/**
 * PollingStationData.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.domain.demographics;

import java.util.Set;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.persistence.UniqueConstraint;

import com.smartmatic.waffle.domain.bulk.CustomCodeAnnotation;
import com.smartmatic.waffle.domain.general.GeneralInfo;
import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;
import com.thoughtworks.xstream.annotations.XStreamConverter;
import com.thoughtworks.xstream.annotations.XStreamOmitField;
import com.thoughtworks.xstream.converters.enums.EnumConverter;

/**
 * Class that represents the Polling Station concept. A Polling Station is a
 * electoral unit where the voters may vote. It can contain one or more
 * VotingBooth.
 */
@XStreamAlias("polling-station")
@Entity
@Table(name = "polling_station", uniqueConstraints = { @UniqueConstraint(columnNames = "polling_station_code") })
public class PollingStationData extends GeneralInfo {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 9025972645196074584L;

	/**
	 * Enum that contains all the possible types that a PollingPlaceStation can
	 * contain.
	 */
	@XStreamConverter(EnumConverter.class)
	public enum Type {

		/**
		 * Constant that is used to indicate that a polling station does not
		 * contain a voting machine. When a polling station does not contain a
		 * voting machine it is called a manual polling station.
		 */

		MANUAL,

		/**
		 * Code of the polling station. This field is the key of this object. Is
		 * not required that the user has to know the polling station code
		 * because this field is only used by the system as a control mechanism.
		 */

		AUTOMATED
	}

	/**
	 * Polling place where the polling station is located.
	 */
	@XStreamOmitField
	private PollingPlaceData pollingPlace;

	/** Polling station customCode inside the polling place. */
	@XStreamAlias("number")
	@XStreamAsAttribute
	private int number;

	/**
	 * Type of the polling station.
	 */
	@XStreamOmitField
	@Enumerated(EnumType.ORDINAL)
	private Type type;

	/** Alternate code given by the user. */
	@XStreamOmitField
	private String customCode;

	/**
	 * Amount of voters registered to vote in the polling station.
	 */
	@XStreamOmitField
	private int amountVoters;

	/**
	 * Criteria used to separate the voters of the different polling station
	 * contained in the polling place.
	 */
	@XStreamAlias("criteria")
	@XStreamAsAttribute
	private String splitCriteria;

	/**
	 * Collection of voting machines contained in this polling station.
	 */
	@XStreamOmitField
	private Set<VotingBoothData> votingBooths;

	/** Indicates if this polling station was generated automatically. */
	@XStreamOmitField
	private boolean generated = false;

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.domain.general.BaseGeneralInfo#getVersion()
	 */
	@Override
	@Transient
	public int getVersion() {
		return 0;
	}

	/**
	 * Getter of <code>code</code> property.
	 * 
	 * @return <code>code</code> property value.
	 * @see #code
	 */
	@Override
	@Id
	@Column(name = "polling_station_code")
	@GeneratedValue(strategy = GenerationType.AUTO, generator = "s_polling_station")
	@SequenceGenerator(name = "s_polling_station", sequenceName = "s_polling_station", allocationSize = 1, initialValue = 1)
	public long getCode() {
		return this.code;
	}

	/**
	 * Getter of <code>pollingPlace</code> property.
	 * 
	 * @return <code>pollingPlace</code> property value.
	 * @see #pollingPlace
	 */
	@ManyToOne(optional = false, targetEntity = PollingPlaceData.class)
	@JoinColumn(name = "polling_place_code", nullable = false, updatable = true)
	public PollingPlaceData getPollingPlace() {
		return this.pollingPlace;
	}

	/**
	 * Setter of <code>pollingPlace</code> property.
	 * 
	 * @param pollingPlace
	 *            The <code>pollingPlace</code> to set.
	 * @see #pollingPlace
	 */
	public void setPollingPlace(PollingPlaceData pollingPlace) {
		this.pollingPlace = pollingPlace;
	}

	/**
	 * Getter of <code>number</code> property.
	 * 
	 * @return <code>number</code> property value.
	 * @see #number
	 */
	@Column(name = "polling_station_number", nullable = false)
	public int getNumber() {
		return this.number;
	}

	/**
	 * Setter of <code>number</code> property.
	 * 
	 * @param number
	 *            The <code>number</code> to set.
	 * @see #number
	 */
	public void setNumber(int number) {
		this.number = number;
	}

	/**
	 * Getter of <code>type</code> property.
	 * 
	 * @return <code>type</code> property value.
	 * @see #type
	 */
	@Column(name = "polling_station_type", nullable = false)
	public Type getType() {
		return this.type;
	}

	/**
	 * Setter of <code>type</code> property.
	 * 
	 * @param type
	 *            The <code>type</code> to set.
	 * @see #type
	 */
	public void setType(Type type) {
		this.type = type;
	}

	/**
	 * Getter of <code>customCode</code> property.
	 * 
	 * @return <code>customCode</code> property value.
	 * @see #customCode
	 */
	@Override
	@CustomCodeAnnotation
	@Column(name = "polling_station_custom_code", nullable = false, length = 80)
	public String getCustomCode() {
		return this.customCode;
	}

	/**
	 * Setter of <code>customCode</code> property.
	 * 
	 * @param customCode
	 *            The <code>customCode</code> to set.
	 * @see #customCode
	 */
	@Override
	public void setCustomCode(String customCode) {
		this.customCode = customCode;
	}

	/**
	 * Getter of <code>amountVoters</code> property.
	 * 
	 * @return <code>amountVoters</code> property value.
	 * @see #amountVoters
	 */
	@Column(name = "voter_number", nullable = false)
	public int getAmountVoters() {
		return this.amountVoters;
	}

	/**
	 * Setter of <code>amountVoters</code> property.
	 * 
	 * @param amountVoters
	 *            The <code>amountVoters</code> to set.
	 * @see #amountVoters
	 */
	public void setAmountVoters(int amountVoters) {
		this.amountVoters = amountVoters;
	}

	/**
	 * Getter of <code>splitCriteria</code> property.
	 * 
	 * @return <code>splitCriteria</code> property value.
	 * @see #splitCriteria
	 */
	@Column(name = "split_criteria", length = 320)
	public String getSplitCriteria() {
		return this.splitCriteria;
	}

	/**
	 * Setter of <code>splitCriteria</code> property.
	 * 
	 * @param splitCriteria
	 *            The <code>splitCriteria</code> to set.
	 * @see #splitCriteria
	 */
	public void setSplitCriteria(String splitCriteria) {
		this.splitCriteria = splitCriteria;
	}

	/**
	 * Getter of <code>votingBooths</code> property.
	 * 
	 * @return <code>votingBooths</code> property value.
	 * @see #votingBooths
	 */
	@OneToMany(mappedBy = "pollingStation")
	public Set<VotingBoothData> getVotingBooths() {
		return this.votingBooths;
	}

	/**
	 * Setter of <code>votingBooths</code> property.
	 * 
	 * @param votingBooths
	 *            The <code>votingBooths</code> to set.
	 * @see #votingBooths
	 */
	public void setVotingBooths(Set<VotingBoothData> votingBooths) {
		this.votingBooths = votingBooths;
	}

	/**
	 * Getter of <code>generated</code> property.
	 * 
	 * @return <code>generated</code> property value.
	 * @see #generated
	 */
	@Column(name = "generated", nullable = false)
	public boolean isGenerated() {
		return this.generated;
	}

	/**
	 * Setter of <code>generated</code> property.
	 * 
	 * @param generated
	 *            The <code>generated</code> to set.
	 * @see #generated
	 */
	public void setGenerated(boolean generated) {
		this.generated = generated;
	}

}
