/**
 * RegionData.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.domain.demographics;

import java.util.Set;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.persistence.UniqueConstraint;

import com.smartmatic.waffle.domain.authority.ElectoralAuthorityData;
import com.smartmatic.waffle.domain.bulk.CustomCodeAnnotation;
import com.smartmatic.waffle.domain.general.GeneralInfo;
import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.lang.annotations.basic.ValidatorListener;
import com.smartmatic.waffle.persistence.metadata.MultilingualPropertyAnnotation;
import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;
import com.thoughtworks.xstream.annotations.XStreamOmitField;

/**
 * This class represents the region table that represents a geographical
 * location such as a country, a state, a municipality, or any other way to
 * divide a particular territory.
 */
@Entity
@Table(name = "region", uniqueConstraints = {
		@UniqueConstraint(columnNames = "region_custom_code"),
		@UniqueConstraint(columnNames = { "region_name" }) })
@ValidatorListener(name = "com.smartmatic.waffle.validator.demographics.RegionDataValidator")
@XStreamAlias("region")
public class RegionData extends GeneralInfo implements Comparable<RegionData> {

	/** The Constant serialVersionUID. */
	@XStreamOmitField
	private static final long serialVersionUID = -5647680735987584916L;

	/** The category. */
	@XStreamOmitField
	private CategoryData category;

	/** The parent. */
	@XStreamOmitField
	private Set<RegionData> parent;

	/** The children. */
	@XStreamOmitField
	private Set<RegionData> children;

	/** The all parents. */
	@XStreamOmitField
	private Set<RegionData> allParents;

	/** The all children. */
	@XStreamOmitField
	private Set<RegionData> allChildren;

	/** The elections language. */
	@XStreamOmitField
	private Set<LanguageData> electionsLanguage;

	/** The polling places. */
	@XStreamOmitField
	private Set<PollingPlaceData> pollingPlaces;

	/** The region name. */
	@XStreamAlias("name-key")
	@XStreamAsAttribute
	String regionName;

	/**
	 * All districts related to this region.
	 */
	@XStreamOmitField
	private Set<DistrictData> districts;

	/** The programming units. */
	private Set<ProgrammingUnitData> programmingUnits;

	/** Electoral Authorities associated with this region. */
	@XStreamOmitField
	private Set<ElectoralAuthorityData> electoralAuthorities;

	/** The nis code. */
	@XStreamOmitField
	private String nisCode;

	/** Audio voting enable for polling place. */
	@XStreamOmitField
	public boolean audioVotingSetting = false;

	/**
	 * Gets the electoral Authorities associated with this region.
	 * 
	 * @return the electoral Authorities associated with this region
	 */
	@OneToMany(mappedBy = "region")
	public Set<ElectoralAuthorityData> getElectoralAuthorities() {
		return this.electoralAuthorities;
	}

	/**
	 * Setter of <code>electoralAuthorities</code> property.
	 * 
	 * @param electoralAuthorities
	 *            The <code>electoralAuthorities</code> to set
	 */
	public void setElectoralAuthorities(
			Set<ElectoralAuthorityData> electoralAuthorities) {
		this.electoralAuthorities = electoralAuthorities;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.domain.general.BaseGeneralInfo#getCode()
	 */
	@Override
	@Id
	@Column(name = "region_code")
	@GeneratedValue(strategy = GenerationType.AUTO, generator = "s_region")
	@SequenceGenerator(name = "s_region", sequenceName = "s_region", allocationSize = 1, initialValue = 1)
	public long getCode() {
		return this.code;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.domain.general.BaseGeneralInfo#setCode(long)
	 */
	@Override
	public void setCode(long code) {
		this.code = code;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.domain.general.BaseGeneralInfo#getCustomCode()
	 */
	@Override
	@CustomCodeAnnotation
	@Column(name = "region_custom_code", nullable = false)
	public String getCustomCode() {
		return this.customCode;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.domain.general.BaseGeneralInfo#setCustomCode(
	 * java.lang.String)
	 */
	@Override
	public void setCustomCode(String customCode) {
		this.customCode = customCode;
	}

	/**
	 * Gets the region name.
	 * 
	 * @return the region name
	 */
	@MultilingualPropertyAnnotation
	@Column(name = "region_name", nullable = false)
	public String getRegionName() {
		return this.regionName;
	}

	/**
	 * Setter of <code>regionName</code> property.
	 * 
	 * @param regionName
	 *            The <code>regionName</code> to set
	 */
	public void setRegionName(String regionName) {
		this.regionName = regionName;
	}

	/**
	 * Gets the category.
	 * 
	 * @return the category
	 */
	@ManyToOne
	@JoinColumn(name = "category_code", nullable = false, updatable = true)
	public CategoryData getCategory() {
		return this.category;
	}

	/**
	 * Setter of <code>category</code> property.
	 * 
	 * @param category
	 *            The <code>category</code> to set
	 */
	public void setCategory(CategoryData category) {
		this.category = category;
	}

	/**
	 * Gets the parent.
	 * 
	 * @return the parent
	 */
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = "region_parent", inverseJoinColumns = @JoinColumn(name = "region_parent_code", referencedColumnName = "region_code"), joinColumns = @JoinColumn(name = "region_child_code", referencedColumnName = "region_code"))
	public Set<RegionData> getParent() {
		return this.parent;
	}

	/**
	 * Setter of <code>parent</code> property.
	 * 
	 * @param parent
	 *            The <code>parent</code> to set
	 */
	public void setParent(Set<RegionData> parent) {
		this.parent = parent;
	}

	/**
	 * Gets the children.
	 * 
	 * @return the children
	 */
	@ManyToMany(mappedBy = "parent")
	public Set<RegionData> getChildren() {
		return this.children;
	}

	/**
	 * Setter of <code>children</code> property.
	 * 
	 * @param children
	 *            The <code>children</code> to set
	 */
	public void setChildren(Set<RegionData> children) {
		this.children = children;
	}

	/**
	 * Gets the all parents.
	 * 
	 * @return the all parents
	 */
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = "region_all_parent", inverseJoinColumns = @JoinColumn(name = "region_parent_code", referencedColumnName = "region_code"), joinColumns = @JoinColumn(name = "region_child_code", referencedColumnName = "region_code"))
	public Set<RegionData> getAllParents() {
		return this.allParents;
	}

	/**
	 * Setter of <code>allParents</code> property.
	 * 
	 * @param allParents
	 *            The <code>allParents</code> to set
	 */
	public void setAllParents(Set<RegionData> allParents) {
		this.allParents = allParents;
	}

	/**
	 * Gets the all children.
	 * 
	 * @return the all children
	 */
	@ManyToMany(mappedBy = "allParents", fetch = FetchType.LAZY)
	public Set<RegionData> getAllChildren() {
		return this.allChildren;
	}

	/**
	 * Setter of <code>allChildren</code> property.
	 * 
	 * @param allChildren
	 *            The <code>allChildren</code> to set
	 */
	public void setAllChildren(Set<RegionData> allChildren) {
		this.allChildren = allChildren;
	}

	/**
	 * Getter of <code>districts</code> property.
	 * 
	 * @return <code>districts</code> property value.
	 * @see #districts
	 */
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = "region_district", inverseJoinColumns = @JoinColumn(name = "district_code", referencedColumnName = "district_code"), joinColumns = @JoinColumn(name = "region_code", referencedColumnName = "region_code"))
	public Set<DistrictData> getDistricts() {
		return this.districts;
	}

	/**
	 * Setter of <code>districts</code> property.
	 * 
	 * @param districts
	 *            The <code>districts</code> to set.
	 * @see #districts
	 */
	public void setDistricts(Set<DistrictData> districts) {
		this.districts = districts;
	}

	/**
	 * Gets the programming units.
	 * 
	 * @return the programmingUnits
	 */
	@ManyToMany(fetch = FetchType.LAZY)
	@JoinTable(name = "region_programming_unit", inverseJoinColumns = @JoinColumn(name = "programming_unit_code", referencedColumnName = "programming_unit_code"), joinColumns = @JoinColumn(name = "region_code", referencedColumnName = "region_code"))
	public Set<ProgrammingUnitData> getProgrammingUnits() {
		return this.programmingUnits;
	}

	/**
	 * Sets the programming units.
	 * 
	 * @param programmingUnits
	 *            the programmingUnits to set
	 */
	public void setProgrammingUnits(Set<ProgrammingUnitData> programmingUnits) {
		this.programmingUnits = programmingUnits;
	}

	/**
	 * Getter of {@link #pollingPlaces} field.
	 * 
	 * @return {@link #pollingPlaces} value.
	 */
	@OneToMany(mappedBy = "region")
	public Set<PollingPlaceData> getPollingPlaces() {
		return this.pollingPlaces;
	}

	/**
	 * Setter of {@link #pollingPlaces} field.
	 * 
	 * @param pollingPlaces
	 *            The Set<PollingPlaceData> to set.
	 */
	public void setPollingPlaces(Set<PollingPlaceData> pollingPlaces) {
		this.pollingPlaces = pollingPlaces;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.domain.general.BaseGeneralInfo#getVersion()
	 */
	@Override
	@Transient
	public int getVersion() {
		return this.version;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.domain.BaseDataObject#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object anObject) {
		if (anObject instanceof RegionData) {
			RegionData otherRegion = (RegionData) anObject;
			return otherRegion.code == this.code;
		}
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.domain.BaseDataObject#hashCode()
	 */
	@Override
	public int hashCode() {
		return (int) this.code;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	@Override
	public int compareTo(RegionData region) {

		if (this.category.getCategoryLevel().ordinal() < region.getCategory()
				.getCategoryLevel().ordinal()) {
			return -1;
		} else if (this.category.getCategoryLevel() == region.getCategory()
				.getCategoryLevel()) {
			return 0;
		} else {
			return 1;
		}
	}

	/**
	 * Getter of {@link #nisCode} field.
	 * 
	 * @return {@link #nisCode} value.
	 */
	@Column(name = "nis_code")
	public String getNisCode() {
		return this.nisCode;
	}

	/**
	 * Setter of {@link #nisCode} field.
	 * 
	 * @param nisCode
	 *            The String to set.
	 */
	public void setNisCode(String nisCode) {
		this.nisCode = nisCode;
	}

	/**
	 * Gets the elections language.
	 * 
	 * @return the elections language
	 */
	@ManyToMany()
	@JoinTable(name = "region_election_language", joinColumns = @JoinColumn(name = "region_code", referencedColumnName = "region_code"), inverseJoinColumns = @JoinColumn(name = "election_language_code", referencedColumnName = "election_language_code"))
	public Set<LanguageData> getElectionsLanguage() {
		return this.electionsLanguage;
	}

	/**
	 * Setter of <code>electionsLanguage</code> property.
	 * 
	 * @param electionsLanguage
	 *            The <code>electionsLanguage</code> to set
	 */
	public void setElectionsLanguage(Set<LanguageData> electionsLanguage) {
		this.electionsLanguage = electionsLanguage;
	}

	/**
	 * Get if audio voting is enable for the region polling places.
	 * 
	 * @return the audioVotingEnable Is audio voting enable for region polling
	 *         places.
	 */
	@Column(name = "audio_voting_enable")
	public boolean isAudioVotingSetting() {
		return this.audioVotingSetting;
	}

	/**
	 * Sets the enable/disable audio voting value for the region polling places.
	 * 
	 * @param audioVotingEnable
	 *            the audio voting value for region polling places.
	 */
	public void setAudioVotingSetting(boolean audioVotingEnable) {
		this.audioVotingSetting = audioVotingEnable;
	}

}
