/**
 * VotingBoothData.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.domain.demographics;

import java.util.Set;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToOne;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.persistence.UniqueConstraint;

import com.smartmatic.waffle.domain.bulk.CustomCodeAnnotation;
import com.smartmatic.waffle.domain.device.LogicalVotingGroupData;
import com.smartmatic.waffle.domain.device.VotingDeviceData;
import com.smartmatic.waffle.domain.general.GeneralInfo;
import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;
import com.thoughtworks.xstream.annotations.XStreamConverter;
import com.thoughtworks.xstream.annotations.XStreamOmitField;
import com.thoughtworks.xstream.converters.enums.EnumConverter;

/**
 * Class that represents the concept of a Voting Booth. A Voting booth is the
 * place where the voter cast his/her vote. A Polling Station can contain
 * several voting booth. Each voting booth is responsible of the transmission of
 * electoral information to the central servers.
 */
@XStreamAlias("voting-booth")
@Entity
@Table(name = "voting_booth", uniqueConstraints = { @UniqueConstraint(columnNames = "voting_booth_custom_code") })
public class VotingBoothData extends GeneralInfo {

	/** The Constant serialVersionUID. */
	@XStreamOmitField
	private static final long serialVersionUID = 7800404945698900079L;

	/**
	 * Enumerate that contains all the possible values of the attribute Type for
	 * a Voting Booth.
	 */
	@XStreamConverter(EnumConverter.class)
	public enum Type {

		/**
		 * Constant that is used to indicate that a Voting Booth is a manual
		 * Voting Booth. This means that the voting process is done manually and
		 * later the information is introduced in the system through the manual
		 * TALLY subsystem
		 */
		MANUAL,

		/**
		 * Constant that is used to indicate that a Voting Booth is a Voting
		 * machine. This means that all the voting process is done in an
		 * automated way.
		 */
		AUTOMATED
	}

	/**
	 * Number of the voting booth. This field is used to numerate all voting
	 * booths belonging to the voting station attribute to this object.
	 */
	@XStreamAlias("number")
	@XStreamAsAttribute
	private int number;

	/**
	 * Amount of voters that can vote in this Voting Booth.
	 */
	@XStreamAlias("registered-electors")
	@XStreamAsAttribute
	private int amountVoters;

	/** Polling Station where the Voting Booth is located. */
	@XStreamOmitField
	private PollingStationData pollingStation;

	/** The polling station code. */
	@XStreamAlias("polling-station-code")
	@XStreamAsAttribute
	Long pollingStationCode;

	/** The polling station number. */
	@XStreamAlias("polling-station-number")
	@XStreamAsAttribute
	int pollingStationNumber;

	/** The polling station criteria. */
	@XStreamAlias("polling-station-criteria")
	@XStreamAsAttribute
	String pollingStationCriteria;

	/** The polling place code. */
	@XStreamAlias("polling-place-code")
	@XStreamAsAttribute
	Long pollingPlaceCode;

	/** The polling place registered electors. */
	@XStreamAlias("polling-place-registered-electors")
	@XStreamAsAttribute
	int pollingPlaceRegisteredElectors;

	/** The polling place address. */
	@XStreamAlias("polling-place-address")
	@XStreamAsAttribute
	String pollingPlaceAddress;

	/** The polling place name. */
	@XStreamAlias("polling-place-name")
	@XStreamAsAttribute
	String pollingPlaceName;

	/** The logical voting group data. */
	@XStreamOmitField
	Set<LogicalVotingGroupData> logicalVotingGroupData;

	/**
	 * Type of the voting Booth. Its possible values are defined in the enum
	 * Type
	 */
	@XStreamOmitField
	@Enumerated(EnumType.ORDINAL)
	private Type type;

	/**
	 * Criteria used to separate the voters of the different Voting Booth
	 * contained in the polling Station.
	 */
	@XStreamOmitField
	private String splitCriteria;

	/**
	 * The device where the voters from this voting booth cast their vote.
	 */
	@XStreamOmitField
	private VotingDeviceData votingDevice;

	/** Indicate if a votingBooth is the default one for a votingDevice. */
	@XStreamAlias("default")
	@XStreamAsAttribute
	private boolean defaultVB;

	/** Indicates if this voting booth was generated automatically. */
	@XStreamOmitField
	private boolean generated = false;

	/** The region hierarchies. */
	@XStreamAlias("region-hierarchies")
	private Set<RegionHierarchy> regionHierarchies;

	/** The region hierarchies. */
	@XStreamAlias("totalization-matches")
	private Set<TotalizationNisCode> totalizationMatches;

	/** The polling place address. */
	@XStreamAlias("audio-voting-enable")
	@XStreamAsAttribute
	private boolean audioVotingEnable;

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.domain.general.BaseGeneralInfo#getVersion()
	 */
	@Override
	@Transient
	public int getVersion() {
		return 0;
	}

	/**
	 * Getter of <code>code</code> property.
	 * 
	 * @return <code>code</code> property value.
	 * @see #code
	 */
	@Override
	@Id
	@Column(name = "voting_booth_code")
	@GeneratedValue(strategy = GenerationType.AUTO, generator = "s_voting_booth")
	@SequenceGenerator(name = "s_voting_booth", sequenceName = "s_voting_booth", allocationSize = 1, initialValue = 1)
	public long getCode() {
		return this.code;
	}

	/**
	 * Getter of <code>customCode</code> property.
	 * 
	 * @return <code>customCode</code> property value.
	 * @see #customCode
	 */
	@Override
	@CustomCodeAnnotation
	@Column(name = "voting_booth_custom_code", nullable = false, length = 80)
	public String getCustomCode() {
		return this.customCode;
	}

	/**
	 * Getter of <code>number</code> property.
	 * 
	 * @return <code>number</code> property value.
	 * @see #number
	 */
	@Column(name = "voting_booth_number", nullable = false)
	public int getNumber() {
		return this.number;
	}

	/**
	 * Setter of <code>number</code> property.
	 * 
	 * @param number
	 *            The <code>number</code> to set.
	 * @see #number
	 */
	public void setNumber(int number) {
		this.number = number;
	}

	/**
	 * Getter of <code>amountVoters</code> property.
	 * 
	 * @return <code>amountVoters</code> property value.
	 * @see #amountVoters
	 */
	@Column(name = "voter_number", nullable = false)
	public int getAmountVoters() {
		return this.amountVoters;
	}

	/**
	 * Setter of <code>amountVoters</code> property.
	 * 
	 * @param amountVoters
	 *            The <code>amountVoters</code> to set.
	 * @see #amountVoters
	 */
	public void setAmountVoters(int amountVoters) {
		this.amountVoters = amountVoters;
	}

	/**
	 * Getter of <code>pollingStation</code> property.
	 * 
	 * @return <code>pollingStation</code> property value.
	 * @see #pollingStation
	 */
	@ManyToOne(optional = false, targetEntity = PollingStationData.class)
	@JoinColumn(name = "polling_station_code", nullable = false, updatable = true)
	public PollingStationData getPollingStation() {
		return this.pollingStation;
	}

	/**
	 * Setter of <code>pollingStation</code> property.
	 * 
	 * @param pollingStation
	 *            The <code>pollingStation</code> to set.
	 * @see #pollingStation
	 */
	public void setPollingStation(PollingStationData pollingStation) {
		this.pollingStation = pollingStation;
	}

	/**
	 * Getter of <code>type</code> property.
	 * 
	 * @return <code>type</code> property value.
	 * @see #type
	 */
	@Column(name = "voting_booth_type")
	public Type getType() {
		return this.type;
	}

	/**
	 * Setter of <code>type</code> property.
	 * 
	 * @param type
	 *            The <code>type</code> to set.
	 * @see #type
	 */
	public void setType(Type type) {
		this.type = type;
	}

	/**
	 * Getter of <code>splitCriteria</code> property.
	 * 
	 * @return <code>splitCriteria</code> property value.
	 * @see #splitCriteria
	 */
	@Column(name = "split_criteria", length = 320)
	public String getSplitCriteria() {
		return this.splitCriteria;
	}

	/**
	 * Setter of <code>splitCriteria</code> property.
	 * 
	 * @param splitCriteria
	 *            The <code>splitCriteria</code> to set.
	 * @see #splitCriteria
	 */
	public void setSplitCriteria(String splitCriteria) {
		this.splitCriteria = splitCriteria;
	}

	/**
	 * Getter of <code>votingDevice</code> property.
	 * 
	 * @return <code>votingDevice</code> property value.
	 * @see #votingDevice
	 */
	@OneToOne(optional = true, targetEntity = VotingDeviceData.class)
	@JoinColumn(name = "voting_device_code", updatable = true)
	public VotingDeviceData getVotingDevice() {
		return this.votingDevice;
	}

	/**
	 * Setter of <code>votingDevice</code> property.
	 * 
	 * @param votingDevice
	 *            The <code>votingDevice</code> to set.
	 * @see #votingDevice
	 */
	public void setVotingDevice(VotingDeviceData votingDevice) {
		this.votingDevice = votingDevice;
	}

	/**
	 * Getter of <code>defaultVB</code> property.
	 * 
	 * @return <code>defaultVB</code> property value.
	 * @see #defaultVB
	 */
	@Column(name = "voting_booth_default", nullable = false)
	public boolean isDefaultVB() {
		return this.defaultVB;
	}

	/**
	 * Setter of <code>defaultVB</code> property.
	 * 
	 * @param defaultVB
	 *            The <code>defaultVB</code> to set.
	 * @see #defaultVB
	 */
	public void setDefaultVB(boolean defaultVB) {
		this.defaultVB = defaultVB;
	}

	/**
	 * Getter of <code>generated</code> property.
	 * 
	 * @return <code>generated</code> property value.
	 * @see #generated
	 */
	@Column(name = "generated", nullable = false)
	public boolean isGenerated() {
		return this.generated;
	}

	/**
	 * Setter of <code>generated</code> property.
	 * 
	 * @param generated
	 *            The <code>generated</code> to set.
	 * @see #generated
	 */
	public void setGenerated(boolean generated) {
		this.generated = generated;
	}

	/**
	 * Getter of <code>totalizationMatches</code> property.
	 * 
	 * @return <code>totalizationMatches</code> property value.
	 * @see #TotalizationNisCode
	 */
	@Transient
	public Set<TotalizationNisCode> getTotalizationMatches() {
		return this.totalizationMatches;
	}

	/**
	 * Setter of <code>totalizationMatches</code> property.
	 * 
	 * @param regionHierarchies
	 *            The <code>totalizationMatches</code> to set.
	 * @see #TotalizationNisCode
	 */
	public void setTotalizationMatches(
			Set<TotalizationNisCode> totalizationMatches) {
		this.totalizationMatches = totalizationMatches;
	}

	/**
	 * Getter of <code>regionHierarchies</code> property.
	 * 
	 * @return <code>regionHierarchies</code> property value.
	 * @see #regionHierarchies
	 */
	@Transient
	public Set<RegionHierarchy> getRegionHierarchies() {
		return this.regionHierarchies;
	}

	/**
	 * Setter of <code>regionHierarchies</code> property.
	 * 
	 * @param regionHierarchies
	 *            The <code>regionHierarchies</code> to set.
	 * @see #regionHierarchies
	 */
	public void setRegionHierarchies(Set<RegionHierarchy> regionHierarchies) {
		this.regionHierarchies = regionHierarchies;
	}

	/**
	 * Getter of {@link #pollingStationCode} field.
	 * 
	 * @return {@link #pollingStationCode} value.
	 */
	@Transient
	public Long getPollingStationCode() {
		return this.pollingStationCode;
	}

	/**
	 * Setter of {@link #pollingStationCode} field.
	 * 
	 * @param pollingStationCode
	 *            The Long to set.
	 */
	public void setPollingStationCode(Long pollingStationCode) {
		this.pollingStationCode = pollingStationCode;
	}

	/**
	 * Getter of {@link #pollingStationNumber} field.
	 * 
	 * @return {@link #pollingStationNumber} value.
	 */
	@Transient
	public int getPollingStationNumber() {
		return this.pollingStationNumber;
	}

	/**
	 * Setter of {@link #pollingStationNumber} field.
	 * 
	 * @param pollingStationNumber
	 *            The int to set.
	 */
	public void setPollingStationNumber(int pollingStationNumber) {
		this.pollingStationNumber = pollingStationNumber;
	}

	/**
	 * Getter of {@link #pollingStationCriteria} field.
	 * 
	 * @return {@link #pollingStationCriteria} value.
	 */
	@Transient
	public String getPollingStationCriteria() {
		return this.pollingStationCriteria;
	}

	/**
	 * Setter of {@link #pollingStationCriteria} field.
	 * 
	 * @param pollingStationCriteria
	 *            The String to set.
	 */
	public void setPollingStationCriteria(String pollingStationCriteria) {
		this.pollingStationCriteria = pollingStationCriteria;
	}

	/**
	 * Getter of {@link #pollingPlaceCode} field.
	 * 
	 * @return {@link #pollingPlaceCode} value.
	 */
	@Transient
	public Long getPollingPlaceCode() {
		return this.pollingPlaceCode;
	}

	/**
	 * Setter of {@link #pollingPlaceCode} field.
	 * 
	 * @param pollingPlaceCode
	 *            The Long to set.
	 */
	public void setPollingPlaceCode(Long pollingPlaceCode) {
		this.pollingPlaceCode = pollingPlaceCode;
	}

	/**
	 * Getter of {@link #pollingPlaceRegisteredElectors} field.
	 * 
	 * @return {@link #pollingPlaceRegisteredElectors} value.
	 */
	@Transient
	public int getPollingPlaceRegisteredElectors() {
		return this.pollingPlaceRegisteredElectors;
	}

	/**
	 * Setter of {@link #pollingPlaceRegisteredElectors} field.
	 * 
	 * @param pollingPlaceRegisteredElectors
	 *            The int to set.
	 */
	public void setPollingPlaceRegisteredElectors(
			int pollingPlaceRegisteredElectors) {
		this.pollingPlaceRegisteredElectors = pollingPlaceRegisteredElectors;
	}

	/**
	 * Getter of {@link #pollingPlaceAddress} field.
	 * 
	 * @return {@link #pollingPlaceAddress} value.
	 */
	@Transient
	public String getPollingPlaceAddress() {
		return this.pollingPlaceAddress;
	}

	/**
	 * Setter of {@link #pollingPlaceAddress} field.
	 * 
	 * @param pollingPlaceAddress
	 *            The String to set.
	 */
	public void setPollingPlaceAddress(String pollingPlaceAddress) {
		this.pollingPlaceAddress = pollingPlaceAddress;
	}

	/**
	 * Getter of {@link #pollingPlaceName} field.
	 * 
	 * @return {@link #pollingPlaceName} value.
	 */
	@Transient
	public String getPollingPlaceName() {
		return this.pollingPlaceName;
	}

	/**
	 * Setter of {@link #pollingPlaceName} field.
	 * 
	 * @param pollingPlaceName
	 *            The String to set.
	 */
	public void setPollingPlaceName(String pollingPlaceName) {
		this.pollingPlaceName = pollingPlaceName;
	}

	/**
	 * Getter of {@link #logicalVotingGroupData} field.
	 * 
	 * @return {@link #logicalVotingGroupData} value.
	 */
	@ManyToMany(mappedBy = "votingBooths")
	public Set<LogicalVotingGroupData> getLogicalVotingGroupData() {
		return this.logicalVotingGroupData;
	}

	/**
	 * Setter of {@link #logicalVotingGroupData} field.
	 * 
	 * @param logicalVotingGroupData
	 *            The Set<LogicalVotingGroupData> to set.
	 */
	public void setLogicalVotingGroupData(
			Set<LogicalVotingGroupData> logicalVotingGroupData) {
		this.logicalVotingGroupData = logicalVotingGroupData;
	}

	/**
	 * Get if audio voting is enable/disable.
	 * 
	 * @return the audioVotingEnable True if audio voting is enable.
	 */
	@Transient
	public boolean isAudioVotingEnable() {
		return this.audioVotingEnable;
	}

	/**
	 * Set enable/disable audio voting value.
	 * 
	 * @param audioVotingEnable
	 *            the audioVotingEnable to set
	 */
	public void setAudioVotingEnable(boolean audioVotingEnable) {
		this.audioVotingEnable = audioVotingEnable;
	}

}
