/**
 * LanguageData.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.domain.i18n;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Locale;
import java.util.Map;
import java.util.Set;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.persistence.UniqueConstraint;

import org.apache.commons.lang.builder.ToStringBuilder;

import com.smartmatic.waffle.domain.bulk.CustomCodeAnnotation;
import com.smartmatic.waffle.domain.demographics.ElectionData;
import com.smartmatic.waffle.domain.demographics.RegionData;
import com.smartmatic.waffle.domain.general.GeneralInfo;
import com.smartmatic.waffle.domain.multiLanguage.StringResourceData;
import com.smartmatic.waffle.domain.multiLanguage.TranslationKeyData;
import com.smartmatic.waffle.lang.annotations.basic.ValidatorListener;
import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;
import com.thoughtworks.xstream.annotations.XStreamOmitField;

/**
 * Class that represents a supported language in the voting experience process.
 */
@Entity
@Table(name = "election_language", uniqueConstraints = { @UniqueConstraint(columnNames = "custom_code") })
@XStreamAlias("language")
@ValidatorListener(name = "com.smartmatic.waffle.validator.i18.LanguageDataValidator")
public class LanguageData extends GeneralInfo {

	/** The default language. */
	@XStreamOmitField
	boolean defaultLanguage;

	/** The Constant serialVersionUID. */
	@XStreamOmitField
	private static final long serialVersionUID = 6081113901441109343L;

	/***
	 * regions list to the current language.
	 */
	@XStreamOmitField
	private Set<RegionData> regions;

	/** Collection of translation key. */
	@XStreamOmitField
	private Set<TranslationKeyData> translationKeys;

	/** The string resources. */
	@XStreamOmitField
	Map<String, StringResourceData> stringResources = new HashMap<String, StringResourceData>();

	/**
	 * Hold the audio resources for the current language this set may defer from
	 * the string resources
	 */
	@XStreamOmitField
	Set<StringResourceData> audioStringResources = new HashSet<StringResourceData>();

	/** The voice hdr file name related to language. */
	@XStreamOmitField
	private String voiceNameHDR;

	/** The voice dat file name related to language. */
	@XStreamOmitField
	private String voiceNameDAT;

	/** The official language. */
	@XStreamAlias("is-default")
	@XStreamAsAttribute
	boolean officialLanguage;

	/** The audio. */
	@XStreamAlias("audio")
	@XStreamAsAttribute
	boolean audio;

	/** The election. */
	@XStreamOmitField
	private ElectionData election;

	/** The language name. */
	@XStreamAlias("name-key")
	@XStreamAsAttribute
	private String languageName;

	/** The language rfc. */
	@XStreamAlias("language")
	@XStreamAsAttribute
	private String languageRFC;

	/**
	 * ISO-3166 That identifies the country to use in the language.
	 * 
	 * @see Locale
	 */
	@XStreamAlias("country")
	@XStreamAsAttribute
	private String country;

	/** Variant of the language [vendor and browser specific code]. @see Locale */

	@XStreamAlias("variant")
	@XStreamAsAttribute
	private String variant;

	/**
	 * Language shortcut key. This key is used as a shortcut to allow the user
	 * select the language pressing the configured shortcut
	 */
	@XStreamAlias("key")
	@XStreamAsAttribute
	private char key = ' ';

	/**
	 * Message displayed to the user to select the language.
	 */
	@XStreamAlias("message")
	@XStreamAsAttribute
	private String message;

	/**
	 * Field used to order the different languages in the User interface.
	 * Languages with lower precedences will be displayed first in the User
	 * Interface.
	 */
	@XStreamAlias("precedence")
	@XStreamAsAttribute
	private int precedence;

	/**
	 * Getter of <code>code</code> property.
	 * 
	 * @return <code>code</code> property value.
	 * @see #code
	 */
	@Override
	@Id
	@Column(name = "election_language_code")
	@GeneratedValue(strategy = GenerationType.AUTO, generator = "s_election_language")
	@SequenceGenerator(name = "s_election_language", sequenceName = "s_election_language", allocationSize = 1, initialValue = 1)
	public long getCode() {
		return this.code;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.domain.general.BaseGeneralInfo#getCustomCode()
	 */
	@Override
	@CustomCodeAnnotation
	@Column(name = "custom_code", nullable = false)
	public String getCustomCode() {
		return this.customCode;
	}

	/**
	 * Getter of {@link #languageCustomCode} field.
	 * 
	 * @return {@link #languageCustomCode} value.
	 */
	@Column(name = "language_rfc", nullable = true, length = 100)
	public String getLanguageRFC() {
		return this.languageRFC;
	}

	/**
	 * Setter of {@link #languageCustomCode} field.
	 * 
	 * @param language
	 *            the new language rfc
	 */
	public void setLanguageRFC(String language) {
		this.languageRFC = language;
	}

	/**
	 * Getter of <code>language</code> property.
	 * 
	 * @return <code>language</code> property value.
	 * @see #languageName
	 */

	@Column(name = "language_name", nullable = true, length = 100)
	public String getLanguageName() {
		return this.languageName;
	}

	/**
	 * Setter of <code>language</code> property.
	 * 
	 * @param language
	 *            The <code>language</code> to set.
	 * @see #languageName
	 */
	public void setLanguageName(String language) {
		this.languageName = language;
	}

	/**
	 * Getter of <code>officialLanguage</code> property.
	 * 
	 * @return <code>officialLanguage</code> property value.
	 * @see #officialLanguage
	 */
	@Column(name = "official_language", nullable = true)
	public boolean getOfficialLanguage() {
		return this.officialLanguage;
	}

	/**
	 * Setter of <code>officialLanguage</code> property.
	 * 
	 * @param officialLanguage
	 *            The <code>officialLanguage</code> to set.
	 * @see #officialLanguage
	 */
	public void setOfficialLanguage(boolean officialLanguage) {
		this.officialLanguage = officialLanguage;
	}

	/**
	 * Getter of <code>translationKeys</code> property.
	 * 
	 * @return <code>translationKeys</code> property value.
	 * @see #translationKeys
	 */

	@OneToMany(mappedBy = "electionLanguage")
	public Set<TranslationKeyData> getTranslationKeys() {
		return this.translationKeys;
	}

	/**
	 * Setter of <code>translationKeys</code> property.
	 * 
	 * @param theTranslationKeys
	 *            The <code>translationKeys</code> to set.
	 * @see #translationKeys
	 */
	public void setTranslationKeys(Set<TranslationKeyData> theTranslationKeys) {
		this.translationKeys = theTranslationKeys;
	}

	/**
	 * Gets the * regions list to the current language.
	 * 
	 * @return the * regions list to the current language
	 */
	@ManyToMany()
	@JoinTable(name = "region_election_language", joinColumns = @JoinColumn(name = "election_language_code", referencedColumnName = "election_language_code"), inverseJoinColumns = @JoinColumn(name = "region_code", referencedColumnName = "region_code"))
	public Set<RegionData> getRegions() {
		return this.regions;
	}

	/**
	 * Sets the * regions list to the current language.
	 * 
	 * @param regions
	 *            the new * regions list to the current language
	 */
	public void setRegions(Set<RegionData> regions) {
		this.regions = regions;
	}

	/**
	 * Overriding of the toString method, in this implementation is used the.
	 * 
	 * @return the string {@link ToStringBuilder} class with the {@link #code}
	 *         and {@link #languageName} properties to perform the method.
	 */
	@Override
	public String toString() {
		return new ToStringBuilder(this).append("code", this.code)
				.append("language", this.languageName).toString();
	}

	/**
	 * Getter of {@link #stringResources} field.
	 * 
	 * @return {@link #stringResources} value.
	 */
	@Transient
	public Map<String, StringResourceData> getStringResources() {
		return this.stringResources;
	}

	/**
	 * Setter of {@link #stringResources} field.
	 * 
	 * @param stringResources
	 *            The List<StringResourcesData> to set.
	 */
	public void setStringResources(StringResourceData stringResources) {
		if (!this.stringResources.containsKey(stringResources.getKey())) {
			this.stringResources.put(stringResources.getKey(), stringResources);
		}
	}

	/**
	 * Getter of {@link #audio} field.
	 * 
	 * @return {@link #audio} value.
	 */
	@Transient
	public boolean isAudio() {
		return this.audio;
	}

	/**
	 * Setter of {@link #audio} field.
	 * 
	 * @param audio
	 *            The boolean to set.
	 */
	public void setAudio(boolean audio) {
		this.audio = audio;
	}

	/**
	 * Getter of {@link #election} field.
	 * 
	 * @return {@link #election} value.
	 */
	@ManyToOne(optional = false, fetch = FetchType.LAZY, targetEntity = ElectionData.class)
	@JoinColumn(name = "election_code", nullable = false, updatable = true)
	public ElectionData getElection() {
		return this.election;
	}

	/**
	 * Setter of {@link #election} field.
	 * 
	 * @param election
	 *            The ElectionData to set.
	 */
	public void setElection(ElectionData election) {
		this.election = election;
	}

	/**
	 * Getter of {@link #country} field.
	 * 
	 * @return {@link #country} value.
	 */
	@Column(name = "country", length = 80)
	public String getCountry() {
		return this.country;
	}

	/**
	 * Setter of {@link #country} field.
	 * 
	 * @param country
	 *            The String to set.
	 */
	public void setCountry(String country) {
		this.country = country;
	}

	/**
	 * Getter of {@link #variant} field.
	 * 
	 * @return {@link #variant} value.
	 */
	@Transient
	public String getVariant() {
		return this.variant;
	}

	/**
	 * Setter of {@link #variant} field.
	 * 
	 * @param variant
	 *            The String to set.
	 */
	public void setVariant(String variant) {
		this.variant = variant;
	}

	/**
	 * Getter of {@link #key} field.
	 * 
	 * @return {@link #key} value.
	 */
	@Transient
	public char getKey() {
		return this.key;
	}

	/**
	 * Setter of {@link #key} field.
	 * 
	 * @param key
	 *            The char to set.
	 */
	public void setKey(char key) {
		this.key = key;
	}

	/**
	 * Getter of {@link #message} field.
	 * 
	 * @return {@link #message} value.
	 */
	@Column(name = "vote_message", length = 80)
	public String getMessage() {
		return this.message;
	}

	/**
	 * Setter of {@link #message} field.
	 * 
	 * @param message
	 *            The String to set.
	 */
	public void setMessage(String message) {
		this.message = message;
	}

	/**
	 * Getter of {@link #precedence} field.
	 * 
	 * @return {@link #precedence} value.
	 */
	@Column(name = "precedence")
	public int getPrecedence() {
		return this.precedence;
	}

	/**
	 * Setter of {@link #precedence} field.
	 * 
	 * @param precedence
	 *            The int to set.
	 */
	public void setPrecedence(int precedence) {
		this.precedence = precedence;
	}

	/**
	 * Getter of <code>type</code> property.
	 * 
	 * @return the default language
	 * @see #type
	 */
	@Column(name = "default_language")
	public boolean getDefaultLanguage() {
		return this.defaultLanguage;
	}

	/**
	 * Setter of <code>type</code> property.
	 * 
	 * @param defaultLanguage
	 *            the new default language
	 * @return <code>type</code> property value.
	 * @see #type
	 */
	public void setDefaultLanguage(boolean defaultLanguage) {
		this.defaultLanguage = defaultLanguage;
	}

	/**
	 * Returns the audio resources associated to the current language.
	 * 
	 * @return the {@link #audioStringResources} set
	 */
	@Transient
	public Set<StringResourceData> getAudioStringResources() {
		return this.audioStringResources;
	}

	/**
	 * Setter of {@link #audioStringResources} property.
	 * 
	 * @param audioStringResources
	 *            the audio string resources set to add to the current language
	 */
	public void setAudioStringResources(
			Set<StringResourceData> audioStringResources) {
		this.audioStringResources = audioStringResources;
	}

	/**
	 * Return the voice hdr file name to use with audio voting.
	 * 
	 * @return the language voice name.
	 */
	@Column(name = "voice_name_hdr")
	public String getVoiceNameHDR() {
		return this.voiceNameHDR;
	}

	/**
	 * Setter of {@link #voiceNameHDR} field.
	 * 
	 * @param voiceName
	 *            the language voice hdr file name to set.
	 */
	public void setVoiceNameHDR(String voiceNameHDR) {
		this.voiceNameHDR = voiceNameHDR;
	}

	/**
	 * Return the voice dat file name to use with audio voting.
	 * 
	 * @return the language voice name.
	 */
	@Column(name = "voice_name_dat")
	public String getVoiceNameDAT() {
		return this.voiceNameDAT;
	}

	/**
	 * Setter of {@link #voiceNameDAT} field.
	 * 
	 * @param voiceName
	 *            the language voice dat file name to set.
	 */
	public void setVoiceNameDAT(String voiceNameDAT) {
		this.voiceNameDAT = voiceNameDAT;
	}

	/**
	 * Hash code.
	 * 
	 * @return the int
	 * @inheritDoc
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = super.hashCode();
		result = prime * result
				+ ((this.name == null) ? 0 : this.name.hashCode());
		return result;
	}

	/**
	 * Equals.
	 * 
	 * @param obj
	 *            the obj
	 * @return true, if successful
	 * @inheritDoc
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (!super.equals(obj)) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		LanguageData other = (LanguageData) obj;
		if (this.name == null) {
			if (other.name != null) {
				return false;
			}
		} else if (!this.name.equals(other.name)) {
			return false;
		}
		return true;
	}
}
