/**
 * BulkXmlValidator.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */

package com.smartmatic.waffle.ecm.bulk.service.util;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringReader;
import java.util.Hashtable;

import org.iso_relax.verifier.Schema;
import org.iso_relax.verifier.Verifier;
import org.xml.sax.InputSource;

import com.smartmatic.waffle.ecm.bulk.service.lang.BulkLoadException;
import com.smartmatic.waffle.log.SMTTLogger;
import com.smartmatic.waffle.properties.ApplicationProperties;
import com.sun.msv.verifier.jarv.FactoryImpl;
import com.sun.msv.verifier.jarv.TheFactoryImpl;

/**
 * Class responsible to validate an xml file used in the bulk data loading
 * module. A xml is validated against a base xsd that defines the common
 * structure of a xml file used in the bulk data loading and if the developer
 * configured a custom xml for a class then is also validated against a custom
 * xsd.
 * 
 * @author Ithamar Torres (itorres@smartmatic.com)
 */

public class BulkXmlValidator {

	private static SMTTLogger logger = SMTTLogger
			.getLogger(BulkXmlValidator.class);

	/**
	 * Holds a list of cached schemas used to validate the bulk files. The
	 * schemas are cached because the schema validation engine used compiles the
	 * schema and produces a class that verifies the validity of a given xml.
	 * The creation of the validation instances is a process that can be avoided
	 * because there is a small set of different xsd to validate so those set is
	 * cached in this hastable.
	 */

	private static Hashtable<String, Schema> schemaCache = new Hashtable<String, Schema>();

	/**
	 * Method used to lookup for a specicfic compiled schema. This method
	 * handles the logic that if the schema was previously cached then returns
	 * the cached instance. If the schema wasn't previously created then creates
	 * the schema and saves its references in the cache table.
	 * 
	 * @param schemaPath
	 *            The path where the xsd is located
	 * @return The compiled schema
	 * @throws BulkLoadException
	 *             If there exist a problem with the xsd path. Mostly a
	 *             exception occurs if the developer provided a wrong xsd path
	 *             or if the xsd is not a valid xsd.
	 */

	private static Schema obtainSchema(String schemaPath)
			throws BulkLoadException {
		try {
			Schema schema = schemaCache.get(schemaPath);
			if (schema == null) {
				FactoryImpl factory = new TheFactoryImpl();
				schema = factory.compileSchema(new File(schemaPath));
				schemaCache.put(schemaPath, schema);
			}
			return schema;
		} catch (Exception ex) {
			throw new BulkLoadException(
					BulkLoadException.ExceptionInfo.XML_SCHEMA_NOT_VALID, ex);
		}
	}

	/**
	 * Method that verifies a xml using the configured base xsd to check that
	 * the information contained in the xml is correct [at least syntactically
	 * correct].
	 * 
	 * @param bulkXml
	 *            The xml string
	 * @return The errors that contains the xml. If the xml does not contain any
	 *         error then this method returns an empty string.
	 * @throws BulkLoadException
	 *             If there is a problem with the xsd used to validate the xml
	 *             information
	 */

	public static String validateBaseXsd(String bulkXml)
			throws BulkLoadException {
		final StringReader reader = new StringReader(bulkXml);
		try {
			InputSource source = new InputSource(reader);
			return validateBaseXsd(source);

		} finally {

			reader.close();
		}
	}

	/**
	 * Method that verifies a xml using the configured base xsd to check that
	 * the information contained in the xml is correct [at least syntactically
	 * correct].
	 * 
	 * @param bulkXml
	 *            The xml string
	 * @return The errors that contains the xml. If the xml does not contain any
	 *         error then this method returns an empty string.
	 * @throws BulkLoadException
	 *             If there is a problem with the xsd used to validate the xml
	 *             information
	 */

	public static String validateBaseXsd(InputSource bulkXml)
			throws BulkLoadException {
		BulkErrorHandler errorHandler = new BulkErrorHandler();
		try {
			String xsdPath = ApplicationProperties
					.getValue("bulk.base.xsd.path");
			return BulkXmlValidator.validateXsd(bulkXml, xsdPath);
		} catch (BulkLoadException e) {
			throw e;
		} catch (Exception e) {
			StringBuffer errorReport = errorHandler.getErrorReport();
			errorReport.append(e.getMessage());
			return errorReport.toString();
		}
	}

	/**
	 * Method that verifies a xml using the provided xsd path to check that the
	 * information contained in the xml is correct. This xsd can be as specific
	 * as the developer wants. This way the system can be configured using very
	 * strict validation rules.
	 * 
	 * @param bulkXml
	 *            The xml string
	 * @param xsdPath
	 *            The path where is the xsd against validate the xml received.
	 * 
	 * @return The errors that contains the xml. If the xml does not contain any
	 *         error then this method returns an empty string.
	 * 
	 * @throws BulkLoadException
	 *             If there is a problem with the xsd used to validate the xml
	 *             information
	 */

	public static String validateXsd(String bulkXml, String xsdPath)
			throws BulkLoadException {

		final StringReader reader = new StringReader(bulkXml);
		try {
			InputSource source = new InputSource(reader);
			return validateXsd(source, xsdPath);

		} finally {

			reader.close();
		}
	}

	/**
	 * Method that verifies a xml using the provided xsd path to check that the
	 * information contained in the xml is correct. This xsd can be as specific
	 * as the developer wants. This way the system can be configured using very
	 * strict validation rules.
	 * 
	 * @param bulkXml
	 *            The xml string
	 * @param xsdPath
	 *            The path where is the xsd against validate the xml received.
	 * 
	 * @return The errors that contains the xml. If the xml does not contain any
	 *         error then this method returns an empty string.
	 * 
	 * @throws BulkLoadException
	 *             If there is a problem with the xsd used to validate the xml
	 *             information
	 */

	public static String validateXsd(byte[] bulkXml, String xsdPath)
			throws BulkLoadException {

		final InputStream reader = new ByteArrayInputStream(bulkXml);
		try {
			InputSource source = new InputSource(reader);
			return validateXsd(source, xsdPath);

		} finally {

			try {
				reader.close();
			} catch (IOException exception) {
				assert false : exception.getMessage();
			}
		}
	}

	/**
	 * Method that verifies a xml using the provided xsd path to check that the
	 * information contained in the xml is correct. This xsd can be as specific
	 * as the developer wants. This way the system can be configured using very
	 * strict validation rules.
	 * 
	 * @param bulkXml
	 *            The xml string
	 * @param xsdPath
	 *            The path where is the xsd against validate the xml received.
	 * 
	 * @return The errors that contains the xml. If the xml does not contain any
	 *         error then this method returns an empty string.
	 * 
	 * @throws BulkLoadException
	 *             If there is a problem with the xsd used to validate the xml
	 *             information
	 */

	public static String validateXsd(InputSource bulkXml, String xsdPath)
			throws BulkLoadException {
		BulkErrorHandler errorHandler = new BulkErrorHandler();
		try {
			Schema compiledSchema = BulkXmlValidator.obtainSchema(xsdPath);
			Verifier verifier = compiledSchema.newVerifier();
			verifier.setErrorHandler(errorHandler);
			verifier.verify(bulkXml);
			return errorHandler.getErrorReport().toString();
		} catch (BulkLoadException e) {
			logger.warn("Error loading schema: " + xsdPath, e);
			throw e;
		} catch (Exception e) {
			StringBuffer errorReport = errorHandler.getErrorReport();
			errorReport.append(e.getMessage());
			return errorReport.toString();
		}
	}

}
