package com.smartmatic.waffle.ecm.client.ui;

import java.util.Map;

import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.ecm.client.ECMClientSideConstants;
import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType;
import com.smartmatic.waffle.ecm.client.worker.MasterUSBFileGenerationWorker;
import com.smartmatic.waffle.ecm.client.worker.MasterUSBPrepareContestsWorker;
import com.smartmatic.waffle.ecm.client.worker.ProcessedContestsPollingWorker;
import com.smartmatic.waffle.ecm.client.worker.data.FileBackup;
import com.smartmatic.waffle.ecm.core.lang.PendriveException.PendriveExceptionInfo;
import com.smartmatic.waffle.ecm.data.transfer.type.USBGenerationType;
import com.smartmatic.waffle.ecm.ui.core.i18n.controller.LanguageController;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Class used to manage the required tasks when performing a Master USB
 * generation, this class will start the workers to execute the server side
 * methods.
 */
public class MasterUSBGenerationController extends USBGenerationController {

	private static SMTTLogger logger = SMTTLogger
			.getLogger(MasterUSBGenerationController.class);

	private MasterUSBPrepareContestsWorker masterUSBPrepareContestsWorker;

	private ProcessedContestsPollingWorker processedContestsPollingWorker;

	private MasterUSBFileGenerationWorker masterUSBFileGenerationWorker;

	public MasterUSBGenerationController(GUIGenerationInterface generationPanel) {
		super(generationPanel);
	}

	@Override
	public void awtOperation(AwtOperationType awtOperationType) {

		switch (awtOperationType) {
		case UPDATE_COMPONENTS:
			updateComponentsOperation();
			break;
		case REFRESH_LABEL:
			refreshLabelOperation();
			break;
		case UPDATE_PROGRESS_BAR:
			updateProgressBarOperation();
			break;
		case SHOW_ERROR_MESSAGE:
			showErrorMessageOperation();
			break;
		default:
			break;
		}
	}

	@Override
	protected USBGenerationType getUSBGenerationType() {
		return USBGenerationType.MASTER_USB;
	}

	@Override
	protected Map<String, FileBackup> getWrittenFiles() {
		return this.masterUSBFileGenerationWorker.getWrittenFiles();
	}

	@Override
	protected boolean isFileGenerationDone() {
		return this.masterUSBFileGenerationWorker.isFileGenerationDone();
	}

	@Override
	protected boolean isContestPreparationWorkerDone() {
		if (this.processedContestsPollingWorker.getProcessedContests() == this.masterUSBPrepareContestsWorker
				.getContestsToProcess()) {
			return true;
		}
		return false;
	}

	@Override
	protected void startPrepareContestsWorker() {

		this.isPreparingContests = true;

		LanguageData language = LanguageController.getInstance()
				.getSelectedLanguage();

		boolean generateUsingProgrammingUnits = ECMClientSideConstants.GENERATE_FILES_USING_PROGRAMMING_UNIT;

		this.masterUSBPrepareContestsWorker = new MasterUSBPrepareContestsWorker(
				getPanel().getSelectedObjects(), language,
				generateUsingProgrammingUnits);
		this.masterUSBPrepareContestsWorker.addListener(this);

		Thread masterUSBPrepareContestsWorkerThread = new Thread(
				this.masterUSBPrepareContestsWorker,
				"MasterUSBPrepareContestsWorkerThread");
		masterUSBPrepareContestsWorkerThread.start();

		startProcessedContestsPollingWorker();
	}

	@Override
	protected void startFileGenerationWorker() {

		this.processedContestsPollingWorker.stopPolling();

		boolean generateUsingProgrammingUnits = ECMClientSideConstants.GENERATE_FILES_USING_PROGRAMMING_UNIT;

		this.masterUSBFileGenerationWorker = new MasterUSBFileGenerationWorker(
				this.mountedPath,
				this.masterUSBPrepareContestsWorker
						.getProgrammingUnitElectionObject(),
				generateUsingProgrammingUnits,
				ECMServiceProvider.getApplicationKey());

		this.masterUSBFileGenerationWorker.addListener(this);

		Thread masterUSBFileGenerationWorkerThread = new Thread(
				this.masterUSBFileGenerationWorker,
				"MasterUSBFileGenerationWorkerThread");
		masterUSBFileGenerationWorkerThread.start();

		this.isGeneratingFiles = true;
	}

	/**
	 * Method used to start the execution of the processed contests polling
	 * worker.
	 */
	private void startProcessedContestsPollingWorker() {
		this.processedContestsPollingWorker = new ProcessedContestsPollingWorker(
				this.mountedPath);

		this.processedContestsPollingWorker.addListener(this);

		Thread processedContestsPollingWorkerThread = new Thread(
				this.processedContestsPollingWorker,
				"ProcessedContestsPollingWorkerThread");
		processedContestsPollingWorkerThread.start();
	}

	/**
	 * Method used to control the logic when an active worker fires the
	 * REFRESH_LABEL operation.
	 */
	private void refreshLabelOperation() {

		if (this.isPreparingContests) {

			getPanel().setPrepareContestIconInPendriveOkLabel();
			getPanel().setPendriveOkLabelVisible(true);

			int processed = this.processedContestsPollingWorker
					.getProcessedContests();
			int toProcess = this.masterUSBPrepareContestsWorker
					.getContestsToProcess();

			getPanel().setLabelStatusProcessingContests(processed, toProcess);
		}
	}

	/**
	 * Method used to control the logic when an active worker fires the
	 * UPDATE_PROGRESS_BAR operation.
	 */
	private void updateProgressBarOperation() {

		if (this.isGeneratingFiles) {

			getPanel().setProgressBarValue(
					this.masterUSBFileGenerationWorker
							.getVotingDevicesProcessed()
							* 100
							/ this.masterUSBFileGenerationWorker
									.getVotingDevicesToProcess());
		}
	}

	/**
	 * Method used to control the logic when an active worker fires the
	 * SHOW_ERROR_MESSAGE operation.
	 */
	private void showErrorMessageOperation() {
		String messageError = "ErrorMessage.GENERIC_ERROR";

		if (this.isCopyingOS) {
			logger.error("Error copying OS");
			PendriveExceptionInfo result = this.getCopyOSWorker().getResult();
			messageError = getCopyOsErrorMessage(result);
		} else if (this.isPreparingContests) {
			logger.error("Error preparing contest");
			messageError = "MasterUsbGenerationPanel.prepare.master_drive.error";
		} else if (this.isGeneratingFiles) {
			logger.error("Error generating files");
			messageError = "MasterUsbGenerationPanel.message.generating.pendrive.failure";
		} else if (this.isVerifyingFiles) {
			logger.error("Error verifiying files");
			messageError = "WritingInPenDriveInfo.GENERATION_INTEGRITY_FAILURE";
		}

		resetOperationStatus();
		getPanel().showErrorMessage(messageError);
	}

	/**
	 * Gets error message key for a CopyOsWorker result
	 * 
	 * @param result
	 *            ExceptionInfo generated by worker
	 * @return The error message key
	 */
	private String getCopyOsErrorMessage(PendriveExceptionInfo result) {

		String messageError = "WritingInPenDriveInfo."
				+ "ERROR_WRITING_OS_IN_PENDRIVE.MASTER.USB";
		if (result != null) {
			if (result == PendriveExceptionInfo.ERROR_EXECUTING_TAR_COMMAND) {
				messageError = "WritingInPenDriveInfo."
						+ "ERROR_WRITING_OS_IN_PENDRIVE.TAR_COMMAND";
			} else if (result == PendriveExceptionInfo.BAD_BLOCKS_DETECTED) {
				messageError = "WritingInPenDriveInfo."
						+ "ERROR_WRITING_OS_IN_PENDRIVE.FS_VERIFICATION";
			}
		}
		return messageError;
	}
}
