package com.smartmatic.waffle.ecm.client.ui;

import java.io.File;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.ecm.client.ECMClientSideConstants;
import com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType;
import com.smartmatic.waffle.ecm.client.ui.common.IAwtOperationListener;
import com.smartmatic.waffle.ecm.client.worker.ConnectedDriveFinderWorker;
import com.smartmatic.waffle.ecm.client.worker.ScreenshotsProcessingWorker;
import com.smartmatic.waffle.ecm.core.pendrive.util.PendriveUtil;
import com.smartmatic.waffle.localpersistence.SAESCommonDestinations;
import com.smartmatic.waffle.localpersistence.SAESLocalPersistenceManager.MountedDrive;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Class used to manage the required tasks when performing a Screenshots
 * processing from the client application, this class will start the workers to
 * execute the server side methods.
 */
public class ScreenshotsProcessingController implements IAwtOperationListener {

	/** Logger. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(ScreenshotsProcessingController.class);

	private ScreenshotGenerationPanel processingPanel;

	private ConnectedDriveFinderWorker connectedDriveFinderWorker;
	private ScreenshotsProcessingWorker screenshotsProcessingWorker;

	private String mountedPath;
	private String partitionName;

	private boolean isFindingDrive;

	private boolean testMode;

	public ScreenshotsProcessingController(
			ScreenshotGenerationPanel processingPanel) {
		this.processingPanel = processingPanel;

		this.testMode = Boolean.getBoolean("test_mode");
	}

	@Override
	public void awtOperation(AwtOperationType awtOperationType) {

		switch (awtOperationType) {
		case UPDATE_COMPONENTS:
			updateComponentsOperation();
			break;
		case REFRESH_LABEL:
			refreshLabelOperation();
			break;
		case UPDATE_PROGRESS_BAR:
			updateProgressBarOperation();
			break;
		case SHOW_ERROR_MESSAGE:
			showErrorMessageOperation();
		default:
			break;
		}

	}

	/**
	 * Method used to start the workers required to the whole screenshots
	 * processing.
	 * 
	 * @param mountedPath
	 *            The path where is mounted the USB drive.
	 * @param partitionName
	 *            The partition name of the USB drive.
	 */
	public void startWorker(String mountedPath, String partitionName) {

		this.mountedPath = mountedPath;
		this.partitionName = partitionName;

		this.startScreenshotsProcessingWorker();
	}

	/**
	 * Method used to start the connected drive finder worker thread.
	 */
	public void startDriveFinderWorker() {

		if (this.testMode) {
			startProcessingTestMode();
		} else {
			Set<String> fileSystemToFilter = new HashSet<String>();
			fileSystemToFilter.add("ext3");

			this.connectedDriveFinderWorker = new ConnectedDriveFinderWorker(
					fileSystemToFilter);
			this.connectedDriveFinderWorker.addListener(this);

			Thread connectedDriveFinderWorkerThread = new Thread(
					this.connectedDriveFinderWorker,
					"ConnectedDriveFinderWorkerThread");
			connectedDriveFinderWorkerThread.start();

			this.isFindingDrive = true;
		}
	}

	/**
	 * Method used to stop the drive finder worker current execution.
	 */
	public void stopDriveFinderWorker() {
		if (this.connectedDriveFinderWorker != null) {
			this.connectedDriveFinderWorker.disableWorker();
		}
		this.isFindingDrive = false;
	}

	/**
	 * Method used to start the screenshots processing worker.
	 */
	public void startScreenshotsProcessingWorker() {
		this.screenshotsProcessingWorker = new ScreenshotsProcessingWorker(
				this.mountedPath, this.processingPanel.getContestList());

		this.screenshotsProcessingWorker.addListener(this);

		Thread screenshotsProcessingWorkerThread = new Thread(
				this.screenshotsProcessingWorker,
				"ScreenshotsProcessingWorkerThread");

		screenshotsProcessingWorkerThread.start();
	}

	private void startProcessingTestMode() {

		String localDirectory = System
				.getProperty("saes_saes_local_copy_directory");

		this.mountedPath = localDirectory + "/ScreenShotFolder";

		this.partitionName = "LocalPartition";

		int result = this.processingPanel
				.showConfirmDialogStartGeneration(this.mountedPath);

		if (result == 0) {

			String folderForScreenshotInDrive = ECMClientSideConstants.PATH_IN_DRIVE_FOR_SCREEN_SHOT;

			File saesFolder = new File(this.mountedPath, "SAES");

			if (checkVotingExperienceFolder(saesFolder.listFiles(),
					folderForScreenshotInDrive)) {

				stopDriveFinderWorker();
				this.processingPanel.preparePanelToProcess();
				startScreenshotsProcessingWorker();

			} else {
				this.processingPanel.pendriveIncompleteInformation();
			}
		}
	}

	/**
	 * Method used to control the logic when an active worker fires the
	 * SHOW_ERROR_MESSAGE operation.
	 */
	private void showErrorMessageOperation() {

		this.processingPanel.showErrorMessage("ErrorMessage.GENERIC_ERROR");
		this.processingPanel.finishSecondProcess(false);
	}

	/**
	 * Method used to control the logic when an active worker fires the
	 * UPDATE_PROGRESS_BAR operation.
	 */
	private void updateProgressBarOperation() {

		this.processingPanel
				.setProgressBarValue(this.screenshotsProcessingWorker
						.getProcessingProgress());

	}

	/**
	 * Method used to control the logic when an active worker fires the
	 * REFRESH_LABEL operation.
	 */
	private void refreshLabelOperation() {

		this.processingPanel
				.setLabelStatusProcess(this.processingPanel
						.getBundle()
						.getString(
								"MasterUsbGenerationPanel.jLabelStatusProccess.proccess.text")
						+ " "
						+ this.screenshotsProcessingWorker.getContestName());
	}

	/**
	 * Method used to control the logic when an active worker fires the
	 * UPDATE_COMPONENTS operation.
	 */
	private void updateComponentsOperation() {

		if (this.isFindingDrive) {
			findingDriveOnUpdate();
		} else if (this.screenshotsProcessingWorker.isProcessingDone()) {

			File destinationFilePath = this.processingPanel
					.chooseFile(this.mountedPath);

			if (destinationFilePath != null) {
				this.processingPanel.setIndeterminateBar(true);

				this.processingPanel.setLabelStatusProcess(this.processingPanel
						.getBundle().getString(
								"ScreenshotGenerationPanel.writing.pdf.file"));

				copyGeneratedZip(destinationFilePath);

				this.processingPanel.setIndeterminateBar(false);
			} else {
				this.processingPanel.finishSecondProcess(false);
			}
		} else {
			this.processingPanel.showLabelStatusProccess(true);
		}
	}

	/**
	 * Performs the logic when an update event has been fired by the running
	 * worker and is finding driving operation.
	 */
	protected void findingDriveOnUpdate() {

		List<SAESCommonDestinations> devices = this.connectedDriveFinderWorker
				.getDevicesAllowed();

		if (ECMClientSideConstants.VALIDATE_DRIVES_CONNECTED_AMOUNT
				&& devices.size() > 1) {
			this.processingPanel.showMessageManyDevicesConnected();
		} else if (devices.size() > 0) {

			MountedDrive mountedDrive = devices.get(0).getDeviceInfo();
			String mountedPath = mountedDrive.mountedPath.getAbsolutePath();
			String partitionName = mountedDrive.partitionName;

			int result = this.processingPanel
					.showConfirmDialogStartGeneration(mountedPath);

			if (result == 0) {
				stopDriveFinderWorker();

				String folderForScreenshotInDrive = ECMClientSideConstants.PATH_IN_DRIVE_FOR_SCREEN_SHOT;

				File screenshotFolder = new File(mountedPath,
						folderForScreenshotInDrive);
				File saesFolder = new File(mountedPath, "SAES");

				this.processingPanel.preparePanelToProcess();

				if (screenshotFolder.isDirectory()) {

					if (checkVotingExperienceFolder(saesFolder.listFiles(),
							folderForScreenshotInDrive)) {

						startWorker(mountedPath, partitionName);

					} else {
						this.processingPanel.pendriveIncompleteInformation();
					}
				} else {
					this.processingPanel
							.showErrorMessage("ScreenshotGenerationPanel.message.process.notADirectory");
				}
			}
		}
	}

	/**
	 * Check if the voting experience folder is contained in an array of files.
	 * 
	 * @param files
	 *            the array of files to check
	 * @param votingExperienceFolderName
	 *            the folder name to check
	 * @return true if exists otherwise false
	 */
	private boolean checkVotingExperienceFolder(File[] files,
			String votingExperienceFolderName) {

		if (files == null || files.length == 0) {
			return false;
		}

		for (File file : files) {
			if (file.isDirectory()) {

				String votingExperiencePath = StringUtils.replaceOnce(
						file.getAbsolutePath(), "/SAES/",
						votingExperienceFolderName);

				if (StringUtils.contains(votingExperiencePath, "images")) {
					continue;
				}

				File votingExperienceFolder = new File(votingExperiencePath);

				if (!votingExperienceFolder.exists()) {
					return false;
				} else {
					File XMLImage = new File(votingExperienceFolder,
							"xml-image.xml");

					if (!XMLImage.exists()) {
						return false;
					}
				}
			}
		}

		return true;
	}

	/**
	 * Copy the generated file to the user selected destination.
	 * 
	 * @param destinationFilePath
	 *            the destination path
	 */
	private void copyGeneratedZip(File destinationFilePath) {

		File tmpScreenshotsFolder = new File(
				ECMClientSideConstants.TEMP_PATH_IN_FILE_SYSTEM_FOR_SCREEN_SHOT);

		File tmpScreenshotsFile = new File(tmpScreenshotsFolder.getParent(),
				ECMClientSideConstants.SCREEN_SHOT_FILE);

		try {

			FileUtils.copyFile(tmpScreenshotsFile, destinationFilePath, true);

			// Sync the Filesystem (in case is a memory)
			PendriveUtil.sync();

			this.processingPanel.finishSecondProcess(true);

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			this.processingPanel
					.showErrorMessage("ExceptionInfo.PROBLEM_COPYING_ZIP_FILE");
			this.processingPanel.finishSecondProcess(false);
		}
	}
}
