/**
 * AsynchronousOperation.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.client.ui.common;

import java.util.ArrayList;
import java.util.List;

import javax.swing.SwingUtilities;

import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Interface for classes that fire AWT asynchronous events. It is useful for
 * processes that need to update GUI components in the main thread, from another
 * thread.
 */
public abstract class AsynchronousOperation implements Runnable {

	/** Logger Static reference. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(AsynchronousOperation.class);

	/**
	 * List of listeners that are subscribing to events which will require GUI
	 * actions (AWT).
	 */
	private List<IAwtOperationListener> listeners = new ArrayList<IAwtOperationListener>();

	/**
	 * Abstract method to perform main operation.
	 */
	protected abstract void performOperation();

	/**
	 * Abstract method to any clean up operation.
	 */
	protected abstract void onCompleted();

	/**
	 * Method to add listeners.
	 * 
	 * @param listener
	 *            IAwtOperationListener reference to be subscribed.
	 */
	public void addListener(IAwtOperationListener listener) {
		this.listeners.add(listener);
	}

	/**
	 * Method to remove listeners.
	 * 
	 * @param listener
	 *            IAwtOperationListener reference to be unsubscribed.
	 */
	public void removeListener(IAwtOperationListener listener) {
		this.listeners.remove(listener);
	}

	/**
	 * Method that waits for all subscribed listeners to have processed an AWT
	 * event which was fired (synchronous notification).
	 * 
	 * @param awtOperationType
	 *            AWT operation type
	 */
	protected void fireAwtEvent(final AwtOperationType awtOperationType) {
		try {
			SwingUtilities.invokeAndWait(new Runnable() {
				@Override
				public void run() {
					for (IAwtOperationListener listener : AsynchronousOperation.this.listeners) {
						listener.awtOperation(awtOperationType);
					}
				}
			});
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * Method that notifies all subscribed listeners that an AWT event was
	 * fired, asynchronously.
	 * 
	 * @param awtOperationType
	 *            AWT operation type
	 */
	protected void fireAndForgetAwtEvent(final AwtOperationType awtOperationType) {
		try {
			SwingUtilities.invokeLater(new Runnable() {
				@Override
				public void run() {
					for (IAwtOperationListener listener : AsynchronousOperation.this.listeners) {
						listener.awtOperation(awtOperationType);
					}
				}
			});
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Runnable#run()
	 */
	@Override
	public void run() {
		try {
			performOperation();
			onCompleted();
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
		}
	}
}
