package com.smartmatic.waffle.ecm.client.ui.login;

import java.lang.reflect.InvocationTargetException;

import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;
import javax.security.auth.login.LoginContext;
import javax.security.auth.login.LoginException;
import javax.swing.SwingUtilities;

import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang.StringUtils;
import org.jboss.security.SecurityContextAssociation;
import org.jboss.security.SimplePrincipal;
import org.jboss.security.auth.callback.SecurityAssociationHandler;

import com.smartmatic.waffle.domain.user.UserData;
import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.service.spec.IECMWorkflowService;
import com.smartmatic.waffle.ecm.ui.core.MachineUtils;
import com.smartmatic.waffle.lang.BaseException;
import com.smartmatic.waffle.localpersistence.SAESCommonDestinations;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Task that will check if the user has access to this application.
 */
class PassphraseUserCheckTask extends
		org.jdesktop.application.Task<Object, Void> {

	/** The unlock success. */
	private boolean unlockSucess;

	/** The user. */
	private UserData user;

	/** The passphraseDialog. */
	protected PassphraseUserInputDialog passphraseDialog;

	/**
	 * For logger purpose.
	 */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(PassphraseUserCheckTask.class);

	/**
	 * Instantiates a new check passphrase user db task.
	 * 
	 * @param app
	 *            the app
	 */
	public PassphraseUserCheckTask(org.jdesktop.application.Application app,
			PassphraseUserInputDialog passphraseDialog) {
		super(app);

		this.passphraseDialog = passphraseDialog;
		this.unlockSucess = false;
	}

	/**
	 * Do in background.
	 * 
	 * @return the object
	 * @inheritDoc
	 */
	@Override
	protected Object doInBackground() {

		try {
			SwingUtilities.invokeAndWait(new Runnable() {

				@Override
				public void run() {
					updateGUIComponents();
				}
			});
		} catch (InterruptedException e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
		} catch (InvocationTargetException e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
		}

		return null;
	}

	/**
	 * Checks if the user exists, has access to the application and has provided
	 * the right password.
	 * 
	 * @param user
	 *            , the user's login to authenticate
	 * @param password
	 *            , the user's password
	 * @return UserData if the user is able to login, otherwise null
	 * @throws PassphraseUserException
	 */
	private UserData authenticateUser(String user, String password)
			throws PassphraseUserException {

		UserData userData = null;

		LoginContext loginContext;
		try {
			loginContext = getLoginContext(user, password);
			loginContext.login();

			configureEcmWorkflowService();

			userData = ECMServiceProvider.getUserById(user);
			if (userData == null) {
				loginContext.logout();
				throw new PassphraseUserException("Invalid user.",
						PassphraseUserException.ErrorType.USER);
			}

			if (!userData.getPassword().equals(password)) {
				loginContext.logout();
				throw new PassphraseUserException("Invalid password.",
						PassphraseUserException.ErrorType.PASSWORD);
			}

			if (userData.getProfile() == null
					|| userData.getProfile().getFunctionalities().isEmpty()) {
				loginContext.logout();
				throw new PassphraseUserException("Invalid user profile.",
						PassphraseUserException.ErrorType.PROFILE);
			}

			logger.info("The user was successfully logged in the server");

		} catch (LoginException e) {
			throw new PassphraseUserException(
					"Login context error during user validation", e);
		} catch (BaseException e) {
			throw new PassphraseUserException(
					"Error retrieving user from server during user login", e);
		}

		return userData;
	}

	/**
	 * Gets login context.
	 * 
	 * @param user
	 * @param password
	 * @return LoginContext
	 * @throws PassphraseUserException
	 */
	private LoginContext getLoginContext(String user, String password)
			throws PassphraseUserException {

		System.setProperty("java.security.auth.login.config",
				"data/conf/auth.conf");
		System.setProperty("org.jboss.security.context.ThreadLocal", "false");
		SecurityAssociationHandler handler = new SecurityAssociationHandler();
		SimplePrincipal userJAAS = new SimplePrincipal(user);
		handler.setSecurityInfo(userJAAS, password.toCharArray());

		LoginContext loginContext;

		try {
			loginContext = new LoginContext("jmx-console", handler);
		} catch (LoginException e) {
			throw new PassphraseUserException(
					"Error configuring login context", e);
		}

		SecurityContextAssociation.setClient();

		return loginContext;
	}

	/**
	 * Configures EcmWorkflowService again, if it is not executed some warning
	 * will be logged on server.
	 * 
	 * @throws PassphraseUserException
	 */
	private void configureEcmWorkflowService() throws PassphraseUserException {

		try {
			InitialContext ctx = new InitialContext();

			Object reference = ctx.lookup("com.smartmatic.waffle.ecm/"
					+ IECMWorkflowService.SERVICE_NAME + "/remote");

			IECMWorkflowService ecmWorkflowService = (IECMWorkflowService) PortableRemoteObject
					.narrow(reference, IECMWorkflowService.class);

			ECMServiceProvider.setEcmWorkflowService(ecmWorkflowService);
		} catch (NamingException e) {
			throw new PassphraseUserException(
					"Error configuring EcmWorkFlowService", e);
		}
	}

	private void updateGUIComponents() {
		/* Increase the wrong password attempts counter */
		this.passphraseDialog.updateCounter();

		String user = this.passphraseDialog.userField.getText();
		String plainPassword = this.passphraseDialog.getEnteredPassword();
		String sha256password = DigestUtils.sha256Hex(plainPassword);

		boolean keepUserValueAfterFailedAttemp = false;

		try {
			this.user = authenticateUser(user, sha256password);
			this.passphraseDialog.okButton.setEnabled(false);
			this.unlockSucess = true;

		} catch (PassphraseUserException e) {
			logger.info("User cannot login, ErrorType: " + e.getErrorType());
			if (PassphraseUserInputDialog.getWrongPasswordCounter() >= 3) {
				logger.error("The user has exceeded the maximum of password attempts permitted");
				this.passphraseDialog.showDialogMaxAttempsPermitted();
				MachineUtils.getInstance().shutdownMachine(
						SAESCommonDestinations.getInstance());
			}

			if (e.getErrorType() == PassphraseUserException.ErrorType.PASSWORD) {
				setErrorMessage("password.invalid");
				keepUserValueAfterFailedAttemp = true;
			} else if (e.getErrorType() == PassphraseUserException.ErrorType.USER) {
				setErrorMessage("user.invalid");
			} else {
				logger.error(e, "Configuration error logging user in server");
				setErrorMessage("passphrase.generic.error");
			}
		} catch (Exception e) {
			logger.error(e, "Unexpected error logging user in server");
			setErrorMessage("passphrase.generic.error");
		} finally {
			this.passphraseDialog.resetButton.setEnabled(true);
		}
		this.passphraseDialog.clearFields(keepUserValueAfterFailedAttemp);
	}

	/**
	 * changes error message to show on GUI
	 * 
	 * @param messageKey
	 */
	private void setErrorMessage(String messageKey) {
		this.passphraseDialog.setErrorMessage(messageKey);
	}

	@Override
	protected void succeeded(Object result) {

		if (this.user != null) {
			this.passphraseDialog.user = this.user;

			if (this.unlockSucess) {
				logger.info("User authorized to login [" + this.user.getLogin()
						+ "] ");
				this.passphraseDialog.setVisible(false);
			}
		}
	}
}
