/**
 * ClientCommon.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.client.util;

import java.io.File;
import java.io.IOException;
import java.util.prefs.Preferences;

import javax.swing.JFileChooser;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.ecm.client.ECMClientSideConstants;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * 
 * Utility class that contains common methods used by the client application.
 * 
 */
public class ClientCommon {

	private final static String PREFERENCE_NODE = "ecm.filechooser.preference";
	public final static String FILE_CHOOSER_DEFAULT = "filechooser";
	public final static String FILE_CHOOSER_ISO = "filechooserISO";

	/**
	 * Gets client preferences from system.
	 * 
	 * @param key
	 *            preference key
	 * @param defaultValue
	 *            default value if key is not found
	 * @return
	 */
	private static String getPreference(String key, String defaultValue) {

		Preferences fileChooserPreferences = Preferences.userRoot().node(
				PREFERENCE_NODE);

		return fileChooserPreferences.get(key, defaultValue);
	}

	/**
	 * Puts or updates a client preferences on the system.
	 * 
	 * @param key
	 *            preference key
	 * @param value
	 *            preference value to save
	 * @return
	 */
	private static void putPreference(String key, String value) {

		Preferences fileChooserPreferences = Preferences.userRoot().node(
				PREFERENCE_NODE);
		fileChooserPreferences.put(key, value);
	}

	/**
	 * Initializes the filechooser to the previously saved location in the
	 * filesystem.
	 * 
	 * @param fileChooser
	 *            the file chooser to set the previous location
	 */
	public static void initFileChooserLocation(JFileChooser fileChooser) {
		initFileChooserLocation(fileChooser, new File(""), FILE_CHOOSER_DEFAULT);
	}

	/**
	 * Initializes the filechooser to the previously saved location in the
	 * filesystem.
	 * 
	 * @param fileChooser
	 *            the file chooser to set the previous location
	 */
	public static void initFileChooserLocation(JFileChooser fileChooser,
			String fileChooserType) {
		initFileChooserLocation(fileChooser, new File(""), fileChooserType);
	}

	/**
	 * Initializes the filechooser to the previously saved location in the
	 * filesystem.
	 * 
	 * @param fileChooser
	 *            the file chooser to set the previous location
	 */
	public static void initFileChooserLocation(JFileChooser fileChooser,
			File defaultFile) {
		initFileChooserLocation(fileChooser, defaultFile, FILE_CHOOSER_DEFAULT);
	}

	/**
	 * Initializes the filechooser to the previously saved location in the
	 * filesystem, and suggests a filename.
	 * 
	 * @param defaultFile
	 *            default filename suggestion to show in the filechooser
	 * @param fileChooser
	 *            the filechooser to configure
	 * @param fileChooserType
	 *            file chooser type
	 */
	private static void initFileChooserLocation(JFileChooser fileChooser,
			File defaultFile, String fileChooserType) {

		try {
			File[] files = { defaultFile };

			fileChooser.setSelectedFile(defaultFile);
			fileChooser.setSelectedFiles(files);

			String preferencesValue = getPreference(fileChooserType,
					ECMClientSideConstants.DEFAULT_FILECHOOSER_PATH);

			File localDirectory = new File(preferencesValue);

			if (!localDirectory.exists()) {
				localDirectory = new File(
						ECMClientSideConstants.DEFAULT_FILECHOOSER_PATH);
			}

			fileChooser.setCurrentDirectory(localDirectory);

		} catch (Exception e) {
			logger.error(e, "Error setting File Chooser directory");
		}
	}

	/**
	 * Sets the last path selected in preferences node
	 * 
	 * @param newPath
	 *            patch selected to set
	 */
	public static void saveFileChooserLocationPreference(String newPath) {
		putPreference(FILE_CHOOSER_DEFAULT, newPath);
	}

	/**
	 * Sets the last patch selected in preferences node
	 * 
	 * @param newPath
	 *            patch selected to set
	 */
	public static void saveFileChooserPreference(String newPath,
			String fileChooserType) {
		putPreference(fileChooserType, newPath);
	}

	/**
	 * Return the total number of pages to process given a number of elements
	 * and a page size.
	 * 
	 * @param totalElements
	 *            total elements to process
	 * @param pageSize
	 *            page size
	 * @return the number of pages
	 */
	public static int getNumberOfPages(int totalElements, int pageSize) {
		return (int) Math.ceil((double) totalElements / (double) pageSize);
	}

	/**
	 * Adds line breaks to an input string by a given maximum line length.
	 * 
	 * @param input
	 *            the string
	 * @param maxLineLength
	 *            maximum line length
	 * @return the modified string
	 */
	public static String addLineBreaks(String input, int maxLineLength) {

		String output = StringUtils.EMPTY;
		int index = 0;

		for (int i = 0; i < input.length() - 1; i++) {
			if (index == maxLineLength) {
				output = output + "\n";
				index = 0;
			}
			index++;
			output = output + String.valueOf(input.charAt(i));
		}

		return output;
	}

	/**
	 * Creates a new <code>directory</code> from the <code>parentPath</code>,
	 * first checks if already exist to delete a create a new one from scratch.
	 * 
	 * @param parentPath
	 *            the path from the new directory should be created
	 * @param directory
	 *            the directory to create
	 * @throws IOException
	 *             signal that an error has occurred creating the directory.
	 */
	public static void createDirectory(String parentPath, String directory)
			throws IOException {

		File directoryToCreate = new File(parentPath, directory);

		if (directoryToCreate.exists()) {
			FileUtils.forceDelete(directoryToCreate);
			directoryToCreate.mkdirs();
		} else {
			directoryToCreate.mkdirs();
		}
	}

	private static SMTTLogger logger = SMTTLogger.getLogger(ClientCommon.class);
}
