package com.smartmatic.waffle.ecm.client.util;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.zip.Deflater;
import java.util.zip.DeflaterOutputStream;

import javax.crypto.SecretKey;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.io.filefilter.DirectoryFileFilter;
import org.apache.commons.io.filefilter.RegexFileFilter;
import org.apache.commons.io.output.ByteArrayOutputStream;
import org.apache.commons.lang.StringUtils;
import org.apache.xml.security.signature.XMLSignature;

import com.smartmatic.waffle.domain.demographics.ElectionData;
import com.smartmatic.waffle.domain.eligibleoption.ImageWrapper;
import com.smartmatic.waffle.domain.general.KeystoreData;
import com.smartmatic.waffle.ecm.client.ECMClientSideConstants;
import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.client.lang.FileGenerationException;
import com.smartmatic.waffle.ecm.client.lang.FileGenerationException.FileGenerationExceptionInfo;
import com.smartmatic.waffle.ecm.client.worker.data.FileData;
import com.smartmatic.waffle.log.SMTTLogger;
import com.smartmatic.waffle.security.asymmetric.AsymmetricKeyManager;
import com.smartmatic.waffle.security.asymmetric.EncryptDecryptUtil;
import com.smartmatic.waffle.security.asymmetric.PBEFileUtil;
import com.smartmatic.waffle.security.asymmetric.PBEFileUtil.CipheringAlgorithm;
import com.smartmatic.waffle.security.lang.CryptographicRuntimeException;
import com.smartmatic.waffle.security.lang.CryptographicStaticException;
import com.smartmatic.waffle.security.sign.SignXMLUtil;
import com.smartmatic.waffle.security.symmetric.SymmetricKeyManager;
import com.thoughtworks.xstream.XStream;

public class MasterFileGenerationUtils {

	private static final String CHARSET_ENCODE = "UTF-8";

	private static final int LINE_LENGHT = 76;

	private static final String LINE_SEPARATOR = "\n";

	/** Static SMTT logger instance. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(MasterFileGenerationUtils.class);

	private static String certToEncryptFilename;

	private static Boolean signXML;

	private static Boolean encryptXML;

	private static Boolean compressElection;

	/**
	 * Method that returns a list of files given a parent directory, this will
	 * lookup inside sub directories recursive.
	 * 
	 * @param parentDirectory
	 *            the directory which the method starts to lookup
	 * @return a list of found files
	 */
	public static List<File> getFilesFromDirectory(File parentDirectory) {

		List<File> files = (List<File>) FileUtils.listFiles(parentDirectory,
				new RegexFileFilter("^(.*?)"), DirectoryFileFilter.DIRECTORY);

		return files;
	}

	/**
	 * Writes a signature file with the given text, will write the file
	 * encrypted if required.
	 * 
	 * @param text
	 *            the text to write in the file
	 * @param signatureXMLFile
	 *            the path of the signature file
	 * @param encrypt
	 *            true if the file should be encrypted
	 * @param protectionKey
	 *            the key to encrypt the file
	 * @return the written file
	 * @throws FileGenerationException
	 *             if an error has occurred writing the signature file
	 */
	public static FileData writeSignatureXML(String text,
			File signatureXMLFile, boolean encrypt, String protectionKey)
			throws FileGenerationException {

		try {

			byte[] plainData = text.getBytes();
			FileData writtenFile = new FileData();
			writtenFile.setFile(signatureXMLFile);

			if (encrypt) {
				byte[] encData = encryptBytes(plainData, protectionKey);
				writtenFile.setData(encData);
				FileUtils.writeByteArrayToFile(signatureXMLFile, encData);
			} else {
				writtenFile.setData(plainData);
				FileUtils.writeByteArrayToFile(signatureXMLFile, plainData);
			}

			return writtenFile;

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new FileGenerationException(e,
					FileGenerationExceptionInfo.WRITING_SIGNATURE_FILE_FAILURE);
		}
	}

	/**
	 * Generates the security certificates using the user authentication
	 * information and writes the files into the media device.
	 * 
	 * @param pkcs12
	 *            the pkcs12 data
	 * @param alias
	 *            Alias of the user writing the certificates
	 * @param password
	 *            Password of the user writing the certificates
	 * @param path
	 *            Path to media device
	 * @return the list which contains the certificates written
	 * @throws FileGenerationException
	 *             Signal that an error occurred writing the certificates
	 * @throws IOException
	 *             signal that an error has occurred closing input stream
	 */
	public static List<FileData> writeCertificates(byte[] pkcs12, String alias,
			String password, String path) throws KeyStoreException,
			NoSuchAlgorithmException, CertificateException,
			FileGenerationException, IOException {

		final InputStream input = new ByteArrayInputStream(pkcs12);
		List<FileData> certificates = new ArrayList<FileData>();
		try {
			KeyStore keyStore = KeyStore
					.getInstance(AsymmetricKeyManager.KEYSTORE_PKCS12);

			keyStore.load(input, password.toCharArray());

			final Certificate[] certificateChain = keyStore
					.getCertificateChain(alias);

			if (certificateChain.length >= 3) {
				final Certificate[] intermediateCAs = Arrays.copyOfRange(
						certificateChain, 1, certificateChain.length - 1);

				certificates.add(writeCertificateFile(intermediateCAs,
						new File(path, "intermediate-cas.crt")));
			}

			if (certificateChain.length >= 2) {
				final Certificate[] rootCA = Arrays.copyOfRange(
						certificateChain, certificateChain.length - 1,
						certificateChain.length);

				final Certificate[] intermediateCAs = Arrays.copyOfRange(
						certificateChain, 0, certificateChain.length - 1);

				certificates.add(writeCertificateFile(rootCA, new File(path,
						"root-ca.crt")));

				certificates.add(writeCertificateFile(intermediateCAs,
						new File(path, "ECM.crt")));

			}
			return certificates;
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new FileGenerationException(
					e,
					FileGenerationExceptionInfo.WRITING_CERTIFICATES_CHAIN_FAILURE);
		} finally {
			input.close();
		}
	}

	/**
	 * Writes the security certificate file into an specific location.
	 * 
	 * @param certificateChain
	 *            the certificate chain
	 * @param pathFile
	 *            Path to where the certificate file will be write into
	 * @throws FileGenerationException
	 *             signal that there was an error writing the certificate file
	 */
	public static FileData writeCertificateFile(
			final Certificate[] certificateChain, File pathFile)
			throws FileGenerationException {

		StringBuffer fileContent = new StringBuffer();

		FileData writtenFile = new FileData();
		try {
			Base64 encoder = new Base64(LINE_LENGHT,
					LINE_SEPARATOR.getBytes(CHARSET_ENCODE));

			for (Certificate certificate : certificateChain) {
				byte[] buf = certificate.getEncoded();

				fileContent.append("-----BEGIN CERTIFICATE-----\n");
				fileContent.append(encoder.encodeToString(buf));
				fileContent.append("-----END CERTIFICATE-----\n");
			}

			writtenFile.setFile(pathFile);
			writtenFile.setData(fileContent.toString().getBytes());

			FileUtils.writeStringToFile(pathFile, fileContent.toString());

			return writtenFile;

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new FileGenerationException(
					e,
					FileGenerationExceptionInfo.WRITING_CERTIFICATE_FILE_FAILURE);
		}
	}

	/**
	 * Get the fingerprint for the certificate.
	 * 
	 * @param certFile
	 *            The certificate file.
	 * 
	 * @return The certificate fingerprint
	 * 
	 * @throws NoSuchAlgorithmException
	 *             Signal that there was an error with the algorithm
	 * @throws FileNotFoundException
	 *             Signal that the file was not found
	 * @throws CertificateException
	 *             signal that there was an error with the certificate
	 */
	public static String getFingerprint(File certFile)
			throws NoSuchAlgorithmException, FileNotFoundException,
			CertificateException {

		MessageDigest md = MessageDigest.getInstance("SHA-256");

		FileInputStream is = new FileInputStream(certFile);
		CertificateFactory certificateFactory = CertificateFactory
				.getInstance("X.509");
		X509Certificate cert = (X509Certificate) certificateFactory
				.generateCertificate(is);

		md.update(cert.getEncoded());

		return new String(Base64.encodeBase64(md.digest()));
	}

	/**
	 * Writes the certificate used to encrypt the voting device files
	 * 
	 * @param votingDevicesFolder
	 *            the voting device folder path
	 * @param publicKey
	 *            the public key to write into the certificate
	 * @throws FileGenerationException
	 *             signal that an error has occurred writing the certificate
	 */
	public static FileData writeCertificateToEncrypt(File votingDevicesFolder,
			ImageWrapper publicKey) throws FileGenerationException {

		FileData writtenFile = new FileData();

		byte[] certificateToEncrypt = null;

		if (publicKey != null) {
			certificateToEncrypt = publicKey.getBinaryData();
		}

		if (certificateToEncrypt != null) {

			File certificateToEncryptFile = new File(votingDevicesFolder,
					getCertToEncryptFilename());

			writtenFile.setFile(certificateToEncryptFile);
			writtenFile.setData(certificateToEncrypt);

			try {
				FileUtils.writeByteArrayToFile(certificateToEncryptFile,
						certificateToEncrypt);

			} catch (IOException e) {
				logger.error(e, StringUtils.defaultString(e.getMessage()));
				throw new FileGenerationException(
						e,
						FileGenerationExceptionInfo.WRITING_CERTIFICATE_FILE_FAILURE);
			}
		}
		return writtenFile;
	}

	/**
	 * Writes a new XML file given an XStream object that defined the rules to
	 * serialize the given <code>object</code>.
	 * 
	 * @param xstreamObject
	 *            the XStream object which contains data to serialize the object
	 *            parameter<code></code>
	 * @param object
	 *            the object to be serialized and put it in the XML file
	 * @param xmlFile
	 *            the file which will contains the serialized data
	 * @param protectionKey
	 *            the protection key to cipher the file if required
	 * @param keyStore
	 *            the keyStore with the security information of the current
	 *            election
	 * @throws FileGenerationException
	 *             signal that an error occurred writing the XML file
	 */
	public static FileData writeXML(XStream xstreamObject, Object object,
			File xmlFile, String protectionKey, KeystoreData keyStore)
			throws FileGenerationException {

		logger.info("Getting election generation properties");

		boolean sign = generateXMLSigned();

		boolean encrypt = generateXMLEncrypted();

		boolean compress = generateElectionCompressed();

		byte[] electionPkcs12 = keyStore.getPkcs12().getBinaryData();

		String alias = keyStore.getAlias();

		String passwordToSign = keyStore.getPassword();

		try {
			return writeXML(xstreamObject, object, xmlFile, electionPkcs12,
					alias, passwordToSign, protectionKey, sign, encrypt,
					compress);
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new FileGenerationException(e,
					FileGenerationExceptionInfo.WRITING_XML_FILE_FAILURE);
		}
	}

	/**
	 * * Writes a new XML file given an XStream object that defined the rules to
	 * serialize the given <code>object</code>.
	 * 
	 * @param xstreamObject
	 *            the XStream object which contains data to serialize the object
	 *            parameter<code></code>
	 * @param object
	 *            the object to be serialized and put it in the XML file
	 * @param xmlFile
	 *            the file which will contains the serialized data
	 * @param pkcs12
	 *            the data which contains security information to sign the file
	 *            if required
	 * @param alias
	 *            the alias of the certificate to sign
	 * @param passwordToSign
	 *            the password of the certificate
	 * @param protectionKey
	 *            the protection key to encrypt the file if required
	 * @param sign
	 *            true if the file must be signed, otherwise false
	 * @param encrypt
	 *            true if the file must be encrypted, otherwise false
	 * @throws FileGenerationException
	 *             signal that an error occurred writing the XML file
	 */
	public static FileData writeXML(XStream xstreamObject, Object object,
			File xmlFile, byte[] pkcs12, String alias, String passwordToSign,
			String protectionKey, boolean sign, boolean encrypt,
			boolean compress) throws FileGenerationException {

		logger.info("Starting XML generation");

		FileData writtenFile = new FileData();

		StringBuilder finalXml = new StringBuilder(
				"<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
		finalXml.append(System.getProperty("line.separator"));

		logger.info("Deserializing object");

		if (xstreamObject != null) {
			finalXml.append((xstreamObject.toXML(object)));
		} else {
			finalXml.append((String) object);
		}

		try {
			if (sign) {

				logger.info("Signing object XML");
				SignXMLUtil.signXML(pkcs12, alias, passwordToSign, finalXml,
						XMLSignature.ALGO_ID_SIGNATURE_ECDSA_SHA256);
				logger.info("XML signed successfully");
			}

			byte[] data = null;

			if (object instanceof ElectionData) {
				if (compress) {
					logger.info("Compressing Election");

					data = getCompressedData(finalXml.toString().getBytes());

					logger.info("Election Compressed successfully");
				} else {
					data = finalXml.toString().getBytes();
				}
			} else {
				data = finalXml.toString().getBytes();
			}

			if (encrypt) {
				logger.info("Encrypting XML data");

				byte[] encData = encryptBytes(data, protectionKey);

				writtenFile.setData(encData);

				logger.info("Copying XML encrypted on disk");

				FileUtils.writeByteArrayToFile(xmlFile, encData);

				logger.info("XML copied successfully on disk");
			} else {
				writtenFile.setData(data);
				FileUtils.writeByteArrayToFile(xmlFile, data);
			}

			writtenFile.setFile(xmlFile);

			return writtenFile;

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new FileGenerationException(e,
					FileGenerationExceptionInfo.WRITING_XML_FILE_FAILURE);
		}
	}

	/**
	 * Writes a file which contains a database dump.
	 * 
	 * @param totalizationFilesPath
	 *            the file path to all the RTS required files
	 * @param masterSymmetricKey
	 *            the master key to encrypt the file
	 * @throws FileGenerationException
	 *             signal that an error occurred writing the database dump file
	 */
	public static FileData writeDatabaseDump(String totalizationFilesPath,
			String masterSymmetricKey) throws FileGenerationException {

		String databaseIp = (Boolean.getBoolean("development_mode") || Boolean
				.getBoolean("stand_alone")) == true ? "127.0.0.1"
				: ECMClientSideConstants.MASTER_DB_IP_ADDRESS;

		try {
			byte[] backupFile = ECMServiceProvider
					.generateBackupDatabase(databaseIp);

			File dumpFile = new File(totalizationFilesPath + File.separator
					+ "dump.ciph");

			FileData writtenFile = new FileData();

			byte encryptedDump[] = encryptBytes(backupFile, masterSymmetricKey);

			writtenFile.setData(encryptedDump);
			writtenFile.setFile(dumpFile);

			FileUtils.writeByteArrayToFile(dumpFile, encryptedDump);

			return writtenFile;

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new FileGenerationException(e,
					FileGenerationExceptionInfo.WRITING_DATABASE_DUMP_FAILURE);
		}
	}

	/**
	 * Writes a Password Based Encryption file using either CBC or ECB modes.
	 * 
	 * @param file
	 *            the file to save the encrypt
	 * @param password
	 *            the password to encrypt the file
	 * @param protectionKey
	 *            the protection key to encrypt
	 * @param seedSalt
	 *            the seed for salt
	 * @param pbKeyFile
	 *            Identify a file that requires HMAC value
	 * @throws FileGenerationException
	 *             Signal that an error has occurred writing the ciphered file.
	 */
	public static FileData writeCipheredFile(File file, String password,
			String protectionKey, String seedSalt, boolean pbKeyFile)
			throws FileGenerationException {

		try {
			byte[] rawData = Base64.decodeBase64(protectionKey);
			byte[] pbFileData = null;

			FileData writtenFile = new FileData();

			if (ECMClientSideConstants.ENCRYPTION_ALGORITHM.equals("CBC")) {

				pbFileData = PBEFileUtil.cipherData(password, rawData,
						seedSalt, CipheringAlgorithm.CBC, pbKeyFile);

			} else if (ECMClientSideConstants.ENCRYPTION_ALGORITHM
					.equals("ECB")) {

				pbFileData = PBEFileUtil.cipherData(password, rawData,
						seedSalt, CipheringAlgorithm.ECB, pbKeyFile);
			}

			writtenFile.setFile(file);
			writtenFile.setData(pbFileData);

			FileUtils.writeByteArrayToFile(file, pbFileData);

			return writtenFile;

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new FileGenerationException(e,
					FileGenerationExceptionInfo.WRITING_CIPHERED_FILE_FAILURE);
		}
	}

	/**
	 * Compress an array of bytes
	 * 
	 * @param data
	 *            the data to compress
	 * @return the compressed data
	 * @throws IOException
	 *             if an error occurred compressing the data
	 */
	public static byte[] getCompressedData(byte[] data) throws IOException {

		ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
		Deflater dfl = new Deflater();
		dfl.setLevel(Integer
				.parseInt(ECMClientSideConstants.LEVEL_COMPRESS_TO_ELECTION));

		DeflaterOutputStream dflOutStream = new DeflaterOutputStream(
				outputStream, dfl);
		dflOutStream.write(data);
		dflOutStream.flush();
		dflOutStream.close();

		data = outputStream.toByteArray();

		return data;
	}

	/**
	 * Setter of the certToEncryptFilename property
	 * 
	 * @param filename
	 *            the file name of the certificate
	 */
	public static void setCertToEncryptFilename(String filename) {
		certToEncryptFilename = filename;
	}

	/**
	 * Setter of the signXML property
	 * 
	 * @param signXMLParam
	 *            true if the XML files must be signed
	 */
	public static void setSignXML(boolean signXMLParam) {
		signXML = signXMLParam;
	}

	/**
	 * Setter of the encryptXML property
	 * 
	 * @param cryptXMLParam
	 *            true if the file must be encrypted
	 */
	public static void setCryptXML(boolean cryptXMLParam) {
		encryptXML = cryptXMLParam;
	}

	/**
	 * Getter of the certToEncryptFilename property
	 * 
	 * @return the certificate filename to encrypt the voting devices data
	 */
	private static String getCertToEncryptFilename() {

		if (certToEncryptFilename != null) {
			return certToEncryptFilename;
		} else {
			return ECMClientSideConstants.CERT_TO_ENCRYPT_FILE_NAME;
		}
	}

	/**
	 * Getter of the signXML property
	 * 
	 * @return true if the XML files must be signed
	 */
	private static boolean generateXMLSigned() {

		if (signXML != null) {
			return signXML.booleanValue();
		} else {
			return ECMClientSideConstants.GENERATE_XML_SIGNED;
		}
	}

	/**
	 * Getter of the encryptXML property
	 * 
	 * @return true if the XML files must be encrypted
	 */
	private static boolean generateXMLEncrypted() {

		if (encryptXML != null) {
			return encryptXML.booleanValue();
		} else {
			return ECMClientSideConstants.GENERATE_XML_ENCRYPTED;
		}
	}

	/**
	 * Getter of the compressElection property
	 * 
	 * @return true if the election file must be compressed
	 */
	private static boolean generateElectionCompressed() {

		if (compressElection != null) {
			return compressElection.booleanValue();
		} else {
			return ECMClientSideConstants.GENERATE_ELECTION_COMPRESS;
		}

	}

	/**
	 * Encrypts an array of bytes and return the bytes encrypted using either
	 * CBC or ECB modes defined in the configuration properties
	 * 
	 * @param data
	 *            the array of bytes to encrypt
	 * @param protectionKey
	 *            the key to encrypt the bytes
	 * @return the array of bytes encrypted
	 * @throws NoSuchAlgorithmException
	 *             if trying to use not supported algorithm
	 * @throws CryptographicRuntimeException
	 *             if an error occurred during the ciphering process
	 * @throws CryptographicStaticException
	 *             if an error occurred during the ciphering process
	 */
	private static byte[] encryptBytes(byte[] data, String protectionKey)
			throws NoSuchAlgorithmException, CryptographicRuntimeException,
			CryptographicStaticException {

		byte[] encData = null;
		byte[] key = Base64.decodeBase64(protectionKey);

		SecretKey secretKey = SymmetricKeyManager.getSymmetricKey(key);

		if (ECMClientSideConstants.ENCRYPTION_ALGORITHM.equals("CBC")) {
			byte[] iVector = SymmetricKeyManager.generateIVectorByte();
			encData = EncryptDecryptUtil.encryptData(data, secretKey, iVector);
		} else if (ECMClientSideConstants.ENCRYPTION_ALGORITHM.equals("ECB")) {
			encData = EncryptDecryptUtil.encryptData(data, secretKey);
		}
		return encData;
	}

	/**
	 * Recursive method to set the files to sign given a parent path
	 * 
	 * @param path
	 *            the parent path to start finding the files
	 * @param fileListToSign
	 *            the files list to sign object reference
	 * @param level
	 *            the maximum level to search
	 * @return an integer that represents the level listed
	 */
	public static int getFilesToSign(String path, List<File> fileListToSign,
			int level) {

		File root = new File(path);
		File[] list = root.listFiles();

		for (File f : list) {
			if (f.isDirectory()
					&& level < ECMClientSideConstants.MAX_LEVEL_TO_SIGN) {
				level = getFilesToSign(f.getAbsolutePath(), fileListToSign,
						++level);
			} else if (!f.isDirectory()) {
				fileListToSign.add(f);
			}
		}

		return --level;
	}

	/**
	 * Recursive method to get the to sign given the main path
	 * 
	 * @param path
	 *            the parent path to start finding the files
	 * @return the files list to sign object reference
	 */
	public static List<File> getFilesToSign(String path) {
		List<File> fileListToSign = new ArrayList<File>();

		File root = new File(path);
		File[] list = root.listFiles();

		for (File f : list) {
			if (f.isDirectory()) {
				fileListToSign.addAll(getFilesToSign(f.getAbsolutePath()));
			} else {
				fileListToSign.add(f);
			}
		}

		return fileListToSign;
	}

	/**
	 * Generate a secure keystore file (.p12) containing the certToEncript.
	 * 
	 * @param path
	 *            Path where the p12 will be located.
	 * @param certToEncrypt
	 *            The certToEncrypt certificate.
	 * @param password
	 *            Password to secure the keystore.
	 * 
	 * @return The FileData wrapper instance with tht certToEncript.p12
	 *         information
	 * @throws FileGenerationException
	 *             if an error occurred during the file generation process
	 * @throws IOException
	 *             if an error occurred during any I/O action
	 */
	public static FileData generateKeyStoreWithCertToEncrypt(String path,
			FileData certToEncrypt, String password)
			throws FileGenerationException, IOException {

		FileData writtenFile = new FileData();

		InputStream keyStoreInputStream = null;

		final InputStream input = new ByteArrayInputStream(
				certToEncrypt.getData());

		try {
			KeyStore keyStore = KeyStore
					.getInstance(AsymmetricKeyManager.KEYSTORE_PKCS12);
			keyStore.load(null, password.toCharArray());

			CertificateFactory certFactory = CertificateFactory
					.getInstance("X.509");
			Certificate cert = null;

			while (input.available() > 0) {
				cert = certFactory.generateCertificate(input);
				keyStore.setCertificateEntry("certToEncrypt", cert);
			}

			keyStore.setCertificateEntry("certToEncrypt", cert);
			keyStore.store(new FileOutputStream(path + "certToEncrypt.p12"),
					password.toCharArray());

			File file = new File(path + "certToEncrypt.p12");
			keyStoreInputStream = new FileInputStream(file);

			writtenFile.setFile(file);
			writtenFile.setData(IOUtils.toByteArray(keyStoreInputStream));

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new FileGenerationException(
					e,
					FileGenerationExceptionInfo.WRITING_CERTIFICATE_FILE_FAILURE);
		} finally {
			input.close();
			keyStoreInputStream.close();
			if (certToEncrypt.getFile() != null) {
				FileUtils.deleteQuietly(certToEncrypt.getFile());
			}
		}
		return writtenFile;
	}
}