/*
 * BulkUploadSmttFilesWorker.java
 * 
 * Copyright (c) 2022 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.client.worker;

import static com.smartmatic.waffle.ecm.service.lang.BulkDataLoadException.BulkDataLoadExceptionOperationExceptionInfo.BADLY_CONFIGURED_AUDIO_VOTING_FOR_REGION;
import static com.smartmatic.waffle.ecm.service.lang.BulkDataLoadException.BulkDataLoadExceptionOperationExceptionInfo.UNEXPECTED_ERROR_EXECUTING_BULK_DATA_LOAD;

import java.util.ArrayList;
import java.util.List;

import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.client.lang.ConnectionException;
import com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation;
import com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType;
import com.smartmatic.waffle.ecm.data.transfer.objects.BulkResultDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.FileToUploadDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.FileToUploadDetailDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.UploadedFileDetailDTO;
import com.smartmatic.waffle.ecm.data.transfer.type.SmttFileType;
import com.smartmatic.waffle.ecm.service.lang.BulkDataLoadException;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Class that executes the main process to execute bulk
 * upload from smartmatic
 * files.
 */
public class BulkUploadSmttFilesWorker extends AsynchronousOperation {

    /**
     * Static reference to log any info or error which occurred executing
     * methods from this class.
     */
    private static SMTTLogger logger =
        SMTTLogger.getLogger(BulkUploadSmttFilesWorker.class);


    /**
     * Why this worker didn't end his work as expected.
     */
    private BulkDataLoadException failure;

    /**
     * Holds the information about the file being processing
     * in the moment to show the process status.
     */
    private String fileBeingProcessed;

    /**
     * Output list with the detail of elements uploaded.
     */
    private List<UploadedFileDetailDTO> fileUploadedDetailList;

    /**
     * Holds the amount of processed files to show the process status.
     */
    private int numberOfProcessedFiles = 0;

    /**
     * Holds the byte array with the resources file.
     */
    private byte[] resourceFileContent;

    /**
     * Input list with files to load.
     */
    private List<FileToUploadDTO> smttFilesToUpĺoadList;


    /**
     * BulkUploadSmttFilesWorker Constructor.
     * 
     * @param smttFilesToUpĺoadList
     *      Input list with files to load
     * @param resourceFileContent
     *      Hold the byte array with the resources file
     */
    public BulkUploadSmttFilesWorker(
        List<FileToUploadDTO> smttFilesToUpĺoadList, byte[] resourceFileContent
    ) {
        this.fileUploadedDetailList = new ArrayList<UploadedFileDetailDTO>();
        this.resourceFileContent    = resourceFileContent;
        this.smttFilesToUpĺoadList  = smttFilesToUpĺoadList;
    }


    /**
     * Returns the reason (i.e. {@code Exception}] this worker didn't end
     * his work as expected.
     *
     * @return
     *     {@code BulkDataLoadException} instance
     */
    public BulkDataLoadException getFailure() {
        return this.failure;
    }


    /**
     * Gets the file being processed.
     * 
     * @return
     *      the file being processed
     */
    public String getFileBeingProcessed() {
        return this.fileBeingProcessed;
    }


    /**
     * Gets the uploaded file's details list.
     * 
     * @return
     *      the file uploaded file's details list
     */
    public List<UploadedFileDetailDTO> getFileUploadedDetailList() {
        return this.fileUploadedDetailList;
    }


    /**
     * Gets the number of processed files.
     * 
     * @return
     *      the number of processed files
     */
    public int getNumberOfProcessedFiles() {
        return this.numberOfProcessedFiles;
    }


    /**
     * Gets the list of smtt files to upĺoad.
     * 
     * @return
     *      the list of smtt files to upĺoad
     */
    public List<FileToUploadDTO> getSmttFilesToUpĺoadList() {
        return this.smttFilesToUpĺoadList;
    }


    /**
     * Method that performs the bulk upload of smartmatic files.
     */
    public void startBulkUploadOperation() {
        try {
            this.executeOperation();

        } catch (BulkDataLoadException ex) {
            logger.error(ex, "Error loading Smtt files");

            this.failure = ex;
            this.numberOfProcessedFiles = 0;

            this.fileUploadedDetailList.clear();
            this.fireAwtEvent(
                (ex.getInfo() == BADLY_CONFIGURED_AUDIO_VOTING_FOR_REGION)
                    ? AwtOperationType.REPORT_ERROR_IN_PROCESS
                    : AwtOperationType.SHOW_ERROR_MESSAGE
            );

        } catch (Exception ex) {
            logger.error(ex, "Unexpected error loading Smtt file");

            this.numberOfProcessedFiles = 0;
            this.failure = new BulkDataLoadException(
                UNEXPECTED_ERROR_EXECUTING_BULK_DATA_LOAD
            );

            this.fileUploadedDetailList.clear();
            this.fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);
         }
    }


    /**
     * {@inheritDoc}
     */
    @Override
    protected void onCompleted() {
    }


    /**
     * {@inheritDoc}
     */
    @Override
    protected void performOperation() {
        this.startBulkUploadOperation();
    }


    /**
     * Validate that the language of the regions with audio voting enabled is
     * configured correctly.
     *
     * @throws BulkDataLoadException
     *      if anything fails
     */
    private void checkConfiguredRegionLaguages() throws BulkDataLoadException {
        ECMServiceProvider.checkConfiguredRegionLaguages();
    }


    /**
     * Method that performs the bulk upload of smartmatic files.
     */
    private void executeOperation()
    throws BulkDataLoadException, ConnectionException {
        final String infoMsg = "Processing %s file";

        BulkResultDTO resultDTO;
        FileToUploadDetailDTO detail;
        UploadedFileDetailDTO uploaded;

        for (FileToUploadDTO fileToUpload : this.smttFilesToUpĺoadList) {
            this.fileBeingProcessed = fileToUpload
                                            .getFileToUploadDetailDTO()
                                            .getFileInfo()
                                            .getName();

            logger.info(String.format(infoMsg, this.fileBeingProcessed));

            detail   = fileToUpload.getFileToUploadDetailDTO();
            uploaded = new UploadedFileDetailDTO();

            uploaded.setFileName(this.fileBeingProcessed);
            uploaded.setSmttFileType( fileToUpload.getSmttFileType() );

            fireAwtEvent(AwtOperationType.UPDATE_COMPONENTS);

            resultDTO = this.uploadSmttFileIntoDataBase(
                this.fileBeingProcessed,
                detail.getFileContent(),
                fileToUpload.getSmttFileType(),
                this.resourceFileContent
            );

            uploaded.setBulkResultDTO(resultDTO);
            this.fileUploadedDetailList.add(uploaded);

            if (resultDTO.getIncorrectCount() > 0) {
                break;
            }
            checkConfiguredRegionLaguages();
            this.numberOfProcessedFiles++;

            fireAwtEvent(AwtOperationType.UPDATE_PROGRESS_BAR);
        }
        fireAwtEvent(AwtOperationType.SHOW_RESULT_MESSAGE);
    }


    /**
     * Method that calls the process in server side to execute bulk upload of
     * smartmatic files
     * 
     * @param fileToUploadName
     *      name of the file to upload
     * @param fileToUploadContent
     *      content of the file to upload
     * @param fileType
     *      type of file
     * @param resourceFileContent
     *      content of the resource file
     * 
     * @return
     *      dto with the result of bulk upload
     * 
     * @throws BulkDataLoadException
     *      signal that an exception occurred executing bulk upload
     * 
     * @throws ConnectionException
     *      Signal that an error occurred with the connection to the server
     */
    private BulkResultDTO uploadSmttFileIntoDataBase(
        String fileToUploadName, byte[] fileToUploadContent,
        SmttFileType fileType, byte[] resourceFileContent
    )
    throws BulkDataLoadException, ConnectionException {
        return ECMServiceProvider.uploadSmttFileIntoDataBase(
            fileToUploadName,
            fileToUploadContent,
            fileType,
            resourceFileContent
        );
    }
}
