/**
 * CopyOSWorker.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.client.worker;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.domain.demographics.ElectionData;
import com.smartmatic.waffle.domain.general.KeystoreData;
import com.smartmatic.waffle.ecm.client.ECMClientSideConstants;
import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.client.lang.ConnectionException;
import com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation;
import com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType;
import com.smartmatic.waffle.ecm.client.ui.common.IAwtOperationListener;
import com.smartmatic.waffle.ecm.client.util.MasterFileGenerationUtils;
import com.smartmatic.waffle.ecm.core.lang.PendriveException;
import com.smartmatic.waffle.ecm.core.lang.PendriveException.PendriveExceptionInfo;
import com.smartmatic.waffle.ecm.core.pendrive.util.PendriveUtil;
import com.smartmatic.waffle.ecm.core.util.SAESCommandUtil;
import com.smartmatic.waffle.ecm.core.util.TimestampUtil;
import com.smartmatic.waffle.ecm.data.transfer.type.USBGenerationType;
import com.smartmatic.waffle.ecm.service.lang.ElectoralOrganizationOperationException;
import com.smartmatic.waffle.localpersistence.SAESCommonDestinations;
import com.smartmatic.waffle.localpersistence.SAESLocalPersistenceManager;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Worker class that copies the President Machine Operating System to a
 * formatted pendrive
 */
public class CopyOSWorker extends AsynchronousOperation implements
		IAwtOperationListener {

	/** Smartmatic Logger. */
	private static SMTTLogger logger = SMTTLogger.getLogger(CopyOSWorker.class);

	/**
	 * Pendrive device path to generate the master USB
	 */
	private String devicePath;

	/**
	 * Master USB generation type
	 */
	private USBGenerationType generationType;

	/**
	 * Used to monitor this copy process.
	 */
	private boolean copyDone;

	/**
	 * Result of process in case of error
	 */
	private PendriveExceptionInfo result;

	/**
	 * Application Key.
	 */
	private String applicationKey;

	/**
	 * Target path for temporal file generation.
	 */
	private String targetPath;

	/**
	 * Instantiates a new CopyOSWorker.
	 * 
	 * @param devicePath
	 *            USB device mounted path
	 * 
	 * @param generationType
	 *            type to generation to perform
	 * 
	 * @param applicationKey
	 *            application key config
	 * 
	 */
	public CopyOSWorker(String devicePath, USBGenerationType generationType,
			String applicationKey) {
		this.devicePath = devicePath;
		this.generationType = generationType;
		this.applicationKey = applicationKey;
	}

	/**
	 * Returns the current copy status.
	 */
	public boolean isCopyDone() {
		return this.copyDone;
	}

	/**
	 * Gets the type of error in case the worker fails.
	 * 
	 * @return PendriveExceptionInfo: ERROR_EXECUTING_TAR_COMMAND,
	 *         BAD_BLOCKS_DETECTED, UNEXPECTED_ERROR_COPYING_OS.
	 */
	public PendriveExceptionInfo getResult() {
		return this.result;
	}

	/**
	 * Copy the President Machine Operating System to formatted pendrive.
	 * 
	 * @throws IOException
	 * @throws PendriveException
	 * @throws ElectoralOrganizationOperationException
	 * @throws ConnectionException
	 */
	protected void copyOSToMasterUSB() throws IOException, PendriveException,
			ConnectionException, ElectoralOrganizationOperationException {

		long start = System.currentTimeMillis();

		logger.info("Gettig election certificates");

		generateCertificatesForActiveElection();

		logger.info("Copying Operating system in pendrive");

		Process process = executeCopyCommand();

		String stdOut = IOUtils.toString(process.getInputStream());

		logger.info("Standard out copy os task:\n" + stdOut);

		String stdErr = parseStandardErrorToString(process);
		int exitValue = process.exitValue();

		if (exitValue == 0) {
			if (!analyzeCopyProcessStdErr(stdErr)) {
				throw new PendriveException(
						PendriveExceptionInfo.UNEXPECTED_ERROR_COPYING_OS,
						"Unexpected error detected during OS install execution."
								+ "\nStandard Error: \n" + stdErr);
			} else {
				logger.info("OS successfully copied in pendrive in "
						+ TimestampUtil.prettyTimeDifference(start,
								System.currentTimeMillis()));
			}
		} else if (exitValue == 2) {
			throw new PendriveException(
					PendriveExceptionInfo.ERROR_EXECUTING_TAR_COMMAND,
					"Error executing tar command during OS install"
							+ "\nstdErr: \n" + stdErr);
		} else if (exitValue == 3) {
			throw new PendriveException(
					PendriveExceptionInfo.BAD_BLOCKS_DETECTED,
					"Error during file system verification" + "\nstdErr: \n"
							+ stdErr);
		} else if (exitValue == 4) {
			throw new PendriveException(
					PendriveExceptionInfo.BAD_BLOCKS_DETECTED,
					"Error during execution of smtt-create-partitions"
							+ "\nstdErr: \n" + stdErr);
		} else {
			throw new PendriveException(
					PendriveExceptionInfo.UNEXPECTED_ERROR_COPYING_OS,
					"Unexpected error during OS install execution."
							+ "\nExit value: " + exitValue + "\nstdErr: \n"
							+ stdErr);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation#
	 * performOperation()
	 */
	@Override
	protected void performOperation() {
		try {
			this.copyDone = false;
			fireAwtEvent(AwtOperationType.UPDATE_COMPONENTS);
			copyOSToMasterUSB();
			this.copyDone = true;
			fireAwtEvent(AwtOperationType.UPDATE_COMPONENTS);

		} catch (PendriveException e) {
			this.result = (PendriveExceptionInfo) e.getExceptionInfo();
			logger.error(e, "Error executing script OS-install");
			this.copyDone = true;
			fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);

		} catch (Exception e) {
			this.result = PendriveExceptionInfo.UNEXPECTED_ERROR_COPYING_OS;
			logger.error(e, "Unexpected error executing script OS-install");
			this.copyDone = true;
			fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);
		}
	}

	/**
	 * Generates application key file and insert it in the filesystem.squashfs.
	 * 
	 * @return
	 * 
	 * @throws IOException
	 * @throws PendriveException
	 * @throws InterruptedException
	 *
	 * @throws Exception
	 *             in case that a problem occurs
	 */
	private String generateApplicationKeyFile() throws IOException {

		File temporaryPathForFileGeneration = new File(
				ECMClientSideConstants.TEMPORARY_PATH_FOR_FILE_GENERATION);

		byte[] compressedApplicationKey = MasterFileGenerationUtils
				.getCompressedData(Base64.decodeBase64(this.applicationKey));

		String base64 = new String(Base64.encodeBase64(
				compressedApplicationKey, false));

		FileUtils.writeStringToFile(
				new File(temporaryPathForFileGeneration.getParent(),
						"swap.prep"), base64);

		return temporaryPathForFileGeneration.getParent();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation#onCompleted
	 * ()
	 */
	@Override
	protected void onCompleted() {

	}

	/**
	 * Executes the command to copy the Operating System in the pendrive
	 * 
	 * @return the process object which can be used to get information about the
	 *         operation
	 * @throws IOException
	 */
	private Process executeCopyCommand() throws IOException {

		if (this.generationType.equals(USBGenerationType.MASTER_USB)) {
			return SAESCommandUtil.execute(getCopyScriptPath(), true,
					this.devicePath, getOSImageFilename(),
					generateApplicationKeyFile());
		} else {
			return SAESCommandUtil.execute(getCopyScriptPath(), true,
					this.devicePath, getOSImageFilename());
		}

	}

	/**
	 * Search the active election and generate the root-ca certificate and the
	 * intermediate certificate.
	 * 
	 * @throws ConnectionException
	 * @throws ElectoralOrganizationOperationException
	 */
	private void generateCertificatesForActiveElection()
			throws ConnectionException, ElectoralOrganizationOperationException {
		this.targetPath = getTargetPath();

		ElectionData activeElection = ECMServiceProvider
				.retrieveActiveElection();

		if (activeElection != null && activeElection.getKeystore() != null) {
			generateCertificates(activeElection);
		}
	}

	/**
	 * Generates the required data to write the voting devices certificates.
	 * 
	 * @param election
	 *            the election to retrieve the security information
	 */
	private void generateCertificates(ElectionData election) {

		logger.info("Generating certificates");

		KeystoreData keystore = election.getKeystore();

		try {
			MasterFileGenerationUtils.writeCertificates(keystore.getPkcs12()
					.getBinaryData(), keystore.getAlias(), keystore
					.getPassword(), this.targetPath);
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * Parses the standard error from process result to string. Needed for
	 * unit-testing.
	 * 
	 * @param process
	 *            the process object to get the operation output
	 * @return error string parsed
	 * @throws IOException
	 *             signals that an I/O exception has occurred.
	 */
	private String parseStandardErrorToString(Process process)
			throws IOException {
		return IOUtils.toString(process.getErrorStream());
	}

	/**
	 * Analyzes the standard error output in the copy process. Will return void
	 * if all is OK, will throw an exception otherwise.
	 * 
	 * @param stdErr
	 * @return true if StdErro is valid.
	 * @throws PendriveException
	 *             signals that the standard error output contains an
	 *             unauthorized error
	 */
	private boolean analyzeCopyProcessStdErr(String stdErr)
			throws PendriveException {

		List<String> authorizedErrorLines = new ArrayList<String>();
		authorizedErrorLines
				.add("Building a new DOS disklabel with disk identifier");
		authorizedErrorLines
				.add("Changes will remain in memory only, until you decide to write them.");
		authorizedErrorLines
				.add("After that, of course, the previous content won't be recoverable.");
		authorizedErrorLines
				.add("Warning: invalid flag 0x0000 of partition table 4 will be corrected by w(rite)");
		authorizedErrorLines.add("mke2fs");
		authorizedErrorLines.add("Installing for x86_64-efi platform.");
		authorizedErrorLines.add("Installing for i386-pc platform.");
		authorizedErrorLines.add("Installation finished");
		authorizedErrorLines
				.add("WARNING: If you have created or modified any DOS 6.x partitions");
		authorizedErrorLines
				.add("WARNING: GPT (GUID Partition Table) detected on");
		authorizedErrorLines.add("blocks");
		authorizedErrorLines.add("% done");
		authorizedErrorLines.add("create_inode");

		return verifyCommandOutput(stdErr, authorizedErrorLines);
	}

	/**
	 * Gets the path of the script needed to copy the Operating System to
	 * generate the Master USB.
	 * 
	 * @return The script path
	 */
	private String getCopyScriptPath() {
		SAESCommonDestinations defaultDestinations = SAESLocalPersistenceManager
				.getInstance().getDefaultDestinations();

		return defaultDestinations.createOsImageScript.getAbsolutePath();
	}

	/**
	 * Gets the filename of the Operating System image to copy in the pendrive.
	 * 
	 * @return the Operating System image filename to copy.
	 */
	private String getOSImageFilename() {
		SAESCommonDestinations defaultDestinations = SAESLocalPersistenceManager
				.getInstance().getDefaultDestinations();

		if (this.generationType.equals(USBGenerationType.SCREENSHOT_USB)) {
			return defaultDestinations.votingDeviceScreenShotImage
					.getAbsolutePath();
		} else {
			return defaultDestinations.votingDeviceImage.getAbsolutePath();
		}
	}

	/**
	 * Verifies if the standard error contains an unauthorized error. Needed for
	 * unit-testing; mocking the call to a static inside a private proved
	 * untractable, so resorted to placing the call inside a stub-able private.
	 * 
	 * @param stdErr
	 *            standard error from process parsed to string
	 * @param authorizedErrorLines
	 *            List of authorized error lines
	 * @return false, if is an unauthorized error was found, true otherwise.
	 */
	private boolean verifyCommandOutput(String stdErr,
			List<String> authorizedErrorLines) {

		boolean messageIsOk = PendriveUtil.isAnExpectedOutput(
				StringUtils.split(stdErr, "\n"), authorizedErrorLines);

		return messageIsOk;
	}

	@Override
	public void awtOperation(AwtOperationType awtOperationType) {
	}

	/**
	 * The path to write the files generated for current invocation, if
	 * necessary to generate the files locally first, then it will return the
	 * temporary path otherwise the mounted path will be the target.
	 * 
	 * @return the target path
	 */
	private String getTargetPath() {

		if (ECMClientSideConstants.WRITE_FILES_IN_DRIVE) {
			return this.devicePath;
		} else {
			return ECMClientSideConstants.TEMPORARY_PATH_FOR_FILE_GENERATION;
		}
	}
}
