package com.smartmatic.waffle.ecm.client.worker;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.io.FileUtils;

import com.smartmatic.waffle.ecm.client.ECMClientSideConstants;
import com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation;
import com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType;
import com.smartmatic.waffle.ecm.client.util.FileVerificationUtils;
import com.smartmatic.waffle.ecm.client.util.MasterFileGenerationUtils;
import com.smartmatic.waffle.ecm.client.worker.data.FileBackup;
import com.smartmatic.waffle.ecm.core.lang.PendriveException;
import com.smartmatic.waffle.ecm.core.lang.PendriveException.PendriveExceptionInfo;
import com.smartmatic.waffle.ecm.data.transfer.type.USBGenerationType;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Worker class used to verify integrity of the files that were generated by the
 * file generation worker.
 *
 */
public class FileIntegrityVerificationWorker extends AsynchronousOperation {

	/** Static SMTT logger instance. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(FileIntegrityVerificationWorker.class);

	private String mountedPath;
	private Map<String, FileBackup> writtenFiles;
	private USBGenerationType generationType;

	private List<String> nonConsistentFiles;
	private List<String> filesHash;

	private boolean fileVerificationDone;

	/**
	 * Drive removed detector
	 */
	private DriveRemovedDetectorWorker driveRemovedDetectorWorker;

	public FileIntegrityVerificationWorker(String mountedPath,
			Map<String, FileBackup> writtenFiles,
			USBGenerationType generationType) {

		this.mountedPath = mountedPath;
		this.writtenFiles = writtenFiles;
		this.generationType = generationType;

		this.nonConsistentFiles = new ArrayList<String>();
		this.filesHash = new ArrayList<String>();
	}

	/**
	 * Method that return true if the verification process is done, otherwise
	 * return false.
	 * 
	 * @return true if the process is done.
	 */
	public boolean isFileVerificationDone() {
		return this.fileVerificationDone;
	}

	/**
	 * Starts the verification process, if a file or files with inconsistencies
	 * are found will call a process to try the recovery of the damage files.
	 * 
	 * @throws IOException
	 *             if an error has occurred verifying the files
	 * @throws PendriveException
	 * @throws InterruptedException
	 */
	private void startVerification() throws IOException, PendriveException {

		logger.info("Starting file verification process");

		this.filesHash.addAll(verifyDirectory("SAES"));

		if (this.generationType.equals(USBGenerationType.MASTER_USB)) {

			String installationFolder = "firstboot/INSTALLATION_FOLDER";
			File dir = new File(this.mountedPath, installationFolder);
			if (dir.exists() && dir.isDirectory()) {
				logger.info("Directory " + dir.getAbsolutePath()
						+ " will be verified");
				this.filesHash.addAll(verifyDirectory(installationFolder));
			}
		}

		if (!this.filesHash.isEmpty()) {

			File digestFile = new File(this.mountedPath, "SAES/sha256sum.txt");

			FileUtils.writeLines(digestFile, this.filesHash);
		}

		if (!this.nonConsistentFiles.isEmpty()) {

			logger.warn("Files with inconsistencies were found. Recovery process will start");
			if (!recoverFiles(this.nonConsistentFiles)) {
				logger.warn("File recovery process finished with un recover files");
			}
		}

	}

	/**
	 * Method that iterates through the list of damage files and try to recover
	 * with the stored data, it will attempt to recover the number of times
	 * defined by the user.
	 * 
	 * @param nonConsistenFiles
	 *            a list of path files to recover
	 * @return true if all files were recovered
	 * 
	 * @throws IOException
	 *             if an error occurred recovering the files
	 * @throws PendriveException
	 * @throws InterruptedException
	 */
	protected boolean recoverFiles(List<String> nonConsistentFiles)
			throws IOException, PendriveException {

		int attempsToFileRecovery = getAttemptsToFileRecover();

		boolean allFilesRecovered = true;
		for (String damageFile : nonConsistentFiles) {

			logger.info("Recovering file: " + damageFile);

			File fileRecovered = new File(this.mountedPath + File.separator
					+ damageFile);

			int i = 1;

			while (i < attempsToFileRecovery) {
				checkDrivePresent();
				FileUtils.deleteQuietly(fileRecovered);

				if (fileRecovered.exists()) {
					logger.info("Not possible recover file: "
							+ fileRecovered.getAbsolutePath());
					break;
				} else {
					FileBackup backup = this.writtenFiles.get(damageFile);
					byte[] recoveredFileData = getFileData(backup.getFile());

					if (recoveredFileData != null) {
						FileUtils.writeByteArrayToFile(fileRecovered,
								recoveredFileData);

						byte[] recoveredHash = backup.getHash();
						byte[] newFileHash = getHash(FileUtils
								.readFileToByteArray(fileRecovered));
						if (Arrays.equals(recoveredHash, newFileHash)) {
							logger.info("File: "
									+ fileRecovered.getAbsolutePath()
									+ " recovered successfuly.");
							break;
						} else {
							logger.warn("File: "
									+ fileRecovered.getAbsolutePath()
									+ " could not be recovered at attempt: "
									+ i);

							if (i == attempsToFileRecovery) {
								allFilesRecovered = false;
							}
						}
					} else {
						logger.warn("No data found for damaged file: "
								+ fileRecovered.getAbsolutePath());
						allFilesRecovered = false;
						break;
					}
				}
				i++;
			}
		}

		return allFilesRecovered;
	}

	/**
	 * Return the digest of a given byte array
	 * 
	 * @param data
	 *            the byte array
	 * @return the data hash
	 */
	protected byte[] getHash(byte[] data) {
		return DigestUtils.sha256(data);
	}

	/**
	 * Method to return the data of the file written by the generation process,
	 * me match will be performed by its relative path.
	 * 
	 * @param filePath
	 *            the file path to search
	 * @return the file data
	 * @throws IOException
	 */
	protected byte[] getFileData(File backupFile) throws IOException {

		if (backupFile == null) {
			return null;
		}

		byte[] data = FileUtils.readFileToByteArray(backupFile);
		return data;
	}

	/**
	 * Method that verifies all the files recursive given the parent directory
	 * 
	 * @param directory
	 *            the directory to verify
	 * @return a list of the files verified
	 * @throws IOException
	 *             if an error occurred verifying the files
	 * @throws PendriveException
	 * @throws InterruptedException
	 * 
	 */
	protected List<String> verifyDirectory(String directory)
			throws IOException, PendriveException {

		logger.info(String.format("Starting directory verification: %s",
				directory));
		List<File> filesToVerify = MasterFileGenerationUtils
				.getFilesFromDirectory(new File(this.mountedPath, directory));

		List<String> filesVerified = new ArrayList<String>();

		logger.info(String.format("Total files to verify: %s",
				filesToVerify.size()));
		for (File file : filesToVerify) {
			checkDrivePresent();

			String relativePath = getRelativePath(file, this.mountedPath);
			byte[] fileData = FileUtils.readFileToByteArray(file);
			byte[] fileHash = getHash(fileData);
			String stringHash = DigestUtils.sha256Hex(fileData);

			FileBackup fileBackup = this.writtenFiles.get(relativePath);

			if (fileBackup != null) {
				byte[] writtenFileHash = fileBackup.getHash();

				if (!Arrays.equals(fileHash, writtenFileHash)) {
					this.nonConsistentFiles.add(relativePath);
				}
			} else {
				this.nonConsistentFiles.add(relativePath);
			}

			filesVerified.add(String.format("./%-120s | %-64s", relativePath,
					stringHash));
		}
		logger.info(String.format("Total verified files: %s",
				filesVerified.size()));
		logger.info(String.format("Total non Consistent Files: %s",
				this.nonConsistentFiles.size()));

		return filesVerified;
	}

	/**
	 * Method that return the relative path of a file given a base path
	 * 
	 * @param file
	 *            the file to get the relative path
	 * @param basePath
	 *            the base path
	 * @return the relative path
	 */
	protected String getRelativePath(File file, String basePath) {

		String path = new File(basePath).toURI().relativize(file.toURI())
				.getPath();

		return path;
	}

	@Override
	protected void performOperation() {

		this.fileVerificationDone = false;

		fireAwtEvent(AwtOperationType.UPDATE_COMPONENTS);

		try {

			if (ECMClientSideConstants.CHECK_GENERATION_FILE_INTEGRITY) {
				startVerification();
			}
			logger.info("File verification process done");
			this.fileVerificationDone = true;
			fireAwtEvent(AwtOperationType.UPDATE_COMPONENTS);

		} catch (Exception e) {
			logger.error(e, "Error during file verification process");
			fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);
		} finally {
			FileVerificationUtils.deleteTemporaryPath();
		}

	}

	@Override
	protected void onCompleted() {
	}

	/**
	 * Return the number of attempts for recover a damage file
	 * 
	 * @return the number of attempts
	 */
	protected int getAttemptsToFileRecover() {
		return ECMClientSideConstants.ATTEMPTS_TO_FILE_RECOVERY;
	}

	private void checkDrivePresent() throws PendriveException {

		this.driveRemovedDetectorWorker = new DriveRemovedDetectorWorker(
				this.mountedPath);
		if (!this.driveRemovedDetectorWorker.isDrivePresent()) {
			throw new PendriveException(
					PendriveExceptionInfo.PENDRIVE_NOT_FOUND,
					"Drive was removed during process");
		}
	}

}
