/**
 * ImageUploadValidatorWorker.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.client.worker;

import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation;
import com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType;
import com.smartmatic.waffle.ecm.core.util.SAESCommandUtil;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Class that execute the main process to validate the image that will be upload
 * in a backend thread.
 */
public class ImagesUploadValidatorWorker extends AsynchronousOperation {

	private static SMTTLogger logger = SMTTLogger
			.getLogger(ImagesUploadValidatorWorker.class);

	/** Contains all file names necessary to consider that the image is valid. */
	private List<String> mandatoryFiles;

	/** tar.gz image to validate. */
	private File srcImage;

	/** Flag to indicate if is a valid image. */
	private boolean isValidImage;

	/** progressBar value. */
	private int progressBarValue;

	/**
	 * Checks if is valid image.
	 * 
	 * @return true, if is a valid image
	 */
	public boolean isValidImage() {
		return this.isValidImage;
	}

	/**
	 * Sets the valid image.
	 * 
	 * @param isValidImage
	 *            the new valid image
	 */
	public void setValidImage(boolean isValidImage) {
		this.isValidImage = isValidImage;
	}

	/**
	 * Gets the progress bar value.
	 * 
	 * @return the progress bar value
	 */
	public int getProgressBarValue() {
		return this.progressBarValue;
	}

	/**
	 * Sets the progress bar value.
	 * 
	 * @param progressBarValue
	 *            the new progress bar value
	 */
	public void setProgressBarValue(int progressBarValue) {
		this.progressBarValue = progressBarValue;
	}

	/**
	 * Instantiates a new ImageUploadValidatorWorker
	 * 
	 * @param requestedFiles
	 *            file names necessary to consider that the image is valid
	 * @param srcImage
	 *            tar.gz image to validate
	 */
	public ImagesUploadValidatorWorker(List<String> mandatoryFiles,
			File srcImage) {
		this.mandatoryFiles = mandatoryFiles;
		this.srcImage = srcImage;

	}

	/**
	 * Validate image.
	 * 
	 * @param requestedFiles
	 *            file names necessary to consider that the image is valid
	 * @param srcImage
	 *            tar.gz image to validate
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	private void validateImage(List<String> mandatoryFiles, File srcImage)
			throws Exception {

		boolean isValidImage = true;

		Process execute = SAESCommandUtil.execute("tar", "-ztf", "\""
				+ srcImage.getAbsolutePath() + "\"");

		String standardError = IOUtils.toString(execute.getErrorStream());

		if (standardError.isEmpty()) {

			String scriptResult = IOUtils.toString(execute.getInputStream());

			List<String> files = Arrays.asList(StringUtils.split(scriptResult,
					"\n"));

			int progress = 0;

			for (String requiredFile : mandatoryFiles) {
				if (!validateFile(requiredFile, files)) {
					isValidImage = false;
					break;
				}
				Thread.sleep(50);
				setProgressBarValue((++progress * 100) / mandatoryFiles.size());
				fireAwtEvent(AwtOperationType.UPDATE_PROGRESS_BAR);
			}

			setProgressBarValue(100);
			fireAwtEvent(AwtOperationType.UPDATE_PROGRESS_BAR);

		} else {
			fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);
			isValidImage = false;
		}

		setValidImage(isValidImage);
	}

	/**
	 * Validate the required files in the OS tar file list.
	 * 
	 * @param filePattern
	 *            The file pattern configured.
	 * @param files
	 *            The files in OS tar.
	 * @return If the file is valid.
	 */
	private boolean validateFile(String filePattern, List<String> files) {
		boolean validFile = false;

		Pattern pattern = Pattern.compile(filePattern);
		Matcher matcher;

		for (String fileName : files) {
			matcher = pattern.matcher(fileName);
			validFile = matcher.matches();

			if (validFile) {
				break;
			}
		}

		logger.debug("Valid required file pattern " + filePattern + " "
				+ validFile);

		return validFile;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation#
	 * performOperation()
	 */
	@Override
	protected void performOperation() {
		try {
			validateImage(this.mandatoryFiles, this.srcImage);
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation#onCompleted
	 * ()
	 */
	@Override
	protected void onCompleted() {
		fireAwtEvent(AwtOperationType.UPDATE_COMPONENTS);
		try {
			Thread.sleep(3000);
		} catch (InterruptedException e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);
		}
		fireAwtEvent(AwtOperationType.DISABLE_COMPONENTS);
	}

}
