package com.smartmatic.waffle.ecm.client.worker;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.domain.BaseDataObject;
import com.smartmatic.waffle.domain.demographics.DistrictData;
import com.smartmatic.waffle.domain.demographics.ElectionData;
import com.smartmatic.waffle.domain.demographics.PollingPlaceData;
import com.smartmatic.waffle.domain.demographics.ProgrammingUnitData;
import com.smartmatic.waffle.domain.demographics.VotingBoothData;
import com.smartmatic.waffle.domain.device.DeviceParameterData;
import com.smartmatic.waffle.domain.device.VotingCentersData;
import com.smartmatic.waffle.domain.device.VotingCentersData.VotingCenter;
import com.smartmatic.waffle.domain.device.VotingDevicesData;
import com.smartmatic.waffle.domain.eligibleoption.PartyData;
import com.smartmatic.waffle.domain.i18n.LanguageCollectionData;
import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.domain.multiLanguage.StringResourceData;
import com.smartmatic.waffle.domain.multiLanguage.StringResourcesData;
import com.smartmatic.waffle.domain.xstream.converters.HibernateCollectionConverter;
import com.smartmatic.waffle.ecm.client.ECMClientSideConstants;
import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.client.lang.ConnectionException;
import com.smartmatic.waffle.ecm.client.lang.FileGenerationException;
import com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation;
import com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType;
import com.smartmatic.waffle.ecm.client.ui.controller.SortedLanguage;
import com.smartmatic.waffle.ecm.client.util.FileVerificationUtils;
import com.smartmatic.waffle.ecm.client.util.MasterFileGenerationUtils;
import com.smartmatic.waffle.ecm.client.worker.data.FileBackup;
import com.smartmatic.waffle.ecm.client.worker.data.FileData;
import com.smartmatic.waffle.ecm.core.lang.PendriveException;
import com.smartmatic.waffle.ecm.core.lang.PendriveException.PendriveExceptionInfo;
import com.smartmatic.waffle.ecm.core.pendrive.util.PendriveUtil;
import com.smartmatic.waffle.ecm.core.util.TimestampUtil;
import com.smartmatic.waffle.ecm.service.lang.ElectoralOrganizationOperationException;
import com.smartmatic.waffle.log.SMTTLogger;
import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.DomDriver;
import com.thoughtworks.xstream.mapper.Mapper;

/**
 * Worker that perform the generation and writes the files for the screenshots
 * USB
 */
public class ScreenshotsFileGenerationWorker extends AsynchronousOperation {

	/** Static SMTT logger instance. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(ScreenshotsFileGenerationWorker.class);


    /**
     * Collection of {@code DeviceParameters} to be exported during
     * the generation of the screenshot memory.
     */
    private List<DeviceParameterData> devParams;

	private boolean fileGenerationDone;

	private String mountedPath;

	private String targetPath;

	/**
	 * Boolean that holds if the generation is being performed in test mode.
	 */
	protected boolean testMode;

	private Map<VotingCenter, ElectionData> votingCenterElectionMap;

	private int votingCentersProcessed = 0;

	private int votingCentersToProcess = 0;

	private Map<String, FileBackup> writtenFilesMap;

	/**
	 * Return the total number of voting devices to process
	 * 
	 * @return the total number of voting devices
	 */
	public int getVotingDevicesToProcess() {
		return this.votingCentersToProcess;
	}

	/**
	 * Return the number of voting devices processed so far
	 * 
	 * @return the number of voting devices processed
	 */
	public int getVotingDevicesProcessed() {
		return this.votingCentersProcessed;
	}

	public ScreenshotsFileGenerationWorker(
        String mountedPath,
        Map<VotingCenter, ElectionData> votingCenterElectionMap,
        List<DeviceParameterData> devParams
    ) {
        this.devParams   = devParams;
        this.mountedPath = mountedPath;
        this.votingCenterElectionMap = votingCenterElectionMap;
        this.writtenFilesMap = new HashMap<String, FileBackup>();
        this.testMode        = Boolean.getBoolean("test_mode");
    }


	/**
	 * Return the set of written files
	 * 
	 * @return the set of written files
	 */
	public Map<String, FileBackup> getWrittenFiles() {
		return this.writtenFilesMap;
	}

	/**
	 * Return the status of the current file generation
	 * 
	 * @return the file generation status
	 */
	public boolean isFileGenerationDone() {
		return this.fileGenerationDone;
	}

	@Override
	protected void performOperation() {

		fireAwtEvent(AwtOperationType.UPDATE_COMPONENTS);
		try {
			FileVerificationUtils.deleteTemporaryPath();
			startFileGeneration();
		} catch (Exception e) {
			FileVerificationUtils.deleteTemporaryPath();
			logger.error(e, "Error on Screenshot file generation");
			fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);
		}
	}

	@Override
	protected void onCompleted() {
		fireAwtEvent(AwtOperationType.UPDATE_COMPONENTS);
	}

	/**
	 * Starts the process of screenshots USB file generation for the selected
	 * contests.
	 * 
	 * @throws FileGenerationException
	 *             signal that an error has occurred writing a file
	 * @throws IOException
	 *             signal that an error has occurred writing a file
	 * @throws ConnectionException
	 *             signal that an error has occurred with the server connection
	 * @throws ElectoralOrganizationOperationException
	 *             an error occurred retrieving the active election
	 * @throws PendriveException
	 */
	protected void startFileGeneration() throws FileGenerationException,
			IOException, ConnectionException,
			ElectoralOrganizationOperationException, PendriveException {

		logger.info("Starting file generation process");

		this.fileGenerationDone = false;

		this.targetPath = getTargetPath();

		ProgrammingUnitData programmingUnit = new ProgrammingUnitData();

		List<VotingCenter> votingCenters = new ArrayList<VotingCenter>();

		this.votingCentersToProcess = this.votingCenterElectionMap.keySet()
				.size();
		this.votingCentersProcessed = 0;
		for (VotingCenter votingCenter : this.votingCenterElectionMap.keySet()) {

			checkPendrive();

			logger.info("Generating files for voting center: "
					+ votingCenter.getFolderName());

			long votingCenterGenerationTime = System.currentTimeMillis();

			programmingUnit.setCustomCode(votingCenter.getFolderName());

			generateXML(
				votingCenter.getFolderName(),
				programmingUnit,
				votingCenter.getVotingDevices(),
				this.votingCenterElectionMap.get(votingCenter),
				this.devParams
			);

			votingCenters.add(votingCenter);

			logger.info("Time generating files for : "
					+ votingCenter.getFolderName()
					+ " "
					+ TimestampUtil.prettyTimeDifference(
							votingCenterGenerationTime,
							System.currentTimeMillis()));
			this.votingCentersProcessed++;
			fireAwtEvent(AwtOperationType.UPDATE_PROGRESS_BAR);
		}

		generateVotingCenters(votingCenters);

		generateDefaultLanguages();

		copyGeneratedFiles();

		mergeSaesFiles();

		this.fileGenerationDone = true;

		logger.info("File generation process done.");

	}

    /**
     * Generates all the required files for a voting device, including XML,
     * resources and protection key files.
     *
     * @param votingDeviceCustomCode
     *      the custom code of the voting device to generate
     * @param programmingUnit
     *      the programming unit that is being generated
     * @param votingDevices
     *      the voting devices list to generate
     * @param activeElection
     *      the active election object
     * @param devParams
     *      device parameters to be exported
     *
     * @throws FileGenerationException
     *      signal that an error has occurred writing a file
     * @throws IOException
     * 
     * @throws PendriveException
     *      if no flashdrive is currently connected
     */
    protected void generateXML(
        String votingDeviceCustomCode,
        ProgrammingUnitData programmingUnit,
        VotingDevicesData votingDevices,
        ElectionData activeElection,
        List<DeviceParameterData> devParams
    ) throws FileGenerationException, IOException, PendriveException {
        File programmingUnitFolder;
        File resourcesFolder;
        File rootSAESDirectory;
        File votingDeviceFolder;
        Set<LanguageData> languages;

        languages = votingDevices.getVotingDeviceData().getLanguages();
        rootSAESDirectory = new File(this.targetPath, "SAES");

        programmingUnitFolder = new File(
            rootSAESDirectory,
            programmingUnit.getCustomCode().replace(" ", "_")
        );
        votingDeviceFolder = new File(
            programmingUnitFolder,
            votingDeviceCustomCode + File.separator + "SAES"
        );

        this.generateElection(activeElection, programmingUnit);
        this.generateDeviceParametersXml(devParams, programmingUnit);

        resourcesFolder = new File(votingDeviceFolder, "StringResources");
        resourcesFolder.mkdirs();

        this.generateStringResources(
            resourcesFolder, languages, activeElection
        );
        this.generateVotingDevice(votingDevices, votingDeviceFolder);
        this.writePartyImages(activeElection.getPartyList());
    }


	/**
	 * Generates the required data to write into the election XML file.
	 * 
	 * @param election
	 *            the election which contains the data
	 * @param programmingUnit
	 *            the programming unit that is being generated
	 * @throws FileGenerationException
	 *             signal that an error has occurred writing a file
	 */
	private void generateElection(ElectionData election,
			ProgrammingUnitData programmingUnit) throws FileGenerationException {

		logger.info("Starting preparation to generate the election file");

		XStream xstreamElection = new XStream(new DomDriver("UTF-8"));

		xstreamElection.processAnnotations(DistrictData.RegionCodeData.class);

		xstreamElection.processAnnotations(PartyData.class);

		xstreamElection.omitField(BaseDataObject.class, "version");

		xstreamElection.setMode(XStream.NO_REFERENCES);

		xstreamElection.addDefaultImplementation(java.util.ArrayList.class,
				org.hibernate.collection.PersistentList.class);
		xstreamElection.addDefaultImplementation(java.util.HashMap.class,
				org.hibernate.collection.PersistentMap.class);
		xstreamElection.addDefaultImplementation(java.util.HashSet.class,
				org.hibernate.collection.PersistentSet.class);

		Mapper mapperElection = xstreamElection.getMapper();

		xstreamElection.registerConverter(new HibernateCollectionConverter(
				mapperElection));

		File rootSAESDirectory = new File(this.targetPath, "SAES");

		File programmingUnitFolder = new File(rootSAESDirectory,
				programmingUnit.getCustomCode().replace(" ", "_"));

		File electionXMLFile = new File(programmingUnitFolder, "election.xml");

		FileData file = MasterFileGenerationUtils.writeXML(xstreamElection,
				election, electionXMLFile, null, null, null, null, false,
				false, false);

		writeLastFileProcessed(file);
	}


    /**
     * Generates the file {@code device-parameters.xml}.
     *
     * @param params
     *      Collection of parameters to be included in the file
     *      {@code device-parameters.xml}
     * @param programmingUnit
     *      the programming unit that is being generated
     *
     * @throws FileGenerationException
     *      signal that an error has occurred writing a file
     */
    private void generateDeviceParametersXml(
        List<DeviceParameterData> params, ProgrammingUnitData progUnit
    ) throws FileGenerationException {
        final String infoMsg = "Generating device-parameters.xml file";

        File xmlFile;         // File being generated
        FileData meta;        // File's meta information
        StringBuilder path;   // Relative path where the file should be created
        XStream xstream;      // XStream

        logger.info(infoMsg);

        path = new StringBuilder()
                       .append("SAES/")
                       .append( progUnit.getCustomCode().replace(" " , "_") )
                       .append("/device-parameters.xml");

        xmlFile = new File(this.targetPath, path.toString());
        xstream = new XStream(new DomDriver("UTF-8"));

        xstream.processAnnotations(DeviceParameterData.class);
        xstream.omitField(BaseDataObject.class, "version");
        xstream.alias("parameters", List.class);

        meta = MasterFileGenerationUtils.writeXML(
            xstream, params, xmlFile,
            null, null, null, null, false, false, false
        );
        writeLastFileProcessed(meta);
    }


	/**
	 * Generate the data required to write voting device XML file.
	 * 
	 * @param votingDevices
	 *            the voting devices data to write
	 * @param votingDeviceFolder
	 *            the folder of the voting device
	 * @throws FileGenerationException
	 *             signal that an error has occurred writing a file
	 */
	private void generateVotingDevice(VotingDevicesData votingDevices,
			File votingDeviceFolder) throws FileGenerationException {

		logger.info("Starting preparation to generate the voting devices file");

		XStream xstreamVotingDevice = new XStream(new DomDriver("UTF-8"));

		xstreamVotingDevice.processAnnotations(VotingDevicesData.class);
		xstreamVotingDevice.processAnnotations(VotingBoothData.class);
		xstreamVotingDevice.processAnnotations(PollingPlaceData.class);

		xstreamVotingDevice.omitField(BaseDataObject.class, "version");

		xstreamVotingDevice.setMode(XStream.NO_REFERENCES);

		xstreamVotingDevice.addDefaultImplementation(java.util.ArrayList.class,
				org.hibernate.collection.PersistentList.class);
		xstreamVotingDevice.addDefaultImplementation(java.util.HashMap.class,
				org.hibernate.collection.PersistentMap.class);
		xstreamVotingDevice.addDefaultImplementation(java.util.HashSet.class,
				org.hibernate.collection.PersistentSet.class);

		Mapper mapperVotingDevice = xstreamVotingDevice.getMapper();

		xstreamVotingDevice.registerConverter(new HibernateCollectionConverter(
				mapperVotingDevice));

		File xmlFile = new File(votingDeviceFolder, "voting-devices.xml");

		FileData file = MasterFileGenerationUtils.writeXML(xstreamVotingDevice,
				votingDevices, xmlFile, null, null, null, null, false, false,
				false);

		writeLastFileProcessed(file);

	}

	/**
	 * Generates the data required to write the string resources file
	 * 
	 * @param resourcesFolder
	 *            the folder where the resources files should be stored
	 * @param languages
	 *            the languages to generate
	 * @throws FileGenerationException
	 *             signal that an error has occurred writing a file
	 * @throws PendriveException
	 */
	private void generateStringResources(File resourcesFolder,
			Set<LanguageData> languages, ElectionData activeElection)
			throws FileGenerationException, PendriveException {

		logger.info("Starting preparation to generate the election resources file");

		XStream xstreamStringResources = new XStream(new DomDriver("UTF-8"));

		xstreamStringResources.processAnnotations(StringResourcesData.class);

		xstreamStringResources.setMode(XStream.NO_REFERENCES);

		for (LanguageData language : languages) {

			checkPendrive();

			List<StringResourceData> list = new ArrayList<StringResourceData>();

			list.addAll(language.getStringResources().values());
			// add the district string resources (from the election level) to
			// the current string resources
			list.addAll(extractDistrictStringResourcesFromElectionDataByLanguage(
					activeElection, language.getCode()));

			StringResourcesData stringResources = new StringResourcesData();

			stringResources
					.setXmlns("http://www.smartmatic.com/Saes/Resources");

			stringResources.setXmlnsXs("http://www.w3.org/2001/XMLSchema");

			stringResources.setStringResources(list);

			File xmlFile = new File(resourcesFolder, "election-resources"
					+ language.getCustomCode() + ".xml");

			FileData file = MasterFileGenerationUtils.writeXML(
					xstreamStringResources, stringResources, xmlFile, null,
					null, null, null, false, false, false);

			writeLastFileProcessed(file);
		}

	}

	/**
	 * Helper method used to locate and extract the {@link StringResourceData
	 * district string resources} (for a particular language) from an
	 * {@link ElectionData election} object
	 * 
	 * @param electionData
	 *            the election object
	 * @param languageCode
	 *            the language's code used to filter the string resources
	 * @return a list with the district's string resources for the indicated
	 *         language
	 */
	private List<StringResourceData> extractDistrictStringResourcesFromElectionDataByLanguage(
			ElectionData electionData, long languageCode) {
		List<StringResourceData> result = new ArrayList<StringResourceData>();
		String districtKeyPrefix = DistrictData.class.getName();
		logger.debug("Locating string resources with key: " + districtKeyPrefix);
		for (LanguageData language : electionData.getLanguages()) {
			if (language.getCode() == languageCode) {
				for (Map.Entry<String, StringResourceData> entry : language
						.getStringResources().entrySet()) {
					if (entry.getKey().toLowerCase()
							.contains(districtKeyPrefix.toLowerCase())) {
						logger.debug("Extracting resource {Key: "
								+ entry.getValue().getKey() + ", Value: "
								+ entry.getValue().getValue() + "}");
						result.add(entry.getValue());
					}
				}
			}
		}
		return result;
	}

	/**
	 * Generates the required data to write into the voting centers file.
	 * 
	 * @param votingCenterList
	 *            the voting centers list to write
	 * @throws FileGenerationException
	 *             signal that an error has occurred writing a file
	 * @throws PendriveException
	 */
	private void generateVotingCenters(List<VotingCenter> votingCenterList)
			throws FileGenerationException, PendriveException {

		logger.info("Starting preparation to generate the voting center file");

		String xmlns = "http://www.smartmatic.com/Saes/Voting/VotingCenters";

		String xmlnsXs = "http://www.w3.org/2001/XMLSchema";

		VotingCentersData votingCenters = new VotingCentersData(xmlnsXs, xmlns,
				votingCenterList);

		XStream xstreamVotingCenter = new XStream(new DomDriver("UTF-8"));

		xstreamVotingCenter.processAnnotations(VotingCentersData.class);

		xstreamVotingCenter.setMode(XStream.NO_REFERENCES);

		File rootSAESDirectory = new File(this.targetPath, "SAES");

		File votingCenterFile = new File(rootSAESDirectory,
				"voting-centers.xml");

		checkPendrive();

		FileData file = MasterFileGenerationUtils.writeXML(xstreamVotingCenter,
				votingCenters, votingCenterFile, null, null, null, null, false,
				false, false);

		writeLastFileProcessed(file);
	}

	/**
	 * Generates the required data to write the default languages file.
	 * 
	 * @throws FileGenerationException
	 *             signal that an error has occurred writing a file
	 * @throws ElectoralOrganizationOperationException
	 * @throws ConnectionException
	 */
	private void generateDefaultLanguages() throws FileGenerationException,
			ConnectionException, ElectoralOrganizationOperationException {

		logger.info("Starting preparation to generate the default languages file");

		String xmlns = "http://www.smartmatic.com/Saes/Voting/Election";

		String xmlnsXs = "http://www.w3.org/2001/XMLSchema";

		XStream xstreamLanguages = new XStream(new DomDriver("UTF-8"));

		xstreamLanguages.omitField(BaseDataObject.class, "version");

		xstreamLanguages.processAnnotations(LanguageCollectionData.class);

		xstreamLanguages.setMode(XStream.NO_REFERENCES);

		ElectionData election = ECMServiceProvider.retrieveActiveElection();

		List<LanguageData> languageList = election.getLanguages();

		Collections.sort(languageList, new SortedLanguage());

		LanguageCollectionData languageCollection = new LanguageCollectionData(
				xmlnsXs, xmlns, languageList);

		File rootSAESDirectory = new File(this.targetPath, "SAES");

		File defaultLanguagesFile = new File(rootSAESDirectory,
				"default-languages.xml");

		FileData file = MasterFileGenerationUtils.writeXML(xstreamLanguages,
				languageCollection, defaultLanguagesFile, null, null, null,
				null, false, false, false);

		writeLastFileProcessed(file);
	}

	/**
	 * Method that return the relative path of a file given a base path
	 * 
	 * @param file
	 *            the file to get the relative path
	 * @param basePath
	 *            the base path
	 * @return the relative path
	 */
	private String getRelativePath(File file, String basePath) {

		String path = new File(basePath).toURI().relativize(file.toURI())
				.getPath();
		return path;
	}

	/**
	 * Check if a file with the same name was processed previously to remove it
	 * and add the last file processed.
	 * 
	 * @param file
	 *            The file data processed.
	 * @throws FileGenerationException
	 */
	private void writeLastFileProcessed(FileData fileData)
			throws FileGenerationException {

		if (fileData == null || fileData.getFile() == null
				|| fileData.getData() == null) {
			logger.warn("It was not possible to persist file for verification process");
		} else {
			byte[] hash = FileVerificationUtils.verifyDiskFile(fileData);
			String relativePath = getRelativePath(fileData.getFile(),
					this.targetPath);
			FileBackup backup = new FileBackup();
			backup.setFile(fileData.getFile());
			backup.setHash(hash);
			this.writtenFilesMap.put(relativePath, backup);
		}
	}

	/**
	 * Writes into the parties folder the set of required party images.
	 * 
	 * @param parties
	 *            the set of parties to write
	 * @return the list of files that were written
	 * @throws IOException
	 *             if an error has occurred writing a party image
	 */
	private List<File> writePartyImages(Set<PartyData> parties)
			throws IOException {

		List<File> imagesFiles = new ArrayList<File>();

		String imagesFolder = this.targetPath
				+ ECMClientSideConstants.PATH_PARTIES_IMAGES_FOLDER;

		File imagePartyDirectory = new File(imagesFolder);

		for (PartyData party : parties) {

			if (party.getImage() != null
					&& party.getImage().getBinaryData() != null) {

				File image = new File(imagePartyDirectory,
						party.getImageFileName());
				FileUtils.writeByteArrayToFile(image, party.getImage()
						.getBinaryData());

				imagesFiles.add(image);

			}
		}

		return imagesFiles;
	}

	/**
	 * The path to write the files generated for current invocation, if
	 * necessary to generate the files locally first, then it will return the
	 * temporary path otherwise the mounted path will be the target.
	 * 
	 * @return the target path
	 */
	private String getTargetPath() {

		if (ECMClientSideConstants.WRITE_FILES_IN_DRIVE) {
			return this.mountedPath;
		} else {
			return ECMClientSideConstants.TEMPORARY_PATH_FOR_FILE_GENERATION;
		}
	}

	/**
	 * Method that copy the generated files from the local device to the USB
	 * drive if the flag to write directly to the USB drive is disabled.
	 * 
	 * @throws IOException
	 *             in an error has occurred copying the files
	 * @throws PendriveException
	 */
	private void copyGeneratedFiles() throws IOException, PendriveException {

		if (!ECMClientSideConstants.WRITE_FILES_IN_DRIVE) {

			checkPendrive();

			File SAESFolder = new File(this.mountedPath, "SAES");
			File SAESLocalFolder = new File(this.mountedPath, "SAESLocal");

			File tmpSAESFolder = new File(
					ECMClientSideConstants.TEMPORARY_PATH_FOR_FILE_GENERATION,
					"SAES");
			File tmpSAESLocalFolder = new File(
					ECMClientSideConstants.TEMPORARY_PATH_FOR_FILE_GENERATION,
					"SAESLocal");

			FileUtils.copyDirectory(SAESLocalFolder, tmpSAESLocalFolder);

			FileUtils.forceDelete(SAESFolder);
			FileUtils.forceDelete(SAESLocalFolder);

			long copyTime = System.currentTimeMillis();

			logger.info("Copying SAES folder");
			FileUtils.copyDirectory(tmpSAESFolder, SAESFolder);

			logger.info("Copying SAESLocal folder");
			FileUtils.copyDirectory(tmpSAESLocalFolder, SAESLocalFolder);

			logger.info("SAES and SAESLocal copied successfully");
			logger.info("Time copying SAES and SAESLocal folder: "
					+ TimestampUtil.prettyTimeDifference(copyTime,
							System.currentTimeMillis()));
		}
	}

	/**
	 * Perform the merge of SAES folders for the current master generation.
	 * 
	 */
	private void mergeSaesFiles() {
		try {
			checkPendrive();

			logger.info("Starting to merge SAESLocal and SAESLocalUnsigned");
			PendriveUtil.mergeSAESLocalSAESLocalUnsigned(this.mountedPath,
					this.testMode);
			logger.info("SAESLocal and SAESLocalUnsigned merged successfully");
		} catch (PendriveException e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * Check if exist the mounted path.
	 * 
	 * @throws PendriveException
	 */
	private void checkPendrive() throws PendriveException {
		File mountedFile = new File(this.mountedPath);

		if (!mountedFile.exists()) {
			logger.error("memory is not mounted on path: " + this.mountedPath);
			throw new PendriveException(
					PendriveExceptionInfo.PENDRIVE_NOT_FOUND);
		}
	}
}
