package com.smartmatic.waffle.ecm.client.worker;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.domain.contest.ContestData;
import com.smartmatic.waffle.domain.demographics.RegionData;
import com.smartmatic.waffle.domain.eligibleoption.EligibleOptionData;
import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.ecm.client.ECMClientSideConstants;
import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.client.lang.ConnectionException;
import com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation;
import com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType;
import com.smartmatic.waffle.ecm.client.util.ClientCommon;
import com.smartmatic.waffle.ecm.data.transfer.objects.EligibleOptionSearchDTO;
import com.smartmatic.waffle.ecm.service.lang.NominationException;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * This class is used to search candidate objects sending the request to the
 * server and updating the status and the data in the client.
 */
public class SearchPartyWorker extends AsynchronousOperation {

	/**
	 * Static reference to log any info or error which occurred executing
	 * methods from this class.
	 */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(SearchPartyWorker.class);

	private List<EligibleOptionSearchDTO> partiesList;

	private String partiesSearchText;
	private List<RegionData> regions;
	private List<ContestData> contests;
	private LanguageData language;
	private int pageNumber;
	private int totalNumberOfPages;

	/**
	 * Creates a new instance of SearchPartyWorker
	 * 
	 * @param partiesSearchText
	 *            the party search text
	 * @param regions
	 *            the regions list filter
	 * @param contests
	 *            the contest list filter
	 * @param language
	 *            the current client language
	 * @param pageNumber
	 *            the page number to fetch data
	 */
	public SearchPartyWorker(String partiesSearchText,
			List<RegionData> regions, List<ContestData> contests,
			LanguageData language, int pageNumber) {
		this.partiesSearchText = partiesSearchText;
		this.regions = regions;
		this.contests = contests;
		this.language = language;
		this.pageNumber = pageNumber;
	}

	/**
	 * Gets the total number of pages from the current search
	 * 
	 * @return the total number of pages
	 */
	public int getTotalNumberOfPages() {
		return this.totalNumberOfPages;
	}

	/**
	 * Gets the parties list from the current search
	 * 
	 * @return the parties list
	 */
	public List<EligibleOptionSearchDTO> getPartiesList() {
		return this.partiesList;
	}

	/**
	 * Sets the total number of pages from the current search
	 * 
	 * @param pageSize
	 *            the page size
	 * @throws NominationException
	 *             a signal that an error occurred retrieving the total number
	 *             of parties
	 * @throws ConnectionException
	 *             a signal that an error occurred with the server connection
	 */
	public void calculateTotalNumberOfPages(int pageSize)
			throws NominationException, ConnectionException {

		int totalNumberOfElements = ECMServiceProvider.retrieveNumberOfParties(
				this.partiesSearchText, this.regions, this.contests);

		this.totalNumberOfPages = ClientCommon.getNumberOfPages(
				totalNumberOfElements, pageSize);

		fireAwtEvent(AwtOperationType.UPDATE_COMPONENTS);
	}

	/**
	 * Gets the list of parties from the current search
	 * 
	 * @param pageSize
	 *            the current page size
	 * @return the list of parties
	 * @throws NominationException
	 *             a signal that an error occurred retrieving the list of
	 *             parties
	 * @throws ConnectionException
	 *             a signal that an error occurred with the server connection
	 */
	public List<EligibleOptionSearchDTO> getParties(int pageSize)
			throws NominationException, ConnectionException {

		List<EligibleOptionData> parties = ECMServiceProvider.getParties(
				this.pageNumber, pageSize, this.partiesSearchText,
				this.regions, this.contests, this.language);

		List<EligibleOptionSearchDTO> partiesList = new ArrayList<EligibleOptionSearchDTO>();

		Object[] values = new Object[1];
		for (EligibleOptionData index : parties) {

			EligibleOptionSearchDTO eligibleOptionDTO = new EligibleOptionSearchDTO();
			eligibleOptionDTO.setEligibleOption(index);
			eligibleOptionDTO.setValues(values);
			partiesList.add(eligibleOptionDTO);
		}

		return partiesList;
	}

	@Override
	protected void performOperation() {
		try {
			fireAwtEvent(AwtOperationType.DISABLE_COMPONENTS);

			int pageSize = Integer
					.valueOf(ECMClientSideConstants.NUMBER_OF_PAGES_FOR_SEARCH_ELIGIBLE_OPTION);

			calculateTotalNumberOfPages(pageSize);
			this.partiesList = getParties(pageSize);

			fireAwtEvent(AwtOperationType.SHOW_RESULT_MESSAGE);

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);
		} finally {
			fireAwtEvent(AwtOperationType.ENABLE_COMPONENTS);
		}
	}

	@Override
	protected void onCompleted() {
		fireAwtEvent(AwtOperationType.ENABLE_COMPONENTS);
	}

}
