/**
 * SecurityComponentsExportWorker.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.client.worker;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.KeyStore;
import java.security.cert.Certificate;
import java.util.ArrayList;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.ecm.client.ECMClientSideConstants;
import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.client.lang.ConnectionException;
import com.smartmatic.waffle.ecm.client.lang.FileGenerationException;
import com.smartmatic.waffle.ecm.client.lang.FileGenerationException.FileGenerationExceptionInfo;
import com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation;
import com.smartmatic.waffle.ecm.client.ui.common.AwtOperationType;
import com.smartmatic.waffle.ecm.client.util.ClientCommon;
import com.smartmatic.waffle.ecm.core.pendrive.util.PendriveUtil;
import com.smartmatic.waffle.ecm.data.transfer.objects.ElectoralAuthoritySecurityComponentsDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.VotingDeviceSecurityComponentsDTO;
import com.smartmatic.waffle.ecm.service.lang.ElectoralOrganizationOperationException;
import com.smartmatic.waffle.ecm.service.lang.ElectoralPhysicalLocationsRegistryException;
import com.smartmatic.waffle.log.SMTTLogger;
import com.smartmatic.waffle.security.asymmetric.AsymmetricKeyManager;

/**
 * Class that executes the main process to export voting devices security
 * components in a backend thread.
 */
public class SecurityComponentsExportWorker extends AsynchronousOperation {

	/** Zip file extension. */
	private static final String ZIP_FILE_EXTENSION = ".zip";

	/** Temporal folder name for security components. */
	private static final String SECURITY_COMPONENTS = "SecurityComponents";

	/** Zip file name for public keys. */
	private static final String PUBLIC_KEYS = "PublicKeys";

	/** Zip file name for machine keys. */
	private static final String MACHINE_KEYS = "MachineKeys";

	/** Temporal folder name for generate security components to export. */
	private static final String SECURITY_COMPONENTS_EXPORT_FOLDER = "SecurityComponentsExportFolder";

	/** Charset encode. */
	private static final String CHARSET_ENCODE = "UTF-8";

	/** Public key line length. */
	private static final int LINE_LENGTH = 65;

	/** File separator to use encoding Base64 byte array. */
	private static final String LINE_SEPARATOR = "\n";

	/** The logger. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(SecurityComponentsExportWorker.class);

	/** Progress bar value. */
	private int progressBarValue;

	/** Destination file. */
	private File destFile;

	/** The maximum value of the progress bar. */
	private int maxProgressBar;

	/** The error bundle key to show to the client. */
	private String errorBundleKey;

	/**
	 * Gets the progress bar value.
	 * 
	 * @return the progress bar value
	 */
	public int getProgressBarValue() {
		return this.progressBarValue;
	}

	/**
	 * Sets the progress bar value.
	 * 
	 * @param progressBarValue
	 *            the new progress bar value
	 */
	public void setProgressBarValue(int progressBarValue) {
		this.progressBarValue = progressBarValue;
	}

	/**
	 * Sets the destination file.
	 * 
	 * @param destFile
	 *            the new destination file
	 */
	public void setDestFile(File destFile) {
		this.destFile = destFile;
	}

	/**
	 * Gets the destination file.
	 * 
	 * @return the destination file
	 */
	public File getDestFile() {
		return this.destFile;
	}

	/**
	 * Gets the maximum value of the progress bar.
	 * 
	 * @return the maximum value of the progress bar
	 */
	public int getMaxProgressBar() {
		return this.maxProgressBar;
	}

	/**
	 * Gets the error message to show to the client.
	 * 
	 * @return the error message to show to the client
	 */
	public String getErrorBundleKey() {
		return this.errorBundleKey;
	}

	/**
	 * Sets the error message to show to the client.
	 * 
	 * @param errorMessage
	 *            the new error message to show to the client
	 */
	public void setErrorBundleKey(String errorMessage) {
		this.errorBundleKey = errorMessage;
	}

	/**
	 * Sets the maximum value of the progress bar.
	 * 
	 * @param maxProgressBar
	 *            the new maximum value of the progress bar
	 */
	public void setMaxProgressBar(int maxProgressBar) {
		this.maxProgressBar = maxProgressBar;
	}

	/**
	 * Generates a folder with all voting devices certificates and a plain file
	 * with the names of the security components.
	 * 
	 * @param votingDeviceSecurityComponentsList
	 *            Voting device security components DTO list
	 * @param exportTemporaryFolder
	 *            temporary export folder
	 * @param exportTemporaryFile
	 *            temporary export file
	 * @param votingDevicesSecurityComponentsLinesList
	 *            Initial file content with the header
	 * @throws ElectoralPhysicalLocationsRegistryException
	 *             Signals that an exception has occurred retrieving some voting
	 *             device security components.
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	public void exportSecurityComponents(
			List<VotingDeviceSecurityComponentsDTO> votingDeviceSecurityComponentsList,
			File exportTemporaryFolder, File exportP12TemporaryFolder,
			File exportPublicKeyTemporaryFolder, File exportTemporaryFile,
			List<String> votingDevicesSecurityComponentsLinesList)
			throws ElectoralPhysicalLocationsRegistryException, IOException,
			FileGenerationException {

		for (VotingDeviceSecurityComponentsDTO indexVotingDevice : votingDeviceSecurityComponentsList) {

			votingDevicesSecurityComponentsLinesList.add(String.format(
					"%-30s | %-40s | %16s | %16s | %13s | %44s",
					indexVotingDevice.getName(),
					indexVotingDevice.getCustomCode(),
					indexVotingDevice.getPresidentKey(),
					indexVotingDevice.getRecounterKey(),
					indexVotingDevice.getNumericKey1(),
					indexVotingDevice.getProtectionKey()));

			File certificateFile = new File(exportP12TemporaryFolder,
					"votingDeviceCertificate_"
							+ indexVotingDevice.getCustomCode() + ".p12");

			FileUtils.writeByteArrayToFile(certificateFile,
					indexVotingDevice.getBinaryCertificate());

			getPublicKey(indexVotingDevice.getBinaryCertificate(),
					indexVotingDevice.getProtectionKey(),
					exportPublicKeyTemporaryFolder, "votingDeviceCertificate_"
							+ indexVotingDevice.getCustomCode() + ".pem");

		}

		FileUtils.writeLines(exportTemporaryFile,
				votingDevicesSecurityComponentsLinesList, true);
	}

	/**
	 * Get the public key from the p12 and write it as a pem file.
	 * 
	 * @param binaryCertificate
	 *            A byte array with the machine key p12 file.
	 * @param protectionKey
	 *            The protection key of the machine key.
	 * @param exportPublicKeyTemporaryFolder
	 *            The folder where will be located the pem with the public key.
	 * @param fileName
	 *            The file name for the pem file.
	 * @throws FileGenerationException
	 */
	private void getPublicKey(byte[] binaryCertificate, String protectionKey,
			File exportPublicKeyTemporaryFolder, String fileName)
			throws FileGenerationException {

		final InputStream input = new ByteArrayInputStream(binaryCertificate);

		try {
			KeyStore keyStore = KeyStore
					.getInstance(AsymmetricKeyManager.KEYSTORE_PKCS12);

			keyStore.load(input, protectionKey.toCharArray());

			final Certificate cert = keyStore.getCertificate("anyAlias");

			File publicKeyFile = new File(exportPublicKeyTemporaryFolder,
					fileName);

			Base64 encoder = new Base64(LINE_LENGTH,
					LINE_SEPARATOR.getBytes(CHARSET_ENCODE));

			byte[] key = cert.getPublicKey().getEncoded();

			FileUtils.writeByteArrayToFile(publicKeyFile, encoder
					.encodeToString(key).toString().getBytes());

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new FileGenerationException(
					e,
					FileGenerationExceptionInfo.WRITING_CERTIFICATES_CHAIN_FAILURE);
		} finally {
			IOUtils.closeQuietly(input);
		}
	}

	/**
	 * Generates a folder with all electoral authorities certificates and a
	 * plain file with the list of its security components.
	 * 
	 * @param electoralAuthoritiesKeysList
	 *            the list of electoral authorities
	 * @param exportTemporaryFolder
	 *            temporary folder before the export
	 * @param exportTemporaryFile
	 *            temporary file with the list of security components
	 * @param electoralAuthoritesSecurityComponentsLinesList
	 *            formatted string with the list of components
	 * @throws IOException
	 *             a signal that an error occurred writing the certificates.
	 */
	public void exportElectoralAuthoritiesSecurityComponents(
			List<ElectoralAuthoritySecurityComponentsDTO> electoralAuthoritiesKeysList,
			File exportTemporaryFolder, File exportTemporaryFile,
			List<String> electoralAuthoritesSecurityComponentsLinesList)
			throws IOException {

		for (ElectoralAuthoritySecurityComponentsDTO electoralAuthorityKeys : electoralAuthoritiesKeysList) {
			electoralAuthoritesSecurityComponentsLinesList.add(String.format(
					"%-15s | %-30s | %-16s | %-50s | %-44s | %-44s",
					electoralAuthorityKeys.getEaUsername(),
					electoralAuthorityKeys.getName(),
					electoralAuthorityKeys.getAuthorityKey(),
					electoralAuthorityKeys.getCustomCode(),
					electoralAuthorityKeys.getSymmetricKey(),
					electoralAuthorityKeys.getProtectionKey()));

			File certificateFile = new File(exportTemporaryFolder,
					"electoralAuthorityCertificate_"
							+ electoralAuthorityKeys.getCustomCode() + ".p12");

			FileUtils.writeByteArrayToFile(certificateFile,
					electoralAuthorityKeys.getBinaryCertificate());

		}

		FileUtils.writeLines(exportTemporaryFile,
				electoralAuthoritesSecurityComponentsLinesList, true);

	}

	/**
	 * Compresses (zip format) the temporary folder containing the security
	 * components.
	 * 
	 * @param exportTemporaryFolder
	 *            temporary export folder
	 * @param zipFile
	 *            zip file reference
	 * @param processedElements
	 *            the processed elements
	 * 
	 * @return Total of processed elements.
	 * 
	 * @throws ElectoralPhysicalLocationsRegistryException
	 *             signal that an exception has occurred compressing voting
	 *             devices security components
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 * @throws InterruptedException
	 *             Signals that an interrupted exception has occurred.
	 */
	public int zipSecurityComponents(File exportTemporaryFolder, File zipFile,
			int processedElements)
			throws ElectoralPhysicalLocationsRegistryException, IOException,
			InterruptedException {

		File[] securityComponentsArray = exportTemporaryFolder.listFiles();

		ZipOutputStream out;

		out = new ZipOutputStream(new FileOutputStream(zipFile));

		try {
			for (File fileToCompress : securityComponentsArray) {
				byte[] bytes = FileUtils.readFileToByteArray(fileToCompress);
				out.putNextEntry(new ZipEntry(fileToCompress.getName()));
				out.write(bytes);
				out.closeEntry();

				setProgressBarValue(++processedElements);
				Thread.sleep(5);
				fireAwtEvent(AwtOperationType.UPDATE_PROGRESS_BAR);
			}
		} finally {
			out.flush();
			out.close();
		}

		return processedElements;
	}

	/**
	 * Copy the zip file to final destination selected by the user.
	 * 
	 * @param srcFile
	 *            an existing zip file to copy
	 * @param destFile
	 *            the file reference to copy
	 * @throws ElectoralPhysicalLocationsRegistryException
	 *             signal that an exception has occurred the copying zip file
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	public void copyZipFileToFinalDestination(File srcFile, File destFile)
			throws ElectoralPhysicalLocationsRegistryException, IOException {

		FileUtils.copyFile(srcFile, destFile);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation#
	 * performOperation()
	 */
	@Override
	protected void performOperation() {

		try {

			fireAwtEvent(AwtOperationType.DISABLE_COMPONENTS);

			int votingDevicesAmount = ECMServiceProvider
					.retrieveVotingDevicesAmount();
			int electoralAuthoritiesAmount = ECMServiceProvider
					.retrieveElectoralAuthoritiesAmount();

			int securityComponentsPerPage = 50;

			int processedElements = 0;

			int numberOfPages = ClientCommon.getNumberOfPages(
					votingDevicesAmount, securityComponentsPerPage);

			int numberOfElectoralAuthoritiesPages = ClientCommon
					.getNumberOfPages(electoralAuthoritiesAmount,
							securityComponentsPerPage);

			if (ECMClientSideConstants.GENERATE_TOTALIZATION_DEVICE) {
				setMaxProgressBar(((votingDevicesAmount + electoralAuthoritiesAmount) * 2) + 2);
			} else {
				setMaxProgressBar((votingDevicesAmount * 3) + 1);
			}
			fireAwtEvent(AwtOperationType.UPDATE_COMPONENTS);

			File saesSaesLocaFolder = new File(
					ECMClientSideConstants.TEMPORARY_PATH_FOR_FILE_GENERATION);

			File exportTemporaryFolder = new File(
					saesSaesLocaFolder.getParentFile(),
					SECURITY_COMPONENTS_EXPORT_FOLDER);

			File exportP12TemporaryFolder = new File(
					saesSaesLocaFolder.getParentFile(), String.format("%s%s%s",
							SECURITY_COMPONENTS_EXPORT_FOLDER, File.separator,
							MACHINE_KEYS));

			File exportPublicKeyTemporaryFolder = new File(
					saesSaesLocaFolder.getParentFile(), String.format("%s%s%s",
							SECURITY_COMPONENTS_EXPORT_FOLDER, File.separator,
							PUBLIC_KEYS));

			File zipFile = createZipFile(exportTemporaryFolder.getParent(),
					String.format("%s%s", SECURITY_COMPONENTS,
							ZIP_FILE_EXTENSION));

			File zipFileP12 = createZipFile(
					exportPublicKeyTemporaryFolder.getParent(),
					String.format("%s%s", MACHINE_KEYS, ZIP_FILE_EXTENSION));

			File zipFilePem = createZipFile(
					exportPublicKeyTemporaryFolder.getParent(),
					String.format("%s%s", PUBLIC_KEYS, ZIP_FILE_EXTENSION));

			removePreviousFolders(exportTemporaryFolder,
					exportP12TemporaryFolder, exportPublicKeyTemporaryFolder);

			File exportTemporaryFile = new File(exportTemporaryFolder,
					"VotingDeviceSecurityComponents.txt");

			List<String> votingDevicesSecurityComponentsLinesList = new ArrayList<String>();

			votingDevicesSecurityComponentsLinesList.add(String.format(
					"%-30s | %-40s | %-16s | %-16s | %-13s | %-44s", "Name",
					"Custom Code", "President Key", "Recounter Key",
					"Numeric Key 1", "Protection Key"));

			for (int pageIndex = 0; pageIndex < numberOfPages; pageIndex++) {
				List<VotingDeviceSecurityComponentsDTO> votingDeviceSecurityComponentsList = ECMServiceProvider
						.retrieveVotingDeviceSecurityComponents(pageIndex,
								securityComponentsPerPage);

				exportSecurityComponents(votingDeviceSecurityComponentsList,
						exportTemporaryFolder, exportP12TemporaryFolder,
						exportPublicKeyTemporaryFolder, exportTemporaryFile,
						votingDevicesSecurityComponentsLinesList);

				votingDevicesSecurityComponentsLinesList.clear();

				processedElements += securityComponentsPerPage;

				setProgressBarValue(processedElements);

				fireAwtEvent(AwtOperationType.UPDATE_PROGRESS_BAR);
			}

			if (ECMClientSideConstants.GENERATE_TOTALIZATION_DEVICE) {

				File exportElectoralAuthoritiesTemporaryFile = new File(
						exportTemporaryFolder,
						"ElectoralAuthoritiesSecurityComponents.txt");

				List<String> electoralAuthoritiesSecurityComponentsLinesList = new ArrayList<String>();

				electoralAuthoritiesSecurityComponentsLinesList
						.add(String
								.format("%-15s | %-30s | %-16s | %-50s | %-44s | %-44s",
										"Wizard Username", "RTS Username",
										"Authority Key", "Custom Code",
										"Symmetric Key", "Protection Key"));

				for (int pageIndex = 0; pageIndex < numberOfElectoralAuthoritiesPages; pageIndex++) {
					List<ElectoralAuthoritySecurityComponentsDTO> electoralAuthoritySecurityComponentsList = ECMServiceProvider
							.retrieveElectoralAuthoritiesSecurityComponents(
									pageIndex, securityComponentsPerPage);

					exportElectoralAuthoritiesSecurityComponents(
							electoralAuthoritySecurityComponentsList,
							exportTemporaryFolder,
							exportElectoralAuthoritiesTemporaryFile,
							electoralAuthoritiesSecurityComponentsLinesList);

					electoralAuthoritiesSecurityComponentsLinesList.clear();

					processedElements += securityComponentsPerPage;

					setProgressBarValue(processedElements);

					fireAwtEvent(AwtOperationType.UPDATE_PROGRESS_BAR);
				}
			}

			processedElements = writeZip(processedElements,
					exportP12TemporaryFolder, zipFileP12);

			processedElements = writeZip(processedElements,
					exportPublicKeyTemporaryFolder, zipFilePem);

			zipSecurityComponents(exportTemporaryFolder, zipFile,
					processedElements);

			copyZipFileToFinalDestination(zipFile, this.destFile);

			PendriveUtil.sync();

			FileUtils.forceDelete(exportTemporaryFolder);
			FileUtils.forceDelete(zipFile);

			fireAwtEvent(AwtOperationType.SHOW_RESULT_MESSAGE);

		} catch (ElectoralPhysicalLocationsRegistryException e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			setErrorBundleKey(e.getBundleKey());
			fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);
		} catch (ElectoralOrganizationOperationException e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			setErrorBundleKey(e.getBundleKey());
			fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);
		} catch (ConnectionException e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			setErrorBundleKey(e.getBundleKey());
			fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			setErrorBundleKey("ecm.generic.error");
			fireAwtEvent(AwtOperationType.SHOW_ERROR_MESSAGE);
		} finally {
			setProgressBarValue(0);
			fireAwtEvent(AwtOperationType.UPDATE_PROGRESS_BAR);
			fireAwtEvent(AwtOperationType.ENABLE_COMPONENTS);
		}

	}

	/**
	 * Add the temporary folder into the zip file.
	 * 
	 * @param processedElements
	 *            Number of processed elements.
	 * @param temporaryFolder
	 *            Temporary folder to zip.
	 * @param zipFile
	 *            Zip file to write.
	 * 
	 * @return Number of total processed elements.
	 * 
	 * @throws ElectoralPhysicalLocationsRegistryException
	 * @throws IOException
	 * @throws InterruptedException
	 */
	private int writeZip(int processedElements, File temporaryFolder,
			File zipFile) throws ElectoralPhysicalLocationsRegistryException,
			IOException, InterruptedException {
		int totalProcessedElements = zipSecurityComponents(temporaryFolder,
				zipFile, processedElements);

		FileUtils.forceDelete(temporaryFolder);
		return totalProcessedElements;
	}

	/**
	 * Remove the given folders from file system.
	 * 
	 * @param exportTemporaryFolder
	 *            The temporary folder for the txt and the certificate folders.
	 * @param exportP12TemporaryFolder
	 *            The temporary folder for the p12 key stores.
	 * @param exportPublicKeyTemporaryFolder
	 *            The temporary folder for the public keys.
	 * @throws IOException
	 */
	private void removePreviousFolders(File exportTemporaryFolder,
			File exportP12TemporaryFolder, File exportPublicKeyTemporaryFolder)
			throws IOException {
		removePreviousFolder(exportTemporaryFolder);
		removePreviousFolder(exportP12TemporaryFolder);
		removePreviousFolder(exportPublicKeyTemporaryFolder);
	}

	/**
	 * Create the zip files.
	 * 
	 * @param temporaryFolder
	 *            The folder where will be located the zip file.
	 * @param zipName
	 *            The name of the zip file.
	 * 
	 * @return The zip file.
	 * @throws IOException
	 */
	private File createZipFile(String temporaryFolder, String zipName)
			throws IOException {
		File zipFile = new File(temporaryFolder, zipName);

		if (zipFile.exists()) {
			FileUtils.forceDelete(zipFile);
		}
		return zipFile;
	}

	/**
	 * Remove the specific folder from the file system.
	 * 
	 * @param temporaryFolder
	 *            The temporary folder to remove.
	 * @throws IOException
	 */
	private void removePreviousFolder(File temporaryFolder) throws IOException {
		if (temporaryFolder.exists()) {
			FileUtils.forceDelete(temporaryFolder);
			temporaryFolder.mkdir();
		} else {
			temporaryFolder.mkdir();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.smartmatic.waffle.ecm.client.ui.common.AsynchronousOperation#onCompleted
	 * ()
	 */
	@Override
	protected void onCompleted() {
	}

}
