#!/bin/bash

# Filename: smtt-write-application-key.sh
# Synopsis: Script that stores the application INIT_KEY,
#			root-ca.crt and ECM.crt into the squashfs
#			file system of the USB.
# Author: Luis Dalo
# Version: 0.0.3
# Last Edited: 6 Dec 2018

set -e

###############
#  VARIABLES  #
###############

HELP="Usage: $0  [PATH_TO_USB]  [PATH_TO_CFG_FILE]\n
where:\n
- PATH_TO_USB: Absolute path of the USB where to install the OS.\n
- PATH_TO_CFG_FILE: Absolute path where the config files are located.\n"

##########################
#  VALIDATING ARGUMENTS  #
##########################

if [ $# -ne 2 ]
then
    echo "E: Wrong usage."
    echo -e ${HELP}
    exit 1
else
    USB_PATH=$1
    PATH_TO_CFG=$2
fi

if [ ! -d ${USB_PATH} ]
then
    echo "E: Not a valid USB Path (${USB_PATH})."
    echo -e ${HELP}
    exit 1
fi

if [ ! -d ${PATH_TO_CFG} ]
then
    echo "E: Not a valid PATH_TO_CFG_FILE (${PATH_TO_CFG})."
    echo -e ${HELP}
    exit 1
fi

if [ "${USB_PATH}" == "/" ]
then
    echo "E: PATH_TO_USB cannot be the root path."
    echo -e ${HELP}
    exit 1
fi

if [ ! -f ${PATH_TO_CFG}/swap.prep ]
then
    echo "E: Path ${PATH_TO_CFG} does not contain a swap.prep file"
    echo -e ${HELP}
    exit 1
fi

if [ ! -f ${PATH_TO_CFG}/drivePathTemp/root-ca.crt ]
then
    echo "E: Path ${PATH_TO_CFG} does not contain a root-ca.crt file"
    echo -e ${HELP}
    exit 1
fi

if [ ! -f ${PATH_TO_CFG}/drivePathTemp/ECM.crt ]
then
    echo "E: Path ${PATH_TO_CFG} does not contain ECM.crt file"
    echo -e ${HELP}
    exit 1
fi

if [ $(find ${USB_PATH}/live/ -type f -name "filesystem*" | wc -l) -eq 0 ]
then
    echo "E: Path ${USB_PATH}/live/ does not contain a squashfs filesystem."
    echo -e ${HELP}
    exit 1
fi

#######################
#  DYNAMIC VARIABLES  #
#######################

SWAP_FILE=${PATH_TO_CFG}/swap.prep
ROOT_CA_CRT_FILE=${PATH_TO_CFG}/drivePathTemp/root-ca.crt
ECM_CRT_FILE=${PATH_TO_CFG}/drivePathTemp/ECM.crt
SQUASHFS_ROOT_DIR=${PATH_TO_CFG}/squashfs-root

###########
#  BEGIN  #
###########

for filesystem in ${USB_PATH}/live/*.squashfs
do
    echo "I: Extracting the $(basename ${filesystem})"
    rm -rf ${SQUASHFS_ROOT_DIR}
    unsquashfs -d ${SQUASHFS_ROOT_DIR} ${filesystem}
    echo "I: Storing the INIT_KEY into the $(basename ${filesystem})"
    cp ${SWAP_FILE} ${SQUASHFS_ROOT_DIR}/etc/election.d/config/
    
    echo "I: Removing mozilla certificate folder into the $(basename ${filesystem})"
    rm -rf ${SQUASHFS_ROOT_DIR}/usr/share/ca-certificates/*
    echo "I: Creating certificate folder into the $(basename ${filesystem})"
    mkdir ${SQUASHFS_ROOT_DIR}/usr/share/ca-certificates/smartmatic/
    echo "I: Storing the root_ca.crt into the $(basename ${filesystem})"
    cp ${ROOT_CA_CRT_FILE} ${SQUASHFS_ROOT_DIR}/usr/share/ca-certificates/smartmatic/
    echo "I: Storing the ECM.crt into the $(basename ${filesystem})"
    cp ${ECM_CRT_FILE} ${SQUASHFS_ROOT_DIR}/usr/share/ca-certificates/smartmatic/
    
    echo "I: Repacking the $(basename ${filesystem})"
    mksquashfs ${SQUASHFS_ROOT_DIR} ${PATH_TO_CFG}/filesystem.squashfs -noappend
    cp ${PATH_TO_CFG}/filesystem.squashfs ${filesystem}
    sync
    echo "I: Done for the $(basename ${filesystem})"
done

echo "I: Done updating swap.prep file on all filesystems."

##############
#  CLEAN UP  #
##############

rm -rf ${SQUASHFS_ROOT_DIR} ${PATH_TO_CFG}/filesystem.squashfs ${SWAP_FILE}

#########
#  END  #
#########
