package com.smartmatic.waffle.ecm.client.util;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.Certificate;
import java.security.cert.CertificateException;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.io.FileUtils;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mockito;
import org.powermock.api.mockito.PowerMockito;
import org.powermock.core.classloader.annotations.PrepareForTest;
import org.powermock.modules.junit4.PowerMockRunner;

import com.smartmatic.waffle.domain.device.VotingCentersData;
import com.smartmatic.waffle.domain.device.VotingCentersData.VotingCenter;
import com.smartmatic.waffle.domain.eligibleoption.ImageWrapper;
import com.smartmatic.waffle.domain.general.KeystoreData;
import com.smartmatic.waffle.ecm.client.ECMClientSideConstants;
import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.client.lang.FileGenerationException;
import com.smartmatic.waffle.security.asymmetric.AsymmetricKeyManager;
import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.DomDriver;

@RunWith(PowerMockRunner.class)
@PrepareForTest({ ECMServiceProvider.class, ECMClientSideConstants.class })
public class MasterFileGenerationUtilsTest {

	@Before
	public void setUp() {

		File parent = new File("test/resources/master_folder");
		parent.mkdirs();

	}

	@Test
	public void testGetFilesFromDirectory() throws IOException {

		File parent = new File("test/resources/master_folder");

		File testFile = new File("test/resources/master_folder/test_file.txt");
		FileUtils.writeStringToFile(testFile, "test");

		List<File> files = MasterFileGenerationUtils
				.getFilesFromDirectory(parent);

		assertTrue(files.contains(new File(
				"test/resources/master_folder/test_file.txt")));
	}

	@Test
	public void testWriteSignatureXMLFile() throws IOException,
			FileGenerationException {

		// arrange
		File signatureFile = new File(
				"test/resources/master_folder/signature.xml");

		String data = "signature data";

		String protectionKey = "protectionKey";

		// act
		MasterFileGenerationUtils.writeSignatureXML(data, signatureFile, false,
				protectionKey);

		// assert
		assertEquals(data, FileUtils.readFileToString(signatureFile));
	}

	@Test
	public void testWriteCertificates() throws IOException, KeyStoreException,
			NoSuchAlgorithmException, CertificateException,
			FileGenerationException {

		// arrange

		File certificate = new File("test/resources/test_cert.p12");

		byte[] pkcs12 = FileUtils.readFileToByteArray(certificate);

		String alias = "test";

		String password = "test";

		String pathToCertificates = "test/resources/master_folder";

		// act
		MasterFileGenerationUtils.writeCertificates(pkcs12, alias, password,
				pathToCertificates);

		File rootCa = new File("test/resources/master_folder/root");

		// assert
		assertNotNull(rootCa);
	}

	@Test
	public void testWriteCertificate() throws IOException, KeyStoreException,
			NoSuchAlgorithmException, CertificateException,
			FileGenerationException {

		// arrange
		KeyStore keyStore = KeyStore
				.getInstance(AsymmetricKeyManager.KEYSTORE_PKCS12);

		File certificateInput = new File("test/resources/test_cert.p12");

		byte[] pkcs12 = FileUtils.readFileToByteArray(certificateInput);

		String alias = "test";

		String password = "test";

		InputStream input = new ByteArrayInputStream(pkcs12);

		keyStore.load(input, password.toCharArray());

		Certificate[] certificateChain = keyStore.getCertificateChain(alias);

		File certFile = new File("test/resources/master_folder/test.crt");

		// act
		MasterFileGenerationUtils.writeCertificateFile(certificateChain,
				certFile);

		// assert
		assertNotNull(FileUtils.readFileToString(certFile));

	}

	@Test
	public void testWriteCertificateToEncrypt() throws IOException,
			FileGenerationException {

		// arrange

		PowerMockito.mockStatic(ECMServiceProvider.class);

		PowerMockito.when(
				ECMServiceProvider.getServerPropertyAsString(Mockito
						.anyString())).thenReturn("1");

		PowerMockito.mockStatic(ECMClientSideConstants.class);

		PowerMockito.when(ECMClientSideConstants.CERT_TO_ENCRYPT_FILE_NAME)
				.thenReturn("cert.crt");

		File certificateInput = new File("test/resources/test_cert.p12");

		byte[] pkcs12 = FileUtils.readFileToByteArray(certificateInput);

		ImageWrapper publicKey = new ImageWrapper();
		publicKey.setBinaryData(pkcs12);

		File outputCert = new File("test/resources/master_folder/");

		MasterFileGenerationUtils.setCertToEncryptFilename("cert.crt");
		// act
		MasterFileGenerationUtils.writeCertificateToEncrypt(outputCert,
				publicKey);

		// assert
		assertNotNull(FileUtils.readFileToByteArray(new File(
				"test/resources/master_folder/cert.crt")));

	}

	@Test
	public void testWriteXMLWithElection() throws Exception {

		// arrange

		File certificateInput = new File("test/resources/test_cert.p12");

		byte[] pkcs12 = FileUtils.readFileToByteArray(certificateInput);

		ImageWrapper publicKey = new ImageWrapper();
		publicKey.setBinaryData(pkcs12);

		KeystoreData keystoreData = new KeystoreData();

		keystoreData.setPkcs12(publicKey);
		keystoreData.setAlias("test");
		keystoreData.setPassword("test");

		String xmlns = "http://www.smartmatic.com/Saes/Voting/VotingCenters";

		String xmlnsXs = "http://www.w3.org/2001/XMLSchema";

		List<VotingCenter> votingCenterList = new ArrayList<VotingCenter>();

		votingCenterList.add(new VotingCentersData.VotingCenter("code", "rnd",
				"folder", "pu"));

		VotingCentersData votingCenters = new VotingCentersData(xmlnsXs, xmlns,
				votingCenterList);

		XStream xstreamVotingCenter = new XStream(new DomDriver("UTF-8"));

		xstreamVotingCenter.processAnnotations(VotingCentersData.class);

		xstreamVotingCenter.setMode(XStream.NO_REFERENCES);

		File xmlFile = new File(
				"test/resources/master_folder/voting-centers.xml");

		MasterFileGenerationUtils.writeXML(xstreamVotingCenter, votingCenters,
				xmlFile, "protectionKey", keystoreData);

		// assert
		assertNotNull(xmlFile);
	}

	@Test
	public void testWriteXML() throws Exception {

		// arrange

		File certificateInput = new File("test/resources/test_cert.p12");

		byte[] pkcs12 = FileUtils.readFileToByteArray(certificateInput);

		String xmlns = "http://www.smartmatic.com/Saes/Voting/VotingCenters";

		String xmlnsXs = "http://www.w3.org/2001/XMLSchema";

		List<VotingCenter> votingCenterList = new ArrayList<VotingCenter>();

		votingCenterList.add(new VotingCentersData.VotingCenter("code", "rnd",
				"folder", "pu"));

		VotingCentersData votingCenters = new VotingCentersData(xmlnsXs, xmlns,
				votingCenterList);

		XStream xstreamVotingCenter = new XStream(new DomDriver("UTF-8"));

		xstreamVotingCenter.processAnnotations(VotingCentersData.class);

		xstreamVotingCenter.setMode(XStream.NO_REFERENCES);

		File xmlFile = new File(
				"test/resources/master_folder/voting-centers.xml");

		MasterFileGenerationUtils.writeXML(xstreamVotingCenter, votingCenters,
				xmlFile, pkcs12, "alias", "password", "protectionKey", false,
				false, false);

		// assert
		assertNotNull(xmlFile);
	}

	@After
	public void tearDown() {
		FileUtils.deleteQuietly(new File("test/resources/master_folder"));
	}
}
