package com.smartmatic.waffle.ecm.client.worker;

import static org.junit.Assert.assertEquals;

import java.io.File;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.io.FileUtils;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mockito;
import org.powermock.api.mockito.PowerMockito;
import org.powermock.core.classloader.annotations.PrepareForTest;
import org.powermock.modules.junit4.PowerMockRunner;

import com.smartmatic.waffle.domain.contest.ContestData;
import com.smartmatic.waffle.domain.contest.ContestGroupData;
import com.smartmatic.waffle.domain.demographics.ElectionData;
import com.smartmatic.waffle.domain.demographics.RegionData;
import com.smartmatic.waffle.domain.device.VotingCentersData.VotingCenter;
import com.smartmatic.waffle.domain.device.VotingDeviceData;
import com.smartmatic.waffle.domain.device.VotingDevicesData;
import com.smartmatic.waffle.domain.i18n.ContestLanguagesPairData;
import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.domain.i18n.LanguageForVotingDeviceData;
import com.smartmatic.waffle.ecm.client.ECMClientSideConstants;
import com.smartmatic.waffle.ecm.client.ECMServiceProvider;
import com.smartmatic.waffle.ecm.client.lang.ConnectionException;
import com.smartmatic.waffle.ecm.core.lang.PendriveException;
import com.smartmatic.waffle.ecm.data.transfer.objects.ElectionXmlComponentsDTO;
import com.smartmatic.waffle.ecm.data.transfer.objects.VotingDeviceDTO;
import com.smartmatic.waffle.ecm.service.lang.ElectoralOrganizationOperationException;
import com.smartmatic.waffle.ecm.service.lang.MasterUSBGenerationException;
import com.smartmatic.waffle.ecm.service.lang.NominationException;
import com.smartmatic.waffle.ecm.service.lang.UtilitiesException;

@RunWith(PowerMockRunner.class)
@PrepareForTest({ ECMServiceProvider.class, ECMClientSideConstants.class,
		ScreenshotsPrepareContestsWorker.class })
public class ScreenshotsPrepareContestsWorkerTest {

	@Before
	public void setUp() {
		File parent = new File("test/resources/master_folder");
		parent.mkdirs();
	}

	@Test
	public void testProcessSelectedObjects() throws NominationException,
			ConnectionException, UtilitiesException,
			MasterUSBGenerationException,
			ElectoralOrganizationOperationException, PendriveException {

		// arrange
		PowerMockito.mockStatic(ECMServiceProvider.class);

		List<Object> selectedObjects = new ArrayList<Object>();

		ContestData contest1 = new ContestData();
		ContestData contest2 = new ContestData();
		ContestData contest3 = new ContestData();

		contest1.setCode(1);

		contest2.setCode(2);

		contest3.setCode(3);

		List<ContestData> contestsResult1 = new ArrayList<ContestData>();
		contestsResult1.add(contest1);
		List<ContestData> contestsResult2 = new ArrayList<ContestData>();
		contestsResult2.add(contest2);
		List<ContestData> contestsResult3 = new ArrayList<ContestData>();
		contestsResult3.add(contest3);

		selectedObjects.add(contest1);
		selectedObjects.add(contest2);
		selectedObjects.add(contest3);

		Set<LanguageData> languages = new HashSet<LanguageData>();

		LanguageData language1 = new LanguageData();
		language1.setName("french");
		LanguageData language2 = new LanguageData();
		language2.setName("dutch");
		LanguageData language3 = new LanguageData();
		language3.setName("english");

		languages.add(language1);
		languages.add(language2);
		languages.add(language3);

		String mountedPath = "test/resources/master_folder";
		File mountedFolder = new File(mountedPath);

		ScreenshotsPrepareContestsWorker screenShotsPrepareContestsWorker = PowerMockito
				.spy(new ScreenshotsPrepareContestsWorker(selectedObjects,
						language1, mountedPath));

		PowerMockito.when(
				ECMServiceProvider.getContests(Mockito.anyListOf(Long.class),
						Mockito.eq(0), Mockito.anyInt())).thenReturn(
				contestsResult1);

		PowerMockito.when(
				ECMServiceProvider.getContests(Mockito.anyListOf(Long.class),
						Mockito.eq(1), Mockito.anyInt())).thenReturn(
				contestsResult2);

		PowerMockito.when(
				ECMServiceProvider.getContests(Mockito.anyListOf(Long.class),
						Mockito.eq(2), Mockito.anyInt())).thenReturn(
				contestsResult3);

		PowerMockito.doReturn(languages).when(screenShotsPrepareContestsWorker)
				.getElectionLanguages(Mockito.any(ContestData.class));

		PowerMockito
				.doNothing()
				.when(screenShotsPrepareContestsWorker)
				.generateForMultipleLanguages(Mockito.any(ContestData.class),
						Mockito.any(VotingDeviceData.class), Mockito.anyInt());

		PowerMockito.doCallRealMethod().when(screenShotsPrepareContestsWorker)
				.processSelectedObjects();

		// act
		screenShotsPrepareContestsWorker.processSelectedObjects();

		// assert
		assertEquals(screenShotsPrepareContestsWorker.getProcessedContests(), 3);
	}

	@Test
	public void testGenerateForMultipleLanguages() throws ConnectionException,
			UtilitiesException, MasterUSBGenerationException,
			ElectoralOrganizationOperationException {

		// arrange
		PowerMockito.mockStatic(ECMServiceProvider.class);

		List<Object> selectedObjects = new ArrayList<Object>();

		String mountedPath = "test/resources/master_folder";
		File mountedFolder = new File(mountedPath);

		ScreenshotsPrepareContestsWorker screenShotsPrepareContestsWorker = PowerMockito
				.spy(new ScreenshotsPrepareContestsWorker(selectedObjects,
						new LanguageData(), mountedPath));

		List<ContestLanguagesPairData> languagesPairs = new ArrayList<ContestLanguagesPairData>();

		ContestLanguagesPairData cl1 = new ContestLanguagesPairData();
		ContestLanguagesPairData cl2 = new ContestLanguagesPairData();

		LanguageData language1 = new LanguageData();
		language1.setLanguageRFC("FR");

		LanguageData language2 = new LanguageData();
		language2.setLanguageRFC("NL");

		cl1.setLanguage1(language1);
		cl1.setLanguage2(language2);

		cl2.setLanguage1(language2);

		languagesPairs.add(cl1);
		languagesPairs.add(cl2);

		PowerMockito.when(
				ECMServiceProvider.getLanguageCombinationsByContest(Mockito
						.any(ContestData.class))).thenReturn(languagesPairs);

		PowerMockito.doReturn("_").when(screenShotsPrepareContestsWorker)
				.getLanguagesSeparator();

		PowerMockito
				.doNothing()
				.when(screenShotsPrepareContestsWorker)
				.generateVotingCenter(Mockito.any(ContestData.class),
						Mockito.any(VotingDeviceData.class),
						Mockito.anySetOf(LanguageData.class), Mockito.anyInt(),
						Mockito.anyString());

		ContestData contest = new ContestData();
		VotingDeviceData votingDevice = new VotingDeviceData();
		votingDevice.setCustomCode("contest");

		// act
		screenShotsPrepareContestsWorker.generateForMultipleLanguages(contest,
				votingDevice, 1);

		// assert
		Mockito.verify(screenShotsPrepareContestsWorker).generateVotingCenter(
				Mockito.any(ContestData.class),
				Mockito.any(VotingDeviceData.class),
				Mockito.anySetOf(LanguageData.class), Mockito.anyInt(),
				Mockito.eq("contest_FR_NL"));

		Mockito.verify(screenShotsPrepareContestsWorker).generateVotingCenter(
				Mockito.any(ContestData.class),
				Mockito.any(VotingDeviceData.class),
				Mockito.anySetOf(LanguageData.class), Mockito.anyInt(),
				Mockito.eq("contest_NL"));
	}

	@SuppressWarnings("unchecked")
	@Test
	public void testGenerateVotingCenter() throws Exception {
		// arrange
		PowerMockito.mockStatic(ECMServiceProvider.class);
		ElectionXmlComponentsDTO electionXmlMock = PowerMockito
				.mock(ElectionXmlComponentsDTO.class);

		List<Object> selectedObjects = new ArrayList<Object>();

		VotingDeviceDTO votingDeviceDto = new VotingDeviceDTO();

		VotingDevicesData votingDevicesData = new VotingDevicesData();

		votingDeviceDto.setVotingDevicesData(votingDevicesData);

		String mountedPath = "test/resources/master_folder";
		File mountedFolder = new File(mountedPath);

		ScreenshotsPrepareContestsWorker screenShotsPrepareContestsWorker = PowerMockito
				.spy(new ScreenshotsPrepareContestsWorker(selectedObjects,
						new LanguageData(), mountedPath));

		PowerMockito.when(
				ECMServiceProvider.generateElectionForContest(
						Mockito.anyListOf(ContestData.class), Mockito.anyInt(),
						Mockito.any(ElectionData.class), Mockito.anyMap(),
						(List<ContestGroupData>) Mockito.isNull())).thenReturn(
				electionXmlMock);

		PowerMockito.when(
				ECMServiceProvider.generateVotingDeviceForRegion(
						Mockito.any(VotingDeviceData.class),
						Mockito.any(RegionData.class),
						Mockito.anyListOf(ContestGroupData.class),
						Mockito.anyMap(), Mockito.anyBoolean(),
						Mockito.any(ElectionData.class), Mockito.anyBoolean()))
				.thenReturn(votingDeviceDto);

		PowerMockito.doReturn(new ArrayList<LanguageForVotingDeviceData>())
				.when(screenShotsPrepareContestsWorker,
						"getVotingDevicesLanguages",
						Mockito.anyListOf(LanguageData.class));

		PowerMockito.doReturn(new ElectionData()).when(electionXmlMock)
				.getElection();

		PowerMockito.doReturn(new ArrayList<ContestGroupData>())
				.when(electionXmlMock).getContestGroup();

		ContestData contest = new ContestData();
		VotingDeviceData votingDevice = new VotingDeviceData();
		votingDevice.setName("CN1");
		votingDevice.setCustomCode("CN1");

		Set<LanguageData> languages = new HashSet<LanguageData>();

		languages.add(new LanguageData());
		languages.add(new LanguageData());

		// act
		screenShotsPrepareContestsWorker.generateVotingCenter(contest,
				votingDevice, languages, 1, "contest1");

		Map<VotingCenter, ElectionData> votingCenterElectionMapResult = screenShotsPrepareContestsWorker
				.getVotingCenterElectionMap();

		Set<VotingCenter> votingCenters = votingCenterElectionMapResult
				.keySet();

		// assert

		for (VotingCenter votingCenter : votingCenters) {
			assertEquals(votingCenter.getCode(), "CN1");
			assertEquals(votingCenter.getFolderName(), "contest1");

			assertEquals(votingCenterElectionMapResult.get(votingCenter),
					electionXmlMock.getElection());
		}

	}

	@After
	public void tearDown() {
		FileUtils.deleteQuietly(new File("test/resources/master_folder"));
	}
}
