/**
 * BatteryStatus.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.core.battery;

import java.io.IOException;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeoutException;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.text.StrTokenizer;

import com.smartmatic.waffle.ecm.core.battery.IBatteryStatusQuery.BatteryState;
import com.smartmatic.waffle.ecm.core.printer.PrinterStatus;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Class that manage the batery power present status.
 */
public class BatteryStatus {

	/** The logger. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(BatteryStatus.class);

	/** The listeners. */
	private List<IBatteryStatusChangedListener> listeners = new LinkedList<IBatteryStatusChangedListener>();

	/** The battery status task. */
	private BatteryStatusTask batteryStatusTask;

	/**
	 * Instantiates a new battery status.
	 * 
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 * @throws TimeoutException
	 *             the timeout exception
	 */
	public BatteryStatus() throws IOException, TimeoutException {
		this.batteryStatusTask = new BatteryStatusTask();
		this.batteryStatusTask.init();
		Thread timer = new Thread(this.batteryStatusTask, "BatteryStatus");
		timer.start();
		return;
	}

	/**
	 * Adds the listener.
	 * 
	 * @param listener
	 *            the listener
	 */
	public void addListener(IBatteryStatusChangedListener listener) {
		this.listeners.add(listener);
	}

	/**
	 * The Class BatteryStatusTask.
	 */
	class BatteryStatusTask implements Runnable, IBatteryStatusQuery {

		/** The device raw state. */
		private String deviceRawState;

		/** The power percentage. */
		private int powerPercentage;

		/*
		 * (non-Javadoc)
		 * 
		 * @see
		 * com.smartmatic.waffle.ecm.core.battery.IBatteryStatusQuery#updateStatus
		 * ()
		 */
		@Override
		public Map<String, Object> updateStatus() throws IOException,
				TimeoutException {

			boolean disconnected = true;

			logger.debug("Processing update status method");

			Map<String, Object> changed = new HashMap<String, Object>();

			List<String> batteryInfoOutput = PrinterStatus.executeProcess(
					"upower", "-i",
					"/org/freedesktop/UPower/devices/battery_BAT0");

			logger.debug("Battery info output ['" + batteryInfoOutput + "']");

			for (String line : batteryInfoOutput) {

				StrTokenizer tokens = new StrTokenizer(line.trim(), " ");

				if (tokens.size() == 2) {
					String[] tokenArray = tokens.getTokenArray();

					final String firstToken = tokenArray[0];
					logger.debug("first Token : [" + firstToken + "]");

					final String secondToken = tokenArray[1];
					logger.debug("first Token : [" + secondToken + "]");

					if ("state:".equals(firstToken)) {
						disconnected = false;
						BatteryState last = this.getBatteryState();
						this.deviceRawState = secondToken;
						BatteryState current = this.getBatteryState();

						if (last != current) {
							changed.put(IBatteryStatusQuery.KEY_BATTERY_STATE,
									current);
						}

					} else if ("percentage:".equals(firstToken)) {
						disconnected = false;
						String percentage = secondToken;
						int i = 1;

						int last = this.powerPercentage;

						while (StringUtils
								.isNumeric(percentage.substring(0, i))) {
							this.powerPercentage = Integer.valueOf(percentage
									.substring(0, i));
							i++;
						}

						int current = this.powerPercentage;

						if (last != current) {
							changed.put(
									IBatteryStatusQuery.KEY_POWER_PERCENTAGE,
									this.powerPercentage);
						}
					}

				}
			}

			if (disconnected) {
				BatteryState last = this.getBatteryState();
				this.deviceRawState = null;
				BatteryState current = this.getBatteryState();

				this.powerPercentage = 0;

				if (last != current) {
					changed.put(IBatteryStatusQuery.KEY_BATTERY_STATE, current);
				}
			}

			return changed;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see java.lang.Runnable#run()
		 */
		@Override
		public void run() {
			try {

				logger.debug("Executing battery monitoring tasks");

				while (true) {

					Map<String, Object> changed = updateStatus();

					if (changed.isEmpty() == false
							&& BatteryStatus.this.listeners.isEmpty() == false) {

						for (IBatteryStatusChangedListener listener : BatteryStatus.this.listeners) {
							listener.batteryStatusChanged(changed);
						}
					}
					Thread.sleep(500);
				}
			} catch (Exception e) {
				logger.error(e, "There is an error executing printing tasks");
			}
		}

		/**
		 * Inits the.
		 * 
		 * @throws IOException
		 *             Signals that an I/O exception has occurred.
		 * @throws TimeoutException
		 *             the timeout exception
		 */
		private void init() throws IOException, TimeoutException {

			List<String> batteryInfoOutput = PrinterStatus.executeProcess(
					"upower", "-i",
					"/org/freedesktop/UPower/devices/battery_BAT0");

			logger.debug("Battery info output ['" + batteryInfoOutput + "']");

			for (String line : batteryInfoOutput) {

				StrTokenizer tokens = new StrTokenizer(line.trim(), " ");

				if (tokens.size() == 2) {
					String[] tokenArray = tokens.getTokenArray();

					final String firstToken = tokenArray[0];
					logger.debug("first Token : [" + firstToken + "]");

					final String secondToken = tokenArray[1];
					logger.debug("first Token : [" + secondToken + "]");

					if ("state:".equals(firstToken)) {
						this.deviceRawState = secondToken;

					} else if ("percentage:".equals(firstToken)) {
						String percentage = secondToken;
						int i = 1;
						while (StringUtils
								.isNumeric(percentage.substring(0, i))) {
							this.powerPercentage = Integer.valueOf(percentage
									.substring(0, i));
							i++;
						}
					}

				}
			}
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see com.smartmatic.waffle.ecm.core.battery.IBatteryStatusQuery#
		 * getBatteryState()
		 */
		@Override
		public BatteryState getBatteryState() {
			if (this.deviceRawState != null) {
				if (this.deviceRawState.equals("fully-charged")) {
					return BatteryState.FULLY_CHARGED;
				} else if (this.deviceRawState.equals("discharging")) {
					return BatteryState.DISCHARGING;
				} else if (this.deviceRawState.equals("charging")) {
					return BatteryState.CHARGING;
				}
			}
			return BatteryState.DISCONNECTED;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see com.smartmatic.waffle.ecm.core.battery.IBatteryStatusQuery#
		 * getPowerPercent()
		 */
		@Override
		public int getPowerPercent() {
			return this.powerPercentage;
		}
	}

	/**
	 * Gets the battery state.
	 * 
	 * @return the battery state
	 */
	public BatteryState getBatteryState() {
		return this.batteryStatusTask.getBatteryState();
	}

	/**
	 * Gets the power percent.
	 * 
	 * @return the power percent
	 */
	public int getPowerPercent() {
		return this.batteryStatusTask.getPowerPercent();
	}
}
