/**
 * ConnectivityManagerMultiple.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.core.monitoring;

import java.io.IOException;
import java.net.ConnectException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.UnknownHostException;
import java.nio.channels.SocketChannel;
import java.util.ArrayList;

import com.smartmatic.waffle.ecm.core.monitoring.spec.IConnectionCheckListener;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Class used to perform a check of the connection to the server.
 */
public class ConnectivityManagerMultiple {

	/**
	 * sleep time in thread execution (2 seg).
	 */
	private final Integer THREAD_DELAY = 2000;

	/**
	 * Field used for logging purpose.
	 */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(ConnectivityManagerMultiple.class);

	/** * the server name / IP. */
	private String server;

	/**
	 * the port number.
	 */
	private Integer port;

	/**
	 * The ip adress.
	 */
	private InetAddress ipAdress;

	/**
	 * The socket address.
	 */
	private InetSocketAddress socketAddress;

	/**
	 * Enum to indicate the conection status.
	 */
	public enum ConnectionState {

		/**
		 * The init.
		 */
		INIT,
		/**
		 * The connected.
		 */
		CONNECTED,
		/**
		 * The disconnected.
		 */
		DISCONNECTED,
		/**
		 * The stopped.
		 */
		STOPPED;
	}

	/**
	 * The current state of the connection.
	 */
	private ConnectionState state = ConnectionState.STOPPED;

	/**
	 * The listener list to notify about status change.
	 */
	private ArrayList<IConnectionCheckListener> stateListeners = new ArrayList<IConnectionCheckListener>();

	/**
	 * Class constructor.
	 * 
	 * @param host
	 *            the server name / IP
	 * @param p
	 *            the port number.
	 */
	public ConnectivityManagerMultiple(String host, Integer p) {
		this.server = host;
		this.port = p;
	}

	/**
	 * The Class ConnectionTask.
	 */
	class ConnectionTask implements Runnable {

		private static final int MAX_NUMBER_OF_TRIES = 3;

		/** The time that the host will be tested before throw a timeout */
		private static final int HOST_REACHABLE_TIMEOUT = 1000;

		/***
		 * {@inheritDoc}<br>
		 * Method used to perform a check of the connection to the server.
		 */
		@Override
		public void run() {
			SocketChannel socketChannel = null;

			while (true) {
				try {
					try {

						if (!ConnectivityManagerMultiple.this.ipAdress
								.isReachable(HOST_REACHABLE_TIMEOUT)) {
							throw new ConnectException("Host: "
									+ ConnectivityManagerMultiple.this.ipAdress
									+ " unreachable");
						}

						socketChannel = SocketChannel.open();
						socketChannel.configureBlocking(false);
						socketChannel
								.connect(ConnectivityManagerMultiple.this.socketAddress);

						int tries = 0;
						while (!socketChannel.finishConnect()
								&& tries < MAX_NUMBER_OF_TRIES) {
							Thread.sleep(1000);
						}

						if (socketChannel.isConnected()) {
							if (ConnectivityManagerMultiple.this.state != ConnectionState.CONNECTED) {
								ConnectivityManagerMultiple.this.state = ConnectionState.CONNECTED;
								ConnectivityManagerMultiple.this
										.fireConnectedHostEvent();
							}
						} else {
							if (ConnectivityManagerMultiple.this.state != ConnectionState.DISCONNECTED) {
								ConnectivityManagerMultiple.this.state = ConnectionState.DISCONNECTED;
								ConnectivityManagerMultiple.this
										.fireDisconnectedHostEvent();
							}
						}
					} catch (Exception e) {

						logger.error(
								e,
								"An error occured checking connection to: "
										+ ConnectivityManagerMultiple.this.socketAddress);

						if (ConnectivityManagerMultiple.this.state != ConnectionState.DISCONNECTED) {
							ConnectivityManagerMultiple.this.state = ConnectionState.DISCONNECTED;
							ConnectivityManagerMultiple.this
									.fireDisconnectedHostEvent();
						}
					} finally {
						if (socketChannel != null) {
							try {
								socketChannel.close();
							} catch (IOException e) {
								logger.info(e,
										"There is a problem closing the socket");
							}
							socketChannel = null;
						}
					}
					Thread.sleep(ConnectivityManagerMultiple.this.THREAD_DELAY);
				} catch (InterruptedException e) {
					logger.error(
							"There was an error trying to sleep connectivity manager thread ",
							e);
				}
			}
		}
	}

	/**
	 * Getter of {@link #state} field.
	 * 
	 * @return {@link #state} value.
	 */
	public ConnectionState getState() {
		return this.state;
	}

	/**
	 * Method used to start the monitoring connectivity.
	 */
	public void start() {
		logger.info("Checking connection with server [" + this.server
				+ "] on port [" + this.port + "]");
		Thread eventThread = new Thread(new ConnectionTask(),
				"MONITORING_MANAGER_THREAD");

		try {
			this.ipAdress = InetAddress.getByName(this.server);
			this.socketAddress = new InetSocketAddress(this.ipAdress, this.port);
		} catch (UnknownHostException e) {
			logger.error(e, "There was an error init connectivity test");
			return;
		}

		eventThread.start();
	}

	/**
	 * Method used to notify to all listeners about the event host connected.
	 */
	private void fireConnectedHostEvent() {
		synchronized (this.stateListeners) {
			for (IConnectionCheckListener listener : this.stateListeners) {
				try {
					logger.info("Host " + this.server + ":" + this.port
							+ " connected");
					listener.hostConnected();
				} catch (Exception e) {
					logger.error(
							"There was an error when throw the host is connected [cause]: "
									+ e.getMessage(), e);
				}
			}
		}
	}

	/**
	 * Method used to notify to all listeners about the event host disconnected.
	 */
	private void fireDisconnectedHostEvent() {
		synchronized (this.stateListeners) {
			for (IConnectionCheckListener listener : this.stateListeners) {
				try {
					logger.info("Host " + this.server + ":" + this.port
							+ " disconnected");
					listener.hostDisconnected();
				} catch (Exception e) {
					logger.error(
							"There was an error when throw the host is disconnected [cause]: "
									+ e.getMessage(), e);
				}
			}
		}
	}

	/**
	 * Method used to add listeners to manager of events, to further
	 * notification.
	 * 
	 * @param client
	 *            listener to add
	 */
	public void addConnectionListener(IConnectionCheckListener client) {
		synchronized (this.stateListeners) {
			if (!this.containsConnectionListener(client)) {
				this.stateListeners.add(client);
			}
		}
	}

	/**
	 * Method used to remove listeners to manager of events, to further
	 * notification.
	 * 
	 * @param client
	 *            listener to remove.
	 */
	public void removeConnectionListener(IConnectionCheckListener client) {
		synchronized (this.stateListeners) {
			if (this.containsConnectionListener(client)) {
				this.stateListeners.remove(client);
			}
		}
	}

	/**
	 * Contains connection listener.
	 * 
	 * @param e
	 *            the e
	 * @return true, if successful
	 */
	public boolean containsConnectionListener(IConnectionCheckListener e) {
		synchronized (this.stateListeners) {
			return this.stateListeners.contains(e);
		}
	}
}
