/**
 * HPLIPPrinterStatusQuery.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.core.printer;

import java.io.IOException;
import java.net.InetAddress;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeoutException;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.SystemUtils;
import org.apache.commons.lang.text.StrTokenizer;

import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Class used to holds each possible printer status. This class is also used as
 * notifier if any status change.
 */
public class HPLIPPrinterStatusQuery implements IPrinterStatusQuery {

	/** For logging purposes. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(HPLIPPrinterStatusQuery.class);

	/** The printer url. */
	public final String printerUrl;

	/** The device raw state. */
	private int deviceRawState;

	/** The status code. */
	private int statusCode = -4;

	/** The status desc. */
	private String statusDesc;

	/** The ink level percent. */
	private int inkLevelPercent;

	/** The hp info output. */
	private List<String> hpInfoOutput;

	/* Device states */
	/** The Constant DEVICE_STATE_NOT_FOUND. */
	public final static int DEVICE_STATE_NOT_FOUND = -1;

	/** The Constant DEVICE_STATE_FOUND. */
	public final static int DEVICE_STATE_FOUND = 0;

	/** The Constant DEVICE_STATE_JUST_FOUND. */
	public final static int DEVICE_STATE_JUST_FOUND = 1;

	/** The Constant DEVICE_STATE_DISCONNECTED. */
	public final static int DEVICE_STATE_DISCONNECTED = -2;

	/** The Constant STATUS_DISCONNECTED. */
	public final static int STATUS_DISCONNECTED = -3;

	/** The Constant STATUS_PRINTER_BASE. */
	public final static int STATUS_PRINTER_BASE = 1000;

	/** The Constant STATUS_PRINTER_IDLE. */
	public final static int STATUS_PRINTER_IDLE = 1000;

	/** The Constant STATUS_PRINTER_BUSY. */
	public final static int STATUS_PRINTER_BUSY = 1001;

	/** The Constant STATUS_PRINTER_PRINTING. */
	public final static int STATUS_PRINTER_PRINTING = 1002;

	/** The Constant STATUS_PRINTER_TURNING_OFF. */
	public final static int STATUS_PRINTER_TURNING_OFF = 1003;

	/** The Constant STATUS_PRINTER_REPORT_PRINTING. */
	public final static int STATUS_PRINTER_REPORT_PRINTING = 1004;

	/** The Constant STATUS_PRINTER_CANCELING. */
	public final static int STATUS_PRINTER_CANCELING = 1005;

	/** The Constant STATUS_PRINTER_IO_STALL. */
	public final static int STATUS_PRINTER_IO_STALL = 1006;

	/** The Constant STATUS_PRINTER_DRY_WAIT_TIME. */
	public final static int STATUS_PRINTER_DRY_WAIT_TIME = 1007;

	/** The Constant STATUS_PRINTER_PEN_CHANGE. */
	public final static int STATUS_PRINTER_PEN_CHANGE = 1008;

	/** The Constant STATUS_PRINTER_OUT_OF_PAPER. */
	public final static int STATUS_PRINTER_OUT_OF_PAPER = 1009;

	/** The Constant STATUS_PRINTER_BANNER_EJECT. */
	public final static int STATUS_PRINTER_BANNER_EJECT = 1010;

	/** The Constant STATUS_PRINTER_BANNER_MISMATCH. */
	public final static int STATUS_PRINTER_BANNER_MISMATCH = 1011;

	/** The Constant STATUS_PRINTER_PHOTO_MISMATCH. */
	public final static int STATUS_PRINTER_PHOTO_MISMATCH = 1012;

	/** The Constant STATUS_PRINTER_DUPLEX_MISMATCH. */
	public final static int STATUS_PRINTER_DUPLEX_MISMATCH = 1013;

	/** The Constant STATUS_PRINTER_MEDIA_JAM. */
	public final static int STATUS_PRINTER_MEDIA_JAM = 1014;

	/** The Constant STATUS_PRINTER_CARRIAGE_STALL. */
	public final static int STATUS_PRINTER_CARRIAGE_STALL = 1015;

	/** The Constant STATUS_PRINTER_PAPER_STALL. */
	public final static int STATUS_PRINTER_PAPER_STALL = 1016;

	/** The Constant STATUS_PRINTER_PEN_FAILURE. */
	public final static int STATUS_PRINTER_PEN_FAILURE = 1017;

	/** The Constant STATUS_PRINTER_HARD_ERROR. */
	public final static int STATUS_PRINTER_HARD_ERROR = 1018;

	/** The Constant STATUS_PRINTER_POWER_DOWN. */
	public final static int STATUS_PRINTER_POWER_DOWN = 1019;

	/** The Constant STATUS_PRINTER_FRONT_PANEL_TEST. */
	public final static int STATUS_PRINTER_FRONT_PANEL_TEST = 1020;

	/** The Constant STATUS_PRINTER_CLEAN_OUT_TRAY_MISSING. */
	public final static int STATUS_PRINTER_CLEAN_OUT_TRAY_MISSING = 1021;

	/** The Constant STATUS_PRINTER_OUTPUT_BIN_FULL. */
	public final static int STATUS_PRINTER_OUTPUT_BIN_FULL = 1022;

	/** The Constant STATUS_PRINTER_MEDIA_SIZE_MISMATCH. */
	public final static int STATUS_PRINTER_MEDIA_SIZE_MISMATCH = 1023;

	/** The Constant STATUS_PRINTER_MANUAL_DUPLEX_BLOCK. */
	public final static int STATUS_PRINTER_MANUAL_DUPLEX_BLOCK = 1024;

	/** The Constant STATUS_PRINTER_SERVCE_STALL. */
	public final static int STATUS_PRINTER_SERVCE_STALL = 1025;

	/** The Constant STATUS_PRINTER_OUT_OF_INK. */
	public final static int STATUS_PRINTER_OUT_OF_INK = 1026;

	/**
	 * The Constant STATUS_PRINTER_LIO_ERROR .
	 */
	public final static int STATUS_PRINTER_LIO_ERROR = 1027;

	/** The Constant STATUS_PRINTER_PUMP_STALL. */
	public final static int STATUS_PRINTER_PUMP_STALL = 1028;

	/** The Constant STATUS_PRINTER_TRAY_2_MISSING. */
	public final static int STATUS_PRINTER_TRAY_2_MISSING = 1029;

	/** The Constant STATUS_PRINTER_DUPLEXER_MISSING. */
	public final static int STATUS_PRINTER_DUPLEXER_MISSING = 1030;

	/** The Constant STATUS_PRINTER_REAR_TRAY_MISSING. */
	public final static int STATUS_PRINTER_REAR_TRAY_MISSING = 1031;

	/** The Constant STATUS_PRINTER_PEN_NOT_LATCHED. */
	public final static int STATUS_PRINTER_PEN_NOT_LATCHED = 1032;

	/** The Constant STATUS_PRINTER_BATTERY_VERY_LOW. */
	public final static int STATUS_PRINTER_BATTERY_VERY_LOW = 1033;

	/** The Constant STATUS_PRINTER_SPITTOON_FULL. */
	public final static int STATUS_PRINTER_SPITTOON_FULL = 1034;

	/** The Constant STATUS_PRINTER_OUTPUT_TRAY_CLOSED. */
	public final static int STATUS_PRINTER_OUTPUT_TRAY_CLOSED = 1035;

	/** The Constant STATUS_PRINTER_MANUAL_FEED_BLOCKED. */
	public final static int STATUS_PRINTER_MANUAL_FEED_BLOCKED = 1036;

	/** The Constant STATUS_PRINTER_REAR_FEED_BLOCKED. */
	public final static int STATUS_PRINTER_REAR_FEED_BLOCKED = 1037;

	/** The Constant TRAY_2_OPEN. */
	public final static int TRAY_2_OPEN = 1138;

	/** The Constant STATUS_PRINTER_TRAY_2_OUT_OF_PAPER. */
	public final static int STATUS_PRINTER_TRAY_2_OUT_OF_PAPER = 1038;

	/** The Constant STATUS_PRINTER_UNABLE_TO_LOAD_FROM_LOCKED_TRAY. */
	public final static int STATUS_PRINTER_UNABLE_TO_LOAD_FROM_LOCKED_TRAY = 1039;

	/** The Constant STATUS_PRINTER_NON_HP_INK. */
	public final static int STATUS_PRINTER_NON_HP_INK = 1040;

	/** The Constant STATUS_PRINTER_PEN_CALIBRATION_RESUME. */
	public final static int STATUS_PRINTER_PEN_CALIBRATION_RESUME = 1041;

	/** The Constant STATUS_PRINTER_MEDIA_TYPE_MISMATCH. */
	public final static int STATUS_PRINTER_MEDIA_TYPE_MISMATCH = 1042;

	/** The Constant STATUS_PRINTER_CUSTOM_MEDIA_MISMATCH. */
	public final static int STATUS_PRINTER_CUSTOM_MEDIA_MISMATCH = 1043;

	/** The Constant STATUS_PRINTER_PEN_CLEANING. */
	public final static int STATUS_PRINTER_PEN_CLEANING = 1044;

	/** The Constant STATUS_PRINTER_PEN_CHECKING. */
	public final static int STATUS_PRINTER_PEN_CHECKING = 1045;

	/** The Constant STATUS_PRINTER_POWER_SAVE. */
	public final static int STATUS_PRINTER_POWER_SAVE = 1046;

	/** The Constant STATUS_PRINTER_CARTRIDGE_WRONG. */
	public final static int STATUS_PRINTER_CARTRIDGE_WRONG = 1047;

	/** The Constant STATUS_PRINTER_CARTRIDGE_MISSING. */
	public final static int STATUS_PRINTER_CARTRIDGE_MISSING = 1048;

	/** The Constant STATUS_PRINTER_PRINTHEAD_MISSING. */
	public final static int STATUS_PRINTER_PRINTHEAD_MISSING = 1049;

	/* derived codes */
	/* set to AGENT_TYPE + base (base: 1500=ink, 1600=laser ) */;

	/** The Constant STATUS_PRINTER_LOW_INK_BASE. */
	public final static int STATUS_PRINTER_LOW_INK_BASE = 1500;

	/** The Constant STATUS_PRINTER_LOW_BLACK_INK. */
	public final static int STATUS_PRINTER_LOW_BLACK_INK = 1501;

	/** The Constant STATUS_PRINTER_LOW_TRI_COLOR_INK. */
	public final static int STATUS_PRINTER_LOW_TRI_COLOR_INK = 1502;

	/** The Constant STATUS_PRINTER_LOW_PHOTO_INK. */
	public final static int STATUS_PRINTER_LOW_PHOTO_INK = 1503;

	/** The Constant STATUS_PRINTER_LOW_CYAN_INK. */
	public final static int STATUS_PRINTER_LOW_CYAN_INK = 1504;

	/** The Constant STATUS_PRINTER_LOW_MAGENTA_INK. */
	public final static int STATUS_PRINTER_LOW_MAGENTA_INK = 1505;

	/** The Constant STATUS_PRINTER_LOW_YELLOW_INK. */
	public final static int STATUS_PRINTER_LOW_YELLOW_INK = 1506;

	/** The Constant STATUS_PRINTER_LOW_PHOTO_CYAN_INK. */
	public final static int STATUS_PRINTER_LOW_PHOTO_CYAN_INK = 1507;

	/** The Constant STATUS_PRINTER_LOW_PHOTO_MAGENTA_INK. */
	public final static int STATUS_PRINTER_LOW_PHOTO_MAGENTA_INK = 1508;

	/** The Constant STATUS_PRINTER_LOW_PHOTO_YELLOW_INK. */
	public final static int STATUS_PRINTER_LOW_PHOTO_YELLOW_INK = 1509;

	/** The Constant STATUS_PRINTER_LOW_PHOTO_GRAY_INK. */
	public final static int STATUS_PRINTER_LOW_PHOTO_GRAY_INK = 1510;

	/** The Constant STATUS_PRINTER_LOW_PHOTO_BLUE_INK. */
	public final static int STATUS_PRINTER_LOW_PHOTO_BLUE_INK = 1511;

	/** The Constant STATUS_PRINTER_LOW_TONER_BASE. */
	public final static int STATUS_PRINTER_LOW_TONER_BASE = 1600;

	/** The Constant STATUS_PRINTER_LOW_BLACK_TONER. */
	public final static int STATUS_PRINTER_LOW_BLACK_TONER = 1601;

	/** The Constant STATUS_PRINTER_LOW_CYAN_TONER. */
	public final static int STATUS_PRINTER_LOW_CYAN_TONER = 1604;

	/** The Constant STATUS_PRINTER_LOW_MAGENTA_TONER. */
	public final static int STATUS_PRINTER_LOW_MAGENTA_TONER = 1605;

	/** The Constant STATUS_PRINTER_LOW_YELLOW_TONER. */
	public final static int STATUS_PRINTER_LOW_YELLOW_TONER = 1606;

	/* derived laserjet codes */
	/** The Constant STATUS_PRINTER_WARMING_UP. */
	public final static int STATUS_PRINTER_WARMING_UP = 1800;

	/** The Constant STATUS_PRINTER_LOW_PAPER. */
	public final static int STATUS_PRINTER_LOW_PAPER = 1801;

	/** The Constant STATUS_PRINTER_DOOR_OPEN. */
	public final static int STATUS_PRINTER_DOOR_OPEN = 1802;

	/** The Constant STATUS_PRINTER_OFFLINE. */
	public final static int STATUS_PRINTER_OFFLINE = 1803;

	/** The Constant STATUS_PRINTER_LOW_TONER. */
	public final static int STATUS_PRINTER_LOW_TONER = 1804;

	/** The Constant STATUS_PRINTER_NO_TONER. */
	public final static int STATUS_PRINTER_NO_TONER = 1805;

	/** The Constant STATUS_PRINTER_SERVICE_REQUEST. */
	public final static int STATUS_PRINTER_SERVICE_REQUEST = 1806;

	/** The Constant STATUS_PRINTER_FUSER_ERROR. */
	public final static int STATUS_PRINTER_FUSER_ERROR = 1807;

	/** The Constant STATUS_PRINTER_LASERJET_BUSY. */
	public final static int STATUS_PRINTER_LASERJET_BUSY = 5002;

	/* other derived codes */
	/** The Constant STATUS_DEVICE_UNSUPPORTED. */
	public final static int STATUS_DEVICE_UNSUPPORTED = 1900;

	/**
	 * Instantiates a new hPLIP printer status query.
	 * 
	 * @param printerUrl
	 *            the printer url
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	public HPLIPPrinterStatusQuery(String printerUrl) throws IOException {
		this.printerUrl = printerUrl;

		return;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.smartmatic.waffle.ecm.core.printer.IPrinterStatusQuery#updateStatus
	 * ()
	 */
	@Override
	public Map<String, Object> updateStatus() throws IOException,
			TimeoutException {

		boolean forceDisconnected = true;

		logger.debug("Processing update status method");
		Map<String, Object> changed = new HashMap<String, Object>();

		InetAddress inet = InetAddress.getByName("10.0.16.80");

		if (!inet.isReachable(500)) {
			final StatusPrinter last = this.getPrinterStatus();

			this.statusCode = STATUS_PRINTER_OFFLINE;

			final StatusPrinter current = this.getPrinterStatus();

			if (last != current) {
				changed.put("statusCode", current);
			}

			return changed;
		}

		this.hpInfoOutput = PrinterStatus.executeProcess("hp-info", "-i",
				"--device=" + this.printerUrl);

		logger.debug("hp-info output ['" + this.hpInfoOutput + "']");

		for (String line : this.hpInfoOutput) {

			StrTokenizer tokens = new StrTokenizer(line.trim(),
					"                   ");

			if (tokens.size() == 2) {
				String[] tokenArray = tokens.getTokenArray();

				final String firstToken = tokenArray[0];
				logger.debug("first Token : [" + firstToken + "]");

				final String secondToken = tokenArray[1];
				logger.debug("first Token : [" + secondToken + "]");

				if ("device-state".equals(firstToken)) {
					forceDisconnected = false;
					DeviceState last = this.getDeviceState();
					this.deviceRawState = Integer.parseInt(secondToken);
					DeviceState current = this.getDeviceState();

					if (last != current) {
						changed.put("deviceState", current);
					}

				} else if ("status-code".equals(firstToken)) {
					forceDisconnected = false;
					final StatusPrinter last = this.getPrinterStatus();
					this.statusCode = Integer.parseInt(secondToken);

					final StatusPrinter current = this.getPrinterStatus();

					if (last != current) {
						changed.put("statusCode", current);
					}

				} else if ("panel-line1".equals(firstToken)
						&& secondToken.equals("TRAY 2 OPEN")) {
					forceDisconnected = false;
					final StatusPrinter last = this.getPrinterStatus();
					this.statusCode = TRAY_2_OPEN;

					final StatusPrinter current = this.getPrinterStatus();

					if (last != current) {
						changed.put("statusCode", current);
					}

					break;

				} else if ("panel-line1".equals(firstToken)
						&& secondToken.contains("Canceling")) {
					forceDisconnected = false;
					final StatusPrinter last = this.getPrinterStatus();
					this.statusCode = STATUS_PRINTER_CANCELING;

					final StatusPrinter current = this.getPrinterStatus();

					if (last != current) {
						changed.put("statusCode", current);
					}

					break;

				} else if ("status-desc".equals(firstToken)) {
					forceDisconnected = false;
					this.statusDesc = secondToken;

				} else if ("agent1-level".equals(firstToken)) {

					this.inkLevelPercent = Integer.parseInt(secondToken);

				}

			}

		}

		if (forceDisconnected) {
			DeviceState lastDeviceState = this.getDeviceState();
			this.deviceRawState = DEVICE_STATE_DISCONNECTED;
			DeviceState currentDeviceState = this.getDeviceState();

			if (lastDeviceState != currentDeviceState) {
				changed.put("deviceState", currentDeviceState);
			}

			final StatusPrinter lastStatusPrinter = this.getPrinterStatus();
			this.statusCode = STATUS_DISCONNECTED;

			final StatusPrinter currentStatusPrinter = this.getPrinterStatus();

			if (lastStatusPrinter != currentStatusPrinter) {
				changed.put("statusCode", currentStatusPrinter);
			}

		}

		logger.debug("Finished update status method");
		return changed;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.smartmatic.waffle.ecm.core.printer.IPrinterStatusQuery#getPrinterStatus
	 * ()
	 */
	@Override
	public StatusPrinter getPrinterStatus() {
		switch (this.statusCode) {
		case STATUS_PRINTER_BUSY:
		case STATUS_PRINTER_TURNING_OFF:
		case STATUS_PRINTER_DRY_WAIT_TIME:
		case STATUS_PRINTER_POWER_DOWN:
		case STATUS_PRINTER_PEN_CLEANING:
		case STATUS_PRINTER_PEN_CHECKING:
		case STATUS_PRINTER_POWER_SAVE:
		case STATUS_PRINTER_WARMING_UP:
		case STATUS_PRINTER_PEN_CALIBRATION_RESUME:
		case STATUS_PRINTER_LASERJET_BUSY:
			return StatusPrinter.BUSY;
		case STATUS_PRINTER_CANCELING:
			return StatusPrinter.CANCELING;
		case STATUS_PRINTER_IO_STALL:
		case STATUS_PRINTER_PEN_CHANGE:
		case STATUS_PRINTER_BANNER_EJECT:
		case STATUS_PRINTER_BANNER_MISMATCH:
		case STATUS_PRINTER_PHOTO_MISMATCH:
		case STATUS_PRINTER_DUPLEX_MISMATCH:
		case STATUS_PRINTER_MEDIA_JAM:
		case STATUS_PRINTER_CARRIAGE_STALL:
		case STATUS_PRINTER_PAPER_STALL:
		case STATUS_PRINTER_PEN_FAILURE:
		case STATUS_PRINTER_HARD_ERROR:
		case STATUS_PRINTER_FRONT_PANEL_TEST:
		case STATUS_PRINTER_CLEAN_OUT_TRAY_MISSING:
		case STATUS_PRINTER_OUTPUT_BIN_FULL:
		case STATUS_PRINTER_MEDIA_SIZE_MISMATCH:
		case STATUS_PRINTER_MANUAL_DUPLEX_BLOCK:
		case STATUS_PRINTER_SERVCE_STALL:
		case STATUS_PRINTER_LIO_ERROR:
		case STATUS_PRINTER_PUMP_STALL:
		case STATUS_PRINTER_TRAY_2_MISSING:
		case STATUS_PRINTER_DUPLEXER_MISSING:
		case STATUS_PRINTER_REAR_TRAY_MISSING:
		case STATUS_PRINTER_PEN_NOT_LATCHED:
		case STATUS_PRINTER_SPITTOON_FULL:
		case STATUS_PRINTER_OUTPUT_TRAY_CLOSED:
		case STATUS_PRINTER_MANUAL_FEED_BLOCKED:
		case STATUS_PRINTER_REAR_FEED_BLOCKED:
		case STATUS_PRINTER_UNABLE_TO_LOAD_FROM_LOCKED_TRAY:
		case STATUS_PRINTER_NON_HP_INK:
		case STATUS_PRINTER_MEDIA_TYPE_MISMATCH:
		case STATUS_PRINTER_CUSTOM_MEDIA_MISMATCH:
		case STATUS_PRINTER_CARTRIDGE_WRONG:
		case STATUS_PRINTER_CARTRIDGE_MISSING:
		case STATUS_PRINTER_PRINTHEAD_MISSING:
		case STATUS_PRINTER_SERVICE_REQUEST:
		case STATUS_PRINTER_FUSER_ERROR:
			return StatusPrinter.ERROR;

		case STATUS_PRINTER_DOOR_OPEN:
			return StatusPrinter.DOOR_OPEN;

		case TRAY_2_OPEN:
			return StatusPrinter.PAPER_DOOR_OPEN;

		case STATUS_PRINTER_BATTERY_VERY_LOW:
			return StatusPrinter.BATTERY_VERY_LOW;

		case STATUS_PRINTER_IDLE:
		case STATUS_PRINTER_PRINTING:
		case STATUS_PRINTER_REPORT_PRINTING:
			return StatusPrinter.READY;

		case STATUS_PRINTER_LOW_BLACK_INK:
		case STATUS_PRINTER_LOW_TRI_COLOR_INK:
		case STATUS_PRINTER_LOW_PHOTO_INK:
		case STATUS_PRINTER_LOW_CYAN_INK:
		case STATUS_PRINTER_LOW_MAGENTA_INK:
		case STATUS_PRINTER_LOW_YELLOW_INK:
		case STATUS_PRINTER_LOW_PHOTO_CYAN_INK:
		case STATUS_PRINTER_LOW_PHOTO_MAGENTA_INK:
		case STATUS_PRINTER_LOW_PHOTO_YELLOW_INK:
		case STATUS_PRINTER_LOW_PHOTO_GRAY_INK:
		case STATUS_PRINTER_LOW_PHOTO_BLUE_INK:
		case STATUS_PRINTER_LOW_CYAN_TONER:
		case STATUS_PRINTER_LOW_MAGENTA_TONER:
		case STATUS_PRINTER_LOW_YELLOW_TONER:
		case STATUS_PRINTER_LOW_TONER:
		case STATUS_PRINTER_LOW_BLACK_TONER:
			return StatusPrinter.LOW_INK;

		case STATUS_PRINTER_TRAY_2_OUT_OF_PAPER:
		case STATUS_PRINTER_OUT_OF_PAPER:
		case STATUS_PRINTER_LOW_PAPER:
			return StatusPrinter.OUT_OF_PAPER;

		case STATUS_PRINTER_OFFLINE:
			return StatusPrinter.OFFLINE;

		case STATUS_PRINTER_OUT_OF_INK:
		case STATUS_PRINTER_NO_TONER:
			return StatusPrinter.OUT_OF_INK;

		case STATUS_DISCONNECTED:
			return StatusPrinter.DISCONNECTED;

		default:
			return StatusPrinter.READY;
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.smartmatic.waffle.ecm.core.printer.IPrinterStatusQuery#getDeviceState
	 * ()
	 */
	@Override
	public DeviceState getDeviceState() {
		switch (this.deviceRawState) {
		case DEVICE_STATE_FOUND:
			return DeviceState.FOUND;

		case DEVICE_STATE_JUST_FOUND:
			return DeviceState.JUST_FOUND;

		case DEVICE_STATE_NOT_FOUND:
			return DeviceState.NOT_FOUND;

		case DEVICE_STATE_DISCONNECTED:
			return DeviceState.DISCONNECTED;

		default:
			return null;
		}
	}

	/**
	 * Getter of <code>inkLevelPercent</code> property.
	 * 
	 * @return <code>inkLevelPercent</code> property value.
	 * @see #inkLevelPercent
	 */
	@Override
	public int getInkLevelPercent() {
		return this.inkLevelPercent;
	}

	/**
	 * Getter of <code>statusCode</code> property.
	 * 
	 * @return <code>statusCode</code> property value.
	 * @see #statusCode
	 */
	public int getStatusCode() {
		return this.statusCode;
	}

	/**
	 * Getter of <code>deviceRawState</code> property.
	 * 
	 * @return <code>deviceRawState</code> property value.
	 * @see #deviceRawState
	 */
	public int getDeviceRawState() {
		return this.deviceRawState;
	}

	/**
	 * Getter of <code>statusCodeDesc</code> property.
	 * 
	 * @return <code>statusCodeDesc</code> property value.
	 * @see #statusCodeDesc
	 */
	public String getStatusDesc() {
		return this.statusDesc;
	}

	/**
	 * Getter of <code>hpInfoOutput</code> property.
	 * 
	 * @return <code>hpInfoOutput</code> property value.
	 * @see #hpInfoOutput
	 */
	public List<String> getHpInfoOutput() {
		return this.hpInfoOutput;
	}

	/**
	 * Gets the hp info output string.
	 * 
	 * @return the hp info output string
	 */
	public String getHpInfoOutputString() {
		return StringUtils.join(this.hpInfoOutput, SystemUtils.LINE_SEPARATOR);
	}

}