/**
 * PrinterWatcher.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.core.printer;

import java.awt.Component;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.JOptionPane;

import com.smartmatic.waffle.ecm.core.printer.IPrinterStatusQuery.DeviceState;
import com.smartmatic.waffle.ecm.core.printer.IPrinterStatusQuery.StatusPrinter;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Class used to validate that the printer is plugged.
 * 
 * @version 1.0 (Apr 11, 2011)
 * @author Fernando Alvarez (fernando.alvarez@smartmatic.com)
 */
public class PrinterWatcher implements IPrinterStatusChangedListener {

	/** For logging purposes. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(PrinterWatcher.class);

	/**
	 * The singleton Instance.
	 */
	private static PrinterWatcher singleton;

	/** The printer. */
	private PrinterStatus printer;

	/**
	 * Default private constructor.
	 * 
	 * @throws IOException
	 *             Signals that an I/O exception of some sort has occurred. This
	 *             class is the general class of exceptions produced by failed
	 *             or interrupted I/O operations.
	 */
	private PrinterWatcher() throws IOException {
		this.printer = new PrinterStatus();
	}

	/** The clients. */
	private List<IPrinterStatusChangedNotifierListener> clients = new ArrayList<IPrinterStatusChangedNotifierListener>();

	/** The last pinter status. */
	private String lastPinterStatus;

	/**
	 * Gets the printer name.
	 * 
	 * @return the printer name
	 */
	public String getPrinterName() {
		return this.printer.getPrinterName();
	}

	/**
	 * Gets the printer url.
	 * 
	 * @return the printer url
	 */
	public String getPrinterUrl() {
		return this.printer.getPrinterUrl();
	}

	/**
	 * Gets the device state.
	 * 
	 * @return the device state
	 */
	public DeviceState getDeviceState() {
		return this.printer.getDeviceState();
	}

	/**
	 * Gets the status printer.
	 * 
	 * @return the status printer
	 */
	public StatusPrinter getStatusPrinter() {
		return this.printer.getPrinterStatus();
	}

	/**
	 * Gets the ink level percent.
	 * 
	 * @return the ink level percent
	 */
	public int getInkLevelPercent() {
		return this.printer.getInkLevelPercent();
	}

	/**
	 * Checks if is device enabled.
	 * 
	 * @return the boolean
	 */
	public Boolean isDeviceEnabled() {
		return this.printer.isEnabled();
	}

	/**
	 * Adds the listener client.
	 * 
	 * @param client
	 *            the client
	 */
	public void addListenerClient(IPrinterStatusChangedNotifierListener client) {
		if (!this.clients.contains(client)) {
			this.clients.add(client);
			this.printer.addListener(this);
		}
	}

	/**
	 * Gets the current status.
	 * 
	 * @return the current status
	 */
	public String getCurrentStatus() {
		if (isDeviceEnabled() != null && isDeviceEnabled() == false) {
			return DeviceState.DISCONNECTED.toString();
		} else {
			return this.printer.getPrinterStatus().toString();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.smartmatic.waffle.ecm.core.printer.IPrinterStatusChangedListener
	 * #printerStatusChanged(java.util.Map)
	 */
	@Override
	public void printerStatusChanged(Map<String, Object> changed) {

		DeviceState deviceState = null;

		StatusPrinter statusPrinter = null;

		if (changed.containsKey("statusCode")) {
			statusPrinter = (StatusPrinter) changed.get("statusCode");
		}

		if (changed.containsKey("deviceState")) {
			deviceState = (DeviceState) changed.get("deviceState");
		} else {
			deviceState = this.printer.getDeviceState();
		}

		String status = null;

		for (Iterator<IPrinterStatusChangedNotifierListener> iterator = this.clients
				.iterator(); iterator.hasNext();) {

			IPrinterStatusChangedNotifierListener iPrinterStatusChangeListener = iterator
					.next();

			if (this.lastPinterStatus != null
					&& this.lastPinterStatus.equals(StatusPrinter.DOOR_OPEN
							.toString())
					&& this.printer.getPrinterStatus().equals(
							StatusPrinter.OFFLINE)) {
				status = StatusPrinter.BUSY.toString();

			} else if (statusPrinter == null
					&& deviceState.toString().equals(
							DeviceState.JUST_FOUND.toString())) {
				status = StatusPrinter.READY.toString();
			} else if (statusPrinter == null
					&& deviceState.toString().equals(
							DeviceState.DISCONNECTED.toString())) {
				status = StatusPrinter.DISCONNECTED.toString();
			} else if (statusPrinter != null) {
				status = statusPrinter.toString();
			}

			this.lastPinterStatus = status;

			iPrinterStatusChangeListener.notifiesPrinterStatusChanged(status);
		}
	}

	/**
	 * Print plugged check.
	 * 
	 * @param parent
	 *            the parent
	 * @param messages
	 *            the messages
	 * @return the int
	 */
	public int checkPrinterPlugged(Component parent, String... messages) {
		int result = 1;
		String[] optionsKey = new String[2];
		System.arraycopy(messages, 2, optionsKey, 0, optionsKey.length);
		DeviceState state;
		while ((state = this.printer.getDeviceState())
				.equals(DeviceState.NOT_FOUND)) {
			logger.info("Current state of printer: [" + state.name() + "]");
			result = JOptionPane.showOptionDialog(parent, messages[0],
					messages[1], JOptionPane.YES_NO_OPTION,
					JOptionPane.QUESTION_MESSAGE, null, optionsKey, null);
			if (result == 0) {
				return result;
			}
		}
		return result;
	}

	/**
	 * Check printer plugged.
	 * 
	 * @return true, if successful
	 */
	public boolean checkPrinterPlugged() {
		DeviceState state = this.printer.getDeviceState();
		return !state.equals(DeviceState.NOT_FOUND);
	}

	/**
	 * Checks if is avalaible printer.
	 * 
	 * @return true, if is avalaible printer
	 */
	public boolean isAvalaiblePrinter() {
		DeviceState state = this.printer.getDeviceState();
		StatusPrinter status = this.printer.getPrinterStatus();
		logger.info("Diagnosis = Current state :[ " + state.name()
				+ " ] and status [" + status.name() + " ] of printer");
		return (!state.equals(DeviceState.NOT_FOUND) && status
				.equals(StatusPrinter.READY));
	}

	/***
	 * Method used to get the singleton instance.
	 * 
	 * @return the singleton instance.
	 * @throws IOException
	 *             Signals that an I/O exception of some sort has occurred. This
	 *             class is the general class of exceptions produced by failed
	 *             or interrupted I/O operations.
	 */
	public static PrinterWatcher getInstance() throws IOException {
		if (singleton == null) {
			singleton = new PrinterWatcher();
		}
		return singleton;
	}

	/**
	 * Gets the printer status.
	 * 
	 * @return the printer status
	 */
	public PrinterStatus getPrinterStatus() {
		return singleton.printer;
	}

}