/**
 * BackupDatabaseUtil.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.core.util;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;

import com.smartmatic.waffle.ecm.core.tar.gz.SAESTARGZUtil;
import com.smartmatic.waffle.log.SMTTLogger;

/**
 * Class used to perform a backup of local database.
 * 
 * @version 1.0 (05/06/2011)
 * @author Hector Rodriguez <hector.rodriguez@smartmatic.com>
 */
public class BackupDatabaseUtil {

	/** Property used to log events of the application. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(BackupDatabaseUtil.class);

	/**
	 * Create a file that contains the electoral authority code for verification
	 * purpose.
	 * 
	 * @param backupDir
	 *            the backup dir
	 * @param authorityCode
	 *            the authority code
	 * @return the file
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	private static File createAuthorityCodeFile(File backupDir,
			Long authorityCode) throws IOException {
		File electoralAuthorityCodeFile = new File(backupDir, "code");
		FileUtils.writeStringToFile(electoralAuthorityCodeFile,
				String.valueOf(authorityCode));
		return electoralAuthorityCodeFile;
	}

	/**
	 * Zip backup file.
	 * 
	 * @param destinationFiles
	 *            the destination files
	 * @return the byte[]
	 */
	private static byte[] zipBackupFile(File... destinationFiles) {
		byte[] dataZipped = null;
		try {
			// added created files on tar file
			dataZipped = SAESTARGZUtil.tarFile(destinationFiles);
		} catch (Exception e) {
			logger.error(
					"There was an error trying to create zipped backup file ",
					e);
		} finally {
			// delete stored files from file system
			for (File file : destinationFiles) {
				if (file != null && file.exists()) {
					file.delete();
				}
			}
		}
		// return compressed data
		return dataZipped;

	}

	/**
	 * Creates the backup file.
	 * 
	 * @param nameDb
	 *            the name db
	 * @param backupDir
	 *            the backup dir
	 * @param hostName
	 *            the host name
	 * @return the file
	 */
	private static File createBackupFile(String nameDb, File backupDir,
			String hostName) {
		File destinationFile = new File(backupDir, "backup.sql");
		Process execute = null;

		ArrayList<String> params = new ArrayList<String>();
		params.add("-Fc");
		params.add("-U");
		params.add("ecm");
		params.add(nameDb);

		if (hostName != null) {
			params.add("-h");
			params.add(hostName);
		}
		params.add(">");
		params.add(destinationFile.getAbsolutePath());

		try {
			// create SQL backup file
			execute = SAESCommandUtil.execute("pg_dump",
					params.toArray(new String[0]));
			SAESCommandUtil.execute("sync");
			return destinationFile;
		} catch (Exception e) {
			logger.error(e, e.getMessage());
		} finally {
			if (execute != null) {
				execute.destroy();
			}
		}
		return null;
	}

	/**
	 * Execute pg restore.
	 * 
	 * @param backupFile
	 *            the backup file
	 * @param dataBaseName
	 *            the data base name
	 */
	public static void executePgRestore(File backupFile, String dataBaseName) {
		executePgRestore(backupFile, dataBaseName, null);
	}

	/**
	 * Execute pg restore.
	 * 
	 * @param backupFile
	 *            the backup file
	 * @param dataBaseName
	 *            the data base name
	 * @param databaseServerHostname
	 *            the database server hostname
	 */
	public static void executePgRestore(File backupFile, String dataBaseName,
			String databaseServerHostname) {
		File destinationFile = new File(backupFile, "backup.sql");
		Process execute = null;
		try {
			String[] params;
			if (databaseServerHostname != null
					&& !databaseServerHostname.isEmpty()) {
				params = new String[] { "-c", "-U", "ecm", "-d", dataBaseName,
						"-w", "-v", "-h", databaseServerHostname,
						destinationFile.getAbsolutePath(), ">/dev/null",
						" 2>&1" };
			} else {
				params = new String[] { "-c", "-U", "ecm", "-d", dataBaseName,
						"-w", "-v", destinationFile.getAbsolutePath(),
						">/dev/null", " 2>&1" };
			}
			execute = SAESCommandUtil.execute("/usr/bin/pg_restore", params);

			String aux = IOUtils.toString(execute.getInputStream());

			if (aux != null && !aux.isEmpty()) {
				logger.info(aux);
			}
		} catch (Exception e) {
			logger.error(e, e.getMessage());
		} finally {
			if (execute != null) {
				execute.destroy();
			}
		}
	}

	/**
	 * Creates the backup tmp.
	 * 
	 * @param backupDir
	 *            the backup dir
	 * @param dataBaseName
	 *            the data base name
	 */
	public static void createBackupTmp(File backupDir, String dataBaseName) {
		createBackupFile(dataBaseName, backupDir, null);
	}

	/**
	 * Creates the backup tmp.
	 * 
	 * @param backupDir
	 *            the backup dir
	 * @param dataBaseName
	 *            the data base name
	 * @param hostName
	 *            the host name
	 */
	public static void createBackupTmp(File backupDir, String dataBaseName,
			String hostName) {
		createBackupFile(dataBaseName, backupDir, hostName);
	}

	/**
	 * Creates the local backup.
	 * 
	 * @param backupDir
	 *            the backup dir
	 * @param authorityCode
	 *            the authority code
	 * @return the byte[]
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	public static byte[] createLocalBackup(File backupDir, Long authorityCode)
			throws IOException {
		ArrayList<File> listOfFilesToZip = new ArrayList<File>();

		File destinationFile = createBackupFile("ecm_db", backupDir, null);
		listOfFilesToZip.add(destinationFile);
		if (authorityCode != null) {
			// create authority code file, for verification purpose
			File authorityCodeFile = createAuthorityCodeFile(
					destinationFile.getParentFile(), authorityCode);
			listOfFilesToZip.add(authorityCodeFile);
		}

		byte[] zipped = zipBackupFile(listOfFilesToZip.toArray(new File[0]));
		return zipped;
	}

	/**
	 * Creates the backup.
	 * 
	 * @param nameDb
	 *            the name db
	 * @param backupDir
	 *            the backup dir
	 * @return the byte[]
	 */
	public static byte[] createBackup(String nameDb, File backupDir) {
		File destinationFile = createBackupFile(nameDb, backupDir, null);
		byte[] zipped = zipBackupFile(destinationFile);
		return zipped;
	}

	/**
	 * Creates the backup.
	 * 
	 * @param nameDb
	 *            the name db
	 * @param backupDir
	 *            the backup dir
	 * @param databaseServerHostname
	 *            the database server hostname
	 * @return the byte[]
	 */
	public static byte[] createBackup(String nameDb, File backupDir,
			String databaseServerHostname) {
		File destinationFile = createBackupFile(nameDb, backupDir,
				databaseServerHostname);
		byte[] zipped = zipBackupFile(destinationFile);
		return zipped;
	}
}
