/**
 * BrowserHierarchyTreeDTO.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.data.transfer.objects;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import com.smartmatic.waffle.ecm.data.transfer.type.BrowserHierarchyStatusType;

/**
 * Class used to transfer information needed by any layer about each node in the
 * hierarchy tree for region or contest browser.
 */
public class BrowserHierarchyTreeDTO {

    /***
     * The node list selected by the user.
     */
    private final Set<BrowserHierarchyDTO> selectedNodes;

    /***
     * The root node.
     */
    private BrowserHierarchyDTO node;

    /***
     * Class constructor.
     */
    public BrowserHierarchyTreeDTO() {
        this.selectedNodes = new HashSet<BrowserHierarchyDTO>();
    }

    /**
     * Getter of {@link #node} field.
     * 
     * @return the node - The root node.
     */
    public BrowserHierarchyDTO getNode() {
        return this.node;
    }

    /**
     * Sets the * The root node.
     * 
     * @param node
     *            the node to set
     */
    public void setNode(BrowserHierarchyDTO node) {
        this.node = node;
    }

    /***
     * The method is used to check if all children have been loaded.
     * 
     * @param parent
     *            - the parent node of children.<br>
     *            The parent cannot be null.
     * @return The method return true if all children have been loaded or false
     *         otherwise.
     */
    public boolean isChildrenLoaded(BrowserHierarchyDTO parent) {
        if (parent == null) {
            throw new IllegalArgumentException("The parent cannot be null");
        }
        return (parent.getChildElements() != null)
                && (!parent.getChildElements().isEmpty());
    }

    /***
     * The method returns all children of a parent.
     * 
     * @param parent
     *            - the parent node of children.<br>
     *            The parent cannot be null.
     * @return the list children.
     */
    public Set<BrowserHierarchyDTO> getChildren(BrowserHierarchyDTO parent) {
        if (parent == null) {
            throw new IllegalArgumentException("The parent node cannot be null");
        }
        return parent.getChildElements();
    }

    /***
     * The method is used to check if the parent of a node have been loaded.
     * 
     * @param child
     *            - the child node.<br>
     *            The child cannot be null.
     * @return The method return true if the parent node have been loaded or
     *         false otherwise.
     */
    public boolean isParentLoaded(BrowserHierarchyDTO child) {
        if (child == null) {
            throw new IllegalArgumentException("The child node cannot be null");
        }
        return (child.getParentElement() != null);
    }

    /***
     * The method returns the parent node of a child node.
     * 
     * @param child
     *            - The child node.<br>
     *            The child cannot be null.
     * @return the method returns the parent node.
     */
    public BrowserHierarchyDTO getParent(BrowserHierarchyDTO child) {
        if (child == null) {
            throw new IllegalArgumentException("The child node cannot be null");
        }
        return child.getParentElement();
    }

    /***
     * the method rturns the First level nodes.
     * 
     * @return -the First level nodes.
     */
    public Set<BrowserHierarchyDTO> getFirstNode() {
        return this.node.getChildElements();
    }

    /***
     * Method used to get the current status of the parent of the child node.
     * 
     * @param child
     *            - the child node.
     * @return The current status of the parent.
     */
    public BrowserHierarchyStatusType getParentStatus(BrowserHierarchyDTO child) {
        if (this.isParentLoaded(child)) {// if the parent have been loaded.
            BrowserHierarchyDTO parent = this.getParent(child);
            if (parent.getCurrentStatus() != null) {// if the parent has a
                                                    // loaded status.
                return parent.getCurrentStatus();
            } else {// if the parent has not a
                // loaded status.

                BrowserHierarchyDTO grandParent = this.getParent(child);// get
                                                                        // the
                                                                        // grandParent.

                if (grandParent != null
                        && BrowserHierarchyStatusType.PARTIAL_SELECTED
                                .equals(grandParent.getCurrentStatus())) {
                    return BrowserHierarchyStatusType.UNSELECTED;
                } else {
                    return this.getParentStatus(parent);
                }

            }
        } else {
            return null;
        }
    }

    /***
     * Method used to get the current status of a node.
     * 
     * @param node
     *            - the current node.
     * @return - the current status of the node.
     */
    public BrowserHierarchyStatusType getCurrentStatus(BrowserHierarchyDTO node) {
        if (node.getCurrentStatus() != null) {
            return node.getCurrentStatus();
        } else {
            BrowserHierarchyDTO parent = this.getParent(node);
            if (parent != null
                    && BrowserHierarchyStatusType.PARTIAL_SELECTED
                            .equals(parent.getCurrentStatus())) {
                return BrowserHierarchyStatusType.UNSELECTED;
            } else {
                return this.getParentStatus(node);
            }
        }
    }

    /**
     * *.
     * 
     * @param parent
     *            the parent
     * @param nodeToExclude
     *            the node to exclude
     * @return true, if is all children un selected
     */
    public boolean isAllChildrenUnSelected(BrowserHierarchyDTO parent,
            List<BrowserHierarchyDTO> nodeToExclude) {
        boolean isAllChildrenUnSelected = true;
        if (this.isChildrenLoaded(parent)) {
            Set<BrowserHierarchyDTO> childern = this.getChildren(parent);
            for (BrowserHierarchyDTO current : childern) {
                if (!BrowserHierarchyStatusType.UNSELECTED
                        .equals(getCurrentStatus(current))) {
                    isAllChildrenUnSelected = false;
                    break;
                }
            }
            return isAllChildrenUnSelected;
        }
        return true;
    }

    /**
     * *.
     * 
     * @param parent
     *            the parent
     * @param nodeToExclude
     *            the node to exclude
     * @return true, if is all children selected
     */
    public boolean isAllChildrenSelected(BrowserHierarchyDTO parent,
            List<BrowserHierarchyDTO> nodeToExclude) {
        boolean isAllChildrenUnSelected = true;
        if (this.isChildrenLoaded(parent)) {
            Set<BrowserHierarchyDTO> childern = this.getChildren(parent);
            for (BrowserHierarchyDTO current : childern) {
                if (!BrowserHierarchyStatusType.FULL_SELECTED
                        .equals(getCurrentStatus(current))) {
                    isAllChildrenUnSelected = false;
                    break;
                }

            }
            return isAllChildrenUnSelected;
        }
        return true;
    }

    /**
     * *.
     * 
     * @param parent
     *            the parent
     * @param nodeToExclude
     *            the node to exclude
     * @param finalStatus
     *            the final status
     * @param initialStatus
     *            the initial status
     */
    public void updateChildrenStatus(BrowserHierarchyDTO parent,
            List<BrowserHierarchyDTO> nodeToExclude,
            BrowserHierarchyStatusType finalStatus,
            BrowserHierarchyStatusType initialStatus) {
        if (((parent != null) && (initialStatus != null) && (!BrowserHierarchyStatusType.PARTIAL_SELECTED
                .equals(initialStatus)))) {
            if (this.isChildrenLoaded(parent)) {
                Set<BrowserHierarchyDTO> children = this.getChildren(parent);
                for (BrowserHierarchyDTO current : children) {
                    if (!nodeToExclude.contains(current)) {
                        if (BrowserHierarchyStatusType.PARTIAL_SELECTED
                                .equals(finalStatus)) {
                            current.setCurrentStatus(initialStatus);
                            if (BrowserHierarchyStatusType.FULL_SELECTED
                                    .equals(current.getCurrentStatus())) {
                                this.addSelectedNode(current);
                                // this.selectedNodes.add(current);
                            } else {
                                this.removeSelectedNode(current);
                                // this.selectedNodes.remove(current);
                            }
                        } else {
                            current.setCurrentStatus(null);
                        }
                    }
                }
            }
        }
    }

    /**
     * *.
     * 
     * @param node
     *            the node
     */
    public void updateParentStatus(BrowserHierarchyDTO node) {
        BrowserHierarchyDTO parent = this.getParent(node);
        List<BrowserHierarchyDTO> nodeToExclude = new ArrayList<BrowserHierarchyDTO>();
        nodeToExclude.add(node);
        if (node.getCurrentStatus().equals(
                BrowserHierarchyStatusType.UNSELECTED)) {

            if (this.isParentLoaded(node)) {
                if (this.isAllChildrenUnSelected(parent, nodeToExclude)) {
                    parent.setCurrentStatus(BrowserHierarchyStatusType.UNSELECTED);
                    this.removeSelectedNode(parent);
                    // this.selectedNodes.remove(parent);
                } else {
                    parent.setCurrentStatus(BrowserHierarchyStatusType.PARTIAL_SELECTED);
                    this.removeSelectedNode(parent);
                    // this.selectedNodes.remove(parent);
                }
                this.updateParentStatus(parent);
            }
        } else {
            if (this.isParentLoaded(node)) {
                if (this.isAllChildrenSelected(parent, nodeToExclude)) {
                    parent.setCurrentStatus(BrowserHierarchyStatusType.FULL_SELECTED);
                    addSelectedNode(parent);
                    // this.selectedNodes.add(parent);
                } else {
                    parent.setCurrentStatus(BrowserHierarchyStatusType.PARTIAL_SELECTED);
                    this.removeSelectedNode(parent);
                    // this.selectedNodes.remove(parent);
                }
                this.updateParentStatus(parent);
            }
        }
    }

    /**
     * *.
     * 
     * @param node
     *            the node
     */
    public void updateCurrentStatus(BrowserHierarchyDTO node) {
        if (node.getCurrentStatus() == null) {// The current node not have an
                                              // status
            if (this.isParentLoaded(node)) {// the node isn't First level node.
                BrowserHierarchyStatusType parentStatus = this
                        .getParentStatus(node);
                if (parentStatus == null
                        || parentStatus
                                .equals(BrowserHierarchyStatusType.UNSELECTED)
                        || parentStatus
                                .equals(BrowserHierarchyStatusType.PARTIAL_SELECTED)) {// the
                    // current
                    // node
                    // is
                    // unselected
                    node.setCurrentStatus(BrowserHierarchyStatusType.FULL_SELECTED);
                    this.addSelectedNode(node);
                    // this.selectedNodes.add(node);
                } else {// the current node is selected
                    node.setCurrentStatus(BrowserHierarchyStatusType.UNSELECTED);
                    this.removeSelectedNode(node);
                    // this.selectedNodes.remove(node);
                }

            } else {// the node is First level node.
                node.setCurrentStatus(BrowserHierarchyStatusType.FULL_SELECTED);
                this.addSelectedNode(node);
                // this.selectedNodes.add(node);
            }

        } else if (node.getCurrentStatus().equals(
                BrowserHierarchyStatusType.UNSELECTED)) {// //the current node
                                                         // is
                                                         // unselected
            node.setCurrentStatus(BrowserHierarchyStatusType.FULL_SELECTED);
            this.addSelectedNode(node);
            // this.selectedNodes.add(node);
        } else {// the current node is selected
            node.setCurrentStatus(BrowserHierarchyStatusType.UNSELECTED);
            this.removeSelectedNode(node);
            // this.selectedNodes.remove(node);
        }
        BrowserHierarchyStatusType initialStatus = this.getParentStatus(node);
        updateParentStatus(node);
        BrowserHierarchyStatusType finalStatus = this.getParentStatus(node);
        BrowserHierarchyDTO parent = this.getParent(node);
        List<BrowserHierarchyDTO> listToExclude = new ArrayList<BrowserHierarchyDTO>();
        listToExclude.add(node);
        // Revisar en este metodo si debo o no enviar el nodo actual para ser
        // excluido de la actualizacion.

        // Debo revisar la condicion bajo la cual no se deben actualizar los
        // nodos hijos
        this.updateChildrenStatus(parent, listToExclude, finalStatus,
                initialStatus);
        updateStatusToAllChildern(node);
    }

    /**
     * *.
     * 
     * @param node
     *            the node
     */
    private void updateStatusToAllChildern(BrowserHierarchyDTO node) {
        if (isChildrenLoaded(node)) {
            Set<BrowserHierarchyDTO> children = this.getChildren(node);
            for (BrowserHierarchyDTO current : children) {
                if (BrowserHierarchyStatusType.UNSELECTED.equals(node
                        .getCurrentStatus())) {
                    current.setCurrentStatus(BrowserHierarchyStatusType.UNSELECTED);
                    this.removeSelectedNode(current);
                    // this.selectedNodes.remove(current);
                } else {
                    current.setCurrentStatus(null);
                }
                updateStatusToAllChildern(current);
            }
        }
    }

    /**
     * *.
     * 
     * @return the * The node list selected by the user
     */
    public Set<BrowserHierarchyDTO> getSelectedNodes() {
        return this.selectedNodes;
    }

    /**
     * Adds the selected node.
     * 
     * @param node
     *            the node
     */
    public void addSelectedNode(BrowserHierarchyDTO node) {
        this.selectedNodes.add(node);
    }

    /**
     * Adds the all selected node.
     * 
     * @param nodes
     *            the nodes
     */
    public void addAllSelectedNode(Set<BrowserHierarchyDTO> nodes) {
        this.selectedNodes.addAll(nodes);
    }

    /**
     * Removes the selected node.
     * 
     * @param node
     *            the node
     */
    public void removeSelectedNode(BrowserHierarchyDTO node) {
        if (this.selectedNodes.contains(node)) {
            this.selectedNodes.remove(node);
        }
    }

    /**
     * *.
     */
    public void selectedAllCurrentNodes() {
        if ((this.selectedNodes != null) && (this.selectedNodes.isEmpty())) {
            this.selectALlChildNodes(this.node);
        } else {
            this.unSelectALlChildNodes(this.node);
        }

    }

    /**
     * *.
     * 
     * @param parentNode
     *            the parent node
     */
    public void selectALlChildNodes(BrowserHierarchyDTO parentNode) {
        BrowserHierarchyStatusType newStatus;
        this.unSelectALlChildNodes(parentNode);
        newStatus = BrowserHierarchyStatusType.FULL_SELECTED;
        Set<BrowserHierarchyDTO> nodeList = parentNode.getChildElements();
        for (BrowserHierarchyDTO current : nodeList) {
            current.setCurrentStatus(newStatus);
        }
        this.addAllSelectedNode(parentNode.getChildElements());
        // this.selectedNodes.addAll(parentNode.getChildElements());
    }

    /**
     * *.
     * 
     * @param parentNode
     *            the parent node
     */
    public void unSelectALlChildNodes(BrowserHierarchyDTO parentNode) {
        BrowserHierarchyStatusType newStatus = null;
        this.setNewStatusToAllChildNodes(parentNode, newStatus);
        this.selectedNodes.clear();
    }

    /**
     * *.
     * 
     * @param currentNode
     *            the current node
     * @param newStatus
     *            the new status
     */
    private void setNewStatusToAllChildNodes(BrowserHierarchyDTO currentNode,
            BrowserHierarchyStatusType newStatus) {
        if (currentNode == null) {
            throw new IllegalArgumentException("The currentNode cannot be null");
        }
        if (this.isChildrenLoaded(currentNode)) {
            Set<BrowserHierarchyDTO> nodeList = currentNode.getChildElements();
            for (BrowserHierarchyDTO current : nodeList) {
                current.setCurrentStatus(newStatus);
                this.setNewStatusToAllChildNodes(current, newStatus);
            }
        }
    }

    /**
     * Clear.
     */
    public void clear() {
        this.selectedNodes.clear();
        this.node = null;
    }

    /**
     * *.
     */
    public void refreshSelectedNodes() {
        this.selectedNodes.clear();
        addselectedNodesIntoList(this.node);
    }

    /**
     * *.
     * 
     * @param parent
     *            the parent
     */
    private void addselectedNodesIntoList(BrowserHierarchyDTO parent) {
        Set<BrowserHierarchyDTO> child = parent.getChildElements();
        for (BrowserHierarchyDTO current : child) {
            if (BrowserHierarchyStatusType.FULL_SELECTED.equals(current
                    .getCurrentStatus())) {
                addSelectedNode(current);
                // this.selectedNodes.add(current);
            } else if (BrowserHierarchyStatusType.PARTIAL_SELECTED
                    .equals(current.getCurrentStatus())) {
                addselectedNodesIntoList(current);
            }
        }
    }
}
