/**
 * ECMElectoralPhysicalLocationsRegistryServiceImpl.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.service.impl;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.security.PublicKey;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import javax.ejb.EJB;
import javax.ejb.Local;
import javax.ejb.Remote;
import javax.ejb.Stateless;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.domain.demographics.PollingPlaceData;
import com.smartmatic.waffle.domain.demographics.RegionData;
import com.smartmatic.waffle.domain.device.VotingDeviceData;
import com.smartmatic.waffle.domain.eligibleoption.ImageWrapper;
import com.smartmatic.waffle.ecm.data.transfer.objects.VotingDeviceSecurityComponentsDTO;
import com.smartmatic.waffle.ecm.service.lang.ElectoralOrganizationOperationException;
import com.smartmatic.waffle.ecm.service.lang.ElectoralPhysicalLocationsRegistryException;
import com.smartmatic.waffle.ecm.service.lang.ElectoralPhysicalLocationsRegistryException.ElectoralPhysicalLocationsRegistryExceptionInfo;
import com.smartmatic.waffle.ecm.service.spec.IECMElectoralOrganizationServiceLocal;
import com.smartmatic.waffle.ecm.service.spec.IECMElectoralPhysicalLocationsRegistryService;
import com.smartmatic.waffle.ecm.service.spec.IECMElectoralPhysicalLocationsRegistryServiceLocal;
import com.smartmatic.waffle.log.SMTTLogger;
import com.smartmatic.waffle.persistence.search.JoinType;
import com.smartmatic.waffle.persistence.search.SearchExpression;
import com.smartmatic.waffle.persistence.search.criteria.CriteriaFactory;
import com.smartmatic.waffle.persistence.search.criteria.OrderCriteria;
import com.smartmatic.waffle.persistence.search.transformation.TransformationFactory;
import com.smartmatic.waffle.persistence.search.transformation.TransformationsList;
import com.smartmatic.waffle.persistence.service.spec.IPersistenceServiceLocal;
import com.smartmatic.waffle.properties.ApplicationProperties;
import com.smartmatic.waffle.security.asymmetric.AsymmetricKeyManager;
import com.smartmatic.waffle.security.lang.CryptographicStaticException;

/**
 * Session bean that handles all server side operations related with Electoral
 * Physical Locations Registry. This class implements all the server services
 * defined in <code>IECMElectoralPhysicalLocationsRegistryService</code>
 */
@Stateless(name = IECMElectoralPhysicalLocationsRegistryService.SERVICE_NAME)
@Remote(IECMElectoralPhysicalLocationsRegistryService.class)
@Local(IECMElectoralPhysicalLocationsRegistryServiceLocal.class)
public class ECMElectoralPhysicalLocationsRegistryServiceImpl implements
		IECMElectoralPhysicalLocationsRegistryServiceLocal {

	/** Object to generate logs during the application execution. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(ECMElectoralPhysicalLocationsRegistryServiceImpl.class);

	/**
	 * PersistenceServiceLocal Instance to do any database persistence
	 * operation.
	 */
	@EJB
	private IPersistenceServiceLocal persistenceService;

	/** The electoral organization service. */
	@EJB
	private IECMElectoralOrganizationServiceLocal electoralOrganizationService;

	/**
	 * Sets the persistenceServiceLocal Instance to do any database persistence
	 * operation.
	 * 
	 * @param persistenceService
	 *            the new persistenceServiceLocal Instance to do any database
	 *            persistence operation
	 */
	public void setPersistenceService(
			IPersistenceServiceLocal persistenceService) {
		this.persistenceService = persistenceService;
	}

	/**
	 * Sets the electoral organization service.
	 * 
	 * @param electoralOrganizationService
	 *            the new electoral organization service
	 */
	public void setElectoralOrganizationService(
			IECMElectoralOrganizationServiceLocal electoralOrganizationService) {
		this.electoralOrganizationService = electoralOrganizationService;
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public int retrieveVotingDevicesAmount()
			throws ElectoralPhysicalLocationsRegistryException {

		try {
			SearchExpression search = new SearchExpression(
					VotingDeviceData.class);

			return this.persistenceService.countObjects(search, null);
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new ElectoralPhysicalLocationsRegistryException(
					e,
					ElectoralPhysicalLocationsRegistryExceptionInfo.RETRIEVING_VOTING_DEVICES_AMOUNT_FAILURE,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * @inheritDoc
	 */
	@Override
	@SuppressWarnings("unchecked")
	public void setSecurityComponentsToVotingDevices(int pageNumber,
			int pageSize) throws ElectoralPhysicalLocationsRegistryException,
			ElectoralOrganizationOperationException {

		try {
			// comma separated characters for replacement in case a restricted
			// characters is generated in president passphrase generation
			char[] whiteListChars = ApplicationProperties
					.getValue("president.key.whitelist.characters").toString()
					.toLowerCase().toCharArray();

			SearchExpression search = new SearchExpression(
					VotingDeviceData.class);

			search.addOrder(OrderCriteria.asc("customCode"));

			List<VotingDeviceData> votingDeviceList = (List<VotingDeviceData>) this.persistenceService
					.searchPagedObjects(search, null, pageNumber, pageSize);

			for (Iterator<VotingDeviceData> iterator = votingDeviceList
					.iterator(); iterator.hasNext();) {
				VotingDeviceData device = iterator.next();

				generateVotingDevicesSecurityComponents(device, whiteListChars);

				this.persistenceService.update(device, null);

				logger.info("Processed " + device.getCustomCode());
			}

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new ElectoralPhysicalLocationsRegistryException(
					e,
					ElectoralPhysicalLocationsRegistryExceptionInfo.SETTING_SECURITY_COMPONENTS_TO_VOTING_DEVICES_FAILURE,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * @inheritDoc
	 */
	@Override
	@SuppressWarnings("unchecked")
	public List<VotingDeviceSecurityComponentsDTO> retrieveVotingDeviceSecurityComponents(
			int pageNumber, int pageSize)
			throws ElectoralPhysicalLocationsRegistryException {

		InputStream byteArrayInputStream = null;

		try {
			SearchExpression SearchExpression = new SearchExpression(
					VotingDeviceData.class);

			SearchExpression.createAlias("votingBooth", "votingBooth");
			SearchExpression.createAlias("pollingStation",
					"votingBooth.pollingStation");
			SearchExpression.createAlias("pollingPlace",
					"pollingStation.pollingPlace");
			SearchExpression.createAlias("region", "pollingPlace.region");
			SearchExpression.addOrder(OrderCriteria.asc("province.customCode"));
			SearchExpression.addOrder(OrderCriteria.asc("parents.customCode"));
			SearchExpression.addOrder(OrderCriteria.asc("customCode"));
			SearchExpression.createAlias("parents", "region.parent");
			SearchExpression.createAlias("arrond", "parents.parent",
					JoinType.LEFT_JOIN);
			SearchExpression.createAlias("province", "arrond.parent",
					JoinType.LEFT_JOIN);

			TransformationsList lightTransformation = TransformationFactory
					.transformationList();

			lightTransformation.add(TransformationFactory
					.distinctTransformation(TransformationFactory
							.propertyTransformation("code")));
			lightTransformation.add(TransformationFactory
					.propertyTransformation("customCode"));
			lightTransformation.add(TransformationFactory
					.propertyTransformation("name"));
			lightTransformation.add(TransformationFactory
					.propertyTransformation("protectionKey"));
			lightTransformation.add(TransformationFactory
					.propertyTransformation("transmissionKey"));
			lightTransformation.add(TransformationFactory
					.propertyTransformation("presidentKey"));
			lightTransformation.add(TransformationFactory
					.propertyTransformation("numericKey1"));
			lightTransformation.add(TransformationFactory
					.propertyTransformation("numericKey2"));
			lightTransformation.add(TransformationFactory
					.propertyTransformation("asymmetricKeys"));
			lightTransformation.add(TransformationFactory
					.propertyTransformation("recounterKey"));
			lightTransformation.add(TransformationFactory
					.propertyTransformation("province.customCode"));
			lightTransformation.add(TransformationFactory
					.propertyTransformation("parents.customCode"));

			SearchExpression.setTransformation(lightTransformation);

			Collection<Object[]> votingDeviceObjects = this.persistenceService
					.searchPagedObjects(SearchExpression, null, pageNumber,
							pageSize);

			List<VotingDeviceSecurityComponentsDTO> votingDeviceSecurityComponentsList = new ArrayList<VotingDeviceSecurityComponentsDTO>();

			for (Object[] votingDeviceObject : votingDeviceObjects) {
				String votingDeviceCustomCode = (String) votingDeviceObject[1];
				String votingDeviceName = (String) votingDeviceObject[2];
				String votingDeviceProtectionKey = (String) votingDeviceObject[3];
				String votingDeviceTransmissionKey = (String) votingDeviceObject[4];
				String votingDevicePresidentKey = (String) votingDeviceObject[5];
				String votingDeviceNumericKey1 = (String) votingDeviceObject[6];
				String votingDeviceNumericKey2 = (String) votingDeviceObject[7];
				ImageWrapper votingDeviceAsymmetricKey = (ImageWrapper) votingDeviceObject[8];
				String votingDeviceRecounterKey = (String) votingDeviceObject[9];

				byteArrayInputStream = new ByteArrayInputStream(
						votingDeviceAsymmetricKey.getBinaryData());

				PublicKey publicKey = AsymmetricKeyManager.getPublicKey(
						byteArrayInputStream, votingDeviceProtectionKey,
						"anyAlias", votingDeviceProtectionKey,
						AsymmetricKeyManager.KEYSTORE_PKCS12);

				String encodePrivateKey = Base64.encodeBase64String(publicKey
						.getEncoded());

				VotingDeviceSecurityComponentsDTO votingDeviceSecurityComponents = new VotingDeviceSecurityComponentsDTO(
						votingDeviceName, encodePrivateKey,
						votingDeviceProtectionKey, votingDeviceCustomCode,
						votingDeviceTransmissionKey, votingDevicePresidentKey,
						votingDeviceNumericKey1, votingDeviceNumericKey2,
						votingDeviceAsymmetricKey.getBinaryData(),
						votingDeviceRecounterKey);

				votingDeviceSecurityComponentsList
						.add(votingDeviceSecurityComponents);
			}

			return votingDeviceSecurityComponentsList;
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new ElectoralPhysicalLocationsRegistryException(
					e,
					ElectoralPhysicalLocationsRegistryExceptionInfo.RETRIEVING_VOTING_DEVICES_SECURITY_COMPONENTS_FAILURE,
					StringUtils.defaultString(e.getMessage()));
		} finally {
			if (byteArrayInputStream != null) {
				IOUtils.closeQuietly(byteArrayInputStream);
			}
		}
	}

	/**
	 * @inheritDoc
	 */
	@Override
	public int deleteVotingDeviceSecurityComponents()
			throws ElectoralPhysicalLocationsRegistryException {
		try {
			return this.persistenceService
					.executeUpdateNativeQuery("update voting_device set protection_key = null, transmission_key = null, asymmetric_key = null, president_key = null, numeric_key2 = null, numeric_key1 = null, recounter_key = null;");
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new ElectoralPhysicalLocationsRegistryException(
					e,
					ElectoralPhysicalLocationsRegistryExceptionInfo.DELETING_VOTING_DEVICES_SECURITY_COMPONENTS_FAILURE,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<VotingDeviceData> getVotingDevicesByRegion(RegionData region)
			throws ElectoralPhysicalLocationsRegistryException {

		SearchExpression searchVotingDevice = new SearchExpression(
				VotingDeviceData.class);

		searchVotingDevice.createAlias("votingBooth", "votingBooth");

		searchVotingDevice.createAlias("pollingStation",
				"votingBooth.pollingStation");

		searchVotingDevice.createAlias("pollingPlace",
				"pollingStation.pollingPlace");

		searchVotingDevice.addSearchCriteria(CriteriaFactory.eq(
				"pollingPlace.region", region));

		try {
			String path[] = { "votingBooth.pollingStation.pollingPlace.region.category.paths" };

			List<VotingDeviceData> votingDeviceList = (List<VotingDeviceData>) this.persistenceService
					.searchObjects(searchVotingDevice, path, null);

			return votingDeviceList;

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new ElectoralPhysicalLocationsRegistryException(
					e,
					ElectoralPhysicalLocationsRegistryExceptionInfo.GETTING_VOTING_DEVICES_BY_REGION_FAILURE,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<VotingDeviceData> getVotingDevicesByPollingPlace(
			PollingPlaceData pollingPlace)
			throws ElectoralPhysicalLocationsRegistryException {

		SearchExpression searchVotingDevice = new SearchExpression(
				VotingDeviceData.class);

		searchVotingDevice.createAlias("votingBooth", "votingBooth");

		searchVotingDevice.createAlias("pollingStation",
				"votingBooth.pollingStation");

		searchVotingDevice.addSearchCriteria(CriteriaFactory.eq(
				"pollingStation.pollingPlace", pollingPlace));

		try {
			String path[] = { "votingBooth.pollingStation.pollingPlace.region.category.paths" };

			List<VotingDeviceData> votingDeviceList = (List<VotingDeviceData>) this.persistenceService
					.searchObjects(searchVotingDevice, path, null);

			return votingDeviceList;

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new ElectoralPhysicalLocationsRegistryException(
					e,
					ElectoralPhysicalLocationsRegistryExceptionInfo.GETTING_VOTING_DEVICES_BY_REGION_FAILURE,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * Generates all keys needed by a voting device.
	 * 
	 * @param votingDevice
	 *            the voting device for which to generate all needed keys
	 * @param restrictedCharacters
	 *            comma separated restricted characters for president passphrase
	 * @param replacementCharacters
	 *            comma separated characters for replacement in case a
	 *            restricted characters is generated in president passphrase
	 *            generation
	 * @throws CryptographicStaticException
	 *             In case decryption fails because of inappropriate system
	 *             configuration. This exception implies that the server is not
	 *             properly configured. If this exception occurs the only way to
	 *             solve the problem is by reconfiguring the server.
	 * @throws ElectoralPhysicalLocationsRegistryException
	 *             the electoral physical locations registry exception
	 * @throws ElectoralOrganizationOperationException
	 *             the electoral organization operation exception
	 */
	private void generateVotingDevicesSecurityComponents(
			VotingDeviceData votingDevice, char[] whiteListChars)
			throws CryptographicStaticException,
			ElectoralPhysicalLocationsRegistryException,
			ElectoralOrganizationOperationException {

		String transmissionKey = ECMServerCommonImpl.generateRandomString(32);

		votingDevice.setTransmissionKey(transmissionKey);

		String protectionKey = ECMServerCommonImpl.generateSymmetricKey();

		votingDevice.setProtectionKey(protectionKey);

		ImageWrapper votingDeviceAsymmetricKey = generateAsymmetricKeyForVotingDevice(
				votingDevice.getCode(), votingDevice.getCustomCode(),
				protectionKey, "anyAlias");

		votingDevice.setAsymmetricKeys(votingDeviceAsymmetricKey);

		String presidentKey = generateRamdomPassword(whiteListChars);

		votingDevice.setPresidentKey(presidentKey);

		String recounterKey = generateRamdomPassword(whiteListChars);

		votingDevice.setRecounterKey(recounterKey);

		String numericKey1 = ECMServerCommonImpl.generateRandomNumericString(6);

		votingDevice.setNumericKey1(numericKey1);

		String numericKey2 = ECMServerCommonImpl.generateRandomNumericString(6);

		votingDevice.setNumericKey2(numericKey2);

	}

	private String generateRamdomPassword(char[] chars) {
		return ECMServerCommonImpl.generateRandomString(16, chars)
				.toLowerCase();
	}

	/**
	 * Generate asymmetric keys for a voting device.
	 * 
	 * @param votingDeviceCode
	 *            the voting device code used as a certificate serial number
	 * @param votingDeviceCustomCode
	 *            the voting device custom code used as a certificate common
	 *            name
	 * @param votingDeviceProtectionKey
	 *            the device protection key used as a certificate passphrase
	 * 
	 * @return the image wrapper object with the asymmetric key built for the
	 *         voting device.
	 * 
	 * @throws ElectoralPhysicalLocationsRegistryException
	 *             Signals that an exception has occurred retrieving signature
	 *             algorithm.
	 * @throws ElectoralOrganizationOperationException
	 *             Signals that an exception has occurred retrieving the active
	 *             election
	 */
	private ImageWrapper generateAsymmetricKeyForVotingDevice(
			long votingDeviceCode, String votingDeviceCustomCode,
			String votingDeviceProtectionKey, String votingDeviceAlias)
			throws ElectoralPhysicalLocationsRegistryException,
			ElectoralOrganizationOperationException {

		try {

			return ECMServerCommonImpl.generateAsymmetricKey(votingDeviceCode,
					votingDeviceCustomCode, votingDeviceProtectionKey,
					votingDeviceAlias, this.electoralOrganizationService
							.getElectionKeyStoreComponents());

		} catch (ElectoralOrganizationOperationException e) {
			throw e;
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new ElectoralPhysicalLocationsRegistryException(
					e,
					ElectoralPhysicalLocationsRegistryExceptionInfo.GENERATING_ASYMMETRIC_KEY_FOR_VOTING_DEVICE_FAILURE,
					StringUtils.defaultString(e.getMessage()));
		}
	}
}
