package com.smartmatic.waffle.ecm.service.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeMap;
import java.util.TreeSet;

import javax.ejb.EJB;
import javax.ejb.Local;
import javax.ejb.Remote;
import javax.ejb.Stateless;

import org.apache.commons.lang.StringUtils;

import com.smartmatic.waffle.domain.ballot.BallotOptionData;
import com.smartmatic.waffle.domain.contest.ContestClassData;
import com.smartmatic.waffle.domain.contest.ContestData;
import com.smartmatic.waffle.domain.contest.ContestViewData;
import com.smartmatic.waffle.domain.demographics.DistrictData;
import com.smartmatic.waffle.domain.demographics.RegionData;
import com.smartmatic.waffle.domain.device.LogicalVotingGroupData;
import com.smartmatic.waffle.domain.eligibleoption.CandidateData;
import com.smartmatic.waffle.domain.eligibleoption.EligibleOptionData;
import com.smartmatic.waffle.domain.eligibleoption.EligibleOptionRoleData;
import com.smartmatic.waffle.domain.eligibleoption.ImageWrapper;
import com.smartmatic.waffle.domain.eligibleoption.PartyData;
import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.domain.multiLanguage.TranslationKeyData;
import com.smartmatic.waffle.ecm.service.lang.NominationException;
import com.smartmatic.waffle.ecm.service.lang.NominationException.NominationExceptionInfo;
import com.smartmatic.waffle.ecm.service.spec.IECMNominationService;
import com.smartmatic.waffle.ecm.service.spec.IECMNominationServiceLocal;
import com.smartmatic.waffle.ecm.service.spec.IECMUtilitiesServiceLocal;
import com.smartmatic.waffle.lang.BaseException;
import com.smartmatic.waffle.log.SMTTLogger;
import com.smartmatic.waffle.persistence.search.SearchExpression;
import com.smartmatic.waffle.persistence.search.criteria.CriteriaFactory;
import com.smartmatic.waffle.persistence.search.criteria.ISearchCriteria;
import com.smartmatic.waffle.persistence.search.criteria.MatchCriteriaMode;
import com.smartmatic.waffle.persistence.search.criteria.OrderCriteria;
import com.smartmatic.waffle.persistence.search.transformation.AliasTransformationsMap;
import com.smartmatic.waffle.persistence.search.transformation.TransformationFactory;
import com.smartmatic.waffle.persistence.search.transformation.TransformationsList;
import com.smartmatic.waffle.persistence.service.spec.IPersistenceServiceLocal;

/**
 * Session beans that handles nomination domain server side operations.This
 * class implements all the server services defined in
 * <code>IECMNominationService</code>
 */
@Stateless(name = IECMNominationService.SERVICE_NAME)
@Remote(IECMNominationService.class)
@Local(IECMNominationServiceLocal.class)
public class ECMNominationServiceImpl implements IECMNominationServiceLocal {

	/** Object to generate logs during the application execution. */
	private static SMTTLogger logger = SMTTLogger
			.getLogger(ECMNominationServiceImpl.class);

	/**
	 * PersistenceServiceLocal Instance to do any database persistence
	 * operation.
	 */
	@EJB
	private IPersistenceServiceLocal persistenceService;

	@EJB
	private IECMUtilitiesServiceLocal ecmUtilitiesService;

	/**
	 * Sets the persistence service to be used by this class.
	 * 
	 * @param persistenceService
	 *            the persistence service class to be used.
	 */
	public void setPersistenceService(
			IPersistenceServiceLocal persistenceService) {
		this.persistenceService = persistenceService;
	}

	public void setECMUtilitiesService(
			IECMUtilitiesServiceLocal ecmUtilitiesService) {
		this.ecmUtilitiesService = ecmUtilitiesService;
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<EligibleOptionData> getCandidates(int pageNumber,
			int numberOfPages, String candidateName, List<RegionData> regions,
			List<ContestData> contests, List<PartyData> parties,
			LanguageData language) throws NominationException {
		SearchExpression search = new SearchExpression(EligibleOptionData.class);
		SearchExpression subQuery = new SearchExpression(
				EligibleOptionData.class);

		search.createAlias("eligibleOptionParent", "eligibleOptionParent");
		search.createAlias("eligibleOptionRoleData", "eligibleOptionRoleData");
		search.createAlias("candidate", "eligibleOptionRoleData.candidate");
		search.createAlias("ballotOptions", "ballotOptions");

		subQuery.createAlias("ballotOptions", "ballotOptions");
		subQuery.createAlias("contest", "contest");
		subQuery.createAlias("eligibleOptionParent", "eligibleOptionParent");
		subQuery.createAlias("eligibleOptionRoleData", "eligibleOptionRoleData");
		subQuery.createAlias("candidate", "eligibleOptionRoleData.candidate");
		subQuery.createAlias("district", "contest.district");
		subQuery.createAlias("regions", "district.regions");
		subQuery.createAlias("allParents", "regions.allParents");
		subQuery.createAlias("party", "ballotOptions.party");

		if (regions != null && !regions.isEmpty()) {
			List<Long> regionCodes = new ArrayList<Long>();
			for (RegionData region : regions) {
				regionCodes.add(region.getCode());
			}
			subQuery.addSearchCriteria(CriteriaFactory.in("allParents.code",
					regionCodes));
		}

		if (contests != null && !contests.isEmpty()) {
			List<Long> contestCodes = new ArrayList<Long>();
			for (ContestData contest : contests) {
				contestCodes.add(contest.getCode());
			}
			subQuery.addSearchCriteria(CriteriaFactory.in("contest.code",
					contestCodes));
		}

		if (parties != null && !parties.isEmpty()) {
			List<Long> partyCodes = new ArrayList<Long>();
			for (PartyData party : parties) {
				partyCodes.add(party.getCode());
			}
			subQuery.addSearchCriteria(CriteriaFactory.in("party.code",
					partyCodes));
		}

		if (candidateName != null && !candidateName.equals("")
				&& !candidateName.isEmpty()) {

			String[] names = candidateName.split(" ");
			ISearchCriteria mainCiteria = null;
			for (String current : names) {
				if (mainCiteria == null) {
					mainCiteria = CriteriaFactory.ilike("ballotName",
							current.trim(), MatchCriteriaMode.ANYWHERE);
				} else {
					mainCiteria = CriteriaFactory.or(mainCiteria,
							CriteriaFactory.ilike("ballotName", current.trim(),
									MatchCriteriaMode.ANYWHERE));
				}
			}
			mainCiteria = CriteriaFactory.or(mainCiteria, CriteriaFactory
					.ilike("candidate.identification", candidateName.trim(),
							MatchCriteriaMode.ANYWHERE));
			subQuery.addSearchCriteria(mainCiteria);
		}

		subQuery.addSearchCriteria(CriteriaFactory
				.isNotNull("eligibleOptionParent"));

		subQuery.setTransformation(TransformationFactory
				.distinctTransformation(TransformationFactory
						.propertyTransformation("code")));

		search.addSearchCriteria(CriteriaFactory.propertyIn("code", subQuery));

		search.addOrder(OrderCriteria.asc("ballotName"));

		try {
			String path[] = { "ballotOptions",
					"eligibleOptionRoleData.candidate" };
			List<EligibleOptionData> candidateDataList = (List<EligibleOptionData>) this.persistenceService
					.searchPagedObjects(search, path, null, pageNumber,
							numberOfPages);

			List<EligibleOptionData> translatedEO = new ArrayList<EligibleOptionData>();

			if (language != null) {
				for (EligibleOptionData indexEO : candidateDataList) {
					indexEO.setContest(this.ecmUtilitiesService
							.translateContestName(indexEO.getContest(),
									language));
					translatedEO.add(indexEO);
				}
			} else {
				return candidateDataList;
			}
			return translatedEO;
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new NominationException(e,
					NominationExceptionInfo.RETRIEVING_CANDIDATES_FAILURE,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	@Override
	public int retrieveNumberOfCandidates(String candidateName,
			List<RegionData> regions, List<ContestData> contests,
			List<PartyData> parties) throws NominationException {

		SearchExpression search = new SearchExpression(EligibleOptionData.class);
		SearchExpression subQuery = new SearchExpression(
				EligibleOptionData.class);

		search.createAlias("eligibleOptionParent", "eligibleOptionParent");
		search.createAlias("eligibleOptionRoleData", "eligibleOptionRoleData");
		search.createAlias("candidate", "eligibleOptionRoleData.candidate");
		search.createAlias("ballotOptions", "ballotOptions");

		subQuery.createAlias("ballotOptions", "ballotOptions");
		subQuery.createAlias("contest", "contest");
		subQuery.createAlias("eligibleOptionParent", "eligibleOptionParent");
		subQuery.createAlias("eligibleOptionRoleData", "eligibleOptionRoleData");
		subQuery.createAlias("candidate", "eligibleOptionRoleData.candidate");
		subQuery.createAlias("district", "contest.district");
		subQuery.createAlias("regions", "district.regions");
		subQuery.createAlias("allParents", "regions.allParents");
		subQuery.createAlias("party", "ballotOptions.party");

		if (regions != null && !regions.isEmpty()) {
			List<Long> regionCodes = new ArrayList<Long>();
			for (RegionData region : regions) {
				regionCodes.add(region.getCode());
			}
			subQuery.addSearchCriteria(CriteriaFactory.in("allParents.code",
					regionCodes));
		}

		if (contests != null && !contests.isEmpty()) {
			List<Long> contestCodes = new ArrayList<Long>();
			for (ContestData contest : contests) {
				contestCodes.add(contest.getCode());
			}
			subQuery.addSearchCriteria(CriteriaFactory.in("contest.code",
					contestCodes));
		}

		if (parties != null && !parties.isEmpty()) {
			List<Long> partyCodes = new ArrayList<Long>();
			for (PartyData party : parties) {
				partyCodes.add(party.getCode());
			}
			subQuery.addSearchCriteria(CriteriaFactory.in("party.code",
					partyCodes));
		}

		if (candidateName != null && !candidateName.equals("")
				&& !candidateName.isEmpty()) {

			String[] names = candidateName.split(" ");
			ISearchCriteria mainCiteria = null;
			for (String current : names) {
				if (mainCiteria == null) {
					mainCiteria = CriteriaFactory.ilike("ballotName",
							current.trim(), MatchCriteriaMode.ANYWHERE);
				} else {
					mainCiteria = CriteriaFactory.or(mainCiteria,
							CriteriaFactory.ilike("ballotName", current.trim(),
									MatchCriteriaMode.ANYWHERE));
				}
			}
			mainCiteria = CriteriaFactory.or(mainCiteria, CriteriaFactory
					.ilike("candidate.identification", candidateName.trim(),
							MatchCriteriaMode.ANYWHERE));
			subQuery.addSearchCriteria(mainCiteria);
		}

		subQuery.addSearchCriteria(CriteriaFactory
				.isNotNull("eligibleOptionParent"));

		subQuery.setTransformation(TransformationFactory
				.distinctTransformation(TransformationFactory
						.propertyTransformation("code")));

		search.addSearchCriteria(CriteriaFactory.propertyIn("code", subQuery));

		try {

			return this.persistenceService.countObjects(search, null);

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new NominationException(
					e,
					NominationExceptionInfo.RETRIEVING_CANDIDATES_AMOUNT_FAILURE,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	@Override
	public int retrieveNumberOfParties(String partyName,
			List<RegionData> regions, List<ContestData> contests)
			throws NominationException {
		SearchExpression search = new SearchExpression(EligibleOptionData.class);
		search.createAlias("ballotOptions", "ballotOptions");
		search.createAlias("party", "ballotOptions.party");
		search.createAlias("contest", "contest");
		search.createAlias("district", "contest.district");
		search.createAlias("regions", "district.regions");
		search.createAlias("allParents", "regions.allParents");
		search.addSearchCriteria(CriteriaFactory.isNull("eligibleOptionParent"));

		if (contests != null && !contests.isEmpty()) {
			List<Long> contestCodes = new ArrayList<Long>();
			for (ContestData contest : contests) {
				contestCodes.add(contest.getCode());
			}
			search.addSearchCriteria(CriteriaFactory.in("contest.code",
					contestCodes));
		}

		if (regions != null && !regions.isEmpty()) {
			List<Long> regionCodes = new ArrayList<Long>();
			for (RegionData region : regions) {
				regionCodes.add(region.getCode());
			}
			search.addSearchCriteria(CriteriaFactory.in("allParents.code",
					regionCodes));
		}

		if (partyName != null && !partyName.equals("") && !partyName.isEmpty()) {
			search.addSearchCriteria(CriteriaFactory.ilike("ballotName", "%"
					+ partyName.trim().replace(" ", "%") + "%"));
		}

		try {
			return this.persistenceService.countObjects(search, null);
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new NominationException(e,
					NominationExceptionInfo.RETRIEVING_PARTIES_AMOUNT_FAILURE,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<EligibleOptionData> getParties(int pageNumber,
			int numberOfPages, String partyName, List<RegionData> regions,
			List<ContestData> contests, LanguageData language)
			throws NominationException {
		SearchExpression searchGeneral = new SearchExpression(
				EligibleOptionData.class);

		searchGeneral.createAlias("ballotOptions", "ballotOptions");
		searchGeneral.createAlias("party", "ballotOptions.party");
		searchGeneral.addSearchCriteria(CriteriaFactory
				.isNull("eligibleOptionParent"));

		SearchExpression search = new SearchExpression(EligibleOptionData.class);
		search.createAlias("ballotOptions", "ballotOptions");
		search.createAlias("party", "ballotOptions.party");
		search.createAlias("contest", "contest");
		search.createAlias("district", "contest.district");
		search.createAlias("regions", "district.regions");
		search.createAlias("allParents", "regions.allParents");
		search.addSearchCriteria(CriteriaFactory.isNull("eligibleOptionParent"));

		if (contests != null && !contests.isEmpty()) {
			List<Long> contestCodes = new ArrayList<Long>();
			for (ContestData contest : contests) {
				contestCodes.add(contest.getCode());
			}
			search.addSearchCriteria(CriteriaFactory.in("contest.code",
					contestCodes));
		}

		if (regions != null && !regions.isEmpty()) {
			List<Long> regionCodes = new ArrayList<Long>();
			for (RegionData region : regions) {
				regionCodes.add(region.getCode());
			}
			search.addSearchCriteria(CriteriaFactory.in("allParents.code",
					regionCodes));
		}

		if (partyName != null && !partyName.equals("") && !partyName.isEmpty()) {
			search.addSearchCriteria(CriteriaFactory.ilike("ballotName", "%"
					+ partyName.trim().replace(" ", "%") + "%"));
		}

		searchGeneral.addOrder(OrderCriteria.asc("ballotName"));
		String path[] = { "ballotOptions", "ballotOptions.party" };

		search.setTransformation(TransformationFactory
				.distinctTransformation(TransformationFactory
						.propertyTransformation("code")));

		searchGeneral.addSearchCriteria(CriteriaFactory.propertyIn("code",
				search));

		try {
			List<EligibleOptionData> partiesList = (List<EligibleOptionData>) this.persistenceService
					.searchPagedObjects(searchGeneral, path, null, pageNumber,
							numberOfPages);

			for (EligibleOptionData indexEO : partiesList) {

				indexEO.setContest(this.ecmUtilitiesService
						.translateContestName(indexEO.getContest(), language));

				indexEO.getContest().setCollegeTranslate(
						this.ecmUtilitiesService.translateCollege(
								indexEO.getContest(), language)
								.getCollegeTranslate());

			}
			return partiesList;
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new NominationException(e,
					NominationExceptionInfo.RETRIEVING_PARTIES_FAILURE,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	@Override
	public PartyData updateEligibleOptionAndParty(
			EligibleOptionData eligibleOption) throws NominationException {
		SearchExpression search = new SearchExpression(PartyData.class);
		search.createAlias("ballotOption", "ballotOptions");
		search.createAlias("party", "ballotOption.party");
		search.createAlias("eligibleOption", "ballotOption.eligibleOption");

		search.addSearchCriteria(CriteriaFactory.eq("eligibleOption.code",
				eligibleOption.getCode()));
		PartyData party;
		try {
			party = (PartyData) this.persistenceService.searchObject(search,
					null);
			party.setName(eligibleOption.getBallotName());

			updateEligibleOption(eligibleOption);

			return this.persistenceService.update(party, null);
		} catch (NominationException e) {
			throw e;
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new NominationException(e,
					NominationExceptionInfo.UPDATING_PARTY_FAILURE,
					StringUtils.defaultString(e.getMessage()));
		}

	}

	@Override
	public EligibleOptionData updateEligibleOption(
			EligibleOptionData eligibleOption) throws NominationException {
		try {
			return this.persistenceService.update(eligibleOption, null);
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new NominationException(e,
					NominationExceptionInfo.UPDATING_ELIGIBLE_OPTION_FAILURE,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<PartyData> getParties(List<RegionData> regions,
			List<ContestData> contests) throws NominationException {

		SearchExpression search = new SearchExpression(PartyData.class);
		SearchExpression subQuery = new SearchExpression(PartyData.class);

		subQuery.createAlias("ballotOptions", "ballotOptions");
		subQuery.createAlias("eligibleOption", "ballotOptions.eligibleOption");
		subQuery.createAlias("contest", "eligibleOption.contest");
		subQuery.createAlias("district", "contest.district");
		subQuery.createAlias("regions", "district.regions");
		subQuery.createAlias("allChildren", "regions.allChildren");

		if (contests != null && !contests.isEmpty()) {
			List<Long> contestCodes = new ArrayList<Long>();
			for (ContestData contest : contests) {
				contestCodes.add(contest.getCode());
			}
			subQuery.addSearchCriteria(CriteriaFactory.in("contest.code",
					contestCodes));
		}

		subQuery.addSearchCriteria(CriteriaFactory
				.isNotNull("eligibleOption.eligibleOptionParent"));

		subQuery.setTransformation(TransformationFactory
				.distinctTransformation(TransformationFactory
						.propertyTransformation("code")));

		search.addSearchCriteria(CriteriaFactory.propertyIn("code", subQuery));

		search.addOrder(OrderCriteria.asc("name"));

		try {
			String path[] = {};

			List<PartyData> partyDataList = (List<PartyData>) this.persistenceService
					.searchObjects(search, path, null);

			return partyDataList;

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new NominationException(e,
					NominationExceptionInfo.RETRIEVING_PARTIES_FAILURE,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<ContestData> getContests(List<RegionData> regions,
			List<PartyData> parties, LanguageData language)
			throws NominationException {

		SearchExpression search = new SearchExpression(ContestData.class);
		SearchExpression subQuery = new SearchExpression(ContestData.class);
		SearchExpression subQueryRegions = new SearchExpression(
				RegionData.class);

		subQuery.createAlias("district", "district");
		subQuery.createAlias("regions", "district.regions");
		subQuery.createAlias("allChildren", "regions.allChildren");
		subQuery.createAlias("allParents", "regions.allParents");
		subQuery.createAlias("eligibleOptions", "eligibleOptions");
		subQuery.createAlias("ballotOptions", "eligibleOptions.ballotOptions");
		subQuery.createAlias("regions", "district.regions");
		subQuery.createAlias("party", "ballotOptions.party");

		subQueryRegions.createAlias("allChildren", "allChildren");
		subQueryRegions.createAlias("allParents", "allParents");

		if (regions != null && !regions.isEmpty()) {
			List<Long> regionCodes = new ArrayList<Long>();
			for (RegionData region : regions) {
				regionCodes.add(region.getCode());
			}
			subQueryRegions.addSearchCriteria(CriteriaFactory.or(
					CriteriaFactory.in("allChildren.code", regionCodes),
					CriteriaFactory.in("allParents.code", regionCodes)));
			subQueryRegions.setTransformation(TransformationFactory
					.distinctTransformation(TransformationFactory
							.propertyTransformation("code")));
			subQuery.addSearchCriteria(CriteriaFactory.propertyIn(
					"regions.code", subQueryRegions));
		}

		if (parties != null && !parties.isEmpty()) {
			List<Long> partyCodes = new ArrayList<Long>();
			for (PartyData party : parties) {
				partyCodes.add(party.getCode());
			}
			subQuery.addSearchCriteria(CriteriaFactory.in("party.code",
					partyCodes));
		}

		subQuery.addSearchCriteria(CriteriaFactory
				.isNotNull("eligibleOptions.eligibleOptionParent"));

		subQuery.setTransformation(TransformationFactory
				.distinctTransformation(TransformationFactory
						.propertyTransformation("code")));

		search.addSearchCriteria(CriteriaFactory.propertyIn("code", subQuery));

		search.addOrder(OrderCriteria.asc("name"));

		try {
			List<ContestData> contestDataList = (List<ContestData>) this.persistenceService
					.searchObjects(search, null, null);

			List<ContestData> contestTranslated = new ArrayList<ContestData>();

			for (ContestData contest : contestDataList) {
				contestTranslated.add(this.ecmUtilitiesService
						.translateContestName(contest, language));
			}

			return contestTranslated;

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new NominationException(e,
					NominationExceptionInfo.RETRIEVING_CONTESTS_FAILURE,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<ContestData> getContests(List<Long> contestCodesList,
			int pageNumber, int pageSize) throws NominationException {

		try {
			SearchExpression search = new SearchExpression(ContestData.class);

			search.addSearchCriteria(CriteriaFactory.in("code",
					contestCodesList));

			search.addOrder(OrderCriteria.asc("code"));

			String path[] = { "contestGroupList" };

			List<ContestData> contestList = (List<ContestData>) this.persistenceService
					.searchPagedObjects(search, path, null, pageNumber,
							pageSize);

			return contestList;
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new NominationException(e,
					NominationExceptionInfo.RETRIEVING_CONTESTS_FAILURE,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	@SuppressWarnings({ "unchecked" })
	@Override
	public List<ContestData> getContestsByContestClass(
			String contestClassCustomCode, LanguageData language)
			throws NominationException {

		List<ContestData> contestList = new ArrayList<ContestData>();

		try {

			SearchExpression search = new SearchExpression(
					LogicalVotingGroupData.class);

			TransformationsList lightTransformation = TransformationFactory
					.transformationList();

			lightTransformation.add(TransformationFactory
					.propertyTransformation("customCode"));

			search.setTransformation(lightTransformation);

			Collection<String> lvgCustomCodeList = this.persistenceService
					.searchObjects(search, null);
			StringBuilder customCodeLVG = new StringBuilder();
			for (String current : lvgCustomCodeList) {
				customCodeLVG.append(current);
			}

			search = new SearchExpression(ContestViewData.class);

			lightTransformation = TransformationFactory.transformationList();

			lightTransformation.add(TransformationFactory
					.distinctTransformation(TransformationFactory
							.propertyTransformation("code")));

			search.setTransformation(lightTransformation);

			Collection<Long> contestCodes = this.persistenceService
					.searchObjects(search, null);
			if ((contestCodes != null) && (!contestCodes.isEmpty())) {
				AliasTransformationsMap aliasTransformation = TransformationFactory
						.aliasTransformationMap();

				search = new SearchExpression(ContestData.class);

				search.createAlias("district", "district");
				search.createAlias("contestClass", "contestClass");

				aliasTransformation.add(
						TransformationFactory.propertyTransformation("code"),
						"contestCode");

				aliasTransformation.add(
						TransformationFactory.propertyTransformation("name"),
						"contestName");

				aliasTransformation.add(TransformationFactory
						.propertyTransformation("customCode"),
						"contestCustomCode");

				aliasTransformation.add(TransformationFactory
						.propertyTransformation("collegeLanguageRFC"),
						"contestCollegeLanguageRFC");

				aliasTransformation
						.add(TransformationFactory
								.propertyTransformation("contestClass"),
								"contestClass");

				aliasTransformation.add(TransformationFactory
						.propertyTransformation("district"), "district");

				search.addSearchCriteria(CriteriaFactory.in("code",
						contestCodes));

				List<Map<String, Object>> contestObjects = null;

				if (contestClassCustomCode != null) {
					search.addSearchCriteria(CriteriaFactory.eq(
							"contestClass.customCode", contestClassCustomCode));

					search.setTransformation(aliasTransformation);

					contestObjects = (List<Map<String, Object>>) this.persistenceService
							.searchObjects(search, null);
				} else {
					search.setTransformation(aliasTransformation);

					String[] path = { "district.regions" };

					contestObjects = (List<Map<String, Object>>) this.persistenceService
							.searchObjects(search, path, null);
				}

				for (Iterator<Map<String, Object>> iterator = contestObjects
						.iterator(); iterator.hasNext();) {

					Map<String, Object> map = iterator.next();
					String customCode = (String) map.get("contestCustomCode");
					if (customCodeLVG.toString().contains(customCode)) {
						long code = (Long) map.get("contestCode");
						String name = (String) map.get("contestName");
						String collegeLanguageRFC = (String) map
								.get("contestCollegeLanguageRFC");

						ContestClassData contestClass = (ContestClassData) map
								.get("contestClass");

						if (language != null) {
							String traducction = getTranslation(
									contestClass.getClass(),
									"contestClassName", contestClass.getCode(),
									language);
							if (traducction != null) {
								contestClass
										.setContestClassNameTranslate(traducction);
							} else {
								contestClass
										.setContestClassNameTranslate(contestClass
												.getContestClassName());
							}
						}

						DistrictData district = (DistrictData) map
								.get("district");

						ContestData contestData = new ContestData();
						contestData.setCode(code);
						contestData.setName(name);
						contestData.setCollegeLanguageRFC(collegeLanguageRFC);
						contestData.setContestClass(contestClass);
						contestData.setDistrict(district);
						contestData.setCustomCode(customCode);
						if (language != null) {
							contestData = this.ecmUtilitiesService
									.translateContestName(contestData, language);
						}
						contestList.add(contestData);
					}
				}
			}

			return contestList;
		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new NominationException(e,
					NominationExceptionInfo.RETRIEVING_CONTESTS_FAILURE,
					StringUtils.defaultString(e.getMessage()));
		}

	}

	@SuppressWarnings("unchecked")
	@Override
	public List<EligibleOptionData> getBallotOptionByEligibleOptions(
			List<EligibleOptionData> eligibleOptionsList)
			throws NominationException {

		Long partyCode;
		Map<Long, PartyData> parties = new TreeMap<Long, PartyData>();
		BallotOptionData ballotOption = null;
		List<Object[]> ballotOptionObjects = null;
		Set<BallotOptionData> ballotOptionSet = null;

		try {

			List<EligibleOptionData> eligibleOptionsWithBallotOptionsList = new ArrayList<EligibleOptionData>();

			for (EligibleOptionData eligibleOption : eligibleOptionsList) {

				SearchExpression search = new SearchExpression(
						BallotOptionData.class);

				search.createAlias("eligibleOption", "eligibleOption");

				search.addSearchCriteria(CriteriaFactory.eq(
						"eligibleOption.code", eligibleOption.getCode()));

				TransformationsList lightTransformation = TransformationFactory
						.transformationList();

				lightTransformation.add(TransformationFactory
						.propertyTransformation("code"));
				lightTransformation.add(TransformationFactory
						.propertyTransformation("customCode"));
				lightTransformation.add(TransformationFactory
						.propertyTransformation("type"));
				lightTransformation.add(TransformationFactory
						.propertyTransformation("tallyPosition"));

				lightTransformation.add(TransformationFactory
						.propertyTransformation("party.code"));

				search.setTransformation(lightTransformation);

				ballotOptionObjects = (List<Object[]>) this.persistenceService
						.searchObjects(search, null);

				ballotOptionSet = new HashSet<BallotOptionData>();

				for (Object[] ballotOptionObject : ballotOptionObjects) {
					ballotOption = new BallotOptionData();
					ballotOption.setCode((Long) ballotOptionObject[0]);
					ballotOption.setCustomCode(String
							.valueOf(ballotOptionObject[1]));
					ballotOption
							.setType((BallotOptionData.Type) ballotOptionObject[2]);
					ballotOption
							.setTallyPosition((Integer) ballotOptionObject[3]);

					partyCode = (Long) ballotOptionObject[4];

					if (!parties.containsKey(partyCode)) {
						parties.put(partyCode, getParty(partyCode));
					}

					ballotOption.setParty(parties.get(partyCode));

					ballotOption.setEligibleOption(eligibleOption);

					ballotOptionSet.add(ballotOption);
				}

				eligibleOption.setBallotOptions(ballotOptionSet);
				eligibleOptionsWithBallotOptionsList.add(eligibleOption);
			}

			return eligibleOptionsWithBallotOptionsList;

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new NominationException(
					e,
					NominationExceptionInfo.GETTING_BALLOT_OPTIONS_BY_ELIGIBLE_OPTION_FAILURE,
					StringUtils.defaultString(e.getMessage()));
		} finally {
			partyCode = null;
			parties = null;
			ballotOption = null;
			ballotOptionObjects = null;
			ballotOptionSet = null;
		}
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<EligibleOptionData> getEligibleOptionRoleByEligibleOption(
			List<EligibleOptionData> eligibleOptionsList)
			throws NominationException {

		try {

			List<EligibleOptionData> eligibleOptionsWithEligibleOptionRole = new ArrayList<EligibleOptionData>();

			for (EligibleOptionData eligibleOption : eligibleOptionsList) {

				SearchExpression search = new SearchExpression(
						EligibleOptionRoleData.class);

				search.createAlias("eligibleOption", "eligibleOption");

				search.addSearchCriteria(CriteriaFactory.eq(
						"eligibleOption.code", eligibleOption.getCode()));

				TransformationsList lightTransformation = TransformationFactory
						.transformationList();

				lightTransformation.add(TransformationFactory
						.propertyTransformation("code"));
				lightTransformation.add(TransformationFactory
						.propertyTransformation("role"));
				lightTransformation.add(TransformationFactory
						.propertyTransformation("candidate.code"));

				search.setTransformation(lightTransformation);

				List<Object[]> eligibleOptionRoleList = (List<Object[]>) this.persistenceService
						.searchObjects(search, null);

				SortedSet<EligibleOptionRoleData> eligibleOptionRoleSet = new TreeSet<EligibleOptionRoleData>(
						new EligibleOptionRoleData.OrderComparator());

				for (Object[] eligibleOptionRolObject : eligibleOptionRoleList) {

					EligibleOptionRoleData eligibleOptionRole = new EligibleOptionRoleData();

					eligibleOptionRole
							.setCode((Long) eligibleOptionRolObject[0]);
					eligibleOptionRole
							.setRole((EligibleOptionRoleData.Role) eligibleOptionRolObject[1]);

					eligibleOptionRole
							.setCandidate(getCandidate((Long) eligibleOptionRolObject[2]));

					eligibleOptionRoleSet.add(eligibleOptionRole);
				}

				eligibleOption.setEligibleOptionRoleData(eligibleOptionRoleSet);

				eligibleOptionsWithEligibleOptionRole.add(eligibleOption);
			}

			return eligibleOptionsWithEligibleOptionRole;

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new NominationException(
					e,
					NominationExceptionInfo.GETTING_ELIGIBLE_OPTION_ROLE_BY_ELIGIBLE_OPTION_FAILURE,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	@SuppressWarnings({ "unchecked" })
	@Override
	public List<EligibleOptionData> getEligibleOptionsByContest(
			ContestData contestData) throws NominationException {
		try {

			SearchExpression search = new SearchExpression(
					EligibleOptionData.class);

			search.createAlias("contest", "contest");

			search.addSearchCriteria(CriteriaFactory.eq("contest.code",
					contestData.getCode()));

			TransformationsList lightTransformation = TransformationFactory
					.transformationList();
			lightTransformation.add(TransformationFactory
					.propertyTransformation("code"));
			lightTransformation.add(TransformationFactory
					.propertyTransformation("customCode"));
			lightTransformation.add(TransformationFactory
					.propertyTransformation("ballotName"));
			lightTransformation.add(TransformationFactory
					.propertyTransformation("eligibleOptionParent.code"));

			search.setTransformation(lightTransformation);

			List<Object[]> eligibleOptionObjects = (List<Object[]>) this.persistenceService
					.searchObjects(search, null);

			List<EligibleOptionData> eligibleOptionList = new ArrayList<EligibleOptionData>();

			EligibleOptionData eligibleOption;

			for (Object[] object : eligibleOptionObjects) {
				eligibleOption = new EligibleOptionData();

				eligibleOption.setCode((Long) object[0]);
				eligibleOption.setCustomCode(String.valueOf(object[1]));
				eligibleOption.setBallotName(String.valueOf(object[2]));
				eligibleOption.setParentCode(object[3] == null ? "" : String
						.valueOf(object[3]));
				eligibleOptionList.add(eligibleOption);

				eligibleOption = null;
			}

			return eligibleOptionList;

		} catch (Exception e) {
			logger.error(e, StringUtils.defaultString(e.getMessage()));
			throw new NominationException(
					e,
					NominationExceptionInfo.GETTING_ELIGIBLE_OPTION_BY_CONTEST_FAILURE,
					StringUtils.defaultString(e.getMessage()));
		}
	}

	/**
	 * Method that returns a party object if the given party code matches a
	 * record stored in database.
	 * 
	 * @param partyCode
	 *            the party code to find
	 * @return the party code object if
	 * @throws BaseException
	 *             Signal that an error occurred getting the party.
	 */
	private PartyData getParty(long partyCode) throws BaseException {

		SearchExpression search = new SearchExpression(PartyData.class);

		TransformationsList lightTransformation = TransformationFactory
				.transformationList();

		search.addSearchCriteria(CriteriaFactory.eq("code", partyCode));

		lightTransformation.add(TransformationFactory
				.propertyTransformation("code"));

		lightTransformation.add(TransformationFactory
				.propertyTransformation("customCode"));

		lightTransformation.add(TransformationFactory
				.propertyTransformation("name"));

		lightTransformation.add(TransformationFactory
				.propertyTransformation("abbreviation"));

		lightTransformation.add(TransformationFactory
				.propertyTransformation("image"));

		lightTransformation.add(TransformationFactory
				.propertyTransformation("imageFileName"));

		search.setTransformation(lightTransformation);

		Object[] partyObject = (Object[]) this.persistenceService.searchObject(
				search, null);

		PartyData party = new PartyData();

		if (partyObject != null) {

			party.setCode((Long) partyObject[0]);
			party.setCustomCode(String.valueOf(partyObject[1]));
			party.setName(String.valueOf(partyObject[2]));
			party.setAbbreviation(String.valueOf(partyObject[3]));
			party.setImage((ImageWrapper) partyObject[4]);
			party.setImageFileName(String.valueOf(partyObject[5]));
		}

		return party;
	}

	/**
	 * Method that returns a candidate object if the given candidate code
	 * matches a record stored in database.
	 * 
	 * @param candidateCode
	 *            the candidate code to find
	 * @return the candidate code object if
	 * @throws BaseException
	 *             Signal that an error occurred getting the candidate.
	 */
	private CandidateData getCandidate(long candidateCode) throws BaseException {

		SearchExpression search = new SearchExpression(CandidateData.class);

		search.addSearchCriteria(CriteriaFactory.eq("code", candidateCode));

		TransformationsList lightTransformation = TransformationFactory
				.transformationList();

		lightTransformation.add(TransformationFactory
				.propertyTransformation("code"));

		lightTransformation.add(TransformationFactory
				.propertyTransformation("identification"));

		lightTransformation.add(TransformationFactory
				.propertyTransformation("name"));

		search.setTransformation(lightTransformation);

		Object[] candidateObject = (Object[]) this.persistenceService
				.searchObject(search, null);

		CandidateData candidate = new CandidateData();

		if (candidateObject != null) {
			candidate.setCode((Long) candidateObject[0]);
			candidate.setIdentification(String.valueOf(candidateObject[1]));
			candidate.setName(String.valueOf(candidateObject[2]));
		}

		return candidate;
	}


    /**
     * Fetch the translations for specific field of the given class and object
     * code for the required language.
     *
     * @param className
     *      The class name
     * @param fieldToSearch
     *      The field to search the translation
     * @param objectCode
     *      The code for the object of given class
     * @param language
     *      The language for the desired translation
     * @return
     *      The translation
     *
     * @throws BaseException
     *      Signal that an error occurred getting the translation
     */
    @SuppressWarnings("unchecked")
    private String getTranslation(
        Class<?> className, String fieldToSearch, Long objectCode,
        LanguageData language
    ) throws BaseException {
        Collection<String> translations;
        SearchExpression mkExpr;
        SearchExpression search;
        String translatedWord;
        TransformationsList lightTransformation;

        translatedWord = null;
        search         = new SearchExpression(TranslationKeyData.class);

        search
            .createAlias("electionLanguage", "electionLanguage")
            .addSearchCriteria(
                CriteriaFactory.eq("customCode", language.getCustomCode())
            );

        mkExpr = search.createAlias("multilingualKey","multilingualKey");

        mkExpr.addSearchCriteria(CriteriaFactory.eq("key",className.getName()));
        mkExpr.addSearchCriteria(CriteriaFactory.eq("field", fieldToSearch));
        mkExpr.addSearchCriteria(
            CriteriaFactory.eq("foreignKey", objectCode)
        );

        lightTransformation = TransformationFactory.transformationList();
        lightTransformation.add(
            TransformationFactory.distinctTransformation(
                TransformationFactory.propertyTransformation("translationText")
            )
        );

        search.setTransformation(lightTransformation);
        translations = this.persistenceService.searchObjects(search, null);

        if ((translations != null) && !translations.isEmpty()) {
            assert translations.size() == 1;
            translatedWord = translations.iterator().next();
        }
        return translatedWord;
    }
}
