/**
 * BulkDataLoadException.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.service.lang;

import java.util.Collections;
import java.util.Map;

import com.smartmatic.waffle.ecm.core.lang.ECMBaseException;
import com.smartmatic.waffle.ecm.core.lang.IECMExceptionInfo;

/**
 * Class to handle any exception that may occur executing any Bulk Data Load
 * operation.
 */
public class BulkDataLoadException extends ECMBaseException {

    /**
     * Holds the version of the class. It is used by the serialization engine to
     * determine when a class its equivalent to another
     */
    private static final long serialVersionUID = 1042224267124632834L;

    /**
     * Map of tuples {@code (K,V)} where {@code K} represents a given
     * parameter and {@code V} its associated value.
     * 
     * Such map contains contextual information associated to this
     * {@link TransformationException}.
     */
    private final Map<String, String> ctxt;


    /**
     * Instantiates a BulkDataLoadException.
     * 
     * @param throwable
     *      a nested exception
     * @param exceptionInfo
     *      an enumeration value regarding the cause of the exception
     */
    public BulkDataLoadException(
        Throwable throwable, IECMExceptionInfo exceptionInfo
    ) {
        this(throwable, exceptionInfo, throwable.getMessage());
    }


    /**
     * Instantiates a BulkDataLoadException.
     * 
     * @param throwable
     *     a nested exception
     * @param exceptionInfo
     *     an enumeration value regarding the cause of the exception
     * @param exceptionMessage
     *     Exception message of nested exception
     */
    public BulkDataLoadException(
        Throwable throwable, IECMExceptionInfo exceptionInfo,
        String exceptionMessage
    ) {
        super(throwable, exceptionInfo, exceptionMessage);
        this.ctxt = (throwable instanceof TransformationException)
                       ? ((TransformationException) throwable).getContext()
                       : Collections.<String,String>emptyMap();
    }


    /**
     * Instantiates a BulkDataLoadException.
     * 
     * @param exceptionInfo
     *      an enumeration value regarding the cause of the exception
     */
    public BulkDataLoadException(IECMExceptionInfo exceptionInfo) {
        this(exceptionInfo, null);
    }


    /**
     * Instantiates a BulkDataLoadException.
     * 
     * @param exceptionInfo
     *      an enumeration value regarding the cause of the exception
     * @param ctxt
     *      Map that contains contextual information of the exception
     *      being instantiated. Such contextual information can be used
     *      later to customize exception messages even more!
     */
    public BulkDataLoadException(
        IECMExceptionInfo exceptionInfo, Map<String,String> ctxt
    ) {
        super(exceptionInfo);
        this.ctxt = (ctxt != null)
                       ? Collections.<String,String>unmodifiableMap(ctxt)
                       : Collections.<String,String>emptyMap();
    }


    /**
     * {@inheritDoc}
     */
    @Override
    public String getBundleKey() {
        return this.getClass().getSimpleName()
             + "." + this.exceptionInfo.getBundleKey();
    }


    /**
     * Returns a map of contextual information associated to this {@link
     * TransformationException}.
     *
     * @return
     *      {@code Map<String, String>} (never {@code null})
     */
    public Map<String, String> getContext() {
        return this.ctxt;
    }



    /**
     * Enum that contains all the possible reasons for which the
     * BulkDataLoadException can be thrown.
     */
    public enum BulkDataLoadExceptionOperationExceptionInfo
    implements IECMExceptionInfo {

        BADLY_CONFIGURED_AUDIO_VOTING_FOR_REGION(
            "badly.configured.audio.voting"
        ),
        MISSING_REQUIRED_FIELDS_ERROR(
            "missing.required.fields.error"
        ),
        RETRIEVING_MULTILINGUAL_AMOUNT_FAILURE(
            "retrieving.multilingual.amount.failure"
        ),
        SELECTED_FILES_AMOUNT_DO_NO_MATCH_WITH_SIGNED_FILES(
            "selected.files.amount.does.not.match.with.signed.files",
            "The number of selected files does not match with the"
                + "signed files"
        ),
        SMTT_FILE_TYPE_NOT_FOUND(
            "unexpected.error.getting.smtt.file.type",
            "Smartmatic file type not found"
        ),
        THE_SIGNATURE_FILE_DOES_NOT_MATCH_WITH_THE_CERTIFICATE(
            "signature.file.does.not.match.with.the.certificate",
            "The signature of the selected files does not match"
                + " with the loaded certificate"
        ),
        THE_SELECTED_FILES_ARE_NOT_SIGNED(
            "the.selected.files.are.not.signed", 
            "The selected files are not signed, please sign them and"
                + " load them with the signature file"
        ),
        UNEXPECTED_ERROR_EXECUTING_BULK_DATA_LOAD(
            "unexpected.error.executing.bulk.data.load"
        ),
        UNEXPECTED_ERROR_EXPORTING_MULTILINGUAL_KEYS(
            "unexpected.error.exporting.multilingual.keys"
        ),
        UNEXPECTED_ERROR_GETTING_SMTT_FILE_TYPE(
            "unexpected.error.getting.smtt.file.type"
        ),
        UNEXPECTED_ERROR_PARSING_SIGNATURE_FILE(
            "unexpected.error.parsing.signature.file"
        ),
        UNEXPECTED_ERROR_REMOVING_ALL_DATABASE_RECORDS(
            "unexpected.error.removing.all.database.records"
        ),
        UNEXPECTED_ERROR_REMOVING_CODI_RECORDS(
            "unexpected.error.removing.codi.records"
        ),
        UNEXPECTED_ERROR_RETRIEVING_ENTITY_RECORDS_QUANTITIES(
            "unexpected.error.retrieving.entity.records.quantities"
        ),
        UNEXPECTED_ERROR_SIGNING_EML_FILES(
            "unexpected.error.signing.codi.files"
        ),
        UNEXPECTED_ERROR_UPDATING_OBJECT_IN_DATABASE(
            "unexpected.error.updating.object.in.database"
        ),
        UNEXPECTED_ERROR_VALIDATING_FILE_INTEGRITY(
            "unexpected.error.validating.file.integrity"
        );

        /**
         * Variable to hold the bundle key to show a user understandable
         * message.
         */
        private String bundleKey;

        /**
         * Variable to hold the exception message.
         */
        private String exceptionMessage;

        /**
         * {@inheritDoc}
         */
        @Override
        public String getBundleKey() {
            return this.bundleKey;
        }

        /**
         * {@inheritDoc}
         */
        @Override
        public String getExceptionMessage() {
            return this.exceptionMessage;
        }

        /**
         * Constructor.
         * 
         * @param bundleKey
         *      Resource bundle's key message
         */
        private BulkDataLoadExceptionOperationExceptionInfo(String bundleKey) {
            this.bundleKey = bundleKey;
        }

        /**
         * Constructor.
         * 
         * @param bundleKey
         *      Resource bundle's key message
         * @param exceptionMessage
         *      Exception message
         */
        private BulkDataLoadExceptionOperationExceptionInfo(
            String bundleKey, String exceptionMessage
        ) {
            this.bundleKey        = bundleKey;
            this.exceptionMessage = exceptionMessage;
        }
    }
}
