/**
 * IECMElectoralDemographyService.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.service.spec;

import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.ejb.Remote;

import com.smartmatic.waffle.domain.contest.ContestData;
import com.smartmatic.waffle.domain.contest.ContestGroupData;
import com.smartmatic.waffle.domain.demographics.DistrictData;
import com.smartmatic.waffle.domain.demographics.RegionData;
import com.smartmatic.waffle.domain.device.LogicalVotingGroupData;
import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.ecm.service.lang.ElectoralDemographyException;

/**
 * Remote interface which establishes methods contract that must be implemented
 * by <code>ECMElectoralDemographyServiceImpl</code> class
 */
@Remote
public interface IECMElectoralDemographyService {

	/** Service name Constant. */
	public static final String SERVICE_NAME = "com.smartmatic.waffle.ecm.service.spec.IECMElectoralDemographyService";

	/**
	 * Retrieves a contest list by district
	 * 
	 * @param districtDataSet
	 *            set of district to get the contest list a associated with it
	 * 
	 * @return the contest list
	 * 
	 * @throws ElectoralDemographyException
	 *             signals that an exception has occurred retrieving the list of
	 *             contest by district.
	 */
	List<ContestData> getContestByDistrict(Set<DistrictData> districtDataSet)
			throws ElectoralDemographyException;

	/**
	 * Retrieves a contest list by last level regions
	 * 
	 * @param lastLevelRegionsCodesList
	 *            contain the codes of last level region selected by the user to
	 *            generate the Master USB
	 * 
	 * @return the contest list
	 * 
	 * @throws ElectoralDemographyException
	 *             signals that an exception has occurred retrieving the list of
	 *             contest by last level regions.
	 */
	List<ContestData> getContestsByLastLevelRegions(List<Long> regionCodes)
			throws ElectoralDemographyException;

	/**
	 * Gets the contest group by last level region.
	 * 
	 * @param lastLevelRegion
	 *            last level region to get the contest group
	 * 
	 * @return the contest group list
	 * 
	 * @throws ElectoralDemographyException
	 *             signals that an exception has occurred retrieving the list of
	 *             contest groups by last level regions.
	 */
	List<ContestGroupData> getContestGroupByLastLevelRegion(
			RegionData lastLevelRegion) throws ElectoralDemographyException;

	/**
	 * Gets the contest group loaded in database.
	 * 
	 * @return the list of contest groups loaded in database
	 * 
	 * @throws ElectoralDemographyException
	 *             signals that an exception has occurred retrieving the list of
	 *             contest groups loaded in database.
	 */
	List<ContestGroupData> getContestGroupsLoadedInDatabase()
			throws ElectoralDemographyException;

	/**
	 * Method to get all possible groups of contests (in a specific
	 * lastLevelRegion). A voter will have to vote for all of the contests of
	 * one of these groups. If lastLevelRegion is null, the process generates
	 * the contest groups for all possible lastLevelRegions in the database. If
	 * lastLevelRegion is not null, the method does the following. The
	 * database-stored contestGroups contain the list of legal contests for each
	 * voter type. This is at the national level, though the contests may only
	 * apply to some regions. Each of the contests belong to a given contest
	 * class. This method will create contest groups for the lastLevelRegion,
	 * such that each contest group contains exactly one contest of each contest
	 * class, and such that all the contests in the contest group are legal for
	 * the region being considered (ie, they all come from the same
	 * national-level contest group and are applicable in that region).
	 * 
	 * @param lastLevelRegion
	 *            the last level region for which we are building the
	 *            region-specific contest groups. If this parameter is null, the
	 *            method will build all possible contest groups in the country.
	 * 
	 * @param regionContests
	 *            all contests which apply to the lastLevelRegion. If
	 *            lastLevelRegion is null, this contains all contests in the
	 *            database.
	 * @param contestGroupCodeReference
	 *            the last contest group code generated for the current election
	 *            when generating this is useful when generating for multiple
	 *            contest to avoid duplicate codes.
	 * 
	 * @return generated list of region-specific contest groups
	 * 
	 * @throws ElectoralDemographyException
	 *             signals that an exception has occurred generating the list of
	 *             contest groups.
	 */
	List<ContestGroupData> makeRegionSpecificContestGroups(
			final RegionData lastLevelRegion,
			final List<ContestData> regionContests,
			long contestGroupCodeReference) throws ElectoralDemographyException;

	/**
	 * Method to get all possible groups of contests from a list of contest. A
	 * voter will have to vote for all of the contests of one of these groups.
	 * The database-stored contestGroups contain the list of legal contests for
	 * each voter type. This is at the national level, though the contests may
	 * only apply to some regions. Each of the contests belong to a given
	 * contest class. This method will create contest groups for all the
	 * contests in the contest group are legal for the region being considered
	 * (ie, they all come from the same national-level contest group and are
	 * applicable in that region).
	 * 
	 * @param regionContests
	 *            all contests which apply to the lastLevelRegion. If
	 *            lastLevelRegion is null, this contains all contests in the
	 *            database.
	 * 
	 * @return generated list of region-specific contest groups
	 * 
	 * @throws ElectoralDemographyException
	 *             signals that an exception has occurred generating the list of
	 *             contest groups.
	 */
	List<ContestGroupData> makeContestsSpecificContestGroups(
			final List<ContestData> regionContests)
			throws ElectoralDemographyException;

	/**
	 * Gets contest object by code
	 * 
	 * @param contestCode
	 *            the contest code to find
	 * @return the contest object matched
	 * @throws ElectoralDemographyException
	 *             A signal that an error has occurred getting the contest
	 */
	ContestData getContestByCode(long contestCode)
			throws ElectoralDemographyException;

	/**
	 * Gets a logical voting group object by custom code
	 * 
	 * @param customCode
	 *            the custom code find
	 * @return the logical voting group object matched
	 * @throws ElectoralDemographyException
	 *             A signal that an error has occurred getting the logical
	 *             voting group.
	 */
	LogicalVotingGroupData getLogicalVotingGroup(String customCode)
			throws ElectoralDemographyException;

	/**
	 * Gets a logical voting group list by custom code and contest group name
	 * 
	 * @param customCode
	 *            the custom code to find
	 * @param contestGroupName
	 *            the contest group name to find
	 * @return the logical voting group list
	 * @throws ElectoralDemographyException
	 *             A signal that an error has occurred getting the logical
	 *             voting group list.
	 */
	List<LogicalVotingGroupData> getLogicalVotingGroupByContestGroup(
			String customCode, String contestGroupName)
			throws ElectoralDemographyException;

	ContestGroupData getContestGroupByCustomCode(String customCode)
			throws ElectoralDemographyException;

	List<LanguageData> getDistrictLanguages(String customCode)
			throws ElectoralDemographyException;

	/**
	 * Gets the contests for last level regions with the minimum data required.
	 * 
	 * @param regionCode
	 *            long value for region code.
	 * @return A map of contest groups with the contest and related data.
	 * @throws ElectoralDemographyException
	 *             A signal that an error has occurred getting the contest.
	 */
	Map<Long, ContestGroupData> getContestsForLastLevelRegion(long regionCode)
			throws ElectoralDemographyException;

	/**
	 * Build the specific contest groups for last level region contests.
	 * 
	 * @param contestsForLastLevelRegion
	 *            contest groups with the contest and related data.
	 * @param regionCode
	 *            The last level region code.
	 * @throws ElectoralDemographyException
	 *             A signal that an error has occurred getting the logical
	 *             voting group list.
	 */
	void buildSpecificContestGroup(
			Map<Long, ContestGroupData> contestsForLastLevelRegion,
			long regionCode) throws ElectoralDemographyException;

	/**
	 * Search the max contest group code on database.
	 * 
	 * @return The max contest group code on database.
	 * 
	 * @throws ElectoralDemographyException
	 */
	long searchMaxContestGroupCode() throws ElectoralDemographyException;
}
