/**
 * IECMService.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.ecm.service.spec;

import java.io.IOException;
import java.sql.SQLException;
import java.util.Date;
import java.util.List;
import java.util.Map;

import javax.ejb.Remote;

import com.smartmatic.waffle.domain.demographics.CategoryData.Level;
import com.smartmatic.waffle.domain.demographics.PollingPlaceData;
import com.smartmatic.waffle.domain.demographics.RegionData;
import com.smartmatic.waffle.domain.device.DeviceParameterData;
import com.smartmatic.waffle.domain.device.LogicalVotingGroupData;
import com.smartmatic.waffle.domain.general.ParameterData;
import com.smartmatic.waffle.domain.i18n.LanguageData;
import com.smartmatic.waffle.ecm.service.lang.BackupDatabaseGenerationExeption;
import com.smartmatic.waffle.ecm.service.lang.ElectoralDemographyException;
import com.smartmatic.waffle.ecm.service.lang.XmlGenerationException;
import com.smartmatic.waffle.lang.BaseException;

/**
 * Remote interface which establishes methods contract that must be implemented
 * by <code>ECMServiceImpl</code> class
 */
@Remote
public interface IECMService {

	/**
	 * Holds the name of the bean that implements this interface.
	 * 
	 */
	public static final String SERVICE_NAME = "com.smartmatic.waffle.ecm.service.spec.IECMService";

	/**
	 * Method used to get the integer value of a parameter in the database.
	 * 
	 * @param key
	 *            The key of the parameter.
	 * 
	 * @return The integer value of the parameter.
	 * 
	 * @throws BaseException
	 *             If an exception occurs interacting with the database.
	 */
	public int getIntParameter(String key) throws IllegalStateException,
			BaseException;

	/**
	 * Method used to get the value of a parameter in the database.
	 * 
	 * @param key
	 *            The key of the parameter.
	 * 
	 * @return The value of the parameter.
	 * 
	 * @throws BaseException
	 *             If an exception occurs interacting with the database.
	 */
	public String getStringParameter(String key) throws BaseException;

	/**
	 * Method used to update any configuration parameter
	 * 
	 * @param parameter
	 *            Parameter to update
	 * 
	 * @author Frederick Faria (frederick.faria@smartmatic.com)
	 */
	public void updateParameter(ParameterData parameter)
			throws XmlGenerationException;

	/**
	 * Method to set or create a new parameter key - value
	 * 
	 * @param key
	 *            The key of the parameter.
	 * @param value
	 *            The value of the parameter.
	 * @throws BaseException
	 */
	public void setParameter(String key, String value) throws BaseException;

	/**
	 * Method used to get all parameters saved in database
	 * 
	 * @return list of ParameterData
	 * 
	 * @throws XmlGenerationException
	 *             Wrapper of Exception to notify the client that a problem
	 *             occurred
	 * 
	 * @author Frederick Faria (frederick.faria@smartmatic.com)
	 */
	public List<ParameterData> getParameters() throws XmlGenerationException;

	/**
	 * Executes the generation process of logical voting groups after the main
	 * data load has ended.
	 * 
	 * @param region
	 *            Region of the logical voting groups that will be generated
	 * 
	 * @param logicalVotingGroupsData
	 *            List of already generated logical voting groups. This is used
	 *            to prevent a logical voting group from repeating itself
	 * 
	 * @return List of logical voting groups with its associated region
	 */
	public Map<String, LogicalVotingGroupData> postLoadProcess(RegionData region)
			throws Exception;

	/**
	 * Stores a list of logical voting groups in the database, keeping their
	 * relationship with the respective regions
	 * 
	 * @param logicalVotingGroupsData
	 *            List of logical voting groups and their respective regions
	 * @throws ElectoralDemographyException
	 * 
	 * @throws BaseException
	 */
	void persistLogicalVotingGroup(
			Map<String, LogicalVotingGroupData> logicalVotingGroupsData)
			throws XmlGenerationException, ElectoralDemographyException;

	/**
	 * Creates the dump of the database and returns it as an array of bytes
	 * 
	 * @param databaseIp
	 *            ip address to reach the database to extract the dump
	 * 
	 * @return Dump of the database as an array of bytes if no errors were
	 *         detected, otherwise it returns null
	 * 
	 * @throws XmlGenerationException
	 */
	public byte[] dumpDatabase(String databaseIp)
			throws BackupDatabaseGenerationExeption;

	/**
	 * Gets all the polling places of a given region
	 * 
	 * @param region
	 *            Region of the polling places
	 * 
	 * @return List of polling places of the given region
	 * 
	 * @throws BaseException
	 */
	public List<PollingPlaceData> getPollingPlacesPerRegion(RegionData region)
			throws XmlGenerationException;

	/**
	 * Method that return the server ip.
	 * 
	 * @return True if the database is working of false in otherwise.
	 * 
	 * @throws BaseException
	 *             Abstract base class to notify the client that a problem
	 *             occurred
	 * 
	 * @throws SQLException
	 *             An exception that provides information on a database access
	 *             error or other errors.
	 */
	public String getServerIP();

	/**
	 * Method that check if exist contest, contest class and contest group in
	 * database.
	 * 
	 * @return True if exist contest, contest class and contest group in
	 *         database.
	 * 
	 * @throws BaseException
	 *             Abstract base class to notify the client that a problem
	 *             occurred
	 */
	public boolean integrityValidationContesContestClassContestGroup()
			throws BaseException;

	/***
	 * Method used to return a list of children regions from a parent region
	 * 
	 * @param parent
	 *            parent region
	 * @param language
	 *            selected language
	 * @param customCodesOfRegionsAvailables
	 * @return the list of children regions
	 * @throws XmlGenerationException
	 */
	public List<RegionData> getChildrenRegions(RegionData parent,
			List<Long> codesOfRegionsAvailables, LanguageData language)
			throws XmlGenerationException;

	/**
	 * Method used to return a list of regions by category level
	 * 
	 * @param categoryLevel
	 *            level of the initial category
	 * @param codesOfRegionsAvailables
	 * @param language
	 *            selected language
	 * @return the list of regions
	 * @throws XmlGenerationException
	 */
	public List<RegionData> getRegionsByCategoryLevel(Level categoryLevel,
			List<Long> codesOfRegionsAvailables, LanguageData language)
			throws XmlGenerationException;

	/**
	 * Method to fill the language combinations table
	 * 
	 * @throws BaseException
	 *             The exception is thrown when there is an error executing the
	 *             post load process.
	 */
	public void fillLanguageCombinations() throws Exception;

	/**
	 * Method to download ECM report from database to server file system
	 * 
	 * @throws BaseException
	 *             The exception is thrown when there is an error executing the
	 *             post load process.
	 */
	public void downloadECMReports() throws BaseException;

	/**
	 * Method used to set the server date and time.
	 * 
	 * @param datePicked
	 *            The date to set.
	 * @param scriptPath
	 *            The path of the script
	 * @param machineDate
	 *            The command to execute
	 * @throws BaseException
	 *             The exception is thrown when there is an error executing the
	 *             post load process.
	 */
	public void setServerDateTime(Date datePicked, String scriptPath,
			String machineDate) throws BaseException;

	/**
	 * Gets the server date time.
	 * 
	 * @param scriptPath
	 *            The script path
	 * @param machineDateTime
	 *            The command to execute
	 * @return the server date time
	 * @throws BaseException
	 *             The exception is thrown when there is an error executing the
	 *             post load process.
	 */
	public String getServerDateTime(String scriptPath, String machineDateTime)
			throws BaseException;

	/**
	 * Gets the log files from the server into a hashMap
	 * 
	 * @return A HashMap with the inputStreams
	 * 
	 * @throws IOException
	 *             Signals that an I/O exception of some sort has occurred.
	 */
	public Map<String, byte[]> getLogFiles() throws IOException;


    /**
     * Retrieves from the persistence layer any persisted device parameter,
     * including a list of default parameter instances.
     * <p>
     * By "default parameters" we're referring to parameters established
     * by the System, for which the application's client hasn't provided
     * or persisted a value.
     * <p>
     * <b>NOTE</b>: Calling this method is equivalent to call:
     * <pre>
     *     IECMService.retrieveDeviceParameters(true);
     * </pre>
     *
     * @return
     *      {@code List<DeviceParameterData>} (never {@code null})
     *
     * @see #retrieveDeviceParameters(boolean)
     */
    List<DeviceParameterData> retrieveDeviceParameters();


    /**
     * Retrieves from the persistence layer any persisted device parameter,
     * including or excluding the list of default parameter instances.
     * <p>
     * By "default parameters" we're referring to parameters established
     * by the System, for which the application's client hasn't provided
     * or persisted a value.
     *
     * @param includeDefs
     *      {@code true} to include the list of default parameter instances;
     *      {@code false} to limit the result to only persisted parameters
     * @return
     *      {@code List<DeviceParameterData>} (never {@code null})
     */
    List<DeviceParameterData> retrieveDeviceParameters(boolean includeDefs);
}
