/**
 * LogConfiguration.java
 *
 * $Revision: 1.6 $ ($Date: 2011-04-06 04:43:04 $)
 *
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.log.configuration;

import java.util.LinkedList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Class used to manage log4j configuration
 * 
 * @author Carlos Lameda (carlos.lameda@smartmatic.com)
 * @version 1.0 (Nov 11, 2009)
 */
public class LogConfiguration {

	/**
	 * Log4j Configuration Parameter name
	 */
	private static final String LOG4J_CONFIG_ELEMENT_NAME = "log4j";

	/**
	 * XML Log document
	 */
	private Document doc;

	/**
	 * XML configuration parameter
	 */
	private ConfigurationParameter config;

	/**
	 * Boolean value indicate the log4j debug Element is enabled
	 */
	private boolean isDebugEnabled = false;

	/**
	 * Getter of <code>isDebugEnabled</code> property.
	 * 
	 * @return <code>isDebugEnabled</code> property value.
	 * @see #isDebugEnabled
	 */
	public boolean isDebugEnabled() {
		return this.isDebugEnabled;
	}

	/**
	 * Setter of <code>isDebugEnabled</code> property.
	 * 
	 * @param isDebugEnabled
	 *            The <code>isDebugEnabled</code> to set.
	 * @see #isDebugEnabled
	 */
	public void setDebugEnabled(boolean isDebugEnabled) {
		this.isDebugEnabled = isDebugEnabled;
	}

	/**
	 * Class LogConfiguration constructor
	 */
	public LogConfiguration() {
		this(null);
	}

	/**
	 * Class LogConfiguration constructor
	 * 
	 * @param log4jConfiguration
	 *            The log4j XML document
	 */
	public LogConfiguration(Document log4jConfiguration) {
		DocumentBuilderFactory factory;
		DocumentBuilder builder;
		NodeList log4jNodes;
		Element log4jElement;
		try {
			factory = DocumentBuilderFactory.newInstance();
			builder = factory.newDocumentBuilder();
			this.doc = builder.newDocument();
			if (log4jConfiguration != null) {
				// TODO Add error checking
				log4jNodes = this.doc
						.getElementsByTagName(LOG4J_CONFIG_ELEMENT_NAME);
				if (log4jNodes != null && log4jNodes.getLength() > 0) {
					log4jElement = (Element) log4jNodes.item(0);
					this.doc.appendChild(this.doc
							.importNode(log4jElement, true));
					if (log4jElement.getLocalName().equals(
							LOG4J_CONFIG_ELEMENT_NAME)) {
						this.config = convert(log4jElement);
					}
				}
				if (this.config == null) {
					throw new IllegalArgumentException(
							"Unable to get a log4j configuration element from the specified xml document");
				}
			} else {
				// TODO Create the root node "log4j node"
				this.config = new ConfigurationParameter(
						LOG4J_CONFIG_ELEMENT_NAME, null, null, null, null, true);
			}
		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * Method that convert an log4j node to a configuration parameter
	 * 
	 * @param log4jElement
	 *            Log4j node
	 * @return The configuration parameter node
	 */
	private ConfigurationParameter convert(Element log4jElement) {
		NodeList configElementList = log4jElement.getChildNodes();
		ConfigurationParameter configTree = new ConfigurationParameter(
				LOG4J_CONFIG_ELEMENT_NAME, null, null, null, null, true), currConfig;
		Element currElement;
		Node node;
		int i, len;

		len = configElementList.getLength();
		for (i = 0; i < len; i++) {
			node = configElementList.item(i);
			currConfig = null;
			if (node.getNodeType() == Node.ELEMENT_NODE) {
				currElement = (Element) node;
				if (currElement.getLocalName().equals(
						RootLoggerConfiguration.ELEMENT_LOCAL_NAME)) {
					currConfig = getLoggerConfiguration(currElement, true);
				} else if (currElement.getLocalName().equals(
						LoggerConfiguration.ELEMENT_LOCAL_NAME)) {
					currConfig = getLoggerConfiguration(currElement, false);
				} else if (currElement.getLocalName().equals(
						AppenderConfiguration.ELEMENT_LOCAL_NAME)) {
					currConfig = getAppenderConfiguration(currElement);
				}
			}
			if (currConfig != null) {
				configTree.addChildParameter(currConfig);
			}
		}

		return configTree;
	}

	/**
	 * Method retrieve a Configuration Attributes from
	 * ConfigurationAttributeType and an element using a bit mask
	 * 
	 * @param configElement
	 *            configuration to set
	 * @param attributesRequested
	 * @return A configuration attribute
	 */
	private ConfigurationAttributes getConfigurationAttributes(
			Element configElement,
			ConfigurationAttributeType attributesRequested) {
		ConfigurationAttributes att = new ConfigurationAttributes();

		if ((attributesRequested.getBitMask() & ConfigurationAttributeType.Name
				.getBitMask()) == ConfigurationAttributeType.Name.getBitMask()) {
			att.setCustomName(configElement
					.getAttribute(ConfigurationParameter.NAME_ATRIBUTE_NAME));
		}
		if ((attributesRequested.getBitMask() & ConfigurationAttributeType.Value
				.getBitMask()) == ConfigurationAttributeType.Value.getBitMask()) {
			att.setCustomName(configElement
					.getAttribute(ConfigurationParameter.VALUE_ATRIBUTE_NAME));
		}
		if ((attributesRequested.getBitMask() & ConfigurationAttributeType.Type
				.getBitMask()) == ConfigurationAttributeType.Type.getBitMask()) {
			att.setCustomName(configElement
					.getAttribute(ConfigurationParameter.TYPE_ATRIBUTE_NAME));
		}
		if ((attributesRequested.getBitMask() & ConfigurationAttributeType.Ref
				.getBitMask()) == ConfigurationAttributeType.Ref.getBitMask()) {
			att.setCustomName(configElement
					.getAttribute(ConfigurationParameter.REFERENCE_ATRIBUTE_NAME));
		}
		if ((attributesRequested.getBitMask() & ConfigurationAttributeType.Additivity
				.getBitMask()) == ConfigurationAttributeType.Additivity
				.getBitMask()) {
			att.setCustomName(configElement
					.getAttribute(ConfigurationParameter.ADDITIVITY_ATRIBUTE_NAME));
		}

		return att;
	}

	/**
	 * Get The appender configuration from the element
	 * 
	 * @param appenderElement
	 *            Appender element
	 * @return The requested appender configuration
	 */
	private ConfigurationParameter getAppenderConfiguration(
			Element appenderElement) {
		AppenderConfiguration config;
		ConfigurationAttributes att = getConfigurationAttributes(
				appenderElement, ConfigurationAttributeType.AppenderElement);
		Element element;
		NodeList nodeList;
		Node node;
		int i, len;

		config = new AppenderConfiguration(att.getCustomName(), att.getType());
		if (appenderElement.hasChildNodes()) {
			/* Special handling for appender-ref elements */
			nodeList = appenderElement.getChildNodes();
			len = nodeList.getLength();
			for (i = 0; i < len; i++) {
				node = nodeList.item(i);
				if (node instanceof Element) {
					element = (Element) node;
					if (element.getLocalName().equals(
							AppenderReferenceParameter.ELEMENT_LOCAL_NAME)) {
						addAppenderReferenceConfiguration(config, element);
					} else {
						addCostumConfiguration(config, element);
					}
				}
			}
		}

		return config;
	}

	/**
	 * Load a ConfigurationParameter from a XML Element
	 * 
	 * @param loggerElement
	 *            the XML Element
	 * @param isRoot
	 *            indicate if the node is a root
	 * @return The requested configuration parameter
	 */
	private ConfigurationParameter getLoggerConfiguration(
			Element loggerElement, boolean isRoot) {
		ConfigurationParameter config;
		ConfigurationAttributes att = getConfigurationAttributes(loggerElement,
				ConfigurationAttributeType.LoggerElement);
		Element element;
		NodeList nodeList;
		Node node;
		int i, len;

		if (isRoot) {
			config = new RootLoggerConfiguration();
		} else {
			config = new LoggerConfiguration(att.getCustomName(),
					att.getAdditivity());
		}
		if (loggerElement.hasChildNodes()) {
			/* Special handling for appender-ref and level elements */
			nodeList = loggerElement.getChildNodes();
			len = nodeList.getLength();
			for (i = 0; i < len; i++) {
				node = nodeList.item(i);
				if (node instanceof Element) {
					element = (Element) node;
					if (element.getLocalName().equals(
							AppenderReferenceParameter.ELEMENT_LOCAL_NAME)) {
						addAppenderReferenceConfiguration(config, element);
					} else if (element.getLocalName().equals(
							LevelParameter.ELEMENT_LOCAL_NAME)) {
						addLevelConfiguration(config, element);
					} else {
						addCostumConfiguration(config, element);
					}
				}
			}
		}

		return config;
	}

	/**
	 * Adds a level configuration element to a Configuration parameter
	 * 
	 * @param config
	 *            Configuration parameter
	 * @param element
	 *            The configuration element
	 */
	private void addLevelConfiguration(ConfigurationParameter config,
			Element element) {
		LevelParameter levelConfig;
		ConfigurationAttributes att = getConfigurationAttributes(element,
				ConfigurationAttributeType.LevelElement);
		levelConfig = new LevelParameter(att.getValue());
		config.addChildParameter(levelConfig);
	}

	/**
	 * Method user to add Appender Reference Configuration from a Configuration
	 * Attributes
	 * 
	 * @param config
	 *            Parameter to configure
	 * @param element
	 *            XML element
	 */
	private void addAppenderReferenceConfiguration(
			ConfigurationParameter config, Element element) {
		AppenderReferenceParameter apdRefConfig;
		ConfigurationAttributes att = getConfigurationAttributes(element,
				ConfigurationAttributeType.AppenderReferenceElement);
		apdRefConfig = new AppenderReferenceParameter(att.getRef());
		config.addChildParameter(apdRefConfig);
	}

	/**
	 * Add a custom(different) configuration parameter
	 * 
	 * @param config
	 *            The Parameter
	 * @param element
	 *            The element to add
	 */
	private void addCostumConfiguration(ConfigurationParameter config,
			Element element) {
		ConfigurationParameter customConfig;
		ConfigurationAttributes att = getConfigurationAttributes(element,
				ConfigurationAttributeType.All);
		NodeList nodeList;
		Node node;
		int i, len;
		boolean useCostumNameForElementName = false;

		if (att.getCustomName() == null || att.getCustomName().length() == 0) {
			/* Name attribute not present */
			att.setCustomName(element.getLocalName());
			/*
			 * In this case, the attribute name will be suppressed at the time
			 * of creation of the log4j element
			 */
			useCostumNameForElementName = true;
		}
		if (!att.getCustomName().equals(element.getLocalName())) {
			/*
			 * In this case, the name attribute was present and it was different
			 * to the element name so don't use the name attribute for the
			 * element name at the time of creation of the log4j element
			 */
			useCostumNameForElementName = false;
		}
		customConfig = new ConfigurationParameter(att.getCustomName(),
				att.getValue(), att.getType(), att.getRef(),
				att.getAdditivity(), useCostumNameForElementName);
		if (!att.getCustomName().equals(element.getLocalName())) {
			/* Set the name of the element */
			customConfig.setDefaultElementName(element.getLocalName());
		}
		if (element.hasChildNodes()) {
			nodeList = element.getChildNodes();
			len = nodeList.getLength();
			for (i = 0; i < len; i++) {
				node = nodeList.item(i);
				if (node instanceof Element) {
					addCostumConfiguration(customConfig, (Element) node);
				}
			}
		}

		/* Finally add the custom configuration */
		config.addChildParameter(customConfig);
	}

	/**
	 * Create a config Log4J Configuration Element
	 * 
	 * @return the element
	 */
	public Element getLog4JConfigurationElement() {
		Element log4jElement = this.config.getXMLElement();

		log4jElement.setAttribute("debug", this.isDebugEnabled ? "true"
				: "false");

		return log4jElement;
	}

	/**
	 * Add a list of appenders to a logger
	 * 
	 * @param loggerName
	 *            The logger configuration name
	 * @param additivity
	 *            Boolean additivity
	 * @param level
	 *            The level name
	 * @param appendersName
	 *            The list of appenders reference names
	 */
	public void addLogger(String loggerName, boolean additivity, String level,
			Iterable<String> appendersName) {
		/* Create a logger */
		LoggerConfiguration config = new LoggerConfiguration(loggerName,
				additivity ? "true" : "false");

		/* Add the level parameter */
		config.addChildParameter(new LevelParameter(level));
		if (appendersName != null) {
			config.addChildParameter(new LevelParameter(level));
			for (String apdRef : appendersName) {
				config.addChildParameter(new AppenderReferenceParameter(apdRef));
			}
		}

		/* Add the logger to the log4j configuration */
		config.addChildParameter(config);
	}

	/**
	 * Add an appender to a logger using their names
	 * 
	 * @param loggerName
	 *            Logger name
	 * @param appenderName
	 *            Appender name
	 */

	public void addAppenderToLogger(String loggerName, String appenderName) {
		LinkedList<ConfigurationParameter> parameters = this.config
				.getInnerParams();
		LoggerConfiguration logger;

		if (parameters != null) {
			for (ConfigurationParameter param : parameters) {
				logger = (LoggerConfiguration) param;
				if (logger != null && logger.getCustomName().equals(loggerName)) {
					logger.addChildParameter(new AppenderReferenceParameter(
							appenderName));
				}
			}
		}
	}

	/**
	 * Append a parameter to a log4j element
	 * 
	 * @param appenderName
	 *            The String contain the appender name
	 * @param appenderType
	 *            The String contain the appender type
	 * @param parameters
	 *            The parameters to add
	 */
	public void addAppender(String appenderName, String appenderType,
			Iterable<ConfigurationParameter> parameters) {
		/* Create the appender */
		AppenderConfiguration config = new AppenderConfiguration(appenderName,
				appenderType);

		if (parameters != null) {
			/* Add the parameters */
			for (ConfigurationParameter parameter : parameters) {
				config.addChildParameter(parameter);
			}
		}

		/* Add the appender to the log4j element */
		config.addChildParameter(config);
	}

	/**
	 * Add AppenderConfiguration to the Configuration Parameter without replace
	 * 
	 * @param appenderName
	 *            The appender configuration name
	 * @param parameter
	 *            The parameter to set
	 */
	public void addParameterToAppender(String appenderName,
			ConfigurationParameter parameter) {
		addParameterToAppender(appenderName, this.config, false);
	}

	/**
	 * Add AppenderConfiguration to the Configuration Parameter
	 * 
	 * @param appenderName
	 *            The appender configuration name
	 * @param parameter
	 *            the parameter to add
	 * @param replace
	 *            indicate if replace the
	 */
	public void addParameterToAppender(String appenderName,
			ConfigurationParameter parameter, boolean replace) {
		LinkedList<ConfigurationParameter> parameters = parameter
				.getInnerParams();
		AppenderConfiguration appender;

		if (parameters != null) {
			for (ConfigurationParameter param : parameters) {
				appender = (AppenderConfiguration) param;
				if (appender != null
						&& appender.getCustomName().equals(appenderName)) {
					appender.addChildParameter(parameter, replace);
				}
			}
		}
	}

	/**
	 * Retrieve a list of XML RootLogger configuration parameter names
	 * 
	 * @return The array with the RootLogger AppendersReference
	 */
	public String[] getRootLoggerAppendersReference() {
		LinkedList<String> appenders = new LinkedList<String>();
		LinkedList<ConfigurationParameter> parameters = this.config
				.getInnerParams();
		RootLoggerConfiguration rootL;
		AppenderReferenceParameter appender;

		if (parameters != null) {
			for (ConfigurationParameter param : parameters) {
				rootL = (RootLoggerConfiguration) param;
				if (rootL != null) {
					for (ConfigurationParameter par : rootL.getInnerParams()) {
						appender = (AppenderReferenceParameter) par;
						if (appender != null) {
							appenders.add(appender.getReference());
						}
					}
				}
			}
		}

		return appenders.toArray(new String[] {});
	}

	/**
	 * Retrieve a list of XML reference from configuration parameter names
	 * 
	 * @param loggerName
	 *            The parameters name to retrieve
	 * @return the List with the references
	 */
	public String[] getAppenderReference(String loggerName) {
		LinkedList<String> appenders = new LinkedList<String>();
		LinkedList<ConfigurationParameter> parameters = this.config
				.getInnerParams();
		LoggerConfiguration logger;
		AppenderReferenceParameter appender;

		if (parameters != null) {
			for (ConfigurationParameter param : parameters) {
				logger = (LoggerConfiguration) param;
				if (logger != null && logger.getCustomName().equals(loggerName)) {
					for (ConfigurationParameter par : logger.getInnerParams()) {
						appender = (AppenderReferenceParameter) par;
						if (appender != null) {
							appenders.add(appender.getReference());
						}
					}
				}
			}
		}

		return appenders.toArray(new String[] {});
	}

	/**
	 * Retrieve a list of XML configuration parameter names
	 * 
	 * @return The array with the names
	 */
	public String[] getLoggersName() {
		LinkedList<String> loggersN = new LinkedList<String>();
		LinkedList<ConfigurationParameter> parameters = this.config
				.getInnerParams();
		LoggerConfiguration log;

		if (parameters != null) {
			for (ConfigurationParameter param : parameters) {
				log = (LoggerConfiguration) param;
				if (log != null) {
					loggersN.add(log.getCustomName());
				}
			}
		}

		return loggersN.toArray(new String[] {});
	}

}
