/**
 * SmartCryptoLayoutTransformer.java
 *
 * $Revision: 1.5 $ ($Date: 2011-04-06 04:43:01 $)
 *
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.log.custom.layout;

import javax.crypto.SecretKey;

import com.smartmatic.waffle.log.SMTTLogger;
import com.smartmatic.waffle.security.lang.CryptographicRuntimeException;
import com.smartmatic.waffle.security.lang.CryptographicStaticException;
import com.smartmatic.waffle.security.symmetric.PoolableSymmetricCipher;

/**
 * Layout Class used with Log4j to encrypt / decrypt the Logger of the
 * application.
 * 
 * @author Daniel Mora daniel.mora@smartmatic.com
 * @version 1.0
 * @since jdk.1.6.0_11
 * 
 */
public class SmartCryptoLayoutTransformer implements CryptoLayoutTransformer {

	/**
	 * Secret key used to encrypt / decrypt the data.
	 */
	SecretKey key;

	/**
	 * Cipher used to encrypt / decrypt the data.
	 */
	PoolableSymmetricCipher poolableCipher;

	/**
	 * Default constructor that assign the key and the cipher to the class.
	 * 
	 * @throws CryptographicStaticException
	 *             Thrown when some errors related to static members ciphering
	 *             the key
	 * @throws CryptographicRuntimeException
	 *             Thrown when some errors occur ciphering the key
	 */
	public SmartCryptoLayoutTransformer() throws CryptographicStaticException,
			CryptographicRuntimeException {
		this.key = SMTTLogger.getKey();
		if (this.key != null) {
			this.poolableCipher = new PoolableSymmetricCipher(this.key);
		}
	}

	/**
	 * Method that receive a String and return it decrypted.
	 * 
	 * @see com.smartmatic.waffle.log.custom.layout.CryptoLayoutTransformer#decryptString
	 *      (java.lang.String)
	 * 
	 * @param encyptedData
	 *            The data to decrypt.
	 * @return the data decrypted if the {@link #key} attribute isn't null, the
	 *         same data received in the parameter encyptedData in other case.
	 */
	@Override
	public String decryptString(String encyptedData) {
		String decryptedData = null;
		if (this.key != null) {
			try {
				decryptedData = this.poolableCipher.decrypt(encyptedData);
			} catch (CryptographicRuntimeException e) {
				e.printStackTrace();
			}
		} else {
			decryptedData = encyptedData;
		}
		return decryptedData;
	}

	/**
	 * Method that receive a String and return it encrypted.
	 * 
	 * @see com.smartmatic.waffle.log.custom.layout.CryptoLayoutTransformer#encryptString
	 *      (java.lang.String)
	 * 
	 * @param plainData
	 *            . The data to encrypt.
	 * @return the data encrypted if the {@link #key} attribute isn't null, the
	 *         same data received in the parameter plainData in other case.
	 */
	@Override
	public String encryptString(String plainData) {
		String encryptedData = null;
		if (this.key != null) {
			try {
				encryptedData = this.poolableCipher.encrypt(plainData);
				encryptedData += "\n";
			} catch (CryptographicRuntimeException e) {
				e.printStackTrace();
			}
		} else {
			encryptedData = plainData;
		}
		return encryptedData;
	}

}
