/**
 * MatchCriteriaMode.java
 *
 * $Revision: 1.3 $ ($Date: 2011-04-06 04:45:07 $)
 *
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.persistence.search.criteria;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import org.hibernate.criterion.MatchMode;

/**
 * Represents an strategy for matching strings using "like".
 * 
 * @see Example#enableLike(MatchMode)
 * @author Ithamar Torres
 */

public abstract class MatchCriteriaMode implements Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Code used to identify a match mode
	 */

	private final int intCode;

	/**
	 * Name of the match mode
	 */

	private final String name;

	/**
	 * Map used to store all the match modes supported
	 */

	private static final Map<Integer, MatchCriteriaMode> INSTANCES = new HashMap<Integer, MatchCriteriaMode>();

	/**
	 * Constructor of a match mode. This method is not public because the way to
	 * access a match mode is through static methods that create the match mode
	 * required
	 * 
	 * @param intCode
	 *            The code used to identify the match mode
	 * @param name
	 *            Name of the match mode
	 */

	protected MatchCriteriaMode(int intCode, String name) {
		this.intCode = intCode;
		this.name = name;
	}

	/**
	 * Match the entire string to the pattern
	 */

	public static final MatchCriteriaMode EXACT = new MatchCriteriaMode(0,
			"EXACT") {

		/**
		 * Holds the serialization id of the class
		 */
		private static final long serialVersionUID = 4366201773862219087L;

		/**
		 * Convert the pattern
		 * 
		 * @param value
		 *            String value to find
		 * @return SQL pattern
		 */
		@Override
		String toMatchString(String pattern) {
			return pattern;
		}
	};

	/**
	 * Match the START of the string to the pattern
	 */

	public static final MatchCriteriaMode START = new MatchCriteriaMode(1,
			"HTTP_PARAM_START") {

		/**
		 * Holds the serialization id of the class
		 */
		private static final long serialVersionUID = -8795975119508167998L;

		/**
		 * Convert the pattern, by appending "%"
		 * 
		 * @param value
		 *            String value to find
		 * @return SQL pattern
		 */
		@Override
		String toMatchString(String pattern) {
			return pattern + '%';
		}
	};

	/**
	 * Match the end of the string to the pattern
	 */

	public static final MatchCriteriaMode END = new MatchCriteriaMode(2, "END") {

		/**
		 * Holds the serialization id of the class
		 */
		private static final long serialVersionUID = 5876278855782780872L;

		/**
		 * Convert the pattern, by prepending "%"
		 * 
		 * @param value
		 *            String value to find
		 * @return SQL pattern
		 */
		@Override
		String toMatchString(String pattern) {
			return '%' + pattern;
		}
	};

	/**
	 * Match the pattern anywhere in the string
	 */

	public static final MatchCriteriaMode ANYWHERE = new MatchCriteriaMode(3,
			"ANYWHERE") {

		/**
		 * Holds the serialization id of the class
		 */
		private static final long serialVersionUID = -763082498129977462L;

		/**
		 * Convert the pattern, by appending,prepending "%"
		 * 
		 * @param value
		 *            String value to find
		 * @return SQL pattern
		 */
		@Override
		String toMatchString(String pattern) {
			return '%' + pattern + '%';
		}
	};

	/**
	 * Method that create all the supported match modes and stores it in a map.
	 */

	static {
		MatchCriteriaMode.INSTANCES.put(new Integer(
				MatchCriteriaMode.EXACT.intCode), MatchCriteriaMode.EXACT);
		MatchCriteriaMode.INSTANCES.put(new Integer(
				MatchCriteriaMode.END.intCode), MatchCriteriaMode.END);
		MatchCriteriaMode.INSTANCES.put(new Integer(
				MatchCriteriaMode.START.intCode), MatchCriteriaMode.START);
		MatchCriteriaMode.INSTANCES
				.put(new Integer(MatchCriteriaMode.ANYWHERE.intCode),
						MatchCriteriaMode.ANYWHERE);
	}

	/**
	 * Method that returns the match criteria mode given the intCode
	 * 
	 * @return The requested match criteria mode
	 */
	private Object readResolve() {
		return MatchCriteriaMode.INSTANCES.get(new Integer(this.intCode));
	}

	/**
	 * convert the pattern, by appending/prepending "%"
	 */

	abstract String toMatchString(String pattern);

	/**
	 * Returns a string representation of the object.
	 * 
	 * @return The string representation of the mode
	 */
	@Override
	public String toString() {
		return this.name;
	}

}
