/**
 * ApplicationProperties.java
 *
 * Revision: $Revision$  (Last Modified: $Date$)
 * Last Modified by: [$Author$]
 * 
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.properties;

import java.io.File;
import java.io.InputStream;
import java.util.Map;
import java.util.Set;

import org.igfay.jfig.JFig;
import org.igfay.jfig.JFigConstants;
import org.igfay.jfig.JFigException;
import org.igfay.jfig.JFigLocator;
import org.igfay.jfig.JFigLocatorIF;

import com.smartmatic.waffle.properties.SmartProperties.Type;

/**
 * Base class used to obtain any property required in the application.
 * 
 * @version 1.0 (May 23, 2007)
 * @author Ithamar Torres (itorres@smartmatic.com)
 */

public class ApplicationProperties {

	/** Object used to get references indicated in properties file. */
	private JFig properties;

	/** The locator. */
	private JFigLocatorIF locator;

	/** Singleton application properties instance. */
	private static ApplicationProperties singletonInstance = null;

	/**
	 * Property used to indicate the path to XSD file to validate config
	 * properties file.
	 */
	private static final String XSD_VALIDATION_PATH = System
			.getProperty("jboss.server.home.dir")
			+ "/xsd/config-properties.xsd";

	/**
	 * Property used to indicate the path to load the config properties file
	 * plain.
	 */
	private static final String CONFIG_PLAIN_PROPERTIES_FILEPATH = System
			.getProperty("jboss.server.home.dir")
			+ "/conf/config-properties.xml";

	/**
	 * Property used to indicate a default section of the configuration files is
	 * goin to be used.
	 */
	private static final String DEFAULT_PROPERTIES_SECTION = "DEFAULT";

	/**
	 * Application properties constructor, it use the file path to a file using
	 * java Properties path.
	 * 
	 * @param filePath
	 *            File path to the property file
	 * @param xsdPath
	 *            the xsd path
	 * @throws ApplicationPropertiesException
	 *             Thrown loading the file property
	 * @see #Properties
	 */
	private ApplicationProperties(String filePath, String xsdPath)
			throws ApplicationPropertiesException {
		try {
			this.locator = new SmartPropertiesLocator(filePath, xsdPath);
			this.properties = (JFig) JFig.getInstance(this.locator);
		} catch (Exception e) {
			throw new ApplicationPropertiesException(
					ApplicationPropertiesException.ExceptionInfo.INITIALIZATION_EXCEPTION);
		}
	}

	/**
	 * Instantiates a new application properties.
	 */
	private ApplicationProperties() {
	}

	/**
	 * Application properties constructor, it use an input stream using java
	 * Properties.
	 * 
	 * @param stream
	 *            Property input stream
	 * @throws ApplicationPropertiesException
	 *             Thrown loading the file property
	 * @see #Properties
	 */
	private ApplicationProperties(InputStream stream)
			throws ApplicationPropertiesException {
		try {
			this.locator = new SmartPropertiesLocator(stream,
					XSD_VALIDATION_PATH);
			this.properties = (JFig) JFig.getInstance(this.locator);
		} catch (Exception e) {
			throw new ApplicationPropertiesException(
					ApplicationPropertiesException.ExceptionInfo.INITIALIZATION_EXCEPTION);
		}
	}

	/**
	 * Checks if is application properties instantiated.
	 * 
	 * @return true, if is application properties instantiated
	 */
	public static boolean isApplicationPropertiesInstantiated() {
		if (singletonInstance != null) {
			return true;
		}

		return false;
	}

	/**
	 * Application properties initializer, it use an input stream using java
	 * Properties.
	 * 
	 * @param fileType
	 *            the file type
	 * @return true, if successful
	 * @throws ApplicationPropertiesException
	 *             Thrown loading the file property
	 * @see #Properties
	 */
	public synchronized boolean init(Type fileType)
			throws ApplicationPropertiesException {
		if (ApplicationProperties.singletonInstance.properties == null) {
			return ApplicationProperties.singletonInstance.startUp(fileType);
		}
		return true;
	}

	/**
	 * Start up.
	 * 
	 * @param fileType
	 *            the file type
	 * @return true, if successful
	 */
	private boolean startUp(Type fileType) {
		try {
			if (fileType == Type.PLAIN) {
				this.locator = new SmartPropertiesLocator(
						ApplicationProperties.CONFIG_PLAIN_PROPERTIES_FILEPATH,
						ApplicationProperties.XSD_VALIDATION_PATH);
				this.properties = (JFig) JFig.getInstance(this.locator);
				return (this.properties != null);
			}
			return false;
		} catch (Exception e) {
			throw new ApplicationPropertiesException(
					ApplicationPropertiesException.ExceptionInfo.INITIALIZATION_EXCEPTION,
					e);
		}
	}

	/**
	 * Application properties initializer, it use an input stream using java
	 * Properties.
	 * 
	 * @param stream
	 *            Property input stream
	 * @throws ApplicationPropertiesException
	 *             Thrown loading the file property
	 * @see #Properties
	 */
	public synchronized static void init(InputStream stream)
			throws ApplicationPropertiesException {
		if (ApplicationProperties.singletonInstance != null) {
			throw new ApplicationPropertiesException(
					ApplicationPropertiesException.ExceptionInfo.SERVICE_ALREADY_INITIALIZED);
		}
		ApplicationProperties.singletonInstance = new ApplicationProperties(
				stream);
	}

	/**
	 * Inits the.
	 * 
	 * @param path
	 *            the path
	 * @param xsd
	 *            the xsd
	 */
	public synchronized static void init(String path, String xsd) {
		if (ApplicationProperties.singletonInstance != null) {
			throw new ApplicationPropertiesException(
					ApplicationPropertiesException.ExceptionInfo.SERVICE_ALREADY_INITIALIZED);
		}
		ApplicationProperties.singletonInstance = new ApplicationProperties(
				path, xsd);
	}

	/**
	 * Method that cast a value to another type.
	 * 
	 * @param <T>
	 *            Destination class type
	 * @param value
	 *            The value to be transformed
	 * @param classType
	 *            Destination class
	 * @return The value casted
	 * @see #Class
	 */
	@SuppressWarnings("unchecked")
	private static <T> T transform(String value, Class<T> classType) {
		T result = null;
		if (value != null) {
			try {
				if (classType == Boolean.class) {
					result = ((Class<T>) Boolean.class).cast(Boolean
							.valueOf(value));
				} else if (classType == Byte.class) {
					result = ((Class<T>) Byte.class).cast(Byte.valueOf(value));
				} else if (classType == Character.class) {
					result = ((Class<T>) Character.class).cast(Character
							.valueOf(value.charAt(0)));
				} else if (classType == Double.class) {
					result = ((Class<T>) Double.class).cast(Double
							.valueOf(value));
				} else if (classType == Float.class) {
					result = ((Class<T>) Float.class)
							.cast(Float.valueOf(value));
				} else if (classType == Integer.class) {
					result = ((Class<T>) Integer.class).cast(Integer
							.valueOf(value));
				} else if (classType == Long.class) {
					result = ((Class<T>) Long.class).cast(Long.valueOf(value));
				} else if (classType == Short.class) {
					result = ((Class<T>) Short.class)
							.cast(Short.valueOf(value));
				} else { // hard cast:
					result = classType.cast(value);
				}
			} catch (Exception ex) {
				throw new ApplicationPropertiesException(
						ApplicationPropertiesException.ExceptionInfo.ERROR_CONVERTING_TO_TYPE);
			}
		}
		return result;
	}

	/**
	 * Get a property contained in the single instance and cast it to Int.
	 * 
	 * @param propertyName
	 *            The property name to get
	 * @return The value casted
	 * @see #Integer
	 */
	public static int getIntValue(String propertyName) {
		String value = getValue(propertyName);
		return transform(value, java.lang.Integer.class);
	}

	/**
	 * Gets the int value.
	 * 
	 * @param section
	 *            the section
	 * @param propertyName
	 *            the property name
	 * @return the int value
	 */
	public static int getIntValue(IPropertySection section, String propertyName) {
		String value = getValue(section, propertyName);
		return transform(value, java.lang.Integer.class);
	}

	/**
	 * Get a property contained in the single instance and cast it to Int.
	 * 
	 * @param propertyName
	 *            The property name to get
	 * @param defaultValue
	 *            The value if the property are not defined
	 * @return The value casted
	 * @see #Integer
	 */
	public static int getIntValue(String propertyName, int defaultValue) {
		String value;
		try {
			value = getValue(propertyName);

			if (value != null) {
				return transform(value, java.lang.Integer.class);
			} else {
				return defaultValue;
			}
		} catch (ApplicationPropertiesException e) {
			if (e.getInfo() == ApplicationPropertiesException.ExceptionInfo.PROPERTY_NOT_FOUND) {
				return defaultValue;
			} else {
				throw e;
			}
		}
	}

	/**
	 * Gets the int value.
	 * 
	 * @param section
	 *            the section
	 * @param propertyName
	 *            the property name
	 * @param defaultValue
	 *            the default value
	 * @return the int value
	 */
	public static int getIntValue(IPropertySection section,
			String propertyName, int defaultValue) {
		String value;
		try {
			value = getValue(section, propertyName);

			if (value != null) {
				return transform(value, java.lang.Integer.class);
			} else {
				return defaultValue;
			}
		} catch (ApplicationPropertiesException e) {
			if (e.getInfo() == ApplicationPropertiesException.ExceptionInfo.PROPERTY_NOT_FOUND) {
				return defaultValue;
			} else {
				throw e;
			}
		}
	}

	/**
	 * Get a property contained in the single instance and cast it to Boolean.
	 * 
	 * @param propertyName
	 *            The property name to get
	 * @return The value casted
	 * @see #Boolean
	 */
	public static boolean getBooleanValue(String propertyName) {
		String value = getValue(propertyName);
		return transform(value, java.lang.Boolean.class);
	}

	/**
	 * Gets the boolean value.
	 * 
	 * @param section
	 *            the section
	 * @param propertyName
	 *            the property name
	 * @return the boolean value
	 */
	public static boolean getBooleanValue(IPropertySection section,
			String propertyName) {
		String value = getValue(section, propertyName);
		return transform(value, java.lang.Boolean.class);
	}

	/**
	 * Get a property contained in the single instance.
	 * 
	 * @param <T>
	 *            The type to cast the value
	 * @param propertyName
	 *            The property name to get
	 * @return The value casted
	 */
	@SuppressWarnings("unchecked")
	public static <T> T getValue(String propertyName) {
		if (ApplicationProperties.singletonInstance == null) {
			throw new ApplicationPropertiesException(
					ApplicationPropertiesException.ExceptionInfo.SERVICE_NOT_INITIALIZED);
		}

		String result;
		try {
			result = singletonInstance.properties.getValue(
					DEFAULT_PROPERTIES_SECTION, propertyName);
		} catch (JFigException e) {
			throw new ApplicationPropertiesException(
					ApplicationPropertiesException.ExceptionInfo.PROPERTY_NOT_FOUND,
					propertyName);
		}

		if (result == null) {
			throw new ApplicationPropertiesException(
					ApplicationPropertiesException.ExceptionInfo.PROPERTY_NOT_FOUND,
					propertyName);
		}

		return (T) result;
	}

	/**
	 * Gets the value.
	 * 
	 * @param <T>
	 *            the generic type
	 * @param propertyName
	 *            the property name
	 * @param defaultValue
	 *            the default value
	 * @return the value
	 */
	@SuppressWarnings("unchecked")
	public static <T> T getValue(String propertyName, String defaultValue) {
		if (ApplicationProperties.singletonInstance == null) {
			throw new ApplicationPropertiesException(
					ApplicationPropertiesException.ExceptionInfo.SERVICE_NOT_INITIALIZED);
		}

		String result = singletonInstance.properties.getValue(
				DEFAULT_PROPERTIES_SECTION, propertyName, defaultValue);

		if (result == null) {
			throw new ApplicationPropertiesException(
					ApplicationPropertiesException.ExceptionInfo.PROPERTY_NOT_FOUND,
					propertyName);
		}

		return (T) result;
	}

	/**
	 * Gets the value.
	 * 
	 * @param <T>
	 *            the generic type
	 * @param section
	 *            the section
	 * @param propertyName
	 *            the property name
	 * @return the value
	 */
	@SuppressWarnings("unchecked")
	public static <T> T getValue(IPropertySection section, String propertyName) {
		if (ApplicationProperties.singletonInstance == null) {
			throw new ApplicationPropertiesException(
					ApplicationPropertiesException.ExceptionInfo.SERVICE_NOT_INITIALIZED);
		}

		String result;
		try {
			result = singletonInstance.properties.getValue(section.getName(),
					propertyName);
		} catch (JFigException e) {
			throw new ApplicationPropertiesException(
					ApplicationPropertiesException.ExceptionInfo.PROPERTY_NOT_FOUND,
					"SECTION[" + section.getName() + "]", propertyName);
		}

		if (result == null) {
			throw new ApplicationPropertiesException(
					ApplicationPropertiesException.ExceptionInfo.PROPERTY_NOT_FOUND,
					"SECTION[" + section.getName() + "]", propertyName);
		}

		return (T) result;
	}

	/**
	 * Gets the value.
	 * 
	 * @param <T>
	 *            the generic type
	 * @param section
	 *            the section
	 * @param propertyName
	 *            the property name
	 * @param defaultValue
	 *            the default value
	 * @return the value
	 */
	@SuppressWarnings("unchecked")
	public static <T> T getValue(IPropertySection section, String propertyName,
			String defaultValue) {
		if (ApplicationProperties.singletonInstance == null) {
			throw new ApplicationPropertiesException(
					ApplicationPropertiesException.ExceptionInfo.SERVICE_NOT_INITIALIZED);
		}

		String result = singletonInstance.properties.getValue(
				section.getName(), propertyName, defaultValue);

		if (result == null) {
			throw new ApplicationPropertiesException(
					ApplicationPropertiesException.ExceptionInfo.PROPERTY_NOT_FOUND,
					"SECTION[" + section.getName() + "]", propertyName);
		}

		return (T) result;
	}

	/**
	 * Set a property contained in the single instance a value in a section
	 * DEFAULT.
	 * 
	 * @param propertyName
	 *            The property name to set
	 * @param value
	 *            Value to be set
	 */
	public static void setValue(String propertyName, String value) {
		if (ApplicationProperties.singletonInstance == null) {
			throw new ApplicationPropertiesException(
					ApplicationPropertiesException.ExceptionInfo.SERVICE_NOT_INITIALIZED);
		}
		singletonInstance.properties.setConfigurationValue(
				DEFAULT_PROPERTIES_SECTION, propertyName, value);
	}

	/**
	 * Set a property contained in the single instance a value in a section.
	 * 
	 * @param sectionName
	 *            The section name
	 * @param propertyName
	 *            The property name to set
	 * @param value
	 *            Value to be set
	 */
	public static void setValue(String sectionName, String propertyName,
			String value) {
		if (ApplicationProperties.singletonInstance == null) {
			throw new ApplicationPropertiesException(
					ApplicationPropertiesException.ExceptionInfo.SERVICE_NOT_INITIALIZED);
		}
		singletonInstance.properties.setConfigurationValue(sectionName,
				propertyName, value);
	}

	/**
	 * Load the property names contained in the single instance.
	 * 
	 * @return a Set of property keys
	 * @see #Properties
	 */
	@SuppressWarnings("unchecked")
	public static Set<String> getPropertyNames() {

		if (ApplicationProperties.singletonInstance == null) {
			throw new ApplicationPropertiesException(
					ApplicationPropertiesException.ExceptionInfo.SERVICE_NOT_INITIALIZED);
		}
		return singletonInstance.properties.getSection(
				DEFAULT_PROPERTIES_SECTION).keySet();
	}

	/**
	 * Gets the property names.
	 * 
	 * @param section
	 *            the section
	 * @return the property names
	 */
	public static Set<String> getPropertyNames(IPropertySection section) {

		if (ApplicationProperties.singletonInstance == null) {
			throw new ApplicationPropertiesException(
					ApplicationPropertiesException.ExceptionInfo.SERVICE_NOT_INITIALIZED);
		}

		return singletonInstance.properties.getSectionAsProperties(
				section.getName()).stringPropertyNames();
	}

	/**
	 * Gets the section.
	 * 
	 * @param section
	 *            the section
	 * @return the section
	 */
	@SuppressWarnings("rawtypes")
	public static Map getSection(IPropertySection section) {

		if (ApplicationProperties.singletonInstance == null) {
			throw new ApplicationPropertiesException(
					ApplicationPropertiesException.ExceptionInfo.SERVICE_NOT_INITIALIZED);
		}

		return singletonInstance.properties.getSection(section.getName());
	}

	/**
	 * Returns the singleton instance.
	 * 
	 * @return the singleton instance
	 */
	public static ApplicationProperties getInstance() {
		if (singletonInstance == null) {
			singletonInstance = new ApplicationProperties();
		}
		return singletonInstance;
	}

	/**
	 * Read again the properties file to see the new changes.
	 * 
	 * @throws JFigException
	 *             the j fig exception
	 */
	public void reprocess() throws JFigException {
		JFigLocator locator = new JFigLocator("config-properties.xml");
		locator.setConfigDirectory(System.getProperty("jboss.server.home.dir")
				+ File.separator + "conf" + File.separator);
		locator.setConfigLocation(JFigConstants.FILE);
		JFig.initialize(locator);
		synchronized (singletonInstance) {
			singletonInstance.properties = (JFig) JFig.getInstance(locator);
			singletonInstance.properties.reprocessConfiguration(locator);
		}
	}

}
