/**
 * BaseDataObject.java
 *
 * $Revision: 1.10 $ ($Date: 2011-04-06 05:08:29 $)
 *
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.domain;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.smartmatic.waffle.lang.annotations.basic.EqualsFieldAnnotation;

/**
 * Abstract class that its the root of all the inheritance tree of the Business
 * components. Every object that should persists in the database must inherits
 * from this class.
 * 
 * @author Ithamar Torres (itorres@smartmatic.com)
 * @version 1.0 (22/06/2006)
 * @refactoredBy Alden Torres (atorres@smartmatic.com)
 */
public abstract class BaseDataObject implements Serializable, IBaseDataObject {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Cached methods for the generic base data objects
	 */
	private final static Map<Class<BaseDataObject>, Method[]> cachedMethodsForHash = new HashMap<Class<BaseDataObject>, Method[]>();

	/**
	 * It is the prefix used by those getter methods required in equals and/or
	 * hashCode method
	 */

	private static final String PREFIX_METHOD = "get";

	/**
	 * It is the prefix optionally used by those getter methods (that have a
	 * <code>boolean</code>turn type) required in equals and/or hashCode method
	 */
	private static final String BOOLEAN_OPT_PREFIX_METHOD = "is";

	/**
	 * Field used to maintain the version of the Business component.
	 * 
	 * This field its used to avoid that two users read the same object and
	 * updates the same object with a change lost because the latest change
	 * overwrites the previous one.
	 * 
	 * @author Ithamar Torres (itorres@smartmatic.com)
	 */

	protected int version;

	/**
	 * Setter of <code>version</code> property.
	 * 
	 * @param version
	 *            The <code>version</code> to set.
	 * @see #version
	 */
	@Override
	public void setVersion(int version) {
		this.version = version;
	}

	/**
	 * Handle optimistic hibernate concurrency
	 * 
	 * @see com.smartmatic.waffle.domain.IBaseDataObject#getVersion()
	 */
	@Override
	public abstract int getVersion();

	/**
	 * Generates a cache for the generic implementation of hashCode method which
	 * needs java.lang.reflect api and EqualsFieldAnnotation annotation.
	 * 
	 * @version 1.0 (Apr 9, 2007)
	 * @param clazz
	 *            Class which will be searched for properties that will be used
	 *            for the hash code method
	 * @return int The hashcode of the object
	 */
	protected Method[] buildCachedMethodsForHash(Class clazz) {

		List<Method> methodList = new ArrayList<Method>();
		while (BaseDataObject.class.isAssignableFrom(clazz)) {
			Field[] fields = clazz.getDeclaredFields();
			for (Field field : fields) {
				/**
				 * boolean value to know if the field is required to calculate
				 * the hashCode value
				 */
				boolean doesItHaveTheAnnotation = field
						.isAnnotationPresent(EqualsFieldAnnotation.class);
				if (doesItHaveTheAnnotation) {
					/** field name */
					String fieldName = field.getName();
					/** field name with capital letter */
					String firstLetterUpper = Character.toUpperCase(fieldName
							.charAt(0)) + fieldName.substring(1);
					/** method name */
					String getMethodName = PREFIX_METHOD + firstLetterUpper;
					/** instance of method */
					try {
						Method getMethod = clazz.getMethod(getMethodName,
								(Class[]) null);
						methodList.add(getMethod);

					} catch (SecurityException exception) {
						exception.printStackTrace();
					} catch (NoSuchMethodException exception) {

						getMethodName = BOOLEAN_OPT_PREFIX_METHOD
								+ firstLetterUpper;
						try {
							Method getMethod = clazz.getMethod(getMethodName,
									(Class[]) null);
							methodList.add(getMethod);

						} catch (SecurityException exceptionInside) {
							exceptionInside.printStackTrace();
						} catch (NoSuchMethodException exceptionInside) {

							/* Show the original exception's stack trace */
							exception.printStackTrace();

						}

					}

				}
			}
			clazz = clazz.getSuperclass();
		}
		return methodList.toArray(new Method[methodList.size()]);
	}

	/**
	 * It is a generic implementation of hashCode method which needs
	 * java.lang.reflect api and EqualsFieldAnnotation annotation.
	 * 
	 * @author Luis Felipe Barrera (lbarrera@smartmatic.com)
	 * @version 1.0 (Apr 9, 2007)
	 * @param instance
	 *            The instance to calculate the hashcode
	 * @return int The hashcode of the object
	 */
	protected int genericHashCode(Object instance) {
		/** initial value to hashCode */
		int hashCode = 1;
		/** constant value, we chose the prime number 31 to our calculate */
		final int constant = 31;
		/** declared fields contained in the class */

		/** It is the class of the parameter value */
		Class clazz = instance.getClass();

		Method[] methods = cachedMethodsForHash.get(clazz);
		if (methods == null) {
			methods = buildCachedMethodsForHash(clazz);

			cachedMethodsForHash.put(clazz, methods);
		}

		try {
			for (Method theMethod : methods) {
				/** the value returned once that the method was applied */
				Object value = theMethod.invoke(instance, (Object[]) null);

				// hashCode is modified holding the value that will be returned
				// to the end of the method
				hashCode = (hashCode * constant) + value.hashCode();
			}

		} catch (Exception e) {
			// If at runtime any operation here
			// declared on the fields marked throws
			// any exception then is impossible to
			// determine if the instances are equals
			hashCode = -1;
			return hashCode;
		}
		return hashCode;
	}

	/**
	 * It is a generic implementation of equals method which needs
	 * java.lang.reflect api and EqualsFieldAnnotation annotation.
	 * 
	 * @author Luis Felipe Barrera (lbarrera@smartmatic.com)
	 * @version 1.0 (Apr 9, 2007)
	 * @param instance
	 *            One of the objects to compare
	 * @param anotherInstance
	 *            The other object to compare
	 * @return boolean true if the two objects are semantically equals and false
	 *         otherwise
	 */
	protected boolean genericEquals(Object instance, Object anotherInstance) {
		/** It is the class of the parameter instance */
		Class clazz = instance.getClass();
		/** It is the class of the parameter anotherInstance */
		if (anotherInstance == null) {
			return false;
		}
		Class anotherClass = anotherInstance.getClass();
		/** initial value to hashCode */
		boolean result = true;
		if (clazz == anotherClass) {

			while (BaseDataObject.class.isAssignableFrom(clazz)) {
				/** declared fields contained in the class */
				Field[] fields = clazz.getDeclaredFields();
				for (Field field : fields) {
					/**
					 * boolean value to know if the field is required to
					 * calculate the hashCode value
					 */
					boolean doesItHaveTheAnnotation = field
							.isAnnotationPresent(EqualsFieldAnnotation.class);
					if (doesItHaveTheAnnotation) {
						/** field name */
						String fieldName = field.getName();
						/** field name with capital letter */
						String firstLetterUpper = Character
								.toUpperCase(fieldName.charAt(0))
								+ fieldName.substring(1);
						try {
							/** method name */
							String getMethodName = PREFIX_METHOD
									+ firstLetterUpper;
							/** instance of method */
							Method getMethod = null;
							try {
								getMethod = clazz.getMethod(getMethodName,
										(Class[]) null);
							} catch (NoSuchMethodException e) {

								/** method name */
								getMethodName = BOOLEAN_OPT_PREFIX_METHOD
										+ firstLetterUpper;
								/** instance of method */
								try {
									getMethod = clazz.getMethod(getMethodName,
											(Class[]) null);
								} catch (NoSuchMethodException eInside) {
									e.printStackTrace();

									throw e;
								}

							}
							/**
							 * the value returned once that the method was
							 * applied
							 */
							Object value = getMethod.invoke(instance,
									(Object[]) null);
							/**
							 * the anotherValue returned once that the method
							 * was applied
							 */
							Object anotherValue = getMethod.invoke(
									anotherInstance, (Object[]) null);

							// result is modified holding the value that will be
							// returned to the end of the method
							result = result && value.equals(anotherValue);
						} catch (Exception e) { // If at runtime any operation
							// here
							// declared on the fields marked
							// throws any exception then is
							// impossible to determine if the
							// instances are equals
							return false;
						}
					}
				}

				clazz = clazz.getSuperclass();
			}
			return result;
		} else {
			// If both instances arent of the same class, they must be
			// different
			return false;
		}

	}

	/**
	 * It is a generic implementation of equals method which needs
	 * java.lang.reflect api and EqualsFieldAnnotation annotation.
	 * 
	 * @param anObject
	 *            The other object to compare
	 * @return True if the objects are equals, false if they don not
	 */
	@Override
	public boolean equals(Object anObject) {
		return this.genericEquals(this, anObject);

	}

	/**
	 * It is a generic implementation of hashCode method which needs
	 * java.lang.reflect api and EqualsFieldAnnotation annotation.
	 * 
	 * @return The integer representing the instance hash code
	 */
	@Override
	public int hashCode() {
		return this.genericHashCode(this);

	}
}
