/*
 * Compressor.java
 *
 * 1.0 (Nov 30, 2009)
 *
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and propietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.lang;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.zip.DataFormatException;
import java.util.zip.Deflater;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;
import java.util.zip.Inflater;

import org.apache.commons.io.IOUtils;

/**
 * Class used to compress and decompress information. This class has as
 * objective reduce the amount of information that needs to be sent in the
 * network.
 * 
 * @author Hector Rodriguez (hector.rodriguez@smartmatic.com)
 * @version 1.0 (Nov 30, 2009)
 */
public class Compressor {
	/**
	 * 
	 * Field that defines the size of the blocks that are going to be
	 * compressed/decompressed. This field does not limit the amount of
	 * information that can be compressed/decompressed. Only is for internal use
	 * and in case of receive a block bigger than the block size the process of
	 * compression / decompression is made spliting the information in blocks
	 * small enough to be processed. This process is done internally and the
	 * caller does not have to know anything about the block size
	 */

	private static final int BLOCK_SIZE = 4096;

	/**
	 * 
	 * Compression rate that will be used by default if the programmer does not
	 * specify another compression rate
	 */

	private static final int DEFAULT_COMPRESSION_METHOD = Deflater.BEST_COMPRESSION;

	/**
	 * 
	 * Method that allows to decompress information that previously was
	 * compressed
	 * 
	 * @paramcompressedData byte[] information that needs to be decompressed
	 * 
	 * @throwsZipWrongMessageFormatException If there is a problem in the format
	 *                                       of the information
	 * @return byte[] the information decompressed
	 */

	public static byte[] decompress(byte[] compressedData)
			throws ZipWrongMessageFormatException {
		try {
			Inflater decompresser = new Inflater(false);
			decompresser.setInput(compressedData);
			byte[] buffer = new byte[Compressor.BLOCK_SIZE];
			ByteArrayOutputStream byteStream = new ByteArrayOutputStream();
			int length = -1;
			while (!decompresser.finished()) {
				length = decompresser.inflate(buffer);
				byteStream.write(buffer, 0, length);

			}

			decompresser.end();
			return byteStream.toByteArray();

		} catch (DataFormatException e) {
			throw new ZipWrongMessageFormatException(
					ZipWrongMessageFormatException.ExceptionInfo.UNABLE_DECOMPRESS_MESSAGE,
					e);
		}
	}

	/**
	 * 
	 * Method that allow to compress information using the default compression
	 * rate
	 * 
	 * @param rawData
	 *            byte[] Information that needs to be compressed
	 * 
	 * @return byte[] The compressed information
	 */

	public static byte[] compress(byte[] rawData) {
		return compress(rawData, Compressor.DEFAULT_COMPRESSION_METHOD);
	}

	/**
	 * 
	 * Method that allow to compress information using a compression rate
	 * specified by the caller.
	 * 
	 * @param rawData
	 * 
	 *            byte[] The information that needs to be compressed
	 * 
	 * @paramcompressionRate int Compression rate that will be used in the
	 *                       compression
	 * 
	 * @return byte[] The compressed information
	 */

	public static byte[] compress(byte[] rawData, int compressionRate) {
		ByteArrayOutputStream byteStream = new ByteArrayOutputStream();
		byte[] buffer = new byte[Compressor.BLOCK_SIZE];
		Deflater compresser = new Deflater(compressionRate, false);
		compresser.setInput(rawData);
		compresser.finish();
		while (!compresser.finished()) {
			int length = compresser.deflate(buffer);
			byteStream.write(buffer, 0, length);
		}
		compresser.end();
		return byteStream.toByteArray();

	}

	/**
	 * Method that allow to compress information.
	 * 
	 * @param rawData
	 *            byte[] The information that needs to be compressed
	 * @return byte[] The compressed information
	 * @throws GZipWrongMessageFormatException
	 *             If there is a problem in the format of the information
	 */
	public static byte[] compressGZip(byte[] content)
			throws GZipWrongMessageFormatException {
		ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
		try {
			GZIPOutputStream gzipOutputStream = new GZIPOutputStream(
					byteArrayOutputStream);
			gzipOutputStream.write(content);
			gzipOutputStream.close();
		} catch (IOException e) {
			throw new GZipWrongMessageFormatException(
					GZipWrongMessageFormatException.ExceptionInfo.UNABLE_COMPRESS_MESSAGE,
					e);
		}
		return byteArrayOutputStream.toByteArray();
	}

	/**
	 * Method that allows to decompress information that previously was
	 * compressed.
	 * 
	 * @param compressedData
	 *            byte[] information that needs to be decompressed
	 * @return byte[] the information decompressed
	 * @throws ZipWrongMessageFormatException
	 *             If there is a problem in the format of the information
	 * @throws GZipWrongMessageFormatException
	 *             If there is a problem in the format of the information
	 */
	public static byte[] decompressGZip(byte[] contentBytes)
			throws GZipWrongMessageFormatException {
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		try {
			IOUtils.copy(new GZIPInputStream(new ByteArrayInputStream(
					contentBytes)), out);
		} catch (IOException e) {
			throw new GZipWrongMessageFormatException(
					GZipWrongMessageFormatException.ExceptionInfo.UNABLE_DECOMPRESS_MESSAGE,
					e);
		}
		return out.toByteArray();
	}
}
