/**
 * CommonObjectSerializer.java
 *
 * $Revision: 1.15 $ ($Date: 2011-04-06 05:08:30 $)
 *
 * Copyright (c) 2004 Smartmatic Corp.
 * 1001 Broken Sound Parkway NW, Suite D
 * Boca Raton FL 33487, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of
 * Smartmatic Corp. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered
 * into with Smartmatic Corp.
 */
package com.smartmatic.waffle.util;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import com.smartmatic.waffle.lang.NotSerializableObjectException;

/**
 * Class used to convert objects to an array of bytes and back to its
 * constituent object using the minimum space to each type.
 * 
 * @author Héctor Rodríguez (hector.rodriguez@smartmatic.com)
 * @version 1.0 (20/01/2009)
 */
public class CommonObjectSerializer {

	/**
	 * Converts an object to an array of bytes.
	 * 
	 * @param object
	 *            the object to convert.
	 * @return the associated byte array.
	 * @throws NotSerializableObjectException
	 *             Thrown when occurs an error during a serialization process
	 */
	public static byte[] toBytes(Object object)
			throws NotSerializableObjectException {
		if (object.getClass().isAssignableFrom(Long.class)
				|| object.getClass().isAssignableFrom(long.class)) {
			return toBytes(((Long) object).longValue());
		} else if (object.getClass().isAssignableFrom(Byte.class)
				|| object.getClass().isAssignableFrom(byte.class)) {
			return new byte[] { ((Byte) object).byteValue() };
		} else if (object.getClass().isAssignableFrom(Integer.class)
				|| object.getClass().isAssignableFrom(int.class)) {
			return toBytes(((Integer) object).intValue());
		}
		if (object.getClass().isAssignableFrom(Float.class)
				|| object.getClass().isAssignableFrom(float.class)) {
			return toBytes(((Float) object).floatValue());
		}
		if (object.getClass().isAssignableFrom(Double.class)
				|| object.getClass().isAssignableFrom(double.class)) {
			return toBytes(((Double) object).doubleValue());
		}
		if (object.getClass().isAssignableFrom(Short.class)
				|| object.getClass().isAssignableFrom(short.class)) {
			return toBytes(((Short) object).shortValue());
		}
		if (object.getClass().isAssignableFrom(Character.class)
				|| object.getClass().isAssignableFrom(char.class)) {
			return toBytes(((Character) object).charValue());
		}
		if (object.getClass().isAssignableFrom(Boolean.class)
				|| object.getClass().isAssignableFrom(boolean.class)) {
			return toBytes(((Boolean) object).booleanValue());
		} else if (object.getClass().isAssignableFrom(String.class)) {
			return toBytes(((String) object).toString());
		} else if (object.getClass().isAssignableFrom(Byte[].class)
				|| object.getClass().isAssignableFrom(byte[].class)) {
			return (byte[]) object;
		}
		return serializeObject(object);
	}

	/**
	 * Converts an array of bytes back to its constituent object. The input
	 * array is assumed to have been created from the original object.
	 * 
	 * @param bytes
	 *            the byte array to convert.
	 * @return the associated object.
	 * @throws NotSerializableObjectException
	 *             Thrown when occurs an error during a serialization process
	 */
	public static Object toObject(byte[] bytes)
			throws NotSerializableObjectException {
		Object object = null;
		try {
			object = new ObjectInputStream(new ByteArrayInputStream(bytes))
					.readObject();
		} catch (IOException e) {
			throw new NotSerializableObjectException(
					NotSerializableObjectException.ExceptionInfo.INVALID_INNER_REFERENCE,
					e);
		} catch (ClassNotFoundException e) {
			throw new NotSerializableObjectException(
					NotSerializableObjectException.ExceptionInfo.UNKNOWN_INNER_REFERENCE,
					e);
		}
		return object;
	}

	/**
	 * Converts an array of bytes back to its constituent object. The input
	 * array is assumed to have been created from the original object.
	 * 
	 * @param bytes
	 *            the byte array to convert.
	 * @param clazz
	 *            the original class of the associated object.
	 * @return the associated object.
	 * @throws NotSerializableObjectException
	 *             Thrown when occurs an error during a serialization process
	 */
	@SuppressWarnings("unchecked")
	public static Object toObject(byte[] bytes, Class clazz)
			throws NotSerializableObjectException {
		if (clazz.isAssignableFrom(Long.class)
				|| clazz.isAssignableFrom(long.class)) {
			return toLong(bytes);
		} else if (clazz.isAssignableFrom(Integer.class)
				|| clazz.isAssignableFrom(int.class)) {
			return toInt(bytes);
		} else if (clazz.isAssignableFrom(Byte.class)
				|| clazz.isAssignableFrom(byte.class)) {
			return toBytes(bytes)[0];
		} else if (clazz.isAssignableFrom(Short.class)
				|| clazz.isAssignableFrom(short.class)) {
			return toShort(bytes);
		} else if (clazz.isAssignableFrom(Double.class)
				|| clazz.isAssignableFrom(double.class)) {
			return toDouble(bytes);
		} else if (clazz.isAssignableFrom(Float.class)
				|| clazz.isAssignableFrom(float.class)) {
			return toFloat(bytes);
		} else if (clazz.isAssignableFrom(Character.class)
				|| clazz.isAssignableFrom(char.class)) {
			return toChar(bytes);
		} else if (clazz.isAssignableFrom(Boolean.class)
				|| clazz.isAssignableFrom(boolean.class)) {
			return toBoolean(bytes);
		} else if (clazz.isAssignableFrom(String.class)) {
			return toString(bytes);
		} else if (clazz.isAssignableFrom(Byte[].class)
				|| clazz.isAssignableFrom(byte[].class)) {
			return bytes;
		}
		return toObject(bytes);
	}

	/**
	 * Converts an object to an array of bytes.
	 * 
	 * @param object
	 *            the object to convert.
	 * @return the associated byte array.
	 * @throws NotSerializableObjectException
	 *             Thrown when occurs an error during a serialization process
	 */
	public static byte[] serializeObject(Object object)
			throws NotSerializableObjectException {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		ObjectOutputStream oos = null;
		try {
			oos = new ObjectOutputStream(baos);
			oos.writeObject(object);
		} catch (IOException e) {
			throw new NotSerializableObjectException(
					NotSerializableObjectException.ExceptionInfo.INVALID_INNER_REFERENCE,
					e);
		} finally {
			if (oos != null) {
				try {
					oos.close();
				} catch (IOException e) {
					throw new NotSerializableObjectException(
							NotSerializableObjectException.ExceptionInfo.INVALID_INNER_REFERENCE,
							e);
				}
			}
		}

		return baos.toByteArray();
	}

	/**
	 * Converts a long to an array of bytes.
	 * 
	 * @param ln
	 *            the long to convert.
	 * @return the associated byte array.
	 * @throws NotSerializableObjectException
	 *             Thrown when occurs an error during a serialization process
	 */
	public static byte[] toBytes(long ln) throws NotSerializableObjectException {
		byte[] ret = new byte[8];
		int i = 0;
		int pot = 0;
		long mask = 0;
		mask = 0xFF00000000000000L;
		pot = 56;
		for (i = 0; i < 8; i++) {
			ret[i] = (byte) (((ln & mask) >> pot) & 0x00000000000000FFL);
			mask >>= 8;
			mask &= 0x00FFFFFFFFFFFFFFL;
			pot -= 8;
		}
		return ret;
	}

	/**
	 * Converts an int to an array of bytes.
	 * 
	 * @param in
	 *            the int to convert.
	 * @return the associated byte array.
	 * @throws NotSerializableObjectException
	 *             Thrown when occurs an error during a serialization process
	 */
	public static byte[] toBytes(int in) throws NotSerializableObjectException {
		byte[] ret = new byte[4];
		int i = 0;
		int pot = 0;
		int mask = 0;
		mask = 0xFF000000;
		pot = 24;
		for (i = 0; i < 4; i++) {
			ret[i] = (byte) (((in & mask) >> pot) & 0x000000FF);
			mask >>= 8;
			mask &= 0x00FFFFFF;
			pot -= 8;
		}
		return ret;
	}

	/**
	 * Converts a short to an array of bytes.
	 * 
	 * @param sh
	 *            the short to convert.
	 * @return the associated byte array.
	 * @throws NotSerializableObjectException
	 *             Thrown when occurs an error during a serialization process
	 */
	public static byte[] toBytes(short sh)
			throws NotSerializableObjectException {
		byte[] ret = new byte[2];
		int i = 0;
		int pot = 0;
		short mask = 0;
		mask = (short) 0xFF00;
		pot = 8;
		for (i = 0; i < 2; i++) {
			ret[i] = (byte) (((sh & mask) >> pot) & (short) 0x00FF);
			mask >>= 8;
			mask &= (short) 0x00FF;
			pot -= 8;
		}
		return ret;
	}

	/**
	 * Converts a double to an array of bytes.
	 * 
	 * @param db
	 *            the double to convert.
	 * @return the associated byte array.
	 * @throws NotSerializableObjectException
	 *             Thrown when occurs an error during a serialization process
	 */
	public static byte[] toBytes(double db)
			throws NotSerializableObjectException {
		return toBytes(Double.doubleToRawLongBits(db));
	}

	/**
	 * Converts a float to an array of bytes.
	 * 
	 * @param fl
	 *            the float to convert.
	 * @return the associated byte array.
	 * @throws NotSerializableObjectException
	 *             Thrown when occurs an error during a serialization process
	 */
	public static byte[] toBytes(float fl)
			throws NotSerializableObjectException {
		return toBytes(Float.floatToRawIntBits(fl));
	}

	/**
	 * Converts a char to an array of bytes.
	 * 
	 * @param ch
	 *            the char to convert.
	 * @return the associated byte array.
	 * @throws NotSerializableObjectException
	 *             Thrown when occurs an error during a serialization process
	 */
	public static byte[] toBytes(char ch) throws NotSerializableObjectException {
		return new byte[] { (byte) ch };
	}

	/**
	 * Converts a boolean to an array of bytes.
	 * 
	 * @param bl
	 *            the boolean to convert.
	 * @return the associated byte array.
	 * @throws NotSerializableObjectException
	 *             Thrown when occurs an error during a serialization process
	 */
	public static byte[] toBytes(boolean bl)
			throws NotSerializableObjectException {
		if (bl) {
			return new byte[] { 1 };
		}
		return new byte[] { 0 };
	}

	/**
	 * Converts a String to an array of bytes.
	 * 
	 * @param str
	 *            the String to convert.
	 * @return the associated byte array.
	 * @throws NotSerializableObjectException
	 *             Thrown when occurs an error during a serialization process
	 */
	public static byte[] toBytes(String str)
			throws NotSerializableObjectException {
		return str.getBytes();
	}

	/**
	 * Converts an array of bytes back to its constituent long. The input array
	 * is assumed to have been created from the original long.
	 * 
	 * @param bytes
	 *            the byte array to convert.
	 * @return the associated long.
	 * @throws NotSerializableObjectException
	 *             Thrown when occurs an error during a serialization process
	 */
	public static long toLong(byte[] bytes)
			throws NotSerializableObjectException {

		return ((long) bytes[0] << 56) + ((long) (bytes[1] & 255) << 48)
				+ ((long) (bytes[2] & 255) << 40)
				+ ((long) (bytes[3] & 255) << 32)
				+ ((long) (bytes[4] & 255) << 24) + ((bytes[5] & 255) << 16)
				+ ((bytes[6] & 255) << 8) + ((bytes[7] & 255) << 0);
	}

	/**
	 * Converts an array of bytes back to its constituent int. The input array
	 * is assumed to have been created from the original int.
	 * 
	 * @param bytes
	 *            the byte array to convert.
	 * @return the associated int.
	 * @throws NotSerializableObjectException
	 *             Thrown when occurs an error during a serialization process
	 */
	public static int toInt(byte[] bytes) throws NotSerializableObjectException {
		int ret = 0;
		int b = 0;
		int i = 0;
		for (i = 0; i < 4; i++) {
			ret <<= 8;
			b = Integer.parseInt(String.format("%02X", bytes[i]), 16);
			ret |= b;
		}
		return ret;
	}

	/**
	 * Converts an array of bytes back to its constituent short. The input array
	 * is assumed to have been created from the original short.
	 * 
	 * @param bytes
	 *            the byte array to convert.
	 * @return the associated short.
	 * @throws NotSerializableObjectException
	 *             Thrown when occurs an error during a serialization process
	 */
	public static short toShort(byte[] bytes)
			throws NotSerializableObjectException {
		short ret = (short) 0;
		short b = 0;
		int i = 0;
		for (i = 0; i < 2; i++) {
			ret <<= 8;
			b = Short.parseShort(String.format("%02X", bytes[i]), 16);
			ret |= b;
		}
		return ret;
	}

	/**
	 * Converts an array of bytes back to its constituent double. The input
	 * array is assumed to have been created from the original double.
	 * 
	 * @param bytes
	 *            the byte array to convert.
	 * @return the associated double.
	 * @throws NotSerializableObjectException
	 *             Thrown when occurs an error during a serialization process
	 */
	public static double toDouble(byte[] bytes)
			throws NotSerializableObjectException {
		return Double.longBitsToDouble(toLong(bytes));
	}

	/**
	 * Converts an array of bytes back to its constituent float. The input array
	 * is assumed to have been created from the original float.
	 * 
	 * @param bytes
	 *            the byte array to convert.
	 * @return the associated float.
	 * @throws NotSerializableObjectException
	 *             Thrown when occurs an error during a serialization process
	 */
	public static float toFloat(byte[] bytes)
			throws NotSerializableObjectException {
		return Float.intBitsToFloat(toInt(bytes));
	}

	/**
	 * Converts an array of bytes back to its constituent char. The input array
	 * is assumed to have been created from the original char.
	 * 
	 * @param bytes
	 *            the byte array to convert.
	 * @return the associated char.
	 * @throws NotSerializableObjectException
	 *             Thrown when occurs an error during a serialization process
	 */
	public static char toChar(byte[] bytes)
			throws NotSerializableObjectException {
		return (char) bytes[0];
	}

	/**
	 * Converts an array of bytes back to its constituent boolean. The input
	 * array is assumed to have been created from the original boolean.
	 * 
	 * @param bytes
	 *            the byte array to convert.
	 * @return the associated boolean.
	 * @throws NotSerializableObjectException
	 *             Thrown when occurs an error during a serialization process
	 */
	public static boolean toBoolean(byte[] bytes)
			throws NotSerializableObjectException {
		if (bytes[0] == 1) {
			return true;
		}
		return false;
	}

	/**
	 * Converts an array of bytes back to its constituent String. The input
	 * array is assumed to have been created from the original String.
	 * 
	 * @param bytes
	 *            the byte array to convert.
	 * @return the associated String.
	 * @throws NotSerializableObjectException
	 *             Thrown when occurs an error during a serialization process
	 */
	public static String toString(byte[] bytes)
			throws NotSerializableObjectException {
		return new String(bytes);
	}

}
