/**
 * @file	s-a-e-s-conf-manager.h
 * @brief	Declares the e s conf manager class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */

#ifndef _S_A_E_S_CONFIGURATION_MANAGER_H_
#define _S_A_E_S_CONFIGURATION_MANAGER_H_

#include <string>
#include <Configuration/SaesConfiguration-schema.hxx>
#include <qrencode.h>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Configuration
        {
            /**
             * @class	SAESConfigurationManager
             * @brief	Manager for saes configurations.
             * @author	Eduardo.campione
             * @date	28/04/2011
             */

            class SAESConfigurationManager
            {
                public:

                    /**
                     * @fn	static SAESConfigurationManager * getInstance( SAESConfigurationManager * externManager = NULL );
                     * @brief	Gets the current SAES Configuration Manager.
                     * @author	Yadickson Soto
                     * @date	30/01/2014
                     * @param 	externManager external manager to unit test
                     * @return	the current instance SAES Configuration Manager.
                     */

                    static SAESConfigurationManager * getInstance ( SAESConfigurationManager * externManager = NULL );

                    /**
                     * @fn	SaesConfiguration * getConfiguration();
                     * @brief	Gets the Saes configuration.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     * @return	null if it fails, else the Saes configuration.
                     */

                    SaesConfiguration * getConfiguration();

                    /**
                     * @fn	void SAESConfigurationManager::loadDefault();
                     * @brief	Loads the default.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     */

                    void loadDefault();

                    /**
                     * @fn	QRecLevel SAESConfigurationManager::GetCorrectionModeQRCode(CorrectionModeQRCode & correction);
                     * @brief	Gets the Correction Mode QR Code.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     * @param [in,out]	correction	The correction.
                     * @return	The correction mode qr code.
                     */

                    QRecLevel GetCorrectionModeQRCode ( CorrectionModeQRCode & correction );

                    /**
                     * @fn	QRCodeConfig SAESConfigurationManager::getQRCodeVoteConfig(int size = -1);
                     * @brief	Gets the QR code vote config.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	31/05/2011
                     * @param	size	(optional) the size.
                     * @return	The QR code vote config.
                     */

                    QRCodeConfig getQRCodeVoteConfig ( int size = -1 );

                    /**
                     * @fn	QRCodeConfig SAESConfigurationManager::getQRCodeBarcodeConfig();
                     * @brief	Gets the QR code barcode config.
                     * @author	Juan Delgado juand.delgado@smartmatic.com
                     * @date	30/08/2011
                     * @return	The QR code barcode config.
                     */

                    QRCodeConfig getQRCodeBarcodeConfig();

                    /**
                     * @fn	QRCodeConfig SAESConfigurationManager::getQRCodeInstallConfig();
                     * @brief	Gets the QR code install config.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	31/05/2011
                     * @return	The QR code install config.
                     */

                    QRCodeConfig getQRCodeInstallConfig();

                    /**
                     * @fn	unsigned int SAESConfigurationManager::getCardValidity();
                     * @brief	Gets the card validity.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     * @return	The card validity.
                     */

                    unsigned int getCardValidity();

                    /**
                     * @fn	bool SAESConfigurationManager::getSaveDemoVotesInTransmissionPackage();
                     * @brief	Gets save demo votes in transmission package.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     * @return	true if it succeeds, false if it fails.
                     */
                    bool getSaveDemoVotesInTransmissionPackage();

                    /**
                     * @brief  Gets the EML file configuration.
                     * @author manuel.arguelles
                     * @date   Jan  8, 2018
                     * @return The EML file configuration object.
                     */
                    EmlFileConfiguration & getEmlFileConfiguration();

                    /**
                     * @brief Gets the file hash configuration.
                     * @author eduardo.fernandez
                     * @date   09/01/2019
                     * @return The file hash configuration.
                     */
                    FileHashConfiguration & getFileHashConfiguration();

                    /**
                     * @fn	PVFileConfiguration & SAESConfigurationManager::getPVFileConfiguration();
                     * @brief	method getter pv file configuration.
                     * @author	Yadickson Soto
                     * @date	10/01/2013
                     * @return	pv file configuration.
                     */
                    PVFileConfiguration & getPVFileConfiguration();

                    /**
                     * @fn	ConfigurationVote & SAESConfigurationManager::getPrintVoteConfiguration();
                     * @brief	Gets the print vote configuration.
                     * @author	Freddy.Gomez
                     * @date	04/05/2011
                     * @return	The print vote configuration.
                     */
                    ConfigurationVote & getPrintVoteConfiguration();

                    /**
                     * @fn	ConfigurationVote& SAESConfigurationManager::getPrintScreenVoteConfiguration();
                     * @brief	Gets the screen vote configuration.
                     * @author	daniel.ampuero
                     * @date	04/05/2011
                     * @return	The screen vote configuration.
                     */
                    ConfigurationVote & getPrintScreenVoteConfiguration();

                    /**
                     * @fn	VoteCodeConfig & SAESConfigurationManager::getVoteCodeConfig();
                     * @brief	Method getter vote code config.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	14/07/2011
                     * @return	The vote code configuration.
                     */

                    VoteCodeConfig & getVoteCodeConfig();

                    /**
                     * @fn	bool SAESConfigurationManager::showBarCodeInstallationReport();
                     * @brief	Shows the bar code installation report.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     * @return	true if it succeeds, false if it fails.
                     */
                    bool showBarCodeInstallationReport();

                    /**
                     * @fn	LayoutContestConfiguration & SAESConfigurationManager::getLayoutContestConfiguration();
                     * @brief	Gets layout contest configuration.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     * @return	The layout contest configuration.
                     */
                    LayoutContestConfiguration & getLayoutContestConfiguration();

                    /**
                     * @fn	LayoutBallotOptiontType & SAESConfigurationManager::getLayoutBallotOptiontType();
                     * @brief	Gets layout ballot optiont type.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     * @return	The layout ballot optiont type.
                     */
                    LayoutBallotOptiontType & getLayoutBallotOptiontType();

                    /**
                     * @fn	LayoutContestConfiguration & SAESConfigurationManager::getLayoutContestConfigurationPreviewScreen();
                     * @brief	Gets layout contest configuration preview screen.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     * @return	The layout contest configuration preview screen.
                     */
                    LayoutContestConfiguration & getLayoutContestConfigurationPreviewScreen();

                    /**
                     * @fn	LayoutBallotOptiontType & SAESConfigurationManager::getLayoutBallotOptiontTypePreviewScreen();
                     * @brief	Gets layout ballot optiont type preview screen.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     * @return	The layout ballot optiont type preview screen.
                     */
                    LayoutBallotOptiontType & getLayoutBallotOptiontTypePreviewScreen();

                    /**
                     * @fn	virtual SAESConfigurationManager::~SAESConfigurationManager();
                     * @brief	Finaliser.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     */
                    virtual ~SAESConfigurationManager();

                protected:

                    /**
                     * @fn	SAESConfigurationManager::SAESConfigurationManager();
                     * @brief	Default constructor.
                     * @author	Yadickson Soto
                     * @date	30/01/2014
                     */

                    SAESConfigurationManager();

                    /**
                     * @fn	virtual void SAESConfigurationManager::initialize();
                     * @brief	Initializes this object.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     */

                    virtual void initialize();

                private:

                    static SAESConfigurationManager * instance; ///< The singleton instance
                    SaesConfiguration * configuration;	///< The configuration
            };
        }
    }
}

#endif // _S_M_T_T_G_U_I_CONFIGURATION_MANAGER_H_
