/**
 * @file ReportPrintingController.cpp
 * @brief Implementation of entity that prints the reports collection obtained  from <tt>SAESCommon</tt> settings.
 * @date 19/04/2011
 * @author Rafael Ferrero
 * @author manuel.arguelles
 */

#include "ReportPrintingController.hxx"
#include "Runtime/Environment/saes-directories.h"
#include <Configuration/s-a-e-s-conf-manager.h>
#include <Configuration/saes-language-controller.h>
#include <Environment/Configurations/resource-path-provider.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Printing/LPSpooler.h>
#include <Security/Encryption.hxx>
#include <System/Exception/CryptoException.hxx>
#include <System/IO/File.h>
#include <System/IO/Path.h>
#include <System/SafeBuffer.h>
#include <System/Security/XmlDigitalSignature/XmlDetachedSignature.hxx>
#include <System/Utils/StringUtils.hxx>
#include <Voting/Election/ElectionInstalationLanguages.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String)
#else
#define _(String) (String)
#endif

#define N_(String) (String)

using namespace Smartmatic::Environment;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Printing;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::SAES::Security;
using namespace Smartmatic::SAES::Voting::Election;
using namespace Smartmatic::System;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::Security;
using namespace Smartmatic::System::Utils;

const std::string  LP_OPTION_FINALCUT         = "finalcut";
const std::string  LP_OPTION_FINALFEED        = "fwdfeed";
const std::string  LP_OPTION_LEFTMARGIN       = "LeftMargin";
const std::string  LP_OPTION_RMTOP            = "rmtop";
const std::string  LP_OPTION_TIMEOUT          = "prtimeout";
const std::string  LP_OPTION_HISTORIC_HEAT    = "HistoricHeat";
const std::string  LP_VALUE_FINALFEED         = "128";
const std::string  LP_VALUE_LEFTMARGIN        = "30";
const std::string  LP_VALUE_PARTIALCUT        = "1";
const std::string  LP_VALUE_RMTOP             = "False";
const std::string  LP_VALUE_TIMEOUT_MS        = "20000";
const std::string  LP_VALUE_HISTORIC_HEAT_OFF = "False";
const std::string  LP_VALUE_HISTORIC_HEAT_ON  = "True";
const unsigned int DELAY_BETWEEN_REPORTS      = 1;

static ISMTTLog* logger = SMTTLogManager::GetLogger(
    "Smartmatic.SAES.Printing.ReportPrintingController",
    GETTEXT_PACKAGE);

ReportPrintingController::ReportPrintingController()
    : m_client(NULL), m_numOfCopies(0), m_numPrinted(0)
{
    SAESCommonConfigurationManager* commonConfigurationManager = \
        SAESCommonConfigurationManager::getInstance();
    SaesCommonConfiguration* commonConfiguration = commonConfigurationManager->getConfiguration();

    m_allLanguages = commonConfiguration->getConfigurationPrinter().getGeneratePVReportInAllLanguages();
}

void ReportPrintingController::loadReports()
{
    m_reportList.clear();
    m_fileNameReportList.clear();

    ConfigurationReportPrinting::ReportFilesConstIterator it;
    ConfigurationReportPrinting reportConfiguration =
        SAESConfigurationManager::getInstance()->getConfiguration()->getReportPrinting();
    ConfigurationReportPrinting::ReportFilesSequence fileQueue = reportConfiguration.getReportFiles();

    for (it = fileQueue.begin(); it != fileQueue.end(); ++it)
    {
        ConfigurationReportFile file = *it;
        file.setReportTitle(_(it->getReportTitle().c_str()));
        file.setFileDescription(_(it->getFileDescription().c_str()));
        m_reportList.push_back(file);
    }
}

int ReportPrintingController::getTotalDocumentsToPrint()
{
    return m_fileNameReportList.size();
}

ReportPrintingController::~ReportPrintingController()
{
}

void ReportPrintingController::Print()
{
    PrintDocumentResult res;
    std::list <std::string>::iterator it;

    res.setResult(false, " ", PrintDocumentResult::PRINTING_UNEXPECTED_ERROR);
    m_numPrinted = 0;

    for (it = m_fileNameReportList.begin(); it != m_fileNameReportList.end(); ++it)
    {
        LPSpooler spooler;
        std::string file = *it;
        SAESCommonConfigurationManager* commonConfigurationManager = SAESCommonConfigurationManager::getInstance();
        SaesCommonConfiguration* commonConfiguration = commonConfigurationManager->getConfiguration();

        logger->Debug("Print report file " + file);

        spooler.SetFileName(file);

        if (getBackendType() == PrinterBackendType::CONFIGURATION_DIRECTPRINTING)
        {
            spooler.CancelJobs();
            spooler.ClearOptions();
        }

        spooler.AddOption(LP_OPTION_TIMEOUT, LP_VALUE_TIMEOUT_MS);
        spooler.AddOption(LP_OPTION_FINALCUT, LP_VALUE_PARTIALCUT);
        spooler.AddOption(LP_OPTION_FINALFEED, LP_VALUE_FINALFEED);
        spooler.AddOption(LP_OPTION_RMTOP, LP_VALUE_RMTOP);
        spooler.AddOption(LP_OPTION_LEFTMARGIN, LP_VALUE_LEFTMARGIN);

        if (commonConfiguration->getConfigurationPrinter().getHistoricHeat().present() &&
            commonConfiguration->getConfigurationPrinter().getHistoricHeat().get() == true)
        {
            spooler.AddOption(LP_OPTION_HISTORIC_HEAT, LP_VALUE_HISTORIC_HEAT_ON);
        }
        else
        {
            spooler.AddOption(LP_OPTION_HISTORIC_HEAT, LP_VALUE_HISTORIC_HEAT_OFF);
        }

        m_numPrinted++;
        m_signalPrintFile.emit();

        LPSpooler::JobStateEnumType printjobStatus = spooler.Print();
        std::string message = spooler.getLastMessage();

        sleep(DELAY_BETWEEN_REPORTS);

        if (printjobStatus != LPSpooler::LPSPOOLER_JOB_COMPLETED)
        {
            if (message.empty())
            {
                message = _("Smartmatic.SAES.Printing.ReportPrintingController.SpoolerDeviceError");
            }

            res.setResult(false, message, PrintDocumentResult::PRINTING_UNEXPECTED_ERROR);
            logger->Debug("Error detected " + message);
            m_signalErrorDetected.emit();
            break;
        }
        else
        {
            res.setResult(true, message, PrintDocumentResult::PRINTING_COMPLETED);
            logger->Debug("Print finish " + message);
        }
    }

    if (m_client)
    {
        m_client->onDocumentPrinted(res);
    }

    m_signalPrintFinished.emit();
}

void ReportPrintingController::setClient(PrintDocumentClient* client)
{
    m_client = client;
}

void ReportPrintingController::setReportsList()
{
    SaesDirectories directories;
    std::vector<ConfigurationReportFile>::const_iterator it;

    for (it = m_reportList.begin(); it != m_reportList.end(); ++it)
    {
        std::string reportPath = Path::Combine( directories.getDocsFolderFullPath(), it->getFileName() );

        for (unsigned int i = 0; i < it->getNumCopies(); ++i)
        {
            m_fileNameReportList.push_back(reportPath);
        }
    }
}

bool ReportPrintingController::verifySignature()
{
    bool success = false;

    std::list<std::string>::const_iterator       it  = m_fileNameReportList.begin();
    const std::list<std::string>::const_iterator end = m_fileNameReportList.end();

    const std::string signatureFileName =
        SAESCommonConfigurationManager::getInstance()->getConfiguration()->
        getConfigurationElectionFilesPath().getSignatureResourceFileName();

    for (; it != end; ++it)
    {
        const std::string directory = Path::GetDirectory(*it);
        std::string signatureFilePath = Path::Combine(directory, signatureFileName);

        try
        {
            logger->Debug("Decrypting file '" + signatureFilePath + "'.");

            SafeBuffer bufferToVerify;
            Encryption::getInstance()->decryptoFileToBuffer(signatureFilePath, bufferToVerify, KeyType::CONFIGURATION_SECRET_KEY);

            logger->Debug("File '" + signatureFilePath + "' decrypted with success.");
            logger->Debug("Verifying file " + *it + "'s signature.");

            XmlDetachedSignature detachedSignature;
            success = detachedSignature.verify(bufferToVerify, NULL, directory);

            if (success == false)
            {
                logger->Debug("Error: file " + *it + "'s signature could not be verified.");
                break;
            }

            logger->Debug("File " + *it + "'s signature verified with success.");
        }
        catch (CryptoException& cryptoException)
        {
            logger->Error("Error decrypting file '" + signatureFilePath + "': " + cryptoException.what());
            success = false;
            break;
        }
    }

    return success;
}

int ReportPrintingController::GetNumOfCopies()
{
    return m_numOfCopies;
}

int ReportPrintingController::GetNumOfCopiesPrinted()
{
    return m_numPrinted;
}

const std::vector<ConfigurationReportFile>& ReportPrintingController::GetReportList()
{
    return m_reportList;
}

Glib::Dispatcher& ReportPrintingController::signalReportChanged()
{
    return m_signalPrintFile;
}

Glib::Dispatcher& ReportPrintingController::signalReportPrintFinished()
{
    return m_signalPrintFinished;
}

Glib::Dispatcher& ReportPrintingController::signalReportPrintErrorDeteced()
{
    return m_signalErrorDetected;
}
