/**
 * @file	VotePrintDocument.h
 * @brief	Declares the vote print document class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */


#ifndef _VOTEPRINTDOCUMENT_H_
#define _VOTEPRINTDOCUMENT_H_

#include "Printing/BasePrintDocument.h"
#include "Configuration/s-a-e-s-conf-manager.h"
#include "Printing/DocumentOptions.h"
#include <Voting/Election/election-schema.hxx>
#include "Voting/Vote/vote-schema.hxx"

#include <vector>
#include <string>
#include <Voting/ElectoralConfiguration.hxx>
#include <System/SafeBuffer.h>

using namespace Smartmatic::SAES::Voting::Election;

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Printing
        {
            /**
             * @class	VotePrintDocument
             * @brief	Vote print document.
             * @author	Eduardo.campione
             * @date	28/04/2011
             */

            class VotePrintDocument : public BasePrintDocument
            {
                public:

                    /**
                     * @fn	VotePrintDocument::VotePrintDocument ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration,
                     * 			const Smartmatic::SAES::Voting::Vote::Vote & vote, bool forceGtkPrinting = false,
                     * 			bool showBarCode = true, bool screenPreview = false);
                     * @brief	Constructor.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     * @param	electoralConfiguration	The electoral configuration.
                     * @param	vote										The vote.
                     * @param	forceGtkPrinting				true to force gtk printing.
                     * @param	showBarCode							(optional) the show bar code.
                     * @param	screenPreview						(optional) the screen preview.
                     */

                    VotePrintDocument ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration,
                                        const Smartmatic::SAES::Voting::Vote::Vote & vote, bool forceGtkPrinting = false,
                                        bool showBarCode = true, bool screenPreview = false );

                    /**
                     * @fn	void VotePrintDocument::PrintVote(const Smartmatic::SAES::Voting::Vote::Vote & vote);
                     * @brief	Print vote.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     * @param	vote	The vote.
                     */

                    void PrintVote ( const Smartmatic::SAES::Voting::Vote::Vote & vote );

                    /**
                     * @fn	Glib::RefPtr<Gdk::Pixbuf> VotePrintDocument::GetQRCodeVote(const Smartmatic::SAES::Voting::Vote::Vote & vote );
                     * @brief	Gets a qr code vote.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     * @param	vote	The vote.
                     * @return	The qr code vote.
                     */

                    Glib::RefPtr<Gdk::Pixbuf> GetQRCodeVote ( const Smartmatic::SAES::Voting::Vote::Vote & vote );

                    /**
                     * @fn	void VotePrintDocument::CommandPrintQRCode (const Smartmatic::SAES::Voting::Vote::Vote & vote);
                     * @brief	Command print qr code.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     * @param	vote	The vote.
                     */
                    void CommandPrintQRCode ( const Smartmatic::SAES::Voting::Vote::Vote & vote );

                    /**
                     * @fn std::map < std::string, std::map < std::string, std::string > > getTextToPrintMap();
                     * @brief Gets the text to print map.
                     * @author  eduardo.fernandez
                     * @date    10/02/2018
                     * @return  The map of the text to print.
                     */
                    std::map < std::string, std::map < std::string, std::string > > getTextToPrintMap();

                protected:

                    /**
                     * @fn	void VotePrintDocument::PrintThread();
                     * @brief	Print thread.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     */

                    void PrintThread();

                    /**
                     * @fn	void VotePrintDocument::PrintDoc();
                     * @brief	Print document.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     */

                    void PrintDoc();

                    /**
                     * @fn	void VotePrintDocument::BeginPrint();
                     * @brief	Begins a print.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     */

                    void BeginPrint();

                    /**
                     * @fn	void VotePrintDocument::EndPrint();
                     * @brief	Ends a print.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     */

                    void EndPrint();

                    /**
                     * @fn	Glib::RefPtr<Gdk::Pixbuf> VotePrintDocument::GetQRCodeVoteInternal(const Smartmatic::SAES::Voting::Vote::Vote & vote,
                     * 			bool useBase64, Smartmatic::SAES::Configuration::QRCodeConfig & config );
                     * @brief	Gets a qr code vote internal.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     * @param	vote							The vote.
                     * @param	useBase64					true to use base 64.
                     * @param [in,out]	config	Size of the pixel.
                     * @return	The qr code vote internal.
                     * ### param	margin				 	The margin.
                     * ### param	version				 	The version.
                     * ### param	correctionLevel	The correction level.
                     */

                    Glib::RefPtr<Gdk::Pixbuf> GetQRCodeVoteInternal ( const Smartmatic::SAES::Voting::Vote::Vote & vote, bool useBase64, Smartmatic::SAES::Configuration::QRCodeConfig & config );
                    Glib::RefPtr<Gdk::Pixbuf> pixbufptrThread;  ///< The pixbufptr thread
                    Glib::RefPtr<Gdk::Drawable> drwptrThread;   ///< The drwptr thread
                    gint widthThread;   ///< The width thread
                    gint heightThread;  ///< The height thread

                    /**
                     * @fn	void VotePrintDocument::AddVoteHeader(const Smartmatic::SAES::Voting::Vote::Vote & vote);
                     * @brief	Adds a vote header.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     * @param	vote	The vote.
                     */

                    void AddVoteHeader ( const Smartmatic::SAES::Voting::Vote::Vote & vote );

                    /**
                     * @fn	void VotePrintDocument::AddVoteText(Glib::ustring toCompose,
                     * 			Smartmatic::SAES::Configuration::TextConfiguration textConfiguration,
                     * 			std::vector<Glib::ustring> & vParam);
                     * @brief	method to add a text line to the document.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     * @param	toCompose				 	text resource with place holders and translated.
                     * @param	textConfiguration	configurations of the text block.
                     * @param	vParam					 	vector of parameters, are taken into account the first 9.
                     */

                    void AddVoteText ( Glib::ustring toCompose,
                                       Smartmatic::SAES::Configuration::TextConfiguration textConfiguration,
                                       std::vector<Glib::ustring> & vParam );

                    /**
                     * @fn	void VotePrintDocument::AddFinalSpaces();
                     * @brief	Adds final spaces.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     */

                    void AddFinalSpaces();

                    /**
                     * @fn	void VotePrintDocument::addCustomAttributesHeader(const Smartmatic::SAES::Voting::Vote::Vote & vote);
                     * @brief	Adds custom attributes header.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     * @param	vote	The vote.
                     */

                    void addCustomAttributesHeader ( const Smartmatic::SAES::Voting::Vote::Vote & vote );

                    /**
                     * @fn	std::string VotePrintDocument::ProcessTextBallotOption(
                     * 			bool isPrincipal,
                     * 			Glib::ustring code,
                     * 			const Smartmatic::SAES::Voting::Election::Eligible_option * eligibleOption,
                     * 			int maxLastNameChars = 0,
                     * 			int maxNameChars = 0,
                     * 			int maxCodeChars = 0);
                     * @brief	Process the text ballot option.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     * @param	isPrincipal				true if is principal.
                     * @param	code					The code.
                     * @param	eligibleOption			The eligible option.
                     * @param	maxLastNameChars		The maximum last name characters.
                     * @param	maxNameChars			The maximum name characters.
                     * @param	maxCodeChars			The maximum code characters.
                     * @return	.
                     */

                    std::string ProcessTextBallotOption (
                        bool isPrincipal,
                        Glib::ustring code,
                        const Smartmatic::SAES::Voting::Election::Eligible_option * eligibleOption,
                        int maxLastNameChars = 0,
                        int maxNameChars = 0,
                        int maxCodeChars = 0 );

                    /**
                     * @fn	Glib::ustring VotePrintDocument::GetCustomAttributeValue( const Smartmatic::SAES::Voting::Election::Contest & contestWrapper,
                     * 			const Smartmatic::SAES::Voting::Vote::Vote & vote,
                     * 			const Smartmatic::SAES::Voting::Election::Contest_class * contestClass);
                     * @brief	Gets a custom attribute value.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     * @param	contestWrapper	The contest wrapper.
                     * @param	vote						The vote.
                     * @param	contestClass		The contest class.
                     * @return	The custom attribute value.
                     */

                    Glib::ustring GetCustomAttributeValue (
                        const Smartmatic::SAES::Voting::Election::Contest & contestWrapper,
                        const Smartmatic::SAES::Voting::Vote::Vote & vote,
                        const Smartmatic::SAES::Voting::Election::Contest_class * contestClass );

                    /**
                     * @fn	Glib::ustring VotePrintDocument::EvaluateText(Glib::ustring key);
                     * @brief	Evaluate text.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     * @param	key	The key.
                     * @return	.
                     */

                    Glib::ustring EvaluateText ( Glib::ustring key );

                    Glib::RefPtr<Gdk::Pixbuf> pixbufBarCode;	///< The pixbuf bar code

                    std::map < std::string, std::map < std::string, std::string > > textToPrintMap;

                private:

                    bool viewIdLine; ///< view id line
                    Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration;	///< The electoral configuration
                    const Smartmatic::SAES::Voting::Vote::Vote & _vote; ///< The vote
                    std::map <std::string, std::string> recoveryVoteMap; ///< Contest->Binary Map

                    Smartmatic::SAES::Voting::VotingDevice::Voting_booth votingBooth; ///< The voting booth
                    /**
                     * @property	std::list < Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition > customAttributeDefinition
                     * @brief	Gets the custom attribute definition.
                     * @return	The custom attribute definition.
                     */
                    std::list < Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition > customAttributeDefinition;

                    int countContest; ///< counter current contest
                    int countLinesList; ///< counter lines of ballot option list in current vote
                    int countLinesBallotOption; ///< counter lines of ballot option in current vote
                    int countLinesTotalList; ///< counter lines of total ballot option in current vote
                    int countLinesTotalBallotOption; ///< counter lines of total ballot option in current vote

                    int countContestName1;	///< The first count contest name
                    int countContestName2;	///< The second count contest name
                    int countContestName3;	///< The third count contest name

                    bool viewPrincipalTitle; ///< flag view principal title
                    bool viewStandInTitle; ///< flag view stand in title

                    int _maxBallotOptionLastNameChars;  ///< The maximum ballot option last name characters
                    int _maxBallotOptionNameChars;  ///< The maximum ballot option name characters
                    int _maxBallotOptionCodeChars;  ///< The maximum ballot option code characters

                    Glib::ustring strContestClassName;	///< Name of the contest class
                    Glib::ustring strContestClassDescription; ///< Information describing the contest class
                    Glib::ustring strContestTitle;	///< The contest title
                    Glib::ustring strContestAbstract; ///< The contest abstract
                    Glib::ustring strContestHelp; ///< The contest help
                    Glib::ustring strContestBinaryRecovery; ///< The contest binary recovery
                    Glib::ustring customAttributeValue; ///< The custom attribute value
                    Glib::ustring strEligibleOptionBallotName;	///< Name of the eligible option ballot
                    Glib::ustring strBallotOptionPartyName; ///< Name of the ballot option party
                    Glib::ustring strBallotOptionPartyAbbreviation; ///< The ballot option party abbreviation

                    Glib::ustring strElectionName; ///< Election name
                    Glib::ustring strElectoralOrg; ///< ElectoralOrganization
                    Glib::ustring strPollingPlaceName; ///< Polling Place Name
                    Glib::ustring strPollingPlaceAddress; ///< Polling Place Address

                    bool showBarCode;  ///< The show barcode

                    DocumentTextOptions _textOptionGeneric; ///< The text option generic

                    Glib::ustring _charPadding; ///< The character padding

                    /**
                     * @fn	virtual Smartmatic::System::SafeBuffer getBuffer ();
                     * @brief	Get buffer to print.
                     * @author	Yadickson Soto
                     * @date	06/08/2015
                     * @result	The QR code data.
                     */
                    virtual Smartmatic::System::SafeBuffer getBuffer ();

                    /**
                     * @fn	void VotePrintDocument::setBallotTitle( const Smartmatic::SAES::Voting::Election::Ballot & ballot );
                     * @brief	Method set ballot title in print vote
                     * 				@param ballot.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	29-07-2011
                     * @param	ballot	The ballot.
                     */
                    void setBallotTitle ( const Smartmatic::SAES::Voting::Election::Ballot & ballot );

                    /**
                     * @fn	void VotePrintDocument::setTopVote( const Smartmatic::SAES::Voting::Vote::Vote & vote);
                     * @brief	Method set top element in print vote.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	29-07-2011
                     * @param	vote	vote.
                     */
                    void setTopVote ( const Smartmatic::SAES::Voting::Vote::Vote & vote );

                    /**
                     * @fn	void VotePrintDocument::setCenterVote( const Smartmatic::SAES::Voting::Vote::Vote & vote);
                     * @brief	Method set center element in print vote
                     * 				@param vote.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	29-07-2011
                     * @param	vote	The vote.
                     */
                    void setCenterVote ( const Smartmatic::SAES::Voting::Vote::Vote & vote );

                    /**
                     * @fn	void VotePrintDocument::setBottomVote( const Smartmatic::SAES::Voting::Vote::Vote & vote);
                     * @brief	Method set bottom element in print vote
                     * 				@param vote.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	29-07-2011
                     * @param	vote	The vote.
                     */
                    void setBottomVote ( const Smartmatic::SAES::Voting::Vote::Vote & vote );

                    /**
                     * @fn	void VotePrintDocument::setSuperNull();
                     * @brief	Sets super null.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     */
                    void setSuperNull();

                    /**
                     * @fn	void VotePrintDocument::setContest( const Smartmatic::SAES::Voting::Vote::Vote & vote,
                     * 			Smartmatic::SAES::Voting::Election::Contest & contest,
                     * 			const Smartmatic::SAES::Voting::Vote::Selection & selection);
                     * @brief	Sets a contest.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     * @param	vote						 	The vote.
                     * @param [in,out]	contest	The contest.
                     * @param	selection				 	The selection.
                     */
                    void setContest (
                        const Smartmatic::SAES::Voting::Vote::Vote & vote,
                        Smartmatic::SAES::Voting::Election::Contest & contest,
                        const Smartmatic::SAES::Voting::Vote::Selection & selection );

                    /**
                     * @fn	void VotePrintDocument::setBallotOptionInfo( const Smartmatic::SAES::Voting::Vote::Vote & vote,
                     * 			Smartmatic::SAES::Voting::Election::Contest & contest,
                     * 			const Smartmatic::SAES::Voting::Vote::Selection & selection);
                     * @brief	Sets ballot option information.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     * @param	vote						 	The vote.
                     * @param [in,out]	contest	The contest.
                     * @param	selection				 	The selection.
                     */
                    void setBallotOptionInfo (
                        const Smartmatic::SAES::Voting::Vote::Vote & vote,
                        Smartmatic::SAES::Voting::Election::Contest & contest,
                        const Smartmatic::SAES::Voting::Vote::Selection & selection );

                    /**
                     * @fn	void VotePrintDocument::setPartyName( const std::string& contestCode, const Voting::Election::Ballot_option & ballotOption );
                     * @brief	Sets the party name and abbreviation for the given ballot option.
                     * @author	Angel Ramirez
                     * @date	31/05/2022
                     * @param	contestCode  The contest code.
                     * @param	ballotOption The ballot option.
                     */
                    void setPartyName ( const std::string& contestCode, const Voting::Election::Ballot_option & ballotOption );

                    /**
                     * @fn	void VotePrintDocument::setContestTitle( const Smartmatic::SAES::Voting::Vote::Vote & vote,
                     * 			Smartmatic::SAES::Voting::Election::Contest & contest);
                     * @brief	Sets contest title.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     * @param	vote						 	The vote.
                     * @param [in,out]	contest	The contest.
                     */
                    void setContestTitle (
                        const Smartmatic::SAES::Voting::Vote::Vote & vote,
                        const Smartmatic::SAES::Voting::Election::Contest & contest );

                    /**
                     * @fn  std::string getDistrictDescription( const Contest & contest );
                     * @brief   Gets the district description for the given contest.
                     * @author  Angel Ramirez
                     * @date    01/06/2022
                     * @param   contest The contest.
                     * @return  The district description.
                     */
                    std::string getDistrictDescription( const Contest & contest );

                    /**
                     * @fn	std::string getHelp();
                     * @brief	Get the contest help message.
                     * @author	Angel Ramirez
                     * @date	07/06/2022
                     * @return	The contest help message.
                     */
                    std::string getHelp( const Contest & contest );

                    /**
                     * @fn	Glib::ustring VotePrintDocument::getComposeMessage( Glib::ustring messageToCompose );
                     * @brief	Gets compose message.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     * @param	messageToCompose	The message to compose.
                     * @return	The compose message.
                     */
                    Glib::ustring getComposeMessage ( Glib::ustring messageToCompose );

                    /**
                     * @fn	void VotePrintDocument::setTitleReferendum();
                     * @brief	Sets title referendum.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     */
                    void setTitleReferendum();

                    /**
                     * @fn	void VotePrintDocument::setContestList( const Smartmatic::SAES::Voting::Vote::Vote & vote,
                     * 			Smartmatic::SAES::Voting::Election::Contest & contest,
                     * 			std::list < Smartmatic::SAES::Voting::Election::Ballot_option * > & list,
                     * 			const Smartmatic::SAES::Voting::Vote::Selection & selection );
                     * @brief	Sets contest list.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     * @param	vote						 	The vote.
                     * @param [in,out]	contest	The contest.
                     * @param [in,out]	list	 	The list.
                     * @param	selection				 	The selection.
                     */
                    void setContestList (
                        const Smartmatic::SAES::Voting::Vote::Vote & vote,
                        Smartmatic::SAES::Voting::Election::Contest & contest,
                        std::list < Smartmatic::SAES::Voting::Election::Ballot_option * > & list,
                        const Smartmatic::SAES::Voting::Vote::Selection & selection );

                    /**
                     * @fn	void VotePrintDocument::setContestBallotOption( const Smartmatic::SAES::Voting::Vote::Vote & vote,
                     * 			Smartmatic::SAES::Voting::Election::Contest & contest,
                     * 			const Smartmatic::SAES::Voting::Vote::Selection & selection,
                     * 			std::list < Smartmatic::SAES::Voting::Election::Ballot_option * > & listBallotOptionPrincipal,
                     * 			std::list < Smartmatic::SAES::Voting::Election::Ballot_option * > & listBallotOptionStandIn,
                     * 			bool isQuestion);
                     * @brief	Sets contest ballot option.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     * @param	vote															 	The vote.
                     * @param [in,out]	contest									 	The contest.
                     * @param	selection													 	The selection.
                     * @param [in,out]	listBallotOptionPrincipal	The list ballot option principal.
                     * @param [in,out]	listBallotOptionStandIn	 	The list ballot option stand in.
                     * @param	isQuestion												 	true if this object is question.
                     */
                    void setContestBallotOption (
                        const Smartmatic::SAES::Voting::Vote::Vote & vote,
                        Smartmatic::SAES::Voting::Election::Contest & contest,
                        const Smartmatic::SAES::Voting::Vote::Selection & selection,
                        std::list < Smartmatic::SAES::Voting::Election::Ballot_option * > & listBallotOptionPrincipal,
                        std::list < Smartmatic::SAES::Voting::Election::Ballot_option * > & listBallotOptionStandIn,
                        bool isQuestion );

                    /**
                     * @fn	void VotePrintDocument::setBallotOptions(Smartmatic::SAES::Voting::Election::Contest & contest,
                     * 			std:: vector < Smartmatic::SAES::Voting::Election::Ballot_option * > & list,
                     * 			Smartmatic::SAES::Configuration::LayoutContestConfiguration & layout, bool isPrincipal,
                     * 			bool isQuestion);
                     * @brief	Sets ballot options.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     * @param [in,out]	contest	The contest.
                     * @param [in,out]	list	 	The list.
                     * @param [in,out]	layout 	The layout.
                     * @param	isPrincipal			 	true if this object is principal.
                     * @param	isQuestion			 	true if this object is question.
                     */
                    void setBallotOptions ( Smartmatic::SAES::Voting::Election::Contest & contest,
                                            std:: vector < Smartmatic::SAES::Voting::Election::Ballot_option * > & list,
                                            Smartmatic::SAES::Configuration::LayoutContestConfiguration & layout,
                                            bool isPrincipal,
                                            bool isQuestion );

                    /**
                     * @fn	void VotePrintDocument::setContestBlankSpace( Glib::ustring contestCode, bool addEndSpace );
                     * @brief	Method set blank space in end contest.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	29-07-2011
                     * @param	contestCode	contest code.
                     * @param	addEndSpace	add end space contest.
                     */
                    void setContestBlankSpace ( Glib::ustring contestCode, bool addEndSpace );

                    /**
                     * @fn	void VotePrintDocument::setContestTitlePrincipalSpace( int count = 1 );
                     * @brief	Method set blank space by title principal config.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	29-07-2011
                     * @param	count	count line space, default 1.
                     */
                    void setContestTitlePrincipalSpace ( int count = 1 );

                    /**
                     * @fn	void VotePrintDocument::setContestTitleStandInSpace( int count = 1 );
                     * @brief	Method set blank space by title stand in config.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	29-07-2011
                     * @param	count	count line space, default 1.
                     */
                    void setContestTitleStandInSpace ( int count = 1 );

                    /**
                     * @fn	void VotePrintDocument::setContestEndSpace( int count = 1 );
                     * @brief	Method set blank space by end config.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	29-07-2011
                     * @param	count	count line space, default 1.
                     */
                    void setContestEndSpace ( int count = 1 );

                    /**
                     * @fn  void VotePrintDocument::setExtraSpaceForMissingContest( int missingContestLines);
                     * @brief   Method to set extra blank spaces for minor difs.
                     * @author  Yadickson Soto yadickson.soto@smartmatic.com
                     * @date    29-07-2011
                     * @param   missingContestLines lines not printed to calculate extra lines
                     */
                    void setExtraSpaceForMissingContest ( int missingContestLines );

                    /**
                     * @fn	void VotePrintDocument::setContestListSpace(int countList);
                     * @brief	Method set blank space by list config
                     * 				@param countList.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	29-07-2011
                     * @param	countList	List of counts.
                     */
                    void setContestListSpace ( int countList );

                    /**
                     * @fn	void VotePrintDocument::setContestBallotSpace(int countBallot);
                     * @brief	Method set blank space bay ballot config
                     * 				@param countBallot.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	29-07-2011
                     * @param	countBallot	The count ballot.
                     */
                    void setContestBallotSpace ( int countBallot );

                    /**
                     * @fn	void VotePrintDocument::setBlankSpaceVote();
                     * @brief	Method set blank space in end vote.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	29-07-2011
                     */
                    void setBlankSpaceVote();
                    /**
                     * @fn	void VotePrintDocument::translateText();
                     * @brief	Method translate text languages.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	19-04-2012
                     */
                    void translateText();
                    /**
                     * @fn	void VotePrintDocument::clearText();
                     * @brief	Method clear text languages.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	19-04-2012
                     */
                    void clearText();
                    /**
                     * @fn	void VotePrintDocument::concatenateText();
                     * @brief	Method concatenate all text languages.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	19-04-2012
                     */
                    void concatenateText();
                    /**
                     * @fn	void VotePrintDocument::concatenateText(Glib::ustring text, Glib::ustring & result);
                     * @brief	Method concatenate all text languages.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	19-04-2012
                     * @param	text		text.
                     * @param	result	result.
                     */
                    void concatenateText ( Glib::ustring text, Glib::ustring & result );
                    /**
                     * @fn	Glib::ustring VotePrintDocument::getResource(Glib::ustring key);
                     * @brief	Method set resource text all languages.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	19-04-2012
                     * @param	key	key name.
                     * @return	text resource.
                     */
                    Glib::ustring getResource ( Glib::ustring key );

                    /**
                     * @fn  void VotePrintDocument::sortBallotOptionsListByTallyPosition ( std::vector < Ballot_option * > & ballotOptionList );
                     * @brief   Sort ballot options list by tally position.
                     * @author  marcel.mancini
                     * @date    07/02/2018
                     * @param   ballotOptionList    List of ballot options.
                     */
                    void sortBallotOptionsListByTallyPosition ( std::vector < Ballot_option * > & ballotOptionList );

                    bool isMultiLanguage; ///< flag multi language vote
                    Glib::ustring separatorLanguage; ///< separator for multi language
                    unsigned maxCharByLanguage; ///< max char by language

                    Glib::ustring strDemoTitle; ///< demo title
                    Glib::ustring strSimulatedTitle; ///< simulated title
                    Glib::ustring strOfficialTitle; ///< official title

                    Glib::ustring strNullReferendum; ///< null referendum
                    Glib::ustring strNullVote; ///< null vote
                    Glib::ustring strTitleReferendum; ///< title referendum

                    Glib::ustring strParentOptionMessage; ///< parent option message
                    Glib::ustring strParentOptionWasSelected; ///< parent option selected

                    Glib::ustring strPrincipalTitleReferendum; ///< principal title referendum
                    Glib::ustring strPrincipalNoSelectectedMessage; ///< principal no selected message
                    Glib::ustring strPrincipalMessage; ///< principal message

                    Glib::ustring strStandInTitleReferendum; ///< stand in title referendum
                    Glib::ustring strStandInNoSelectedMessage; ///< stand in no selected message
                    Glib::ustring strStandInMessage; ///< stand in message

                    Glib::ustring strDemoEnd; ///< demo end
                    Glib::ustring strRecoveryEnd; ///< recovery end
                    Glib::ustring strSimulatedEnd; ///< simulate end
                    Glib::ustring strOfficialEnd; ///< official end
                    Glib::ustring strTextBottomDemo; ///< text bottom demo
                    Glib::ustring strTextBottomSimulate; ///< text bottom simulate
                    Glib::ustring strTextBottomOfficial; ///< text bottom official

                    Glib::ustring separatorName; ///< separator name config
                    int maxList; ///<max list count
                    int maxBallotOptions; ///< max ballot option count

                    static Smartmatic::Log::ISMTTLog * logger; ///<logger

                    Smartmatic::SAES::Configuration::ConfigurationVote printConfiguration;	///< The print configuration
                    Smartmatic::SAES::Configuration::LayoutContestConfiguration layout; ///< The layout

                    std::map < std::string, std::map < std::string, std::string > > m_signalTextToPrintCompleted;
            };
        }
    }
}

#endif // _VOTEPRINTDOCUMENT_H_
