/**
 * @file   GeneralAudioResourcesManager.cxx
 * @brief  Implementation of GeneralAudioResourcesManager class.
 * @author manuel.arguelles
 * @date   Oct 12, 2017
 */

#include "GeneralAudioResourcesManager.hxx"
#include <Log/SMTTLogManager.h>
#include <Log/ISMTTLog.h>
#include <System/IO/Directory.h>
#include <System/IO/Path.h>
#include <Security/Encryption.hxx>
#include <System/Utils/StringUtils.hxx>

#include <vector>
#include <string>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

using namespace Smartmatic::Log;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::System::Utils;

ISMTTLog* GeneralAudioResourcesManager::m_logger = SMTTLogManager::GetLogger(
    "Smartmatic.SAES.Resources.GeneralAudioResourcesManager",
    GETTEXT_PACKAGE);

GeneralAudioResourcesManager* GeneralAudioResourcesManager::m_instance = NULL;

GeneralAudioResourcesManager::GeneralAudioResourcesManager()
{
}

GeneralAudioResourcesManager::~GeneralAudioResourcesManager()
{
}

GeneralAudioResourcesManager* GeneralAudioResourcesManager::getInstance(GeneralAudioResourcesManager* externResource)
{
    if (externResource != NULL)
    {
        m_instance = externResource;
    }
    else if (m_instance == NULL)
    {
        m_instance = new GeneralAudioResourcesManager();
    }

    return m_instance;
}

void GeneralAudioResourcesManager::init(SaesDirectories* directories, BasePathType type)
{
    m_fileName = directories->getGeneralAudioResourcesFileName();
    m_folderPath = directories->getElectoralResourceFullFolderPath();
    Initialize();
}

std::string& GeneralAudioResourcesManager::getFileName()
{
    return m_fileName;
}

std::string& GeneralAudioResourcesManager::getFullFolderPath()
{
    return m_folderPath;
}

void GeneralAudioResourcesManager::Initialize()
{
    std::vector<std::string> files;
    std::vector<std::string>::const_iterator it;

    Dispose();

    if (Directory::getInstance()->DirectoryExists(getFullFolderPath()))
    {
        m_logger->Debug("Open Directory: " + getFullFolderPath());
    }
    else
    {
        m_logger->Debug("Directory " + getFullFolderPath() + " not found.");
        return;
    }

    Directory::getInstance()->GetFiles(getFullFolderPath(), files);

    for (it = files.begin(); it != files.end(); ++it)
    {
        std::string filePresent = Path::GetFileNameWithOutExtension(*it);
        std::string fileWanted = Path::GetFileNameWithOutExtension(getFileName());

        std::transform(fileWanted.begin(), fileWanted.end(), fileWanted.begin(), ::toupper);
        std::transform(filePresent.begin(), filePresent.end(), filePresent.begin(), ::toupper);

        if (filePresent.find(fileWanted) != std::string::npos)
        {
            try
            {
                std::string culture;
                std::auto_ptr<Strings_resources> resources;
                std::string file = Path::Combine(getFullFolderPath(), *it);
                std::string data;
                std::map<std::string, std::string>* cultureResources;
                Strings_resources::SrIterator iterator;

                m_logger->Debug("Open file: " + file);
                data = Smartmatic::SAES::Security::Encryption::getInstance()->openXmlFile (
                    file,
                    Smartmatic::SAES::Security::Encryption::EXTERNAL_FILE,
                    Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_UP_KEY,
                    false);

                std::istringstream istr(data);

                resources = parseStrings_resources(istr, ResourcesTypes::Flags::dont_validate);
                filePresent.erase(filePresent.begin(), filePresent.begin() + fileWanted.size());
                culture = StringUtils::toLower(filePresent);
                cultureResources = new std::map<std::string, std::string>();

                m_logger->Debug("Load culture: " + culture + " - number resources keys: " +
                                StringUtils::intToString(resources->getSr().size()));

                for (iterator = resources->getSr().begin(); iterator != resources->getSr().end(); ++iterator)
                {
                    const Sr& resource(*iterator);
                    (*cultureResources)[resource.getKey()] = resource.getValue();
                }

                cultureMap[culture] = cultureResources;
            }
            catch (const ::xsd::cxx::tree::exception<char>& e)
            {
                std::stringstream ss;
                ss << "FATAL - " << __func__ << " " << e;
                m_logger->Debug(ss.str());
            }
            catch (Smartmatic::System::Exception::CryptoException& e)
            {
                m_logger->Debug("FATAL - " + std::string(__func__) + e.what());
            }
            catch ( Smartmatic::System::Exception::SignedException & e )
            {
                m_logger->Debug("FATAL - " + std::string(__func__) + e.what());
            }
            catch ( Smartmatic::System::Exception::XmlException & e )
            {
                m_logger->Debug("FATAL - " + std::string(__func__) + e.what());
            }
            catch (...)
            {
                m_logger->Debug("FATAL - " + std::string(__func__));
            }
        }
    }
}
