/**
 * @file	DeviceParametersManager.h
 * @brief	Implementation of the device parameters manager class.
 * @author	angel.ramirez
 * @date	05/07/2022
 */

#include "DeviceParametersManager.hxx"

#include <Exceptions/Voting/ElectoralConfiguration/ConfigurationException.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Security/Encryption.hxx>
#include <Voting/DeviceParameters/device-parameters-schema.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Exceptions::Voting;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::SAES::Voting;

ISMTTLog * DeviceParametersManager::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Voting.DeviceParametersManager", GETTEXT_PACKAGE );

DeviceParametersManager* DeviceParametersManager::singleton_instance = NULL;

DeviceParametersManager::DeviceParametersManager()
{
}

DeviceParametersManager::~DeviceParametersManager()
{
    singleton_instance = NULL;
}

DeviceParametersManager* DeviceParametersManager::getInstance ( DeviceParametersManager* externResource )
{
    if ( !singleton_instance )
    {
        singleton_instance = ( externResource == NULL ? new DeviceParametersManager() : externResource );
    }

    return singleton_instance;
}

void DeviceParametersManager::init (
    Smartmatic::SAES::Runtime::Environment::SaesDirectories* saesDir,
    Smartmatic::SAES::Common::Configuration::BasePathType type )
{
    filePath = saesDir->getDeviceParametersFileFullPath();
    Initialize();
}

bool DeviceParametersManager::getParameter ( const std::string& key ) const
{
    std::map<std::string, bool>::const_iterator parameter = parametersMap.find ( key );
    bool value;

    if ( parameter != parametersMap.end() )
    {
        value = parameter->second;
    }
    else
    {
        throw ConfigurationException("The given key '" + key + "' was not found in device-parameters file.",
            "Key not found");
    }

    return value;
}

void DeviceParametersManager::Initialize()
{
    try
    {
        logger->Debug ( "Open file: " + filePath );

        std::string data = Smartmatic::SAES::Security::Encryption::getInstance()->openXmlFile ( 
            filePath, 
            Smartmatic::SAES::Security::Encryption::EXTERNAL_FILE,
            Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_UP_KEY,
            false,
            true,
            false);

        std::istringstream istr ( data );
        std::auto_ptr< Parameters > parameters = parseParameters ( istr, ::xml_schema::Flags::dont_validate );
        Parameters::ParameterIterator iterator ( parameters->getParameter().begin() );
        Parameters::ParameterIterator enditerator ( parameters->getParameter().end() );

        while ( iterator != enditerator )
        {
            const Parameter& parameter ( *iterator++ );
            parametersMap[parameter.getName()] = parameter;
        }
    }
    catch ( const ::xsd::cxx::tree::exception< char > & e )
    {
        std::stringstream ss;
        ss << "FATAL - " << __func__ << " " << e;
        logger->Debug ( ss.str() );
        throw;
    }
    catch ( Smartmatic::System::Exception::CryptoException & e )
    {
        std::stringstream ss;
        ss << "FATAL - " << __func__ << " " << e.what();
        logger->Debug ( ss.str() );
        throw;
    }
    catch ( Smartmatic::System::Exception::SignedException & e )
    {
        std::stringstream ss;
        ss << "FATAL - " << __func__ << " " << e.what();
        logger->Debug ( ss.str() );
        throw;
    }
    catch ( Smartmatic::System::Exception::XmlException & e )
    {
        std::stringstream ss;
        ss << "FATAL - " << __func__ << " " << e.what();
        logger->Debug ( ss.str() );
        throw;
    }
    catch ( ... )
    {
        std::stringstream ss;
        ss << "FATAL - " << __func__;
        logger->Debug ( ss.str() );
        throw;
    }
}
