/**
 * @file	DeviceParametersManager.h
 * @brief	Declaration of the device parameters manager class.
 * @author	angel.ramirez
 * @date	05/07/2022
 */

#ifndef _DEVICE_PARAMETERS_MANAGER_H_
#define _DEVICE_PARAMETERS_MANAGER_H_

#include "Runtime/Environment/saes-directories.h"

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Voting
        {
            /**
             * @class	DeviceParametersManager
             * @brief	Manager for device parameters configuration file.
             * @author	Angel Ramirez
             * @date	05/07/2022
             */
            class DeviceParametersManager
            {
                public:

                    /**
                     * @fn	DeviceParametersManager::DeviceParametersManager();
                     * @brief	Default constructor.
                     * @author	Angel Ramirez
                     * @date	05/07/2022
                     */
                    DeviceParametersManager();

                    /**
                     * @fn	virtual DeviceParametersManager::~DeviceParametersManager();
                     * @brief	Class destructor.
                     * @author	Angel Ramirez
                     * @date	05/07/2022
                     */
                    virtual ~DeviceParametersManager();

                    /**
                     * @fn	static DeviceParametersManager* DeviceParametersManager::getInstance( DeviceParametersManager * externResource = NULL );
                     * @brief	Gets the current Device Parameters Manager.
                     * @author	Angel Ramirez
                     * @date	05/07/2022
                     * @param externResource extern resource only to unit test
                     * @return	instance the Device Parameters Manager.
                     */
                    static DeviceParametersManager* getInstance ( DeviceParametersManager* externResource = NULL );

                    /**
                     * @fn	void DeviceParametersManager::init(Smartmatic::SAES::Runtime::Environment::SaesDirectories * saesDir,
                     * 			Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM);
                     * @brief	Initialises this object.
                     * @author	Angel Ramirez
                     * @date	05/07/2022
                     * @param	saesDir	The saes dir.
                     * @param	type	(optional) the type.
                     */
                    void init ( Smartmatic::SAES::Runtime::Environment::SaesDirectories* saesDir,
                        Smartmatic::SAES::Common::Configuration::BasePathType type = Smartmatic::SAES::Common::Configuration::BasePathType::CONFIGURATION_DOM );

                    /**
                     * @fn	bool DeviceParametersManager::getParameter( const std::string& key ) const;
                     * @brief	Gets a parameter value.
                     * @author	angel Ramirez
                     * @date	05/07/2022
                     * @param	key The parameter key.
                     * @return	The parameter value.
                     * @throws  ConfigurationException If the given key is not found.
                     */
                    bool getParameter ( const std::string& key ) const;

                private:

                    /**
                     * @fn	void DeviceParametersManager::Initialize();
                     * @brief	Initializes the device parameters information.
                     * @author	Angel Ramirez
                     * @date	05/07/2022
                     */
                    void Initialize();

                    std::string filePath;   ///< Filename of the file
                    static DeviceParametersManager* singleton_instance; ///< The singleton instance of the Device Parameters Manager
                    std::map<std::string , bool> parametersMap; ///< The parameters map
                    static Smartmatic::Log::ISMTTLog* logger; ///<logger
            };
        }
    }
}

#endif // _DEVICE_PARAMETERS_MANAGER_H_
