/**
 * @file   EmlFileManager.cxx
 * @brief  Implementation of EmlFileManager class.
 * @author manuel.arguelles
 * @date   Jan  8, 2018
 */

#include "EmlFileManager.hxx"
#include "EmlFileInfo.hxx"
#include "EmlWrapper.hxx"
#include "X7SEmlFileWrapper.hxx"
#include <Configuration/SaesCommonConfiguration-schema.hxx>
#include <Configuration/s-a-e-s-conf-manager.h>
#include <Configuration/s-a-e-s-common-conf-manager.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Runtime/Environment/saes-directories.h>
#include <System/Exception/CryptoException.hxx>
#include <System/IO/Directory.h>
#include <System/IO/FatFileManager.hxx>
#include <System/IO/File.h>
#include <System/IO/Path.h>
#include <System/Runtime/ProcessLauncher.h>
#include <System/SafeBuffer.h>
#include <System/Utils/StringUtils.hxx>
#include <Voting/Election/election-schema.hxx>
#include <Voting/ElectoralConfiguration.hxx>
#include <Voting/PBReports/PVContestWrapper.hxx>
#include <Voting/ResultValidation.hxx>
#include <Voting/Vote/vote-schema.hxx>
#include <Voting/VotingDevice/voting-devices-schema.hxx>
#include <vector>
#include <list>



#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::SAES::Voting::Election;
using namespace Smartmatic::SAES::Voting::Eml;
using namespace Smartmatic::SAES::Voting::EmlX7S;
using namespace Smartmatic::SAES::Voting::Vote;
using namespace Smartmatic::SAES::Voting::VotingDevice;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::Runtime;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::System;

ISMTTLog* EmlFileManager::m_logger = SMTTLogManager::GetLogger(
    "Smartmatic.SAES.Voting.Eml.EmlFileManager",
    GETTEXT_PACKAGE);

/**
 * @brief  Evaluates if a vote should be considered or not.
 * @author manuel.arguelles
 * @date   Feb 28, 2018
 * @param  vote The vote.
 * @return True if the vote should be counted, false if not.
 */
static bool countVote(const Smartmatic::SAES::Voting::Vote::Vote& vote)
{
    bool count = false;

    switch (vote.getMode())
    {
        case VoteMode::VOTE_D:
            if (SAESConfigurationManager::getInstance()->getSaveDemoVotesInTransmissionPackage())
                count = true;
            else
                count = false;
            break;

        case VoteMode::VOTE_O:
            count = true;
            break;

        case VoteMode::VOTE_S:
            count = false;
            break;

        default:
            break;
    }

    return count;
}


/**
 * @brief  Gets the template of the X7S file without language.
 * @author manuel.arguelles
 * @date   Mar  2, 2018
 * @param  filename The original EML file name.
 * @return The template of the file.
 */
static std::string getX7SFileTemplate(const std::string& filename)
{
    std::string templateFile = Path::GetFileNameWithOutExtension(filename);
    std::string::size_type lastDashPosition;

    lastDashPosition = templateFile.find_last_of("_");

    if (lastDashPosition != std::string::npos)
    {
        templateFile.replace(lastDashPosition, 1, ".");
    }

    return templateFile;
}

static std::vector<std::string> getX7SFilesFromEml(
    Smartmatic::SAES::Voting::ElectoralConfiguration* configuration,
    const std::string& filename)
{
    std::string templateFile = getX7SFileTemplate(filename);
    std::vector<std::string> x7sFiles;
    std::list<std::string> files;
    std::list<std::string>::iterator it;

    configuration->getFileNamesByLanguages(templateFile, files, false, 1, true);

    for (it = files.begin(); it != files.end(); ++it)
    {
        std::string x7sWithoutExt  = Path::GetFileNameWithOutExtension(*it);
        std::string x7sOriginalExt = Path::GetFileExtension(*it).substr(1);

        std::string x7s = x7sWithoutExt + "_" + x7sOriginalExt + ".EML.X7S";

        x7sFiles.push_back(x7s);
    }

    return x7sFiles;
}

EmlFileManager::EmlFileManager()
    : m_directories(NULL)
    , m_electoralConfiguration(NULL)
{
}

EmlFileManager::~EmlFileManager()
{
}

void EmlFileManager::init(ElectoralConfiguration* electoralConfiguration, SaesDirectories* directories)
{
    m_directories = directories;
    m_electoralConfiguration = electoralConfiguration;

    if (!SAESConfigurationManager::getInstance()->getEmlFileConfiguration().getEnableEml()) return;

    try
    {
        VotingDevice::Voting_booths::Voting_boothSequence seq;
        std::string votingBoothCustomCode;

        seq = m_electoralConfiguration->getVotingDevice()->getVotingBooths().getVoting_booth();

        if (seq.size() > 0)
        {
            VotingDevice::Voting_booth& votingBooth = seq[0];
            votingBoothCustomCode = votingBooth.getCustom_code();
            std::string find = SAESConfigurationManager::getInstance()->getEmlFileConfiguration().getSeparatorVotingDevice();
            std::vector<std::string> vector;
            StringUtils::splitToVector(votingBoothCustomCode, find, vector);

            if (vector.size() >= 2)
            {
                m_machine = vector[1];
            }

            if (vector.size() >= 3)
            {
                m_customNisCode = vector[2];
            }
        }

        m_region = votingBoothCustomCode;
    }
    catch (...)
    {
        m_logger->Debug("Error initializing EML files");
    }

    loadEmlFiles();
}

void EmlFileManager::loadEmlFiles()
{
    std::list<Voting::Election::Contest*>& contests = m_electoralConfiguration->getContestList();
    std::list<Voting::Election::Contest*>::const_iterator it;
    std::string emlFolder = m_directories->getEmlFilesFolderFullPath(BasePathType::CONFIGURATION_DOM);

    for (it = contests.begin(); it != contests.end(); ++it)
    {
        std::string contestCode = (*it)->getCode();
        std::string customCode  = (*it)->getCustom_code();
        std::string fileName    = EmlFileInfo::getInputContestFileName(customCode);

        m_logger->Debug("Initializing contest: " + contestCode + " / " + customCode + " into " + fileName);
        m_emlFiles[customCode] = EmlFileInfo::loadEmlFromFile(Path::Combine(emlFolder, fileName));

        m_logger->Debug("Preparing EML for " + customCode);
        EmlWrapper emlWrapper(m_emlFiles[customCode]);

        emlWrapper.updateTotalVotesSelectionCategory(customCode);
        emlWrapper.removePartiesOfDifferentLinguisticRegime(customCode);
        emlWrapper.updatePartyAndCandidateNames(customCode, *m_electoralConfiguration);
    }
}

std::map<std::string, std::string> EmlFileManager::getRelationFiles() const
{
    std::map<std::string, std::string> files;
    const std::list<Voting::Election::Contest*>& contest = m_electoralConfiguration->getContestList();
    std::list<Voting::Election::Contest*>::const_iterator it;

    for (it = contest.begin(); it != contest.end(); ++it)
    {
        std::string customCode;
        std::string inputFile;
        std::string outputFile;

        customCode = (*it)->getCustom_code();
        inputFile = EmlFileInfo::getInputContestFileName(customCode);
        outputFile = EmlFileInfo::getOutputContestFileName(customCode, m_machine, "", getOutputNisCodeFromBooth(customCode));
        files[inputFile] = outputFile;
    }

    return files;
}

std::string EmlFileManager::getOutputContestFileName(const std::string& customCode) const
{
    std::string inFileName = EmlFileInfo::getInputContestFileName(customCode);
    std::map<std::string, std::string> filesMap = getRelationFiles();

    return filesMap[inFileName];
}

std::string EmlFileManager::getEmlContestCode(const std::string& fileName) const
{
    std::string emlContestCode = "K";
    std::list<Voting::Election::Contest*>& contests = m_electoralConfiguration->getContestList();
    std::list<Voting::Election::Contest*>::const_iterator it;

    for (it = contests.begin(); it != contests.end(); ++it)
    {
        std::string customCode     = (*it)->getCustom_code();
        std::string outputFileName = getOutputContestFileName(customCode);

        if (fileName == outputFileName)
        {
            emlContestCode = getOutputShortCodeFromBooth(customCode);
            m_logger->Debug("Short code for contest code " + customCode + " is " + emlContestCode);
            break;
        }
    }

    return emlContestCode;
}

bool EmlFileManager::isListSelection(const std::string& contestCode, const std::string& ballotCode)
{
    bool isList = false;
    std::list<Voting::Election::Ballot_option*> options = m_electoralConfiguration->getEligibleOptionsListByContestCode(contestCode);
    std::list<Voting::Election::Ballot_option*>::const_iterator it;

    for (it = options.begin(); it != options.end(); ++it)
    {
        if ((*it)->getCode() == ballotCode)
        {
            isList = true;
            break;
        }
    }

    return isList;
}

bool EmlFileManager::isPrincipalSelection(const std::string& contestCode, const std::string& ballotCode)
{
    bool isPrincipal = false;
    std::list<Voting::Election::Ballot_option*> options = m_electoralConfiguration->getEligibleOptionsPrincipalByContestCode(contestCode);
    std::list<Voting::Election::Ballot_option*>::const_iterator it;

    for (it = options.begin(); it != options.end(); ++it)
    {
        if ((*it)->getCode() == ballotCode)
        {
            isPrincipal = true;
            break;
        }
    }

    return isPrincipal;
}

bool EmlFileManager::isStandInSelection(const std::string& contestCode, const std::string& ballotCode)
{
    bool isStandIn = false;
    std::list<Voting::Election::Ballot_option*> options = m_electoralConfiguration->getEligibleOptionsStandInByContestCode(contestCode);
    std::list<Voting::Election::Ballot_option*>::const_iterator it;

    for (it = options.begin(); it != options.end(); ++it)
    {
        if ((*it)->getCode() == ballotCode)
        {
            isStandIn = true;
            break;
        }
    }

    return isStandIn;
}

void EmlFileManager::updateCounters(const Vote::Vote& vote)
{
    Vote::Vote::SSequence selections = vote.getSelections();
    Vote::Vote::SIterator sIt;

    if (!countVote(vote))
    {
        m_logger->Debug("Ignoring vote");
        return;
    }

    for (sIt = selections.begin(); sIt != selections.end(); ++sIt)
    {
        std::string contestCode = sIt->getContest();
        Vote::Selection::BSequence ballotOptions = sIt->getBallotOptions();
        Vote::Selection::BIterator bIt;
        std::string customCode;

        bool hasListBeenSelected       = false;
        bool hasPrincipalsBeenSelected = false;
        bool hasStandInBeenSelected    = false;

        std::string partyCustomCode;

        customCode = m_electoralConfiguration->getContestByCode(contestCode)->getCustom_code();

        m_logger->Debug("Contest code: " + contestCode);
        m_logger->Debug("Custom code: " + customCode);

        EmlWrapper emlWrapper(m_emlFiles[customCode]);

        for (bIt = ballotOptions.begin(); bIt != ballotOptions.end(); ++bIt)
        {
            std::string ballotCode = *bIt;
            std::string candidateCode;
            std::string partyCode = m_electoralConfiguration->getPartyCodeByBallotOptionCode(ballotCode);

            m_logger->Debug("Ballot code: " + ballotCode);
            Voting::Election::Eligible_option* option = m_electoralConfiguration->getEligibleOptionFromBallotOptionCode(contestCode, ballotCode);
            m_logger->Debug(" Ballot name: " + option->getBallot_name_key());
            m_logger->Debug(" Ballot candidate id: " + option->getCandidate_id());
            m_logger->Debug(" Ballot eligible option code: " + option->getCode());
            m_logger->Debug(" Ballot custom code: " + option->getCustom_code());
            m_logger->Debug(" Ballot parent code: " + option->getParent_code());
            m_logger->Debug(" Party code: " + partyCode);

            if (m_electoralConfiguration->getPartyByCode(partyCode) != NULL)
            {
                partyCustomCode = m_electoralConfiguration->getPartyByCode(partyCode)->getCustom_code();
            }

            m_logger->Debug(" Party custom code: " + partyCustomCode);

            candidateCode = option->getCustom_code();

            m_logger->Debug(" Candidate code: " + candidateCode);

            if (isListSelection(contestCode, ballotCode))
            {
                hasListBeenSelected = true;
            }
            else if (isPrincipalSelection(contestCode, ballotCode))
            {
                hasPrincipalsBeenSelected = true;
                emlWrapper.increaseCandidateVote(customCode, candidateCode);
            }
            else if (isStandInSelection(contestCode, ballotCode))
            {
                hasStandInBeenSelected = true;
                emlWrapper.increaseCandidateVote(customCode, candidateCode);
            }
        }

        if (ballotOptions.empty())
        {
            m_logger->Debug("No ballot options");
            emlWrapper.increaseBlankInvalid(customCode);
        }
        else
        {
            if (hasPrincipalsBeenSelected && !hasStandInBeenSelected)
            {
                emlWrapper.increasePartyPrincipalBallotCount(customCode, partyCustomCode);
            }
            else if (!hasPrincipalsBeenSelected && hasStandInBeenSelected)
            {
                emlWrapper.increasePartyStandInBallotCount(customCode, partyCustomCode);
            }
            else if (hasPrincipalsBeenSelected && hasStandInBeenSelected)
            {
                emlWrapper.increasePartyPrincipalStandInBallotCount(customCode, partyCustomCode);
            }
            else if (hasListBeenSelected)
            {
                emlWrapper.increasePartyListBallotCount(customCode, partyCustomCode);
            }
        }
    }
}

void EmlFileManager::setTotalsByType(const std::map<std::string, long>& totalsByType)
{
    std::map<std::string, Voting::Eml::EML>::iterator it;

    for (it = m_emlFiles.begin(); it != m_emlFiles.end(); ++it)
    {
        EmlWrapper emlWrapper(it->second);

        emlWrapper.setTotalsByType(totalsByType);
    }
}

std::vector<std::pair<std::string, std::string> > EmlFileManager::getEmlFileHashes() const
{
    std::vector<std::pair<std::string, std::string> > result;
    std::map<std::string, EML>::const_iterator it;
    std::string emlFolder = m_directories->getEmlTempFolderFullPath();

    for (it = m_outputEmlFiles.begin(); it != m_outputEmlFiles.end(); ++it)
    {
        SafeBuffer buffer;
        std::pair<std::string, std::string> entry;
        std::string file = Path::Combine(emlFolder, it->first);

        File::getInstance()->getHashFile(file, buffer);
        entry = std::make_pair(it->first, buffer.GetBase64Data());
        result.push_back(entry);
    }

    return result;
}

void EmlFileManager::persistPlainEmlFiles()
{
    std::map<std::string, EML>::iterator it;
    const std::string emlTempFolder = m_directories->getEmlTempFolderFullPath();

    Directory::getInstance()->Mkdir(emlTempFolder);

    buildOutputEml();

    for (it = m_outputEmlFiles.begin(); it != m_outputEmlFiles.end(); ++it)
    {
        std::string fileName = it->first;
        EmlWrapper emlWrapper(it->second);

        try
        {
            long checksum;
            std::string closingDateTime = \
                m_electoralConfiguration->getVotingDevice()->getFirstVotingDevice().getOperation_modes().getOperation_mode().front().getClosing_date_time();

            emlWrapper.prepareForOutput(fileName, getEmlContestCode(fileName), m_machine);
            emlWrapper.setElectionDate(closingDateTime);
            emlWrapper.sort();
            checksum = emlWrapper.calculateChecksums();
            updateContestChecksum(fileName, checksum);
        }
        catch (...)
        {
            m_logger->Debug("Checksum failed for " + fileName);
            throw;
        }

        const std::string emlTempFile = Path::Combine(emlTempFolder, fileName);

        if (emlWrapper.serialize(emlTempFile))
        {
            m_logger->Debug("Serializing eml file " + fileName + ": OK");
        }
        else
        {
            m_logger->Debug("Serializing eml file " + fileName + ": Failed!");
        }
    }

    addHashesToResultValidation();

    Directory::getInstance()->addAllPermission(emlTempFolder);
}

void EmlFileManager::removePlainEmlFiles()
{
    if (m_directories == NULL)
    {
        return;
    }

    m_logger->Debug("Deleting EML and docs files directories because an error occurred during election closing.");

    clearTempResultsDirectory();
}

void EmlFileManager::persistX7SEmlFiles()
{
    std::string outputDirDom = m_directories->getEmlFolderFullPath(BasePathType::CONFIGURATION_DOM);
    std::string outputDirFlash = m_directories->getEmlFolderFullPath(BasePathType::CONFIGURATION_FLASH);
    std::map<std::string, Voting::Eml::EML>::iterator it;

    for (it = m_outputEmlFiles.begin(); it != m_outputEmlFiles.end(); ++it)
    {
        X7SEmlFileWrapper x7sEmlFileWrapper(m_electoralConfiguration, m_directories);
        std::string emlName = it->first;
        std::vector<std::string> x7sLangFiles = getX7SFilesFromEml(m_electoralConfiguration, it->first);
        std::vector<std::string>::iterator x7sIt;
        std::string pvreport = m_directories->getGeneratedPVReportFileFullPath(BasePathType::CONFIGURATION_DOM);

        pvreport = Path::GetFileName(pvreport);

        for (x7sIt = x7sLangFiles.begin(); x7sIt != x7sLangFiles.end(); ++x7sIt)
        {
            x7sEmlFileWrapper.persistX7SEmlFile(emlName, *x7sIt, pvreport);
        }
    }

    copyPlainEmlFiles();

    Directory::getInstance()->addAllPermission(outputDirDom);
    Directory::getInstance()->addAllPermission(outputDirFlash);
}

bool EmlFileManager::validate() const
{
    bool valid = true;
    std::map<std::string, std::string> emlFiles = EmlFileManager::getRelationFiles();
    std::map<std::string, std::string>::const_iterator it;

    if (!SAESConfigurationManager::getInstance()->getEmlFileConfiguration().getEnableEml()) return valid;

    for (it = emlFiles.begin(); it != emlFiles.end(); ++it)
    {
        std::string emlFileName = it->second;
        std::string domFolder   = m_directories->getEmlFolderFullPath(BasePathType::CONFIGURATION_DOM);
        std::string flashFolder = m_directories->getEmlFolderFullPath(BasePathType::CONFIGURATION_FLASH);

        if (SAESConfigurationManager::getInstance()->getEmlFileConfiguration().getPersistenceEml())
        {
            std::string domFile   = Path::Combine(domFolder, emlFileName);
            std::string flashFile = Path::Combine(flashFolder, emlFileName);

            if (!File::getInstance()->compareFiles(domFile, flashFile))
            {
                valid = false;
                break;
            }
        }

        if (SAESConfigurationManager::getInstance()->getEmlFileConfiguration().getPersistenceX7SEml())
        {
            std::vector<std::string> x7sFiles = getX7SFilesFromEml(m_electoralConfiguration, it->second);
            std::vector<std::string>::const_iterator x7sIt;

            for (x7sIt = x7sFiles.begin(); x7sIt != x7sFiles.end(); ++x7sIt)
            {
                std::string domFile   = Path::Combine(domFolder, *x7sIt);
                std::string flashFile = Path::Combine(flashFolder, *x7sIt);

                if (!File::getInstance()->compareFiles(domFile, flashFile))
                {
                    valid = false;
                    break;
                }
            }
        }
    }

    return valid;
}

void EmlFileManager::deleteFiles() const
{
    std::string domFolder   = m_directories->getEmlFolderFullPath(BasePathType::CONFIGURATION_DOM);
    std::string flashFolder = m_directories->getEmlFolderFullPath(BasePathType::CONFIGURATION_FLASH);

    Directory::getInstance()->Delete(domFolder);
    Directory::getInstance()->Delete(flashFolder);
}

std::string EmlFileManager::getMachineCode() const
{
    return m_machine;
}

std::string EmlFileManager::getCustomNisCode() const
{
    return m_customNisCode;
}

std::map<std::string, EML> EmlFileManager::getOutputEmlFiles() const
{
    return m_outputEmlFiles;
}

void EmlFileManager::reloadEmlFiles()
{
    m_emlFiles.clear();
    m_outputEmlFiles.clear();
    loadEmlFiles();
}

void EmlFileManager::buildOutputEml()
{
    std::map<std::string, EML>::iterator it;

    for (it = m_emlFiles.begin(); it != m_emlFiles.end(); ++it)
    {
        std::string emlOutName  = getOutputContestFileName(it->first);
        EmlWrapper emlWrapper(it->second);

        if (m_outputEmlFiles.find(emlOutName) == m_outputEmlFiles.end())
        {
            m_logger->Debug("New output file: " + emlOutName);
            m_outputEmlFiles[emlOutName] = it->second;
        }
        else
        {
            EmlWrapper finalEml(m_outputEmlFiles[emlOutName]);

            m_logger->Debug("Merge of EML files required for: " + emlOutName);
            finalEml.mergeEml(it->second);
        }
    }
}

void EmlFileManager::updateContestChecksum(const std::string& emlFilename, const long checksum)
{
    std::list<Voting::Election::Contest*>& contests = m_electoralConfiguration->getContestList();
    std::list<Voting::Election::Contest*>::const_iterator it;

    for (it = contests.begin(); it != contests.end(); ++it)
    {
        std::string customCode  = (*it)->getCustom_code();
        std::string filename = getOutputContestFileName(customCode);

        if (filename == emlFilename)
        {
            m_logger->Debug("Updating checksum for contest " + customCode + " (" + filename + ")");
            PBReports::PVContestWrapper::getInstance()->getContest(customCode)->setChecksum(checksum);
        }
    }
}

void EmlFileManager::addHashesToResultValidation() const
{
    std::map<std::string, EML>::const_iterator it;
    bool mustPersistPlain = SAESConfigurationManager::getInstance()->getEmlFileConfiguration().getPersistenceEml();
    bool mustPersistX7S   = SAESConfigurationManager::getInstance()->getEmlFileConfiguration().getPersistenceX7SEml();

    if (!mustPersistPlain && !mustPersistX7S) return;

    for (it = m_outputEmlFiles.begin(); it != m_outputEmlFiles.end(); ++it)
    {
        std::string domFolder   = m_directories->getEmlFolderFullPath(BasePathType::CONFIGURATION_DOM);
        std::string flashFolder = m_directories->getEmlFolderFullPath(BasePathType::CONFIGURATION_FLASH);
        std::string fileName    = it->first;
        std::string domFile     = Path::Combine(domFolder, fileName);
        std::string flashFile   = Path::Combine(flashFolder, fileName);

        SafeBuffer domHash;
        SafeBuffer flashHash;

        File::getInstance()->getHashFile(domFile, domHash);
        File::getInstance()->getHashFile(flashFile, flashHash);

        m_logger->Audit(fileName + " " + domHash.GetBase64Data());

        ResultValidation::getInstance()->addBuffer(fileName, domHash);
        ResultValidation::getInstance()->addBuffer(fileName, flashHash);
    }
}

void EmlFileManager::copyPlainEmlFiles() const
{
    const std::string tempFolder = m_directories->getEmlTempFolderFullPath();

    std::map<std::string, EML>::const_iterator       it  = m_outputEmlFiles.begin();
    const std::map<std::string, EML>::const_iterator end = m_outputEmlFiles.end();

    if (SAESConfigurationManager::getInstance()->getEmlFileConfiguration().getPersistenceEml())
    {
        for (; it != end; ++it)
        {
            const std::string domFolder   = m_directories->getEmlFolderFullPath(BasePathType::CONFIGURATION_DOM);
            const std::string flashFolder = m_directories->getEmlFolderFullPath(BasePathType::CONFIGURATION_FLASH);

            const std::string tempFile  = Path::Combine(tempFolder, it->first);
            const std::string domFile   = Path::Combine(domFolder, it->first);
            const std::string flashFile = Path::Combine(flashFolder, it->first);

            File::getInstance()->copyFile(tempFile,domFile);
            File::getInstance()->copyFile(tempFile, flashFile);
        }
    }

    Directory::getInstance()->Delete(tempFolder);
}

void EmlFileManager::clearTempResultsDirectory()
{
    ConfigurationElectionFilesPath& configElectionFiles(
        SAESCommonConfigurationManager::getInstance()->getConfiguration()->getConfigurationElectionFilesPath());

    const std::string tempFolder = m_directories->getEmlTempFolderFullPath();

    std::string outputDirDom   = m_directories->getEmlFolderFullPath(BasePathType::CONFIGURATION_DOM);
    std::string outputDirFlash = m_directories->getEmlFolderFullPath(BasePathType::CONFIGURATION_FLASH);
    std::string emlRelativeFolder = configElectionFiles.getEmlFolder();

    if (Path::GetDirectory(configElectionFiles.getEmlFolder()) != "")
    {
        outputDirDom = Path::GetDirectory(outputDirDom);
        outputDirFlash = Path::GetDirectory(outputDirFlash);

        emlRelativeFolder = Path::GetDirectory(emlRelativeFolder);
    }

    Directory::getInstance()->Delete(outputDirDom);
    Directory::getInstance()->Delete(outputDirFlash);
    Directory::getInstance()->Delete(tempFolder);
}

std::string EmlFileManager::getOutputNisCodeFromBooth(const std::string& contestCode) const
{
    std::string outputNisCode;

    VotingDevice::Voting_booths::Voting_boothSequence& votingBooths = \
        m_electoralConfiguration->getVotingDevice()->getVotingBooths().getVoting_booth();

    if (!votingBooths.empty())
    {
        VotingDevice::Voting_booth::Totalization_matchesType::Totalization_matchConstIterator it;
        VotingDevice::Voting_booth::Totalization_matchesType::Totalization_matchSequence& matches = \
            votingBooths.front().getTotalization_matches().getTotalization_match();

        for (it = matches.begin(); it != matches.end(); ++it)
        {
            if (it->getContest_custom_code() == contestCode)
            {
                outputNisCode = it->getOutput_nis_code();
                break;
            }
        }
    }

    return outputNisCode;
}

std::string EmlFileManager::getOutputShortCodeFromBooth(const std::string& contestCode) const
{
    std::string outputShortCode;

    VotingDevice::Voting_booths::Voting_boothSequence& votingBooths = \
        m_electoralConfiguration->getVotingDevice()->getVotingBooths().getVoting_booth();

    if (!votingBooths.empty())
    {
        VotingDevice::Voting_booth::Totalization_matchesType::Totalization_matchConstIterator it;
        VotingDevice::Voting_booth::Totalization_matchesType::Totalization_matchSequence& matches = \
            votingBooths.front().getTotalization_matches().getTotalization_match();

        for (it = matches.begin(); it != matches.end(); ++it)
        {
            if (it->getContest_custom_code() == contestCode)
            {
                outputShortCode = it->getOutput_short_code();
                break;
            }
        }
    }

    return outputShortCode;
}
