/**
 * @file   EmlFileManager.hxx
 * @brief  Declaration of EmlFileManager class.
 * @author manuel.arguelles
 * @date   Jan  8, 2018
 */

#ifndef EML_FILE_MANAGER_HXX
#define EML_FILE_MANAGER_HXX

#include <Voting/Eml/510.hxx>
#include <string>
#include <map>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Runtime
        {
            namespace Environment
            {
                class SaesDirectories;
            }
        }

        namespace Voting
        {
            class ElectoralConfiguration;

            namespace Vote
            {
                class Vote;
            }

            namespace Eml
            {
                /**
                 * @class  EmlFileManager
                 * @brief  Wrapper class for the EML files.
                 * @author manuel.arguelles
                 * @date   Jan  8, 2018
                 */
                class EmlFileManager
                {
                    public:
                        /**
                         * @brief  Create an instance of the class.
                         * @author manuel.arguelles
                         * @date   Jan  8, 2018
                         */
                        EmlFileManager();

                        /**
                         * @brief  Destroys an instance of the class.
                         * @author manuel.arguelles
                         * @date   Jan  8, 2018
                         */
                        ~EmlFileManager();

                    public:

                        /**
                         * @brief  Initialize the files.
                         * @author manuel.arguelles
                         * @date   Jan  8, 2018
                         */
                        void init(SAES::Voting::ElectoralConfiguration* electoralConfiguration,
                                  SAES::Runtime::Environment::SaesDirectories* directories);

                        /**
                         * @brief  Updates the EML counters with the vote.
                         * @author manuel.arguelles
                         * @date   Feb 28, 2018
                         * @param  vote The vote with the selections to update.
                         */
                        void updateCounters(const Voting::Vote::Vote& vote);

                        /**
                         * @brief  Add the totals to all the EML files.
                         * @author manuel.arguelles
                         * @date   Mar  2, 2018
                         * @param  totalsByType A map with the type and totals.
                         */
                        void setTotalsByType(const std::map<std::string, long>& totalsByType);

                        /**
                         * @brief  Gets the list of EML files and their hashes.
                         * @author manuel.arguelles
                         * @date   Mar  2, 2018
                         * @return A vector holding the EML file name and its hash.
                         */
                        std::vector<std::pair<std::string, std::string> > getEmlFileHashes() const;

                        /**
                         * @brief  Saves to disk the EML files in plain text format.
                         * @author manuel.arguelles
                         * @date   Mar  2, 2018
                         */
                        void persistPlainEmlFiles();

                        /**
                         * @brief  Removes all plain files in case of an error during election closing.
                         * @author eduardo.fernandez
                         * @date   Dec 28, 2018
                         */
                        void removePlainEmlFiles();

                        /**
                         * @brief  Saves to disk the EML files in X7S format.
                         * @author manuel.arguelles
                         * @date   Mar  2, 2018
                         * @throws XmlException        If the serialization operation failed.
                         * @throws FileSystemException If required files are missing.
                         * @throws CryptoException     If a crypto error condition occurs.
                         * @throws SignedException     If errors while signing occur.
                         */
                        void persistX7SEmlFiles();

                        /**
                         * @brief  Validates the persisted EML files.
                         * @author manuel.arguelles
                         * @date   Mar  2, 2018
                         * @return True if all the resulting EML are equal in both memories, false
                         * otherwise.
                         */
                        bool validate() const;

                        /**
                         * @brief  Deletes persisted EML files.
                         * @author manuel.arguelles
                         * @date   Mar  2, 2018
                         */
                        void deleteFiles() const;

                        /**
                         * @brief  Gets the machine code.
                         * @author manuel.arguelles
                         * @date   Mar  5, 2018
                         * @return The machine code.
                         */
                        std::string getMachineCode() const;

                        /**
                         * @brief  Gets the custom NIS code.
                         * @author manuel.arguelles
                         * @date   Mar  5, 2018
                         * @return The custom NIS code.
                         */
                        std::string getCustomNisCode() const;

                        /**
                         * @brief  Gets the output EML files.
                         * @author eduardo.fernandez
                         * @date   Nov 14, 2018
                         * @return The output EML files.
                         */
                        std::map<std::string, Voting::Eml::EML> getOutputEmlFiles() const;

                        /**
                         * @brief  Reloads the EML files.
                         * @author manuel.arguelles
                         * @date   Jun 24, 2022
                         */
                        void reloadEmlFiles();

                    private:

                        /**
                         * @brief  Gets the EML files
                         * @author manuel.arguelles
                         * @date   Jan  8, 2018
                         * @return A map with the input / output EML files.
                         */
                        std::map<std::string, std::string> getRelationFiles() const;

                        /**
                         * @brief  Gets the output contest file name.
                         * @author manuel.arguelles
                         * @date   Mar 13, 2018
                         * @param  customCode The custom code.
                         * @return The file name of the contests.
                         */
                        std::string getOutputContestFileName(const std::string& customCode) const;

                        /**
                         * @brief  Parse and prepare the input EML files.
                         * @author manuel.arguelles
                         * @date   Feb 28, 2018
                         * @throws FileSystemException If EML files can't be loaded.
                         */
                        void loadEmlFiles();

                        /**
                         * @brief  Checks if a ballot option is a list selection.
                         * @author manuel.arguelles
                         * @date   Jan 11, 2018
                         * @param  contestCode The code of the contest.
                         * @param  ballotCode  The code of the ballot option.
                         * @return True if it is a list selection, false if not.
                         */
                        bool isListSelection(const std::string& contestCode, const std::string& ballotCode);

                        /**
                         * @brief  Checks if a ballot option is a principal selection.
                         * @author manuel.arguelles
                         * @date   Jan 11, 2018
                         * @param  contestCode The code of the contest.
                         * @param  ballotCode  The code of the ballot option.
                         * @return True if it is a principal selection, false if not.
                         */
                        bool isPrincipalSelection(const std::string& contestCode, const std::string& ballotCode);

                        /**
                         * @brief  Checks if a ballot option is a stand in selection.
                         * @author manuel.arguelles
                         * @date   Jan 11, 2018
                         * @param  contestCode The code of the contest.
                         * @param  ballotCode  The code of the ballot option.
                         * @return True if it is a stand in selection, false if not.
                         */
                        bool isStandInSelection(const std::string& contestCode, const std::string& ballotCode);

                        /**
                         * @brief  Generates the output EMLs from the input ones.
                         * @author manuel.arguelles
                         * @date   Mar  2, 2018
                         */
                        void buildOutputEml();

                        /**
                         * @brief  Sets the checksum in the global Contest object.
                         * @author manuel.arguelles
                         * @date   Mar  2, 2018
                         * @param  emlFilename The file name of the EML.
                         * @param  checksum    The checksum of such EML file.
                         */
                        void updateContestChecksum(const std::string& emlFilename, const long checksum);

                        /**
                         * @brief  Adds the hashes to the ResultValidation object.
                         * @author manuel.arguelles
                         * @date   Mar  2, 2018
                         */
                        void addHashesToResultValidation() const;

                        /**
                         * @brief  Copies plain text EML files if required.
                         * @author eduardo.fernandez
                         * @date   Jan 3, 2019
                         */
                        void copyPlainEmlFiles() const;

                        /**
                         * @brief  Clears plain text EML files.
                         * @author eduardo.fernandez
                         * @date   Nov 16, 2018
                         */
                        void clearTempResultsDirectory();

                        /**
                         * @brief  Get the output NIS code from the voting booth.
                         * @author manuel.arguelles
                         * @date   Jun 12, 2018
                         * @param  contestCode The contest code.
                         * @return The output NIS code.
                         */
                        std::string getOutputNisCodeFromBooth(const std::string& contestCode) const;

                        /**
                         * @brief  Get the output short code from the voting booth.
                         * @author manuel.arguelles
                         * @date   Jun 12, 2018
                         * @param contestCode The contest code.
                         * @return The output short code.
                         */
                        std::string getOutputShortCodeFromBooth(const std::string& contestCode) const;

                        /**
                         * @brief  Gets the EML contest code short code.
                         * @author manuel.arguelles
                         * @date   Feb 16, 2018
                         * @param  fileName The file name of the EML.
                         * @return The contests code (K, M, I, X)
                         */
                        std::string getEmlContestCode(const std::string& fileName) const;

                    private:

                        SAES::Runtime::Environment::SaesDirectories* m_directories;
                        SAES::Voting::ElectoralConfiguration*        m_electoralConfiguration;
                        std::string                                  m_machine;
                        std::string                                  m_customNisCode;
                        std::string                                  m_region;
                        std::map<std::string, Voting::Eml::EML>      m_emlFiles;
                        std::map<std::string, Voting::Eml::EML>      m_outputEmlFiles;
                        static Smartmatic::Log::ISMTTLog*            m_logger;
                };
            }
        }
    }
}

#endif /* EML_FILE_MANAGER_HXX */
