/**
 * @file   EmlInfo.hxx
 * @brief  Declaration of EML information functions.
 * @author manuel.arguelles
 * @date   Jan  5, 2018
 */

#ifndef EML_INFO_HXX
#define EML_INFO_HXX

#include <string>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Voting
        {
            namespace Eml
            {
                namespace EmlInfo
                {
                    /**
                     * @brief  Get the election type.
                     * @author manuel.arguelles
                     * @date   Jan  5, 2018
                     * @param  customCode The custom code.
                     * @return The election type of the custom code.
                     */
                    std::string getElectionType(const std::string& customCode);

                    /**
                     * @brief  Get the input NIS code.
                     * @author manuel.arguelles
                     * @date   Jan  5, 2018
                     * @param  customCode The custom code.
                     * @return The input NIS code of the custom code.
                     */
                    std::string getInputNisCode(const std::string& customCode);

                    /**
                     * @brief  Get the output NIS code.
                     * @author manuel.arguelles
                     * @date   Jan  5, 2018
                     * @param  customCode    The custom code.
                     * @param  customNisCode The custom NIS code.
                     * @return The output NIS code from the custom and NIS code.
                     */
                    std::string getOutputNisCode(const std::string& customCode,
                                                 const std::string& customNisCode);

                    /**
                     * @brief  Get the language.
                     * @author manuel.arguelles
                     * @date   Jan  5, 2018
                     * @param  customCode The custom code.
                     * @return The language of the custom code.
                     */
                    std::string getLanguage(const std::string& customCode);

                    /**
                     * @brief  Get the category code for the total votes selection.
                     * @author manuel.arguelles
                     * @date   Jan 19, 2018
                     * @param  customCode The custom code.
                     * @return The selection category (niscode + language code).
                     */
                    std::string getTotalVotesSelectionCategory(const std::string& customCode);

                    /**
                     * @brief  Gets the numeric linguistic regime of the custom code.
                     * @author manuel.arguelles
                     * @date   Feb 10, 2018
                     * @param  customCode The custom code.
                     * @return The linguistic regime number.
                     */
                    int getLinguisticRegime(const std::string& customCode);

                    /**
                     * @brief  Gets the event qualifier of the EML filename.
                     * @author manuel.arguelles
                     * @date   Feb 14, 2018
                     * @param  emlFilename The filename of the EML.
                     * @return The event qualifier.
                     */
                    std::string getEventQualifier(const std::string& emlFilename);

                    /**
                     * @brief  Gets the election identifier id.
                     * @author manuel.arguelles
                     * @date   Feb 14, 2018
                     * @param  customCode The custom code.
                     * @return The election identifier id.
                     */
                    std::string getElectionIdentifierId(const std::string& customCode);

                    /**
                     * @brief  Gets election identifier name.
                     * @author manuel.arguelles
                     * @date   Feb 14, 2018
                     * @param  customCode The custom code.
                     * @return The election identifier name.
                     */
                    std::string getElectionIdentifierName(const std::string &customCode);

                    /**
                     * @brief  Gets the NIS code from the EML output filename.
                     * @author manuel.arguelles
                     * @date   Feb 16, 2018
                     * @param  outputFilename The output EML filename.
                     * @return The NIS code.
                     */
                    std::string getOutputNisCode(const std::string& outputFilename);

                } /* namespace EmlInfo */
            } /* namespace Eml */
        } /* namespace Voting */
    } /* namespace SAES */
} /* namespace Smartmatic */

#endif /* EML_INFO_HXX */
