/**
 * @file	OperationStatus.cxx
 * @brief	operation status class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */


#include "OperationStatus.hxx"
#include <System/IO/File.h>
#include <System/IO/Path.h>
#include <System/IO/Directory.h>
#include <System/IO/FatFileManager.hxx>
#include <Log/SMTTLogManager.h>
#include <Runtime/Environment/DirectoryManager.hxx>
#include <System/Utils/StringUtils.hxx>
#include <Configuration/s-a-e-s-conf-manager.h>
#include <System/Runtime/ProcessLauncher.h>
#include <System/Utils/DateTimeUtils.hxx>
#include <System/Compression/ZipCompression.hxx>
#include <System/Exception/FileSystemException.hxx>
#include <Configuration/DiagnosticLogConfigurationManager.hxx>
#include <Configuration/SaesCommonConfiguration-schema.hxx>
#include <Configuration/s-a-e-s-common-conf-manager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::SAES::Voting::Status;
using namespace Smartmatic::System;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::System::Runtime;
using namespace Smartmatic::System::Compression;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::System::Exception;

ISMTTLog * OperationStatus::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Voting.OperationStatus", GETTEXT_PACKAGE );

/**
 * @fn static bool isLiveFileSystemBasePath(const std::string &path)
 * @brief Checks whether given path is a live file system base path.
 * @param path The path to check.
 * @return Whether given path is a live file system base path.
 */
static bool isLiveFileSystemBasePath(const std::string &path)
{
    ConfigurationElectionFilesPath & configElectionFiles ( SAESCommonConfigurationManager::getInstance()->getConfiguration()->getConfigurationElectionFilesPath() );
    ConfigurationBasePath configBasePath = configElectionFiles.getConfigBasePath();

    return path == configBasePath.getLiveFileSystemBasePath();
}

OperationStatus::OperationStatus ( Smartmatic::SAES::Runtime::Environment::SaesDirectories * saesDirectories, std::list<std::string> runFunctionalityDescriptors )
    : vmStatus ( NULL ), electionStatus ( NULL ), saesDirectories ( saesDirectories )
{
    ElectionStatus base;
    std::string now = DateTimeUtils::getCurrentDateTime();
    std::string openCode = GuidClass::NewGuid().ToString();

    base.setState ( Status::ProcessStatus::ELECTION_STATUS_NOTINSTALLED );
    base.setOpeningCode ( openCode );
    base.setVotersRegisteredInBook ( 0 );
    base.setOperationModeCode ( "0" );
    base.setElectionConfigurationReportPrinted ( false );
    base.setOpenElectionDate ( now );
    base.setCloseVotingDate ( now );
    base.setCloseElectionDate ( now );
    base.setSynchronizeVotes ( false );

    electionStatus = new ElectionStatusWrapper ( base );
    vmStatus = new VotingMachineStatusWrapper ( runFunctionalityDescriptors );
}

OperationStatus::~OperationStatus()
{
    dispose();
}

void OperationStatus::initialize () throw ( CryptoException, SignedException, XmlException )
{
    Glib::Mutex::Lock lock ( mutex );

    logger->Debug ( "initialize" );

    std::string fileDom = saesDirectories->getElectionStatusFileFullPath ( BasePathType::CONFIGURATION_DOM );
    std::string fileFlash = saesDirectories->getElectionStatusFileFullPath ( BasePathType::CONFIGURATION_FLASH );

    std::auto_ptr<ElectionStatusWrapper> statusDOM(loadElectionStatusFile(fileDom));
    std::auto_ptr<ElectionStatusWrapper> statusFlash(loadElectionStatusFile(fileFlash));

    if ( statusDOM.get() == NULL && statusFlash.get() == NULL )
    {
        throw XmlException ( _ ( "Smartmatic.SAES.Voting.OperationStatus.ErrorSerializaData" ), N_ ( "Smartmatic.SAES.Voting.OperationStatus.ErrorSerializaData" ), logger );
    }

    if ( statusDOM.get() != NULL && statusFlash.get() == NULL )
    {
        logger->Debug ( "Load DOM" );
        delete electionStatus;
        electionStatus = statusDOM.release();
    }
    else if ( statusDOM.get() == NULL && statusFlash.get() != NULL )
    {
        logger->Debug ( "Load Flash" );
        delete electionStatus;
        electionStatus = statusFlash.release();
    }
    else if ( statusDOM.get() != NULL && statusFlash.get() != NULL )
    {
        if ( statusDOM->getSynchronizeVotes() == true )
        {
            logger->Debug ( "Synchronize votes in DOM" );
            delete electionStatus;
            electionStatus = statusDOM.release();
        }
        else if ( statusFlash->getSynchronizeVotes() == true )
        {
            logger->Debug ( "Synchronize votes in Flash" );
            delete electionStatus;
            electionStatus = statusFlash.release();
        }
        else if ( statusDOM->getEletionState() >= statusFlash->getEletionState() )
        {
            logger->Debug ( "Load current DOM" );
            delete electionStatus;
            electionStatus = statusDOM.release();
        }
        else
        {
            logger->Debug ( "Load current FLASH" );
            delete electionStatus;
            electionStatus = statusFlash.release();
        }
    }
}

void OperationStatus::dispose ()
{
    Glib::Mutex::Lock lock ( mutex );

    if ( electionStatus != NULL )
    {
        delete electionStatus;
        electionStatus = NULL;
    }

    if ( vmStatus != NULL )
    {
        delete vmStatus;
        vmStatus = NULL;
    }
}

ElectionStatusWrapper * OperationStatus::getElectionStatusWrapper()
{
    Glib::Mutex::Lock lock ( mutex );
    return electionStatus;
}

ProcessStatus OperationStatus::getElectionState()
{
    Glib::Mutex::Lock lock ( mutex );
    return electionStatus->getEletionState();
}

VotingMachineStatusWrapper * OperationStatus::getVotingMachineStatusWrapper()
{
    Glib::Mutex::Lock lock ( mutex );
    return vmStatus;
}

ElectionStatusWrapper * OperationStatus::loadElectionStatusFile ( std::string file )
{
    ElectionStatusWrapper * status = NULL;

    if ( File::getInstance()->canOpenFile ( file ) )
    {
        try
        {
            std::string data = Smartmatic::SAES::Security::Encryption::getInstance()->openXmlFile ( file, Smartmatic::SAES::Security::Encryption::INTERNAL_FILE );
            std::istringstream istr ( data );
            status = new ElectionStatusWrapper ( * ( parseElectionStatus ( istr, ElectionStatusTypes::Flags::dont_validate ) ) );
        }
        catch ( const ::xsd::cxx::tree::exception< char > & e )
        {
            std::stringstream ss;
            ss << "FATAL - " << __func__ << " " << e;
            logger->Debug ( ss.str() );
        }
        catch ( const std::exception & e )
        {
            std::stringstream ss;
            ss << "FATAL - " << __func__ << " " << e.what();
            logger->Debug ( ss.str() );
        }
    }

    return status;
}

void  OperationStatus::persistElectionStatus ( bool createBackup ) throw ( CryptoException, SignedException, XmlException )
{
    Glib::Mutex::Lock lock ( mutex );

    logger->Debug ( "Persist Election Status" );

    std::ostringstream os;
    std::string fileDom = saesDirectories->getElectionStatusFileFullPath ( BasePathType::CONFIGURATION_DOM );
    std::string fileFlash = saesDirectories->getElectionStatusFileFullPath ( BasePathType::CONFIGURATION_FLASH );
    std::string fileDomTemp = fileDom + ".tmp";
    std::string fileFlashTemp = fileFlash + ".tmp";

    Smartmatic::SAES::Voting::ElectionStatusTypes::NamespaceInfomap map;
    map[""].name = NAMESPACE_VOTING_STATUS;
    map[""].schema = "";
    serializeElectionStatus ( os, electionStatus->getElectionStatus(), map, "UTF-8" );

    std::string data = os.str();
    std::string devicePathDom = saesDirectories->getDevicePath ( BasePathType::CONFIGURATION_DOM );
    std::string devicePathFlash = saesDirectories->getDevicePath ( BasePathType::CONFIGURATION_FLASH );

    if ( createBackup )
    {
        if ( Directory::getInstance()->DirectoryExists ( devicePathDom ) )
        {
            Smartmatic::SAES::Security::Encryption::getInstance()->saveXmlFile ( data, fileDomTemp,
                    Smartmatic::SAES::Security::Encryption::INTERNAL_FILE );
            File::getInstance()->copyFile ( fileDomTemp, fileFlashTemp );
            File::getInstance()->renameFile ( fileDomTemp, fileDom );
            File::getInstance()->renameFile ( fileFlashTemp, fileFlash );
        }
        else if ( Directory::getInstance()->DirectoryExists ( devicePathFlash ) )
        {
            Smartmatic::SAES::Security::Encryption::getInstance()->saveXmlFile ( data, fileFlashTemp,
                    Smartmatic::SAES::Security::Encryption::INTERNAL_FILE );
            File::getInstance()->copyFile ( fileFlashTemp, fileDomTemp );
            File::getInstance()->renameFile ( fileDomTemp, fileDom );
            File::getInstance()->renameFile ( fileFlashTemp, fileFlash );
        }
    }
    else
    {
        if ( Directory::getInstance()->FileExists ( fileDom ) )
        {
            Smartmatic::SAES::Security::Encryption::getInstance()->saveXmlFile ( data, fileDomTemp,
                    Smartmatic::SAES::Security::Encryption::INTERNAL_FILE );
            File::getInstance()->copyFile ( fileDomTemp, fileFlashTemp );
            File::getInstance()->renameFile ( fileDomTemp, fileDom );
            File::getInstance()->renameFile ( fileFlashTemp, fileFlash );
        }
        else if ( Directory::getInstance()->FileExists ( fileFlash ) )
        {
            Smartmatic::SAES::Security::Encryption::getInstance()->saveXmlFile ( data, fileFlashTemp,
                    Smartmatic::SAES::Security::Encryption::INTERNAL_FILE );
            File::getInstance()->copyFile ( fileFlashTemp, fileDomTemp );
            File::getInstance()->renameFile ( fileDomTemp, fileDom );
            File::getInstance()->renameFile ( fileFlashTemp, fileFlash );
        }
    }

    ProcessLauncher::getInstance()->RunProcessThroughSystem ( "sync" );
}

bool OperationStatus::persistDiagnosticFile(const bool able20IsConnected)
{
    bool success = true;

    if ( DiagnosticLogConfigurationManager::getInstance()->shouldPersistDiagnosticLogFile() )
    {
        try
        {
            std::string result;
            std::string file;
            FatFileManager fatFiles;
            bool persisted = false;
            fatFiles.addFatDevices();
            getDiagnosticResult ( result, file, able20IsConnected );
            logger->Debug ( "Save file " + file );
            persistResultToFile ( result, file, BasePathType::CONFIGURATION_DOM, persisted );

            if ( saesDirectories->getDevicePath ( BasePathType::CONFIGURATION_FLASH ).empty() == false )
            {
                persistResultToFile ( result, file, BasePathType::CONFIGURATION_FLASH, persisted );
            }

            persistResultToFile ( result, file, BasePathType::CONFIGURATION_BACKUP, persisted );
            std::string  resultFilesInDOM   = saesDirectories->getDiagnosticResultFolderFullPath(BasePathType::CONFIGURATION_DOM);
            fatFiles.copyFiles(resultFilesInDOM, Path::GetDirectory(saesDirectories->getDiagnosticResultFolderFullPath(BasePathType::CONFIGURATION_DOM)));
        }
        catch ( FileSystemException &fse )
        {
            logger->Error(fse.getMessage());
            success = false;
        }
        catch ( ... )
        {
            success = false;
        }
    }

    return success;
}

void OperationStatus::getDiagnosticResult ( std::string & result, std::string & file, const bool able20IsConnected )
{
    std::string serialDetected = getVotingMachineStatusWrapper()->getSerialDetected();
    std::string serial = getVotingMachineStatusWrapper()->getSerial();

    std::string separator = DiagnosticLogConfigurationManager::getInstance()->getSeparator();
    std::string ok = DiagnosticLogConfigurationManager::getInstance()->getStringOk();
    std::string nok = DiagnosticLogConfigurationManager::getInstance()->getStringNotOk();
    std::string na = DiagnosticLogConfigurationManager::getInstance()->getStringNA();

    result.clear();
    result.append ( getVotingMachineStatusWrapper()->getVersion ( separator, ":" ) );
    result.append ( getVotingMachineStatusWrapper()->getResult ( separator, ":", ok, nok, na, able20IsConnected ) );
    file = ( serial.empty() ? serialDetected : serial );

    if ( file.empty() )
    {
        file = "SERIAL_EMPTY";
    }
}

void OperationStatus::persistResultToFile ( std::string & result, std::string & file, BasePathType type, bool &persisted )
{
    std::string device = saesDirectories->getDevicePath ( type );

    if ( !device.empty() && Directory::getInstance()->DirectoryExists ( device ) )
    {
        std::string folder = saesDirectories->getDiagnosticResultFolderFullPath ( type );

        if ( !Directory::getInstance()->DirectoryExists ( folder ) )
        {
            Directory::getInstance()->Mkdir ( folder );
        }

        std::string fileFullPath = Path::Combine ( folder, file );
        SafeBuffer buffer ( result );
        File::getInstance()->saveBufferToFile ( buffer, fileFullPath );

        Directory::getInstance()->addAllPermission ( folder );

        if (isLiveFileSystemBasePath(device) == false)
        {
            persisted = true;
        }
    }
    else if (persisted == false)
    {
        const std::string errorPersistingMessage = Glib::ustring::compose(_("Smartmatic.SAES.Voting.OperationStatus.DirectoryNotFound"), device, file);
        throw Smartmatic::System::Exception::FileSystemException(
                errorPersistingMessage, N_("Smartmatic.SAES.Voting.OperationStatus.ErrorPersistingResults"), logger);
    }
}

SafeBuffer OperationStatus::getQRDiagnosticResult ( bool process )
{
    std::string data, file;
    getDiagnosticResult ( data, file, false );

    SafeBuffer buffer ( data );
    SafeBuffer bufferOut;

    if ( process )
    {
        ZipCompression zip;
        SafeBuffer deflate;
        zip.Deflate ( buffer, deflate );
        bufferOut.AddData ( deflate.GetBase64Data() );
    }
    else
    {
        bufferOut.AddData ( buffer );
    }

    return bufferOut;
}
