/**
 * @file	OperationStatus.hxx
 * @brief	Declares the operation status class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */

#ifndef _OPERATION_STATUS_H_
#define _OPERATION_STATUS_H_

#include <string>
#include <Voting/Status/ElectionStatusWrapper.hxx>
#include <Voting/Status/VMStatusWrapper.hxx>
#include <Runtime/Environment/saes-directories.h>
#include <System/Exception/CryptoException.hxx>
#include <System/Exception/SignedException.hxx>
#include <System/Exception/XmlException.hxx>
#include <Log/ISMTTLog.h>
#include <glibmm.h>
#include <System/SafeBuffer.h>
#include <list>
#include <Runtime/Environment/saes-directories.h>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Voting
        {
            /**
             * @class	OperationStatus
             * @brief	Operation status.
             * @author	Eduardo.campione
             * @date	28/04/2011
             */

            class OperationStatus
            {
                public:

                    /**
                     * @fn	OperationStatus::OperationStatus( Smartmatic::SAES::Runtime::Environment::SaesDirectories * saesDirectories );
                     * @brief	Default constructor.
                     * @author	Yadickson Soto
                     * @date	28/04/2014
                     * @param	saesDirectories the saes directories
                     * @param   runFunctionalityDescriptors run functionality descriptors.
                     */
                    explicit OperationStatus ( Smartmatic::SAES::Runtime::Environment::SaesDirectories * saesDirectories, std::list<std::string> runFunctionalityDescriptors );

                    /**
                     * @fn	virtual ~OperationStatus();
                     * @brief	Finaliser.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     */
                    virtual ~OperationStatus();

                    /**
                     * @fn	void OperationStatus::initialize() throw (Smartmatic::System::Exception::CryptoException,
                     * 			Smartmatic::System::Exception::SignedException, Smartmatic::System::Exception::XmlException);
                     * @brief	Initialises this object.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition occurs.
                     * @exception	Smartmatic::System::Exception::SignedException	Thrown when a signed error condition occurs.
                     * @exception	Smartmatic::System::Exception::XmlException			Thrown when an XML error condition occurs.
                     */
                    void initialize()
                    throw ( Smartmatic::System::Exception::CryptoException,
                            Smartmatic::System::Exception::SignedException,
                            Smartmatic::System::Exception::XmlException );

                    /**
                     * @fn	void OperationStatus::dispose();
                     * @brief	Dispose of this object, cleaning up any resources it uses.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     */
                    void dispose();

                    /**
                     * @fn	Smartmatic::SAES::Voting::Status::ElectionStatusWrapper * OperationStatus::getElectionStatusWrapper();
                     * @brief	Gets the election status wrapper.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     * @return	The election status wrapper.
                     */

                    Smartmatic::SAES::Voting::Status::ElectionStatusWrapper * getElectionStatusWrapper();

                    /**
                     * @fn     Smartmatic::SAES::Voting::Status::ProcessStatus getElectionState();
                     * @brief  Gets the election state.
                     * @author manuel.arguelles
                     * @date   20/07/2022
                     * @return The election state.
                     */
                    Smartmatic::SAES::Voting::Status::ProcessStatus getElectionState();

                    /**
                     * @fn	Smartmatic::SAES::Voting::Status::VotingMachineStatusWrapper * OperationStatus::getVotingMachineStatusWrapper();
                     * @brief	Gets the voting machine status wrapper.
                     * @author	Eduardo.campione
                     * @date	28/04/2011
                     * @return	The voting machine status wrapper.
                     */

                    Smartmatic::SAES::Voting::Status::VotingMachineStatusWrapper * getVotingMachineStatusWrapper();

                    /**
                     * @fn	void OperationStatus::persistElectionStatus( bool createBackup = true ) throw (Smartmatic::System::Exception::CryptoException,
                     * 			Smartmatic::System::Exception::SignedException, Smartmatic::System::Exception::XmlException);
                     * @brief	Persist election status.
                     * @author	Yadickson Soto
                     * @date	28/04/2011
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::SignedException	Thrown when a signed error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::XmlException			Thrown when an XML error condition
                     * 																														occurs.
                     * @param	createBackup	 	create backup file.
                     */
                    void persistElectionStatus ( bool createBackup = true )
                    throw ( Smartmatic::System::Exception::CryptoException,
                            Smartmatic::System::Exception::SignedException,
                            Smartmatic::System::Exception::XmlException );

                    /**
                     * @fn	void OperationStatus::persistDiagnosticFile();
                     * @brief	Persist diagnostic file.
                     * @author	Yadickson Soto
                     * @date	19/09/2012
                     * @param   able20IsConnected Whether the able 2.0 is connected.
                     */
                    bool persistDiagnosticFile(const bool able20IsConnected);

                    /**
                     * @fn	void OperationStatus::getDiagnosticResult(std::string & result, std::string & file);
                     * @brief	method getter result and file.
                     * @author	Yadickson Soto
                     * @date	20/09/2012
                     * @param	result	result data.
                     * @param	file		file of data.
                     * @param   able20IsConnected Whether the able 2.0 is connected.
                     */
                    void getDiagnosticResult ( std::string & result, std::string & file, const bool able20IsConnected );

                    /**
                     * @fn	Smartmatic::System::SafeBuffer getQRDiagnosticResult ( bool process );
                     * @brief	Optionally activate gzip + base64encode of the QRcode in the Diagnostic Only if process is true
                     * @author	Yadickson Soto
                     * @date	28/01/2014
                     * @param 	process 	activate gzip + base64encode of the buffer
                     * @return	Result QR buffer
                     */
                    Smartmatic::System::SafeBuffer getQRDiagnosticResult ( bool process );

                private:

                    /**
                     * @brief  Disable copy constructor.
                     * @author manuel.arguelles
                     * @date   Sep 12, 2017
                     */
                    OperationStatus(const OperationStatus&);

                    /**
                     * @brief  Disable copy.
                     * @author manuel.arguelles
                     * @date   Sep 12, 2017
                     */
                    OperationStatus& operator=(const OperationStatus&);

                    /**
                     * @fn	bool OperationStatus::persistResultToFile(std::string & result, std::string & file,
                     * 			Smartmatic::SAES::Common::Configuration::BasePathType type);
                     * @brief	Saves a result string into a file.
                     * @author	Yadickson Soto
                     * @date	20/09/2012
                     * @param	result	result data.
                     * @param	file		filename in which to save the result.
                     * @param	type		folder type.
                     * @param   persisted   whether the file was already persisted.
                     */
                    void persistResultToFile ( std::string & result, std::string & file, Smartmatic::SAES::Common::Configuration::BasePathType type, bool &persisted );

                    /**
                     * @fn	Smartmatic::SAES::Voting::Status::ElectionStatusWrapper * OperationStatus::loadElectionStatusFile(std::string file);
                     * @brief	Loads election status file.
                     * @author	Youssef.tourkmani
                     * @date	31/10/2012
                     * @param	file	The file.
                     * @return	null if it fails, else the election status file.
                     */
                    Smartmatic::SAES::Voting::Status::ElectionStatusWrapper * loadElectionStatusFile ( std::string file );

                    Smartmatic::SAES::Voting::Status::VotingMachineStatusWrapper * vmStatus; ///< The voting machine status
                    Smartmatic::SAES::Voting::Status::ElectionStatusWrapper * electionStatus;	///< The election status
                    Smartmatic::SAES::Runtime::Environment::SaesDirectories * saesDirectories; ///< the saes directories

                    Glib::Mutex mutex;	///< The mutex
                    static Smartmatic::Log::ISMTTLog * logger; ///< logger
            };
        }
    }
}

#endif // _OPERATION_STATUS_H_
