/**
 * @file	PVFormWrapper.cxx
 * @brief	pv form wrapper class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */


#include "PVFormWrapper.hxx"
#include "VoteStatisticsWrapper.hxx"
#include "Resources/election-string-resources-manager.h"
#include "VoteResultsWrapper.hxx"
#include "Voting/PBReports/pvform-schema.hxx"
#include "activated-card-statistics-wrapper.h"
#include <Configuration/s-a-e-s-common-conf-manager.h>
#include <Configuration/s-a-e-s-conf-manager.h>
#include <Environment/Configurations/base-language-controller.h>
#include <Log/SMTTLogManager.h>
#include <Runtime/Environment/DirectoryManager.hxx>
#include <Runtime/Environment/saes-directories.h>
#include <Security/CertificateStore.h>
#include <Security/XmlFileSignature.hxx>
#include <System/IO/File.h>
#include <System/IO/Path.h>
#include <System/Security/XmlDigitalSignature/XmlDetachedSignature.hxx>
#include <System/SafeBuffer.h>
#include <System/Security/Cryptography/CertificateExtensionBase.hxx>
#include <System/Security/Cryptography/CertUtil.h>
#include <Security/Encryption.hxx>
#include <System/Utils/DateTimeUtils.hxx>
#include <System/XSLFO/XSLFOTransformer.h>
#include <Voting/PBReports/PVContestWrapper.hxx>
#include <Voting/ElectoralConfiguration.hxx>
#include <Voting/Eml/EmlFileManager.hxx>
#include <Voting/OperationStatus.hxx>
#include <Voting/Vote/StationTallyWrapper.hxx>
#include <Voting/Vote/TransmissionPackageWrapper.hxx>
#include <Voting/Vote/VoteWrapper.hxx>
#include <Voting/set-language-functionality.h>
#include <set>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Environment;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::SAES::Security;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::SAES::Voting::PBReports;
using namespace Smartmatic::System;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::Security;
using namespace Smartmatic::System::Security::Cryptography;
using namespace Smartmatic::System::Utils;

ISMTTLog * PVFormWrapper::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Voting.PBReports.PVFormWrapper", GETTEXT_PACKAGE );

PVFormWrapper::PVFormWrapper()
{
    this->transformer = NULL;
    this->electoralConfiguration = NULL;
    this->operationStatus = NULL;
    this->activatedCardStatisticsWrapper = NULL;
    this->voteStatisticsWrapper = NULL;
    this->directories = NULL;
    this->stationTallyWrapper = NULL;
    this->transmissionPackageWrapper = NULL;
    this->emlFiles = NULL;
}

PVFormWrapper::~PVFormWrapper()
{
    dispose();
}

void PVFormWrapper::dispose()
{
    this->fileFullPath = "";
}

void PVFormWrapper::init ( SAES::Runtime::Environment::SaesDirectories * directories,
                           ElectoralConfiguration * electoralConfiguration,
                           OperationStatus * operationStatus,
                           ActivatedCardStatisticsWrapper * activatedCardStatisticsWrapper,
                           VoteStatisticsWrapper * voteStatisticsWrapper,
                           Voting::Vote::StationTallyWrapper * stationTallyWrapper,
                           Voting::Vote::TransmissionPackageWrapper * transmissionPackageWrapper,
                           Voting::Eml::EmlFileManager * emlFiles )
{
    this->directories = directories;
    this->electoralConfiguration = electoralConfiguration;
    this->operationStatus = operationStatus;
    this->activatedCardStatisticsWrapper = activatedCardStatisticsWrapper;
    this->voteStatisticsWrapper = voteStatisticsWrapper;
    this->stationTallyWrapper = stationTallyWrapper;
    this->transmissionPackageWrapper = transmissionPackageWrapper;
    this->emlFiles = emlFiles;

    dispose();
}

bool PVFormWrapper::persistencePVForm (	Smartmatic::System::XSLFO::XSLFOTransformer * transformer )
{
    this->transformer = transformer;

    std::string pvformFileDom = directories->getPVFormFileFullPath ( BasePathType::CONFIGURATION_DOM );
    std::string pvformFileFlash = directories->getPVFormFileFullPath ( BasePathType::CONFIGURATION_FLASH );
    std::string pvreportFileDom = directories->getGeneratedPVReportFileFullPath ( BasePathType::CONFIGURATION_DOM );
    std::string pvreportFileFlash = directories->getGeneratedPVReportFileFullPath ( BasePathType::CONFIGURATION_FLASH );
    std::string pvtemplateFile = directories->getGlobalPVTemplateFileFullPath();

    bool validate = persistence ( pvformFileDom, pvformFileFlash,
                                  pvreportFileDom, pvreportFileFlash,
                                  pvtemplateFile );
    std::string pvreportTestFileDom = directories->getGeneratedPVReportTestFullPath ( BasePathType::CONFIGURATION_DOM );
    std::string pvreportTestFileFlash = directories->getGeneratedPVReportTestFullPath ( BasePathType::CONFIGURATION_FLASH );
    std::string pvtemplateTestFile = directories->getTestPVTemplateFileFullPath();

    validate = persistence ( pvformFileDom, pvformFileFlash,
                             pvreportTestFileDom,
                             pvreportTestFileFlash,
                             pvtemplateTestFile ) || validate;
    return validate;
}

bool PVFormWrapper::persistence ( std::string & pvformFileDom,
                                  std::string & pvformFileFlash,
                                  std::string & pvreportFileDom,
                                  std::string & pvreportFileFlash,
                                  std::string & pvtemplateFile )
{
    bool validate = true;
    std::string actualLanguage;
    std::string actualCountry;
    std::string currentLanguage;
    currentLanguage = BaseLanguageController::getActualLanguage();
    SetLanguageFunctionality::getCurrentLanguageCountry ( actualLanguage, actualCountry );
    std::list <std::string> languages = getLanguages();
    std::list <std::string>::iterator it;

    PVForms::LanguageSequence pvLanguages;

    for ( it = languages.begin();
            it != languages.end();
            ++it )
    {
        PVLanguage language;
        language.setCode(*it);

        std::string lang = ( *it );
        logger->Debug ( "Generate pv-report in language " + lang );

        if ( lang.empty() )
        {
            BaseLanguageController::FullLocalizeApplication ( currentLanguage );
            SetLanguageFunctionality::setNewVotingLanguageByLanguageCountry ( actualLanguage, actualCountry );
        }
        else
        {
            BaseLanguageController::FullLocalizeApplication ( lang );
            std::string aLand, aCountry;
            BaseLanguageController::GetActualLanguage ( aLand, aCountry );
            SetLanguageFunctionality::setNewVotingLanguageByLanguageCountry ( aLand, aCountry );
        }

        loadPVForm( language );

        pvLanguages.push_back( language );
    }

    PVForms pvForms;
    pvForms.setLanguage( pvLanguages );

    SetLanguageFunctionality::setNewVotingLanguageByLanguageCountry ( actualLanguage, actualCountry );

    std::ostringstream os;

    try
    {
        Smartmatic::SAES::Voting::PBReports::NamespaceInfomap map;
        map[""].name = NAMESPACE_PBREPORTS;
        map[""].schema = "";
        serializePVForms ( os, pvForms, map );
    }
    catch ( const ::xsd::cxx::tree::exception< char > & e )
    {
        std::stringstream ss;
        ss << "FATAL - " << __func__ << e.what();
        logger->Debug ( ss.str() );
        throw Smartmatic::System::Exception::XmlException ( _ ( "Smartmatic.SAES.Voting.PBReports.PVFormWrapper.ErrorSerializaData" ),
                N_ ( "Smartmatic.SAES.Voting.PBReports.PVFormWrapper.ErrorSerializaData" ),
                logger );
    }
    catch ( ... )
    {
        std::stringstream ss;
        ss << "FATAL - " << __func__ << __LINE__;
        logger->Debug ( ss.str() );
        throw Smartmatic::System::Exception::XmlException ( _ ( "Smartmatic.SAES.Voting.PBReports.PVFormWrapper.ErrorSerializaData" ),
                N_ ( "Smartmatic.SAES.Voting.PBReports.PVFormWrapper.ErrorSerializaData" ),
                logger );
    }

    std::string data = os.str();
    std::string file;

    if ( fileFullPath.empty() )
    {
        file = directories->getPVFormFileFullPath();
    }
    else
    {
        file = fileFullPath;
    }

    Smartmatic::SAES::Security::Encryption::getInstance()->saveXmlFile ( data, file,
            Smartmatic::SAES::Security::Encryption::UTIL_FILE );

    logger->Debug ( "Persist pv report data " + pvformFileDom );
    fileFullPath = pvformFileDom;

    File::getInstance()->copyFile ( pvformFileDom, pvformFileFlash );

    logger->Debug ( "input pv report file " + pvformFileDom );
    logger->Debug ( "template pv report file " + pvtemplateFile );
    logger->Debug ( "Generate pv report file " + pvreportFileDom );

    if ( !File::getInstance()->canOpenFile ( pvtemplateFile ) )
    {
        logger->Debug ( "Error pv template file " + pvtemplateFile + " not found" );
        validate = false;
    }
    else
    {
        try
        {
            const bool mustBeValidated = Encryption::getInstance()->validateSigned(Encryption::UTIL_FILE);

            SafeBuffer pvFormBuffer;
            File::getInstance()->openFileToBuffer(pvformFileDom, pvFormBuffer);

            XmlFileSignature xmlFileSignature;
            xmlFileSignature.validateSignatureCertificate(pvFormBuffer, mustBeValidated);
        }
        catch (CryptoException &cryptoException)
        {
            logger->Error("An error occurred while trying to validate file " + pvformFileDom + "'s signature's certificate chain of trust.");
            return false;
        }

        if ( !transformer || !transformer->transformFile ( pvreportFileDom, pvformFileDom, pvtemplateFile ) )
        {
            logger->Debug ( "Error transform file " + pvformFileDom + " to " + pvreportFileDom );
            validate = false;
        }
    }

    File::getInstance()->copyFile ( pvreportFileDom, pvreportFileFlash );

    const std::string signatureFileName =
        SAESCommonConfigurationManager::getInstance()->getConfiguration()->
        getConfigurationElectionFilesPath().getSignatureResourceFileName();

    std::string signatureFileDom = Path::Combine(Path::GetDirectory(pvreportFileDom), signatureFileName);

    validate = signPVReport(pvreportFileDom, signatureFileDom);

    if (validate)
    {
        const std::string signatureFileFlash = Path::Combine(Path::GetDirectory(pvreportFileFlash), signatureFileName);
        File::getInstance()->copyFile ( signatureFileDom, signatureFileFlash );
    }

    if ( !SAESConfigurationManager::getInstance()->getPVFileConfiguration().getPersistenceBaseFile() )
    {
        File::getInstance()->DeleteFile ( pvformFileDom );
        File::getInstance()->DeleteFile ( pvformFileFlash );
    }

    return validate;
}

bool PVFormWrapper::signPVReport(const std::string& pvReportFilePath, std::string& signatureFilePath)
{
    logger->Debug ( "Signing file '" + pvReportFilePath + "'." );

    if (File::getInstance()->canOpenFile(pvReportFilePath) == false)
    {
        logger->Debug ( "Error pv report file " + pvReportFilePath + " not found" );
        return false;
    }

    std::string pvReportFileContents;

    File::getInstance()->readFile(pvReportFilePath, pvReportFileContents);
    SafeBuffer bufferToSign(pvReportFileContents);
    SafeBuffer signedBuffer;

    CertificateExtensionBase * const clientCertificate =
        CertificateStore::getInstance(directories)->getClientCertificate(
            Encryption::PRINTED_VOTE,
            Encryption::getInstance()->getSignedSecretKey());

    XmlDetachedSignature detachedSignature;
    bool success = detachedSignature.sign(
        bufferToSign, signedBuffer, Signed::FULL_SIGNED, clientCertificate, HASH_SHA256, pvReportFilePath);

    if ((success) && (signedBuffer.GetSize() > 0))
    {
        logger->Debug ( "File '" + pvReportFilePath + "' signed with success." );
        logger->Debug ( "Encrypting file '" + signatureFilePath + "'." );

        SafeBuffer encryptedBuffer;
        Encryption::getInstance()->cryptoBufferToBuffer ( signedBuffer, encryptedBuffer, KeyType::CONFIGURATION_SECRET_KEY );
        File::getInstance()->DeleteFile(signatureFilePath);
        File::getInstance()->addBufferToFile(encryptedBuffer, signatureFilePath);

        logger->Debug ( "File '" + signatureFilePath + "' encrypted with success." );

        success = true;
    }
    else
    {
        logger->Debug ( "Error: file '" + pvReportFilePath + "' could not be signed." );
    }

    return success;
}

void PVFormWrapper::loadPVForm( PVLanguage &language )
{
    ElectionStringResourcesManager * resource = ElectionStringResourcesManager::getInstance();
    Election::ElectionWrapper * election = electoralConfiguration->getElection();
    VotingDevice::VotingDeviceWrapper * votingDevice = electoralConfiguration->getVotingDevice();
    VotingDevice::Voting_booth votingBooth = votingDevice->getFirstVotingDevice().getVoting_booths().getVoting_booth() [0];

    Form form;
    PVInfoElection infoElection;
    PVVotes votes;

    /*******************************************************************/

    std::string electionDateString = votingDevice->getFirstVotingDevice().getOperation_modes().getOperation_mode() [0].getOpening_date_time();
    DateTimeUtils electionDate ( electionDateString );

    std::string openingDateString = operationStatus->getElectionStatusWrapper()->getOpenElectionDate();
    DateTimeUtils openingDate ( openingDateString );

    std::string closeVotingDateString = operationStatus->getElectionStatusWrapper()->getCloseVotingDate();
    DateTimeUtils closeVotingDate ( closeVotingDateString );

    std::string closeElectionDateString = operationStatus->getElectionStatusWrapper()->getCloseElectionDate();
    DateTimeUtils closeElectionDate ( closeElectionDateString );

    infoElection.setElection_date ( electionDate.getDate() );
    infoElection.setElection_name ( resource->getResource ( election->getElection().getName_key() ) );
    infoElection.setElectoral_organization ( resource->getResource ( election->getElection().getElectoral_organization() ) );
    infoElection.setCreation_time ( electionDate.getDate() + " " + electionDate.getTime() );
    infoElection.setVoting_device_code ( votingDevice->getFirstVotingDevice().getCode() );
    infoElection.setPolling_station_code ( votingBooth.getPolling_station_code() );
    infoElection.setPolling_station_number ( votingBooth.getPolling_station_number() );
    infoElection.setPolling_station_criteria ( resource->getResource ( votingBooth.getPolling_station_criteria() ) );
    infoElection.setPolling_place_code ( votingBooth.getPolling_place_code() );
    infoElection.setPolling_place_registered_electors ( 0 );
    infoElection.setPolling_place_address ( resource->getResource ( votingBooth.getPolling_place_address() ) );
    infoElection.setPolling_place_name ( resource->getResource ( votingBooth.getPolling_place_name() ) );
    infoElection.setOpening_date ( openingDate.getDate() + " " + openingDate.getTime() );
    infoElection.setClose_voting_date ( closeVotingDate.getDate() + " " + closeVotingDate.getTime() );
    infoElection.setClose_election_date ( closeElectionDate.getDate() + " " + closeElectionDate.getTime() );
    int numberSenators = 0;
    int numberMembers = 0;

    std::list < Smartmatic::SAES::Voting::PBReports::Contest * > & pbcontestList ( PVContestWrapper::getInstance()->getContestList () );
    std::list < Smartmatic::SAES::Voting::PBReports::Contest * >::iterator pbcit;

    Smartmatic::SAES::Voting::PBReports::Contests pvContests;
    Smartmatic::SAES::Voting::PBReports::Contests::ContestSequence pvConstestSeq;

    for ( pbcit = pbcontestList.begin();
            pbcit != pbcontestList.end();
            ++pbcit )
    {
        Smartmatic::SAES::Voting::PBReports::Contest * c = ( *pbcit );
        pvConstestSeq.push_back ( *c );
    }

    pvContests.setContest ( pvConstestSeq );
    form.setContests ( pvContests );
    infoElection.setNumber_senators ( numberSenators );
    infoElection.setNumber_members ( numberMembers );
    infoElection.setDuration_election_date ( durationElection );

    infoElection.setNiscode ( emlFiles->getCustomNisCode() );

    EmlInfo emlInfo;
    EmlInfo::EmlSequence emlSeq;
    std::vector<std::pair<std::string, std::string> > emlFilesHash = emlFiles->getEmlFileHashes();
    std::vector<std::pair<std::string, std::string> >::const_iterator itHashes;

    for (itHashes = emlFilesHash.begin(); itHashes != emlFilesHash.end(); ++itHashes)
    {
        Eml eml(itHashes->first, itHashes->second);
        emlSeq.push_back(eml);
    }

    emlInfo.setEml ( emlSeq );
    form.setEmlInfo ( emlInfo );

    PVInfoElection::RegionLevelSequence regions;
    std::list <VotingDevice::Region_category> & regionList ( electoralConfiguration->getRegionCategoryList() );
    std::list <VotingDevice::Region_category>::iterator regionIt;

    for ( regionIt = regionList.begin(); regionIt != regionList.end(); ++regionIt )
    {
        VotingDevice::Region_category category = ( *regionIt );
        RegionLevel regionLevel;
        regionLevel.setCategory_level ( category.getCategory_level().present() ? category.getCategory_level().get() : "" );
        regionLevel.setCategory ( resource->getResource ( category.getName_key() ) );
        regionLevel.setName ( resource->getResource ( category.getRegion().getName_key() ) );
        regions.push_back ( regionLevel );
    }

    infoElection.setRegionLevel ( regions );
    form.setInfoElection ( infoElection );

    /*******************************************************************/

    //Vote Statistics
    VoteStatistics newVoteStatistics = voteStatisticsWrapper->convertToGeneratedClass();
    VotesByCA::VoteByCASequence & sequenceVotesByCA = newVoteStatistics.getVotesByCA().getVoteByCA();

    for ( unsigned int i = 0; i < sequenceVotesByCA.size(); i++ )
    {
        sequenceVotesByCA[i].setCAValue ( resource->getResource ( sequenceVotesByCA[i].getCAValue() ) );
    }

    form.setVoteStatistics ( newVoteStatistics );
    //Activated Card Statistics
    ActivatedCardStatistics * newActivatedCardStatistics = activatedCardStatisticsWrapper->getCardStatistics();

    if (newActivatedCardStatistics == NULL)
    {
        throw Smartmatic::System::Exception::XmlException ( _ ( "Smartmatic.SAES.Voting.PBReports.PVFormWrapper.ErrorSerializaData" ),
                N_ ( "Smartmatic.SAES.Voting.PBReports.PVFormWrapper.ErrorSerializaData" ),
                logger );
    }

    ActivatedOfficialAudioByCA::CardByCASequence & cardOfficialAudioSeq = newActivatedCardStatistics->getActivatedOfficialAudioByCA().getCardByCA();

    for ( unsigned int i = 0; i < cardOfficialAudioSeq.size(); i++ )
    {
        cardOfficialAudioSeq[i].setCAValue ( resource->getResource ( cardOfficialAudioSeq[i].getCAValue() ) );
    }

    ActivatedOfficialNonAudioByCA::CardByCASequence & cardOfficialNonAudioSeq = newActivatedCardStatistics->getActivatedOfficialNonAudioByCA().getCardByCA();

    for ( unsigned int i = 0; i < cardOfficialNonAudioSeq.size(); i++ )
    {
        cardOfficialNonAudioSeq[i].setCAValue ( resource->getResource ( cardOfficialNonAudioSeq[i].getCAValue() ) );
    }

    ActivatedDemoAudioByCA::CardByCASequence & cardDemoAudioSeq = newActivatedCardStatistics->getActivatedDemoAudioByCA().getCardByCA();

    for ( unsigned int i = 0; i < cardDemoAudioSeq.size(); i++ )
    {
        cardDemoAudioSeq[i].setCAValue ( resource->getResource ( cardDemoAudioSeq[i].getCAValue() ) );
    }

    ActivatedDemoNonAudioByCA::CardByCASequence & cardDemoNonAudioSeq = newActivatedCardStatistics->getActivatedDemoNonAudioByCA().getCardByCA();

    for ( unsigned int i = 0; i < cardDemoNonAudioSeq.size(); i++ )
    {
        cardDemoNonAudioSeq[i].setCAValue ( resource->getResource ( cardDemoNonAudioSeq[i].getCAValue() ) );
    }

    form.setActivatedCardStatistics ( *newActivatedCardStatistics );

    /*******************************************************************/
    std::string fileTransmissionPackages = directories->getTransmissionPackagesFileFullPath();
    Smartmatic::System::SafeBuffer bufferFile;
    File::getInstance()->getHashFile ( fileTransmissionPackages, bufferFile );
    std::string hash = bufferFile.GetBase64Data();
    form.setHashOfTheResults ( hash );
    /*******************************************************************/
    std::list <Vote::Vote> demoVotes = transmissionPackageWrapper->getAllVoteDemo();
    std::list <Vote::Vote>::iterator itv;
    PVVotes::VoteSequence voteSeq;

    for ( itv = demoVotes.begin();
            itv != demoVotes.end();
            ++itv )
    {
        PVVote vote = Vote::VoteWrapper::voteToPVVote ( electoralConfiguration, *itv );
        voteSeq.push_back ( vote );
    }

    votes.setVote ( voteSeq );
    votes.setTotal_votes ( demoVotes.size() );
    form.setReferenceVotes ( votes );

    /*******************************************************************/

    VoteResultsWrapper voteResults ( electoralConfiguration, stationTallyWrapper );
    m_voteResults = voteResults.getVoteResults ();
    form.setVoteResults ( m_voteResults );

    /*******************************************************************/

    language.setPVForm(form);
}

void PVFormWrapper::setFileFullPath ( const std::string& fileFullPath )
{
    this->fileFullPath = fileFullPath;
}

void PVFormWrapper::setDurationElection ( const std::string& durationElection )
{
    this->durationElection = durationElection;
}

std::string PVFormWrapper::getDurationElection()
{
    return this->durationElection;
}

std::list <std::string> PVFormWrapper::getLanguages()
{
    std::list <std::string> languages;
    std::string currentLanguage = BaseLanguageController::getActualLanguage();
    bool allLanguages = SAESCommonConfigurationManager::getInstance()->isPVInAllLanguages();

    if ( allLanguages )
    {
        languages = electoralConfiguration->getLanguageListFullName();
    }

    if ( languages.empty() )
    {
        languages.push_back ( currentLanguage ); // generate default
    }

    return languages;
}

bool PVFormWrapper::validate()
{
    std::string pvformFileDom = directories->getPVFormFileFullPath ( BasePathType::CONFIGURATION_DOM );
    std::string pvformFileFlash = directories->getPVFormFileFullPath ( BasePathType::CONFIGURATION_FLASH );
    bool validate = true;

    if ( SAESConfigurationManager::getInstance()->getPVFileConfiguration().getPersistenceBaseFile() )
    {
        try
        {
            ( void ) Smartmatic::SAES::Security::Encryption::getInstance()->openXmlFile ( pvformFileDom,
                    Smartmatic::SAES::Security::Encryption::UTIL_FILE );
        }
        catch ( ... )
        {
            logger->Debug ( "FATAL - error open " + pvformFileDom );
            validate = false;
        }

        if ( ( !File::getInstance()->compareFiles ( pvformFileDom, pvformFileFlash ) ) && ( validate ) )
        {
            logger->Debug ( "FATAL - error compare files " + pvformFileDom + " " + pvformFileFlash );
            validate = false;
        }
    }

    if ( validate )
    {
        const std::string pvreportFileDom   =
            directories->getGeneratedPVReportFileFullPath ( BasePathType::CONFIGURATION_DOM );
        const std::string pvreportFileFlash =
            directories->getGeneratedPVReportFileFullPath ( BasePathType::CONFIGURATION_FLASH );

        if (File::getInstance()->compareFiles(pvreportFileDom, pvreportFileFlash) == false)
        {
            logger->Debug ( "FATAL - error compare files " + pvreportFileDom + " " + pvreportFileFlash );
            validate = false;
        }

        const std::string signatureFileName =
            SAESCommonConfigurationManager::getInstance()->getConfiguration()->
            getConfigurationElectionFilesPath().getSignatureResourceFileName();

        const std::string pvReportSignatureFileDom   =
            Path::Combine(directories->getDocsFolderFullPath(BasePathType::CONFIGURATION_DOM), signatureFileName);
        const std::string pvReportSignatureFileFlash =
            Path::Combine(directories->getDocsFolderFullPath(BasePathType::CONFIGURATION_FLASH), signatureFileName);

        if (File::getInstance()->compareFiles(pvReportSignatureFileDom, pvReportSignatureFileFlash) == false)
        {
            logger->Debug("FATAL - error compare files " + pvReportSignatureFileDom + " " + pvReportSignatureFileFlash);
            validate = false;
        }
    }

    return validate;
}

void PVFormWrapper::deleteFiles()
{
    std::string pvformFileDom = directories->getPVFormFileFullPath ( BasePathType::CONFIGURATION_DOM );
    std::string pvformFileFlash = directories->getPVFormFileFullPath ( BasePathType::CONFIGURATION_FLASH );
    std::string pvreportFileDom = directories->getGeneratedPVReportFileFullPath ( BasePathType::CONFIGURATION_DOM );
    std::string pvreportFileFlash = directories->getGeneratedPVReportFileFullPath ( BasePathType::CONFIGURATION_FLASH );
    std::list <std::string>::iterator itl;
    std::list <std::string> languages = getLanguages();

    for ( itl = languages.begin();
            itl != languages.end();
            ++itl )
    {
        std::string lang = ( *itl );
        std::string pvformFileDomTemp = Path::combineFileName ( pvformFileDom, lang );
        std::string pvformFileFlashTemp = Path::combineFileName ( pvformFileFlash, lang );
        std::string pvreportFileDomTemp = Path::combineFileName ( pvreportFileDom, lang );
        std::string pvreportFileFlashTemp = Path::combineFileName ( pvreportFileFlash, lang );
        File::getInstance()->DeleteFile ( pvformFileDomTemp );
        File::getInstance()->DeleteFile ( pvformFileFlashTemp );
        File::getInstance()->DeleteFile ( pvreportFileDomTemp );
        File::getInstance()->DeleteFile ( pvreportFileFlashTemp );
    }
}

void PVFormWrapper::getPVFileNames ( std::string file, std::list <std::string> & fileNames )
{
    fileNames.clear();
    std::list <std::string>::iterator itl;
    std::list <std::string> languages = getLanguages();

    for ( itl = languages.begin();
            itl != languages.end();
            ++itl )
    {
        std::string lang = ( *itl );
        std::string tmpFile = Path::GetFileNameWithOutExtension ( file ) + lang + Path::GetFileExtension ( file );
        fileNames.push_back ( tmpFile );
    }
}

VoteResults PVFormWrapper::getVoteResults()
{
    return m_voteResults;
}

size_t PVFormWrapper::getTotalVotes()
{
    return voteStatisticsWrapper->getTotalVotes();
}
