/**
 * @file   VoteStatisticsWrapper.cxx
 * @brief  Implementation of VoteStatisticsWrapper class.
 * @author Youssef.tourkmani
 * @author manuel.arguelles
 * @date   Jun 27, 2017
 */

#include "VoteStatisticsWrapper.hxx"
#include "Configuration/s-a-e-s-conf-manager.h"
#include <Log/SMTTLogManager.h>
#include <Security/Encryption.hxx>
#include <System/Exception/VoteException.hxx>
#include <System/IO/Directory.h>
#include <System/IO/File.h>
#include <Voting/VotingDevice/voting-devices-schema.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::SAES::Security;
using namespace Smartmatic::SAES::Voting::PBReports;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::System::IO;

#define LOGGER_PROJECT "Smartmatic.SAES.Voting.PBReports.VoteStatistics"

#ifdef GETTEXT_PACKAGE
ISMTTLog* VoteStatisticsWrapper::m_logger = SMTTLogManager::GetLogger(
    LOGGER_PROJECT,
    GETTEXT_PACKAGE);
#else
ISMTTLog* VoteStatisticsWrapper::m_logger = SMTTLogManager::GetLogger(
    LOGGER_PROJECT,
    "");
#endif

struct checkCustomCode
{
    explicit checkCustomCode(const VoteCounterByContest& voteCounterByContest)
    : m_voteCounterByContest(voteCounterByContest)
    {

    }

    bool operator()(VoteCounterByContest& voteCounterByContest)
    {
        return (m_voteCounterByContest.getCustomCode() == voteCounterByContest.getCustomCode());
    }

    VoteCounterByContest m_voteCounterByContest;
};

VoteStatisticsWrapper::VoteStatisticsWrapper()
    : m_votingDeviceWrapper(NULL)
    , m_directories(NULL)
    , m_statistics(0)
    , m_deltaVotes(0)
{
}

VoteStatisticsWrapper::~VoteStatisticsWrapper()
{
    Dispose();
}

unsigned int VoteStatisticsWrapper::getVoteCounterByCACodeAndValue(
    const std::string& caCode,
    const std::string& caValue)
{
    unsigned int counter = 0;

    for (size_t j = 0; j < m_voteCountersByCA.size(); ++j)
    {
        if (caCode.compare(m_voteCountersByCA[j].getCACode()) == 0
            && caValue.compare(m_voteCountersByCA[j].getCAValue()) == 0)
        {
            counter = m_voteCountersByCA[j].getCounter();
            break;
        }
    }

    return counter;
}

VoteStatistics VoteStatisticsWrapper::convertToGeneratedClass()
{
    VoteStatistics statistics;

    VotesByCA votesCA;
    VotesByCA::VoteByCASequence voteCAseq;

    VotesByContests votesContest;
    VotesByContests::VotesByContestSequence voteContestSeq;

    statistics.setTotalVotes(getTotalVotes());
    statistics.setTotalOfficialVotes(getOfficialVotes());
    statistics.setTotalDemoVotes(getDemoVotes());
    statistics.setTotalDuplicatedVotes(getDuplicatedVotes());
    statistics.setTotalSimulatedVotes(getSimulatedVotes());

    for (size_t j = 0; j < m_voteCountersByCA.size(); ++j)
    {
        VoteByCA voteCA;

        voteCA.setVotesCounter(m_voteCountersByCA[j].getCounter());
        voteCA.setCACode(m_voteCountersByCA[j].getCACode());
        voteCA.setCAValue(m_voteCountersByCA[j].getCAValue());
        voteCAseq.push_back(voteCA);
    }

    for (size_t i = 0; i < m_voteCountersByContest.size(); ++i)
    {
        VotesByContest voteContest;

        voteContest.setCustomCode(m_voteCountersByContest[i].getCustomCode());
        voteContest.setVotesCounter(m_voteCountersByContest[i].getCounter());
        voteContestSeq.push_back(voteContest);
    }

    votesCA.setVoteByCA(voteCAseq);
    statistics.setVotesByCA(votesCA);

    votesContest.setVotesByContest(voteContestSeq);
    statistics.setVotesByContests(votesContest);

    return statistics;
}

bool VoteStatisticsWrapper::IsVoteProcessed(const Vote::Vote& vote)
{
    for (size_t j = 0; j < m_voteGUUIDsRegistered.size(); ++j)
    {
        if (m_voteGUUIDsRegistered[j].compare(vote.getCode()) == 0)
        {
            return true;
        }
    }

    return false;
}

bool VoteStatisticsWrapper::IsVoteProcessed(const std::string& voteGUUID)
{
    for (size_t j = 0; j < m_voteGUUIDsRegistered.size(); ++j)
    {
        if (m_voteGUUIDsRegistered[j].compare(voteGUUID) == 0)
        {
            return true;
        }
    }

    return false;
}

bool  VoteStatisticsWrapper::ProcessVote(Vote::Vote& vote)
{
    bool validate = false;

    try
    {
        if (!IsVoteProcessed(vote))
        {
            incrementTotalVotes();
            m_voteGUUIDsRegistered.push_back(vote.getCode());

            switch (vote.getMode())
            {
                case Vote::VoteMode::VOTE_O:
                    incrementVotesByCA(vote);
                    incrementVotesByContest(vote);
                    incrementOfficialVotes();
                    break;

                case Vote::VoteMode::VOTE_D:
                    incrementDemoVotes();
                    break;

                case Vote::VoteMode::VOTE_S:
                    incrementSimulateVotes();
                    break;

                default:
                    break;
            }

            validate = true;
        }
        else
        {
            incrementTotalDuplicatedVotes();
        }
    }
    catch (const VotingDeviceException& ex)
    {
        std::stringstream ss;

        ss << "FATAL " << __LINE__ << " " << __FILE__ << ": " << ex.getMessage();
        m_logger->Debug(ss.str());
        return false;
    }

    return validate;
}

std::vector<VoteCounterByCA> VoteStatisticsWrapper::getVoteCountersByCA()
{
    return m_voteCountersByCA;
}

std::vector<VoteCounterByContest> VoteStatisticsWrapper::getVoteCountersByContest()
{
    return m_voteCountersByContest;
}

bool VoteStatisticsWrapper::init(
    SaesDirectories* directories,
    ElectoralConfiguration* elecConf)
{
    Dispose();

    m_directories = directories;

    try
    {
        m_votingDeviceWrapper = elecConf->getVotingDevice();
        m_votingDevice = m_votingDeviceWrapper->getFirstVotingDevice();
        VotingDevice::Logical_voter_groups lvgs = m_votingDeviceWrapper->getLogicalVoterGroups(m_votingDevice);

        if (lvgs.getCustom_attribute_definitions() != NULL)
        {
            VotingDevice::Custom_attribute_definitions caDefs = lvgs.getCustom_attribute_definitions().get();

            for (size_t i = 0; i < caDefs.getCustom_attribute_definition().size(); ++i)
            {
                for (size_t j = 0; j < caDefs.getCustom_attribute_definition()[i].getCustom_attribute_definition_values().getCustom_attribute_definition_value().size(); ++j)
                {
                    VoteCounterByCA voteCounter(
                        caDefs.getCustom_attribute_definition()[i].getCode(),
                        caDefs.getCustom_attribute_definition()[i].getCustom_attribute_definition_values().getCustom_attribute_definition_value()[j].getValue());

                    m_voteCountersByCA.push_back(voteCounter);
                }
            }
        }
        else
        {
            m_logger->Debug("ERROR - Custom attribute definitions not found.");
        }

        if (lvgs.getLogical_voter_group().size() > 0)
        {
            VotingDevice::Logical_voter_groups::Logical_voter_groupSequence logicalVoterGroups = lvgs.getLogical_voter_group();

            VotingDevice::Logical_voter_groups::Logical_voter_groupSequence::const_iterator itLvg = logicalVoterGroups.begin();
            const VotingDevice::Logical_voter_groups::Logical_voter_groupSequence::const_iterator endLvg = logicalVoterGroups.end();

            for (; itLvg != endLvg; ++itLvg)
            {
                const std::string contestGroupCode = itLvg->getContest_group_code();
                m_contestGroupsMap[contestGroupCode] = elecConf->getListContestByContestGroupCode(contestGroupCode);
            }

            if (m_contestGroupsMap.empty() == false)
            {
                std::map<std::string, std::list<Election::Contest*> >::const_iterator itMap = m_contestGroupsMap.begin();
                const std::map<std::string, std::list<Election::Contest*> >::const_iterator endMap = m_contestGroupsMap.end();

                for (; itMap != endMap; ++itMap)
                {
                    std::list<Election::Contest*>::const_iterator itContest = itMap->second.begin();
                    const std::list<Election::Contest*>::const_iterator endContest = itMap->second.end();

                    for (; itContest != endContest; ++itContest)
                    {
                        const std::string customCode = (*itContest)->getCustom_code();
                        const VoteCounterByContest voteCounter(customCode);

                        if (std::find_if(
                            m_voteCountersByContest.begin(),
                            m_voteCountersByContest.end(),
                            checkCustomCode(voteCounter)) == m_voteCountersByContest.end())
                        {
                            m_voteCountersByContest.push_back(voteCounter);
                        }
                    }
                }
            }
            else
            {
                m_logger->Debug("ERROR - Contest groups not found.");
            }
        }
        else
        {
            m_logger->Debug("ERROR - Logical voter groups not found.");
        }
    }
    catch (const VotingDeviceException& ex)
    {
        std::stringstream ss;

        ss << "FATAL " << __LINE__ << " " << __FILE__ << ": " << ex.getMessage();
        m_logger->Debug(ss.str());
    }

    Glib::Mutex::Lock lock(m_mutex);
    int totalVotes = -1;

    if (m_statistics != NULL)
    {
        totalVotes = m_statistics->getTotalVotes();
        delete m_statistics;
        m_statistics = NULL;
    }

    VoteStatistic* statisticDom = Load(directories->getVoteStatisticsFileFullPath(BasePathType::CONFIGURATION_DOM));
    VoteStatistic* statisticFlash = Load(directories->getVoteStatisticsFileFullPath(BasePathType::CONFIGURATION_FLASH));
    bool load = false;

    if (statisticDom != NULL && statisticFlash == NULL)
    {
        m_logger->Debug("Load vote statistics in DOM.");
        m_statistics = statisticDom;
        m_deltaVotes = m_statistics->getTotalVotes();
        load = true;
    }
    else if (statisticDom == NULL && statisticFlash != NULL)
    {
        m_logger->Debug("Load vote statistics in Flash.");
        m_statistics = statisticFlash;
        m_deltaVotes = m_statistics->getTotalVotes();
        load = true;
    }
    else if (statisticDom != NULL && statisticFlash != NULL)
    {
        load = true;
        m_deltaVotes =(unsigned int)abs((int)(statisticDom->getTotalVotes() - statisticFlash->getTotalVotes()));

        if (statisticDom->getTotalVotes() > statisticFlash->getTotalVotes())
        {
            m_logger->Debug("Load vote statistics in DOM by total votes.");
            m_statistics = statisticDom;
            delete statisticFlash;
        }
        else if (statisticDom->getDuplicatedVotes() > statisticFlash->getDuplicatedVotes())
        {
            m_logger->Debug("Load vote statistics in DOM by duplicate votes.");
            m_statistics = statisticDom;
            delete statisticFlash;
        }
        else
        {
            m_logger->Debug("Load vote statistics in Flash by default.");
            m_statistics = statisticFlash;
            delete statisticDom;
        }

        if (totalVotes >= 0 && totalVotes != ( int ) m_statistics->getTotalVotes())
        {
            throw VoteException(
                _("Smartmatic.SAES.Voting.PBReports.VoteStatisticsWrapper.ErrorLoadStatisticVote"),
                N_("Smartmatic.SAES.Voting.PBReports.VoteStatisticsWrapper.ErrorLoadStatisticVote" ),
                m_logger);
        }
    }
    else
    {
        m_logger->Debug("New vote statistics by default.");
        m_statistics = new VoteStatistic();
    }

    return load;
}

unsigned int VoteStatisticsWrapper::getTotalVotes()
{
    Glib::Mutex::Lock lock(m_mutex);
    return (m_statistics != NULL ? m_statistics->getTotalVotes() : 0);
}

unsigned int VoteStatisticsWrapper::getOfficialVotes()
{
    Glib::Mutex::Lock lock(m_mutex);
    return (m_statistics != NULL ? m_statistics->getOfficialVotes() : 0);
}

unsigned int VoteStatisticsWrapper::getSimulatedVotes()
{
    Glib::Mutex::Lock lock (m_mutex);
    return (m_statistics != NULL ? m_statistics->getSimulateVotes() : 0);
}

unsigned int VoteStatisticsWrapper::getDemoVotes()
{
    Glib::Mutex::Lock lock(m_mutex);
    return (m_statistics != NULL ? m_statistics->getDemoVotes() : 0);
}

unsigned int VoteStatisticsWrapper::getDuplicatedVotes()
{
    Glib::Mutex::Lock lock(m_mutex);
    return (m_statistics != NULL ? m_statistics->getDuplicatedVotes() : 0);
}

void VoteStatisticsWrapper::incrementTotalVotes()
{
    if (m_statistics)
    {
        m_statistics->setTotalVotes(m_statistics->getTotalVotes() + 1);
    }
}

void VoteStatisticsWrapper::incrementTotalDuplicatedVotes()
{
    if (m_statistics)
    {
        m_statistics->setDuplicatedVotes(m_statistics->getDuplicatedVotes() + 1);
        std::stringstream ss;
        ss << "Increment duplicate vote to " << m_statistics->getDuplicatedVotes();
        m_logger->Debug(ss.str());
    }
}

void VoteStatisticsWrapper::incrementOfficialVotes()
{
    if (m_statistics)
    {
        m_statistics->setOfficialVotes(m_statistics->getOfficialVotes() + 1);
        std::stringstream ss;
        ss << "Increment official vote to " << m_statistics->getOfficialVotes();
        m_logger->Debug(ss.str());
    }
}

void VoteStatisticsWrapper::incrementDemoVotes()
{
    if (m_statistics)
    {
        m_statistics->setDemoVotes(m_statistics->getDemoVotes() + 1);
        std::stringstream ss;
        ss << "Increment demo vote to " << m_statistics->getDemoVotes();
        m_logger->Debug(ss.str());
    }
}

void VoteStatisticsWrapper::incrementSimulateVotes()
{
    if (m_statistics)
    {
        m_statistics->setSimulateVotes(m_statistics->getSimulateVotes() + 1);
        std::stringstream ss;
        ss << "Increment simulate vote to " << m_statistics->getSimulateVotes();
        m_logger->Debug(ss.str());
    }
}

void VoteStatisticsWrapper::Dispose()
{
    if (m_statistics)
    {
        m_statistics->clearAll();
    }

    m_voteGUUIDsRegistered.clear();
    m_voteCountersByCA.clear();
    m_voteCountersByContest.clear();
}

void VoteStatisticsWrapper::clearCounter(bool cleanDuplicated)
{
    if (m_statistics)
    {
        if (cleanDuplicated)
        {
            m_statistics->clearAll();
        }
        else
        {
            m_statistics->clearAllNotDuplicate();
        }
    }

    m_voteGUUIDsRegistered.clear();

    for (size_t j = 0; j < m_voteCountersByCA.size(); ++j)
    {
        m_voteCountersByCA[j].clearCounter();
    }

    m_logger->Debug("Clear counters");
}

void VoteStatisticsWrapper::persistence()
{
    m_logger->Debug("Start persistence vote statistics file");
    VoteStatistics statistic = convertToGeneratedClass();
    std::ostringstream os;

    try
    {
        Smartmatic::SAES::Voting::PBReports::NamespaceInfomap map;
#ifdef NAMESPACE_PBREPORTS
        map[""].name = NAMESPACE_PBREPORTS;
#endif
        map[""].schema = "";
        serializeVoteStatistics(os, statistic, map);
    }
    catch (const ::xsd::cxx::tree::exception<char>& e)
    {
        std::stringstream ss;
        ss << "FATAL - " << __func__ << e.what();
        m_logger->Debug(ss.str());
        throw XmlException(
            _("Smartmatic.SAES.Voting.PBReports.VoteStatisticsWrapper.ErrorSerializaData"),
            N_("Smartmatic.SAES.Voting.PBReports.VoteStatisticsWrapper.ErrorSerializaData"),
            m_logger);
    }
    catch (...)
    {
        std::stringstream ss;
        ss << "FATAL - " << __func__ << __LINE__;
        m_logger->Debug(ss.str());
        throw XmlException(
            _("Smartmatic.SAES.Voting.PBReports.VoteStatisticsWrapper.ErrorSerializaData"),
            N_("Smartmatic.SAES.Voting.PBReports.VoteStatisticsWrapper.ErrorSerializaData"),
            m_logger);
    }

    std::string data = os.str();
    std::string devicePathDom = m_directories->getDevicePath(BasePathType::CONFIGURATION_DOM);
    std::string devicePathFlash = m_directories->getDevicePath(BasePathType::CONFIGURATION_FLASH);
    std::string fileDom = m_directories->getVoteStatisticsFileFullPath(BasePathType::CONFIGURATION_DOM);
    std::string fileFlash = m_directories->getVoteStatisticsFileFullPath(BasePathType::CONFIGURATION_FLASH);

    if (Directory::getInstance()->DirectoryExists(devicePathDom))
    {
        Encryption::getInstance()->saveXmlFile(
            data,
            fileDom,
            Encryption::INTERNAL_FILE);
        File::getInstance()->copyFile(fileDom, fileFlash);
    }
    else if (Directory::getInstance()->DirectoryExists(devicePathFlash))
    {
        Encryption::getInstance()->saveXmlFile(
            data,
            fileFlash,
            Encryption::INTERNAL_FILE);
        File::getInstance()->copyFile(fileFlash, fileDom);
    }

    m_logger->Debug("Success persistence vote statistics file");
}

VoteStatistic* VoteStatisticsWrapper::Load(std::string xmlFile)
{
    VoteStatistic* statisticTemp = NULL;

    if (File::getInstance()->canOpenFile(xmlFile))
    {
        try
        {
            std::string data = Encryption::getInstance()->openXmlFile(
                xmlFile,
                Encryption::INTERNAL_FILE);
            std::istringstream istr(data);
            VoteStatistics* statistic = new VoteStatistics(
                *(parseVoteStatistics(istr, Flags::dont_validate)));

            statisticTemp = new VoteStatistic();
            statisticTemp->setDemoVotes(statistic->getTotalDemoVotes());
            statisticTemp->setDuplicatedVotes(statistic->getTotalDuplicatedVotes());
            statisticTemp->setOfficialVotes(statistic->getTotalOfficialVotes());
            statisticTemp->setTotalVotes(statistic->getTotalVotes());
            statisticTemp->setSimulateVotes(statistic->getTotalSimulatedVotes() != NULL ? statistic->getTotalSimulatedVotes().get() : 0);
            delete statistic;
            statistic = NULL;
        }
        catch (const ::xsd::cxx::tree::exception<char>& e)
        {
            std::stringstream ss;

            ss  << "FATAL - " << __func__ << " " << e;
            m_logger->Debug(ss.str());
            throw XmlException(
                _("Smartmatic.SAES.Voting.PBReports.VoteStatisticsWrapper.ErrorParseFile"),
                N_("Smartmatic.SAES.Voting.PBReports.VoteStatisticsWrapper.ErrorParseFile"),
                m_logger);
        }
        catch ( ... )
        {
            std::stringstream ss;

            ss  << "FATAL - " << __func__;
            m_logger->Debug(ss.str());
            throw XmlException(
                _("Smartmatic.SAES.Voting.PBReports.VoteStatisticsWrapper.ErrorParseFile"),
                N_("Smartmatic.SAES.Voting.PBReports.VoteStatisticsWrapper.ErrorParseFile"),
                m_logger);
        }
    }

    return statisticTemp;
}

unsigned int VoteStatisticsWrapper::getDeltaVotes()
{
    return m_deltaVotes;
}

void VoteStatisticsWrapper::incrementVotesByCA(const Vote::Vote& vote)
{
    VotingDevice::Logical_voter_group& lvg(m_votingDeviceWrapper->getLogicalVoterGroup(m_votingDevice, vote.getLogicalVoterGroup()));

    for (size_t i = 0; i < lvg.getCustom_attributes().get().getCustom_attribute().size(); ++i)
    {
        for (size_t j = 0; j < m_voteCountersByCA.size(); ++j)
        {
            if (lvg.getCustom_attributes().get().getCustom_attribute()[i].getCode().compare(m_voteCountersByCA[j].getCACode()) == 0
                && lvg.getCustom_attributes().get().getCustom_attribute()[i].getValue().compare(m_voteCountersByCA[j].getCAValue()) == 0)
            {
                m_voteCountersByCA[j].incrementCounter();
                break;
            }
        }
    }
}

void VoteStatisticsWrapper::incrementVotesByContest(const Vote::Vote& vote)
{
    const Vote::Vote::SSequence selections = vote.getSelections();

    Vote::Vote::SSequence::const_iterator       it  = selections.begin();
    const Vote::Vote::SSequence::const_iterator end = selections.end();

    const VotingDevice::Logical_voter_group voteLvg     = m_votingDeviceWrapper->getLogicalVoterGroup(vote.getLogicalVoterGroup());
    const std::list<Election::Contest*>     lvgContests = m_contestGroupsMap[voteLvg.getContest_group_code()];

    for (; it != end; ++it)
    {
        updateCounter(it->getContest(), lvgContests);
    }
}

void VoteStatisticsWrapper::updateCounter(
    const std::string& voteContestCode,
    const std::list<Election::Contest*>& lvgContests)
{
    std::list<Election::Contest*>::const_iterator       it  = lvgContests.begin();
    const std::list<Election::Contest*>::const_iterator end = lvgContests.end();

    for (; it != end; ++it)
    {
        if ((*it)->getCode() == voteContestCode)
        {
            updateVotesCounterByContest((*it)->getCustom_code());
            break;
        }
    }
}

void VoteStatisticsWrapper::updateVotesCounterByContest(const std::string& customCode)
{
    for (size_t i = 0; i < m_voteCountersByContest.size(); ++i)
    {
        if (m_voteCountersByContest[i].getCustomCode() == customCode)
        {
            m_voteCountersByContest[i].incrementCounter();
            break;
        }
    }
}
