/**
 * @file   VoteStatisticsWrapper.hxx
 * @brief  Declaration of VoteStatisticsWrapper class.
 * @author Youssef.tourkmani
 * @author manuel.arguelles
 * @date   Jun 27, 2017
 */

#ifndef _VOTE_STATISTICS_WRAPPER_HXX_
#define _VOTE_STATISTICS_WRAPPER_HXX_

#include "VoteCounterByCA.hxx"
#include "VoteCounterByContest.hxx"
#include "VoteStatistic.hxx"
#include "Voting/PBReports/vote-statistics-schema.hxx"
#include <Voting/ElectoralConfiguration.hxx>
#include <Voting/Vote/vote-schema.hxx>
#include <glibmm.h>


namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Voting
        {
            namespace PBReports
            {

                /**
                 * @class  VoteStatisticsWrapper.
                 * @brief  Wrapper class for VoteStatistics.
                 * @author Marcel Mancini &lt;marcel.mancini@smartmatic.com&gt;
                 * @date   29-03-2011
                 */
                class VoteStatisticsWrapper
                {
                    public:
                        /**
                         * @brief  Class constructor.
                         * @author Eduardo.campione
                         * @date   28/04/2011
                         */
                        VoteStatisticsWrapper();

                        /**
                         * @brief  Class destructorer.
                         * @author Eduardo.campione
                         * @date   28/04/2011
                         */
                        virtual ~VoteStatisticsWrapper();

                        /**
                         * @brief  Gets the official votes.
                         * @author Eduardo.campione
                         * @date   28/04/2011
                         * @return The official votes.
                         */
                        virtual unsigned int getOfficialVotes();

                        /**
                         * @brief  Gets the simulated votes.
                         * @author Eduardo.campione
                         * @date   28/04/2011
                         * @return The simulated votes.
                         */
                        virtual unsigned int getSimulatedVotes();

                        /**
                         * @brief  Gets the demo votes.
                         * @author Eduardo.campione
                         * @date   28/04/2011
                         * @return The demo votes.
                         */
                        virtual unsigned int getDemoVotes();

                        /**
                         * @brief  Convertes to generated class.
                         * @author Eduardo.campione
                         * @date   28/04/2011
                         * @return The vote statistic generated class.
                         */
                        VoteStatistics convertToGeneratedClass();

                        /**
                         * @brief  Gets the vote counters by ca.
                         * @author Eduardo.campione
                         * @date   28/04/2011
                         * @return The vote counters by ca.
                         */
                        virtual std::vector<VoteCounterByCA> getVoteCountersByCA();

                        /**
                         * @brief  Gets the vote counters by contest.
                         * @author eduardo.fernandez
                         * @date   05/12/2018
                         * @return The vote counters by contest.
                         */
                        virtual std::vector<VoteCounterByContest> getVoteCountersByContest();

                        /**
                         * @brief  Gets a vote counter by ca code and value.
                         * @author Eduardo.campione
                         * @date   28/04/2011
                         * @param  caCode The ca code.
                         * @param  caValue The ca value.
                         * @return The vote counter by ca code and value.
                         */
                        unsigned int getVoteCounterByCACodeAndValue(
                            const std::string& caCode,
                            const std::string& caValue);

                        /**
                         * @brief  Gets the total votes.
                         * @author Eduardo.campione
                         * @date   28/04/2011
                         * @return The total votes.
                         */
                        virtual unsigned int getTotalVotes();

                        /**
                         * @brief  Gets the duplicated votes.
                         * @author Eduardo.campione
                         * @date   28/04/2011
                         * @return The duplicated votes.
                         */
                        unsigned int getDuplicatedVotes();

                        /**
                         * @brief  Clears the counter described by cleanDuplicated.
                         * @author Youssef.tourkmani
                         * @date   31/10/2012
                         * @param  cleanDuplicated (optional) clean duplicated.
                         */
                        void clearCounter(bool cleanDuplicated = false);

                        /**
                         * @brief         Process the vote described by vote.
                         * @author        Youssef.tourkmani
                         * @date          31/10/2012
                         * @param[in,out] vote The vote.
                         * @return        true if it succeeds, false if it fails.
                         */
                        bool ProcessVote(Smartmatic::SAES::Voting::Vote::Vote& vote);

                        /**
                         * @brief  Check if vote was processed.
                         * @author Eduardo.campione
                         * @date   28/04/2011
                         * @param  vote The vote.
                         * @return true if vote was processed, false if not.
                         */
                        bool IsVoteProcessed(const Smartmatic::SAES::Voting::Vote::Vote& vote);

                        /**
                         * @brief  Check if voteGUUID was processed.
                         * @author Eduardo.campione
                         * @date   28/04/2011
                         * @param  voteGUUID The vote guuid.
                         * @return true if vote was processed, false if not.
                         */
                        bool IsVoteProcessed(const std::string& voteGUUID);

                        /**
                         * @brief  Initialises this object.
                         * @author Eduardo.campione
                         * @date   28/04/2011
                         * @throws CryptoException Thrown when a crypto error condition occurs.
                         * @throws SignedException Thrown when a signed error condition occurs.
                         * @throws XmlException    Thrown when an XML error condition occurs.
                         * @throws VoteException   Thrown when a vote error condition occurs.
                         * @param  directories The SAES Directories.
                         * @param  elecConf The electoral configuration.
                         * @return True if it succeeds, false if it fails.
                         */
                        bool init(
                            Smartmatic::SAES::Runtime::Environment::SaesDirectories* directories,
                            Smartmatic::SAES::Voting::ElectoralConfiguration* elecConf);

                        /**
                         * @brief  Dispose of this object, cleaning up any resources it uses.
                         * @author Eduardo.campione
                         * @date   28/04/2011
                         */
                        void Dispose();

                        /**
                         * @brief     Load vote statistic file.
                         * @author    Yadickson Soto yadickson.soto@smartmatic.com
                         * @date      21/05/2011
                         * @throws    CryptoException Thrown when a crypto error condition occurs.
                         * @throws    SignedException Thrown when a signed error condition occurs.
                         * @throws    XmlException    Thrown when an XML error condition occurs.
                         * @param     xmlFile File full path vote statistic.
                         * @return    null if it fails, true otherwise.
                         */
                        VoteStatistic* Load(std::string xmlFile);

                        /**
                         * @brief     Persist the vote statistic file.
                         * @author    Yadickson Soto yadickson.soto@smartmatic.com
                         * @date      20/05/2011
                         * @throws    CryptoException Thrown when a crypto error condition occurs.
                         * @throws    SignedException Thrown when a signed error condition occurs.
                         * @throws    XmlException    Thrown when an XML error condition occurs.
                         */
                        void persistence();

                        /**
                         * @brief  Gets delta votes.
                         * @author Youssef.tourkmani
                         * @date   31/10/2012
                         * @return The delta votes.
                         */
                        unsigned int getDeltaVotes();

                    protected:
                        /**
                         * @brief  Increment official votes.
                         * @author Eduardo.campione
                         * @date   28/04/2011
                         */
                        void incrementOfficialVotes();

                        /**
                         * @brief  Increment demo votes.
                         * @author Eduardo.campione
                         * @date   28/04/2011
                         */
                        void incrementDemoVotes();

                        /**
                         * @brief  Increment simulate votes.
                         * @author Eduardo.campione
                         * @date   28/04/2011
                         */
                        void incrementSimulateVotes();

                        /**
                         * @brief  Increment total votes.
                         * @author Eduardo.campione
                         * @date   28/04/2011
                         */
                        void incrementTotalVotes();

                        /**
                         * @brief  Increment total duplicated votes.
                         * @author Eduardo.campione
                         * @date   28/04/2011
                         */
                        void incrementTotalDuplicatedVotes();

                        /**
                         * @brief  Increment the votes counter by CA.
                         * @author manuel.arguelles
                         * @date   Jun 26, 2017
                         * @param  vote The vote.
                         */
                        void incrementVotesByCA(const Smartmatic::SAES::Voting::Vote::Vote& vote);

                        /**
                         * @brief  Increment the votes counter by CA.
                         * @author manuel.arguelles
                         * @date   Jun 26, 2017
                         * @param  vote The vote.
                         */
                        void incrementVotesByContest(const Smartmatic::SAES::Voting::Vote::Vote& vote);

                    private:

                        /**
                         * @brief Updates votes counter by contest
                         * @author eduardo.fernandez
                         * @date   06/12/2018
                         * @param  voteContestCode Current vote contest code.
                         * @param  contests Configured contests.
                         */
                        void updateCounter(
                            const std::string& voteContestCode,
                            const std::list<Election::Contest*>& contests);

                        /**
                         * @brief Updates vote counter by given contest's custom code.
                         * @author eduardo.fernandez
                         * @date   06/12/2018
                         * @param  customCode Contest's custom code to be updated.
                         */
                        void updateVotesCounterByContest(const std::string& customCode);

                    private:
                        VotingDevice::VotingDeviceWrapper*                    m_votingDeviceWrapper;
                        VotingDevice::Voting_device                           m_votingDevice;
                        Runtime::Environment::SaesDirectories*                m_directories;
                        VoteStatistic*                                        m_statistics;
                        unsigned int                                          m_deltaVotes;
                        std::vector<VoteCounterByCA>                          m_voteCountersByCA;
                        std::vector<VoteCounterByContest>                     m_voteCountersByContest;
                        std::map<std::string, std::list<Election::Contest*> > m_contestGroupsMap;
                        std::vector<std::string>                              m_voteGUUIDsRegistered;
                        Glib::Mutex                                           m_mutex;
                        static Smartmatic::Log::ISMTTLog*                     m_logger;
                };

            }
        }
    }
}

#endif /* _VOTE_STATISTICS_WRAPPER_HXX_ */
