/**
 * @file	StationTallyWrapper.cxx
 * @brief	station tally wrapper class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */

#include "StationTallyWrapper.hxx"
#include <Configuration/s-a-e-s-conf-manager.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Voting/Election/election-schema.hxx>
#include <Voting/ElectoralConfiguration.hxx>
#include <Voting/Vote/station-tally-schema.hxx>
#include <Voting/Vote/vote-schema.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Voting::Vote;
using namespace Smartmatic::SAES::Voting;

ISMTTLog * StationTallyWrapper::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Voting.Vote.StationTallyWrapper", GETTEXT_PACKAGE );

StationTallyWrapper::StationTallyWrapper ()
{
    this->electoralConfiguration = NULL;
}

void StationTallyWrapper::init ( ElectoralConfiguration * electoralConfiguration )
{
    this->electoralConfiguration = electoralConfiguration;
}

void StationTallyWrapper::initialize()
{
    clear();
    std::list<Election::Contest * > & contests = electoralConfiguration->getContestList();
    std::list<Election::Contest * >::iterator it;

    for ( it = contests.begin();
            it != contests.end();
            ++it )
    {
        std::string contest_code = ( *it )->getCode();
        StationTally * st = new StationTally();

        st->setContest ( contest_code );
        st->setNumber ( ( *it )->getCustom_code() );
        st->setEmptyVotes ( 0 );
        st->setProcessedVotes ( 0 );
        st->setSelectedVotes ( 0 );
        st->setChecksum ( 0 );
        st->setType ( Type::STATION_TALLY_AUTOMATED );

        stationTallyMap[contest_code] = st;

        std::list<Election::Ballot_option * > & list ( electoralConfiguration->getEligibleOptionsListByContestCode ( contest_code ) );
        std::list<Election::Ballot_option * > & principal ( electoralConfiguration->getEligibleOptionsPrincipalByContestCode ( contest_code ) );
        std::list<Election::Ballot_option * > & standIn ( electoralConfiguration->getEligibleOptionsStandInByContestCode ( contest_code ) );

        registerMap & listMap ( listRegisterMap[contest_code] );
        registerMap & principalMap ( principalRegisterMap[contest_code] );
        registerMap & standInMap ( standInRegisterMap[contest_code] );

        initRegisterMap ( contest_code, listMap, list );
        initRegisterMap ( contest_code, principalMap, principal );
        initRegisterMap ( contest_code, standInMap, standIn );
    }
}

StationTallyWrapper::~StationTallyWrapper()
{
    clear();
}

void StationTallyWrapper::initRegisterMap (
    std::string contestCode
    , registerMap & map
    , std::list<Election::Ballot_option * > & list )
{
    std::list<Election::Ballot_option * >::iterator it;

    for ( it = list.begin();
            it != list.end();
            ++it )
    {
        Register * r = new Register();
        std::string ballotCode = ( *it )->getCode();
        r->setBallotOption ( ballotCode );
        r->setTotal ( 0 );

        r->setCountList ( 0 ); // Category 1
        r->setCountListP ( 0 ); // Category 2
        r->setCountListPS ( 0 ); // Category 3
        r->setCountListS ( 0 ); // Category 4
        r->setCheckSum ( 0 );  // checksum solo para registro L

        map[ballotCode] = r;
        ( allRegisterMap[contestCode] ) [ballotCode] = r;
    }
}

void StationTallyWrapper::cleanRegisterMap ( registerMap & map )
{
    registerMap::iterator it;

    for ( it = map.begin();
            it != map.end();
            ++it )
    {
        Register * r = ( *it ).second;
        delete r;
    }

    map.clear();
}

void StationTallyWrapper::update ( Vote & vote )
{
    bool count = false;

    switch ( vote.getMode() )
    {
        case VoteMode::VOTE_D:

            if ( SAESConfigurationManager::getInstance()->getSaveDemoVotesInTransmissionPackage() )
            {
                count = true;
            }

            break;

        case VoteMode::VOTE_O:
            count = true;
            break;

        default:
        case VoteMode::VOTE_S:
            break;
    }

    if ( !count )
    {
        return;
    }

    Vote::SSequence sequenceS = vote.getSelections();
    Vote::SIterator its;

    for ( its = sequenceS.begin();
            its != sequenceS.end();
            its++ )
    {
        Selection & selection ( *its );
        Selection::BSequence sequenceB = selection.getBallotOptions();
        Selection::BIterator itb;
        std::string contestCode = selection.getContest();
        long countList = 0;
        long countPrincipal = 0;
        long countStandIn = 0;

        for ( itb = sequenceB.begin();
                itb != sequenceB.end();
                itb++ )
        {
            std::string ballotCode = ( *itb );
            Register * r = ( allRegisterMap[contestCode] ) [ballotCode];
            r->setTotal ( r->getTotal().get() + 1 ); // increments L (list) total counter

            if ( countList == 0 && listRegisterMap[contestCode].find ( ballotCode ) != listRegisterMap[contestCode].end() )
            {
                countList++;
            }
            else if ( countPrincipal == 0 && principalRegisterMap[contestCode].find ( ballotCode ) != principalRegisterMap[contestCode].end() )
            {
                countPrincipal++;
            }
            else if ( countStandIn == 0 && standInRegisterMap[contestCode].find ( ballotCode ) != standInRegisterMap[contestCode].end() )
            {
                countStandIn++;
            }
        }

        if ( sequenceB.size() > 0 )
        {
            std::string ballotCode = * ( sequenceB.begin() );
            Election::Ballot_option * option = electoralConfiguration->getBallotOptionListFromBallotOptionCode ( contestCode, ballotCode );
            Register * r = ( allRegisterMap[contestCode] ) [option->getCode()];

            if ( countPrincipal > 0 && countStandIn == 0 )
            {
                r->setCountListP ( r->getCountListP().get() + 1 );
            }
            else if ( countPrincipal == 0 && countStandIn > 0 )
            {
                r->setCountListS ( r->getCountListS().get() + 1 );
            }
            else if ( countPrincipal > 0 && countStandIn > 0 )
            {
                r->setCountListPS ( r->getCountListPS().get() + 1 );
            }
            else if ( countList > 0 )
            {
                r->setCountList ( r->getCountList().get() + 1 );
            }
        }

        StationTally * st = stationTallyMap[contestCode];
        st->setProcessedVotes ( st->getProcessedVotes().get() + 1 );

        if ( sequenceB.size() > 0L )
        {
            st->setSelectedVotes ( st->getSelectedVotes().get() + 1 );
        }
        else
        {
            st->setEmptyVotes ( st->getEmptyVotes().get() + 1 );
        }
    }
}

std::list <StationTally *> StationTallyWrapper::getStationTally ()
{
    std::list <StationTally *> list;
    std::map <std::string, StationTally *>::iterator it;

    for ( it = stationTallyMap.begin();
            it != stationTallyMap.end();
            ++it )
    {
        StationTally * st = ( *it ).second;
        std::string contestCode = st->getContest().get();
        StationTally::RSequence registers;
        registerMap::iterator itr;

        for ( itr = allRegisterMap[contestCode].begin();
                itr != allRegisterMap[contestCode].end();
                ++itr )
        {
            Register * r = ( *itr ).second;
            registers.push_back ( *r );
        }

        st->setRegisters ( registers );
        list.push_back ( st );
    }

    return list;
}


void StationTallyWrapper::clear()
{
    std::map <std::string, StationTally *>::iterator it;

    for ( it = stationTallyMap.begin();
            it != stationTallyMap.end();
            ++it )
    {
        StationTally * st = ( *it ).second;
        delete st;
    }

    fullRegisterMap::iterator itm;

    for ( itm = allRegisterMap.begin();
            itm != allRegisterMap.end();
            ++itm )
    {
        registerMap::iterator itr;
        registerMap & map ( ( *itm ).second );

        for ( itr = map.begin();
                itr != map.end();
                ++itr )
        {
            Register * r = ( *itr ).second;
            delete r;
        }
    }

    stationTallyMap.clear();
    allRegisterMap.clear();
}
