/**
 * @file	TransmissionPackageWrapper.cxx
 * @brief	transmission package wrapper class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */

#include "TransmissionPackageWrapper.hxx"
#include <Configuration/s-a-e-s-common-conf-manager.h>
#include <Configuration/s-a-e-s-conf-manager.h>
#include <Log/SMTTLogManager.h>
#include <Runtime/Environment/DirectoryManager.hxx>
#include <Runtime/Environment/saes-directories.h>
#include <Security/KeyManager.hxx>
#include <System/Exception/CryptoException.hxx>
#include <System/Exception/XmlException.hxx>
#include <System/IO/Directory.h>
#include <System/IO/File.h>
#include <System/Security/Cryptography/MsgDigestUtil.h>
#include <System/Security/Cryptography/PBEKeyDerivatorHMAC.h>
#include <System/Security/Cryptography/RsaUtil.h>
#include <Voting/ElectoralConfiguration.hxx>
#include <Voting/Vote/station-tally-schema.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Voting::Vote;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::Security::Cryptography;
using namespace Smartmatic::System;

ISMTTLog * TransmissionPackageWrapper::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Voting.Vote.TransmissionPackageWrapper", GETTEXT_PACKAGE );

TransmissionPackageWrapper::TransmissionPackageWrapper()
{
    this->electoralConfiguration = NULL;
    this->directories = NULL;
}

TransmissionPackageWrapper::~TransmissionPackageWrapper()
{
    dispose();
}

void TransmissionPackageWrapper::dispose()
{
    votes.clear();
    demoVotes.clear();
}

void TransmissionPackageWrapper::init ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration,
                                        Smartmatic::SAES::Runtime::Environment::SaesDirectories * directories)
{
    dispose();
    this->electoralConfiguration = electoralConfiguration;
    this->directories = directories;
}

void TransmissionPackageWrapper::addVote ( Vote & vote )
{
    switch ( vote.getMode() )
    {
        case VoteMode::VOTE_D:

            if ( SAESConfigurationManager::getInstance()->getSaveDemoVotesInTransmissionPackage() )
            {
                votes.push_back ( vote );
            }

            demoVotes.push_back ( vote );
            break;

        case VoteMode::VOTE_O:
            votes.push_back ( vote );
            break;

        default:
        case VoteMode::VOTE_S:
            break;
    }
}

void TransmissionPackageWrapper::deleteVotes()
{
    votes.clear();
    demoVotes.clear();
}

const std::list<Vote> & TransmissionPackageWrapper::getAllVote()
{
    return votes;
}

const std::list<Vote> & TransmissionPackageWrapper::getAllVoteDemo()
{
    return demoVotes;
}

void TransmissionPackageWrapper::persistTransmissionPackage(const std::list<StationTally*>& stationTallies)
{
    Transmission transmission;
    TransmissionPackage transmissionPackage;
    StationTallyPackages stnTlyPcks;
    StationTallyPackages::Stn_tly_pckSequence sequence;
    StationTallyPackage stnTlyPck;
    ValidVotes validVotes;
    ValidVotes::VSequence vsequence;
    std::list <Vote>::iterator it;

    for ( it = votes.begin();
            it != votes.end();
            ++it )
    {
        Vote & vote ( *it );
        vsequence.push_back ( vote );
    }

    // Calculate checksums
    StationTallyPackage::Stn_tlySequence stnSequence;
    std::list <StationTally *>::const_iterator itst;

    for (itst = stationTallies.begin(); itst != stationTallies.end(); ++itst)
    {
        stnSequence.push_back ( * ( *itst ) );
    }

    stnTlyPck.setStationTally ( stnSequence );
    stnTlyPck.setVoting_device_code ( electoralConfiguration->getVotingDevice()->getFirstVotingDevice().getCode() );
    validVotes.setVotes ( vsequence );
    stnTlyPck.setValidVotes ( validVotes );
    sequence.push_back ( stnTlyPck );
    stnTlyPcks.setStationTallyPackage ( sequence );
    transmissionPackage.setStationTallyPackages ( stnTlyPcks );
    transmission.setTransmissionPackage ( transmissionPackage );

    if ( SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getPersistenceTransmissionPackages() )
    {
        std::ostringstream os;

        try
        {
            Smartmatic::SAES::Voting::VoteTypes::NamespaceInfomap map;
            map[""].name = NAMESPACE_VOTING_VOTE;
            map[""].schema = "";
            serializeTransmission ( os, transmission, map );
        }
        catch ( const ::xsd::cxx::tree::exception< char > & e )
        {
            std::stringstream ss;
            ss << "FATAL - " << __func__ << e.what();
            logger->Debug ( ss.str() );
            throw XmlException ( _ ( "Smartmatic.SAES.Voting.Vote.TransmissionPackageWrapper.ErrorSerializaData" ),
                                 N_ ( "Smartmatic.SAES.Voting.Vote.TransmissionPackageWrapper.ErrorSerializaData" ),
                                 logger );
        }
        catch ( ... )
        {
            std::stringstream ss;
            ss << "FATAL - " << __func__ << __LINE__;
            logger->Debug ( ss.str() );
            throw XmlException ( _ ( "Smartmatic.SAES.Voting.Vote.TransmissionPackageWrapper.ErrorSerializaData" ),
                                 N_ ( "Smartmatic.SAES.Voting.Vote.TransmissionPackageWrapper.ErrorSerializaData" ),
                                 logger );
        }

        std::string data = os.str();

        if ( !SAESCommonConfigurationManager::getInstance()->isUseAsymmetricKeyForTransmissionPackages() )
        {
            logger->Debug ( "Start persistence transmission package with secret-key" );
            persistTransmissionPackageSecretKey ( data );
            logger->Debug ( "Success persistence transmission package with secret-key" );
        }
        else
        {
            logger->Debug ( "Start persistence transmission package with asymmetric-key" );
            persistTransmissionPackageAsymmetricKey ( data );
            logger->Debug ( "Success persistence transmission package with asymmetric-key" );
        }

        persistTransmissionPackageHash();
    }
}

void TransmissionPackageWrapper::persistTransmissionPackageSecretKey ( std::string & data )
{
    std::string devicePathDom = directories->getDevicePath ( BasePathType::CONFIGURATION_DOM );
    std::string devicePathFlash = directories->getDevicePath ( BasePathType::CONFIGURATION_FLASH );
    std::string fileDom = directories->getTransmissionPackagesFileFullPath ( BasePathType::CONFIGURATION_DOM );
    std::string fileFlash = directories->getTransmissionPackagesFileFullPath ( BasePathType::CONFIGURATION_FLASH );

    if ( Directory::getInstance()->DirectoryExists ( devicePathDom ) )
    {
        Smartmatic::SAES::Security::Encryption::getInstance()->saveXmlFile ( data, fileDom,
                Smartmatic::SAES::Security::Encryption::EXCHANGE_FILE );
        File::getInstance()->copyFile ( fileDom, fileFlash );
    }
    else if ( Directory::getInstance()->DirectoryExists ( devicePathFlash ) )
    {
        Smartmatic::SAES::Security::Encryption::getInstance()->saveXmlFile ( data, fileFlash,
                Smartmatic::SAES::Security::Encryption::EXCHANGE_FILE );
        File::getInstance()->copyFile ( fileFlash, fileDom );
    }
}

void TransmissionPackageWrapper::persistTransmissionPackageAsymmetricKey ( std::string & data )
{
    SafeBuffer dataTransmissionPackage;
    SafeBuffer asymmetricKeyCrypto;
    //---------------------------------------------------------
    bool isSignedData = Smartmatic::SAES::Security::Encryption::getInstance()->signedData (
                            Smartmatic::SAES::Security::Encryption::EXCHANGE_FILE );

    if ( isSignedData )
    {
        Smartmatic::SAES::Security::Encryption::getInstance()->signedXml (
            data,
            Smartmatic::SAES::Security::Encryption::EXCHANGE_FILE );
    }

    SafeBuffer dataIn ( ( unsigned char * ) data.c_str(), data.length() );
    //---------------------------------------------------------
    bool isCryptoFile = Smartmatic::SAES::Security::Encryption::getInstance()->cryptoData (
                            Smartmatic::SAES::Security::Encryption::EXCHANGE_FILE );

    if ( isCryptoFile )
    {
        PBEKeyDerivatorHMAC pbe;

        SafeBuffer passPhrase ( Smartmatic::System::GuidClass::NewGuid().ToString() );
        SafeBuffer salt ( Smartmatic::System::GuidClass::NewGuid().ToString() );
        SafeBuffer asymmetricKey;

        pbe.SetSalt ( salt );
        pbe.GenerateKey ( passPhrase, asymmetricKey, MsgDigestUtil::getDefaultGenerateKeyDigest() );

        //---------------------------------------------------------
        std::string certPath = directories->getTransmissionPackagesCrtFileFullPath();

        if ( !File::getInstance()->canOpenFile ( certPath ) )
        {
            logger->Debug ( "Fatal - file cert not found : " + certPath );
            throw CryptoException (
                _ ( "Smartmatic.SAES.Voting.Vote.TransmissionPackageWrapper.ErrorCertNotFound" ),
                N_ ( "Smartmatic.SAES.Voting.Vote.TransmissionPackageWrapper.ErrorCertNotFound" ),
                logger );
        }

        RsaUtil util;
        util.SetCertificate ( certPath, CertUtil::CertFormat_PEM );
        SafeBuffer dataToCrypto;

        try
        {
            dataToCrypto.AddData ( asymmetricKey );
            asymmetricKeyCrypto = util.PublicEncrypt ( dataToCrypto );
        }
        catch ( ... )
        {
            logger->Debug ( "Fatal - error cert." );
            throw CryptoException (
                _ ( "Smartmatic.SAES.Voting.Vote.TransmissionPackageWrapper.ErrorCert" ),
                N_ ( "Smartmatic.SAES.Voting.Vote.TransmissionPackageWrapper.ErrorCert" ),
                logger );
        }

        //---------------------------------------------------------
        Smartmatic::System::Security::Crypto crypto;
        crypto.init ( Smartmatic::SAES::Common::Security::KeyManager::getInstance()->getKeySize(),
                      Smartmatic::SAES::Common::Security::KeyManager::getInstance()->getModeType(), true );
        crypto.encryptBufferToBufferAndCreateIV ( dataIn, dataTransmissionPackage, ( unsigned char * ) asymmetricKey.GetData());
    }
    else
    {
        dataTransmissionPackage.AddData ( dataIn );
    }

    //---------------------------------------------------------
    std::string devicePathDom = directories->getDevicePath ( BasePathType::CONFIGURATION_DOM );
    std::string devicePathFlash = directories->getDevicePath ( BasePathType::CONFIGURATION_FLASH );
    std::string fileTransDom = directories->getTransmissionPackagesFileFullPath ( BasePathType::CONFIGURATION_DOM );
    std::string fileTransKeyDom = directories->getTransmissionPackagesKeyFileFullPath ( BasePathType::CONFIGURATION_DOM );
    std::string fileTransFlash = directories->getTransmissionPackagesFileFullPath ( BasePathType::CONFIGURATION_FLASH );
    std::string fileTransKeyFlash = directories->getTransmissionPackagesKeyFileFullPath ( BasePathType::CONFIGURATION_FLASH );

    if ( Directory::getInstance()->DirectoryExists ( devicePathDom ) )
    {
        File::getInstance()->saveBufferToFile ( dataTransmissionPackage, fileTransDom );
        File::getInstance()->saveBufferToFile ( asymmetricKeyCrypto, fileTransKeyDom );
        File::getInstance()->copyFile ( fileTransDom, fileTransFlash );
        File::getInstance()->copyFile ( fileTransKeyDom, fileTransKeyFlash );
    }
    else if ( Directory::getInstance()->DirectoryExists ( devicePathFlash ) )
    {
        File::getInstance()->saveBufferToFile ( dataTransmissionPackage, fileTransFlash );
        File::getInstance()->saveBufferToFile ( asymmetricKeyCrypto, fileTransKeyFlash );
        File::getInstance()->copyFile ( fileTransFlash, fileTransDom );
        File::getInstance()->copyFile ( fileTransKeyFlash, fileTransKeyDom );
    }
}

void TransmissionPackageWrapper::persistTransmissionPackageHash()
{
    std::string devicePathDom = directories->getDevicePath ( BasePathType::CONFIGURATION_DOM );
    std::string devicePathFlash = directories->getDevicePath ( BasePathType::CONFIGURATION_FLASH );
    std::string fileDOM = directories->getTransmissionPackagesFileFullPath ( BasePathType::CONFIGURATION_DOM );
    std::string fileFlash = directories->getTransmissionPackagesFileFullPath ( BasePathType::CONFIGURATION_FLASH );
    std::string fileHashDom = directories->getTransmissionPackagesHashFileFullPath ( BasePathType::CONFIGURATION_DOM );
    std::string fileHashFlash = directories->getTransmissionPackagesHashFileFullPath ( BasePathType::CONFIGURATION_FLASH );

    if ( Directory::getInstance()->DirectoryExists ( devicePathDom ) )
    {
        SafeBuffer bufferFile;
        File::getInstance()->getHashFile ( fileDOM, bufferFile );
        std::string hash = bufferFile.GetBase64Data();
        Smartmatic::SAES::Security::Encryption::getInstance()->cryptoStringToFile ( hash, fileHashDom,
                Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY );
        File::getInstance()->copyFile ( fileHashDom, fileHashFlash );
    }
    else if ( Directory::getInstance()->DirectoryExists ( devicePathFlash ) )
    {
        SafeBuffer bufferFile;
        File::getInstance()->getHashFile ( fileFlash, bufferFile );
        std::string hash = bufferFile.GetBase64Data();
        Smartmatic::SAES::Security::Encryption::getInstance()->cryptoStringToFile ( hash, fileHashFlash,
                Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY );
        File::getInstance()->copyFile ( fileHashFlash, fileHashDom );
    }
}

bool TransmissionPackageWrapper::validate()
{
    if ( !SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getPersistenceTransmissionPackages() )
    {
        return true;
    }

    bool validate = false;

    try
    {
        bool validateKey = true;
        std::string dom = directories->getTransmissionPackagesHashFileFullPath ( BasePathType::CONFIGURATION_DOM );
        std::string flash = directories->getTransmissionPackagesHashFileFullPath ( BasePathType::CONFIGURATION_FLASH );
        std::string fileDom = directories->getTransmissionPackagesFileFullPath ( BasePathType::CONFIGURATION_DOM );
        std::string fileFlash = directories->getTransmissionPackagesFileFullPath ( BasePathType::CONFIGURATION_FLASH );
        std::string hashDom;
        std::string hashFlash;
        SafeBuffer bufferFile;
        File::getInstance()->getHashFile ( fileDom, bufferFile );
        std::string hash = bufferFile.GetBase64Data();
        logger->Debug ( "hash: " + hash );
        Smartmatic::SAES::Security::Encryption::getInstance()->decryptoFileToString (
            dom,
            hashDom,
            Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY );
        logger->Debug ( "hashDom: " + hashDom );
        Smartmatic::SAES::Security::Encryption::getInstance()->decryptoFileToString (
            flash,
            hashFlash,
            Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY );
        logger->Debug ( "hashFlash: " + hashFlash );
        bool isCryptoFile = Smartmatic::SAES::Security::Encryption::getInstance()->cryptoData (
                                Smartmatic::SAES::Security::Encryption::EXCHANGE_FILE );

        if ( isCryptoFile && SAESCommonConfigurationManager::getInstance()->isUseAsymmetricKeyForTransmissionPackages() )
        {
            std::string domKey = directories->getTransmissionPackagesKeyFileFullPath ( BasePathType::CONFIGURATION_DOM );
            std::string flashKey = directories->getTransmissionPackagesKeyFileFullPath ( BasePathType::CONFIGURATION_FLASH );
            validateKey = File::getInstance()->compareFiles ( domKey, flashKey );
        }

        validate = validateKey &&
                   File::getInstance()->compareFiles ( dom, flash ) &&
                   File::getInstance()->compareFiles ( fileDom, fileFlash ) &&
                   hashDom == hashFlash &&
                   hashDom == hash;
    }
    catch ( ... )
    {
        logger->Debug ( "Fatal - " + std::string ( __func__ ) + " Error decrypto data." );
    }

    return validate;
}

void TransmissionPackageWrapper::deleteFiles()
{
    std::string transmissionPackageDom = directories->getTransmissionPackagesFileFullPath ( BasePathType::CONFIGURATION_DOM );
    std::string transmissionPackageHashDom = directories->getTransmissionPackagesHashFileFullPath ( BasePathType::CONFIGURATION_DOM );
    std::string transmissionPackageKeyDom = directories->getTransmissionPackagesKeyFileFullPath ( BasePathType::CONFIGURATION_DOM );
    File::getInstance()->DeleteFile ( transmissionPackageDom );
    File::getInstance()->DeleteFile ( transmissionPackageHashDom );
    File::getInstance()->DeleteFile ( transmissionPackageKeyDom );
    std::string transmissionPackageFlash = directories->getTransmissionPackagesFileFullPath ( BasePathType::CONFIGURATION_FLASH );
    std::string transmissionPackageHashFlash = directories->getTransmissionPackagesHashFileFullPath ( BasePathType::CONFIGURATION_FLASH );
    std::string transmissionPackageKeyFlash = directories->getTransmissionPackagesKeyFileFullPath ( BasePathType::CONFIGURATION_FLASH );
    File::getInstance()->DeleteFile ( transmissionPackageFlash );
    File::getInstance()->DeleteFile ( transmissionPackageHashFlash );
    File::getInstance()->DeleteFile ( transmissionPackageKeyFlash );
    deleteVotes();
}
