/**
 * @file	VoteWrapper.hxx
 * @brief	vote wrapper class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */

#ifndef _VOTE_WRAPPER_H_
#define _VOTE_WRAPPER_H_

#include <Voting/Vote/vote-schema.hxx>
#include <string>
#include <Voting/PBReports/pvform-schema.hxx>
#include <Voting/ElectoralConfiguration.hxx>
#include <System/SafeBuffer.h>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }
    namespace SAES
    {
        namespace Voting
        {
            class OperationStatus;

            namespace Vote
            {
                /**
                 * @class	VoteWrapper
                 * @brief	Vote wrapper.
                 * @author	Eduardo.campione
                 * @date	28/04/2011
                 */

                class VoteWrapper
                {
                    protected:

                        /**
                         * @fn	VoteWrapper::VoteWrapper();
                         * @brief	Default constructor.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         */

                        VoteWrapper();

                    public:

                        /**
                         * @fn	virtual VoteWrapper::~VoteWrapper();
                         * @brief	Finaliser.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         */

                        virtual ~VoteWrapper();

                        /**
                         * @fn	static std::string VoteWrapper::generateCode(Vote & vote);
                         * @brief	Method generate code.
                         * @author	Yadickson Soto yadickson.soto@smartmatic.com
                         * @date	07/05/2011
                         * @param	vote	object vote.
                         * @return	vote code.
                         */

                        static std::string generateCode ( Vote & vote );

                        /**
                         * @fn	static bool VoteWrapper::voteFileToVote(std::string & file, Vote & vote);
                         * @brief	Vote file to string.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @param [in,out]	file	The file.
                         * @param	vote						The vote.
                         * @return	The vote into a string.
                         */

                        static bool voteFileToVote ( std::string & file, Vote & vote );

                        /**
                         * @brief	Vote to string.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @throws      XmlException  Thrown when an XML error condition occurs.
                         * @param	vote          The vote.
                         * @param	outStringVote The out string vote.
                         */
                        static void voteToString ( const Vote & vote, std::string & outStringVote );

                        /**
                         * @fn	static Smartmatic::SAES::Voting::PBReports::PVVote VoteWrapper::voteToPVVote( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration,
                         * 			Vote & vote);
                         * @brief	transform Vote to PVVote.
                         * @author	Yadickson Soto yadickson.soto@smartmatic.com
                         * @date	06/05/2011
                         * @param	electoralConfiguration	electoral configuration.
                         * @param	vote										The vote.
                         * @return	The vote into a PVVote.
                         */

                        static Smartmatic::SAES::Voting::PBReports::PVVote voteToPVVote (
                            Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration,
                            Vote & vote );

                        /**
                         * @fn	static Smartmatic::SAES::Voting::PBReports::PVCandidate VoteWrapper::getCandidateInfo( Smartmatic::SAES::Voting::Election::Eligible_option & eligibleOption,
                         * 			Smartmatic::SAES::Voting::Election::Ballot_option & ballotOption);
                         * @brief	Gets candidate information.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @param	eligibleOption	The eligible option.
                         * @param	ballotOption		The ballot option.
                         * @return	The candidate information.
                         */
                        static Smartmatic::SAES::Voting::PBReports::PVCandidate getCandidateInfo (
                            Smartmatic::SAES::Voting::Election::Eligible_option & eligibleOption,
                            Smartmatic::SAES::Voting::Election::Ballot_option & ballotOption );

                        /**
                         * @fn	static void VoteWrapper::getPartyInfo( Smartmatic::SAES::Voting::Election::Ballot_option & ballotOption,
                         * 			std::string & pvPartyAbreviation, std::string & pvPartyName);
                         * @brief	Gets party information.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @param	ballotOption				The ballot option.
                         * @param	pvPartyAbreviation	The pv party abreviation.
                         * @param	pvPartyName					Name of the pv party.
                         */
                        static void getPartyInfo (
                            Smartmatic::SAES::Voting::Election::Ballot_option & ballotOption,
                            std::string & pvPartyAbreviation,
                            std::string & pvPartyName );

                        /**
                         * @fn	static bool VoteWrapper::validateVote( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration,
                         * 			Vote & vote);
                         * @brief	validate vote.
                         * @author	Yadickson Soto yadickson.soto@smartmatic.com
                         * @date	07/05/2011
                         * @param	electoralConfiguration	electoral configuration.
                         * @param	vote										The vote.
                         * @return	true is validate vote.
                         */
                        static bool validateVote (
                            Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration,
                            Vote & vote );

                        /**
                         * @fn	static bool VoteWrapper::validateVoteCode(Vote & vote);
                         * @brief	validate vote code.
                         * @author	Yadickson Soto yadickson.soto@smartmatic.com
                         * @date	07/05/2011
                         * @param	vote	The vote.
                         * @return	true is validate vote.
                         */
                        static bool validateCbcMac ( Vote & vote );

                        /**
                         * @fn	static bool VoteWrapper::validateLogicalVoterGroupVote( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration,
                         * 			Vote & vote);
                         * @brief	validate logical voter group vote.
                         * @author	Yadickson Soto yadickson.soto@smartmatic.com
                         * @date	07/05/2011
                         * @param	electoralConfiguration	electoral configuration.
                         * @param	vote										The vote.
                         * @return	true is validate vote.
                         */
                        static bool validateLogicalVoterGroupVote (
                            Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration,
                            Vote & vote );

                        /**
                         * @fn	static bool VoteWrapper::validateVotingDeviceCodeVote( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration,
                         * 			Vote & vote);
                         * @brief	validate voting device code vote.
                         * @author	Yadickson Soto yadickson.soto@smartmatic.com
                         * @date	07/05/2011
                         * @param	electoralConfiguration	electoral configuration.
                         * @param	vote										The vote.
                         * @return	true is validate vote.
                         */
                        static bool validateVotingDeviceCodeVote (
                            Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration,
                            Vote & vote );

                        /**
                         * @brief	Pack vote for qr.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @throws	CryptoException Thrown when a crypto error condition occurs.
                         * @throws      SignedException Thrown when a signed error condition occurs.
                         * @param       electoralConfiguration If non-null, the electoral configuration.
                         * @param       vote                   The vote.
                         * @param       outBuffer              Buffer for out data.
                         * @param       deflate                (optional) the deflate.
                         */
                        static void packVoteForQR(
                            Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration,
                            const Vote & vote,
                            Smartmatic::System::SafeBuffer & outBuffer,
                            bool deflate = true);

                        /**
                         * @brief Unpack vote from qr.
                         * @author Youssef.tourkmani
                         * @date   31/10/2012
                         * @throws CryptoException              Thrown when a crypto error condition occurs.
                         * @throws SignedException              Thrown when a signed error condition occurs.
                         * @throws QRVoteInvalidFormatException Thrown when a qr vote invalid format error condition occurs.
                         * @throws QRVoteSignatureException     Thrown when a qr vote signature error condition occurs.
                         * @param  electoralConfiguration       If non-null, the electoral configuration.
                         * @param  intBuffer                    Buffer for int data.
                         * @param  outVote                      The out vote.
                         * @param  inflate                      (optional) the inflate.
                         * @return true if it succeeds, false if it fails.
                         */
                        static bool unpackVoteFromQR(
                            Smartmatic::SAES::Voting::ElectoralConfiguration* electoralConfiguration,
                            Smartmatic::System::SafeBuffer& intBuffer,
                            Vote& outVote,
                            bool inflate = true);

                        /**
                         * @fn	static void VoteWrapper::getReducedXmlVoteString(Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration,
                         * 			Vote vote, std::string& outStringVote);
                         * @brief	Gets reduced XML vote string.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @param [in,out]	electoralConfiguration	If non-null, the electoral configuration.
                         * @param	vote															The vote.
                         * @param [in,out]	outStringVote						The out string vote.
                         */
                        static void getReducedXmlVoteString ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration, Vote vote, std::string & outStringVote );

                        /**
                         * @brief  Gets expanded XML vote.
                         * @author Youssef.tourkmani
                         * @date   31/10/2012
                         * @throws QRVoteInvalidFormatException Thrown when a qr vote invalid format error condition occurs.
                         * @param  electoralConfiguration       If non-null, the electoral configuration.
                         * @param  inOutVote                    The in out vote.
                         */
                        static void getExpandedXmlVote ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration, Vote & inOutVote );

                        /**
                         * @brief  Gets size optimized vote string.
                         * @author Youssef.tourkmani
                         * @date   31/10/2012
                         * @throws CryptoException        Thrown when a crypto error condition occurs.
                         * @param  electoralConfiguration If non-null, the electoral configuration.
                         * @param  vote                   The vote.
                         * @param  outStringVote          The out string vote.
                         */
                        static void getSizeOptimizedVoteString ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration, const Vote & vote, std::string & outStringVote );

                        /**
                         * @brief  Gets vote from size optimized string.
                         * @author Youssef.tourkmani
                         * @date   31/10/2012
                         * @throws CryptoException              Thrown when a crypto error condition occurs.
                         * @throws QRVoteInvalidFormatException Thrown when a qr vote invalid format error condition occurs.
                         * @throws QRVoteSignatureException     Thrown when a qr vote signature error condition occurs.
                         * @param  electoralConfiguration       If non-null, the electoral configuration.
                         * @param  stringVote                   The string vote.
                         * @param  outVote                      The out vote.
                         */
                        static void getVoteFromSizeOptimizedString ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration, std::string & stringVote, Vote & outVote );

                        /**
                         * @fn	static Vote * VoteWrapper::createRandomVote(Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration, Smartmatic::SAES::Voting::OperationStatus * operationStatus);
                         * @brief	Method create random contest group vote.
                         * @author	Youssef.tourkmani
                         * @date	16/10/2012
                         * @param	electoralConfiguration	electoral configuration.
                         * @param	operationStatus The operation status.
                         * @return	vote result.
                         */
                        static Vote * createRandomVote ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration, Smartmatic::SAES::Voting::OperationStatus * operationStatus );
                        /**
                         * @fn	static Vote * VoteWrapper::createEmptyRandomVote(Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration, Smartmatic::SAES::Voting::OperationStatus * operationStatus);
                         * @brief	Method create empty random contest group vote.
                         * @author	Youssef.tourkmani
                         * @date	16/10/2012
                         * @param	electoralConfiguration	electoral configuration.
                         * @param	operationStatus The operation status.
                         * @return	vote result.
                         */
                        static Vote * createEmptyRandomVote ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration, Smartmatic::SAES::Voting::OperationStatus * operationStatus );
                        /**
                         * @fn	static Vote * VoteWrapper::createFullRandomVote(Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration, Smartmatic::SAES::Voting::OperationStatus * operationStatus);
                         * @brief	Method create full random contest group vote.
                         * @author	Youssef.tourkmani
                         * @date	16/10/2012
                         * @param	electoralConfiguration	electoral configuration.
                         * @param	operationStatus The operation status.
                         * @return	vote result.
                         */
                        static Vote * createFullRandomVote ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration, Smartmatic::SAES::Voting::OperationStatus * operationStatus );
                        /**
                         * @fn	static std::string getFileName ( const Vote & vote );
                         * @brief	Getter method: Relationship vote code to file name vote.
                         * @author	Yadickson Soto
                         * @date	05/06/2013
                         * @param	vote vote to getter file name.
                         * @return	file name.
                         */
                        static std::string getFileName ( const Vote & vote );

                    private:
                        /**
                         * @fn	static Vote * VoteWrapper::createRandomVote(Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration,
                         * 			bool full, bool empty);
                         * @brief	Method create random contest group vote.
                         * @author	Youssef.tourkmani
                         * @date	16/10/2012
                         * @param	electoralConfiguration	electoral configuration.
                         * @param	operationStatus The operation status.
                         * @param	full										full vote, is false them normal vote.
                         * @param	empty										empty vote, is true them empty contest.
                         * @return	vote result.
                         */
                        static Vote * createRandomVote ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration, Smartmatic::SAES::Voting::OperationStatus * operationStatus, bool full, bool empty );

                        static Smartmatic::Log::ISMTTLog * logger; ///< The logger
                };
            }
        }
    }
}

#endif /* _VOTE_WRAPPER_H_ */
