/**
 * @file	VotingDeviceWrapper.hxx
 * @brief	voting device wrapper class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */

#ifndef __VOTING_DEVICE_WRAPPER_H__
#define __VOTING_DEVICE_WRAPPER_H__

#include <string>
#include <list>
#include <Voting/VotingDevice/voting-devices-schema.hxx>
#include <System/Exception/VotingDeviceException.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Voting
        {
            namespace VotingDevice
            {
                /**
                 * @class	VotingDeviceWrapper
                 * @brief	Voting device wrapper.
                 * @author	Eduardo.campione
                 * @date	28/04/2011
                 */

                class VotingDeviceWrapper
                {
                    public:

                        /**
                         * @fn	VotingDeviceWrapper::VotingDeviceWrapper(Voting_devices & votingDevices);
                         * @brief	Constructor.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @param	votingDevices	The voting devices.
                         */

                        explicit VotingDeviceWrapper ( Voting_devices & votingDevices );

                        /**
                         * @fn	virtual VotingDeviceWrapper::~VotingDeviceWrapper();
                         * @brief	Finaliser.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         */

                        virtual ~VotingDeviceWrapper();

                        /**
                         * @fn	Voting_devices & VotingDeviceWrapper::getVotingDevices ();
                         * @brief	Gets the voting devices.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @return	The voting devices.
                         */

                        Voting_devices & getVotingDevices ();

                        /**
                         * @fn	Voting_device & VotingDeviceWrapper::getFirstVotingDevice() throw (Smartmatic::System::Exception::VotingDeviceException);
                         * @brief	Gets the first voting device.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @exception	Smartmatic::System::Exception::VotingDeviceException	Thrown when a voting device
                         * 																																	error condition occurs.
                         * @return	The first voting device.
                         */

                        Voting_device & getFirstVotingDevice()
                        throw ( Smartmatic::System::Exception::VotingDeviceException );

                        /**
                         * @fn	Voting_device & VotingDeviceWrapper::getVotingDevice( std::string codeVotingDevice ) throw (Smartmatic::System::Exception::VotingDeviceException);
                         * @brief	Gets a voting device.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @exception	Smartmatic::System::Exception::VotingDeviceException	Thrown when a voting device
                         * 																																	error condition occurs.
                         * @param	codeVotingDevice	The code voting device.
                         * @return	The voting device.
                         */

                        Voting_device & getVotingDevice ( std::string codeVotingDevice )
                        throw ( Smartmatic::System::Exception::VotingDeviceException );

                        /**
                         * @fn	Operation_modes & VotingDeviceWrapper::getOperationModes ( Voting_device & votingDevice );
                         * @brief	Gets an operation modes.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @param	votingDevice	The voting device.
                         * @return	The operation modes.
                         */

                        Operation_modes & getOperationModes ( Voting_device & votingDevice );

                        /**
                         * @fn	Voting_booths & VotingDeviceWrapper::getVotingBooths () throw (Smartmatic::System::Exception::VotingDeviceException);
                         * @brief	Gets voting booths.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @exception	Smartmatic::System::Exception::VotingDeviceException	Thrown when a voting device
                         * 																																	error condition occurs.
                         * @return	The voting booths.
                         */
                        Voting_booths & getVotingBooths ()
                        throw ( Smartmatic::System::Exception::VotingDeviceException );

                        /**
                         * @fn	Voting_booths & VotingDeviceWrapper::getVotingBooths ( Voting_device & votingDevice );
                         * @brief	Gets a voting booths.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @param	votingDevice	The voting device.
                         * @return	The voting booths.
                         */

                        Voting_booths & getVotingBooths ( Voting_device & votingDevice );

                        /**
                         * @fn	Logical_voter_groups & VotingDeviceWrapper::getLogicalVoterGroups() throw (Smartmatic::System::Exception::VotingDeviceException);
                         * @brief	Gets logical voter groups.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @exception	Smartmatic::System::Exception::VotingDeviceException	Thrown when a voting device
                         * 																																	error condition occurs.
                         * @return	The logical voter groups.
                         */
                        Logical_voter_groups & getLogicalVoterGroups()
                        throw ( Smartmatic::System::Exception::VotingDeviceException );

                        /**
                         * @fn	Logical_voter_groups & VotingDeviceWrapper::getLogicalVoterGroups( Voting_device & votingDevice );
                         * @brief	Gets a logical voter groups.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @param	votingDevice	The voting device.
                         * @return	The logical voter groups.
                         */

                        Logical_voter_groups & getLogicalVoterGroups ( Voting_device & votingDevice );

                        /**
                         * @fn	Logical_voter_group & VotingDeviceWrapper::getLogicalVoterGroup( Voting_device & votingDevice,
                         * 			std::string codeLVG ) throw (Smartmatic::System::Exception::VotingDeviceException);
                         * @brief	Gets a logical voter group.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @exception	Smartmatic::System::Exception::VotingDeviceException	Thrown when a voting device
                         * 																																	error condition occurs.
                         * @param	votingDevice	The voting device.
                         * @param	codeLVG				The code lvg.
                         * @return	The logical voter group.
                         */

                        Logical_voter_group & getLogicalVoterGroup ( Voting_device & votingDevice, std::string codeLVG )
                        throw ( Smartmatic::System::Exception::VotingDeviceException );

                        /**
                         * @fn	Logical_voter_group & VotingDeviceWrapper::getLogicalVoterGroup(std::string codeLVG ) throw (Smartmatic::System::Exception::VotingDeviceException);
                         * @brief	Gets a logical voter group.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @exception	Smartmatic::System::Exception::VotingDeviceException	Thrown when a voting device
                         * 																																	error condition occurs.
                         * @param	codeLVG	The code lvg.
                         * @return	The logical voter group.
                         */

                        Logical_voter_group & getLogicalVoterGroup ( const std::string& codeLVG )
                        throw ( Smartmatic::System::Exception::VotingDeviceException );

                        /**
                         * @fn	Logical_voter_group & VotingDeviceWrapper::getLogicalVoterGroupByContestGroupCode( std::string contestGroupCode ) throw (Smartmatic::System::Exception::VotingDeviceException);
                         * @brief	Gets a logical voter group.
                         * @author	Yadickson Soto yadickson.soto@smartmatic.com
                         * @date	28/04/2011
                         * @exception	Smartmatic::System::Exception::VotingDeviceException	Thrown when a voting device
                         * 																																	error condition occurs.
                         * @param	contestGroupCode	The contest group code.
                         * @return	The logical voter group.
                         */

                        Logical_voter_group & getLogicalVoterGroupByContestGroupCode ( std::string contestGroupCode )
                        throw ( Smartmatic::System::Exception::VotingDeviceException );

                        /**
                         * @fn	std::list < Custom_attribute_definition > VotingDeviceWrapper::getCustomAttributeDefinitionList( Machine_type machineType = Machine_type::VOTING_DEVICES_VM ) throw (Smartmatic::System::Exception::VotingDeviceException);
                         * @brief	Gets custom attribute definition list.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @exception	Smartmatic::System::Exception::VotingDeviceException	Thrown when a voting device
                         * 																																	error condition occurs.
                         * @param	machineType	(optional) type of the machine.
                         * @return	The custom attribute definition list.
                         */
                        std::list < Custom_attribute_definition > getCustomAttributeDefinitionList (
                            Machine_type machineType = Machine_type::VOTING_DEVICES_VM )
                        throw ( Smartmatic::System::Exception::VotingDeviceException );

                        /**
                         * @fn	std::list < Custom_attribute_definition > VotingDeviceWrapper::getCustomAttributeDefinitionList( Voting_device & votingDevice,
                         * 			Machine_type machineType = Machine_type::VOTING_DEVICES_VM );
                         * @brief	Gets a custom attribute definition list.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @param	votingDevice	The voting device.
                         * @param	machineType 	Type of the machine.
                         * @return	The custom attribute definition list.
                         */

                        std::list < Custom_attribute_definition > getCustomAttributeDefinitionList (
                            Voting_device & votingDevice,
                            Machine_type machineType = Machine_type::VOTING_DEVICES_VM );

                        /**
                         * @fn	std::list < Custom_attribute > VotingDeviceWrapper::getCustomAttributeList( Logical_voter_group & lvg);
                         * @brief	Gets a custom attribute list.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @param	lvg	The lvg.
                         * @return	The custom attribute list.
                         */

                        std::list < Custom_attribute > getCustomAttributeList (
                            Logical_voter_group & lvg );

                        /**
                         * @fn	void VotingDeviceWrapper::getLogicalVoterGroupList( Voting_device & votingDevice,
                         * 			std::list < Logical_voter_group * > & list );
                         * @brief	Gets a logical voter group list.
                         * @author	Yadickson Soto
                         * @date	23/02/2012
                         * @param	votingDevice	The voting device.
                         * @param	list					result the logical voter group list.
                         */

                        void getLogicalVoterGroupList ( Voting_device & votingDevice, std::list < Logical_voter_group * > & list );

                        /**
                         * @fn	bool VotingDeviceWrapper::IsLogicalVoterGroupPresent(std::string lvgCode);
                         * @brief	Test whether the Logical Voter Group is registered in the ocnfiguration.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @param [in]	lvgCode	LVG Code.
                         * @return	true whether the LVG is present in the configuration.
                         */
                        bool IsLogicalVoterGroupPresent ( const std::string& lvgCode );

                        /**
                         * @fn	std::list < Custom_attribute > VotingDeviceWrapper::getCustomAttributeListFromLVGCode(std::string lvgCode) throw (Smartmatic::System::Exception::VotingDeviceException);
                         * @brief	Gets a logical voter group list from a LVG code.
                         * @author	Eduardo.campione
                         * @date	28/04/2011
                         * @exception	Smartmatic::System::Exception::VotingDeviceException	Thrown when a voting device
                         * 																																	error condition occurs.
                         * @param [in]	lvgCode	LVG Code.
                         * @return	The custom attribute list.
                         */
                        std::list < Custom_attribute > getCustomAttributeListFromLVGCode ( const std::string& lvgCode )
                        throw ( Smartmatic::System::Exception::VotingDeviceException );

                        /**
                         * @fn	bool VotingDeviceWrapper::GetCustomAttributeDefinition(std::string caCode,
                         * 			Custom_attribute_definition& customAttrDef );
                         * @brief	Return the custom attribute definition belongiing to the specified code.
                         * @author	Freddy.Gomez
                         * @date	12/05/2011
                         * @param [in]	caCode			 	Custom attribute Code.
                         * @param [out]	customAttrDef	Custom attribute Definition.
                         * @return	true whether the Custom Attribute is found, otherwise false.
                         */
                        bool GetCustomAttributeDefinition ( const std::string& caCode, Custom_attribute_definition & customAttrDef );

                        /**
                         * @fn	bool VotingDeviceWrapper::IsLanguagesPresent ();
                         * @brief	Query if this object is languages present.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @return	true if languages present, false if not.
                         */
                        bool IsLanguagesPresent ();
                        /**
                         * @fn	void VotingDeviceWrapper::setLanguages (Smartmatic::SAES::Voting::Election::Languages & lang);
                         * @brief	Sets the languages.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @param [in,out]	lang	The language.
                         */
                        void setLanguages ( Smartmatic::SAES::Voting::Election::Languages & lang );
                        /**
                         * @fn	Smartmatic::SAES::Voting::Election::Languages & VotingDeviceWrapper::getLanguages ();
                         * @brief	Gets the languages.
                         * @author	Youssef.tourkmani
                         * @date	31/10/2012
                         * @return	The languages.
                         */
                        Smartmatic::SAES::Voting::Election::Languages & getLanguages ();

                        /**
                         * @fn	void VotingDeviceWrapper::getRegionCategoryList(std::list <Smartmatic::SAES::Voting::VotingDevice::Region_category> & list) throw (Smartmatic::System::Exception::VotingDeviceException);
                         * @brief	method getter all region category.
                         * @author	Yadickson Soto
                         * @date	04/04/2012
                         * @exception	Smartmatic::System::Exception::VotingDeviceException	Thrown when a voting device
                         * 																																	error condition occurs.
                         * @param	list	all region category.
                         */
                        void getRegionCategoryList ( std::list <Smartmatic::SAES::Voting::VotingDevice::Region_category> & list )
                        throw ( Smartmatic::System::Exception::VotingDeviceException );


                    private:
                        Voting_devices votingDevices;	///< The voting devices
                        static Smartmatic::Log::ISMTTLog * logger;	///< The logger
                };
            }
        }
    }
}

#endif /*__VOTING_DEVICE_WRAPPER_H__*/
