/**
 * @file VotingResultsWrapper.cxx
 * @brief Body of voting results class, responsible for the generation of voting results by nationality.
 * @date 27/08/2013
 * @author Yadickson Soto
 */

#include "VotingResultsWrapper.hxx"
#include <Configuration/saes-language-controller.h>
#include <Voting/set-language-functionality.h>
#include <Environment/Configurations/base-language-controller.h>
#include <Resources/election-string-resources-manager.h>
#include <Voting/voting-results-schema.hxx>
#include <Security/Encryption.hxx>
#include <System/IO/File.h>
#include <System/IO/Path.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Voting/PBReports/VoteStatisticsWrapper.hxx>
#include <System/Exception/XmlException.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::SAES::Security;
using namespace Smartmatic::Environment;

VotingResultsWrapper * VotingResultsWrapper::instance = NULL;
ISMTTLog * VotingResultsWrapper::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Voting.VotingResultsWrapper", GETTEXT_PACKAGE );

VotingResultsWrapper::VotingResultsWrapper ()
{
    electoralConfiguration = NULL;
    directories = NULL;
    voteStatistics = NULL;
    recountVoteStatistics = NULL;
}

VotingResultsWrapper::~VotingResultsWrapper()
{
    instance = NULL;
}
void VotingResultsWrapper::init ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration,
                                  Smartmatic::SAES::Runtime::Environment::SaesDirectories * directories,
                                  Smartmatic::SAES::Voting::PBReports::VoteStatisticsWrapper * voteStatistics,
                                  Smartmatic::SAES::Voting::RecountVoteStatistics * recountVoteStatistics )
{
    this->electoralConfiguration = electoralConfiguration;
    this->directories = directories;
    this->voteStatistics = voteStatistics;
    this->recountVoteStatistics = recountVoteStatistics;
}

void VotingResultsWrapper::persistence()
{
    std::string votingResultFileDom = directories->getVotingResultsFileFullPath ( BasePathType::CONFIGURATION_DOM );
    std::string votingResultFileFlash = directories->getVotingResultsFileFullPath ( BasePathType::CONFIGURATION_FLASH );

    std::string actualLanguage;
    std::string actualCountry;
    std::string currentLanguage;

    ElectionStringResourcesManager * resource = ElectionStringResourcesManager::getInstance();
    std::list < std::string > types = recountVoteStatistics->GetVoterTypes();
    std::vector < PBReports::VoteCounterByCA > votes = voteStatistics->getVoteCountersByCA();

    currentLanguage = BaseLanguageController::getActualLanguage();
    SetLanguageFunctionality::getCurrentLanguageCountry ( actualLanguage, actualCountry );

    std::list <std::string> languages = electoralConfiguration->getLanguageListFullName();
    std::list <std::string>::iterator it;

    for ( it = languages.begin();
            it != languages.end();
            ++it )
    {
        std::string lang = ( *it );
        logger->Debug ( "Generate voting results in language " + lang );

        if ( lang.empty() )
        {
            BaseLanguageController::FullLocalizeApplication ( currentLanguage );
            SetLanguageFunctionality::setNewVotingLanguageByLanguageCountry ( actualLanguage, actualCountry );
        }
        else
        {
            BaseLanguageController::FullLocalizeApplication ( lang );
            std::string aLand, aCountry;
            BaseLanguageController::GetActualLanguage ( aLand, aCountry );
            SetLanguageFunctionality::setNewVotingLanguageByLanguageCountry ( aLand, aCountry );
        }

        std::string domTmp = Path::combineFileName ( votingResultFileDom, lang );
        std::string flashTmp = Path::combineFileName ( votingResultFileFlash, lang );

        logger->Debug ( "Persist voting results " + domTmp );
        logger->Debug ( "Persist voting results " + flashTmp );

        VotingResults voteResults;
        voteResults.setTotalVotes ( voteStatistics->getOfficialVotes() );

        VotesByNationality votesByNationality;
        VotesByNationality::NationalitySequence seq;

        std::list < std::string >::iterator ittypes;

        for ( ittypes = types.begin();
                ittypes != types.end();
                ++ittypes )
        {
            Nationality n;
            n.setName ( resource->getResource ( *ittypes ) );

            for ( std::vector< PBReports::VoteCounterByCA>::iterator it = votes.begin(); it != votes.end(); ++it )
            {
                if ( *ittypes == ( *it ).getCAValue() )
                {
                    n.setVotes ( ( *it ).getCounter() );
                    break;
                }
            }

            seq.push_back ( n );
        }

        votesByNationality.setNationality ( seq );
        voteResults.setVotesByNationality ( votesByNationality );

        SetLanguageFunctionality::setNewVotingLanguageByLanguageCountry ( actualLanguage, actualCountry );

        std::ostringstream str;

        try
        {
            Smartmatic::SAES::Voting::NamespaceInfomap mapv;
            mapv[""].name = NAMESPACE_VOTING;
            mapv[""].schema = "";
            serializeVotingResults ( str, voteResults, mapv, "UTF-8", Smartmatic::SAES::Voting::Flags::dont_validate );
        }
        catch ( ... )
        {
            throw Smartmatic::System::Exception::XmlException ( _ ( "Smartmatic.SAES.Voting.VotingResultsWrapper.ErrorSerializaData" ),
                    N_ ( "Smartmatic.SAES.Voting.VotingResultsWrapper.ErrorSerializaData" ),
                    logger );
        }

        std::string data = str.str();
        Encryption::getInstance()->saveXmlFile ( data, domTmp, Encryption::VOTING_RESULT_FILE );
        File::getInstance()->copyFile ( domTmp, flashTmp );
    }
}

bool VotingResultsWrapper::validate()
{
    bool validate = true;

    std::string votingResultFileDom = directories->getVotingResultsFileFullPath ( BasePathType::CONFIGURATION_DOM );
    std::string votingResultFileFlash = directories->getVotingResultsFileFullPath ( BasePathType::CONFIGURATION_FLASH );

    std::list <std::string> languages = electoralConfiguration->getLanguageListFullName();
    std::list <std::string>::iterator it;

    for ( it = languages.begin();
            it != languages.end() && validate;
            ++it )
    {
        std::string lang = ( *it );

        std::string domTmp = Path::combineFileName ( votingResultFileDom, lang );
        std::string flashTmp = Path::combineFileName ( votingResultFileFlash, lang );

        try
        {
            Encryption::getInstance()->openXmlFile ( domTmp, Encryption::VOTING_RESULT_FILE );
            Encryption::getInstance()->openXmlFile ( flashTmp, Encryption::VOTING_RESULT_FILE );

            validate = validate && File::getInstance()->compareFiles ( domTmp, flashTmp );
        }
        catch ( ... )
        {
            validate = false;
        }
    }

    return validate;
}

void VotingResultsWrapper::deleteFiles()
{
    std::string votingResultFileDom = directories->getVotingResultsFileFullPath ( BasePathType::CONFIGURATION_DOM );
    std::string votingResultFileFlash = directories->getVotingResultsFileFullPath ( BasePathType::CONFIGURATION_FLASH );

    std::list <std::string> languages = electoralConfiguration->getLanguageListFullName();
    std::list <std::string>::iterator it;

    for ( it = languages.begin();
            it != languages.end();
            ++it )
    {
        std::string lang = ( *it );

        std::string domTmp = Path::combineFileName ( votingResultFileDom, lang );
        std::string flashTmp = Path::combineFileName ( votingResultFileFlash, lang );

        File::getInstance()->DeleteFile ( domTmp );
        File::getInstance()->DeleteFile ( flashTmp );
    }
}

VotingResultsWrapper * VotingResultsWrapper::getInstance()
{
    if ( instance == NULL )
    {
        instance = new VotingResultsWrapper();
    }

    return instance;
}
