#ifndef TEST_NATIONALITY_MANAGER_TEST
#define TEST_NATIONALITY_MANAGER_TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <tests/MockElectionStringResourcesManager.hxx>
#include <tests/MockRecountVoteStatistics.hxx>
#include <tests/MockVoteStatistics.hxx>
#include <Voting/NationalityTypeManager.hxx>
#include <System/SafeBuffer.h>

class NationalityTest : public CPPUNIT_NS::TestFixture
{
    public:

        typedef Smartmatic::SAES::Resources::ElectionStringResourcesManager ElectionStringResourcesManager;
        typedef Smartmatic::SAES::Voting::NationalityTypeManager NationalityTypeManager;
        typedef Smartmatic::SAES::Voting::PBReports::VoteCounterByCA VoteCounterByCA;
        typedef Smartmatic::System::SafeBuffer SafeBuffer;

        NationalityTypeManager * nationalityManager;

        NationalityTest ()
        {
            nationalityManager = NULL;
        }

        void setUp()
        {
            nationalityManager = NationalityTypeManager::getInstance();
        }

        void tearDown()
        {
            delete nationalityManager;
        }

        void testParseEmptyFileToMap()
        {
            SafeBuffer buffer;
            std::map < std::string, std::string > map;

            nationalityManager->getNationalityMap ( buffer, map );

            CPPUNIT_ASSERT ( true == map.empty() );
        }

        void testParseFileToMap()
        {
            SafeBuffer buffer ( "N,belgie" );
            std::map < std::string, std::string > map;

            nationalityManager->getNationalityMap ( buffer, map );

            CPPUNIT_ASSERT ( 1 == map.size() );
            CPPUNIT_ASSERT ( true == ( map.find ( "belgie" ) != map.end() ) );
            CPPUNIT_ASSERT ( std::string ( "N" ) == map.at ( "belgie" ) );
        }

        void testParseFileTwoRegistersToMap()
        {
            SafeBuffer buffer ( "N,belgie\nE,european" );
            std::map < std::string, std::string > map;

            nationalityManager->getNationalityMap ( buffer, map );

            CPPUNIT_ASSERT ( 2 == map.size() );
            CPPUNIT_ASSERT ( true == ( map.find ( "belgie" ) != map.end() ) );
            CPPUNIT_ASSERT ( true == ( map.find ( "european" ) != map.end() ) );
            CPPUNIT_ASSERT ( std::string ( "N" ) == map.at ( "belgie" ) );
            CPPUNIT_ASSERT ( std::string ( "E" ) == map.at ( "european" ) );
        }

        void testParseMapToString()
        {
            MockElectionStringResourcesManager manager;
            MockRecountVoteStatistics recountVote;
            MockVoteStatistics voteStatistics;

            ElectionStringResourcesManager::getInstance ( &manager );

            MOCKPP_CHAINER_FOR_EXT ( MockElectionStringResourcesManager, getResource, ext ) getResource_mock ( &manager );

            getResource_mock
            .expects ( MOCKPP_NS::exactly ( 2 ) )
            .will ( MOCKPP_NS::onConsecutiveCalls (
                        new MOCKPP_NS::ReturnStub<std::string> ( "belgie" ),
                        new MOCKPP_NS::ReturnStub<std::string> ( "european" ) ) );

            MOCKPP_CHAINER_FOR ( MockVoteStatistics, getOfficialVotes ) getOfficialVotes_mock ( &voteStatistics );

            getOfficialVotes_mock
            .expects ( MOCKPP_NS::once() )
            .will ( new MOCKPP_NS::ReturnStub< unsigned int > ( 5 ) );

            recountVote.list.push_back ( "belgie" );
            recountVote.list.push_back ( "european" );

            voteStatistics.vector.push_back ( VoteCounterByCA ( "belgie", "belgie", 3 ) );
            voteStatistics.vector.push_back ( VoteCounterByCA ( "european", "european", 2 ) );

            SafeBuffer buffer ( "N,belgie\nE,european" );
            std::map < std::string, std::string > map;

            std::string result;
            std::string expected = "T=5|N=3|E=2";

            nationalityManager->init ( NULL, &voteStatistics, &recountVote, NULL );
            nationalityManager->getNationalityMap ( buffer, map );

            result = nationalityManager->getStringResult ( map );

            CPPUNIT_ASSERT ( expected == result );

            manager.verify();
            voteStatistics.verify();
        }

        CPPUNIT_TEST_SUITE ( NationalityTest );
        CPPUNIT_TEST ( testParseEmptyFileToMap );
        CPPUNIT_TEST ( testParseFileToMap );
        CPPUNIT_TEST ( testParseFileTwoRegistersToMap );
        CPPUNIT_TEST ( testParseMapToString );
        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_NATIONALITY_MANAGER_TEST */
