/**
 * @file ConnectionBehavior.cxx
 * @brief The high-level behaviour of the connection to the smartcard reader and provides operations of connection, commands, and manages error cases.
 * @author Yadickson Soto
 * @date 28/03/2014
 */

#include "ConnectionBehavior.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Configuration/SmartcardConfigurationManager.hxx>
#include <Card/ISmartcardReaderPowerManager.hxx>

#include <System/SafeBuffer.h>
#include <System/IO/File.h>
#include <string>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

static const double SECONDS_BETWEEN_RESTARTS = 5;

using namespace Smartmatic::SAES::Card;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::System;
using namespace Smartmatic::Log;

ISMTTLog * ConnectionBehavior::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Card.ConnectionBehavior", GETTEXT_PACKAGE );

ConnectionBehavior::ConnectionBehavior ( unsigned int numberOfTriesToRead, ISmartcardReaderPowerManager* powerManager )
    : numberOfTriesToRead ( numberOfTriesToRead )
    , m_powerManager(powerManager)
    , m_lastRestart(std::time(NULL))
{
    connectionDriver = NULL;
}

ConnectionBehavior::~ConnectionBehavior()
{
    if ( connectionDriver )
    {
        delete connectionDriver;
    }
}

bool ConnectionBehavior::establishContext ()
{
    return checkConnectionStatus ( connectionDriver->establishContext () );
}

bool ConnectionBehavior::releaseContext()
{
    return checkConnectionStatus ( connectionDriver->releaseContext() );
}

bool ConnectionBehavior::selectAID ( const SafeBuffer & aid )
{
    unsigned long status;
    int count = numberOfTriesToRead;

    do
    {
        status = connectionDriver->selectAID ( aid );
    }
    while ( --count > 0 && !checkConnectionStatus ( status ) );

    return processConnectionStatus ( status );
}

bool ConnectionBehavior::sendCommand ( const SafeBuffer & command )
{
    return processConnectionStatus ( connectionDriver->sendCommand ( command ) );
}

SafeBuffer ConnectionBehavior::sendCommandWithResponse ( const SafeBuffer & command )
{
    SafeBuffer response;

    if ( sendCommand ( command ) )
    {
        response = connectionDriver->getResponse();
    }

    return response;
}

bool ConnectionBehavior::processConnectionStatus ( unsigned long status )
{
    bool validate = true;

    if ( !checkConnectionStatus ( status ) )
    {
        validate = false;

        if ( SmartcardConfigurationManager::getInstance()->isErrorCode ( status ) )
        {
            connectionDriver->releaseContext();

            if (m_powerManager.get() != NULL &&
                std::difftime(std::time(NULL), m_lastRestart) > SECONDS_BETWEEN_RESTARTS)
            {
                m_powerManager->doReaderPowerCycle();
                m_lastRestart = std::time(NULL);
            }

            validate = checkConnectionStatus ( connectionDriver->establishContext () );
        }
    }

    return validate;
}
