/**
 * @file GlobalPlatformMode.cxx
 * @brief The global platform connection mode
 * @author Yadickson Soto
 * @date 20/05/2014
 */

#include <Configuration/SmartcardConfigurationManager.hxx>
#include <Card/GlobalPlatformMode.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#define KEY_STRING (unsigned char *)"@ABCDEFGHIJKLMNO" ///< The security string
#define RECEIVEAPDULEN 261 ///< max receive data file

using namespace Smartmatic::SAES::Card;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::Log;

ISMTTLog * GlobalPlatformMode::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Card.GlobalPlatformMode", GETTEXT_PACKAGE );

GlobalPlatformMode::GlobalPlatformMode ( OPGP_CARD_INFO & cardInfo, OPGP_CARD_CONTEXT & cardContext )
    : cardInfo ( cardInfo ), cardContext ( cardContext )
{
    keyIndex = SmartcardConfigurationManager::getInstance()->getSecurityKeyIndex();
    keySetVersion = SmartcardConfigurationManager::getInstance()->getSecurityKeySetVersion();
    keyDerivation = 0x00;
    secureChannelEnabled = SmartcardConfigurationManager::getInstance()->hasSecureChannel();

    if ( secureChannelEnabled )
    {
        SecurityKeyDerivation derivation = SmartcardConfigurationManager::getInstance()->getSecurityKeyDerivation();

        switch ( derivation )
        {
            default:
            case SecurityKeyDerivation::CONFIGURATION_NONE:
                keyDerivation = OPGP_DERIVATION_METHOD_NONE;
                break;

            case SecurityKeyDerivation::CONFIGURATION_VISA2:
                keyDerivation = OPGP_DERIVATION_METHOD_VISA2;
                break;

            case SecurityKeyDerivation::CONFIGURATION_EMVCPS11:
                keyDerivation = OPGP_DERIVATION_METHOD_EMV_CPS11;
                break;
        }
    }
}

GlobalPlatformMode::~GlobalPlatformMode()
{

}

OPGP_ERROR_STATUS GlobalPlatformMode::openSecureChannel()
{
    OPGP_ERROR_STATUS status;
    OPGP_ERROR_CREATE_NO_ERROR ( status );

    if ( secureChannelEnabled )
    {
        logger->Debug ( "Open security channel" );
        status = setupSecureChannel();
    }

    return status;
}

OPGP_ERROR_STATUS GlobalPlatformMode::sendCommand ( const Smartmatic::System::SafeBuffer & command, Smartmatic::System::SafeBuffer & response )
{
    OPGP_ERROR_STATUS status;

    unsigned char recvAPDU[RECEIVEAPDULEN + 1];
    unsigned long recvAPDULen = RECEIVEAPDULEN;

    response.Clear();

    status = processCommand ( command, recvAPDU, recvAPDULen );

    if ( !OPGP_ERROR_CHECK ( status ) )
    {
        response.AddData ( ( const unsigned char * ) recvAPDU, recvAPDULen );
    }

    return status;

}

unsigned char * GlobalPlatformMode::getSecurityKey()
{
    return KEY_STRING;
}
