/**
 * @file JavaCardOperations.cxx
 * @brief Standard generic JavaCard operations
 * @date 28/03/2014
 * @author Yadickson Soto
 */

#include "JavaCardOperations.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#define PASSWORD_FILE_LEN 0x30 ///< The password file length

#define CLA 0x80 ///< command apdu CLA

#define INS_VERIFY_GLOBAL_PIN 0xF1  ///< command verify global pin
#define INS_VERIFY_PASS_PIN 0xF2  ///< command verify password pin
#define INS_SET_PASS_PIN 0xF3 ///< command set password pin
#define INS_GET_PASS_PIN 0xF4 ///< command get password pin
#define INS_SET_PASS_FILE 0xF5 ///< command set password file
#define INS_GET_PASS_FILE 0xF6 ///< command get password file
#define INS_WRITE 0xF7 ///< command write
#define INS_READ  0xF8 ///< command read

#define P1_NORMAL 0x00 ///< command APDU P1
#define P2_NORMAL 0x00 ///< command APDU P2

#define P1_OPEN_WRITE_FILE 0x01 ///< command open write file
#define P1_NORMAL_WRITE_FILE 0x02 ///< command write file
#define P1_ABORT_WRITE_FILE 0x03  ///< command abort write file
#define P2_NORMAL_WRITE_FILE 0x00 ///< command write file
#define P2_CLOSE_WRITE_FILE 0x01 ///< command close write file

#define P1_OPEN_READ_FILE 0x01 ///< command open read file
#define P1_NORMAL_READ_FILE 0x00 ///< command read file
#define P2_NORMAL_READ_FILE 0x00 ///< command read file
#define LE_GET_LENGTH 0x02 ///< command get length file

#define MAX_DATA_APDU 200 ///<

using namespace Smartmatic::SAES::Card;
using namespace Smartmatic::System;
using namespace Smartmatic::Log;

ISMTTLog * JavaCardOperations::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Card.JavaCardOperations", GETTEXT_PACKAGE );

JavaCardOperations::JavaCardOperations ( ConnectionBehavior * connectionBehavior )
    : connectionBehavior ( connectionBehavior )
{

}

JavaCardOperations::~JavaCardOperations()
{

}

bool JavaCardOperations::verifyGlobalPIN ( const SafeBuffer & pin )
{
    logger->Debug ( "Verify Global PIN" );

    bool validate = false;

    SafeBuffer apdu;
    SafeBuffer buffer;

    apdu.push_back ( CLA );
    apdu.push_back ( INS_VERIFY_GLOBAL_PIN );
    apdu.push_back ( P1_NORMAL );
    apdu.push_back ( P2_NORMAL );
    apdu.push_back ( pin.GetSize() );
    apdu.AddData ( pin );
    apdu.push_back ( 0x01 );

    buffer = connectionBehavior->sendCommandWithResponse ( apdu );

    if ( buffer.GetSize() == 3 )
    {
        validate = buffer.GetData() [0] == 0x01 &&
                   buffer.GetData() [1] == 0x90 &&
                   buffer.GetData() [2] == 0x00;
    }

    return validate;
}

bool JavaCardOperations::verifyPasswordPIN ( const SafeBuffer & pin )
{
    logger->Debug ( "Verify Password PIN" );

    bool validate = false;

    SafeBuffer apdu;
    SafeBuffer buffer;

    apdu.push_back ( CLA );
    apdu.push_back ( INS_VERIFY_PASS_PIN );
    apdu.push_back ( P1_NORMAL );
    apdu.push_back ( P2_NORMAL );
    apdu.push_back ( pin.GetSize() );
    apdu.AddData ( pin );
    apdu.push_back ( 0x01 );

    buffer = connectionBehavior->sendCommandWithResponse ( apdu );

    if ( buffer.GetSize() == 3 )
    {
        validate = buffer.GetData() [0] == 0x01 &&
                   buffer.GetData() [1] == 0x90 &&
                   buffer.GetData() [2] == 0x00;
    }

    return validate;
}

bool JavaCardOperations::setPasswordPIN ( const SafeBuffer & pin )
{
    logger->Debug ( "Set Password PIN" );

    SafeBuffer apdu;

    apdu.push_back ( CLA );
    apdu.push_back ( INS_SET_PASS_PIN );
    apdu.push_back ( P1_NORMAL );
    apdu.push_back ( P2_NORMAL );
    apdu.push_back ( pin.GetSize() );
    apdu.AddData ( pin );

    return connectionBehavior->sendCommand ( apdu );
}

bool JavaCardOperations::setPasswordFile ( const SafeBuffer & password )
{
    logger->Debug ( "Set Password File" );

    SafeBuffer apdu;
    apdu.push_back ( CLA );
    apdu.push_back ( INS_SET_PASS_FILE );
    apdu.push_back ( P1_NORMAL );
    apdu.push_back ( P2_NORMAL );
    apdu.push_back ( password.GetSize() );
    apdu.AddData ( password );

    return connectionBehavior->sendCommand ( apdu );
}

SafeBuffer JavaCardOperations::getPasswordFile ()
{
    logger->Debug ( "Get Password File" );

    SafeBuffer apdu;
    SafeBuffer buffer;

    apdu.push_back ( CLA );
    apdu.push_back ( INS_GET_PASS_FILE );
    apdu.push_back ( P1_NORMAL );
    apdu.push_back ( P2_NORMAL );
    apdu.push_back ( PASSWORD_FILE_LEN );

    buffer = connectionBehavior->sendCommandWithResponse ( apdu );

    if ( buffer.GetSize() == ( PASSWORD_FILE_LEN + 2 ) )
    {
        SafeBuffer key ( ( unsigned char * ) buffer.GetData(), PASSWORD_FILE_LEN );
        buffer.Clear();
        buffer.AddData ( key );
    }
    else
    {
        buffer.Clear();
    }

    return buffer;
}

bool JavaCardOperations::openWrite()
{
    SafeBuffer apdu;

    apdu.push_back ( CLA );
    apdu.push_back ( INS_WRITE );
    apdu.push_back ( P1_OPEN_WRITE_FILE );
    apdu.push_back ( P2_NORMAL_WRITE_FILE );

    return connectionBehavior->sendCommand ( apdu );
}

bool JavaCardOperations::continueWrite ( const SafeBuffer & buffer )
{
    SafeBuffer apdu;
    apdu.push_back ( CLA );
    apdu.push_back ( INS_WRITE );
    apdu.push_back ( P1_NORMAL_WRITE_FILE );
    apdu.push_back ( P2_NORMAL_WRITE_FILE );
    apdu.push_back ( buffer.GetSize() );
    apdu.AddData ( buffer );

    return connectionBehavior->sendCommand ( apdu );
}

bool JavaCardOperations::closeWrite()
{
    SafeBuffer apdu;

    apdu.push_back ( CLA );
    apdu.push_back ( INS_WRITE );
    apdu.push_back ( P1_NORMAL_WRITE_FILE );
    apdu.push_back ( P2_CLOSE_WRITE_FILE );
    apdu.push_back ( 0x00 );

    return connectionBehavior->sendCommand ( apdu );
}

bool JavaCardOperations::abortWrite ()
{
    SafeBuffer apdu;

    apdu.push_back ( CLA );
    apdu.push_back ( INS_WRITE );
    apdu.push_back ( P1_ABORT_WRITE_FILE );
    apdu.push_back ( P2_NORMAL_WRITE_FILE );

    return connectionBehavior->sendCommand ( apdu );
}

unsigned int JavaCardOperations::openRead ()
{
    unsigned int left = 0;

    SafeBuffer apdu;
    SafeBuffer buffer;

    apdu.push_back ( CLA );
    apdu.push_back ( INS_READ );
    apdu.push_back ( P1_OPEN_READ_FILE );
    apdu.push_back ( P2_NORMAL_READ_FILE );
    apdu.push_back ( LE_GET_LENGTH );

    buffer = connectionBehavior->sendCommandWithResponse ( apdu );

    if ( buffer.GetSize() == 4 &&
            buffer.GetData() [2] == 0x90 &&
            buffer.GetData() [3] == 0x00 )
    {
        left = buffer.GetData() [0];
        left <<= 8;
        left += buffer.GetData() [1];
    }

    return left;
}

SafeBuffer JavaCardOperations::continueRead ( unsigned char size, unsigned int & left )
{
    left = 0;

    SafeBuffer apdu;
    SafeBuffer buffer;
    SafeBuffer response;

    apdu.push_back ( CLA );
    apdu.push_back ( INS_READ );
    apdu.push_back ( P1_NORMAL_READ_FILE );
    apdu.push_back ( P2_NORMAL_READ_FILE );
    apdu.push_back ( size + 2 );

    response = connectionBehavior->sendCommandWithResponse ( apdu );

    if ( response.GetSize() == ( unsigned int ) ( 4 + size ) &&
            response.GetData() [2 + size] == 0x90 &&
            response.GetData() [3 + size] == 0x00 )
    {
        left = response.GetData() [0];
        left <<= 8;
        left += response.GetData() [1];

        buffer.AddData ( response.GetData() + 2, size );
    }

    return buffer;
}

SafeBuffer JavaCardOperations::getDataFile ()
{
    SafeBuffer buffer;

    unsigned int leng = openRead ();

    if ( leng > 0 )
    {
        do
        {
            unsigned int size = leng > MAX_DATA_APDU ? MAX_DATA_APDU : leng;
            buffer.AddData ( continueRead ( size, leng ) );
        }
        while ( leng != 0 );
    }

    return buffer;
}

bool JavaCardOperations::setDataFile ( const SafeBuffer & buffer )
{
    bool validate = openWrite();

    if ( validate )
    {
        unsigned int leng = buffer.GetSize();

        if ( leng > 0 )
        {
            unsigned int offset = 0;

            do
            {
                unsigned int size = leng > MAX_DATA_APDU ? MAX_DATA_APDU : leng;

                SafeBuffer temp;

                temp.AddData ( buffer.GetData() + offset, size );

                if ( !continueWrite ( temp ) )
                {
                    abortWrite();
                    validate = false;
                    logger->Debug ( "ERROR - writing file" );
                    break;
                }

                leng -= size;
                offset += size;
            }
            while ( leng != 0 );
        }

        if ( !closeWrite() )
        {
            abortWrite();
        }
    }
    else
    {
        abortWrite();
        logger->Debug ( "ERROR - open file" );
    }

    return validate;
}
