/**
 * @file	SimulatorSmartcard.hxx
 * @brief	Declaration of the simulator smartcard reader controller which implements operations of
 * reading and writing the device.
 * @author	Yadickson Soto
 * @date	13/05/2015
 */

#ifndef _SIMULATOR_SMARTCARD_HXX_
#define _SIMULATOR_SMARTCARD_HXX_

#include <Card/SmartcardInterface.hxx>
#include <Security/CryptoData.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Card
        {
            /**
             * @class	SimulatorSmartcard
             * @brief Represents the controller of a smartcard reader device which implements of operations
             *        the activation, reading, writing, and restarting the device.
             * @author	Yadickson Soto
             * @date	13/05/2015
             */
            class SimulatorSmartcard : public SmartcardInterface
            {
                public:

                    /**
                     * @fn	SimulatorSmartcard( const std::string & eventFile, const std::string & contentFile, unsigned int timeAfterRead = 0, unsigned int timeBeforeWrite = 0 );
                     * @brief   Class constructor to initialize parameters from get card events, read and write smartcard.
                     * @param	eventFile path file to read event file (I: inserted and R: removed).
                     * @param	contentFile path file to write and read the password file and content file.
                     * @param	timeAfterRead time after of the read operation in seconds.
                     * @param	timeBeforeWrite time before of the write operation in seconds.
                     * @author	Yadickson Soto
                     * @date	13/05/2015
                     */
                    SimulatorSmartcard (
                        const std::string & eventFile,
                        const std::string & contentFile,
                        unsigned int timeAfterRead = 0,
                        unsigned int timeBeforeWrite = 0 );

                    /**
                     * @fn	virtual ~SimulatorSmartcard();
                     * @brief	class destructor
                     * @author	Yadickson Soto
                     * @date	13/05/2015
                     */
                    virtual ~SimulatorSmartcard();

                    /**
                     * @fn	virtual bool open ( const std::string & device );
                     * @brief	Establishes the context dummy.
                     * @author	Yadickson Soto
                     * @date	13/05/2015
                     * @param 	device device path
                     * @return true if success
                     */
                    virtual bool open ( const std::string & device );

                    /**
                     * @fn	virtual bool close();
                     * @brief	Release context dummy.
                     * @author	Yadickson Soto
                     * @date	13/05/2015
                     * @return true if success
                     */
                    virtual bool close();

                    /**
                     * @fn	virtual bool isCardPresent();
                     * @brief	Whether a card is present.
                     * @author	Yadickson Soto
                     * @date	13/05/2015
                     * @return	true if the content inside of eventFile is I, otherwise false.
                     */
                    virtual bool isCardPresent();

                    /**
                     * @fn	virtual bool read(Smartmatic::System::SafeBuffer & buffer,, Smartmatic::SAES::Security::CryptoData & cryptoData );
                     * @brief	Reads the contents of the smartcard
                     * @author	Yadickson Soto
                     * @date	13/05/2015
                     * @param	buffer			The reference to the buffer used to store the information received.
                     * @param	cryptoData	Contains common context information for the cryptographic checking of the operation.
                     * @return	true if the reading was successful; otherwise false.
                     */
                    virtual bool read ( Smartmatic::System::SafeBuffer & buffer, Smartmatic::SAES::Security::CryptoData & cryptoData );

                    /**time after of the read operation in seconds.
                     * @fn	virtual bool write(Smartmatic::System::SafeBuffer & buffer,, Smartmatic::SAES::Security::CryptoData & cryptoData );
                     * @brief	Writes the contents of the buffer and verifies the data in the smartcard.
                     * @author	Yadickson Soto
                     * @date	13/05/2015
                     * @param [in,out]	buffer			Contains information that should to be written.
                     * @param [in,out]	cryptoData	Contains common context information for the cryptographic checking of the operation.
                     * @return	true if the writing was successful; otherwise false.
                     */
                    virtual bool write ( Smartmatic::System::SafeBuffer & buffer, Smartmatic::SAES::Security::CryptoData & cryptoData );

                    /**
                     * @fn	virtual bool test();
                     * @brief	Test smartcard to write and read dummy string.
                     * @author	Yadickson Soto
                     * @date	13/05/2015
                     * @return	true if the test was successful; otherwise false.
                     */
                    virtual bool test();

                private:

                    std::string eventFile; ///< path file to read event file (I: inserted and R: removed).
                    std::string contentFile; ///< path file to write and read the password file and content file.
                    unsigned int timeAfterRead; ///< time after of the read operation in seconds.
                    unsigned int timeBeforeWrite; ///< time before of the write operation in seconds.

                    static Smartmatic::Log::ISMTTLog * logger; ///< The pointer to the logger instance.
            };
        }
    }
}

#endif /* _SIMULATOR_SMARTCARD_HXX_ */
