/**
 * @file	Smartcard.hxx
 * @brief	Declaration of the smartcard reader controller which implements operations of activation,
 * reading, writing, and restarting the device.
 * @author	Yadickson Soto
 * @date	21/05/2014
 */

#ifndef _SMARTCARD_HXX_
#define _SMARTCARD_HXX_

#include <Card/SmartcardInterface.hxx>
#include <Card/JavaCardOperations.hxx>
#include <Card/ConnectionBehavior.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Card
        {
            /**
             * @class	Smartcard
             * @brief Represents the controller of a smartcard reader device which implements of operations
             *        the activation, reading, writing, and restarting the device.
             * @author	Yadickson Soto
             * @date	21/05/2014
             */
            class Smartcard : public SmartcardInterface
            {
                public:

                    /**
                     * @fn	Smartcard( ConnectionBehavior * connectionBehavior, JavaCardOperations * operations, const Smartmatic::System::SafeBuffer & appletID, const Smartmatic::System::SafeBuffer & globalPIN );
                     * @brief
                     * @author	Yadickson Soto
                     * @date	21/05/2014
                     * @param 	connectionBehavior The connection behavior (driver wrapper) for the reader which connects, sends commands, and manages error cases.
                     * @param	operations The java card operations manager.
                     * @param 	appletID Applet ID for smartcard communication.
                     * @param 	globalPIN Global PIN configuration for smartcard communication.
                     * @param 	numberOfTriesToRead Maximum number of tries to read from the smarcard.
                     */
                    Smartcard ( ConnectionBehavior * connectionBehavior, JavaCardOperations * operations, const Smartmatic::System::SafeBuffer & appletID, const Smartmatic::System::SafeBuffer & globalPIN );

                    /**
                     * @fn	virtual ~Smartcard();
                     * @brief	class destructor
                     * @author	Yadickson Soto
                     * @date	21/05/2014
                     */
                    virtual ~Smartcard();

                    /**
                     * @fn	virtual bool open ( const std::string & device );
                     * @brief	Establishes the context. Call only once.
                     * @author	Yadickson Soto
                     * @date	21/05/2014
                     * @param 	device device path
                     * @return true if success
                     */
                    virtual bool open ( const std::string & device );

                    /**
                     * @fn	virtual bool close();
                     * @brief	Release context
                     * @author	Yadickson Soto
                     * @date	21/05/2014
                     * @return true if success
                     */
                    virtual bool close();

                    /**
                     * @fn	virtual bool isCardPresent();
                     * @brief	Whether a card is present.
                     * @author	Yadickson Soto
                     * @date	21/05/2014
                     * @return	true if a smartcard is present; otherwise false.
                     */
                    virtual bool isCardPresent();

                    /**
                     * @fn	virtual bool read(Smartmatic::System::SafeBuffer & buffer,, Smartmatic::SAES::Security::CryptoData & cryptoData );
                     * @brief	Reads the contents of the smartcard
                     * @author	Yadickson Soto
                     * @date	21/05/2014
                     * @param	buffer			The reference to the buffer used to store the information received.
                     * @param	cryptoData	Contains common context information for the cryptographic checking of the operation.
                     * @return	true if the reading was successful; otherwise false.
                     */
                    virtual bool read ( Smartmatic::System::SafeBuffer & buffer, Smartmatic::SAES::Security::CryptoData & cryptoData );

                    /**
                     * @fn	virtual bool write(Smartmatic::System::SafeBuffer & buffer,, Smartmatic::SAES::Security::CryptoData & cryptoData );
                     * @brief	Writes the contents of the buffer and verifies the data in the smartcard.
                     * @author	Yadickson Soto
                     * @date	21/05/2014
                     * @param [in,out]	buffer			Contains information that should to be written.
                     * @param [in,out]	cryptoData	Contains common context information for the cryptographic checking of the operation.
                     * @return	true if the writing was successful; otherwise false.
                     */
                    virtual bool write ( Smartmatic::System::SafeBuffer & buffer, Smartmatic::SAES::Security::CryptoData & cryptoData );

                    /**
                     * @fn	virtual bool test();
                     * @brief	Determines whether the ID can be read from the smartcard.
                     * @author	Yadickson Soto
                     * @date	21/05/2014
                     * @return	true if the test was successful; otherwise false.
                     */
                    virtual bool test();

                private:

                    ConnectionBehavior * connectionBehavior; ///< The connection behavior (driver wrapper) for the reader which connects, sends commands, and manages error cases.
                    JavaCardOperations * operations; ///< The java card operations

                    Smartmatic::System::SafeBuffer appletID; ///< Applet ID for smartcard communication.
                    Smartmatic::System::SafeBuffer globalPIN; ///< Global PIN configuration for smartcard communication.

                    bool initStatus;	///< This flag represents the initialization state of the controller.

                    static Smartmatic::Log::ISMTTLog * logger; ///< The pointer to the logger instance.
            };
        }
    }
}

#endif /* _SMARTCARD_MANAGER_HXX_ */
