/**
 * @file	s-a-e-s-common-conf-manager.cc
 * @brief	Implementation the configuration manager which provides and persists the SAESCommon settings.
 * This class is implemented in terms of SaesConfiguration.
 * @author	Youssef.tourkmani
 * @date	30/10/2012
 */

#include <Configuration/ConfigurationBase-schema.hxx>
#include <Configuration/configuration-persistence-manager.h>
#include "s-a-e-s-common-conf-manager.h"
#include <System/IO/Path.h>
#include <System/ZenityWrapper.hxx>
#include <System/Runtime/ExitProcess.hxx>
#include <System/Encoding/Base64Encoder.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::Configuration;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System;
using namespace Smartmatic::System::Runtime;
using namespace Smartmatic::System::Encoding;

SAESCommonConfigurationManager * SAESCommonConfigurationManager::instance = NULL;

SAESCommonConfigurationManager::SAESCommonConfigurationManager()
{
    configuration = NULL;
}

SAESCommonConfigurationManager::~SAESCommonConfigurationManager()
{
    if ( configuration )
    {
        delete configuration;
        configuration = NULL;
    }

    instance = NULL;
}

SaesCommonConfiguration * SAESCommonConfigurationManager::getConfiguration()
{
    if ( configuration == NULL )
    {
        initialize();
    }

    return configuration;
}

void SAESCommonConfigurationManager::initialize()
{
    try
    {
        std::string semiFile ( ConfigurationPersistenceManager::getInstance()->GetLibraryConfigFile ( "SAESCommon" ) );
        std::istringstream ss ( semiFile );
        configuration = new SaesCommonConfiguration ( * parseSaesCommonConfiguration ( ss, ConfigurationTypes::Flags::dont_validate ).get() );
    }
    catch ( const ::xsd::cxx::tree::exception< char > & e )
    {
        std::stringstream ss;
        ss << e;
        ZenityWrapper::showError ( "Config SAESCOMMON Error \n\n" + ss.str(), "Error" );
        ExitProcess::getInstance()->exitProcess();
    }
}

void SAESCommonConfigurationManager::loadDefault()
{
    SaesCommonConfiguration configSAESCommon;
    ConfigurationElectionFilesPath configElectionFiles;
    ConfigurationPrinter configPrinter;
    OSConfigurationFiles osConfig;
    LogConfigurationReader logConfigReader;
    SmartcardConfiguration smartcardConfiguration;
    PCSCDConfiguration pcscdConfiguration;

    SecurityFileConfiguration installationFiles;
    SecurityFileConfiguration externalFiles;
    SecurityFileConfiguration resourcesFiles;
    SecurityFileConfiguration internalFiles;
    SecurityFileConfiguration utilFiles;
    SecurityFileConfiguration votingResultFiles;
    SecurityFileConfiguration exchangeFiles;
    SecurityFileConfiguration printedVoteData;

    configElectionFiles.setEmlFilesFolder ( "firstboot/INSTALLATION_FOLDER/eml_files" );
    configElectionFiles.setSAESFolder ( "SAES" );
    configElectionFiles.setResourcesFolder ( "StringResources" );
    configElectionFiles.setImagesFolder ( "Images" );
    configElectionFiles.setVotesFolder ( "Votes" );
    configElectionFiles.setDocsFolder ( "Docs" );
    configElectionFiles.setEmlFolder ( "Eml" );
    configElectionFiles.setEmlTempFolder( "/tmp/eml" );
    configElectionFiles.setLogsFolderName ( "Logs" );
    configElectionFiles.setDiagnosticResultFolder ( "Results" );
    configElectionFiles.setTransmissionPackageFileName ( "transmission-package.xml" );
    configElectionFiles.setTransmissionPackageHashFileName ( "transmission-package.hash" );
    configElectionFiles.setTransmissionPackageKeyFileName ( "transmission-package.key" );
    configElectionFiles.setTransmissionPackageCrtFileName ( "totKey.crt" );
    configElectionFiles.setPVFormFileName ( "pv-report-data.xml" );
    configElectionFiles.setElectionFileName ( "election.xml" );
    configElectionFiles.setDeviceParametersFileName ( "device-parameters.xml" );
    configElectionFiles.setElectionStatusFileName ( "election-status.xml" );
    configElectionFiles.setElectionResourcesFileName ( "election-resources.xml" );
    configElectionFiles.setVotingDeviceFileName ( "voting-devices.xml" );
    configElectionFiles.setActivatedCardStatisticsFileName ( "activated-card-statistics.xml" );
    configElectionFiles.setXMLImageFileName ( "xml-image.xml" );
    configElectionFiles.setLanguageFileName ( "default-languages.xml" );
    configElectionFiles.setVotingCentersFileName ( "voting-centers.xml" );
    configElectionFiles.setSecretKeyFile ( "secret-key" );
    configElectionFiles.setUPKeyFile ( "up-key" );
    configElectionFiles.setMachineKeysFile ( "machineKeys.p12" );
    configElectionFiles.setX7SSignedKeyCertifiedFileName ( "signed-key.pem" );
    configElectionFiles.setX7SSignedCertifiedFileName ( "signed-cert.pem" );
    configElectionFiles.setX7SCryptoKeystoreFileName( "certToEncrypt.p12" );
    configElectionFiles.setDateDiagnosticFileName ( "diagnostic-date.txt" );
    configElectionFiles.setZipCodeFileName ( "zip-code.txt" );
    configElectionFiles.setStatisticConfigFileName ( "statistic-configuration.xml" );
    configElectionFiles.setWizardConfigFileName ( "wizard-configuration.xml" );
    configElectionFiles.setMenuConfigFileName ( "menu-configuration.xml" );
    configElectionFiles.setVotingExperienceConfigFileName ( "voting-experience-configuration.xml" );
    configElectionFiles.setProcessStatusResultsFileName ( "ProcessStatusResultsFileName.txt" );
    configElectionFiles.setLoginKeyFileName ( "login-key" );
    configElectionFiles.setRecountSecretKeyFileName ( "recount-key" );
    configElectionFiles.setRecountUPKeyFileName ( "up-recount-key" );
    configElectionFiles.setIssuerCertificatesFolderName ( "ca-certificates/smartmatic" );
    configElectionFiles.setRootCACertificateFileName ( "root-ca.crt" );
    configElectionFiles.setIntermediateCACertificateFileName ( "ECM.crt" );
    configElectionFiles.setPVTemplateFileName ( "PVTemplate.xslt" );
    configElectionFiles.setGeneratedPVReportFileName ( "pv-report.pdf" );
    configElectionFiles.setPVTemplateFolderName ( "PVTemplate" );
    configElectionFiles.setVoteStatisticsFileName ( "vote-statistics.xml" );
    configElectionFiles.setSignatureResourceFileName ( "Signature.xml" );
    configElectionFiles.setSignatureRootFileName ( "Signature.xml" );
    configElectionFiles.setSignatureEmlFileName ( "Signature.xml" );
    configElectionFiles.setSignatureSAESLocalFileName ( "Signature.xml" );
    configElectionFiles.setLogErrorFileName ( "error-log-file.log" );
    configElectionFiles.setLogAuditFileName ( "audit-log-file.log" );
    configElectionFiles.setVotingResultsFileName ( "Eml/voting-results.xml" );
    configElectionFiles.setVotingResultsFilePlainName ( "Eml/V[n][nis].txt" );
    configElectionFiles.setCheckListDiagnoseFileName ( "checklist-diagnose.xml" );
    configElectionFiles.setSkipCalibrationToolFileName ( "skip-calibration-tool" );
    configElectionFiles.setVotingResultsFilePlainName ( "V[n][nis].txt" );
    configElectionFiles.setNationalityDefinitionTypeFileName ( "nationality-code.txt" );
    configElectionFiles.setSwapBasePath("/etc/election.d/config");
    configElectionFiles.setSwapFileName("swap.prep");
    configElectionFiles.setElectionAudioResourcesFileName ( "election-audio-resources.xml" );
    configElectionFiles.setGeneralAudioResourcesFileName ( "general-audio-resources.xml" );
    configElectionFiles.setTextToSpeechVoicesFolder ( "Voices" );

    configPrinter.setTimeToPrintTicket ( 500000 );
    configPrinter.setUsePresentPaperOnline ( false );
    configPrinter.setDefaultPrinterFontFamily ( "FreeSans" );
    configPrinter.setEndPaddingDirectPrint ( 3 );
    configPrinter.setCharsetDirectPrint ( 0 );
    configPrinter.setCharPaddingDirectPrint ( " " );
    configPrinter.setCharIndentationDirectPrint ( " " );
    configPrinter.setCutType ( Smartmatic::SAES::Common::Configuration::CutType::CONFIGURATION_PARTIALCUT );
    configPrinter.setPrinterBackendType ( Smartmatic::SAES::Common::Configuration::PrinterBackendType::CONFIGURATION_GTKPRINTING );
    configPrinter.setDirectPrintDeleteFiles ( true );
    configPrinter.setGeneratePVReportInAllLanguages ( true );
    configPrinter.setCommandPrintingMode ( Smartmatic::SAES::Common::Configuration::CommandPrintingModeType::CONFIGURATION_EMBEDDED_TEXT_CMD );
    configPrinter.setStatusMonitorTimeOut ( 1500 );
    configPrinter.setStatusMonitorCancel ( false );
    configPrinter.setDirectQrUseOptimization ( false );
    configPrinter.setDirectQrCommandHeader ( "<GS>s<10><128>" );
    configPrinter.setDirectQrCommandFooter ( "<GS>s<4><18>" );
    configPrinter.setScreenPreviewWidth ( 450 );
    configPrinter.setScreenPreviewResolutionFactor ( 72 );

    osConfig.setIdHardwareFileFullPath ( "/live/image/osconf/machine.id" );
    osConfig.setScriptRunSmartcard ( "smtt-smartcard" );
    osConfig.setScriptRunPrintBlank ( "smtt-printblank" );
    osConfig.setScriptRunDSortVote ( "smtt-dsort" );

    logConfigReader.setLogAuditReader ( CryptoType::CONFIGURATION_GLOBAL );
    logConfigReader.setLogErrorReader ( CryptoType::CONFIGURATION_GLOBAL );

    SmartcardConfiguration::NivelTestSequence nivelTest;

    nivelTest.push_back ( NivelTestConfig::CONFIGURATION_SIMPLE_SELECT_APPLET );
    nivelTest.push_back ( NivelTestConfig::CONFIGURATION_SIMPLE_WRITE_READ );

    smartcardConfiguration.setNivelTest ( nivelTest );
    smartcardConfiguration.setCallRestartBeforeInitialize ( false );

    SmartcardConfiguration::FatalErrorCodesSequence fatalErrorCodes;

    fatalErrorCodes.push_back ( "80100009" );
    fatalErrorCodes.push_back ( "8010000D" );

    smartcardConfiguration.setFatalErrorCodes ( fatalErrorCodes );
    smartcardConfiguration.setAppletID ( "F00000006203010C0101" );
    smartcardConfiguration.setGlobalPIN ( "3333726314155697" );
    smartcardConfiguration.setPlatformMode ( PlatformMode::CONFIGURATION_GP_211 );
    smartcardConfiguration.setSecureChannelEnabled ( false );
    smartcardConfiguration.setSecurityKeySetVersion ( 0 );
    smartcardConfiguration.setSecurityKeyIndex ( 0 );
    smartcardConfiguration.setSecurityChannelProtocol ( SecurityChannelProtocol::CONFIGURATION_SCP02 );
    smartcardConfiguration.setSecurityChannelProtocolImpl ( SecurityChannelProtocolImpl::CONFIGURATION_SCP02_IMPL_I15 );
    smartcardConfiguration.setSecurityLevel ( SecurityLevel::CONFIGURATION_C_DEC_C_MAC );
    smartcardConfiguration.setSecurityKeyDerivation ( SecurityKeyDerivation::CONFIGURATION_NONE );
    smartcardConfiguration.setDelayAfterOn ( 2 );
    smartcardConfiguration.setConnectionPluginName ( "gppcscconnectionplugin" );
    smartcardConfiguration.setConnectionPluginVersion ( "1.0.1" );
    smartcardConfiguration.setForceRestartSmartcardReader ( false );
    smartcardConfiguration.setForceRestartPCSCD ( false );
    smartcardConfiguration.setWaitStopPCSCD ( 2 );
    smartcardConfiguration.setWaitStartPCSCD ( 4 );
    smartcardConfiguration.setReadRetry ( 3 );

    pcscdConfiguration.setStartService ( false );
    pcscdConfiguration.setStopService ( false );

    SmartcardSimulation smartcardSimulation;

    smartcardSimulation.setEventFile ( "/tmp/SmartcardEvent" );
    smartcardSimulation.setContentFile ( "/tmp/SmartcardContent" );
    smartcardSimulation.setTimeAfterRead ( 1 );
    smartcardSimulation.setTimeBeforeWrite ( 1 );

    smartcardConfiguration.setSmartcardSimulation ( smartcardSimulation );

    ConfigurationBasePath configBasePath;

    configBasePath.setProjectName ( "SAES" );
    configBasePath.setWorkingDirectory ( "SAESLocal" );
    configBasePath.setLanguageImageDirectory ( "images" );
    configBasePath.setBackupFileName ( "machineId" );
    configBasePath.setVotingExperienceValidationScreensDirectory ( "VotingExperienceImages" );
    configBasePath.setRulesBasePath ( Smartmatic::SAES::Common::Configuration::RulesBasePathType::CONFIGURATION_PM_RULE );
    configBasePath.setLiveFileSystemBasePath ( "/live/image" );
    configElectionFiles.setConfigBasePath ( configBasePath );

    SecurityConfiguration security;
    SaltSourceType saltSource;

    unsigned char saltArray[20] =
    {
        0x1, 0x2, 0x3, 0x4, 0x5
        , 0x6, 0x7, 0x8, 0x9, 0xa
        , 0xb, 0xc, 0xd, 0xe, 0xf
        , 0x10, 0x11, 0x12, 0x13, 0x14
    };

    Smartmatic::System::SafeBuffer buffer ( saltArray, sizeof ( saltArray ) );
    std::string defaultSalt;

    Base64Encoder::ConvertToBase64 ( buffer, defaultSalt );

    saltSource.setCalcSalt ( true );
    saltSource.setSepChar ( "|" );
    saltSource.setDefaultSalt ( defaultSalt );

    SecurityFileConfiguration::ValidateCertifiedConfigSequence installationVCS;

    installationVCS.push_back ( ValidateCertifiedConfiguration::CONFIGURATION_DO_NOT_VALIDATE );
    installationFiles.setEncryptedData ( true );
    installationFiles.setSignedData ( Smartmatic::SAES::Common::Configuration::SignedType::CONFIGURATION_NONE );
    installationFiles.setValidateSigned ( false );
    installationFiles.setValidateCertifiedConfig ( installationVCS );
    installationFiles.setCertifiedFilePath ( "" );
    installationFiles.setCertifiedType ( CertifiedType::CONFIGURATION_NONE );
    installationFiles.setValidateCertified ( false );

    SecurityFileConfiguration::ValidateCertifiedConfigSequence externalVCS;

    externalVCS.push_back ( ValidateCertifiedConfiguration::CONFIGURATION_DO_NOT_VALIDATE );
    externalFiles.setEncryptedData ( true );
    externalFiles.setSignedData ( Smartmatic::SAES::Common::Configuration::SignedType::CONFIGURATION_NONE );
    externalFiles.setValidateSigned ( true );
    externalFiles.setValidateCertifiedConfig ( externalVCS );
    externalFiles.setCertifiedFilePath ( "" );
    externalFiles.setCertifiedType ( CertifiedType::CONFIGURATION_NONE );
    externalFiles.setValidateCertified ( false );

    SecurityFileConfiguration::ValidateCertifiedConfigSequence resourcesVCS;

    resourcesVCS.push_back ( ValidateCertifiedConfiguration::CONFIGURATION_DO_NOT_VALIDATE );
    resourcesFiles.setEncryptedData ( true );
    resourcesFiles.setSignedData ( Smartmatic::SAES::Common::Configuration::SignedType::CONFIGURATION_NONE );
    resourcesFiles.setValidateSigned ( true );
    resourcesFiles.setValidateCertifiedConfig ( resourcesVCS );
    resourcesFiles.setCertifiedFilePath ( "" );
    resourcesFiles.setCertifiedType ( CertifiedType::CONFIGURATION_NONE );
    resourcesFiles.setValidateCertified ( false );

    SecurityFileConfiguration::ValidateCertifiedConfigSequence internalVCS;

    internalVCS.push_back ( ValidateCertifiedConfiguration::CONFIGURATION_DO_NOT_VALIDATE );
    internalFiles.setEncryptedData ( true );
    internalFiles.setSignedData ( Smartmatic::SAES::Common::Configuration::SignedType::CONFIGURATION_FULL );
    internalFiles.setValidateSigned ( true );
    internalFiles.setValidateCertifiedConfig ( internalVCS );
    internalFiles.setCertifiedFilePath ( Path::Combine ( configBasePath.getProjectName(), "machineKeys.p12" ) );
    internalFiles.setCertifiedType ( CertifiedType::CONFIGURATION_PCKS12 );
    internalFiles.setValidateCertified ( false );

    SecurityFileConfiguration::ValidateCertifiedConfigSequence exchangeVCS;

    exchangeVCS.push_back ( ValidateCertifiedConfiguration::CONFIGURATION_DO_NOT_VALIDATE );
    exchangeFiles.setEncryptedData ( true );
    exchangeFiles.setSignedData ( Smartmatic::SAES::Common::Configuration::SignedType::CONFIGURATION_FULL );
    exchangeFiles.setValidateSigned ( true );
    exchangeFiles.setValidateCertifiedConfig ( exchangeVCS );
    exchangeFiles.setCertifiedFilePath ( Path::Combine ( configBasePath.getProjectName(), "machineKeys.p12" ) );
    exchangeFiles.setCertifiedType ( CertifiedType::CONFIGURATION_PCKS12 );
    exchangeFiles.setValidateCertified ( false );

    SecurityFileConfiguration::ValidateCertifiedConfigSequence voteVCS;

    voteVCS.push_back ( ValidateCertifiedConfiguration::CONFIGURATION_DO_NOT_VALIDATE );
    printedVoteData.setEncryptedData ( false );
    printedVoteData.setSignedData ( Smartmatic::SAES::Common::Configuration::SignedType::CONFIGURATION_SIMPLE );
    printedVoteData.setValidateSigned ( true );
    printedVoteData.setValidateCertifiedConfig ( voteVCS );
    printedVoteData.setCertifiedFilePath ( Path::Combine ( configBasePath.getProjectName(), "machineKeys.p12" ) );
    printedVoteData.setCertifiedType ( CertifiedType::CONFIGURATION_PCKS12 );
    printedVoteData.setValidateCertified ( false );

    SecurityFileConfiguration::ValidateCertifiedConfigSequence utilFilesVCS;

    utilFilesVCS.push_back ( ValidateCertifiedConfiguration::CONFIGURATION_DO_NOT_VALIDATE );
    utilFiles.setEncryptedData ( false );
    utilFiles.setSignedData ( Smartmatic::SAES::Common::Configuration::SignedType::CONFIGURATION_FULL );
    utilFiles.setValidateSigned ( true );
    utilFiles.setValidateCertifiedConfig ( utilFilesVCS );
    utilFiles.setCertifiedFilePath ( Path::Combine ( configBasePath.getProjectName(), "machineKeys.p12" ) );
    utilFiles.setCertifiedType ( CertifiedType::CONFIGURATION_PCKS12 );
    utilFiles.setValidateCertified ( false );

    SecurityFileConfiguration::ValidateCertifiedConfigSequence votingResultFilesVCS;
    votingResultFilesVCS.push_back ( ValidateCertifiedConfiguration::CONFIGURATION_DO_NOT_VALIDATE );
    votingResultFiles.setEncryptedData ( false );
    votingResultFiles.setSignedData ( Smartmatic::SAES::Common::Configuration::SignedType::CONFIGURATION_FULL );
    votingResultFiles.setValidateSigned ( true );
    votingResultFiles.setValidateCertifiedConfig ( votingResultFilesVCS );
    votingResultFiles.setCertifiedFilePath ( Path::Combine ( configBasePath.getProjectName(), "machineKeys.p12" ) );
    votingResultFiles.setCertifiedType ( CertifiedType::CONFIGURATION_PCKS12 );
    votingResultFiles.setValidateCertified ( false );

    security.setSecurityCryptoMode ( SecurityCryptoMode::CONFIGURATION_CBC );
    security.setInstallationFiles ( installationFiles );
    security.setExternalFiles ( externalFiles );
    security.setResourcesFiles ( resourcesFiles );
    security.setInternalFiles ( internalFiles );
    security.setExchangeFiles ( exchangeFiles );
    security.setUtilFiles ( utilFiles );
    security.setVotingResultFiles ( votingResultFiles );
    security.setPrintedVoteData ( printedVoteData );
    security.setValidateCertifiedTime ( TimeConfiguration::CONFIGURATION_NONE );
    security.setCertifiedTime ( "1970-01-01T00:00:00.00Z" );
    security.setSaltSource ( saltSource );
    security.setSecretKeyEncrypted ( true );
    security.setForceReduceElection ( false );
    security.setVerifySAESFolder ( true );
    security.setVerifyEmlFolder ( false );
    security.setVerifySAESLocalFolder ( true );
    security.setImplementCACertificateFileName ( false );
    security.setImplementItermediateCACertificates ( false );
    security.setImplementTrustedCertificates ( false );
    security.setIterations ( 2048 );
    security.setVerifyExternElements ( false );
    security.setUseAsymmetricKeyForTransmissionPackages ( false );
    security.setPersistenceTransmissionPackages ( false );
    security.setIsCompressElection ( true );
    security.setIsCompressVotingDevices ( false );

    FileSystemVoteConfiguration fileSystemVote;

    fileSystemVote.setImplementRandom ( true );
    fileSystemVote.setMaxfiles ( 5 );
    fileSystemVote.setImplementTime ( TimeConfiguration::CONFIGURATION_CONFIG_TIME );
    fileSystemVote.setFileSystemVoteTime ( "1970-01-01T00:00:00.00Z" );
    fileSystemVote.setBackupMarkFile ( "markFile" );

    security.setFileSystemVote ( fileSystemVote );

    configSAESCommon.setMachineSecuritySchema ( security );
    configSAESCommon.setConfigurationElectionFilesPath ( configElectionFiles );
    configSAESCommon.setConfigurationPrinter ( configPrinter );
    configSAESCommon.setOSConfigurationFiles ( osConfig );
    configSAESCommon.setLogConfigurationReader ( logConfigReader );
    configSAESCommon.setSmartcardConfiguration ( smartcardConfiguration );
    configSAESCommon.setPCSCDConfiguration ( pcscdConfiguration );

    DeviceTypeMonitorPoolConfiguration deviceMonitorConfiguration;
    DeviceTypeMonitorPoolConfiguration::DeviceTypeMonitorSequence deviceMonitorSeq;

    /// Configure Barcode Device Monitor

    DeviceTypeMonitorConfiguration barcodeDeviceConfiguration;
    DeviceTypeMonitorConfiguration::PropertySequence barcodeDeviceProperties;

    barcodeDeviceConfiguration.setType ( DeviceType::CONFIGURATION_BARCODE );
    barcodeDeviceConfiguration.setNode ( "DEVNAME" );
    barcodeDeviceConfiguration.setCustomNode ( "%1" );

    barcodeDeviceProperties.push_back ( PropertyConfiguration ( "ID_VENDOR_ID", "1eab" ) );
    barcodeDeviceProperties.push_back ( PropertyConfiguration ( "ID_MODEL_ID", "0102" ) );
    barcodeDeviceProperties.push_back ( PropertyConfiguration ( "DEVNAME", "/dev/ttyUSB[0-9]" ) );
    barcodeDeviceProperties.push_back ( PropertyConfiguration ( "SUBSYSTEM",  "tty" ) );

    barcodeDeviceConfiguration.setProperty ( barcodeDeviceProperties );

    /// Configure APS and HP Printer Device Monitor

    DeviceTypeMonitorConfiguration printerDeviceConfiguration;
    DeviceTypeMonitorConfiguration::PropertySequence printerDeviceProperties;

    printerDeviceConfiguration.setType ( DeviceType::CONFIGURATION_PRINTER );
    printerDeviceConfiguration.setNode ( "DEVPATH" );
    printerDeviceConfiguration.setCustomNode ( "%1" );

    printerDeviceProperties.push_back ( PropertyConfiguration ( "ID_VENDOR_ID", "1ad4|03f0" ) );
    printerDeviceProperties.push_back ( PropertyConfiguration ( "ID_MODEL_ID", "0003|9411" ) );
    printerDeviceProperties.push_back ( PropertyConfiguration ( "ID_VENDOR_FROM_DATABASE", "APS|Hewlett-Packard" ) );
    printerDeviceProperties.push_back ( PropertyConfiguration ( "SUBSYSTEM",  "usb" ) );
    printerDeviceProperties.push_back ( PropertyConfiguration ( "DEVTYPE",  "usb_device" ) );

    printerDeviceConfiguration.setProperty ( printerDeviceProperties );

    /// Configure Smartcard Device Monitor

    DeviceTypeMonitorConfiguration smartcardDeviceConfiguration;
    DeviceTypeMonitorConfiguration::PropertySequence smartcardDeviceProperties;

    smartcardDeviceConfiguration.setType ( DeviceType::CONFIGURATION_SMARTCARD );
    smartcardDeviceConfiguration.setNode ( "DEVPATH" );
    smartcardDeviceConfiguration.setCustomNode ( "%1" );

    smartcardDeviceProperties.push_back ( PropertyConfiguration ( "PRODUCT", "95[24]0" ) );
    smartcardDeviceProperties.push_back ( PropertyConfiguration ( "ID_VENDOR_FROM_DATABASE", "Alcor Micro Corp" ) );
    smartcardDeviceProperties.push_back ( PropertyConfiguration ( "SUBSYSTEM",  "usb" ) );
    smartcardDeviceProperties.push_back ( PropertyConfiguration ( "DEVTYPE",  "usb_interface" ) );

    smartcardDeviceConfiguration.setProperty ( smartcardDeviceProperties );

    /// Configure USB Storage Device Monitor

    DeviceTypeMonitorConfiguration usbStorageDeviceConfiguration;
    DeviceTypeMonitorConfiguration::PropertySequence usbStorageDeviceProperties;

    usbStorageDeviceConfiguration.setType ( DeviceType::CONFIGURATION_USBSTORAGEMOUNTED );
    usbStorageDeviceConfiguration.setNode ( "DEVPATH" );
    usbStorageDeviceConfiguration.setCustomNode ( "%1" );

    usbStorageDeviceProperties.push_back ( PropertyConfiguration ( "ID_VENDOR_ID", "090c|28cd" ) );
    usbStorageDeviceProperties.push_back ( PropertyConfiguration ( "ID_MODEL_ID", "1000|42ff" ) );
    usbStorageDeviceProperties.push_back ( PropertyConfiguration ( "ID_VENDOR", "SMTT" ) );
    usbStorageDeviceProperties.push_back ( PropertyConfiguration ( "ID_VENDOR_FROM_DATABASE", "Feiya Technology Corp" ) );
    usbStorageDeviceProperties.push_back ( PropertyConfiguration ( "SUBSYSTEM",  "usb" ) );
    usbStorageDeviceProperties.push_back ( PropertyConfiguration ( "DEVTYPE",  "usb_device" ) );

    usbStorageDeviceConfiguration.setProperty ( usbStorageDeviceProperties );

    /// Configure Flash Storage Volume Mount Monitor

    DeviceTypeMonitorConfiguration flashDeviceConfiguration;
    DeviceTypeMonitorConfiguration::PropertySequence flashDeviceProperties;

    flashDeviceConfiguration.setType ( DeviceType::CONFIGURATION_FLASH );
    flashDeviceConfiguration.setNode ( "DEVNAME" );
    flashDeviceConfiguration.setCustomNode ( "grep %1 /proc/mounts | awk '{print $2}'" );

    flashDeviceProperties.push_back ( PropertyConfiguration ( "ID_VENDOR_ID", "090c|28cd" ) );
    flashDeviceProperties.push_back ( PropertyConfiguration ( "ID_MODEL_ID", "1000|42ff" ) );
    flashDeviceProperties.push_back ( PropertyConfiguration ( "ID_VENDOR", "SMTT" ) );
    flashDeviceProperties.push_back ( PropertyConfiguration ( "ID_FS_TYPE",  "ext3" ) );
    flashDeviceProperties.push_back ( PropertyConfiguration ( "DEVNAME",  "/dev/sd[a-z][0-9]" ) );
    flashDeviceProperties.push_back ( PropertyConfiguration ( "SUBSYSTEM",  "block" ) );
    flashDeviceProperties.push_back ( PropertyConfiguration ( "DEVTYPE",  "partition" ) );

    flashDeviceConfiguration.setProperty ( flashDeviceProperties );

    /// Configure Ethernet Monitor

    DeviceTypeMonitorConfiguration ethernetDeviceConfiguration;
    DeviceTypeMonitorConfiguration::PropertySequence ethernetDeviceProperties;

    ethernetDeviceConfiguration.setType ( DeviceType::CONFIGURATION_ETHERNET );
    ethernetDeviceConfiguration.setNode ( "INTERFACE" );
    ethernetDeviceConfiguration.setCustomNode ( "/sys/class/net/%1/carrier" );

    ethernetDeviceProperties.push_back ( PropertyConfiguration ( "INTERFACE", "eth[0-9]" ) );
    ethernetDeviceProperties.push_back ( PropertyConfiguration ( "SUBSYSTEM", "net" ) );

    ethernetDeviceConfiguration.setProperty ( ethernetDeviceProperties );

    /// Configure AlarmBox Monitor

    DeviceTypeMonitorConfiguration alarmBoxDeviceConfiguration;
    DeviceTypeMonitorConfiguration::PropertySequence alarmBoxDeviceProperties;

    alarmBoxDeviceConfiguration.setType ( DeviceType::CONFIGURATION_ALARMBOX );
    alarmBoxDeviceConfiguration.setNode ( "DEVNAME" );
    alarmBoxDeviceConfiguration.setCustomNode ( "/dev/%1" );

    alarmBoxDeviceProperties.push_back ( PropertyConfiguration ( "DEVNAME", "ttyS2" ) );
    alarmBoxDeviceProperties.push_back ( PropertyConfiguration ( "SUBSYSTEM", "tty" ) );

    alarmBoxDeviceConfiguration.setProperty ( alarmBoxDeviceProperties );

    /// Configure P168A Monitor

    DeviceTypeMonitorConfiguration p168aDeviceConfiguration;
    DeviceTypeMonitorConfiguration::PropertySequence p168aDeviceProperties;

    p168aDeviceConfiguration.setType ( DeviceType::CONFIGURATION_P168A );
    p168aDeviceConfiguration.setNode ( "DEVNAME" );
    p168aDeviceConfiguration.setCustomNode ( "/dev/%1" );

    p168aDeviceProperties.push_back ( PropertyConfiguration ( "DEVNAME", "ttyS1" ) );
    p168aDeviceProperties.push_back ( PropertyConfiguration ( "SUBSYSTEM", "tty" ) );

    p168aDeviceConfiguration.setProperty ( p168aDeviceProperties );

    deviceMonitorSeq.push_back ( alarmBoxDeviceConfiguration );
    deviceMonitorSeq.push_back ( p168aDeviceConfiguration );
    deviceMonitorSeq.push_back ( barcodeDeviceConfiguration );
    deviceMonitorSeq.push_back ( printerDeviceConfiguration );
    deviceMonitorSeq.push_back ( smartcardDeviceConfiguration );
    deviceMonitorSeq.push_back ( flashDeviceConfiguration );
    deviceMonitorSeq.push_back ( usbStorageDeviceConfiguration );
    deviceMonitorSeq.push_back ( ethernetDeviceConfiguration );

    deviceMonitorConfiguration.setDeviceTypeMonitor ( deviceMonitorSeq );
    deviceMonitorConfiguration.setSimulatorEventDeviceFilePath ( "/tmp/event-devices" );

    configSAESCommon.setDeviceTypeMonitorPoolConfiguration ( deviceMonitorConfiguration );

    Smartmatic::ConfigurationTypes::NamespaceInfomap map;
#ifdef NAMESPACE_SAES_COMMON_CONFIGURATION
    map[""].name = NAMESPACE_SAES_COMMON_CONFIGURATION;
#endif
    map[""].schema = "";

    std::stringstream ss;
    serializeSaesCommonConfiguration ( ss, configSAESCommon, map, "UTF-8" );
    ConfigurationPersistenceManager::getInstance()->SetLibraryConfigFile ( "SAESCommon", ss.str() );
}

SAESCommonConfigurationManager * SAESCommonConfigurationManager::getInstance ( SAESCommonConfigurationManager * externManager )
{
    if ( ! instance )
    {
        instance = ( externManager == NULL ? new SAESCommonConfigurationManager() : externManager );
    }

    return instance;
}

PrinterBackendType & SAESCommonConfigurationManager::getPrinterBackendType()
{
    return getConfiguration()->getConfigurationPrinter().getPrinterBackendType();
}

bool SAESCommonConfigurationManager::getStatusMonitorCancel()
{
    return getConfiguration()->getConfigurationPrinter().getStatusMonitorCancel();
}

bool SAESCommonConfigurationManager::getDirectQrUseOptimization ()
{
    return getConfiguration()->getConfigurationPrinter().getDirectQrUseOptimization ();
}

std::string SAESCommonConfigurationManager::getDirectQrCommandHeader ()
{
    return getConfiguration()->getConfigurationPrinter().getDirectQrCommandHeader ();
}

std::string SAESCommonConfigurationManager::getDirectQrCommandFooter ()
{
    return getConfiguration()->getConfigurationPrinter().getDirectQrCommandFooter ();
}

int SAESCommonConfigurationManager::getStatusMonitorTimeOut()
{
    return getConfiguration()->getConfigurationPrinter().getStatusMonitorTimeOut();
}

bool SAESCommonConfigurationManager::isVerifyExternElements()
{
    return getConfiguration()->getMachineSecuritySchema().getVerifyExternElements();
}

bool SAESCommonConfigurationManager::isCompressElection()
{
    return getConfiguration()->getMachineSecuritySchema().getIsCompressElection();
}

bool SAESCommonConfigurationManager::isCompressVotingDevices()
{
    return getConfiguration()->getMachineSecuritySchema().getIsCompressVotingDevices();
}

bool SAESCommonConfigurationManager::isUseAsymmetricKeyForTransmissionPackages()
{
    return getConfiguration()->getMachineSecuritySchema().getUseAsymmetricKeyForTransmissionPackages();
}

SecurityConfiguration & SAESCommonConfigurationManager::getSecurityConfiguration()
{
    return getConfiguration()->getMachineSecuritySchema();
}

PCSCDConfiguration & SAESCommonConfigurationManager::getPCSCDConfiguration()
{
    return getConfiguration()->getPCSCDConfiguration();
}

FileSystemVoteConfiguration & SAESCommonConfigurationManager::getFileSystemVoteConfiguration()
{
    return getSecurityConfiguration().getFileSystemVote();
}

ConfigurationBasePath & SAESCommonConfigurationManager::getConfigBasePath()
{
    return getConfiguration()->getConfigurationElectionFilesPath().getConfigBasePath();
}

std::string SAESCommonConfigurationManager::getDateDiagnosticFileName()
{
    return getConfiguration()->getConfigurationElectionFilesPath().getDateDiagnosticFileName();
}

bool SAESCommonConfigurationManager::isPVInAllLanguages()
{
    return getConfiguration()->getConfigurationPrinter().getGeneratePVReportInAllLanguages();
}

bool SAESCommonConfigurationManager::isForceReduceElection()
{
    return getConfiguration()->getMachineSecuritySchema().getForceReduceElection();
}

OSConfigurationFiles & SAESCommonConfigurationManager::getOSConfigurationFiles()
{
    return getConfiguration()->getOSConfigurationFiles();
}

ConfigurationPrinter & SAESCommonConfigurationManager::getConfigurationPrinter()
{
    return getConfiguration()->getConfigurationPrinter();
}
