/**
 * @file	LogCryptoPool.hxx
 * @brief	Declaration of the singleton entity that contains the cryptographic instance collection of the log type.
 * @author	Youssef.tourkmani
 * @date	30/10/2012
 */

#ifndef LOGTRANSFORMATION_HXX_
#define LOGTRANSFORMATION_HXX_

#include <map>
#include <Configuration/SaesCommonConfiguration-schema.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ICryptoLayoutTransformer;
    }

    namespace SAES
    {
        namespace Log
        {
            /**
             * @class	LogCryptoPool
             * @brief	Container that saves the references to the cryptographic instances of log type.
             * This container index those instances according to its code.
             * @author	Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
             * @date	25-07-2011
             */
            class LogCryptoPool
            {
                private:
                    /**
                     * @fn	LogCryptoPool::LogCryptoPool();
                     * @brief	Default constructor initializes an instance of the LogCryptoPool Class.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     */
                    LogCryptoPool();


                    std::map < std::string, Smartmatic::Log::ICryptoLayoutTransformer * > cryptoMap; /**<Map that contains the references to the cryptographic instances of log type. This map index those instances according to its code.*/
                    static LogCryptoPool * instance;	///< The pointer reference to LogCryptoPoll required to access to the instance.

                public:
                    /**
                     * @fn	virtual LogCryptoPool::~LogCryptoPool();
                     * @brief	Finalizes an instance of the LogCryptoPool class.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     */
                    virtual ~LogCryptoPool();

                    /**
                     * @fn	void LogCryptoPool::putLogCrypto( Smartmatic::Log::ICryptoLayoutTransformer * iCrypto );
                     * @brief	Puts a reference to a cryptographic instance of log type in the map. It also index this reference by its code.
                     *  The code must be unique for each instance in order to avoid override other references.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param [in,out]	iCrypto	The pointer reference to the selected cryptographic instance of the log type.
                     */
                    void putLogCrypto ( Smartmatic::Log::ICryptoLayoutTransformer * iCrypto );
                    /**
                     * @fn	Smartmatic::Log::ICryptoLayoutTransformer * LogCryptoPool::getLogCrypto( Smartmatic::SAES::Common::Configuration::CryptoType type );
                     * @brief	Gets a pointer reference to the selected cryptographic instance.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param	type	The value used to select the reference from the map. It represents the code of the log instance.
                     * @return	The pointer reference to the cryptographic instance of the log type if matches successful; otherwise <tt>NULL</tt>;
                     */
                    Smartmatic::Log::ICryptoLayoutTransformer * getLogCrypto ( Smartmatic::SAES::Common::Configuration::CryptoType type );
                    /**
                     * @fn	std::map < std::string, Smartmatic::Log::ICryptoLayoutTransformer * > & LogCryptoPool::getCryptoMap();
                     * @brief	Gets a reference to map that contains the references to the cryptographic instances of log type.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @return	The reference to map that contains the references to the cryptographic instances of log type.
                     */
                    std::map < std::string, Smartmatic::Log::ICryptoLayoutTransformer * > & getCryptoMap();

                    /**
                     * @fn	static LogCryptoPool * LogCryptoPool::getInstance();
                     * @brief	Gets the only instance of the container. If this instance does not exist, then it creates a new one.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @return	The pointer reference to LogCryptoPoll required to access to the instance.
                     */
                    static LogCryptoPool * getInstance();
            };
        }
    }
}

#endif /* LOGTRANSFORMATION_HXX_ */
