/**
 * @file	LogReader.cc
 * @brief	Implementation of reader that provides access to log data obtained from the files.
 * @author	Youssef.tourkmani
 * @date	30/10/2012
 */

#include "LogReader.h"
#include "LogCryptoPool.hxx"
#include <System/IO/File.h>
#include <System/Utils/StringUtils.hxx>
#include <System/Utils/SafeBufferUtils.hxx>
#include <Log/ICryptoLayoutTransformer.hxx>
#include <Log/SMTTLogManager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Log;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::System;

ISMTTLog * LogReader::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Log.LogReader", GETTEXT_PACKAGE );

/**
 * @enum LogPart
 * @brief Define parts log line
 */
enum LogPart
{
    LOG_DATE
    , LOG_MESSAGE
    , LOG_LEVEL
    , LOG_TIME_STAMP
    , LOG_END
};

/**
 * @enum LogMessagePart
 * @brief Define message parts log line
 */
enum LogMessagePart
{
    LOG_MESSAGE_LIBRARY
    , LOG_MESSAGE_STR
};

LogReader::LogReader ( const std::string& logFile, unsigned int sizePage, const Smartmatic::SAES::Common::Configuration::CryptoType& type )
{
    this->sizePage = sizePage;
    this->type = type;
    ProcessFilesToGetLogLines ( logFile );
}

std::vector<LogLine> LogReader::getLogLines()
{
    return this->logLines;
}

void  LogReader::ProcessFilesToGetLogLines ( const std::string& logFile )
{
    SafeBuffer buffer;

    if ( File::getInstance ()->openFileToBuffer ( logFile, buffer ) )
    {
        SafeBufferUtils util ( buffer );
        std::vector < SafeBuffer > result = util.splitByEndLine ();
        std::vector < SafeBuffer >::iterator it;

        for ( it = result.begin ();
                it != result.end ();
                ++it )
        {
            std::string line ( ( const char * ) ( *it ).GetData(), ( *it ).GetSize() );

            if ( line.empty() )
            {
                continue;
            }

            line.erase(std::remove(line.begin(), line.end(), 0x00), line.end());

            ICryptoLayoutTransformer * interface = LogCryptoPool::getInstance()->getLogCrypto ( type );

            if ( interface )
            {
                line = interface->decode ( line );
            }

            LogLine log = ReadLine ( line );

            if ( !log.getDate().empty() )
            {
                this->logLines.push_back ( log );
            }
        }
    }
}

LogLine LogReader::ReadLine ( std::string line )
{
    std::list<std::string> lines;
    std::list<std::string>::iterator it;

    StringUtils::splitToList ( line, " - ", lines );

    LogLine logLine;
    int count = LOG_DATE;

    for ( it = lines.begin();
            it != lines.end();
            ++it, count++ )
    {
        switch ( ( LogPart ) count )
        {
            case LOG_DATE:
                logLine.setDate ( *it );
                break;

            case LOG_MESSAGE:
                StringToMessagelog ( *it, logLine );
                break;

            case LOG_LEVEL:
                logLine.setLevel ( StringToLevelLog ( *it ) );
                break;

            case LOG_TIME_STAMP:
                logLine.setTimeSinceStart ( *it );
                break;

            case LOG_END:
            default:
                break;
        }
    }

    return logLine;
}

void LogReader::StringToMessagelog ( const std::string& line, LogLine& logLine )
{
    std::list<std::string> list;
    std::list<std::string>::iterator it;

    StringUtils::splitToList ( line, "#", list );

    int count = LOG_MESSAGE_LIBRARY;

    for ( it = list.begin();
            it != list.end();
            ++it, count++ )
    {
        switch ( ( LogMessagePart ) count )
        {
            default:
            case LOG_MESSAGE_LIBRARY:
                logLine.setLibrary ( *it );
                break;

            case LOG_MESSAGE_STR:
                logLine.setMessage ( dgettext ( logLine.getLibrary().c_str(), ( *it ).c_str() ) );
                break;
        }
    }
}

LogLine::LevelLog LogReader::StringToLevelLog ( const std::string& level )
{
    LogLine::LevelLog result;

    if ( level == "TRACE" )
    {
        result = LogLine::TRACE;
    }
    else if ( level == "DEBUG" )
    {
        result = LogLine::DEBUG;
    }
    else if ( level == "INFO" )
    {
        result = LogLine::INFO;
    }
    else if ( level == "WARN" )
    {
        result = LogLine::WARNING;
    }
    else if ( level == "ERROR" )
    {
        result = LogLine::ERROR;
    }
    else if ( level == "FATAL" )
    {
        result = LogLine::FATAL;
    }
    else
    {
        result = LogLine::TRACE;
    }

    return result;
}

LogPage LogReader::getPage ( unsigned int page ) throw ( LogReaderException )
{
    if ( page == 0 || page > getPageCount() )
    {
        page = 1;
    }

    unsigned int start = ( page - 1 ) * this->sizePage;
    std::vector<LogLine> logLinesPage;

    for ( unsigned int i = start;
            i < start + this->sizePage &&
            i < logLines.size(); i++ )
    {
        logLinesPage.push_back ( this->logLines[i] );
    }

    LogPage logPage ( page, logLinesPage );
    return logPage;
}

unsigned int LogReader::getPageCount()
{
    int result = 0;
    int sizeTotal = this->logLines.size();

    if ( sizeTotal == 0 )
    {
        result = 1;
    }
    else
    {
        result = sizeTotal / this->sizePage;

        if ( sizeTotal % this->sizePage != 0 )
        {
            result++;
        }
    }

    return result;
}

