/**
 * @file DirectPrintingBackend.cpp
 * @brief Implementation of direct printing strategy which provides the functionalities to render and print documents.
 * @date 06/03/2011
 * @author Freddy Gomez
 */

#include "DirectPrintingBackend.h"
#include "Runtime/Environment/saes-directories.h"
#include "Configuration/s-a-e-s-common-conf-manager.h"
#include <Log/SMTTLogManager.h>
#include <Graphics/Encoding2D/QREncoding.h>
#include <System/IO/File.h>
#include <System/IO/Directory.h>
#include <System/IO/Path.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::SAES::Printing;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::System;
using namespace Smartmatic::Log;
using namespace Smartmatic::Graphics::Encoding2D;
using namespace Smartmatic::System::IO;

ISMTTLog * DirectPrintingBackend::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Printing.DirectPrintingBackend", GETTEXT_PACKAGE );

DirectPrintingBackend::DirectPrintingBackend() :
    currentFilename ( "" )
    , currentOptions ()
    , mOverallResult ( false, "", PrintDocumentResult::PRINTING_NONE )
{
    std::stringstream ss;
    ss << __FILE__ << " " << __func__ << " Line " << __LINE__;
    logger->Debug ( ss.str() );
    // Initialize french codepage mapping
    /*iso_8859_1_2_aps_french [0xe0] = 0x40;
    iso_8859_1_2_aps_french [0xb0] = 0x5b;
    iso_8859_1_2_aps_french [0xe7] = 0x5c;
    iso_8859_1_2_aps_french [0xa7] = 0x5d;
    iso_8859_1_2_aps_french [0xe9] = 0x7b;
    iso_8859_1_2_aps_french [0xf9] = 0x7c;
    iso_8859_1_2_aps_french [0xe8] = 0x7d;
    iso_8859_1_2_aps_french [0xeb] = 0x89;

    iso_8859_1_2_aps_french [0xfc] = 0x81;
    iso_8859_1_2_aps_french [0xe2] = 0x83;
    iso_8859_1_2_aps_french [0xe4] = 0x84;
    iso_8859_1_2_aps_french [0xea] = 0x88;
    iso_8859_1_2_aps_french [0xef] = 0x8b;
    iso_8859_1_2_aps_french [0xee] = 0x8c;
    iso_8859_1_2_aps_french [0xc4] = 0x8e;
    iso_8859_1_2_aps_french [0xc9] = 0x90;
    iso_8859_1_2_aps_french [0xf4] = 0x93;
    iso_8859_1_2_aps_french [0xf6] = 0x94;
    iso_8859_1_2_aps_french [0xf2] = 0x95;
    iso_8859_1_2_aps_french [0xfb] = 0x96;
    iso_8859_1_2_aps_french [0xed] = 0xa1;
    iso_8859_1_2_aps_french [0xf3] = 0xa2;
    iso_8859_1_2_aps_french [0xfa] = 0xa3;*/
    utf8_to_aps[0xC2A0] = 0x20;
    utf8_to_aps[0xC380] = 0xB7;
    utf8_to_aps[0xC381] = 0xB5;
    utf8_to_aps[0xC382] = 0xB6;
    utf8_to_aps[0xC383] = 0xC7;
    utf8_to_aps[0xC384] = 0x8E;
    utf8_to_aps[0xC385] = 0x8F;
    utf8_to_aps[0xC386] = 0x92;
    utf8_to_aps[0xC387] = 0x87;
    utf8_to_aps[0xC388] = 0xD4;
    utf8_to_aps[0xC389] = 0x90;
    utf8_to_aps[0xC38A] = 0x90;
    utf8_to_aps[0xC38B] = 0xD3;
    utf8_to_aps[0xC38C] = 0x8D;
    utf8_to_aps[0xC38D] = 0xD6;
    utf8_to_aps[0xC38E] = 0x8B;
    utf8_to_aps[0xC38F] = 0xD8;
    utf8_to_aps[0xC390] = 0xD1;
    utf8_to_aps[0xC391] = 0xA5;
    utf8_to_aps[0xC392] = 0xE3;
    utf8_to_aps[0xC393] = 0xE0;
    utf8_to_aps[0xC394] = 0xE2;
    utf8_to_aps[0xC395] = 0xE5;
    utf8_to_aps[0xC396] = 0x99;
    utf8_to_aps[0xC397] = 0x2A;
    utf8_to_aps[0xC398] = 0x9D;
    utf8_to_aps[0xC399] = 0xEB;
    utf8_to_aps[0xC39A] = 0xE9;
    utf8_to_aps[0xC39B] = 0xEA;
    utf8_to_aps[0xC39C] = 0x9A;
    utf8_to_aps[0xC39D] = 0xED;
    utf8_to_aps[0xC39E] = 0xE8;
    utf8_to_aps[0xC39F] = 0xE1;
    utf8_to_aps[0xC3A0] = 0x85;
    utf8_to_aps[0xC3A1] = 0xA0;
    utf8_to_aps[0xC3A2] = 0x83;
    utf8_to_aps[0xC3A3] = 0xC6;
    utf8_to_aps[0xC3A4] = 0x84;
    utf8_to_aps[0xC3A5] = 0x86;
    utf8_to_aps[0xC3A6] = 0x91;
    utf8_to_aps[0xC3A7] = 0x87;
    utf8_to_aps[0xC3A8] = 0x8A;
    utf8_to_aps[0xC3A9] = 0x82;
    utf8_to_aps[0xC3AA] = 0x88;
    utf8_to_aps[0xC3AB] = 0x89;
    utf8_to_aps[0xC3AC] = 0x8D;
    utf8_to_aps[0xC3AD] = 0xA1;
    utf8_to_aps[0xC3AE] = 0xA1;
    utf8_to_aps[0xC3AF] = 0x8B;
    utf8_to_aps[0xC3B0] = 0xE4;
    utf8_to_aps[0xC3B1] = 0xA4;
    utf8_to_aps[0xC3B2] = 0x95;
    utf8_to_aps[0xC3B3] = 0xA2;
    utf8_to_aps[0xC3B4] = 0x93;
    utf8_to_aps[0xC3B5] = 0xE4;
    utf8_to_aps[0xC3B6] = 0x94;
    utf8_to_aps[0xC3B7] = 0xF6;
    utf8_to_aps[0xC3B8] = 0x9B;
    utf8_to_aps[0xC3B9] = 0x97;
    utf8_to_aps[0xC3BA] = 0xA3;
    utf8_to_aps[0xC3BB] = 0x96;
    utf8_to_aps[0xC3BC] = 0x81;
    utf8_to_aps[0xC3BD] = 0xEC;
    utf8_to_aps[0xC3BE] = 0xE7;
    utf8_to_aps[0xC3BF] = 0x98;
    deleteFiles = Smartmatic::SAES::Common::Configuration::SAESCommonConfigurationManager::getInstance()->getConfiguration()->getConfigurationPrinter().getDirectPrintDeleteFiles();
    CommandPrintingModeType cmd_print_mode = Smartmatic::SAES::Common::Configuration::SAESCommonConfigurationManager::getInstance()->getConfiguration()->getConfigurationPrinter().getCommandPrintingMode();

    switch ( cmd_print_mode )
    {
        case CommandPrintingModeType::CONFIGURATION_BINARY_CMD:
            binaryOutput = true;
            break;

        case CommandPrintingModeType::CONFIGURATION_EMBEDDED_TEXT_CMD:
            binaryOutput = false;
            break;

        default:
            binaryOutput = false;
            break;
    }
}

DirectPrintingBackend::~DirectPrintingBackend()
{
    std::stringstream ss;
    ss << __FILE__ << " " << __func__ << " Line " << __LINE__;
    logger->Debug ( ss.str() );
    utf8_to_aps.clear();
}

void DirectPrintingBackend::Init()
{
    std::stringstream ss;
    ss << __FILE__ << " " << __func__ << " Line " << __LINE__;
    logger->Debug ( ss.str() );
    SaesDirectories directories;
    currentFilename = directories.getTemporaryFileFullPath() + ".txt";
    unsigned char charCode = SAESCommonConfigurationManager::getInstance()->getConfiguration()->getConfigurationPrinter().getCharsetDirectPrint();
    AddNewCharSetCommand ( charCode );
}

void DirectPrintingBackend::cleanPrintQueue()
{
    logger->Debug ( "Emptying filequeue" );

    while ( !filequeue.empty() )
    {
        filequeue.pop();
    }
}

void DirectPrintingBackend::ChangeFile()
{
    std::stringstream ss;
    ss << __FILE__ << " " << __func__ << " Line " << __LINE__;
    logger->Debug ( ss.str() );

    bool addFile = !currentFilename.empty() && File::getInstance()->canOpenFile ( currentFilename );

    if ( addFile )
    {
        filequeue.push ( currentFilename );
    }
    else //just remove the file from the system
    {
        File::getInstance()->DeleteFile ( currentFilename );
    }

    SaesDirectories directories;
    currentFilename = directories.getTemporaryFileFullPath() + ".txt";
}

void DirectPrintingBackend::End()
{
    std::stringstream ss;
    ss << __FILE__ << " " << __func__ << " Line " << __LINE__;
    logger->Debug ( ss.str() );

    LPSpooler * tmp_spool;
    bool addFile = !currentFilename.empty() && File::getInstance()->canOpenFile ( currentFilename );
    int endPadding = SAESCommonConfigurationManager::getInstance()->getConfiguration()->getConfigurationPrinter().getEndPaddingDirectPrint();
    CutType cutType = SAESCommonConfigurationManager::getInstance()->getConfiguration()->getConfigurationPrinter().getCutType();
    bool needToAddMoreDataToFile = false;

    if ( endPadding > 0 )
    {
        needToAddMoreDataToFile = true;
    }

    if ( cutType != CutType::CONFIGURATION_NOCUT )
    {
        needToAddMoreDataToFile = true;
    }

    if ( needToAddMoreDataToFile == false && addFile )
    {
        filequeue.push ( currentFilename );
    }

    //file is empty and no data to add, erase the file
    if ( needToAddMoreDataToFile == false && !addFile )
    {
        File::getInstance()->DeleteFile ( currentFilename );
    }

    //there is data to add
    if ( needToAddMoreDataToFile == true )
    {
        if ( !addFile ) //file does not exist, create one
        {
            SaesDirectories directories;
            currentFilename = directories.getTemporaryFileFullPath() + ".txt";
        }
    }

    for ( int i = 0; i < endPadding; i++ )
    {
        this->AddFeedPaperCommand ( 40 );
    }

    if ( cutType == CutType::CONFIGURATION_CUT )
    {
        this->AddNewFullCutCommand();
    }
    else if ( cutType == CutType::CONFIGURATION_PARTIALCUT )
    {
        this->AddNewPartialCutCommand();
    }

    if ( needToAddMoreDataToFile == true || addFile )
    {
        filequeue.push ( currentFilename );
    }

    //Print the list of files
    while ( !filequeue.empty() )
    {
        LPSpooler::JobStateEnumType printop_res;
        std::string file = filequeue.front();
        std::stringstream ss;
        ss << __FILE__ << " " << __func__ << " Line " << __LINE__ << " - Printing File " << file;
        logger->Debug ( ss.str() );
        tmp_spool = new LPSpooler ( file );
        tmp_spool->AddOption ( "rmtop", "True" );
        printop_res = tmp_spool->Print();

        if ( printop_res != LPSpooler::LPSPOOLER_JOB_COMPLETED )
        {
            logger->Debug ( "Error " + tmp_spool->GetSpoolerErrorMessage ( printop_res ) );
            mOverallResult.setResult (
                false
                , tmp_spool->GetSpoolerErrorMessage ( printop_res )
                , SpoolerResToPrintingRes ( printop_res ) );
            delete ( tmp_spool );
            break;
        }
        else
        {
            mOverallResult.setResult (
                true
                , tmp_spool->GetSpoolerErrorMessage ( printop_res )
                , SpoolerResToPrintingRes ( printop_res ) );
        }

        delete ( tmp_spool );

        if ( deleteFiles )
        {
            if ( !File::getInstance()->DeleteFile ( file ) )
            {
                std::stringstream ss;
                ss << __FILE__ << " " << __func__ << " Line " << __LINE__ << " Error deleting file";
                logger->Debug ( ss.str() );
            }
        }

        filequeue.pop();
    }
}

void DirectPrintingBackend::JobNotifListenerThreadBody()
{
    std::stringstream ss;
    ss << __FILE__ << " " << __func__ << " Line " << __LINE__;
    logger->Debug ( ss.str() );
    Glib::Thread::Exit();
}

void DirectPrintingBackend::onPrintJobStateChanged ( LPSpooler & sender )
{
    std::stringstream ss;
    ss << __FILE__ << " " << __func__ << " Line " << __LINE__;
    logger->Debug ( ss.str() );
}

void DirectPrintingBackend::UtfToPrinterCharSet ( const Glib::ustring & utf8_string, std::string & decodeResult )
{
    unsigned int siz = utf8_string.bytes();
    unsigned char * data = new unsigned char[siz];
    utf8_string.copy ( ( char * ) data, siz, 0 );

    for ( unsigned int i = 0; i < siz; i++ )
    {
        if ( data[i] == 0xC3 || data[i] == 0xC2 )
        {
            i++; //goto next char

            if ( i < siz )
            {
                int code = ( data[i - 1] << 8 ) |  data[i];
                decodeResult.push_back ( utf8_to_aps[code] );
            }
        }
        else
        {
            decodeResult.push_back ( data[i] );
        }
    }

    delete [] data;
}

//TODO: Add parameters to the methods


void DirectPrintingBackend::DrawString (
    std::list<Glib::ustring> & textContent
    , std::list<DocumentTextOptions> & textContentOptionList )
{
    std::vector<DocumentTextOptions> textOptionList ( textContentOptionList.begin(), textContentOptionList.end() );
    std::vector<Glib::ustring> text;
    std::_List_iterator<Glib::ustring> iterator = textContent.begin();

    while ( iterator != textContent.end() )
    {
        text.push_back ( *iterator++ );
    }

    if ( textOptionList.size() > 1 )
    {
        //columns number
        int columns = textOptionList.size();
        int rows = text.size() / columns;
        int reminder = text.size() % columns;

        for ( int r = 0; r < rows; r++ )
        {
            for ( int c = 0; c < columns; c++ )
            {
                std::string decodedInner;
                UtfToPrinterCharSet ( text[ ( r * columns ) + c], decodedInner );
                DrawCell (
                    textOptionList[c]
                    , decodedInner
                    , columns );
            }

            AddLineFeedCommand();
        }

        //Draw incomplete row
        for ( int e = 0; e < reminder; e++ )
        {
            std::string decodedInner;
            UtfToPrinterCharSet ( text[ ( rows * columns ) + e], decodedInner );
            DrawCell (
                textOptionList[e]
                , decodedInner
                , columns );
        }

        //New line for the uncomplete row
        if ( reminder > 0 )
        {
            AddLineFeedCommand();
        }
    }
    else
    {
        std::string decoded;
        //Update parameters
        UpdateParameters ( textOptionList.front(), true );
        //Send text one by one
        std::_List_const_iterator<Glib::ustring> iterator = textContent.begin();

        while ( iterator != textContent.end() )
        {
            UtfToPrinterCharSet ( *iterator++, decoded );
            SafeBuffer buffer ( decoded );
            File::getInstance()->addBufferToFile ( buffer, currentFilename );
        }

        AddLineFeedCommand();
    }
}

void DirectPrintingBackend::DrawCell ( DocumentTextOptions & options, std::string & text, int columns )
{
    //int allowedIndent = 0;
    int counter = 0;
    std::string finalText;
    // update the options of the text
    UpdateParameters ( options, false );
    //sub string to the
    int maxChars = getCharactersPerline ( options.getFontSize() ) / columns;
    /*Glib::ustring cell;
    cell.assign(text);*/
    int identationLevel = options.getIndentationLevel();
    //add indentation
    std::string identationChar = SAESCommonConfigurationManager::getInstance()->getConfiguration()->getConfigurationPrinter().getCharIndentationDirectPrint();

    while ( counter++ < identationLevel )
    {
        finalText.append ( identationChar );
    }

    finalText.append ( text );

    //trunc the string
    if ( finalText.size() > static_cast<unsigned int> ( maxChars ) )
    {
        finalText = finalText.substr ( 0, maxChars );
    }

    int padding = maxChars - finalText.size();
    std::string paddingChar = SAESCommonConfigurationManager::getInstance()->getConfiguration()->getConfigurationPrinter().getCharPaddingDirectPrint();
    counter = 0;

    while ( counter++ < padding )
    {
        finalText.append ( paddingChar );
    }

    SafeBuffer buffer ( finalText );
    File::getInstance()->addBufferToFile ( buffer, currentFilename );
}

void DirectPrintingBackend::UpdateParameters ( DocumentTextOptions & newParams, bool includeJustification )
{
    if ( currentOptions.getFontSize() != newParams.getFontSize() )
    {
        ChangeFontSize ( newParams.getFontSize() );
        currentOptions.setFontSize ( newParams.getFontSize() );
    }

    if ( currentOptions.getJustification() != newParams.getJustification() && includeJustification )
    {
        ChangeJustification ( newParams.getJustification() );
        currentOptions.setJustification ( newParams.getJustification() );
    }

    if ( currentOptions.getFontStyle() != newParams.getFontStyle() )
    {
        ChangeFontStyle ( newParams.getFontStyle() );
        currentOptions.setFontStyle ( newParams.getFontStyle() );
    }

    if ( currentOptions.getIndentationLevel() != newParams.getIndentationLevel() )
    {
        currentOptions.setIndentationLevel ( newParams.getIndentationLevel() );
    }

    if ( currentOptions.getLineSpacing() != newParams.getLineSpacing() )
    {
        currentOptions.setLineSpacing ( newParams.getLineSpacing() );
    }
}

void DirectPrintingBackend::DrawImage ( Glib::RefPtr<Gdk::Pixbuf> image )
{
    std::stringstream ss;
    ss << __FILE__ << " " << __func__ << " Line " << __LINE__;
    logger->Debug ( ss.str() );

    if ( !currentFilename.empty() )
    {
        //push the file on the queue
        ChangeFile();
    }

    //create a file with the image and push it to the queque
    SaesDirectories directories;
    std::string imageFileName = directories.getTemporaryFileFullPath() + ".png" ;

    std::string path = Path::GetDirectory ( imageFileName );
    Directory::getInstance()->Mkdir ( path );

    image->save ( imageFileName , "png" );
    filequeue.push ( imageFileName );
}

void DirectPrintingBackend::DrawQRCode ( QRCodeContent & qr )
{
    if ( binaryOutput )
    {
        DrawQRCodeBin ( qr );
    }
    else
    {
        DrawQRCodeText ( qr );
    }
}

void DirectPrintingBackend::DrawQRCodeText ( QRCodeContent qr )
{
    std::stringstream ss;
    int correction_level = 0;

    switch ( qr.getCorrectionLevel() )
    {
        case QR_ECLEVEL_L:
            correction_level = 0;
            break;

        case QR_ECLEVEL_M:
            correction_level = 1;
            break;

        case QR_ECLEVEL_Q:
            correction_level = 2;
            break;

        case QR_ECLEVEL_H:
            correction_level = 3;
            break;

        default:
            break;
    }

    if ( SAESCommonConfigurationManager::getInstance()->getDirectQrUseOptimization() )
    {
        ss << SAESCommonConfigurationManager::getInstance()->getDirectQrCommandHeader ();
    }

    ss << "<ESC><.><"
       << qr.getPixelSize() << "><"                       /* module size */
       << qr.getVersion () << "><"                        /* qr version */
       << correction_level << "><"                        /* correction level */
       << 2 << "><"                                       /* 8 bit mode */
       << 0 << "><"                                       /* case sensitvity */
       << 1 << "><"                                       /* bin mode */
       << qr.getData ().GetBase64Data ().c_str ()         /* data argument */
       << ">";

    if ( SAESCommonConfigurationManager::getInstance()->getDirectQrUseOptimization() )
    {
        ss << SAESCommonConfigurationManager::getInstance()->getDirectQrCommandFooter ();
    }

    logger->Debug ( ss.str () );
    SafeBuffer buffer ( ss.str () );
    File::getInstance()->addBufferToFile ( buffer, currentFilename );
}

void DirectPrintingBackend::DrawQRCodeBin ( QRCodeContent qr )
{
    const char * cmd_data = qr.getData().GetBase64Data().c_str();
    int cmd_length = strlen ( cmd_data ) + 9; // plus arguments and the trailing zero
    char * cmdbuff = new char[cmd_length];
    char ver0 = '0'
                , ver1 = '0'
                         , correction_level
                         , mode = '2'
                                  , bin_mode = '1';
    memset ( cmdbuff, 0, cmd_length );

    switch ( qr.getCorrectionLevel() )
    {
        case QR_ECLEVEL_L:
            correction_level = '0';
            break;

        case QR_ECLEVEL_M:
            correction_level = '1';
            break;

        case QR_ECLEVEL_Q:
            correction_level = '2';
            break;

        case QR_ECLEVEL_H:
            correction_level = '3';
            break;

        default:
            correction_level = '0';
            break;
    }

    snprintf ( cmdbuff, cmd_length, "%c%c%c%c%c%c%c%s%c"
               , ESC
               , '.'
               , ver0
               , ver1
               , correction_level
               , mode
               , bin_mode
               , cmd_data
               , ESC );

    SafeBuffer buffer ( ( unsigned char * ) cmdbuff, cmd_length );
    File::getInstance()->addBufferToFile ( buffer, currentFilename );

    delete [] cmdbuff;
}



void DirectPrintingBackend::ChangeJustification ( DocumentJustifitacion justification )
{
    unsigned char code ( 0 );

    switch ( justification )
    {
        default:
        case LeftJustified:
            code = 2;
            break;

        case Centered:
            code = 0;
            break;

        case RightJustified:
            code = 1;
            break;
    }

    AddNewJustificationCommand ( code );
}
void DirectPrintingBackend::ChangeFontSize ( DocumentFontSize newFont )
{
    unsigned char code ( 0 );

    switch ( newFont )
    {
        default:
        case TinyFont:
        case SmallFont:
            code = 2;
            break;

        case MediumFont:
            code = 0;
            break;

        case LargeFont:
        case Biggest:
            code = 1;
            break;
    }

    AddNewFontCommand ( code );
}

void DirectPrintingBackend::ChangeFontStyle ( DocumentFontStyle newFontStyle )
{
    bool underlined = ( ( Underlined & newFontStyle ) != Normal );
    AddNewTextModeCommand ( underlined );
}
int DirectPrintingBackend::getCharactersPerline ( DocumentFontSize fontSize )
{
    int size ( 64 );

    switch ( fontSize )
    {
        default:
        case TinyFont:
        case SmallFont:
            size = 64;
            break;

        case MediumFont:
            size = 56;
            break;

        case LargeFont:
        case Biggest:
            size = 40;
            break;
    }

    return size;
}


void DirectPrintingBackend::AddNewFontCommand ( unsigned char code )
{
    char cmdbuff[32];

    if ( ! binaryOutput )
    {
        sprintf ( cmdbuff, "<ESC>%%<%d>", code );
    }
    else
    {
        sprintf ( cmdbuff, "%c%%<%d>", ESC, code );
    }

    std::string cmdstr ( cmdbuff );

    SafeBuffer buffer ( cmdstr );
    File::getInstance()->addBufferToFile ( buffer, currentFilename );
}
void DirectPrintingBackend::AddNewCharSetCommand ( unsigned char code )
{
    if ( code > 12 )
    {
        code = 12;
    }

    char cmdbuff[32] ;

    if ( ! binaryOutput )
    {
        sprintf ( cmdbuff, "<ESC>R<%d>", code );
    }
    else
    {
        sprintf ( cmdbuff, "%cR<%d>", ESC, code );
    }

    std::string cmdstr ( cmdbuff );
    SafeBuffer buffer ( cmdstr );
    File::getInstance()->addBufferToFile ( buffer, currentFilename );
}

void DirectPrintingBackend::AddNewJustificationCommand ( unsigned char code )
{
    char cmdbuff[32];

    if ( ! binaryOutput )
    {
        sprintf ( cmdbuff, "<ESC>C<%d>", code );
    }
    else
    {
        sprintf ( cmdbuff, "%cC<%d>", ESC, code );
    }

    std::string cmdstr ( cmdbuff );
    SafeBuffer buffer ( cmdstr );
    File::getInstance()->addBufferToFile ( buffer, currentFilename );
}

void DirectPrintingBackend::AddNewTextModeCommand ( unsigned char underlined )
{
    char cmdbuff[32];

    if ( underlined )
    {
        underlined = 0x80;
    }

    if ( ! binaryOutput )
    {
        sprintf ( cmdbuff, "<ESC>!<%d>", underlined );
    }
    else
    {
        sprintf ( cmdbuff, "%c!<%d>", ESC, underlined );
    }

    std::string cmdstr ( cmdbuff );
    SafeBuffer buffer ( cmdstr );
    File::getInstance()->addBufferToFile ( buffer, currentFilename );
}

void DirectPrintingBackend::AddNewFullCutCommand()
{
    std::string cmdstr;

    if ( ! binaryOutput )
    {
        cmdstr = "<ESC>i";
    }
    else
    {
        char cmdbuff [32];
        sprintf ( cmdbuff, "%ci", ESC );
        cmdstr.assign ( cmdbuff );
    }

    SafeBuffer buffer ( cmdstr );
    File::getInstance()->addBufferToFile ( buffer, currentFilename );
}

void DirectPrintingBackend::AddNewPartialCutCommand()
{
    std::string cmdstr = "<ESC>m";

    if ( ! binaryOutput )
    {
        cmdstr = "<ESC>m";
    }
    else
    {
        char cmdbuff [32];
        sprintf ( cmdbuff, "%cm", ESC );
        cmdstr.assign ( cmdbuff );
    }

    SafeBuffer buffer ( cmdstr );
    File::getInstance()->addBufferToFile ( buffer, currentFilename );
}

void DirectPrintingBackend::AddLineFeedCommand()
{
    std::string cmdstr;// ("<LF>");

    if ( ! binaryOutput )
    {
        cmdstr = "<LF>";
    }
    else
    {
        char cmdbuff [32];
        sprintf ( cmdbuff, "%c", LF );
        cmdstr.assign ( cmdbuff );
    }

    SafeBuffer buffer ( cmdstr );
    File::getInstance()->addBufferToFile ( buffer, currentFilename );
}
/**
 * @brief Feed paper (n dot lines) forward Paper is fed for val (val<256) dot lines (val times 0.125 mm).
 * The print position is at the beginning of the next line.
 */
void DirectPrintingBackend::AddFeedPaperCommand ( unsigned char val )
{
    char cmdbuff[32];

    if ( ! binaryOutput )
    {
        sprintf ( cmdbuff, "<ESC>J<%d>", val );
    }
    else
    {
        sprintf ( cmdbuff, "%cJ<%d>", ESC, val );
    }

    std::string cmdstr ( cmdbuff );
    SafeBuffer buffer ( cmdstr );
    File::getInstance()->addBufferToFile ( buffer, currentFilename );
}

void DirectPrintingBackend::setDeleteFiles ( bool arg )
{
    deleteFiles = arg;
}


PrintDocumentResult::PrintingResultEnumType DirectPrintingBackend::SpoolerResToPrintingRes (
    LPSpooler::JobStateEnumType spoolerRes )
{
    switch ( spoolerRes )
    {
        case LPSpooler::LPSPOOLER_DEVICE_ERROR:
            return PrintDocumentResult::PRINTING_DEVICE_ERROR;

        case LPSpooler::LPSPOOLER_SERVER_ERROR:
            return PrintDocumentResult::PRINTING_SERVER_ERROR;

        case LPSpooler::LPSPOOLER_NO_PRINTER_FOUND:
            return PrintDocumentResult::PRINTING_NO_PRINTER_FOUND;

        case LPSpooler::LPSPOOLER_NO_JOBS:
            return PrintDocumentResult::PRINTING_NO_JOBS;

        case LPSpooler::LPSPOOLER_JOB_PENDING:
            return PrintDocumentResult::PRINTING_PENDING;

        case LPSpooler::LPSPOOLER_JOB_HELD:
            return PrintDocumentResult::PRINTING_HELD;

        case LPSpooler::LPSPOOLER_JOB_PROCESSING:
            return PrintDocumentResult::PRINTING_PROCESSING;

        case LPSpooler::LPSPOOLER_JOB_STOPPED:
            return PrintDocumentResult::PRINTING_STOPPED;

        case LPSpooler::LPSPOOLER_JOB_CANCELED:
            return PrintDocumentResult::PRINTING_CANCELED;

        case LPSpooler::LPSPOOLER_JOB_ABORTED:
            return PrintDocumentResult::PRINTING_ABORTED;

        case LPSpooler::LPSPOOLER_JOB_COMPLETED:
            return PrintDocumentResult::PRINTING_COMPLETED;

        default:
            return PrintDocumentResult::PRINTING_NO_JOBS;
    }
}

PrintDocumentResult DirectPrintingBackend::getPrintResult ()
{
    std::stringstream ss;
    ss << "Status : " << ( mOverallResult.getSuccess() ? "success" : "error" ) << " - ";
    ss << "PrinterStatusDetail : " << mOverallResult.getPrinterStatusDetail() << " - ";
    ss << "PrintingResult : " << ( int ) mOverallResult.getPrintingResult();
    logger->Debug ( ss.str() );
    return mOverallResult;
}
