/**
 * @file	GraphicContentPage.h
 * @brief	Declares the graphic content page class.
 * @author	Youssef.tourkmani
 * @date	30/10/2012
 */

#ifndef GRAPHICCONTENTPAGE_H_
#define GRAPHICCONTENTPAGE_H_


#include <list>
#include <gtkmm.h>
#include <gdkmm.h>

namespace Smartmatic
{

    namespace SAES
    {

        namespace Printing
        {

            /**
             * @class	GraphicContentPage
             * @brief	Graphic content page.
             * @author	Youssef.tourkmani
             * @date	30/10/2012
             */
            class GraphicContentPage
            {
                public:

                    /**
                     * @typedef	enum PageAlignmentEnum
                     * @brief	Defines an alias representing the page alignment enum.
                     */
                    typedef enum PageAlignmentEnum
                    {
                        GCPAGE_LEFT,
                        GCPAGE_CENTER,
                        GCPAGE_RIGHT,
                        GCPAGE_EXPANDED

                    } PageAlignmentEnumType;	///< Type of the page alignment enum

                    /**
                     * @typedef	enum ColumnModeEnum
                     * @brief	Defines an alias representing the column mode enum.
                     */
                    typedef enum ColumnModeEnum
                    {
                        GCPAGE_SINGLE_COLUMN,
                        GCPAGE_DOUBLE_COLUMN

                    } ColumnModeEnumType; ///< Type of the column mode enum

                    /**
                     * @fn	GraphicContentPage::GraphicContentPage();
                     * @brief	Life Cycle.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     */

                    GraphicContentPage();

                    /**
                     * @fn	GraphicContentPage::GraphicContentPage(const GraphicContentPage& other);
                     * @brief	Constructor.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param	other	The other.
                     */
                    GraphicContentPage ( const GraphicContentPage & other );


                    /**
                     * @fn	GraphicContentPage::GraphicContentPage(double pageWidth, double pageHeight);
                     * @brief	Constructor.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param	pageWidth 	Width of the page.
                     * @param	pageHeight	Height of the page.
                     */
                    GraphicContentPage ( double pageWidth, double pageHeight );


                    /**
                     * @fn	virtual GraphicContentPage::~GraphicContentPage();
                     * @brief	Destructor.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     */
                    virtual ~GraphicContentPage();


                    /**
                     * @fn	void GraphicContentPage::GetCurrentPosition (int& x, int& y);
                     * @brief	Interface.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param [in,out]	x	The x coordinate.
                     * @param [in,out]	y	The y coordinate.
                     */

                    void GetCurrentPosition ( int & x, int & y );

                    /**
                     * @fn	double GraphicContentPage::GetCurrentTextAreaWidth ();
                     * @brief	Gets current text area width.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @return	The current text area width.
                     */
                    double GetCurrentTextAreaWidth ();

                    /**
                     * @fn	double GraphicContentPage::GetWidth ()
                     * @brief	Gets the width.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @return	The width.
                     */
                    double GetWidth ()
                    {
                        return m_Width;
                    }

                    /**
                     * @fn	bool GraphicContentPage::IsBeyondEndOfPage (double elementHeight);
                     * @brief	Query if 'elementHeight' is beyond end of page.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param	elementHeight	Height of the element.
                     * @return	true if beyond end of page, false if not.
                     */
                    bool IsBeyondEndOfPage ( double elementHeight );

                    /**
                     * @fn	void GraphicContentPage::AddTextRow ( const std::vector<Glib::RefPtr<Pango::Layout> >& row_content ,
                     * 			const std::vector<int>& cell_indent , int column_width , int row_height);
                     * @brief	Adds a text row.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param	row_content 	The row content.
                     * @param	cell_indent 	The cell indent.
                     * @param	column_width	Width of the column.
                     * @param	row_height		Height of the row.
                     */
                    void AddTextRow (
                        const std::vector<Glib::RefPtr<Pango::Layout> > & row_content
                        , const std::vector<int> & cell_indent
                        , int column_width
                        , int row_height );

                    /**
                     * @fn	void GraphicContentPage::AddTextContent ( Glib::RefPtr<Pango::Layout> textContent ,
                     * 			int initX , int initY);
                     * @brief	Adds a text content.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param	textContent	The text content.
                     * @param	initX			 	The initialise x coordinate.
                     * @param	initY			 	The initialise y coordinate.
                     */
                    void AddTextContent (
                        Glib::RefPtr<Pango::Layout> textContent
                        , int initX
                        , int initY );

                    /**
                     * @fn	void GraphicContentPage::AddImageContent ( Glib::RefPtr<Gdk::Pixbuf> image , int init_x ,
                     * 			int init_y , double element_width , double element_height , PageAlignmentEnumType alignment ,
                     * 			bool floating);
                     * @brief	Adds an image content.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param	image						The image.
                     * @param	init_x					The initialise x coordinate.
                     * @param	init_y					The initialise y coordinate.
                     * @param	element_width 	Width of the element.
                     * @param	element_height	Height of the element.
                     * @param	alignment				The alignment.
                     * @param	floating				true to floating.
                     */
                    void AddImageContent (
                        Glib::RefPtr<Gdk::Pixbuf> image
                        , int init_x
                        , int init_y
                        , double element_width
                        , double element_height
                        , PageAlignmentEnumType alignment
                        , bool floating );

                    /**
                     * @fn	void GraphicContentPage::AddSepContent (int initX, int initY, int endX, int endY,
                     * 			double weight);
                     * @brief	Adds a separator content.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param	initX 	The initialise x coordinate.
                     * @param	initY 	The initialise y coordinate.
                     * @param	endX		The end x coordinate.
                     * @param	endY		The end y coordinate.
                     * @param	weight	The weight.
                     */
                    void AddSepContent ( int initX, int initY, int endX, int endY, double weight );

                    /**
                     * @fn	void GraphicContentPage::JumpInlineImage ();
                     * @brief	Jump inline image.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     */
                    void JumpInlineImage ();

                    /**
                     * @fn	void GraphicContentPage::Render (Cairo::RefPtr<Cairo::Context> drawableContext);
                     * @brief	Renders the given drawableContext.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param	drawableContext	Context for the drawable.
                     */
                    void Render ( Cairo::RefPtr<Cairo::Context> drawableContext );

                    /**
                     * @fn	void GraphicContentPage::SetSingleLayout ();
                     * @brief	Sets single layout.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     */
                    void SetSingleLayout ();

                    /**
                     * @fn	void GraphicContentPage::SetTwoColumnsLayout ();
                     * @brief	Sets two columns layout.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     */
                    void SetTwoColumnsLayout ();

                    /**
                     * @fn  void GraphicContentPage::operator= ();
                     * @brief   Assignment operator.
                     * @author  eduardo.fernandez
                     * @date    21/01/2019
                     * @param   obj Object
                     * @return  return object
                     */
                    GraphicContentPage & operator = ( const GraphicContentPage & obj );

                private:


                    /**
                     * @class	GraphicContextTextCell
                     * @brief	Graphic context cell.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     */
                    class GraphicContextTextCell
                    {
                        public:

                            /**
                             * @fn	GraphicContextTextCell::GraphicContextTextCell ( Glib::RefPtr<Pango::Layout> textContent ,
                             * 			int initX , int initY)
                             * @brief	Constructor.
                             * @author	Youssef.tourkmani
                             * @date	30/10/2012
                             * @param	textContent	The text content.
                             * @param	initX			 	The initialise x coordinate.
                             * @param	initY			 	The initialise y coordinate.
                             */
                            GraphicContextTextCell (
                                Glib::RefPtr<Pango::Layout> textContent
                                , int initX
                                , int initY )
                                : CellLayout ( textContent )
                                , init_x ( initX )
                                , init_y ( initY ) { }

                            /**
                             * @fn	GraphicContextTextCell::GraphicContextTextCell (const GraphicContextTextCell& other)
                             * @brief	Constructor.
                             * @author	Youssef.tourkmani
                             * @date	30/10/2012
                             * @param	other	The other.
                             */
                            GraphicContextTextCell ( const GraphicContextTextCell & other )
                            {
                                CellLayout = other.CellLayout;
                                init_x = other.init_x;
                                init_y = other.init_y;
                            }

                            /**
                             * @fn	virtual GraphicContextTextCell::~GraphicContextTextCell ()
                             * @brief	Destructor.
                             * @author	Youssef.tourkmani
                             * @date	30/10/2012
                             */
                            virtual ~GraphicContextTextCell () {};

                            /**
                             * @fn  void GraphicContextTextCell::operator= ();
                             * @brief   Assignment operator.
                             * @author  eduardo.fernandez
                             * @date    21/01/2019
                             * @param   obj Object
                             * @return  return object
                             */
                            GraphicContextTextCell & operator = ( const GraphicContextTextCell & obj )
                            {
                                init_x = obj.init_x;
                                init_y = obj.init_y;
                                return *this;
                            }

                            Glib::RefPtr<Pango::Layout> CellLayout; ///< The cell layout

                            int init_x; ///< The initialise x coordinate
                            int init_y; ///< The initialise y coordinate
                    };

                    /**
                     * @class	GraphicContextImage
                     * @brief	Graphic context image.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     */
                    class GraphicContextImage
                    {
                        public:

                            /**
                             * @fn	GraphicContextImage::GraphicContextImage ( Glib::RefPtr<Gdk::Pixbuf> image , int initX ,
                             * 			int initY , int width , int height , PageAlignmentEnumType align)
                             * @brief	Constructor.
                             * @author	Youssef.tourkmani
                             * @date	30/10/2012
                             * @param	image 	The image.
                             * @param	initX 	The initialise x coordinate.
                             * @param	initY 	The initialise y coordinate.
                             * @param	width 	The width.
                             * @param	height	The height.
                             * @param	align 	The align.
                             */
                            GraphicContextImage (
                                Glib::RefPtr<Gdk::Pixbuf> image
                                , int initX
                                , int initY
                                , int width
                                , int height
                                , PageAlignmentEnumType align )
                                : Image ( image )
                                , init_x ( initX )
                                , init_y ( initY )
                                , element_width ( width )
                                , element_height ( height )
                                , alignment ( align )
                            { }

                            /**
                             * @fn	virtual GraphicContextImage::~GraphicContextImage ()
                             * @brief	Destructor.
                             * @author	Youssef.tourkmani
                             * @date	30/10/2012
                             */
                            virtual ~GraphicContextImage () {};

                            Glib::RefPtr<Gdk::Pixbuf> Image;	///< The image

                            int init_x; ///< The initialise x coordinate
                            int init_y; ///< The initialise y coordinate
                            int element_width;	///< Width of the element
                            int element_height; ///< Height of the element

                            PageAlignmentEnumType alignment;	///< The alignment
                    };

                    /**
                     * @class	GraphicContextSep
                     * @brief	Graphic context separator.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     */
                    class GraphicContextSep
                    {
                        public:

                            /**
                             * @fn	GraphicContextSep::GraphicContextSep ( int initX, int initY, int endX, int endY,
                             * 			double weight)
                             * @brief	Constructor.
                             * @author	Youssef.tourkmani
                             * @date	30/10/2012
                             * @param	initX 	The initialise x coordinate.
                             * @param	initY 	The initialise y coordinate.
                             * @param	endX		The end x coordinate.
                             * @param	endY		The end y coordinate.
                             * @param	weight	The weight.
                             */
                            GraphicContextSep (
                                int initX, int initY, int endX, int endY, double weight )
                                : init_x ( initX )
                                , init_y ( initY )
                                , end_x ( endX )
                                , end_y ( endY )
                                , m_weight ( weight )
                            { }

                            /**
                             * @fn	virtual GraphicContextSep::~GraphicContextSep ()
                             * @brief	Destructor.
                             * @author	Youssef.tourkmani
                             * @date	30/10/2012
                             */
                            virtual ~GraphicContextSep () {};

                            int init_x; ///< The initialise x coordinate
                            int init_y; ///< The initialise y coordinate
                            int end_x;	///< The end x coordinate
                            int end_y;	///< The end y coordinate
                            double m_weight;	///< The weight
                    };


                    static int PageCount; ///< Number of pages
                    int m_PageNum;	///< The page number

                    double m_DoubleColumnStartPos;	///< The double column start position
                    int m_CurrentColumn;	///< The current column
                    ColumnModeEnumType m_CurrentColumnMode; ///< The current column mode

                    double m_Width; ///< The width
                    double m_Height;	///< The height

                    double m_Cursor_X;	///< The cursor x coordinate
                    double m_Cursor_Y;	///< The cursor y coordinate

                    double m_TextLeftMargin;	///< The text left margin
                    double m_TextRightMargin; ///< The text right margin
                    double m_CurrentImageBottom;	///< The current image bottom

                    std::list<GraphicContextTextCell> m_TextElements; ///< The text elements
                    std::list<GraphicContextImage> m_ImageElements; ///< The image elements
                    std::list<GraphicContextSep> m_SepElements; ///< The separator elements

            };

        }
    }
}

#endif /* GRAPHICCONTENTPAGE_H_ */
