/**
 * @file   IB31K1PrintingBackend.cxx
 * @brief  Implementation of IB31K1PrintingBackend class.
 * @author manuel.arguelles
 * @date   Sep  8, 2017
 */

#include "IB31K1PrintingBackend.hxx"
#include "Runtime/Environment/saes-directories.h"
#include "Configuration/s-a-e-s-common-conf-manager.h"
#include "LPSpooler.h"
#include <System/IO/File.h>
#include <System/IO/Directory.h>
#include <System/IO/Path.h>
#include <Graphics/Encoding2D/QREncoding.h>
#include <map>

using namespace Smartmatic::Graphics::Encoding2D;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Printing;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System;

const std::string  LP_OPTION_HISTORIC_HEAT    = "HistoricHeat";
const std::string  LP_VALUE_HISTORIC_HEAT_OFF = "False";
const std::string  LP_VALUE_HISTORIC_HEAT_ON  = "True";

/**
 * @brief  Return the mapping characters between utf-8 and ib31k1 printer.
 * @author manuel.arguelles
 * @date   Nov 17, 2017
 * @return A map with the characters.
 */
static std::map<int, unsigned char> utfToIB31K1Map()
{
    std::map<int, unsigned char> map;

    map[0xC2A0] = 0x20;
    map[0xC380] = 0xB7;
    map[0xC381] = 0xB5;
    map[0xC382] = 0xB6;
    map[0xC383] = 0xC7;
    map[0xC384] = 0x8E;
    map[0xC385] = 0x8F;
    map[0xC386] = 0x92;
    map[0xC387] = 0x87;
    map[0xC388] = 0xD4;
    map[0xC389] = 0x90;
    map[0xC38A] = 0x90;
    map[0xC38B] = 0xD3;
    map[0xC38C] = 0x8D;
    map[0xC38D] = 0xD6;
    map[0xC38E] = 0x8B;
    map[0xC38F] = 0xD8;
    map[0xC390] = 0xD1;
    map[0xC391] = 0xA5;
    map[0xC392] = 0xE3;
    map[0xC393] = 0xE0;
    map[0xC394] = 0xE2;
    map[0xC395] = 0xE5;
    map[0xC396] = 0x99;
    map[0xC397] = 0x2A;
    map[0xC398] = 0x9D;
    map[0xC399] = 0xEB;
    map[0xC39A] = 0xE9;
    map[0xC39B] = 0xEA;
    map[0xC39C] = 0x9A;
    map[0xC39D] = 0xED;
    map[0xC39E] = 0xE8;
    map[0xC39F] = 0xE1;
    map[0xC3A0] = 0x85;
    map[0xC3A1] = 0xA0;
    map[0xC3A2] = 0x83;
    map[0xC3A3] = 0xC6;
    map[0xC3A4] = 0x84;
    map[0xC3A5] = 0x86;
    map[0xC3A6] = 0x91;
    map[0xC3A7] = 0x87;
    map[0xC3A8] = 0x8A;
    map[0xC3A9] = 0x82;
    map[0xC3AA] = 0x88;
    map[0xC3AB] = 0x89;
    map[0xC3AC] = 0x8D;
    map[0xC3AD] = 0xA1;
    map[0xC3AE] = 0xA1;
    map[0xC3AF] = 0x8B;
    map[0xC3B0] = 0xE4;
    map[0xC3B1] = 0xA4;
    map[0xC3B2] = 0x95;
    map[0xC3B3] = 0xA2;
    map[0xC3B4] = 0x93;
    map[0xC3B5] = 0xE4;
    map[0xC3B6] = 0x94;
    map[0xC3B7] = 0xF6;
    map[0xC3B8] = 0x9B;
    map[0xC3B9] = 0x97;
    map[0xC3BA] = 0xA3;
    map[0xC3BB] = 0x96;
    map[0xC3BC] = 0x81;
    map[0xC3BD] = 0xEC;
    map[0xC3BE] = 0xE7;
    map[0xC3BF] = 0x98;

    return map;
}

static std::map<int, unsigned char> UTF_TO_IB31K1 = utfToIB31K1Map();

/**
 * @brief  Gets the maximum amount of characters than can be printed per line.
 * @author manuel.arguelles
 * @date   Sep 11, 2017
 * @param  fontSize The font size in use.
 * @return The maximum amount of characters per line.
 */
static int getCharactersPerLine(const DocumentFontSize fontSize)
{
    int size;

    switch (fontSize)
    {
        default:
        case TinyFont:
        case SmallFont:
            size = 57;
            break;

        case MediumFont:
            size = 57;
            break;

        case LargeFont:
        case Biggest:
            size = 40;
            break;
    }

    return size;
}

/**
 * @brief  Converts UTF-8 strings to IB31K1 format.
 * @author manuel.arguelles
 * @date   Nov 17, 2017
 * @param  input  The input UTF-8 string.
 * @param  output The converted string.
 */
static void utfToIB31K1(const Glib::ustring& input, std::string& output)
{
    unsigned int size = input.bytes();
    unsigned char* data = new unsigned char[size];

    input.copy ( ( char * ) data, size, 0 );

    for ( unsigned int i = 0; i < size; ++i )
    {
        if ( data[i] == 0xC3 || data[i] == 0xC2 )
        {
            i++; //goto next char

            if ( i < size )
            {
                int code = ( data[i - 1] << 8 ) |  data[i];
                output.push_back ( UTF_TO_IB31K1[code] );
            }
        }
        else
        {
            output.push_back ( data[i] );
        }
    }

    delete [] data;
}

/**
 * @brief  Convert spooler state to printing result enumerate.
 * @author manuel.arguelles
 * @date   Sep 11, 2017
 * @param  spoolerState The spooler state.
 * @return The corresponding printing result enumerate of the given state.
 */
static PrintDocumentResult::PrintingResultEnumType spoolerStateToPrintingResult(
    LPSpooler::JobStateEnumType spoolerState)
{
    switch (spoolerState)
    {
        case LPSpooler::LPSPOOLER_DEVICE_ERROR:
            return PrintDocumentResult::PRINTING_DEVICE_ERROR;

        case LPSpooler::LPSPOOLER_SERVER_ERROR:
            return PrintDocumentResult::PRINTING_SERVER_ERROR;

        case LPSpooler::LPSPOOLER_NO_PRINTER_FOUND:
            return PrintDocumentResult::PRINTING_NO_PRINTER_FOUND;

        case LPSpooler::LPSPOOLER_NO_JOBS:
            return PrintDocumentResult::PRINTING_NO_JOBS;

        case LPSpooler::LPSPOOLER_JOB_PENDING:
            return PrintDocumentResult::PRINTING_PENDING;

        case LPSpooler::LPSPOOLER_JOB_HELD:
            return PrintDocumentResult::PRINTING_HELD;

        case LPSpooler::LPSPOOLER_JOB_PROCESSING:
            return PrintDocumentResult::PRINTING_PROCESSING;

        case LPSpooler::LPSPOOLER_JOB_STOPPED:
            return PrintDocumentResult::PRINTING_STOPPED;

        case LPSpooler::LPSPOOLER_JOB_CANCELED:
            return PrintDocumentResult::PRINTING_CANCELED;

        case LPSpooler::LPSPOOLER_JOB_ABORTED:
            return PrintDocumentResult::PRINTING_ABORTED;

        case LPSpooler::LPSPOOLER_JOB_COMPLETED:
            return PrintDocumentResult::PRINTING_COMPLETED;

        default:
            return PrintDocumentResult::PRINTING_NO_JOBS;
    }
}

/**
 * @brief  Calculate the possible left margin based on the size of the qr code
 * used for diagnostics report.
 * @author manuel.arguelles
 * @date   Dec  5, 2017
 * @param  size The size of the data (76 to 424).
 * @return The left margin from 130 to 180.
 */
static int getQrMarginFromDataSize(int size)
{
    int margin;

    if (size <= 76)
    {
        margin = 180;
    }
    else if (size >= 424)
    {
        margin = 130;
    }
    else
    {
        margin = (size / -6.96) + 191;
    }

    return margin;
}

IB31K1PrintingBackend::IB31K1PrintingBackend()
    : m_overallResult(false, "", PrintDocumentResult::PRINTING_NONE)
{
}

IB31K1PrintingBackend::~IB31K1PrintingBackend()
{
}

void IB31K1PrintingBackend::Init()
{
    SaesDirectories directories;

    m_currentFile = directories.getTemporaryFileFullPath() + ".txt";
}

void IB31K1PrintingBackend::addFinalCommands()
{
    SAESCommonConfigurationManager* manager = SAESCommonConfigurationManager::getInstance();
    SaesCommonConfiguration* configuration = manager->getConfiguration();
    CutType cutType = configuration->getConfigurationPrinter().getCutType();
    int endPadding = configuration->getConfigurationPrinter().getEndPaddingDirectPrint();

    for (int i = 0; i < endPadding; ++i)
    {
        addFeedPaperCommand(40);
    }

    if (cutType != CutType::CONFIGURATION_NOCUT)
    {
        addNewFullCutCommand();
    }
}

void IB31K1PrintingBackend::configureNeopSensor()
{
    SAESCommonConfigurationManager* manager = SAESCommonConfigurationManager::getInstance();
    SaesCommonConfiguration* configuration = manager->getConfiguration();

    if (configuration->getConfigurationPrinter().getDirectNEOP().present())
    {
        SafeBuffer comment("#CUPS-COMMAND\n");
        SafeBuffer command("SetNEOP ");
        std::string neopValue = configuration->getConfigurationPrinter().getDirectNEOP().get();

        neopValue += std::string("\n");

        command.AddData(neopValue);

        changeFile();
        File::getInstance()->addBufferToFile(comment, m_currentFile);
        File::getInstance()->addBufferToFile(command, m_currentFile);
        changeFile();
    }
}

void IB31K1PrintingBackend::updateStatus()
{
    SafeBuffer command("#CUPS-COMMAND\nReportStatus\n");

    changeFile();
    File::getInstance()->addBufferToFile(command, m_currentFile);
    changeFile();
}

void IB31K1PrintingBackend::End()
{
    if (File::getInstance()->canOpenFile(m_currentFile))
    {
        addFinalCommands();
    }

    changeFile();

    while(!m_files.empty())
    {
        LPSpooler                   spooler;
        LPSpooler::JobStateEnumType status;
        std::string                 file;
        bool                        deleteFiles;
        SaesCommonConfiguration*    configuration;

        configuration = SAESCommonConfigurationManager::getInstance()->getConfiguration();
        deleteFiles = configuration->getConfigurationPrinter().getDirectPrintDeleteFiles();
        file = m_files.front();

        if (configuration->getConfigurationPrinter().getHistoricHeat().present() &&
            configuration->getConfigurationPrinter().getHistoricHeat().get() == true)
        {
            spooler.AddOption(LP_OPTION_HISTORIC_HEAT, LP_VALUE_HISTORIC_HEAT_ON);
        }
        else
        {
            spooler.AddOption(LP_OPTION_HISTORIC_HEAT, LP_VALUE_HISTORIC_HEAT_OFF);
        }

        spooler.SetFileName(file);

        status = spooler.Print();

        if (status != LPSpooler::LPSPOOLER_JOB_COMPLETED)
        {
            m_overallResult.setResult(
                false,
                spooler.GetSpoolerErrorMessage(status),
                spoolerStateToPrintingResult(status));
            break;
        }
        else
        {
            m_overallResult.setResult(
                true,
                spooler.GetSpoolerErrorMessage(status),
                spoolerStateToPrintingResult(status));
        }

        if (deleteFiles)
        {
            File::getInstance()->DeleteFile(file);
        }

        m_files.pop();
    }
}

void IB31K1PrintingBackend::DrawString(std::list<Glib::ustring>& textContent,
                                       std::list<DocumentTextOptions>& textContentOptions)
{
    std::vector<DocumentTextOptions> options(textContentOptions.begin(), textContentOptions.end());
    std::vector<Glib::ustring>       text(textContent.begin(), textContent.end());

    if (options.size() > 1)
    {
        int columns  = options.size();
        int rows     = text.size() / columns;
        int reminder = text.size() % columns;

        for (int r = 0; r < rows; ++r)
        {
            for (int c = 0; c < columns; ++c)
            {
                std::string decoded;

                utfToIB31K1(text[(r * columns) + c], decoded);
                drawCell(options[c], decoded, columns);
            }

            addLineFeedCommand();
        }

        for (int i = 0; i < reminder; ++i)
        {
            std::string decoded;

            utfToIB31K1(text[(rows * columns) + i], decoded);
            drawCell(options[i], decoded, columns);
        }

        if (reminder > 0)
        {
            addLineFeedCommand();
        }
    }
    else
    {
        std::vector<Glib::ustring>::const_iterator it;

        updateTextOptions(options.front(), true);

        for (it = text.begin(); it != text.end(); ++it)
        {
            std::string decoded;

            utfToIB31K1(*it, decoded);

            SafeBuffer buffer(decoded);

            File::getInstance()->addBufferToFile(buffer, m_currentFile);
        }

        addLineFeedCommand();
    }
}

void IB31K1PrintingBackend::DrawImage(Glib::RefPtr<Gdk::Pixbuf> image)
{
    SaesDirectories directories;
    std::string     imageFile;

    changeFile();
    imageFile = directories.getTemporaryFileFullPath() + ".png";
    image->save(imageFile, "png");
    m_files.push(imageFile);
}

void IB31K1PrintingBackend::DrawQRCode(QRCodeContent& qrCode)
{
    DrawQRCodeBin(qrCode);
}

void IB31K1PrintingBackend::DrawQRCodeBin(QRCodeContent& qrCode)
{
    SafeBuffer buffer;
    int correctionLevel = 0;
    std::string base64Data = qrCode.getData().GetBase64Data();
    std::size_t base64Size = base64Data.size();
    unsigned char base64SizeN7;
    unsigned char base64SizeN8;
    unsigned char margin = qrCode.getMargin();

    if (qrCode.getPixelSize() >= 6)
    {
        addFeedPaperCommand(0x44);
        changeLeftMargin(60);
        margin = 0;
    }
    else
    {
        changeLeftMargin(getQrMarginFromDataSize(base64Size));
    }

    switch ( qrCode.getCorrectionLevel() )
    {
        case QR_ECLEVEL_L:
            correctionLevel = 0;
            break;

        case QR_ECLEVEL_M:
            correctionLevel = 1;
            break;

        case QR_ECLEVEL_Q:
            correctionLevel = 2;
            break;

        default:
        case QR_ECLEVEL_H:
            correctionLevel = 3;
            break;
    }

    buffer.push_back(0x1b);
    buffer.push_back(0x27);
    buffer.push_back(0x0a);
    buffer.push_back(0x01);

    buffer.push_back(0x1b);
    buffer.push_back(0x2e);
    buffer.push_back(qrCode.getPixelSize());
    buffer.push_back(qrCode.getVersion());
    buffer.push_back(correctionLevel);
    buffer.push_back(0x03);
    buffer.push_back(0x00);
    buffer.push_back(margin);

    base64SizeN7 = (base64Size & 0xff00) >> 8;
    base64SizeN8 = (base64Size & 0xff);

    buffer.push_back(base64SizeN7);
    buffer.push_back(base64SizeN8);
    buffer.AddData(base64Data);

    File::getInstance()->addBufferToFile(buffer, m_currentFile);

    addFeedPaperCommand(0x0C);
}

PrintDocumentResult IB31K1PrintingBackend::getPrintResult()
{
    return m_overallResult;
}

void IB31K1PrintingBackend::cleanPrintQueue()
{
    while (!m_files.empty())
    {
        m_files.pop();
    }
}

void IB31K1PrintingBackend::addLineFeedCommand()
{
    SafeBuffer command;

    command.push_back(0x0A);

    File::getInstance()->addBufferToFile(command, m_currentFile);
}

void IB31K1PrintingBackend::addNewFullCutCommand()
{
    SafeBuffer command;

    command.push_back(0x1b);
    command.push_back(0x69);

    File::getInstance()->addBufferToFile(command, m_currentFile);
}

void IB31K1PrintingBackend::addFeedPaperCommand(unsigned char dots)
{
    SafeBuffer command;

    command.push_back(0x1b);
    command.push_back(0x4a);
    command.push_back(dots);

    File::getInstance()->addBufferToFile(command, m_currentFile);
}

void IB31K1PrintingBackend::changeFont(const DocumentFontSize fontSize, const DocumentFontStyle fontStyle)
{
    SafeBuffer    command;
    unsigned char sizeCode;

    switch (fontSize)
    {
        default:
        case TinyFont:
        case SmallFont:
            sizeCode = 0x02;
            break;

        case MediumFont:
            sizeCode = 0x00;
            break;

        case LargeFont:
        case Biggest:
            sizeCode = 0x01;
            break;
    }

    changeCharactersPerLine(getCharactersPerLine(fontSize));

    command.push_back(0x1b);
    command.push_back(0x25);
    command.push_back(sizeCode);

    if ((fontStyle & Underlined) != Normal)
    {
        command.push_back(0x1b);
        command.push_back(0x21);
        command.push_back(0x80);
    }

    File::getInstance()->addBufferToFile(command, m_currentFile);
}

void IB31K1PrintingBackend::changeLeftMargin(unsigned char dots)
{
    SafeBuffer command;

    command.push_back(0x1d);
    command.push_back(0x4c);
    command.push_back(dots);
    command.push_back(0x00);

    File::getInstance()->addBufferToFile(command, m_currentFile);
}

void IB31K1PrintingBackend::changeCharactersPerLine(unsigned char characters)
{
    SafeBuffer command;

    command.push_back(0x1b);
    command.push_back(0x63);
    command.push_back(characters);

    File::getInstance()->addBufferToFile(command, m_currentFile);
}

void IB31K1PrintingBackend::changeJustification(const DocumentJustifitacion justification)
{
    SafeBuffer    command;
    unsigned char justificationCode;

    switch (justification)
    {
        default:
        case LeftJustified:
            changeLeftMargin(0x10);
            justificationCode = 0x02;
            break;
        case Centered:
            changeLeftMargin(0x00);
            justificationCode = 0x00;
            break;
        case RightJustified:
            changeLeftMargin(0x00);
            justificationCode = 0x01;
            break;
    }

    command.push_back(0x1B);
    command.push_back(0x43);
    command.push_back(justificationCode);

    File::getInstance()->addBufferToFile(command, m_currentFile);
}

void IB31K1PrintingBackend::updateTextOptions(DocumentTextOptions& options, const bool includeJustification)
{
    if (m_textOptions.getFontSize() != options.getFontSize() ||
        m_textOptions.getFontStyle() != options.getFontStyle())
    {
        changeFont(options.getFontSize(), options.getFontStyle());
        m_textOptions.setFontSize(options.getFontSize());
        m_textOptions.setFontStyle(options.getFontStyle());
    }

    if (m_textOptions.getJustification() != options.getJustification() && includeJustification)
    {
        changeJustification(options.getJustification());
        m_textOptions.setJustification(options.getJustification());
    }
}

void IB31K1PrintingBackend::drawCell(DocumentTextOptions& options, const std::string& text, const int columns)
{
    SaesCommonConfiguration* configuration;
    int                      counter = 0;
    int                      indentationLevel;
    int                      maxCharacters;
    int                      padding;
    std::string              finalText;
    std::string              indentationCharacter;
    std::string              paddingCharacter;

    configuration        = SAESCommonConfigurationManager::getInstance()->getConfiguration();
    indentationLevel     = options.getIndentationLevel();
    maxCharacters        = getCharactersPerLine(options.getFontSize()) / columns;
    indentationCharacter = configuration->getConfigurationPrinter().getCharIndentationDirectPrint();
    paddingCharacter     = configuration->getConfigurationPrinter().getCharPaddingDirectPrint();

    updateTextOptions(options, false);

    while (counter++ < indentationLevel)
    {
        finalText.append(indentationCharacter);
    }

    finalText.append(text);

    if (finalText.size() > static_cast<std::string::size_type>(maxCharacters))
    {
        finalText = finalText.substr(0, maxCharacters);
    }

    padding = maxCharacters - finalText.size();
    counter = 0;

    while (counter++ < padding)
    {
        finalText.append(paddingCharacter);
    }

    SafeBuffer buffer(finalText);
    File::getInstance()->addBufferToFile(buffer, m_currentFile);
}

void IB31K1PrintingBackend::changeFile()
{
    SaesDirectories directories;

    if (!m_currentFile.empty() && File::getInstance()->canOpenFile(m_currentFile))
    {
        m_files.push(m_currentFile);
    }

    m_currentFile = directories.getTemporaryFileFullPath() + ".txt";
}
