/**
 * @file	MemoryPathController.cpp
 * @brief	Implements the memory path controller class.
 * @author	Youssef.tourkmani
 * @date	30/10/2012
 */


#include "MemoryPathController.h"
#include <System/IO/File.h>
#include <System/IO/Path.h>
#include <System/IO/Directory.h>
#include <sys/mount.h>
#include <System/OptParser.hxx>
#include <System/Utils/StringUtils.hxx>
#include <iostream>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::System;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::Utils;

MemoryPathController * MemoryPathController::instance = NULL;

ISMTTLog * MemoryPathController::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Runtime.Environment.MemoryPathController", GETTEXT_PACKAGE );

/**
 * @brief  Removes a path from the path list.
 * @author manuel.arguelles
 * @date   Sep  7, 2017
 * @param  pathList The vector with the pathList.
 * @param  path     The entry to remove.
 */
static void removeFromPathList(std::vector<std::string>& pathList, const std::string& path)
{
    std::vector<std::string>::iterator it;

    for (it = pathList.begin(); it != pathList.end(); ++it)
    {
        if (*it == path)
        {
            pathList.erase(it);
            break;
        }
    }
}

MemoryPathController::MemoryPathController()
{
    ConfigurationElectionFilesPath & configElectionFiles ( SAESCommonConfigurationManager::getInstance()->getConfiguration()->getConfigurationElectionFilesPath() );
    ConfigurationBasePath configBasePath = configElectionFiles.getConfigBasePath();
    rule = configBasePath.getRulesBasePath();
    liveFileSystemBasePath = configBasePath.getLiveFileSystemBasePath();

    std::string issuerCertificatesBasePath;
    std::string issuerCertificatesFolderName = configElectionFiles.getIssuerCertificatesFolderName();

    if ((OptParser::getInstance()->isSet(OptParser::SHARE_DIR) == false) ||
        (OptParser::getInstance()->getArg(OptParser::SHARE_DIR, issuerCertificatesBasePath) == false))
    {
        issuerCertificatesBasePath = DATADIR;
    }

    issuerCertificatesFolderFullPath = Path::Combine(issuerCertificatesBasePath, issuerCertificatesFolderName);
}

MemoryPathController::~MemoryPathController()
{
}

std::string & MemoryPathController::getStartupMemoryBasePath()
{
    Glib::Mutex::Lock lock ( mutex );
    return domMemoryBasePath; ///< path of the startup memory
}

std::string & MemoryPathController::getLiveFileSystemBasePath()
{
    Glib::Mutex::Lock lock ( mutex );
    return liveFileSystemBasePath; ///< path of the live folder
}

std::string MemoryPathController::getIssuerCertificatesFolderFullPath()
{
    return issuerCertificatesFolderFullPath; ///< path of the issuer certificates folder
}

std::string & MemoryPathController::getDomMemoryBasePath()
{
    Glib::Mutex::Lock lock ( mutex );
    return domMemoryBasePath; ///< path of the DOM memory
}

std::string & MemoryPathController::getFlashMemoryBasePath()
{
    Glib::Mutex::Lock lock ( mutex );
    return flashMemoryBasePath;///< path of the Flash Memory
}

std::string & MemoryPathController::getBackupMemoryBasePath()
{
    Glib::Mutex::Lock lock ( mutex );
    return backupMemoryBasePath;
}

void MemoryPathController::updatePaths()
{
    domMemoryBasePath.clear();
    flashMemoryBasePath.clear();
    backupMemoryBasePath.clear();

    std::vector < std::string >::iterator it;
    int count = 0;

    for ( it = pathList.begin(); it != pathList.end(); ++it )
    {
        const std::string path = *it;
        const std::string votingCenters = Path::Combine(path, "SAES/voting-centers.xml");

        if ( Directory::getInstance()->DirectoryExists ( path ) &&
             Directory::getInstance()->FileExists( votingCenters ) )
        {
            switch ( count )
            {
                default:
                case 0:
                    domMemoryBasePath = path;
                    break;

                case 1:
                    flashMemoryBasePath = path;
                    break;

                case 2:
                    backupMemoryBasePath = path;
                    break;
            }

            count++;
        }
    }

    switch ( rule )
    {
        default:
        case RulesBasePathType::CONFIGURATION_PM_RULE:
            break;

        case RulesBasePathType::CONFIGURATION_VM_RULE:
            backupMemoryBasePath = domMemoryBasePath;
            domMemoryBasePath = liveFileSystemBasePath;
            break;
    }
}

RulesBasePathType & MemoryPathController::getRule()
{
    return rule;
}

bool MemoryPathController::isAllDevices()
{
    Glib::Mutex::Lock lock ( mutex );
    bool result = true;

    switch ( rule )
    {
        default:
        case RulesBasePathType::CONFIGURATION_PM_RULE:
            result = Directory::getInstance()->DirectoryExists ( domMemoryBasePath ) && Directory::getInstance()->DirectoryExists ( flashMemoryBasePath );
            break;

        case RulesBasePathType::CONFIGURATION_VM_RULE:
            result = Directory::getInstance()->DirectoryExists ( liveFileSystemBasePath );
            break;
    }

    return result;
}

MemoryPathController * MemoryPathController::getInstance()
{
    if ( MemoryPathController::instance == NULL )
    {
        MemoryPathController::instance = new MemoryPathController();
    }

    return MemoryPathController::instance;
}

void MemoryPathController::addPath ( std::string path )
{
    Glib::Mutex::Lock lock ( mutex );
    logger->Debug("Adding path " + path);
    removeFromPathList(pathList, path);
    pathList.push_back(path);
    updatePaths();
}

void MemoryPathController::removePath ( const std::string& path )
{
    Glib::Mutex::Lock lock ( mutex );
    logger->Debug("Removing path " + path);
    removeFromPathList(pathList, path);
    updatePaths();
}
