/*
 * CertificateStore.cxx
 *
 *  Created on: Oct 25, 2018
 *      Author: eduardo.fernandez
 */

#include "CertificateStore.h"

#include <Configuration/s-a-e-s-common-conf-manager.h>
#include <Configuration/SaesCommonConfiguration-schema.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <System/Exception/CryptoException.hxx>
#include <System/IO/File.h>
#include <System/IO/Path.h>
#include <System/OptParser.hxx>
#include <System/Security/Cryptography/CertificateExtensionBase.hxx>
#include <System/Security/Cryptography/CertUtil.h>
#include <System/Security/OpenSSLCertificate.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::SAES::Security;
using namespace Smartmatic::System;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::Security;
using namespace Smartmatic::System::Security::Cryptography;

ISMTTLog *CertificateStore::m_logger =
    SMTTLogManager::GetLogger ( "Smartmatic.SAES.Security.CertificateStore", GETTEXT_PACKAGE );

CertificateStore *CertificateStore::m_instance = NULL;

/**
 * @brief Gets the client certificate full path.
 * @author eduardo.fernandez
 * @date   07/11/2018
 * @param  directories SAES directories.
 * @param  relativePath Client certificate relative path.
 * @return The client certificate full path.
 */
static std::string
getClientCertificateFullPath(SaesDirectories * const directories, const std::string& relativePath)
{
    const std::string domBasePath   = directories->getBasePath();
    const std::string flashBasePath = directories->getBasePath(BasePathType::CONFIGURATION_FLASH);

    const std::string fullPath = File::getInstance()->canOpenFile(Path::Combine(domBasePath, relativePath)) ?
        Path::Combine(domBasePath, relativePath) :
        Path::Combine(flashBasePath, relativePath);

    return fullPath;
}

/**
 * @brief Gets the client certificate format.
 * @author eduardo.fernandez
 * @date   07/11/2018
 * @param  type Client certificate configuration type.
 * @return The client certificate format.
 */
static CertUtil::CertFileFormat
getClientCertificateFormat(const CertifiedType& type)
{
    CertUtil::CertFileFormat format = CertUtil::CertFormat_PKCS12;

    switch (type)
    {
        default:
        case CertifiedType::CONFIGURATION_PCKS12:
            format = CertUtil::CertFormat_PKCS12;
            break;

        case CertifiedType::CONFIGURATION_X509:
            format = CertUtil::CertFormat_PEM;
            break;

        case CertifiedType::CONFIGURATION_NONE:
            break;
    }

    return format;
}

/**
 * @brief Gets the client certificate full path and format.
 * @author eduardo.fernandez
 * @date   07/11/2018
 * @param[out] fullPath Client certificate full path.
 * @param[out] format Client certificate format.
 * @param[in]  SAES directories.
 * @param[in]  resourceFileType Type of the resource file.
 */
static void
getClientCertificateParameters(
    std::string& fullPath,
    CertUtil::CertFileFormat& format,
    SaesDirectories * const directories,
    const Encryption::ResourceFileType& resourceFileType)
{
    SecurityConfiguration securityConfiguration =
        SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration();

    std::string   relativePath;
    CertifiedType type = CertifiedType::CONFIGURATION_NONE;

    switch (resourceFileType)
    {
        default:
        case Encryption::INSTALL_FILE:
            relativePath = securityConfiguration.getInstallationFiles().getCertifiedFilePath();
            type         = securityConfiguration.getInstallationFiles().getCertifiedType();
            break;

        case Encryption::EXTERNAL_FILE:
            relativePath = securityConfiguration.getExternalFiles().getCertifiedFilePath();
            type         = securityConfiguration.getExternalFiles().getCertifiedType();
            break;

        case Encryption::INTERNAL_FILE:
            relativePath = securityConfiguration.getInternalFiles().getCertifiedFilePath();
            type         = securityConfiguration.getInternalFiles().getCertifiedType();
            break;

        case Encryption::RESOURCE_FILE:
            relativePath = securityConfiguration.getResourcesFiles().getCertifiedFilePath();
            type         = securityConfiguration.getResourcesFiles().getCertifiedType();
            break;

        case Encryption::EXCHANGE_FILE:
            relativePath = securityConfiguration.getExchangeFiles().getCertifiedFilePath();
            type         = securityConfiguration.getExchangeFiles().getCertifiedType();
            break;

        case Encryption::VOTING_RESULT_FILE:
            relativePath = securityConfiguration.getVotingResultFiles().getCertifiedFilePath();
            type         = securityConfiguration.getVotingResultFiles().getCertifiedType();
            break;

        case Encryption::UTIL_FILE:
            relativePath = securityConfiguration.getUtilFiles().getCertifiedFilePath();
            type         = securityConfiguration.getUtilFiles().getCertifiedType();
            break;

        case Encryption::PRINTED_VOTE:
            relativePath = securityConfiguration.getPrintedVoteData().getCertifiedFilePath();
            type         = securityConfiguration.getPrintedVoteData().getCertifiedType();
            break;
    }

    fullPath = getClientCertificateFullPath(directories, relativePath);
    format   = getClientCertificateFormat(type);
}

CertificateStore::CertificateStore(SaesDirectories * const directories)
: m_directories(directories)
, m_validatorWrapper(CertificateValidatorWrapper::getInstance())
, m_isRootCertificateValid(false)
, m_isIntermediateCertificateValid(false)
{

}

CertificateStore::~CertificateStore()
{
    dispose();

    for (std::map<std::string, CertificateExtensionBase*>::const_iterator it = m_store.begin(); it != m_store.end(); ++it)
        delete it->second;
}

CertificateStore*
CertificateStore::getInstance(SaesDirectories * const directories, CertificateStore * const externalInstance)
{
    if ( m_instance == NULL || externalInstance != NULL)
    {
        m_instance = (externalInstance == NULL ? new CertificateStore(directories) : externalInstance);
    }

    return m_instance;
}

void
CertificateStore::loadIssuerCertificates(const std::string& rootFingerprint)
{
    loadRootCertificate(rootFingerprint);
    loadIntermediateCertificate();
}

OpenSSLCryptoX509 *
CertificateStore::getRootCertificate()
{
    OpenSSLCryptoX509 *x509 = NULL;

    if (m_isRootCertificateValid)
    {
        SafeBuffer buffer;
        File::getInstance()->openFileToBuffer(m_directories->getRootCACertificateFileFullPath(), buffer);
        CertUtil::updateCertificateBuffer(buffer);
        OpenSSLCertificate *certificate =  new OpenSSLCertificate(buffer);

        x509 = certificate->getX509();
    }

    return x509;
}

OpenSSLCryptoX509 *
CertificateStore::getIntermediateCertificate()
{
    OpenSSLCryptoX509 *x509 = NULL;

    if (m_isIntermediateCertificateValid)
    {
        SafeBuffer buffer;
        File::getInstance()->openFileToBuffer(m_directories->getIntermediateCACertificateFileFullPath(), buffer);
        CertUtil::updateCertificateBuffer(buffer);
        OpenSSLCertificate *certificate =  new OpenSSLCertificate(buffer);

        x509 = certificate->getX509();
    }

    return x509;
}

CertificateExtensionBase*
CertificateStore::getClientCertificate(
    const Encryption::ResourceFileType& resourceFileType,
    const std::string& protectionKey)
{
    std::string fullPath;
    std::string storeId;
    CertUtil::CertFileFormat format = CertUtil::CertFormat_PEM;

    getClientCertificateParameters(fullPath, format, m_directories, resourceFileType);

    storeId = Path::GetFileName(fullPath).empty() ? fullPath : Path::GetFileName(fullPath);

    if (m_store.find(storeId) == m_store.end())
    {
        m_validatorWrapper->validateClientCertificate(fullPath, format, protectionKey);

        CertificateExtensionBase * const clientCertificate =
            CertUtil::getCertificateExtensionBase(fullPath, format, protectionKey);

        clientCertificate->LoadCertifiedX509();
        clientCertificate->LoadCertifiedPrivateKey();
        clientCertificate->LoadCertifiedPublicKey();

        m_store[storeId] = clientCertificate;
    }

    return m_store[storeId]->clone();
}

void
CertificateStore::loadRootCertificate(const std::string& rootFingerprint)
{
    const std::string rootCertificatePath = m_directories->getRootCACertificateFileFullPath();
    m_validatorWrapper->validateRootCertificate(rootCertificatePath, rootFingerprint);
    m_isRootCertificateValid = true;
}

void
CertificateStore::loadIntermediateCertificate()
{
    const std::string intermediateCertificatePath = m_directories->getIntermediateCACertificateFileFullPath();
    m_validatorWrapper->validateIntermediateCertificate(intermediateCertificatePath);
    m_isIntermediateCertificateValid = true;
}

void
CertificateStore::dispose()
{
    if (m_directories != NULL)
    {
        delete m_directories;
        m_directories = NULL;
    }

    if (m_validatorWrapper != NULL)
    {
        delete m_validatorWrapper;
        m_validatorWrapper = NULL;
    }

    m_isRootCertificateValid         = false;
    m_isIntermediateCertificateValid = false;
}
