/**
 * @file	Encryption.cxx
 * @brief	encryption class.
 * @author	Youssef.tourkmani
 * @date	30/10/2012
 */

#include <System/XML/DomXml.hxx>
#include <System/Compression/compression-class.h>
#include <glibmm.h>
#include <System/IO/File.h>
#include <System/Encoding/Base64Encoder.h>
#include "Configuration/s-a-e-s-common-conf-manager.h"
#include "Encryption.hxx"
#include <System/IO/Path.h>
#include <System/IO/File.h>
#include <Log/SMTTLogManager.h>
#include <Security/CertificateStore.h>
#include <System/Security/Cryptography/CertUtil.h>
#include <Security/KeyManager.hxx>
#include <Security/XmlFileSignature.hxx>
#include <Security/Passphrase.hxx>
#include <System/OptParser.hxx>
#include <System/Security/Cryptography/SignUtils.hxx>
#include <System/Utils/StringUtils.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::System::Encoding;
using namespace Smartmatic::SAES::Security;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::System::Security;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::System::XML;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Common::Security;
using namespace Smartmatic::System::Security::Cryptography;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::Log;
using namespace Smartmatic::System::Compression;
using namespace Smartmatic::System;

static const std::string SCREEN_GENERATOR_MODE = "SG";

ISMTTLog * Encryption::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Security.Encryption", GETTEXT_PACKAGE );

Encryption * Encryption::instance = NULL;

Encryption::Encryption()
{
    machineTime = 0;
    m_SecretKey = NULL;
    directories = NULL;

    Smartmatic::System::Security::Crypto::KeySize keySize = Smartmatic::System::Security::Crypto::KEY_AES_256;
    Smartmatic::System::Security::Crypto::Mode modeType = Crypto::MODE_AES_CBC;

    KeyManager::getInstance()->initialize ( keySize, modeType );

    SafeBuffer initKey;

    KeyManager::getInstance()->getKey ( Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_INIT_KEY, initKey );
}

Encryption::~Encryption()
{
    dispose();
}

void Encryption::initialize()
{
    SecurityCryptoMode mode;

    if ( SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getSecurityCryptoMode() != NULL )
    {
        mode = SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getSecurityCryptoMode().get();
    }
    else
    {
        mode = SecurityCryptoMode::CONFIGURATION_CBC;
    }

    Smartmatic::System::Security::Crypto::KeySize keySize = Smartmatic::System::Security::Crypto::KEY_AES_256;
    Smartmatic::System::Security::Crypto::Mode modeType;

    switch ( mode )
    {
        case SecurityCryptoMode::CONFIGURATION_ECB:
            modeType = Crypto::MODE_AES_ECB;
            break;

        case SecurityCryptoMode::CONFIGURATION_CBC:
        default:
            modeType = Crypto::MODE_AES_CBC;
            break;
    }

    KeyManager::getInstance()->initialize ( keySize, modeType );
}

void Encryption::dispose()
{
    if ( m_SecretKey )
    {
        delete m_SecretKey;
        m_SecretKey = NULL;
    }
}

void Encryption::initCrypto ( SecretKey & sk, Smartmatic::SAES::Common::Configuration::KeyType ukt ) throw ( CryptoException )
{
    SafeBuffer buffer;

    switch ( ukt )
    {
        case Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_UP_KEY:
        case Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECURITY_KEY:
            sk.getSafeBuffer ( buffer );
            break;

        case Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY:
        case Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_VERIFICATION_KEY:
        case Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_INIT_KEY:
        case Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_RECOUNT_KEY:
        default:

            if ( m_SecretKey )
            {
                delete m_SecretKey;
                m_SecretKey = NULL;
            }

            this->m_SecretKey = new SecretKey ( sk );
            this->m_SecretKey->getSafeBuffer ( buffer );

            Passphrase::getInstance()->setPlainPassphrase ( m_SecretKey->getPlainPassphrase() );
            m_SecretKey->clearPlainPassphrase();

            break;
    }

    KeyManager::getInstance()->addKeyByType ( buffer, ukt );
}

void Encryption::initSigned ( Smartmatic::SAES::Runtime::Environment::SaesDirectories * directories, std::string & signedKey, std::string & cbcMacKey ) throw ( SignedException, CryptoException )
{
    this->directories = directories;
    this->signedSecretKey = signedKey;
    this->cbcMacKey = cbcMacKey;
}

void Encryption::signedXml ( std::string & xml, ResourceFileType rft )
throw ( CryptoException, SignedException )
{
    SafeBuffer bufferIn ( xml );
    SafeBuffer bufferOut;

    signedXmlBuffer ( bufferIn, bufferOut, rft );
    xml.clear();
    xml.assign ( ( const char * ) bufferOut.GetData(), bufferOut.GetSize() );
}

void Encryption::signedXmlBuffer ( SafeBuffer & bufferIn,
                                   SafeBuffer & bufferOut,
                                   Encryption::ResourceFileType rft )
throw ( CryptoException, SignedException )
{
    Signed objSigned;
    CertificateExtensionBase * const clientCertificate =
        CertificateStore::getInstance(directories)->getClientCertificate(rft, signedSecretKey);
    objSigned.SetCertificate(clientCertificate);
    objSigned.signBuffer ( bufferIn, bufferOut, signedData ( rft ) );
}

void Encryption::signedFile ( std::string & file,
                              Encryption::ResourceFileType rft ) throw ( CryptoException, SignedException,
                                      XmlException )
{
    SafeBuffer bufferIn;
    SafeBuffer bufferOut;

    File::getInstance()->openFileToBuffer ( file, bufferIn );
    signedXmlBuffer ( bufferIn, bufferOut, rft );

    File::getInstance()->saveBufferToFile ( bufferOut, file );
}

void Encryption::cryptoDOMToBuffer ( xercesc::DOMDocument * doc, SafeBuffer & dataOut,
                                     Encryption::ResourceFileType rft,
                                     Smartmatic::SAES::Common::Configuration::KeyType ukt,
                                     bool pad ) throw ( CryptoException )
{
    std::string dataIn;
    Smartmatic::System::XML::DomXml dom;
    dom.domToString ( doc, dataIn );
    cryptoStringToBuffer ( dataIn, dataOut, rft, ukt, pad );
}

void Encryption::cryptoDOMToFile ( xercesc::DOMDocument * doc, std::string & file,
                                   Encryption::ResourceFileType rft,
                                   Smartmatic::SAES::Common::Configuration::KeyType ukt,
                                   bool pad ) throw ( CryptoException )
{
    std::string dataIn;
    Smartmatic::System::XML::DomXml dom;
    dom.domToString ( doc, dataIn );
    cryptoStringToFile ( dataIn, file, rft, ukt, pad );
}

void Encryption::cryptoStringToFile ( std::string & data, std::string & file,
                                      Encryption::ResourceFileType rft,
                                      Smartmatic::SAES::Common::Configuration::KeyType ukt,
                                      bool pad ) throw ( CryptoException )
{
    SafeBuffer dataIn ( ( unsigned char * ) data.c_str(), data.length() );
    cryptoBufferToFile ( dataIn, file, rft, ukt, pad );
}

void Encryption::cryptoStringToFile ( std::string & data, std::string & file,
                                      Smartmatic::SAES::Common::Configuration::KeyType ukt,
                                      bool pad ) throw ( CryptoException )
{
    SafeBuffer dataIn ( ( unsigned char * ) data.c_str(), data.length() );
    cryptoBufferToFile ( dataIn, file, ukt, pad );
}

void Encryption::cryptoBufferToFile ( const SafeBuffer & dataIn, std::string & file,
                                      Encryption::ResourceFileType rft,
                                      Smartmatic::SAES::Common::Configuration::KeyType ukt,
                                      bool pad ) throw ( CryptoException )
{
    if ( cryptoData ( rft ) )
    {
        cryptoBufferToFile ( dataIn, file, ukt );
    }
    else if ( dataIn.GetSize() > 0 )
    {
        File::getInstance()->saveBufferToFile ( dataIn, file );
    }
}

void Encryption::cryptoBufferToFile ( const SafeBuffer & dataIn, std::string & file,
                                      Smartmatic::SAES::Common::Configuration::KeyType ukt,
                                      bool pad ) throw ( CryptoException )
{
    Crypto objCrypto;
    objCrypto.init ( KeyManager::getInstance()->getKeySize(), KeyManager::getInstance()->getModeType(), pad );

    SafeBuffer key;
    SafeBuffer iv;

    KeyManager::getInstance()->getKey ( ukt, key );
    objCrypto.encryptBufferToFileAndCreateIV( dataIn, file, key.GetData() );
}

void Encryption::cryptoBufferToBuffer ( const SafeBuffer & dataIn,
                                        SafeBuffer & dataOut,
                                        SafeBuffer key, bool pad )
throw ( CryptoException )
{
    Crypto objCrypto;
    objCrypto.init ( KeyManager::getInstance()->getKeySize(), KeyManager::getInstance()->getModeType(), pad );
    objCrypto.encryptBufferToBufferAndCreateIV( dataIn, dataOut, key.GetData());
}

void Encryption::decryptoFileToString ( std::string & file, std::string & data,
                                        Encryption::ResourceFileType rft,
                                        Smartmatic::SAES::Common::Configuration::KeyType ukt,
                                        bool pad )
throw ( CryptoException )
{
    SafeBuffer dataOut;
    decryptoFileToBuffer ( file, dataOut, rft, ukt, pad );

    if ( dataOut.GetSize() > 0 )
    {
        data.assign ( ( const char * ) dataOut.GetData(), dataOut.GetSize() );
        data.resize ( dataOut.GetSize() );
    }
}

void Encryption::decryptoFileToString ( std::string & file, std::string & data,
                                        Smartmatic::SAES::Common::Configuration::KeyType ukt,
                                        bool pad )
throw ( CryptoException )
{
    SafeBuffer dataOut;
    decryptoFileToBuffer ( file, dataOut, ukt, pad );

    if ( dataOut.GetSize() > 0 )
    {
        data.assign ( ( const char * ) dataOut.GetData(), dataOut.GetSize() );
        data.resize ( dataOut.GetSize() );
    }
}

void Encryption::decryptoFileToBuffer ( std::string & file,
                                        SafeBuffer & dataOut,
                                        Encryption::ResourceFileType rft,
                                        Smartmatic::SAES::Common::Configuration::KeyType ukt,
                                        bool pad )
throw ( CryptoException )
{
    if ( cryptoData ( rft ) )
    {
        decryptoFileToBuffer ( file, dataOut, ukt, pad );
    }
    else
    {
        File::getInstance()->openFileToBuffer ( file, dataOut );
    }
}

void Encryption::decryptoFileToBuffer ( std::string & file,
                                        SafeBuffer & dataOut,
                                        Smartmatic::SAES::Common::Configuration::KeyType ukt,
                                        bool pad )
throw ( CryptoException )
{
    Crypto objCrypto;
    objCrypto.init ( KeyManager::getInstance()->getKeySize(), KeyManager::getInstance()->getModeType(), pad );
    SafeBuffer key;
    KeyManager::getInstance()->getKey ( ukt, key );
    objCrypto.decryptFileToBufferAndGetIV( file, dataOut, key.GetData());
}

void Encryption::decryptoBufferToBuffer ( const SafeBuffer & dataIn,
        SafeBuffer & dataOut,
        SafeBuffer key, bool pad )
throw ( CryptoException )
{
    Crypto objCrypto;
    objCrypto.init ( KeyManager::getInstance()->getKeySize(), KeyManager::getInstance()->getModeType(), pad );
    objCrypto.decryptBufferToBufferAndGetIV ( dataIn, dataOut, key.GetData());
}

void Encryption::validateSignatureCertificate(const SafeBuffer& xmlFileBuffer, Encryption::ResourceFileType rft)
{
    const bool mustBeValidated = validateSigned(rft);

    XmlFileSignature xmlFileSignature;
    xmlFileSignature.validateSignatureCertificate(xmlFileBuffer, mustBeValidated);
}

void Encryption::validateSignedString ( std::string & data,
                                        Encryption::ResourceFileType rft )
throw ( CryptoException, SignedException )
{
    SafeBuffer buffer ( data );
    validateSignedXmlBuffer ( buffer, rft );
}

void Encryption::validateSignedXmlBuffer ( SafeBuffer & buffer,
        Encryption::ResourceFileType rft )
throw ( CryptoException, SignedException )
{
    if ( validateSigned ( rft ) )
    {
        Signed objSigned;
        Smartmatic::System::Security::Cryptography::CertificateValidator validator;
        CertificateExtensionBase * const clientCertificate =
            CertificateStore::getInstance(directories)->getClientCertificate(rft, signedSecretKey);
        objSigned.SetCertificate(clientCertificate);

        if ( !objSigned.verifyBuffer ( buffer ) )
        {
            logger->Debug ( "Error validate signed" );
            throw SignedException ( _ ( "Smartmatic.SAES.Security.Encryption.ErrorSignedValidate" ), N_ ( "Smartmatic.SAES.Security.Encryption.ErrorSignedValidate" ), logger );
        }
    }
}

void Encryption::validateSignedFile ( std::string & file,
                                      Encryption::ResourceFileType rft )
throw ( CryptoException, SignedException, XmlException )
{
    SafeBuffer buffer;
    File::getInstance()->openFileToBuffer ( file, buffer );
    validateSignedXmlBuffer ( buffer, rft );
}

std::string Encryption::openXmlFile ( std::string & file,
                                      Encryption::ResourceFileType rft,
                                      Smartmatic::SAES::Common::Configuration::KeyType ukt,
                                      bool isCompressed,
                                      bool pad,
                                      bool validateChainOfTrust )
throw ( Smartmatic::System::Exception::CryptoException,
        Smartmatic::System::Exception::SignedException,
        Smartmatic::System::Exception::XmlException )
{
    std::stringstream ss;
    ss << "Open xml file " << file;
    logger->Debug ( ss.str() );
    SafeBuffer buffer;
    decryptoFileToBuffer ( file, buffer, rft, ukt, pad );
    std::string data;

    if ( isCompressed )
    {
        std::stringstream ssc;
        ssc << "Decompress xml data of file " << file;
        logger->Debug ( ssc.str() );
        SafeBuffer inflate;
        CompressionClass::getInstance()->Inflate ( buffer, inflate );

        if ( inflate.GetSize() > 0 )
        {
            data.assign ( ( char * ) inflate.GetData(), inflate.GetSize() );
            data.resize ( inflate.GetSize() );
        }
        else
        {
            logger->Debug ( "File " + file + " not compress." );
        }
    }
    else if ( buffer.GetSize() > 0 )
    {
        data.assign ( ( char * ) buffer.GetData(), buffer.GetSize() );
        data.resize ( buffer.GetSize() );
    }
    else
    {
        logger->Debug ( "Error open xml file " + file );
        return data;
    }

    if ( validateChainOfTrust )
    {
        logger->Debug ( "Validating file '" + file + "''s signature certificate." );
        validateSignatureCertificate(buffer, rft);

        logger->Debug ( "Verifying file '" + file + "''s signature." );
        validateSignedString ( data, rft );
    }

    logger->Debug ( "Success open xml file " + file + " with " + getUseKeyString ( ukt ) );
    return data;
}

void Encryption::saveXmlFile ( std::string & data, std::string & file,
                               Encryption::ResourceFileType rft,
                               Smartmatic::SAES::Common::Configuration::KeyType ukt,
                               bool compress,
                               bool pad )
throw ( Smartmatic::System::Exception::CryptoException,
        Smartmatic::System::Exception::SignedException,
        Smartmatic::System::Exception::XmlException )
{
    logger->Debug ( "Save xml file " + file );

    std::string dataStr = data;
    signedXml ( dataStr, rft );
    SafeBuffer dataBuffer ( dataStr );

    if ( compress )
    {
        std::stringstream ssc;
        ssc << "Compress xml data of file " << file;
        logger->Debug ( ssc.str() );
        SafeBuffer deflate;
        CompressionClass::getInstance()->Deflate ( dataBuffer, deflate );

        if ( deflate.GetSize() > 0 )
        {
            dataBuffer.Clear();
            dataBuffer.AddData ( deflate );
        }
        else
        {
            logger->Debug ( "Error deflate data" );
            return;
        }
    }
    else if ( dataBuffer.GetSize() == 0 )
    {
        logger->Debug ( "Error save xml file " + file );
        return;
    }

    SafeBuffer buffer;
    cryptoBufferToBuffer ( dataBuffer, buffer, rft, ukt, pad );
    File::getInstance()->saveBufferToFile ( buffer, file );
    logger->Debug ( "Success save xml file " + file + " with " + getUseKeyString ( ukt ) );
}

Encryption * Encryption::getInstance()
{
    if ( instance == NULL )
    {
        instance = new Encryption();
    }

    return instance;
}

std::string Encryption::cryptoStringToB64 ( std::string & data,
        Encryption::ResourceFileType rft,
        Smartmatic::SAES::Common::Configuration::KeyType ukt,
        bool pad )
throw ( CryptoException )
{
    SafeBuffer dataOut;
    cryptoStringToBuffer ( data, dataOut, rft, ukt, pad );
    return dataOut.GetBase64Data();
}

std::string Encryption::cryptoStringToB64 ( std::string & data,
        Smartmatic::SAES::Common::Configuration::KeyType ukt,
        bool pad )
throw ( CryptoException )
{
    SafeBuffer dataIn ( ( unsigned char * ) data.c_str(), data.length() );
    SafeBuffer dataOut;
    Crypto objCrypto;
    objCrypto.init ( KeyManager::getInstance()->getKeySize(), KeyManager::getInstance()->getModeType(), pad );

    SafeBuffer key;

    KeyManager::getInstance()->getKey ( ukt, key );
    objCrypto.encryptBufferToBufferAndCreateIV ( dataIn, dataOut, key.GetData());
    return dataOut.GetBase64Data();
}

std::string Encryption::decryptoB64ToString ( std::string & data,
        Encryption::ResourceFileType rft,
        Smartmatic::SAES::Common::Configuration::KeyType ukt,
        bool pad )
throw ( CryptoException )
{
    SafeBuffer dataIn;
    Base64Encoder::ConvertFromBase64 ( data, dataIn );
    return decryptoBufferToString ( dataIn, rft, ukt, pad );
}

std::string Encryption::decryptoB64ToString ( std::string & data,
        Smartmatic::SAES::Common::Configuration::KeyType ukt,
        bool pad )
throw ( CryptoException )
{
    SafeBuffer dataIn;
    decryptoB64ToBuffer ( data, dataIn, ukt, pad );
    std::string dataReturn;

    if ( dataIn.GetSize() > 0 )
    {
        dataReturn.assign ( ( const char * ) dataIn.GetData(), dataIn.GetSize() );
        dataReturn.resize ( dataIn.GetSize() );
    }

    return dataReturn;
}

void Encryption::decryptoB64ToBuffer ( std::string & data,
                                       SafeBuffer & dataOut,
                                       Smartmatic::SAES::Common::Configuration::KeyType ukt,
                                       bool pad )
throw ( Smartmatic::System::Exception::CryptoException )
{
    SafeBuffer dataIn;
    Base64Encoder::ConvertFromBase64 ( data, dataIn );
    Crypto objCrypto;
    objCrypto.init ( KeyManager::getInstance()->getKeySize(), KeyManager::getInstance()->getModeType(), pad );
    SafeBuffer key;

    KeyManager::getInstance()->getKey ( ukt, key );
    objCrypto.decryptBufferToBufferAndGetIV ( dataIn, dataOut, key.GetData());
}

void Encryption::cryptoBufferToBuffer (
    SafeBuffer & data,
    SafeBuffer & dataOut,
    Encryption::ResourceFileType rft,
    Smartmatic::SAES::Common::Configuration::KeyType ukt,
    bool pad )
throw ( CryptoException )
{
    if ( cryptoData ( rft ) )
    {
        cryptoBufferToBuffer ( data, dataOut, ukt, pad );
    }
    else
    {
        dataOut.AddData ( data );
    }
}

void Encryption::cryptoBufferToBuffer (
    SafeBuffer & data,
    SafeBuffer & dataOut,
    Smartmatic::SAES::Common::Configuration::KeyType ukt,
    bool pad )
throw ( CryptoException )
{
    Crypto objCrypto;
    objCrypto.init ( KeyManager::getInstance()->getKeySize(), KeyManager::getInstance()->getModeType(), pad );
    SafeBuffer key;
    KeyManager::getInstance()->getKey ( ukt, key );
    objCrypto.encryptBufferToBufferAndCreateIV ( data, dataOut, key.GetData() );
}

void Encryption::cryptoStringToBuffer ( std::string & data,
                                        SafeBuffer & dataOut,
                                        Encryption::ResourceFileType rft,
                                        Smartmatic::SAES::Common::Configuration::KeyType ukt,
                                        bool pad )
throw ( CryptoException )
{
    SafeBuffer dataIn ( ( unsigned char * ) data.c_str(), data.length() );
    cryptoBufferToBuffer ( dataIn, dataOut, rft, ukt, pad );
}

std::string Encryption::decryptoBufferToString ( const SafeBuffer & dataIn,
        Encryption::ResourceFileType rft,
        Smartmatic::SAES::Common::Configuration::KeyType ukt,
        bool pad )
throw ( CryptoException )
{
    std::string dataReturn;
    SafeBuffer dataOut;
    decryptoBufferToBuffer ( dataIn, dataOut, rft, ukt, pad );

    if ( dataOut.GetSize() > 0 )
    {
        dataReturn.assign ( ( const char * ) dataOut.GetData(), dataOut.GetSize() );
        dataReturn.resize ( dataOut.GetSize() );
    }

    return dataReturn;
}

void Encryption::decryptoBufferToBuffer (
    const SafeBuffer & dataIn,
    SafeBuffer & dataOut,
    Encryption::ResourceFileType rft,
    Smartmatic::SAES::Common::Configuration::KeyType ukt,
    bool pad )
throw ( CryptoException )
{
    if ( cryptoData ( rft ) )
    {
        decryptoBufferToBuffer ( dataIn, dataOut, ukt, pad );
    }
    else
    {
        dataOut.AddData ( dataIn );
    }
}

void Encryption::decryptoBufferToBuffer (
    const SafeBuffer & dataIn,
    SafeBuffer & dataOut,
    Smartmatic::SAES::Common::Configuration::KeyType ukt,
    bool pad )
throw ( CryptoException )
{
    Crypto objCrypto;
    objCrypto.init ( KeyManager::getInstance()->getKeySize(), KeyManager::getInstance()->getModeType(), pad );
    SafeBuffer key;
    KeyManager::getInstance()->getKey ( ukt, key );
    objCrypto.decryptBufferToBufferAndGetIV ( dataIn, dataOut, key.GetData() );
}

void Encryption::setOpeningCode ( Smartmatic::System::GuidClass & openingCode )
{
    this->openingCode = openingCode;

    SafeBuffer secretKey;
    SafeBuffer verificationKey;

    KeyManager::getInstance()->getKey ( Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY, secretKey );

    Glib::ustring strKey;

    strKey = secretKey.GetBase64Data();
    strKey += "_";
    strKey += openingCode.ToBase64String();

    SafeBuffer buffer ( strKey );

    MsgDigestUtil digest ( buffer );
    digest.getDigest ( verificationKey, MsgDigestUtil::getDefaultOpeningCodeDigest() );

    KeyManager::getInstance()->addKeyByType ( verificationKey, Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_VERIFICATION_KEY );
}

const Smartmatic::System::GuidClass & Encryption::getOpeningCode()
{
    return this->openingCode;
}

Signed::SignedType Encryption::signedData ( Encryption::ResourceFileType rft )
{
    Smartmatic::System::Security::Signed::SignedType signedType;
    Smartmatic::SAES::Common::Configuration::SignedType type;

    switch ( rft )
    {
        default:
        case INSTALL_FILE:
            type = SAESCommonConfigurationManager::getInstance()->getConfiguration()->getMachineSecuritySchema().getInstallationFiles().getSignedData();
            break;

        case EXTERNAL_FILE:
            type = SAESCommonConfigurationManager::getInstance()->getConfiguration()->getMachineSecuritySchema().getExternalFiles().getSignedData();
            break;

        case INTERNAL_FILE:
            type = SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getInternalFiles().getSignedData();
            break;

        case RESOURCE_FILE:
            type = SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getResourcesFiles().getSignedData();
            break;

        case EXCHANGE_FILE:
            type = SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getExchangeFiles().getSignedData();
            break;

        case VOTING_RESULT_FILE:
            type = SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getVotingResultFiles().getSignedData();
            break;

        case UTIL_FILE:
            type = SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getUtilFiles().getSignedData();
            break;

        case PRINTED_VOTE:
            type = SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getPrintedVoteData().getSignedData();
            break;
    }

    switch ( type )
    {
        case SignedType::CONFIGURATION_SIMPLE:
            signedType = Signed::SIMPLE_SIGNED;
            break;

        case SignedType::CONFIGURATION_FULL:
            signedType = Signed::FULL_SIGNED;
            break;

        case SignedType::CONFIGURATION_NONE:
        default:
            signedType = Signed::NOT_SIGNED;
            break;
    }

    return signedType;
}

bool Encryption::SignDataWithPersonalCertificate ( const SafeBuffer & dataIn, SafeBuffer & dataOut )
{
    bool success = false;

    try
    {
        CertificateExtensionBase * const clientCertificate =
            CertificateStore::getInstance(directories)->getClientCertificate(PRINTED_VOTE, signedSecretKey);

        success = SignUtils::signBuffer ( clientCertificate, dataIn, dataOut, MsgDigestUtil::getDefaultCertificateDigest() );
    }
    catch (CryptoException &cryptoException)
    {
        logger->Error("An error occurred while trying to validate the certificate chain of trust during the vote signing.");
    }

    return success;
}

bool Encryption::VerifySignatureWithPersonalCertificate ( const SafeBuffer & dataIn, SafeBuffer & signature )
{
    bool success = false;

    try
    {
        CertificateExtensionBase * const clientCertificate =
            CertificateStore::getInstance(directories)->getClientCertificate(PRINTED_VOTE, signedSecretKey);

        success = SignUtils::verifyBuffer ( clientCertificate, dataIn, signature, MsgDigestUtil::getDefaultCertificateDigest() );
    }
    catch (CryptoException &cryptoException)
    {
        logger->Error("An error occurred while trying to validate the certificate chain of trust during the vote's signature verification.");
    }

    return success;
}

bool Encryption::validateSigned ( Encryption::ResourceFileType rft )
{
    bool validate = false;

    switch ( rft )
    {
        default:
        case INSTALL_FILE:
            validate = SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getInstallationFiles().getValidateSigned();
            break;

        case EXTERNAL_FILE:
            validate = SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getExternalFiles().getValidateSigned();
            break;

        case INTERNAL_FILE:
            validate = SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getInternalFiles().getValidateSigned();
            break;

        case RESOURCE_FILE:
            validate = SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getResourcesFiles().getValidateSigned();
            break;

        case EXCHANGE_FILE:
            validate = SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getExchangeFiles().getValidateSigned();
            break;

        case VOTING_RESULT_FILE:
            validate = SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getVotingResultFiles().getValidateSigned();
            break;

        case UTIL_FILE:
            validate = SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getUtilFiles().getValidateSigned();
            break;

        case PRINTED_VOTE:
            validate = SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getPrintedVoteData().getValidateSigned();
            break;
    }

    return validate;
}

bool Encryption::cryptoData ( Encryption::ResourceFileType rft )
{
    bool crypto = false;

    switch ( rft )
    {
        default:
        case INSTALL_FILE:
            crypto = SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getInstallationFiles().getEncryptedData();
            break;

        case EXTERNAL_FILE:
            crypto = SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getExternalFiles().getEncryptedData();
            break;

        case INTERNAL_FILE:
            crypto = SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getInternalFiles().getEncryptedData();
            break;

        case RESOURCE_FILE:
            crypto = SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getResourcesFiles().getEncryptedData();
            break;

        case EXCHANGE_FILE:
            crypto = SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getExchangeFiles().getEncryptedData();
            break;

        case VOTING_RESULT_FILE:
            crypto = SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getVotingResultFiles().getEncryptedData();
            break;

        case UTIL_FILE:
            crypto = SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getUtilFiles().getEncryptedData();
            break;

        case PRINTED_VOTE:
            crypto = SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getPrintedVoteData().getEncryptedData();
            break;
    }

    return crypto;
}

std::string Encryption::getUseKeyString ( Smartmatic::SAES::Common::Configuration::KeyType ukt )
{
    std::string value;

    switch ( ukt )
    {
        case Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_UP_KEY:

            if ( KeyManager::getInstance()->isKeyByType ( ukt ) )
            {
                value = ukt;
            }
            else
            {
                value = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY;
            }

            break;

        case Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECURITY_KEY:

            if ( KeyManager::getInstance()->isKeyByType ( ukt ) )
            {
                value = ukt;
            }
            else
            {
                value = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_INIT_KEY;
            }

            break;

        case Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY:
        case Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_VERIFICATION_KEY:
        case Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_INIT_KEY:
        case Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_RECOUNT_KEY:
            value = ukt;
            break;

        default:
            break;
    }

    return value;
}

int Encryption::getCertificateValidatorFlags ( Encryption::ResourceFileType rft )
{
    int flags = 0;
    SecurityFileConfiguration::ValidateCertifiedConfigSequence sequence;

    switch ( rft )
    {
        default:
        case INSTALL_FILE:
            sequence = SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getInstallationFiles().getValidateCertifiedConfig();
            break;

        case EXTERNAL_FILE:
            sequence = SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getExternalFiles().getValidateCertifiedConfig();
            break;

        case INTERNAL_FILE:
            sequence = SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getInternalFiles().getValidateCertifiedConfig();
            break;

        case RESOURCE_FILE:
            sequence = SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getResourcesFiles().getValidateCertifiedConfig();
            break;

        case EXCHANGE_FILE:
            sequence = SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getExchangeFiles().getValidateCertifiedConfig();
            break;

        case VOTING_RESULT_FILE:
            sequence = SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getVotingResultFiles().getValidateCertifiedConfig();
            break;

        case UTIL_FILE:
            sequence = SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getUtilFiles().getValidateCertifiedConfig();
            break;

        case PRINTED_VOTE:
            sequence = SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getPrintedVoteData().getValidateCertifiedConfig();
            break;
    }

    for ( unsigned int i = 0; i < sequence.size(); i++ )
    {
        switch ( sequence[i] )
        {
            default:
            case ValidateCertifiedConfiguration::CONFIGURATION_NO_FLAGS:
                flags |= Cryptography::VERIFY_NO_FLAGS;
                break;

            case ValidateCertifiedConfiguration::CONFIGURATION_DO_NOT_VALIDATE:
                flags |= Cryptography::VERIFY_DO_NOT_VALIDATE;
                break;

            case ValidateCertifiedConfiguration::CONFIGURATION_IGNORE_CRITICAL:
                flags |= Cryptography::VERIFY_IGNORE_CRITICAL;
                break;

            case ValidateCertifiedConfiguration::CONFIGURATION_CB_ISSUER_CHECK:
                flags |= Cryptography::VERIFY_CB_ISSUER_CHECK;
                break;

            case ValidateCertifiedConfiguration::CONFIGURATION_CRL_CHECK:
                flags |= Cryptography::VERIFY_CRL_CHECK;
                break;

            case ValidateCertifiedConfiguration::CONFIGURATION_CRL_CHECK_ALL:
                flags |= Cryptography::VERIFY_CRL_CHECK_ALL;
                break;

            case ValidateCertifiedConfiguration::CONFIGURATION_POLICY_CHECK:
                flags |= Cryptography::VERIFY_POLICY_CHECK;
                break;

            case ValidateCertifiedConfiguration::CONFIGURATION_EXPLICIT_POLICY:
                flags |= Cryptography::VERIFY_EXPLICIT_POLICY;
                break;

            case ValidateCertifiedConfiguration::CONFIGURATION_INHIBIT_ANY:
                flags |= Cryptography::VERIFY_INHIBIT_ANY;
                break;

            case ValidateCertifiedConfiguration::CONFIGURATION_INHIBIT_MAP:
                flags |= Cryptography::VERIFY_INHIBIT_MAP;
                break;

            case ValidateCertifiedConfiguration::CONFIGURATION_X509_STRICT:
                flags |= Cryptography::VERIFY_X509_STRICT;
                break;

            case ValidateCertifiedConfiguration::CONFIGURATION_EXTENDED_CRL_SUPPORT:
                flags |= Cryptography::VERIFY_EXTENDED_CRL_SUPPORT;
                break;

            case ValidateCertifiedConfiguration::CONFIGURATION_USE_DELTAS:
                flags |= Cryptography::VERIFY_USE_DELTAS;
                break;

            case ValidateCertifiedConfiguration::CONFIGURATION_NOTIFY_POLICY:
                flags |= Cryptography::VERIFY_NOTIFY_POLICY;
                break;

            case ValidateCertifiedConfiguration::CONFIGURATION_CHECK_SS_SIGNATURE:
                flags |= Cryptography::VERIFY_CHECK_SS_SIGNATURE;
                break;

            case ValidateCertifiedConfiguration::CONFIGURATION_SET_CHECK_TIME:
                flags |= Cryptography::VERIFY_SET_CHECK_TIME;
                break;

            case ValidateCertifiedConfiguration::CONFIGURATION_ALLOW_PROXY_CERTS:
                flags |= Cryptography::VERIFY_ALLOW_PROXY_CERTS;
                break;
        }
    }

    return flags;
}

void Encryption::setMachineTime ( double machineTime )
{
    this->machineTime = machineTime;
}

time_t Encryption::getCertificateValidatorTime()
{
    Glib::TimeVal time;
    Glib::ustring dateStr = "";
    double timeD = 0;
    TimeConfiguration type =
        SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getValidateCertifiedTime();

    switch ( type )
    {
        case TimeConfiguration::CONFIGURATION_CONFIG_TIME:
            dateStr = SAESCommonConfigurationManager::getInstance()->getSecurityConfiguration().getCertifiedTime();
            time.assign_from_iso8601 ( dateStr );
            timeD = time.as_double();
            break;

        case TimeConfiguration::CONFIGURATION_MACHINE_TIME:
            timeD = machineTime;
            break;

        default:
        case TimeConfiguration::CONFIGURATION_NONE:
            timeD = 0;
            break;
    }

    return ( time_t ) timeD;
}

bool Encryption::validateIssuerCertificates(const std::string& rootCertificateFingerprint)
{
    bool success = false;

    try
    {
        std::string operationMode;
        OptParser::getInstance()->getArg(OptParser::MODE, operationMode);

        if (StringUtils::toUpper(operationMode) != SCREEN_GENERATOR_MODE)
        {
            CertificateStore::getInstance(directories)->loadIssuerCertificates(rootCertificateFingerprint);
        }

        success = true;
    }
    catch (CryptoException& cryptoException)
    {
        logger->Error("Issuer certificates could not be validated.");
        throw;
    }

    return success;
}

std::string Encryption::getSignedSecretKey()
{
    return signedSecretKey;
}

std::string Encryption::getCbcMacKey()
{
    return cbcMacKey;
}

std::string Encryption::getRootCAFingerprint(const std::string& rootCAPath)
{
    const std::string fingerprint = CertUtil::getFingerprint(rootCAPath);

    return fingerprint;
}
