/**
 * @file	Encryption.hxx
 * @brief	encryption class.
 * @author	Youssef.tourkmani
 * @date	30/10/2012
 */

#ifndef __ENCRYPTION__H__
#define __ENCRYPTION__H__

#include <System/SafeBuffer.h>
#include <System/guid-class.h>
#include <System/Security/Signed.hxx>
#include <System/Security/Cryptography/CertificateValidator.h>
#include <Runtime/Environment/saes-directories.h>
#include <System/Exception/XmlException.hxx>
#include <System/Security/Crypto.hxx>
#include <vector>
#include <Configuration/key-type-schema.hxx>
#include "Security/SecretKey.h"

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }
    namespace SAES
    {
        namespace Security
        {
            /**
             * @class	Encryption
             * @brief	Header class Encryption.
             * @author	Yadickson Soto
             * @date	04-04-2011
             */
            class Encryption
            {
                public:

                    /**
                     * @enum ResourceFileType
                     * @brief	Type of the resource file
                     */
                    typedef enum
                    {
                        INSTALL_FILE ///< install file
                        , EXTERNAL_FILE ///< external file
                        , INTERNAL_FILE ///< internal file
                        , RESOURCE_FILE ///< resource file
                        , EXCHANGE_FILE ///< exchange file
                        , VOTING_RESULT_FILE ///< voting result file
                        , UTIL_FILE ///< util file
                        , PRINTED_VOTE ///< printed vote
                    } ResourceFileType;

                private:

                    /**
                     * @fn	Encryption::Encryption();
                     * @brief	Class constructor.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     */

                    Encryption();

                public:

                    /**
                     * @fn	virtual Encryption::~Encryption();
                     * @brief	Class destructor.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     */

                    virtual ~Encryption();

                    /**
                     * @fn	void initialize();
                     * @brief	Initialize configuration parameters
                     * @author	Yadickson Soto
                     * @date	09/12/2013
                     */

                    void initialize();

                    /**
                     * @fn	void Encryption::initCrypto(Smartmatic::SAES::Common::Security::SecretKey& key,
                     * 			Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY ) throw (Smartmatic::System::Exception::CryptoException);
                     * @brief	Method for init crypto class.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @param [in,out]	key	SecretKey object containing the key for decryption of the secret file.
                     * @param	ukt					 	The ukt.
                     * 											@throw CryptoException.
                     */

                    void initCrypto ( Smartmatic::SAES::Common::Security::SecretKey & key,
                                      Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY )
                    throw ( Smartmatic::System::Exception::CryptoException );

                    /**
                     * @fn	void dispose ();
                     * @brief	Method delete all elements
                     * @author	Yadickson Soto
                     * @date	05/11/2015
                     */
                    void dispose ();

                    /**
                     * @fn	void Encryption::initSigned(Smartmatic::SAES::Runtime::Environment::SaesDirectories * directories,
                     * 			std::string & signedKey) throw (Smartmatic::System::Exception::SignedException,
                     * 			Smartmatic::System::Exception::CryptoException);
                     * @brief	Method for init class.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @exception	Smartmatic::System::Exception::SignedException	Thrown when a signed error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @param	directories	The directories.
                     * @param	signedKey	 	secret key for signed
                     * 										@throw SignedException.
                     */

                    void initSigned ( Smartmatic::SAES::Runtime::Environment::SaesDirectories * directories, std::string & signedKey, std::string & cbcMacKey )
                    throw ( Smartmatic::System::Exception::SignedException,
                            Smartmatic::System::Exception::CryptoException );

                    /**
                     * @fn	void Encryption::signedXml(std::string & xml,
                     * 			ResourceFileType rft) throw (Smartmatic::System::Exception::CryptoException,
                     * 			Smartmatic::System::Exception::SignedException);
                     * @brief	method to signed DOMElement.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::SignedException	Thrown when a signed error condition
                     * 																														occurs.
                     * @param	xml	DOM document signed
                     * 						@throw SignedException.
                     * @param	rft	The rft.
                     */

                    void signedXml ( std::string & xml, ResourceFileType rft )
                    throw ( Smartmatic::System::Exception::CryptoException,
                            Smartmatic::System::Exception::SignedException );

                    /**
                     * @fn	void Encryption::signedXmlBuffer( Smartmatic::System::SafeBuffer & bufferIn,
                     * 			Smartmatic::System::SafeBuffer & bufferOut,
                     * 			ResourceFileType rft) throw (Smartmatic::System::Exception::CryptoException,
                     * 			Smartmatic::System::Exception::SignedException);
                     * @brief	method to signed DOMElement.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::SignedException	Thrown when a signed error condition
                     * 																														occurs.
                     * @param	bufferIn	document to signed
                     * @param	bufferOut	document signed
                     * 						@throw SignedException.
                     * @param	rft	The rft.
                     */

                    void signedXmlBuffer ( Smartmatic::System::SafeBuffer & bufferIn,
                                           Smartmatic::System::SafeBuffer & bufferOut, ResourceFileType rft )
                    throw ( Smartmatic::System::Exception::CryptoException,
                            Smartmatic::System::Exception::SignedException );

                    /**
                     * @fn	void Encryption::signedFile(std::string & file,
                     * 			ResourceFileType rft) throw (Smartmatic::System::Exception::CryptoException,
                     * 			Smartmatic::System::Exception::SignedException, Smartmatic::System::Exception::XmlException);
                     * @brief	method to signed file.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::SignedException	Thrown when a signed error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::XmlException			Thrown when an XML error condition
                     * 																														occurs.
                     * @param	file	file signed.
                     * @param	rft 	The rft.
                     */

                    void signedFile ( std::string & file, ResourceFileType rft )
                    throw ( Smartmatic::System::Exception::CryptoException,
                            Smartmatic::System::Exception::SignedException,
                            Smartmatic::System::Exception::XmlException );

                    /**
                     * @fn	void Encryption::cryptoDOMToBuffer(xercesc::DOMDocument * doc,
                     * 			Smartmatic::System::SafeBuffer & dataOut, ResourceFileType rft, Smartmatic::SAES::Common::Configuration::KeyType ukt,
                     * 			bool pad = true) throw (Smartmatic::System::Exception::CryptoException);
                     * @brief	Crypto dom to buffer.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @param [in,out]	doc		 	If non-null, the document.
                     * @param [in,out]	dataOut	The data out.
                     * @param	rft							 	The rft.
                     * @param	ukt							 	The ukt.
                     * @param	pad							 	(optional) the pad.
                     */
                    void cryptoDOMToBuffer ( xercesc::DOMDocument * doc, Smartmatic::System::SafeBuffer & dataOut,
                                             ResourceFileType rft,
                                             Smartmatic::SAES::Common::Configuration::KeyType ukt,
                                             bool pad = true ) throw ( Smartmatic::System::Exception::CryptoException );


                    /**
                     * @fn	void Encryption::cryptoDOMToFile(xercesc::DOMDocument * doc, std::string & file,
                     * 			ResourceFileType rft, Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY,
                     * 			bool pad = true) throw (Smartmatic::System::Exception::CryptoException);
                     * @brief	method to crypto DOMElement.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @param	doc 	DOM document signed.
                     * @param	file	crypto file
                     * 							@throw CryptoException.
                     * @param	rft 	The rft.
                     * @param	ukt 	The ukt.
                     * @param	pad 	(optional) the pad.
                     */

                    void cryptoDOMToFile ( xercesc::DOMDocument * doc, std::string & file,
                                           ResourceFileType rft,
                                           Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY,
                                           bool pad = true )
                    throw ( Smartmatic::System::Exception::CryptoException );

                    /**
                     * @fn	void Encryption::cryptoStringToFile(std::string & data, std::string & file,
                     * 			ResourceFileType rft, Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY,
                     * 			bool pad = true) throw (Smartmatic::System::Exception::CryptoException);
                     * @brief	method to crypto file.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @param	data	data to crypto.
                     * @param	file	crypto file
                     * 							@throw CryptoException.
                     * @param	rft 	The rft.
                     * @param	ukt 	The ukt.
                     * @param	pad 	(optional) the pad.
                     */

                    void cryptoStringToFile ( std::string & data, std::string & file,
                                              ResourceFileType rft,
                                              Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY,
                                              bool pad = true )
                    throw ( Smartmatic::System::Exception::CryptoException );

                    /**
                     * @fn	void Encryption::cryptoStringToFile(std::string & data, std::string & file,
                     * 			Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY,
                     * 			bool pad = true) throw (Smartmatic::System::Exception::CryptoException);
                     * @brief	Crypto string to file.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @param [in,out]	data	The data.
                     * @param [in,out]	file	The file.
                     * @param	ukt							(optional) the ukt.
                     * @param	pad							(optional) the pad.
                     */
                    void cryptoStringToFile ( std::string & data, std::string & file,
                                              Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY,
                                              bool pad = true )
                    throw ( Smartmatic::System::Exception::CryptoException );

                    /**
                     * @fn	void Encryption::cryptoBufferToFile(const Smartmatic::System::SafeBuffer & dataIn,
                     * 			std::string & file, Encryption::ResourceFileType rft,
                     * 			Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY,
                     * 			bool pad = true) throw (Smartmatic::System::Exception::CryptoException);
                     * @brief	Crypto buffer to file.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @param [in,out]	dataIn	The data in.
                     * @param [in,out]	file		The file.
                     * @param	rft								The rft.
                     * @param	ukt								(optional) the ukt.
                     * @param	pad								(optional) the pad.
                     */
                    void cryptoBufferToFile ( const Smartmatic::System::SafeBuffer & dataIn, std::string & file,
                                              Encryption::ResourceFileType rft,
                                              Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY,
                                              bool pad = true )
                    throw ( Smartmatic::System::Exception::CryptoException );

                    /**
                     * @fn	void Encryption::cryptoBufferToFile(const Smartmatic::System::SafeBuffer & dataIn,
                     * 			std::string & file,
                     * 			Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY,
                     * 			bool pad = true) throw (Smartmatic::System::Exception::CryptoException);
                     * @brief	Crypto buffer to file.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @param [in,out]	dataIn	The data in.
                     * @param [in,out]	file		The file.
                     * @param	ukt								(optional) the ukt.
                     * @param	pad								(optional) the pad.
                     */
                    void cryptoBufferToFile ( const Smartmatic::System::SafeBuffer & dataIn, std::string & file,
                                              Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY,
                                              bool pad = true )
                    throw ( Smartmatic::System::Exception::CryptoException );

                    /**
                     * @fn	void Encryption::cryptoBufferToBuffer(const Smartmatic::System::SafeBuffer & dataIn,
                     * 			Smartmatic::System::SafeBuffer & dataOut, Smartmatic::System::SafeBuffer key,
                     * 			bool pad = true) throw (Smartmatic::System::Exception::CryptoException);
                     * @brief	method crypto buffer to buffer.
                     * @author	Yadickson Soto
                     * @date	02/03/2012
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @param	dataIn 	data in.
                     * @param	dataOut	data out.
                     * @param	key		 	key.
                     * @param	pad		 	padding.
                     */
                    void cryptoBufferToBuffer ( const Smartmatic::System::SafeBuffer & dataIn,
                                                Smartmatic::System::SafeBuffer & dataOut,
                                                Smartmatic::System::SafeBuffer key,
                                                bool pad = true )
                    throw ( Smartmatic::System::Exception::CryptoException );

                    /**
                     * @fn	void Encryption::decryptoFileToString(std::string & file, std::string & data,
                     * 			ResourceFileType rft, Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY,
                     * 			bool pad = true) throw (Smartmatic::System::Exception::CryptoException);
                     * @brief	method to decrypto file to string.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @param	file	crypto file.
                     * @param	data	data string.
                     * @param	rft 	The rft.
                     * @param	ukt 	The ukt.
                     * @param	pad 	(optional) the pad.
                     */

                    void decryptoFileToString ( std::string & file, std::string & data,
                                                ResourceFileType rft,
                                                Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY,
                                                bool pad = true )
                    throw ( Smartmatic::System::Exception::CryptoException );


                    /**
                     * @fn	void Encryption::decryptoFileToString(std::string & file, std::string & data,
                     * 			Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY,
                     * 			bool pad = true) throw (Smartmatic::System::Exception::CryptoException);
                     * @brief	Decodes the data obtained from the file and retrieves the result into a string.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * occurs.
                     * @param [in,out]	file	The reference to file name which contained the data
                     * @param [in,out]	data	The reference to string used to retrieve the result.
                     * @param	ukt							(optional) the ukt.
                     * @param	pad							(optional) the pad.
                     */
                    void decryptoFileToString ( std::string & file, std::string & data,
                                                Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY,
                                                bool pad = true )
                    throw ( Smartmatic::System::Exception::CryptoException );

                    /**
                     * @fn	void Encryption::decryptoFileToBuffer(std::string & file,
                     * 			Smartmatic::System::SafeBuffer & dataOut, ResourceFileType rft, Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY,
                     * 			bool pad = true) throw (Smartmatic::System::Exception::CryptoException);
                     * @brief	Decrypto file to buffer.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @param [in,out]	file	 	The file.
                     * @param [in,out]	dataOut	The data out.
                     * @param	rft							 	The rft.
                     * @param	ukt							 	(optional) the ukt.
                     * @param	pad							 	(optional) the pad.
                     */
                    void decryptoFileToBuffer ( std::string & file,
                                                Smartmatic::System::SafeBuffer & dataOut,
                                                ResourceFileType rft,
                                                Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY,
                                                bool pad = true )
                    throw ( Smartmatic::System::Exception::CryptoException );

                    /**
                     * @fn	void Encryption::decryptoFileToBuffer(std::string & file,
                     * 			Smartmatic::System::SafeBuffer & dataOut, Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY,
                     * 			bool pad = true) throw (Smartmatic::System::Exception::CryptoException);
                     * @brief	Decrypto file to buffer.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @param [in,out]	file	 	The file.
                     * @param [in,out]	dataOut	The data out.
                     * @param	ukt							 	(optional) the ukt.
                     * @param	pad							 	(optional) the pad.
                     */
                    void decryptoFileToBuffer ( std::string & file,
                                                Smartmatic::System::SafeBuffer & dataOut,
                                                Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY,
                                                bool pad = true )
                    throw ( Smartmatic::System::Exception::CryptoException );

                    /**
                     * @fn	void Encryption::decryptoBufferToBuffer(const Smartmatic::System::SafeBuffer & dataIn,
                     * 			Smartmatic::System::SafeBuffer & dataOut, Smartmatic::System::SafeBuffer key,
                     * 			bool pad = true) throw (Smartmatic::System::Exception::CryptoException);
                     * @brief	method decrypto buffer to buffer.
                     * @author	Yadickson Soto
                     * @date	02/03/2012
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @param	dataIn 	data in.
                     * @param	dataOut	data out.
                     * @param	key		 	key.
                     * @param	pad		 	padding.
                     */
                    void decryptoBufferToBuffer ( const Smartmatic::System::SafeBuffer & dataIn,
                                                  Smartmatic::System::SafeBuffer & dataOut,
                                                  Smartmatic::System::SafeBuffer key,
                                                  bool pad = true )
                    throw ( Smartmatic::System::Exception::CryptoException );

                    /**
                     * @fn bool validateSignatureCertificate(const Smartmatic::System::SafeBuffer& xmlFileBuffer);
                     * @brief  Validates whether the certificate contained in the given XML file's signature is valid.
                     * @author eduardo.fernandez
                     * @date   04/12/2018
                     * @param  xmlFileBuffer Buffer of the XML file to validate.
                     * @param  rft Resource file type.
                     */
                    void validateSignatureCertificate(
                        const Smartmatic::System::SafeBuffer& xmlFileBuffer,
                        ResourceFileType rft);

                    /**
                     * @fn	void Encryption::validateSignedString(std::string & data,
                     * 			ResourceFileType rft) throw (Smartmatic::System::Exception::CryptoException,
                     * 			Smartmatic::System::Exception::SignedException);
                     * @brief	method to validate DOMElement.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::SignedException	Thrown when a signed error condition
                     * 																														occurs.
                     * @param	data	string data to validate
                     * 							@throw SignedException.
                     * @param	rft 	The rft.
                     */

                    void validateSignedString ( std::string & data, ResourceFileType rft )
                    throw ( Smartmatic::System::Exception::CryptoException,
                            Smartmatic::System::Exception::SignedException );

                    /**
                     * @fn	void Encryption::validateSignedXmlBuffer(Smartmatic::System::SafeBuffer & buffer,
                     * 			ResourceFileType rft) throw (Smartmatic::System::Exception::CryptoException,
                     * 			Smartmatic::System::Exception::SignedException);
                     * @brief	method to validate DOMElement.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::SignedException	Thrown when a signed error condition
                     * 																														occurs.
                     * @param	buffer	document to validate
                     * 						@throw SignedException.
                     * @param	rft	The rft.
                     */

                    void validateSignedXmlBuffer ( Smartmatic::System::SafeBuffer & buffer, ResourceFileType rft )
                    throw ( Smartmatic::System::Exception::CryptoException,
                            Smartmatic::System::Exception::SignedException );

                    /**
                     * @fn	void Encryption::validateSignedFile(std::string & file,
                     * 			ResourceFileType rft) throw (Smartmatic::System::Exception::CryptoException,
                     * 			Smartmatic::System::Exception::SignedException, Smartmatic::System::Exception::XmlException);
                     * @brief	method to validate file.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::SignedException	Thrown when a signed error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::XmlException			Thrown when an XML error condition
                     * 																														occurs.
                     * @param	file	file to validate.
                     * @param	rft 	The rft.
                     */

                    void validateSignedFile ( std::string & file, ResourceFileType rft )
                    throw ( Smartmatic::System::Exception::CryptoException,
                            Smartmatic::System::Exception::SignedException,
                            Smartmatic::System::Exception::XmlException );

                    /**
                     * @fn	std::string Encryption::openXmlFile(std::string & file, ResourceFileType rft,
                     * 			Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY, bool isCompressed = false,
                     * 			bool pad = true) throw (Smartmatic::System::Exception::CryptoException,
                     * 			Smartmatic::System::Exception::SignedException, Smartmatic::System::Exception::XmlException);
                     * @brief	method to open xml file.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::SignedException	Thrown when a signed error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::XmlException			Thrown when an XML error condition
                     * 																														occurs.
                     * @param	file					file to validate.
                     * @param	rft						The rft.
                     * @param	ukt						The ukt.
                     * @param	isCompressed	(optional) the is compressed.
                     * @param	pad						(optional) the pad.
                     * @param   validateChainOfTrust    (optional) whether to validate the chain of trust
                     *                                             when verifying signature.
                     * @return	istream file.
                     */

                    std::string openXmlFile ( std::string & file,
                                              ResourceFileType rft,
                                              Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY,
                                              bool isCompressed = false,
                                              bool pad = true,
                                              bool validateChainOfTrust = true)
                    throw ( Smartmatic::System::Exception::CryptoException,
                            Smartmatic::System::Exception::SignedException,
                            Smartmatic::System::Exception::XmlException );

                    /**
                     * @fn	void Encryption::saveXmlFile(std::string & data, std::string & file, ResourceFileType rft,
                     * 			Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY, bool compress = false,
                     * 			bool pad = true) throw (Smartmatic::System::Exception::CryptoException,
                     * 			Smartmatic::System::Exception::SignedException, Smartmatic::System::Exception::XmlException);
                     * @brief	method to save data to xml file.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::SignedException	Thrown when a signed error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::XmlException			Thrown when an XML error condition
                     * 																														occurs.
                     * @param	data			data to file.
                     * @param	file			save file
                     * 									@throw CryptoException.
                     * @param	rft				The rft.
                     * @param	ukt				The ukt.
                     * @param	compress	(optional) the compress.
                     * @param	pad				(optional) the pad.
                     */

                    void saveXmlFile ( std::string & data, std::string & file,
                                       ResourceFileType rft,
                                       Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY,
                                       bool compress = false,
                                       bool pad = true )
                    throw ( Smartmatic::System::Exception::CryptoException,
                            Smartmatic::System::Exception::SignedException,
                            Smartmatic::System::Exception::XmlException );

                    /**
                     * @fn	static Encryption * Encryption::getInstance();
                     * @brief	Method getter instance class.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @return	instance class.
                     */

                    static Encryption * getInstance();

                    /**
                     * @fn	void Encryption::cryptoBufferToBuffer(Smartmatic::System::SafeBuffer & data,
                     * 			Smartmatic::System::SafeBuffer & dataOut, ResourceFileType rft, Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY,
                     * 			bool pad = true) throw (Smartmatic::System::Exception::CryptoException);
                     * @brief	method to crypto string.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @param	data	 	to crypto.
                     * @param	dataOut	The data out.
                     * @param	rft		 	The rft.
                     * @param	ukt		 	The ukt.
                     * @param	pad		 	(optional) the pad.
                     */

                    void cryptoBufferToBuffer ( Smartmatic::System::SafeBuffer & data,
                                                Smartmatic::System::SafeBuffer & dataOut,
                                                ResourceFileType rft,
                                                Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY,
                                                bool pad = true )
                    throw ( Smartmatic::System::Exception::CryptoException );

                    /**
                     * @fn	void Encryption::cryptoBufferToBuffer(Smartmatic::System::SafeBuffer & data,
                     * 			Smartmatic::System::SafeBuffer & dataOut, Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY,
                     * 			bool pad = true) throw (Smartmatic::System::Exception::CryptoException);
                     * @brief	Crypto buffer to buffer.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @param [in,out]	data	 	The data.
                     * @param [in,out]	dataOut	The data out.
                     * @param	ukt							 	(optional) the ukt.
                     * @param	pad							 	(optional) the pad.
                     */
                    void cryptoBufferToBuffer ( Smartmatic::System::SafeBuffer & data,
                                                Smartmatic::System::SafeBuffer & dataOut,
                                                Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY,
                                                bool pad = true )
                    throw ( Smartmatic::System::Exception::CryptoException );

                    /**
                     * @fn	void Encryption::cryptoStringToBuffer(std::string & data,
                     * 			Smartmatic::System::SafeBuffer & dataOut, ResourceFileType rft, Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY,
                     * 			bool pad = true) throw (Smartmatic::System::Exception::CryptoException);
                     * @brief	method to crypto string.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @param	data	 	to crypto.
                     * @param	dataOut	The data out.
                     * @param	rft		 	The rft.
                     * @param	ukt		 	The ukt.
                     * @param	pad		 	(optional) the pad.
                     */

                    void cryptoStringToBuffer ( std::string & data,
                                                Smartmatic::System::SafeBuffer & dataOut,
                                                ResourceFileType rft,
                                                Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY,
                                                bool pad = true )
                    throw ( Smartmatic::System::Exception::CryptoException );

                    /**
                     * @fn	std::string Encryption::decryptoBufferToString(const Smartmatic::System::SafeBuffer & dataIn,
                     * 			ResourceFileType rft, Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY,
                     * 			bool pad = true) throw (Smartmatic::System::Exception::CryptoException);
                     * @brief	method to decrypto string.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @param	dataIn	decrypto.
                     * @param	rft			The rft.
                     * @param	ukt			The ukt.
                     * @param	pad			(optional) the pad.
                     * @return	data decrypto
                     * 					@throw CryptoException.
                     */

                    std::string decryptoBufferToString ( const Smartmatic::System::SafeBuffer & dataIn,
                                                         ResourceFileType rft,
                                                         Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY,
                                                         bool pad = true )
                    throw ( Smartmatic::System::Exception::CryptoException );

                    /**
                     * @fn	void Encryption::decryptoBufferToBuffer(const Smartmatic::System::SafeBuffer & dataIn,
                     * 			Smartmatic::System::SafeBuffer & dataOut, ResourceFileType rft, Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY,
                     * 			bool pad = true) throw (Smartmatic::System::Exception::CryptoException);
                     * @brief	method to decrypto string.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @param	dataIn					 	decrypto.
                     * @param [in,out]	dataOut	The data out.
                     * @param	rft							 	The rft.
                     * @param	ukt							 	The ukt.
                     * @param	pad							 	(optional) the pad.
                     * ### return	data decrypto
                     * 						@throw CryptoException.
                     */

                    void decryptoBufferToBuffer ( const Smartmatic::System::SafeBuffer & dataIn,
                                                  Smartmatic::System::SafeBuffer & dataOut,
                                                  ResourceFileType rft,
                                                  Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY,
                                                  bool pad = true )
                    throw ( Smartmatic::System::Exception::CryptoException );

                    /**
                     * @fn	void Encryption::decryptoBufferToBuffer(const Smartmatic::System::SafeBuffer & buffer,
                     * 			Smartmatic::System::SafeBuffer & dataOut, Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY,
                     * 			bool pad = true) throw (Smartmatic::System::Exception::CryptoException);
                     * @brief	Decrypto buffer to buffer.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @param [in,out]	buffer 	The buffer.
                     * @param [in,out]	dataOut	The data out.
                     * @param	ukt							 	(optional) the ukt.
                     * @param	pad							 	(optional) the pad.
                     */
                    void decryptoBufferToBuffer ( const Smartmatic::System::SafeBuffer & buffer,
                                                  Smartmatic::System::SafeBuffer & dataOut,
                                                  Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY,
                                                  bool pad = true )
                    throw ( Smartmatic::System::Exception::CryptoException );

                    /**
                     * @fn	std::string Encryption::cryptoStringToB64(std::string & data, ResourceFileType rft,
                     * 			Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_VERIFICATION_KEY,
                     * 			bool pad = true) throw (Smartmatic::System::Exception::CryptoException);
                     * @brief	method to crypto string.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @param	data	to crypto.
                     * @param	rft 	The rft.
                     * @param	ukt 	The ukt.
                     * @param	pad 	(optional) the pad.
                     * @return	data crypto to b64
                     * 					@throw CryptoException.
                     */

                    std::string cryptoStringToB64 ( std::string & data,
                                                    ResourceFileType rft,
                                                    Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_VERIFICATION_KEY,
                                                    bool pad = true )
                    throw ( Smartmatic::System::Exception::CryptoException );

                    /**
                     * @fn	std::string Encryption::cryptoStringToB64(std::string & data,
                     * 			Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_VERIFICATION_KEY,
                     * 			bool pad = true) throw (Smartmatic::System::Exception::CryptoException);
                     * @brief	method to crypto string.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @param	data	to crypto.
                     * @param	ukt 	The ukt.
                     * @param	pad 	(optional) the pad.
                     * @return	data crypto to b64
                     * 					@throw CryptoException.
                     */

                    std::string cryptoStringToB64 ( std::string & data,
                                                    Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_VERIFICATION_KEY,
                                                    bool pad = true )
                    throw ( Smartmatic::System::Exception::CryptoException );

                    /**
                     * @fn	std::string Encryption::decryptoB64ToString(std::string & data, ResourceFileType rft,
                     * 			Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_VERIFICATION_KEY,
                     * 			bool pad = true) throw (Smartmatic::System::Exception::CryptoException);
                     * @brief	method to decrypto string.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @param	data	b64 to decrypto.
                     * @param	rft 	The rft.
                     * @param	ukt 	The ukt.
                     * @param	pad 	(optional) the pad.
                     * @return	data decrypto
                     * 					@throw CryptoException.
                     */

                    std::string decryptoB64ToString ( std::string & data,
                                                      ResourceFileType rft,
                                                      Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_VERIFICATION_KEY,
                                                      bool pad = true )
                    throw ( Smartmatic::System::Exception::CryptoException );

                    /**
                     * @fn	std::string Encryption::decryptoB64ToString(std::string & data,
                     * 			Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_VERIFICATION_KEY,
                     * 			bool pad = true) throw (Smartmatic::System::Exception::CryptoException);
                     * @brief	method to decrypto string.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @param	data	b64 to decrypto.
                     * @param	ukt 	The ukt.
                     * @param	pad 	(optional) the pad.
                     * @return	data decrypto
                     * 					@throw CryptoException.
                     */

                    std::string decryptoB64ToString ( std::string & data,
                                                      Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_VERIFICATION_KEY,
                                                      bool pad = true )
                    throw ( Smartmatic::System::Exception::CryptoException );

                    /**
                     * @fn	void Encryption::decryptoB64ToBuffer(std::string & data,
                     * 			Smartmatic::System::SafeBuffer & dataOut, Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_VERIFICATION_KEY,
                     * 			bool pad = true) throw (Smartmatic::System::Exception::CryptoException);
                     * @brief	Decrypto b 64 to buffer.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @param [in,out]	data	 	The data.
                     * @param [in,out]	dataOut	The data out.
                     * @param	ukt							 	(optional) the ukt.
                     * @param	pad							 	(optional) the pad.
                     */
                    void decryptoB64ToBuffer ( std::string & data,
                                               Smartmatic::System::SafeBuffer & dataOut,
                                               Smartmatic::SAES::Common::Configuration::KeyType ukt = Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_VERIFICATION_KEY,
                                               bool pad = true )
                    throw ( Smartmatic::System::Exception::CryptoException );


                    /**
                     * @fn	void Encryption::SignDataWithPersonalCertificate(const Smartmatic::System::SafeBuffer & dataIn, Smartmatic::System::SafeBuffer & dataOut);
                     * @brief	Sign data with personal certificate.
                     * @author	Yadickson Soto
                     * @date	30/10/2012
                     * 																														occurs.
                     * @param [in,out]	dataIn 	The data in.
                     * @param [in,out]	dataOut	The data out.
                     * @return true if successfully
                     */
                    bool SignDataWithPersonalCertificate ( const Smartmatic::System::SafeBuffer & dataIn, Smartmatic::System::SafeBuffer & dataOut );

                    /**
                     * @fn	bool Encryption::VerifySignatureWithPersonalCertificate(const Smartmatic::System::SafeBuffer & dataIn,
                     * 			Smartmatic::System::SafeBuffer & signature);
                     * @brief	Verify signature with personal certificate.
                     * @author	Yadickson Soto
                     * @date	30/10/2012
                     * 																														occurs.
                     * @param [in,out]	dataIn	 	The data in.
                     * @param [in,out]	signature	The signature.
                     * @return	true if it succeeds, false if it fails.
                     */
                    bool VerifySignatureWithPersonalCertificate ( const Smartmatic::System::SafeBuffer & dataIn, Smartmatic::System::SafeBuffer & signature );

                    /**
                     * @fn	void Encryption::setOpeningCode(Smartmatic::System::GuidClass & openingCode);
                     * @brief	method setter opening code.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	openingCode	opening code.
                     */

                    void setOpeningCode ( Smartmatic::System::GuidClass & openingCode );

                    /**
                     * @fn	const Smartmatic::System::GuidClass & Encryption::getOpeningCode();
                     * @brief	Gets opening code.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @return	The opening code.
                     */
                    const Smartmatic::System::GuidClass & getOpeningCode();

                    /**
                     * @fn	void Encryption::setMachineTime(double machineTime);
                     * @brief	Sets a machine time.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	machineTime	Time of the machine.
                     */

                    void setMachineTime ( double machineTime );

                    /**
                     * @fn	Smartmatic::System::Security::Signed::SignedType Encryption::signedData(ResourceFileType rft);
                     * @brief	Signed data.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	rft	The rft.
                     * @return	.
                     */

                    Smartmatic::System::Security::Signed::SignedType signedData ( ResourceFileType rft );

                    /**
                     * @fn	bool Encryption::validateSigned(ResourceFileType rft);
                     * @brief	Validate signed.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	rft	The rft.
                     * @return	true if it succeeds, false if it fails.
                     */

                    bool validateSigned ( ResourceFileType rft );

                    /**
                     * @fn	bool Encryption::cryptoData(ResourceFileType rft);
                     * @brief	Crypto data.
                     * @author	Cesar.bonilla
                     * @date	4/28/2011
                     * @param	rft	The rft.
                     * @return	true if it succeeds, false if it fails.
                     */

                    bool cryptoData ( ResourceFileType rft );

                    /**
                     * @fn	std::string Encryption::getUseKeyString(Smartmatic::SAES::Common::Configuration::KeyType ukt);
                     * @brief	method getter user key string.
                     * @author	Yadickson Soto
                     * @date	15/02/2012
                     * @param	ukt	use key type.
                     * @return	string use key type.
                     */

                    std::string getUseKeyString ( Smartmatic::SAES::Common::Configuration::KeyType ukt );

                    /**
                     * @fn	std::string Encryption::getSignedSecretKey();
                     * @brief	method getter signed secret key.
                     * @author	Yadickson Soto
                     * @date	02/03/2012
                     * @return	signed secret key.
                     */
                    std::string getSignedSecretKey();

                    /**
                     * @fn  std::string Encryption::getCbcMacKey();
                     * @brief   method getter cbc mac key.
                     * @author  eduardo.fernandez
                     * @date    22/01/2019
                     * @return  cbc mac key.
                     */
                    std::string getCbcMacKey();

                    /**
                     * @fn  std::string Encryption::getRootCAFingerprint(const std::string& rootCAPath);
                     * @brief   method getter root CA fingerprint.
                     * @author  Eduardo Fernández
                     * @date    22/10/2018
                     * @param   rootCAPath root CA file path.
                     * @return  root CA file fingerprint.
                     */
                    std::string getRootCAFingerprint(const std::string& rootCAPath);

                    /**
                     * @fn  int Encryption::getCertificateValidatorFlags(ResourceFileType rft);
                     * @brief   method getter certificate validator flags.
                     * @author  Cesar.bonilla
                     * @date    4/28/2011
                     * @param   rft resource file type.
                     * @return  certificate validator flags.
                     */

                    int getCertificateValidatorFlags ( ResourceFileType rft );

                    /**
                     * @fn  time_t Encryption::getCertificateValidatorTime();
                     * @brief   method getter certificate validator time.
                     * @author  Cesar.bonilla
                     * @date    4/28/2011
                     * @return  certificate validator time.
                     */

                    time_t getCertificateValidatorTime();

                    /**
                     * @brief Validates the issuer certificates involved in the electoral process.
                     * @author eduardo.fernandez
                     * @date   26/10/2018
                     * @param  rootCertificateFingerprint Root certificate fingerprint.
                     * @return true if the validation is successful.
                     */
                    bool validateIssuerCertificates(const std::string& rootCertificateFingerprint);

                private:

                    static Encryption * instance; ///< The instance
                    Smartmatic::System::GuidClass openingCode; ///< The opening code
                    Smartmatic::SAES::Common::Security::SecretKey * m_SecretKey;  ///< The secret key

                    std::string signedSecretKey;	///< The signed secret key
                    std::string cbcMacKey;
                    double machineTime; ///< Time of the machine

                    static Smartmatic::Log::ISMTTLog * logger; ///< The logger
                    Smartmatic::SAES::Runtime::Environment::SaesDirectories * directories;	///< The directories
            };
        }
    }
}

#endif /*__ENCRYPTION__H__*/
