/**
 * @file KeyManager.hxx
 * @brief
 * @date 06/05/2013
 * @author Yadickson Soto
 */

#ifndef KEYMANAGER_HXX_
#define KEYMANAGER_HXX_

#include <System/SafeBuffer.h>
#include <Configuration/key-type-schema.hxx>
#include <System/Security/Crypto.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }
    namespace SAES
    {
        namespace Common
        {
            namespace Security
            {
                /**
                 * @class KeyManager
                 * @brief This class contains all keys for PM and VM Security process
                 * @date 06/05/2013
                 * @author Yadickson Soto
                 */
                class KeyManager
                {

                    private:

                        /**
                         * @fn	KeyManager();
                         * @brief	Constructor, default initialize keySize, modeType and initkey.
                         * @author	Yadickson Soto
                         * @date	06/05/2013
                         */
                        KeyManager();

                        static KeyManager * instance; ///< The pointer reference to KeyManager required to access to the instance.

                    public:

                        /**
                         * @fn	virtual ~KeyManager();
                         * @brief  Finalizes the instance of the KeyManager class.
                         * @author	Yadickson Soto
                         * @date	06/05/2013
                         */
                        virtual ~KeyManager();

                        /**
                         * @fn	void initialize(
                        		const Smartmatic::System::Security::Crypto::KeySize & keySize,
                        		const Smartmatic::System::Security::Crypto::Mode & modeType );
                         * @brief	Initialize property basic for crypto configuration.
                         * @author	Yadickson Soto
                         * @date	06/05/2013
                         * @param 	keySize new crypto key size.
                         * @param	modeType new crypto mode type.
                         */
                        void initialize (
                            const Smartmatic::System::Security::Crypto::KeySize & keySize,
                            const Smartmatic::System::Security::Crypto::Mode & modeType );

                        /**
                         * @fn	unsigned int getKeyLength();
                         * @brief	Gets key length.
                         * @author	Yadickson Soto
                         * @date	06/05/2013
                         * @return	The key length.
                         */
                        unsigned int getKeyLength();

                        /**
                         * @fn	Smartmatic::System::Security::Crypto::KeySize getKeySize();
                         * @brief	Gets a enumerator that represent the key size.
                         * @author	Yadickson Soto
                         * @date	06/05/2013
                         * @return	The key size.
                         */
                        Smartmatic::System::Security::Crypto::KeySize getKeySize();

                        /**
                         * @fn	Smartmatic::System::Security::Crypto::Mode getModeType();
                         * @brief	Gets a enumerator that represent the mode Crypto.
                         * @author	Yadickson Soto
                         * @date	06/05/2013
                         * @return	The mode type.
                         */
                        Smartmatic::System::Security::Crypto::Mode getModeType();

                        /**
                         * @fn	void addKeyByType ( const Smartmatic::System::SafeBuffer & key, const Smartmatic::SAES::Common::Configuration::KeyType & type );
                         * @brief	Add new keys in the maps, this method is called from Encryption Class, create initialization vector from the key
                         * @author	Yadickson Soto
                         * @date	06/05/2013
                         * @param 	key new key
                         * @param	type type of key
                         */
                        void addKeyByType ( const Smartmatic::System::SafeBuffer & key, const Smartmatic::SAES::Common::Configuration::KeyType & type );

                        /**
                         * @fn	bool isKeyByType ( const Smartmatic::SAES::Common::Configuration::KeyType & type );
                         * @brief	Gets whether the type is contained by the key collection.
                         * @author	Yadickson Soto
                         * @date	06/05/2013
                         * @param	type type of key
                         * @return	<tt>true</tt> if key was found; otherwise <tt>false</tt>.
                         */
                        bool isKeyByType ( const Smartmatic::SAES::Common::Configuration::KeyType & type );

                        /**
                         * @fn	void getKey ( const Smartmatic::SAES::Common::Configuration::KeyType & type, Smartmatic::System::SafeBuffer & key );
                         * @brief	Gets the key corresponding to the type.
                         * @author	Yadickson Soto
                         * @date	06/05/2013
                         * @param	type type of key
                         * @param 	key result key
                         */
                        void getKey ( const Smartmatic::SAES::Common::Configuration::KeyType & type, Smartmatic::System::SafeBuffer & key );

                        /**
                         * @fn	static KeyManager * getInstance();
                         * @brief	Gets the only instance of the KeyManager. If this instance does not exist, then it creates a new one.
                         * @author	Yadickson Soto
                         * @date	06/05/2013
                         * @param	externKeyManager External key manager for unit tests.
                         * @return 	The pointer reference to KeyManager required to access to the instance.
                         */
                        static KeyManager * getInstance ( KeyManager* externKeyManager = NULL );

                    protected:

                        /**
                         * @brief  Constructor for unit tests.
                         * @remark No init key will be added.
                         * @author manuel.arguelles
                         * @date   Aug 31, 2017
                         * @param  size The size of the key.
                         * @param  mode The cipher mode.
                         */
                        KeyManager ( Smartmatic::System::Security::Crypto::KeySize size,  Smartmatic::System::Security::Crypto::Mode mode );

                    private:

                        Smartmatic::System::Security::Crypto::KeySize keySize; ///< key size
                        Smartmatic::System::Security::Crypto::Mode modeType; ///< symmetric cipher

                        std::map < Smartmatic::SAES::Common::Configuration::KeyType, Smartmatic::System::SafeBuffer > keysMap; ///< The map index those keys according to its type.
                        static Smartmatic::Log::ISMTTLog * logger; ///< The logger
                };

            } /* namespace Security */
        } /* namespace Common */
    } /* namespace SAES */
} /* namespace Smartmatic */
#endif /* KEYMANAGER_HXX_ */
