#ifndef TEST_SIMULATOR_MONITOR_TEST
#define TEST_SIMULATOR_MONITOR_TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <Operation/Monitor/SimulatorMonitor.hxx>
#include <tests/MockFile.hxx>
#include <tests/StoreMockFile.hxx>

class SimulatorMonitorTest : public CPPUNIT_NS::TestFixture
{
    private:

        typedef Smartmatic::SAES::Common::Operation::Monitor::SimulatorMonitor SimulatorMonitor;
        typedef Smartmatic::SAES::Common::Operation::Monitor::MonitorInfo MonitorInfo;
        typedef Smartmatic::System::SafeBuffer SafeBuffer;
        typedef Smartmatic::System::IO::File File;

    public:

        SimulatorMonitorTest ()
        {

        }

        void setUp()
        {

        }

        void tearDown()
        {

        }

        void testInitialize()
        {
            MockFile file;
            File::getInstance ( &file );

            MOCKPP_CHAINER_FOR_EXT ( MockFile, canOpenFile, ext ) canOpenFile_mock ( &file );

            canOpenFile_mock
            .expects ( MOCKPP_NS::exactly ( 2 ) )
            .will ( MOCKPP_NS::onConsecutiveCalls (
                        MOCKPP_NS::returnValue<bool> ( false ),
                        MOCKPP_NS::returnValue<bool> ( true ) ) );

            SimulatorMonitor monitor ( "eventFile" );

            CPPUNIT_ASSERT ( false == monitor.initialize() );
            CPPUNIT_ASSERT ( true == monitor.initialize() );

            file.verify();
        }

        void testRead_File_Not_Found()
        {
            MockFile file;
            File::getInstance ( &file );

            MOCKPP_CHAINER_FOR_EXT ( MockFile, openFileToBuffer, ext ) openFileToBuffer_mock ( &file );

            openFileToBuffer_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue<bool> ( false ) );

            SimulatorMonitor monitor ( "eventFile" );

            Smartmatic::System::SafeList < MonitorInfo * > list = monitor.searchDevices ();

            CPPUNIT_ASSERT ( true == list.empty() );

            file.verify();
        }

        void testReadFile_Buffer_Empty()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( SafeBuffer (), true ) );

            SimulatorMonitor monitor ( "eventFile" );

            Smartmatic::System::SafeList < MonitorInfo * > list = monitor.searchDevices ();

            CPPUNIT_ASSERT ( true == list.empty() );
            CPPUNIT_ASSERT ( false == file.openFileListValues.empty() );
            CPPUNIT_ASSERT ( "eventFile" == file.openFileListValues.front() );
        }

        void testReadFile_Action_Add()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            SafeBuffer fileBuffer;

            fileBuffer.AddData ( SafeBuffer ( "add," ) );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( fileBuffer, true ) );

            SimulatorMonitor monitor ( "eventFile" );

            Smartmatic::System::SafeList < MonitorInfo * > list = monitor.searchDevices ();
            Smartmatic::System::SafeList < MonitorInfo * >::iterator it;

            CPPUNIT_ASSERT ( false == list.empty() );

            it = list.begin();

            CPPUNIT_ASSERT ( true == ( *it )->isAdded() );

            for ( it = list.begin();
                    it != list.end();
                    ++it )
            {
                delete *it;
            }
        }

        void testReadFile_Action_Remove()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            SafeBuffer fileBuffer;

            fileBuffer.AddData ( SafeBuffer ( "remove," ) );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( fileBuffer, true ) );

            SimulatorMonitor monitor ( "eventFile" );

            Smartmatic::System::SafeList < MonitorInfo * > list = monitor.searchDevices ();
            Smartmatic::System::SafeList < MonitorInfo * >::iterator it;

            CPPUNIT_ASSERT ( false == list.empty() );

            it = list.begin();

            CPPUNIT_ASSERT ( false == ( *it )->isAdded() );

            for ( it = list.begin();
                    it != list.end();
                    ++it )
            {
                delete *it;
            }
        }

        void testReadFile_Action_Add_Node()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            SafeBuffer fileBuffer;

            fileBuffer.AddData ( SafeBuffer ( "add,NODE:/media/usb" ) );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( fileBuffer, true ) );

            SimulatorMonitor monitor ( "eventFile" );

            Smartmatic::System::SafeList < MonitorInfo * > list = monitor.searchDevices ();
            Smartmatic::System::SafeList < MonitorInfo * >::iterator it;

            CPPUNIT_ASSERT ( false == list.empty() );

            it = list.begin();

            CPPUNIT_ASSERT ( true == ( *it )->isAdded() );
            CPPUNIT_ASSERT ( "/media/usb" == ( *it )->getValue ( "NODE" ) );

            for ( it = list.begin();
                    it != list.end();
                    ++it )
            {
                delete *it;
            }
        }

        void testReadFile_Action_Add_Multi_Node()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            SafeBuffer fileBuffer;

            fileBuffer.AddData ( SafeBuffer ( "add,NODE:/media/usb1,TYPE:usb" ) );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( fileBuffer, true ) );

            SimulatorMonitor monitor ( "eventFile" );

            Smartmatic::System::SafeList < MonitorInfo * > list = monitor.searchDevices ();
            Smartmatic::System::SafeList < MonitorInfo * >::iterator it;

            CPPUNIT_ASSERT ( 1 == list.size() );

            it = list.begin();

            CPPUNIT_ASSERT ( true == ( *it )->isAdded() );
            CPPUNIT_ASSERT ( "/media/usb1" == ( *it )->getValue ( "NODE" ) );
            CPPUNIT_ASSERT ( "usb" == ( *it )->getValue ( "TYPE" ) );

            for ( it = list.begin();
                    it != list.end();
                    ++it )
            {
                delete *it;
            }
        }

        void testReadFile_Action_Add_And_Remove_Node()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            SafeBuffer fileBuffer;

            fileBuffer.AddData ( SafeBuffer ( "add,NODE:/media/usb1\nremove,NODE:/media/usb2" ) );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( fileBuffer, true ) );

            SimulatorMonitor monitor ( "eventFile" );

            Smartmatic::System::SafeList < MonitorInfo * > list = monitor.searchDevices ();
            Smartmatic::System::SafeList < MonitorInfo * >::iterator it;

            CPPUNIT_ASSERT ( 2 == list.size() );

            it = list.begin();

            CPPUNIT_ASSERT ( true == ( *it )->isAdded() );
            CPPUNIT_ASSERT ( "/media/usb1" == ( *it )->getValue ( "NODE" ) );

            ++it;

            CPPUNIT_ASSERT ( false == ( *it )->isAdded() );
            CPPUNIT_ASSERT ( "/media/usb2" == ( *it )->getValue ( "NODE" ) );

            for ( it = list.begin();
                    it != list.end();
                    ++it )
            {
                delete *it;
            }
        }

        void testReadFileEvent_Action_Multi_Node()
        {
            StoreMockFile file;
            File::getInstance ( &file );

            SafeBuffer fileBuffer;

            fileBuffer.AddData ( SafeBuffer ( "add,NODE:/media/usb0,TYPE:usb" ) );
            fileBuffer.AddData ( SafeBuffer ( "\n" ) );
            fileBuffer.AddData ( SafeBuffer ( "remove,NODE:/media/usb1,TYPE:usb_interface" ) );

            file.openFileValues.push_back ( StoreMockFile::OpenFileValues ( fileBuffer, true ) );

            SimulatorMonitor monitor ( "eventFile" );

            Smartmatic::System::SafeList < MonitorInfo * > list = monitor.eventDevices();
            Smartmatic::System::SafeList < MonitorInfo * >::iterator it;

            CPPUNIT_ASSERT ( 2 == list.size() );

            it = list.begin();

            CPPUNIT_ASSERT ( true == ( *it )->isAdded() );
            CPPUNIT_ASSERT ( "/media/usb0" == ( *it )->getValue ( "NODE" ) );
            CPPUNIT_ASSERT ( "usb" == ( *it )->getValue ( "TYPE" ) );

            ++it;

            CPPUNIT_ASSERT ( false == ( *it )->isAdded() );
            CPPUNIT_ASSERT ( "/media/usb1" == ( *it )->getValue ( "NODE" ) );
            CPPUNIT_ASSERT ( "usb_interface" == ( *it )->getValue ( "TYPE" ) );

            for ( it = list.begin();
                    it != list.end();
                    ++it )
            {
                delete *it;
            }
        }

        CPPUNIT_TEST_SUITE ( SimulatorMonitorTest );

        CPPUNIT_TEST ( testInitialize );
        CPPUNIT_TEST ( testRead_File_Not_Found );
        CPPUNIT_TEST ( testReadFile_Buffer_Empty );

        CPPUNIT_TEST ( testReadFile_Action_Add );
        CPPUNIT_TEST ( testReadFile_Action_Remove );
        CPPUNIT_TEST ( testReadFile_Action_Add_Node );
        CPPUNIT_TEST ( testReadFile_Action_Add_Multi_Node );
        CPPUNIT_TEST ( testReadFile_Action_Add_And_Remove_Node );

        CPPUNIT_TEST ( testReadFileEvent_Action_Multi_Node );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_SIMULATOR_MONITOR_TEST */
