/**
 * @file	functionality-manager.h
 *
 * @brief	Header file to the functionality manager class.
 */

#ifndef _FUNCTIONALITY_MANAGER_H_
#define _FUNCTIONALITY_MANAGER_H_

#include <map>
#include <list>
#include "Functionalities/functionality.h"
#include <string>

namespace Smartmatic
{
    namespace Functionality
    {
        /**
         * @class	FunctionalityManager
         * @brief	Manager for functionalities class.
         * @author	Rafael.gasperi
         * @date	28/04/2011
         * @tparam	TMenuOptionDescriptor	Type of the menu option descriptor.
         */
        template <class TMenuOptionDescriptor>
        class FunctionalityManager
        {
            protected:

                /**
                 * @typedef	std::map<TMenuOptionDescriptor, Functionality<TMenuOptionDescriptor>* > MapFunctionality
                 * @brief	Defines an alias representing the map functionality.
                 */
                typedef std::map<TMenuOptionDescriptor, Functionality<TMenuOptionDescriptor>* > MapFunctionality;

                MapFunctionality functionalities; ///< The functionalities

                std::list<TMenuOptionDescriptor> descriptors;///< Functionalities to be executed sequentialy

                Functionality<TMenuOptionDescriptor> * pntr; ///< current function

            public:

                /**
                 * @fn	FunctionalityManager::FunctionalityManager();
                 * @brief	Default constructor.
                 * @author	Rafael.gasperi
                 * @date	28/04/2011
                 */
                FunctionalityManager();

                /**
                 * @fn	bool FunctionalityManager::ExecuteOptionSelected(TMenuOptionDescriptor menuDesc);
                 * @brief	Executes the option selected operation.
                 * @author	Rafael.gasperi
                 * @date	28/04/2011
                 * @param	menuDesc	Information describing the menu.
                 * @return	true if it succeeds, false if it fails.
                 * ### exception	e	Thrown when .
                 */
                bool ExecuteOptionSelected ( TMenuOptionDescriptor menuDesc );


                /**
                 * @fn	void FunctionalityManager::ExecuteOptionList(const std::list<TMenuOptionDescriptor>& descriptors);
                 * @brief	Executes a list of options.
                 * @author	Daniel Ampuero
                 * @date	18/09/2012
                 * @param	descriptors	List of options.
                 */
                void ExecuteOptionList ( const std::list<TMenuOptionDescriptor> & descriptors );

                /**
                 * @fn	void AddOptionList ( const std::list<TMenuOptionDescriptor>& descriptors );
                 * @brief	Add new options list to execute.
                 * @author	Yadikckson Soto
                 * @date	24/05/2013
                 * @param	descriptors	List of options.
                 */
                void AddOptionList ( const std::list<TMenuOptionDescriptor> & descriptors );

                /**
                 * @fn	void FunctionalityManager::Register(Functionality<TMenuOptionDescriptor> *funct);
                 * @brief	Registers this object.
                 * @author	Rafael.gasperi
                 * @date	28/04/2011
                 * @param	funct	If non-null, the funct.
                 * ### exception	e	Thrown when .
                 */
                void Register ( Functionality<TMenuOptionDescriptor> * funct );

                /**
                 * @fn	virtual FunctionalityManager::~FunctionalityManager();
                 * @brief	Gets the functionality manager.
                 * @author	Rafael.gasperi
                 * @date	28/04/2011
                 */
                virtual ~FunctionalityManager();

            private:

                bool forceExit; ///< flag force exit
                sigc::signal<void> signalNextFunctionality; ///< signal emit execute next functionality

                /**
                 * @fn	void FunctionalityManager::onExecuteNextFunctionality();
                 * @brief	Raises a Next functionality signal.
                 * @author	Youssef.tourkmani
                 * @date	24/10/2012
                 */
                void onExecuteNextFunctionality();

                /**
                 * @fn	void FunctionalityManager::onExecuteNext();
                 * @brief	Execute the next functionality after the termination of any.
                 * @author	Daniel Ampuero
                 * @date	18/09/2012
                 */
                void onExecuteNext();

                /**
                 * @fn	void FunctionalityManager::ExecuteNext();
                 * @brief	Execute the next functionality after the termination of any.
                 * @author	Daniel Ampuero
                 * @date	18/09/2012
                 */
                void ExecuteNext();

                /**
                 * @fn	void FunctionalityManager::ExecuteNextFunctionality();
                 * @brief	Executes the next functionality operation.
                 * @author	Youssef.tourkmani
                 * @date	24/10/2012
                 */
                void ExecuteNextFunctionality();

        };
    }
}

#include "functionality-manager.hpp"

#endif // _FUNCTIONALITY_MANAGER_H_
