/**
 * @file CardController.cxx
 * @brief Body class card controller
 * @date 15/08/2011
 * @author Yadickson Soto
 */

#include "CardController.hxx"
#include <Security/Passphrase.hxx>
#include <Voting/CardData/card-data-wrapper.h>
#include <Configuration/s-a-e-s-functionality-conf-manager.h>
#include <Card/SmartcardFactory.hxx>
#include <Operation/Devices/SmartcardReaderPowerManagerFactory.hxx>
#include <Card/ISmartcardReaderPowerManager.hxx>
#include <Operation/Security/CryptoSmartcard.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::System;
using namespace Smartmatic::SAES::Card;
using namespace Smartmatic::SAES::Security;
using namespace Smartmatic::SAES::Common::Security;
using namespace Smartmatic::SAES::Functionality::Configuration;
using namespace Smartmatic::SAES::Operation::Security;
using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::SAES::Voting::CardData;
using namespace Smartmatic::Log;

ISMTTLog * CardController::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Operation.Card.CardController", GETTEXT_PACKAGE );
CardController * CardController::instance = NULL;

CardController::CardController ( Smartmatic::SAES::Card::SmartcardInterface * smartcard )
{
    this->smartcard = smartcard;
    this->cryptoCardData = NULL;
}

CardController::~CardController()
{
    if ( smartcard )
    {
        powerCycleConnection.disconnect();
        delete smartcard;
        smartcard = NULL;
    }

    if ( cryptoCardData )
    {
        delete cryptoCardData;
        cryptoCardData = NULL;
    }

    instance = NULL;
}

bool CardController::open ( const std::string & device )
{
    Glib::Mutex::Lock lock ( mutex );

    if (smartcard == NULL)
    {
        ISmartcardReaderPowerManager* powerManager =
            SmartcardReaderPowerManagerFactory::getSmartcardReaderPowerManager();

        powerCycleConnection.disconnect();
        powerCycleConnection = powerManager->signalPowerCycled().connect(
            sigc::mem_fun(*this, &CardController::onPowerCycled));

        smartcard = SmartcardFactory::getSmartcard(powerManager);
    }

    cryptoCardData = ( cryptoCardData == NULL ? new CryptoSmartcard() : cryptoCardData );

    return smartcard->open ( device );
}

bool CardController::isCardPresent()
{
    Glib::Mutex::Lock lock ( mutex );

    return smartcard != NULL && smartcard->isCardPresent();
}

bool CardController::read ( SafeBuffer & buffer, CryptoData & cryptoData )
{
    SafeBuffer bufferRead;
    buffer.Clear();

    return smartcard != NULL && smartcard->read ( bufferRead, cryptoData ) && cryptoData.decrypto ( bufferRead, buffer );
}

bool CardController::write ( SafeBuffer & buffer, CryptoData & cryptoData )
{
    SafeBuffer bufferWrite;

    return smartcard != NULL && cryptoData.crypto ( buffer, bufferWrite ) && smartcard->write ( bufferWrite, cryptoData );
}

bool CardController::test()
{
    return testSelect() && testWriteRead();
}

bool CardController::testSelect()
{
    bool validate = smartcard != NULL && smartcard->test();

    std::stringstream ss;
    ss << "Simple select test " << ( validate ? "success" : "fail" );
    logger->Debug ( ss.str() );

    return validate;
}

bool CardController::testWriteRead()
{
    bool validate = false;

    if ( !cryptoCardData )
    {
        return false;
    }

    SafeBuffer bufferWrite;
    SafeBuffer bufferRead;

    bufferWrite.push_back ( 0x31 );
    bufferWrite.push_back ( 0x32 );
    bufferWrite.push_back ( 0x33 );
    bufferWrite.push_back ( 0x34 );
    bufferWrite.push_back ( 0x35 );
    bufferWrite.push_back ( 0x36 );
    bufferWrite.push_back ( 0x37 );
    bufferWrite.push_back ( 0x38 );

    cryptoCardData->generateKeys();
    cryptoCardData->generatePin();

    if ( write ( bufferWrite, *cryptoCardData ) && read ( bufferRead, *cryptoCardData ) )
    {
        validate = bufferWrite == bufferRead;
    }

    std::stringstream ss;
    ss << "Simple write read test " << ( validate ? "success" : "fail" );
    logger->Debug ( ss.str() );

    return validate;
}

bool CardController::close()
{
    Glib::Mutex::Lock lock ( mutex );

    return smartcard != NULL && smartcard->close();
}

bool CardController::readCardData ( Card_data & cardData )
{
    Glib::Mutex::Lock lock ( mutex );

    if ( !cryptoCardData )
    {
        return false;
    }

    bool validate = false;
    SafeBuffer buffer;

    cryptoCardData->generatePin();

    if ( read ( buffer, *cryptoCardData ) )
    {
        CardDataWrapper::DecompressCardData ( buffer, cardData );
        validate = ! cardData.getCard_type().empty();
    }
    else
    {
        logger->Debug ( "ERROR - read card data" );
    }

    return validate;
}

bool CardController::writeCardData ( Card_data & cardData )
{
    Glib::Mutex::Lock lock ( mutex );

    if ( !cryptoCardData )
    {
        return false;
    }

    bool validate = false;
    SafeBuffer buffer;

    CardDataWrapper::CompressCardData ( cardData, buffer );

    cryptoCardData->generateKeys();
    cryptoCardData->generatePin();

    if ( write ( buffer, *cryptoCardData ) )
    {
        SafeBuffer bufferRead;

        if ( read ( bufferRead, *cryptoCardData ) )
        {
            validate = buffer == bufferRead;
        }
        else
        {
            logger->Debug ( "ERROR - read card data" );
        }
    }
    else
    {
        logger->Debug ( "ERROR - write card data" );
    }

    return validate;
}

CardController * CardController::getInstance ( CardController * externManager )
{
    if ( instance == NULL )
    {
        instance = ( externManager == NULL ? new CardController ( NULL ) : externManager );
    }

    return instance;
}

void CardController::onPowerCycled()
{
    m_powerCycledSignal.emit();
}

CardController::PowerCycledSignal CardController::signalPowerCycled()
{
    return m_powerCycledSignal;
}
