/**
 * @file AlarmPolling.hxx
 * @brief Declaration of the AlarmPolling class.
 * @date 07/06/2015
 * @author Yadickson Soto
 */

#ifndef ALARM_POLLING_HXX_
#define ALARM_POLLING_HXX_

#include <System/SafeThread.hxx>
#include <System/SafeDispatcher.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Operation
        {
            namespace Devices
            {
                /**
                 * @class AlarmPolling
                 * @brief Class alarm controller.
                 * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                 * @date 07/06/2015
                 */
                class AlarmPolling : public Smartmatic::System::SafeThread
                {
                    public:

                        /**
                         * @enum AlarmButtonStatus
                         * @brief Represents the alarm button status.
                         */

                        enum AlarmButtonStatus
                        {
                            PRESSED,
                            RELEASED
                        };

                        typedef sigc::signal<void, AlarmButtonStatus > SignalPressButton; ///< typedef signal press button

                    protected:

                        /**
                         * @fn AlarmPolling::AlarmPolling();
                         * @brief Default constructor for the AlarmPolling class.
                         * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date 07/06/2015
                         */
                        AlarmPolling();

                        /**
                         * @fn	    virtual void preStart ();
                         * @brief	Callback from thread before start thread.
                         * @author	Yadickson Soto
                         * @date	07/06/2015
                         */
                        virtual void preStart ();

                        /**
                         * @fn	virtual bool process ();
                         * @brief	Callback from thread any time.
                         * @author	Yadickson Soto
                         * @date	07/06/2015
                         * @return true to continue process
                         */
                        virtual bool process ();

                        /**
                         * @fn	    virtual void postStop ();
                         * @brief	Callback from thread after stop thread.
                         * @author	Yadickson Soto
                         * @date	07/06/2015
                         */
                        virtual void postStop ();

                        /**
                         * @fn	    void onRawObject ( void * object );
                         * @brief	Callback when emit sync signal.
                         * @author	Yadickson Soto
                         * @date	01/08/2015
                         * @param 	object The object to process
                         */
                        void onRawObject ( void * object );

                    public:

                        /**
                         * @fn virtual AlarmPolling::~AlarmPolling();
                         * @brief Finalizes an instance of the AlarmPolling class.
                         * @author Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                         * @date 07/06/2015
                         */
                        virtual ~AlarmPolling();

                        /**
                         * @fn void pause ();
                         * @brief Pause read device.
                         * @author Yadickson Soto
                         * @date 13/07/2015
                         */
                        void pause ();

                        /**
                         * @fn void resume ();
                         * @brief Resume read device.
                         * @author Yadickson Soto
                         * @date 13/07/2015
                         */
                        void resume ();

                        /**
                         * @fn	SignalPressButton getSignalPressButton ();
                         * @brief	Gets signal press button.
                         * @author	Yadickson Soto
                         * @date	07/06/2015
                         * @return	signal press button.
                         */
                        SignalPressButton getSignalPressButton ();

                        /**
                         * @fn static AlarmPolling * AlarmPolling::getInstance();
                         * @brief Gets the current instance of the AlarmPolling.
                         * @author Yadickson Soto
                         * @date 07/06/2015
                         * @return pointer to the current instance of the AlarmPolling class.
                         */

                        static AlarmPolling * getInstance();

                    private:

                        Smartmatic::System::SafeDispatcher signalAlarmBoxPressButton;   ///< The signal alarm box disconnect
                        SignalPressButton pressButton; ///< The signal press button

                        static AlarmPolling * instance; ///< instance
                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger

                        bool flagPause; ///< The flag pause
                        bool pressed; ///< flag button pressed
                };
            }
        }
    }
}

#endif /* ALARM_POLLING_HXX_ */
