/**
 * @file IDiagnosis.hxx
 * @brief Declaration of the IDiagnosis class.
 * @date 20/10/2011
 * @author Yadickson Soto
 */

#ifndef IDIAGNOSIS_HXX_
#define IDIAGNOSIS_HXX_

#include <Configuration/s-a-e-s-functionality-conf-manager.h>
#include <Operation/MemoryEventManager.hxx>
#include <Voting/Status/voting-machine-status-schema.hxx>
#include <Voting/OperationStatus.hxx>
#include <System/SafeThread.hxx>
#include <System/SafeDispatcher.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }
    namespace SAES
    {
        namespace Operation
        {
            namespace Diagnosis
            {
                /**
                 * @class IDiagnosis
                 * @brief Represents the common properties and operations of a device diagnostic.
                 * @author Yadickson Soto
                 * @date 20/10/2011
                 */
                class IDiagnosis : public Smartmatic::System::SafeThread
                {
                    public:

                        /**
                         * @fn IDiagnosis::IDiagnosis ( Smartmatic::SAES::Common::Configuration::DeviceType type );
                         * @brief Constructor for the IDiagnosis class.
                         * @author Yadickson Soto
                         * @date 20/10/2011
                         * @param type diagnosis type.
                         */
                        explicit IDiagnosis ( Smartmatic::SAES::Common::Configuration::DeviceType type );

                        /**
                         * @fn virtual IDiagnosis::~IDiagnosis();
                         * @brief Finalizes the IDiagnosis class.
                         * @author Yadickson Soto
                         * @date 20/10/2011
                         */
                        virtual ~IDiagnosis();

                        /**
                         * @fn Smartmatic::SAES::Common::Configuration::DeviceType IDiagnosis::getType();
                         * @brief Gets the diagnostics type.
                         * @author Yadickson Soto
                         * @date 20/10/2011
                         * @return diagnostics type.
                         */
                        Smartmatic::SAES::Common::Configuration::DeviceType getType();

                        /**
                         * @fn void IDiagnosis::setMandatory ( bool mandatory );
                         * @brief Marks this diagnostics as mandatory.
                         * @author Yadickson Soto
                         * @date 20/10/2011
                         * @param mandatory true if the diagnostic must be mandatory. Otherwise, false.
                         */
                        void setMandatory ( bool mandatory );

                        /**
                         * @fn bool IDiagnosis::getMandatory();
                         * @brief Verifies if the diagnostic is set as mandatory.
                         * @author Yadickson Soto
                         * @date 20/10/2011
                         * @return true if the diagnostic is mandatory. Otherwise, false.
                         */
                        bool getMandatory();

                        /**
                         * @fn void IDiagnosis::setMessage ( const std::string& message );
                         * @brief Sets the status message of this diagnostic.
                         * @author Yadickson Soto
                         * @date 20/10/2011
                         * @param message new message.
                         */
                        void setMessage ( const std::string& message );

                        /**
                         * @fn std::string IDiagnosis::getMessage();
                         * @brief Gets the status message of this diagnostic.
                         * @author Yadickson Soto
                         * @date 20/10/2011
                         * @return current status message.
                         */
                        std::string getMessage();

                        /**
                         * @fn void IDiagnosis::setMessageFooter ( const std::string& messageFooter );
                         * @brief Sets the footer message for this diagnostic.
                         * @author Yadickson Soto
                         * @date 08/05/2012
                         * @param messageFooter new footer message.
                         */
                        void setMessageFooter ( const std::string& messageFooter );

                        /**
                         * @fn std::string IDiagnosis::getMessageFooter();
                         * @brief Gets the footer message for this diagnostic.
                         * @author Yadickson Soto
                         * @date 08/05/2012
                         * @return current footer message.
                         */
                        std::string getMessageFooter();

                        /**
                         * @fn void IDiagnosis::setTime ( int time );
                         * @brief Sets the timeout in milliseconds for this diagnostic.
                         * @author Yadickson Soto
                         * @date 20/10/2011
                         * @param time timeout in milliseconds for this diagnostic..
                         */
                        void setTime ( int time );

                        /**
                         * @fn int IDiagnosis::getTime();
                         * @brief Gets the timeout in milliseconds for this diagnostic.
                         * @author Yadickson Soto
                         * @date 20/10/2011
                         * @return timeout in milliseconds for this diagnostic.
                         */
                        int getTime();

                        /**
                         * @fn void IDiagnosis::setStatus ( Smartmatic::SAES::Voting::Status::VMStatus status );
                         * @brief Sets the status for this diagnostic.
                         * @author Yadickson Soto
                         * @date 20/10/2011
                         * @param status new status for this diagnostic.
                         */
                        void setStatus ( Smartmatic::SAES::Voting::Status::VMStatus status );

                        /**
                         * @fn Smartmatic::SAES::Voting::Status::VMStatus IDiagnosis::getStatus();
                         * @brief Gets the current status for this diagnostic.
                         * @author Yadickson Soto
                         * @date 20/10/2011
                         * @return current status for this diagnostic.
                         */
                        Smartmatic::SAES::Voting::Status::VMStatus getStatus();

                        /**
                         * @fn virtual void IDiagnosis::updateMessage() = 0;
                         * @brief Updates the status message of this diagnostic.
                         * @author Yadickson Soto
                         * @date 20/10/2011
                         */
                        virtual void updateMessage() = 0;

                        /**
                         * @fn virtual void IDiagnosis::test();
                         * @brief Makes a test run of this diagnostic.
                         * @author Yadickson Soto
                         * @date 20/10/2011
                         */
                        virtual void test();

                        /**
                         * @fn sigc::signal<void> IDiagnosis::getSignalEnd();
                         * @brief Gets the signal that occurs when the diagnostic is finished.
                         * @author Yadickson Soto
                         * @date 20/10/2011
                         * @return signal end.
                         */
                        sigc::signal<void> getSignalEnd();

                    protected:

                        /**
                         * @fn virtual void IDiagnosis::run() = 0;
                         * @brief Runs this diagnostic.
                         * @author Yadickson Soto
                         * @date 28/09/2012
                         */
                        virtual void run() = 0;

                        /**
                         * @fn virtual bool process();
                         * @brief Runs this diagnostic only one time.
                         * @author Yadickson Soto
                         * @date 10/07/2015
                         * @return always return false
                         */
                        virtual bool process();

                        /**
                         * @fn	    virtual void postStop ();
                         * @brief	Callback from thread after stop thread.
                         * @author	Yadickson Soto
                         * @date	08/07/2015
                         */
                        virtual void postStop ();

                        /**
                         * @fn	    void onRawObject ( void * object );
                         * @brief	Callback when emit sync signal. Force the exit of this diagnostic.
                         * @author	Yadickson Soto
                         * @date	01/08/2015
                         * @param 	object The object to process
                         */
                        void onRawObject ( void * object );

                    private:

                        bool mandatory; ///< mandatory
                        int valueTime; ///< timeout

                        std::string message; ///< message status
                        std::string messageFooter; ///< message footer

                        Smartmatic::SAES::Common::Configuration::DeviceType type; ///< type diagnosis
                        Smartmatic::SAES::Voting::Status::VMStatus status; ///< status

                        sigc::signal<void> signalEnd; ///< signal end

                        Smartmatic::System::SafeDispatcher signalDistpacherEnd; ///< signal dispatcher end diagnostic

                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger
                };
            }
        }
    }
}

#endif /* IDIAGNOSIS_HXX_ */
