/**
 * @file PrinterDiagnosis.hxx
 * @brief Declaration of the PrinterDiagnosis class.
 * @author Youssef.tourkmani
 * @date 01/11/2012
 */

#ifndef PRINTERDIAGNOSIS_HXX_
#define PRINTERDIAGNOSIS_HXX_

#include "IDiagnosis.hxx"
#include <Printing/TestPrinting.hxx>
#include <Operation/Devices/PrinterController.hxx>
#include <glibmm.h>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }
    namespace SAES
    {
        namespace Operation
        {
            namespace Diagnosis
            {
                /**
                 * @class PrinterDiagnosis
                 * @brief Represents a printer device diagnostic.
                 * @author Youssef.tourkmani
                 * @date 01/11/2012
                 */

                class PrinterDiagnosis : public IDiagnosis
                {
                    public:

                        /**
                         * @fn PrinterDiagnosis::PrinterDiagnosis ( const std::string& data, bool neop );
                         * @brief Constructor for the PrinterDiagnosis class.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         * @param data     The data.
                         * @param neop true to check neop printer.
                         */

                        PrinterDiagnosis ( const std::string& data, bool neop );

                        /**
                         * @fn virtual PrinterDiagnosis::~PrinterDiagnosis();
                         * @brief Finalizes an instance of the PrinterDiagnosis class.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         */

                        virtual ~PrinterDiagnosis();

                        /**
                         * @fn void setCheckNeop( bool neop );
                         * @brief Constructor for the PrinterDiagnosis class.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         * @param data     The data.
                         * @param neop true to check neop printer.
                         */
                        void setCheckNeop ( bool neop );

                        /**
                         * @fn virtual void IDiagnosis::test();
                         * @brief Makes a test run of this diagnostic.
                         * @author Yadickson Soto
                         * @date 13/07/2015
                         */
                        virtual void test();

                    protected:

                        /**
                         * @fn virtual void PrinterDiagnosis::run();
                         * @brief Runs the printer device diagnostic.
                         * @author Yadickson Soto
                         * @date 28/09/2012
                         */

                        virtual void run();

                        /**
                         * @fn virtual void PrinterDiagnosis::updateMessage();
                         * @brief Updates the status message of this diagnostic.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         */

                        virtual void updateMessage();

                        /**
                         * @fn void PrinterDiagnosis::endPrint();
                         * @brief Signals the end of the print diagnostic successfully.
                         * @author Youssef.tourkmani
                         * @date 01/11/2012
                         */

                        void endPrint();

                        /**
                         * @fn void errorPrint( const std::string& message );
                         * @brief Signals that the print diagnostic finished with errors.
                         * @author Yadickson Soto
                         * @date 09/08/2013
                         * @param message error message.
                         */

                        void errorPrint ( const std::string& message );

                    private:

                        bool neop; ///< true to check neop printer
                        std::string errorMessage; ///< Message describing the error
                        Smartmatic::SAES::Printing::TestPrinting * testPrinter; ///< The test printer
                        Smartmatic::SAES::Operation::Devices::PrinterController * printerController;

                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger

                        std::string m_data;
                };
            }
        }
    }
}

#endif /* PRINTERDIAGNOSIS_HXX_ */
