/**
 * @file VMachineInitiator.cxx
 * @brief Implementation of the VMachineInitiator class.
 * @author Youssef.tourkmani
 * @date 01/11/2012
 */

#include "VMachineInitiator.hxx"
#include <Configuration/s-a-e-s-functionality-conf-manager.h>
#include <Log/SMTTLogManager.h>
#include <Operation/Initiator/CheckDirectoryStructure.hxx>
#include <Operation/Initiator/CheckFilesPresence.hxx>
#include <Operation/Initiator/CheckResourceFiles.hxx>
#include <Operation/MachineOperationManager.hxx>
#include <Operation/Voting/VotingEnqueuer.hxx>
#include <Security/Encryption.hxx>
#include <System/Exception/FileSystemException.hxx>
#include <unistd.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::Functionality::Configuration;
using namespace Smartmatic::SAES::Operation::Initiator;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Operation::Voting;

ISMTTLog * VMachineInitiator::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Operation.Initiator.VMachineInitiator", GETTEXT_PACKAGE );

VMachineInitiator::VMachineInitiator()
{
    dirCondition = new CheckDirectoryStructure ( MachineOperationManager::getInstance()->getSaesDirectories() );
    filesCondition = new CheckFilesPresence ( MachineOperationManager::getInstance()->getSaesDirectories() );
    resourceFiles = new CheckResourceFiles ( MachineOperationManager::getInstance()->getSaesDirectories() );
    addCondition ( dirCondition );
    addCondition ( filesCondition );
    addCondition ( resourceFiles );
    init ( MachineOperationManager::getInstance()->getElectoralConfiguration() );
}

VMachineInitiator::~VMachineInitiator()
{
    delete dirCondition;
    delete filesCondition;
    delete resourceFiles;
}

void VMachineInitiator::preStart()
{
    sleep ( 1 );
    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Initiator.StartVMAchine" ) );
}

bool VMachineInitiator::process()
{
    std::list < ICheckCondition * >::iterator it;
    bool success = true;

    InitializationStatus status;

    for ( it = conditions.begin();
            it != conditions.end();
            ++it )
    {
        ICheckCondition * condition = ( *it );
        status.setMessage ( condition->getMessage() );

        TestStatus::Tests test = condition->check();
        success = success && ( test == TestStatus::Passed );
        status.setStatus ( success ? TestStatus::Passed : TestStatus::Failed );

        addMessage ( status, false );
    }

    success = success && CheckConfigurationLoad();

    removeUnusedAudioVotingVoices();

    if ( success || SAESFunctionalityConfigurationManager::getInstance()->getOperationConfiguration().getDiagnosesConsideredApproved() )
    {
        status.setMessage ( _ ( "Smartmatic.SAES.Operation.Initiator.VMachineInitiator.CheckingConfigurationSuccess" ) );
        status.setStatus ( TestStatus::Passed );
    }
    else
    {
        status.setMessage ( _ ( "Smartmatic.SAES.Operation.Initiator.VMachineInitiator.CheckingConfigurationError" ) );
        status.setStatus ( TestStatus::Failed );
    }

    if ( status.getStatus() == TestStatus::Passed )
    {
        VotingEnqueuer::getInstance()->enableSaveVote ( false );
        VotingEnqueuer::getInstance()->validateID ( false );
    }

    addMessage ( status, true );

    return false;
}

bool VMachineInitiator::checkData()
{
    return true;
}

bool VMachineInitiator::CheckConfigurationLoad()
{
    try
    {
        electoralConfiguration->init ( MachineOperationManager::getInstance()->getSaesDirectories() );
        electoralConfiguration->loadAllData();

        std::string openingDateTime = electoralConfiguration->getVotingDevice()->getFirstVotingDevice().getOperation_modes().getOperation_mode() [0].getOpening_date_time();
        std::string closingDateTime = electoralConfiguration->getVotingDevice()->getFirstVotingDevice().getOperation_modes().getOperation_mode() [0].getClosing_date_time();
        Glib::TimeVal openingTime;
        Glib::TimeVal closingTime;
        openingTime.assign_from_iso8601 ( openingDateTime );
        double openingD = openingTime.as_double();
        closingTime.assign_from_iso8601 ( closingDateTime );
        double closingD = closingTime.as_double();
        Smartmatic::SAES::Security::Encryption::getInstance()->setMachineTime ( openingD + ( openingD + closingD ) / 2 );
        MachineOperationManager::getInstance()->getOperationStatus()->initialize();
        Glib::ustring openingCode = MachineOperationManager::getInstance()->getOperationStatus()->getElectionStatusWrapper()->getOpeningCode();
        Smartmatic::System::GuidClass openCodeGUID = Smartmatic::System::GuidClass::Parse ( openingCode );
        Smartmatic::SAES::Security::Encryption::getInstance()->setOpeningCode ( openCodeGUID );
        return true;
    }
    catch ( ElectionException & ex )
    {
        std::stringstream ss;
        ss << "FATAL " << __func__ << ex.what();
        logger->Debug ( ss.str() );
    }
    catch ( VotingDeviceException & ex )
    {
        std::stringstream ss;
        ss << "FATAL " << __func__ << ex.what();
        logger->Debug ( ss.str() );
    }
    catch ( CryptoException & ex )
    {
        std::stringstream ss;
        ss << "FATAL " << __func__ << ex.what();
        logger->Debug ( ss.str() );
    }
    catch ( SignedException & ex )
    {
        std::stringstream ss;
        ss << "FATAL " << __func__ << ex.what();
        logger->Debug ( ss.str() );
    }
    catch ( XmlException & ex )
    {
        std::stringstream ss;
        ss << "FATAL " << __func__ << ex.what();
        logger->Debug ( ss.str() );
    }
    catch ( GuIdException & ex )
    {
        std::stringstream ss;
        ss << "FATAL " << __func__ << ex.what();
        logger->Debug ( ss.str() );
    }
    catch ( FileSystemException & ex )
    {
        std::stringstream ss;
        ss << "FATAL " << __func__ << ex.what();
        logger->Debug ( ss.str() );
    }
    catch ( std::exception & ex )
    {
        std::stringstream ss;
        ss << "FATAL " << __func__ << ex.what();
        logger->Debug ( ss.str() );
    }
    catch ( ... )
    {
        std::stringstream ss;
        ss << "FATAL " << __func__;
        logger->Debug ( ss.str() );
    }

    return false;
}

void VMachineInitiator::dispose()
{
}

int VMachineInitiator::getSuccessMessageTime()
{
    return Smartmatic::SAES::Functionality::Configuration::SAESFunctionalityConfigurationManager::getInstance()->getSuccesMessageTime();
}

Smartmatic::Log::ISMTTLog * VMachineInitiator::getLogger()
{
    return logger;
}

void VMachineInitiator::removeUnusedAudioVotingVoices()
{
    if (!MachineOperationManager::getInstance()->getAudioVotingCheckUtility().pollingPlaceConfiguredForAudioVoting())
    {
        MachineOperationManager::getInstance()->getAudioVotingCheckUtility().removeVoices();
    }
}
