/**
 * @file	MachineOperationManager.cxx
 * @brief	machine operation manager class.
 * @author	Gustavo Garcia
 * @date	30/01/2017
 */

#include "MachineOperationManager.hxx"
#include <Voting/VotingCenters/VotingCentersWrapper.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Voting/Vote/TransmissionPackageWrapper.hxx>
#include <Voting/Election/ElectionInstalationLanguages.h>
#include <Runtime/Environment/DirectoryManager.hxx>
#include <System/IO/File.h>
#include <System/IO/Path.h>
#include <System/IO/Directory.h>
#include <System/Encoding/Base64Encoder.h>
#include <Configuration/saes-language-controller.h>
#include <Runtime/Environment/MemoryPathController.h>
#include <System/XSLFO/FOPTransformer.hxx>
#include <System/XSLFO/XMLRoffTransformer.h>
#include "PersistSortFile.hxx"
#include "InstallMachine.hxx"
#include <System/Runtime/ProcessLauncher.h>
#include <Runtime/Environment/saes-temporary-directories.h>
#include <Runtime/Environment/SaesProgramUnitDirectories.hxx>
#include <Operation/Initiator/CheckSAESResourceFiles.hxx>
#include <Operation/Initiator/CheckEmlResourceFiles.hxx>
#include <Operation/Initiator/CheckSaesLocalResourceFiles.hxx>
#include <Operation/Monitor/DeviceTypeMonitorPool.hxx>
#include <Operation/LoginKeyManager.hxx>
#include <Voting/VotingResultsWrapper.hxx>
#include <Voting/NationalityTypeManager.hxx>
#include <Voting/ResultValidation.hxx>
#include <Voting/Eml/EmlFileManager.hxx>
#include <Operation/ResultsPackageTransmissionManager.hxx>
#include <Configuration/DeviceTypeMonitorConfigurationManager.hxx>
#include <System/Runtime/ExitProcess.hxx>
#include <Configuration/s-a-e-s-functionality-conf-manager.h>
#include <Operation/Monitor/DeviceTypeMonitorPool.hxx>
#include <Voting/Vote/VoteWrapper.hxx>
#include <Operation/CheckDataManager.hxx>
#include <System/IO/FatFileManager.hxx>
#include <Operation/Voting/VoteCompliance/VoteComplianceVerificator.hxx>
#include <System/Exception/FileSystemException.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Common::Operation::Monitor;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Functionality::Configuration;
using namespace Smartmatic::SAES::Log;
using namespace Smartmatic::SAES::Operation::Initiator;
using namespace Smartmatic::SAES::Operation::Monitor;
using namespace Smartmatic::SAES::Operation::VoteCompliance;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::SAES::Security;
using namespace Smartmatic::SAES::Voting::PBReports;
using namespace Smartmatic::SAES::Voting::Status;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::System::Encoding;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::Runtime;
using namespace Smartmatic::System::Runtime;
using namespace Smartmatic::System::XSLFO;
using namespace Smartmatic::System;

MachineOperationManager * MachineOperationManager::instance = NULL;

ISMTTLog * MachineOperationManager::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Operation.MachineOperationManager", GETTEXT_PACKAGE );

static std::list<std::string> getRunFunctionalityDescriptorsStrings(const std::list<MenuOptionType> &runFunctionalityDescriptors)
{
    std::list<std::string> runFunctionalityDescriptorsStrings;
    std::list<MenuOptionType>::const_iterator it = runFunctionalityDescriptors.begin();

    for (; it != runFunctionalityDescriptors.end(); ++it)
    {
        runFunctionalityDescriptorsStrings.push_back(it->c_str());
    }

    return runFunctionalityDescriptorsStrings;
}

MachineOperationManager::MachineOperationManager()
    : machineInitiator(NULL)
    , authenticator(NULL)
    , electoralConfiguration(NULL)
    , activatedCardStatistics(NULL)
    , saesDirectories(NULL)
    , saesInstallDirectories(NULL)
    , operationStatus(NULL)
    , votingExperienceController(NULL)
    , voteStatistics(NULL)
    , recountVoteStatistics(NULL)
    , basePathDom(NULL)
    , basePathFlash(NULL)
    , pvtransformer(NULL)
    , pvFormWrapper(NULL)
    , stationTallyWrapper(NULL)
    , transmissionPackageWrapper(NULL)
    , emlFileManager(NULL)
    , sortDomFile(NULL)
    , sortFlashFile(NULL)
    , objInstallMachine(NULL)
    , audioVotingExperienceController(NULL)
{
    waitingToCloseElection = false;
}

void MachineOperationManager::initialize()
{
    delete this->authenticator;
    this->authenticator = NULL;

    Encryption::getInstance()->initialize();

    this->electoralConfiguration = new ElectoralConfiguration();
    this->operationStatus = NULL;
    this->pvtransformer = NULL;
    this->saesDirectories = NULL;
    this->votingExperienceController = NULL;
    this->audioVotingExperienceController = NULL;
    this->machineInitiator = NULL;
    this->recountVoteStatistics = new RecountVoteStatistics();

    this->saesDirectories = new Smartmatic::SAES::Runtime::Environment::SaesDirectories();
    std::string markFile = SAESCommonConfigurationManager::getInstance()->getFileSystemVoteConfiguration().getBackupMarkFile();
    bool pesistence = SAESCommonConfigurationManager::getInstance()->getFileSystemVoteConfiguration().getImplementRandom();
    this->sortDomFile = new PersistSortFile ( BasePathType::CONFIGURATION_DOM, markFile, pesistence );
    this->sortFlashFile = new PersistSortFile ( BasePathType::CONFIGURATION_FLASH, markFile, pesistence );
    basePathDom = new LogBasePath ( BasePathType::CONFIGURATION_DOM );
    basePathFlash = new LogBasePath ( BasePathType::CONFIGURATION_FLASH );
    PVFormTransformationType type = SAESFunctionalityConfigurationManager::getInstance()->getPVFormTransformation().getType();

    switch ( type )
    {
        default:
        case PVFormTransformationType::CONFIGURATION_FOP:
            pvtransformer = new FOPTransformer();
            break;

        case PVFormTransformationType::CONFIGURATION_XMLROFF:
            pvtransformer = new XMLRoffTransformer();
            break;
    }

    saesInstallDirectories = NULL;
    StructureInstallationType structType = SAESFunctionalityConfigurationManager::getInstance()->getStructureInstallationType();

    switch ( structType )
    {
        default:
        case StructureInstallationType::CONFIGURATION_SEPARATE:
            saesInstallDirectories = new SaesTemporaryDirectories();
            break;

        case StructureInstallationType::CONFIGURATION_GROUPED:
            saesInstallDirectories = new SaesProgramUnitDirectories();
            break;
    }

    this->objInstallMachine = new InstallMachine ();

    activatedCardStatistics = new ActivatedCardStatisticsWrapper();

    delete voteStatistics;
    voteStatistics = new VoteStatisticsWrapper();

    stationTallyWrapper = new Smartmatic::SAES::Voting::Vote::StationTallyWrapper();
    transmissionPackageWrapper = new Smartmatic::SAES::Voting::Vote::TransmissionPackageWrapper();
    pvFormWrapper = new Smartmatic::SAES::Voting::PBReports::PVFormWrapper();
    std::list<std::string> runFunctionalityDescriptors = getRunFunctionalityDescriptorsStrings(SAESFunctionalityConfigurationManager::getInstance()->getRunFunctionalityDescriptors());
    operationStatus = new OperationStatus ( saesDirectories, runFunctionalityDescriptors );
    emlFileManager = new SAES::Voting::Eml::EmlFileManager();
}

void MachineOperationManager::cleanResources()
{
    sortDomFile = NULL;
    sortFlashFile = NULL;
    basePathDom = NULL;
    basePathFlash = NULL;
    pvtransformer = NULL;
    saesDirectories = NULL;
    saesInstallDirectories = NULL;
    objInstallMachine = NULL;
}

MachineOperationManager::~MachineOperationManager()
{
    if ( sortDomFile )
    {
        delete sortDomFile;
        sortDomFile = NULL;
    }

    if ( sortFlashFile )
    {
        delete sortFlashFile;
        sortFlashFile = NULL;
    }

    if ( basePathDom )
    {
        delete basePathDom;
        basePathDom = NULL;
    }

    if ( basePathFlash )
    {
        delete basePathFlash;
        basePathFlash = NULL;
    }

    if ( pvtransformer )
    {
        delete pvtransformer;
        pvtransformer = NULL;
    }

    if ( saesDirectories )
    {
        delete saesDirectories;
        saesDirectories = NULL;
    }

    if ( saesInstallDirectories )
    {
        delete saesInstallDirectories;
        saesInstallDirectories = NULL;
    }

    if ( objInstallMachine )
    {
        delete objInstallMachine;
        objInstallMachine = NULL;
    }

    delete activatedCardStatistics;

    delete voteStatistics;

    delete stationTallyWrapper;
    delete transmissionPackageWrapper;
    delete pvFormWrapper;
    delete operationStatus;
    delete emlFileManager;
}

bool MachineOperationManager::getIsPersistence()
{
    ConfigurationElectionFilesPath & configElectionFiles ( SAESCommonConfigurationManager::getInstance()->getConfiguration()->getConfigurationElectionFilesPath() );
    ConfigurationBasePath configBasePath = configElectionFiles.getConfigBasePath();
    RulesBasePathType rule = configBasePath.getRulesBasePath();

    if ( rule == RulesBasePathType::CONFIGURATION_VM_RULE )
    {
        return true;
    }

    std::string folderDOM = sortDomFile->getFolder();
    std::string folderFlash = sortFlashFile->getFolder();

    if ( !Directory::getInstance()->DirectoryExists ( folderDOM ) || !Directory::getInstance()->DirectoryExists ( folderFlash ) )
    {
        logger->Debug ( "ERROR - Device Dom or Flash not found, not persistence vote" );
        logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.ErrorPersistenceVote" ) );
        return false;
    }

    return true;
}

bool MachineOperationManager::synchronize ()
{
    bool status = sortDomFile->recoverVotes() && sortFlashFile->recoverVotes();

    LoginKeyManager::getInstance()->write();

    try
    {
        status = status && getMachineInitiator()->checkData();

        if (status)
        {
          saveVoteInOneMemory();
          SMTTLogManager::getInstance()->shutdown();
        }
    }
    catch ( ... )
    {
        status = false;
    }

    ProcessLauncher::getInstance()->RunProcessThroughSystem ( "sync" );

    return status;
}

bool MachineOperationManager::checkData ()
{
    bool status = true;

    try
    {
        restoreVoteInAllMemory();
    }
    catch ( ... )
    {
        status = false;
    }

    return status && getMachineInitiator()->checkData();
}

void MachineOperationManager::setMachineInitiator ( Smartmatic::SAES::Operation::Initiator::IMachineInitiator * machineInitiator )
{
    this->machineInitiator = machineInitiator;
}

Smartmatic::SAES::Operation::Initiator::IMachineInitiator * MachineOperationManager::getMachineInitiator()
{
    return this->machineInitiator;
}

void MachineOperationManager::setAuthenticator ( Smartmatic::SAES::Operation::Authenticator::IAuthenticator * authenticator )
{
    delete this->authenticator;
    this->authenticator = authenticator;
}

Smartmatic::SAES::Operation::Authenticator::IAuthenticator * MachineOperationManager::getAuthenticator()
{
    return this->authenticator;
}

void MachineOperationManager::setElectoralConfiguration ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration )
{
    this->electoralConfiguration = electoralConfiguration;
}

void MachineOperationManager::setActivatedCardStatistics ( Smartmatic::SAES::Voting::PBReports::ActivatedCardStatisticsWrapper * activatedCardStatistics )
{
    this->activatedCardStatistics = activatedCardStatistics;
}

Smartmatic::SAES::Voting::PBReports::ActivatedCardStatisticsWrapper * MachineOperationManager::getActivatedCardStatistics()
{
    return this->activatedCardStatistics;
}

Smartmatic::SAES::Voting::ElectoralConfiguration * MachineOperationManager::getElectoralConfiguration()
{
    return this->electoralConfiguration;
}

Smartmatic::SAES::Runtime::Environment::SaesDirectories * MachineOperationManager::getSaesDirectories()
{
    return saesDirectories;
}

Smartmatic::SAES::Voting::OperationStatus * MachineOperationManager::getOperationStatus()
{
    return operationStatus;
}

void MachineOperationManager::setVotingExperienceController ( Smartmatic::SAES::Operation::Voting::VotingExperienceController * votingExperienceController )
{
    this->votingExperienceController = votingExperienceController;
}

void MachineOperationManager::setAudioVotingExperienceController ( Smartmatic::SAES::Operation::AudioVoting::AudioVotingExperienceController * audioVotingExperienceController)
{
    this->audioVotingExperienceController = audioVotingExperienceController;
}

Smartmatic::SAES::Operation::Voting::VotingExperienceController * MachineOperationManager::getVotingExperienceController()
{
    return votingExperienceController;
}

Smartmatic::SAES::Operation::AudioVoting::AudioVotingExperienceController * MachineOperationManager::getAudioVotingExperienceController()
{
    return audioVotingExperienceController;
}

Smartmatic::SAES::Operation::AudioVoting::AudioVotingCheckUtility  MachineOperationManager::getAudioVotingCheckUtility()
{
    return audioVotingCheckUtility;
}

bool MachineOperationManager::validateIssuerCertificates()
{
    return Encryption::getInstance()->validateIssuerCertificates(electoralConfiguration->getElection()->getElection().getRoot_fp());
}

size_t MachineOperationManager::getNumberOfConnectedFlashDevices() const
{
    const size_t connectedDevices =
        DeviceTypeMonitorPool::getInstance()->getDeviceTypeMonitor(DeviceType::CONFIGURATION_FLASH)->getNumberOfDevices();

    return connectedDevices;
}

bool MachineOperationManager::hasVotes() const
{
    const size_t votes = voteStatistics != NULL ? voteStatistics->getTotalVotes() : 0;

    return votes > 0;
}

void MachineOperationManager::refreshOperationStatus()
{
    getOperationStatus()->initialize();
}

std::string MachineOperationManager::getMachineOperationCode()
{
    std::string code;
    std::string votingDeviceCode = electoralConfiguration->getVotingDevice()->getFirstVotingDevice().getCode();
    code += votingDeviceCode;
    std::string openingCode = getOperationStatus()->getElectionStatusWrapper()->getOpeningCode();
    code += "_" + openingCode;
    std::string pathIdHardware = saesDirectories->getIdHardwareFileFullPath();

    SafeBuffer hardwareId;

    if ( File::getInstance()->openFileToBuffer ( pathIdHardware, hardwareId ) )
    {
        code += "_" + hardwareId.GetBase64Data();
    }
    else
    {
        logger->Debug ( "Hardware id file " + pathIdHardware + " not found." );
    }

    return code;
}

Voting::VotePersistence::PersistenceStatusVote MachineOperationManager::persistenceVote ( Smartmatic::SAES::Voting::Vote::Vote & vote, bool saveVoteOneMemory )
{
    logger->Debug ( "Start persistence vote - OneMemory : " + std::string ( saveVoteOneMemory ? "TRUE" : "FALSE" ) );

    std::string folderDOM = sortDomFile->getFolder();
    std::string folderFlash = sortFlashFile->getFolder();

    if ( !Vote::VoteWrapper::validateVote ( electoralConfiguration, vote ) )
    {
        logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.ErrorValidatingVote" ) );
        throw Smartmatic::System::Exception::XmlException (
            _ ( "Smartmatic.SAES.Operation.MachineOperationManager.ErrorValidatedVote" ),
            N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.ErrorValidatedVote" ), logger );
    }

    if ( !getVoteStatistics()->IsVoteProcessed ( vote ) )
    {
        logger->Debug ( "Start persistence vote file" );

        std::string file = Smartmatic::SAES::Voting::Vote::VoteWrapper::getFileName ( vote );
        std::string fullPathDOM = Path::Combine ( folderDOM, file );
        std::string fullPathFlash = Path::Combine ( folderFlash, file );

        if ( File::getInstance()->canOpenFile ( fullPathDOM ) || File::getInstance()->canOpenFile ( fullPathFlash ) )
        {
            Vote::Vote existVote;
            Vote::VoteWrapper::voteFileToVote ( fullPathDOM, existVote );
            logger->Debug ( "ERROR - vote file " + file + " with code=" + existVote.getCode() + " exist." );
            logger->Debug ( "ERROR - vote file " + file + " with code=" + vote.getCode() + " not persistence." );
            logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.ErrorValidatingVote" ) );
            throw Smartmatic::System::Exception::XmlException (
                _ ( "Smartmatic.SAES.Operation.MachineOperationManager.ErrorValidatedVote" ),
                N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.ErrorValidatedVote" ), logger );
        }

        std::string strVote;
        Smartmatic::SAES::Voting::Vote::VoteWrapper::voteToString ( vote, strVote );

        try
        {
            Encryption::getInstance()->saveXmlFile ( strVote, fullPathDOM, Encryption::EXCHANGE_FILE );
            if(!saveVoteOneMemory)
            	File::getInstance()->copyFile ( fullPathDOM, fullPathFlash );
        }
        catch ( ... )
        {
            logger->Debug ( "FATAL - persistence vote, read vote file " + file );
            File::getInstance()->DeleteFile ( fullPathDOM );
            File::getInstance()->DeleteFile ( fullPathFlash );
            logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.ErrorPersistenceVote" ) );
            throw Smartmatic::System::Exception::XmlException (
                _ ( "Smartmatic.SAES.Operation.MachineOperationManager.ErrorPersistenceVote" ),
                N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.ErrorPersistenceVote" ), logger );
        }

        Vote::Vote vote;

        if ( !Vote::VoteWrapper::voteFileToVote ( fullPathDOM, vote ) ||
                ( !saveVoteOneMemory && !Vote::VoteWrapper::voteFileToVote ( fullPathFlash, vote ) ) )
        {
            logger->Debug ( "FATAL - persistence vote, read vote file " + file );
            File::getInstance()->DeleteFile ( fullPathDOM );
            File::getInstance()->DeleteFile ( fullPathFlash );
            logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.ErrorPersistenceVote" ) );
            throw Smartmatic::System::Exception::XmlException (
                _ ( "Smartmatic.SAES.Operation.MachineOperationManager.ErrorPersistenceVote" ),
                N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.ErrorPersistenceVote" ), logger );
        }

        sortDomFile->addFile ( file );
        sortFlashFile->addFile ( file );
        persistenceFileSystemVote();
        logger->Debug ( "Success persistence vote file" );
        getVoteStatistics()->ProcessVote ( vote );
        getRecountVoteStatistics()->ProcessVote ( vote );
        getVoteStatistics()->persistence();

        hashVoteManager.processVote ( vote );

        switch ( vote.getMode() )
        {
            case Vote::VoteMode::VOTE_O:
                logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.SucessfullPersistenceOfficialVote" ) );
                break;

            case Vote::VoteMode::VOTE_D:
                logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.SucessfullPersistenceDemoVote" ) );
                break;

            case Vote::VoteMode::VOTE_S:
                logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.SucessfullPersistenceSimulateVote" ) );
                break;

            default:
                break;
        }
    }
    else
    {
        getVoteStatistics()->ProcessVote ( vote );
        getRecountVoteStatistics()->ProcessVote ( vote );
        getVoteStatistics()->persistence();

        logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.SucessfullPersistenceDuplicatedVote" ) );

        return Voting::VotePersistence::PERSISTENCE_VOTE_DUPLICATE;
    }

    return Voting::VotePersistence::PERSISTENCE_VOTE_SUCCESS;
}

void MachineOperationManager::deleteVote ( Smartmatic::SAES::Voting::Vote::Vote & vote )
{
    if ( !getVoteStatistics()->IsVoteProcessed ( vote ) )
    {
        std::string file = Smartmatic::SAES::Voting::Vote::VoteWrapper::getFileName ( vote );
        std::string folderDOM = sortDomFile->getFolder();
        std::string folderFlash = sortFlashFile->getFolder();
        std::string fullPathDOM = Path::Combine ( folderDOM, file );
        std::string fullPathFlash = Path::Combine ( folderFlash, file );
        File::getInstance()->DeleteFile ( fullPathDOM );
        File::getInstance()->DeleteFile ( fullPathFlash );
    }
}

void MachineOperationManager::persistenceFileSystemVote()
{
    sortDomFile->persistFiles();
    sortFlashFile->persistFiles();
}

void MachineOperationManager::setVoteStatistics ( Smartmatic::SAES::Voting::PBReports::VoteStatisticsWrapper * voteStatistics )
{
    delete this->voteStatistics;
    this->voteStatistics = voteStatistics;
}

Smartmatic::SAES::Voting::PBReports::VoteStatisticsWrapper * MachineOperationManager::getVoteStatistics()
{
    return this->voteStatistics;
}

Smartmatic::SAES::Voting::RecountVoteStatistics * MachineOperationManager::getRecountVoteStatistics()
{
    return this->recountVoteStatistics;
}

MachineOperationManager * MachineOperationManager::getInstance(MachineOperationManager * externInstance)
{
    if ( instance == NULL || externInstance != NULL)
    {
        instance = (externInstance == NULL ? new MachineOperationManager() : externInstance);
        instance->initialize();
    }

    return instance;
}

bool MachineOperationManager::loadVoteFiles ( bool validate )
{
    logger->Debug ( "Start load file system votes" );

    int deltaP = 0;

    std::string votesFolderDom = sortDomFile->getFolder();
    std::string votesFolderFlash = sortFlashFile->getFolder();

    std::vector<std::string> listDom;
    std::vector<std::string> listFlash;
    std::vector<std::string>::iterator itDom;

    System::IO::Directory::getInstance()->GetFiles ( votesFolderDom, listDom );
    System::IO::Directory::getInstance()->GetFiles ( votesFolderFlash, listFlash );

    if ( validate && !votingIsPossible() && !listDom.empty() )
    {
        logger->Debug ( "ERROR - Check file system votes, exist votes in dom folder" );
        logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.ErrorFoundFolderDomVotes" ) );
        return false;
    }

    if ( validate && !votingIsPossible() && !listFlash.empty() )
    {
        logger->Debug ( "ERROR - Check file system votes, exist votes in flash folder" );
        logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.ErrorFoundFolderFlashVotes" ) );
        return false;
    }

    int officialVotes = getVoteStatistics()->getOfficialVotes();
    int demoVotes = getVoteStatistics()->getDemoVotes();
    int totalVotes = getVoteStatistics()->getTotalVotes();
    int delta = listDom.size() - totalVotes;

    if ( validate && ! ( delta >= 0 && delta <= deltaP ) && VotingCenters::VotingCentersWrapper::getInstance()->getRunningMode () != VotingCenters::RunningMode::VOTING_CENTER_RECOUNT )
    {
        {
            std::stringstream ss;
            ss << "ERROR - Check file system votes, file delta votes > ";
            ss << deltaP << " - Dom files = " << listDom.size() << ", Total votes = " << totalVotes;
            logger->Debug ( ss.str() );
        }
        logger->Audit ( N_ ( "Smartmatic.SAES.Functionality.Operation.MachineOperationManager.ErrorCountVotes" ) );
        return false;
    }

    delta = listDom.size() - listFlash.size();

    if ( validate && ! ( delta >= 0 && delta <= deltaP ) )
    {
        {
            std::stringstream ss;
            ss << "ERROR - Check file system votes, file delta votes > ";
            ss << deltaP << " - Dom files = " << listDom.size() << ", Flash files = " << listFlash.size();
            logger->Debug ( ss.str() );
        }
        logger->Audit ( N_ ( "Smartmatic.SAES.Functionality.Operation.MachineOperationManager.ErrorCountVotes" ) );
        return false;
    }
    else
    {
        getVoteStatistics()->clearCounter();
        getRecountVoteStatistics()->clear();
        hashVoteManager.clear();
    }

    for ( itDom = listDom.begin();
            itDom != listDom.end();
            ++itDom )
    {
        std::string fileDom = Path::Combine ( votesFolderDom, ( *itDom ) );
        std::string fileFlash = Path::Combine ( votesFolderFlash, ( *itDom ) );

        Smartmatic::SAES::Voting::Vote::Vote vote;

        if ( !Vote::VoteWrapper::voteFileToVote ( fileDom, vote ) && ( !validate || !Vote::VoteWrapper::voteFileToVote ( fileFlash, vote ) ) )
        {
            logger->Audit ( N_ ( "Smartmatic.SAES.Functionality.Operation.MachineOperationManager.ErrorVote" ) );
            return false;
        }

        if ( !validate || Vote::VoteWrapper::validateVote ( electoralConfiguration, vote ) )
        {
            std::string fileCode = Smartmatic::SAES::Voting::Vote::VoteWrapper::getFileName ( vote );

            if ( ( *itDom ) == fileCode )
            {
                getVoteStatistics()->ProcessVote ( vote );
                hashVoteManager.processVote ( vote );
                getRecountVoteStatistics()->ProcessVote ( vote );
            }
            else
            {
                logger->Debug ( "VoteCode: " + vote.getCode() + " -> " + fileCode + " (" + *itDom + ")" );
                logger->Audit ( N_ ( "Smartmatic.SAES.Functionality.Operation.MachineOperationManager.ErrorFileNameVote" ) );
                return false;
            }
        }
        else
        {
            logger->Audit ( N_ ( "Smartmatic.SAES.Functionality.Operation.MachineOperationManager.ErrorValidateVote" ) );
            return false;
        }
    }

    if ( validate && VotingCenters::VotingCentersWrapper::getInstance()->getRunningMode () != VotingCenters::RunningMode::VOTING_CENTER_RECOUNT )
    {

        delta = getVoteStatistics()->getOfficialVotes() - officialVotes;

        if ( ! ( delta >= 0 && delta <= deltaP ) )
        {
            std::stringstream ss;
            ss << "OfficialVotesStatistics: " << getVoteStatistics()->getOfficialVotes() << " OfficialVotes: " << officialVotes;
            logger->Debug ( ss.str() );
            logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.ErrorLostOfficialVotes" ) );
            return false;
        }

        delta = getVoteStatistics()->getDemoVotes() - demoVotes;

        if ( ! ( delta >= 0 && delta <= deltaP ) )
        {
            logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.ErrorLostDemoVotes" ) );
            return false;
        }

        delta = getVoteStatistics()->getTotalVotes() - totalVotes;

        if ( ! ( delta >= 0 && delta <= deltaP ) )
        {
            logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.ErrorLostTotalVotes" ) );
            return false;
        }

    }

    try
    {
        if ( validate )
        {
            getVoteStatistics()->persistence();
        }
    }
    catch ( ... )
    {
        logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.ErrorPersistenceVoteStatistics" ) );
        return false;
    }

    logger->Debug ( "Success load file system votes" );
    return true;
}

bool MachineOperationManager::checkVotes ()
{
    logger->Debug ( "Start Check Hash Votes" );

    std::string votesFolderDom = sortDomFile->getFolder();
    std::string votesFolderFlash = sortFlashFile->getFolder();

    std::list < std::string > & list ( hashVoteManager.getFileNameList() );
    std::vector<std::string> listDom;
    std::vector<std::string> listFlash;

    System::IO::Directory::getInstance()->GetFiles ( votesFolderDom, listDom );
    System::IO::Directory::getInstance()->GetFiles ( votesFolderFlash, listFlash );

    if ( listDom.size() != list.size() || listFlash.size() != list.size() )
    {
        std::stringstream ss;
        ss << "ERROR - List votes (" << list.size() << ")";
        ss << " - List Dom votes (" << listDom.size() << ")";
        ss << " - List Flash votes (" << listFlash.size() << ")";
        logger->Debug ( ss.str() );
        return false;
    }

    std::list < std::string >::iterator it;

    std::stringstream ss;
    ss << "List votes (" << list.size() << ")";

    logger->Debug ( ss.str() );

    for ( it = list.begin();
            it != list.end();
            ++it )
    {
        std::string fileDom = Path::Combine ( votesFolderDom, ( *it ) );
        std::string fileFlash = Path::Combine ( votesFolderFlash, ( *it ) );

        if ( ! ( File::getInstance()->compareFiles ( fileDom, fileFlash ) && hashVoteManager.validateVote ( *it ) ) )
        {
            logger->Debug ( "Check Hash Votes Fail" );
            return false;
        }
    }

    try
    {
        getVoteStatistics()->persistence();
    }
    catch ( ... )
    {
        logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.ErrorPersistenceVoteStatistics" ) );
        return false;
    }

    logger->Debug ( "Check Hash Votes Success" );

    return true;
}

void MachineOperationManager::configurationReset()
{
}

bool MachineOperationManager::authentication ( std::string & center, std::string & password, bool centerCaseSensitive )
{
    return objInstallMachine->authentication ( center, password, centerCaseSensitive, saesDirectories, saesInstallDirectories );
}

SaesInstallDirectories * MachineOperationManager::getSaesInstallDirectories()
{
    return saesInstallDirectories;
}

bool MachineOperationManager::install()
{
    return objInstallMachine->install();
}

std::list<Smartmatic::SAES::Voting::Election::Language> MachineOperationManager::getLanguagesElection()
{
    std::list<Smartmatic::SAES::Voting::Election::Language> languages;
    std::list<Smartmatic::SAES::Voting::Election::Language> langs;

    if ( electoralConfiguration == NULL )
    {
        return languages;
    }

    //Initialize the language handling class with the items in the list.
    Election::ElectionInstalationLanguages::getInstance()->clearLanguage();
    Smartmatic::SAES::Voting::Election::Languages::LanguageSequence & sequence ( electoralConfiguration->getLanguages().getLanguage() );
    Smartmatic::SAES::Voting::Election::Languages::LanguageIterator it;

    for ( it = sequence.begin();
            it != sequence.end();
            ++it )
    {
        languages.push_back ( *it );
    }

    std::list<Smartmatic::SAES::Voting::Election::Language>::iterator itl;

    for ( itl = languages.begin();
            itl != languages.end();
            ++itl )
    {
        if ( ( *itl ).getIs_default() )
        {
            langs.push_back ( ( *itl ) );
        }

        ( *itl ).setIs_default ( false );
        Election::ElectionInstalationLanguages::getInstance()->addInstallationLanguages ( ( *itl ) );
    }

    return langs;
}

std::list<Election::Language> MachineOperationManager::getLanguagesInstall()
{
    std::list<Election::Language> languages;
    Election::ElectionInstalationLanguages::getInstance()->ReloadLanguages();
    languages = Election::ElectionInstalationLanguages::getInstance()->getLanguages();
    //Initialize the language handling class with the items in the list.
    Election::ElectionInstalationLanguages::getInstance()->clearLanguage();
    std::list<Election::Language> langs;
    std::list<Election::Language>::iterator itl;

    for ( itl = languages.begin();
            itl != languages.end();
            ++itl )
    {
        if ( ( *itl ).getIs_default() )
        {
            langs.push_back ( ( *itl ) );
        }

        ( *itl ).setIs_default ( false );
        Election::ElectionInstalationLanguages::getInstance()->addInstallationLanguages ( ( *itl ) );
    }

    return langs;
}

void MachineOperationManager::startUp ( bool firstTimeMode )
{
    logger->Debug ( "Start load configuration machine" );

    if ( SAESFunctionalityConfigurationManager::getInstance()->getOperationConfiguration().getConfigurationReset() )
    {
        configurationReset();
    }

    electoralConfiguration->init ( saesDirectories );
    electoralConfiguration->loadAllData();

    std::string openingDateTime =
        electoralConfiguration->getVotingDevice()->getFirstVotingDevice()
        .getOperation_modes().getOperation_mode() [0].getOpening_date_time();

    Glib::TimeVal openingTime;
    openingTime.assign_from_iso8601 ( openingDateTime );

    initializeClass ( saesDirectories, firstTimeMode );

    TimeConfiguration type = SAESCommonConfigurationManager::getInstance()->getFileSystemVoteConfiguration().getImplementTime();

    std::string date;

    switch ( type )
    {
        case TimeConfiguration::CONFIGURATION_CONFIG_TIME:
            date = SAESCommonConfigurationManager::getInstance()->getFileSystemVoteConfiguration().getFileSystemVoteTime();
            break;

        case TimeConfiguration::CONFIGURATION_MACHINE_TIME:
            date = openingTime.as_iso8601();
            break;

        default:
        case TimeConfiguration::CONFIGURATION_NONE:
            date = "";
            break;
    }

    sortDomFile->updateDate ( date );
    sortFlashFile->updateDate ( date );

    logger->Debug ( "Success load configuration machine" );
}

void MachineOperationManager::initializeClass (
    Smartmatic::SAES::Runtime::Environment::SaesDirectories * directories,
    bool firstTimeMode,
    bool persist )
{
    if ( !getActivatedCardStatistics()->init ( directories, electoralConfiguration ) && persist )
    {
        getActivatedCardStatistics()->Persist();
    }

    if ( persist )
    {
        getActivatedCardStatistics()->RestoreFile ( electoralConfiguration );
    }

    if ( !firstTimeMode && persist )
    {
        getOperationStatus()->initialize ();
    }

    static bool starting = true;

    if ( starting )
    {
        starting = false;

        if ( !getVoteStatistics()->init ( directories, electoralConfiguration ) && persist )
        {
            getVoteStatistics()->persistence();
        }

        Smartmatic::System::IO::Directory::getInstance()->Mkdir ( directories->getDocsFolderFullPath() );
        Smartmatic::System::IO::Directory::getInstance()->Mkdir ( directories->getVotesFolderFullPath() );

        getRecountVoteStatistics()->initialize ( electoralConfiguration->getVotingDevice() );

        Glib::ustring openingCode = getOperationStatus()->getElectionStatusWrapper()->getOpeningCode();
        logger->Debug ( "startup opening code " + openingCode );

        Smartmatic::System::GuidClass openCodeGUID = Smartmatic::System::GuidClass::Parse ( openingCode );
        Smartmatic::SAES::Security::Encryption::getInstance()->setOpeningCode ( openCodeGUID );

        if ( persist )
        {
            getOperationStatus()->persistElectionStatus ();
        }

        emlFileManager->init ( electoralConfiguration, directories);
    }

    if ( SAESFunctionalityConfigurationManager::getInstance()->getOperationConfiguration().getStartOpenElection() )
    {
        openElection();
    }

    stationTallyWrapper->init ( electoralConfiguration );

    pvFormWrapper->init (
        directories,
        electoralConfiguration,
        operationStatus,
        activatedCardStatistics,
        voteStatistics,
        stationTallyWrapper,
        transmissionPackageWrapper,
        emlFileManager );

    transmissionPackageWrapper->init ( electoralConfiguration, directories );
    Smartmatic::SAES::Voting::VotingResultsWrapper::getInstance()->init ( electoralConfiguration, directories, voteStatistics, recountVoteStatistics );
    Smartmatic::SAES::Voting::NationalityTypeManager::getInstance()->init ( directories, voteStatistics, recountVoteStatistics, emlFileManager );
}

void MachineOperationManager::createMachineId ( bool force )
{
    std::string code = getMachineOperationCode();
    Smartmatic::SAES::Runtime::Environment::DirectoryManager directorymanager ( *saesDirectories );
    directorymanager.createMachineIdFile ( code, force );
}

bool MachineOperationManager::validateMachineId()
{
    std::string code = getMachineOperationCode();
    Smartmatic::SAES::Runtime::Environment::DirectoryManager directorymanager ( *saesDirectories );
    return directorymanager.validateMachineIdFile ( code );
}

bool MachineOperationManager::checkVoteFiles()
{
    logger->Debug ( "Start check vote files" );

    if ( !sortDomFile->recoverVotes() || !sortFlashFile->recoverVotes() )
    {
        return false;
    }

    bool saveVoteOneMemory = getOperationStatus()->getElectionStatusWrapper()->getSynchronizeVotes();

    std::vector<std::string> listDom;
    std::vector<std::string> listFlash;

    std::string votesFolderDom = sortDomFile->getFolder();
    std::string votesFolderFlash = sortFlashFile->getFolder();

    System::IO::Directory::getInstance()->GetFiles ( votesFolderDom, listDom );
    System::IO::Directory::getInstance()->GetFiles ( votesFolderFlash, listFlash );

    std::vector<std::string> listTempFlash;
    std::vector<std::string>::iterator itDom;
    std::vector<std::string>::iterator itFlash;

    std::stringstream ss1;
    ss1 << "votes dom = " << listDom.size();
    logger->Debug ( ss1.str() );
    std::stringstream ss2;
    ss2 << "votes flash = " << listFlash.size();
    logger->Debug ( ss2.str() );

    listTempFlash = listFlash;

    if ( saveVoteOneMemory )
    {
        logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.RestoreVoteInAllMemory" ) );
    }

    /// validate votes
    for ( itDom = listDom.begin();
            itDom != listDom.end();
            ++itDom )
    {
        for ( itFlash = listFlash.begin();
                itFlash != listFlash.end();
                ++itFlash )
        {
            if ( ( *itFlash ) == ( *itDom ) )
            {
                std::string file = *itDom;
                std::string fileDomFullPath = Path::Combine ( votesFolderDom, ( file ) );
                std::string fileFlashFullPath = Path::Combine ( votesFolderFlash, ( file ) );
                listFlash.erase ( itFlash );

                if ( !File::getInstance()->compareFiles ( fileDomFullPath, fileFlashFullPath ) )
                {
                    Vote::Vote voteDom;
                    Vote::Vote voteFlash;
                    bool successDom = Vote::VoteWrapper::voteFileToVote ( fileDomFullPath, voteDom );
                    bool successFlash = Vote::VoteWrapper::voteFileToVote ( fileFlashFullPath, voteFlash );

                    if ( successDom && !successFlash )
                    {
                        logger->Debug ( "Restore file vote dom to flash " + file );
                        File::getInstance()->copyFile ( fileDomFullPath, fileFlashFullPath );
                    }
                    else if ( !successDom && successFlash )
                    {
                        logger->Debug ( "Restore file vote flash to dom " + file );
                        File::getInstance()->copyFile ( fileFlashFullPath, fileDomFullPath );
                    }
                    else if ( successDom && successFlash )
                    {
                        logger->Error ( "Error file dom and flash " + file );
                        logger->Debug ( "Vote code in dom " + voteDom.getCode() );
                        logger->Debug ( "Vote code in flash " + voteFlash.getCode() );

                        std::string fileDom = Smartmatic::SAES::Voting::Vote::VoteWrapper::getFileName ( voteDom );
                        std::string fileFlash = Smartmatic::SAES::Voting::Vote::VoteWrapper::getFileName ( voteFlash );

                        logger->Debug ( "Vote file in dom " + fileDom );
                        logger->Debug ( "Vote file in flash " + fileFlash );

                        return false;
                    }
                    else
                    {
                        logger->Error ( "Error structure file dom and flash " + file );
                        return false;
                    }
                }

                break;
            }
        }
    }

    // clear all file
    for ( itFlash = listTempFlash.begin();
            itFlash != listTempFlash.end();
            ++itFlash )
    {
        for ( itDom = listDom.begin();
                itDom != listDom.end();
                ++itDom )
        {
            if ( ( *itFlash ) == ( *itDom ) )
            {
                listDom.erase ( itDom );
                break;
            }
        }
    }

    /// copy file vote to flash
    for ( itDom = listDom.begin();
            itDom != listDom.end();
            ++itDom )
    {
        std::string fileName = *itDom;
        std::string fileDomFullPath = Path::Combine ( votesFolderDom, fileName );
        std::string fileFlashFullPath = Path::Combine ( votesFolderFlash, fileName );
        File::getInstance()->copyFile ( fileDomFullPath, fileFlashFullPath );
        logger->Debug ( "Restore file vote to flash " + fileName );
    }

    /// copy file vote to dom
    for ( itFlash = listFlash.begin();
            itFlash != listFlash.end();
            ++itFlash )
    {
        std::string fileName = *itFlash;
        std::string fileDomFullPath = Path::Combine ( votesFolderDom, fileName );
        std::string fileFlashFullPath = Path::Combine ( votesFolderFlash, fileName );
        File::getInstance()->copyFile ( fileFlashFullPath, fileDomFullPath );
        logger->Debug ( "Restore file vote to dom " + fileName );
    }

    try
    {
        if ( saveVoteOneMemory )
        {
            restoreVoteInAllMemory();
        }
    }
    catch ( ... )
    {
        return false;
    }

    logger->Debug ( "End check vote files" );

    return true;
}

bool MachineOperationManager::checkPVReport()
{
    bool validate = false;

    if ( isCloseElection() )
    {
        validate = pvFormWrapper->validate();
    }
    else
    {
        validate = true;
    }

    return validate;
}

bool MachineOperationManager::checkTransmissionPackages()
{
    bool validate = false;

    if ( isCloseElection() )
    {
        validate = transmissionPackageWrapper->validate() && emlFileManager->validate();
    }
    else
    {
        validate = true;
    }

    return validate;
}

bool MachineOperationManager::checkVotingResultFiles()
{
    bool validate = false;

    if ( isCloseElection() )
    {
        validate = Smartmatic::SAES::Voting::VotingResultsWrapper::getInstance()->validate() &&
                   Smartmatic::SAES::Voting::NationalityTypeManager::getInstance()->validate();
    }
    else
    {
        validate = true;
    }

    return validate;
}

bool MachineOperationManager::checkNationalitiesFile()
{
    std::string dom = saesDirectories->getNationalityDefinitionTypeFileFullPath( BasePathType::CONFIGURATION_DOM );
    std::string flash = saesDirectories->getNationalityDefinitionTypeFileFullPath( BasePathType::CONFIGURATION_FLASH );
    return File::getInstance()->compareFiles( dom, flash );
}

bool MachineOperationManager::checkMachineId()
{
    std::string dom = saesDirectories->getBackupFileFullPath ( BasePathType::CONFIGURATION_DOM );
    std::string flash = saesDirectories->getBackupFileFullPath ( BasePathType::CONFIGURATION_FLASH );
    return File::getInstance()->compareFiles ( dom, flash );
}

bool MachineOperationManager::checkVotingCentersFiles()
{
    std::string vdom = saesDirectories->getVotingCentersInstalationFileFullPath ( BasePathType::CONFIGURATION_DOM );
    std::string vflash = saesDirectories->getVotingCentersInstalationFileFullPath ( BasePathType::CONFIGURATION_FLASH );
    return File::getInstance()->compareFiles ( vdom, vflash );
}

bool MachineOperationManager::checkSAESLocal()
{
    std::string langDom = saesDirectories->getLanguageLocalFileFullPath(BasePathType::CONFIGURATION_DOM);
    std::string langFlash = saesDirectories->getLanguageLocalFileFullPath(BasePathType::CONFIGURATION_FLASH);
    std::string nationalityCodeDom = saesDirectories->getNationalityDefinitionTypeFileFullPath(
        BasePathType::CONFIGURATION_DOM);
    std::string nationalityCodeFlash = saesDirectories->getNationalityDefinitionTypeFileFullPath(
        BasePathType::CONFIGURATION_FLASH);

    return File::getInstance()->compareFiles(langDom, langFlash) &&
        File::getInstance()->compareFiles(nationalityCodeDom, nationalityCodeFlash);
}

bool MachineOperationManager::checkVoteStatistics()
{
    std::string dom = saesDirectories->getVoteStatisticsFileFullPath ( BasePathType::CONFIGURATION_DOM );
    std::string flash = saesDirectories->getVoteStatisticsFileFullPath ( BasePathType::CONFIGURATION_FLASH );
    return File::getInstance()->canOpenFile ( dom ) && File::getInstance()->canOpenFile ( flash );
}

bool MachineOperationManager::checkElectionStatus()
{
    std::string dom = saesDirectories->getElectionStatusFileFullPath ( BasePathType::CONFIGURATION_DOM );
    std::string flash = saesDirectories->getElectionStatusFileFullPath ( BasePathType::CONFIGURATION_FLASH );
    return File::getInstance()->canOpenFile ( dom ) && File::getInstance()->canOpenFile ( flash );
}

bool MachineOperationManager::checkElection()
{
    std::string dom = saesDirectories->getElectionFileFullPath ( BasePathType::CONFIGURATION_DOM );
    std::string flash = saesDirectories->getElectionFileFullPath ( BasePathType::CONFIGURATION_FLASH );
    return File::getInstance()->compareFiles ( dom, flash );
}

bool MachineOperationManager::checkDeviceParameters()
{
    std::string dom = saesDirectories->getDeviceParametersFileFullPath ( BasePathType::CONFIGURATION_DOM );
    std::string flash = saesDirectories->getDeviceParametersFileFullPath ( BasePathType::CONFIGURATION_FLASH );
    return File::getInstance()->compareFiles ( dom, flash );
}

bool MachineOperationManager::checkVotingDevices()
{
    std::string dom = saesDirectories->getVotingDeviceFileFullPath ( BasePathType::CONFIGURATION_DOM );
    std::string flash = saesDirectories->getVotingDeviceFileFullPath ( BasePathType::CONFIGURATION_FLASH );
    return File::getInstance()->compareFiles ( dom, flash );
}

bool MachineOperationManager::checkActivatedCardStatistics()
{
    std::string dom = saesDirectories->getActivatedCardStatisticsFileFullPath ( BasePathType::CONFIGURATION_DOM );
    std::string flash = saesDirectories->getActivatedCardStatisticsFileFullPath ( BasePathType::CONFIGURATION_FLASH );
    return File::getInstance()->canOpenFile ( dom ) && File::getInstance()->canOpenFile ( flash );
}

bool MachineOperationManager::checkSignature()
{
    std::string dom = saesDirectories->getSignatureResourceFileFullPath ( BasePathType::CONFIGURATION_DOM );
    std::string flash = saesDirectories->getSignatureResourceFileFullPath ( BasePathType::CONFIGURATION_FLASH );
    return File::getInstance()->canOpenFile ( dom ) && File::getInstance()->canOpenFile ( flash );
}

bool MachineOperationManager::checkFatFiles()
{
    bool validate = true;;

    if ( isCloseElection() )
    {
        validate = verifyFilesInFat();
    }

    if (!validate)
    {
        try
        {
            reporFatExportError(N_("Error verifying files in fat partition."));
            validate = true;
        }
        catch (const Smartmatic::System::Exception::FileSystemException&)
        {
        }
    }

    return validate;
}

bool MachineOperationManager::validateSAESSignature()
{
    bool success = true;

    CheckSAESResourceFiles checkDOMSAESResourceFiles(BasePathType::CONFIGURATION_DOM);
    CheckSAESResourceFiles checkFLASHSAESResourceFiles(BasePathType::CONFIGURATION_FLASH);

    std::string domSignature = saesDirectories->getSignatureRootFileFullPath(BasePathType::CONFIGURATION_DOM);
    std::string flashSignature = saesDirectories->getSignatureRootFileFullPath(BasePathType::CONFIGURATION_FLASH);

    if (!File::getInstance()->compareFiles(domSignature, flashSignature) ||
        checkDOMSAESResourceFiles.check() == TestStatus::Failed ||
        checkFLASHSAESResourceFiles.check() == TestStatus::Failed)
    {
        success = false;
    }

    return success;
}

bool MachineOperationManager::validateEmlSignature()
{
    bool success = true;

    CheckEmlResourceFiles checkDOMEmlResourceFiles ( BasePathType::CONFIGURATION_DOM );
    CheckEmlResourceFiles checkFLASHEmlResourceFiles ( BasePathType::CONFIGURATION_FLASH );

    if (checkDOMEmlResourceFiles.check() == TestStatus::Failed ||
        checkFLASHEmlResourceFiles.check() == TestStatus::Failed)
    {
        success = false;
    }

    return success;
}

bool MachineOperationManager::validateSaesLocalSignature()
{
    bool success = true;

    CheckSaesLocalResourceFiles checkDomSaesLocal(BasePathType::CONFIGURATION_DOM);
    CheckSaesLocalResourceFiles checkFlashSaesLocal(BasePathType::CONFIGURATION_FLASH);

    if (checkDomSaesLocal.check() == TestStatus::Failed ||
        checkFlashSaesLocal.check() == TestStatus::Failed)
    {
        success = false;
    }

    return success;
}

void MachineOperationManager::installMachine()
{
    if ( isNotInstalled() )
    {
        logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.StatingInstallMachine" ) );
        logger->Debug ( "Start install machine" );
        Smartmatic::SAES::Voting::Status::ProcessStatus nextStatus = SAESFunctionalityConfigurationManager::getInstance()->getNextProcessStatus ( getOperationStatus()->getElectionStatusWrapper()->getEletionState() );
        getOperationStatus()->getElectionStatusWrapper()->setEletionState ( nextStatus );
        getOperationStatus()->persistElectionStatus();
        logger->Debug ( "Success install machine" );
        logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.MachineInstalled" ) );
    }
}

void MachineOperationManager::skipInstallMachine()
{
    if ( isNotInstalled() )
    {
        logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.SkipInstallMachine" ) );
        logger->Debug ( "Skip install machine" );
        Smartmatic::SAES::Voting::Status::ProcessStatus nextStatus = SAESFunctionalityConfigurationManager::getInstance()->getNextProcessStatus ( getOperationStatus()->getElectionStatusWrapper()->getEletionState() );
        getOperationStatus()->getElectionStatusWrapper()->setEletionState ( nextStatus );
        getOperationStatus()->persistElectionStatus();
        logger->Debug ( "Success skip install machine" );
        logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.MachineNotInstalled" ) );
    }
}

void MachineOperationManager::demostrationVote()
{
    if ( isNotDemostrationVote() )
    {
        logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.StatingDemostrationVote" ) );
        logger->Debug ( "Start demostration vote" );
        Smartmatic::SAES::Voting::Status::ProcessStatus nextStatus = SAESFunctionalityConfigurationManager::getInstance()->getNextProcessStatus ( getOperationStatus()->getElectionStatusWrapper()->getEletionState() );
        getOperationStatus()->getElectionStatusWrapper()->setEletionState ( nextStatus );
        getOperationStatus()->persistElectionStatus();
        logger->Debug ( "Success demostration vote" );
        logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.DemostrationVote" ) );
    }
}

void MachineOperationManager::skipDemostrationVote()
{
    if ( isNotDemostrationVote() )
    {
        logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.SkipDemostrationVote" ) );
        logger->Debug ( "Skip demostration vote" );
        Smartmatic::SAES::Voting::Status::ProcessStatus nextStatus = SAESFunctionalityConfigurationManager::getInstance()->getNextProcessStatus ( getOperationStatus()->getElectionStatusWrapper()->getEletionState() );
        getOperationStatus()->getElectionStatusWrapper()->setEletionState ( nextStatus );
        getOperationStatus()->persistElectionStatus();
        logger->Debug ( "Success demostration vote" );
        logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.NotDemostrationVote" ) );
    }
}

void MachineOperationManager::openElection()
{
    if ( isNotInitiatedElection() )
    {
        logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.StatingOpenElection" ) );
        logger->Debug ( "Start open election" );
        Glib::TimeVal now;
        now.assign_current_time();
        getOperationStatus()->getElectionStatusWrapper()->setOpenElectionDate ( now.as_iso8601() );
        Smartmatic::SAES::Voting::Status::ProcessStatus nextStatus = SAESFunctionalityConfigurationManager::getInstance()->getNextProcessStatus ( getOperationStatus()->getElectionStatusWrapper()->getEletionState() );
        getOperationStatus()->getElectionStatusWrapper()->setEletionState ( nextStatus );
        getOperationStatus()->persistElectionStatus();
        logger->Debug ( "Success open election" );
        logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.ElectionOpened" ) );
    }
}

void MachineOperationManager::startClosingElection()
{
    const RunningMode runningMode   = VotingCenters::VotingCentersWrapper::getInstance()->getRunningMode ();
    const bool        isRecountMode = runningMode == RunningMode::VOTING_CENTER_RECOUNT;

    if ( ( isOpenElection() ) || ( ( isRecountMode ) && ( isNotInstalled() ) ) )
    {
        logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.StartClosingElection" ) );
        logger->Debug ( "Start closing election" );

        ProcessStatus current = getOperationStatus()->getElectionStatusWrapper()->getEletionState();

        if ( isRecountMode )
        {
            current = ProcessStatus ( ProcessStatus::ELECTION_STATUS_INITIATED );
        }

        Smartmatic::SAES::Voting::Status::ProcessStatus nextStatus = SAESFunctionalityConfigurationManager::getInstance()->getNextProcessStatus ( current );
        getOperationStatus()->getElectionStatusWrapper()->setEletionState ( nextStatus );
        getOperationStatus()->persistElectionStatus();
        logger->Debug ( "Success start closing election" );
        logger->Audit ( N_ ( "Start of election closing" ) );
    }
}

void MachineOperationManager::closeElection()
{
    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.StartingCloseElection" ) );
    logger->Debug ( "Start close election" );
    persistenceFileSystemVote();
    Glib::TimeVal now;
    now.assign_current_time();

    getOperationStatus()->getElectionStatusWrapper()->setClosedElection ( true );
    getOperationStatus()->getElectionStatusWrapper()->setCloseElectionDate ( now.as_iso8601() );

    Smartmatic::SAES::Voting::Status::ProcessStatus current = getOperationStatus()->getElectionStatusWrapper()->getEletionState();
    Smartmatic::SAES::Voting::Status::ProcessStatus nextStatus = SAESFunctionalityConfigurationManager::getInstance()->getNextProcessStatus ( current );
    getOperationStatus()->getElectionStatusWrapper()->setEletionState ( nextStatus );
    getOperationStatus()->persistElectionStatus();
    std::string folderDom = saesDirectories->getBasePath ( BasePathType::CONFIGURATION_DOM );
    std::string folderFlash = saesDirectories->getBasePath ( BasePathType::CONFIGURATION_FLASH );

    Smartmatic::System::IO::Directory::getInstance()->addAllPermission ( folderDom );
    Smartmatic::System::IO::Directory::getInstance()->addAllPermission ( folderFlash );

    logger->Debug ( "Success close election" );
    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.ElectionClosed" ) );
}

void MachineOperationManager::loadStationTally()
{
    logger->Debug ( "Start load station tally info" );
    logger->Debug ( "Success load station tally info" );
}

void MachineOperationManager::persistTransmissionPackage()
{
    logger->Debug ( "Start persistence transmission package" );

    transmissionPackageWrapper->deleteVotes();
    stationTallyWrapper->initialize ();

    std::string votesFolderDom = sortDomFile->getFolder();
    std::string votesFolderFlash = sortFlashFile->getFolder();

    std::list<std::string> & listDom ( hashVoteManager.getFileNameList() );
    std::list<std::string>::iterator itDom;

    for ( itDom = listDom.begin();
            itDom != listDom.end();
            ++itDom )
    {
        std::string fileDom = Path::Combine ( votesFolderDom, ( *itDom ) );
        std::string fileFlash = Path::Combine ( votesFolderFlash, ( *itDom ) );
        Smartmatic::SAES::Voting::Vote::Vote vote;

        if ( !Vote::VoteWrapper::voteFileToVote ( fileDom, vote ) &&
                !Vote::VoteWrapper::voteFileToVote ( fileFlash, vote ) )
        {
            logger->Audit ( N_ ( "Smartmatic.SAES.Functionality.Operation.MachineOperationManager.ErrorVote" ) );

            transmissionPackageWrapper->deleteVotes();

            throw Smartmatic::System::Exception::XmlException (
                _ ( "Smartmatic.SAES.Functionality.Operation.MachineOperationManager.ErrorVote" ),
                N_ ( "Smartmatic.SAES.Functionality.Operation.MachineOperationManager.ErrorVote" ),
                logger );
        }
        else
        {
            transmissionPackageWrapper->addVote ( vote );
            stationTallyWrapper->update ( vote );
            emlFileManager->updateCounters(vote);
        }
    }

    transmissionPackageWrapper->persistTransmissionPackage(stationTallyWrapper->getStationTally());

    logger->Debug ( "Success persistence transmission package" );
}

void MachineOperationManager::persistVotingResultsFile()
{
    logger->Debug ( "Start persistence voting results file" );
    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.StartPersistVotingResultsFile" ) );

    Smartmatic::SAES::Voting::VotingResultsWrapper::getInstance()->persistence();
    Smartmatic::SAES::Voting::NationalityTypeManager::getInstance()->persistence();

    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.PersistVotingResultsFileSuccessful" ) );
}

void MachineOperationManager::persistPVForm()
{
    logger->Debug ( "Start persistence pv report data" );
    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.StartingPersistPVForm" ) );

    if ( !pvFormWrapper->persistencePVForm ( pvtransformer ) )
    {
        logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.ErrorPersistPVForm" ) );
        throw Smartmatic::System::Exception::XmlException (
            _ ( "Smartmatic.SAES.Operation.MachineOperationManager.ErrorPersistPVForm" ),
            N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.ErrorPersistPVForm" ),
            logger );
    }

    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.PersistPVFormSuccess" ) );
    logger->Debug ( "Success persistence pv report data" );
}

void MachineOperationManager::closeVotingExperience()
{
    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.StartingCloseVotingExperience" ) );
    logger->Debug ( "Start close voting experience" );
    Glib::TimeVal now;
    now.assign_current_time();
    getOperationStatus()->getElectionStatusWrapper()->setCloseVotingDate ( now.as_iso8601() );
    Smartmatic::SAES::Voting::Status::ProcessStatus nextStatus = SAESFunctionalityConfigurationManager::getInstance()->getNextProcessStatus ( getOperationStatus()->getElectionStatusWrapper()->getEletionState() );
    getOperationStatus()->getElectionStatusWrapper()->setEletionState ( nextStatus );
    getOperationStatus()->persistElectionStatus();
    logger->Debug ( "Success close voting experience" );
    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.VotingExperienceClosed" ) );
}

bool MachineOperationManager::isNotInstalled()
{
    return getOperationStatus()->getElectionState() ==
           Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_NOTINSTALLED;
}

bool MachineOperationManager::isNotDemostrationVote()
{
    return getOperationStatus()->getElectionState() ==
           Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_NOTDEMOSTRATIONVOTE;
}

bool MachineOperationManager::isOpenElection()
{
    return getOperationStatus()->getElectionState() ==
           Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_INITIATED;
}

bool MachineOperationManager::isNotOpenElection()
{
    return getOperationStatus()->getElectionState() ==
           Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_NOTINITIATED;
}

bool MachineOperationManager::votingIsPossible()
{
    ElectionStatusWrapper * status = getOperationStatus()->getElectionStatusWrapper();

    std::stringstream ss;
    ss << "getInstance election state is " << status->getEletionState();
    logger->Debug ( ss.str() );

    bool validate = 	status->getEletionState() == Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_NOTDEMOSTRATIONVOTE ||
                        status->getEletionState() == Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_NOTINITIATED ||
                        status->getEletionState() == Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_INITIATED ||
                        status->getEletionState() == Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_CLOSED ||
                        status->getEletionState() == Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_RESULTSTRANSMITTED ||
                        VotingCenters::VotingCentersWrapper::getInstance()->getRunningMode () == VotingCenters::RunningMode::VOTING_CENTER_RECOUNT;

    return validate;
}

bool MachineOperationManager::isNotInitiatedElection()
{
    return getOperationStatus()->getElectionState() ==
           Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_NOTINITIATED;
}

bool MachineOperationManager::isClosingElection()
{
    return getOperationStatus()->getElectionState() ==
        Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_CLOSING;
}

bool MachineOperationManager::isCloseElection()
{
    return getOperationStatus()->getElectionState() ==
        Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_RESULTSTRANSMITTED;
}

bool MachineOperationManager::isCloseVotingExperience()
{
    return getOperationStatus()->getElectionState() ==
           Smartmatic::SAES::Voting::Status::ProcessStatus::ELECTION_STATUS_CLOSED;
}

bool MachineOperationManager::isPrintingInstallReport()
{
    bool isPrinting = false;

    try
    {
        isPrinting = getOperationStatus()->getElectionStatusWrapper()->getElectionConfigurationReportPrinted();
    }
    catch ( ... )
    {
        logger->Debug ( "ERRRO - read electoral configuration" );
    }

    return isPrinting;
}

bool MachineOperationManager::isWaitingConfirmationToCloseElection()
{
    return waitingToCloseElection;
}

void MachineOperationManager::setWaitingConfirmationToCloseElection(const bool waitingForUser)
{
    waitingToCloseElection = waitingForUser;
}

void MachineOperationManager::saveElectoralConfigurationReportPrinted()
{
    try
    {
        getOperationStatus()->getElectionStatusWrapper()->setElectionConfigurationReportPrinted ( true );
        logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.ElectoralConfigurationReportPrinted" ) );
        getOperationStatus()->persistElectionStatus();
    }
    catch ( ... )
    {
        logger->Debug ( "Error save electoral configuration report printed" );
    }
}

void MachineOperationManager::deleteCloseFiles()
{
    transmissionPackageWrapper->deleteFiles();
    emlFileManager->deleteFiles();
    pvFormWrapper->deleteFiles();
    Smartmatic::SAES::Voting::VotingResultsWrapper::getInstance()->deleteFiles();
    Smartmatic::SAES::Voting::NationalityTypeManager::getInstance()->deleteFiles();
}

LogBasePath * MachineOperationManager::getLogBasePathDom()
{
    return basePathDom;
}

LogBasePath * MachineOperationManager::getLogBasePathFlash()
{
    return basePathFlash;
}

void MachineOperationManager::saveVoteInOneMemory()
{
    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.SaveVoteInOneMemory" ) );
    getOperationStatus()->getElectionStatusWrapper()->setSynchronizeVotes ( true );
    getOperationStatus()->persistElectionStatus ( false );
    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.SaveVoteInOneMemorySuccess" ) );
}

void MachineOperationManager::restoreVoteInAllMemory()
{
    getOperationStatus()->getElectionStatusWrapper()->setSynchronizeVotes ( false );
    getOperationStatus()->persistElectionStatus();
    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.MachineOperationManager.RestoreVoteInAllMemorySuccess" ) );
}

void MachineOperationManager::exitMachine()
{
    ( void ) signal ( SIGUSR2, SIG_DFL );
    ( void ) signal ( SIGSEGV, SIG_DFL );
    ( void ) signal ( SIGABRT, SIG_DFL );

    ExitProcess::getInstance()->exitProcess();
}

void MachineOperationManager::removeDocsFiles()
{
    std::vector<std::string> rootDirectories;
    rootDirectories.push_back(saesDirectories->getDocsFolderFullPath(BasePathType::CONFIGURATION_DOM));
    rootDirectories.push_back(saesDirectories->getDocsFolderFullPath(BasePathType::CONFIGURATION_FLASH));

    for (size_t i = 0; i < rootDirectories.size(); ++i)
    {
        std::vector<std::string> directories;
        Directory::getInstance()->GetDirectories(rootDirectories[i], directories);

        for (size_t j = 2; j < directories.size(); ++j)
        {
            const std::string directoryToDelete = Path::Combine(rootDirectories[i], directories[j]);
            Directory::getInstance()->Delete(directoryToDelete);
        }

        std::vector<std::string> files;
        Directory::getInstance()->GetFiles(rootDirectories[i], files);

        for (size_t k = 0; k < files.size(); ++k)
        {
            const std::string fileToDelete = Path::Combine(rootDirectories[i], files[k]);
            File::getInstance()->DeleteFile(fileToDelete);
        }
    }
}

void MachineOperationManager::removePlainFiles()
{
    emlFileManager->removePlainEmlFiles();
    emlFileManager->reloadEmlFiles();

    removeDocsFiles();

    ProcessLauncher::getInstance()->RunProcessThroughSystem("sync");
}

void MachineOperationManager::persistPlainEmlFiles()
{
    logger->Debug ( "Start persistence plain eml files" );
    emlFileManager->setTotalsByType(NationalityTypeManager::getInstance()->getTotalsByType());
    emlFileManager->persistPlainEmlFiles();
}

void MachineOperationManager::persistX7SEmlFiles()
{
    logger->Debug ( "Start persistence x7s eml files" );
    emlFileManager->persistX7SEmlFiles();
}

bool MachineOperationManager::persistAllResult ()
{
    bool validate = false;

    try
    {
        removePlainFiles();

        ResultValidation::getInstance()->initialize();

        persistTransmissionPackage();
        logger->Audit ( N_ ( "Persist Transmission Package Successfully" ) );

        persistPlainEmlFiles();
        logger->Audit ( N_ ( "Persist Plain EML Files Successfully" ) );

        logger->Audit ( N_ ( "Start Validation All Result" ) );
        persistVotingResultsFile();

        if ( ResultValidation::getInstance()->validate() )
        {
            logger->Audit ( N_ ( "Validation All Result Successfully" ) );

            ResultValidation::getInstance()->clear();

            persistPVForm();

            logger->Audit ( N_ ( "Persist PVForm Successfully" ) );

            if (VoteComplianceVerificator::getInstance()->checkAllBeforeGeneratingResultsConditions(pvFormWrapper->getTotalVotes(), pvFormWrapper->getVoteResults(), emlFileManager->getOutputEmlFiles(), voteStatistics))
            {
                logger->Audit ( N_ ( "Vote results meet the compliance." ) );

                persistX7SEmlFiles();

                logger->Audit ( N_ ( "Persist X7SEml Files Successfully" ) );

                copyFilesToFat();

                logger->Audit ( N_ ( "Files Exported to FAT Partition Successfully" ) );

                validate = true;

            }
            else
            {
                logger->Audit ( N_ ( "Vote results do not meet the compliance." ) );
            }
        }
    }
    catch ( ... )
    {
    }

    transmissionPackageWrapper->deleteVotes();

    if ( validate )
    {
        logger->Audit ( N_ ( "Persist All Result Successfully" ) );
    }
    else
    {
        logger->Audit ( N_ ( "Persist All Result Fail" ) );
    }

    return validate;
}

bool MachineOperationManager::checkNetworkDevice ()
{
    DeviceTypeMonitor * deviceTypeMonitor = DeviceTypeMonitorPool::getInstance()->getDeviceTypeMonitor ( DeviceType::CONFIGURATION_ETHERNET );
    ResultsPackageTransmissionManager manager ( deviceTypeMonitor );

    return manager.mountNetworkDevice();
}

bool MachineOperationManager::sendResultsPackages ( std::string & message )
{
    VotingCenters::VotingCenter votingCenter;
    VotingCenters::VotingCentersWrapper::getInstance()->getInstallationVotingCenter ( votingCenter );

    DeviceTypeMonitor * deviceTypeMonitor = DeviceTypeMonitorPool::getInstance()->getDeviceTypeMonitor ( DeviceType::CONFIGURATION_ETHERNET );
    ResultsPackageTransmissionManager manager ( deviceTypeMonitor );

    bool result = manager.sendResultsPackages ( votingCenter.getCode(), getSaesDirectories()->getDevicePath(), message );

    if ( !result && getOperationStatus()->getElectionStatusWrapper()->getResultsPackageTransmissionState () != ResultsPackageTransmissionStateConfiguration::ELECTION_STATUS_TRANSMITTED )
    {
        getOperationStatus()->getElectionStatusWrapper()->setResultsPackageTransmissionState ( ResultsPackageTransmissionStateConfiguration::ELECTION_STATUS_NOTTRANSMITTED );
    }
    else
    {
        getOperationStatus()->getElectionStatusWrapper()->setResultsPackageTransmissionState ( ResultsPackageTransmissionStateConfiguration::ELECTION_STATUS_TRANSMITTED );
    }

    try
    {
        getOperationStatus()->persistElectionStatus();
    }
    catch ( ... )
    {
        result = false;
        message = N_ ( "Error while storing the state of the transmission of the results package." );
    }

    return result;
}

void MachineOperationManager::copyFilesToFat()
{
    FatFileManager fatFiles;
    BasePathType type        = BasePathType::CONFIGURATION_DOM;
    std::string  emlFiles    = saesDirectories->getEmlFolderFullPath(type);
    std::string  voteFiles   = saesDirectories->getVotesFolderFullPath(type);
    std::string  reportFiles = saesDirectories->getDocsFolderFullPath(type);

    fatFiles.addFatDevices();

    if (!checkVoteFiles())
    {
        reporFatExportError(N_("Error checking vote files. Can't copy files to fat partition."));

        return;
    }

    if (fatFiles.getNumberOfDevices() != 2)
    {
        reporFatExportError(N_("Incorrect number of fat partitions."));
    }

    if (!fatFiles.copyFiles(emlFiles, saesDirectories->getDevicePath(type)))
    {
        reporFatExportError(N_("Error copying eml files to fat partition"));
    }

    if (!fatFiles.copyFiles(voteFiles, saesDirectories->getSaesDataFolderFullPath(type)))
    {
        reporFatExportError(N_("Error copying vote files to fat partition"));
    }

    if (!fatFiles.copyFiles(reportFiles, saesDirectories->getSaesDataFolderFullPath(type)))
    {
        reporFatExportError(N_("Error copying report files to fat partition"));
    }

    if (fatFiles.verifyFiles(emlFiles, saesDirectories->getDevicePath(type)) &&
        fatFiles.verifyFiles(voteFiles, saesDirectories->getSaesDataFolderFullPath(type)) &&
        fatFiles.verifyFiles(reportFiles, saesDirectories->getSaesDataFolderFullPath(type)))
    {
        logger->Debug("Files successfully copied and verified to fat partition.");
    }
    else
    {
        reporFatExportError(N_("Error verifying files in fat partition."));
    }
}

bool MachineOperationManager::verifyFilesInFat()
{
    FatFileManager fatFiles;
    BasePathType type        = BasePathType::CONFIGURATION_DOM;
    std::string  emlFiles    = saesDirectories->getEmlFolderFullPath(type);
    std::string  voteFiles   = saesDirectories->getVotesFolderFullPath(type);
    std::string  reportFiles = saesDirectories->getDocsFolderFullPath(type);
    bool         valid       = false;

    fatFiles.addFatDevices();

    if (fatFiles.getNumberOfDevices() == 2)
    {
        valid = fatFiles.verifyFiles(emlFiles, saesDirectories->getDevicePath(type)) &&
                fatFiles.verifyFiles(voteFiles, saesDirectories->getSaesDataFolderFullPath(type)) &&
                fatFiles.verifyFiles(reportFiles, saesDirectories->getSaesDataFolderFullPath(type));
    }

    return valid;
}

void MachineOperationManager::reporFatExportError(const std::string& message)
{
    bool ignoreError = SAESFunctionalityConfigurationManager::getInstance()->getOperationConfiguration().getIgnoreFatErrors();

    logger->Debug(message);

    if (ignoreError)
    {
        std::string ignoreErrorMessage = N_("Fatal error suppressed by configuration.");

        logger->Audit(message + " " + ignoreErrorMessage);
    }
    else
    {
        throw Smartmatic::System::Exception::FileSystemException(
            message,
            N_("Smartmatic.SAES.Operation.MachineOperationManager.ErrorExportingFiles"),
            logger);
    }
}
