/**
 * @file	MachineOperationManager.hxx
 * @brief	machine operation manager class.
 * @author	Gustavo Garcia
 * @date	30/01/2017
 */

#ifndef _MACHINE_OPERATION_MANAGER_H_
#define _MACHINE_OPERATION_MANAGER_H_

#include <Log/LogBasePath.hxx>
#include "Initiator/IMachineInitiator.hxx"
#include "Authenticator/IAuthenticator.hxx"
#include "Voting/VotingExperienceController.hxx"
#include "Voting/AudioVoting/AudioVotingExperienceController.hxx"
#include <Operation/Monitor/DeviceTypeMonitor.hxx>
#include <Operation/Voting/AudioVoting/AudioVotingCheckUtility.hxx>
#include <Voting/ElectoralConfiguration.hxx>
#include <Voting/OperationStatus.hxx>
#include "Voting/PBReports/activated-card-statistics-wrapper.h"
#include <Runtime/Environment/saes-directories.h>
#include <Runtime/Environment/SaesInstallDirectories.hxx>
#include <System/SafeBuffer.h>
#include <Voting/Vote/vote-schema.hxx>
#include <Voting/PBReports/VoteStatisticsWrapper.hxx>
#include <Voting/Recount/RecountVoteStatistics.h>
#include <Log/ISMTTLog.h>
#include <System/XSLFO/XSLFOTransformer.h>
#include <glibmm.h>
#include <Voting/PBReports/PVFormWrapper.hxx>
#include <Voting/Vote/StationTallyWrapper.hxx>
#include <Voting/Vote/TransmissionPackageWrapper.hxx>
#include <Voting/Vote/HashVoteManager.hxx>
#include <Operation/Voting/VotePersistence.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace Voting
        {
            namespace Eml
            {
                class EmlFileManager;
            }
        }

        namespace Operation
        {
            /**
             * @class	PersistSortFile
             * @brief	Persist sort file.
             * @author	Youssef.tourkmani
             * @date	01/11/2012
             */
            class PersistSortFile;
            /**
             * @class	InstallMachine
             * @brief	Install machine.
             * @author	Youssef.tourkmani
             * @date	01/11/2012
             */
            class InstallMachine;

            /**
             * @class	MachineOperationManager
             * @brief	Header class MachineOperationManager.
             * @author	Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
             * @date	02-03-2011
             */

            class MachineOperationManager
            {
                protected:

                    /**
                     * @fn	MachineOperationManager::MachineOperationManager();
                     * @brief	Class constructor.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     */
                    MachineOperationManager();

                private:

                    static MachineOperationManager * instance;  ///< instance of class MachineOperationManager
                    Smartmatic::SAES::Operation::Initiator::IMachineInitiator * machineInitiator;   ///< instance of object machine initiator
                    Smartmatic::SAES::Operation::Authenticator::IAuthenticator * authenticator; ///< instance of object authnticator
                    Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration;  ///< instance of object electoral configuration
                    Smartmatic::SAES::Voting::PBReports::ActivatedCardStatisticsWrapper * activatedCardStatistics;	///< instance of object activated card statistics list
                    Smartmatic::SAES::Runtime::Environment::SaesDirectories * saesDirectories;  ///< instance of object SAES directories
                    Smartmatic::SAES::Runtime::Environment::SaesInstallDirectories * saesInstallDirectories; ///< instance saes install directories
                    Smartmatic::SAES::Voting::OperationStatus * operationStatus;	///< instance of object operation status
                    Smartmatic::SAES::Operation::Voting::VotingExperienceController * votingExperienceController;   ///< instance of object voting experience controller
                    Smartmatic::SAES::Voting::PBReports::VoteStatisticsWrapper * voteStatistics;   ///< instance of vote statistics
                    Smartmatic::SAES::Voting::RecountVoteStatistics * recountVoteStatistics;   ///< instance of vote statistics
                    Smartmatic::SAES::Log::LogBasePath * basePathDom; ///< The base path dom
                    Smartmatic::SAES::Log::LogBasePath * basePathFlash; ///< The base path flash
                    Smartmatic::System::XSLFO::XSLFOTransformer * pvtransformer;	///< The pvtransformer
                    Smartmatic::SAES::Voting::PBReports::PVFormWrapper * pvFormWrapper; ///< pv form wrapper class
                    Smartmatic::SAES::Voting::Vote::StationTallyWrapper * stationTallyWrapper; ///< The StationTally Wrapper
                    Smartmatic::SAES::Voting::Vote::TransmissionPackageWrapper * transmissionPackageWrapper; ///< The transmission package wrapper
                    //Smartmatic::SAES::Voting::Codi::CodiFileWrapper * codiFileWrapper; ///< The codi file wrapper
                    Smartmatic::SAES::Operation::AudioVoting::AudioVotingCheckUtility  audioVotingCheckUtility; ///< The wrapper for audio voting check utility
                    Smartmatic::SAES::Voting::Eml::EmlFileManager * emlFileManager;

                public:

                    typedef sigc::signal<void> SignalShowShutdownWidget;

                public:

                    /**
                     * @fn	bool synchronize();
                     * @brief	Stop dsort, write login-key and synchronize all votes.
                     * @author	Yadickson Soto
                     * @date	30/07/2015
                     * @return  True if success
                     */
                    bool synchronize ();

                    /**
                     * @fn	bool checkData();
                     * @brief	Check all data from devices.
                     * @author	Yadickson Soto
                     * @date	30/07/2015
                     * @return  True if success
                     */
                    bool checkData ();

                    /**
                     * @fn	void MachineOperationManager::getIsPersistence();
                     * @brief	Getter if is possible persistence vote.
                     * @author	Yadickson Soto
                     * @date	20/06/2013
                     * @return	false if not is possible persistence vote.
                     */
                    bool getIsPersistence();

                    /**
                     * @fn	virtual MachineOperationManager::~MachineOperationManager();
                     * @brief	Finaliser.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     */

                    virtual ~MachineOperationManager();

                    /**
                     * @fn	void MachineOperationManager::setMachineInitiator ( Smartmatic::SAES::Operation::Initiator::IMachineInitiator * machineInitiator );
                     * @brief	method to set machine initiator instance.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @param	machineInitiator	object machine initiator.
                     */

                    void setMachineInitiator ( Smartmatic::SAES::Operation::Initiator::IMachineInitiator * machineInitiator );

                    /**
                     * @fn	Smartmatic::SAES::Operation::Initiator::IMachineInitiator * MachineOperationManager::getMachineInitiator();
                     * @brief	method to get machine initiator instance.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @return	machine initiator instance.
                     */

                    Smartmatic::SAES::Operation::Initiator::IMachineInitiator * getMachineInitiator();

                    /**
                     * @fn	void MachineOperationManager::setAuthenticator ( Smartmatic::SAES::Operation::Authenticator::IAuthenticator * authenticator );
                     * @brief	method to set authenticator instance.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @param	authenticator	object authenticator.
                     */

                    void setAuthenticator ( Smartmatic::SAES::Operation::Authenticator::IAuthenticator * authenticator );

                    /**
                     * @fn	Smartmatic::SAES::Operation::Authenticator::IAuthenticator * MachineOperationManager::getAuthenticator();
                     * @brief	method to get authenticator instance.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @return	authenticator instance.
                     */

                    Smartmatic::SAES::Operation::Authenticator::IAuthenticator * getAuthenticator();

                    /**
                     * @fn	void MachineOperationManager::setElectoralConfiguration ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration );
                     * @brief	method to set the electoral configuration instance.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @param	electoralConfiguration	object electoral configuration.
                     */

                    void setElectoralConfiguration ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration );
                    /**
                     * @fn	void MachineOperationManager::setActivatedCardStatistics ( Smartmatic::SAES::Voting::PBReports::ActivatedCardStatisticsWrapper * activatedCardStatistics );
                     * @brief	method to set activated card statistics list instance.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @param [in,out]	activatedCardStatistics	card statistics	object activated card statistics.
                     */

                    void setActivatedCardStatistics ( Smartmatic::SAES::Voting::PBReports::ActivatedCardStatisticsWrapper * activatedCardStatistics );
                    /**
                     * @fn	Smartmatic::SAES::Voting::PBReports::ActivatedCardStatisticsWrapper * MachineOperationManager::getActivatedCardStatistics();
                     * @brief	method to get activated card statistics.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @return	activated card statistics instance.
                     */

                    Smartmatic::SAES::Voting::PBReports::ActivatedCardStatisticsWrapper * getActivatedCardStatistics();

                    /**
                     * @fn	Smartmatic::SAES::Voting::ElectoralConfiguration * MachineOperationManager::getElectoralConfiguration();
                     * @brief	method to get electoral configuration.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @return	electoral configuration instance.
                     */

                    Smartmatic::SAES::Voting::ElectoralConfiguration * getElectoralConfiguration();

                    /**
                     * @fn	Smartmatic::SAES::Runtime::Environment::SaesDirectories * MachineOperationManager::getSaesDirectories();
                     * @brief	method to get saes directories.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @return	saes directories instance.
                     */

                    Smartmatic::SAES::Runtime::Environment::SaesDirectories * getSaesDirectories();

                    /**
                     * @fn	Smartmatic::SAES::Runtime::Environment::SaesInstallDirectories * MachineOperationManager::getSaesInstallDirectories();
                     * @brief	Gets saes install directories.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @return	null if it fails, else the saes install directories.
                     */
                    Smartmatic::SAES::Runtime::Environment::SaesInstallDirectories * getSaesInstallDirectories();

                    /**
                     * @fn	Smartmatic::SAES::Voting::OperationStatus * MachineOperationManager::getOperationStatus();
                     * @brief	method to get operation status.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @return	operation status instance.
                     */

                    Smartmatic::SAES::Voting::OperationStatus * getOperationStatus();

                    /**
                     * @fn	void MachineOperationManager::setVotingExperienceController ( Smartmatic::SAES::Operation::Voting::VotingExperienceController * votingExperienceController );
                     * @brief	method to set voting experience controller.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @param	votingExperienceController	object voting experience controller.
                     */

                    void setVotingExperienceController ( Smartmatic::SAES::Operation::Voting::VotingExperienceController * votingExperienceController );

                    /**
                     * @fn	Smartmatic::SAES::Operation::Voting::VotingExperienceController * MachineOperationManager::getVotingExperienceController();
                     * @brief	method to get voting experience controller.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @return	voting experience controller instance.
                     */

                    Smartmatic::SAES::Operation::Voting::VotingExperienceController * getVotingExperienceController();

                    /**
                     * @fn	std::string MachineOperationManager::getMachineOperationCode();
                     * @brief	method to get machine operation code.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @return	string machine operation code.
                     */

                    std::string getMachineOperationCode();

                    /**
                     * @fn	Voting::VotePersistence::PersistenceStatusVote MachineOperationManager::persistenceVote ( Smartmatic::SAES::Voting::Vote::Vote & vote, bool saveVoteOneMemory )
                     *  throw ( Smartmatic::System::Exception::CryptoException,
                     * 			Smartmatic::System::Exception::SignedException, Smartmatic::System::Exception::XmlException );
                     * @brief	method for persistence vote.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::SignedException	Thrown when a signed error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::XmlException			Thrown when an XML error condition
                     * 																														occurs.
                     * @param	vote	vote to persistence.
                     * @param	saveVoteOneMemory flag save vote in one memory
                     * @return	status of persistence vote
                     */

                    Voting::VotePersistence::PersistenceStatusVote persistenceVote ( Smartmatic::SAES::Voting::Vote::Vote & vote, bool saveVoteOneMemory );

                    /**
                     * @fn	void MachineOperationManager::deleteVote( Smartmatic::SAES::Voting::Vote::Vote & vote );
                     * @brief	method for delete vote in file system.
                     * @param vote vote
                     * @author	Yadickson Soto
                     * @date	01/11/2012
                     */
                    void deleteVote ( Smartmatic::SAES::Voting::Vote::Vote & vote );

                    /**
                     * @fn	void MachineOperationManager::persistenceFileSystemVote();
                     * @brief	method for persistence vote in file system.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     */

                    void persistenceFileSystemVote();

                    /**
                     * @fn	void MachineOperationManager::setVoteStatistics ( Smartmatic::SAES::Voting::PBReports::VoteStatisticsWrapper * voteStatistics );
                     * @brief	method to set vote statistics.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @param	voteStatistics	vote statistics.
                     */

                    void setVoteStatistics ( Smartmatic::SAES::Voting::PBReports::VoteStatisticsWrapper * voteStatistics );

                    /**
                     * @fn	Smartmatic::SAES::Voting::PBReports::VoteStatisticsWrapper * MachineOperationManager::getVoteStatistics();
                     * @brief	method to get vote statistics.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @return	vote statistics.
                     */

                    Smartmatic::SAES::Voting::PBReports::VoteStatisticsWrapper * getVoteStatistics();

                    /**
                     * @fn	Smartmatic::SAES::Voting::RecountVoteStatistics * MachineOperationManager::getRecountVoteStatistics();
                     * @brief	method to get vote statistics.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @return	vote statistics.
                     */

                    Smartmatic::SAES::Voting::RecountVoteStatistics * getRecountVoteStatistics();
                    /**
                     * @fn	bool MachineOperationManager::checkVotes ();
                     * @brief	method to validate all votes.
                     * @author	Yadickson Soto
                     * @date	31/05/2013
                     * @return	true if it succeeds, false if it fails.
                     */
                    bool checkVotes ();

                    /**
                     * @fn	void MachineOperationManager::configurationReset();
                     * @brief	configuration reset.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     */

                    void configurationReset();

                    /**
                     * @fn	bool MachineOperationManager::authentication ( std::string & center, std::string & password,
                     * 			bool centerCaseSensitive );
                     * @brief	Method authentication machine.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	18/10/2011
                     * 				@param center
                     * @param [in,out]	center	 	The center.
                     * @param	password the password.
                     * @param	centerCaseSensitive	true to center case sensitive.
                     * @return	true is success.
                     */
                    bool authentication ( std::string & center, std::string & password, bool centerCaseSensitive );

                    /**
                     * @fn	bool MachineOperationManager::install();
                     * @brief	Installs this object.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @return	true if it succeeds, false if it fails.
                     */
                    bool install();

                    /**
                     * @fn	std::list<Smartmatic::SAES::Voting::Election::Language> MachineOperationManager::getLanguagesElection();
                     * @brief	Gets languages election.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @return	The languages election.
                     */
                    std::list<Smartmatic::SAES::Voting::Election::Language> getLanguagesElection();
                    /**
                     * @fn	std::list<Smartmatic::SAES::Voting::Election::Language> MachineOperationManager::getLanguagesInstall();
                     * @brief	Gets languages install.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @return	The languages install.
                     */
                    std::list<Smartmatic::SAES::Voting::Election::Language> getLanguagesInstall();

                    /**
                     * @fn	void MachineOperationManager::startUp( bool firstTime ) throw ( Smartmatic::System::Exception::ElectionException,
                     * 			Smartmatic::System::Exception::VotingDeviceException,
                     * 			Smartmatic::System::Exception::CryptoException,
                     * 			Smartmatic::System::Exception::SignedException, Smartmatic::System::Exception::XmlException,
                     * 			Smartmatic::System::Exception::GuIdException, Smartmatic::System::Exception::VoteException,
                     * 			Smartmatic::System::Exception::FileSystemException );
                     * @brief	method start up machine.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @exception	Smartmatic::System::Exception::ElectionException			Thrown when an election error
                     * 																																	condition occurs.
                     * @exception	Smartmatic::System::Exception::VotingDeviceException	Thrown when a voting device
                     * 																																	error condition occurs.
                     * @exception	Smartmatic::System::Exception::CryptoException				Thrown when a crypto error
                     * 																																	condition occurs.
                     * @exception	Smartmatic::System::Exception::SignedException				Thrown when a signed error
                     * 																																	condition occurs.
                     * @exception	Smartmatic::System::Exception::XmlException						Thrown when an XML error
                     * 																																	condition occurs.
                     * @exception	Smartmatic::System::Exception::GuIdException					Thrown when a gu identifier
                     * 																																	error condition occurs.
                     * @exception	Smartmatic::System::Exception::VoteException					Thrown when a vote error
                     * 																																	condition occurs.
                     * @exception	Smartmatic::System::Exception::FileSystemException		Thrown when a file system error
                     * 																																	condition occurs.
                     * @param firstTimeMode True if first time mode running machine
                     */
                    void startUp ( bool firstTimeMode );

                    void initializeClass ( Smartmatic::SAES::Runtime::Environment::SaesDirectories * directories, bool firstTimeMode, bool persist = true );

                    /**
                     * @fn	void MachineOperationManager::createMachineId ( bool force = false );
                     * @brief	Creates machine identifier.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @param	force	(optional) the force.
                     */
                    void createMachineId ( bool force = false );
                    /**
                     * @fn	bool MachineOperationManager::validateMachineId();
                     * @brief	Determines if we can validate machine identifier.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @return	true if it succeeds, false if it fails.
                     */
                    bool validateMachineId();
                    /**
                     * @fn	bool MachineOperationManager::checkMachineId();
                     * @brief	Determines if we can check machine identifier.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @return	true if it succeeds, false if it fails.
                     */
                    bool checkMachineId();
                    /**
                     * @fn	bool MachineOperationManager::checkPVReport();
                     * @brief	Determines if we can check pv report.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @return	true if it succeeds, false if it fails.
                     */
                    bool checkPVReport();

                    /**
                     * @fn	bool MachineOperationManager::checkTransmissionPackages();
                     * @brief	Method for check transmission package files.
                     * @author	Yadickson Soto
                     * @date	17/02/2012
                     * @return	true is valid files.
                     */
                    bool checkTransmissionPackages();

                    /**
                     * @fn	bool checkVotingResultFiles();
                     * @brief	Method for check voting result files.
                     * @author	Yadickson Soto
                     * @date	27/08/2013
                     * @return	true is valid files.
                     */
                    bool checkVotingResultFiles();

                    /**
                     * @fn  bool checkNationalitiesFile();
                     * @brief   Method for checking the nationalities file.
                     * @author  Eduardo Fernández
                     * @date    11/08/2017
                     * @return  true is valid files.
                    */
                    bool checkNationalitiesFile();

                    /**
                     * @fn	bool MachineOperationManager::checkVoteFiles();
                     * @brief	check all vote files from usb.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	31/05/2013
                     * @return	true if it succeeds, false if it fails.
                     */
                    bool checkVoteFiles();

                    /**
                     * @fn	bool MachineOperationManager::loadVoteFiles( bool validate = true );
                     * @brief	load all vote files names and hash from usb.
                     * @author	Yadickson Soto yadickson.soto@smartmatic.com
                     * @date	31/05/2013
                     * @param 	validate validate statistics votes if not recount mode
                     * @return	true if it succeeds, false if it fails.
                     */
                    bool loadVoteFiles ( bool validate = true );
                    /**
                     * @fn	bool MachineOperationManager::checkElectionStatus();
                     * @brief	method for validate election status.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @return	true if success.
                     */
                    bool checkElectionStatus();

                    /**
                     * @fn	bool MachineOperationManager::checkSAESFiles();
                     * @brief	Determines if we can check voting centers files.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @return	true if it succeeds, false if it fails.
                     */
                    bool checkVotingCentersFiles();
                    /**
                     * @fn	bool MachineOperationManager::checkSAESLocal();
                     * @brief	Determines if we can check saes local.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @return	true if it succeeds, false if it fails.
                     */
                    bool checkSAESLocal();
                    /**
                     * @fn	bool MachineOperationManager::checkVoteStatistics();
                     * @brief	Determines if we can check vote statistics.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @return	true if it succeeds, false if it fails.
                     */
                    bool checkVoteStatistics();
                    /**
                     * @fn	bool MachineOperationManager::checkElection();
                     * @brief	Determines if we can check election.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @return	true if it succeeds, false if it fails.
                     */
                    bool checkElection();
                    /**
                     * @fn	bool MachineOperationManager::checkDeviceParameters();
                     * @brief	Checks if device parameter files are valid.
                     * @author	Angel.Ramirez
                     * @date	07/05/2022
                     * @return	true if it succeeds, false if it fails.
                     */
                    bool checkDeviceParameters();
                    /**
                     * @fn	bool MachineOperationManager::checkVotingDevices();
                     * @brief	Determines if we can check voting devices.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @return	true if it succeeds, false if it fails.
                     */
                    bool checkVotingDevices();
                    /**
                     * @fn	bool MachineOperationManager::checkActivatedCardStatistics();
                     * @brief	Determines if we can check activated card statistics.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @return	true if it succeeds, false if it fails.
                     */
                    bool checkActivatedCardStatistics();
                    /**
                     * @fn	bool MachineOperationManager::checkSignature();
                     * @brief	Determines if we can check signature.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @return	true if it succeeds, false if it fails.
                     */
                    bool checkSignature();

                    /**
                     * @fn	bool MachineOperationManager::checkFatFiles();
                     * @brief	Determines if files in fat are valid.
                     * @author  manuel.arguelles
                     * @date	04/08/2017
                     * @return	true if it succeeds, false if it fails.
                     */
                    bool checkFatFiles();

                    /**
                     * @fn	bool validateSAESSignature();
                     * @brief	Validate Signature SAES Folder Signature.
                     * @author	Yadickson Soto
                     * @date	03/06/2013
                     * @return true if success
                     */
                    bool validateSAESSignature();

                    /**
                     * @fn	bool validateEmlSignature();
                     * @brief	Validate Signature of Eml Files.
                     * @author	Yadickson Soto
                     * @date	03/06/2013
                     * @return true if success
                     */
                    bool validateEmlSignature();

                    /**
                     * @brief  Validate SAESLocal folder and files signature.
                     * @author manuel.arguelles
                     * @date   Mar 23, 2018
                     * @return True if the files matches the signature, false otherwise.
                     */
                    bool validateSaesLocalSignature();

                    /**
                     * @fn	void MachineOperationManager::installMachine() throw ( Smartmatic::System::Exception::CryptoException,
                     * 			Smartmatic::System::Exception::SignedException, Smartmatic::System::Exception::XmlException );
                     * @brief	Installs the machine.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::SignedException	Thrown when a signed error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::XmlException			Thrown when an XML error condition
                     * 																														occurs.
                     */
                    void installMachine();

                    /**
                     * @fn	void MachineOperationManager::skipInstallMachine() throw ( Smartmatic::System::Exception::CryptoException,
                     * 			Smartmatic::System::Exception::SignedException, Smartmatic::System::Exception::XmlException );
                     * @brief	Skip install machine.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::SignedException	Thrown when a signed error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::XmlException			Thrown when an XML error condition
                     * 																														occurs.
                     */
                    void skipInstallMachine();

                    /**
                     * @fn	void MachineOperationManager::demostrationVote() throw ( Smartmatic::System::Exception::CryptoException,
                     * 			Smartmatic::System::Exception::SignedException, Smartmatic::System::Exception::XmlException );
                     * @brief	Demostration vote.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::SignedException	Thrown when a signed error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::XmlException			Thrown when an XML error condition
                     * 																														occurs.
                     */
                    void demostrationVote();

                    /**
                     * @fn	void MachineOperationManager::skipDemostrationVote() throw ( Smartmatic::System::Exception::CryptoException,
                     * 			Smartmatic::System::Exception::SignedException, Smartmatic::System::Exception::XmlException );
                     * @brief	Skip demostration vote.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::SignedException	Thrown when a signed error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::XmlException			Thrown when an XML error condition
                     * 																														occurs.
                     */
                    void skipDemostrationVote();

                    /**
                     * @fn	void MachineOperationManager::openElection() throw ( Smartmatic::System::Exception::CryptoException,
                     * 			Smartmatic::System::Exception::SignedException, Smartmatic::System::Exception::XmlException );
                     * @brief	Opens the election.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::SignedException	Thrown when a signed error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::XmlException			Thrown when an XML error condition
                     * 																														occurs.
                     */

                    void openElection();

                    /**
                     * @fn void MachineOperationManager::startClosingElection()
                     * @brief Starts closing election.
                     * @author eduardo.fernandez
                     * @date   10/12/2018
                     */
                    void startClosingElection();

                    /**
                     * @fn	void MachineOperationManager::closeElection() throw ( Smartmatic::System::Exception::CryptoException,
                     * 			Smartmatic::System::Exception::SignedException, Smartmatic::System::Exception::XmlException );
                     * @brief	Closes the election.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::SignedException	Thrown when a signed error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::XmlException			Thrown when an XML error condition
                     * 																														occurs.
                     */

                    void closeElection();

                    /**
                     * @fn	void MachineOperationManager::closeVotingExperience() throw ( Smartmatic::System::Exception::CryptoException,
                     * 			Smartmatic::System::Exception::SignedException, Smartmatic::System::Exception::XmlException );
                     * @brief	Closes the voting experience.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::SignedException	Thrown when a signed error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::XmlException			Thrown when an XML error condition
                     * 																														occurs.
                     */

                    void closeVotingExperience();

                    /**
                     * @fn	bool MachineOperationManager::isNotInstalled();
                     * @brief	Query if this object is not installed.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @return	true if not installed.
                     */

                    bool isNotInstalled();

                    /**
                     * @fn	bool MachineOperationManager::isNotDemostrationVote();
                     * @brief	Query if this object is not demostration vote.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @return	true if not demostration vote.
                     */

                    bool isNotDemostrationVote();

                    /**
                     * @fn	bool MachineOperationManager::isOpenElection();
                     * @brief	Query if this object is open election.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @return	true if open election, false if not.
                     */

                    bool isOpenElection();

                    /**
                     * @fn     bool MachineOperationManager::isNotOpenElection();
                     * @brief  Query if the election is not open.
                     * @author manuel.arguelles
                     * @date   14/07/2022
                     * @return true if the election is not open, false if it is.
                     */

                    bool isNotOpenElection();

                    /**
                     * @fn	bool MachineOperationManager::votingIsPossible();
                     * @brief	Determines if we can voting is possible.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @return	true if it succeeds, false if it fails.
                     */
                    bool votingIsPossible();

                    /**
                     * @fn	bool MachineOperationManager::isNotInitiatedElection();
                     * @brief	Query if this is not an initiated election.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @return	true if not initiated election, false if not.
                     */

                    bool isNotInitiatedElection();

                    /**
                     * @fn     bool MachineOperationManager::isClosingElection();
                     * @brief  Checks whether the current election status is closing.
                     * @author manuel.arguelles
                     * @date   20/07/2022
                     * @return true if the election is closing, false if not.
                     */
                    bool isClosingElection();

                    /**
                     * @fn	bool MachineOperationManager::isCloseElection();
                     * @brief	Query if this is a close election.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @return	true if close election, false if not.
                     */

                    bool isCloseElection();

                    /**
                     * @fn	bool MachineOperationManager::isCloseVotingExperience();
                     * @brief	Query if this is a close voting experience.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @return	true if close voting experience, false if not.
                     */

                    bool isCloseVotingExperience();

                    /**
                     * @fn	bool MachineOperationManager::isPrintingInstallReport();
                     * @brief	Query if this object is printing install report.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @return	true if printing install report, false if not.
                     */
                    bool isPrintingInstallReport();

                    /**
                     * @brief  Query if we are waiting for user to confirm the
                     * closure of the election.
                     * @author manuel.arguelles
                     * @date   Apr 26, 2018
                     */
                    bool isWaitingConfirmationToCloseElection();

                    /**
                     * @brief  Sets if we are waiting for user to close the election.
                     * @author manuel.arguelles
                     * @date   Apr 26, 2018
                     * @param  waitingForUser True if we are waiting, false if not.
                     */
                    void setWaitingConfirmationToCloseElection(bool waitingForUser);


                    /**
                     * @fn	void MachineOperationManager::persistVotingResultsFile() throw ( Smartmatic::System::Exception::CryptoException,
                     * 			Smartmatic::System::Exception::SignedException, Smartmatic::System::Exception::XmlException );
                     * @brief	Method persist transmission package.
                     * @author	Yadickson Soto
                     * @date	17/02/2012
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::SignedException	Thrown when a signed error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::XmlException		Thrown when an XML error condition
                     * 																														occurs.
                     */
                    void persistVotingResultsFile();

                    /**
                     * @fn	void MachineOperationManager::saveElectoralConfigurationReportPrinted();
                     * @brief	Saves the electoral configuration report printed.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     */

                    void saveElectoralConfigurationReportPrinted();

                    /**
                     * @fn	void MachineOperationManager::deleteCloseFiles();
                     * @brief	Deletes the close files.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     */
                    void deleteCloseFiles();

                    /**
                     * @fn	void MachineOperationManager::saveVoteInOneMemory() throw ( Smartmatic::System::Exception::CryptoException,
                     * 			Smartmatic::System::Exception::SignedException, Smartmatic::System::Exception::XmlException );
                     * @brief	Saves the vote in one memory.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::SignedException	Thrown when a signed error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::XmlException			Thrown when an XML error condition
                     * 																														occurs.
                     */
                    void saveVoteInOneMemory();

                    /**
                     * @fn	void MachineOperationManager::restoreVoteInAllMemory() throw ( Smartmatic::System::Exception::CryptoException,
                     * 			Smartmatic::System::Exception::SignedException, Smartmatic::System::Exception::XmlException );
                     * @brief	Restore vote in all memory.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::SignedException	Thrown when a signed error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::XmlException			Thrown when an XML error condition
                     * 																														occurs.
                     */
                    void restoreVoteInAllMemory();

                    /**
                     * @fn	static MachineOperationManager * MachineOperationManager::getInstance(MachineOperationManager * externInstance);
                     * @brief	Gets an instance of the clas.
                     * @author	Juan Valera
                     * @date	07/02/2017
                     * @param MachineOperationManager extern instance.
                     * @return	null if it fails, else the instance.
                     */

                    static MachineOperationManager * getInstance(MachineOperationManager * externInstance = NULL);

                    /**
                     * @fn	Smartmatic::SAES::Log::LogBasePath * MachineOperationManager::getLogBasePathDom();
                     * @brief	Gets log base path dom.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @return	null if it fails, else the log base path dom.
                     */
                    Smartmatic::SAES::Log::LogBasePath * getLogBasePathDom();

                    /**
                     * @fn	Smartmatic::SAES::Log::LogBasePath * MachineOperationManager::getLogBasePathFlash();
                     * @brief	Gets log base path flash.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @return	null if it fails, else the log base path flash.
                     */
                    Smartmatic::SAES::Log::LogBasePath * getLogBasePathFlash();

                    /**
                     * @fn	void MachineOperationManager::loadStationTally() throw ( Smartmatic::System::Exception::CryptoException,
                     * 			Smartmatic::System::Exception::SignedException, Smartmatic::System::Exception::XmlException );
                     * @brief	Method for load station tally.
                     * @author	Yadickson Soto
                     * @date	17/02/2012
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::SignedException	Thrown when a signed error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::XmlException			Thrown when an XML error condition
                     * 																														occurs.
                     */
                    void loadStationTally();

                    /**
                     * @fn	void MachineOperationManager::exitMachine();
                     * @brief	Method exit machine.
                     * @author	Yadickson Soto
                     * @date	23/02/2012
                     */
                    void exitMachine();

                    /**
                     * @fn virtual bool persistAllResult ();
                     * @brief Persist all result file, transmission packages, eml and pv
                     * @date 24/03/2014
                     * @author Yadickson Soto
                     * @return true if is success
                     */
                    virtual bool persistAllResult ();

                    /**
                     * @fn virtual bool checkNetworkDevice ();
                     * @brief Check network device
                     * @date 08/04/2014
                     * @author Yadickson Soto
                     * @return true if is success
                     */
                    virtual bool checkNetworkDevice ();

                    /**
                     * @fn virtual bool sendResultsPackages ( std::string & message );
                     * @brief Send all results packages using the network connection.
                     * @date 07/04/2014
                     * @author Yadickson Soto
                     * @param message Message of the response
                     * @return true if is success
                     */
                    virtual bool sendResultsPackages ( std::string & message );

                    /**
                     * @brief  Gets the audio voting experience.
                     * @author manuel.arguelles
                     * @date   Oct 12, 2017
                     * @return The audio voting experience controller.
                     */
                    Smartmatic::SAES::Operation::AudioVoting::AudioVotingExperienceController * getAudioVotingExperienceController();

                    /**
                     * @brief  Sets the audio voting experience controller.
                     * @author manuel.arguelles
                     * @date   Oct 12, 2017
                     * @param  audioVotingExperienceController The controller.
                     */
                    void setAudioVotingExperienceController(Smartmatic::SAES::Operation::AudioVoting::AudioVotingExperienceController * audioVotingExperienceController);
                    /**
                     * @brief  Gets the audio voting check utility instance.
                     * @author marcel.mancini
                     * @date   Jan 23, 2018
                     * @return The audio voting check utility.
                     */
                    Smartmatic::SAES::Operation::AudioVoting::AudioVotingCheckUtility getAudioVotingCheckUtility();

                    /**
                     * @brief Validates the issuer certificates involved in the electoral process.
                     * @author eduardo.fernandez
                     * @date   08/11/2018
                     * @return true if the validation is successful.
                     */
                    bool validateIssuerCertificates();

                    /**
                     * @brief Gets the number of connected flash devices
                     * @author eduardo.fernandez
                     * @date   22/01/2019
                     * @return The number of connected flash devices.
                     */
                    size_t getNumberOfConnectedFlashDevices() const;

                    /**
                     * @brief  Indicates whether the machine has votes or not.
                     * @author manuel.arguelles
                     * @date   30/06/2022
                     * @return True if there are votes present; otherwhise, false.
                     */
                    bool hasVotes() const;

                    /**
                     * @brief  Refresh the operation status from the memories.
                     * @author manuel.arguelles
                     * @date   26/07/2022
                     */
                    void refreshOperationStatus();

                protected:

                    /**
                     * @fn	void MachineOperationManager::initialize();
                     * @brief	Initialize the machine operation manager.
                     * @author	Juan Valera
                     * @date	07/02/2017
                     */
                    virtual void initialize();

                    /**
                     * @fn	void MachineOperationManager::cleanResources();
                     * @brief	Clean some fields of the MachineOperationManager setting this values in NULL.
                     * @author	Juan Valera
                     * @date	07/02/2017
                     */
                    void cleanResources();

                    /**
                     * @fn	void MachineOperationManager::persistTransmissionPackage() throw ( Smartmatic::System::Exception::CryptoException,
                     * 			Smartmatic::System::Exception::SignedException, Smartmatic::System::Exception::XmlException );
                     * @brief	Method persist transmission package.
                     * @author	Yadickson Soto
                     * @date	17/02/2012
                     * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::SignedException	Thrown when a signed error condition
                     * 																														occurs.
                     * @exception	Smartmatic::System::Exception::XmlException			Thrown when an XML error condition
                     * 																														occurs.
                     */
                    void persistTransmissionPackage();

                    /**
                     * @brief Removes directories and files contained in the Docs directory in case of error.
                     * @author eduardo.fernandez
                     * @date   15/01/2019
                     */
                    void removeDocsFiles();

                    /**
                     * @brief Removes plain files in case of error.
                     * @author eduardo.fernandez
                     * @date   10/01/2019
                     */
                    void removePlainFiles();

                    /**
                     * @brief	Method persist plain eml files.
                     * @author	Yadickson Soto
                     * @date	17/02/2012
                     * @throws CryptoException Thrown when a crypto error condition occurs.
                     * @throws SignedException Thrown when a signed error condition occurs.
                     * @throws XmlException    Thrown when an XML error condition occurs.
                     */
                    void persistPlainEmlFiles();

                    /**
                     * @brief	Method persist x7s eml files.
                     * @author	Yadickson Soto
                     * @date	17/02/2012
                     * @throws  CryptoException Thrown when a crypto error condition occurs.
                     * @throws  SignedException Thrown when a signed error condition occurs.
                     * @throws  XmlException    Thrown when an XML error condition occurs.
                     */
                    void persistX7SEmlFiles();

                    /**
                     * @brief	Persist pv form.
                     * @author	Youssef.tourkmani
                     * @date	01/11/2012
                     * @throws  CryptoException Thrown when a crypto error condition occurs.
                     * @throws  SignedException	Thrown when a signed error condition occurs.
                     * @throws  XmlException    Thrown when an XML error condition occurs.
                     */
                    void persistPVForm();

                private:

                    /**
                     * @brief  Copy files to fat partitions.
                     * @author manuel.arguelles
                     * @date   Sep  6, 2017
                     * @throws FileSystemException.
                     */
                    void copyFilesToFat();

                    /**
                     * @brief  Verify files in the fat partition.
                     * @author manuel.arguelles
                     * @date   Sep  6, 2017
                     * @return True if the files are ok, false if not.
                     */
                    bool verifyFilesInFat();

                    /**
                     * @brief  Logs the error and throw the exception
                     * @author manuel.arguelles
                     * @date   Sep  6, 2017
                     * @throws FileSystemException.
                     * @param  message The error message to log.
                     */
                    void reporFatExportError(const std::string& message);

                private:

                    PersistSortFile * sortDomFile;	///< The sort dom file
                    PersistSortFile * sortFlashFile;	///< The sort flash file
                    InstallMachine * objInstallMachine; ///< The object install machine

                    std::string openingCodeRecoveryMode; ///< opening code recovery mode
                    Smartmatic::SAES::Voting::Vote::HashVoteManager hashVoteManager; ///< Hash vote manager
                    Smartmatic::SAES::Operation::AudioVoting::AudioVotingExperienceController * audioVotingExperienceController;
                    bool waitingToCloseElection;
                    static Smartmatic::Log::ISMTTLog * logger;  ///< The logger
            };
        }
    }
}

#endif // _MACHINE_OPERATION_MANAGER_H_
