/**
 * @file MemoryEventManager.hxx
 * @brief Header class for send memory event to gui
 * @date 29/07/2015
 * @author Yadickson Soto
 */

#ifndef _MEMORY_EVENT_MANAGER_HXX_
#define _MEMORY_EVENT_MANAGER_HXX_

#include <System/SafeDispatcher.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Operation
        {

            class MemoryEventManager
            {
                protected:

                    /**
                     * @fn	MemoryEventManager();
                     * @brief	Class constructor
                     * @date 29/07/2015
                     * @author Yadickson Soto
                     */
                    MemoryEventManager();

                    static MemoryEventManager * instance; ///< The instance of the class

                public:

                    /**
                     * @enum	MemoryEvent
                     * @brief	Values that represent MemoryEvent.
                     */
                    enum MemoryEvent
                    {
                        ONE_MEMORY, ///< Error remove USB
                        TWO_MEMORY, ///< Insert two USB
                        CHECK_MEMORY_OK, ///< Check memory data OK
                        CHECK_MEMORY_ERROR, ///< Check memory error
                        START_SYNCHRONIZE, ///< Start Synchronize USB for safe eject
                        END_SYNCHRONIZE, ///< Safe eject success
                        SAFE_EJECT, ///< Removed USB with safe eject
                        INTERRUPTED, ///< Synchronization or memory check interrupted
                        INTERRUPTED_NO_MEMORY ///< Synchronization or memory check interrupted without memories
                    };

                    typedef sigc::signal<void, MemoryEvent> MemorySignal; ///< The device events' signal

                    /**
                     * @fn	virtual ~MemoryEventManager();
                     * @brief	Class destroyer
                     * @date 29/07/2015
                     * @author Yadickson Soto
                     */
                    virtual ~MemoryEventManager();

                    /**
                     * @fn	void startEmitSignal();
                     * @brief	Start emit signals
                     * @date 29/07/2015
                     * @author Yadickson Soto
                     */
                    void startEmitSignal();

                    /**
                     * @fn	void stopEmitSignal();
                     * @brief	Stop emit signals
                     * @date 29/07/2015
                     * @author Yadickson Soto
                     */
                    void stopEmitSignal();

                    /**
                     * @fn void addMemoryEvent( MemoryEvent event );
                     * @brief Sets the signal that corresponds to the change event of a device flash.
                     * @date 29/07/2015
                     * @author Yadickson Soto
                     * @param event The new event
                     */
                    void addMemoryEvent ( MemoryEvent event );

                    /**
                     * @fn bool isCanEmitSignal ();
                     * @brief Gets if can emit signal.
                     * @date 29/07/2015
                     * @author Yadickson Soto
                     * @return True if can emit signal
                     */
                    bool isCanEmitSignal ();

                    /**
                     * @fn MemorySignal getMemorySignal();
                     * @brief Gets the signal that corresponds to the change event of a device flash.
                     * @date 29/07/2015
                     * @author Yadickson Soto
                     * @return The signal for the changing event.
                     */
                    MemorySignal getMemorySignal();

                    /**
                     * @fn	static MemoryEventManager * getInstance();
                     * @brief	Getter instance of the class
                     * @date 29/07/2015
                     * @author Yadickson Soto
                     * @return The instance of the class
                     */
                    static MemoryEventManager * getInstance();

                private:

                    MemorySignal memoryEventSignal;	///< The device event's signal
                    bool flagEmitSignal; ///< The flag emit signal

                    Smartmatic::System::SafeDispatcher rawSignal; ///< The safe signal

                    /**
                     * @fn	    void onRawObject ( void * object );
                     * @brief	Callback when emit sync signal.
                     * @author	Yadickson Soto
                     * @date	29/07/2015
                     * @param 	object The object to process
                     */
                    void onRawObject ( void * object );
            };

        } /* namespace Operation */
    } /* namespace SAES */
} /* namespace Smartmatic */

#endif /* SRC_OPERATION_MEMORYEVENTMANAGER_HXX_ */
