/**
 * @file   AbledController.hxx
 * @brief  Declaration of Abled controller.
 * @author manuel.arguelles
 * @date   Dec 21, 2017
 */

#ifndef ABLED_CONTROLLER_HXX
#define ABLED_CONTROLLER_HXX

#include <sigc++/sigc++.h>

namespace Smartmatic
{
    namespace SAES
    {
        namespace HW
        {
            namespace Operation
            {
                class AbleDButtonState;
            }
        }

        namespace Operation
        {
            namespace AudioVoting
            {
                /**
                 * @class  AbledController
                 * @brief  Basic abled controller class.
                 * @author manuel.arguelles
                 * @date   Dec 21, 2017
                 */
                class AbledController
                {
                    public:

                        /**
                         * @brief Declaration of up button signal type.
                         */
                        typedef sigc::signal<void> UpButtonSignal;

                        /**
                         * @brief Declaration of down button signal type.
                         */
                        typedef sigc::signal<void> DownButtonSignal;

                        /**
                         * @brief Declaration of left button signal type.
                         */
                        typedef sigc::signal<void> LeftButtonSignal;

                        /**
                         * @brief Declaration of right button signal type.
                         */
                        typedef sigc::signal<void> RightButtonSignal;

                        /**
                         * @brief Declaration of select button signal type.
                         */
                        typedef sigc::signal<void> SelectButtonSignal;

                        /**
                         * @brief Declaration of help button signal type.
                         */
                        typedef sigc::signal<void> HelpButtonSignal;

                        /**
                         * @brief Declaration of the speed up button signal type.
                         */
                        typedef sigc::signal<void> SpeedUpButtonSignal;

                        /**
                         * @brief Declaration of the speed down button signal type.
                         */
                        typedef sigc::signal<void> SpeedDownButtonSignal;

                        /**
                         * @brief Declaration of the volume up button signal type.
                         */
                        typedef sigc::signal<void> VolumeUpButtonSignal;

                        /**
                         * @brief Declaration of the volume down button signal type.
                         */
                        typedef sigc::signal<void> VolumeDownButtonSignal;

                    public:

                        /**
                         * @brief  Create an instance of the class
                         * @author manuel.arguelles
                         * @date   Dec 21, 2017
                         */
                        AbledController();

                        /**
                         * @brief  Destroy an instance of the class.
                         * @author manuel.arguelles
                         * @date   Dec 21, 2017
                         */
                        ~AbledController();

                    public:

                        /**
                         * @brief  Start broadcasting events.
                         * @author manuel.arguelles
                         * @date   Dec 21, 2017
                         */
                        void start();

                        /**
                         * @brief  Stop broadcasting events.
                         * @author manuel.arguelles
                         * @date   Dec 21, 2017
                         */
                        void stop();

                        /**
                         * @brief  Up button signal.
                         * @author manuel.arguelles
                         * @date   Dec 21, 2017
                         * @return The up button signal.
                         */
                        UpButtonSignal signalUpButton();

                        /**
                         * @brief  Down button signal.
                         * @author manuel.arguelles
                         * @date   Dec 21, 2017
                         * @return The down button signal.
                         */
                        DownButtonSignal signalDownButton();

                        /**
                         * @brief  Left button signal.
                         * @author manuel.arguelles
                         * @date   Dec 21, 2017
                         * @return The left button signal.
                         */
                        LeftButtonSignal signalLeftButton();

                        /**
                         * @brief  Right button signal.
                         * @author manuel.arguelles
                         * @date   Dec 21, 2017
                         * @return The right button signal.
                         */
                        RightButtonSignal signalRightButton();

                        /**
                         * @brief  Select button signal.
                         * @author manuel.arguelles
                         * @date   Dec 21, 2017
                         * @return The select button signal.
                         */
                        SelectButtonSignal signalSelectButton();

                        /**
                         * @brief  Help button signal.
                         * @author manuel.arguelles
                         * @date   Dec 21, 2017
                         * @return The help button signal.
                         */
                        HelpButtonSignal signalHelpButton();

                        /**
                         * @brief  Speed up button signal.
                         * @author eduardo.fernandez
                         * @date   Jan 24, 2018
                         */
                        SpeedUpButtonSignal signalSpeedUpButton();

                        /**
                         * @brief  Speed down button signal.
                         * @author manuel.arguelles
                         * @date   Jan 24, 2018
                         */
                        SpeedUpButtonSignal signalSpeedDownButton();

                        /**
                         * @brief  Volume up button signal.
                         * @author manuel.arguelles
                         * @date   Dec 29, 2017
                         */
                        VolumeUpButtonSignal signalVolumeUpButton();

                        /**
                         * @brief  Volume down button signal.
                         * @author manuel.arguelles
                         * @date   Dec 29, 2017
                         */
                        VolumeDownButtonSignal signalVolumeDownButton();

                    private:

                        /**
                         * @brief  Called when abled button is pressed.
                         * @author manuel.arguelles
                         * @date   Dec 21, 2017
                         * @param  state The state of the button.
                         */
                        void onButtonPressed(SAES::HW::Operation::AbleDButtonState* state);

                    private:

                        UpButtonSignal         m_signalUpButton;
                        DownButtonSignal       m_signalDownButton;
                        LeftButtonSignal       m_signalLeftButton;
                        RightButtonSignal      m_signalRightButton;
                        SelectButtonSignal     m_signalSelectButton;
                        HelpButtonSignal       m_signalHelpButton;
                        SpeedUpButtonSignal    m_signalSpeedUpButton;
                        SpeedDownButtonSignal  m_signalSpeedDownButton;
                        VolumeUpButtonSignal   m_signalVolumeUpButton;
                        VolumeDownButtonSignal m_signalVolumeDownButton;
                };

            } /* namespace AudioVoting */
        } /* namespace Operation */
    } /* namespace SAES */
} /* namespace Smartmatic */

#endif /* ABLED_CONTROLLER_HXX */
