/**
 * @file   AudioVotingPartySelection.cxx
 * @brief  Implementation of AudioVotingPartySelection class.
 * @author manuel.arguelles
 * @date   Oct 13, 2017
 */

#include "AudioVotingPartySelection.hxx"
#include <Configuration/MachineModelEnum.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Operation/MachineOperationManager.hxx>
#include <Resources/ElectionAudioResourcesManager.hxx>
#include <System/Audio/TextToSpeech/ITextToSpeech.hxx>
#include <System/OptParser.hxx>
#include <Voting/Election/election-schema.hxx>
#include <Voting/ElectoralConfiguration.hxx>

using namespace Smartmatic::Configuration;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::HW::Operation;
using namespace Smartmatic::SAES::Operation::AudioVoting;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::SAES::Voting::Election;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::System::Audio::TextToSpeech;
using namespace Smartmatic::System;

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

static const unsigned int INTERVAL_MS                      = 10;
static const unsigned int IDLE_INTERVAL_S                  = 10;
static const unsigned int SLEEP_AFTER_STOP_US              = 200000;
static const std::string  CONTEST_ANNOUNCEMENT_MESSAGE_KEY = "smartmatic.saes.operation.audiovoting.AudioVotingPartySelection.ContestAnnouncementMessage";
static const std::string  GENERAL_INSTRUCTIONS_MESSAGE_KEY = "smartmatic.saes.operation.audiovoting.AudioVotingPartySelection.GeneralInstructionsMessage";
static const std::string  PRESELECTION_MESSAGE_KEY         = "smartmatic.saes.operation.audiovoting.AudioVotingPartySelection.PreselectionMessage";
static const std::string  CONFIRMATION_MESSAGE_KEY         = "smartmatic.saes.operation.audiovoting.AudioVotingPartySelection.ConfirmationMessage";
static const std::string  EMPTY_PARTY_MESSAGE_KEY          = "smartmatic.saes.operation.audiovoting.AudioVotingPartySelection.EmptyPartyMessage";

ISMTTLog* AudioVotingPartySelection::m_logger = SMTTLogManager::GetLogger(
    "Smartmatic.SAES.Operation.AudioVoting.AudioVotingPartySelection",
    GETTEXT_PACKAGE);

/**
 * @brief  Gets the contest name key
 * @author manuel.arguelles
 * @date   Oct 13, 2017
 * @param  contest       The contest.
 * @param  configuration The electoral configuration.
 * @return The key of the contest name.
 */
static std::string getContestNameKey(Contest* contest, ElectoralConfiguration* configuration)
{
    Contest_class* contestClass;

    contestClass = configuration->getContestClassByCode(contest->getContest_class_code());

    return contestClass->getName_key();
}

AudioVotingPartySelection::AudioVotingPartySelection(ITextToSpeech* textToSpeech)
    : m_textToSpeech(textToSpeech)
    , m_contest()
    , m_electoralAudioResources(ElectionAudioResourcesManager::getInstance())
    , m_canReturn(false)
    , m_canInstructionsBeInterrupted(false)
    , m_optionPlayed(false)
{
}

AudioVotingPartySelection::~AudioVotingPartySelection()
{
}

void AudioVotingPartySelection::start(Contest* contest, bool canReturn)
{
    m_contest = contest;
    m_canReturn = canReturn;
    m_canInstructionsBeInterrupted = false;
    m_currentStage = INITIAL_STAGE;

    setInitialOption();

    SafeThread::start(100);
}

void AudioVotingPartySelection::preStart()
{
    std::string currentMachineModel;
    OptParser::getInstance()->getArg( OptParser::MACHINE_MODEL, currentMachineModel );

    m_selectConnection = m_abled.signalSelectButton().connect(
        sigc::mem_fun(*this, &AudioVotingPartySelection::handleSelectPress));
    m_helpConnection = m_abled.signalHelpButton().connect(
        sigc::mem_fun(*this, &AudioVotingPartySelection::handleHelpPress));
    m_leftConnection = m_abled.signalLeftButton().connect(
        sigc::mem_fun(*this, &AudioVotingPartySelection::handleLeftPress));
    m_rightConnection = m_abled.signalRightButton().connect(
        sigc::mem_fun(*this, &AudioVotingPartySelection::handleRightPress));
    m_upConnection = m_abled.signalUpButton().connect(
        sigc::mem_fun(*this, &AudioVotingPartySelection::handleUpPress));
    m_moreSpeedConnection = m_abled.signalSpeedUpButton().connect(
        sigc::mem_fun(*this, &AudioVotingPartySelection::handleSpeedPress));
    m_lessSpeedConnection = m_abled.signalSpeedDownButton().connect(
        sigc::mem_fun(*this, &AudioVotingPartySelection::handleSpeedPress));
}

void AudioVotingPartySelection::postStop()
{
    m_selectConnection.disconnect();
    m_helpConnection.disconnect();
    m_leftConnection.disconnect();
    m_rightConnection.disconnect();
    m_upConnection.disconnect();
    m_moreSpeedConnection.disconnect();
    m_lessSpeedConnection.disconnect();
}

bool AudioVotingPartySelection::process()
{
    if (m_currentStage == INITIAL_STAGE)
    {
        playInstructions();
        m_currentStage = SELECTION_STAGE;
        m_idleTime.reset();
    }
    else if (m_currentStage == SELECTION_STAGE && m_textToSpeech->isSpeaking())
    {
        m_idleTime.reset();
    }
    else if (m_currentStage == CONFIRMATION_STAGE)
    {
        confirmSelection();
        m_currentStage = END_STAGE;
        m_idleTime.reset();
    }
    else if (m_currentStage == RETURN_STAGE)
    {
        Glib::signal_timeout().connect_once(
            sigc::mem_fun(*this, &AudioVotingPartySelection::returnToCustomAttributeSelection),
            INTERVAL_MS);
    }
    else if (m_currentStage == END_STAGE)
    {
        Glib::signal_timeout().connect_once(
            sigc::mem_fun(*this, &AudioVotingPartySelection::end),
            INTERVAL_MS);
    }

    if (m_idleTime.elapsed() > IDLE_INTERVAL_S)
    {
        m_currentStage = INITIAL_STAGE;
    }

    return true;
}

void AudioVotingPartySelection::setInitialOption()
{
    ElectoralConfiguration* electoralConfiguration = \
        MachineOperationManager::getInstance()->getElectoralConfiguration();

    m_ballotOptions = electoralConfiguration->getBallotOptionPartyByContest(m_contest->getCode());
    m_ballotIterator = m_ballotOptions.begin();
}

void AudioVotingPartySelection::end()
{
    m_logger->Debug("Party selection ended");

    stop();

    if (m_ballotIterator == m_ballotOptions.end())
    {
        m_emptyPartySelectedSignal.emit();
    }
    else
    {
        Party party = (*m_ballotIterator)->getParty();

        m_partySelectedSignal.emit(party);
    }
}

std::string AudioVotingPartySelection::getCurrentPartyName()
{
    std::string partyName = m_generalAudioResources.getResource(EMPTY_PARTY_MESSAGE_KEY);

    if (m_ballotIterator != m_ballotOptions.end())
    {
        Party party = (*m_ballotIterator)->getParty();
        partyName = m_electoralAudioResources->getResource(party.getName() != NULL ? party.getName().get() : "null");
    }

    return partyName;
}

void AudioVotingPartySelection::playInstructions()
{
    std::string contestKey = getContestNameKey(m_contest, MachineOperationManager::getInstance()->getElectoralConfiguration());
    std::string contestName = m_electoralAudioResources->getResource(contestKey);
    std::string audioText = Glib::ustring::compose(m_generalAudioResources.getResource(CONTEST_ANNOUNCEMENT_MESSAGE_KEY), contestName);
    std::string partyName;

    m_optionPlayed = false;

    if (m_ballotIterator == m_ballotOptions.end())
    {
        partyName = m_generalAudioResources.getResource(EMPTY_PARTY_MESSAGE_KEY);
    }
    else
    {
        Party party = (*m_ballotIterator)->getParty();

        partyName = m_electoralAudioResources->getResource(party.getName().get());
    }

    std::string instructionsAudioText = Glib::ustring::compose(
        m_generalAudioResources.getResource(GENERAL_INSTRUCTIONS_MESSAGE_KEY),
        partyName);

    std::string instructions = audioText + " " + instructionsAudioText;

    m_logger->Debug(instructions);
    m_textToSpeech->stop();
    m_textToSpeech->speak(instructions);
    m_textToSpeech->waitWhileSpeaking();
    m_canInstructionsBeInterrupted = true;
}

void AudioVotingPartySelection::playSelection()
{
    std::string selection = "";

    if (m_ballotIterator == m_ballotOptions.end())
    {
        selection = m_generalAudioResources.getResource(EMPTY_PARTY_MESSAGE_KEY);
    }
    else
    {
        Party party = (*m_ballotIterator)->getParty();

        selection = m_electoralAudioResources->getResource(party.getName() != NULL ? party.getName().get() : "null");
    }

    std::string audioText = Glib::ustring::compose(m_generalAudioResources.getResource(PRESELECTION_MESSAGE_KEY), selection);

    m_logger->Debug(audioText);
    m_textToSpeech->stop();
    Glib::usleep(SLEEP_AFTER_STOP_US);
    m_textToSpeech->speak(audioText);
    m_optionPlayed = true;
}

void AudioVotingPartySelection::nextSelection()
{
    if (m_ballotIterator == m_ballotOptions.end())
    {
        m_ballotIterator = m_ballotOptions.begin();
    }
    else
    {
        ++m_ballotIterator;
    }

    playSelection();
}

void AudioVotingPartySelection::previousSelection()
{
    if (m_ballotIterator == m_ballotOptions.begin())
    {
        m_ballotIterator = m_ballotOptions.end();
    }
    else
    {
        --m_ballotIterator;
    }

    playSelection();
}

void AudioVotingPartySelection::confirmSelection()
{
    std::string confirmation = Glib::ustring::compose(m_generalAudioResources.getResource(CONFIRMATION_MESSAGE_KEY), getCurrentPartyName());

    m_logger->Debug(confirmation);
    m_textToSpeech->stop();
    m_textToSpeech->speak(confirmation);
    m_textToSpeech->waitWhileSpeaking();
}

void AudioVotingPartySelection::returnToCustomAttributeSelection()
{
    m_logger->Debug("Party selection ended");

    stop();
    m_returnRequestedSignal.emit();
}

void AudioVotingPartySelection::handleLeftPress()
{
    m_idleTime.reset();

    if (m_currentStage == INITIAL_STAGE && m_canInstructionsBeInterrupted)
    {
        playSelection();
    }
    else if (m_currentStage == SELECTION_STAGE)
    {
        previousSelection();
    }
}

void AudioVotingPartySelection::handleHelpPress()
{
    m_idleTime.reset();
    m_currentStage = INITIAL_STAGE;
}

void AudioVotingPartySelection::handleRightPress()
{
    m_idleTime.reset();

    if (m_currentStage == INITIAL_STAGE && m_canInstructionsBeInterrupted)
    {
        playSelection();
    }
    else if (m_currentStage == SELECTION_STAGE)
    {
        nextSelection();
    }
}

void AudioVotingPartySelection::handleSelectPress()
{
    m_idleTime.reset();

    if (m_currentStage == INITIAL_STAGE && m_canInstructionsBeInterrupted)
    {
        playSelection();
    }
    else if (m_currentStage == SELECTION_STAGE)
    {
        m_currentStage = CONFIRMATION_STAGE;
    }
}

void AudioVotingPartySelection::handleUpPress()
{
    m_idleTime.reset();

    if (m_currentStage == INITIAL_STAGE && m_canInstructionsBeInterrupted)
    {
        playSelection();
    }
    else if (m_currentStage == SELECTION_STAGE && m_canReturn)
    {
        m_currentStage = RETURN_STAGE;
    }
}

void AudioVotingPartySelection::handleSpeedPress()
{
    m_idleTime.reset();

    if (m_currentStage == INITIAL_STAGE && m_canInstructionsBeInterrupted)
    {
        m_textToSpeech->stop();
        Glib::usleep(SLEEP_AFTER_STOP_US);
        m_currentStage = INITIAL_STAGE;
    }
    else if (m_currentStage == SELECTION_STAGE && m_optionPlayed)
    {
        playSelection();
    }
    else if (m_currentStage == SELECTION_STAGE && !m_optionPlayed)
    {
        m_currentStage = INITIAL_STAGE;
    }
}

AudioVotingPartySelection::EmptyPartySelectedSignal AudioVotingPartySelection::signalEmptyPartySelected()
{
    return m_emptyPartySelectedSignal;
}

AudioVotingPartySelection::PartySelectedSignal AudioVotingPartySelection::signalPartySelected()
{
    return m_partySelectedSignal;
}

AudioVotingPartySelection::ReturnRequestedSignal AudioVotingPartySelection::signalReturnRequested()
{
    return m_returnRequestedSignal;
}
