
/**
 * @file   AudioVotingPartySelection.hxx
 * @brief  Declaration of AudioVotingPartySelection class.
 * @author manuel.arguelles
 * @date   Oct 13, 2017
 */

#ifndef AUDIO_VOTING_PARTY_SELECTION_HXX
#define AUDIO_VOTING_PARTY_SELECTION_HXX

#include "AbledController.hxx"
#include "AudioVotingResources.hxx"
#include <System/SafeThread.hxx>
#include <list>
#include <string>
#include <sigc++/sigc++.h>
#include <glibmm.h>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace System
    {
        namespace Audio
        {
            namespace TextToSpeech
            {
                class ITextToSpeech;
            }
        }
    }

    namespace SAES
    {
        namespace Voting
        {
            namespace Election
            {
                class Ballot_option;
                class Contest;
                class Party;
            }
        }

        namespace Resources
        {
            class ElectionAudioResourcesManager;
        }

        namespace Operation
        {
            namespace AudioVoting
            {
                /**
                 * @class  AudioVotingPartySelection
                 * @brief  Class to handle the party selection.
                 * @author manuel.arguelles
                 * @date   Oct 13, 2017
                 */
                class AudioVotingPartySelection : public System::SafeThread
                {
                    public:

                        typedef sigc::signal<void> EmptyPartySelectedSignal;

                        typedef sigc::signal<void, SAES::Voting::Election::Party&> PartySelectedSignal;

                        typedef sigc::signal<void> ReturnRequestedSignal;

                    public:

                        /**
                         * @brief  Creates an instance of the class.
                         * @author manuel.arguelles
                         * @date   Oct 13, 2017
                         * @param  textToSpeech The text to speech engine to use.
                         * Ownership of the pointer is not transferred.
                         */
                        explicit AudioVotingPartySelection(System::Audio::TextToSpeech::ITextToSpeech* textToSpeech);

                        /**
                         * @brief  Destroys an instance of the class.
                         * @author manuel.arguelles
                         * @date   Oct 13, 2017
                         */
                        ~AudioVotingPartySelection();

                    public:

                        /**
                         * @brief  Starts the selection process.
                         * @author manuel.arguelles
                         * @date   Oct 13, 2017
                         * @param  contest The contest with the parties.
                         * Ownership of contest is not transferred.
                         */
                        void start(SAES::Voting::Election::Contest* contest, bool canReturn);

                        /**
                         * @brief  Emitted when a party is selected.
                         * @author manuel.arguelles
                         * @date   Oct 13, 2017
                         * @return The party selected signal.
                         */
                        PartySelectedSignal signalPartySelected();

                        /**
                         * @brief  Emitted when no party is selected.
                         * @author manuel.arguelles
                         * @date   Oct 13, 2017
                         * @return The empty party selected signal.
                         */
                        EmptyPartySelectedSignal signalEmptyPartySelected();

                        /**
                         * @brief  Emitted when the user presses the back button.
                         * @author eduardo.fernandez
                         * @date   Nov 15, 2017
                         * @return The return requested signal.
                         */
                        ReturnRequestedSignal signalReturnRequested();

                    private:

                        /**
                         * @brief  Main loop of the selection process.
                         * @author manuel.arguelles
                         * @date   Dec 22, 2017
                         * @return True while the selection is in process, false when done.
                         */
                        virtual bool process();

                        /**
                         * @brief  Called before starting the selection process.
                         * @author manuel.arguelles
                         * @date   Dec 22, 2017
                         */
                        virtual void preStart();

                        /**
                         * @brief  Called right after the selection process.
                         * @author manuel.arguelles
                         * @date   Dec 22, 2017
                         */
                        virtual void postStop();

                        /**
                         * @brief  Finalizes the selection process.
                         * @author manuel.arguelles
                         * @date   Oct 13, 2017
                         */
                        void end();

                        /**
                         * @brief  Gets the current party option name.
                         * @author eduardo.fernandez
                         * @date   Nov 15, 2017
                         */
                        std::string getCurrentPartyName();

                        /**
                         * @brief  Plays the general instructions.
                         * @author manuel.arguelles
                         * @date   Oct 13, 2017
                         */
                        void playInstructions();

                        /**
                         * @brief  Goes to the next selection.
                         * @author manuel.arguelles
                         * @date   Oct 13, 2017
                         */
                        void nextSelection();

                        /**
                         * @brief  Goes to the previous selection.
                         * @author manuel.arguelles
                         * @date   Oct 13, 2017
                         */
                        void previousSelection();

                        /**
                         * @brief  Returns to the custom attribute selection stage.
                         * @author eduardo.fernandez
                         * @date   Nov 15, 2017
                         */
                        void returnToCustomAttributeSelection();

                        /**
                         * @brief  Plays the current party.
                         * @author manuel.arguelles
                         * @date   Oct 13, 2017
                         */
                        void playSelection();

                        /**
                         * @brief  Plays current selection audio..
                         * @author eduardo.fernandez
                         * @date   Nov 15, 2017
                         */
                        void confirmSelection();

                        /**
                         * @brief  Sets the initial option.
                         * @author eduardo.fernandez
                         * @date   Nov 15, 2017
                         */
                        void setInitialOption();

                        /**
                         * @brief  Handles the left button press.
                         * @author manuel.arguelles
                         * @date   Dec 22, 2017
                         */
                        void handleLeftPress();

                        /**
                         * @brief  Handles the help button press.
                         * @author manuel.arguelles
                         * @date   Dec 22, 2017
                         */
                        void handleHelpPress();

                        /**
                         * @brief  Handles the right button press.
                         * @author manuel.arguelles
                         * @date   Dec 22, 2017
                         */
                        void handleRightPress();

                        /**
                         * @brief  Handles the select button press.
                         * @author manuel.arguelles
                         * @date   Dec 22, 2017
                         */
                        void handleSelectPress();

                        /**
                         * @brief  Handles the up button press.
                         * @author manuel.arguelles
                         * @date   Dec 22, 2017
                         */
                        void handleUpPress();

                        /**
                         * @brief  Handles the speed button press.
                         * @author manuel.arguelles
                         * @date   Mar 28, 2018
                         */
                        void handleSpeedPress();

                    private:

                        enum Stage
                        {
                            INITIAL_STAGE,
                            SELECTION_STAGE,
                            CONFIRMATION_STAGE,
                            RETURN_STAGE,
                            END_STAGE
                        };

                    private:

                        System::Audio::TextToSpeech::ITextToSpeech*                 m_textToSpeech;
                        SAES::Voting::Election::Contest*                            m_contest;
                        std::list<SAES::Voting::Election::Ballot_option*>           m_ballotOptions;
                        std::list<SAES::Voting::Election::Ballot_option*>::iterator m_ballotIterator;
                        SAES::Resources::ElectionAudioResourcesManager*             m_electoralAudioResources;
                        AudioVotingResources                                        m_generalAudioResources;
                        PartySelectedSignal                                         m_partySelectedSignal;
                        EmptyPartySelectedSignal                                    m_emptyPartySelectedSignal;
                        ReturnRequestedSignal                                       m_returnRequestedSignal;
                        bool                                                        m_canReturn;
                        Glib::Timer                                                 m_idleTime;
                        AbledController                                             m_abled;
                        sigc::connection                                            m_selectConnection;
                        sigc::connection                                            m_helpConnection;
                        sigc::connection                                            m_leftConnection;
                        sigc::connection                                            m_rightConnection;
                        sigc::connection                                            m_upConnection;
                        sigc::connection                                            m_moreSpeedConnection;
                        sigc::connection                                            m_lessSpeedConnection;
                        Stage                                                       m_currentStage;
                        bool                                                        m_canInstructionsBeInterrupted;
                        bool                                                        m_optionPlayed;
                        static Smartmatic::Log::ISMTTLog*                           m_logger;
                };

            } /* namespace AudioVoting */
        } /* namespace Operation */
    } /* namespace SAES */
} /* namespace Smartmatic */

#endif /* AUDIO_VOTING_PARTY_SELECTION_HXX */
