/**
 * @file   AudioVotingReviewConfirmation.hxx
 * @brief  Declaration of AudioVotingReviewConfirmation class.
 * @author eduardo.fernandez
 * @date   Dec 20, 2017
 */

#ifndef AUDIO_VOTING_REVIEW_CONFIRMATION_HXX
#define AUDIO_VOTING_REVIEW_CONFIRMATION_HXX

#include "AbledController.hxx"
#include "AudioVotingResources.hxx"
#include <string>
#include <list>
#include <sigc++/sigc++.h>
#include <glibmm.h>
#include <System/SafeThread.hxx>
#include <Operation/Voting/VotingExperienceController.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace System
    {
        namespace Audio
        {
            namespace TextToSpeech
            {
                class ITextToSpeech;
            }
        }
    }

    namespace SAES
    {
        namespace Operation
        {
            namespace AudioVoting
            {

                /**
                 * @class  AudioVotingReviewConfirmation
                 * @brief  Class to handle the audio voting review confirmation.
                 * @author eduardo.fernandez
                 * @date   Dec 20, 2017
                 */
                class AudioVotingReviewConfirmation : public System::SafeThread
                {
                    public:

                        typedef sigc::signal<void, bool> DoneSignal;

                        typedef sigc::signal<void> ReturnSignal;

                    public:

                        /**
                         * @brief  Creates an instance of the class.
                         * @author eduardo.fernandez
                         * @date   Dec 20, 2017
                         * @param  textToSpeech The text to speech engine to use.
                         * Ownership of the pointer is not transferred.
                         */
                        explicit AudioVotingReviewConfirmation(System::Audio::TextToSpeech::ITextToSpeech* textToSpeech);

                        /**
                         * @brief  Destroys an instance of the class.
                         * @author eduardo.fernandez
                         * @date   Dec 20, 2017
                         */
                        ~AudioVotingReviewConfirmation();

                    public:

                        /**
                         * @brief  Starts the review confirmation process.
                         * @author eduardo.fernandez
                         * @date   Dec 20, 2017
                         */
                        void start();

                        /**
                         * @brief  Emitted when the review confirmation process is completed.
                         * @author eduardo.fernandez
                         * @date   Dec 20, 2017
                         * @return The done signal.
                         */
                        DoneSignal signalDone();

                        /**
                         * @brief  Emitted on request to return to the previous stage.
                         * @author manuel.arguelles
                         * @date   Feb 23, 2018
                         * @return The return signal.
                         */
                        ReturnSignal signalReturn();

                    private:

                        /**
                         * @brief  Called before starting the selection process.
                         * @author manuel.arguelles
                         * @date   Dec 26, 2017
                         */
                        virtual void preStart();

                        /**
                         * @brief  Main loop of the selection process.
                         * @author manuel.arguelles
                         * @date   Dec 26, 2017
                         * @return True while the selection process is active, false when not.
                         */
                        virtual bool process();

                        /**
                         * @brief  Called right after the selection process.
                         * @author manuel.arguelles
                         * @date   Dec 26, 2017
                         */
                        virtual void postStop();

                        /**
                         * @brief  Finalizes the review confirmation process.
                         * @author eduardo.fernandez
                         * @date   Dec 20, 2017
                         */
                        void end();

                        /**
                         * @brief  Plays the stage instructions.
                         * @author manuel.arguelles
                         * @date   Dec 28, 2017
                         */
                        void playInstructions();

                        /**
                         * @brief  Plays the current option.
                         * @author manuel.arguelles
                         * @date   Dec 28, 2017
                         */
                        void playCurrentOption();

                        /**
                         * @brief  Returns to the previous stage.
                         * @author manuel.arguelles
                         * @date   Feb 23, 2018
                         */
                        void returnToPreviousStage();

                        /**
                         * @brief  Handles the help button press.
                         * @author manuel.arguelles
                         * @date   Dec 26, 2017
                         */
                        void handleHelpPress();

                        /**
                         * @brief  Handles the left/right buttons press.
                         * @author manuel.arguelles
                         * @date   Dec 26, 2017
                         */
                        void handleLeftOrRightPress();

                        /**
                         * @brief  Handles the select button press.
                         * @author manuel.arguelles
                         * @date   Dec 26, 2017
                         */
                        void handleSelectPress();

                        /**
                         * @brief  Handles the up button press.
                         * @author manuel.arguelles
                         * @date   Feb 23, 2018
                         */
                        void handleUpPress();

                        /**
                         * @brief  Handles the speed button press.
                         * @author manuel.arguelles
                         * @date   Mar 28, 2018
                         */
                        void handleSpeedPress();

                    private:

                        enum Stage
                        {
                            INITIAL_STAGE,
                            SELECTION_STAGE,
                            END_STAGE
                        };

                    private:

                        System::Audio::TextToSpeech::ITextToSpeech*    m_textToSpeech;
                        AudioVotingResources                           m_generalAudioResources;
                        DoneSignal                                     m_doneSignal;
                        ReturnSignal                                   m_returnSignal;
                        Glib::Timer                                    m_idleTime;
                        bool                                           m_editSelections;
                        AbledController                                m_abled;
                        Stage                                          m_currentStage;
                        sigc::connection                               m_selectConnection;
                        sigc::connection                               m_helpConnection;
                        sigc::connection                               m_leftConnection;
                        sigc::connection                               m_rightConnection;
                        sigc::connection                               m_upConnection;
                        sigc::connection                               m_moreSpeedConnection;
                        sigc::connection                               m_lessSpeedConnection;
                        bool                                           m_canInstructionsBeInterrupted;
                        bool                                           m_optionPlayed;
                        static Smartmatic::Log::ISMTTLog*              m_logger;
                };

            } /* namespace AudioVoting */
        } /* namespace Operation */
    } /* namespace SAES */
} /* namespace Smartmatic */

#endif /* AUDIO_VOTING_REVIEW_CONFIRMATION_HXX */

