/**
 * @file   AudioVotingSelectionsEdition.hxx
 * @brief  Declaration of AudioVotingSelectionsEdition class.
 * @author eduardo.fernandez
 * @date   Dec 11, 2017
 */

#ifndef AUDIO_VOTING_SELECTIONS_EDITION_HXX
#define AUDIO_VOTING_SELECTIONS_EDITION_HXX

#include "AbledController.hxx"
#include "AudioVotingResources.hxx"
#include <string>
#include <list>
#include <sigc++/sigc++.h>
#include <System/SafeThread.hxx>
#include <Operation/Voting/VotingExperienceController.hxx>
#include <Voting/Vote/vote-schema.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace System
    {
        namespace Audio
        {
            namespace TextToSpeech
            {
                class ITextToSpeech;
            }
        }
    }

    namespace SAES
    {
        namespace Voting
        {
            namespace Election
            {
                class Eligible_option;
                class Contest;
            }
        }

        namespace Operation
        {
            namespace AudioVoting
            {

                /**
                 * @class  AudioVotingSelectionsEdition
                 * @brief  Class to handle the audio voting selections edition.
                 * @author eduardo.fernandez
                 * @date   Dec 11, 2017
                 */
                class AudioVotingSelectionsEdition : public System::SafeThread
                {
                    public:

                        typedef sigc::signal<void, int, int> OptionSelectedSignal;
                        typedef sigc::signal<void> DoneSignal;

                    public:

                        /**
                         * @brief  Creates an instance of the class.
                         * @author eduardo.fernandez
                         * @date   Dec 11, 2017
                         * @param  textToSpeech The text to speech engine to use.
                         * Ownership of the pointer is not transferred.
                         */
                        explicit AudioVotingSelectionsEdition(System::Audio::TextToSpeech::ITextToSpeech* textToSpeech);

                        /**
                         * @brief  Destroys an instance of the class.
                         * @author eduardo.fernandez
                         * @date   Dec 11, 2017
                         */
                        ~AudioVotingSelectionsEdition();

                    public:

                        /**
                         * @brief  Starts the edition process.
                         * @author eduardo.fernandez
                         * @date   Dec 11, 2017
                         */
                        void start();

                        /**
                         * @brief  Emitted when an option is selected for edition.
                         * @author eduardo.fernandez
                         * @date   Dec 11, 2017
                         */
                        OptionSelectedSignal signalOptionSelected();

                        /**
                         * @brief  Emitted when the edition process is completed.
                         * @author eduardo.fernandez
                         * @date   Dec 11, 2017
                         */
                        DoneSignal signalDone();

                    private:

                        /**
                         * @brief  Called before starting the selection process.
                         * @author manuel.arguelles
                         * @date   Dec 27, 2017
                         */
                        virtual void preStart();

                        /**
                         * @brief  Called right after the selection process.
                         * @author manuel.arguelles
                         * @date   Dec 27, 2017
                         */
                        virtual void postStop();

                        /**
                         * @brief  Main loop of the selection process.
                         * @author manuel.arguelles
                         * @date   Dec 27, 2017
                         * @return True while the selection is in process, false when done.
                         */
                        virtual bool process();

                        /**
                         * @brief  Finalizes the edition process.
                         * @author eduardo.fernandez
                         * @date   Dec 11, 2017
                         */
                        void end();

                        /**
                         * @brief  Finalizes the edition process with a request
                         * to the previous stage.
                         * @author manuel.arguelles
                         * @date   Dec 27, 2017
                         */
                        void returnRequested();

                        /**
                         * @brief  Moves and plays the next selection.
                         * @author manuel.arguelles
                         * @date   Feb 19, 2018
                         */
                        void nextSelection();

                        /**
                         * @brief  Moves and plays the previous selection.
                         * @author manuel.arguelles
                         * @date   Feb 19, 2018
                         */
                        void previousSelection();

                        /**
                         * @brief  Parse the selection option into an audible message to play.
                         * @author manuel.arguelles
                         * @date   Feb 19, 2018
                         * @param  selectedOption The selected option text.
                         * @return The text with the message to play.
                         */
                        std::string parseSelectedOptionAudioMessage(const std::string& selectedOption);

                        /**
                         * @brief  Plays the given option.
                         * @author manuel.arguelles
                         * @date   Feb 19, 2018
                         * @param  selectedOption The option to play.
                         */
                        void playSelectedOption(const std::string& selectedOption);

                        /**
                         * @brief  Plays the general instructions.
                         * @author manuel.arguelles
                         * @date   Feb 19, 2018
                         */
                        void playInstructions();

                        /**
                         * @brief  Handles the left button press.
                         * @author manuel.arguelles
                         * @date   Dec 27, 2017
                         */
                        void handleLeftPress();

                        /**
                         * @brief  Handles the right button press.
                         * @author manuel.arguelles
                         * @date   Dec 27, 2017
                         */
                        void handleRightPress();

                        /**
                         * @brief  Handles the select button press.
                         * @author manuel.arguelles
                         * @date   Dec 27, 2017
                         */
                        void handleSelectPress();

                        /**
                         * @brief  Handles the help button press.
                         * @author manuel.arguelles
                         * @date   Dec 27, 2017
                         */
                        void handleHelpPress();

                        /**
                         * @brief  Handles the up button press.
                         * @author manuel.arguelles
                         * @date   Dec 27, 2017
                         */
                        void handleUpPress();

                        /**
                         * @brief  Handles the speed button press.
                         * @author manuel.arguelles
                         * @date   Mar 28, 2018
                         */
                        void handleSpeedPress();

                    private:

                        enum Stage
                        {
                            INITIAL_STAGE,
                            SELECTION_STAGE,
                            RETURN_STAGE,
                            FINAL_STAGE
                        };

                    private:

                        System::Audio::TextToSpeech::ITextToSpeech*     m_textToSpeech;
                        AudioVotingResources                            m_generalAudioResources;
                        OptionSelectedSignal                            m_optionSelectedSignal;
                        DoneSignal                                      m_doneSignal;
                        Smartmatic::SAES::Voting::Vote::Vote::SIterator m_currentSelection;
                        std::vector<std::string>                        m_selectedOptions;
                        size_t                                          m_selectedOptionsIndex;
                        size_t                                          m_currentMementoPosition;
                        size_t                                          m_currentMementoContestsIndex;
                        AbledController                                 m_abled;
                        Stage                                           m_currentStage;
                        sigc::connection                                m_leftConnection;
                        sigc::connection                                m_rightConnection;
                        sigc::connection                                m_selectConnection;
                        sigc::connection                                m_helpConnection;
                        sigc::connection                                m_upConnection;
                        sigc::connection                                m_moreSpeedConnection;
                        sigc::connection                                m_lessSpeedConnection;
                        Glib::Timer                                     m_idleTime;
                        bool                                            m_canInstructionsBeInterrupted;
                        bool                                            m_optionPlayed;
                        static Smartmatic::Log::ISMTTLog*               m_logger;
                };

            } /* namespace AudioVoting */
        } /* namespace Operation */
    } /* namespace SAES */
} /* namespace Smartmatic */

#endif /* AUDIO_VOTING_SUCCESSOR_SELECTION_HXX */

