/**
 * @file   AudioVotingSelectionsReview.CXX
 * @brief  Implementation of AudioVotingSelectionsReview class.
 * @author eduardo.fernandez
 * @date   Oct 31, 2017
 */

#include "AudioVotingSelectionsReview.hxx"
#include <Configuration/MachineModelEnum.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Operation/MachineOperationManager.hxx>
#include <System/Audio/TextToSpeech/ITextToSpeech.hxx>
#include <System/OptParser.hxx>
#include <System/Utils/StringUtils.hxx>
#include <Voting/Election/election-schema.hxx>
#include <Voting/ElectoralConfiguration.hxx>
#include <Voting/Vote/vote-schema.hxx>

using namespace Smartmatic::Configuration;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::HW::Operation;
using namespace Smartmatic::SAES::Operation::AudioVoting;
using namespace Smartmatic::SAES::Operation::Voting;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::SAES::Voting::Election;
using namespace Smartmatic::SAES::Voting::Vote;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::System::Audio::TextToSpeech;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::System;

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

static const unsigned int INTERVAL_MS                      = 10;
static const unsigned int IDLE_INTERVAL_S                  = 10;
static const unsigned int SLEEP_AFTER_STOP_US              = 200000;
static const std::string  GENERAL_INSTRUCTIONS_MESSAGE_KEY = "smartmatic.saes.operation.AudioVoting.AudioVotingSelectionsReview.GeneralInstructionsMessage";

ISMTTLog* AudioVotingSelectionsReview::m_logger = SMTTLogManager::GetLogger(
    "Smartmatic.SAES.Operation.AudioVoting.AudioVotingSelectionsReview",
    GETTEXT_PACKAGE);

AudioVotingSelectionsReview::AudioVotingSelectionsReview(ITextToSpeech* textToSpeech)
    : m_textToSpeech(textToSpeech)
    , m_canInstructionsBeInterrupted(false)
    , m_playOneReview(true)
    , m_shouldAutoPlay(true)
    , m_optionPlayed(false)
{
}

AudioVotingSelectionsReview::~AudioVotingSelectionsReview()
{
}

void AudioVotingSelectionsReview::start()
{
    m_currentStage = INITIAL_STAGE;
    m_canInstructionsBeInterrupted = false;
    m_selectedOptions = MachineOperationManager::getInstance()->getVotingExperienceController()->getSelectedOptions(false);
    m_currentOption = m_selectedOptions.begin();

    SafeThread::start(100);
}

void AudioVotingSelectionsReview::preStart()
{
    std::string currentMachineModel;

    OptParser::getInstance()->getArg( OptParser::MACHINE_MODEL, currentMachineModel );

    m_downConnection = m_abled.signalDownButton().connect(
        sigc::mem_fun(*this, &AudioVotingSelectionsReview::handleDownPress));
    m_helpConnection = m_abled.signalHelpButton().connect(
        sigc::mem_fun(*this, &AudioVotingSelectionsReview::handleHelpPress));
    m_upConnection = m_abled.signalUpButton().connect(
        sigc::mem_fun(*this, &AudioVotingSelectionsReview::handleUpPress));
    m_leftConnection = m_abled.signalLeftButton().connect(
        sigc::mem_fun(*this, &AudioVotingSelectionsReview::handleLeftPress));
    m_rightConnection = m_abled.signalRightButton().connect(
        sigc::mem_fun(*this, &AudioVotingSelectionsReview::handleRightPress));
    m_moreSpeedConnection = m_abled.signalSpeedUpButton().connect(
        sigc::mem_fun(*this, &AudioVotingSelectionsReview::handleSpeedPress));
    m_lessSpeedConnection = m_abled.signalSpeedDownButton().connect(
        sigc::mem_fun(*this, &AudioVotingSelectionsReview::handleSpeedPress));
}

void AudioVotingSelectionsReview::postStop()
{
    m_downConnection.disconnect();
    m_helpConnection.disconnect();
    m_upConnection.disconnect();
    m_leftConnection.disconnect();
    m_rightConnection.disconnect();
    m_moreSpeedConnection.disconnect();
    m_lessSpeedConnection.disconnect();
}

bool AudioVotingSelectionsReview::process()
{
    if (m_currentStage == INITIAL_STAGE)
    {
        m_playOneReview = true;
        m_shouldAutoPlay = true;
        m_currentOption = m_selectedOptions.begin();
        playInstructions();
        m_idleTime.reset();
        m_currentStage = REVIEW_STAGE;
    }
    else if (m_currentStage == REVIEW_STAGE && m_idleTime.elapsed() > IDLE_INTERVAL_S)
    {
        m_currentStage = INITIAL_STAGE;
        m_idleTime.reset();
    }
    else if (m_currentStage == REVIEW_STAGE && m_playOneReview)
    {
        playCurrentSelection();
        m_idleTime.reset();
        m_playOneReview = false;
    }
    else if (m_currentStage == REVIEW_STAGE && m_textToSpeech->isSpeaking())
    {
        m_idleTime.reset();
    }
    else if (m_currentStage == REVIEW_STAGE)
    {
        autoPlay();
    }
    else if (m_currentStage == END_STAGE)
    {
        Glib::signal_timeout().connect_once(
            sigc::mem_fun(*this, &AudioVotingSelectionsReview::end),
            INTERVAL_MS);
    }

    return true;
}

void AudioVotingSelectionsReview::playCurrentSelection()
{
    std::string selectedOption = (m_currentOption != m_selectedOptions.end() ? *m_currentOption : *(m_currentOption - 1));
    std::vector<std::string> fields;
    std::string audioText;

    StringUtils::splitToVector(selectedOption, " | ", fields);

    for (size_t i = 2; i < fields.size(); ++i)
    {
        audioText += Glib::ustring::compose("%1 ", fields[i]);
    }

    audioText = audioText.substr(0, audioText.size() - 1);

    m_logger->Debug(audioText);
    m_textToSpeech->stop();
    m_textToSpeech->speak(audioText);
    m_optionPlayed = true;
}

void AudioVotingSelectionsReview::playInstructions()
{
    std::string generalInstructions = m_generalAudioResources.getResource(GENERAL_INSTRUCTIONS_MESSAGE_KEY);

    m_optionPlayed = false;

    m_logger->Debug(generalInstructions);
    m_textToSpeech->stop();
    m_textToSpeech->speak(generalInstructions);
    m_textToSpeech->waitWhileSpeaking();
    m_canInstructionsBeInterrupted = true;
}

void AudioVotingSelectionsReview::end()
{
    stop();
    m_doneSignal.emit();
}

void AudioVotingSelectionsReview::nextSelection()
{
    ++m_currentOption;

    if (m_currentOption == m_selectedOptions.end())
    {
        m_currentOption = m_selectedOptions.begin();
    }

    playCurrentSelection();
}

void AudioVotingSelectionsReview::previousSelection()
{
    if (m_currentOption == m_selectedOptions.begin())
    {
        m_currentOption = m_selectedOptions.end();
    }

    --m_currentOption;
    playCurrentSelection();
}

void AudioVotingSelectionsReview::handleDownPress()
{
    m_idleTime.reset();

    if (m_currentStage == REVIEW_STAGE)
    {
        m_shouldAutoPlay = false;
        m_textToSpeech->stop();
        Glib::usleep(SLEEP_AFTER_STOP_US);
        m_currentStage = END_STAGE;
    }
}

void AudioVotingSelectionsReview::handleLeftPress()
{
    m_idleTime.reset();

    if (m_currentStage == REVIEW_STAGE || (m_currentStage == INITIAL_STAGE && m_canInstructionsBeInterrupted))
    {
        m_currentStage = REVIEW_STAGE;
        m_playOneReview = false;
        previousSelection();
    }
}

void AudioVotingSelectionsReview::handleRightPress()
{
    m_idleTime.reset();

    if (m_currentStage == REVIEW_STAGE || (m_currentStage == INITIAL_STAGE && m_canInstructionsBeInterrupted))
    {
        m_currentStage = REVIEW_STAGE;
        m_playOneReview = false;
        nextSelection();
    }
}

void AudioVotingSelectionsReview::handleUpPress()
{
    m_idleTime.reset();
}

void AudioVotingSelectionsReview::handleHelpPress()
{
    m_idleTime.reset();

    if (m_currentStage == REVIEW_STAGE)
    {
        m_textToSpeech->stop();
        Glib::usleep(SLEEP_AFTER_STOP_US);
        m_currentStage = INITIAL_STAGE;
    }
}

void AudioVotingSelectionsReview::handleSpeedPress()
{
    m_idleTime.reset();

    if (m_currentStage == INITIAL_STAGE && m_canInstructionsBeInterrupted)
    {
        m_shouldAutoPlay = false;
        m_playOneReview = false;
        m_textToSpeech->stop();
        Glib::usleep(SLEEP_AFTER_STOP_US);
        m_currentStage = INITIAL_STAGE;
    }
    else if (m_currentStage == REVIEW_STAGE && m_optionPlayed)
    {
        playCurrentSelection();
    }
}

AudioVotingSelectionsReview::DoneSignal AudioVotingSelectionsReview::signalDone()
{
    return m_doneSignal;
}

void AudioVotingSelectionsReview::autoPlay()
{
    std::vector<std::string>::const_iterator lastOption = m_selectedOptions.end();

    --lastOption;

    if (!m_textToSpeech->isSpeaking() && m_shouldAutoPlay)
    {
        if (m_currentOption != lastOption)
        {
            nextSelection();
        }
        else
        {
            m_shouldAutoPlay = false;
        }
    }
}
