/**
 * @file   AudioVotingSelectionsReview.hxx
 * @brief  Declaration of AudioVotingSelectionsReview class.
 * @author eduardo.fernandez
 * @date   Oct 31, 2017
 */

#ifndef AUDIO_VOTING_SELECTIONS_REVIEW_HXX
#define AUDIO_VOTING_SELECTIONS_REVIEW_HXX

#include "AbledController.hxx"
#include "AudioVotingResources.hxx"
#include <string>
#include <list>
#include <sigc++/sigc++.h>
#include <System/SafeThread.hxx>
#include <Operation/Voting/VotingExperienceController.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace System
    {
        namespace Audio
        {
            namespace TextToSpeech
            {
                class ITextToSpeech;
            }
        }
    }

    namespace SAES
    {
        namespace Voting
        {
            namespace Election
            {
                class Eligible_option;
                class Contest;
            }
        }

        namespace Operation
        {
            namespace AudioVoting
            {

                /**
                 * @class  AudioVotingSelectionsReview
                 * @brief  Class to handle the audio voting selections review.
                 * @author eduardo.fernandez
                 * @date   Oct 31, 2017
                 */
                class AudioVotingSelectionsReview : public System::SafeThread
                {
                    public:

                        typedef sigc::signal<void> DoneSignal;

                    public:

                        /**
                         * @brief  Creates an instance of the class.
                         * @author eduardo.fernandez
                         * @date   Oct 31, 2017
                         * @param  textToSpeech The text to speech engine to use.
                         * Ownership of the pointer is not transferred.
                         */
                        explicit AudioVotingSelectionsReview(System::Audio::TextToSpeech::ITextToSpeech* textToSpeech);

                        /**
                         * @brief  Destroys an instance of the class.
                         * @author eduardo.fernandez
                         * @date   Oct 31, 2017
                         */
                        ~AudioVotingSelectionsReview();

                    public:

                        /**
                         * @brief  Starts the review process.
                         * @author eduardo.fernandez
                         * @date   Oct 31, 2017
                         */
                        void start();

                        /**
                         * @brief  Emitted when the review process is completed.
                         * @author eduardo.fernandez
                         * @date   Oct 31, 2017
                         */
                        DoneSignal signalDone();

                    private:

                        /**
                         * @brief  Main loop of the selection process.
                         * @author manuel.arguelles
                         * @date   Dec 22, 2017
                         * @return True while the selection process is active, false when not.
                         */
                        virtual bool process();

                        /**
                         * @brief  Called before starting the selection process.
                         * @author manuel.arguelles
                         * @date   Dec 22, 2017
                         */
                        virtual void preStart();

                        /**
                         * @brief  Called right after the selection process.
                         * @author manuel.arguelles
                         * @date   Dec 22, 2017
                         */
                        virtual void postStop();

                        /**
                         * @brief  Finalizes the review process.
                         * @author eduardo.fernandez
                         * @date   Oct 31, 2017
                         */
                        void end();

                        /**
                         * @brief  Plays the current selected option.
                         * @author manuel.arguelles
                         * @date   Feb 19, 2018
                         */
                        void playCurrentSelection();

                        /**
                         * @brief  Plays the stage instructions.
                         * @author manuel.arguelles
                         * @date   Dec 28, 2017
                         */
                        void playInstructions();

                        /**
                         * @brief  Plays the next selection.
                         * @author manuel.arguelles
                         * @date   Feb 19, 2018
                         */
                        void nextSelection();

                        /**
                         * @brief  Plays the previous selection.
                         * @author manuel.arguelles
                         * @date   Feb 19, 2018
                         */
                        void previousSelection();

                        /**
                         * @brief  Handles the select button press.
                         * @author manuel.arguelles
                         * @date   Dec 22, 2017
                         */
                        void handleDownPress();

                        /**
                         * @brief  Handles the left button press.
                         * @author manuel.arguelles
                         * @date   Feb 19, 2018
                         */
                        void handleLeftPress();

                        /**
                         * @brief  Handles the right button press.
                         * @author manuel.arguelles
                         * @date   Feb 19, 2018
                         */
                        void handleRightPress();

                        /**
                         * @brief  Handles the up button press.
                         * @author manuel.arguelles
                         * @date   Dec 22, 2017
                         */
                        void handleUpPress();

                        /**
                         * @brief  Handles the help button press.
                         * @author manuel.arguelles
                         * @date   Dec 22, 2017
                         */
                        void handleHelpPress();

                        /**
                         * @brief  Handles the speed button press.
                         * @author manuel.arguelles
                         * @date   Mar 28, 2018
                         */
                        void handleSpeedPress();

                        /**
                         * @brief  Plays the next selection automatically.
                         * @author manuel.arguelles
                         * @date   Feb 22, 2018
                         */
                        void autoPlay();

                    private:

                        enum Stage
                        {
                            INITIAL_STAGE,
                            REVIEW_STAGE,
                            END_STAGE
                        };

                    private:

                        System::Audio::TextToSpeech::ITextToSpeech*     m_textToSpeech;
                        AudioVotingResources                            m_generalAudioResources;
                        DoneSignal                                      m_doneSignal;
                        Stage                                           m_currentStage;
                        sigc::connection                                m_downConnection;
                        sigc::connection                                m_helpConnection;
                        sigc::connection                                m_upConnection;
                        sigc::connection                                m_leftConnection;
                        sigc::connection                                m_rightConnection;
                        sigc::connection                                m_moreSpeedConnection;
                        sigc::connection                                m_lessSpeedConnection;
                        AbledController                                 m_abled;
                        Glib::Timer                                     m_idleTime;
                        std::vector<std::string>                        m_selectedOptions;
                        std::vector<std::string>::const_iterator        m_currentOption;
                        bool                                            m_canInstructionsBeInterrupted;
                        bool                                            m_playOneReview;
                        bool                                            m_shouldAutoPlay;
                        bool                                            m_optionPlayed;
                        static Smartmatic::Log::ISMTTLog*               m_logger;
                };

            } /* namespace AudioVoting */
        } /* namespace Operation */
    } /* namespace SAES */
} /* namespace Smartmatic */

#endif /* AUDIO_VOTING_SUCCESSOR_SELECTION_HXX */

