/*
 * BeforePrintingVoteComplianceVerificator.hxx
 *
 *  Created on: Feb 06, 2018
 *      Author: eduardo.fernandez
 */

#ifndef BEFORE_PRINTING_VOTE_COMPLIANCE_VERIFICATOR_HXX
#define BEFORE_PRINTING_VOTE_COMPLIANCE_VERIFICATOR_HXX

#include <string>
#include <list>
#include <vector>
#include <map>
#include <Log/ISMTTLog.h>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Voting
        {
            class ElectoralConfiguration;

            namespace Vote
            {
                class Vote;
                class Selection;
            }
        }

        namespace Operation
        {
            namespace VoteCompliance
            {
                class VoteComplianceVerificationManager;

                /**
                 * @class	BeforePrintingVoteComplianceVerificator
                 * @brief	Header class BeforePrintingVoteComplianceVerificator.
                 * @author	eduardo.fernandez
                 * @date	Feb 06, 2018
                 */
                class BeforePrintingVoteComplianceVerificator
                {
                    public:

                        /**
                         * @fn	BeforePrintingVoteComplianceVerificator::BeforePrintingVoteComplianceVerificator ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration );
                         * @brief	Class constructor.
                         * @param	electoralConfiguration	electoral configuration.
                         * @author	eduardo.fernandez
                         * @date	Feb 06, 2018
                         */
                        explicit BeforePrintingVoteComplianceVerificator(Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration);

                        /**
                         * @fn	virtual BeforePrintingVoteComplianceVerificator::~BeforePrintingVoteComplianceVerificator();
                         * @brief	Finalizer.
                         * @author	eduardo.fernandez
                         * @date	Feb 06, 2018
                         */
                        virtual ~BeforePrintingVoteComplianceVerificator();


                        /**
                         * @fn  bool BeforePrintingVoteComplianceVerificator::checkMinimumNumberOfSelections(const Smartmatic::SAES::Voting::Vote::Vote &vote);
                         * @brief    Checks minimum number of selections in the given vote.
                         * @param    vote Vote containing the voter selections.
                         * @return   Whether the vote satisfies the minimum number of selections.
                         * @author   eduardo.fernandez
                         * @date     Feb 07, 2018
                         */
                        bool checkMinimumNumberOfSelections(const Smartmatic::SAES::Voting::Vote::Vote &vote);

                        /**
                         * @fn  bool BeforePrintingVoteComplianceVerificator::checkSelectionsBelongToSameParty(const Smartmatic::SAES::Voting::Vote::Vote &vote);
                         * @brief    Checks whether the given vote's selections belong to the same party.
                         * @param    vote Vote containing the voter selections.
                         * @return   Whether the given vote's selections belong to the same party.
                         * @author   eduardo.fernandez
                         * @date     Feb 07, 2018
                         */
                        bool checkSelectionsBelongToSameParty(const Smartmatic::SAES::Voting::Vote::Vote &vote);

                        /**
                         * @fn  bool BeforePrintingVoteComplianceVerificator::checkSelectionsApplicableToConstituency(const Smartmatic::SAES::Voting::Vote::Vote &vote);
                         * @brief    Checks whether the given vote's selections' party belong to the same contest.
                         * @param    vote Vote containing the voter selections.
                         * @return   Whether the given vote's selections' party belong to the same contest.
                         * @author   eduardo.fernandez
                         * @date     Feb 08, 2018
                         */
                        bool checkSelectionsApplicableToConstituency(const Smartmatic::SAES::Voting::Vote::Vote &vote);

                        /**
                         * @fn  bool BeforePrintingVoteComplianceVerificator::checkUniquenessInSelections(const Smartmatic::SAES::Voting::Vote::Vote &vote);
                         * @brief    Checks whether the given vote's selections are unique qithin the vote.
                         * @param    vote Vote containing the voter selections.
                         * @return   Whether the given vote's selections' party belong to the same contest.
                         * @author   eduardo.fernandez
                         * @date     Feb 08, 2018
                         */
                        bool checkUniquenessInSelections(const Smartmatic::SAES::Voting::Vote::Vote &vote);

                        /**
                         * @fn  bool BeforePrintingVoteComplianceVerificator::checkListHeaderResourceCorrectness(const Smartmatic::SAES::Voting::Vote::Vote &vote, const std::map < std::string, std::map < std::string, std::string > > &textToPrintMap);
                         * @brief    Checks whether the string resource of the parties to print matches the string resource
                         *           of the vote's parties.
                         * @param    vote Vote containing the voter selections.
                         * @param    textToPrintMap Map containing the parties to print.
                         * @return   Whether the string resource of the parties to print matches the string resource
                         *           of the vote's parties.
                         * @author   eduardo.fernandez
                         * @date     Feb 09, 2018
                         */
                        bool checkListHeaderResourceCorrectness(
                            const Smartmatic::SAES::Voting::Vote::Vote &vote,
                            std::map < std::string, std::map < std::string, std::string > > textToPrintMap);

                        /**
                         * @fn  bool BeforePrintingVoteComplianceVerificator::checkCumulativeNumberOfSelections(const Smartmatic::SAES::Voting::Vote::Vote &vote);
                         * @brief    Checks whether the cumulative number of selected candidates are less than or equal
                         *           to the number of candidates registered on the party.
                         * @param    vote Vote containing the voter selections.
                         * @return   Whether the cumulative number of selected candidates are less than or equal
                         *           to the number of candidates registered on the party.
                         * @author   eduardo.fernandez
                         * @date     Feb 08, 2018
                         */
                        bool checkCumulativeNumberOfSelections(const Smartmatic::SAES::Voting::Vote::Vote &vote);

                        /**
                         * @fn  bool BeforePrintingVoteComplianceVerificator::checkAllConditions(const Smartmatic::SAES::Voting::Vote::Vote &vote);
                         * @brief    Checks whether the given vote's selections are unique qithin the vote.
                         * @param    vote Vote containing the voter selections.
                         * @return   Whether the given vote's selections' party belong to the same contest.
                         * @author   eduardo.fernandez
                         * @date     Feb 08, 2018
                         */
                        bool checkAllConditions();

                    private:

                        static Smartmatic::Log::ISMTTLog * m_logger;	///< the logger
                        VoteComplianceVerificationManager * m_voteComplianceVerificationManager;
                };
            }
        }
    }
}

#endif /* BEFORE_PRINTING_VOTE_COMPLIANCE_VERIFICATOR_HXX */
