/*
 * VoteComplianceVerificationManager.cxx
 *
 *  Created on: Feb 10, 2018
 *      Author: eduardo.fernandez
 */

#include "VoteComplianceVerificationManager.hxx"

#include <glibmm.h>

#include <Configuration/s-a-e-s-functionality-conf-manager.h>
#include <Log/SMTTLogManager.h>
#include <Resources/election-string-resources-manager.h>
#include <Voting/Election/election-schema.hxx>
#include <Voting/ElectoralConfiguration.hxx>
#include <Voting/Eml/510.hxx>
#include <Voting/VotingDevice/logical-voter-group-schema.hxx>
#include <Voting/Vote/vote-schema.hxx>
#include <Voting/PBReports/vote-results-schema.hxx>
#include <Voting/PBReports/VoteStatisticsWrapper.hxx>
#include <System/Utils/StringUtils.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::Functionality::Configuration;
using namespace Smartmatic::SAES::Operation::VoteCompliance;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::SAES::Voting::Election;
using namespace Smartmatic::SAES::Voting::PBReports;
using namespace Smartmatic::SAES::Voting::VotingDevice;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::Log;

ISMTTLog * VoteComplianceVerificationManager::m_logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Operation.Voting.VoteComplianceVerificationManager", GETTEXT_PACKAGE );

/**
 * @fn static bool VoteComplianceVerificationManager::blankVoteIsSelected(const std::string &partyCode)
 * @brief  Checks whether given party code corresponds to a blank vote.
 * @param  partyCode Party code to check.
 * @return Whether given party code corresponds to a blank vote.
 * @author eduardo.fernandez
 * @date   Feb 07, 2018
 */
static bool blankVoteIsSelected(const std::string &partyCode)
{
    return partyCode.empty();
}

/**
 * @fn static bool VoteComplianceVerificationManager::listVoteIsSelected(const std::list < Eligible_option * > eligibleOptions)
 * @brief  Checks whether there is a list vote selected.
 * @param  eligibleOption Eligible option to check.
 * @return Whether there is a list vote selected.
 * @author eduardo.fernandez
 * @date   Feb 07, 2018
 */
static bool listVoteIsSelected(const Eligible_option *eligibleOption)
{
    return eligibleOption->getParent_code().empty();
}

/**
 * @fn static bool VoteComplianceVerificationManager::principalCandidatesAreSelected(const std::list < Eligible_option * > eligibleOptions)
 * @brief  Checks whether there is a principal candidate selected.
 * @param  eligibleOption Eligible option to check.
 * @return Whether there is a principal candidate selected.
 * @author eduardo.fernandez
 * @date   Feb 07, 2018
 */
static bool principalCandidatesAreSelected(const Eligible_option *eligibleOption)
{
    return eligibleOption->getBallot_options().getBallot_option()[0].getCategory() == Category::ELECTION_PRINCIPAL;
}

/**
 * @fn static bool VoteComplianceVerificationManager::standInCandidatesAreSelected(const std::list < Eligible_option * > eligibleOptions)
 * @brief  Checks whether there is a stand-in candidate selected.
 * @param  eligibleOption Eligible option to check.
 * @return Whether there is a stand-in candidate selected.
 * @author eduardo.fernandez
 * @date   Feb 07, 2018
 */
static bool standInCandidatesAreSelected(const Eligible_option *eligibleOption)
{
    return eligibleOption->getBallot_options().getBallot_option()[0].getCategory() == Category::ELECTION_STAND_IN;
}

/**
 * @fn static bool VoteComplianceVerificationManager::partyBelongsToPartyList(const std::list < Party * > partyList, const std::string &partyCode)
 * @brief  Checks whether the given party code belongs to the given party list.
 * @param  partyCode Party code to search.
 * @param  partyList Party list to check.
 * @return Whether the given party code belongs to the given party list.
 * @author eduardo.fernandez
 * @date   Feb 08, 2018
 */
static bool partyCodeBelongsToPartyList(const std::string &partyCode, const std::list < Party * >& partyList)
{
    bool found = false;
    std::list < Party * >::const_iterator partyIterator = partyList.begin();

    for (; partyIterator != partyList.end(); ++partyIterator)
    {
        if ((*partyIterator)->getCode() == partyCode)
        {
            found = true;
            break;
        }
    }

    return found;
}

/**
 * @fn static bool VoteComplianceVerificationManager::contestAppearsOnlyOnce(const std::string &contestCode, const Vote::Vote &vote)
 * @brief  Checks whether the given contest appears only once in the given vote.
 * @param  contestCode Contest code to check.
 * @param  vote Vote containing given contest.
 * @return Whether the given contest appears only once in the given vote.
 * @author eduardo.fernandez
 * @date   Feb 08, 2018
 */
static bool contestAppearsOnlyOnce(const std::string &contestCode, const Vote::Vote &vote)
{
    bool contestAppearsOnlyOnce = false;
    const Vote::Vote::SSequence selectionsSequence = vote.getSelections();
    Vote::Vote::SSequence::const_iterator selectionsSequenceIterator = selectionsSequence.begin();

    for (; selectionsSequenceIterator != selectionsSequence.end(); ++selectionsSequenceIterator)
    {
        const std::string currentContestCode = selectionsSequenceIterator->getContest();

        if (currentContestCode == contestCode)
        {
            if (contestAppearsOnlyOnce == false)
            {
                contestAppearsOnlyOnce = true;
            }
            else
            {
                contestAppearsOnlyOnce = false;
                break;
            }
        }
    }

    return contestAppearsOnlyOnce;
}

/**
 * @fn static bool VoteComplianceVerificationManager::partyAppearsOnlyOnce(const std::string &partyCode, const Vote::Vote &vote, ElectoralConfiguration * electoralConfiguration)
 * @brief  Checks whether the given party appears only once in the given vote.
 * @param  partyCode Party code to check.
 * @param  vote Vote containing given party.
 * @param  electoralConfiguration Electoral configuration.
 * @return Whether the given party appears only once in the given vote.
 * @author eduardo.fernandez
 * @date   Feb 08, 2018
 */
static bool partyAppearsOnlyOnce(const std::string &partyCode, const Vote::Vote &vote, ElectoralConfiguration * electoralConfiguration)
{
    bool partyAppearsOnlyOnce = false;
    const Vote::Vote::SSequence selectionsSequence = vote.getSelections();
    Vote::Vote::SSequence::const_iterator selectionsSequenceIterator = selectionsSequence.begin();

    for (; selectionsSequenceIterator != selectionsSequence.end(); ++selectionsSequenceIterator)
    {
        const std::string currentPartyCode = (selectionsSequenceIterator->getBallotOptions().size() > 0) ?
            electoralConfiguration->getPartyCodeByBallotOptionCode(selectionsSequenceIterator->getBallotOptions()[0]) : "";

        if ((currentPartyCode == partyCode) && (currentPartyCode.empty() == false))
        {
            if (partyAppearsOnlyOnce == false)
            {
                partyAppearsOnlyOnce = true;
            }
            else
            {
                partyAppearsOnlyOnce = false;
                break;
            }
        }
    }

    return partyAppearsOnlyOnce;
}

/**
 * @fn static bool VoteComplianceVerificationManager::ballotOptionAppearsOnlyOnce(const std::string &ballotOptionCode, const Vote::Vote &vote, ElectoralConfiguration * electoralConfiguration)
 * @brief  Checks whether the given ballot option appears only once in the given vote.
 * @param  ballotOptionCode Ballot option code to check.
 * @param  vote Vote containing given ballot option.
 * @param  electoralConfiguration Electoral configuration.
 * @return Whether the given ballot option appears only once in the given vote.
 * @author eduardo.fernandez
 * @date   Feb 08, 2018
 */
static bool ballotOptionAppearsOnlyOnce(const std::string &ballotOptionCode, const Vote::Vote &vote, ElectoralConfiguration * electoralConfiguration)
{
    bool ballotOptionAppearsOnlyOnce = false;
    bool continueSequenceIteration = true;
    const Vote::Vote::SSequence selectionsSequence = vote.getSelections();
    Vote::Vote::SSequence::const_iterator selectionsSequenceIterator = selectionsSequence.begin();

    for (; selectionsSequenceIterator != selectionsSequence.end() && continueSequenceIteration; ++selectionsSequenceIterator)
    {
        const Vote::Selection::BSequence ballotOptionsSequence = selectionsSequenceIterator->getBallotOptions();
        Vote::Selection::BSequence::const_iterator ballotOptionsSequenceIterator = ballotOptionsSequence.begin();

        for (; ballotOptionsSequenceIterator != ballotOptionsSequence.end(); ++ballotOptionsSequenceIterator)
        {
            const std::string currentBallotOptionCode = *ballotOptionsSequenceIterator;

            if (currentBallotOptionCode == ballotOptionCode)
            {
                if (ballotOptionAppearsOnlyOnce == false)
                {
                    ballotOptionAppearsOnlyOnce = true;
                }
                else
                {
                    ballotOptionAppearsOnlyOnce = false;
                    continueSequenceIteration = false;
                    break;
                }
            }
        }
    }

    return ballotOptionAppearsOnlyOnce;
}

/**
 * @fn static bool VoteComplianceVerificationManager::eligibleOptionsContainsBallotOption(const std::list < Eligible_option * > eligibleOptions, const std::string &ballotOptionCode)
 * @brief  Checks whether given eligible options contains given ballot option code.
 * @param  eligibleOptions Eligible options to iterate.
 * @param  ballotOptionCode Ballot option to search.
 * @return Whether given eligible options contains given ballot option code.
 * @author eduardo.fernandez
 * @date   Feb 09, 2018
 */
static bool eligibleOptionsContainsBallotOption(
    const std::list < Eligible_option * > &eligibleOptions,
    const std::string &ballotOptionCode)
{
    bool found = false;
    std::list < Eligible_option * >::const_iterator eligibleOptionsIterator = eligibleOptions.begin();

    for (; eligibleOptionsIterator != eligibleOptions.end() && found == false; ++eligibleOptionsIterator)
    {
        const Ballot_options::Ballot_optionSequence ballotOptionsSequence = (*eligibleOptionsIterator)->getBallot_options().getBallot_option();
        Ballot_options::Ballot_optionSequence::const_iterator ballotOptionsSequenceIterator = ballotOptionsSequence.begin();

        for (; ballotOptionsSequenceIterator != ballotOptionsSequence.end(); ++ballotOptionsSequenceIterator)
        {
            if (ballotOptionsSequenceIterator->getCode() == ballotOptionCode)
            {
                found = true;
                break;
            }
        }
    }

    return found;
}

/**
 * @fn static bool VoteComplianceVerificationManager::getSelectedCandidatesByCategory(ElectoralConfiguration * electoralConfiguration, const Vote::Selection &selection, const Category &category)
 * @brief  Gets the number of selected candidates by given category.
 * @param  electoralConfiguration Electoral configuration.
 * @param  selection Current voter selections.
 * @param  category Ballot option category to search.
 * @return The number of selected candidates by given category.
 * @author eduardo.fernandez
 * @date   Feb 09, 2018
 */
static size_t getSelectedCandidatesByCategory(
    ElectoralConfiguration * electoralConfiguration,
    const Vote::Selection &selection,
    const Category &category)
{
    size_t numberOfCandidates = 0;
    std::list < Eligible_option * > eligibleOptions;
    const std::string contestCode = selection.getContest();
    const std::string partyCode = (selection.getBallotOptions().size() > 0) ?
        electoralConfiguration->getPartyCodeByBallotOptionCode(selection.getBallotOptions()[0]) : "";

    if (partyCode.empty() == false)
    {
        if (category == Category::ELECTION_LIST)
        {
            eligibleOptions = electoralConfiguration->getEligibleOptionsListByContestParty(contestCode, partyCode);
        }
        else if (category == Category::ELECTION_PRINCIPAL)
        {
            eligibleOptions = electoralConfiguration->getEligibleOptionsPrincipalByContestParty(contestCode, partyCode);
        }
        else
        {
            eligibleOptions = electoralConfiguration->getEligibleOptionsStandInByContestParty(contestCode, partyCode);
        }

        const Vote::Selection::BSequence ballotOptionsSequence = selection.getBallotOptions();
        Vote::Selection::BSequence::const_iterator ballotOptionsSequenceIterator = ballotOptionsSequence.begin();

        for (; ballotOptionsSequenceIterator != ballotOptionsSequence.end(); ++ballotOptionsSequenceIterator)
        {
            if (eligibleOptionsContainsBallotOption(eligibleOptions, *ballotOptionsSequenceIterator))
            {
                ++numberOfCandidates;
            }
        }
    }

    return numberOfCandidates;
}

/**
 * @fn static bool VoteComplianceVerificationManager::mapPartyKeyMatchesVoteParty(ElectoralConfiguration *electoralConfiguration, const Vote::Vote &vote, const std::string &mapContestCode, const std::string &mapBallotOptionCode, const std::string &mapPartyKey)
 * @brief  Gets whether the party to print matches the vote party.
 * @param  electoralConfiguration Electoral configuration.
 * @param  vote Current voter selections.
 * @param  mapContestCode Contest code to print.
 * @param  mapBallotOptionCode Ballot option to print.
 * @param  mapPartyKey Key of the party to print.
 * @return Whether the party to print matches the vote party.
 * @author eduardo.fernandez
 * @date   Feb 09, 2018
 */
static bool mapPartyMatchesVoteParty(
    ElectoralConfiguration *electoralConfiguration,
    const Vote::Vote &vote,
    const std::string &mapContestCode,
    const std::string &mapBallotOptionCode,
    const std::string &mapPartyKey)
{
    bool partyKeyMatches = false;
    const Vote::Vote::SSequence selectionsSequence = vote.getSelections();
    Vote::Vote::SSequence::const_iterator selectionsSequenceIterator = selectionsSequence.begin();

    for (; selectionsSequenceIterator != selectionsSequence.end() && partyKeyMatches == false; ++selectionsSequenceIterator)
    {
        if (selectionsSequenceIterator->getContest() == mapContestCode)
        {
            const Vote::Selection::BSequence ballotOptionsSequence = selectionsSequenceIterator->getBallotOptions();
            Vote::Selection::BSequence::const_iterator ballotOptionsSequenceIterator = ballotOptionsSequence.begin();

            for (; ballotOptionsSequenceIterator != ballotOptionsSequence.end() && partyKeyMatches == false; ++ballotOptionsSequenceIterator)
            {
                if (*ballotOptionsSequenceIterator == mapBallotOptionCode)
                {
                    const std::string partyCode = electoralConfiguration->getPartyCodeByBallotOptionCode(*ballotOptionsSequenceIterator);

                    const std::list < Party * > partyList = electoralConfiguration->getMapContestParty()[mapContestCode];
                    std::list < Party * >::const_iterator partyListIterator = partyList.begin();

                    for (; partyListIterator != partyList.end(); ++partyListIterator)
                    {
                        if ((*partyListIterator)->getCode() == partyCode)
                        {
                            const std::string mapPartyName = ElectionStringResourcesManager::getInstance()->getResource(mapPartyKey);
                            const std::string partyName = ElectionStringResourcesManager::getInstance()->getResource((*partyListIterator)->getName().get());

                            if (mapPartyName == partyName)
                            {
                                partyKeyMatches = true;
                                break;
                            }
                        }
                    }
                }
            }
        }
    }

    return partyKeyMatches;
}

/**
 * @fn static bool VoteComplianceVerificationManager::preferenceVotesAreLessThanOrEqualToTotalVotes(const size_t totalVotes, const PartyResults::CandidateResultsOptional &candidateResults)
 * @brief  Gets whether the candidates total votes don't exceed the total number of votes.
 * @param  totalVotes The total number of votes.
 * @param  candidateResults The candidate results.
 * @return Whether the candidates total votes don't exceed the total number of votes.
 * @author eduardo.fernandez
 * @date   Feb 15, 2018
 */
static bool preferenceVotesAreLessThanOrEqualToTotalVotes(
    const size_t totalVotes,
    const PartyResults::CandidateResultsOptional &candidateResults)
{
    bool success = true;

    if (candidateResults.present())
    {
        ItemsResults::CandidateSequence candidatesSequence =  candidateResults->getCandidate();
        ItemsResults::CandidateSequence::const_iterator candidateResultsIterator = candidatesSequence.begin();

        for (; candidateResultsIterator != candidatesSequence.end(); ++candidateResultsIterator)
        {
            if (candidateResultsIterator->getPreference_votes() > totalVotes)
            {
                success = false;
                break;
            }
        }
    }

    return success;
}

/**
 * @fn static Contest_group *VoteComplianceVerificationManager::getContestGroupFromLogicalVoterGroup(ElectoralConfiguration *electoralConfiguration, const std::string &logicalVoterGroupCode)
 * @brief  Gets contest group from given logical voter group code.
 * @param  electoralConfiguration Electoral configuration.
 * @param  logicalVoterGroupCode Logical voter group code.
 * @return Contest group from given logical voter group code.
 * @author eduardo.fernandez
 * @date   Apr 03, 2018
 */
static Contest_group *getContestGroupFromLogicalVoterGroup(
    ElectoralConfiguration *electoralConfiguration,
    const std::string &logicalVoterGroupCode)
{
    std::string contestGroupCode = "";
    std::list < Logical_voter_group * > logicalVoterGroups = electoralConfiguration->getLogicalVoterGroupList();
    std::list < Logical_voter_group * >::const_iterator it = logicalVoterGroups.begin();
    const std::list < Logical_voter_group * >::const_iterator end = logicalVoterGroups.end();

    for (; it != end; ++it)
    {
        if ((*it)->getCode() == logicalVoterGroupCode)
        {
            contestGroupCode = (*it)->getContest_group_code();
            break;
        }
    }

    return electoralConfiguration->getContestGroup(contestGroupCode);
}

/**
 * @fn static bool VoteComplianceVerificationManager::currentContestHasSelections(const Vote::Vote &vote, const std::string &contestCode)
 * @brief  Gets whether given contest has selections.
 * @param  vote Current voter selections.
 * @param  contestCode Code of the contest to check.
 * @return Whether given contest has selections
 * @author eduardo.fernandez
 * @date   Apr 03, 2018
 */
static bool currentContestHasSelections(const Vote::Vote &vote, const std::string &contestCode)
{
    bool success = false;
    Vote::Vote::SSequence::const_iterator it = vote.getSelections().begin();
    const Vote::Vote::SSequence::const_iterator end = vote.getSelections().end();

    for (; it != end; ++it)
    {
        if (it->getContest() == contestCode)
        {
            success = true;
            break;
        }
    }

    return success;
}

/**
 * @fn static bool VoteComplianceVerificationManager::everyContestHasSelections(const Vote::Vote &vote, const Contest_group *contestGroup)
 * @brief  Gets whether every given contest group's contest has selections.
 * @param  vote Current voter selections.
 * @param  contestGroup Contest group to check.
 * @return Whether every given contest group's contest has selections.
 * @author eduardo.fernandez
 * @date   Apr 03, 2018
 */
static bool everyContestHasSelections(const Vote::Vote &vote, const Contest_group *contestGroup)
{
    bool success = true;
    Contest_group::Contest_codeSequence::const_iterator it = contestGroup->getContest_code().begin();
    const Contest_group::Contest_codeSequence::const_iterator end = contestGroup->getContest_code().end();

    for (; it != end; ++it)
    {
        if (currentContestHasSelections(vote, *it) == false)
        {
            success = false;
            break;
        }
    }

    return success;
}

/**
 * @fn static size_t VoteComplianceVerificationManager::getEmlTotalVoters(const Eml::EML& eml)
 * @brief  Gets a single EML total voters.
 * @param  eml EML structure to check.
 * @return A single EML total voters.
 * @author eduardo.fernandez
 * @date   Nov 14, 2018
 */
static size_t getEmlTotalVoters(const Eml::EML& eml)
{
    size_t totalVoters = 0;

    Eml::Count::ElectronicVotingCountersConstIterator       it  = eml.getCount().getElectronicVotingCounters().begin();
    const Eml::Count::ElectronicVotingCountersConstIterator end = eml.getCount().getElectronicVotingCounters().end();

    for (; it != end; ++it)
    {
        if (it->getType().get() == "T_TotalVoters")
        {
            totalVoters = StringUtils::stringToLong(*it);
            break;
        }
    }

    return totalVoters;
}

/**
 * @fn static size_t VoteComplianceVerificationManager::getEmlSumOfVoters(const Eml::EML& eml)
 * @brief  Gets the sum of voters.
 * @param  eml EML structure to check.
 * @return A single EML total voters.
 * @author eduardo.fernandez
 * @date   Nov 14, 2018
 */
static size_t getEmlSumOfVoters(const Eml::EML& eml)
{
    size_t sumOfVoters = 0;

    Eml::Count::ElectronicVotingCountersConstIterator       it  = eml.getCount().getElectronicVotingCounters().begin();
    const Eml::Count::ElectronicVotingCountersConstIterator end = eml.getCount().getElectronicVotingCounters().end();

    for (; it != end; ++it)
    {
        if (it->getType().get() != "T_TotalVoters")
        {
            sumOfVoters += StringUtils::stringToLong(*it);
        }
    }

    return sumOfVoters;
}

/**
 * @fn static void VoteComplianceVerificationManager::isVotesDepositedNumberValid(const Eml::Election::ContestsType::ContestConstIterator& itContest, std::string& errorMessage)
 * @brief Gets whether the votes deposited number matches the sum of valid and invalid votes.
 * @param[in]  itContest Current contest iterator.
 * @param[out] errorMessage Error message in case of failure.
 * @return Whether the votes deposited number matches the sum of valid and invalid votes.
 * @author eduardo.fernandez
 * @date   Nov 14, 2018
 */
static bool isVotesDepositedNumberValid(const Eml::Election::ContestsType::ContestConstIterator& itContest, std::string& errorMessage)
{
    bool success = true;

    if (itContest->getTotalVotes().present())
    {
        const Eml::TotalVotes::SelectionSequence selections = itContest->getTotalVotes().get().getSelection();

        Eml::TotalVotes::SelectionConstIterator       itSelection  = selections.begin();
        const Eml::TotalVotes::SelectionConstIterator endSelection = selections.end();

        size_t votesDeposited = 0;
        size_t validVotesSum  = 0;
        size_t invalidVotes   = 0;

        for (; itSelection != endSelection; ++itSelection)
        {
            if (itSelection->getCategory().get() != "GlobalSituation")
            {
                validVotesSum += itSelection->getValidVotes();

                Eml::Selection::CountMetricConstIterator       itMetric  = itSelection->getCountMetric().begin();
                const Eml::Selection::CountMetricConstIterator endMetric = itSelection->getCountMetric().end();

                for (; itMetric != endMetric; ++itMetric)
                {
                    if (itMetric->getType() == "VotesDeposited")
                    {
                        votesDeposited = *itMetric;
                    }
                    else if (itMetric->getType() == "BlankAndInvalidVotes")
                    {
                        invalidVotes = *itMetric;
                    }
                }
            }
        }

        if (votesDeposited != validVotesSum + invalidVotes)
        {
            errorMessage = Glib::ustring::compose( _( "Check EML votes deposited: votes deposited (%1) does not match the sum of valid (%2) and invalid votes (%3)." ), votesDeposited, validVotesSum, invalidVotes );
            success = false;
        }
    }

    return success;
}

/**
 * @fn static size_t VoteComplianceVerificationManager::getVotesDeposited(const Eml::TotalVotes::SelectionConstIterator& itSelection)
 * @brief  Gets given metric's votes deposited number.
 * @param  itSelection Current selection iterator.
 * @return Given metric's votes deposited number.
 * @author eduardo.fernandez
 * @date   Dec 07, 2018
 */
static size_t getVotesDeposited(const Eml::TotalVotes::SelectionConstIterator& itSelection)
{
    const Eml::Selection::CountMetricSequence metricSequence = itSelection->getCountMetric();

    Eml::Selection::CountMetricSequence::const_iterator       itMetric  = metricSequence.begin();
    const Eml::Selection::CountMetricSequence::const_iterator endMetric = metricSequence.end();

    size_t votesDeposited = 0;

    for (; itMetric != endMetric; ++itMetric)
    {
        if (itMetric->getType() == "VotesDeposited")
        {
            votesDeposited = *itMetric;
            break;
        }
    }

    return votesDeposited;
}

/**
 * @fn static bool VoteComplianceVerificationManager::isVotesDepositedValid(const size_t votesDeposited, const std::vector<std::string>& contestCustomCodes, VoteStatisticsWrapper * const voteStatisticsWrapper)
 * @brief  Checks whether the given votes deposited match the sum of votes counter for a list of selections' custom codes.
 * @param  votesDeposited Current selection's votes deposited number.
 * @param  contestCustomCodes Custom codes from current contest's selections.
 * @param  voteStatisticsWrapper The wrapper of the vote statistics.
 * @return Whether the given votes deposited match the sum of votes counter for a list of selections' custom codes.
 * @author eduardo.fernandez
 * @date   Dec 07, 2018
 */
static bool isVotesDepositedValid(
    const size_t votesDeposited,
    const std::vector<std::string>& contestCustomCodes,
    VoteStatisticsWrapper * const voteStatisticsWrapper)
{
    std::vector<VoteCounterByContest> votesCounterByContest = voteStatisticsWrapper->getVoteCountersByContest();

    size_t votesCounterSum = 0;

    for (size_t i = 0; i < votesCounterByContest.size(); ++i)
    {
        const std::string voteCounterCustomCode = votesCounterByContest[i].getCustomCode();
        const size_t      voteCounter           = votesCounterByContest[i].getCounter();

        for (size_t j = 0; j < contestCustomCodes.size(); ++j)
        {
            if (contestCustomCodes[j] == voteCounterCustomCode)
            {
                votesCounterSum += voteCounter;
            }
        }
    }

    const bool isValid = votesDeposited == votesCounterSum;

    return isValid;
}

/**
 * @fn static std::vector<std::string> VoteComplianceVerificationManager::getContestCustomCodes(const std::string& electionCategory, const Eml::TotalVotes::SelectionSequence& selections)
 * @brief  Gets the contest custom codes corresponding to the current contest's selections.
 * @param  electionCategory Current election category.
 * @param  selections Current contest's selections.
 * @return The contest custom codes corresponding to the current contest's selections.
 * @author eduardo.fernandez
 * @date   Dec 07, 2018
 */
static std::vector<std::string> getContestCustomCodes(
    const std::string& electionCategory,
    const Eml::TotalVotes::SelectionSequence& selections)
{
    std::vector<std::string> contestCustomCodes;

    Eml::TotalVotes::SelectionConstIterator       itSelection  = selections.begin();
    const Eml::TotalVotes::SelectionConstIterator endSelection = selections.end();

    for (; itSelection != endSelection; ++itSelection)
    {
        if (itSelection->getCategory().get() != "GlobalSituation")
        {
            const std::string customCode = electionCategory + itSelection->getCategory().get().substr(1);

            contestCustomCodes.push_back(customCode);
        }
    }

    return contestCustomCodes;
}

/**
 * @fn static bool VoteComplianceVerificationManager::areContestsSelectionsValid(const Eml::Count::ElectionConstIterator& itElection, VoteStatisticsWrapper * const voteStatisticsWrapper, std::string& errorMessage)
 * @brief Checks whether the votes deposited match the sum of vote counters in all given election's selections.
 * @param[in]  itElection Current election.
 * @param[in]  voteStatisticsWrapper The wrapper of the vote statistics.
 * @param[out] errorMessage The error message in case of an error.
 * @return Whether the votes deposited match the sum of vote counters in all given election's selections.
 * @author eduardo.fernandez
 * @date   Dec 07, 2018
 */
static bool areContestsSelectionsValid(
    const Eml::Count::ElectionConstIterator& itElection,
    VoteStatisticsWrapper * const voteStatisticsWrapper,
    std::string& errorMessage)
{
    bool success = true;

    const Eml::Contests::ContestSequence contestsSequence = itElection->getContests().getContest();

    Eml::Contests::ContestConstIterator       itContest  = contestsSequence.begin();
    const Eml::Contests::ContestConstIterator endContest = contestsSequence.end();

    for (; itContest != endContest && success; ++itContest)
    {
        if (itContest->getTotalVotes().present())
        {
            const Eml::TotalVotes::SelectionSequence selections = itContest->getTotalVotes().get().getSelection();
            const std::vector<std::string> contestCustomCodes =
                getContestCustomCodes(itElection->getElectionIdentifier().getElectionCategory().get(), selections);

            Eml::TotalVotes::SelectionConstIterator       itSelection  = selections.begin();
            const Eml::TotalVotes::SelectionConstIterator endSelection = selections.end();

            for (; itSelection != endSelection && success; ++itSelection)
            {
                if (itSelection->getCategory().get() != "GlobalSituation")
                {
                    const size_t      votesDeposited    = getVotesDeposited(itSelection);
                    const std::string selectionCategory = itSelection->getCategory().get();

                    if (isVotesDepositedValid(votesDeposited, contestCustomCodes, voteStatisticsWrapper) == false)
                    {
                        errorMessage = Glib::ustring::compose( _( "Check EML votes counter: votes deposited (%1) for selection category (%2) does not match the votes counter sum in the vote statistics file." ), votesDeposited, selectionCategory );
                        success = false;
                    }
                }
            }
        }
    }

    return success;
}

VoteComplianceVerificationManager::VoteComplianceVerificationManager (ElectoralConfiguration *electoralConfiguration)
: m_electoralConfiguration(electoralConfiguration)
{
}

VoteComplianceVerificationManager::~VoteComplianceVerificationManager()
{
}

bool VoteComplianceVerificationManager::checkMinimumNumberOfSelectionsInContest(const Vote::Selection &selection)
{
    bool success = false;
    std::string partyCode = (selection.getBallotOptions().size() > 0) ?
        m_electoralConfiguration->getPartyCodeByBallotOptionCode(selection.getBallotOptions()[0]) : "";

    if (blankVoteIsSelected(partyCode))
    {
        m_logger->Debug( Glib::ustring::compose( _( "Check minimum number of selections: blank vote has been selected for contest %1." ), selection.getContest() ) );
        return true;
    }

    const std::string contestCode = selection.getContest();
    const Vote::Selection::BSequence ballotOptionsSequence = selection.getBallotOptions();
    Vote::Selection::BSequence::const_iterator ballotOptionsSequenceIterator = ballotOptionsSequence.begin();

    for (; ballotOptionsSequenceIterator != ballotOptionsSequence.end(); ++ballotOptionsSequenceIterator)
    {
        const Eligible_option *eligibleOption = m_electoralConfiguration->getEligibleOptionFromBallotOptionCode(contestCode, *ballotOptionsSequenceIterator);

        if (eligibleOption == NULL)
        {
            m_logger->Debug( Glib::ustring::compose( _( "Check minimum number of selections: ballot option %1 is not registered for contest %2." ), *ballotOptionsSequenceIterator, selection.getContest() ) );
            break;
        }

        if (listVoteIsSelected(eligibleOption))
        {
            m_logger->Debug( Glib::ustring::compose( _( "Check minimum number of selections: list vote %1 has been selected for contest %2." ), eligibleOption->getCode(), selection.getContest() ) );
            success = true;
            break;
        }

        if (principalCandidatesAreSelected(eligibleOption))
        {
            m_logger->Debug( Glib::ustring::compose( _( "Check minimum number of selections: principal candidate %1 has been selected for contest %2." ), eligibleOption->getCode(), selection.getContest() ) );
            success = true;
            break;
        }

        if (standInCandidatesAreSelected(eligibleOption))
        {
            m_logger->Debug( Glib::ustring::compose( _( "Check minimum number of selections: stand-in candidate %1 has been selected for contest %2." ), eligibleOption->getCode(), selection.getContest() ) );
            success = true;
            break;
        }
    }

    if (success == false)
    {
        m_logger->Debug( Glib::ustring::compose( _( "Check minimum number of selections: minimum number of selections have not been satisfied for contest %1." ), selection.getContest() ) );
    }

    return success;
}

bool VoteComplianceVerificationManager::checkMinimumNumberOfSelections(const Vote::Vote &vote)
{
    m_logger->Debug( _( "Starting to check minimum number of selections." ) );

    bool success = false;
    const Vote::Vote::SSequence selectionsSequence = vote.getSelections();

    if (selectionsSequence.size() > 0)
    {
        Contest_group *contestGroup = getContestGroupFromLogicalVoterGroup(m_electoralConfiguration, vote.getLogicalVoterGroup());

        if (selectionsSequence.size() == contestGroup->getContest_code().size())
        {
            if (everyContestHasSelections(vote, contestGroup))
            {
                success = true;
                Vote::Vote::SSequence::const_iterator selectionsSequenceIterator = selectionsSequence.begin();

                for (; selectionsSequenceIterator != selectionsSequence.end(); ++selectionsSequenceIterator)
                {
                    success &= checkMinimumNumberOfSelectionsInContest(*selectionsSequenceIterator);
                }
            }
            else
            {
                m_logger->Debug( Glib::ustring::compose( _( "Not all logical voter group %1's contests have selections in vote %2" ), vote.getLogicalVoterGroup(), vote.getCode() ) );
            }
        }
        else if (selectionsSequence.size() > contestGroup->getContest_code().size())
        {
            m_logger->Debug( Glib::ustring::compose( _( "There are more contests in vote %1's selections than the registered ones in logical voter group %2" ), vote.getCode(), vote.getLogicalVoterGroup() ) );
        }
        else
        {
            m_logger->Debug( Glib::ustring::compose( _( "There are less contests in vote %1's selections than the registered ones in logical voter group %2" ), vote.getCode(), vote.getLogicalVoterGroup() ) );
        }
    }

    return success;
}

bool VoteComplianceVerificationManager::checkSelectionsBelongToSamePartyInContest(const Vote::Selection &selection)
{
    bool success = true;
    std::string currentPartyCode = "";
    const Vote::Selection::BSequence ballotOptionsSequence = selection.getBallotOptions();
    Vote::Selection::BSequence::const_iterator ballotOptionsSequenceIterator = ballotOptionsSequence.begin();

    for (; ballotOptionsSequenceIterator != ballotOptionsSequence.end(); ++ballotOptionsSequenceIterator)
    {
        std::string partyCode = m_electoralConfiguration->getPartyCodeByBallotOptionCode(*ballotOptionsSequenceIterator);

        if ((currentPartyCode.empty() == false) && (currentPartyCode != partyCode))
        {
            m_logger->Debug( Glib::ustring::compose( _( "Check if all selections belong to the same party: ballot option %1 does not belong to the same party in contest %2." ), *ballotOptionsSequenceIterator, selection.getContest() ) );
            success = false;
            break;
        }

        currentPartyCode = partyCode;
    }

    return success;
}

bool VoteComplianceVerificationManager::checkSelectionsBelongToSameParty(const Vote::Vote &vote)
{
    m_logger->Debug( _( "Starting to check if all selections belong to the same party." ) );

    bool success = true;
    const Vote::Vote::SSequence selectionsSequence = vote.getSelections();
    Vote::Vote::SSequence::const_iterator selectionsSequenceIterator = selectionsSequence.begin();

    for (; selectionsSequenceIterator != selectionsSequence.end(); ++selectionsSequenceIterator)
    {
        success = checkSelectionsBelongToSamePartyInContest(*selectionsSequenceIterator);

        if (success == false)
        {
            break;
        }
    }

    if (success)
    {
        m_logger->Debug( _( "Check if all selections belong to the same party: all selections belong to the same party." ) );
    }

    return success;
}

bool VoteComplianceVerificationManager::checkSelectionsApplicableToConstituencyInContest(const Vote::Selection &selection)
{
    bool success = true;
    const Vote::Selection::BSequence ballotOptionsSequence = selection.getBallotOptions();
    Vote::Selection::BSequence::const_iterator ballotOptionsSequenceIterator = ballotOptionsSequence.begin();

    for (; ballotOptionsSequenceIterator != ballotOptionsSequence.end(); ++ballotOptionsSequenceIterator)
    {
        const std::list < Party * > contestPartyList = m_electoralConfiguration->getMapContestParty()[selection.getContest()];
        const std::string ballotOptionPartyCode = m_electoralConfiguration->getPartyCodeByBallotOptionCode(*ballotOptionsSequenceIterator);

        if (partyCodeBelongsToPartyList(ballotOptionPartyCode, contestPartyList) == false)
        {
            m_logger->Debug( Glib::ustring::compose( _( "Check if all selections apply to the corresponding constituency: ballot option %1 does not apply to the corresponding constituency in contest %2." ), *ballotOptionsSequenceIterator, selection.getContest() ) );
            success = false;
            break;
        }
    }

    return success;
}

bool VoteComplianceVerificationManager::checkSelectionsApplicableToConstituency(const Vote::Vote &vote)
{
    m_logger->Debug( _( "Starting to check if all selections apply to the corresponding constituency." ) );

    bool success = true;
    const Vote::Vote::SSequence selectionsSequence = vote.getSelections();
    Vote::Vote::SSequence::const_iterator selectionsSequenceIterator = selectionsSequence.begin();

    for (; selectionsSequenceIterator != selectionsSequence.end(); ++selectionsSequenceIterator)
    {
        success = checkSelectionsApplicableToConstituencyInContest(*selectionsSequenceIterator);

        if (success == false)
        {
            break;
        }
    }

    if (success)
    {
        m_logger->Debug( _( "Check if all selections apply to the corresponding constituency: all selections apply to the corresponding constituency." ) );
    }

    return success;
}

bool VoteComplianceVerificationManager::checkUniquenessInSelectionsInContest(
    const Vote::Selection &currentSelection,
    const Vote::Vote &vote)
{
    bool success = true;
    const Vote::Selection::BSequence ballotOptionsSequence = currentSelection.getBallotOptions();
    Vote::Selection::BSequence::const_iterator ballotOptionsSequenceIterator = ballotOptionsSequence.begin();

    for (; ballotOptionsSequenceIterator != ballotOptionsSequence.end(); ++ballotOptionsSequenceIterator)
    {
        const std::string ballotOptionCode = *ballotOptionsSequenceIterator;
        const std::string partyCode = m_electoralConfiguration->getPartyCodeByBallotOptionCode(ballotOptionCode);

        if (partyAppearsOnlyOnce(partyCode, vote, m_electoralConfiguration) == false)
        {
            m_logger->Debug( Glib::ustring::compose( _( "Check uniqueness in selections: party %1 appears more than once in the vote %2." ), partyCode, vote.getCode() ) );
            success = false;
            break;
        }

        if (ballotOptionAppearsOnlyOnce(ballotOptionCode, vote, m_electoralConfiguration) == false)
        {
            m_logger->Debug( Glib::ustring::compose( _( "Check uniqueness in selections: ballot option %1 appears more than once in the vote %2." ), ballotOptionCode, vote.getCode() ) );
            success = false;
            break;
        }
    }

    return success;
}

bool VoteComplianceVerificationManager::checkUniquenessInSelections(const Vote::Vote &vote)
{
    m_logger->Debug( _( "Starting to check uniqueness in selections." ) );

    bool success = true;
    const Vote::Vote::SSequence selectionsSequence = vote.getSelections();
    Vote::Vote::SSequence::const_iterator selectionsSequenceIterator = selectionsSequence.begin();

    for (; selectionsSequenceIterator != selectionsSequence.end(); ++selectionsSequenceIterator)
    {
        const std::string contestCode = selectionsSequenceIterator->getContest();

        if (contestAppearsOnlyOnce(contestCode, vote) == false)
        {
            m_logger->Debug( Glib::ustring::compose( _( "Check uniqueness in selections: contest %1 appears more than once in the vote %2." ), contestCode, vote.getCode() ) );
            success = false;
            break;
        }

        success = checkUniquenessInSelectionsInContest(*selectionsSequenceIterator, vote);

        if (success == false)
        {
            break;
        }
    }

    if (success)
    {
        m_logger->Debug( _( "Check uniqueness in selections: all selections are unique" ) );
    }

    return success;
}

bool VoteComplianceVerificationManager::checkListHeaderResourceCorrectness(
    const Smartmatic::SAES::Voting::Vote::Vote &vote,
    std::map < std::string, std::map < std::string, std::string > > textToPrintMap)
{
    m_logger->Debug( _( "Starting to check correctness of the list header resource." ) );

    bool success = true;
    std::map < std::string, std::map < std::string, std::string > >::const_iterator textToPrintMapIterator = textToPrintMap.begin();

    for (; textToPrintMapIterator != textToPrintMap.end() && success; ++textToPrintMapIterator)
    {
        const std::string contestCode = textToPrintMapIterator->first;
        const std::map < std::string, std::string > ballotOptionsMap = textToPrintMapIterator->second;

        std::map < std::string, std::string >::const_iterator ballotOptionsIterator = ballotOptionsMap.begin();

        for (; ballotOptionsIterator != ballotOptionsMap.end(); ++ballotOptionsIterator)
        {
            const std::string ballotOptionCode = ballotOptionsIterator->first;
            const std::string partyKey = ballotOptionsIterator->second;

            if (mapPartyMatchesVoteParty(m_electoralConfiguration, vote, contestCode, ballotOptionCode, partyKey) == false)
            {
                m_logger->Debug( Glib::ustring::compose( _( "Check correctness of the list header resource: text of the party to print %1 does not match the text of the configured party." ), partyKey ) );
                success = false;
                break;
            }
        }
    }

    if (success)
    {
        m_logger->Debug( _( "Check correctness of the list header resource: texts to print match the configured text." ) );
    }

    return success;
}

bool VoteComplianceVerificationManager::checkCumulativeNumberOfSelections(const Vote::Vote &vote)
{
    m_logger->Debug( _( "Starting to check cumulative number of selections." ) );

    bool success = true;
    const Vote::Vote::SSequence selectionsSequence = vote.getSelections();
    Vote::Vote::SSequence::const_iterator selectionsSequenceIterator = selectionsSequence.begin();

    for (; selectionsSequenceIterator != selectionsSequence.end(); ++selectionsSequenceIterator)
    {
        const std::string contestCode = selectionsSequenceIterator->getContest();
        std::string partyCode = (selectionsSequenceIterator->getBallotOptions().size() > 0) ?
            m_electoralConfiguration->getPartyCodeByBallotOptionCode(selectionsSequenceIterator->getBallotOptions()[0]) : "";

        if (partyCode.empty() == false)
        {
            const size_t listOptionsSize = m_electoralConfiguration->getEligibleOptionsListByContestParty(contestCode, partyCode).size();
            const size_t principalOptionsSize = m_electoralConfiguration->getEligibleOptionsPrincipalByContestParty(contestCode, partyCode).size();
            const size_t standInOptionsSize = m_electoralConfiguration->getEligibleOptionsStandInByContestParty(contestCode, partyCode).size();

            const size_t selectedListSize = getSelectedCandidatesByCategory(m_electoralConfiguration, *selectionsSequenceIterator, Category::ELECTION_LIST);
            const size_t selectedPrincipalSize = getSelectedCandidatesByCategory(m_electoralConfiguration, *selectionsSequenceIterator, Category::ELECTION_PRINCIPAL);
            const size_t selectedStandInSize = getSelectedCandidatesByCategory(m_electoralConfiguration, *selectionsSequenceIterator, Category::ELECTION_STAND_IN);

            if (selectedListSize > listOptionsSize)
            {
                m_logger->Debug( Glib::ustring::compose( _( "Check cumulative number of selections: selected vote lists (%1) are greater than configured vote lists (%2)." ), selectedListSize, listOptionsSize ) );
                success = false;
                break;
            }

            if (selectedPrincipalSize > principalOptionsSize)
            {
                m_logger->Debug( Glib::ustring::compose( _( "Check cumulative number of selections: selected principal candidates (%1) are greater than configured principal candidates (%2)." ), selectedPrincipalSize, principalOptionsSize ) );
                success = false;
                break;
            }

            if (selectedStandInSize > standInOptionsSize)
            {
                m_logger->Debug( Glib::ustring::compose( _( "Check cumulative number of selections: selected stand-in candidates (%1) are greater than configured stand-in candidates (%2)." ), selectedStandInSize, standInOptionsSize ) );
                success = false;
                break;
            }
        }
    }

    if (success)
    {
        m_logger->Debug( _( "Check cumulative number of selections: all selected candidates are less than or equal to configured candidates." ) );
    }

    return success;
}

bool VoteComplianceVerificationManager::checkTotalCumulativeSelections(
    const size_t totalVotes,
    const VoteResults &voteResults)
{
    m_logger->Debug( _( "Starting to check total cumulative selections." ) );

    bool success = true;
    const VoteResults::ContestResultsSequence contestResults = voteResults.getContestResults();
    VoteResults::ContestResultsSequence::const_iterator contestResultsIterator = contestResults.begin();

    for (; contestResultsIterator != contestResults.end() && success; ++contestResultsIterator)
    {
        const ContestResults::PartyResultsSequence partyResults = contestResultsIterator->getPartyResults();
        ContestResults::PartyResultsSequence::const_iterator partyResultsIterator = partyResults.begin();

        for (; partyResultsIterator != partyResults.end() && success; ++partyResultsIterator)
        {
            const size_t partyTotalVotes = partyResultsIterator->getTotal_votes();

            if (partyTotalVotes > totalVotes)
            {
                m_logger->Debug( Glib::ustring::compose( _( "Check total cumulative selections: party %1 total votes (%2) are greater than total votes (%3) in contest %4." ), partyResultsIterator->getCode(), partyTotalVotes, totalVotes, contestResultsIterator->getCode() ) );
                success = false;
                break;
            }

            if (preferenceVotesAreLessThanOrEqualToTotalVotes(totalVotes, partyResultsIterator->getCandidateResults()) == false)
            {
                m_logger->Debug( Glib::ustring::compose( _( "Check total cumulative selections: one of the principal candidates' preference votes are greater than total votes (%1) in contest %2." ), totalVotes, contestResultsIterator->getCode() ) );
                success = false;
                break;
            }

            if (preferenceVotesAreLessThanOrEqualToTotalVotes(totalVotes, partyResultsIterator->getSubstituteResults()) == false)
            {
                m_logger->Debug( Glib::ustring::compose( _( "Check total cumulative selections: one of the stand-in candidates' preference votes are greater than total votes (%1) in contest %2." ), totalVotes, contestResultsIterator->getCode() ) );
                success = false;
                break;
            }
        }
    }

    if (success)
    {
        m_logger->Debug( _( "Check total cumulative selections: total cumulative selections have been satisfied." ) );
    }

    return success;
}

bool VoteComplianceVerificationManager::checkIndividualCumulativeSelections(const VoteResults &voteResults)
{
    m_logger->Debug( _( "Starting to check individual cumulative selections." ) );

    bool success = true;
    const VoteResults::ContestResultsSequence contestResults = voteResults.getContestResults();
    VoteResults::ContestResultsSequence::const_iterator contestResultsIterator = contestResults.begin();

    for (; contestResultsIterator != contestResults.end() && success; ++contestResultsIterator)
    {
        const ContestResults::PartyResultsSequence partyResults = contestResultsIterator->getPartyResults();
        ContestResults::PartyResultsSequence::const_iterator partyResultsIterator = partyResults.begin();

        for (; partyResultsIterator != partyResults.end() && success; ++partyResultsIterator)
        {
            const size_t partyTotalVotes = partyResultsIterator->getTotal_votes();
            const size_t partyListVotes = partyResultsIterator->getList_votes();

            if (partyListVotes > partyTotalVotes)
            {
                m_logger->Debug( Glib::ustring::compose( _( "Check individual cumulative selections: party %1 list votes (%2) are greater than its total votes (%3) in contest %4." ), partyResultsIterator->getCode(), partyListVotes, partyTotalVotes, contestResultsIterator->getCode() ) );
                success = false;
                break;
            }

            if (preferenceVotesAreLessThanOrEqualToTotalVotes(partyTotalVotes, partyResultsIterator->getCandidateResults()) == false)
            {
                m_logger->Debug( Glib::ustring::compose( _( "Check individual cumulative selections: one of the principal candidates' preference votes are greater than party %1 total votes (%2) in contest %3." ), partyResultsIterator->getCode(), partyTotalVotes, contestResultsIterator->getCode() ) );
                success = false;
                break;
            }

            if (preferenceVotesAreLessThanOrEqualToTotalVotes(partyTotalVotes, partyResultsIterator->getSubstituteResults()) == false)
            {
                m_logger->Debug( Glib::ustring::compose( _( "Check individual cumulative selections: one of the stand-in candidates' preference votes are greater than party %1 total votes (%2) in contest %3." ), partyResultsIterator->getCode(), partyTotalVotes, contestResultsIterator->getCode() ) );
                success = false;
                break;
            }
        }
    }

    if (success)
    {
        m_logger->Debug( _( "Check individual cumulative selections: individual cumulative selections have been satisfied." ) );
    }

    return success;
}

bool VoteComplianceVerificationManager::checkEmlVotesDeposited(const std::map<std::string, Eml::EML>& plainEmlFiles)
{
    m_logger->Debug( _( "Starting to check EML votes deposited." ) );

    bool success = true;

    std::map<std::string, Voting::Eml::EML>::const_iterator       itEml  = plainEmlFiles.begin();
    const std::map<std::string, Voting::Eml::EML>::const_iterator endEml = plainEmlFiles.end();

    for (; itEml != endEml && success; ++itEml)
    {
        Eml::Count::ElectionConstIterator       itElection  = itEml->second.getCount().getElection().begin();
        const Eml::Count::ElectionConstIterator endElection = itEml->second.getCount().getElection().end();

        for (; itElection != endElection && success; ++itElection)
        {
            const Eml::Contests::ContestSequence contests = itElection->getContests().getContest();

            Eml::Election::ContestsType::ContestConstIterator       itContest  = contests.begin();
            const Eml::Election::ContestsType::ContestConstIterator endContest = contests.end();

            for (; itContest != endContest && success; ++itContest)
            {
                std::string errorMessage;

                if (isVotesDepositedNumberValid(itContest, errorMessage) == false)
                {
                    errorMessage += Glib::ustring::compose(_(" Error detected in file '%1'." ), itEml->first);
                    m_logger->Debug( errorMessage );
                    success = false;
                }
            }
        }
    }

    if (success)
    {
        m_logger->Debug( _( "Check EML votes deposited: EML votes deposited have been satisfied." ) );
    }

    return success;
}

bool VoteComplianceVerificationManager::checkEmlVotesByVoterType(const std::map<std::string, Eml::EML>& plainEmlFiles)
{
    m_logger->Debug( _( "Starting to check EML votes by voter type." ) );

    bool success = true;

    const size_t totalVoters = getEmlTotalVoters(plainEmlFiles.begin()->second);
    const size_t sumOfVoters = getEmlSumOfVoters(plainEmlFiles.begin()->second);

    if (totalVoters != sumOfVoters)
    {
        m_logger->Debug( Glib::ustring::compose( _( "Check EML votes by voter type: total voters (%1) does not match the sum of voters (%2) in the file '%3'." ), totalVoters, sumOfVoters, plainEmlFiles.begin()->first ) );
        success = false;
    }
    else
    {
        m_logger->Debug( _( "Check EML votes by voter type: EML sum of voters have been satisfied." ) );
    }

    return success;
}

bool VoteComplianceVerificationManager::checkEmlVotesCounter(
    const std::map<std::string, Eml::EML>& plainEmlFiles,
    VoteStatisticsWrapper * const voteStatisticsWrapper)
{
    m_logger->Debug( _( "Starting to check EML votes counter." ) );

    bool success = true;

    std::map<std::string, Voting::Eml::EML>::const_iterator       itEml  = plainEmlFiles.begin();
    const std::map<std::string, Voting::Eml::EML>::const_iterator endEml = plainEmlFiles.end();

    for (; itEml != endEml && success; ++itEml)
    {
        const Eml::Count::ElectionSequence electionsSequence = itEml->second.getCount().getElection();

        Eml::Count::ElectionConstIterator       itElection  = electionsSequence.begin();
        const Eml::Count::ElectionConstIterator endElection = electionsSequence.end();

        for (; itElection != endElection && success; ++itElection)
        {
            std::string errorMessage;

            if (areContestsSelectionsValid(itElection, voteStatisticsWrapper, errorMessage) == false)
            {
                m_logger->Debug(errorMessage);
                success = false;
            }
        }
    }

    if (success)
    {
        m_logger->Debug( _( "Check EML votes counter: EML votes counters have been satisfied." ) );
    }

    return success;
}
