/*
 * VoteComplianceVerificator.cxx
 *
 *  Created on: Nov 21, 2017
 *      Author: juanvalera
 */

#include "VoteComplianceVerificator.hxx"

#include "AfterScanningVoteComplianceVerificator.hxx"
#include "BeforeGeneratingResultsComplianceVerificator.hxx"
#include "BeforePrintingVoteComplianceVerificator.hxx"
#include <Log/SMTTLogManager.h>
#include <Voting/ElectoralConfiguration.hxx>
#include <Voting/Eml/510.hxx>
#include <Voting/Vote/StationTallyWrapper.hxx>
#include <Voting/Vote/vote-schema.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Operation::VoteCompliance;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::SAES::Voting::Eml;
using namespace Smartmatic::SAES::Voting::PBReports;
using namespace Smartmatic::SAES::Voting::Vote;

VoteComplianceVerificator* VoteComplianceVerificator::m_instance = NULL;

ISMTTLog * VoteComplianceVerificator::m_logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Operation.Voting.VoteComplianceVerificator", GETTEXT_PACKAGE );

VoteComplianceVerificator::VoteComplianceVerificator (ElectoralConfiguration * electoralConfiguration)
: m_electoralConfiguration(electoralConfiguration)
, m_beforePrintingVoteComplianceVerificator(new BeforePrintingVoteComplianceVerificator(electoralConfiguration))
, m_afterScanningVoteComplianceVerificator(new AfterScanningVoteComplianceVerificator(electoralConfiguration))
, m_beforeGeneratingResultsComplianceVerificator(new BeforeGeneratingResultsComplianceVerificator(electoralConfiguration))
{
}

VoteComplianceVerificator::~VoteComplianceVerificator()
{
    dispose();
}

void VoteComplianceVerificator::dispose ()
{
    m_instance = NULL;
}

bool VoteComplianceVerificator::checkMinimumNumberOfSelections(const Voting::Vote::Vote &vote)
{
    return m_beforePrintingVoteComplianceVerificator->checkMinimumNumberOfSelections(vote);
}

bool VoteComplianceVerificator::checkSelectionsBelongToSameParty(const Voting::Vote::Vote &vote)
{
    return m_beforePrintingVoteComplianceVerificator->checkSelectionsBelongToSameParty(vote);
}

bool VoteComplianceVerificator::checkSelectionsApplicableToConstituency(const Voting::Vote::Vote &vote)
{
    return m_beforePrintingVoteComplianceVerificator->checkSelectionsApplicableToConstituency(vote);
}

bool VoteComplianceVerificator::checkUniquenessInSelections(const Voting::Vote::Vote &vote)
{
    return m_beforePrintingVoteComplianceVerificator->checkUniquenessInSelections(vote);
}

bool VoteComplianceVerificator::checkListHeaderResourceCorrectness(
    const Smartmatic::SAES::Voting::Vote::Vote &vote,
    const std::map < std::string, std::map < std::string, std::string > > &textToPrintMap)
{
    return m_beforePrintingVoteComplianceVerificator->checkListHeaderResourceCorrectness(vote, textToPrintMap);
}

bool VoteComplianceVerificator::checkCumulativeNumberOfSelections(const Voting::Vote::Vote &vote)
{
    return m_beforePrintingVoteComplianceVerificator->checkCumulativeNumberOfSelections(vote);
}

bool VoteComplianceVerificator::checkMinimumNumberOfScannings(const Voting::Vote::Vote &scannedVote)
{
    return m_afterScanningVoteComplianceVerificator->checkMinimumNumberOfScannings(scannedVote);
}

bool VoteComplianceVerificator::checkScanningsBelongToSameParty(const Voting::Vote::Vote &scannedVote)
{
    return m_afterScanningVoteComplianceVerificator->checkScanningsBelongToSameParty(scannedVote);
}

bool VoteComplianceVerificator::checkScanningsApplicableToConstituency(const Voting::Vote::Vote &scannedVote)
{
    return m_afterScanningVoteComplianceVerificator->checkScanningsApplicableToConstituency(scannedVote);
}

bool VoteComplianceVerificator::checkUniquenessInScannings(const Voting::Vote::Vote &scannedVote)
{
    return m_afterScanningVoteComplianceVerificator->checkUniquenessInScannings(scannedVote);
}

bool VoteComplianceVerificator::checkCumulativeNumberOfScannings(const Voting::Vote::Vote &scannedVote)
{
    return m_afterScanningVoteComplianceVerificator->checkCumulativeNumberOfScannings(scannedVote);
}

bool VoteComplianceVerificator::checkTotalCumulativeSelections(
    const size_t totalVotes,
    const Smartmatic::SAES::Voting::PBReports::VoteResults &voteResults)
{
    return m_beforeGeneratingResultsComplianceVerificator->checkTotalCumulativeSelections(totalVotes, voteResults);
}

bool VoteComplianceVerificator::checkIndividualCumulativeSelections(const VoteResults &voteResults)
{
    return m_beforeGeneratingResultsComplianceVerificator->checkIndividualCumulativeSelections(voteResults);
}

bool VoteComplianceVerificator::checkEmlVotesDeposited(const std::map<std::string, EML>& plainEmlFiles)
{
    return m_beforeGeneratingResultsComplianceVerificator->checkEmlVotesDeposited(plainEmlFiles);
}

bool VoteComplianceVerificator::checkEmlVotesByVoterType(const std::map<std::string, EML>& plainEmlFiles)
{
    return m_beforeGeneratingResultsComplianceVerificator->checkEmlVotesByVoterType(plainEmlFiles);
}

bool VoteComplianceVerificator::checkEmlVotesCounter(
    const std::map<std::string, EML>& plainEmlFiles,
    VoteStatisticsWrapper * const voteStatisticsWrapper)
{
    return m_beforeGeneratingResultsComplianceVerificator->checkEmlVotesCounter(plainEmlFiles, voteStatisticsWrapper);
}

bool VoteComplianceVerificator::checkAllBeforePrintingVoteConditions(
    const Voting::Vote::Vote &vote,
    std::map < std::string, std::map < std::string, std::string > > textToPrintMap)
{
    bool success = true;

    m_logger->Audit(_("Starting verification of conformity test before printing"));

    if ((checkMinimumNumberOfSelections(vote) == false) ||
        (checkSelectionsBelongToSameParty(vote) == false) ||
        (checkSelectionsApplicableToConstituency(vote) == false) ||
        (checkUniquenessInSelections(vote) == false) ||
        (checkListHeaderResourceCorrectness(vote, textToPrintMap) == false) ||
        (checkCumulativeNumberOfSelections(vote) == false))
    {
        m_logger->Audit(_("Verification of conformity test before printing failed"));
        success = false;
    }
    else
    {
        m_logger->Audit(_("Verification of conformity test before printing succeeded"));
    }

    return success;
}

bool VoteComplianceVerificator::checkAllAfterScanningVoteConditions(const Voting::Vote::Vote &scannedVote)
{
    bool success = true;

    m_logger->Audit(_("Starting verification of conformity test after vote scanned"));

    if ((checkMinimumNumberOfScannings(scannedVote) == false) ||
        (checkScanningsBelongToSameParty(scannedVote) == false) ||
        (checkScanningsApplicableToConstituency(scannedVote) == false) ||
        (checkUniquenessInScannings(scannedVote) == false) ||
        (checkCumulativeNumberOfScannings(scannedVote) == false))
    {
        m_logger->Audit(_("Verification of conformity test after vote scanned failed"));
        success = false;
    }
    else
    {
        m_logger->Audit(_("Verification of conformity test after vote scanned succeeded"));
    }

    return success;
}

bool VoteComplianceVerificator::checkAllBeforeGeneratingResultsConditions(
    const size_t totalVotes,
    const Smartmatic::SAES::Voting::PBReports::VoteResults &voteResults,
    const std::map<std::string, EML>& plainEmlFiles,
    VoteStatisticsWrapper * const voteStatisticsWrapper)
{
    bool success = true;

    m_logger->Audit(_("Starting verification of conformity test before generating results"));

    if ((checkTotalCumulativeSelections(totalVotes, voteResults) == false) ||
        (checkIndividualCumulativeSelections(voteResults) == false) ||
        (checkEmlVotesDeposited(plainEmlFiles) == false) ||
        (checkEmlVotesByVoterType(plainEmlFiles) == false) ||
        (checkEmlVotesCounter(plainEmlFiles, voteStatisticsWrapper) == false))
    {
        m_logger->Audit(_("Verification of conformity test before generating results failed"));
        success = false;
    }
    else
    {
        m_logger->Audit(_("Verification of conformity test before generating results succeeded"));
    }

    return success;
}

VoteComplianceVerificator * VoteComplianceVerificator::getInstance(ElectoralConfiguration * electoralConfiguration)
{
    if (m_instance == NULL)
    {
        m_instance = new VoteComplianceVerificator(electoralConfiguration);
    }

    return m_instance;
}
