/*
 * VoteComplianceVerificator.hxx
 *
 *  Created on: Nov 21, 2017
 *      Author: juanvalera
 */

#ifndef VOTECOMPLIANCEVERIFICATOR_HXX_
#define VOTECOMPLIANCEVERIFICATOR_HXX_

#include <string>
#include <list>
#include <vector>
#include <map>
#include <Log/ISMTTLog.h>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Voting
        {
            class ElectoralConfiguration;

            namespace Vote
            {
                class Vote;
            }

            namespace PBReports
            {
                class VoteResults;
                class VoteStatisticsWrapper;
            }

            namespace Eml
            {
                class EML;
            }
        }

        namespace Operation
        {
            namespace VoteCompliance
            {
                class BeforePrintingVoteComplianceVerificator;
                class AfterScanningVoteComplianceVerificator;
                class BeforeGeneratingResultsComplianceVerificator;

                /**
                 * @class	VoteComplianceVerificator
                 * @brief	Header class VoteComplianceVerificator.
                 * @author	Juan Valera
                 * @date	22/11/2017
                 */
                class VoteComplianceVerificator
                {
                    public:

                        /**
                         * @brief  Returns the instance of the vote compliance verificator.
                         * @author eduardo.fernandez
                         * @date   Feb 07, 2018
                         * @param  electoralConfiguration The electoral configuration.
                         * @return The instance of the vote compliance verificator.
                         */
                        static VoteComplianceVerificator * getInstance(Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration = NULL);

                        /**
                         * @fn	virtual VoteComplianceVerificator::~VoteComplianceVerificator();
                         * @brief	Finaliser.
                         * @author	Juan Valera
                         * @date	01/11/2012
                         */
                        virtual ~VoteComplianceVerificator();


                        /**
                         * @fn	void VoteComplianceVerificator::dispose();
                         * @brief	Dispose of this object, cleaning up any resources it uses.
                         * @author	Juan Valera
                         * @date	01/11/2012
                         */
                        void dispose();

                        /**
                         * @fn  bool VoteComplianceVerificator::checkMinimumNumberOfSelections(const Smartmatic::SAES::Voting::Vote::Vote &vote);
                         * @brief    Checks minimum number of selections in the given vote.
                         * @param    vote Vote containing the voter selections.
                         * @return   Whether the vote satisfies the minimum number of selections.
                         * @author   eduardo.fernandez
                         * @date     Feb 07, 2018
                         */
                        bool checkMinimumNumberOfSelections(const Smartmatic::SAES::Voting::Vote::Vote &vote);

                        /**
                         * @fn  bool VoteComplianceVerificator::checkSelectionsBelongToSameParty(const Smartmatic::SAES::Voting::Vote::Vote &vote);
                         * @brief    Checks whether the given vote's selections belong to the same party.
                         * @param    vote Vote containing the voter selections.
                         * @return   Whether the given vote's selections belong to the same party.
                         * @author   eduardo.fernandez
                         * @date     Feb 07, 2018
                         */
                        bool checkSelectionsBelongToSameParty(const Smartmatic::SAES::Voting::Vote::Vote &vote);

                        /**
                         * @fn  bool VoteComplianceVerificator::checkSelectionsApplicableToConstituency(const Smartmatic::SAES::Voting::Vote::Vote &vote);
                         * @brief    Checks whether the given vote's selections' party belong to the same contest.
                         * @param    vote Vote containing the voter selections.
                         * @return   Whether the given vote's selections' party belong to the same contest.
                         * @author   eduardo.fernandez
                         * @date     Feb 08, 2018
                         */
                        bool checkSelectionsApplicableToConstituency(const Smartmatic::SAES::Voting::Vote::Vote &vote);

                        /**
                         * @fn  bool VoteComplianceVerificator::checkUniquenessInSelections(const Smartmatic::SAES::Voting::Vote::Vote &vote);
                         * @brief    Checks whether the given vote's selections are unique within the vote.
                         * @param    vote Vote containing the voter selections.
                         * @return   Whether the given vote's selections' party belong to the same contest.
                         * @author   eduardo.fernandez
                         * @date     Feb 08, 2018
                         */
                        bool checkUniquenessInSelections(const Smartmatic::SAES::Voting::Vote::Vote &vote);

                        /**
                         * @fn  bool VoteComplianceVerificator::checkListHeaderResourceCorrectness(const Smartmatic::SAES::Voting::Vote::Vote &vote, const std::map < std::string, std::map < std::string, std::string > > &textToPrintMap);
                         * @brief    Checks whether the string resource of the parties to print matches the string resource
                         *           of the vote's parties.
                         * @param    vote Vote containing the voter selections.
                         * @param    textToPrintMap Map containing the parties to print.
                         * @return   Whether the string resource of the parties to print matches the string resource
                         *           of the vote's parties.
                         * @author   eduardo.fernandez
                         * @date     Feb 09, 2018
                         */
                        bool checkListHeaderResourceCorrectness(
                            const Smartmatic::SAES::Voting::Vote::Vote &vote,
                            const std::map < std::string, std::map < std::string, std::string > > &textToPrintMap);

                        /**
                         * @fn  bool VoteComplianceVerificator::checkCumulativeNumberOfSelections(const Smartmatic::SAES::Voting::Vote::Vote &vote);
                         * @brief    Checks whether the cumulative number of selected candidates are less than or equal
                         *           to the number of candidates registered on the party.
                         * @param    vote Vote containing the voter selections.
                         * @return   Whether the cumulative number of selected candidates are less than or equal
                         *           to the number of candidates registered on the party.
                         * @author   eduardo.fernandez
                         * @date     Feb 08, 2018
                         */
                        bool checkCumulativeNumberOfSelections(const Smartmatic::SAES::Voting::Vote::Vote &vote);

                        /**
                         * @fn  bool VoteComplianceVerificator::checkMinimumNumberOfScannings(const Smartmatic::SAES::Voting::Vote::Vote &scannedVote);
                         * @brief    Checks minimum number of scannings in the given scanned vote.
                         * @param    scannedVote Scanned vote containing the voter selections.
                         * @return   Whether the scanned vote satisfies the minimum number of scannings.
                         * @author   eduardo.fernandez
                         * @date     Feb 09, 2018
                         */
                        bool checkMinimumNumberOfScannings(const Smartmatic::SAES::Voting::Vote::Vote &scannedVote);

                        /**
                         * @fn  bool VoteComplianceVerificator::checkScanningsBelongToSameParty(const Smartmatic::SAES::Voting::Vote::Vote &scannedVote);
                         * @brief    Checks whether the given scanned vote's selections belong to the same party.
                         * @param    scannedVote Scanned vote containing the voter selections.
                         * @return   Whether the given scanned vote's selections belong to the same party.
                         * @author   eduardo.fernandez
                         * @date     Feb 09, 2018
                         */
                        bool checkScanningsBelongToSameParty(const Smartmatic::SAES::Voting::Vote::Vote &scannedVote);

                        /**
                         * @fn  bool VoteComplianceVerificator::checkScanningsApplicableToConstituency(const Smartmatic::SAES::Voting::Vote::Vote &scannedVote);
                         * @brief    Checks whether the given scanned vote's selections' party belong to the same contest.
                         * @param    scannedVote Scanned vote containing the voter selections.
                         * @return   Whether the given scanned vote's selections' party belong to the same contest.
                         * @author   eduardo.fernandez
                         * @date     Feb 09, 2018
                         */
                        bool checkScanningsApplicableToConstituency(const Smartmatic::SAES::Voting::Vote::Vote &scannedVote);

                        /**
                         * @fn  bool VoteComplianceVerificator::checkUniquenessInScannings(const Smartmatic::SAES::Voting::Vote::Vote &scannedVote);
                         * @brief    Checks whether the given scanned vote's selections are unique within the vote.
                         * @param    scannedVote Scanned vote containing the voter selections.
                         * @return   Whether the given scanned vote's selections' party belong to the same contest.
                         * @author   eduardo.fernandez
                         * @date     Feb 09, 2018
                         */
                        bool checkUniquenessInScannings(const Smartmatic::SAES::Voting::Vote::Vote &scannedVote);

                        /**
                         * @fn  bool VoteComplianceVerificator::checkCumulativeNumberOfScannings(const Smartmatic::SAES::Voting::Vote::Vote &scannedVote);
                         * @brief    Checks whether the cumulative number of scanned candidates are less than or equal
                         *           to the number of candidates registered on the party.
                         * @param    scannedVote Scanned vote containing the voter selections.
                         * @return   Whether the cumulative number of scanned candidates are less than or equal
                         *           to the number of candidates registered on the party.
                         * @author   eduardo.fernandez
                         * @date     Feb 09, 2018
                         */
                        bool checkCumulativeNumberOfScannings(const Smartmatic::SAES::Voting::Vote::Vote &scannedVote);

                        /**
                         * @fn  bool VoteComplianceVerificator::checkTotalCumulativeSelections(const size_t totalVotes, const Smartmatic::SAES::Voting::PBReports::VoteResults &voteResults);
                         * @brief    Checks whether the cumulative number of categorized votes don't exceed the total number of votes.
                         * @param    totalVotes The total number of votes.
                         * @param    voteResults The vote results.
                         * @return   Whether the cumulative number of categorized votes don't exceed the total number of votes.
                         * @author   eduardo.fernandez
                         * @date     Feb 14, 2018
                         */
                        bool checkTotalCumulativeSelections(
                            const size_t totalVotes,
                            const Smartmatic::SAES::Voting::PBReports::VoteResults &voteResults);

                        /**
                         * @fn  bool VoteComplianceVerificator::checkIndividualCumulativeSelections(const Smartmatic::SAES::Voting::PBReports::VoteResults &voteResults);
                         * @brief    Checks whether the cumulative number of categorized votes for a single list don't
                         *           exceed the total number of votes for that list.
                         * @param    voteResults The vote results.
                         * @return   Whether the cumulative number of categorized votes for a single list don't
                         *           exceed the total number of votes for that list.
                         * @author   eduardo.fernandez
                         * @date     Feb 15, 2018
                         */
                        bool checkIndividualCumulativeSelections(const Smartmatic::SAES::Voting::PBReports::VoteResults &voteResults);

                        /*
                         * @fn bool VoteComplianceVerificator::checkEmlVotesDeposited(const std::map<std::string, Smartmatic::SAES::Voting::Eml::EML>& plainEmlFiles);
                         * @brief   Checks whether the total votes deposited matches the sum of valid and invalid votes.
                         * @param   plainEmlFiles The map containing the plain EML files to verify.
                         * @return  Whether the total votes deposited matches the sum of valid and invalid votes.
                         * @author  eduardo.fernandez
                         * @date    Nov 13, 2018
                         */
                        bool checkEmlVotesDeposited(const std::map<std::string, Smartmatic::SAES::Voting::Eml::EML>& plainEmlFiles);

                        /*
                         * @fn bool VoteComplianceVerificator::checkEmlVotesByVoterType(const std::map<std::string, Smartmatic::SAES::Voting::Eml::EML>& plainEmlFiles);
                         * @brief   Checks whether the sum of votes by voter type matches the total voters number.
                         * @param   plainEmlFiles The map containing the plain EML files to verify.
                         * @return  Whether the sum of votes by voter type matches the total voters number.
                         * @author  eduardo.fernandez
                         * @date    Nov 13, 2018
                         */
                        bool checkEmlVotesByVoterType(const std::map<std::string, Smartmatic::SAES::Voting::Eml::EML>& plainEmlFiles);

                        /**
                         * @fn bool VoteComplianceVerificator::checkEmlVotesCounter(const std::map<std::string, Voting::Eml::EML>& plainEmlFiles, Smartmatic::SAES::Voting::PBReports::VoteStatisticsWrapper * const voteStatisticsWrapper);
                         * @brief Checks whether the EML votes deposited match the vote statistics votes counter by contest.
                         * @param plainEmlFiles The map containing the plain EML files to verify.
                         * @param voteStatisticsWrapper Thw vote statistics wrapper.
                         */
                        bool checkEmlVotesCounter(
                            const std::map<std::string, Smartmatic::SAES::Voting::Eml::EML>& plainEmlFiles,
                            Smartmatic::SAES::Voting::PBReports::VoteStatisticsWrapper * const voteStatisticsWrapper);

                        /**
                         * @fn  bool VoteComplianceVerificator::checkAllBeforePrintingVoteConditions(const Smartmatic::SAES::Voting::Vote::Vote &vote, const std::map < std::string, std::map < std::string, std::string > > &textToPrintMap);
                         * @brief    Checks whether all the before printing vote validations are successful.
                         * @param    vote Vote containing the voter selections.
                         * @param    textToPrintMap Map containing the parties to print.
                         * @return   Whether all the before printing vote validations are successful.
                         * @author   eduardo.fernandez
                         * @date     Feb 10, 2018
                         */
                        bool checkAllBeforePrintingVoteConditions(
                            const Smartmatic::SAES::Voting::Vote::Vote &vote,
                            std::map < std::string, std::map < std::string, std::string > > textToPrintMap);

                        /**
                         * @fn  bool VoteComplianceVerificator::checkAllAfterScanningVoteConditions(const Smartmatic::SAES::Voting::Vote::Vote &scannedVote);
                         * @brief    Checks whether all the after scanning vote validations are successful.
                         * @param    scannedVote Scanned vote containing the voter selections.
                         * @return   Whether all the after scanning vote validations are successful.
                         * @author   eduardo.fernandez
                         * @date     Feb 10, 2018
                         */
                        bool checkAllAfterScanningVoteConditions(const Smartmatic::SAES::Voting::Vote::Vote &vote);

                        /**
                         * @fn  bool VoteComplianceVerificator::checkAllBeforeGeneratingResultsConditions(const size_t totalVotes, const Smartmatic::SAES::Voting::PBReports::VoteResults &voteResults);
                         * @brief    Checks whether all the before generating results validations are successful.
                         * @param    totalVotes The total number of votes.
                         * @param    voteResults The vote results.
                         * @param    plainEmlFiles The map containing the plain EML files to verify.
                         * @return   Whether all the before generating results validations are successful.
                         * @author   eduardo.fernandez
                         * @date     Feb 15, 2018
                         */
                        bool checkAllBeforeGeneratingResultsConditions(
                            const size_t totalVotes,
                            const Smartmatic::SAES::Voting::PBReports::VoteResults &voteResults,
                            const std::map<std::string, Smartmatic::SAES::Voting::Eml::EML>& plainEmlFiles,
                            Smartmatic::SAES::Voting::PBReports::VoteStatisticsWrapper * const voteStatisticsWrapper);

                    private:

                        /**
                         * @fn  VoteComplianceVerificator::VoteComplianceVerificator ( Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration );
                         * @brief   Class constructor.
                         * @author  Juan Valera
                         * @date    01/11/2012
                         * @param   electoralConfiguration The electoral configuration.
                         */
                        VoteComplianceVerificator(Smartmatic::SAES::Voting::ElectoralConfiguration * electoralConfiguration);

                    private:

                        static VoteComplianceVerificator * m_instance;
                        static Smartmatic::Log::ISMTTLog * m_logger;	///< the logger
                        Smartmatic::SAES::Voting::ElectoralConfiguration * m_electoralConfiguration;   ///< electoral configuration
                        BeforePrintingVoteComplianceVerificator * m_beforePrintingVoteComplianceVerificator;
                        AfterScanningVoteComplianceVerificator * m_afterScanningVoteComplianceVerificator;
                        BeforeGeneratingResultsComplianceVerificator * m_beforeGeneratingResultsComplianceVerificator;
                };
            }
        }
    }
}



#endif /* VOTECOMPLIANCEVERIFICATOR_HXX_ */
