/**
 * @file	VotingExperienceController.cxx
 * @brief	voting experience controller class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#include "VotingExperienceController.hxx"
#include "VotingOriginator.hxx"
#include "Configuration/s-a-e-s-functionality-conf-manager.h"
#include "Configuration/s-a-e-s-conf-manager.h"
#include "VotingOriginator.hxx"
#include "FillVote.hxx"
#include <Voting/Vote/VoteWrapper.hxx>
#include <Voting/Vote/VoteCode.hxx>
#include <Voting/Vote/VoteFactory.hxx>
#include <Runtime/Environment/DirectoryManager.hxx>
#include <Log/SMTTLogManager.h>
#include <Voting/Status/VMProcessStatusWrapper.hxx>
#include <Resources/election-string-resources-manager.h>
#include <Resources/GeneralAudioResourcesManager.hxx>
#include <Resources/ElectionAudioResourcesManager.hxx>
#include <Voting/Election/election-schema.hxx>
#include <System/Utils/StringUtils.hxx>
#include <glibmm.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::Operation::Voting;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::SAES::Security;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Functionality::Configuration;
using namespace Smartmatic::SAES::Voting::Vote;
using namespace Smartmatic::SAES::Voting::Status;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::System::Utils;

ISMTTLog * VotingExperienceController::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.Operation.Voting.VotingExperienceController", GETTEXT_PACKAGE );

static Election::ElectionWrapper * electionWrapper = NULL; /**<instance of election wrapper*/
static VotingDevice::VotingDeviceWrapper * votingDeviceWrapper = NULL; /**<instance of voting device wrapper*/

static const std::string CONTEST_SELECTION_MESSAGE_KEY               = "smartmatic.saes.operation.AudioVoting.AudioVotingSelectionsReview.ContestMessage";
static const std::string PARTY_SELECTION_MESSAGE_KEY                 = "smartmatic.saes.operation.AudioVoting.AudioVotingSelectionsReview.PartyMessage";
static const std::string EMPTY_PARTY_MESSAGE_KEY                     = "smartmatic.saes.operation.audiovoting.AudioVotingSelectionsReview.EmptyPartyMessage";
static const std::string LIST_VOTE_MESSAGE_KEY                       = "smartmatic.saes.operation.AudioVoting.AudioVotingSelectionsReview.ListVoteMessage";
static const std::string PRINCIPAL_CANDIDATES_LIST_MESSAGE_KEY       = "smartmatic.saes.operation.AudioVoting.AudioVotingSelectionsReview.PrincipalCandidatesListMessage";
static const std::string STAND_IN_CANDIDATES_LIST_MESSAGE_KEY        = "smartmatic.saes.operation.AudioVoting.AudioVotingSelectionsReview.StandInCandidatesListMessage";
static const std::string STAND_IN_WITHOUT_PRINCIPAL_LIST_MESSAGE_KEY = "smartmatic.saes.operation.AudioVoting.AudioVotingSelectionsReview.StandInWithoutPrincipalListMessage";

/**
 * @brief  Gets the contest name key
 * @author manuel.arguelles
 * @date   Oct 13, 2017
 * @param  contest       The contest.
 * @param  configuration The electoral configuration.
 * @return The key of the contest name.
 */
static std::string getContestNameKey(Smartmatic::SAES::Voting::Election::Contest* contest, ElectoralConfiguration* configuration)
{
    Smartmatic::SAES::Voting::Election::Contest_class* contestClass;

    contestClass = configuration->getContestClassByCode(contest->getContest_class_code());

    return contestClass->getName_key();
}

/**
 * @brief  Creates a map with the contest code and the amount of times present in the memento.
 * @author manuel.arguelles
 * @date   Sep 14, 2018
 * @param  memento The voting memento.
 * @return The map with the contest code and the amount of times present in the memento.
 */
static std::map<std::string, int> createContestsCountMap(VotingMemento& memento)
{
    std::map<std::string, int>             contestsMap;
    VotingState*                           state                 = memento.getState();
    const std::list<std::string>           contestGroups         = state->getContestGroups();
    std::list<std::string>::const_iterator contestGroupsIterator = contestGroups.begin();

    for (; contestGroupsIterator != contestGroups.end(); ++contestGroupsIterator)
    {
        std::list<std::string>                 contests         = state->getContestsByContestGroup(*contestGroupsIterator);
        std::list<std::string>::const_iterator contestsIterator = contests.begin();

        for (; contestsIterator != contests.end(); ++contestsIterator)
        {
            const std::string code = *contestsIterator;

            contestsMap[code] = (contestsMap.find(code) != contestsMap.end()) ? contestsMap[code] + 1 : 1;
        }
    }

    return contestsMap;
}

/**
 * @brief  Create a list of contests present in the memento.
 * @author manuel.arguelles
 * @date   Sep 14, 2018
 * @param  memento The voting memento.
 * @return The list of the contests.
 */
static std::list<Election::Contest*> createContestsList(VotingMemento& memento)
{
    std::list<std::string>                     contests;
    const std::map<std::string, int>           contestsMap         = createContestsCountMap(memento);
    std::map<std::string, int>::const_iterator contestsMapIterator = contestsMap.begin();

    for (; contestsMapIterator != contestsMap.end(); ++contestsMapIterator)
    {
        const size_t contestMapCount = contestsMapIterator->second;

        if (memento.getState()->getContestGroups().size() == contestMapCount)
        {
            contests.push_back(contestsMapIterator->first);
        }
    }

    std::list<Election::Contest*> contestList;
    electionWrapper->getContestByContestClass(contests, contestList);

    return contestList;
}

VotingExperienceController::VotingExperienceController ( ElectoralConfiguration * electoralConfiguration )
{
    this->memento = NULL;
    this->electoralConfiguration = electoralConfiguration;
    this->originator = new VotingOriginator();
    this->fillVote = new FillVote ( electoralConfiguration );
    this->votingMode = VoteMode::VOTE_O;
    this->vote = NULL;
    this->m_electoralResources = ElectionStringResourcesManager::getInstance();
    this->m_generalAudioResources = GeneralAudioResourcesManager::getInstance();
    this->m_electoralAudioResources = ElectionAudioResourcesManager::getInstance();
}

VotingExperienceController::~VotingExperienceController()
{
    dispose();

    if ( originator != NULL )
    {
        delete originator;
        originator = NULL;
    }

    if ( fillVote != NULL )
    {
        delete fillVote;
        fillVote = NULL;
    }
}

bool VotingExperienceController::init ( CardData::Card_data & cardData )
throw ( Smartmatic::System::Exception::ElectionException,
        Smartmatic::System::Exception::VotingDeviceException )
{
    VMProcessStatusWrapper::getInstance()->addSuccess ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.Operation.Voting.VotingExperienceController.StartingVotingExperience" ) );

    dispose();
    this->cardData = cardData;
    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Voting.VotingExperienceController.StartingVotingExperience" ) );
    votingDeviceWrapper = electoralConfiguration->getVotingDevice();
    electionWrapper = electoralConfiguration->getElection();

    if ( !validateCardData ( cardData ) )
    {
        logger->Debug ( "ERROR - validate card data" );
        logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Voting.VotingExperienceController.StartVotingExperienceFailture" ) );
        VMProcessStatusWrapper::getInstance()->addFail ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.Operation.Voting.VotingExperienceController.StartVotingExperienceFailture" ) );
        return false;
    }

    switch ( cardData.getCard_type() )
    {
        case Smartmatic::SAES::Voting::CardData::Card_type::CARD_DATA_OFFICIAL_VOTE_CARD:
            votingMode = VoteMode::VOTE_O;
            break;

        default:
        case Smartmatic::SAES::Voting::CardData::Card_type::CARD_DATA_DEMOSTRATION_VOTE_CARD:
            votingMode = VoteMode::VOTE_D;
            break;
    }

    VotingDevice::Voting_device votingDevice = votingDeviceWrapper->getFirstVotingDevice();
    std::list <VotingDevice::Logical_voter_group * > & logicalVoterGroups ( electoralConfiguration->getLogicalVoterGroupList() );
    std::list <VotingDevice::Custom_attribute_definition> customAttributeDefinitions;
    std::list <std::string> contestByContestClass;
    customAttributeDefinitions = votingDeviceWrapper->getCustomAttributeDefinitionList ( votingDevice );
    electionWrapper->getContestCodeOrderByContestClass ( contestByContestClass );
    /*first configuration*/
    VotingState * state = new VotingState ( "", "", VotingState::START );
    state->setLogicalVoterGroups ( logicalVoterGroups );
    state->setCustomAttributeDefinitions ( customAttributeDefinitions );
    state->setContestByContestClass ( contestByContestClass );
    /*set contest group*/
    std::list < VotingDevice::Logical_voter_group * >::iterator itl;

    for ( itl = logicalVoterGroups.begin();
            itl != logicalVoterGroups.end();
            ++itl )
    {
        Election::Contest_group * cg = electoralConfiguration->getContestGroup ( ( *itl )->getContest_group_code() );
        Election::Contest_group::Contest_codeSequence & sequence ( cg->getContest_code() );
        unsigned int size = sequence.size();

        for ( unsigned int i = 0; i < size ; i++ )
        {
            state->addContestByContestGroup ( ( *itl )->getContest_group_code(), sequence[i] );
        }
    }

    originator->setState ( state );
    memento = originator->createMemento();
    caretaker.push_back ( memento );

    /*second configuration - filter from card data*/
    if ( !filterFromCardData ( cardData ) )
    {
        logger->Debug ( "ERROR - filter from card data" );
        logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Voting.VotingExperienceController.StartVotingExperienceFailture" ) );
        VMProcessStatusWrapper::getInstance()->addFail ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.Operation.Voting.VotingExperienceController.StartVotingExperienceFailture" ) );
        return false;
    }

    /*third configuration - filter custom attribute definitions*/
    filterEmptyCustomAttributeDefinition ();
    /*find contest fixed*/
    findFixedContests();
    logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Voting.VotingExperienceController.StartedVotingExperience" ) );
    return true;
}

void VotingExperienceController::dispose ()
{
    this->voteCode = "";
    this->operationModeCode = "";
    this->votingBoothCode = "";
    this->vdCode = "";
    this->regionCode = "";
    this->allContestsFixed.clear();
    this->allContests.clear();
    this->checksumCode = "";
    this->activationCode = "";
    int size = caretaker.size();

    for ( int i = 0; i < size ; i++ )
    {
        VotingMemento * vm = caretaker[i];
        delete vm;
    }

    caretaker.clear();
    memento = NULL;

    if ( vote )
    {
        delete vote;
        vote = NULL;
    }

    this->votingMode = VoteMode::VOTE_O;
}

bool VotingExperienceController::validateCardData ( CardData::Card_data & cardData )
{
    VotingExperienceControllerValidateCardDataType type;
    type = SAESFunctionalityConfigurationManager::getInstance()->getVotingExperienceControllerConfiguration().getValidateCardDataType();

    if ( type == VotingExperienceControllerValidateCardDataType::CONFIGURATION_ALL )
    {
        return validateLogicalVoterGroupCardData ( cardData ) && validateCustomAttributeDefinitionCardData ( cardData );
    }

    if ( type == VotingExperienceControllerValidateCardDataType::CONFIGURATION_LOGICAL_VOTER_GROUP_ONLY )
    {
        return validateLogicalVoterGroupCardData ( cardData );
    }

    return true;
}

bool VotingExperienceController::validateLogicalVoterGroupCardData ( CardData::Card_data & cardData )
{
    VMProcessStatusWrapper::getInstance()->addSuccess ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.Operation.Voting.VotingExperienceController.ValidateLogicalVoterGroupCardData" ) );

    VotingDevice::Logical_voter_group lvgcd = cardData.getLogical_voter_group ();
    std::list <VotingDevice::Logical_voter_group * > & logicalVoterGroups ( electoralConfiguration->getLogicalVoterGroupList() );
    std::list <VotingDevice::Logical_voter_group * >::iterator itlvg;

    for ( itlvg = logicalVoterGroups.begin();
            itlvg != logicalVoterGroups.end();
            ++itlvg )
    {
        VotingDevice::Logical_voter_group * lvg = *itlvg;

        if ( lvgcd.getOperation_mode_code() == lvg->getOperation_mode_code() &&
                lvgcd.getVoting_booth_code () == lvg->getVoting_booth_code () &&
                lvgcd.getRegion_code () == lvg->getRegion_code () )
        {
            return true;
        }
    }

    std::stringstream ss;
    ss << "ERROR - " << __func__ << " Info CardData not found." << std::endl;
    ss << " operation-mode-code = " << lvgcd.getOperation_mode_code();
    ss << " voting-booth-code = " << lvgcd.getVoting_booth_code ();
    ss << " region-code = " << lvgcd.getRegion_code ();
    logger->Debug ( ss.str() );
    return false;
}

bool VotingExperienceController::validateCustomAttributeDefinitionCardData ( CardData::Card_data & cardData )
{
    VMProcessStatusWrapper::getInstance()->addSuccess ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.Operation.Voting.VotingExperienceController.ValidateCustomAttributeDefinitionCardData" ) );

    VotingDevice::Voting_device votingDevice = votingDeviceWrapper->getFirstVotingDevice();
    VotingDevice::Logical_voter_group lvgcd = cardData.getLogical_voter_group ();
    std::list <VotingDevice::Custom_attribute_definition> cad;
    std::list <VotingDevice::Custom_attribute_definition>::iterator itcad;
    cad = votingDeviceWrapper->getCustomAttributeDefinitionList ( votingDevice, VotingDevice::Machine_type::VOTING_DEVICES_PM );

    if ( lvgcd.getCustom_attributes () == NULL )
    {
        return true;
    }

    unsigned int count = 0;
    VotingDevice::Custom_attributes & ca ( lvgcd.getCustom_attributes ().get () );

    for ( VotingDevice::Custom_attributes::Custom_attributeConstIterator
            c ( ca.getCustom_attribute ().begin () );
            c != ca.getCustom_attribute ().end ();
            ++c )
    {
        bool found = false;
        std::string code = ( *c ).getCode ();
        std::string value = ( *c ).getValue ();

        for ( itcad = cad.begin();
                itcad != cad.end();
                ++itcad )
        {
            if ( code == ( *itcad ).getCode() )
            {
                VotingDevice::Custom_attribute_definition_values::Custom_attribute_definition_valueSequence & v ( ( *itcad ).getCustom_attribute_definition_values().getCustom_attribute_definition_value() );

                for ( unsigned int i = 0; i < v.size(); i++ )
                {
                    if ( value == v[i].getValue() )
                    {
                        found = true;
                        count++;
                    }
                }

                if ( found )
                {
                    break;
                }
            }
        }

        if ( !found )
        {
            std::stringstream ss;
            ss << "ERROR - " << __func__ << " Info CardData not found.";
            ss << " code=" << code;
            ss << " value=" << value;
            logger->Debug ( ss.str() );
        }
    }

    return count == cad.size();
}

bool VotingExperienceController::filterFromCardData ( CardData::Card_data & cardData ) throw ( VotingDeviceException )
{
    VMProcessStatusWrapper::getInstance()->addSuccess ( GETTEXT_PACKAGE, N_ ( "Smartmatic.SAES.Operation.Voting.VotingExperienceController.FilterFromCardData" ) );

    VotingDevice::Logical_voter_group lvg;
    VotingDevice::Voting_device votingDevice = votingDeviceWrapper->getFirstVotingDevice();
    vdCode = votingDevice.getCode();


    voteCode = "";
    lvg = cardData.getLogical_voter_group ();

    activationCode = cardData.getActivation_code();
    operationModeCode = lvg.getOperation_mode_code ();
    votingBoothCode = lvg.getVoting_booth_code ();
    regionCode = lvg.getRegion_code ();

    if ( lvg.getCustom_attributes () != NULL )
    {
        VotingDevice::Custom_attributes & ca ( lvg.getCustom_attributes ().get () );

        for ( VotingDevice::Custom_attributes::Custom_attributeConstIterator
                c ( ca.getCustom_attribute ().begin () );
                c != ca.getCustom_attribute ().end ();
                ++c )
        {
            std::string code = ( *c ).getCode ();
            std::string value = ( *c ).getValue ();
            setResponse ( code, value, VotingState::START );
        }
    }

    return true;
}

void VotingExperienceController::filterEmptyCustomAttributeDefinition()
{
    std::list < VotingDevice::Custom_attribute_definition >::iterator it;
    std::list < VotingDevice::Custom_attribute_definition > customAttributeDefinitions = memento->getState()->getCustomAttributeDefinitions();

    for ( it = customAttributeDefinitions.begin();
            it != customAttributeDefinitions.end();
            ++it )
    {
        VotingDevice::Custom_attribute_definition_values::Custom_attribute_definition_valueSequence v = ( *it ).getCustom_attribute_definition_values().getCustom_attribute_definition_value ();
        int sizeLVG = memento->getState()->getLogicalVoterGroups().size();

        if ( v.size () == 1 && sizeLVG > 1 )
        {
            std::string code = ( *it ).getCode();
            std::string value = ( * ( v.begin() ) ).getValue();
            setResponse ( code, value, VotingState::AUTOMATIC_RESPONSE );
        }
        else
        {
            return;
        }
    }
}

std::list<VotingDevice::Logical_voter_group * > VotingExperienceController::filterLogicalVoterGroup ( const std::string& code, const std::string& value )
{
    std::list<VotingDevice::Logical_voter_group * >::iterator itl;
    std::list<VotingDevice::Logical_voter_group * > logicalVoterGroups;
    std::list<VotingDevice::Logical_voter_group * > templvg = memento->getState()->getLogicalVoterGroups();

    for ( itl = templvg.begin();
            itl != templvg.end();
            ++itl )
    {
        VotingDevice::Logical_voter_group * logical = *itl;

        if ( ! ( logical->getOperation_mode_code() == operationModeCode &&
                 logical->getVoting_booth_code() == votingBoothCode &&
                 logical->getRegion_code() == regionCode ) )
        {
            continue;
        }

        if ( logical->getCustom_attributes() == NULL )
        {
            continue;
        }

        VotingDevice::Custom_attributes attributes = logical->getCustom_attributes().get();
        VotingDevice::Custom_attributes::Custom_attributeSequence sequence;
        VotingDevice::Custom_attributes attributesTemp;
        bool remove = false;

        for ( VotingDevice::Custom_attributes::Custom_attributeIterator
                ca ( attributes.getCustom_attribute ().begin () );
                ca != attributes.getCustom_attribute ().end ();
                ++ca )
        {
            std::string customCode = ( *ca ).getCode();
            std::string customValue = ( *ca ).getValue();

            if ( code == customCode && value != customValue )
            {
                remove = true;
                break;
            }

            sequence.push_back ( ( *ca ) );
        }

        if ( !remove )
        {
            attributesTemp.setCustom_attribute ( sequence );
            logical->setCustom_attributes ( attributesTemp );
            logicalVoterGroups.push_back ( logical );
        }
    }

    return logicalVoterGroups;
}

bool VotingExperienceController::findCustomAttribute ( std::list<VotingDevice::Logical_voter_group * > & logicalVoterGroups, std::string code )
{
    std::list < VotingDevice::Logical_voter_group * >::iterator itl;

    /*check logical voter group value in custom attribute*/
    for ( itl = logicalVoterGroups.begin();
            itl != logicalVoterGroups.end();
            ++itl )
    {
        VotingDevice::Custom_attributes attributes = ( *itl )->getCustom_attributes().get();

        for ( VotingDevice::Custom_attributes::Custom_attributeIterator
                ca ( attributes.getCustom_attribute ().begin () );
                ca != attributes.getCustom_attribute ().end ();
                ++ca )
        {
            std::string aCode = ( *ca ).getCode();

            if ( code == aCode )
            {
                return true;
            }
        }
    }

    return false;
}

std::list < VotingDevice::Custom_attribute_definition >
VotingExperienceController::filterCustomAttributeDefinitionByLogicalVoterGroup (
    std::list<VotingDevice::Logical_voter_group * > & logicalVoterGroups, std::string code )
{
    std::list < VotingDevice::Custom_attribute_definition >::iterator it;
    std::list < VotingDevice::Custom_attribute_definition > customAttributeDefinitions;
    std::list < VotingDevice::Custom_attribute_definition > list = memento->getState()->getCustomAttributeDefinitions();

    /*clear custom attribute definition by response*/
    for ( it = list.begin();
            it != list.end();
            ++it )
    {
        std::string customAttributeCode = ( *it ).getCode();

        if ( code == customAttributeCode || !findCustomAttribute ( logicalVoterGroups , customAttributeCode ) )
        {
            continue;
        }

        customAttributeDefinitions.push_back ( *it );
    }

    return customAttributeDefinitions;
}

void VotingExperienceController::findFixedContests () throw ( ElectionException )
{
    findContests ( true );
}

void VotingExperienceController::findNewContests () throw ( ElectionException )
{
    findContests ( false );
}

void VotingExperienceController::findContests ( bool fixeds ) throw ( ElectionException )
{
    if ( memento == NULL )
    {
        logger->Error ( "getInstance memento is NULL." );
        return;
    }

    std::list < std::string >::iterator it;
    std::map < std::string, int > contestMap;
    std::map < std::string, int >::iterator itMap;
    std::list < std::string > temp = memento->getState()->getContestGroups();

    for ( it = temp.begin();
            it != temp.end();
            ++it )
    {
        std::list < std::string > sequence = memento->getState()->getContestsByContestGroup ( *it );
        std::list < std::string >::iterator its;

        for ( its = sequence.begin();
                its != sequence.end();
                ++its )
        {
            std::string code = ( *its );
            contestMap[ code ] = ( contestMap.find ( code ) != contestMap.end() ) ? contestMap[ code ] + 1 : 1;
        }
    }

    int size = temp.size();

    for ( itMap = contestMap.begin();
            itMap != contestMap.end();
            ++itMap )
    {
        int count = ( *itMap ).second;

        if ( size == count )
        {
            if ( fixeds )
            {
                allContestsFixed.push_back ( ( *itMap ).first );
            }
            else
            {
                allContests.push_back ( ( *itMap ).first );
            }

            /*clear contest of contest group*/
            std::list < std::string > temp = memento->getState()->getContestGroups();

            for ( it = temp.begin();
                    it != temp.end();
                    ++it )
            {
                std::list < std::string > sequence = memento->getState()->getContestsByContestGroup ( *it );
                std::list < std::string >::iterator its;

                for ( its = sequence.begin();
                        its != sequence.end();
                        ++its )
                {
                    std::string scode = ( *its );

                    if ( scode != ( *itMap ).first )
                    {
                        continue;
                    }

                    memento->getState()->removeContestByContestGroup ( ( *it ), scode );
                }
            }
        }
    }
}

VotingExperienceController::VotingExperienceType VotingExperienceController::getNextTypeAction()
{
    if ( memento == NULL )
    {
        std::stringstream ss;
        ss << __func__ << " - ERROR current memento is NULL.";
        logger->Debug ( ss.str() );
        logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Voting.VotingExperienceController.NextActionVotingExperienceClose" ) );
        return VOTING_CONTROLLER_CLOSE;
    }

    if ( vote )
    {
        logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Voting.VotingExperienceController.NextActionVotingExperienceClose" ) );
        return VOTING_CONTROLLER_CLOSE;
    }

    VotingExperienceType type;
    //    ALTERNATE_FIRST_QUESTIONS /**<operation type alternate questions, first questions and last contests fixed*/
    //    ALTERNATE_FIRST_CONTESTS_FIXED /**<operation type alternate questions, first contests fixed*/
    //    ALL_QUESTIONS_FIRST_QUESTIONS /**<operation type all questions, first questions and last contests*/
    //    ALL_QUESTIONS_FIRST_CONTESTS_FIXED /**<operation type all questions, first contests fixed, all questions and last constest*/
    findNewContests();

    switch ( SAESFunctionalityConfigurationManager::getInstance()->getVotingExperienceControllerConfiguration().getOperationType() )
    {
        case VotingExperienceControllerOperationType::CONFIGURATION_ALTERNATE_FIRST_QUESTIONS:
            type = alternateFirstQuestions();
            break;

        case VotingExperienceControllerOperationType::CONFIGURATION_ALTERNATE_FIRST_CONTESTS_FIXED:
            type = alternateFirstContestsFixed();
            break;

        case VotingExperienceControllerOperationType::CONFIGURATION_ALL_QUESTIONS_FIRST_QUESTIONS:
            type = allQuestionsFirstQuestions();
            break;

        case VotingExperienceControllerOperationType::CONFIGURATION_ALL_QUESTIONS_FIRST_CONTESTS_FIXED:
            type = allQuestionsFirstContestsFixed();
            break;

        case VotingExperienceControllerOperationType::CONFIGURATION_PRIORITY_BY_CONTEST_CLASS:
            type = priorityByContestClass();
            break;

        default:
            type = VOTING_CONTROLLER_CLOSE;
            break;
    }

    switch ( type )
    {
        case VOTING_CONTROLLER_CONTEST:
            logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Voting.VotingExperienceController.NextActionVotingExperienceContest" ) );
            break;

        case VOTING_CONTROLLER_CUSTOM_ATTRIBUTE_DEFINITION:
            logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Voting.VotingExperienceController.NextActionVotingExperienceCustomAttributeDefinition" ) );
            break;

        default:
        case VOTING_CONTROLLER_CLOSE:
            logger->Audit ( N_ ( "Smartmatic.SAES.Operation.Voting.VotingExperienceController.NextActionVotingExperienceClose" ) );
            break;
    }

    return type;
}

VotingExperienceController::VotingExperienceType VotingExperienceController::alternateFirstQuestions()
{
    unsigned int sizeLVG = memento->getState()->getLogicalVoterGroups().size();
    unsigned int sizeCAD = memento->getState()->getCustomAttributeDefinitions().size();

    if ( !allContests.empty() )
    {
        return VOTING_CONTROLLER_CONTEST;
    }

    if ( !allContestsFixed.empty() && sizeCAD == 0 )
    {
        allContests.insert ( allContests.end(), allContestsFixed.begin(), allContestsFixed.end() );
        allContestsFixed.clear();
        return VOTING_CONTROLLER_CONTEST;
    }

    if ( sizeCAD > 0 && sizeLVG > 1 )
    {
        return VOTING_CONTROLLER_CUSTOM_ATTRIBUTE_DEFINITION;
    }

    return VOTING_CONTROLLER_CLOSE;
}

VotingExperienceController::VotingExperienceType VotingExperienceController::alternateFirstContestsFixed()
{
    unsigned int sizeLVG = memento->getState()->getLogicalVoterGroups().size();
    unsigned int sizeCAD = memento->getState()->getCustomAttributeDefinitions().size();

    if ( !allContestsFixed.empty() )
    {
        allContests.insert ( allContests.end(), allContestsFixed.begin(), allContestsFixed.end() );
        allContestsFixed.clear();
        return VOTING_CONTROLLER_CONTEST;
    }

    if ( !allContests.empty() )
    {
        return VOTING_CONTROLLER_CONTEST;
    }

    if ( sizeCAD > 0 && sizeLVG > 1 )
    {
        return VOTING_CONTROLLER_CUSTOM_ATTRIBUTE_DEFINITION;
    }

    return VOTING_CONTROLLER_CLOSE;
}

VotingExperienceController::VotingExperienceType VotingExperienceController::allQuestionsFirstQuestions()
{
    unsigned int sizeLVG = memento->getState()->getLogicalVoterGroups().size();
    unsigned int sizeCAD = memento->getState()->getCustomAttributeDefinitions().size();

    if ( sizeCAD > 0 && sizeLVG > 1 )
    {
        return VOTING_CONTROLLER_CUSTOM_ATTRIBUTE_DEFINITION;
    }

    if ( !allContests.empty() )
    {
        return VOTING_CONTROLLER_CONTEST;
    }

    if ( !allContestsFixed.empty() )
    {
        allContests.insert ( allContests.end(), allContestsFixed.begin(), allContestsFixed.end() );
        allContestsFixed.clear();
        return VOTING_CONTROLLER_CONTEST;
    }

    return VOTING_CONTROLLER_CLOSE;
}

VotingExperienceController::VotingExperienceType VotingExperienceController::allQuestionsFirstContestsFixed()
{
    unsigned int sizeLVG = memento->getState()->getLogicalVoterGroups().size();
    unsigned int sizeCAD = memento->getState()->getCustomAttributeDefinitions().size();

    if ( !allContestsFixed.empty() )
    {
        allContests.insert ( allContests.end(), allContestsFixed.begin(), allContestsFixed.end() );
        allContestsFixed.clear();
        return VOTING_CONTROLLER_CONTEST;
    }

    if ( !allContests.empty() )
    {
        return VOTING_CONTROLLER_CONTEST;
    }

    if ( sizeCAD > 0 && sizeLVG > 1 )
    {
        return VOTING_CONTROLLER_CUSTOM_ATTRIBUTE_DEFINITION;
    }

    return VOTING_CONTROLLER_CLOSE;
}

VotingExperienceController::VotingExperienceType VotingExperienceController::priorityByContestClass()
{
    /*filter contest by contest class*/
    std::list <std::string> contestByContestGroup = memento->getState()->getAllContestsByContestGroup();
    std::list <std::string> contestFixed = allContestsFixed;
    contestFixed.insert ( contestFixed.end(), allContests.begin(), allContests.end() );
    allContests.clear();
    std::list <std::string> contestByContestClass;
    std::list <std::string> tempContestByContestClass = memento->getState()->getContestByContestClass();
    std::list <std::string>::iterator it;
    contestByContestGroup.insert ( contestByContestGroup.end(), contestFixed.begin(), contestFixed.end() );

    for ( it = tempContestByContestClass.begin();
            it != tempContestByContestClass.end();
            ++it )
    {
        std::list <std::string>::iterator itc;

        for ( itc = contestByContestGroup.begin();
                itc != contestByContestGroup.end();
                ++itc )
        {
            if ( ( *it ) == ( *itc ) )
            {
                contestByContestClass.push_back ( ( *it ) );
                contestByContestGroup.remove ( ( *itc ) );
                break;
            }
        }
    }

    bool next = false;

    /*order list contest by contest class*/

    do
    {
        next = false;

        if ( !contestByContestClass.empty() )
        {
            std::string c = ( * ( contestByContestClass.begin() ) );

            for ( it = contestFixed.begin();
                    it != contestFixed.end();
                    ++it )
            {
                if ( ( *it ) == c )
                {
                    allContests.push_back ( ( *it ) );
                    contestByContestClass.remove ( ( *it ) );
                    contestFixed.remove ( ( *it ) );
                    next = true;
                    break;
                }
            }
        }
    }
    while ( next );

    memento->getState()->setContestByContestClass ( contestByContestClass );
    int sizeLVG = memento->getState()->getLogicalVoterGroups().size();
    int sizeCAD = memento->getState()->getCustomAttributeDefinitions().size();

    if ( !allContests.empty() )
    {
        return VOTING_CONTROLLER_CONTEST;
    }

    if ( sizeCAD > 0 && sizeLVG > 1 )
    {
        return VOTING_CONTROLLER_CUSTOM_ATTRIBUTE_DEFINITION;
    }

    return VOTING_CONTROLLER_CLOSE;
}

VotingDevice::Custom_attribute_definition & VotingExperienceController::getPreviousCustomAttributeDefinition () throw ( VotingDeviceException )
{
    throw VotingDeviceException ( _ ( "Smartmatic.SAES.Operation.Voting.VotingExperienceController.UnderConstruction" ),
                                  "Smartmatic.SAES.Operation.Voting.VotingExperienceController.UnderConstruction",
                                  logger );
}

VotingDevice::Custom_attribute_definition & VotingExperienceController::getNextCustomAttributeDefinition () throw ( VotingDeviceException )
{
    if ( !memento->getState()->getCustomAttributeDefinitions().empty() )
    {
        currentCustomAttributeDefinition = memento->getState()->getCustomAttributeDefinitions().front();
        return getCurrentCustomAttributeDefinition();
    }

    throw VotingDeviceException ( _ ( "Smartmatic.SAES.Operation.Voting.VotingExperienceController.NextCustomAttributeDefinitionNotFound" ),
                                  "Smartmatic.SAES.Operation.Voting.VotingExperienceController.NextCustomAttributeDefinitionNotFound",
                                  logger );
}

VotingDevice::Custom_attribute_definition & VotingExperienceController::getCurrentCustomAttributeDefinition ()
{
    return currentCustomAttributeDefinition;
}

void VotingExperienceController::setCustomAttributeDefinitionResponse ( const std::string& code, const std::string& value ) throw ( VotingDeviceException )
{
    setResponse ( code, value, VotingState::USER_RESPONSE );
    filterEmptyCustomAttributeDefinition ();
}

void VotingExperienceController::setNullPartyResponse ( Smartmatic::SAES::Voting::Election::Contest & contest )
{
    memento->getState()->addBallotOption ( contest.getCode(), "", "" );
}

void VotingExperienceController::setResponse ( const std::string& code, const std::string& value, VotingState::TypeResponse response )
{
    if ( memento == NULL )
    {
        std::stringstream ss;
        ss << __func__ << " - ERROR current memento is NULL.";
        logger->Debug ( ss.str() );
        return;
    }

    VotingState * state = createVotingState(code, value, response);

    originator->setState ( state );
    memento = originator->createMemento();
    caretaker.push_back ( memento );
}

Smartmatic::SAES::Voting::Vote::Vote * VotingExperienceController::getVote () throw ( ElectionException )
{
    if ( vote )
    {
        return vote;
    }
    else
    {
        vote = Smartmatic::SAES::Voting::Vote::VoteFactory::createVote ( cardData );
    }

    if ( memento == NULL )
    {
        std::stringstream ss;
        ss << __func__ << " - ERROR current memento is NULL.";
        logger->Debug ( ss.str() );
        vote->setCode ( "" );
        vote->setVotingDeviceCode ( "" );
        vote->setLogicalVoterGroup ( "" );
        return vote;
    }

    int contestCount, ballotOptionCount;

    vote->setVotingDeviceCode ( vdCode );
    vote->setMode ( votingMode );
    fillVote->fillNormalVote ( *vote, contestCount, ballotOptionCount, caretaker );

    if ( voteCode.empty() )
    {
        voteCode = Smartmatic::SAES::Voting::Vote::VoteWrapper::generateCode ( *vote );
    }

    vote->setCode ( voteCode );
    return vote;
}

void VotingExperienceController::setBallotOptionState (
    std::string contestCode,
    std::string partyCode,
    std::string ballotOptionCode,
    bool state )
{
    if ( memento == NULL )
    {
        std::stringstream ss;
        ss << __func__ << " - ERROR current memento is NULL.";
        logger->Debug ( ss.str() );
        return;
    }

    if ( state )
    {
        memento->getState()->addBallotOption ( contestCode, partyCode, ballotOptionCode );
    }
    else
    {
        memento->getState()->removeBallotOption ( contestCode, partyCode, ballotOptionCode );
    }
}

std::list < Smartmatic::SAES::Voting::Election::Contest * >
VotingExperienceController::getCurrentContestList ()
throw ( Smartmatic::System::Exception::ElectionException )
{
    std::list < Smartmatic::SAES::Voting::Election::Contest * > list;
    electionWrapper->getContestByContestClass ( allContests, list );
    return list;
}

std::list < Smartmatic::SAES::Voting::Election::Contest * >
VotingExperienceController::getCurrentContestList (const int mementoPosition)
throw ( Smartmatic::System::Exception::ElectionException )
{
    std::list<std::string> currentContests;

    for (size_t i = 0; i < caretaker.size(); ++i)
    {
        if (caretaker[i]->getPosition() == mementoPosition)
        {
            currentContests = caretaker[i]->getState()->getContests();
            i = caretaker.size();
        }
    }

    std::list < Smartmatic::SAES::Voting::Election::Contest * > list;
    electionWrapper->getContestByContestClass ( currentContests, list );
    return list;
}

std::list < Smartmatic::SAES::Voting::Election::Ballot_option * >
VotingExperienceController::getBallotOptionsSelectedByContest (
    Smartmatic::SAES::Voting::Election::Contest & contest )
{
    std::list < Smartmatic::SAES::Voting::Election::Ballot_option * > b;
    return b;
}

std::list < Smartmatic::SAES::Voting::Election::Ballot_option * >
VotingExperienceController::getBallotOptionsSelectedByContestParty (
    Smartmatic::SAES::Voting::Election::Contest & contest,
    Smartmatic::SAES::Voting::Election::Party & party )
throw ( Smartmatic::System::Exception::ElectionException )
{
    std::list < Smartmatic::SAES::Voting::Election::Ballot_option * > list;
    std::list < std::string > codes = memento->getState()->getBallotOptions ( contest.getCode(), party.getCode() );
    electionWrapper->getBallotOptionsByContestParty ( contest, party, codes, list );
    return list;
}

void VotingExperienceController::clearBallotOptionsSelectedByContest (
    Smartmatic::SAES::Voting::Election::Contest & contest )
{
    memento->getState()->removeContest ( contest.getCode() );
}

void VotingExperienceController::resetVotingExperience ()
{
    init ( cardData );
}

void VotingExperienceController::clearBallotOptionsSelectedByContestParty (
    Smartmatic::SAES::Voting::Election::Contest & contest,
    Smartmatic::SAES::Voting::Election::Party & party )
{
    memento->getState()->removePartyByContest ( contest.getCode(), party.getCode() );
}

Smartmatic::SAES::Voting::Election::Party
VotingExperienceController::getPartySelectedFromContest (
    Smartmatic::SAES::Voting::Election::Contest & contest )
throw ( Smartmatic::System::Exception::ElectionException )
{
    std::list < std::string > partys;
    partys = memento->getState()->getPartys ( contest.getCode() );

    if ( !partys.empty() )
    {
        std::string partyCode = partys.front();
        return electionWrapper->getPartyByContest ( contest, partyCode );
    }

    throw Smartmatic::System::Exception::ElectionException (
        _ ( "Smartmatic.SAES.Operation.Voting.VotingExperienceController.PartyNotFound" ),
        _ ( "Smartmatic.SAES.Operation.Voting.VotingExperienceController.PartyNotFound" ),
        logger );
}

bool VotingExperienceController::isThereAnyPartySelectedFromContest (
    Smartmatic::SAES::Voting::Election::Contest & contest )
{
    std::list<std::string> parties = memento->getState()->getPartys (contest.getCode());

    if ((parties.empty()) || (parties.front().empty()))
    {
        return false;
    }
    else
    {
        return true;
    }
}

bool VotingExperienceController::moveContestListForward()
{
    allContests.clear();
    return true;
}

bool VotingExperienceController::canMoveBack ( const std::string& contestCode )
{
    bool navigateBetweenContest =
        SAESFunctionalityConfigurationManager::getInstance()->getVotingExperienceControllerConfiguration().getNavigateBetweenContest();
    bool canMove = false;

    if ( navigateBetweenContest )
    {
        bool first = firstContest ( contestCode );
        canMove = canMoveContestListBack();

        logger->Debug ( "Contest " + contestCode + " " + std::string ( first ? "is first" : "not is first" ) );
        logger->Debug ( "Contest " + contestCode + " " + std::string ( canMove ? "can move back" : "not can move back" ) );
    }

    return canMove;
}

bool VotingExperienceController::firstContest ( std::string contestCode )
{
    return !allContests.empty() && ( * ( allContests.begin() ) ).compare ( contestCode ) == 0;
}

bool VotingExperienceController::canMoveContestListBack()
{
    std::vector < VotingMemento * >::reverse_iterator rit;
    bool skipCustomAttributeAutomatic =
        SAESFunctionalityConfigurationManager::getInstance()->getVotingExperienceControllerConfiguration().getSkipCustomAttributeAutomatic();

    for ( rit = caretaker.rbegin();
            rit != caretaker.rend();
            ++rit )
    {
        VotingMemento * m = ( *rit );

        if ( !skipCustomAttributeAutomatic &&
                m->getState()->getResponse() == VotingState::AUTOMATIC_RESPONSE )
        {
            return false;
        }
        else if ( m->getState()->getResponse() == VotingState::USER_RESPONSE )
        {
            return true;
        }
    }

    return false;
}

void VotingExperienceController::deleteVote()
{
    if (vote)
    {
        delete vote;
        vote = NULL;
    }
}

void VotingExperienceController::returnToSpecificContest(const int mementoPosition)
{
    std::vector<VotingMemento*> tempCaretaker = caretaker;
    std::vector<VotingMemento*>::reverse_iterator caretakerReverseIterator = tempCaretaker.rbegin();

    for (; caretakerReverseIterator != tempCaretaker.rend(); ++caretakerReverseIterator )
    {
        VotingMemento *m = *caretakerReverseIterator;

        if (m->getPosition() <= mementoPosition)
        {
            memento = m;
            break;
        }
    }
}

void VotingExperienceController::setFinalMemento()
{
    memento = *(caretaker.end() - 1);
}

void VotingExperienceController::setCustomAttributeInformation(
    std::string &selectedOption,
    VotingMemento *currentMemento,
    const VotingState::TypeResponse &response)
{
    if (response == VotingState::USER_RESPONSE)
    {
        selectedOption += Glib::ustring::compose(" | %1", m_electoralAudioResources->getResource(currentMemento->getState()->getValue()));
    }
}

void VotingExperienceController::setContestInformation(std::string &selectedOption, const std::string &contestCode, const bool listOnlyContests)
{
    std::string contestNameKey = getContestNameKey(electoralConfiguration->getContestByCode(contestCode), electoralConfiguration);
    std::string contestName = m_electoralAudioResources->getResource(contestNameKey);

    if (listOnlyContests)
    {
        selectedOption += Glib::ustring::compose(" | %1", contestName);
    }
    else
    {
        std::string contestSelectionMessageKey = m_generalAudioResources->getResource(CONTEST_SELECTION_MESSAGE_KEY);
        Glib::ustring contestSelectionMessage = Glib::ustring::compose (contestSelectionMessageKey, contestName);
        selectedOption += Glib::ustring::compose(" | %1", contestSelectionMessage);
    }
}

void VotingExperienceController::setPartyInformation(std::string &selectedOption, const std::list<std::string>::iterator &partiesIterator)
{
    std::string partyNameKey = electoralConfiguration->getPartyByCode(*partiesIterator)->getName().get();
    std::string partyName = m_electoralAudioResources->getResource(partyNameKey);
    std::string partySelectionMessageKey = m_generalAudioResources->getResource(PARTY_SELECTION_MESSAGE_KEY);
    std::string partySelectionMessage = Glib::ustring::compose (partySelectionMessageKey, partyName);
    selectedOption += Glib::ustring::compose(" | %1", partySelectionMessage);
}

void VotingExperienceController::setListVoteInformation(
    std::string &selectedOption,
    bool &anyListVote,
    const std::list<std::string> &ballotOptions,
    const std::string &contestsIterator,
    const std::string &partiesIterator)
{
    const std::list<Smartmatic::SAES::Voting::Election::Ballot_option*> listBallotOptions = electoralConfiguration->getBallotOptionsListByContestParty(contestsIterator, partiesIterator);

    if (!listBallotOptions.empty())
    {
        std::string selectedListOption = "";
        std::list<Smartmatic::SAES::Voting::Election::Ballot_option*>::const_iterator configurationListBallotOptionsIterator = listBallotOptions.begin();

        for (; configurationListBallotOptionsIterator != listBallotOptions.end(); ++configurationListBallotOptionsIterator)
        {
            for (std::list< std::string>::const_iterator ballotOptionsIterator = ballotOptions.begin(); ballotOptionsIterator != ballotOptions.end(); ++ballotOptionsIterator)
            {
                if (((*ballotOptionsIterator).empty() == false) && ((*ballotOptionsIterator) == (*configurationListBallotOptionsIterator)->getCode()))
                {
                    anyListVote = true;
                    std::string ballotOptionNameKey = electoralConfiguration->getEligibleOptionFromBallotOptionCode(contestsIterator, *ballotOptionsIterator)->getBallot_name_key();
                    selectedListOption = Glib::ustring::compose(m_generalAudioResources->getResource(LIST_VOTE_MESSAGE_KEY), m_electoralAudioResources->getResource(ballotOptionNameKey));
                    break;
                }
            }
        }

        if (anyListVote)
        {
            std::string selectionMessage = Glib::ustring::compose(" | %1", m_generalAudioResources->getResource(PRINCIPAL_CANDIDATES_LIST_MESSAGE_KEY));
            selectedOption += Glib::ustring::compose(selectionMessage, selectedListOption);
        }
    }
}

void VotingExperienceController::setCandidatesInformation(
    std::string &selectedOption,
    bool &anyPrincipal,
    const std::list<std::string> &ballotOptions,
    const std::list<Smartmatic::SAES::Voting::Election::Ballot_option*> &configurationBallotOptions,
    const std::string &candidatesListMessageKey,
    const std::string &contestsIterator,
    const std::string &partiesIterator)
{
    if (configurationBallotOptions.empty() == false)
    {
        std::string selectedCandidates = "";
        std::list<Smartmatic::SAES::Voting::Election::Ballot_option*>::const_iterator configurationBallotOptionsIterator = configurationBallotOptions.begin();

        for (; configurationBallotOptionsIterator != configurationBallotOptions.end(); ++configurationBallotOptionsIterator)
        {
            for (std::list< std::string>::const_iterator ballotOptionsIterator = ballotOptions.begin(); ballotOptionsIterator != ballotOptions.end(); ++ballotOptionsIterator)
            {
                if (((*ballotOptionsIterator).empty() == false) && ((*ballotOptionsIterator) == (*configurationBallotOptionsIterator)->getCode()))
                {
                    anyPrincipal = true;
                    std::string ballotOptionNameKey = electoralConfiguration->getEligibleOptionFromBallotOptionCode(contestsIterator, *ballotOptionsIterator)->getBallot_name_key();
                    selectedCandidates += Glib::ustring::compose("%1. ", m_electoralAudioResources->getResource(ballotOptionNameKey));
                }
            }
        }

        if (selectedCandidates.size() > 0)
        {
            selectedCandidates = selectedCandidates.substr(0, selectedCandidates.size() - 2);

            std::string selectionMessage = Glib::ustring::compose(" | %1", m_generalAudioResources->getResource(candidatesListMessageKey));
            selectedOption += Glib::ustring::compose(selectionMessage, selectedCandidates);
        }
    }
}

std::list<std::string> VotingExperienceController::getOrderedContestsList(VotingMemento *currentMemento)
{
    std::list<SAES::Voting::Election::Contest*> currentContestList = getCurrentContestList(currentMemento->getPosition());
    std::list<SAES::Voting::Election::Contest*>::iterator it = currentContestList.begin();
    std::list<std::string> contests;

    for (; it != currentContestList.end(); ++it)
    {
        contests.push_back((*it)->getCode());
    }

    return contests;
}

std::vector<std::string> VotingExperienceController::getSelectedOptions(const bool listOnlyContests)
{
    std::vector<std::string> selectedOptions;

    for (size_t i = 0; i < caretaker.size(); ++i)
    {
        VotingMemento *currentMemento = caretaker[i];
        std::list<std::string> contests = getOrderedContestsList(currentMemento);

        if (!contests.empty())
        {
            int currentMementoContestsIndex = 0;

            std::list<std::string>::iterator contestsIterator = contests.begin();

            for (; contestsIterator != contests.end(); ++contestsIterator )
            {
                std::string selectedOption = Glib::ustring::compose("%1 | %2", currentMemento->getPosition(), currentMementoContestsIndex++);
                setContestInformation(selectedOption, *contestsIterator, listOnlyContests);
                std::list<std::string> parties = currentMemento->getState()->getPartys(*contestsIterator);

                for (std::list<std::string>::iterator partiesIterator = parties.begin(); partiesIterator != parties.end() && listOnlyContests == false; ++partiesIterator)
                {
                    if ((*partiesIterator).empty() == false)
                    {
                        bool anyListVote = false;
                        bool anyPrincipal = false;
                        std::string principalCandidatesMessageKey = PRINCIPAL_CANDIDATES_LIST_MESSAGE_KEY;
                        std::string standInCandidatesMessageKey = STAND_IN_CANDIDATES_LIST_MESSAGE_KEY;
                        std::list<std::string> ballotOptions = currentMemento->getState()->getBallotOptions(*contestsIterator, *partiesIterator);

                        setListVoteInformation(selectedOption, anyListVote, ballotOptions, *contestsIterator, *partiesIterator);

                        if (anyListVote)
                        {
                            principalCandidatesMessageKey = "%1.";
                        }

                        std::list<Smartmatic::SAES::Voting::Election::Ballot_option*> principalBallotOptions = electoralConfiguration->getBallotOptionsPrincipalByContestParty(*contestsIterator, *partiesIterator);
                        setCandidatesInformation(selectedOption, anyPrincipal, ballotOptions, principalBallotOptions, principalCandidatesMessageKey, *contestsIterator, *partiesIterator);

                        if (anyListVote == false && anyPrincipal == false)
                        {
                            standInCandidatesMessageKey = STAND_IN_WITHOUT_PRINCIPAL_LIST_MESSAGE_KEY;
                        }

                        std::list<Smartmatic::SAES::Voting::Election::Ballot_option*> standInBallotOptions = electoralConfiguration->getBallotOptionsStandInByContestParty(*contestsIterator, *partiesIterator);
                        setCandidatesInformation(selectedOption, anyPrincipal, ballotOptions, standInBallotOptions, standInCandidatesMessageKey, *contestsIterator, *partiesIterator);

                        setPartyInformation(selectedOption, partiesIterator);
                    }
                    else
                    {
                        selectedOption += Glib::ustring::compose(" | %1", m_generalAudioResources->getResource(EMPTY_PARTY_MESSAGE_KEY));
                    }
                }

                selectedOptions.push_back(selectedOption);
            }
        }
    }

    return selectedOptions;
}

void VotingExperienceController::deselectOptions(int currentContestsIndex)
{
    std::list<std::string> contests = getOrderedContestsList(memento);
    std::list<std::string>::iterator contestsIterator = contests.begin();
    int index = 0;

    for (; contestsIterator != contests.end(); ++contestsIterator )
    {
        if (index == currentContestsIndex)
        {
            std::list<std::string> parties = memento->getState()->getPartys(*contestsIterator);

            for (std::list<std::string>::iterator partiesIterator = parties.begin(); partiesIterator != parties.end(); ++partiesIterator)
            {
                std::list<std::string> ballotOptions = memento->getState()->getBallotOptions(*contestsIterator, *partiesIterator);

                for (std::list< std::string>::const_iterator ballotOptionsIterator = ballotOptions.begin(); ballotOptionsIterator != ballotOptions.end(); ++ballotOptionsIterator)
                {
                    if ((*ballotOptionsIterator).empty() == false)
                    {
                        memento->getState()->removeBallotOption(*contestsIterator, *partiesIterator, *ballotOptionsIterator);
                    }
                }

                memento->getState()->removePartyByContest(*contestsIterator, *partiesIterator);
            }
        }

        ++index;
    }
}

bool VotingExperienceController::moveContestListBack()
{
    if ( !canMoveContestListBack() )
    {
        return false;
    }

    /*delete current memento*/
    std::vector < VotingMemento * >::reverse_iterator rit;
    std::vector < VotingMemento * > tempCaretaker = caretaker;
    int position = 0;
    bool moveForware = false;

    for ( rit = tempCaretaker.rbegin();
            rit != tempCaretaker.rend();
            ++rit )
    {
        VotingMemento * m = ( *rit );
        position = m->getPosition();

        if ( moveForware )
        {
            break;
        }

        if ( m->getState()->getResponse() == VotingState::START )
        {
            // no move forware one position
            break;
        }
        else if ( m->getState()->getResponse() == VotingState::USER_RESPONSE )
        {
            // move forware one position
            moveForware = true;
        }
    }

    for ( rit = tempCaretaker.rbegin();
            rit != tempCaretaker.rend();
            ++rit )
    {
        VotingMemento * m = ( *rit );

        if ( m->getPosition() > position )
        {
            delete m;
            caretaker.pop_back();
        }
        else
        {
            memento = m;
            break;
        }
    }

    allContests.clear();
    return true;
}

Smartmatic::SAES::Voting::Vote::VoteMode VotingExperienceController::getVotingMode()
{
    return votingMode;
}

void VotingExperienceController::setVotingMode ( Smartmatic::SAES::Voting::Vote::VoteMode mode )
{
    votingMode = mode;
}

std::string VotingExperienceController::getChecksumVote()
{
    return checksumCode;
}

std::string VotingExperienceController::getActivationCode()
{
    return this->activationCode;
}

void VotingExperienceController::addContestsByContestGroup(
  VotingState* state,
  const std::list<VotingDevice::Logical_voter_group*>& logicalVoterGroups)
{
    std::list<VotingDevice::Logical_voter_group*>::const_iterator logicalVoterGroupsIterator = logicalVoterGroups.begin();

    for (; logicalVoterGroupsIterator != logicalVoterGroups.end(); ++logicalVoterGroupsIterator)
    {
        const std::list<std::string>           contestGroups         = memento->getState()->getContestGroups();
        std::list<std::string>::const_iterator contestGroupsIterator = contestGroups.begin();

        for (; contestGroupsIterator != contestGroups.end(); ++contestGroupsIterator)
        {
            if ((*logicalVoterGroupsIterator)->getContest_group_code() == *contestGroupsIterator)
            {
                std::list<std::string> contests = memento->getState()->getContestsByContestGroup(*contestGroupsIterator);
                std::list<std::string>::iterator contestsIterator = contests.begin();

                for (; contestsIterator != contests.end(); ++contestsIterator)
                {
                    state->addContestByContestGroup(*contestGroupsIterator, *contestsIterator);
                }
            }
        }
    }
}

VotingState* VotingExperienceController::createVotingState(const std::string& code, const std::string& value, const VotingState::TypeResponse response)
{
    std::list<VotingDevice::Logical_voter_group*>        logicalVoterGroups         = filterLogicalVoterGroup(code, value);
    const std::list<VotingDevice::Custom_attribute_definition> customAttributeDefinitions = filterCustomAttributeDefinitionByLogicalVoterGroup(logicalVoterGroups, code);
    const std::list<std::string>                               contestByContestClass      = memento->getState()->getContestByContestClass();

    VotingState* const state = new VotingState(code, value, response);

    state->setLogicalVoterGroups(logicalVoterGroups);
    state->setCustomAttributeDefinitions(customAttributeDefinitions);
    state->setContestByContestClass(contestByContestClass);

    addContestsByContestGroup(state, logicalVoterGroups);

    return state;
}

Election::Contest* VotingExperienceController::getNextContestFromCustomAttribute(
    const std::string& code,
    const std::string& value)
{
    VotingState*  const                 votingState  = createVotingState(code, value, VotingState::USER_RESPONSE);
    VotingMemento                       tmpMemento(votingState, 0);
    const std::list<Election::Contest*> contestsList = createContestsList(tmpMemento);

    return contestsList.front();
}

std::string VotingExperienceController::getNextContestCodeFromCustomAttribute(
    const std::string& code,
    const std::string& value)
{
    Election::Contest* contest = getNextContestFromCustomAttribute(code, value);

    return contest->getCode();
}
