#ifndef TEST_CARD_CONTROLLER_TEST
#define TEST_CARD_CONTROLLER_TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <Security/KeyManager.hxx>
#include <Security/Passphrase.hxx>
#include <System/Security/OpenSSLSecurity.hxx>
#include <Voting/CardData/CardDataFactory.hxx>
#include <Voting/VotingDevice/LogicalVoterGroupFactory.hxx>
#include <Operation/Devices/CardController.hxx>
#include <tests/MockCardController.hxx>
#include <tests/MockSmartcard.hxx>
#include <System/SafeBuffer.h>
#include <tests/MockKeyManager.hxx>

class MockDummyCardController : public Smartmatic::SAES::Operation::Devices::CardController
{
    public:
        explicit MockDummyCardController ( Smartmatic::SAES::Card::SmartcardInterface * smartcard )
            : Smartmatic::SAES::Operation::Devices::CardController ( smartcard )
        {

        }
};

class MockDummySmartcard : public MockSmartcard
{
    public:
        MockDummySmartcard ()
        {

        }

        virtual bool read ( Smartmatic::System::SafeBuffer & buffer, Smartmatic::SAES::Security::CryptoData & cryptoData )
        {
            buffer = bufferToProcess;
            return true;
        }

        virtual bool write ( Smartmatic::System::SafeBuffer & buffer, Smartmatic::SAES::Security::CryptoData & cryptoData )
        {
            bufferToProcess = buffer;
            return true;
        }

        Smartmatic::System::SafeBuffer bufferToProcess;
};

class CardControllerTest : public CPPUNIT_NS::TestFixture
{
    private:

        typedef Smartmatic::SAES::Operation::Devices::CardController CardController;

        Smartmatic::SAES::Common::Security::KeyManager * keyManager;
        Smartmatic::SAES::Common::Security::Passphrase * passManager;
        Smartmatic::SAES::Voting::CardData::Card_data cardData;

    public:

        CardControllerTest ()
        {
            keyManager = NULL;
            passManager = NULL;
        }

        void setUp()
        {
            MockKeyManager* mockedKeyManager = new MockKeyManager();

            Smartmatic::System::Security::OpenSSLSecurity::init();

            keyManager  = Smartmatic::SAES::Common::Security::KeyManager::getInstance(mockedKeyManager);
            passManager = Smartmatic::SAES::Common::Security::Passphrase::getInstance();

            Smartmatic::System::SafeBuffer key ( "abcd1234567890" );
            keyManager->addKeyByType ( key, Smartmatic::SAES::Common::Configuration::KeyType::CONFIGURATION_SECRET_KEY );

            Smartmatic::SAES::Voting::VotingDevice::Logical_voter_group clvg;

            clvg = Smartmatic::SAES::Voting::VotingDevice::LogicalVoterGroupFactory::createLogicalVoterGroup ();
            cardData = Smartmatic::SAES::Voting::CardData::CardDataFactory::createCardData ( clvg, NULL );
        }

        void tearDown()
        {
            if ( keyManager )
            {
                delete keyManager;
                keyManager = NULL;
            }

            if ( passManager )
            {
                delete passManager;
                passManager = NULL;
            }

            Smartmatic::System::Security::OpenSSLSecurity::clear();
        }

        void testCheckOpenDevice()
        {
            MockSmartcard  * mockSmartcard = new MockSmartcard ();

            MOCKPP_CHAINER_FOR_EXT ( MockSmartcard, open, ext ) open_mock ( mockSmartcard );

            open_mock
            .expects ( MOCKPP_NS::exactly ( 2 ) )
            .will ( MOCKPP_NS::onConsecutiveCalls (
                        MOCKPP_NS::returnValue <bool> ( false ),
                        MOCKPP_NS::returnValue <bool> ( true ) ) );

            MockDummyCardController mockSmartcardController ( mockSmartcard );
            CardController::getInstance ( &mockSmartcardController );

            CPPUNIT_ASSERT ( false == CardController::getInstance()->open ( "" ) );
            CPPUNIT_ASSERT ( true == CardController::getInstance()->open ( "" ) );

            mockSmartcard->verify();
        }

        void testCheckCloseDeviceWithoutSmartcard()
        {
            MockDummyCardController mockSmartcardController ( NULL );
            CardController::getInstance ( &mockSmartcardController );

            CPPUNIT_ASSERT ( false == CardController::getInstance()->close() );
            CPPUNIT_ASSERT ( false == CardController::getInstance()->close() );
        }

        void testCheckCloseDevice()
        {
            MockSmartcard  * mockSmartcard = new MockSmartcard ();

            MOCKPP_CHAINER_FOR_EXT ( MockSmartcard, open, ext ) open_mock ( mockSmartcard );
            MOCKPP_CHAINER_FOR ( MockSmartcard, close ) close_mock ( mockSmartcard );

            open_mock.expects ( MOCKPP_NS::never () );

            close_mock
            .expects ( MOCKPP_NS::exactly ( 3 ) )
            .will ( MOCKPP_NS::onConsecutiveCalls (
                        MOCKPP_NS::returnValue <bool> ( true ),
                        MOCKPP_NS::returnValue <bool> ( false ),
                        MOCKPP_NS::returnValue <bool> ( true ) ) );

            MockDummyCardController mockSmartcardController ( mockSmartcard );
            CardController::getInstance ( &mockSmartcardController );

            CPPUNIT_ASSERT ( true == CardController::getInstance()->close() );
            CPPUNIT_ASSERT ( false == CardController::getInstance()->close() );
            CPPUNIT_ASSERT ( true == CardController::getInstance()->close() );

            mockSmartcard->verify();
        }

        void testCheckIsCardPresentWithoutSmartcard()
        {
            MockDummyCardController mockSmartcardController ( NULL );
            CardController::getInstance ( &mockSmartcardController );

            CPPUNIT_ASSERT ( false == CardController::getInstance()->isCardPresent() );
            CPPUNIT_ASSERT ( false == CardController::getInstance()->isCardPresent() );
        }

        void testIsCardPresent()
        {
            MockSmartcard  * mockSmartcard = new MockSmartcard ();

            MOCKPP_CHAINER_FOR_EXT ( MockSmartcard, open, ext ) open_mock ( mockSmartcard );
            MOCKPP_CHAINER_FOR ( MockSmartcard, isCardPresent ) isCardPresent_mock ( mockSmartcard );

            open_mock.expects ( MOCKPP_NS::never () );

            isCardPresent_mock
            .expects ( MOCKPP_NS::exactly ( 3 ) )
            .will ( MOCKPP_NS::onConsecutiveCalls (
                        MOCKPP_NS::returnValue <bool> ( true ),
                        MOCKPP_NS::returnValue <bool> ( false ),
                        MOCKPP_NS::returnValue <bool> ( true ) ) );

            MockDummyCardController mockSmartcardController ( mockSmartcard );
            CardController::getInstance ( &mockSmartcardController );

            CPPUNIT_ASSERT ( true == CardController::getInstance()->isCardPresent() );
            CPPUNIT_ASSERT ( false == CardController::getInstance()->isCardPresent() );
            CPPUNIT_ASSERT ( true == CardController::getInstance()->isCardPresent() );

            mockSmartcard->verify();
        }

        void testWriteWithoutSmartcard()
        {
            MockDummyCardController mockSmartcardController ( NULL );
            CardController::getInstance ( &mockSmartcardController );

            CPPUNIT_ASSERT ( false == CardController::getInstance()->writeCardData ( cardData ) );
        }

        void testWriteSmartcard()
        {
            MockSmartcard  * mockSmartcard = new MockSmartcard ();

            MOCKPP_CHAINER_FOR_EXT ( MockSmartcard, open, ext ) open_mock ( mockSmartcard );
            MOCKPP_CHAINER_FOR_EXT ( MockSmartcard, write, ext ) write_mock ( mockSmartcard );
            MOCKPP_CHAINER_FOR_EXT ( MockSmartcard, read, ext ) read_mock ( mockSmartcard );

            open_mock
            .expects ( MOCKPP_NS::once () )
            .will ( MOCKPP_NS::returnValue <bool> ( true ) );

            write_mock
            .expects ( MOCKPP_NS::exactly ( 3 ) )
            .will ( MOCKPP_NS::onConsecutiveCalls (
                        MOCKPP_NS::returnValue <bool> ( false ),
                        MOCKPP_NS::returnValue <bool> ( true ),
                        MOCKPP_NS::returnValue <bool> ( true ) ) );

            read_mock
            .expects ( MOCKPP_NS::exactly ( 2 ) )
            .will ( MOCKPP_NS::onConsecutiveCalls (
                        MOCKPP_NS::returnValue <bool> ( false ),
                        MOCKPP_NS::returnValue <bool> ( true ) ) );

            MockDummyCardController mockSmartcardController ( mockSmartcard );
            CardController::getInstance ( &mockSmartcardController );

            CPPUNIT_ASSERT ( true == CardController::getInstance()->open ( "" ) );
            CPPUNIT_ASSERT ( false == CardController::getInstance()->writeCardData ( cardData ) );
            CPPUNIT_ASSERT ( false == CardController::getInstance()->writeCardData ( cardData ) );
            CPPUNIT_ASSERT ( false == CardController::getInstance()->writeCardData ( cardData ) );

            mockSmartcard->verify();
        }

        void testReadWithoutSmartcard()
        {
            MockDummyCardController mockSmartcardController ( NULL );
            CardController::getInstance ( &mockSmartcardController );

            CPPUNIT_ASSERT ( false == CardController::getInstance()->readCardData ( cardData ) );
        }

        void testReadSmartcard()
        {
            MockSmartcard  * mockSmartcard = new MockSmartcard ();

            MOCKPP_CHAINER_FOR_EXT ( MockSmartcard, open, ext ) open_mock ( mockSmartcard );
            MOCKPP_CHAINER_FOR_EXT ( MockSmartcard, write, ext ) write_mock ( mockSmartcard );
            MOCKPP_CHAINER_FOR_EXT ( MockSmartcard, read, ext ) read_mock ( mockSmartcard );

            open_mock
            .expects ( MOCKPP_NS::once () )
            .will ( MOCKPP_NS::returnValue <bool> ( true ) );

            write_mock.expects ( MOCKPP_NS::never() );

            read_mock
            .expects ( MOCKPP_NS::exactly ( 2 ) )
            .will ( MOCKPP_NS::onConsecutiveCalls (
                        MOCKPP_NS::returnValue <bool> ( false ),
                        MOCKPP_NS::returnValue <bool> ( true ) ) );

            MockDummyCardController mockSmartcardController ( mockSmartcard );
            CardController::getInstance ( &mockSmartcardController );

            CPPUNIT_ASSERT ( true == CardController::getInstance()->open ( "" ) );
            CPPUNIT_ASSERT ( false == CardController::getInstance()->readCardData ( cardData ) );
            CPPUNIT_ASSERT ( false == CardController::getInstance()->readCardData ( cardData ) );

            mockSmartcard->verify();
        }

        void testWriteAndReadSmartcard()
        {
            MockDummySmartcard  * mockSmartcard = new MockDummySmartcard ();

            MOCKPP_CHAINER_FOR_EXT ( MockSmartcard, open, ext ) open_mock ( mockSmartcard );

            open_mock
            .expects ( MOCKPP_NS::once () )
            .will ( MOCKPP_NS::returnValue <bool> ( true ) );

            MockDummyCardController mockSmartcardController ( mockSmartcard );
            CardController::getInstance ( &mockSmartcardController );

            CPPUNIT_ASSERT ( true == CardController::getInstance()->open ( "" ) );
            CPPUNIT_ASSERT ( true == CardController::getInstance()->writeCardData ( cardData ) );
            CPPUNIT_ASSERT ( true == CardController::getInstance()->readCardData ( cardData ) );

            mockSmartcard->verify();
        }

        void testCallTestFail_SelectFail()
        {
            MockDummyCardController mockSmartcardController ( NULL );
            CardController::getInstance ( &mockSmartcardController );

            CPPUNIT_ASSERT ( false == CardController::getInstance()->test() );
        }

        void testCallTestFail_SelectFailAgain()
        {
            MockSmartcard  * mockSmartcard = new MockSmartcard ();

            MOCKPP_CHAINER_FOR_EXT ( MockSmartcard, open, ext ) open_mock ( mockSmartcard );
            MOCKPP_CHAINER_FOR_EXT ( MockSmartcard, write, ext ) write_mock ( mockSmartcard );
            MOCKPP_CHAINER_FOR_EXT ( MockSmartcard, read, ext ) read_mock ( mockSmartcard );
            MOCKPP_CHAINER_FOR ( MockSmartcard, test ) test_mock ( mockSmartcard );

            open_mock.expects ( MOCKPP_NS::never () );
            write_mock.expects ( MOCKPP_NS::never() );
            read_mock.expects ( MOCKPP_NS::never() );

            test_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue <bool> ( false ) );

            MockDummyCardController mockSmartcardController ( mockSmartcard );
            CardController::getInstance ( &mockSmartcardController );

            CPPUNIT_ASSERT ( false == CardController::getInstance()->test() );

            mockSmartcard->verify();
        }

        void testCallTestFail_SelectSuccess_WriteFail()
        {
            MockSmartcard  * mockSmartcard = new MockSmartcard ();

            MOCKPP_CHAINER_FOR_EXT ( MockSmartcard, open, ext ) open_mock ( mockSmartcard );
            MOCKPP_CHAINER_FOR ( MockSmartcard, test ) test_mock ( mockSmartcard );
            MOCKPP_CHAINER_FOR_EXT ( MockSmartcard, write, ext ) write_mock ( mockSmartcard );
            MOCKPP_CHAINER_FOR_EXT ( MockSmartcard, read, ext ) read_mock ( mockSmartcard );

            open_mock.expects ( MOCKPP_NS::never () );
            read_mock.expects ( MOCKPP_NS::never() );
            write_mock.expects ( MOCKPP_NS::never() );

            test_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue <bool> ( true ) );

            MockDummyCardController mockSmartcardController ( mockSmartcard );
            CardController::getInstance ( &mockSmartcardController );

            CPPUNIT_ASSERT ( false == CardController::getInstance()->test() );

            mockSmartcard->verify();
        }

        void testCallTestFail_SelectSuccess_WriteFailAgain()
        {
            MockSmartcard  * mockSmartcard = new MockSmartcard ();

            MOCKPP_CHAINER_FOR_EXT ( MockSmartcard, open, ext ) open_mock ( mockSmartcard );
            MOCKPP_CHAINER_FOR ( MockSmartcard, test ) test_mock ( mockSmartcard );
            MOCKPP_CHAINER_FOR_EXT ( MockSmartcard, write, ext ) write_mock ( mockSmartcard );
            MOCKPP_CHAINER_FOR_EXT ( MockSmartcard, read, ext ) read_mock ( mockSmartcard );

            open_mock
            .expects ( MOCKPP_NS::once () )
            .will ( MOCKPP_NS::returnValue <bool> ( true ) );

            read_mock.expects ( MOCKPP_NS::never() );

            write_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue <bool> ( false ) );

            test_mock
            .expects ( MOCKPP_NS::once() )
            .will ( MOCKPP_NS::returnValue <bool> ( true ) );

            MockDummyCardController mockSmartcardController ( mockSmartcard );
            CardController::getInstance ( &mockSmartcardController );

            CPPUNIT_ASSERT ( true == CardController::getInstance()->open ( "" ) );
            CPPUNIT_ASSERT ( false == CardController::getInstance()->test() );

            mockSmartcard->verify();
        }

        void testCallTestFail_SelectSuccess_WriteSuccess_Read()
        {
            MockSmartcard  * mockSmartcard = new MockSmartcard ();

            MOCKPP_CHAINER_FOR_EXT ( MockSmartcard, open, ext ) open_mock ( mockSmartcard );
            MOCKPP_CHAINER_FOR ( MockSmartcard, test ) test_mock ( mockSmartcard );
            MOCKPP_CHAINER_FOR_EXT ( MockSmartcard, write, ext ) write_mock ( mockSmartcard );
            MOCKPP_CHAINER_FOR_EXT ( MockSmartcard, read, ext ) read_mock ( mockSmartcard );

            open_mock
            .expects ( MOCKPP_NS::once () )
            .will ( MOCKPP_NS::returnValue <bool> ( true ) );

            write_mock
            .expects ( MOCKPP_NS::exactly ( 2 ) )
            .will ( MOCKPP_NS::returnValue <bool> ( true ) );

            read_mock
            .expects ( MOCKPP_NS::exactly ( 2 ) )
            .will ( MOCKPP_NS::onConsecutiveCalls (
                        MOCKPP_NS::returnValue <bool> ( false ),
                        MOCKPP_NS::returnValue <bool> ( true ) ) );

            test_mock
            .expects ( MOCKPP_NS::exactly ( 2 ) )
            .will ( MOCKPP_NS::returnValue <bool> ( true ) );

            MockDummyCardController mockSmartcardController ( mockSmartcard );
            CardController::getInstance ( &mockSmartcardController );

            CPPUNIT_ASSERT ( true == CardController::getInstance()->open ( "" ) );
            CPPUNIT_ASSERT ( false == CardController::getInstance()->test() );
            CPPUNIT_ASSERT ( false == CardController::getInstance()->test() );

            mockSmartcard->verify();
        }

        void testCallTestSuccess()
        {
            MockDummySmartcard  * mockSmartcard = new MockDummySmartcard ();

            MOCKPP_CHAINER_FOR_EXT ( MockDummySmartcard, open, ext ) open_mock ( mockSmartcard );
            MOCKPP_CHAINER_FOR ( MockDummySmartcard, test ) test_mock ( mockSmartcard );

            open_mock
            .expects ( MOCKPP_NS::once () )
            .will ( MOCKPP_NS::returnValue <bool> ( true ) );

            test_mock
            .expects ( MOCKPP_NS::once () )
            .will ( MOCKPP_NS::returnValue <bool> ( true ) );

            MockDummyCardController mockSmartcardController ( mockSmartcard );
            CardController::getInstance ( &mockSmartcardController );

            CPPUNIT_ASSERT ( true == CardController::getInstance()->open ( "" ) );
            CPPUNIT_ASSERT ( true == CardController::getInstance()->test() );

            mockSmartcard->verify();
        }

        CPPUNIT_TEST_SUITE ( CardControllerTest );

        CPPUNIT_TEST ( testCheckOpenDevice );

        CPPUNIT_TEST ( testCheckCloseDeviceWithoutSmartcard );
        CPPUNIT_TEST ( testCheckCloseDevice );

        CPPUNIT_TEST ( testCheckIsCardPresentWithoutSmartcard );
        CPPUNIT_TEST ( testIsCardPresent );

        CPPUNIT_TEST ( testWriteWithoutSmartcard );
        CPPUNIT_TEST ( testWriteSmartcard );

        CPPUNIT_TEST ( testReadWithoutSmartcard );
        CPPUNIT_TEST ( testReadSmartcard );

        CPPUNIT_TEST ( testWriteAndReadSmartcard );

        CPPUNIT_TEST ( testCallTestFail_SelectFail );
        CPPUNIT_TEST ( testCallTestFail_SelectFailAgain );
        CPPUNIT_TEST ( testCallTestFail_SelectSuccess_WriteFail );
        CPPUNIT_TEST ( testCallTestFail_SelectSuccess_WriteFailAgain );
        CPPUNIT_TEST ( testCallTestFail_SelectSuccess_WriteSuccess_Read );
        CPPUNIT_TEST ( testCallTestSuccess );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_CARD_CONTROLLER_TEST */
