/**
 * @file ActivationConsolidationFunctionality.cpp
 * @brief Body class activation consolidation functionality
 * @date 02/09/2011
 * @author Juan Delgado
 */

#include <Configuration/s-a-e-s-g-u-i-conf-manager.h>
#include <Configuration/s-m-t-t-g-u-i-conf-manager.h>
#include <Configuration/MachineModelEnum.hxx>
#include <Functionalities/ActivationConsolidationFunctionality.h>
#include <Log/SMTTLogManager.h>
#include <Gui/Widgets/AbstractFactoryWidget.hxx>
#include <Gui/Windows/VoteProcessWindow.hxx>
#include <Operation/Devices/Polling/CardPolling.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

#define EVENT_BOX_WIDTH 330
#define ACTIVATION_WIDGET_WIDTH 330

using namespace Gtk;
using namespace Smartmatic::Configuration;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::GUI::Functionality;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::Functionality;
using namespace Smartmatic::Functionality;
using namespace Smartmatic::SAES::GUI::Windows;
using namespace Smartmatic::SAES::Operation::Devices;

ISMTTLog * ActivationConsolidationFunctionality::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Functionality.ActivationConsolidationFunctionality", GETTEXT_PACKAGE );

ActivationConsolidationFunctionality * ActivationConsolidationFunctionality::current_instance = NULL;

ActivationConsolidationFunctionality * ActivationConsolidationFunctionality::getInstance()
{
    if ( !current_instance )
    {
        current_instance = new ActivationConsolidationFunctionality();
    }

    return current_instance;
}

ActivationConsolidationFunctionality::ActivationConsolidationFunctionality()
{
    outsideBox = NULL;
    activationWidget = NULL;
    consolidationWidget = NULL;
    cardActivatorBox = NULL;
}

ActivationConsolidationFunctionality::~ActivationConsolidationFunctionality()
{
    deleteElements();
}

Smartmatic::SAES::Functionality::Configuration::MenuOptionType ActivationConsolidationFunctionality::GetMenuOptionType()
{
    return MenuOptionType::CONFIGURATION_XMLMACTIVATIONCONSOLIDATIONFUNCTIONALITY;
}

Gtk::Widget * ActivationConsolidationFunctionality::GetFrame()
{
    int activationWidgetWidth = ACTIVATION_WIDGET_WIDTH;
    int eventBoxWidth = EVENT_BOX_WIDTH;

    if ( SMTTGUIConfigurationManager::getInstance()->getSmartmaticGUIMachineModelConfiguration().getId() == EnumToString( PM_VIU805 ) )
    {
        double guiReconfigurationFactor = SMTTGUIConfigurationManager::getInstance()->getConfiguration()->getSmartmaticGUIMachineModelConfiguration().getGuiReconfigurationFactor();

        activationWidgetWidth *= guiReconfigurationFactor;
        eventBoxWidth *= guiReconfigurationFactor;
    }

    CountConfiguration totalActivationCountType = SAESGUIConfigurationManager::getInstance()->GetActivationConsolidationFunctionalityConfiguration().getCardActivationTotalCountConfiguration();
    bool showStatistic = SAESGUIConfigurationManager::getInstance()->GetActivationConsolidationFunctionalityConfiguration().getShowActivateCards();
    activationWidget = new ActivationStatsWidget ( totalActivationCountType, showStatistic );
    activationWidget->getSignalStatusActivated().connect ( sigc::mem_fun ( *this, &ActivationConsolidationFunctionality::onSignalStatusActivated ) );
    activationWidget->getSignalAudioVotingButtonChecked().connect( sigc::mem_fun( *this, &ActivationConsolidationFunctionality::writeVotingCard ) );
    activationWidget->getSignalDone().connect( sigc::mem_fun( *this, &ActivationConsolidationFunctionality::onActivationSignalDone ) );

    CountConfiguration totalConsolidationCountType = SAESGUIConfigurationManager::getInstance()->GetActivationConsolidationFunctionalityConfiguration().getConsolidationTotalCountConfiguration();
    CountConfiguration officialConsolidationCountType = SAESGUIConfigurationManager::getInstance()->GetActivationConsolidationFunctionalityConfiguration().getConsolidationOfficialCountConfiguration();
    consolidationWidget = new ConsolidationStatsWidget ( totalConsolidationCountType, officialConsolidationCountType );
    consolidationWidget->barcode_SendCommands().connect ( sigc::mem_fun ( *this, &ActivationConsolidationFunctionality::statusSendCommands ) );

    cardActivatorBox = new CardActivatorEventBox ( eventBoxWidth );
    cardActivatorBox->signal_vectorCustomAttribute_changed().connect ( sigc::mem_fun ( *this, &ActivationConsolidationFunctionality::SetCustomAttributeVector ) );
    cardActivatorBox->getCustomAttributeSelectedSignal().connect( sigc::mem_fun( *this, &ActivationConsolidationFunctionality::writeVotingCard ) );
    /**
     * activation stats
     */
    activationWidget->set_size_request ( activationWidgetWidth, -1 );
    outsideBox = new Gtk::HBox();
    outsideBox->set_spacing ( 15 );
    outsideBox->show();
    outsideBox->pack_start ( *consolidationWidget );
    outsideBox->pack_start ( *cardActivatorBox, false, false );
    outsideBox->pack_end ( *activationWidget, false, false );
    consolidationWidget->set_no_show_all();
    consolidationWidget->show();
    cardActivatorBox->set_no_show_all();
    cardActivatorBox->show();
    activationWidget->set_no_show_all();
    activationWidget->show();

    consolidationWidget->setAccessibleName ( "StatisticContainer" );
    cardActivatorBox->setAccessibleName ( "VoterTypeSelectionContainer" );
    activationWidget->setAccessibleName ( "CardActivationContainer" );

    return outsideBox;
}

void ActivationConsolidationFunctionality::statusSendCommands ( bool status )
{
    hideBarcodeConnected();
}

void ActivationConsolidationFunctionality::BarcodeConnected ( bool isConnected )
{
    static bool activateCardsWithoutBarcode =
        Smartmatic::SAES::GUI::Configuration::SAESGUIConfigurationManager::getInstance (
        )->GetActivationConsolidationFunctionalityConfiguration().getActivateCardsWithoutBarcode();

    if ( !activateCardsWithoutBarcode )
    {
        activationWidget->AllowCardActivation ( isConnected );
    }
}

void ActivationConsolidationFunctionality::SetCustomAttributeVector ( std::vector< Smartmatic::SAES::Voting::VotingDevice::Custom_attribute > vecCA )
{
    activationWidget->SetAttributes ( vecCA );
}

void ActivationConsolidationFunctionality::writeVotingCard()
{
    if (CardPolling::getInstance()->isCardPresent() && CardPolling::getInstance()->certaintyOfCardPresence())
    {
        cardActivatorBox->disconnectEventBox();
        activationWidget->disconnectAudioVotingCheckBox();
        activationWidget->cardInsert();
    }
}

void ActivationConsolidationFunctionality::onBeforeExit()
{
    activationWidget->StopOperation();
    VoteProcessWindow::getInstance()->showReduceMode();
}

void ActivationConsolidationFunctionality::Dispose()
{
    activationWidget->Dispose();
    consolidationWidget->Dispose();
}

void ActivationConsolidationFunctionality::Initialize()
{
    activationWidget->Initialize();
    activationWidget->SetAttributes ( cardActivatorBox->GetCustomAttributes() );
    consolidationWidget->Initialize();
    exitWidget.setInfo ( N_ ( "Smartmatic.SAES.GUI.Functionality.ActivationConsolidationFunctionality.FinalMessage" ) );
    VoteProcessWindow::getInstance()->showNormalMode();
    enableExitButton();
}

std::string ActivationConsolidationFunctionality::GetWindowTitle()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.ActivationConsolidationFunctionality.ActivationConsolidation" );
}

Smartmatic::Log::ISMTTLog * ActivationConsolidationFunctionality::getLogger()
{
    return logger;
}

Glib::ustring ActivationConsolidationFunctionality::getStartLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.ActivationConsolidationFunctionality.LogStartFunctionality" );
}

Glib::ustring ActivationConsolidationFunctionality::getFinishLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.ActivationConsolidationFunctionality.LogCloseFunctionality" );
}

void ActivationConsolidationFunctionality::block()
{
    logger->Debug ( "Block functionality" );
    activationWidget->SetOneMemoryMode ( true );
    consolidationWidget->SetOneMemoryMode ( true );
    activationWidget->StopOperation();
    consolidationWidget->pauseOperation();
}

void ActivationConsolidationFunctionality::partial()
{
    logger->Debug ( "Partial functionality" );
    activationWidget->ContinueOperation();
    consolidationWidget->resumeOperation();
    consolidationWidget->UpdateMemoryPresence();
    consolidationWidget->UpdateBarcodePresence();
}

void ActivationConsolidationFunctionality::resume()
{
    logger->Debug ( "Resume functionality" );
    activationWidget->SetOneMemoryMode ( false );
    consolidationWidget->SetOneMemoryMode ( false );
    activationWidget->ContinueOperation();
    consolidationWidget->resumeOperation();
    consolidationWidget->UpdateMemoryPresence();
    consolidationWidget->UpdateBarcodePresence();
}

void ActivationConsolidationFunctionality::deleteElements()
{
    if ( outsideBox )
    {
        delete outsideBox;
        outsideBox = NULL;
    }

    if ( activationWidget )
    {
        delete activationWidget;
        activationWidget = NULL;
    }

    if ( consolidationWidget )
    {
        delete consolidationWidget;
        consolidationWidget = NULL;
    }

    if ( cardActivatorBox )
    {
        delete cardActivatorBox;
        cardActivatorBox = NULL;
    }
}

void ActivationConsolidationFunctionality::hideBarcodeConnected()
{
    logger->Debug ( "hideBarcodeConnected" );
    BarcodeConnected ( false );
    activationWidget->ContinueOperation();
    activationWidget->show();
    consolidationWidget->show();
    cardActivatorBox->show();
    enableExitButton();
}

void ActivationConsolidationFunctionality::refresh()
{
    consolidationWidget->UpdateMemoryPresence();
    consolidationWidget->UpdateBarcodePresence();
    GraphicDisplayFunctionality::refresh();
}

void ActivationConsolidationFunctionality::onSignalStatusActivated ( bool status )
{
    if ( status && SAESGUIConfigurationManager::getInstance()->GetActivationConsolidationFunctionalityConfiguration().getResetDefaultCustomAttribute() )
    {
        cardActivatorBox->reset();
        activationWidget->resetAudioVotingSelection();
    }
}

void ActivationConsolidationFunctionality::onActivationSignalDone()
{
    cardActivatorBox->reconnectEventBox();
    activationWidget->reconnectAudioVotingCheckBox();
}

void ActivationConsolidationFunctionality::onHelpShown()
{
    CardPolling::getInstance()->pause();
    activationWidget->setIsHelpShown( true );
}

void ActivationConsolidationFunctionality::onHelpClosed()
{
    CardPolling::getInstance()->resume();
    activationWidget->setIsHelpShown( false );
}
