/**
 * @file   VotingMachineInstallationFunctionality.hxx
 * @brief  Declares the voting machine installation functionality class.
 * @author Youssef.tourkmani
 * @author manuel.arguelles
 * @date   01/11/2012
 */


#ifndef _VOTING_MACHINE_INSTALLATION_FUNCTIONALITY_H_
#define _VOTING_MACHINE_INSTALLATION_FUNCTIONALITY_H_

#include <gtkmm.h>
#include <Gui/Widgets/VotingMachineInstallationWizardWidget.hxx>
#include <Functionalities/graphic-display-functionality.h>
#include <Log/ISMTTLog.h>

namespace Smartmatic
{
    namespace SAES
    {
        namespace GUI
        {
            namespace Functionality
            {
                /**
                 * @class VotingMachineInstallationFunctionality
                 * @brief Voting machine installation functionality.
                 * @author Youssef.tourkmani
                 * @date 01/11/2012
                 */
                class VotingMachineInstallationFunctionality : public GraphicDisplayFunctionality
                {
                    public:
                        /**
                         * @brief  Gets the current instance of the class.
                         * @author Luiggi.mendez
                         * @date   28/04/2011
                         * @return The instance of the class, or null on error.
                         */
                        static VotingMachineInstallationFunctionality* getInstance();

                    protected:
                        /**
                         * @brief  Gets the frame.
                         * @author Luiggi.mendez
                         * @date   28/04/2011
                         * @return null if it fails, else the frame.
                         */
                        virtual Gtk::Widget* GetFrame();

                        /**
                         * @brief  Dispose of this object, cleaning up any resources it uses.
                         * @author Luiggi.mendez
                         * @date   28/04/2011
                         */
                        virtual void Dispose();

                        /**
                         * @brief  Initializes this object.
                         * @author Luiggi.mendez
                         * @date   28/04/2011
                         */
                        virtual void Initialize();

                        /**
                         * @brief  Gets the window title.
                         * @author Luiggi.mendez
                         * @date   28/04/2011
                         * @return The window title.
                         */
                        std::string GetWindowTitle();

                        /**
                         * @brief  Gets the menu option type.
                         * @author Luiggi.mendez
                         * @date   28/04/2011
                         * @return The menu option type.
                         */
                        Smartmatic::SAES::Functionality::Configuration::MenuOptionType GetMenuOptionType();

                        /**
                         * @brief  Finaliser.
                         * @author Luiggi.mendez
                         * @date   28/04/2011
                         */
                        virtual ~VotingMachineInstallationFunctionality();

                        /**
                         * @brief  Gets the logger.
                         * @author Luiggi.mendez
                         * @date   28/04/2011
                         * @return null if it fails, else the logger.
                         */
                        Smartmatic::Log::ISMTTLog* getLogger();

                        /**
                         * @brief  Gets the start log message.
                         * @author Luiggi.mendez
                         * @date   28/04/2011
                         * @return The start log message.
                         */
                        Glib::ustring getStartLogMessage();

                        /**
                         * @brief  Gets the finish log message.
                         * @author Luiggi.mendez
                         * @date   28/04/2011
                         * @return The finish log message.
                         */
                        Glib::ustring getFinishLogMessage();

                        /**
                         * @brief  virtual method before exit.
                         * @author Yadickson Soto
                         * @date   03/10/2012
                         */
                        virtual void onBeforeExit();

                        /**
                         * @brief  refresh widget method
                         * @date   01/11/2012
                         * @author Yadickson Soto
                         */
                        virtual void refresh();

                    private:
                        /**
                         * @brief  Default constructor.
                         * @author Luiggi.mendez
                         * @date   28/04/2011
                         */
                        VotingMachineInstallationFunctionality();

                        /**
                         * @brief  Executes the finish action.
                         * @author Youssef.tourkmani
                         * @date   01/11/2012
                         */
                        void onFinish();

                        /**
                         * @brief  Hides the on finish.
                         * @author Luiggi.mendez
                         * @date   28/04/2011
                         */
                        void HideOnFinish();

                        /**
                         * @brief  Deletes all elements.
                         * @author Yadickson Soto
                         * @date   03/04/2012
                         */
                        virtual void deleteElements();

                    private:
                        sigc::connection connHide;
                        sigc::connection connFinish;
                        Smartmatic::SAES::GUI::Widgets::VotingMachineInstallationWizardWidget* wizardWidget;
                        static VotingMachineInstallationFunctionality* current_instance;
                        static Smartmatic::Log::ISMTTLog* logger;
                };
            }
        }
    }
}

#endif // _VOTING_MACHINE_INSTALLATION_FUNCTIONALITY_H_
