/**
 * @file	all-diagnosis-functionality.h
 * @brief	Declares all diagnosis functionality class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */

#ifndef _ALL_DIAGNOSIS_FUNCTIONALITY_H_
#define _ALL_DIAGNOSIS_FUNCTIONALITY_H_

#include <gtkmm.h>
#include <string>
#include <Functionalities/graphic-display-functionality.h>
#include <Gui/Widgets/Diagnose/AlarmBoxDiagnoseWidget.hxx>
#include <Gui/Widgets/Diagnose/SmartcardDiagnosisWidget.hxx>
#include <Gui/Widgets/Diagnose/PrinterDiagnosisWidget.hxx>
#include <Gui/Widgets/Diagnose/BarcodeReaderDiagnoseWidget.hxx>
#include <Gui/Widgets/Diagnose/CalibrationDiagnosisWidget.hxx>
#include <Gui/Widgets/Diagnose/FlashDiagnosisWidget.hxx>
#include <Gui/Widgets/Diagnose/AbleDiagnosticWidget.hxx>
#include <Gui/Widgets/Diagnose/DiagnosticResultWidget.hxx>
#include <Log/ISMTTLog.h>
#include "Configuration/s-a-e-s-g-u-i-conf-manager.h"
#include <Voting/OperationStatus.hxx>
#include <Gui/Widgets/Diagnose/DeviceDiagnosticPrintWidget.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace GUI
        {
            namespace Functionality
            {
                /**
                 * @class	AllDiagnosisFunctionality
                 * @brief	All diagnosis functionality.
                 * @author	Youssef.tourkmani
                 * @date	01/11/2012
                 */
                class AllDiagnosisFunctionality: public GraphicDisplayFunctionality
                {

                    public:

                        /**
                         * @fn	bool AllDiagnosisFunctionality::DontAllowHelp();
                         * @brief	Gets the dont allow help.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	true if it succeeds, false if it fails.
                         */

                        bool DontAllowHelp();

                        /**
                         * @fn	std::vector<Smartmatic::GUI::Widgets::KeysActionsCode> AllDiagnosisFunctionality::getHelpSeqKey();
                         * @brief	Gets the help sequence key.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The help sequence key.
                         */

                        std::vector<Smartmatic::GUI::Widgets::KeysActionsCode> getHelpSeqKey();

                        /**
                         * @fn	bool AllDiagnosisFunctionality::DontAllowExit();
                         * @brief	Gets the dont allow exit.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	true if it succeeds, false if it fails.
                         */

                        bool DontAllowExit();

                        /**
                         * @fn	std::vector<Smartmatic::GUI::Widgets::KeysActionsCode> AllDiagnosisFunctionality::getSeqKey();
                         * @brief	Gets the sequence key.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The sequence key.
                         */

                        std::vector<Smartmatic::GUI::Widgets::KeysActionsCode> getSeqKey();

                        /**
                         * @fn	static AllDiagnosisFunctionality * AllDiagnosisFunctionality::getInstance();
                         * @brief	Gets the current.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	null if it fails, else.
                         */

                        static AllDiagnosisFunctionality * getInstance();

                    protected:

                        /**
                         * @fn	Smartmatic::Log::ISMTTLog * AllDiagnosisFunctionality::getLogger();
                         * @brief	Gets the logger.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	null if it fails, else the logger.
                         */

                        Smartmatic::Log::ISMTTLog * getLogger();

                        /**
                         * @fn	Glib::ustring AllDiagnosisFunctionality::getStartLogMessage();
                         * @brief	Gets the start log message.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The start log message.
                         */

                        Glib::ustring getStartLogMessage();

                        /**
                         * @fn	Glib::ustring AllDiagnosisFunctionality::getFinishLogMessage();
                         * @brief	Gets the finish log message.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The finish log message.
                         */

                        Glib::ustring getFinishLogMessage();

                    private:

                        Gtk::Frame * framePrincipal; ///< The frame principal
                        bool startPrint; ///< flag start printing

                        sigc::connection connEnd; ///< The connection end diagnostic
                        sigc::connection connShowResults; ///< The connection show results

                        sigc::signal<void> signalDispatcherShowResults; ///< signal dispatcher show results
                        sigc::signal<void> signalDispatcherEnd; ///< signal dispatcher end all diagnostic

                        /**
                         * @fn	void AllDiagnosisFunctionality::on_AlarmDiagFinished();
                         * @brief	Handles alarm diagram finished signals.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void on_AlarmDiagFinished();

                        /**
                         * @fn	void AllDiagnosisFunctionality::on_BCDiagFinished();
                         * @brief	Handles bc diagram finished signals.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void on_BCDiagFinished();

                        /**
                         * @fn	void AllDiagnosisFunctionality::on_SCDiagFinished();
                         * @brief	Handles screen diagram finished signals.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void on_SCDiagFinished();

                        /**
                         * @fn	void AllDiagnosisFunctionality::on_PrinterDiagFinished();
                         * @brief	Handles printer diagram finished signals.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void on_PrinterDiagFinished();

                        /**
                         * @fn	void AllDiagnosisFunctionality::on_PrinterDiagFinishedOnOk();
                         * @brief	Handles printer diagram finished on ok signals.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void on_PrinterDiagFinishedOnOk();

                        /**
                         * @fn	void AllDiagnosisFunctionality::on_PrinterDiagFinishedOnCancel();
                         * @brief	Handles printer diagram finished on cancel signals.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void on_PrinterDiagFinishedOnCancel();

                        /**
                         * @fn	void AllDiagnosisFunctionality::on_CalibrationDiagFinished();
                         * @brief	Handles calibration diagram finished signals.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void on_CalibrationDiagFinished();

                        /**
                         * @fn	void AllDiagnosisFunctionality::on_FlashDiagFinished();
                         * @brief	Handles flash diagram finished signals.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void on_FlashDiagFinished();
                        /**
                         * @fn	void AllDiagnosisFunctionality::on_AbleDiagFinished();
                         * @brief	Handles able diagram finished signals.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void on_AbleDiagFinished();

                        /**
                         * @fn	void AllDiagnosisFunctionality::printResults();
                         * @brief	Print results.
                         * @author	Yadickson Soto
                         * @date	20/09/2012
                         */
                        void printResults();
                        /**
                         * @fn	void AllDiagnosisFunctionality::endPrintResults();
                         * @brief	End print results.
                         * @author	Yadickson Soto
                         * @date	20/09/2012
                         */
                        void endPrintResults();

                        /**
                         * @fn	void AllDiagnosisFunctionality::showResults();
                         * @brief	Shows the results.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */
                        void showResults();

                        static AllDiagnosisFunctionality * current_instance; ///< The current instance

                        /**
                         * @fn	virtual void AllDiagnosisFunctionality::deleteElements();
                         * @brief	Deletes all elements.
                         * @author	Yadickson Soto
                         * @date	03/04/2012
                         */
                        virtual void deleteElements();

                        /**
                         * @fn	void AllDiagnosisFunctionality::deleteResultElement();
                         * @brief	Deletes the result element.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void deleteResultElement();

                        /**
                         * @fn	AllDiagnosisFunctionality::AllDiagnosisFunctionality();
                         * @brief	Default constructor.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        AllDiagnosisFunctionality();
                        /**
                         * @fn	virtual AllDiagnosisFunctionality::~AllDiagnosisFunctionality();
                         * @brief	Finaliser.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        virtual ~AllDiagnosisFunctionality();

                        /**
                         * @fn	Smartmatic::SAES::Functionality::Configuration::MenuOptionType AllDiagnosisFunctionality::GetMenuOptionType();
                         * @brief	Gets the menu option type.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The menu option type.
                         */

                        Smartmatic::SAES::Functionality::Configuration::MenuOptionType GetMenuOptionType();

                        /**
                         * @fn	void AllDiagnosisFunctionality::Dispose();
                         * @brief	Dispose of this object, cleaning up any resources it uses.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void Dispose();

                        /**
                         * @fn	void AllDiagnosisFunctionality::Initialize();
                         * @brief	Initializes this object.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void Initialize();

                        /**
                         * @fn	Gtk::Widget * AllDiagnosisFunctionality::GetFrame();
                         * @brief	Gets the frame.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	null if it fails, else the frame.
                         */

                        Gtk::Widget * GetFrame();

                        unsigned int index; ///< Zero-based index of the
                        std::vector < Smartmatic::GUI::Widgets::WizardWidgetBase *> listOfDiagnosis;	///< The list of diagnosis

                        /**
                         * @fn	bool AllDiagnosisFunctionality::RefreshAlarm ( int time_num );
                         * @brief	Refresh alarm.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @param	time_num	The time number.
                         * @return	true if it succeeds, false if it fails.
                         */

                        bool RefreshAlarm ( int time_num );

                        /**
                         * @fn	bool AllDiagnosisFunctionality::RefreshBC ( int time_num );
                         * @brief	Refresh bc.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @param	time_num	The time number.
                         * @return	true if it succeeds, false if it fails.
                         */

                        bool RefreshBC ( int time_num );

                        /**
                         * @fn	bool AllDiagnosisFunctionality::RefreshPrinter ( int time_num );
                         * @brief	Refresh printer.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @param	time_num	The time number.
                         * @return	true if it succeeds, false if it fails.
                         */

                        bool RefreshPrinter ( int time_num );

                        /**
                         * @fn	void AllDiagnosisFunctionality::SetPrinterWidget();
                         * @brief	Sets the printer widget.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void SetPrinterWidget();

                        /**
                         * @fn	bool AllDiagnosisFunctionality::RefreshSC ( int time_num );
                         * @brief	Refresh screen.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @param	time_num	The time number.
                         * @return	true if it succeeds, false if it fails.
                         */

                        bool RefreshSC ( int time_num );

                        /**
                         * @fn	bool AllDiagnosisFunctionality::RefreshCalib ( int time_num );
                         * @brief	Refresh calib.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @param	time_num	The time number.
                         * @return	true if it succeeds, false if it fails.
                         */

                        bool RefreshCalib ( int time_num );

                        /**
                         * @fn	bool AllDiagnosisFunctionality::RefreshFlash ( int time_num );
                         * @brief	Refresh flash.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @param	time_num	The time number.
                         * @return	true if it succeeds, false if it fails.
                         */

                        bool RefreshFlash ( int time_num );

                        /**
                         * @fn	bool AllDiagnosisFunctionality::RefreshAble ( int time_num );
                         * @brief	Refresh able.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	time_num	The time number.
                         * @return	true if it succeeds, false if it fails.
                         */
                        bool RefreshAble ( int time_num );

                        /**
                         * @fn	std::string AllDiagnosisFunctionality::GetWindowTitle();
                         * @brief	Gets the window title.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The window title.
                         */

                        std::string GetWindowTitle();
                        /**
                         * @fn	virtual void AllDiagnosisFunctionality::onBeforeExit();
                         * @brief	virtual method before exit.
                         * @author	Yadickson Soto
                         * @date	23/10/2012
                         */
                        virtual void onBeforeExit();
                        /**
                         * @fn	void AllDiagnosisFunctionality::endDiagnostic();
                         * @brief	Ends diagnostic.
                         * @author	Yadickson Soto
                         * @date	29/11/2012
                         */
                        void endDiagnostic();
                        /**
                         * @fn	void AllDiagnosisFunctionality::endAllDiagnostic();
                         * @brief	Ends all diagnostic.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void endAllDiagnostic();

                        sigc::connection connAlarmDiag; ///< The connection alarm diagram
                        sigc::connection connBCDiag;	///< The connection bc diagram
                        sigc::connection connSCDiag;	///< The connection screen diagram
                        sigc::connection connPrinterDiag;   ///< The connection printer diagram
                        sigc::connection connPrinterDiagOnOk;   ///< The connection printer diagram on ok
                        sigc::connection connPrinterDiagOnCancel;   ///< The connection printer diagram on cancel
                        sigc::connection connCalibDiag; ///< The connection calib diagram
                        sigc::connection connFlashDiag; ///< The connection flash diagram
                        sigc::connection connRefreshAlarm;  ///< The connection refresh alarm
                        sigc::connection connRefreshBC; ///< The connection refresh bc
                        sigc::connection connRefreshSC; ///< The connection refresh screen
                        sigc::connection connRefreshPrinter;	///< The connection refresh printer
                        sigc::connection connRefreshPrinterOnOk;	///< The connection refresh printer on ok
                        sigc::connection connRefreshPrinterOnCancel;	///< The connection refresh printer on cancel
                        sigc::connection connRefreshCalib;  ///< The connection refresh calib
                        sigc::connection connRefreshFlash;  ///< The connection refresh flash
                        sigc::connection connAbleDiag; ///< The connection able
                        sigc::connection connRefreshAble;  ///< The connection refresh able

                        Smartmatic::SAES::GUI::Widgets::CalibrationDiagnosisWidget * calibrationWidget; ///< The calibration widget
                        Smartmatic::SAES::GUI::Widgets::BarcodeReaderDiagnoseWidget * bcreaderWidget; ///< The bcreader widget
                        Smartmatic::SAES::GUI::Widgets::PrinterDiagnosisWidget * printerWidget; ///< The printer widget
                        Smartmatic::SAES::GUI::Widgets::SmartcardDiagnosisWidget * screaderWidget; ///< The screader widget
                        Smartmatic::SAES::GUI::Widgets::AlarmBoxDiagnoseWidget * alarmWidget;  ///< The alarm widget
                        Smartmatic::SAES::GUI::Widgets::FlashDiagnosisWidget * flashWidget; ///< The flash widget
                        Smartmatic::SAES::GUI::Widgets::AbleDiagnosticWidget * ableWidget; ///< the able diagnostic widhet
                        Smartmatic::SAES::GUI::Widgets::DeviceDiagnosticPrintWidget * printResultWidget; ///< Diagnostic print document
                        Smartmatic::SAES::GUI::Widgets::DiagnosticResultWidget * resultWidget; ///< result widget
                        static Smartmatic::Log::ISMTTLog * logger; /**<logger*/ ///< The logger
                };
            }
        }
    }
}


#endif // _ALL_DIAGNOSIS_FUNCTIONALITY_H_
