/**
 * @file	print-preview-functionality.cc
 * @brief	print preview functionality class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#include <sstream>
#include "Configuration/s-a-e-s-conf-manager.h"
#include "Functionalities/print-preview-functionality.h"
#include "Resources/election-string-resources-manager.h"
#include <Configuration/s-a-e-s-g-u-i-conf-manager.h>
#include "System/Compression/compression-class.h"
#include "Gui/Widgets/AbstractFactoryWidget.hxx"
#include <Log/SMTTLogManager.h>
#include <Operation/Devices/BCRController.hxx>
#include <Gui/Windows/VoteProcessWindow.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::Functionality;
using namespace Smartmatic::System::Compression;
using namespace Smartmatic::SAES::Printing;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::SAES::GUI::Windows;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Functionality;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::Environment;
using namespace Smartmatic::GUI::Windows;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::Functionality;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Gtk;
using namespace Smartmatic::SAES::Operation::Devices;

ISMTTLog * PrintPreviewFunctionality::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Functionality.PrintPreviewFunctionality", GETTEXT_PACKAGE );

PrintPreviewFunctionality * PrintPreviewFunctionality::current_instance = NULL;

PrintPreviewFunctionality * PrintPreviewFunctionality::getInstance()
{
    if ( !current_instance )
    {
        current_instance = new PrintPreviewFunctionality();
    }

    return current_instance;
}

PrintPreviewFunctionality::PrintPreviewFunctionality()
{
    docMagnify = NULL;
    printWidgetMagnify = NULL;
    labelMessage = NULL;
    frameMagnify = NULL;
    vboxMagnify = NULL;
    scrolledMagnify = NULL;
    viewMagnify = NULL;
    validVote = false;
}

PrintPreviewFunctionality::~PrintPreviewFunctionality()
{
    deleteElements();
}

void PrintPreviewFunctionality::Initialize()
{
    VoteProcessWindow::getInstance()->stop();
    validVote = false;
    labelMessage->set_label ( _ ( "Smartmatic.SAES.GUI.Functionality.PrintPreviewFunctionality.WaitingForBarcode" ) );
    Pango::FontDescription fontTitle = labelMessage->get_style()->get_font();
    fontTitle.set_size ( 13 * Pango::SCALE );
    fontTitle.set_weight ( Pango::WEIGHT_ULTRABOLD );
    labelMessage->modify_font ( fontTitle );
    viewMagnify->remove();
    viewMagnify->show_all_children ();
    vboxMagnify->show_all_children ();
    frameMagnify->show_all_children ();
    signalVote.disconnect();
    signalStatusVote.disconnect();
    //signalVote = BCRController::getInstance()->getSignalVote().connect ( sigc::mem_fun ( *this, &PrintPreviewFunctionality::onSignalVote ) );
    //signalStatusVote = BCRController::getInstance()->getSignalStatus().connect ( sigc::mem_fun ( *this, &PrintPreviewFunctionality::onSignalState ) );
    //BCRController::getInstance()->enableSaveVote ( false );
    //BCRController::getInstance()->resume ( true );
    //BCRController::getInstance()->validateID ( false );
    enableExitButton();
}

void PrintPreviewFunctionality::BarcodeError()
{
    validVote = false;
    viewMagnify->remove();
    labelMessage->set_label ( _ ( "Smartmatic.SAES.GUI.Functionality.PrintPreviewFunctionality.BarcodeErrorTryAgain" ) );
    logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Functionality.PrintPreviewFunctionality.BarcodeError" ) );
}

bool PrintPreviewFunctionality::BarcodeRefreshVoteMode ( int time_num )
{
    switch ( voteMode )
    {
        case Vote::VoteMode::VOTE_D:
            labelMessage->set_label ( _ ( "Smartmatic.SAES.GUI.Functionality.PrintPreviewFunctionality.VoteMarkedAsDemo" ) );
            logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Functionality.PrintPreviewFunctionality.VoteMarkedAsDemo" ) );
            break;

        case Vote::VoteMode::VOTE_O:
            labelMessage->set_label ( _ ( "Smartmatic.SAES.GUI.Functionality.PrintPreviewFunctionality.VoteMarkedAsOfficial" ) );
            logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Functionality.PrintPreviewFunctionality.VoteMarkedAsOfficial" ) );
            break;

        case Vote::VoteMode::VOTE_S:
            labelMessage->set_label ( _ ( "Smartmatic.SAES.GUI.Functionality.PrintPreviewFunctionality.VoteMarkedAsSimulated" ) );
            logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Functionality.PrintPreviewFunctionality.VoteMarkedAsSimulated" ) );
            break;

        default:
            break;
    }

    return false;
}

void PrintPreviewFunctionality::CreateNormalFrame()
{
    Glib::RefPtr<Gtk::Builder> builder;

    try
    {
        builder = Gtk::Builder::create_from_file ( ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/vote-preview-functionality.glade" ) );
    }
    catch ( const Glib::FileError & ex )
    {
        std::stringstream ss;
        ss  << "FATAL - " << __func__ << " " << ex.what();
        logger->Debug ( ss.str() );
    }

    builder->get_widget ( "labelMessage", labelMessage );
    builder->get_widget ( "frameMagnify", frameMagnify );
    builder->get_widget ( "vboxMagnify", vboxMagnify );
    builder->get_widget ( "viewportMagnify", viewMagnify );
    builder->get_widget_derived ( "scrolledwindowMagnify", scrolledMagnify );

    labelMessage->set_size_request ( 600, 50 );
    labelMessage->set_alignment ( 0.5, 0.5 );
    normalFrameWidget = frameMagnify;
}

Gtk::Widget * PrintPreviewFunctionality::GetFrame()
{
    CreateNormalFrame();
    return normalFrameWidget;
}

std::string PrintPreviewFunctionality::GetWindowTitle()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.PrintPreviewFunctionality.WindowTitle" );
}

Smartmatic::SAES::Functionality::Configuration::MenuOptionType PrintPreviewFunctionality::GetMenuOptionType()
{
    return MenuOptionType::CONFIGURATION_XMLMPRINTPREVIEWFUNCTIONALITY;
}

void PrintPreviewFunctionality::onBeforeExit()
{
    //BCRController::getInstance()->enableSaveVote();
    //BCRController::getInstance()->resume();
    VoteProcessWindow::getInstance()->start();
}

void PrintPreviewFunctionality::Dispose()
{
    validVote = false;
    signalVote.disconnect();
    signalStatusVote.disconnect();
    viewMagnify->remove();
    //BCRController::getInstance()->validateID ( true );
}

ISMTTLog * PrintPreviewFunctionality::getLogger()
{
    return logger;
}

Glib::ustring PrintPreviewFunctionality::getStartLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.PrintPreviewFunctionality.LogStartFunctionality" );
}

Glib::ustring PrintPreviewFunctionality::getFinishLogMessage()
{
    return N_ ( "Smartmatic.SAES.GUI.Functionality.PrintPreviewFunctionality.LogCloseFunctionality" );
}

void PrintPreviewFunctionality::refresh()
{
    GraphicDisplayFunctionality::refresh();
    Glib::Mutex::Lock lock ( mutex );

    if ( !validVote )
    {
        return;
    }

    if ( docMagnify )
    {
        delete ( docMagnify );
        docMagnify = NULL;
    }

    bool showBarcode = SAESGUIConfigurationManager::getInstance()->getPrintPreviewFunctionalityConfiguration().getShowBarcodeInPreview();
    docMagnify = new VotePrintDocument ( Smartmatic::SAES::Operation::MachineOperationManager::getInstance()->getElectoralConfiguration(), vote, true, showBarcode );
    viewMagnify->remove();

    if ( printWidgetMagnify != NULL )
    {
        delete ( printWidgetMagnify );
        printWidgetMagnify = NULL;
    }

    printWidgetMagnify = new PrintPreviewWidget ( docMagnify );
    viewMagnify->remove();
    viewMagnify->add ( *printWidgetMagnify );
    viewMagnify->show_all_children ();

    switch ( vote.getMode() )
    {
        case Vote::VoteMode::VOTE_D:
            voteMode = Vote::VoteMode::VOTE_D;
            logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Functionality.PrintPreviewFunctionality.VoteMarkedAsDemo" ) );
            break;

        case Vote::VoteMode::VOTE_O:
            voteMode = Vote::VoteMode::VOTE_O;
            logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Functionality.PrintPreviewFunctionality.VoteMarkedAsOfficial" ) );
            break;

        case Vote::VoteMode::VOTE_S:
            voteMode = Vote::VoteMode::VOTE_S;
            logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Functionality.PrintPreviewFunctionality.VoteMarkedAsSimulated" ) );
            break;

        default:
            break;
    }

    BarcodeRefreshVoteMode ( 0 );
}

void PrintPreviewFunctionality::deleteElements()
{
    if ( docMagnify )
    {
        delete ( docMagnify );
        docMagnify = NULL;
    }

    if ( printWidgetMagnify )
    {
        delete ( printWidgetMagnify );
        printWidgetMagnify = NULL;
    }

    if ( labelMessage != NULL )
    {
        delete ( labelMessage );
        labelMessage = NULL;
    }

    if ( frameMagnify != NULL )
    {
        delete ( frameMagnify );
        frameMagnify = NULL;
    }

    if ( vboxMagnify != NULL )
    {
        delete ( vboxMagnify );
        vboxMagnify = NULL;
    }

    if ( scrolledMagnify != NULL )
    {
        delete ( scrolledMagnify );
        scrolledMagnify = NULL;
    }

    if ( viewMagnify != NULL )
    {
        delete ( viewMagnify );
        viewMagnify = NULL;
    }
}

void PrintPreviewFunctionality::block()
{
    logger->Debug ( "Block functionality" );
}

void PrintPreviewFunctionality::partial()
{
    logger->Debug ( "Partial functionality" );
}

void PrintPreviewFunctionality::resume()
{
    logger->Debug ( "Resume functionality" );
}

void PrintPreviewFunctionality::onSignalVote ( Smartmatic::SAES::Voting::Vote::Vote * vote )
{
    this->vote = *vote;
    validVote = true;
}

void PrintPreviewFunctionality::onSignalState ( Smartmatic::SAES::Operation::Voting::VotePersistence::PersistenceStatusVote status )
{
    switch ( status )
    {
        case Smartmatic::SAES::Operation::Voting::VotePersistence::PERSISTENCE_VOTE_FAIL:
        default:
            BarcodeError();
            break;

        case Smartmatic::SAES::Operation::Voting::VotePersistence::PERSISTENCE_VOTE_DUPLICATE:
        case Smartmatic::SAES::Operation::Voting::VotePersistence::PERSISTENCE_VOTE_SUCCESS:
            refresh();
            break;
    }
}
