/**
 * @file	printer-diagnosis-functionality.h
 * @brief	Declares the printer diagnosis functionality class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#ifndef _PRINTER_DIAGNOSIS_FUNCTIONALITY_H_
#define _PRINTER_DIAGNOSIS_FUNCTIONALITY_H_

#include <gtkmm.h>
#include <string>
#include <Functionalities/graphic-display-functionality.h>
#include <Gui/Widgets/Diagnose/PrinterDiagnosisWidget.hxx>
#include "Configuration/s-a-e-s-conf-manager.h"
#include "Printing/DocumentOptions.h"
#include <Log/ISMTTLog.h>

namespace Smartmatic
{
    namespace SAES
    {
        namespace GUI
        {
            namespace Functionality
            {

                /**
                 * @class	PrinterDiagnosisFunctionality
                 * @brief	Printer diagnosis functionality.
                 * @author	Youssef.tourkmani
                 * @date	01/11/2012
                 */
                class PrinterDiagnosisFunctionality: public GraphicDisplayFunctionality
                {

                    public:

                        /**
                         * @fn	bool PrinterDiagnosisFunctionality::DontAllowHelp();
                         * @brief	Gets the dont allow help.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	true if it succeeds, false if it fails.
                         */

                        bool DontAllowHelp();

                        /**
                         * @fn	std::vector<Smartmatic::GUI::Widgets::KeysActionsCode> PrinterDiagnosisFunctionality::getHelpSeqKey();
                         * @brief	Gets the help sequence key.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The help sequence key.
                         */

                        std::vector<Smartmatic::GUI::Widgets::KeysActionsCode> getHelpSeqKey();

                        /**
                         * @fn	bool PrinterDiagnosisFunctionality::DontAllowExit();
                         * @brief	Gets the dont allow exit.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	true if it succeeds, false if it fails.
                         */

                        bool DontAllowExit();

                        /**
                         * @fn	std::vector<Smartmatic::GUI::Widgets::KeysActionsCode> PrinterDiagnosisFunctionality::getSeqKey();
                         * @brief	Gets the sequence key.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The sequence key.
                         */

                        std::vector<Smartmatic::GUI::Widgets::KeysActionsCode> getSeqKey();

                        /**
                         * @fn	static PrinterDiagnosisFunctionality * PrinterDiagnosisFunctionality::getInstance();
                         * @brief	Gets the current.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	null if it fails, else.
                         */

                        static PrinterDiagnosisFunctionality * getInstance();

                    protected:

                        /**
                         * @fn	Smartmatic::Log::ISMTTLog * PrinterDiagnosisFunctionality::getLogger();
                         * @brief	Gets the logger.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	null if it fails, else the logger.
                         */

                        Smartmatic::Log::ISMTTLog * getLogger();

                        /**
                         * @fn	Glib::ustring PrinterDiagnosisFunctionality::getStartLogMessage();
                         * @brief	Gets the start log message.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The start log message.
                         */

                        Glib::ustring getStartLogMessage();

                        /**
                         * @fn	Glib::ustring PrinterDiagnosisFunctionality::getFinishLogMessage();
                         * @brief	Gets the finish log message.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The finish log message.
                         */

                        Glib::ustring getFinishLogMessage();

                        /**
                         * @fn	virtual void PrinterDiagnosisFunctionality::deleteElements();
                         * @brief	virtual Deletes all elements.
                         * @author	Yadickson Soto
                         * @date	03/04/2012
                         */
                        virtual void deleteElements();

                    private:

                        sigc::connection connPrinterDiag;   ///< The connection printer diagram
                        sigc::connection connPrinterDiagOnOk;   ///< The connection printer diagram on ok
                        sigc::connection connPrinterDiagOnCancel;   ///< The connection printer diagram on cancel

                        static PrinterDiagnosisFunctionality * current_instance; ///< The current instance
                        Smartmatic::SAES::GUI::Widgets::PrinterDiagnosisWidget * printerWidget; ///< The printer widget

                        /**
                         * @fn	PrinterDiagnosisFunctionality::PrinterDiagnosisFunctionality();
                         * @brief	Default constructor.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        PrinterDiagnosisFunctionality();

                        /**
                         * @fn	virtual PrinterDiagnosisFunctionality::~PrinterDiagnosisFunctionality();
                         * @brief	Finaliser.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        virtual ~PrinterDiagnosisFunctionality();

                        /**
                         * @fn	void PrinterDiagnosisFunctionality::Dispose();
                         * @brief	Dispose of this object, cleaning up any resources it uses.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void Dispose();

                        /**
                         * @fn	void PrinterDiagnosisFunctionality::Initialize();
                         * @brief	Initializes this object.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void Initialize();

                        /**
                         * @fn	Gtk::Widget * PrinterDiagnosisFunctionality::GetFrame();
                         * @brief	Gets the frame.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	null if it fails, else the frame.
                         */

                        Gtk::Widget * GetFrame();

                        /**
                         * @fn	std::string PrinterDiagnosisFunctionality::GetWindowTitle();
                         * @brief	Gets the window title.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The window title.
                         */

                        std::string GetWindowTitle();

                        /**
                         * @fn	Smartmatic::SAES::Functionality::Configuration::MenuOptionType PrinterDiagnosisFunctionality::GetMenuOptionType();
                         * @brief	Gets the menu option type.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         * @return	The menu option type.
                         */

                        Smartmatic::SAES::Functionality::Configuration::MenuOptionType GetMenuOptionType();

                        static Smartmatic::Log::ISMTTLog * logger; ///< The logger

                        /**
                         * @fn	void PrinterDiagnosisFunctionality::on_PrinterDiagFinished();
                         * @brief	Handles printer diagram finished signals.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void on_PrinterDiagFinished();

                        /**
                         * @fn	void PrinterDiagnosisFunctionality::on_PrinterDiagFinishedOnOk();
                         * @brief	Handles printer diagram finished on ok signals.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void on_PrinterDiagFinishedOnOk();

                        /**
                         * @fn	void PrinterDiagnosisFunctionality::on_PrinterDiagFinishedOnCancel();
                         * @brief	Handles printer diagram finished on cancel signals.
                         * @author	Luiggi.mendez
                         * @date	28/04/2011
                         */

                        void on_PrinterDiagFinishedOnCancel();

                };
            }
        }
    }
}

#endif // _PRINTER_DIAGNOSIS_FUNCTIONALITY_H_
