/**
 * @file	ActivationStatsWidget.cpp
 * @brief	Implements the activation stats widget class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#include "ActivationStatsWidget.h"
#include <Configuration/label-configuration.h>
#include <Configuration/MachineModelEnum.hxx>
#include <Configuration/s-a-e-s-g-u-i-conf-manager.h>
#include <Configuration/s-m-t-t-g-u-i-conf-manager.h>
#include <Environment/Configurations/resource-path-provider.h>
#include <Log/SMTTLogManager.h>
#include <Operation/Devices/CardController.hxx>
#include <Operation/Devices/Polling/CardPolling.hxx>
#include <Operation/MachineOperationManager.hxx>
#include <System/Exception/VotingDeviceException.hxx>
#include "Voting/CardData/card-data-wrapper.h"
#include <System/Utils/StringUtils.hxx>
#include <Operation/Monitor/DeviceTypeMonitorPool.hxx>
#include <Operation/Monitor/SmartcardMonitor.hxx>
#include <System/OptParser.hxx>
#include <Widgets/SmartmaticBasicEventBox.hxx>
#include <Widgets/SmartmaticBasicCheckButton.hxx>
#include <Resources/election-string-resources-manager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

static const int          CARD_ACTIVATION_LABEL_MIN_WIDTH  = 310;
static const int          CARD_ACTIVATION_LABEL_MAX_WIDTH  = 380;
static const int          CARD_ACTIVATION_LABEL_MIN_HEIGHT = 400;
static const unsigned int WAIT_BEFORE_SHOW_REMOVED         = 2000;

using namespace Gtk;
using namespace Smartmatic::Configuration;
using namespace Smartmatic::Environment;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Common::Operation::Monitor;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::SAES::Operation::Monitor;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::SAES::Voting::CardData;
using namespace Smartmatic::SAES::Voting::VotingDevice;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::System;

bool ActivationStatsWidget::oneMemory = false;
ISMTTLog * ActivationStatsWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Functionality.ActivationStatsWidget", GETTEXT_PACKAGE );

ActivationStatsWidget::ActivationStatsWidget ( CountConfiguration & totalCountType, bool showStatistic )
{
    forceUpdate = true;
    m_thread = NULL;
    m_audioVotingEnabled = MachineOperationManager::getInstance()->getAudioVotingCheckUtility().pollingPlaceConfiguredForAudioVoting();

    this->totalCountType = totalCountType;
    ActivationConsolidationFunctionalityConfiguration config =
        SAESGUIConfigurationManager::getInstance()->GetActivationConsolidationFunctionalityConfiguration();
    LabelConfiguration titleLabelConfiguration =
        SAESGUIConfigurationManager::ConvertToLabelConfiguration ( config.getTitleFontConfiguration() );
    LabelConfiguration statLabelConfiguration =
        SAESGUIConfigurationManager::ConvertToLabelConfiguration ( config.getStatsActivationFontConfiguration() );
    LabelConfiguration instructionLabelConfiguration =
        SAESGUIConfigurationManager::ConvertToLabelConfiguration ( config.getInstructionFontConfiguration() );
    imageInsert = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/smartcard_activation.gif" );
    imageExtract = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/wiz_extract_identity_card.gif" );
    imageExtractError = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/wiz_extract_identity_card_error.gif" );
    imageDeviceNotFound = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/smartcard-disconnected.gif" );
    imageCardReaderUnresponsive = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/smartcard-disabled.gif" );

    std::map < std::string, std::map < std::string, std::string > > statisticMap;
    std::map < std::string, std::string > insideStats;

    if ( showStatistic )
    {
        insideStats[N_ ( "Smartmatic.SAES.GUI.Widgets.CustomAttributesCounterWidget.TotalActivated" )] =
            _ ( "Smartmatic.SAES.GUI.Widgets.CustomAttributesCounterWidget.TotalActivated" );
    }

    int cardActivationLabelWidth = CARD_ACTIVATION_LABEL_MIN_WIDTH;

    if ( SMTTGUIConfigurationManager::getInstance()->getSmartmaticGUIMachineModelConfiguration().getId() == EnumToString( PM_VIU805 ) )
    {
        cardActivationLabelWidth = CARD_ACTIVATION_LABEL_MAX_WIDTH;
    }

    statisticMap[_ ( "Smartmatic.SAES.GUI.Functionality.CardGeneratorFunctionality.GeneratedCards" )] = insideStats;
    statsWidget = manage ( new StatisticWidget ( statisticMap, "CardActivationTitle" ) );
    statsWidget->show();
    statsWidget->SetValuesWidth ( 60 );
    statsWidget->SetTitlesSizeRequest ( cardActivationLabelWidth, config.getTitleHeight() );
    statsWidget->SetTitlesLabelConfiguration ( titleLabelConfiguration );
    statsWidget->SetStatsLabelConfiguration ( statLabelConfiguration );
    pack_start ( *statsWidget, false, false );

    setVotingExperienceTypeWidgets(instructionLabelConfiguration);

    instructionLabel = manage ( new SmartmaticBasicLabel() );
    instructionLabel->show();
    instructionLabel->set_size_request ( cardActivationLabelWidth, CARD_ACTIVATION_LABEL_MIN_HEIGHT );
    LabelConfiguration::SetLabelConfiguration ( instructionLabel, instructionLabelConfiguration );
    instructionLabel->set_line_wrap ( true );

    pack_start ( *instructionLabel, true, false);

    cardImage = manage ( new SmartmaticBasicImage() );
    cardImage->show();

    pack_end ( *cardImage, false, false, 20 );
    timerRefresh = SAESGUIConfigurationManager::getInstance()->getPollingCardConfiguration().getActivationStatsWidget();
    show();
    disposeExecuted = false;
    error = true;
    m_powerCycled = false;

    instructionLabel->setAccessibleName ( "CardActivationInfo" );
    cardImage->setAccessibleName ( "CardActivation" );

    signalUpdate.getSyncSignal().connect ( sigc::mem_fun ( *this, &ActivationStatsWidget::UpdateLabel ) );

    m_isHelpShown = false;
    m_writingCard = false;

    m_audioVotingChecked = false;
}

void ActivationStatsWidget::setVotingExperienceTypeWidgets(LabelConfiguration labelConfiguration)
{
    HBox* insideHbox  = manage(new HBox(false, 1));

    insideHbox->show();

    if ( SMTTGUIConfigurationManager::getInstance()->getSmartmaticGUIMachineModelConfiguration().getId() == EnumToString( PM_VIU805 ) )
    {
        pack_start ( *insideHbox, false, false, 50 );
    }
    else
    {
        pack_start ( *insideHbox, false, false);
    }

    HBox* hbox1 = manage(new HBox());
    HBox* hbox2 = manage(new HBox());
    HBox* hbox3 = manage(new HBox());

    hbox1->show();
    hbox2->show();
    hbox3->show();

    m_audioVotingCheckBox.add(*hbox1);
    m_audioVotingCheckBox.show();

    insideHbox->pack_end(*hbox3, true, false);
    insideHbox->pack_end(*hbox2, true, false);
    insideHbox->pack_end(m_audioVotingCheckBox, true, false);

    if (m_audioVotingEnabled)
    {
        m_audioVotingCheckBoxImage.set ( ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/audioVotingUnchecked.png" ) );
        m_audioVotingCheckBoxImage.set_alignment(ALIGN_CENTER, ALIGN_CENTER);
        m_audioVotingCheckBoxImage.show();
        hbox1->pack_end ( m_audioVotingCheckBoxImage, false, false );

        SmartmaticBasicLabel *audioVotingTypeLabel = manage ( new SmartmaticBasicLabel() );
        audioVotingTypeLabel->show();
        audioVotingTypeLabel->set_text(_("Audio voting"));
        labelConfiguration.SetLabelAlignment(LeftAlignment);
        LabelConfiguration::SetLabelConfiguration ( audioVotingTypeLabel, labelConfiguration );
        audioVotingTypeLabel->set_line_wrap ( true );
        hbox2->pack_end ( *audioVotingTypeLabel, false, false );

        SmartmaticBasicImage *audioVotingExperienceTypeImage = manage ( new SmartmaticBasicImage() );

        audioVotingExperienceTypeImage->set ( ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/audioVoting.png" ) );
        audioVotingExperienceTypeImage->set_alignment(ALIGN_LEFT, ALIGN_CENTER);
        audioVotingExperienceTypeImage->show();
        hbox3->pack_end ( *audioVotingExperienceTypeImage, false, false );

        m_audioVotingCheckBoxConnection = m_audioVotingCheckBox.signal_button_press_event().connect(
            sigc::mem_fun(*this, &ActivationStatsWidget::onPressAudioVotingType));

    }
}

bool ActivationStatsWidget::onPressAudioVotingType(GdkEventButton * event)
{
    if (m_audioVotingChecked)
    {
        deselectAudioVotingAttribute();
    }
    else
    {
        selectAudioVotingAttribute();
    }

    m_signalAudioVotingButtonChecked.emit();

    return true;
}

void ActivationStatsWidget::resetAudioVotingSelection()
{
    deselectAudioVotingAttribute();
}

void ActivationStatsWidget::deselectAudioVotingAttribute()
{
    std::string image = ResourcePathProvider::getInstance()->GetResourcePath(
        PACKAGE_NAME,
        "ui/audioVotingUnchecked.png");

    m_audioVotingChecked = false;
    m_audioVotingCheckBoxImage.set(image);
}

void ActivationStatsWidget::selectAudioVotingAttribute()
{
    std::string image = ResourcePathProvider::getInstance()->GetResourcePath(
        PACKAGE_NAME,
        "ui/audioVotingChecked.png");

    m_audioVotingChecked = true;
    m_audioVotingCheckBoxImage.set(image);
}

ActivationStatsWidget::~ActivationStatsWidget()
{
    if ( m_thread != NULL )
    {
        m_thread->join();
        m_thread = NULL;
    }

    conn_checkCard.disconnect();
    conn_updateLabel.disconnect();
    conn_updateStat.disconnect();
    connSleepWriteCard.disconnect();

    m_audioVotingCheckBoxConnection.disconnect();

    if ( !disposeExecuted )
    {
        CardPolling::getInstance()->pause();
    }
}

void ActivationStatsWidget::Initialize()
{
    logger->Debug ( "Initialize" );

    m_audioVotingChecked = false;

    disposeExecuted = false;

    if ( !CardPolling::getInstance()->isRunning() )
    {
        CardPolling::getInstance()->start ( timerRefresh );
        CardPolling::getInstance()->resume();
    }
    else
    {
        CardPolling::getInstance()->resume();
    }

    signalChange.disconnect();
    signalChange = DeviceTypeMonitorPool::getInstance()->getDeviceChangedSignal().connect ( sigc::mem_fun ( *this, &ActivationStatsWidget::deviceChange ) );

    conn_checkCard.disconnect();
    conn_checkCard = CardPolling::getInstance()->getSignalCard().connect ( sigc::mem_fun ( *this, &ActivationStatsWidget::onSignalCardChange ) );

    ContinueOperation();
}

void ActivationStatsWidget::Dispose()
{
    logger->Debug ( "Dispose" );

    disposeExecuted = true;
    signalChange.disconnect();
    conn_checkCard.disconnect();
    conn_updateLabel.disconnect();
    conn_updateStat.disconnect();

    m_audioVotingCheckBoxConnection.disconnect();

    CardPolling::getInstance()->pause();
}

void ActivationStatsWidget::SetAttributes ( std::vector<Custom_attribute> attributes )
{
    Glib::Mutex::Lock lock(m_attributesMutex);

    m_attributes = attributes;
}

void ActivationStatsWidget::WriteCard()
{
    std::vector<Custom_attribute> ptrAttributes;

    {
        Glib::Mutex::Lock lock(m_attributesMutex);

        ptrAttributes = m_attributes;
    }

    error = true;

    if ( WriteVotingCard ( ptrAttributes ) )
    {
        updateLabel = _ ( "Smartmatic.SAES.GUI.Functionality.CardGeneratorFunctionality.CardWasActivated" );
        error = false;
    }
    else
    {
        updateLabel = _ ( "Smartmatic.SAES.GUI.Functionality.CardGeneratorFunctionality.ErrorValidatingData" );
    }

    connSleepWriteCard.disconnect();
    signalUpdate.emit ( NULL );
    m_signalDone.emit();

    m_writingCard = false;
}

void ActivationStatsWidget::UpdateLabel ( void * p )
{
    if ( error )
    {
        updateImage = imageExtractError;
        cardImage->set ( imageExtractError );
    }
    else
    {
        updateImage = imageExtract;
        cardImage->set ( imageExtract );
    }

    instructionLabel->set_label ( updateLabel );
}

bool ActivationStatsWidget::UpdateStat()
{
    CountConfiguration::CountTypeSequence seq = totalCountType.getCountType();
    CountConfiguration::CountTypeIterator it;
    int total = 0;

    for ( it = seq.begin();
            it != seq.end();
            it++ )
    {
        switch ( *it )
        {
            case CountTypeConfig::CONFIGURATION_TOTAL:
                total += MachineOperationManager::getInstance()->getActivatedCardStatistics()->getTotalActivated();
                break;

            case CountTypeConfig::CONFIGURATION_OFFICIAL:
                total += MachineOperationManager::getInstance()->getActivatedCardStatistics()->getTotalOfficialActivated();
                break;

            case CountTypeConfig::CONFIGURATION_DEMO:
                total += MachineOperationManager::getInstance()->getActivatedCardStatistics()->getTotalDemoActivated();
                break;

            default:
            case CountTypeConfig::CONFIGURATION_SIMULATE:
                break;
        }
    }

    if ( statsWidget )
    {
        statsWidget->SetValue ( N_ ( "Smartmatic.SAES.GUI.Widgets.CustomAttributesCounterWidget.TotalActivated" ), StringUtils::intToString ( total ) );
    }

    return false;
}

bool ActivationStatsWidget::WriteVotingCard ( std::vector<Smartmatic::SAES::Voting::VotingDevice::Custom_attribute > attributes )
{
    bool activationOK = false;

    try
    {
        Card_data cardData;
        const bool audioVoting = m_audioVotingEnabled ? m_audioVotingChecked : false;
        CardDataWrapper::GenerateCardData ( MachineOperationManager::getInstance()->getElectoralConfiguration(), MachineOperationManager::getInstance()->getOperationStatus(), attributes, cardData, true, Card_type::CARD_DATA_OFFICIAL_VOTE_CARD, audioVoting );

        if ( !CardController::getInstance()->writeCardData ( cardData ) )
        {
            logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Functionality.ActivationStatsWidget.ErrorActivationCard" ) );
        }
        else
        {
            logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Functionality.ActivationStatsWidget.CardWasActivatedSuccessfully" ) );
            MachineOperationManager::getInstance()->getActivatedCardStatistics()->RefreshStats ( cardData );
            activationOK = true;
        }
    }
    catch ( CryptoException & ex )
    {
        logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Functionality.ActivationStatsWidget.ErrorActivatingCard" ) );
    }
    catch ( SignedException & ex )
    {
        logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Functionality.ActivationStatsWidget.ErrorActivatingCard" ) );
    }
    catch ( XmlException & ex )
    {
        logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Functionality.ActivationStatsWidget.ErrorActivatingCard" ) );
    }
    catch ( VotingDeviceException & ex )
    {
        logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Functionality.ActivationStatsWidget.ErrorActivatingCard" ) );
    }
    catch ( ... )
    {
        logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Functionality.ActivationStatsWidget.ErrorActivatingCard" ) );
    }

    return activationOK;
}

void ActivationStatsWidget::SetOneMemoryMode ( bool oneMemory )
{
    ActivationStatsWidget::oneMemory = oneMemory;
}

void ActivationStatsWidget::AllowCardActivation ( bool allow )
{
    if ( allow )
    {
        ContinueOperation ();
    }
    else
    {
        updateLabel = _ ( "Smartmatic.SAES.GUI.Widgets.ActivationStatsWidget.ActivationNotAllowed" );
        instructionLabel->set_label ( updateLabel );
        smartcardError = true;
    }
}

void ActivationStatsWidget::ContinueOperation()
{
    unsigned int numberOfDevices = DeviceTypeMonitorPool::getInstance()->getNumberOfDevicesByType ( DeviceType::CONFIGURATION_SMARTCARD );

    smartcardError = ( numberOfDevices == 0 );

    UpdateStat();

    bool cardPresent = CardPolling::getInstance()->isCardPresent();

    updateLabel = updateLabel == "" ? _ ( "Smartmatic.SAES.GUI.Functionality.CardGeneratorFunctionality.InsertCardToActivate" ) : updateLabel;
    updateImage = updateImage == "" ? imageInsert : updateImage;

    std::string text = !cardPresent ? _ ( "Smartmatic.SAES.GUI.Functionality.CardGeneratorFunctionality.InsertCardToActivate" ) : updateLabel;
    std::string image = !cardPresent ? imageInsert : updateImage;

    instructionLabel->set_label ( text );
    cardImage->set ( !smartcardError ? image : imageDeviceNotFound );
}

void ActivationStatsWidget::StopOperation()
{
    smartcardError = true;
}

void ActivationStatsWidget::disconnectAudioVotingCheckBox()
{
    m_audioVotingCheckBoxConnection.disconnect();
}

void ActivationStatsWidget::reconnectAudioVotingCheckBox()
{
    m_audioVotingCheckBoxConnection = m_audioVotingCheckBox.signal_button_press_event().connect(
        sigc::mem_fun(*this, &ActivationStatsWidget::onPressAudioVotingType));
}

bool ActivationStatsWidget::showMessageSleepWriteCard()
{
    updateLabel = _ ( "Smartmatic.SAES.GUI.Widgets.ActivationStatsWidget.MessageSleepWriteCard" );
    instructionLabel->set_label ( updateLabel );
    return false;
}

sigc::signal< void, bool > ActivationStatsWidget::getSignalStatusActivated()
{
    return signalStatusActivated;
}

void ActivationStatsWidget::setIsHelpShown( bool isHelpShown )
{
    m_isHelpShown = isHelpShown;
}

ActivationStatsWidget::SignalAudioVotingButtonChecked ActivationStatsWidget::getSignalAudioVotingButtonChecked()
{
    return m_signalAudioVotingButtonChecked;
}

ActivationStatsWidget::SignalDone ActivationStatsWidget::getSignalDone()
{
    return m_signalDone;
}

void ActivationStatsWidget::refreshRemovedAfterPowerCycle()
{
    if (!CardPolling::getInstance()->isCardPresent())
    {
        logger->Debug( "Performing refresh if card is removed in VIU" );
        updateLabel = _ ( "Smartmatic.SAES.GUI.Functionality.CardGeneratorFunctionality.InsertCardToActivate" );
        updateImage = imageInsert;
        instructionLabel->set_label ( updateLabel );
        cardImage->set ( updateImage );
    }
}

void ActivationStatsWidget::deviceChange ( Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor * deviceTypeMonitor )
{
    if ( deviceTypeMonitor->getType() == DeviceType::CONFIGURATION_SMARTCARD )
    {
        if ( deviceTypeMonitor->getNumberOfDevices() != 0 )
        {
            logger->Debug ( "Smartcard connect" );

            if ( m_isHelpShown == false )
            {
                CardPolling::getInstance()->resume ();
            }

            std::string machineModel;
            OptParser::getInstance()->getArg(OptParser::MACHINE_MODEL, machineModel);

            if (StringUtils::toUpper(machineModel) == "PM-NOTEBOOK-A" ||
                StringUtils::toUpper(machineModel) == "PM-NOTEBOOK-B" )
            {
                logger->Debug ( "Connecting normally as a PM" );
                updateLabel = _ ( "Smartmatic.SAES.GUI.Functionality.CardGeneratorFunctionality.InsertCardToActivate" );
                updateImage = imageInsert;
            }
            else
            {
                logger->Debug( "Recovering from a power cycle in VIU" );
                updateLabel = _ ( "Smartmatic.SAES.GUI.Functionality.CardGeneratorFunctionality.ErrorValidatingData" );
                updateImage = imageExtractError;
                m_powerCycled = true;
            }


            smartcardError = false;
            forceUpdate = true;
        }
        else
        {
            smartcardError = true;
            logger->Debug ( "Smartcard disconnect" );

            updateLabel = _ ( "Smartmatic.SAES.GUI.Functionality.CardGeneratorFunctionality.DeviceNotFound" );
            updateImage = imageDeviceNotFound;
        }

        instructionLabel->set_label ( updateLabel );
        cardImage->set ( updateImage );
    }
}

void ActivationStatsWidget::onSignalCardChange ( CardPolling::CardPollingStatus status )
{
    if ( smartcardError )
    {
        return;
    }

    switch ( status )
    {
        case CardPolling::INSERTED:
            cardInsert();
            break;

        case CardPolling::REMOVED:
            cardRemove();
            break;

        default:
        case CardPolling::ERROR_SCR:
            cardReaderUnresponsive();
            break;
    }

    UpdateStat();
}

void ActivationStatsWidget::cardInsert()
{
    Glib::Mutex::Lock lock(m_insertCardMutex);

    if (m_writingCard == false)
    {
        logger->Debug ( "Card Inserted" );

        error = true;
        m_powerCycled = false;
        updateLabel = _ ( "Smartmatic.SAES.GUI.Functionality.ActivationStatsWidget.ActivatingCardPleaseWait" );
        instructionLabel->set_label ( updateLabel );
        updateImage = imageInsert;
        cardImage->set(updateImage);

        if ( m_thread != NULL )
        {
            m_thread->join();
            m_thread = NULL;
        }

        m_writingCard = true;
        m_thread = Glib::Thread::create ( ( sigc::mem_fun ( *this, &ActivationStatsWidget::WriteCard ) ), true );
    }
}

void ActivationStatsWidget::cardRemove()
{
    logger->Debug ( "Card Removed" );

    if (!m_powerCycled)
    {
        updateLabel = _ ( "Smartmatic.SAES.GUI.Functionality.CardGeneratorFunctionality.InsertCardToActivate" );
        updateImage = imageInsert;
        instructionLabel->set_label ( updateLabel );
        cardImage->set ( updateImage );
    }
    else
    {
        logger->Debug( "Card was removed, but is coming from VIU power cycle. Waiting..." );
        Glib::signal_timeout().connect_once(
            sigc::mem_fun(*this, &ActivationStatsWidget::refreshRemovedAfterPowerCycle), WAIT_BEFORE_SHOW_REMOVED);
        m_powerCycled = false;
    }

    signalStatusActivated.emit ( !error );
}

void ActivationStatsWidget::cardReaderUnresponsive()
{
    logger->Debug ( "Smartcard unresponsive" );

    updateLabel = _ ( "Smartmatic.SAES.GUI.Functionality.CardGeneratorFunctionality.CardReaderUnresponsive" );
    updateImage = imageCardReaderUnresponsive;
    instructionLabel->set_label ( updateLabel );
    cardImage->set ( updateImage );

    CardPolling::getInstance()->pause();
}
