/**
 * @file	ConsolidationStatsWidget.cpp
 * @brief	Implements the consolidation stats widget class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#include "ConsolidationStatsWidget.h"
#include <Configuration/MachineModelEnum.hxx>
#include <Configuration/s-a-e-s-g-u-i-conf-manager.h>
#include <Configuration/s-m-t-t-g-u-i-conf-manager.h>
#include <Environment/Configurations/resource-path-provider.h>
#include <Configuration/label-configuration.h>
#include <Log/SMTTLogManager.h>
#include <System/Sounds/smartmatic-sounds-manager.h>
#include "Configuration/label-configuration.h"
#include <System/Utils/StringUtils.hxx>
#include <Runtime/Environment/MemoryPathController.h>
#include <Operation/Devices/BCRController.hxx>
#include <Operation/Devices/Polling/CardPolling.hxx>
#include <Operation/Monitor/DeviceTypeMonitorPool.hxx>
#include <Widgets/SmartmaticBasicHBox.hxx>
#include <Operation/Monitor/DeviceTypeMonitorPool.hxx>
#include <Operation/CheckDataManager.hxx>
#include <Operation/Voting/VotingEnqueuer.hxx>
#include <Voting/PBReports/VoteStatisticsWrapper.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

#define INSTRUCTION_LABEL_WIDTH 310
#define INFO_SCAN_VOTE_LABEL_MIN_WIDTH 300
#define INFO_SCAN_VOTE_LABEL_MAX_WIDTH 420
#define BARCODE_LABEL_MIN_WIDTH 230
#define BARCODE_LABEL_MAX_WIDTH 315
#define EURN_LABEL_MIN_WIDTH 230
#define EURN_LABEL_MAX_WIDTH 315
#define MEMORY_LABEL_MIN_WIDTH 230
#define MEMORY_LABEL_MAX_WIDTH 270
#define SAFE_EJECT_LABEL_MIN_WIDTH 230
#define SAFE_EJECT_LABEL_MAX_WIDTH 300
#define V_BOX_MIN_PADDING 5
#define V_BOX_MAX_PADDING 20

using namespace Gtk;
using namespace Smartmatic::Environment;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Widgets::Voting;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::SAES::Voting::Vote;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::SAES::Common::Operation::Monitor;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Operation::Monitor;
using namespace Smartmatic::SAES::Operation::Voting;
using namespace Smartmatic::Configuration;

Smartmatic::Log::ISMTTLog * ConsolidationStatsWidget::logger = Smartmatic::Log::SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.ConsolidationStatsWidget", GETTEXT_PACKAGE );

ConsolidationStatsWidget::ConsolidationStatsWidget ( CountConfiguration & totalCountType, CountConfiguration & officialCountType )
{
    this->totalCountType = totalCountType;
    this->officialCountType = officialCountType;
    this->infoScanVoteLabel = NULL;

    this->safeEjectLabel = NULL;
    this->safeEjectImage = NULL;
    detailedStatistics = NULL;

    eurnConnectedAndWorkingImage = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/eurn_connected_and_working.gif" );
    eurnConnectedAndNotWorkingImage = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/eurn_connected_and_not_working.gif" );
    eurnNotConnectedImage = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/eurn_not_connected.gif" );

    barcodeConnectedImage = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/barcode_connected_small.gif" );
    barcodeNotConnectedImage = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/barcode_noConnected_small.gif" );
    oneMemoryConnectedImage = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/one-memory-small.gif" );
    twoMemoryConnectedImage = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/both-memory-small.gif" );

    oneMemorySafeEjectImage = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/one-memory-safe-eject.png" );
    twoMemorySafeEjectImage = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/safe-eject.png" );
    twoMemorySafeEjectOverImage = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/safe-eject-over.png" );

    m_isSafeEjectRunning = false;

    static ActivationConsolidationFunctionalityConfiguration config =
        SAESGUIConfigurationManager::getInstance()->GetActivationConsolidationFunctionalityConfiguration();
    static LabelConfiguration titleLabelConfiguration =
        SAESGUIConfigurationManager::ConvertToLabelConfiguration ( config.getTitleFontConfiguration() );
    static LabelConfiguration statLabelConfiguration =
        SAESGUIConfigurationManager::ConvertToLabelConfiguration ( config.getStatsConsolidationFontConfiguration() );
    static LabelConfiguration instructionLabelConfiguration =
        SAESGUIConfigurationManager::ConvertToLabelConfiguration ( config.getInstructionFontConfiguration() );
    static LabelConfiguration devicesLabelConfiguration =
        SAESGUIConfigurationManager::ConvertToLabelConfiguration ( config.getDevicesLabelConfiguration() );

    int statsConsolidationHeight = config.getStatsConsolidationHeight();
    int titleHeight = config.getTitleHeight();
    int infoScanVoteLabelWidth = INFO_SCAN_VOTE_LABEL_MIN_WIDTH;
    int instructionLabelWidth = INSTRUCTION_LABEL_WIDTH;
    int barcodeLabelWidth = BARCODE_LABEL_MIN_WIDTH;
    int eurnLabelWidth = EURN_LABEL_MIN_WIDTH;
    int memoryLabelWidth = MEMORY_LABEL_MIN_WIDTH;
    int safeEjectLabelWidth = SAFE_EJECT_LABEL_MIN_WIDTH;
    int vBoxPadding = V_BOX_MIN_PADDING;

    if ( SMTTGUIConfigurationManager::getInstance()->getSmartmaticGUIMachineModelConfiguration().getId() == EnumToString( PM_VIU805 ) )
    {
        double guiReconfigurationFactor = SMTTGUIConfigurationManager::getInstance()->getConfiguration()->getSmartmaticGUIMachineModelConfiguration().getGuiReconfigurationFactor();

        statsConsolidationHeight *= guiReconfigurationFactor;
        titleHeight *= guiReconfigurationFactor;
        infoScanVoteLabelWidth = INFO_SCAN_VOTE_LABEL_MAX_WIDTH;
        instructionLabelWidth *= guiReconfigurationFactor;
        barcodeLabelWidth = BARCODE_LABEL_MAX_WIDTH;
        eurnLabelWidth = EURN_LABEL_MAX_WIDTH;
        memoryLabelWidth = MEMORY_LABEL_MAX_WIDTH;
        safeEjectLabelWidth = SAFE_EJECT_LABEL_MAX_WIDTH;
        vBoxPadding = V_BOX_MAX_PADDING;
    }

#if	!ENABLE_EXTENDED_STATISTICS
    std::map < std::string, std::map < std::string, std::string > > statisticMap;
    std::map < std::string, std::string > insideStats;

    if ( SAESGUIConfigurationManager::getInstance()->GetActivationConsolidationFunctionalityConfiguration().getShowTotalVotes() )
    {
        insideStats["1Smartmatic.SAES.GUI.Widgets.Voting.VoteStatisticsWidget.TotalVotes"] = _ ( "Smartmatic.SAES.GUI.Widgets.Voting.VoteStatisticsWidget.TotalVotes" );
    }

    if ( SAESGUIConfigurationManager::getInstance()->GetActivationConsolidationFunctionalityConfiguration().getShowOfficialVotes() )
    {
        insideStats["2Smartmatic.SAES.GUI.Widgets.Voting.VoteStatisticsWidget.TotalOfficialVotes"] = _ ( "Smartmatic.SAES.GUI.Widgets.Voting.VoteStatisticsWidget.TotalOfficialVotes" );
    }

    if ( SAESGUIConfigurationManager::getInstance()->GetActivationConsolidationFunctionalityConfiguration().getShowDuplicatesVotes() )
    {
        insideStats["3Smartmatic.SAES.GUI.Widgets.Voting.VoteStatisticsWidget.TotalDuplicateVotes"] = _ ( "Smartmatic.SAES.GUI.Widgets.Voting.VoteStatisticsWidget.TotalDuplicateVotes" );
    }

    statisticMap[_ ( "Smartmatic.SAES.GUI.Widgets.ConsolidationStatsWidget.StatisticTitle" )] = insideStats;
    statsWidget = manage ( new StatisticWidget ( statisticMap ) );
    statsWidget->show();
    statsWidget->set_size_request ( -1, statsConsolidationHeight );
    statsWidget->SetTitlesSizeRequest ( 310, titleHeight );
    statsWidget->SetTitlesLabelConfiguration ( titleLabelConfiguration );
    statsWidget->SetStatsLabelConfiguration ( statLabelConfiguration );
    statsWidget->SetValuesWidth ( 60 );
    statsWidget->set_no_show_all();
    statsWidget->show();
    pack_start ( *statsWidget, false, false );
#else	//0
    detailedStatistics = new DetailedStatistics();
    detailedStatistics->show();
    detailedStatistics->set_size_request ( -1, statsConsolidationHeight );
    detailedStatistics->SetTitle ( _ ( "Smartmatic.SAES.GUI.Widgets.ConsolidationStatsWidget.StatisticTitle" ),
                                   titleLabelConfiguration, infoScanVoteLabelWidth, titleHeight );
    pack_start ( *detailedStatistics, false, false );
#endif
    infoScanVoteLabel = manage ( new SmartmaticBasicLabel() );
    infoScanVoteLabel->show();
    infoScanVoteLabel->set_size_request ( infoScanVoteLabelWidth, -1 );
    LabelConfiguration::SetLabelConfiguration ( infoScanVoteLabel, instructionLabelConfiguration );
    infoScanVoteLabel->set_line_wrap ( true );
    infoScanVoteLabel->set_no_show_all();
    //infoScanVoteLabel->set_visible ( config.getShowScanMessage() );
    infoScanVoteLabel->set_visible ( false );
    pack_start ( *infoScanVoteLabel );
    instructionLabel = manage ( new SmartmaticBasicLabel() );
    instructionLabel->show();
    instructionLabel->set_size_request ( instructionLabelWidth, -1 );
    LabelConfiguration::SetLabelConfiguration ( instructionLabel, instructionLabelConfiguration );
    instructionLabel->set_line_wrap ( true );
    instructionLabel->set_no_show_all();
    instructionLabel->set_visible ( config.getShowBarcodeMessage() );
    instructionLabel->set_visible ( false );
    pack_start ( *instructionLabel );
    vbox = manage ( new VBox() );
    vbox->show();
    pack_end ( *vbox, false, false, 10 );
    SmartmaticBasicHBox * hbox0 = manage ( new SmartmaticBasicHBox() );
    hbox0->show();
    vbox->pack_start ( *hbox0, false, false, vBoxPadding );
    eurnLabel = manage ( new SmartmaticBasicLabel() );
    eurnLabel->show();
    eurnLabel->set_size_request ( eurnLabelWidth, -1 );
    eurnLabel->set_line_wrap();
    eurnLabel->set_justify ( JUSTIFY_CENTER );
    eurnLabel->set_no_show_all();
    hbox0->pack_end ( *eurnLabel );
    eurnImage = manage ( new SmartmaticBasicImage() );
    eurnImage->show();
    eurnImage->set ( eurnConnectedAndWorkingImage );
    hbox0->pack_start ( *eurnImage, false, false, 5 );

    SmartmaticBasicHBox * hbox1 = manage ( new SmartmaticBasicHBox() );
    hbox1->show();
    vbox->pack_start ( *hbox1, false, false, vBoxPadding );
    barcodeLabel = manage ( new SmartmaticBasicLabel() );
    barcodeLabel->show();
    barcodeLabel->set_size_request ( barcodeLabelWidth, -1 );
    barcodeLabel->set_line_wrap();
    barcodeLabel->set_justify ( JUSTIFY_CENTER );
    barcodeLabel->set_no_show_all();

    hbox1->pack_end ( *barcodeLabel );
    barcodeImage = manage ( new SmartmaticBasicImage() );
    barcodeImage->show();
    barcodeImage->set ( barcodeConnectedImage );
    hbox1->pack_start ( *barcodeImage, false, false, 5 );
    SmartmaticBasicHBox * hbox2 = manage ( new SmartmaticBasicHBox() );
    hbox2->show();
    vbox->pack_start ( *hbox2, false, false, vBoxPadding );

    memoryLabel = manage ( new SmartmaticBasicLabel ( _ ( "Smartmatic.SAES.GUI.Widgets.ConsolidationStatsWidget.BothMemoryConnected" ) ) );
    memoryLabel->show();
    memoryLabel->set_size_request ( memoryLabelWidth, -1 );
    memoryLabel->set_line_wrap();
    memoryLabel->set_justify ( JUSTIFY_CENTER );

    hbox2->pack_end ( *memoryLabel );
    memoryImage = manage ( new SmartmaticBasicImage() );
    memoryImage->show();
    memoryImage->set ( twoMemoryConnectedImage );
    hbox2->pack_start ( *memoryImage, false, false, 5 );

    SmartmaticBasicHBox * hbox3 = manage ( new SmartmaticBasicHBox() );
    hbox3->show();
    vbox->pack_start ( *hbox3, false, false, vBoxPadding );

    safeEjectLabel = manage ( new SmartmaticBasicLabel () );
    safeEjectLabel->show();
    safeEjectLabel->set_size_request ( safeEjectLabelWidth, -1 );
    safeEjectLabel->set_line_wrap();
    safeEjectLabel->set_justify ( JUSTIFY_CENTER );
    hbox3->pack_end ( *safeEjectLabel );

    safeEjectEventBox = manage ( new SmartmaticBasicEventBox() );
    safeEjectEventBox->set_events ( Gdk::BUTTON_PRESS_MASK | Gdk::BUTTON_RELEASE_MASK );
    safeEjectEventBox->signal_button_press_event().connect ( sigc::mem_fun ( *this, &ConsolidationStatsWidget::onPressSafeEject ) );
    safeEjectEventBox->signal_button_release_event().connect ( sigc::mem_fun ( *this, &ConsolidationStatsWidget::onReleaseSafeEject ) );
    safeEjectEventBox->show();

    safeEjectImage = manage ( new SmartmaticBasicImage() );
    safeEjectImage->show();
    safeEjectImage->set ( twoMemorySafeEjectImage );

    safeEjectEventBox->add ( *safeEjectImage );
    hbox3->pack_start ( *safeEjectEventBox, false, false, 5 );

    LabelConfiguration::SetLabelConfiguration ( eurnLabel, devicesLabelConfiguration );
    LabelConfiguration::SetLabelConfiguration ( barcodeLabel, devicesLabelConfiguration );
    LabelConfiguration::SetLabelConfiguration ( memoryLabel, devicesLabelConfiguration );
    LabelConfiguration::SetLabelConfiguration ( safeEjectLabel, devicesLabelConfiguration );

    show_all_children();
    show();

    infoScanVoteLabel->setAccessibleName ( "StatisticInfoTop" );
    instructionLabel->setAccessibleName ( "StatisticInfoBottom" );

    hbox0->setAccessibleName ( "EurnStatusContainer" );
    eurnLabel->setAccessibleName ( "EurnStatus" );
    eurnImage->setAccessibleName ( "EurnStatus" );

    hbox1->setAccessibleName ( "BarcodeReaderStatusContainer" );
    barcodeLabel->setAccessibleName ( "BarcodeReaderStatus" );
    barcodeImage->setAccessibleName ( "BarcodeReaderStatus" );

    hbox2->setAccessibleName ( "USBStatusContainer" );
    memoryLabel->setAccessibleName ( "USBStatus" );
    memoryImage->setAccessibleName ( "USBStatus" );

    hbox3->setAccessibleName ( "SafeEjectContainer" );
    safeEjectLabel->setAccessibleName ( "SafeEjectStatus" );
    safeEjectEventBox->setAccessibleName ( "SafeEject" );
    safeEjectImage->setAccessibleName ( "SafeEject" );

    voteWrapper = MachineOperationManager::getInstance()->getVoteStatistics();
    consolidateVoteWithOneMemory = SAESGUIConfigurationManager::getInstance()->GetActivationConsolidationFunctionalityConfiguration().getConsolidateVoteWithOneMemory();
}

ConsolidationStatsWidget::~ConsolidationStatsWidget()
{
    if ( !disposeExecuted )
    {
        barcodeConnected = false;
        Dispose();
    }
}

void ConsolidationStatsWidget::Initialize()
{
    disposeExecuted = false;
    sendingCommands = false;

    DeviceTypeMonitor * deviceTypeMonitor = DeviceTypeMonitorPool::getInstance()->getDeviceTypeMonitor ( DeviceType::CONFIGURATION_BARCODE );
    barcodeConnected = deviceTypeMonitor != NULL && deviceTypeMonitor->getNumberOfDevices() > 0;

    oneMemory = false;
    UpdateBarcodePresence();
    UpdateMemoryPresence();
    UpdateInstructionLabel();
    UpdateStats();
    ConnectSignals();

    EurnController::getInstance()->notifyEurnStatus();
}

void ConsolidationStatsWidget::ConnectSignals()
{
    signalChange.disconnect();
    signalStatusVote.disconnect();
    conn_updateEurnStatus.disconnect();

    signalChange = DeviceTypeMonitorPool::getInstance()->getDeviceChangedSignal().connect ( sigc::mem_fun ( *this, &ConsolidationStatsWidget::deviceChange ) );
    signalStatusVote = VotingEnqueuer::getInstance()->getSignalStatus().connect ( sigc::mem_fun ( *this, &ConsolidationStatsWidget::onSignalState ) );
    conn_updateEurnStatus = EurnController::getInstance()->getSignalEurnSatusChanged().connect ( sigc::mem_fun ( *this, &ConsolidationStatsWidget::onSignalEurnStatus ) );
}

void ConsolidationStatsWidget::DisconnectSignals()
{
    conn_updateEurnStatus.disconnect();
    signalChange.disconnect();
    conn_updateLabelInstruction.disconnect();
    conn_barcodeCommandsSend.disconnect();
    signalStatusVote.disconnect();
}

bool ConsolidationStatsWidget::UpdateInstructionLabel()
{
    if ( !oneMemory && barcodeConnected )
    {
        instructionLabel->set_text ( _ ( "Smartmatic.SAES.GUI.Widgets.ConsolidationStatsWidget.InstructionBarcodeBothMemory" ) );
        barcodeLabel->set_text ( _ ( "Smartmatic.SAES.GUI.Widgets.ConsolidationStatsWidget.WaitScanVoteBarcodeBothMemory" ) );
    }
    else if ( oneMemory && barcodeConnected )
    {
        instructionLabel->set_text ( _ ( "Smartmatic.SAES.GUI.Widgets.ConsolidationStatsWidget.InstructionBarcodeOneMemory" ) );
        barcodeLabel->set_text ( _ ( "Smartmatic.SAES.GUI.Widgets.ConsolidationStatsWidget.WaitScanVoteBarcodeOneMemory" ) );
    }
    else if ( oneMemory && !barcodeConnected )
    {
        instructionLabel->set_text ( _ ( "Smartmatic.SAES.GUI.Widgets.ConsolidationStatsWidget.InstructionNoBarcodeOneMemory" ) );
        barcodeLabel->set_text ( _ ( "Smartmatic.SAES.GUI.Widgets.ConsolidationStatsWidget.WaitScanVoteNoBarcodeOneMemory" ) );
    }
    else
    {
        instructionLabel->set_text ( _ ( "Smartmatic.SAES.GUI.Widgets.ConsolidationStatsWidget.InstructionNoBarcodeBothMemory" ) );
        barcodeLabel->set_text ( _ ( "Smartmatic.SAES.GUI.Widgets.ConsolidationStatsWidget.WaitScanVoteNoBarcodeBothMemory" ) );
    }

    return false;
}

void ConsolidationStatsWidget::BarcodeConnected()
{
    stopOperationAfterSend = false;
    BarcodeCommandsSent ( true );
}

void ConsolidationStatsWidget::BarcodeCommandsSent ( bool commandsOk )
{
    sendingCommands = false;
    m_signal_sendCommands.emit ( commandsOk );

    if ( commandsOk )
    {
        barcodeConnected = true;
        UpdateBarcodePresence();
        vbox->show();

        if ( SAESGUIConfigurationManager::getInstance()->GetActivationConsolidationFunctionalityConfiguration().getShowBarcodeMessage() )
        {
            instructionLabel->show();
        }

        if ( SAESGUIConfigurationManager::getInstance()->GetActivationConsolidationFunctionalityConfiguration().getShowScanMessage() )
        {
            infoScanVoteLabel->show();
        }

#if	!ENABLE_EXTENDED_STATISTICS
        statsWidget->show();
#endif	//0
        UpdateInstructionLabel();
        UpdateStats();

        if ( stopOperationAfterSend )
        {
            stopOperationAfterSend = false;
            StopOperation();
        }
    }
    else
    {
        logger->Debug ( "Send barcode command error" );
        BarcodeDisconnected();
    }

    conn_barcodeCommandsSend.disconnect();
}

void ConsolidationStatsWidget::ShowNormalState()
{
    m_signal_sendCommands.emit ( true );
    vbox->show();

    if ( SAESGUIConfigurationManager::getInstance()->GetActivationConsolidationFunctionalityConfiguration().getShowBarcodeMessage() )
    {
        instructionLabel->show();
    }

    if ( SAESGUIConfigurationManager::getInstance()->GetActivationConsolidationFunctionalityConfiguration().getShowScanMessage() )
    {
        infoScanVoteLabel->show();
    }

#if	!ENABLE_EXTENDED_STATISTICS
    statsWidget->show();
#endif	//0
    //ConnectSignals();
}

void ConsolidationStatsWidget::BarcodeDisconnected()
{
    barcodeConnected = false;
    UpdateBarcodePresence();
    UpdateInstructionLabel();
    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &ConsolidationStatsWidget::ShowNormalState ), 3000 );
}

void ConsolidationStatsWidget::Dispose()
{
    Glib::Mutex::Lock lock ( mutex );
    disposeExecuted = true;
    DisconnectSignals();
}

std::string ConsolidationStatsWidget::getValue ( CountConfiguration::CountTypeSequence & seq )
{
    CountConfiguration::CountTypeIterator it;
    int total = 0;

    for ( it = seq.begin();
            it != seq.end();
            it++ )
    {
        switch ( *it )
        {
            case CountTypeConfig::CONFIGURATION_TOTAL:
                total += voteWrapper->getTotalVotes();
                break;

            case CountTypeConfig::CONFIGURATION_OFFICIAL:
                total += voteWrapper->getOfficialVotes();
                break;

            case CountTypeConfig::CONFIGURATION_DEMO:
                total += voteWrapper->getDemoVotes();
                break;

            case CountTypeConfig::CONFIGURATION_SIMULATE:
                total += voteWrapper->getSimulatedVotes();
                break;

            default:
                break;
        }
    }

    return StringUtils::intToString ( total );
}

void ConsolidationStatsWidget::UpdateStats()
{
#if	!ENABLE_EXTENDED_STATISTICS
    std::string total = getValue ( totalCountType.getCountType() );
    std::string official = getValue ( officialCountType.getCountType() );
    statsWidget->SetValue ( "1Smartmatic.SAES.GUI.Widgets.Voting.VoteStatisticsWidget.TotalVotes", total );
    statsWidget->SetValue ( "2Smartmatic.SAES.GUI.Widgets.Voting.VoteStatisticsWidget.TotalOfficialVotes", official );
    statsWidget->SetValue ( "3Smartmatic.SAES.GUI.Widgets.Voting.VoteStatisticsWidget.TotalDuplicateVotes", StringUtils::intToString ( voteWrapper->getDuplicatedVotes() ) );
#else
    detailedStatistics->Update();
#endif
}

void ConsolidationStatsWidget::SetOneMemoryMode ( bool oneMemory )
{
    this->oneMemory = oneMemory;
    UpdateMemoryPresence();
    UpdateInstructionLabel();

    if ( oneMemory )
    {
        if ( consolidateVoteWithOneMemory )
        {
        }
        else
        {
            instructionLabel->set_text ( _ ( "Smartmatic.SAES.GUI.Widgets.ConsolidationStatsWidget.ConnectMemoryToConsolidate" ) );
        }
    }
}

void ConsolidationStatsWidget::ContinueOperation()
{
    stopOperationAfterSend = false;
}

void ConsolidationStatsWidget::StopOperation()
{
    stopOperationAfterSend = true;
}

void ConsolidationStatsWidget::pauseOperation()
{
}

void ConsolidationStatsWidget::resumeOperation()
{
    logger->Debug ( "resumeOperation" );
    m_isSafeEjectRunning = false;
}

ConsolidationStatsWidget::signal_void_bool ConsolidationStatsWidget::barcode_SendCommands()
{
    return m_signal_sendCommands;
}

void ConsolidationStatsWidget::UpdateMemoryPresence()
{
    if ( !MemoryPathController::getInstance()->isAllDevices() )
    {
        memoryLabel->set_text ( _ ( "Smartmatic.SAES.GUI.Widgets.ConsolidationStatsWidget.OneMemoryConnected" ) );
        memoryImage->set ( oneMemoryConnectedImage );

        safeEjectLabel->set_text ( _ ( "Cannot safe eject with one memory" ) );
        safeEjectImage->set ( oneMemorySafeEjectImage );
        safeEjectEventBox->set_sensitive ( false );
    }
    else
    {
        memoryLabel->set_text ( _ ( "Smartmatic.SAES.GUI.Widgets.ConsolidationStatsWidget.BothMemoryConnected" ) );
        memoryImage->set ( twoMemoryConnectedImage );

        safeEjectLabel->set_text ( _ ( "Press button to safe eject" ) );
        safeEjectImage->set ( twoMemorySafeEjectImage );
        safeEjectEventBox->set_sensitive ( true );
    }
}

void ConsolidationStatsWidget::updateEurnStatus(EurnController::DeviceStatus status)
{
     switch(status)
     {
         case EurnController::CONNECTED_WORKING:
             eurnLabel->set_text ( _ ( "eUrn is responsive" ) );
             eurnImage->set ( eurnConnectedAndWorkingImage );
         break;
         case EurnController::CONNECTED_NOT_RESPONDING:
             eurnLabel->set_text ( _ ( "eUrn is not responsive" ) );
             eurnImage->set ( eurnConnectedAndNotWorkingImage );
         break;
         default:
         case EurnController::NOT_CONNECTED:
            eurnLabel->set_text ( _ ( "eUrn is not connected" ) );
            eurnImage->set ( eurnNotConnectedImage );
         break;
     }
}

void ConsolidationStatsWidget::UpdateBarcodePresence()
{
    if ( barcodeConnected )
    {
        //barcodeLabel->set_text ( _ ( "Smartmatic.SAES.GUI.Widgets.ConsolidationStatsWidget.BarcodeConnected" ) );
        barcodeImage->set ( barcodeConnectedImage );
    }
    else
    {
        //barcodeLabel->set_text ( _ ( "Smartmatic.SAES.GUI.Widgets.ConsolidationStatsWidget.BarcodeNotConnected" ) );
        barcodeImage->set ( barcodeNotConnectedImage );
    }
}

void ConsolidationStatsWidget::onSignalState ( VotePersistence::PersistenceStatusVote status )
{
    logger->Debug ( "Signal State" );

    if(status == VotePersistence::PERSISTENCE_VOTE_SUCCESS)
    {
        UpdateStats();
    }
}

void ConsolidationStatsWidget::onSignalEurnStatus ( EurnController::DeviceStatus status )
{
    Glib::signal_timeout().connect_once (sigc::bind<EurnController::DeviceStatus> (sigc::mem_fun ( *this, &ConsolidationStatsWidget::updateEurnStatus ), status), 10 );
}

void ConsolidationStatsWidget::deviceChange ( Smartmatic::SAES::Common::Operation::Monitor::DeviceTypeMonitor * deviceTypeMonitor )
{
    if ( deviceTypeMonitor->getType() == DeviceType::CONFIGURATION_BARCODE )
    {
        if ( deviceTypeMonitor->getNumberOfDevices() != 0 )
        {
            barcodeConnected = true;
            BarcodeConnected();
        }
        else
        {
            barcodeConnected = false;
            BarcodeDisconnected();
        }
    }
    else if ( deviceTypeMonitor->getType() == DeviceType::CONFIGURATION_SMARTCARD )
    {
        if (m_isSafeEjectRunning)
        {
            CardPolling::getInstance()->pause();
        }
    }
}

bool ConsolidationStatsWidget::onPressSafeEject ( GdkEventButton * event )
{
    safeEjectImage->set ( twoMemorySafeEjectOverImage );
    return false;
}

bool ConsolidationStatsWidget::onReleaseSafeEject ( GdkEventButton * event )
{
    logger->Audit ( _ ( "Call safe eject action" ) );

    m_isSafeEjectRunning = true;
    safeEjectLabel->set_text ( _ ( "Waiting by safe eject" ) );
    safeEjectImage->set ( twoMemorySafeEjectImage );
    safeEjectEventBox->set_sensitive ( false );
    CheckDataManager::getInstance()->startSafeSynchronize();
    return false;
}
