/**
 * @file	DiagnosticResultWidget.cxx
 * @brief	diagnostic result widget class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */

#include "DiagnosticResultWidget.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Environment/Configurations/resource-path-provider.h>
#include <Configuration/MachineModelEnum.hxx>
#include <Configuration/s-a-e-s-g-u-i-conf-manager.h>
#include <Configuration/s-m-t-t-g-u-i-conf-manager.h>
#include <System/Runtime/PowerDown.hxx>
#include <Operation/MachineOperationManager.hxx>
#include <Operation/AccessibleManager.hxx>
#include <Voting/VotingCenters/VotingCentersWrapper.hxx>
#include <Gui/Windows/MemoryMessageWindow.hxx>
#include <Voting/Status/voting-machine-status-schema.hxx>
#include <System/IO/Directory.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate
#define BUTTON_MIN_WIDTH 250
#define BUTTON_MIN_HEIGHT 70
#define BUTTON_MAX_WIDTH 300
#define BUTTON_MAX_HEIGHT 90

using namespace Smartmatic::Configuration;
using namespace Smartmatic::Environment;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::GUI::Operation;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Windows;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::SAES::Voting::VotingCenters;
using namespace Smartmatic::SAES::Voting::Status;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::Runtime;

static const int DEFAULT_TIME_TO_SHOW_MS = 5000;

ISMTTLog* DiagnosticResultWidget::logger = SMTTLogManager::GetLogger(
    "Smartmatic.SAES.GUI.Widgets.DiagnosticResultWidget",
    GETTEXT_PACKAGE);

DiagnosticResultWidget::DiagnosticResultWidget ( bool forceButtonFailed, int labelWidth, int labelHeight )
    : StepWidget()
    , mandatoryFail(false)
    , forceButtonFailed(forceButtonFailed)
    , mainFrame(NULL)
    , centerFrame(NULL)
    , mainVbox(NULL)
    , centerVbox(NULL)
    , centerHbox(NULL)
    , infoLabel(NULL)
    , buttonbox(NULL)
    , buttonHBox(NULL)
    , labelWidth(labelWidth)
    , labelHeight(labelHeight)
    , actionButton(NULL)
    , shutdownButton(NULL)
    , showInstallationScreen(true)
    , timeToShow(DEFAULT_TIME_TO_SHOW_MS)
{
    diagnosis.clear();
    this->initialize();
    setWidget ( mainFrame );
}

DiagnosticResultWidget::~DiagnosticResultWidget()
{
    dispose();
}

void DiagnosticResultWidget::initialize()
{
    dispose();
    Glib::RefPtr<Gtk::Builder> builder;
    std::string gladeFile;
    gladeFile = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/DiagnosticResultWidget.glade" );

    try
    {
        builder = Gtk::Builder::create_from_file ( gladeFile );
    }
    catch ( const Glib::FileError & ex )
    {
        logger->Debug ( "FATAL - ContingencyWidget Initialize" );
    }

    builder->get_widget ( "main-frame", mainFrame );
    builder->get_widget ( "main-vbox", mainVbox );
    builder->get_widget ( "center-frame", centerFrame );
    builder->get_widget ( "center-vbox", centerVbox );
    builder->get_widget ( "center-hbox", centerHbox );
    builder->get_widget ( "vbuttonbox", buttonbox );
    builder->get_widget ( "info-label", infoLabel );
    builder->get_widget ( "hbox1", buttonHBox );
    builder->get_widget_derived ( "action-button", actionButton );
    builder->get_widget_derived ( "shutdown-button", shutdownButton );

    int buttonWidth = BUTTON_MIN_WIDTH;
    int buttonHeight = BUTTON_MIN_HEIGHT;

    if ( SMTTGUIConfigurationManager::getInstance()->getSmartmaticGUIMachineModelConfiguration().getId() == EnumToString( PM_VIU805 ) )
    {
        buttonWidth = BUTTON_MAX_WIDTH;
        buttonHeight = BUTTON_MAX_HEIGHT;
    }

    actionButton->set_size_request( buttonWidth, buttonHeight );
    shutdownButton->set_size_request( buttonWidth, buttonHeight );

    actionButton->setAccessibleName ("actionButton");
    AccessibleManager::setName ( centerFrame, "diagnosisMainPanel" );
    AccessibleManager::setName ( infoLabel, "informationLabel" );
    AccessibleManager::setName ( centerVbox, "diagnosisResultsPanel" );
    diagnosis = MachineOperationManager::getInstance()->getOperationStatus()->getVotingMachineStatusWrapper()->getDeviceStatusList ();
    std::list <Smartmatic::SAES::Voting::Status::DeviceStatus >::iterator it;

    for ( it = diagnosis.begin(); it != diagnosis.end(); ++it )
    {
        const bool isAbleDevice   = (it->getVMType() == VMType::CONFIGURATION_ABLED) || (it->getVMType() == VMType::CONFIGURATION_P168A);
        const bool isNotDiagnosed = it->getVMStatus().getStatus().get() == VMStatus::VOTING_MACHINE_STATUS_NOTDIAGNOSED;
        const bool isMouseDevice  = it->getVMType() == VMType::CONFIGURATION_MOUSE;

        if (((isAbleDevice == false) || (isNotDiagnosed == false)) && (isMouseDevice == false))
        {
            DeviceResultWidget * widget = new DeviceResultWidget ( *it, labelWidth, labelHeight );
            deviceList.push_back ( widget );
            centerVbox->add ( *widget );
        }
    }

    LabelConfiguration diagConf = SAESGUIConfigurationManager::getInstance()->getResultErrorLabelConfiguration();
    LabelConfiguration::SetLabelConfiguration ( infoLabel, diagConf );
    infoLabel->set_visible ( false );
    infoLabel->set_no_show_all ( true );
    buttonbox->set_visible ( false );
    buttonbox->set_no_show_all ( true );
}

void DiagnosticResultWidget::dispose()
{
    this->remove();

    if ( mainFrame )
    {
        delete mainFrame;
        mainFrame = NULL;
    }

    if ( mainVbox )
    {
        delete mainVbox;
        mainVbox = NULL;
    }

    if ( centerFrame )
    {
        delete centerFrame;
        centerFrame = NULL;
    }

    if ( centerHbox )
    {
        delete centerHbox;
        centerHbox = NULL;
    }

    if ( centerVbox )
    {
        delete centerVbox;
        centerVbox = NULL;
    }

    if ( actionButton )
    {
        delete actionButton;
        actionButton = NULL;
    }

    if ( buttonbox )
    {
        delete buttonbox;
        buttonbox = NULL;
    }

    if ( infoLabel )
    {
        delete infoLabel;
        infoLabel = NULL;
    }

    if ( buttonHBox )
    {
        delete buttonHBox;
        buttonHBox = NULL;
    }

    if ( shutdownButton )
    {
        delete shutdownButton;
        shutdownButton = NULL;
    }

    std::list < DeviceResultWidget * >::iterator it;

    for ( it = deviceList.begin();
            it != deviceList.end();
            ++it )
    {
        DeviceResultWidget * widget = ( *it );
        delete widget;
    }
}

void DiagnosticResultWidget::refresh()
{
    memorySignal = MemoryMessageWindow::getInstance()->getSignalBlock().connect( sigc::mem_fun ( *this, &DiagnosticResultWidget::onSignalBlock ) );

    if ( !forceButtonFailed )
    {
        logger->Debug ( "Exit diagnostic" );
        actionButton->set_visible ( false );
        buttonbox->remove ( *actionButton );
        Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this , &DiagnosticResultWidget::end ), timeToShow );
        return;
    }

    logger->Debug ( "End diagnostic" );
    std::list < DeviceResultWidget * >::iterator it;
    bool fail = false;
    mandatoryFail = false;
    for ( it = deviceList.begin();
            it != deviceList.end();
            ++it )
    {
        DeviceResultWidget * widget = ( *it );
        Status::DeviceStatus & interface ( widget->getDiagnosis() );
        widget->update();

        if ( interface.getVMStatus().getStatus().get() != Status::VMStatus::VOTING_MACHINE_STATUS_OK )
        {

            fail = true;
            if ( interface.getMandatory().present() && interface.getMandatory().get() )
            {
                mandatoryFail = true;
            }
        }
    }

    actionButton->set_label ( _ ( "Smartmatic.SAES.GUI.Windows.AutomaticDiagnosticWindow.DiagnosticWithErrorFailedButtonText" ) );
    actionButton->signal_clicked().connect ( sigc::mem_fun ( *this, &DiagnosticResultWidget::end ) );

    shutdownButton->set_label ( _ ( "Smartmatic.SAES.GUI.Windows.AutomaticDiagnosticWindow.ButtonShutDown" ) );
    shutdownButton->signal_clicked().connect ( sigc::mem_fun ( *this, &DiagnosticResultWidget::shutDown ) );

    buttonbox->set_visible ( true );
    actionButton->set_visible ( false );
    shutdownButton->set_visible ( false );
    bool useOkButtonFailed = SAESGUIConfigurationManager::getInstance()->GetAutomaticWindowConfigurationForOfficialMode().getUseOkButtonOnSomeNonMandatoryDiagnosticFailed();
    bool useShutDownButtonMandatoryFailed = SAESGUIConfigurationManager::getInstance()->GetAutomaticWindowConfigurationForOfficialMode().getUseShutDownButtonOnSomeMandatoryDiagnosticFailed();

    SaesDirectories directories;
    bool installationFileExists = Directory::getInstance()->FileExists( directories.getVotingCentersInstalationFileFullPath() );
    bool domFileExists = Directory::getInstance()->FileExists( directories.getVotingCentersLocalFileFullPath ( BasePathType::CONFIGURATION_DOM ) );
    bool flashFileExists = Directory::getInstance()->FileExists( directories.getVotingCentersLocalFileFullPath ( BasePathType::CONFIGURATION_FLASH ) );

    if ( installationFileExists || domFileExists || flashFileExists )
    {
        if ( VotingCentersWrapper::getInstance()->getRunningMode() == RunningMode::VOTING_CENTER_RECOUNT )
        {
            useOkButtonFailed = SAESGUIConfigurationManager::getInstance()->GetAutomaticWindowConfigurationForRecountMode().getUseOkButtonOnSomeNonMandatoryDiagnosticFailed();
            useShutDownButtonMandatoryFailed = SAESGUIConfigurationManager::getInstance()->GetAutomaticWindowConfigurationForRecountMode().getUseShutDownButtonOnSomeMandatoryDiagnosticFailed();
        }
    }

    if ( fail && !mandatoryFail && useOkButtonFailed == true )
    {
        actionButton->set_visible ( true );
        shutdownButton->set_visible ( true );
    }
    else if ( mandatoryFail )
    {
        shutdownButton->set_visible ( true );

        if ( useShutDownButtonMandatoryFailed )
        {
            infoLabel->set_visible ( true );
            infoLabel->set_text ( _ ( "Smartmatic.SAES.GUI.Windows.AutomaticDiagnosticWindow.HardwareDiagnosticFailedText" ) );
        }
        else
        {
            actionButton->set_visible ( false );
            shutdownButton->set_visible ( false );
            buttonHBox->remove ( *actionButton );
            buttonHBox->remove ( *shutdownButton );
            Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this , &DiagnosticResultWidget::fail ), 5000 );
        }
    }
    else
    {
        actionButton->set_visible ( false );
        shutdownButton->set_visible ( false );
        buttonHBox->remove ( *actionButton );
        buttonHBox->remove ( *shutdownButton );
        Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this , &DiagnosticResultWidget::end ), 3000 );
    }
}

void DiagnosticResultWidget::setTimeToShow(int timeToShow)
{
    this->timeToShow = timeToShow;
}

void DiagnosticResultWidget::shutDown()
{
    PowerDown::shutdown ( );
}

void DiagnosticResultWidget::fail()
{
    centerFrame->remove();
    Gtk::Image * imagePen = manage ( new Gtk::Image() );
    imagePen->set ( Gdk::PixbufAnimation::create_from_file ( ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/AutoDiagnosticFailed.img" ) ) );
    infoLabel->set_visible ( true );
    infoLabel->set_text ( _ ( "Smartmatic.SAES.GUI.Windows.AutomaticDiagnosticWindow.HardwareDiagnosticFailedText" ) );
    actionButton->set_visible ( true );
    buttonbox->add ( *actionButton );
    centerFrame->add ( *imagePen );
    centerFrame->show_all_children();
}

void DiagnosticResultWidget::end()
{
    if ( showInstallationScreen )
        emit();

    memorySignal.disconnect();
}

void DiagnosticResultWidget::onSignalBlock()
{
    showInstallationScreen = false;
}
