/**
 * @file SerialNumberWidget.cxx
 * @brief Body class serial number widget
 * @date 21/09/2012
 * @author Yadickson Soto
 */

#include "SerialNumberWidget.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Gui/Widgets/FactoryMethodWidget.hxx>
#include <Environment/Configurations/resource-path-provider.h>
#include "Configuration/label-configuration.h"
#include "Configuration/s-a-e-s-g-u-i-conf-manager.h"
#include <Widgets/SmartmaticRadioButton.hxx>
#include <System/Utils/StringUtils.hxx>
#include <System/Sounds/smartmatic-sounds-manager.h>
#include <Configuration/s-m-t-t-g-u-i-conf-manager.h>
#include <Configuration/MachineModelEnum.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate
#define LABEL_MIN_WIDTH 160
#define LABEL_MAX_WIDTH 200
#define ENTRY_MIN_WIDTH 100
#define ENTRY_MAX_WIDTH 125
#define KEYPAD_TABLE_MAX_HEIGHT 250
#define KEYPAD_BUTTONS_MAX_WIDTH 60
#define KEYPAD_BUTTONS_MAX_HEIGHT 60

using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::Log;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::Environment;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::HW::Configuration;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::System::Sounds;
using namespace Smartmatic::Configuration;

ISMTTLog * SerialNumberWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.SerialNumberWidget", GETTEXT_PACKAGE );

SerialNumberWidget::SerialNumberWidget ( bool showKeyBoard )
    : BasicDiagnosticWidget ( true )
{
    this->showKeyBoard = showKeyBoard;
    mainFrame = NULL;
    keyBoardAlignment = NULL;
    mainVbox = NULL;
    titleText = NULL;
    typeText = NULL;
    serialText = NULL;
    serialTitle = NULL;
    separator = NULL;
    keyboard = NULL;
    serialEntry = NULL;
    groupButton = NULL;
    radioButtonSerial = NULL; // don't delete
    tableText = NULL;
    currentMax = 0;

    initialize();
    setWidget ( mainFrame );
}

SerialNumberWidget::~SerialNumberWidget()
{
    remove();

    if ( mainFrame )
    {
        delete mainFrame;
        mainFrame = NULL;
    }

    if ( keyBoardAlignment )
    {
        delete keyBoardAlignment;
        keyBoardAlignment = NULL;
    }

    if ( mainVbox )
    {
        delete mainVbox;
        mainVbox = NULL;
    }

    if ( titleText )
    {
        delete titleText;
        titleText = NULL;
    }

    if ( typeText )
    {
        delete typeText;
        typeText = NULL;
    }

    if ( serialText )
    {
        delete serialText;
        serialText = NULL;
    }

    if ( serialTitle )
    {
        delete serialTitle;
        serialTitle = NULL;
    }

    if ( separator )
    {
        delete separator;
        separator = NULL;
    }

    if ( keyboard )
    {
        delete keyboard;
        keyboard = NULL;
    }

    if ( serialEntry )
    {
        delete serialEntry;
        serialEntry = NULL;
    }

    if ( groupButton )
    {
        delete groupButton;
        groupButton = NULL;
    }

    if ( tableText )
    {
        delete tableText;
        tableText = NULL;
    }
}

void SerialNumberWidget::initialize()
{
    groupButton = new SmartmaticRadioButtonVerticalGroup();

    Glib::RefPtr<Gtk::Builder> builder;
    std::string gladeFile = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/SerialNumberWidget.glade" );

    if ( SAESGUIConfigurationManager::getInstance()->getStartConfiguration().getInfoMachine() == InfoMachineConfiguration::CONFIGURATION_VM )
    {
        gladeFile = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/SerialNumberWidget-VM.glade" );
    }

    try
    {
        builder = Gtk::Builder::create_from_file ( gladeFile );
    }
    catch ( const Glib::FileError & ex )
    {
        logger->Debug ( "FATAL - SerialNumberWidget Initialize" );
    }

    keyboard = new KeypadWidget ( CorrigeConfirma, true, KEYPAD_TABLE_MAX_HEIGHT, KEYPAD_BUTTONS_MAX_WIDTH, KEYPAD_BUTTONS_MAX_HEIGHT, true );
    keyboard->keypadinterface = this;

    builder->get_widget ( "main-frame", mainFrame );
    builder->get_widget ( "main-vbox", mainVbox );
    builder->get_widget ( "table1", tableText );
    builder->get_widget ( "keyboard", keyBoardAlignment );
    builder->get_widget_derived ( "label-title", titleText );
    builder->get_widget_derived ( "label-serial", serialText );
    builder->get_widget_derived ( "label1", serialTitle );
    builder->get_widget_derived ( "label-type", typeText );
    builder->get_widget ( "hseparator", separator );
    builder->get_widget_derived ( "entry-serial", serialEntry );
    builder->get_widget_derived ( "group-button", groupButton );

    keyBoardAlignment->add ( *keyboard );
    keyBoardAlignment->show_all_children();
    serialEntry->SetType ( SmartmaticEntry::Mask );
    serialEntry->setMandatory ( true );
    serialEntry->setBGError();

    SerialNumberConfiguration::OptionSequence & seq ( SAESGUIConfigurationManager::getInstance()->getSerialNumberConfiguration().getOption() );
    SerialNumberConfiguration::OptionIterator it;

    LabelConfiguration infoConf = SAESGUIConfigurationManager::getInstance()->getSerialNumberInfoConfiguration();

    int labelWidth = LABEL_MIN_WIDTH;
    int entryWidth = ENTRY_MIN_WIDTH;

    bool runningVIU = SMTTGUIConfigurationManager::getInstance()->getSmartmaticGUIMachineModelConfiguration().getId() == EnumToString( PM_VIU805 );

    if ( runningVIU )
    {
        labelWidth = LABEL_MAX_WIDTH;
        entryWidth = ENTRY_MAX_WIDTH;
    }

    for ( it = seq.begin();
            it != seq.end();
            it++ )
    {
        SmartmaticRadioButton * radioButton = new SmartmaticRadioButton ( ( *it ).getName(), ( *it ).getCode() );
        radioButton->setFontSize ( infoConf.GetSize() );

        if ( ( *it ).getCode().empty() )
        {
            radioButtonSerial = radioButton;
            radioButtonSerial->setName ( _ ( "Find Serial Number" ) );
            radioButtonSerial->setFontSize ( infoConf.GetSize() + 4 );
        }
        else
        {
            currentMax = ( *it ).getMax();
            currentMask = ( *it ).getMask();
            currentRegex = ( *it ).getRegex();
            currentPrefix = ( *it ).getCode();

            std::stringstream ss;
            ss << currentMax << "##" << currentMask << "##" << currentRegex;

            radioButton->setAttribute ( ss.str() );
            allRadioButton.push_back ( radioButton );

            serialText->set_text ( ( *it ).getCode() );
        }

        groupButton->addRadioButton ( radioButton );
        radioButton->set_active ( false );
        radioButton->show_all_children();
    }

    SmartmaticRadioButton * radioButton = new SmartmaticRadioButton ( ".", "." );
    groupButton->addRadioButton ( radioButton );
    radioButton->set_active ( true );
    radioButton->set_child_visible ( false );

    groupButton->set_no_show_all();
    groupButton->set_visible ( seq.size() > 1 );

    keyBoardAlignment->set_no_show_all();
    separator->set_no_show_all();
    typeText->set_no_show_all();

    keyBoardAlignment->set_visible ( showKeyBoard || runningVIU );
    separator->set_visible ( showKeyBoard || runningVIU );

    tableText->set_no_show_all();
    tableText->set_visible ( false );

    serialTitle->set_no_show_all();
    serialTitle->set_visible ( false );

    serialEntry->signal_changed().connect ( sigc::mem_fun ( *this, &SerialNumberWidget::clearMessage ) );

    signalOkDetected.connect ( sigc::mem_fun ( *this, &SerialNumberWidget::validate ) );
    signalCancelDetected.connect ( sigc::mem_fun ( *this, &SerialNumberWidget::cancel ) );

    setOkButtonSensitive ( false );

    groupButton->getSignalChange().connect ( sigc::mem_fun ( *this, &SerialNumberWidget::changeOption ) );
    LabelConfiguration titleConf = SAESGUIConfigurationManager::getInstance()->getSerialNumberTitleConfiguration();
    LabelConfiguration::SetLabelConfiguration ( titleText, titleConf );
    LabelConfiguration::SetLabelConfigurationWithoutAlignment ( serialTitle, infoConf );
    LabelConfiguration::SetLabelConfiguration ( serialText, infoConf );
    LabelConfiguration::SetLabelConfiguration ( typeText, infoConf );
    LabelConfiguration messageConf = SAESGUIConfigurationManager::getInstance()->getSerialNumberMessageConfiguration();

    switch ( SAESGUIConfigurationManager::getInstance()->getStartConfiguration().getInfoMachine() )
    {
        case InfoMachineConfiguration::CONFIGURATION_PM:
            {
                titleText->set_text ( _ ( "PM Serial Number Info" ) );
            }
            break;

        default:
        case InfoMachineConfiguration::CONFIGURATION_VM:
            {
                titleText->set_text ( _ ( "VM Serial Number Info" ) );
            }
            break;
    }

    typeText->set_text ( _ ( "(fill in)" ) );
    serialTitle->set_text ( _ ( "Serial Number:" ) );

    serialText->set_size_request( labelWidth, -1 );
    serialEntry->set_size_request( entryWidth, -1 );

    groupButton->show_all_children();
    tableText->set_visible ( seq.size() <= 1 );
    serialTitle->set_visible ( true );

    serialEntry->setMax ( currentMax );
    serialEntry->setMask ( currentMask );
    serialEntry->setRegex ( currentRegex );

    clearMessage();
    show_all_children();

    serialEntry->set_sensitive ( false );
}

void SerialNumberWidget::setSerialNumber ( const std::string & serial )
{
    bool validate = false;
    this->serialDetected = serial;

    std::list < SmartmaticRadioButton * >::iterator it;

    for ( it = allRadioButton.begin();
            it != allRadioButton.end() && !validate;
            ++it )
    {
        validate = StringUtils::regex ( ( *it )->getName(), serial );
    }

    groupButton->clearAll();
    setOkButtonSensitive ( validate );
    keyboard->setSensitiveButton ( kConfirma, validate );

    if ( radioButtonSerial )
    {
        Glib::ustring key = ( validate ? _ ( "Smartmatic.SAES.GUI.Widgets.SerialNumberWidget.Custom" ) : _ ( "Serial Number Not Detected" ) );
        Glib::ustring composed = Glib::ustring::compose ( key, serial );
        radioButtonSerial->setName ( composed );
        radioButtonSerial->setCode ( serial );
        radioButtonSerial->set_active ( validate );
        radioButtonSerial->set_sensitive ( validate );
    }

    serialEntry->set_sensitive ( true );
    serialEntry->focus();

    groupButton->start();
}

std::string SerialNumberWidget::getSerialNumber()
{
    serialEntry->set_sensitive ( false );
    setOkButtonSensitive ( false );
    keyboard->setSensitiveButton ( kConfirma, false );
    keyboard->setSensitiveButton ( kCorrige, false );

    return currentPrefix + serialEntry->get_text();
}

void SerialNumberWidget::clearMessage()
{
    setOkButtonSensitive ( serialEntry->validate() );
    keyboard->setSensitiveButton ( kConfirma, serialEntry->validate() );
}

void SerialNumberWidget::SetKey ( KeysActionsCode value )
{
    switch ( value )
    {
        case Key_0:
        case Key_Aaccent:
        case Key_Arroba:
        case Key_Pad0:
        case Key_Caps0:
            OnKeyPress ( 0 );
            break;

        case Key_1:
        case Key_And:
        case Key_Alt1:
        case Key_F1:
        case Key_Pad1:
            OnKeyPress ( 1 );
            break;

        case Key_2:
        case Key_Eaccent:
        case Key_Alt2:
        case Key_F2:
        case Key_Pad2:
        case Key_Caps2:
            OnKeyPress ( 2 );
            break;

        case Key_3:
        case Key_Quote:
        case Key_F3:
        case Key_Pad3:
            OnKeyPress ( 3 );
            break;

        case Key_4:
        case Key_SingleQuote:
        case Key_OpenKey:
        case Key_F4:
        case Key_Pad4:
            OnKeyPress ( 4 );
            break;

        case Key_5:
        case Key_OpenParentheses:
        case Key_OpenBrace:
        case Key_F5:
        case Key_Pad5:
            OnKeyPress ( 5 );
            break;

        case Key_6:
        case Key_Minus:
        case Key_Or:
        case Key_F6:
        case Key_Pad6:
            OnKeyPress ( 6 );
            break;

        case Key_7:
        case Key_Eaccent2:
        case Key_Accent2:
        case Key_F7:
        case Key_Pad7:
        case Key_Caps7:
            OnKeyPress ( 7 );
            break;

        case Key_8:
        case Key_MinusBig:
        case Key_BackSlash:
        case Key_F8:
        case Key_Pad8:
            OnKeyPress ( 8 );
            break;

        case Key_9:
        case Key_Caccent:
        case Key_InversedV:
        case Key_F9:
        case Key_Pad9:
        case Key_Caps9:
            OnKeyPress ( 9 );
            break;

        case Key_Numeral:
            clearMessage();
            validate();
            break;

        case Key_BS:
        case Key_Asterisk:
            clearMessage();
            serialEntry->set_text ( "" );
            break;

        case Key_ESC:
        case Key_DEL:
        case Key_space:
        case Key_ENTER:
        case Key_Dollar:
        case Key_Percent:
        case Key_Plus:
        case Key_Comma:
        case Key_Point:
        case Key_Slash:
        case Key_TwoPoints:
        case Key_PointComma:
        case Key_Equal:
        case Key_Question:
        case Key_A:
        case Key_B:
        case Key_C:
        case Key_D:
        case Key_E:
        case Key_F:
        case Key_G:
        case Key_H:
        case Key_I:
        case Key_J:
        case Key_K:
        case Key_L:
        case Key_M:
        case Key_N:
        case Key_O:
        case Key_P:
        case Key_Q:
        case Key_R:
        case Key_S:
        case Key_T:
        case Key_U:
        case Key_V:
        case Key_W:
        case Key_X:
        case Key_Y:
        case Key_Z:
        case Key_a:
        case Key_b:
        case Key_c:
        case Key_d:
        case Key_e:
        case Key_f:
        case Key_g:
        case Key_h:
        case Key_i:
        case Key_j:
        case Key_k:
        case Key_l:
        case Key_m:
        case Key_n:
        case Key_o:
        case Key_p:
        case Key_q:
        case Key_r:
        case Key_s:
        case Key_t:
        case Key_u:
        case Key_v:
        case Key_w:
        case Key_x:
        case Key_y:
        case Key_z:
        case Key_PadAsterisk:
        case Key_PadPlus:
        case Key_PadMinus:
        case Key_Invalid:
        default:
            break;
    }
}

void SerialNumberWidget::OnKeyPress ( int data )
{
    if ( !tableText->is_visible() )
    {
        return;
    }

    clearMessage();

    if ( data == kConfirma )
    {
        validate();
    }
    else if ( data == kCorrige )
    {
        serialEntry->set_text ( "" );
    }
    else
    {
        std::string text = serialEntry->get_text();
        text.append ( Glib::Ascii::dtostr ( data ) );
        serialEntry->set_text ( text );
    }
}

void SerialNumberWidget::OnKeyPressT9 ( char data )
{
}

void SerialNumberWidget::validate()
{
    if ( serialEntry->validate() )
    {
        emit();
    }
}

void SerialNumberWidget::cancel()
{
    serialEntry->set_text ( "" );
    on_stop_signal_emit().emit();
}

void SerialNumberWidget::changeOption ( SmartmaticRadioButton * radioButton )
{
    if ( radioButton )
    {
        std::vector < std::string > vector;
        StringUtils::splitToVector ( radioButton->getAttribute(), "##", vector );

        if ( vector.size() >= 3 )
        {
            currentMax = StringUtils::stringToInt ( vector[0] );
            currentMask = vector[1];
            currentRegex = vector[2];
            currentPrefix = radioButton->getCode();

            serialEntry->setMax ( currentMax );
            serialEntry->setMask ( currentMask );
            serialEntry->setRegex ( currentRegex );
            serialEntry->set_text ( "" );
            serialText->set_text ( currentPrefix );
        }
        else
        {
            serialEntry->setMax ( radioButton->getCode().size() );
            serialEntry->setMask ( "" );
            serialEntry->setRegex ( "" );
            serialEntry->set_text ( radioButton->getCode() );
            currentPrefix.clear();
        }

        tableText->set_visible ( vector.size() >= 3 );
    }

    serialEntry->focus();
}
