/**
 * @file	SmartcardActionsWidget.hxx
 * @brief	Declares the smartcard reader diagnosis widget class.
 * @author	Yadickson Soto
 * @date	12/09/2013
 */

#ifndef _SMART_CARD_ACTIONS_WIDGET_H_
#define _SMART_CARD_ACTIONS_WIDGET_H_

#include <Gui/Widgets/Diagnose/BasicDiagnosticWidget.hxx>
#include <Gui/Widgets/ProgressWidget.hxx>
#include <Operation/Devices/Polling/CardPolling.hxx>
#include <System/SafeThread.hxx>
#include <System/SafeDispatcher.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }
    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                /**
                 * @class	SmartcardActionsWidget
                 * @brief	Smart card reader diagnosis widget.
                 * @author	Yadickson Soto
                 * @date	12/09/2013
                 */
                class SmartcardActionsWidget: public BasicDiagnosticWidget, private Smartmatic::System::SafeThread
                {
                    public:

                        /**
                         * @fn	SmartcardActionsWidget::SmartcardActionsWidget ( Smartmatic::SAES::GUI::Widgets::ProgressWidget * progressWidget );
                         * @brief	Constructor.
                         * @author	Yadickson Soto
                         * @date	12/09/2013
                         * @param	progressWidget visual container
                         */
                        explicit SmartcardActionsWidget ( Smartmatic::SAES::GUI::Widgets::ProgressWidget * progressWidget );

                        /**
                         * @fn	SmartcardActionsWidget::~SmartcardActionsWidget();
                         * @brief	Class destructor.
                         * @author	Yadickson Soto
                         * @date	12/09/2013
                         */
                        virtual ~SmartcardActionsWidget();

                        /**
                         * @fn	void SmartcardActionsWidget::dispose();
                         * @brief	Dispose of this object, cleaning up any resources it uses.
                         * @author	Yadickson Soto
                         * @date	12/09/2013
                         */
                        void dispose();

                        /**
                         * @fn	void SmartcardActionsWidget::initialize();
                         * @brief	Initializes this object.
                         * @author	Yadickson Soto
                         * @date	12/09/2013
                         */
                        void initialize();

                    private:

                        /**
                         * @enum SmartcardDiagnoseStep
                         * @brief Smartcard diagnose step definitions
                         * @author	Yadickson Soto
                         * @date	12/09/2013
                         */
                        enum SmartcardDiagnoseStep
                        {
                            INSERT, ///< Insert Card
                            WRITE, ///< Write the card
                            READ, ///< Read the card
                            REMOVE ///< Remove Card
                        };

                        int timeOk;   ///< The time diagnose successfully
                        int timeFail;   ///< The time diagnose fail
                        int timeStep;   ///< The time diagnose step
                        int timeRetry; ///< The waiting time before the retry (after retry button is pressed)
                        int numberOfRetries; ///< "numberOfRetries" is the number of retries set in the configuration
                        int currentRetries; ///< retry button is the only button enabled until the test is retried "numberOfRetries" number of times.

                        int currentStep; ///< current step
                        bool checkRemoveInError; ///< flag check remove device in error

                        std::string messageError; ///< The message error
                        std::string messageErrorInfo; ///< The message error info

                        sigc::connection connChangue;   ///< The connection change
                        Smartmatic::System::SafeDispatcher signalThread; ///< The signal end thread write or read process

                        Smartmatic::SAES::GUI::Widgets::ProgressWidget * progressWidget; ///< The visual container
                        static Smartmatic::Log::ISMTTLog * logger; ///< The logger

                    protected:

                        /**
                         * @fn	    virtual bool process ();
                         * @brief	Callback from thread one time. Write and read process
                         * @author	Yadickson Soto
                         * @date	03/08/2015
                         * @return always false
                         */
                        virtual bool process ();

                        /**
                         * @fn	    void onRawObject ( void * object );
                         * @brief	Callback when emit sync signal.
                         * @author	Yadickson Soto
                         * @date	03/08/2015
                         * @param 	object The object to process
                         */
                        void onRawObject ( void * object );

                        /**
                         * @fn	void nextStep();
                         * @brief	Wizard steps
                         * @author	Yadickson Soto
                         * @date	12/09/2013
                         */
                        void nextStep();

                        /**
                         * @fn	void inserted();
                         * @brief	Inserted smartcard reader step.
                         * @author	Yadickson Soto
                         * @date	12/09/2013
                         */
                        void insert();

                        /**
                         * @fn	void writeCardData();
                         * @brief	Write card data step.
                         * @author	Yadickson Soto
                         * @date	12/09/2013
                         */
                        void writeCardData();

                        /**
                         * @fn	void readCardData();
                         * @brief	Read card data step.
                         * @author	Yadickson Soto
                         * @date	12/09/2013
                         */
                        void readCardData();

                        /**
                         * @fn	void remove();
                         * @brief	Remove smartcard reader step.
                         * @author	Yadickson Soto
                         * @date	12/09/2013
                         */
                        void remove();

                        /**
                         * @fn	void success();
                         * @brief	Success write and read smartcard reader.
                         * @author	Yadickson Soto
                         * @date	12/09/2013
                         */
                        void success();

                        /**
                         * @fn	void viewMessageError ();
                         * @brief	View message error in step of diagnose.
                         * @author	Yadickson Soto
                         * @date	03/02/2014
                         */
                        void viewMessageError ();

                        /**
                         * @fn	void checkDevices ();
                         * @brief	View message error is card present.
                         * @author	Yadickson Soto
                         * @date	03/02/2014
                         */
                        void checkDevices();

                        /**
                         * @fn	void viewWaitRetry ();
                         * @brief	View message wait after click retry button.
                         * @author	Yadickson Soto
                         * @date	03/02/2014
                         */
                        void viewWaitRetry();

                        /**
                         * @fn	void viewError ();
                         * @brief	View end error in diagnose.
                         * @author	Yadickson Soto
                         * @date	03/02/2014
                         */
                        void viewError ();

                        /**
                         * @fn	void viewRetry ();
                         * @brief	View retry diagnose.
                         * @author	Yadickson Soto
                         * @date	12/09/2013
                         */
                        void viewRetry();

                        /**
                         * @fn	void error ();
                         * @brief	End with error in diagnose.
                         * @author	Yadickson Soto
                         * @date	12/09/2013
                         */
                        void error ();

                        /**
                         * @fn	void onCardPollingStatus ( Smartmatic::SAES::Operation::Devices::CardPolling::CardPollingStatus status );
                         * @brief	Callback status change smartcard reader.
                         * @author	Yadickson Soto
                         * @date	12/09/2013
                         * @param status report new status device
                         */
                        void onCardPollingStatus ( Smartmatic::SAES::Operation::Devices::CardPolling::CardPollingStatus status );
                };
            }
        }
    }
}

#endif // _SMART_CARD_ACTIONS_WIDGET_H_
