/**
 * @file	SmartcardPowerWidget.hxx
 * @brief	Declares the smartcard reader power OFF/ON diagnosis widget class.
 * @author	Yadickson Soto
 * @date	12/09/2013
 */

#ifndef _SMART_CARD_POWER_WIDGET_H_
#define _SMART_CARD_POWER_WIDGET_H_

#include <Widgets/wizard-widget-base.h>
#include <Gui/Widgets/ProgressWidget.hxx>
#include <System/SafeThread.hxx>
#include <System/SafeDispatcher.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }
    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                /**
                 * @class	SmartcardPowerWidget
                 * @brief	Smart card reader power OFF/ON diagnosis widget.
                 * @author	Yadickson Soto
                 * @date	12/09/2013
                 */
                class SmartcardPowerWidget: public Smartmatic::GUI::Widgets::WizardWidgetBase, private Smartmatic::System::SafeThread
                {
                    public:

                        /**
                         * @fn	SmartcardPowerWidget::SmartcardPowerWidget ( Smartmatic::SAES::GUI::Widgets::ProgressWidget * progressWidget );
                         * @brief	Constructor.
                         * @author	Yadickson Soto
                         * @date	12/09/2013
                         * @param 	progressWidget visual container
                         */
                        explicit SmartcardPowerWidget ( Smartmatic::SAES::GUI::Widgets::ProgressWidget * progressWidget );

                        /**
                         * @fn	SmartcardPowerWidget::~SmartcardPowerWidget();
                         * @brief	Class destructor.
                         * @author	Yadickson Soto
                         * @date	12/09/2013
                         */
                        virtual ~SmartcardPowerWidget();

                        /**
                         * @fn	void SmartcardPowerWidget::dispose();
                         * @brief	Dispose of this object, cleaning up any resources it uses.
                         * @author	Yadickson Soto
                         * @date	12/09/2013
                         */
                        void dispose();

                        /**
                         * @fn	void SmartcardPowerWidget::initialize();
                         * @brief	Initializes this object.
                         * @author	Yadickson Soto
                         * @date	12/09/2013
                         */
                        void initialize();

                    private:

                        /**
                         * @enum SmartcardDiagnoseStep
                         * @brief Smartcard diagnose step definitions
                         * @author	Yadickson Soto
                         * @date	12/09/2013
                         */
                        enum SmartcardPowerStep
                        {
                            OFF, ///< Power off smartcard reader
                            ON ///< Power on Card
                        };

                        int timeOk;   ///< The time diagnose successfully
                        int timeFail;   ///< The time diagnose fail
                        int timeStep;   ///< The time diagnose step

                        int currentStep; ///< current step
                        std::string messageError; ///< The message error diagnose

                        Smartmatic::System::SafeDispatcher signalThread; ///< The signal end thread for power on/off process

                        Smartmatic::SAES::GUI::Widgets::ProgressWidget * progressWidget; ///< The visual container
                        static Smartmatic::Log::ISMTTLog * logger; ///< The logger

                    protected:

                        /**
                         * @fn	    virtual bool process ();
                         * @brief	Callback from thread one time. Power on/off
                         * @author	Yadickson Soto
                         * @date	03/08/2015
                         * @return always false
                         */
                        virtual bool process ();

                        /**
                         * @fn	    void onRawObject ( void * object );
                         * @brief	Callback when emit sync signal.
                         * @author	Yadickson Soto
                         * @date	03/08/2015
                         * @param 	object The object to process
                         */
                        void onRawObject ( void * object );

                        /**
                         * @fn	void nextStep();
                         * @brief	Wizard steps
                         * @author	Yadickson Soto
                         * @date	12/09/2013
                         */
                        void nextStep();

                        /**
                         * @fn	void powerOff();
                         * @brief	Power OFF smartcard reader
                         * @author	Yadickson Soto
                         * @date	12/09/2013
                         */
                        void powerOff();

                        /**
                         * @fn	void powerOn();
                         * @brief	Power ON smartcard reader
                         * @author	Yadickson Soto
                         * @date	12/09/2013
                         */
                        void powerOn();

                        /**
                         * @fn	void error ();
                         * @brief	Error in diagnose.
                         * @author	Yadickson Soto
                         * @date	12/09/2013
                         */
                        void error ();

                        /**
                         * @fn	void success ();
                         * @brief	Success diagnose.
                         * @author	Yadickson Soto
                         * @date	12/09/2013
                         */
                        void success ();
                };
            }
        }
    }
}

#endif // _SMART_CARD_POWER_WIDGET_H_
