/**
 * @file HelpWidget.cxx
 * @brief Body help widget
 * @date 29/03/2012
 * @author Yadickson Soto
 */

#include "HelpWidget.hxx"
#include <Environment/Configurations/base-language-controller.h>
#include <Environment/Configurations/resource-path-provider.h>
#include "Configuration/s-a-e-s-g-u-i-conf-manager.h"
#include <Printing/PrinterStatusNotifier.h>
#include <System/IO/Path.h>
#include <System/IO/File.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include "AbstractFactoryWidget.hxx"

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::Environment;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Printing;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::GUI::Operation;

ISMTTLog * HelpWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.HelpWidget", GETTEXT_PACKAGE );

HelpWidget::HelpWidget()
    : StepWidget()
{
    this->genericPrinting = NULL;
    this->helpEventBox = NULL;
    this->mainVBox = NULL;
    this->helpVBox = NULL;
    this->image = NULL;
    this->mainVBox = NULL;
    this->separator = NULL;
    this->printHBox = NULL;
    this->printButton = NULL;
    this->printing = NULL;
    this->enableFilePrinting = true;

    initialize();
    setWidget ( helpEventBox );
}

HelpWidget::HelpWidget ( const std::string& titleKey, const std::string& image, const std::string& file, bool enableFilePrinting )
    : StepWidget()
{
    this->genericPrinting = NULL;
    this->helpEventBox = NULL;
    this->mainVBox = NULL;
    this->helpVBox = NULL;
    this->image = NULL;
    this->mainVBox = NULL;
    this->separator = NULL;
    this->printHBox = NULL;
    this->printButton = NULL;
    this->printing = NULL;

    initialize();

    setInfo ( titleKey, imageFile, file, enableFilePrinting );
    setWidget ( helpEventBox );
}

HelpWidget::~HelpWidget()
{
    dispose();
}

bool HelpWidget::isExit()
{
    return true;
}

void HelpWidget::initialize()
{
    dispose();

    genericPrinting = AbstractFactoryWidget::createGenericPrintingWidget();
    genericPrinting->getSignalEnd().connect ( sigc::mem_fun ( *this, &HelpWidget::endPrinting ) );
    genericPrinting->getSignalError().connect ( sigc::mem_fun ( *this, &HelpWidget::endPrinting ) );
    getSignalStart().connect( sigc::mem_fun ( *this, &HelpWidget::onStart ) );
    getSignalEnd().connect( sigc::mem_fun ( *this, &HelpWidget::onEnd ) );

    Glib::RefPtr<Gtk::Builder> builder;
    std::string gladeFile;
    gladeFile = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/HelpWidget.glade" );

    try
    {
        builder = Gtk::Builder::create_from_file ( gladeFile );
    }
    catch ( const Glib::FileError & ex )
    {
        logger->Debug ( "FATAL - HelpWidget Initialize" );
    }

    builder->get_widget ( "help-eventbox", helpEventBox );
    builder->get_widget ( "main-vbox", mainVBox );
    builder->get_widget ( "help-vbox", helpVBox );
    builder->get_widget ( "main-image", image );
    builder->get_widget ( "hseparator", separator );
    builder->get_widget ( "hbuttonbox", printHBox );
    builder->get_widget_derived ( "print-button", printButton );

    image->set_no_show_all();
    separator->set_no_show_all();
    printHBox->set_no_show_all();
    printButton->set_no_show_all();

    int fontSize = SAESGUIConfigurationManager::getInstance ()->getHelpWidgetConfiguration().getPrintButtonConf().getFontSize();
    int widthB = SAESGUIConfigurationManager::getInstance ()->getHelpWidgetConfiguration().getPrintButtonConf().getButtonWidth();
    int heightB = SAESGUIConfigurationManager::getInstance ()->getHelpWidgetConfiguration().getPrintButtonConf().getButtonHeight();

    printButton->ModifyFontSize ( fontSize );
    printButton->set_size_request ( widthB, heightB );
    printButton->signal_clicked().connect ( sigc::mem_fun ( *this, &HelpWidget::startPrintProcess ) );

    this->show_all_children();
}

void HelpWidget::dispose()
{
    this->remove();

    if ( helpEventBox )
    {
        delete helpEventBox;
        helpEventBox = NULL;
    }

    if ( mainVBox )
    {
        delete mainVBox;
        mainVBox = NULL;
    }

    if ( genericPrinting )
    {
        delete genericPrinting;
        genericPrinting = NULL;
    }

    if ( helpVBox )
    {
        delete helpVBox;
        helpVBox = NULL;
    }

    if ( image )
    {
        delete image;
        image = NULL;
    }

    if ( separator )
    {
        delete separator;
        separator = NULL;
    }

    if ( printHBox )
    {
        delete printHBox;
        printHBox = NULL;
    }

    if ( printButton )
    {
        delete printButton;
        printButton = NULL;
    }

    if ( printing )
    {
        delete printing;
        printing = NULL;
    }
}

void HelpWidget::setInfo ( const std::string& titleKey, const std::string& image, const std::string& file, bool enableFilePrinting )
{
    setTitleText ( titleKey );

    this->imageFile = image;
    this->file = file;
    this->enableFilePrinting = enableFilePrinting;

    refresh();
}

void HelpWidget::refresh()
{
    std::string toFile = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, file );
    std::string toImageFile = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, imageFile );

    if ( File::getInstance()->canOpenFile ( toImageFile ) )
    {
        image->clear();
        image->set ( toImageFile );
        image->show();

        printButton->SetLabel ( _ ( "Smartmatic.SAES.GUI.Windows.HelpWindow.PrintButton" ) );
    }
    else
    {
        image->hide();
    }

    enableFilePrinting = enableFilePrinting && File::getInstance()->canOpenFile ( toFile );

    separator->set_visible ( enableFilePrinting );
    printHBox->set_visible ( enableFilePrinting );
    printButton->set_visible ( enableFilePrinting );
}

void HelpWidget::startPrintProcess()
{
    printButton->set_sensitive ( false );
    signalStartPrint.emit();

    setWidget ( genericPrinting );

    if ( printing )
    {
        delete printing;
        printing = NULL;
    }

    std::string toFile = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, file );
    printing = new HelpPrinting ( toFile );

    genericPrinting->setInfoTitle ( _ ( "Smartmatic.SAES.GUI.Windows.HelpWindow.TitleHelpPrinting" ) );
    genericPrinting->setInfoText ( _ ( "Smartmatic.SAES.GUI.Windows.HelpWindow.TextPrinting" ) );
    genericPrinting->start ( printing, false, false );
}

void HelpWidget::endPrinting()
{
    setWidget ( helpEventBox );
    printButton->set_sensitive ( true );
    signalEndPrint.emit();
}

void HelpWidget::onStart()
{
    signalShown.emit();
}

void HelpWidget::onEnd()
{
    signalClosed.emit();
}

sigc::signal<void> HelpWidget::getSignalStartPrint()
{
    return signalStartPrint;
}

sigc::signal<void> HelpWidget::getSignalEndPrint()
{
    return signalEndPrint;
}

sigc::signal<void> HelpWidget::getSignalShown()
{
    return signalShown;
}

sigc::signal<void> HelpWidget::getSignalClosed()
{
    return signalClosed;
}
