/**
 * @file	LogPagingTreeViewWidget.cc
 * @brief	log paging tree view widget class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#include "LogPagingTreeViewWidget.h"
#include <Configuration/MachineModelEnum.hxx>
#include <Configuration/s-m-t-t-g-u-i-conf-manager.h>
#include <Environment/Configurations/resource-path-provider.h>
#include <Log/LogLine.h>
#include <glibmm.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

#define BUTTON_WIDTH  120
#define BUTTON_HEIGHT 50

using namespace Smartmatic::Environment;
using namespace Smartmatic::SAES::Log;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::Configuration;

LogPagingTreeViewWidget::LogPagingTreeViewWidget ( LogReader * logReader, bool enablePrint )
{
    this->logReader = logReader;
    this->enablePrint = enablePrint;

    m_VTable = NULL;
    vBoxRight = NULL;
    m_ButtonBoxNav = NULL;
    m_LabelPage = NULL;
    buttonBack = NULL;
    buttonNext = NULL;
    buttonPrint = NULL;
    eventboxLog = NULL;
    LoadGUI();
}

LogPagingTreeViewWidget::~LogPagingTreeViewWidget()
{
    if ( m_VTable )
    {
        delete m_VTable;
        m_VTable = NULL;
    }

    if ( eventboxLog )
    {
        delete eventboxLog;
        eventboxLog = NULL;
    }

    if ( vBoxRight )
    {
        delete vBoxRight;
        vBoxRight = NULL;
    }

    if ( m_ButtonBoxNav )
    {
        delete m_ButtonBoxNav;
        m_ButtonBoxNav = NULL;
    }

    if ( m_LabelPage )
    {
        delete m_LabelPage;
        m_LabelPage = NULL;
    }

    if ( buttonBack )
    {
        delete buttonBack;
        buttonBack = NULL;
    }

    if ( buttonNext )
    {
        delete buttonNext;
        buttonNext = NULL;
    }

    if ( buttonPrint )
    {
        delete buttonPrint;
        buttonPrint = NULL;
    }
}

void LogPagingTreeViewWidget::on_buttonPrint_clicked()
{
    signalPrint_detected.emit();
}

void LogPagingTreeViewWidget::LoadGUI ()
{
    Glib::RefPtr<Gtk::Builder> widget_builder =
        Gtk::Builder::create_from_file ( ResourcePathProvider::getInstance()->
                                         GetResourcePath ( PACKAGE_NAME, "ui/PagingTreeViewWidget.glade" ) );
    widget_builder->get_widget ( "m_VTable", m_VTable );
    widget_builder->get_widget ( "eventbox1", eventboxLog );
    widget_builder->get_widget ( "vbox-right", vBoxRight );
    widget_builder->get_widget ( "m_ButtonBoxNav", m_ButtonBoxNav );
    widget_builder->get_widget ( "m_LabelPage", m_LabelPage );
    widget_builder->get_widget_derived ( "button-back", buttonBack );
    widget_builder->get_widget_derived ( "button-next", buttonNext );
    widget_builder->get_widget_derived ( "button-print", buttonPrint );

    int buttonWidth  = BUTTON_WIDTH;
    int buttonHeight = BUTTON_HEIGHT;

    if ( SMTTGUIConfigurationManager::getInstance()->getSmartmaticGUIMachineModelConfiguration().getId() == EnumToString( PM_VIU805 ) )
    {
        double guiReconfigurationFactor = SMTTGUIConfigurationManager::getInstance()->getConfiguration()->getSmartmaticGUIMachineModelConfiguration().getGuiReconfigurationFactor();

        buttonWidth *= guiReconfigurationFactor;
        buttonHeight *= guiReconfigurationFactor;
    }

    buttonPrint->set_no_show_all();
    buttonPrint->set_visible ( enablePrint );

    buttonPrint->set_sensitive ( true );
    buttonPrint->WidgetsPosition ( 5, 0, 0 );
    buttonPrint->SetLabel ( _ ( "Smartmatic.SAES.GUI.Widgets.PagingTreeViewWidget.Print" ) );
    buttonPrint->set_size_request( buttonWidth, buttonHeight );
    buttonPrint->signal_clicked().connect ( sigc::mem_fun ( this, &LogPagingTreeViewWidget::on_buttonPrint_clicked ) );

    this->logTreeViewWidget = new LogTreeViewWidget();
    eventboxLog->add ( *logTreeViewWidget );
    LoadPage ( 1 );
    std::stringstream ss;
    ss << 1 << " / " << this->logReader->getPageCount();
    m_LabelPage->set_text ( ss.str() );
    buttonBack->set_sensitive ( false );
    buttonBack->SetNavigationKey ( Smartmatic::GUI::Widgets::Key_Pad1 );
    buttonBack->signal_clicked().connect ( sigc::mem_fun ( this, &LogPagingTreeViewWidget::on_buttonBack_clicked ) );
    //buttonBack->set_size_request (100,50);
    buttonBack->SetButtonType ( SmartmaticButton::ArrowLeft );
    buttonBack->WidgetsPosition ( 0, 5, 0 );
    buttonBack->set_size_request( buttonWidth, buttonHeight );

    if ( this->logReader->getPageCount() > 1 )
    {
        buttonNext->set_sensitive ( true );
    }
    else
    {
        buttonNext->set_sensitive ( false );
    }

    buttonNext->SetNavigationKey ( Smartmatic::GUI::Widgets::Key_Pad2 );
    buttonNext->signal_clicked().connect ( sigc::mem_fun ( this, &LogPagingTreeViewWidget::on_buttonNext_clicked ) );
    //buttonNext->set_size_request (100,50);
    buttonNext->SetButtonType ( SmartmaticButton::ArrowRight );
    buttonNext->WidgetsPosition ( 0, 5, 0 );
    buttonNext->set_size_request( buttonWidth, buttonHeight );
    this->add ( *m_VTable );
    this->show_all_children ( true );
}

void LogPagingTreeViewWidget::LoadPage ( unsigned int page )
{
    this->logTreeViewWidget->CleanAllRows();
    this->logPage = this->logReader->getPage ( page );
    std::vector<LogLine> lines = this->logPage.getPage();

    for ( unsigned int i = 0; i < lines.size(); i++ )
    {
        this->logTreeViewWidget->AddRow ( lines[i] );
    }

    std::stringstream ss;
    ss << page << " / " << this->logReader->getPageCount();
    m_LabelPage->set_text ( ss.str() );
}

void LogPagingTreeViewWidget::on_buttonBack_clicked()
{
    unsigned int page = this->logPage.getPageNumber();
    unsigned int pageTotal = this->logReader->getPageCount();

    if ( page > 1 )
    {
        LoadPage ( page - 1 );
    }

    if ( page - 1 == 1 )
    {
        buttonBack->set_sensitive ( false );
    }
    else
    {
        buttonBack->set_sensitive ( true );
    }

    if ( page - 1 == pageTotal )
    {
        buttonNext->set_sensitive ( false );
    }
    else
    {
        buttonNext->set_sensitive ( true );
    }
}

void LogPagingTreeViewWidget::SetPrintButtonSensitive ( bool sensitive )
{
    buttonPrint->set_sensitive ( sensitive );
}

void LogPagingTreeViewWidget::on_buttonNext_clicked()
{
    unsigned int page = this->logPage.getPageNumber();
    unsigned int pageTotal = this->logReader->getPageCount();

    if ( page < pageTotal )
    {
        LoadPage ( page + 1 );
    }

    if ( page + 1 == pageTotal )
    {
        buttonNext->set_sensitive ( false );
    }
    else
    {
        buttonNext->set_sensitive ( true );
    }

    if ( page + 1 == 1 )
    {
        buttonBack->set_sensitive ( false );
    }
    else
    {
        buttonBack->set_sensitive ( true );
    }
}
