/**
 * @file MemoryStateWidget.hxx
 * @brief Header class memory state widget
 * @date 29/03/2012
 * @author Yadickson Soto
 */

#ifndef MEMORYSTATEWIDGET_HXX_
#define MEMORYSTATEWIDGET_HXX_

#include <gtkmm.h>
#include <string>
#include <Widgets/smartmatic-button.h>
#include <Widgets/SmartmaticBasicEventBox.hxx>
#include <Widgets/SmartmaticBasicVBox.hxx>
#include <Widgets/SmartmaticBasicHBox.hxx>
#include <Widgets/StepWidget.hxx>
#include "ProgressWidget.hxx"

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                /**
                 * @class	MemoryStateWidget
                 * @brief	Class memory state widget.
                 * @author	Yadickson Soto
                 * @date	30/03/2012
                 */
                class MemoryStateWidget : public Smartmatic::GUI::Widgets::StepWidget
                {
                    public:
                        /**
                         * @fn	MemoryStateWidget::MemoryStateWidget ( ProgressWidget * progress );
                         * @brief	Class constructor.
                         * @author	Yadickson Soto
                         * @date	30/03/2012
                         * @param	progress	progress widget.
                         */
                        explicit MemoryStateWidget ( ProgressWidget * progress );
                        /**
                         * @fn	virtual MemoryStateWidget::~MemoryStateWidget();
                         * @brief	Class destructor.
                         * @author	Yadickson Soto
                         * @date	30/03/2012
                         */
                        virtual ~MemoryStateWidget();
                        /**
                         * @fn	virtual void MemoryStateWidget::refresh();
                         * @brief	refresh widget method.
                         * @author	Yadickson Soto
                         * @date	30/03/2012
                         */
                        virtual void refresh();
                        /**
                         * @fn	virtual std::string MemoryStateWidget::getTitleText();
                         * @brief	virtual method getter title.
                         * @author	Yadickson Soto
                         * @date	30/03/2012
                         * @return	title.
                         */
                        virtual std::string getTitleText();
                        /**
                         * @fn	virtual void showSynchronize();
                         * @brief	Show synchronize process message, call thread checkEndSyncDevice
                         * @author	Yadickson Soto
                         * @date	11/06/2015
                         */
                        virtual void showSynchronize();
                        /**
                         * @fn	virtual void showOneMemory();
                         * @brief	Show message if one memory is removed
                         * @author	Yadickson Soto
                         * @date	11/06/2015
                         */
                        virtual void showOneMemory();
                        /**
                         * @fn	virtual void showAllMemoryRemoved();
                         * @brief	Show message if all memory is removed, the application is stopped
                         * @author	Yadickson Soto
                         * @date	11/06/2015
                         */
                        virtual void showAllMemoryRemoved();
                        /**
                         * @fn	virtual void showVerifyTwoMemory();
                         * @brief	Show verification message, call thread checkEndDataDevice
                         * @author	Yadickson Soto
                         * @date	11/06/2015
                         */
                        virtual void showVerifyTwoMemory();
                        /**
                         * @fn	virtual void showSafeEject();
                         * @brief	Show safe eject message and cancel button.
                         * @author	Yadickson Soto
                         * @date	18/06/2013
                         */
                        virtual void showSafeEject();
                        /**
                         * @fn	sigc::signal<void> getCancelSafeEject();
                         * @brief	method getter signal cancel safe eject.
                         * @author	Yadickson Soto
                         * @date	30/07/2015
                         * @return	signal cancel safe eject.
                         */
                        sigc::signal<void> getCancelSafeEject();
                        /**
                         * @fn	virtual void showMemoryCheckError();
                         * @brief	Show memory verification error.
                         * @author	Yadickson Soto
                         * @date	10/06/2015
                         */
                        virtual void showMemoryCheckError();

                        /**
                         * @brief  Shows memory verification error when we have votes.
                         * @author manuel.arguelles
                         * @date   01/06/2022
                         */
                        virtual void showMemoryCheckErrorToRecount();

                        /**
                         * @brief  Shows return memory warning.
                         * @author manuel.arguelles
                         * @date   26/05/2022
                         */
                        void showReturnMemory();

                        /**
                         * @brief  Shows return both memories warning.
                         * @author manuel.arguelles
                         * @date   26/05/2022
                         */
                        void showReturnBothMemories();

                    protected:

                        /**
                         * @fn	virtual void MemoryStateWidget::initialize();
                         * @brief	Initializer method.
                         * @author	Yadickson Soto
                         * @date	08/03/2012
                         */
                        virtual void initialize();
                        /**
                         * @fn	virtual void MemoryStateWidget::dispose();
                         * @brief	virtual method stop.
                         * @author	Yadickson Soto
                         * @date	08/03/2012
                         */
                        virtual void dispose();

                        /**
                         * @fn	void MemoryStateWidget::configureWidget ( bool showHideButton, bool buttonClose, bool showSafeEjectButton = false );
                         * @brief	Configure widget.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @param	showHideButton	true to show, false to hide the hide button.
                         * @param	buttonClose			true to button close.
                         * @param	showSafeEjectButton show safe eject button
                         */
                        void configureWidget ( bool showHideButton, bool buttonClose, bool showSafeEjectButton = false );
                        /**
                         * @fn	void MemoryStateWidget::enabledButton();
                         * @brief	Enabled button.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void enabledButton();
                        /**
                         * @fn	void MemoryStateWidget::shutDown();
                         * @brief	Shuts down this object and frees any resources it is using.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         */
                        void shutDown();

                        /**
                         * @fn	bool MemoryStateWidget::checkPrinter();
                         * @brief	Determines if we can check printer.
                         * @author	Youssef.tourkmani
                         * @date	01/11/2012
                         * @return	true if it succeeds, false if it fails.
                         */
                        bool checkPrinter();
                        /**
                         * @fn	virtual void cancelSafeEject();
                         * @brief	Callback cancel safe eject signal button.
                         * @author	Yadickson Soto
                         * @date	18/06/2013
                         */
                        virtual void cancelSafeEject();
                        /**
                         * @fn	virtual void showMemoryError();
                         * @brief	Show memory error.
                         * @author	Yadickson Soto
                         * @date	10/06/2015
                         */
                        virtual void showMemoryError();

                    private:

                        std::string labelKey; ///< The label key
                        std::string image;	///< The image

                        Smartmatic::GUI::Widgets::SmartmaticBasicEventBox * mainEventBox; ///< The main event box
                        Smartmatic::GUI::Widgets::SmartmaticBasicEventBox * progressEventBox; ///< The progress event box
                        Smartmatic::GUI::Widgets::SmartmaticBasicVBox * memoryVBox; ///< The memory v box
                        Smartmatic::GUI::Widgets::SmartmaticBasicHBox * shutdownHBox; ///< The shutdown h box
                        Smartmatic::GUI::Widgets::SmartmaticButton * shutdownButton;	///< The shutdown button
                        Smartmatic::GUI::Widgets::SmartmaticButton * cancelSafeEjectButton;	///< The cancel safe eject button

                        sigc::connection connectButton; ///< The connect button
                        sigc::signal<void> signalCancel; ///< The signal cancel safe eject

                        ProgressWidget * progress;	///< The progress
                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger
                };
            }
        }
    }
}

#endif /* MEMORYSTATEWIDGET_HXX_ */
