/**
 * @file menu-widgetmm.h
 * @brief Header menu widget class
 * @date 29/10/2012
 * @author Juan Delgado, Yadickson Soto
 */

#ifndef _MENU_WIDGETMM_H_
#define _MENU_WIDGETMM_H_

#include <map>
#include <gdk/gdk.h>
#include "Widgets/smartmatic-button.h"
#include "Widgets/keyword-navigation-interface.h"
#include <Gui/Widgets/Menu/menu-option.h>
#include <Widgets/SmartmaticBasicTable.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {


                /**
                 * @class	OptionsList
                 * @brief	List of options.
                 * @author	Youssef.tourkmani
                 * @date	25/10/2012
                 */
                class OptionsList
                {
                    public:

                        /**
                         * @fn	OptionsList::OptionsList();
                         * @brief	Default constructor.
                         * @author	Luiggi.mendez
                         * @date	27/04/2011
                         */

                        OptionsList();

                        /**
                         * @fn	virtual OptionsList::~OptionsList();
                         * @brief	Finaliser.
                         * @author	Luiggi.mendez
                         * @date	27/04/2011
                         */
                        virtual ~OptionsList();

                        /**
                         * @fn	OptionsList::OptionsList( const std::string& name );
                         * @brief	Constructor.
                         * @author	Luiggi.mendez
                         * @date	27/04/2011
                         * @param	name	The name.
                         */
                        explicit OptionsList ( const std::string& name );

                        /**
                         * @fn	void OptionsList::Add(MenuOption & option);
                         * @brief	Adds option..
                         * @author	Luiggi.mendez
                         * @date	27/04/2011
                         * @param	option	The MenuOption to add.
                         */
                        void Add ( MenuOption & option );

                        /**
                         * @fn	void OptionsList::Remove(MenuOption & option);
                         * @brief	Removes the given option.
                         * @author	Luiggi.mendez
                         * @date	27/04/2011
                         * @param	option	The MenuOption to remove.
                         */
                        void Remove ( MenuOption & option );

                        std::string name;   ///< The name
                        std::vector<MenuOption> options;	///< Options for controlling the operation

                };

                /**
                 * @enum	OptionsType
                 * @brief	Values that represent OptionsType.
                 */

                enum OptionsType
                {
                    FirstLevel, ///< Value that represent firstlevel type.
                    SecondLevel ///< Value that represent secondlevel type.
                };


                /**
                 * @class	OptionByKeyItem
                 * @brief	Option by key item.
                 * @author	Youssef.tourkmani
                 * @date	25/10/2012
                 */
                class OptionByKeyItem
                {
                    public:

                        /**
                         * @fn	OptionByKeyItem::OptionByKeyItem();
                         * @brief	Default constructor.
                         * @author	Luiggi.mendez
                         * @date	27/04/2011
                         */
                        OptionByKeyItem();

                        /**
                         * @fn	OptionByKeyItem::OptionByKeyItem(int _keyCode, const std::string& keyName, MenuOption _option);
                         * @brief	Constructor.
                         * @author	Luiggi.mendez
                         * @date	27/04/2011
                         * @param	_keyCode	The key code.
                         * @param	keyName 	Name of the key.
                         * @param	_option 	The option.
                         */
                        OptionByKeyItem ( int _keyCode, const std::string& keyName, MenuOption _option );

                        /**
                         * @fn	virtual OptionByKeyItem::~OptionByKeyItem();
                         * @brief	Finaliser.
                         * @author	Luiggi.mendez
                         * @date	27/04/2011
                         */
                        virtual ~OptionByKeyItem();
                        int keyCode;	///< The key code
                        std::string keyName;	///< Name of the key
                        MenuOption option;  ///< The option

                        /**
                         * @fn	void OptionByKeyItem::Set(int _keyCode, const std::string& keyName, MenuOption _option);
                         * @brief	Sets.
                         * @author	Luiggi.mendez
                         * @date	27/04/2011
                         * @param	_keyCode	The key code.
                         * @param	keyName 	Name of the key.
                         * @param	_option 	The option.
                         */
                        void Set ( int _keyCode, const std::string& keyName, MenuOption _option );
                };


                /**
                 * @class	OptionByKeyCollection
                 * @brief	Collection of option by keys.
                 * @author	Youssef.tourkmani
                 * @date	25/10/2012
                 */
                class OptionByKeyCollection
                {
                    public:

                        /**
                         * @fn	OptionByKeyCollection::OptionByKeyCollection();
                         * @brief	Default constructor.
                         * @author	Luiggi.mendez
                         * @date	27/04/2011
                         */

                        OptionByKeyCollection();

                        /**
                         * @fn	OptionByKeyCollection::OptionByKeyCollection( const OptionByKeyItem& option );
                         * @brief	Constructor.
                         * @author	Luiggi.mendez
                         * @date	27/04/2011
                         * @param	option	The option.
                         */
                        explicit OptionByKeyCollection ( const OptionByKeyItem& option );

                        /**
                         * @fn	virtual OptionByKeyCollection::~OptionByKeyCollection();
                         * @brief	Finaliser.
                         * @author	Luiggi.mendez
                         * @date	27/04/2011
                         */
                        virtual ~OptionByKeyCollection();
                        std::vector<OptionByKeyItem> options;   ///< Options for controlling the operation

                        /**
                         * @fn	void OptionByKeyCollection::Add(OptionByKeyItem option);
                         * @brief	Adds option..
                         * @author	Luiggi.mendez
                         * @date	27/04/2011
                         * @param	option	The OptionByKeyItem to add.
                         */
                        void Add ( OptionByKeyItem option );

                        /**
                         * @fn	void OptionByKeyCollection::Remove( const OptionByKeyItem& option );
                         * @brief	Removes the given option.
                         * @author	Luiggi.mendez
                         * @date	27/04/2011
                         * @param	option	The OptionByKeyItem to remove.
                         */
                        void Remove ( const OptionByKeyItem& option );
                };


                /**
                 * @class	MenuWidgetmm
                 * @brief	Menu widgetmm.
                 * @author	Youssef.tourkmani
                 * @date	25/10/2012
                 */
                class MenuWidgetmm : public Smartmatic::GUI::Widgets::SmartmaticBasicTable,
                    public Smartmatic::GUI::Widgets::keywordNavigationInterface
                {

                    public:

                        /**
                         * @fn	MenuWidgetmm::MenuWidgetmm();
                         * @brief	Default constructor.
                         * @author	Luiggi.mendez
                         * @date	27/04/2011
                         */
                        MenuWidgetmm();

                        /**
                         * @fn	MenuWidgetmm::MenuWidgetmm( const std::vector<MenuOption>& _firstLevelOptions, const std::map<std::string, OptionsList>& _secondLevelOptions );
                         * @brief	Constructor.
                         * @author	Luiggi.mendez
                         * @date	27/04/2011
                         * @param	_firstLevelOptions 	Options for controlling the first level.
                         * @param	_secondLevelOptions	Options for controlling the second level.
                         */
                        MenuWidgetmm ( const std::vector<MenuOption>& _firstLevelOptions, const std::map<std::string, OptionsList>& _secondLevelOptions );

                        /**
                         * @fn	virtual MenuWidgetmm::~MenuWidgetmm();
                         * @brief	Finaliser.
                         * @author	Luiggi.mendez
                         * @date	27/04/2011
                         */
                        virtual ~MenuWidgetmm();

                        /**
                         * @fn	GtkType MenuWidgetmm::gtk_saesmenu_get_type(void);
                         * @brief	Gets the gtk saesmenu get type.
                         * @author	Luiggi.mendez
                         * @date	27/04/2011
                         * @return	GtkType.
                         */
                        GtkType gtk_saesmenu_get_type ( void );

                        /**
                         * @fn	GtkWidget* MenuWidgetmm::saesmenu_new(void);
                         * @brief	Gets the saesmenu new.
                         * @author	Luiggi.mendez
                         * @date	27/04/2011
                         * @return	null if it fails, else a GtkWidget.
                         */
                        GtkWidget * saesmenu_new ( void );

                        /**
                         * @fn	void MenuWidgetmm::gtk_init_menu(std::vector<MenuOption> _firstLevelOptions,
                         * 			std::map<std::string, OptionsList> _secondLevelOptions);
                         * @brief	Gtk initialise menu.
                         * @author	Luiggi.mendez
                         * @date	27/04/2011
                         * @param	_firstLevelOptions 	Options for controlling the first level.
                         * @param	_secondLevelOptions	Options for controlling the second level.
                         */

                        void gtk_init_menu ( std::vector<MenuOption>   _firstLevelOptions, std::map<std::string, OptionsList> _secondLevelOptions );

                        /**
                         * @fn	void MenuWidgetmm::SetMenuLabel(Gtk::Label* title_label);
                         * @brief	Sets a menu label.
                         * @author	Luiggi.mendez
                         * @date	27/04/2011
                         * @param	title_label	If non-null, the title label.
                         */
                        void SetMenuLabel ( Gtk::Label * title_label );

                        /**
                         * @fn	virtual void MenuWidgetmm::SetKey(KeysActionsCode value);
                         * @brief	Sets a key.
                         * @author	Luiggi.mendez
                         * @date	27/04/2011
                         * @param	value	The value.
                         */
                        virtual void SetKey ( Smartmatic::GUI::Widgets::KeysActionsCode value );

                        /**
                         * @fn	void MenuWidgetmm::Dispose();
                         * @brief	Dispose of this object, cleaning up any resources it uses.
                         * @author	Luiggi.mendez
                         * @date	27/04/2011
                         */
                        void Dispose();

                        /**
                         * @fn	void MenuWidgetmm::SetPrincipalFrameTitle( const std::string& title );
                         * @brief	Sets a principal frame title.
                         * @author	Luiggi.mendez
                         * @date	27/04/2011
                         * @param	title	The title.
                         */
                        void SetPrincipalFrameTitle ( const std::string& title );

                        /**
                         * @fn	void MenuWidgetmm::SetSecondLevelTitle( const std::string& title );
                         * @brief	Sets a second level title.
                         * @author	Luiggi.mendez
                         * @date	27/04/2011
                         * @param	title	The title.
                         */
                        void SetSecondLevelTitle ( const std::string& title );

                    protected:

                        std::map<Smartmatic::GUI::Widgets::SmartmaticButton *, MenuOption> optionsBybuttonList; ///< List of options by correspond buttons
                        std::map<std::string, Gtk::ButtonBox *> hBoxByOption;	///< List of options agrupped for second level

                        std::vector<MenuOption> firstLeveloptions;  ///< List of options agrupped for first level
                        std::vector<Gtk::ButtonBox *> buttonBoxes;  ///< List of button boxes
                        std::map<std::string, OptionsList> secondLevelOptions;  ///< Options for controlling the second level sorted by their names
                        bool withOnlyOptions;   ///< true to with only options
                        std::vector<MenuOption> initOptions;	///< Options for controlling the initialise
                        std::map<std::string, OptionsList> secondLevelInitOptions;  ///< Options for controlling the second level initialise
                        std::map<OptionsType, OptionByKeyCollection> buttonsByKey;  ///< The buttons by key
                        Gtk::Label * main_label; ///< The main label
                        MenuOption currentFirstLevelMenuOption; ///< The current first level menu option

                        /**
                         * @fn	void MenuWidgetmm::menu_init();
                         * @brief	Menu initialise.
                         * @author	Luiggi.mendez
                         * @date	27/04/2011
                         */
                        void menu_init();

                        /**
                         * @fn	void MenuWidgetmm::BuildChildren( const std::string& fatherName, std::vector<MenuOption> options );
                         * @brief	Builds a children.
                         * @author	Luiggi.mendez
                         * @date	27/04/2011
                         * @param	fatherName	Name of the father.
                         * @param	options			Options for controlling the operation.
                         */
                        void BuildChildren ( const std::string& fatherName, std::vector<MenuOption> options );

                        /**
                         * @fn	void MenuWidgetmm::BuildFirstLevelOptions();
                         * @brief	Builds the first level options.
                         * @author	Luiggi.mendez
                         * @date	27/04/2011
                         */
                        void BuildFirstLevelOptions();

                        /**
                         * @fn	OptionsList MenuWidgetmm::GetSecondInitOptionList(std::string optionNameKey,
                         * 			MenuOption & option);
                         * @brief	Gets a second initialise option list.
                         * @author	Luiggi.mendez
                         * @date	27/04/2011
                         * @param	optionNameKey	The option name key.
                         * @param	option			 	The option.
                         * @return	The second initialise option list.
                         */
                        OptionsList GetSecondInitOptionList ( std::string optionNameKey, MenuOption & option );

                        /**
                         * @fn	OptionsList MenuWidgetmm::GetOptionList(std::string optionNameKey, MenuOption & option);
                         * @brief	Gets an option list.
                         * @author	Luiggi.mendez
                         * @date	27/04/2011
                         * @param	optionNameKey	The option name key.
                         * @param	option			 	The option.
                         * @return	The option list.
                         */
                        OptionsList GetOptionList ( std::string optionNameKey, MenuOption & option );

                        /**
                         * @fn	Gtk::ButtonBox MenuWidgetmm::*create_bbox( gint horizontal, gint spacing,
                         * 			Gtk::ButtonBoxStyle layout, gint buttonWidth, gint buttonHeight,
                         * 			std::vector<MenuOption> options, int boxBorderWidth, bool useImage);
                         * @brief	Creates a bounding box.
                         * @author	Luiggi.mendez
                         * @date	27/04/2011
                         * @param	horizontal			The horizontal.
                         * @param	spacing					The spacing.
                         * @param	layout					The layout.
                         * @param	buttonWidth			Width of the button.
                         * @param	buttonHeight		Height of the button.
                         * @param	options					Options for controlling the operation.
                         * @param	boxBorderWidth	Width of the box border.
                         * @param	useImage				true to use image.
                         * @return	null if it fails, else.
                         */
                        Gtk::ButtonBox * create_bbox ( gint  horizontal,
                                                       gint  spacing/*espacio entre los botones del box de botones*/,
                                                       Gtk::ButtonBoxStyle  layout/*layout del button box*/,
                                                       gint buttonWidth/*width de los botones*/,
                                                       gint buttonHeight, std::vector<MenuOption> options,
                                                       int boxBorderWidth,
                                                       bool useImage );

                        /**
                         * @fn	bool MenuWidgetmm::CurrentLevelContainThisSecondOption(MenuOption currentOption,
                         * 			int secondOptionCode);
                         * @brief	Current level contain this second option.
                         * @author	Luiggi.mendez
                         * @date	27/04/2011
                         * @param	currentOption			The current option.
                         * @param	secondOptionCode	The second option code.
                         * @return	true if it succeeds, false if it fails.
                         */
                        bool CurrentLevelContainThisSecondOption ( MenuOption currentOption, int secondOptionCode );

                        /**
                         * @fn	void MenuWidgetmm::on_button_clicked(Smartmatic::GUI::Widgets::SmartmaticButton *widget,
                         * 			MenuOption _option);
                         * @brief	Executes the button clicked action.
                         * @author	Luiggi.mendez
                         * @date	27/04/2011
                         * @param	widget 	If non-null, the widget.
                         * @param	_option	The option.
                         */
                        void on_button_clicked ( Smartmatic::GUI::Widgets::SmartmaticButton * widget, MenuOption _option );

                    private:

                        int buttonIndex;	///< Zero-based index of the button

                        Gtk::Frame * principalOptionsFrame; ///< The principal options frame
                        Gtk::Frame * levelsOptionsFrame; ///< The levels options frame
                        Gtk::ButtonBox * vBox;  ///< The v box
                        std::string principalOptionsFrameTitle; ///< The principal options frame title
                        std::string levelsOptionsFrameTitle;	///< The levels options frame title

                        /**
                         * @fn	void MenuWidgetmm::update_h_box_state();
                         * @brief	Updates the h box state.
                         * @author	Luiggi.mendez
                         * @date	27/04/2011
                         */
                        void update_h_box_state();

                        /**
                         * @fn	std::string MenuWidgetmm::GetButtonName( const std::string& suffix );
                         * @brief	Gets a button name.
                         * @author	Luiggi.mendez
                         * @date	27/04/2011
                         * @param	suffix	The suffix.
                         * @return	The button name.
                         */
                        std::string GetButtonName ( const std::string& suffix );
                };
            }
        }
    }
}

#endif // _MENU_WIDGETMM_H_
