/**
 * @file PMInstallMachineWidget.cxx
 * @brief Body class pm installation widget
 * @date 19/10/2011
 * @author Yadickson Soto
 */

#include "PMInstallMachineWidget.hxx"
#include <Gui/Widgets/AbstractFactoryWidget.hxx>
#include <Operation/MachineOperationManager.hxx>
#include <Configuration/SaesGUIConfiguration-schema.hxx>
#include <Configuration/s-a-e-s-g-u-i-conf-manager.h>
#include <Configuration/s-a-e-s-functionality-conf-manager.h>
#include <Voting/VotingCenters/VotingCentersWrapper.hxx>
#include <System/IO/File.h>
#include <System/Sounds/smartmatic-sounds-manager.h>
#include <Log/SMTTLogManager.h>
#include <Log/ISMTTLog.h>
#include <System/OptParser.hxx>
#include <Functionalities/saes-functionality-manager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::Functionality::Configuration;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Voting::VotingCenters;
using namespace Smartmatic::System::Sounds;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::Log;
using namespace Smartmatic::Environment;
using namespace Smartmatic::SAES::Operation::Initiator;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::SAES::Functionality;

ISMTTLog * PMInstallMachineWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Windows.PMInstallMachineWidget", GETTEXT_PACKAGE );

PMInstallMachineWidget::PMInstallMachineWidget ( ProgressWidget * progressWidget, PollingPlaceInfoWidget * pollingPlaceInfoWidget )
    : InstallMachineWidget(), progressWidget ( progressWidget )
{
    showImages = !SAESGUIConfigurationManager::getInstance()->getStartConfiguration().getLoadSelectionLanguage();
    centerCaseSensitive = SAESGUIConfigurationManager::getInstance()->getPMInstallationWidgetConfig().getLoginCaseSensitiveConfiguration();
    forceText = false;
    loginOk = false;
    m_thread = NULL;
    this->loginWidget = NULL;
    this->infoMessage = NULL;
    this->message = NULL;
    this->image = NULL;
    this->languageWidget = NULL;
    this->reportInstall = NULL;
    this->pollingPlaceInfoWidget = pollingPlaceInfoWidget;
    this->progressWidget->setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::PROGRESS );
    EntryDividedWidgetConfiguration & passwordConfig ( SAESGUIConfigurationManager::getInstance()->getPMInstallationWidgetConfig().getPasswordWidgetConfiguration() );
    loginWidget = new LoginWidget ( showImages, passwordConfig );

    infoMessage = new InfoMessageWidget();
    languageWidget = new LanguageInstallWidget();
    reportInstall = AbstractFactoryWidget::createReportInstallWidget();
    message = new Gtk::Label();
    image = new Gtk::Image();
    loginWidget->getSignalButtonOkClicked().connect ( sigc::mem_fun ( *this, &PMInstallMachineWidget::authentication ) );
    infoMessage->getSignalButtonOkClicked().connect ( sigc::mem_fun ( *this, &PMInstallMachineWidget::install ) );
    infoMessage->getSignalButtonBackClicked().connect ( sigc::mem_fun ( *this, &PMInstallMachineWidget::initialize ) );
    infoMessage->getSignalButtonPrintClicked().connect ( sigc::mem_fun ( *this, &PMInstallMachineWidget::print ) );
    languageWidget->getSignalEnd().connect ( sigc::mem_fun ( *this, &PMInstallMachineWidget::info ) );
    languageWidget->getSignalLanguageNotFound().connect ( sigc::mem_fun ( *this, &PMInstallMachineWidget::error ) );
    reportInstall->getSignalEnd().connect ( sigc::mem_fun ( *this, &PMInstallMachineWidget::info ) );

    signalLoginDispatcher.connect ( sigc::mem_fun ( *this, &PMInstallMachineWidget::onLogin ) );

    firstTime = !VotingCentersWrapper::getInstance()->isInstall();

    if ( !firstTime )
    {
        VotingCenter votingCenter;
        VotingCentersWrapper::getInstance()->getInstallationVotingCenter ( votingCenter );
        center = votingCenter.getCode();
        loginWidget->setLoginText ( center );
        loginWidget->enabledLogin ( false );
    }
    else if ( Smartmatic::System::OptParser::getInstance()->getLoginOption ( center ) )
    {
        loginWidget->setLoginText ( center );
        loginWidget->enabledLogin ( false );
    }

    Pango::FontDescription font = message->get_style()->get_font();
    font.set_size ( 15 * Pango::SCALE );
    message->modify_font ( font );
    initialize();

    if ( Smartmatic::System::OptParser::getInstance()->getPasswordOption ( password ) )
    {
        showProgressLogin ();
    }
}

PMInstallMachineWidget::~PMInstallMachineWidget()
{
    dispose();
}

void PMInstallMachineWidget::initialize()
{
    password.clear();

    loginOk = false;
    forceText = false;
    infoMessage->hideButtonPrint();

    setWidget ( loginWidget );

    loginWidget->focusLogin();

    if ( !firstTime )
    {
        loginWidget->focusPassword();
    }
}

void PMInstallMachineWidget::authentication()
{
    loginWidget->getLoginText ( center );
    loginWidget->getPasswordText ( password );
    showProgressLogin();
}

void PMInstallMachineWidget::error()
{
    SMTTSoundsManager::getInstance()->Sound ( SAESGUIConfigurationManager::getInstance()->getAuthenticationInvalidPasswordSound() );
    infoMessage->setInfoWidget ( getMessageLoginError() );
    infoMessage->enableButtonOk ( false );
    infoMessage->hideText();
    setWidget ( infoMessage );
}

void PMInstallMachineWidget::language()
{
    setWidget ( languageWidget );
    languageWidget->start ( true );
}

void PMInstallMachineWidget::info()
{
    forceText = true;

    infoMessage->setInfoWidget ( getMessageLoginOk() );
    infoMessage->enableButtonOk ( true );
    infoMessage->showText();

    if ( !firstTime && SAESGUIConfigurationManager::getInstance()->GetPMInstallationWidgetShowPrintPasswordButton() &&
            MachineOperationManager::getInstance()->isPrintingInstallReport() )
    {
        infoMessage->showButtonPrint();
    }

    setWidget ( infoMessage );
}

void PMInstallMachineWidget::print()
{
    setWidget ( reportInstall );
    reportInstall->start ( false, false, false );
}

void PMInstallMachineWidget::install()
{
    if ( MachineOperationManager::getInstance()->install() )
    {
        end();
    }
    else
    {
        infoMessage->setInfoWidget ( getMessageLoginError() );
        setWidget ( infoMessage );
    }
}

void PMInstallMachineWidget::login()
{
    loginWidget->clearPassword();

    if ( !loginOk )
    {
        error();
    }
    else if ( SAESGUIConfigurationManager::getInstance()->getStartConfiguration().getLoadSelectionLanguage() )
    {
        info();
    }
    else
    {
        language();
    }
}

void PMInstallMachineWidget::stop()
{

}

void PMInstallMachineWidget::dispose()
{
    this->remove();

    if ( loginWidget )
    {
        delete loginWidget;
        loginWidget = NULL;
    }

    if ( infoMessage )
    {
        delete infoMessage;
        infoMessage = NULL;
    }

    if ( message )
    {
        delete message;
        message = NULL;
    }

    if ( image )
    {
        delete image;
        image = NULL;
    }

    if ( pollingPlaceInfoWidget )
    {
        delete pollingPlaceInfoWidget;
        pollingPlaceInfoWidget = NULL;
    }

    if ( languageWidget )
    {
        delete languageWidget;
        languageWidget = NULL;
    }

    if ( progressWidget )
    {
        delete progressWidget;
        progressWidget = NULL;
    }

    if ( reportInstall )
    {
        delete reportInstall;
        reportInstall = NULL;
    }

    if ( m_thread != NULL )
    {
        m_thread->join();
        m_thread = NULL;
    }
}

Gtk::Widget * PMInstallMachineWidget::getMessageLoginError()
{
    Gtk::Widget * widget;

    if ( showImages )
    {
        std::string imageFile = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/PM_passWrong.gif" );
        image->set ( Gdk::PixbufAnimation::create_from_file ( imageFile ) );
        widget = image;
    }
    else
    {
        message->set_label ( _ ( "Smartmatic.SAES.GUI.Widgets.PMInstallationWidget.LoginAndPasswordIncorrect" ) );
        widget = message;
    }

    return widget;
}

Gtk::Widget * PMInstallMachineWidget::getMessageLoginOk()
{
    Gtk::Widget * widget;

    if ( showImages && !forceText )
    {
        std::string passOkPath = ResourcePathProvider::getInstance()->GetResourcePath ( PACKAGE_NAME, "ui/PM_passOK.gif" );
        image->set ( Gdk::PixbufAnimation::create_from_file ( passOkPath ) );
        widget = image;
    }
    else
    {
        pollingPlaceInfoWidget->fillInfoWidget();
        widget = pollingPlaceInfoWidget;
    }

    return widget;
}

void PMInstallMachineWidget::showProgressLogin()
{
    progressWidget->setInfoTitle ( "  " );
    progressWidget->setInfoText ( "  " );
    setWidget ( progressWidget );

    if ( !Glib::thread_supported() )
    {
        Glib::thread_init();
    }

    if ( m_thread != NULL )
    {
        m_thread->join();
        m_thread = NULL;
    }

    m_thread = Glib::Thread::create ( sigc::mem_fun ( *this, &PMInstallMachineWidget::checkData ), true );
}

void PMInstallMachineWidget::checkData()
{
    loginOk = MachineOperationManager::getInstance()->authentication ( center, password, centerCaseSensitive );
    signalLoginDispatcher.emit();
}

void PMInstallMachineWidget::onLogin()
{
    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &PMInstallMachineWidget::login ), 0 );
}

void PMInstallMachineWidget::end()
{
    try
    {
        SaesFunctionalityManager::getInstance()->AddRunningSelectedModeForPM();
        MachineOperationManager::getInstance()->getElectoralConfiguration()->notReload();
        logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Widgets.PMInstallationWidget.LoadCenterSuccess" ) );
        stop();
        emit();
    }
    catch ( ... )
    {
        error();
    }
}

bool PMInstallMachineWidget::isTitle()
{
    return false;
}

std::string PMInstallMachineWidget::getTitleText()
{
    return N_ ( "Smartmatic.SAES.GUI.Windows.PMInstallationWindow.Title" );
}
