/**
 * @file	ReportLogWidget.cc
 * @brief	report log widget class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#include "ReportLogWidget.h"
#include <Environment/Configurations/resource-path-provider.h>
#include <Configuration/s-a-e-s-g-u-i-conf-manager.h>
#include "Printing/AuditLogPrintDocument.h"
#include "Printing/ErrorLogPrintDocument.h"
#include "Printing/PrinterStatusNotifier.h"
#include "Operation/MachineOperationManager.hxx"
#include <Log/SMTTLogManager.h>
#include <glibmm.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::Environment;
using namespace Smartmatic::SAES::Log;
using namespace Smartmatic::SAES::Printing;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Operation;

ISMTTLog * ReportLogWidget::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Widgets.ReportLogWidget", GETTEXT_PACKAGE );

ReportLogWidget::ReportLogWidget ( LogReader * logReaderAudit,
                                   LogReader * logReaderError,
                                   GenericPrintingWidget * genericPrintingWidget )
{
    this->genericPrintingWidget = genericPrintingWidget;

    m_TopWidgetTableLayout = NULL;
    m_NotebookLogs = NULL;
    m_FrameAudit = NULL;
    m_FrameErrors = NULL;
    logPagingTreeViewAudit = NULL;
    logPagingTreeViewError = NULL;
    this->logReaderAudit = logReaderAudit;
    this->logReaderError = logReaderError;
    this->m_AuditLogPrintReport = new AuditLogPrintDocument ( logReaderAudit, MachineOperationManager::getInstance()->getElectoralConfiguration() );
    this->m_ErrorLogPrintReport = new ErrorLogPrintDocument ( logReaderError, MachineOperationManager::getInstance()->getElectoralConfiguration() );

    genericPrintingWidget->getSignalEnd().connect ( sigc::mem_fun ( *this, &ReportLogWidget::endPrint ) );
    genericPrintingWidget->getSignalError().connect ( sigc::mem_fun ( *this, &ReportLogWidget::endPrint ) );

    LoadGUI();
}

ReportLogWidget::~ReportLogWidget()
{
    if ( !m_TopWidgetTableLayout )
    {
        delete ( m_TopWidgetTableLayout );
        m_TopWidgetTableLayout = NULL;
    }

    if ( !m_NotebookLogs )
    {
        delete ( m_NotebookLogs );
        m_NotebookLogs = NULL;
    }

    if ( !m_FrameAudit )
    {
        delete ( m_FrameAudit );
        m_FrameAudit = NULL;
    }

    if ( !m_FrameErrors )
    {
        delete ( m_FrameErrors );
        m_FrameErrors = NULL;
    }

    if ( m_AuditLogPrintReport != NULL )
    {
        delete m_AuditLogPrintReport;
    }

    if ( m_ErrorLogPrintReport != NULL )
    {
        delete m_ErrorLogPrintReport;
    }

    if ( logPagingTreeViewAudit )
    {
        delete logPagingTreeViewAudit;
    }

    if ( logPagingTreeViewError )
    {
        delete logPagingTreeViewError;
    }

    connPrintAudit.disconnect();
    connPrintError.disconnect();
}

void ReportLogWidget::on_buttonPrint_clicked()
{
    setWidget ( genericPrintingWidget );

    m_signalPrint_begin.emit();

    logPagingTreeViewAudit->SetPrintButtonSensitive ( false );
    logPagingTreeViewError->SetPrintButtonSensitive ( false );

    int page = m_NotebookLogs->get_current_page ();

    if ( page == 0 )
    {
        printAuditLogger ();
    }
    else
    {
        printErrorLogger ();
    }
}

void ReportLogWidget::printAuditLogger()
{
    logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Widgets.ReportLogWidget.PrintingAuditLog" ) );
    genericPrintingWidget->setInfoTitle ( _ ( "Smartmatic.SAES.GUI.Widgets.ReportLogWidget.PrintingAuditLog" ) );
    genericPrintingWidget->start ( m_AuditLogPrintReport , false, false );
}

void ReportLogWidget::printErrorLogger()
{
    logger->Audit ( N_ ( "Smartmatic.SAES.GUI.Widgets.ReportLogWidget.PrintingErrorLog" ) );
    genericPrintingWidget->setInfoTitle ( _ ( "Smartmatic.SAES.GUI.Widgets.ReportLogWidget.PrintingErrorLog" ) );
    genericPrintingWidget->start ( m_ErrorLogPrintReport , false, false );
}

ReportLogWidget::signalPrint ReportLogWidget::on_print_begin()
{
    return m_signalPrint_begin;
}

ReportLogWidget::signalPrint ReportLogWidget::on_print_end()
{
    return m_signalPrint_end;
}

void ReportLogWidget::LoadGUI ()
{
    Glib::RefPtr<Gtk::Builder> widget_builder =
        Gtk::Builder::create_from_file ( ResourcePathProvider::getInstance()->
                                         GetResourcePath ( PACKAGE_NAME, "ui/ReportLogWidget.glade" ) );
    widget_builder->get_widget ( "m_TopWidgetTableLayout", m_TopWidgetTableLayout );
    widget_builder->get_widget ( "m_NotebookLogs", m_NotebookLogs );
    widget_builder->get_widget ( "m_FrameAudit", m_FrameAudit );
    widget_builder->get_widget ( "m_FrameErrors", m_FrameErrors );

    bool enablePrint = true;

    if ( SAESGUIConfigurationManager::getInstance()->GetLogReportFunctionalityConfiguration().getEnablePrintReport().present() )
    {
        enablePrint = SAESGUIConfigurationManager::getInstance()->GetLogReportFunctionalityConfiguration().getEnablePrintReport().get();
    }

    if ( this->logReaderAudit != NULL )
    {
        logPagingTreeViewAudit = new  LogPagingTreeViewWidget ( this->logReaderAudit, enablePrint );
        m_FrameAudit->add ( *logPagingTreeViewAudit );
        m_FrameAudit->show_all_children();
    }

    if ( this->logReaderError != NULL )
    {
        logPagingTreeViewError = new  LogPagingTreeViewWidget ( this->logReaderError, enablePrint );
        m_FrameErrors->add ( *logPagingTreeViewError );
        m_FrameErrors->show_all_children();
    }

    Gtk::Label * auditTitleLabel = manage ( new Gtk::Label() );
    Gtk::Label * errorTitleLabel = manage ( new Gtk::Label() );
    auditTitleLabel->set_label ( _ ( "Smartmatic.SAES.Glade.ReportLogWidget.TagAudit" ) );
    errorTitleLabel->set_label ( _ ( "Smartmatic.SAES.Glade.ReportLogWidget.TagError" ) );
    m_NotebookLogs->set_tab_label ( *m_FrameAudit, *auditTitleLabel );
    m_NotebookLogs->set_tab_label ( *m_FrameErrors, *errorTitleLabel );

    if ( logPagingTreeViewAudit )
    {
        connPrintAudit = logPagingTreeViewAudit->signalPrint_detected.connect ( sigc::mem_fun ( this, &ReportLogWidget::on_buttonPrint_clicked ) );
    }

    if ( logPagingTreeViewError )
    {
        connPrintError = logPagingTreeViewError->signalPrint_detected.connect ( sigc::mem_fun ( this, &ReportLogWidget::on_buttonPrint_clicked ) );
    }

    showInitialImage();
    setWidget ( m_TopWidgetTableLayout );
}

void ReportLogWidget::showInitialImage()
{
    if ( logPagingTreeViewAudit )
    {
        logPagingTreeViewAudit->SetPrintButtonSensitive ( true );
    }

    if ( logPagingTreeViewError )
    {
        logPagingTreeViewError->SetPrintButtonSensitive ( true );
    }
}

void ReportLogWidget::endPrint()
{
    setWidget ( m_TopWidgetTableLayout );
    showInitialImage();
    m_signalPrint_end.emit ();
}
