/**
 * @file SystemFilesIntegrityCheckWidget.cxx
 * @brief Body class system files integrity check widget
 * @date 15/01/2019
 * @author eduardo.fernandez
 */

#include "SystemFilesIntegrityCheckWidget.hxx"

#include "Configuration/s-a-e-s-conf-manager.h"
#include "Configuration/s-a-e-s-g-u-i-conf-manager.h"
#include <Gui/Widgets/FactoryMethodWidget.hxx>
#include <Gui/Windows/USBMonitorWindow.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Operation/MachineOperationManager.hxx>
#include <Operation/Monitor/DeviceTypeMonitorPool.hxx>
#include <Runtime/Environment/MemoryPathController.h>
#include <Runtime/Environment/saes-directories.h>
#include <Voting/Election/election-schema.hxx>
#include <Voting/Election/HashValidator.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate


using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::Common::Configuration;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::GUI::Windows;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Operation::Monitor;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::SAES::Voting::Election;

ISMTTLog *SystemFilesIntegrityCheckWidget::m_logger =
    SMTTLogManager::GetLogger("Smartmatic.SAES.Voting.Election.SystemFilesIntegrityCheckWidget", GETTEXT_PACKAGE);

SystemFilesIntegrityCheckWidget::SystemFilesIntegrityCheckWidget ( ProgressWidget * progress )
    : StepWidget(), m_progress ( progress )
{
    m_progress->setInfoTextConfig ( SAESGUIConfigurationManager::getInstance()->GetVotingWindowEndMessagelabelConfig() );
    refresh();
    setTitle ( true );
    setShowBorder ( true );

    m_progress->setImage ( Smartmatic::SAES::GUI::Operation::ImageResourcePath::PROGRESS );
    m_progress->setInfoTitle ( "  " );
    m_progress->setInfoText ( "Smartmatic.SAES.GUI.Windows.VMInstallationWindow.ProgressMessage" );

    m_errorWidget = FactoryMethodWidget::createProgressWidget();
    m_errorWidget->setImage ( "ui/authentication-error.gif" );
    m_errorWidget->setInfoTitle ( "  " );
    m_errorWidget->setInfoText ( _ ( "Smartmatic.SAES.GUI.Windows.VMInstallationWindow.AuthenticationFailedAutomatic" ) );

    m_election = MachineOperationManager::getInstance()->getElectoralConfiguration()->getElection();
    m_directories = new SaesDirectories();

    m_signalDispatcher.connect ( sigc::mem_fun ( *this, &SystemFilesIntegrityCheckWidget::endVerifyingChecksums ) );

    m_thread = NULL;
    m_successfulVerification = true;
}

SystemFilesIntegrityCheckWidget::~SystemFilesIntegrityCheckWidget()
{
    this->remove();

    if ( m_progress )
    {
        delete m_progress;
        m_progress = NULL;
    }

    if ( m_errorWidget )
    {
        delete m_errorWidget;
        m_errorWidget = NULL;
    }

    if ( m_thread != NULL )
    {
        m_thread->join();
        m_thread = NULL;
    }

    if ( m_directories != NULL )
    {
        delete m_directories;
        m_directories = NULL;
    }
}

void SystemFilesIntegrityCheckWidget::initialize()
{
    setWidget ( m_progress );

    if ( !Glib::thread_supported() )
    {
        Glib::thread_init();
    }

    if ( m_thread != NULL )
    {
        m_thread->join();
        m_thread = NULL;
    }

    m_thread = Glib::Thread::create ( sigc::mem_fun ( *this, &SystemFilesIntegrityCheckWidget::process ), true );
}

bool SystemFilesIntegrityCheckWidget::verifyChecksums()
{
    bool success = true;
    std::string errorMessage;

    if (SAESConfigurationManager::getInstance()->getFileHashConfiguration().getEnableHashVerification())
    {
        Election::ChecksumsOptional checksums = m_election->getChecksums();

        if (checksums.present())
        {
            HashValidator hashValidator(m_directories, false);

            try
            {
                success = hashValidator.validateChecksums(checksums, errorMessage);
            }
            catch (...)
            {
                success = false;
            }
        }
        else
        {
            success = false;
            errorMessage = "Hashes verification setting is enabled, but no checksums node was found in the election.xml file.";
        }
    }

    if (success == false)
    {
        m_logger->Debug(errorMessage);
    }

    return success;
}

void SystemFilesIntegrityCheckWidget::showError()
{
    setWidget ( m_errorWidget );
}

void SystemFilesIntegrityCheckWidget::process()
{
    USBMonitorWindow::getInstance()->stop();

    unsigned int numberOfDevices = DeviceTypeMonitorPool::getInstance()->getNumberOfDevicesByType ( DeviceType::CONFIGURATION_FLASH );

    if ( numberOfDevices >= 1 )
    {
        m_successfulVerification = verifyChecksums();
    }
    else if ( SAESConfigurationManager::getInstance()->getFileHashConfiguration().getEnableHashVerification() )
    {
    	m_successfulVerification = false;
    }

    m_signalDispatcher.emit();
}

void SystemFilesIntegrityCheckWidget::endVerifyingChecksums()
{
    if (m_successfulVerification)
    {
        USBMonitorWindow::getInstance()->start();

        emit();
    }
    else
    {
        showError();
    }
}

std::string SystemFilesIntegrityCheckWidget::getTitleText()
{
    return N_ ( "Smartmatic.SAES.GUI.Windows.VMInstallationWindow.Title" );
}
