/**
 * @file ContestLayout.hxx
 * @brief Header Contest Layout Class
 * @date 22/04/2013
 * @author Yadickson Soto
 */

#ifndef CONTESTLAYOUT_HXX_
#define CONTESTLAYOUT_HXX_

#include <Gui/Widgets/Voting/IContestLayout.hxx>
#include <Voting/Election/election-schema.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace GUI
        {
            namespace Widgets
            {
                namespace Voting
                {
                    /**
                     * @class ContestLayout
                     * @brief Contest Layout Class, contains the logic to find the settings of visual behavior of candidates.
                     * If the contest does not have information, is obtained from the main configuration file.
                     * @date 22/04/2013
                     * @author Yadickson Soto
                     * @see IContestLayout
                     */
                    class ContestLayout : public IContestLayout
                    {
                        public:
                            /**
                             * @fn ContestLayout( Smartmatic::SAES::Voting::Election::Contest * contest );
                             * @brief Class constructor
                             * @date 22/04/2013
                             * @author Yadickson Soto
                             * @param contest contest
                             */
                            explicit ContestLayout ( Smartmatic::SAES::Voting::Election::Contest * contest );

                            /**
                             * @fn virtual ~ContestLayout();
                             * @brief Class destructor
                             * @date 22/04/2013
                             * @author Yadickson Soto
                             */
                            virtual ~ContestLayout();

                            /**
                             * @fn virtual int getColumns();
                             * @brief Getter Method: number of columns by contest
                             * @date 22/04/2013
                             * @author Yadickson Soto
                             * @return number of columns, zero if not found
                             */
                            virtual int getColumns();

                            /**
                             * @fn virtual int getPivot();
                             * @brief Getter Method: value of pivot by contest
                             * @date 22/04/2013
                             * @author Yadickson Soto
                             * @return value of pivot, zero if not found
                             */
                            virtual int getPivot();

                            /**
                             * @fn virtual bool isSwing();
                             * @brief Getter method: is swing (applies to all principal columns). Is swing: balances candidate numbers across columns
                             * @date 22/04/2013
                             * @author Yadickson Soto
                             * @return true is swing
                             */
                            virtual bool isSwing();

                            /**
                             * @brief  Partial swing is a basic swing when the first column has one entry more than the rest.
                             * @author manuel.arguelles
                             * @date   Jun 22, 2018
                             */
                            virtual bool isPartialSwing();

                            /**
                             * @fn virtual float getSpacing();
                             * @brief Getter method: value of box spacing
                             * @date 23/04/2013
                             * @author Yadickson Soto
                             * @return value of spacing box
                             */
                            virtual float getSpacing();

                            /**
                             * @fn virtual int firstnameAlignment();
                             * @brief Method getter value of firstname alignment
                             * @date 01/07/2022
                             * @author Gabriel Jaimes
                             * @return value of firstname alignment
                             */
                            virtual Smartmatic::GUI::Configuration::LabelAlignmentEnum getFirstnameAlignment();

                            /**
                             * @fn virtual int lastnameAlignment()
                             * @brief Method getter value of lastname alignment
                             * @date 01/07/2022
                             * @author Gabriel Jaimes
                             * @return value of lastname alignment
                             */
                            virtual Smartmatic::GUI::Configuration::LabelAlignmentEnum getLastnameAlignment();

                        private:

                            int columns; ///< number of columns
                            int pivot; ///< value of pivot
                            bool swing; ///< swing flag
                            bool partialSwing; ///< partial swing flag
                            float spacing; /**< box spacing */
                            Smartmatic::GUI::Configuration::LabelAlignmentEnum firstnameAlignment; ///< firstname alignment
                            Smartmatic::GUI::Configuration::LabelAlignmentEnum lastnameAlignment; ///< lastname alignment
                    };

                }
            }
        }
    }
}
#endif /* CONTESTLAYOUT_HXX_ */
