/**
 * @file	CustomAttributeSelectionBoxWidget.hxx
 * @brief	Header custom attribute selection box widget class.
 * @author	Yadickson Soto
 * @date	13/11/2012
 */

#ifndef _CUSTOM_ATTRIBUTE_SELECTION_BOX_WIDGET_H_
#define _CUSTOM_ATTRIBUTE_SELECTION_BOX_WIDGET_H_

#include "ICustomAttributeSelectionWidget.hxx"
#include "CustomAttributeSelectionButtonWidget.hxx"

#include <Widgets/BoxSelectionWidget.hxx>
#include <Widgets/SmartmaticBasicFrame.hxx>
#include <Widgets/SmartmaticBasicLabel.hxx>
#include <Widgets/SmartmaticBasicVBox.hxx>
#include <Widgets/TableBoxSelectionWidget.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace Configuration
        {
            class SelectionWidgetConfiguration;
        }

        namespace Voting
        {
            namespace VotingDevice
            {
                class Custom_attribute_definition;
            }
        }

        namespace GUI
        {
            namespace Widgets
            {
                namespace Voting
                {
                    class PartyPreview;
                }

                namespace Voting
                {
                    /**
                     * @class	CustomAttributeSelectionBoxWidget
                     * @brief	Custom attribute selection box widget.
                     * @author	Yadickson Soto
                     * @date	13/11/2012
                     */
                    class CustomAttributeSelectionBoxWidget: public ICustomAttributeSelectionWidget
                    {
                        public:

                            /**
                             * @brief	Constructor.
                             * @author	Yadickson Soto
                             * @date	13/11/2012
                             * @param	customAttr					The custom attribute.
                             * @param	showConfirmation		true to show, false to hide the confirmation.
                             * @param	sortAlphabetically	true to sort alphabetically.
                             */

                            CustomAttributeSelectionBoxWidget ( Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition & customAttr,
                                                                bool showConfirmation, bool sortAlphabetically );

                            /**
                             * @brief	Finaliser.
                             * @author	Yadickson Soto
                             * @date	13/11/2012
                             */
                            virtual ~CustomAttributeSelectionBoxWidget();

                            /**
                             * @brief	Raises the answer event.
                             * @author	Yadickson Soto
                             * @date	13/11/2012
                             * @param	index index	to be selected.
                             */
                            virtual void ForceSelectedOption ( unsigned int index );

                            /**
                             * @brief	refresh widget method.
                             * @author	Yadickson Soto
                             * @date	13/11/2012
                             */
                            virtual void refresh();

                        private:

                            /**
                             * @brief initializes the main table widget.
                             * @param enableClick whether the click is enabled in the configuration.
                             * @author eduardo.fernandez
                             * @date 12/09/2018
                             */
                            void initializeTableWidget(const bool enableClick);

                            /**
                             * @brief initializes the button widget.
                             * @param enableClick whether the click is enabled in the configuration.
                             * @author eduardo.fernandez
                             * @date 12/09/2018
                             */
                            void initializeBottomButtonWidget(const bool enableClick);

                            /**
                             * @brief initializes the header label.
                             * @param headerHeight the header height.
                             * @author eduardo.fernandez
                             * @date 12/09/2018
                             */
                            void initializeHeaderLabel(const size_t headerHeight);

                            /**
                             * @brief initializes the button frame.
                             * @param buttonHeight the button height.
                             * @author eduardo.fernandez
                             * @date 12/09/2018
                             */
                            void initializeBottomFrame(const size_t bottomHeight);

                            /**
                             * @brief initializes the button frame.
                             * @param headerHeight the header height.
                             * @author eduardo.fernandez
                             * @date 12/09/2018
                             */
                            void initializeExternalBox(const size_t headerHeight);

                            /**
                             * @brief creates the party preview frame.
                             * @param customAttributeCode the custom attribute code.
                             * @param widgetConfiguration the widget configuration.
                             * @param partyPreviewLabel the party preview label.
                             * @author eduardo.fernandez
                             * @date 12/09/2018
                             * @return the party preview frame.
                             */
                            PartyPreview* createPartyPreviewFrame(
                                const std::string& customAttributeCode,
                                const Smartmatic::SAES::Configuration::SelectionWidgetConfiguration& widgetConfiguration,
                                Smartmatic::GUI::Widgets::SmartmaticBasicLabel* const partyPreviewLabel,
                                size_t maxParties);

                            /**
                             * @brief creates the custom attribute label.
                             * @param customAttributeCode the custom attribute code.
                             * @param widgetConfiguration the widget configuration.
                             * @author eduardo.fernandez
                             * @date 12/09/2018
                             * @return the custom attribute label.
                             */
                            Smartmatic::GUI::Widgets::SmartmaticBasicLabel* createCustomAttributeLabel(
                                const std::string& customAttributeCode,
                                const Smartmatic::SAES::Configuration::SelectionWidgetConfiguration& widgetConfiguration);

                            /**
                             * @brief creates the custom attribute box.
                             * @param customAttributeCode the custom attribute code.
                             * @param enableClick whether the click is enabled in the configuration.
                             * @param widgetConfiguration the widget configuration.
                             * @param widget The widget to show.
                             * @author eduardo.fernandez
                             * @date 12/09/2018
                             * @return the custom attribute box.
                             */
                            Smartmatic::GUI::Widgets::BoxSelectionWidget* createCustomAttributeBox(
                                const std::string& customAttributeCode,
                            	const bool enableClick,
                                const Smartmatic::SAES::Configuration::SelectionWidgetConfiguration& widgetConfiguration,
                                Widget* const widget);

                            /**
                             * @brief list the custom attributes.
                             * @param enableClick whether the click is enabled in the configuration.
                             * @author eduardo.fernandez
                             * @date 12/09/2018
                             */
                            void listCustomAttributes(const bool enableClick);

                            /**
                             * @brief	Initializer method.
                             * @author	Yadickson Soto
                             * @date	13/11/2012
                             */
                            virtual void initialize();

                            /**
                             * @brief	virtual method stop.
                             * @author	Yadickson Soto
                             * @date	13/11/2012
                             */
                            virtual void dispose();

                            /**
                             * @brief	method accept click button.
                             * @author	Yadickson Soto
                             * @date	14/11/2012
                             */
                            void onAcceptClicked();

                            /**
                             * @brief	method cancel click button.
                             * @author	Yadickson Soto
                             * @date	14/11/2012
                             */
                            void onClearClicked();

                            /**
                             * @brief	method box click.
                             * @author	Yadickson Soto
                             * @date	14/11/2012
                             * @param list box selected
                             */
                            void onBoxClicked ( std::list < Smartmatic::GUI::Widgets::BoxSelectionWidget * > & list );

                            /**
                             * @brief  Checks if the party preview is enabled or not.
                             * @author manuel.arguelles
                             * @date   sep 21, 2018
                             * @return True if the party preview is enabled, false if not.
                             */
                            bool shouldShowPartyPreview() const;

                            /**
                             * @brief  Gets the maximum amount of parties from the following contests.
                             * @author manuel.arguelles
                             * @date   sep 28, 2018
                             * @return The maximum amount of parties.
                             */
                            size_t maxPartiesInContests() const;

                            /**
                             * @brief  Gets the smallest font size of all the party previews.
                             * @author manuel.arguelles
                             * @date   oct  2, 2018
                             * @return The smallest front size of the party preview.
                             */
                            int getSmallestPartyPreviewFontSize();

                            /**
                             * @brief  Sets a font size for the party preview.
                             * @author manuel.arguelles
                             * @date   oct  2, 2018
                             * @param  fontSize The font size to use.
                             */
                            void setPartyPreviewFontSize(int fontSize);

                            /**
                             * @brief  Called when the party preview widget requires size
                             * allocation.
                             * @author manuel.arguelles
                             * @date   oct  2, 2018
                             * @param  request The size request.
                             */
                            void on_party_preview_size_request(Gtk::Requisition* request);

                        private:

                            Smartmatic::GUI::Widgets::SmartmaticBasicLabel * headerLabel; ///< header label
                            Smartmatic::GUI::Widgets::SmartmaticBasicFrame * bottomFrame; ///< bottom frame
                            Smartmatic::GUI::Widgets::SmartmaticBasicVBox * externalBox; ///< external vbox
                            CustomAttributeSelectionButtonWidget * bottomButtonWidget; ///< button widget
                            Smartmatic::GUI::Widgets::BoxSelectionWidget * currentBox; ///< current box
                            Smartmatic::GUI::Widgets::TableBoxSelectionWidget * tableWidget; ///< table widgets
                            int partyPreviewFont; ///< Font size of party preview
                    };
                }
            }
        }
    }
}

#endif // _CUSTOM_ATTRIBUTE_SELECTION_BOX_WIDGET_H_
