/**
 * @file RemoveCardWidget.cxx
 * @brief Body class card removed widget
 * @date 29/02/2012
 * @author Yadickson Soto
 */

#include "RemoveCardWidget.hxx"
#include <Configuration/AlarmConfigurationManager.hxx>
#include <Configuration/s-a-e-s-g-u-i-conf-manager.h>
#include <Operation/AlarmMessage.hxx>
#include <Operation/Devices/AlarmController.hxx>
#include <Operation/Devices/CardController.hxx>
#include <Operation/Devices/Polling/CardPolling.hxx>
#include <Operation/MachineOperationManager.hxx>
#include <Voting/DeviceParametersManager.hxx>
#include <Voting/Election/ElectionWrapper.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::SAES::Functionality::Configuration;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Operation;
using namespace Smartmatic::SAES::GUI::Widgets::Voting;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::HW::Devices;
using namespace Smartmatic::SAES::Operation::Devices;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::SAES::Voting::Election;

static const std::string REMOVE_CARD_MESSAGE_KEY  = _("smartmatic.saes.gui.widgets.voting.RemoveCardWidget.RemoveCardMessage");
static const std::string HAND_SCANNER_USE_ENABLED = "hand-scanner-use-enabled";

RemoveCardWidget::RemoveCardWidget ( ProgressWidget * progress )
    : WizardWidgetBase(), progress ( progress )
{
    progress->setInfoTitleConfig ( SAESGUIConfigurationManager::getInstance()->GetVotingWindowEndMessagelabelConfig() );
    progress->setInfoTextConfig ( SAESGUIConfigurationManager::getInstance()->GetVotingWindowEndMessagelabelConfig() );

    progress->setImage ( "ui/RemoveTheCard.gif" );
    setAccessibleName ( "RemoveCard" );

    refresh();
    setTitle ( false );
    setShowBorder ( true );
    setWidget ( progress );
}

RemoveCardWidget::~RemoveCardWidget()
{
    this->remove();
    connRefresh.disconnect();

    if ( progress )
    {
        delete progress;
        progress = NULL;
    }
}

void RemoveCardWidget::refresh()
{
    const bool handScannerEnabled = DeviceParametersManager::getInstance()->getParameter ( HAND_SCANNER_USE_ENABLED );

    progress->setInfoTitle ( N_ ( "Smartmatic.SAES.GUI.Widgets.Voting.RemoveCardWidget.TitleRemoveTheCard" ) );

    if ( handScannerEnabled )
    {
        progress->setInfoText ( N_ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.PleaseRemoveTheCardAndValidateVote" ) );
    }
    else
    {
        progress->setInfoText ( N_ ( "Smartmatic.SAES.GUI.Windows.VotingWindow.PleaseRemoveTheCard" ) );
    }

    m_play_audio_signal.emit(REMOVE_CARD_MESSAGE_KEY);
}

void RemoveCardWidget::initialize()
{
    timeOut = SAESGUIConfigurationManager::getInstance()->getVotingExperienceConfiguration().getTimeOutCardEndVoting();

    if ( SAESGUIConfigurationManager::getInstance()->getVotingExperienceConfiguration().getCheckCardEndVoting() )
    {
        count = 0;
        safeTime.start();
        connRefresh.disconnect();
        connRefresh = Glib::signal_timeout().connect ( sigc::bind ( sigc::mem_fun ( *this, &RemoveCardWidget::checkCard ), 0 ), 1000 );
    }
    else
    {
        unsigned int time = SAESGUIConfigurationManager::getInstance()->getWaitTimeByBasicImageType ( Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_REMOVE_CARD );
        Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &RemoveCardWidget::endOK ), time );
    }
}

void RemoveCardWidget::endOK()
{
    emit();
}

void RemoveCardWidget::endError()
{
    AlarmMessage::alarm ( AlarmEnum::CONFIGURATION_CARDNOTREMOVEDAFTERVOTING );
    emit();
    m_stop_signal.emit();
}

bool RemoveCardWidget::checkCard ( int timer_num )
{
    if ( !CardPolling::getInstance()->isCardPresent() )
    {
        if ( count >= timeOut && AlarmConfigurationManager::getInstance()->getCategoryOfAlarm ( AlarmEnum::CONFIGURATION_CARDNOTREMOVEDAFTERVOTING ) == AlarmCategoryEnum::CONFIGURATION_NOTALARM )
        {
            AlarmController::getInstance()->shutDownAlarm();
        }

        endCount();
        return false;
    }
    else if ( count == timeOut )
    {
        count++;

        if ( AlarmConfigurationManager::getInstance()->getCategoryOfAlarm ( AlarmEnum::CONFIGURATION_CARDNOTREMOVEDAFTERVOTING ) == AlarmCategoryEnum::CONFIGURATION_NOTALARM )
        {
            AlarmController::getInstance()->soundAlarm ( AlarmEnum::CONFIGURATION_CARDNOTREMOVEDAFTERVOTING );
        }
        else
        {
            endError();
            return false;
        }
    }
    else
    {
        count++;
    }

    return true;
}

void RemoveCardWidget::endCount()
{
    safeTime.stop();

    int voteTime = safeTime.getAllSeconds() * 1000;
    int time = SAESGUIConfigurationManager::getInstance()->getWaitTimeByBasicImageType ( Smartmatic::SAES::Voting::Screen::BasicImageType::GENERATED_IMAGES_REMOVE_CARD );

    int result = time - voteTime;

    if ( result <= 0 )
    {
        result = 0;
    }

    Glib::signal_timeout().connect_once ( sigc::mem_fun ( *this, &RemoveCardWidget::endOK ), result );
}
