/**
 * @file	party-widget.cc
 * @brief	party widget class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#include "party-widget.h"
#include "Configuration/label-configuration.h"
#include "Configuration/s-a-e-s-g-u-i-conf-manager.h"
#include "System/Sounds/smartmatic-sounds-manager.h"
#include "Environment/Configurations/resource-path-provider.h"
#include "Resources/election-string-resources-manager.h"
#include "System/IO/File.h"
#include <Log/SMTTLogManager.h>
#include <Log/ISMTTLog.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Gtk;
using namespace Smartmatic::Environment;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::Configuration;
using namespace Smartmatic::SAES::GUI::Widgets;
using namespace Smartmatic::SAES::Voting::Election;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::Log;

#define LOGGER_PROJECT "Smartmatic.SAES.GUI.Widgets.PartyWidget" ///< logger project name

#ifdef GETTEXT_PACKAGE
ISMTTLog * PartyWidget::logger = SMTTLogManager::GetLogger ( LOGGER_PROJECT, GETTEXT_PACKAGE );
#else
ISMTTLog * PartyWidget::logger = SMTTLogManager::GetLogger ( LOGGER_PROJECT, "" );
#endif

PartyWidget::PartyWidget ( Smartmatic::SAES::Voting::Election::Ballot_option & ballotParty,
                           Smartmatic::SAES::Runtime::Environment::SaesDirectories & saesDir,
                           int width, int height )
    : SmartmaticBasicFrame ()
{
    static PartyWidgetConfiguration config =
        SAESGUIConfigurationManager::getInstance()->GetPartyWidgetConfiguration();

    m_widgetEnabled = true;
    set_size_request ( width, height );
    this->ballotParty = ballotParty;
    this->party = this->ballotParty.getParty();
    std::string image_path = party.getImage_file_name() != NULL ?  saesDir.getImageElectoralResource ( party.getImage_file_name().get() )  : "";

    set_shadow_type ( SHADOW_NONE );

    bool useAbbreviation = config.getUseAbbreviation();
    std::string label_name;

    if ( useAbbreviation )
    {
        label_name = ElectionStringResourcesManager::getInstance()->getResource ( party.getAbbreviation() != NULL ? party.getAbbreviation().get() : "null" );
    }
    else
    {
        label_name = ElectionStringResourcesManager::getInstance()->getResource ( party.getName() != NULL ? party.getName().get() : "null" );
    }

    std::stringstream ss;
    ss << ballotParty.getTally_position();
    std::string number = ss.str();
    int numberWidth = config.getNumberWidth();
    Smartmatic::SAES::Configuration::WidgetDisplayType display = config.getDisplay();
    LabelConfiguration labelConf = SAESGUIConfigurationManager::getInstance()->GetVotingWidgetPartySelectionLabelChildLabelConfiguration();

    if ( display.getType() != DisplayType::CONFIGURATION_DUAL_TEXT && display.getType() != DisplayType::CONFIGURATION_TEXT )
    {
        if ( File::getInstance()->canOpenFile ( image_path ) )
        {
            votingWidget = manage ( new VotingWidgetBase ( display, image_path, number, config.getPadding(), numberWidth, labelConf, width, height ) );
        }
        else
        {
            display.setType ( DisplayType::CONFIGURATION_TEXT );
            display.setXAlign ( 0 );
            display.setYAlign ( 0 );
            votingWidget = manage ( new VotingWidgetBase ( display, label_name, number, config.getPadding(), numberWidth, labelConf, width, height ) );
        }
    }
    else
    {
        votingWidget = manage ( new VotingWidgetBase ( display, label_name, number, config.getPadding(), numberWidth, labelConf, width, height ) );
    }

    add ( *votingWidget );
    show();
    labelString = NULL;
    isPartyNull = false;
}

PartyWidget::PartyWidget ( std::string text, int width, int height )
    : votingWidget(NULL)
{
    m_widgetEnabled = true;
    set_size_request ( width, height );
    set_shadow_type ( SHADOW_NONE );

    LabelConfiguration labelConf = SAESGUIConfigurationManager::getInstance()->GetPartyNullLabelConfiguration();
    labelString = manage ( new Label ( text ) );
    LabelConfiguration::SetLabelConfiguration ( labelString, labelConf );

    add ( *labelString );
    show();
    isPartyNull = true;
}

void PartyWidget::RefreshLabel ( std::string text )
{
    if ( labelString )
    {
        labelString->set_text ( text );
    }
}

PartyWidget::~PartyWidget()
{
}

Smartmatic::SAES::Voting::Election::Party & PartyWidget::getParty()
{
    return party;
}

std::string PartyWidget::getPartyCode()
{
    return ( isPartyNull ? "" : party.getCode() );
}

int PartyWidget::getSmallestFontSize() const
{
    int fontSize = -1;

    if (labelString)
    {
        Glib::RefPtr<Gtk::Style> labelStyle = labelString->get_style();

        if (labelStyle)
        {
            fontSize = labelStyle->get_font().get_size() / Pango::SCALE;
        }
    }

    if (votingWidget)
    {
        fontSize = votingWidget->getSmallestAdjustedFontSize();
    }

    return fontSize;
}

void PartyWidget::setFontSize(int fontSize)
{
    if (labelString)
    {
        Glib::RefPtr<Gtk::Style> labelStyle = labelString->get_style();

        if (labelStyle)
        {
            Pango::FontDescription font = labelStyle->get_font();

            font.set_size(fontSize * Pango::SCALE);
            labelString->modify_font(font);
        }

    }

    if (votingWidget)
    {
        votingWidget->setFontSize(fontSize);
    }
}

