/**
 * @file	GenerateXMLImage.cpp
 * @brief	Implements the generate XML image class.
 * @author	Youssef.tourkmani
 * @date	01/11/2012
 */


#include "GenerateXMLImage.h"
#include "Runtime/Environment/saes-temporary-directories.h"
#include <Voting/VotingCenters/VotingCentersWrapper.hxx>
#include "Resources/election-string-resources-manager.h"
#include "System/IO/Path.h"
#include "System/IO/Directory.h"
#include <Log/SMTTLogManager.h>
#include <Voting/set-language-functionality.h>
#include "Functionalities/voting-experience-validation-functionality.h"
#include <System/ZenityWrapper.hxx>
#include <string>
#include <sstream>
#include <Functionalities/voting-experience-validation-functionality.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Functionality;
using namespace Smartmatic::SAES::Functionality;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::GUI::Configuration;
using namespace Smartmatic::SAES::GUI::Functionality;
using namespace Smartmatic::SAES::GUI::Operation;
using namespace Smartmatic::SAES::Resources;
using namespace Smartmatic::SAES::Runtime::Environment;
using namespace Smartmatic::SAES::Operation;
using namespace Smartmatic::SAES::Operation::Voting;
using namespace Smartmatic::SAES::Voting;
using namespace Smartmatic::SAES::Voting::Status;
using namespace Smartmatic::SAES::Voting::Screen;
using namespace Smartmatic::System;

ISMTTLog * GenerateXMLImage::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.GUI.Operation.GenerateXMLImage", GETTEXT_PACKAGE );

GenerateXMLImage::GenerateXMLImage()
    : electoralConfiguration(NULL)
    , machineOperationManager(NULL)
    , imagesWrapper(NULL)
{
}

GenerateXMLImage::~GenerateXMLImage()
{
    dispose();
}

void GenerateXMLImage::dispose()
{
    if ( imagesWrapper )
    {
        delete imagesWrapper;
        imagesWrapper = NULL;
    }
}

void GenerateXMLImage::init()
{
    dispose();
    machineOperationManager = Smartmatic::SAES::Operation::MachineOperationManager::getInstance();
    questionConfig = SAESGUIConfigurationManager::getInstance()->GetCustomAtributtesWindowConfiguration();
    screenConfig = SAESGUIConfigurationManager::getInstance()->GetVotingExperienceValidationScreensFunctionalityConfiguration();

    imagesWrapper = new GeneratedImagesWrapper();
    electoralConfiguration = machineOperationManager->getElectoralConfiguration();
    logger->Debug ( "Generating info XML image" );
}

void GenerateXMLImage::persist ( std::string folderName )
{
    try
    {
        imagesWrapper->Persist ( electoralConfiguration, folderName );
        logger->Debug ( "Successfully created XML image" );
    }
    catch ( ... )
    {
        logger->Debug ( "Error created XML image" );
    }
}

void GenerateXMLImage::addImage (
    std::string fileName,
    std::string language,
    std::string country,
    Smartmatic::SAES::Voting::Screen::BasicImageType type )

{
    BasicImage image;
    image.setFile ( fileName );
    image.setLanguage_code ( language );
    image.setCountry_code ( country );
    image.setType ( type );
    imagesWrapper->addBasicImage ( image );
}

void GenerateXMLImage::addSelectionImage (
    std::string fileName,
    std::string language,
    std::string country,
    Smartmatic::SAES::Voting::Screen::BasicImageType type,
    std::string selection )
{
    SelectionImage image;
    image.setFile ( fileName );
    image.setLanguage_code ( language );
    image.setCountry_code ( country );
    image.setType ( type );
    image.setSelection ( selection );
    imagesWrapper->addSelectionImage ( image );
}

void GenerateXMLImage::addCustomAttributes ( std::string fileName, std::string language, std::string country, Smartmatic::SAES::Voting::VotingDevice::Custom_attribute_definition & customAttribute )
{
    SetLanguageFunctionality::setNewVotingLanguageByLanguageCountry ( language, country );

    CustomAttributeImage image;
    image.setFile ( fileName );
    image.setLanguage_code ( language );
    image.setCountry_code ( country );
    image.setType ( BasicImageType::GENERATED_IMAGES_CUSTOM_ATTRIBUTE );
    image.setCustom_attribute_code ( customAttribute.getCode() );
    image.setCustom_attribute_name ( ElectionStringResourcesManager::getInstance()->getResource ( customAttribute.getName_key() ) );
    image.setCustom_attribute_description ( ElectionStringResourcesManager::getInstance()->getResource ( customAttribute.getDescription_key() ) );
    imagesWrapper->addCustomAttributeImage ( image );
}

void GenerateXMLImage::addParty ( std::string fileName, std::string language, std::string country, Smartmatic::SAES::Voting::Election::Contest * contest )
{
    SetLanguageFunctionality::setNewVotingLanguageByLanguageCountry ( language, country );

    PartySelectionImage image;
    image.setFile ( fileName );
    image.setLanguage_code ( language );
    image.setCountry_code ( country );
    image.setType ( BasicImageType::GENERATED_IMAGES_PARTY );
    image.setContest_code ( contest->getCode() );
    image.setContest_custom_code ( contest->getCustom_code() );
    image.setContest_title ( ElectionStringResourcesManager::getInstance()->getResource ( contest->getTitle_key() ) );
    image.setContest_abstract ( ElectionStringResourcesManager::getInstance()->getResource ( contest->getAbstract_key() ) );
    image.setContest_help ( ElectionStringResourcesManager::getInstance()->getResource ( contest->getHelp_key() ) );
    imagesWrapper->addPartySelectionImage ( image );
}

void GenerateXMLImage::addBallot ( std::string fileName, std::string language, std::string country, Smartmatic::SAES::Voting::Election::Party * party )
{
    SetLanguageFunctionality::setNewVotingLanguageByLanguageCountry ( language, country );

    BallotSelectionImage image;
    image.setFile ( fileName );
    image.setLanguage_code ( language );
    image.setCountry_code ( country );
    image.setType ( BasicImageType::GENERATED_IMAGES_BALLOT );
    image.setParty_name ( party->getName().present() ? party->getName().get() : " " );
    image.setParty_abbreviation ( party->getAbbreviation().present() ?  party->getAbbreviation().get() : " " );
    image.setParty_code ( party->getCode() );
    image.setParty_custom_code ( party->getCustom_code() );
    imagesWrapper->addBallotSelectionImage ( image );
}
