/**
 * @file   CommandBaseEurn.hxx
 * @brief  Declaration of the command base for EURN.
 * @author manuel.arguelles
 * @date   Aug 25, 2017
 */

#ifndef COMMAND_BASE_EURN_HXX
#define COMMAND_BASE_EURN_HXX

#include "EurnError.hxx"
#include <Commands/command-structure.h>
#include <Configuration/EurnDeviceSchema.hxx>
#include <vector>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace HW
        {
            namespace Connections
            {
                class IConnection;
            }

            namespace Commands
            {

                /**
                 * @class  CommandBaseEurn
                 * @brief  Base clase for the commands of the EURN.
                 * @author manuel.arguelles
                 * @date   Aug 25, 2017
                 */
                class CommandBaseEurn : public CommandStructure
                {
                    public:

                        /**
                         * @brief  CommandBaseEurn constructor.
                         * @author manuel.arguelles
                         * @date   Aug 25, 2017
                         * @param  connection The connection to the device.
                         */
                        explicit CommandBaseEurn(HW::Connections::IConnection* connection);

                        /**
                         * @brief  CommandBaseEurn destructor.
                         * @author manuel.arguelles
                         * @date   Aug 25, 2017
                         */
                        virtual ~CommandBaseEurn();

                    protected:

                        /**
                         * @brief  Get the complete command list.
                         * @author manuel.arguelles
                         * @date   Aug 25, 2017
                         * @return The list of commands for the device.
                         */
                        virtual CommandStructure::CommandList getCommandList();

                        /**
                         * @brief  Process the response of the command.
                         * @author manuel.arguelles
                         * @date   Aug 28, 2017
                         * @param  response The response.
                         * @return True if the response was as expected, false if not.
                         */
                        virtual bool processResponse(const std::vector<unsigned char>& response);

                        /**
                         * @brief  Get the status byte of the response.
                         * @author manuel.arguelles
                         * @date   Aug 29, 2017
                         * @return The status byte.
                         */
                        unsigned char getStatus();

                        /**
                         * @brief  Get the version byte of the response.
                         * @author manuel.arguelles
                         * @date   Aug 29, 2017
                         * @return The version byte.
                         */
                        unsigned char getVersionByte();

                    private:

                        /**
                         * @brief  Get the errors.
                         * @author manuel.arguelles
                         * @date   Aug 29, 2017
                         * @param  response The response vector.
                         * @return The error code.
                         */
                        EurnError getError(const std::vector<unsigned char>& response);

                    private:

                        static Smartmatic::Log::ISMTTLog* logger;
                };

            } /* namespace Commands */
        } /* namespace HW */
    } /* namespace SAES */
} /* namespace Smartmatic */

#endif /* COMMAND_BASE_EURN_HXX */
