/**
 * @file	BarcodeDeviceConfigurationManager.hxx
 * @brief	Declares the barcode configuration manager class.
 */

#ifndef _BARCODE_CONFIGURATION_MANAGER_H_
#define _BARCODE_CONFIGURATION_MANAGER_H_

#include <string>
#include <Configuration/SaesHardwareConfiguration-schema.hxx>

namespace Smartmatic
{
    namespace SAES
    {
        namespace HW
        {
            namespace Configuration
            {
                /**
                 * @class	BarcodeDeviceConfigurationManager
                 * @brief	Manager for barcode configurations class.
                 * @author	Yadickson Soto
                 * @date	11/02/2014
                 */
                class BarcodeDeviceConfigurationManager
                {
                    public:

                        /**
                         * @fn	static BarcodeDeviceConfigurationManager *getInstance(BarcodeDeviceConfigurationManager * externManager = NULL);
                         * @brief	Gets the current barcode Configuration Manager.
                         * @author	Yadickson Soto
                         * @date	11/02/2014
                         * @param 	externManager external manager to unit test
                         * @return	instance of the class
                         */
                        static BarcodeDeviceConfigurationManager * getInstance ( BarcodeDeviceConfigurationManager * externManager = NULL );

                        /**
                         * @fn	BarcodeCommands::CommandSequence getBarcodeConfigurationCommands ( const std::string & commandBlock, const std::string & barcodeModel );
                         * @brief	Gets the barcode configuration commands for a command block. Will first search for commands for the specific model, and if not available, will revert to a default command set.
                         * @author	Yadickson Soto
                         * @date	11/02/2014
                         * @param	commandBlock	command block name.
                         * @param	barcodeModel	model of bardcode reader.
                         * @return 	The configuration commands result.
                         */
                        BarcodeCommands::CommandSequence getBarcodeConfigurationCommands ( const std::string & commandBlock, const std::string & barcodeModel );

                        /**
                         * @fn	std::string getModelFile ();
                         * @brief	Gets the model file to simulate device.
                         * @author	Yadickson Soto
                         * @date	08/07/2015
                         * @return 	The model file path
                         */
                        std::string getModelFile ();

                        /**
                         * @fn	std::string getSerialFile ();
                         * @brief	Gets the serial file to simulate device.
                         * @author	Yadickson Soto
                         * @date	08/07/2015
                         * @return 	The serial file path
                         */
                        std::string getSerialFile ();

                        /**
                         * @fn	std::string getCommandsFile ();
                         * @brief	Gets the commands file to simulate device.
                         * @author	Yadickson Soto
                         * @date	08/07/2015
                         * @return 	The commands file path
                         */
                        std::string getCommandsFile ();

                        /**
                         * @fn	std::string getBufferFile ();
                         * @brief	Gets the buffer file to simulate device.
                         * @author	Yadickson Soto
                         * @date	08/07/2015
                         * @return 	The buffer file path
                         */
                        std::string getBufferFile ();

                        /**
                         * @fn	bool simulationEnabled ();
                         * @brief	Get if simulation mode is enabled
                         * @author	Yadickson Soto
                         * @date	08/07/2015
                         * @return	true if the simulation mode is enabled.
                         */
                        bool simulationEnabled ();

                        /**
                         * @fn	virtual ~BarcodeDeviceConfigurationManager();
                         * @brief	Class destructorer
                         * @author	Yadickson Soto
                         * @date	11/02/2014
                         */
                        virtual ~BarcodeDeviceConfigurationManager();

                    protected:

                        /**
                         * @fn	virtual BarcodeDeviceConfiguration & getBarcodeDeviceConfiguration();
                         * @brief	Gets barcode device configuration.
                         * @author	Yadickson Soto
                         * @date	11/02/2014
                         * @return	The barcode device configuration.
                         */
                        virtual BarcodeDeviceConfiguration & getBarcodeDeviceConfiguration();

                        /**
                         * @fn	BarcodeDeviceConfigurationManager();
                         * @brief	Default constructor.
                         * @author	Yadickson Soto
                         * @date	11/02/2014
                         */
                        BarcodeDeviceConfigurationManager();

                        static BarcodeDeviceConfigurationManager * instance; ///< The singleton instance
                };
            }
        }
    }
}

#endif // _BARCODE_CONFIGURATION_MANAGER_H_
