/**
 * @file	s-a-e-s-hardware-conf-manager.cc
 * @brief	e s hardware conf manager class.
 * @author	Youssef.tourkmani
 * @date	31/10/2012
 */

#include "s-a-e-s-hardware-conf-manager.h"
#include <System/ZenityWrapper.hxx>
#include <System/Runtime/ExitProcess.hxx>
#include <Configuration/ConfigurationBase-schema.hxx>
#include <Configuration/configuration-persistence-manager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Configuration;
using namespace Smartmatic::SAES::HW::Configuration;
using namespace Smartmatic::System;
using namespace Smartmatic::System::Runtime;

SAESHardwareConfigurationManager * SAESHardwareConfigurationManager::instance = NULL;

SAESHardwareConfigurationManager::SAESHardwareConfigurationManager()
{
    configuration = NULL;
}

SAESHardwareConfigurationManager::~SAESHardwareConfigurationManager()
{
    if ( configuration != NULL )
    {
        delete ( configuration );
        configuration = NULL;
    }

    instance = NULL;
}

SaesHardwareConfiguration * SAESHardwareConfigurationManager::getConfiguration()
{
    if ( configuration == NULL )
    {
        initialize();
    }

    return configuration;
}

void SAESHardwareConfigurationManager::initialize()
{
    try
    {
        std::string semiFile ( ConfigurationPersistenceManager::getInstance()->GetLibraryConfigFile ( "SAESHardware" ) );
        std::istringstream ss ( semiFile );
        configuration = new SaesHardwareConfiguration ( * parseSaesHardwareConfiguration ( ss, ConfigurationTypes::Flags::dont_validate ).get() );
    }
    catch ( const ::xsd::cxx::tree::exception< char > & e )
    {
        std::stringstream ss;
        ss << e;
        ZenityWrapper::showError ( "Config SAESHW Error \n\n" + ss.str(), "Error" );
        ExitProcess::getInstance()->exitProcess ();
    }
}

void SAESHardwareConfigurationManager::loadDefault()
{
    SaesHardwareConfiguration configSAESHardware;

    std::stringstream ss;
    Smartmatic::ConfigurationTypes::NamespaceInfomap map;
#ifdef NAMESPACE_SAES_HARDWARE_CONFIGURATION
    map[""].name = NAMESPACE_SAES_HARDWARE_CONFIGURATION;
#endif
    map[""].schema = "";

    P168ADeviceConfiguration p168AConfig;

    P168ADeviceConfiguration::ButtonTypeSequence buttonsType;
    buttonsType.push_back ( AbleButtonType::P168A_DEVICE_PAUSE );
    buttonsType.push_back ( AbleButtonType::P168A_DEVICE_HELP );
    buttonsType.push_back ( AbleButtonType::P168A_DEVICE_UP );
    buttonsType.push_back ( AbleButtonType::P168A_DEVICE_DOWN );
    buttonsType.push_back ( AbleButtonType::P168A_DEVICE_LEFT );
    buttonsType.push_back ( AbleButtonType::P168A_DEVICE_RIGHT );
    buttonsType.push_back ( AbleButtonType::P168A_DEVICE_SELECT );
    buttonsType.push_back ( AbleButtonType::P168A_DEVICE_VOLUME_DOWN );
    buttonsType.push_back ( AbleButtonType::P168A_DEVICE_VOLUME_UP );
    buttonsType.push_back ( AbleButtonType::P168A_DEVICE_SPEED_DOWN );
    buttonsType.push_back ( AbleButtonType::P168A_DEVICE_SPEED_UP );

    p168AConfig.setButtonType ( buttonsType );
    p168AConfig.setSerialMachineFile ( "SerialMachineFile" );
    p168AConfig.setAbleDButtonsFile ( "AbleDButtonsFile" );

    A4517AbledDeviceConfiguration a4517AbledConfig;

    A4517AbledDeviceConfiguration::ButtonTypeSequence a4517AbledButtonsType;
    a4517AbledButtonsType.push_back(A4517AbleButtonType::A4517_ABLED_DEVICE_UNKNOWN);
    a4517AbledButtonsType.push_back(A4517AbleButtonType::A4517_ABLED_DEVICE_HELP);
    a4517AbledButtonsType.push_back(A4517AbleButtonType::A4517_ABLED_DEVICE_VOLUME_DOWN);
    a4517AbledButtonsType.push_back(A4517AbleButtonType::A4517_ABLED_DEVICE_DOWN);
    a4517AbledButtonsType.push_back(A4517AbleButtonType::A4517_ABLED_DEVICE_SPEED_DOWN);
    a4517AbledButtonsType.push_back(A4517AbleButtonType::A4517_ABLED_DEVICE_LEFT);
    a4517AbledButtonsType.push_back(A4517AbleButtonType::A4517_ABLED_DEVICE_SELECT);
    a4517AbledButtonsType.push_back(A4517AbleButtonType::A4517_ABLED_DEVICE_RIGHT);
    a4517AbledButtonsType.push_back(A4517AbleButtonType::A4517_ABLED_DEVICE_VOLUME_UP);
    a4517AbledButtonsType.push_back(A4517AbleButtonType::A4517_ABLED_DEVICE_UP);
    a4517AbledButtonsType.push_back(A4517AbleButtonType::A4517_ABLED_DEVICE_SPEED_UP);

    a4517AbledConfig.setButtonType( a4517AbledButtonsType );

    PDBDeviceConfiguration pdbConfig;
    pdbConfig.setPDBPortForA4517SCRPowerCycle ( PDBPortType::PDB_DEVICE_USB_4 );

    IOBDeviceConfiguration iobConfig;
    iobConfig.setIOBPortForVIUSCRPowerCycle ( IOBPortType::IOB_DEVICE_USB_4 );

    BarcodeDeviceConfiguration barcodeConf;

    /**
     * baseconfiguration-mb2028
     */
    BarcodeCommands::CommandSequence barcodeCommandsSeq;
    BarcodeCommand command;
    command.setRetries ( 2 );
    command.setDelayResponse ( 3000 );
    command.setMandatorySent ( true );
    //1. Desabilitar todos los codigos de barra: NLS0001010
    command.setCommand ( "NLS0001010" );
    barcodeCommandsSeq.push_back ( command );
    //2. Apagar el envio de informacion de settings: NLS0002000
    command.setCommand ( "NLS0002000" );
    barcodeCommandsSeq.push_back ( command );
    //3. Apagar posibilidad de configuracion con codigos: NLS0006000
    command.setCommand ( "NLS0006000" );
    barcodeCommandsSeq.push_back ( command );
    //4. Apagar el envio de informacion cuando se enciende la lectora: NLS0007000
    command.setCommand ( "NLS0007000" );
    barcodeCommandsSeq.push_back ( command );
    //5. Asignar modo de Light a LedOnWhenScan: NLS0200030
    command.setCommand ( "NLS0200030" );
    barcodeCommandsSeq.push_back ( command );
    //6. Asignar modo de Aiming a FlashWhenScan: NLS0201000
    command.setCommand ( "NLS0201000" );
    barcodeCommandsSeq.push_back ( command );
    //7. Desabilitar el Beep de lectura: NLS0203000 = off NLS0203010 = on
    command.setCommand ( "NLS0203010" );
    barcodeCommandsSeq.push_back ( command );
    //8. Apagar el uso de cualquier prefijo o sufijo en la comunicacion: NLS0311000
    command.setCommand ( "NLS0311000" );
    barcodeCommandsSeq.push_back ( command );
    //9. Apagar el empaquetado de datos: NLS0314000
    command.setCommand ( "NLS0314000" );
    barcodeCommandsSeq.push_back ( command );
    //10. Apagar intercepcion de mensajes: NLS0315000
    command.setCommand ( "NLS0315000" );
    barcodeCommandsSeq.push_back ( command );
    //11. Apagar terminador NLS0309000
    command.setCommand ( "NLS0309000" );
    barcodeCommandsSeq.push_back ( command );
    //12. Asingnar el modo de lectura a Auto Scan: NLS0302010
    command.setCommand ( "NLS0302010" );
    barcodeCommandsSeq.push_back ( command );
    //13. Asignar la sensibilidad como la mas alta: NLS0312030
    command.setCommand ( "NLS0312030" );
    barcodeCommandsSeq.push_back ( command );
    //14. Asignar el delay entre lecturas a 0: NLS0313000=0
    command.setCommand ( "NLS0313000=0" );
    barcodeCommandsSeq.push_back ( command );
    //15. Asignar el delay entre lecturas del mismo codigo de barras a 10 segundos (o mas): NLS0313010=10000
    command.setCommand ( "NLS0313010=10000" );
    barcodeCommandsSeq.push_back ( command );
    //16. Asignar el tipo de delay a completely: NLS0313030
    command.setCommand ( "NLS0313030" );
    barcodeCommandsSeq.push_back ( command );
    //17. Apagar el motor de vibracion: NLS1216000
    command.setCommand ( "NLS1216000" );
    barcodeCommandsSeq.push_back ( command );
    //18. Habilitar el codigo de barras QR: NLS0502020
    command.setCommand ( "NLS0502020" );
    barcodeCommandsSeq.push_back ( command );
    //19. Asignar el tamano del mensaje QR al maximo : NLS0502040
    command.setCommand ( "NLS0502040" );
    barcodeCommandsSeq.push_back ( command );
    //20. Indicar que la lectora solo debe aceptar codigos QR simples: NLS0502070
    command.setCommand ( "NLS0502070" );
    barcodeCommandsSeq.push_back ( command );
    BarcodeCommands barcodeCommands;
    barcodeCommands.setCommandBlockName ( "baseconfiguration" );
    barcodeCommands.setCommand ( barcodeCommandsSeq );

    /**
     * baseconfiguration-hr200
     */
    BarcodeCommands barcodeCommands1;
    barcodeCommands1.setCommandBlockName ( "baseconfiguration-hr200" );
    barcodeCommandsSeq.clear();
    command.setCommand ( "NLS0001010" );
    barcodeCommandsSeq.push_back ( command );
    command.setCommand ( "NLS0002000" );
    barcodeCommandsSeq.push_back ( command );
    command.setCommand ( "NLS0006000" );
    barcodeCommandsSeq.push_back ( command );
    command.setCommand ( "NLS0007000" );
    barcodeCommandsSeq.push_back ( command );
    command.setCommand ( "NLS0200030" );
    barcodeCommandsSeq.push_back ( command );
    command.setCommand ( "NLS0201000" );
    barcodeCommandsSeq.push_back ( command );
    command.setCommand ( "NLS0203010" );
    barcodeCommandsSeq.push_back ( command );
    command.setCommand ( "NLS0311000" );
    barcodeCommandsSeq.push_back ( command );
    command.setCommand ( "NLS0314000" );
    barcodeCommandsSeq.push_back ( command );
    command.setCommand ( "NLS0315000" );
    barcodeCommandsSeq.push_back ( command );
    command.setCommand ( "NLS0309000" );
    barcodeCommandsSeq.push_back ( command );
    command.setCommand ( "NLS0302000" );
    barcodeCommandsSeq.push_back ( command );
    command.setCommand ( "NLS0312030" );
    barcodeCommandsSeq.push_back ( command );
    command.setCommand ( "NLS0313000=0" );
    barcodeCommandsSeq.push_back ( command );
    command.setCommand ( "NLS0313010=10000" );
    barcodeCommandsSeq.push_back ( command );
    command.setCommand ( "NLS0313030" );
    barcodeCommandsSeq.push_back ( command );
    command.setCommand ( "NLS1216000" );
    barcodeCommandsSeq.push_back ( command );
    command.setCommand ( "NLS0502020" );
    barcodeCommandsSeq.push_back ( command );
    command.setCommand ( "NLS0502040" );
    barcodeCommandsSeq.push_back ( command );
    command.setCommand ( "NLS0502070" );
    barcodeCommandsSeq.push_back ( command );
    barcodeCommands1.setCommand ( barcodeCommandsSeq );

    /**
     * LightsON
     */
    BarcodeCommands barcodeCommands2;
    barcodeCommands2.setCommandBlockName ( "LightsON" );
    barcodeCommandsSeq.clear();
    command.setCommand ( "NLS0200030" );
    barcodeCommandsSeq.push_back ( command );
    command.setCommand ( "NLS0201000" );
    barcodeCommandsSeq.push_back ( command );
    barcodeCommands2.setCommand ( barcodeCommandsSeq );

    /**
     * LightsOFF
     */
    BarcodeCommands barcodeCommands3;
    barcodeCommands3.setCommandBlockName ( "LightsOFF" );
    barcodeCommandsSeq.clear();
    command.setCommand ( "nls0200020" );
    barcodeCommandsSeq.push_back ( command );
    command.setCommand ( "nls0201020" );
    barcodeCommandsSeq.push_back ( command );
    barcodeCommands3.setCommand ( barcodeCommandsSeq );

    BarcodeDeviceConfiguration::ConfigurationCommandsSequence confCommandsSeq;
    confCommandsSeq.push_back ( barcodeCommands );
    confCommandsSeq.push_back ( barcodeCommands1 );
    confCommandsSeq.push_back ( barcodeCommands2 );
    confCommandsSeq.push_back ( barcodeCommands3 );
    barcodeConf.setConfigurationCommands ( confCommandsSeq );

    barcodeConf.setModelFile ( "BarcodeModelFile" );
    barcodeConf.setSerialFile ( "BarcodeSerialFile" );
    barcodeConf.setCommandsFile ( "BarcodeCommandsFile" );
    barcodeConf.setBufferFile ( "BarcodeBufferFile" );

    AlarmBoxConfiguration alarmBoxConfiguration ( "/tmp/AlarmBoxPingFile", "/tmp/AlarmBoxButtonFile", "/tmp/AlarmBoxBehaviorFile" );

    configSAESHardware.setBarcodeDeviceConfiguration ( barcodeConf );
    configSAESHardware.setP168ADeviceConfiguration ( p168AConfig );
    configSAESHardware.setA4517AbledDeviceConfiguration( a4517AbledConfig );
    configSAESHardware.setPDBDeviceConfiguration( pdbConfig );
    configSAESHardware.setIOBDeviceConfiguration( iobConfig );
    configSAESHardware.setAlarmBoxConfiguration ( alarmBoxConfiguration );

    serializeSaesHardwareConfiguration ( ss, configSAESHardware, map, "UTF-8" );

    ConfigurationPersistenceManager::getInstance()->SetLibraryConfigFile ( "SAESHardware", ss.str() );
}

SAESHardwareConfigurationManager * SAESHardwareConfigurationManager::getInstance ( SAESHardwareConfigurationManager * externConfig )
{
    if ( ! instance )
    {
        instance = ( externConfig == NULL ? new SAESHardwareConfigurationManager () : externConfig );
    }

    return instance;
}
