/**
 * @file   UsbConnection.cxx
 * @brief  Implementation of UsbConnection class.
 * @author manuel.arguelles
 * @date   Aug 30, 2017
 */

#include "UsbConnection.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <System/Utils/StringUtils.hxx>
#include <stdexcept>
#include <iostream>
#include <sstream>
#include <iomanip>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

using namespace Smartmatic::Log;
using namespace Smartmatic::SAES::HW::Connections;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::System;

ISMTTLog * UsbConnection::m_logger = SMTTLogManager::GetLogger(
    "Smartmatic.SAES.HW.Connections.UsbConnection",
    GETTEXT_PACKAGE);

UsbConnection::UsbConnection(
    const std::string& vendor,
    const std::string& product,
    const int interface)
    : m_interface(interface)
    , m_context(NULL)
    , m_device(NULL)
    , m_product(0)
    , m_vendor(0)
    , m_timeout(2000)
{
    std::istringstream ssVendor(vendor);
    std::istringstream ssProduct(product);

    if (libusb_init(&m_context) != LIBUSB_SUCCESS)
    {
        m_logger->Debug("Failed to initialize libusb context");
        throw std::invalid_argument("Failed to initialize libusb context");
    }

    ssVendor  >> std::hex >> m_vendor;
    ssProduct >> std::hex >> m_product;
}

UsbConnection::~UsbConnection()
{
    if (m_device != NULL)
    {
        closeConnection();
    }

    libusb_exit(m_context);
}

bool UsbConnection::openConnection(const std::string&)
{
    bool result = true;

    if (m_device == NULL)
    {
        std::string message = "Openning connection to " +
                              StringUtils::intToString(m_vendor) + ":" +
                              StringUtils::intToString(m_product);

        m_logger->Debug(message);
        m_device = libusb_open_device_with_vid_pid(m_context, m_vendor, m_product);

        if (m_device != NULL)
        {
            if (!claimInterface())
            {
                m_logger->Debug("Error claiming interface.");
                closeConnection();
                result = false;
            }
        }
        else
        {
            m_logger->Debug("Error opening connection.");
            result = false;
        }
    }

    return result;
}

bool UsbConnection::claimInterface()
{
    bool result = false;

    if (m_device != NULL &&
        libusb_claim_interface(m_device, m_interface) == LIBUSB_SUCCESS)
    {
        result = true;
    }

    return result;
}


bool UsbConnection::sendCommand(const std::vector<unsigned char>& command)
{
    size_t size;

    m_logger->Debug("Sending: " + StringUtils::arrayHexToString(command, "0x", " "));
    size = transfer(0x01, const_cast<unsigned char*>(&command[0]), command.size());

    return (size > 0);
}

SafeVector<unsigned char> UsbConnection::readBuffer()
{
    size_t size;
    unsigned char buffer[1024];
    SafeVector<unsigned char> result;

    size = transfer(0x81, buffer, static_cast<int>(sizeof buffer));

    if (size > 0)
    {
        result.assign(buffer, buffer + size);
        m_logger->Debug("Received: " +
                        StringUtils::arrayHexToString(result, "0x", " "));
    }

    return result;
}

unsigned int UsbConnection::getTimeOut()
{
    return m_timeout;
}

void UsbConnection::setTimeOut(unsigned int timeout)
{
    m_timeout = timeout;
}

bool UsbConnection::closeConnection()
{
    bool result = false;

    if (m_device != NULL)
    {
        libusb_release_interface(m_device, m_interface);
        libusb_close(m_device);
        m_device = NULL;
        result = true;
    }

    return result;
}

size_t UsbConnection::transfer(uint8_t endpoint, unsigned char* data, int size)
{
    int transfered = 0;

    if (m_device != NULL)
    {
        int status = 0;

        status = libusb_bulk_transfer(
            m_device,
            endpoint,
            data,
            size,
            &transfered,
            m_timeout);

        if (status != LIBUSB_SUCCESS)
        {
            transfered = 0;
        }
    }

    return static_cast<size_t>(transfered);
}

bool UsbConnection::resetDevice()
{
    bool result = false;

    if (m_device != NULL)
    {
        result = (libusb_reset_device(m_device) == LIBUSB_SUCCESS);
    }

    return result;
}
