/**
 * @file	P168ADevice.cxx
 * @brief	Body P168A device class.
 * @author	Yadickson Soto
 * @date	30/06/2015
 */

#include "P168ADevice.hxx"
#include <Commands/P168AReadMemoryCommand.hxx>
#include <Commands/P168AWriteMemoryCommand.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <Operation/AbleDButtonState.hxx>
#include <System/Utils/StringUtils.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

using namespace Smartmatic::SAES::HW::Connections;
using namespace Smartmatic::SAES::HW::Devices;
using namespace Smartmatic::SAES::HW::Commands;
using namespace Smartmatic::SAES::HW::Operation;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::Log;

ISMTTLog * P168ADevice::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.HW.Devices.P168ADevice", GETTEXT_PACKAGE );

static AbleDButtonState::ButtonType getButtonType(const unsigned char value)
{
    AbleDButtonState::ButtonType type;

    switch (value)
    {
        case 0x31:
            type = AbleDButtonState::HELP;
            break;

        case 0x32:
            type = AbleDButtonState::SELECT;
            break;

        case 0x33:
            type = AbleDButtonState::UP;
            break;

        case 0x34:
            type = AbleDButtonState::DOWN;
            break;

        case 0x35:
            type = AbleDButtonState::RIGHT;
            break;

        case 0x36:
            type = AbleDButtonState::LEFT;
            break;

        case 0x55:
            type = AbleDButtonState::PAUSE;
            break;

        case 0x50:
            type = AbleDButtonState::SPEED_UP;
            break;

        case 0x51:
            type = AbleDButtonState::SPEED_DOWN;
            break;

        case 0xAE:
            type = AbleDButtonState::VOLUME_UP;
            break;

        case 0xAF:
            type = AbleDButtonState::VOLUME_DOWN;
            break;

        case 0x54:
            type = AbleDButtonState::ACTIVATE;
            break;

        default:
        case 0x00:
            type = AbleDButtonState::UNKNOW;
            break;
    }

    return type;
}

static AbleDButtonState::ButtonState getButtonState(const unsigned char value)
{
    AbleDButtonState::ButtonState state;

    switch (value)
    {
        case 0x01:
            state = AbleDButtonState::PRESS;
            break;

        case 0x02:
            state = AbleDButtonState::RELEASE;
            break;

        case 0x03:
            state = AbleDButtonState::HOLD;
            break;

        default:
        case 0x00:
            state = AbleDButtonState::NONE;
            break;
    }

    return state;
}

P168ADevice::P168ADevice ( IConnection * connection )
    : connection ( connection )
{
    buttonStatus = new AbleDButtonState();
}

P168ADevice::~P168ADevice()
{
    if ( connection )
    {
        delete ( connection );
        connection = NULL;
    }

    if ( buttonStatus )
    {
        delete ( buttonStatus );
        buttonStatus = NULL;
    }
}

bool P168ADevice::open ( const std::string & device )
{
    bool result;

    logger->Debug ( "Open p168a device" );
    result = connection->openConnection ( device );

    if (result)
    {
        enableVolumeControl();
    }

    return result;
}

bool P168ADevice::close()
{
    logger->Debug ( "Close p168a device" );
    return connection->closeConnection();
}

std::string P168ADevice::getSerialMachine()
{
    logger->Debug ( "Get serial machine" );

    std::string serial;

    P168AReadMemoryCommand read ( connection, 0x80, 0x00, 0x10 );

    if ( read.execute () )
    {
        const std::vector <unsigned char> & response ( read.getResponse () );
        serial = StringUtils::arrayHexToAsciiString ( response );
    }

    logger->Debug ( "Result : " + serial );
    return serial;
}

bool P168ADevice::setSerialMachine ( const std::string & serial )
{
    logger->Debug ( "Set serial machine : " + serial );

    std::vector <unsigned char> data;

    StringUtils::asciiStringToArrayHex ( serial, data );

    P168AWriteMemoryCommand write ( connection, 0x80, 0x00, data );

    return write.execute ();
}

bool P168ADevice::enableVolumeControl()
{
    logger->Debug ( "Enabling volume control" );

    std::vector <unsigned char> data;
    data.push_back(0x8A);

    P168AWriteMemoryCommand write ( connection, 0xC0, 0x00, data );

    return write.execute ();
}

Smartmatic::SAES::HW::Operation::AbleDButtonState * P168ADevice::getAbleDButtonState ()
{
    std::vector<unsigned char> data = connection->readBuffer();

    for ( unsigned int i = 0; i < data.size(); i += 5 )
    {
        if ( data[i + 0] == 0x1b && data[i + 1] == 0x64 && data[i + 2] == 0x02 )
        {
            AbleDButtonState::ButtonType  type   = getButtonType(data[i+3]);
            AbleDButtonState::ButtonState status = getButtonState(data[i+4]);

            logger->Debug ( "Button Event ( " + StringUtils::toHexString ( type, "", 2 ) + " - " + StringUtils::toHexString ( status, "", 2 ) + " )" );

            buttonStatus->updateStatus ( type, status );
        }
    }

    return buttonStatus;
}
