/**
 * @file	SimulatorBarcodeReaderDevice.cxx
 * @brief	Body of the simulator barcode reader device class.
 * @author	Yadickson Soto
 * @date	08/07/2015
 */

#include "SimulatorBarcodeReaderDevice.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <System/Utils/StringUtils.hxx>
#include <System/Utils/SafeBufferUtils.hxx>
#include <System/IO/File.h>
#include <System/Security/Cryptography/MsgDigestUtil.h>
#include <unistd.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

using namespace Smartmatic::SAES::HW::Devices;
using namespace Smartmatic::System::Security::Cryptography;
using namespace Smartmatic::Log;
using namespace Smartmatic::System;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::Utils;

ISMTTLog * SimulatorBarcodeReaderDevice::logger = SMTTLogManager::GetLogger ( "Smartmatic.SAES.HW.Devices.SimulatorBarcodeReaderDevice", GETTEXT_PACKAGE );

SimulatorBarcodeReaderDevice::SimulatorBarcodeReaderDevice (
    const std::string & modelFile,
    const std::string & serialFile,
    const std::string & commandFile,
    const std::string & bufferFile )
    : modelFile ( modelFile ), serialFile ( serialFile ), commandFile ( commandFile ), bufferFile ( bufferFile )
{

}

SimulatorBarcodeReaderDevice::~SimulatorBarcodeReaderDevice()
{

}

bool SimulatorBarcodeReaderDevice::open ( const std::string & device )
{
    logger->Debug ( "Simulator Open barcode reader device" );
    return File::getInstance()->canOpenFile ( device );
}

bool SimulatorBarcodeReaderDevice::close()
{
    logger->Debug ( "Simulator Close barcode reader device" );
    return true;
}

std::string SimulatorBarcodeReaderDevice::getDeviceModel()
{
    logger->Debug ( "Simulator Get device model from barcode reader device" );

    SafeBuffer data;
    std::string model;

    if ( File::getInstance()->openFileToBuffer ( modelFile, data ) && data.GetSize() > 0 )
    {
        SafeBufferUtils buffUtil ( data );
        std::vector < SafeBuffer > result = buffUtil.splitByEndLine();

        if ( ! result.empty() )
        {
            SafeBuffer item ( result.front() );
            model.assign ( ( const char * ) item.GetData(), item.GetSize() );
            model = StringUtils::trim ( model );
            model = StringUtils::toLower ( model );
        }
    }
    else
    {
        logger->Debug ( "Error load file " + modelFile );
    }

    return model;
}

std::string SimulatorBarcodeReaderDevice::getDeviceSerial(const bool)
{
    logger->Debug ( "Simulator Get serial number from barcode reader device" );

    SafeBuffer data;
    std::string serial;

    if ( File::getInstance()->openFileToBuffer ( serialFile, data ) && data.GetSize() > 0 )
    {
        SafeBufferUtils buffUtil ( data );
        std::vector < SafeBuffer > result = buffUtil.splitByEndLine();

        if ( ! result.empty() )
        {
            SafeBuffer item ( result.front() );
            serial.assign ( ( const char * ) item.GetData(), item.GetSize() );
            serial = StringUtils::trim ( serial );
        }
    }
    else
    {
        logger->Debug ( "Error load file " + serialFile );
    }

    return serial;
}

bool SimulatorBarcodeReaderDevice::setDeviceSerial ( const std::string & serial )
{
    logger->Debug ( "Simulator Set serial number (" + serial + ") to barcode reader device" );

    return File::getInstance()->saveBufferToFile ( SafeBuffer ( serial ), serialFile );
}

bool SimulatorBarcodeReaderDevice::sendReconfigureCommand ( const std::string & command,
        unsigned int timeout,
        unsigned int retries,
        bool mandatory )
{
    logger->Debug ( "Send command (" + command + ") to barcode reader device" );

    SafeBuffer data;
    bool result = false;

    do
    {
        if ( File::getInstance()->openFileToBuffer ( commandFile, data ) && data.GetSize() > 0 )
        {
            SafeBufferUtils buffUtil ( data );
            std::vector < SafeBuffer > commands = buffUtil.splitByEndLine();
            std::vector < SafeBuffer >::iterator it;

            for ( it = commands.begin();
                    it != commands.end ();
                    ++it )
            {
                std::string strCommand;
                std::string strStatus;

                SafeBufferUtils pair ( *it );

                std::vector < SafeBuffer > fields = pair.splitByBuffer ( SafeBuffer ( ":" ) );

                if ( fields.size() >= 2 )
                {
                    strCommand.assign ( ( const char * ) ( fields[0] ).GetData(), ( fields[0] ).GetSize() );
                    strStatus.assign ( ( const char * ) ( fields[1] ).GetData(), ( fields[1] ).GetSize() );
                }

                if ( command == strCommand )
                {
                    result = strStatus == "OK";
                    break;
                }
            }

        }
        else
        {
            logger->Debug ( "Error load file " + commandFile );
        }

        usleep ( timeout * 1000 );
        retries--;
    }
    while ( mandatory && !result && retries != 0 );

    return !mandatory || result;
}

SafeBuffer SimulatorBarcodeReaderDevice::readBuffer ()
{
    SafeBuffer data;

    if ( File::getInstance()->openFileToBuffer ( bufferFile, data ) )
    {
        SafeBuffer hash;
        MsgDigestUtil digest ( data );
        digest.getDigest ( hash, MsgDigestUtil::getDefaultFileDigest() );

        if ( hash == currentHash )
        {
            data.Clear();
        }
        else
        {
            currentHash = hash;
        }
    }
    else
    {
        data.Clear();
        currentHash.Clear();
    }

    return data;
}
