/**
 * @file	SimulatorP168ADevice.hxx
 * @brief	Declares the simulator p168a device class.
 * @author	Yadickson Soto
 * @date	07/06/2015
 */

#ifndef _SIMULATOR_P168A_DEVICE_H_
#define _SIMULATOR_P168A_DEVICE_H_

#include <Devices/P168AInterface.hxx>
#include <string>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace SAES
    {
        namespace HW
        {
            namespace Devices
            {
                /**
                 * @class	SimulatorP168ADevice
                 * @brief	Manage the simulator of the P168A device.
                 * @author	Yadickson Soto
                 * @date	07/06/2015
                 */
                class SimulatorP168ADevice : public P168AInterface
                {
                    public:

                        /**
                         * @fn	SimulatorP168ADevice( const std::string & serialFile, const std::string & buttonsFile );
                         * @brief	Default constructor.
                         * @author	Yadickson Soto
                         * @date	07/06/2015
                         * @param   serialFile The serial file path to read and write serial of the machine
                         * @param   buttonsFile The button file path to read able-d button status
                         */
                        SimulatorP168ADevice ( const std::string & serialFile, const std::string & buttonsFile );

                        /**
                         * @fn	virtual ~SimulatorP168ADevice()
                         * @brief	Class destroyer.
                         * @author	Yadickson Soto
                         * @date	07/06/2015
                         */
                        virtual ~SimulatorP168ADevice();

                        /**
                         * @fn	virtual bool open( const std::string & device );
                         * @brief	Opens the connections.
                         * @author	Yadickson Soto
                         * @date	07/06/2015
                         * @param 	device device path
                         * @return	true if successful.
                         */
                        virtual bool open ( const std::string & device );

                        /**
                         * @fn	virtual bool close();
                         * @brief	Closes the connections.
                         * @author	Yadickson Soto
                         * @date	07/06/2015
                         * @return	true if successful.
                         */
                        virtual bool close();

                        /**
                         * @fn	virtual std::string getSerialMachine();
                         * @brief	Method getter serial machine.
                         * @author	Yadickson Soto
                         * @date	07/06/2015
                         * @return	serial machine.
                         */
                        virtual std::string getSerialMachine();

                        /**
                         * @fn	virtual bool setSerialMachine ( const std::string & serial );
                         * @brief	Method setter serial machine.
                         * @author	Yadickson Soto
                         * @date	07/06/2015
                         * @param	serial	serial machine.
                         * return  true if success
                         */
                        virtual bool setSerialMachine ( const std::string & serial );

                        /**
                         * @brief  Enables able-d volume control.
                         * @author eduardo.fernandez
                         * @date   24/01/2018
                         * @return Whether the volume control was correctly enabled.
                         */
                        virtual bool enableVolumeControl();

                        /**
                         * @fn	virtual Smartmatic::SAES::HW::Operation::AbleButtonState * getAbleDButtonState();
                         * @brief	Method getter able-d button status.
                         * @author	Yadickson Soto
                         * @date	07/06/2015
                         * @return AbleD button status
                         */
                        virtual Smartmatic::SAES::HW::Operation::AbleDButtonState * getAbleDButtonState ();

                    private:

                        std::string serialFile; ///< The serial file path to read and write serial of the machine
                        std::string buttonsFile;	///< The button file path to read able-d button status
                        Smartmatic::SAES::HW::Operation::AbleDButtonState * buttonStatus; ///< The Able-D button status
                        static Smartmatic::Log::ISMTTLog * logger;  ///< The logger
                };
            }
        }
    }
}

#endif // _SIMULATOR_P168A_DEVICE_H_
