/**
 * @file	LanguageController.cxx
 * @brief	language controller class.
 * @author Yadickson Soto
 * @date 27/11/2013
 */

#include "LanguageController.hxx"
#include <Log/SMTTLogManager.h>
#include <Log/ISMTTLog.h>
#include <System/Runtime/ProcessLauncher.h>
#include <System/ZenityWrapper.hxx>
#include <System/Utils/StringUtils.hxx>
#include <System/Runtime/ExitProcess.hxx>
#include <clocale>
#include <sstream>
#include <stdlib.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

static const std::string ENV_LANGUAGE  = "LANGUAGE";
static const std::string ENV_LANG      = "LANG";
static const int         ENV_OVERWRITE = 1;

using namespace Smartmatic::Environment;
using namespace Smartmatic::Log;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::System::Runtime;
using namespace Smartmatic::System;

ISMTTLog * LanguageController::logger = SMTTLogManager::GetLogger ( "Smartmatic.Environment.LanguageController", GETTEXT_PACKAGE ); ///< The base language controller logger
LanguageController * LanguageController::instance = NULL;

LanguageController::LanguageController()
{
    flagLanguage = false;
}

LanguageController::~LanguageController()
{
    languagesSupports.clear();
    instance = NULL;
}

bool LanguageController::setLanguage ( const std::string & newLanguage, const std::string & newCountry )
{
    if ( newLanguage.empty() )
    {
        logger->Debug ( "ERROR - language requested" );
        return false;
    }

    currentCountry = newCountry.empty() ? newLanguage : newCountry;

    currentLanguage = StringUtils::toLower ( newLanguage );
    currentCountry = StringUtils::toUpper ( currentCountry );

    std::string fullLanguage = currentLanguage + "_" + currentCountry;

    loadLangSupports ();

    if ( languagesSupports.find ( fullLanguage + ".utf8" ) != languagesSupports.end() )
    {
        fullLanguage = fullLanguage + ".utf8";
    }
    else if ( languagesSupports.find ( fullLanguage ) == languagesSupports.end() )
    {
        ZenityWrapper::showWarning ( "Support language " + fullLanguage + " not found.", "Warning" );
        ExitProcess::getInstance()->exitProcess();
        return false;
    }

    std::string curLocale = getLangSystem ();

    if ( curLocale != fullLanguage )
    {
        std::stringstream ss;
        ss << "Changing Locale. From: ";
        ss << curLocale << " To: " << fullLanguage;
        logger->Debug ( ss.str() );
        setLangSystem ( fullLanguage );

        flagLanguage = true;
    }

    return true;
}

void LanguageController::loadLangSupports()
{
    if ( languagesSupports.empty() )
    {
        std::list < std::string > localeSupportList;
        localeSupportList = ProcessLauncher::getInstance()->RunProcessThroughPipe ( "locale -a" );
        std::list < std::string >::iterator it;

        for ( it = localeSupportList.begin();
                it != localeSupportList.end();
                ++it )
        {
            std::string locale = *it;
            languagesSupports.insert ( locale );
        }
    }
}

std::string LanguageController::getLangSystem()
{
    std::string curLocale;
    curLocale.append ( std::setlocale ( LC_ALL, NULL ) );
    return curLocale;
}

void LanguageController::setLangSystem ( const std::string & fullLanguage )
{
    setenv(ENV_LANG.c_str(), fullLanguage.c_str(), ENV_OVERWRITE);
    setenv(ENV_LANGUAGE.c_str(), fullLanguage.c_str(), ENV_OVERWRITE);
    std::setlocale ( LC_ALL, fullLanguage.c_str() );
}

bool LanguageController::isSetLang()
{
    return flagLanguage;
}

std::string LanguageController::getCurrentLanguage ()
{
    return currentLanguage;
}

std::string LanguageController::getCurrentCountry ()
{
    return currentCountry;
}

std::string LanguageController::getCurrentFullLanguage()
{
    return currentLanguage + "_" + currentCountry;
}

LanguageController * LanguageController::getInstance ( LanguageController * externController )
{
    if ( instance == NULL )
    {
        instance = ( externController == NULL ? new LanguageController() : externController );
    }

    return instance;
}
