/**
 * @file   MixerControlImpl.cxx
 * @brief  Implementation of MixerControlImpl class.
 * @author manuel.arguelles
 * @date   Dec 13, 2017
 */

#include "MixerControlImpl.hxx"
#include <System/Utils/StringUtils.hxx>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

using namespace Smartmatic::Log;
using namespace Smartmatic::System::Audio;
using namespace Smartmatic::System::Utils;

static const std::string MASTER_ELEMENT_NAME = "Master";
static const std::string PCM_ELEMENT_NAME    = "PCM";

ISMTTLog * MixerControlImpl::logger = SMTTLogManager::GetLogger ( "Smartmatic.System.Audio.MixerControlImpl", GETTEXT_PACKAGE );

/**
 * @brief  Gets the card id number of the first card containing the given name
 * within its name.
 * @author manuel.arguelles
 * @date   Jan 31, 2018
 * @return The id number or -1 if not found.
 */
static int getCardIdByName(const std::string& cardName)
{
    int index = -1;

    while (snd_card_next(&index) == 0 && index != -1)
    {
        char* name;

        if (snd_card_get_name(index, &name) == 0)
        {
            std::string fullName(name);

            free(name);

            if (fullName.find(cardName) != std::string::npos)
                break;
        }
    }

    return index;
}

/**
 * @brief  Gets the mixer device name of the card by id.
 * @author manuel.arguelles
 * @date   Jan 31, 2018
 * @param  id The id of the card.
 * @return The mixer device name.
 */
static std::string getMixerDeviceById(const int id)
{
    std::string device = "hw:" + StringUtils::intToString(id);

    return device;
}

/**
 * @brief  Gets the playback device name of the card by id.
 * @author manuel.arguelles
 * @date   Jan 31, 2018
 * @param  id The id of the card.
 * @return The playback device name.
 */
static std::string getPlaybackDeviceById(const int id)
{
    std::string device = "plughw:" + StringUtils::intToString(id);

    return device;
}

MixerControlImpl::MixerControlImpl(const std::string& cardName, const int defaultVolume)
    : m_mixer(NULL)
    , m_element(NULL)
    , m_volumeMax(0)
    , m_volumeMin(0)
    , m_defaultVolume(defaultVolume)
    , m_cardId(getCardIdByName(cardName))
{
    open();
}

MixerControlImpl::~MixerControlImpl()
{
    close();
}

void MixerControlImpl::open()
{
    std::string deviceName = getMixerDeviceById(m_cardId);

    if (snd_mixer_open(&m_mixer, 0) == 0 &&
        snd_mixer_attach(m_mixer, deviceName.c_str()) == 0 &&
        snd_mixer_load(m_mixer) == 0 &&
        snd_mixer_selem_register(m_mixer, NULL, NULL) == 0)
    {
        if (setVolumeElement(MASTER_ELEMENT_NAME) || setVolumeElement(PCM_ELEMENT_NAME))
        {
            snd_mixer_selem_get_playback_volume_range(m_element, &m_volumeMin, &m_volumeMax);

            logger->Debug("Max volume = " + StringUtils::longToString(m_volumeMax));
            logger->Debug("Min volume = " + StringUtils::longToString(m_volumeMin));

            setPlaybackVolume(m_defaultVolume);
        }
    }
    else
    {
        close();
    }
}

void MixerControlImpl::close()
{
    if (m_mixer != NULL)
    {
        snd_mixer_close(m_mixer);
        m_mixer = NULL;
        m_element = NULL;
    }
}

void MixerControlImpl::setPlaybackVolume(const int volume)
{
    long volumeInRange = volume;

    if (volume < m_volumeMin)
        volumeInRange = m_volumeMin;
    if (volume > m_volumeMax)
        volumeInRange = m_volumeMax;

    if (m_element != NULL)
    {
        if (snd_mixer_selem_set_playback_volume_all(m_element, volumeInRange) == 0)
        {
            m_volume = volumeInRange;
        }
        else
        {
            close();
            open();
        }
    }
    else
    {
        open();
    }
}

int MixerControlImpl::getPlaybackVolume()
{
    return m_volume;
}

bool MixerControlImpl::setVolumeElement(const std::string& name)
{
    bool                  result;
    snd_mixer_elem_t*     element;
    snd_mixer_selem_id_t* selemId;

    snd_mixer_selem_id_malloc(&selemId);
    snd_mixer_selem_id_set_index(selemId, 0);
    snd_mixer_selem_id_set_name(selemId, name.c_str());

    element = snd_mixer_find_selem(m_mixer, selemId);

    snd_mixer_selem_id_free(selemId);

    if (element != NULL)
    {
        m_element = element;
        result = true;
    }
    else
    {
        result = false;
    }

    return result;
}

std::string MixerControlImpl::getPlaybackDeviceName() const
{
    return getPlaybackDeviceById(m_cardId);
}
