/**
 * @file   NuanceVocalizerUtility.cxx
 * @brief  Implementation of utility / helper functions required by
 * Nuance Vocalizer engine.
 * @author manuel.arguelles
 * @date   Oct  7, 2017
 */

#include "NuanceVocalizerUtility.hxx"
#include <System/IO/Directory.h>
#include <System/IO/File.h>
#include <System/IO/Path.h>
#include <System/SafeBuffer.h>
#include <algorithm>
#include <cstdlib>
#include <cstdio>
#include <string>
#include <limits>
#include <vector>

using namespace Smartmatic::System::Audio::TextToSpeech;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System;

/**
 * @brief  Find filename recursively from the given path.
 * @author manuel.arguelles
 * @date   Oct  9, 2017
 * @param  path     The base path where to search.
 * @param  filename The name of the file to find.
 * @return A string with the full path of the file or an empty string if
 * the file was not found.
 */
static std::string findFile(const std::string& path, const std::string& filename)
{
    std::vector<std::string> directories;
    std::vector<std::string>::const_iterator it;

    if (Directory::getInstance()->FileExists(Path::Combine(path, filename)))
    {
        return Path::Combine(path, filename);
    }

    Directory::getInstance()->GetDirectories(path, directories);

    for (it = directories.begin(); it != directories.end(); ++it)
    {
        if (it->at(0) != '.')
        {
            std::string foundFile = findFile(Path::Combine(path, *it), filename);

            if (!foundFile.empty())
            {
                return foundFile;
            }
        }
    }

    return std::string();
}

/**
 * @brief  Concatenate the context of all the hdr file.
 * @author manuel.arguelles
 * @date   Oct  9, 2017
 * @param  path The base path where to search.
 * @return The concatenated context of all the hdr files found or and empty
 * string if no hdr files were found.
 */
static std::string loadHdrFiles(const std::string& path)
{
    std::string content;
    std::vector<std::string> files;
    std::vector<std::string>::const_iterator it;

    Directory::getInstance()->GetFiles(path, files);

    for (it = files.begin(); it != files.end(); ++it)
    {
        if (Path::GetFileExtension(Path::Combine(path, *it)) == ".hdr")
        {
            SafeBuffer data;

            if (File::getInstance()->openFileToBuffer(Path::Combine(path, *it), data))
            {
                content.append(data.GetData(), data.GetData() + data.GetSize());
            }
        }
    }

    return content;
}

std::string NuanceUtility::getBrokerInfo(const std::string& basePath)
{
    std::string brokerInfo = loadHdrFiles(basePath);
    std::vector<std::string> directories;
    std::vector<std::string>::const_iterator it;

    Directory::getInstance()->GetDirectories(basePath, directories);

    for (it = directories.begin(); it != directories.end(); ++it)
    {
        if (it->at(0) != '.')
        {
            std::string content = getBrokerInfo(Path::Combine(basePath,*it));

            brokerInfo += content;
        }
    }

    return brokerInfo;
}

void* NuanceUtility::heapMalloc(void* heap, std::size_t bytes)
{
    (void)heap;
    return std::malloc(bytes);
}

void* NuanceUtility::heapCalloc(void* heap, std::size_t elements, std::size_t elementBytes)
{
    (void)heap;
    return std::calloc(elements, elementBytes);
}

void* NuanceUtility::heapRealloc(void* heap, void* data, std::size_t bytes)
{
    (void)heap;
    return std::realloc(data, bytes);
}

void NuanceUtility::heapFree(void* heap, void* data)
{
    (void)heap;
    return std::free(data);
}

NUAN_ERROR NuanceUtility::fileOpen(void* data, void* heap, const char* name,
                                   const char* mode, void** stream)
{
    std::FILE* file;
    std::string fileName(name);
    std::string basePath;

    if (data != NULL)
    {
        basePath = *(static_cast<std::string*>(data));
    }

    std::replace(fileName.begin(), fileName.end(), '/', '_');
    fileName += ".dat";
    fileName = findFile(basePath, fileName);

    file = std::fopen(fileName.c_str(), mode);
    *stream = file;

    return (file == NULL) ? NUAN_E_COULDNOTOPENFILE : NUAN_OK;
}

NUAN_ERROR NuanceUtility::fileClose(void* stream)
{
    std::FILE* file = static_cast<std::FILE*>(stream);

    std::fclose(file);

    return NUAN_OK;
}

std::size_t NuanceUtility::fileRead(void* buffer, std::size_t elementBytes,
                                    std::size_t elements, void* stream)
{
    std::FILE* file = static_cast<std::FILE*>(stream);

    return std::fread(buffer, elementBytes, elements, file);
}

NUAN_ERROR NuanceUtility::fileSeek(void* stream, std::size_t offset,
                                   VE_STREAM_ORIGIN origin,
                                   VE_STREAM_DIRECTION direction)
{
    std::FILE* file   = static_cast<std::FILE*>(stream);
    NUAN_ERROR status = NUAN_OK;

    std::numeric_limits<long> limit;

    if (offset > static_cast<std::size_t>(limit.max()))
    {
        status = NUAN_E_FILESEEK;
    }
    else
    {
        int fromPosition;

        switch (origin)
        {
            case VE_STREAM_SEEK_SET:
                fromPosition = SEEK_SET;
                break;
            case VE_STREAM_SEEK_CUR:
                fromPosition = SEEK_CUR;
                break;
            case VE_STREAM_SEEK_END:
                fromPosition = SEEK_END;
                break;
            default:
                return NUAN_E_INVALIDARG;
                break;
        }

        if (std::fseek(file, static_cast<long>(offset), fromPosition) != 0)
        {
            status = NUAN_E_FILESEEK;
        }
    }

    return status;
}

std::size_t NuanceUtility::fileGetSize(void* stream)
{
    std::FILE* file = static_cast<std::FILE*>(stream);
    long current;
    long size;

    current = std::ftell(file);
    if (current == -1L)
    {
        return 0;
    }

    if (fseek(file, 0, SEEK_END) != 0)
    {
        return 0;
    }

    size = ftell(file);
    if (size == -1L)
    {
        return 0;
    }

    if (fseek(file, current, SEEK_SET) != 0)
    {
        return 0;
    }

    return static_cast<std::size_t>(size);
}

NUAN_ERROR NuanceUtility::fileError(void* stream)
{
    std::FILE* file = static_cast<std::FILE*>(stream);

    return (std::ferror(file) ? NUAN_E_FILEREADERROR : NUAN_OK);
}

std::size_t NuanceUtility::fileWrite(const void* buffer, std::size_t elementBytes,
                                     std::size_t elements, void* stream)
{
    std::FILE* file = static_cast<std::FILE*>(stream);

    return std::fwrite(buffer, elementBytes, elements, file);
}

