/**
 * @file	ZipCompression.cxx
 * @brief	Body zip compression class.
 * @author	Yadickson Soto
 * @date	08/11/2012
 */

#include "ZipCompression.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <zlib.h>
#include <string.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

using namespace Smartmatic::System::Compression;
using namespace Smartmatic::System;
using namespace Smartmatic::Log;

ISMTTLog * ZipCompression::logger = SMTTLogManager::GetLogger ( "Smartmatic.System.Compression.ZipCompression", GETTEXT_PACKAGE );

#define CHUNK 16384 ///< chunk size

ZipCompression::ZipCompression()
{
    logger->Debug ( "Create ZIP Compression Class" );
}

ZipCompression::~ZipCompression()
{

}

bool ZipCompression::Deflate ( Smartmatic::System::SafeBuffer & dataIn,
                               Smartmatic::System::SafeBuffer & dataOut )
{
    logger->Debug ( "Deflate ZIP Compression" );

    bool validate = false;
    int level = Z_BEST_COMPRESSION;
    int ret;
    z_stream strm;
    unsigned char opaque[CHUNK];
    unsigned char out[CHUNK];

    dataOut.Clear();

    strm.zalloc = ZipCompression::zipAllocFunc;
    strm.zfree = ZipCompression::zipFreeFunc;
    strm.opaque = opaque;

    ret = deflateInit ( &strm, level );

    if ( ret != Z_OK || dataIn.GetSize() == 0 )
    {
        return validate;
    }

    strm.avail_in = dataIn.GetSize();
    strm.next_in = ( unsigned char * ) dataIn.GetData();

    do
    {
        strm.avail_out = CHUNK;
        strm.next_out = out;

        deflate ( &strm, Z_FINISH );

        unsigned bytesReadyThisTime = CHUNK - strm.avail_out;

        if ( bytesReadyThisTime )
        {
            validate = true;
            dataOut.AddData ( out, bytesReadyThisTime );
        }

        out [0] = 0;
    }
    while ( strm.avail_out == 0 );

    ( void ) deflateReset ( &strm );
    ( void ) deflateEnd ( &strm );

    return validate;
}

bool ZipCompression::Inflate ( Smartmatic::System::SafeBuffer & dataIn,
                               Smartmatic::System::SafeBuffer & dataOut,
                               int & avail )
{
    logger->Debug ( "Inflate ZIP Compression" );

    bool validate = false;
    int ret;
    z_stream strm;
    unsigned char opaque[CHUNK];
    unsigned char out[CHUNK];
    avail = 0;

    dataOut.Clear();

    strm.zalloc = ZipCompression::zipAllocFunc;
    strm.zfree = ZipCompression::zipFreeFunc;
    strm.opaque = opaque;

    ret = inflateInit ( &strm );

    if ( ret != Z_OK || dataIn.GetSize() == 0 )
    {
        return validate;
    }

    strm.avail_in = dataIn.GetSize();
    strm.next_in = ( unsigned char * ) dataIn.GetData();

    do
    {
        strm.avail_out = CHUNK;
        strm.next_out = out;

        ret = inflate ( &strm, Z_SYNC_FLUSH );

        if ( ret < 0 )
        {
            validate = false;
            break;
        }

        unsigned bytesReadyThisTime = CHUNK - strm.avail_out;

        if ( bytesReadyThisTime )
        {
            validate = true;
            dataOut.AddData ( out, bytesReadyThisTime );
        }
    }
    while ( strm.avail_out == 0 );

    ( void ) inflateReset ( &strm );
    ( void ) inflateEnd ( &strm );

    if ( !validate )
    {
        avail = 0;
        dataOut.Clear();
    }
    else
    {
        avail = strm.avail_in;
    }

    return validate;
}

void * ZipCompression::zipAllocFunc ( void * opaque, unsigned int items, unsigned int size )
{
    int msize = items * size;
    void * point = new unsigned char [msize];
    memset ( point, 0, msize );
    return point;
}

void ZipCompression::zipFreeFunc ( void * opaque, void * address )
{
    delete [] static_cast<unsigned char*>(address);
}
