/**
 * @file Directory.cc
 * @brief Body class Directory
 */

/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * smartmatic
 * Copyright (C) Freddy Gomez 2011 <fgomez@smartmatic.com>
 *
 * smartmatic is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * smartmatic is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "Directory.h"
#include "Path.h"
#include "File.h"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <sys/stat.h>
#include <dirent.h>
#include <string.h>
#include <sstream>
#include <fstream>
#include <stdlib.h>
#include <ftw.h>
#include <cerrno>
#include <stdio.h>
#include <unistd.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::System::IO;
using namespace Smartmatic::Log;

ISMTTLog * Directory::logger = SMTTLogManager::GetLogger ( "Smartmatic.System.IO.Directory", GETTEXT_PACKAGE );
Directory * Directory::instance = NULL;

static std::vector<std::string> getDirectoryContent(const std::string& path, unsigned char type)
{
    std::vector<std::string> result;
    struct dirent** inputs = NULL;
    int num = scandir(path.c_str(), &inputs, NULL, NULL);

    for (int i = 0; i < num; ++i)
    {
        if (inputs[i]->d_type == type)
        {
            result.push_back(inputs[i]->d_name);
        }

        free(inputs[i]);
        inputs[i] = NULL;
    }

    if (inputs)
    {
        free(inputs);
    }

    return result;
}

Directory::Directory()
{

}

Directory::~Directory()
{
    instance = NULL;
}

void Directory::GetFiles ( const std::string & path, std::vector<std::string> & list )
{
    list = getDirectoryContent(path, DT_REG);
}

void Directory::GetDirectories ( const std::string & path, std::vector<std::string> & list )
{
    list = getDirectoryContent(path, DT_DIR);
}

bool Directory::DirectoryExists ( const std::string & path )
{
    if ( path.empty() )
    {
        return false;
    }

    struct stat sb;

    if ( stat ( path.c_str(), &sb ) == -1 )
    {
        return false;
    }

    return ( ( sb.st_mode & S_IFMT ) == S_IFDIR );
}

bool Directory::Delete ( const std::string & path )
{
    return nftw ( path.c_str(), executeRm, 16, FTW_DEPTH ) == 0;
}

bool Directory::Mkdir ( const std::string & path )
{
    bool validate = true;

    if ( !DirectoryExists ( path ) )
    {
        logger->Debug ( "Create path " + path );

        std::string s = path;
        size_t pre = 0, pos;
        std::string dir;

        if ( s[s.size() - 1] != Path::getPathSeparator() )
        {
            // force trailing / so we can handle everything in loop
            s += Path::getPathSeparator();
        }

        while ( ( pos = s.find_first_of ( Path::getPathSeparator(), pre ) ) != std::string::npos )
        {
            dir = s.substr ( 0, pos++ );
            pre = pos;

            if ( dir.size() == 0 )
            {
                continue;    // if leading / first time is 0 length
            }

            if ( !executeMkdir ( dir ) )
            {
                validate = false;
                break;
            }
        }
    }

    return validate;
}

bool Directory::FileExists ( const std::string & path )
{
    if ( path.empty() )
    {
        return false;
    }

    std::ifstream ifile ( path.c_str() );
    return ifile.is_open();
}

bool Directory::addAllPermission ( const std::string & folder )
{
    bool validate = true;

    if ( DirectoryExists ( folder ) )
    {
        logger->Debug ( "Add all permission : " + folder );
        validate = nftw ( folder.c_str(), executeChmod, 16, FTW_DEPTH ) == 0;
    }

    return validate;
}

int Directory::executeRm ( const char * path, const struct stat * s, int flag, struct FTW * f )
{
    int ( *rm_func ) ( const char * );

    switch ( flag )
    {
        default:
            rm_func = unlink;
            break;

        case FTW_DP:
            rm_func = rmdir;
            break;
    }

    return rm_func ( path );
}

int Directory::executeChmod ( const char * path, const struct stat * s, int flag, struct FTW * f )
{
    switch ( flag )
    {
        default:
            return 0;

        case FTW_F:
        case FTW_DP:
            return chmod ( path, 0777 );
    }
}

bool Directory::executeMkdir ( const std::string & dir )
{
    return ! ( mkdir ( dir.c_str(), 0777 ) != 0 && errno != EEXIST );
}

std::string Directory::getCurrentDirectory ()
{
    char cCurrentPath[FILENAME_MAX];

    std::string currentPath;

    if ( getcwd ( cCurrentPath, sizeof ( cCurrentPath ) ) != NULL )
    {
        currentPath.append ( cCurrentPath );
    }

    return currentPath;
}

Directory * Directory::getInstance ( Directory * externDirectory )
{
    if ( instance == NULL )
    {
        instance = ( externDirectory == NULL ? new Directory() : externDirectory );
    }

    return instance;
}
