/**
 * @file OptParser.hxx
 * @brief Header class OptParser
 * @author Yadickson Soto
 * @date 23/08/2011
 */

#ifndef OPTPARSER_HXX_
#define OPTPARSER_HXX_

#include <string>
#include <map>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace System
    {
        /**
         * @class	OptParser
         * @brief	Class for parser parameters.
         * @author	Yadickson Soto
         * @date	23/08/2011
         */
        class OptParser
        {
            private:

                /**
                 * @fn	OptParser::OptParser();
                 * @brief	Class constructor.
                 * @author	Yadickson Soto
                 * @date	13/05/2013
                 */
                OptParser ();

                static OptParser * instance; ///< instance of the class

            public:

                /**
                 * @fn	virtual ~OptParser ();
                 * @brief	Class destructor.
                 * @author	Yadickson Soto
                 * @date	13/05/2013
                 */
                virtual ~OptParser ();

                /**
                 * @enum	Options
                 * @brief	Options available.
                 */
                enum Options
                {
                    CONFIG = 'c', ///< Config parameters
                    SHARE_DIR = 's', ///< Data directory path parameters (DATADIR default)
                    LOCALE_DIR = 'o', ///< Locale mo files directory path parameters (GNOMELOCALEDIR default)
                    LOGGER = 'g', ///< Logger parameters
                    XSLFO = 'x', ///< xslfo file
                    MODE = 'm', ///< mode start machine
                    LOGIN = 'l', ///< login machine
                    PASSWORD = 'p', ///< password machine
                    DISABLE_BID_SID_VALIDATE = 'd', ///< disable ballot id and session id validate
                    DISABLE_SAVE_VOTE = 'e', ///< disable save vote
                    DBUS_INFO = 'i', ///< view dbus information
                    HELP = 'h', ///< Help option
                    VERS = 'v', ///< Version option
                    MACHINE_MODEL = 't' ///< Machine model option
                };

                /**
                 * @fn	void initialize ( int argc, char **argv );
                 * @brief	Initialize class.
                 * @author	Yadickson Soto
                 * @date	01/08/2012
                 * @param	argc			Count arg.
                 * @param	argv			Vector of data.
                 */
                void initialize ( int argc, char ** argv );
                /**
                 * @fn	bool OptParser::isSet(Options opt);
                 * @brief	Method getter is option enabled.
                 * @author	Yadickson Soto
                 * @date	23/08/2011
                 * @param	opt	option to evaluate.
                 * @return	true option is enable.
                 */
                bool isSet ( Options opt );
                /**
                 * @fn	bool OptParser::getArg(Options opt, std::string & str);
                 * @brief	Method getter option value.
                 * @author	Yadickson Soto
                 * @date	23/08/2011
                 * @param	opt	option to evaluate.
                 * @param	str	in, out option value string.
                 * @return	true if result != empty and option required argument.
                 */
                bool getArg ( Options opt, std::string & str );
                /**
                 * @fn	void OptParser::runApplicationProcess(const std::string & package, std::string basepath);
                 * @brief	Method process args.
                 * @author	Yadickson Soto
                 * @date	01/08/2012
                 * @param	package 	package name.
                 * @param	basepath	base path config dir.
                 */
                void runApplicationProcess ( const std::string & package, std::string basepath );
                /**
                 * @fn	bool OptParser::getHelpOption(std::string & help);
                 * @brief	Method getter option help value.
                 * @author	Yadickson Soto
                 * @date	09/02/2012
                 * @param	help	in,out string.
                 * @return	true is help option enable.
                 */
                bool getHelpOption ( std::string & help );
                /**
                 * @fn	bool getLoginOption ( std::string & login );
                 * @brief	Getter Method: option login value.
                 * @author	Yadickson Soto
                 * @date	13/05/2013
                 * @param	login	in,out stringoptarg.
                 * @return	true is login option enable.
                 */
                bool getLoginOption ( std::string & login );
                /**
                 * @fn	bool getPasswordOption ( std::string & password );
                 * @brief	Getter Method: option password value.
                 * @author	Yadickson Soto
                 * @date	13/05/2013
                 * @param	password	in,out string.
                 * @return	true is password option enable.
                 */
                bool getPasswordOption ( std::string & password );
                /**
                 * @fn	static OptParser * getInstance();
                 * @brief	Getter Method: instance of class.
                 * @author	Yadickson Soto
                 * @date	13/05/2013
                 * @return	instance of class.
                 */
                static OptParser * getInstance();

            protected:

                /**
                 * @fn	void OptParser::setMap(Options opt);
                 * @brief	Method set is option enabled.
                 * @author	Yadickson Soto
                 * @date	09/06/2015
                 * @param	opt	option to evaluate.
                 */
                void setMap ( Options opt );

            private:

                int argc; ///< Count arg
                char ** argv; ///< Vector of data
                std::string shortopts; ///< The short options
                static Smartmatic::Log::ISMTTLog * logger; ///< The logger
                std::map < char, std::string > mapOptions; ///< The full map options

                /**
                 * @fn	bool OptParser::getStringOption(Options opt, std::string & str);
                 * @brief	Method getter option value.
                 * @author	Yadickson Soto
                 * @date	09/02/2012
                 * @param	opt	option to evaluate.
                 * @param	str	in,out string.
                 * @return	true is option enable.
                 */
                bool getStringOption ( Options opt, std::string & str );
        };
    }
}

#endif /* OPTPARSER_HXX_ */
