/**
 * @file   IStorageDevice.hxx
 * @brief  Declaration of IStorageDevice interface.
 * @author manuel.arguelles
 * @date   Aug  8, 2017
 */

#ifndef I_STORAGE_DEVICE_HXX
#define I_STORAGE_DEVICE_HXX

#include <string>

namespace Smartmatic
{
    namespace System
    {
        namespace Runtime
        {
            /**
             * @class  IStorageDevice.
             * @brief  Interface for storage devices.
             * @author manuel.arguelles
             * @date   Aug  8, 2017
             */
            class IStorageDevice
            {
                public:

                    /**
                     * @brief  Finalize an instance of IStorageDevice.
                     * @author manuel.arguelles
                     * @date   Aug  8, 2017
                     */
                    virtual ~IStorageDevice() {}

                public:

                    /**
                     * @brief  Check if the device is mounted.
                     * @author manuel.arguelles
                     * @date   Aug  8, 2017
                     * @return True if mounted, false if not.
                     */
                    virtual bool isMounted() const = 0;

                    /**
                     * @brief  Mount the device.
                     * @author manuel.arguelles
                     * @date   Aug  8, 2017
                     * @return True if it was mounted, false if not.
                     */
                    virtual bool mount() = 0;

                    /**
                     * @brief  Unmount the device.
                     * @author manuel.arguelles
                     * @date   Aug  8, 2017
                     * @return True if the device was unmounted, false if not.
                     */
                    virtual bool unmount() = 0;

                    /**
                     * @brief  Gets the path were the device is mounted.
                     * @author manuel.arguelles
                     * @date   Aug  8, 2017
                     * @return The mount point of the device.
                     */
                    virtual std::string getMountPoint() const = 0;

                    /**
                     * @brief  Gets the device path.
                     * @author manuel.arguelles
                     * @date   Aug  8, 2017
                     * @return The device path.
                     */
                    virtual std::string getDevicePath() const = 0;

                    /**
                     * @brief  Checks if the storage device belong to the same
                     *         device.
                     * @author manuel.arguelles
                     * @date   Sep  6, 2017
                     * @param  other The other StorageDevice to compare.
                     * @return True if both storage devices belong to the same
                     *         device, false if not.
                     */
                    virtual bool isSameDevice(const IStorageDevice& other) const = 0;
            };

        } /* namespace Runtime */
    } /* namespace System */
} /* namespace Smartmatic */

#endif /* I_STORAGE_DEVICE_HXX */
