/**
 * @file   SorageDevice.cxx
 * @brief  Implementation of SotrageDevice class
 * @author manuel.arguelles
 * @date   Aug  2, 2017
 */

#include "StorageDevice.hxx"

#include <libudev.h>
#include <mntent.h>
#include <stdio.h>
#include <sys/mount.h>
#include <errno.h>
#include <stdexcept>
#include <sigc++/sigc++.h>

using namespace Smartmatic::System::Runtime;

static const std::string   DEVNAME            = "DEVNAME";
static const std::string   DEV_NUMBERS        = "0123456789";
static const std::string   ID_FS_TYPE         = "ID_FS_TYPE";
static const std::string   ID_PART_ENTRY_TYPE = "ID_PART_ENTRY_TYPE";
static const std::string   MOUNT_TABLE        = "/proc/mounts";
static const std::string   VFAT_FS_TYPE       = "vfat";
static const std::string   VFAT_PARTITION_ID  = "0xb";
static const std::string   UNIX_DEVICE_ID     = "unix-device";
static const unsigned long ONE_MILLISECOND    = 1000;
static const unsigned long MAX_TIMEOUT        = 5000;

/**
 * @brief  Queries the mount table to get the path where the device is mounted.
 * @author manuel.arguelles
 * @date   Sep  6, 2017
 * @param  devicePath The device path to check.
 * @return The path where the device is mounted.
 */
static std::string getPathFromMountTable(const std::string& devicePath)
{
    std::string mountedPath;
    FILE* mountTable = setmntent(MOUNT_TABLE.c_str(), "r");

    if (mountTable != NULL)
    {
        struct mntent* mountEntity = NULL;

        while ((mountEntity = getmntent(mountTable)) != NULL)
        {
            if (mountEntity->mnt_fsname &&
                devicePath.compare(mountEntity->mnt_fsname) == 0)
            {
                mountedPath = mountEntity->mnt_dir;
                break;
            }
        }

        endmntent(mountTable);
    }

    return mountedPath;
}

/**
 * @brief  Checks if two device paths belong to the same device.
 * @author manuel.arguelles
 * @date   Sep  6, 2017
 * @param  firstDevicePath  The first device path to compare with.
 * @param  secondDevicePath The second device path to compare with.
 * @return True if both devices paths belong to the same device, false if not.
 */
static bool devicesEqual(const std::string& firstDevicePath, const std::string& secondDevicePath)
{
    std::string::size_type devicePosition;
    std::string firstDevice;
    std::string secondDevice;

    devicePosition = firstDevicePath.find_last_not_of(DEV_NUMBERS);

    if (devicePosition != std::string::npos)
    {
        ++devicePosition;
    }

    try
    {
        firstDevice = firstDevicePath.substr(0, devicePosition);
        secondDevice = secondDevicePath.substr(0, devicePosition);
    }
    catch (const std::out_of_range& exception)
    {
        return false;
    }

    return firstDevice == secondDevice;
}

/**
 * @brief  Gets the volume from the gvfs.
 * @author manuel.arguelles
 * @date   Sep 14, 2017
 * @param  devicePath The device path to find.
 * @return The volume of the given device path or null if not found.
 */
static Glib::RefPtr<Gio::Volume> getGvfsVolume(const std::string& devicePath)
{
    Glib::RefPtr<Gio::VolumeMonitor>                        monitor;
    Glib::RefPtr<Gio::Volume>                               result;
    std::vector<Glib::RefPtr<Gio::Volume> >                 volumes;
    std::vector<Glib::RefPtr<Gio::Volume> >::const_iterator it;

    monitor = Gio::VolumeMonitor::get();
    volumes = monitor->get_volumes();

    for (it = volumes.begin(); it != volumes.end(); ++it)
    {
        std::string path = (*it)->get_identifier(UNIX_DEVICE_ID);
        if (path == devicePath)
        {
            result = *it;
            break;
        }
    }

    return result;
}

StorageDevice::StorageDevice(const std::string& devicePath)
    : m_devicePath(devicePath)
{
    m_isReady = true;
}

StorageDevice::~StorageDevice()
{
}

std::vector<StorageDevice> StorageDevice::getAllDevicesByFileSystem(const std::string& fileSystem)
{
    std::vector<StorageDevice> result;
    struct udev*               udevContext;
    struct udev_enumerate*     udevList;
    struct udev_list_entry*    matchedList;
    struct udev_list_entry*    matchedEntry = NULL;

    udevContext = udev_new();
    udevList = udev_enumerate_new(udevContext);

    udev_enumerate_add_match_property(udevList, ID_FS_TYPE.c_str(), fileSystem.c_str());
    udev_enumerate_scan_devices(udevList);

    matchedList = udev_enumerate_get_list_entry(udevList);

    udev_list_entry_foreach(matchedEntry, matchedList)
    {
        const char*         deviceName  = udev_list_entry_get_name(matchedEntry);
        struct udev_device* device      = udev_device_new_from_syspath(udevContext, deviceName);
        const char*         devicePath  = udev_device_get_property_value(device, DEVNAME.c_str());
        const char*         partitionId = udev_device_get_property_value(device, ID_PART_ENTRY_TYPE.c_str());
        StorageDevice matchingDevice(devicePath);

        if (fileSystem != VFAT_FS_TYPE || VFAT_PARTITION_ID.compare(partitionId) == 0)
        {
            result.push_back(matchingDevice);
        }

        udev_device_unref(device);
    }

    udev_enumerate_unref(udevList);
    udev_unref(udevContext);

    return result;
}

std::vector<StorageDevice> StorageDevice::getUniqueDevicesByFileSystem(const std::string& fileSystem)
{
    std::vector<StorageDevice> result;
    std::vector<StorageDevice> allDevices;
    std::vector<StorageDevice>::const_iterator it;

    allDevices = StorageDevice::getAllDevicesByFileSystem(fileSystem);

    for (it = allDevices.begin(); it != allDevices.end(); ++it)
    {
        bool uniqueDevice = true;
        std::vector<StorageDevice>::const_iterator uniqueIt;

        for (uniqueIt = result.begin(); uniqueIt != result.end(); ++uniqueIt)
        {
            if (uniqueIt->isSameDevice(*it))
            {
                uniqueDevice = false;
                break;
            }
        }

        if (uniqueDevice)
        {
            result.push_back(*it);
        }
    }

    return result;
}

bool StorageDevice::isMounted() const
{
    std::string mountPoint = getPathFromMountTable(m_devicePath);

    return !mountPoint.empty();
}

void StorageDevice::mountOperationReady(const Glib::RefPtr<Gio::AsyncResult>& result)
{
    if (m_volume)
    {
        m_volume->mount_finish(result);
    }

    m_isReady = true;
}

void StorageDevice::unmountOperationReady(const Glib::RefPtr<Gio::AsyncResult>& result)
{
    if (m_mount)
    {
        m_mount->unmount_finish(result);
    }

    m_isReady = true;
}

bool StorageDevice::mount()
{
    Glib::RefPtr<Gio::Cancellable> cancelOperation(Gio::Cancellable::create());
    m_volume  = getGvfsVolume(m_devicePath);
    m_isReady = true;

    if (m_volume)
    {
        Glib::RefPtr<Gio::MountOperation> mountOperation;

        m_isReady = false;
        m_volume->mount(mountOperation, sigc::mem_fun(*this, &StorageDevice::mountOperationReady), cancelOperation);
    }

    waitUntilReady(cancelOperation);

    return isMounted();
}

bool StorageDevice::unmount()
{
    Glib::RefPtr<Gio::Cancellable> cancelOperation(Gio::Cancellable::create());
    m_volume  = getGvfsVolume(m_devicePath);
    m_isReady = true;

    if (m_volume)
    {
        m_mount = m_volume->get_mount();

        if (m_mount)
        {
            m_isReady = false;
            m_mount->unmount(sigc::mem_fun(*this, &StorageDevice::unmountOperationReady), cancelOperation, Gio::MOUNT_UNMOUNT_FORCE );
        }
    }

    waitUntilReady(cancelOperation);

    return !isMounted();
}

void StorageDevice::waitUntilReady(Glib::RefPtr<Gio::Cancellable>& cancellable) const
{
    size_t count = 0;

    while (!m_isReady && count < MAX_TIMEOUT)
    {
        usleep(ONE_MILLISECOND);
        ++count;
    }

    if (!m_isReady)
        cancellable->cancel();
}

std::string StorageDevice::getMountPoint() const
{
    return getPathFromMountTable(m_devicePath);
}

std::string StorageDevice::getDevicePath() const
{
    return m_devicePath;
}

bool StorageDevice::isSameDevice(const IStorageDevice& other) const
{
    return devicesEqual(getDevicePath(), other.getDevicePath());
}
