/**
 * @file Crypto.cxx
 * @brief Body class for Crypto file
 */

#include "Crypto.hxx"
#include "System/Security/Cryptography/CryptoAES.hxx"
#include <openssl/evp.h>
#include <openssl/rand.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <System/IO/File.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///< macro for gettext
#else
#define _(String) (String) ///< macro without gettext
#endif

#define N_(String) (String) ///< macro not translate

using namespace Smartmatic::System::Security;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System;
using namespace Smartmatic::Log;

ISMTTLog * Crypto::logger = SMTTLogManager::GetLogger ( "Smartmatic.System.Security.Cryptography.Crypto", GETTEXT_PACKAGE );

Crypto::Crypto()
    : cryptoAES(NULL)
    , mode(Crypto::MODE_AES_CBC)
    , pad(true)
    , keySize(Crypto::KEY_AES_256)
    , ivSize(0)
{
}

Crypto::~Crypto()
{
    if ( cryptoAES )
    {
        delete cryptoAES;
        cryptoAES = NULL;
    }
}

void Crypto::init ( Crypto::KeySize keySize, Crypto::Mode mode, bool pad ) throw ( CryptoException )
{
    this->mode = mode;
    this->pad = pad;
    this->keySize = keySize;

    if ( cryptoAES )
    {
        delete cryptoAES;
        cryptoAES = NULL;
    }

    switch ( keySize )
    {
        case KEY_AES_128:
            configCryptoAESKey128();
            break;

        case KEY_AES_192:
            configCryptoAESKey192();
            break;

        default:
        case KEY_AES_256:
            configCryptoAESKey256();
            break;
    }
}

void Crypto::generateRandomIV(Smartmatic::System::SafeBuffer & iv) throw ( Smartmatic::System::Exception::CryptoException )
{
    iv.Clear();
    unsigned char ivRaw[EVP_MAX_IV_LENGTH];
    if (RAND_bytes(ivRaw, ivSize) != 1)
    {
        logger->Debug ( "ERROR - generating IV " );
        throw CryptoException ( _ ( "Smartmatic.System.Security.Crypto.ErrorGeneratingIV" ), N_ ( "Smartmatic.System.Security.Crypto.ErrorGeneratingIV" ), logger );
    }
    iv.AddData ( ivRaw, ivSize );
}

void Crypto::generateZeroIV(SafeBuffer& iv)
{
    iv.Clear();
    unsigned char ivRaw[EVP_MAX_IV_LENGTH];

    for (size_t i = 0; i < EVP_MAX_IV_LENGTH; ++i)
    {
        ivRaw[i] = 0x00;
    }

    iv.AddData(ivRaw, ivSize);
}

void Crypto::encryptBufferToFileAndCreateIV ( const SafeBuffer & dataIn, const std::string & file, const unsigned char * key) throw ( CryptoException )
{
    SafeBuffer encryptData;
    encryptBufferToBufferAndCreateIV ( dataIn, encryptData, key);

    if ( !File::getInstance()->saveBufferToFile ( encryptData, file ) )
    {
        logger->Debug ( "ERROR - write file " + file );
        throw CryptoException ( _ ( "Smartmatic.System.Security.Crypto.ErrorWriteFile" ), N_ ( "Smartmatic.System.Security.Crypto.ErrorWriteFile" ), logger );
    }
}

void Crypto::encryptFileToBufferAndCreateIV ( const std::string & fileIn, SafeBuffer & dataOut, const unsigned char * key) throw ( CryptoException )
{
    SafeBuffer bufferFile;

    if ( File::getInstance()->openFileToBuffer ( fileIn, bufferFile ) )
    {
        encryptBufferToBufferAndCreateIV ( bufferFile, dataOut, key);
    }
    else
    {
        logger->Debug ( "ERROR - open file " + fileIn );
        throw CryptoException ( _ ( "Smartmatic.System.Security.Crypto.ErrorOpenFile" ), N_ ( "Smartmatic.System.Security.Crypto.ErrorOpenFile" ), logger );
    }
}

void Crypto::encryptFileToFileAndCreateIV ( const std::string & fileIn, const std::string & fileOut, const unsigned char * key) throw ( CryptoException )
{
    SafeBuffer bufferFile;

    if ( File::getInstance()->openFileToBuffer ( fileIn, bufferFile ) )
    {
        encryptBufferToFileAndCreateIV ( bufferFile, fileOut, key);
    }
    else
    {
        logger->Debug ( "ERROR - open file " + fileIn );
        throw CryptoException ( _ ( "Smartmatic.System.Security.Crypto.ErrorOpenFile" ), N_ ( "Smartmatic.System.Security.Crypto.ErrorOpenFile" ), logger );
    }
}

void Crypto::encryptBufferToBufferAndCreateIV ( const SafeBuffer & dataIn, SafeBuffer & dataOut, const unsigned char * key, bool useCbcMac) throw ( CryptoException )
{
    if ( cryptoAES && dataIn.GetSize() > 0 )
    {
        SafeBuffer iv;
        useCbcMac ? generateZeroIV(iv) : generateRandomIV(iv);
        cryptoAES->encryptData ( dataIn, dataOut, key, iv.GetData(), pad );
        useCbcMac ? dataOut.removeData(0, dataOut.GetSize() - iv.GetSize()) : dataOut.insertData(0, iv);
    }
    else
    {
        throw CryptoException ( _ ( "Smartmatic.System.Security.Crypto.ErrorEncryptBuffer" ), N_ ( "Smartmatic.System.Security.Crypto.ErrorEncryptBuffer" ), logger );
    }
}

void Crypto::decryptFileToBufferAndGetIV ( const std::string & file, SafeBuffer & dataOut, const unsigned char * key) throw ( CryptoException )
{
    SafeBuffer bufferFile;

    if ( File::getInstance()->openFileToBuffer ( file, bufferFile ) )
    {
        decryptBufferToBufferAndGetIV ( bufferFile, dataOut, key );
    }
    else
    {
        logger->Debug ( "ERROR - open file " + file );
        throw CryptoException ( _ ( "Smartmatic.System.Security.Crypto.ErrorOpenFile" ), N_ ( "Smartmatic.System.Security.Crypto.ErrorOpenFile" ), logger );
    }
}

void Crypto::decryptBufferToFileAndGetIV ( const SafeBuffer & dataIn, const std::string & file, const unsigned char * key) throw ( CryptoException )
{
    SafeBuffer encryptData;
    decryptBufferToBufferAndGetIV ( dataIn, encryptData, key );

    if ( !File::getInstance()->saveBufferToFile ( encryptData, file ) )
    {
        logger->Debug ( "ERROR - write file " + file );
        throw CryptoException ( _ ( "Smartmatic.System.Security.Crypto.ErrorWriteFile" ), N_ ( "Smartmatic.System.Security.Crypto.ErrorWriteFile" ), logger );
    }
}

void Crypto::decryptFileToFileAndGetIV ( const std::string & fileIn, const std::string & fileOut, const unsigned char * key) throw ( CryptoException )
{
    SafeBuffer encryptData;
    decryptFileToBufferAndGetIV ( fileIn, encryptData, key );

    if ( !File::getInstance()->saveBufferToFile ( encryptData, fileOut ) )
    {
        logger->Debug ( "ERROR - write file " + fileIn );
        throw CryptoException ( _ ( "Smartmatic.System.Security.Crypto.ErrorWriteFile" ), N_ ( "Smartmatic.System.Security.Crypto.ErrorWriteFile" ), logger );
    }
}

void Crypto::decryptBufferToBufferAndGetIV ( SafeBuffer  dataIn, SafeBuffer & dataOut, const unsigned char * key, const bool useCbcMac) throw ( CryptoException )
{
    if ( cryptoAES && dataIn.GetSize() > static_cast<size_t>(ivSize) )
    {
        SafeBuffer iv;

        iv.AddData(dataIn.GetData(), ivSize);
        dataIn.removeData(0, ivSize);
        cryptoAES->decryptData ( dataIn, dataOut, key, iv.GetData(), pad );
    }
    else
    {
        throw CryptoException ( _ ( "Smartmatic.System.Security.Crypto.ErrorDecryptBuffer" ), N_ ( "Smartmatic.System.Security.Crypto.ErrorDecryptBuffer" ), logger );
    }
}

void Crypto::configCryptoAESKey128()
{
    switch ( mode )
    {
        default:
        case MODE_AES_CBC:
            cryptoAES = new Cryptography::CryptoAES ( EVP_aes_128_cbc() );
            ivSize = EVP_CIPHER_iv_length ( EVP_aes_128_cbc() );
            break;

        case MODE_AES_CFB1:
            cryptoAES = new Cryptography::CryptoAES ( EVP_aes_128_cfb1() );
            ivSize = EVP_CIPHER_iv_length ( EVP_aes_128_cfb1() );
            break;

        case MODE_AES_CFB8:
            cryptoAES = new Cryptography::CryptoAES ( EVP_aes_128_cfb8() );
            ivSize = EVP_CIPHER_iv_length ( EVP_aes_128_cfb8() );
            break;

        case MODE_AES_CFB128:
            cryptoAES = new Cryptography::CryptoAES ( EVP_aes_128_cfb128() );
            ivSize = EVP_CIPHER_iv_length ( EVP_aes_128_cfb128() );
            break;

        case MODE_AES_ECB:
            cryptoAES = new Cryptography::CryptoAES ( EVP_aes_128_ecb() );
            ivSize = EVP_CIPHER_iv_length ( EVP_aes_128_ecb() );
            break;

        case MODE_AES_OFB:
            cryptoAES = new Cryptography::CryptoAES ( EVP_aes_128_ofb() );
            ivSize = EVP_CIPHER_iv_length (  EVP_aes_128_ofb() );
            break;
    }
}

void Crypto::configCryptoAESKey192()
{
    switch ( mode )
    {
        default:
        case MODE_AES_CBC:
            cryptoAES = new Cryptography::CryptoAES ( EVP_aes_192_cbc() );
            ivSize = EVP_CIPHER_iv_length (  EVP_aes_192_cbc() );
            break;

        case MODE_AES_CFB1:
            cryptoAES = new Cryptography::CryptoAES ( EVP_aes_192_cfb1() );
            ivSize = EVP_CIPHER_iv_length ( EVP_aes_192_cfb1() );
            break;

        case MODE_AES_CFB8:
            cryptoAES = new Cryptography::CryptoAES ( EVP_aes_192_cfb8() );
            ivSize = EVP_CIPHER_iv_length ( EVP_aes_192_cfb8() );
            break;

        case MODE_AES_CFB128:
            cryptoAES = new Cryptography::CryptoAES ( EVP_aes_192_cfb128() );
            ivSize = EVP_CIPHER_iv_length ( EVP_aes_192_cfb128() );
            break;

        case MODE_AES_ECB:
            cryptoAES = new Cryptography::CryptoAES ( EVP_aes_192_ecb() );
            ivSize = EVP_CIPHER_iv_length ( EVP_aes_192_ecb() );
            break;

        case MODE_AES_OFB:
            cryptoAES = new Cryptography::CryptoAES ( EVP_aes_192_ofb() );
            ivSize = EVP_CIPHER_iv_length ( EVP_aes_192_ofb() );
            break;
    }
}

void Crypto::configCryptoAESKey256()
{
    switch ( mode )
    {
        default:
        case MODE_AES_CBC:
            cryptoAES = new Cryptography::CryptoAES ( EVP_aes_256_cbc() );
            ivSize = EVP_CIPHER_iv_length ( EVP_aes_256_cbc() );
            break;

        case MODE_AES_CFB1:
            cryptoAES = new Cryptography::CryptoAES ( EVP_aes_256_cfb1() );
            ivSize = EVP_CIPHER_iv_length ( EVP_aes_256_cfb1() );
            break;

        case MODE_AES_CFB8:
            cryptoAES = new Cryptography::CryptoAES ( EVP_aes_256_cfb8() );
            ivSize = EVP_CIPHER_iv_length ( EVP_aes_256_cfb8() );
            break;

        case MODE_AES_CFB128:
            cryptoAES = new Cryptography::CryptoAES ( EVP_aes_256_cfb128() );
            ivSize = EVP_CIPHER_iv_length ( EVP_aes_256_cfb128() );
            break;

        case MODE_AES_ECB:
            cryptoAES = new Cryptography::CryptoAES ( EVP_aes_256_ecb() );
            ivSize = EVP_CIPHER_iv_length ( EVP_aes_256_ecb() );
            break;

        case MODE_AES_OFB:
            cryptoAES = new Cryptography::CryptoAES ( EVP_aes_256_ofb() );
            ivSize = EVP_CIPHER_iv_length ( EVP_aes_256_ofb() );
            break;
    }
}
