/*
 * CMSBase.hxx
 *
 *  Created on: Jul 12, 2017
 *      Author: marcelmancini
 */

#ifndef CMSBASE_HXX_
#define CMSBASE_HXX_

#include <System/Security/SecurityBase.hxx>
#include <openssl/bio.h>

namespace Smartmatic
{
    namespace System
    {
        namespace Security
        {
            namespace Cryptography
            {

                /**
                 * @class	CMSBase
                 * @brief	Class crypto CMS.
                 * @author	Marcel Mancini
                 * @date	12/07/2017
                 */
                class CMSBase: public Smartmatic::System::Security::SecurityBase
                {
                    public:

                        /**
                         * @fn	CMSBase( int flags );
                         * @brief	Class constructor.
                         * @author	Marcel Mancini
                         * @date	12/07/2017
                         * @param 	flags flags to process CMSBase object
                         */
                        explicit CMSBase ( int flags );

                        /**
                         * @fn	virtual ~CMSBase();
                         * @brief	Class destructor.
                         * @author	Marcel Mancini
                         * @date	12/07/2017
                         */
                        virtual ~CMSBase();

                    protected:

                        /**
                         * @fn	void processBuffer(
                         * 				const Smartmatic::System::SafeBuffer & bufferIn,
                         *				Smartmatic::System::SafeBuffer & bufferOut);
                         *
                         * @brief	method to encrypt or sign a buffer.
                         * @author	Marcel Mancini
                         * @date	12/07/2017
                         *
                         * @param	bufferIn 	buffer in.
                         * @param	bufferOut	buffer out result.
                         * @return true if successfully
                         */
                        bool processBuffer ( const Smartmatic::System::SafeBuffer & bufferIn,
                                             Smartmatic::System::SafeBuffer & bufferOu);

                        /**
                         * @fn    void reprocessBuffer(
                         * 				const Smartmatic::System::SafeBuffer & bufferIn,
                         *				Smartmatic::System::SafeBuffer & bufferOut);
                         *
                         * @brief	method to decrypt of verify the signature of the buffer.
                         * @author	Marcel Mancini
                         * @date	12/07/2017
                         *
                         * @param    bufferIn 	buffer in.
                         * @param    bufferOut	buffer out result.
                         * @return true if successfully
                         */
                        bool reprocessBuffer ( const Smartmatic::System::SafeBuffer & bufferIn,
                                               Smartmatic::System::SafeBuffer & bufferOut);
                        /**
                         * @fn	virtual void * CMSProcess() = 0;
                         * @brief	Method to process CMS object
                         * @author	Marcel Mancini
                         * @date	12/07/2017
                         * @return	CMS pointer
                         */
                        virtual void * CMSProcess() = 0;

                        /**
                         * @fn	virtual int CMSProcess ( void * cms, BIO * result ) = 0;
                         * @brief	Method to process cms object and get a result value
                         * @author	Marcel Mancini
                         * @date	12/07/2017
                         * @param	cms CMS pointer to process
                         * @param 	result result buffer
                         * @return	successfully if return value > 0
                         */
                        virtual int CMSProcess ( void * cms, BIO * result ) = 0;

                        BIO * memoryBIO; ///< The BIO memory
                        int flags; ///< flag to process message

                    private:

                        /**
                         * @fn	BIO * getMemoryBIO ( const Smartmatic::System::SafeBuffer & bufferIn );
                         * @brief	Method to get a BIO pointer from a buffer.
                         * @author	Marcel Mancini
                         * @date	12/07/2017
                         * @param    bufferIn	buffer in.
                         * @return	memory OpenSSL BIO
                         */
                        BIO * getMemoryBIO ( const Smartmatic::System::SafeBuffer & bufferIn );

                        /**
                         * @fn	void safeToBuffer( BIO * message, Smartmatic::System::SafeBuffer & bufferOut );
                         * @brief	Method to convert a BIO to a safeBuffer
                         * @author	Marcel Mancini
                         * @date	12/07/2017
                         * @param    message 	OpenSSL BIO message.
                         * @param    bufferOut	buffer out result.
                         */
                        void safeToBuffer ( BIO * message, Smartmatic::System::SafeBuffer & bufferOut );
                };
            }
	}
    }
}
#endif /* CMSBASE_HXX_ */
