/**
 * @file CryptoAES.cxx
 * @brief Body class for CryptoAES file
 */

#include "CryptoAES.hxx"
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <openssl/evp.h>
#include <openssl/aes.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///< macro for gettext
#else
#define _(String) (String) ///< macro without gettext
#endif

#define N_(String) (String) ///< macro not translate

using namespace Smartmatic::System::Security::Cryptography;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::System;
using namespace Smartmatic::Log;

ISMTTLog * CryptoAES::logger = SMTTLogManager::GetLogger ( "Smartmatic.System.Security.Cryptography.CryptoAES", GETTEXT_PACKAGE );

CryptoAES::CryptoAES ( const void * cipherMode )
{
    this->cipherMode = cipherMode;
}

CryptoAES::~CryptoAES()
{
}

void CryptoAES::process ( const SafeBuffer & dataIn, SafeBuffer & buffer, const unsigned char * key, const unsigned char * iv, int enc, bool pad ) throw ( CryptoException )
{
    if ( key == NULL || cipherMode == NULL )
    {
        logger->Debug ( "Key is NULL" );
        throw CryptoException ( _ ( "Smartmatic.System.Security.Cryptography.CryptoAES.ErrorEncryptoData" ), N_ ( "Smartmatic.System.Security.Cryptography.CryptoAES.ErrorEncryptoData" ), logger );
    }

    int sizeOut = 0;
    EVP_CIPHER_CTX * ctx = NULL;
    BIO_METHOD * method = BIO_f_cipher();

    BIO *benc = BIO_new ( method );

    BIO_get_cipher_ctx ( benc, &ctx );
    EVP_CipherInit_ex ( ctx, static_cast < const EVP_CIPHER * > ( cipherMode ), NULL, NULL, NULL, enc );
    EVP_CIPHER_CTX_set_padding ( ctx, !pad ? 0 : 1 );
    EVP_CipherInit_ex ( ctx, NULL, NULL, key, iv, enc );

    unsigned char * dataOut = ( unsigned char * ) malloc ( dataIn.GetSize() + 256 );

    EVP_CipherUpdate ( ctx, dataOut, &sizeOut, dataIn.GetData(), dataIn.GetSize() );

    int finalSize = 0;

    if ( !EVP_CipherFinal_ex ( ctx, dataOut + sizeOut, &finalSize ) )
    {
        free ( dataOut );
        BIO_free_all ( benc );
        throw CryptoException ( _ ( "Smartmatic.System.Security.Cryptography.CryptoAES.ErrorEncryptoData" ), N_ ( "Smartmatic.System.Security.Cryptography.CryptoAES.ErrorEncryptoData" ), logger );
    }

    sizeOut += finalSize;

    EVP_CIPHER_CTX_cleanup ( ctx );

    buffer.Clear();
    buffer.AddData ( dataOut, sizeOut );

    free ( dataOut );
    BIO_free_all ( benc );
}

void CryptoAES::encryptData ( const SafeBuffer & dataIn, SafeBuffer & dataOut, const unsigned char * key, const unsigned char * iv, bool pad ) throw ( CryptoException )
{
    process ( dataIn, dataOut, key, iv, 1, pad );
}

void CryptoAES::decryptData ( const SafeBuffer & dataIn, SafeBuffer & dataOut, const unsigned char * key, const unsigned char * iv, bool pad ) throw ( CryptoException )
{
    process ( dataIn, dataOut, key, iv, 0, pad );
}
