/**
 * @file CryptoAES.hxx
 * @brief Header class for CryptoAES file
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */

#ifndef __CRYPTOAES__H__
#define __CRYPTOAES__H__

#include "System/Exception/CryptoException.hxx"
#include "System/SafeBuffer.h"

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace System
    {
        namespace Security
        {
            namespace Cryptography
            {
                /**
                 * @class	CryptoAES
                 * @brief	Header class Symmetric Crypto.
                 * @author	Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
                 * @date	05-04-2011
                 */
                class CryptoAES
                {
                    public:
                        /**
                         * @fn	CryptoAES::CryptoAES(const void * cipherMode);
                         * @brief	Class constructor.
                         * @author	Youssef.tourkmani
                         * @date	29/10/2012
                         * @param	cipherMode	cipher mode crypto
                         */
                        explicit CryptoAES ( const void * cipherMode );

                        /**
                        * @fn    CryptoAES::~CryptoAES();
                        * @brief    Class destructor.
                        * @author    Youssef.tourkmani
                        * @date    29/10/2012
                        */
                        virtual ~CryptoAES();

                        /**
                         * @fn	void CryptoAES::encryptData(const Smartmatic::System::SafeBuffer & dataIn,
                         * 			Smartmatic::System::SafeBuffer & dataOut, const unsigned char * key, const unsigned char * iv,
                         * 			bool pad) throw (Smartmatic::System::Exception::CryptoException);
                         * @brief	method encrypto data.
                         * @author	Youssef.tourkmani
                         * @date	29/10/2012
                         * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                         * 																														occurs.
                         * @param	dataIn 	data to crypto.
                         * @param	dataOut	data out of crypto.
                         * @param	key		 	key crypto.
                         * @param	iv		 	iv crypto.
                         * @param	pad		 	padding true=padding
                         * 								@throw CryptoException.
                         */
                        void encryptData ( const Smartmatic::System::SafeBuffer & dataIn,
                                           Smartmatic::System::SafeBuffer & dataOut,
                                           const unsigned char * key,
                                           const unsigned char * iv,
                                           bool pad)
                        throw ( Smartmatic::System::Exception::CryptoException );

                        /**
                         * @fn	void CryptoAES::decryptData(const Smartmatic::System::SafeBuffer & dataIn,
                         * 			Smartmatic::System::SafeBuffer & dataOut, const unsigned char * key, const unsigned char * iv,
                         * 			bool pad) throw (Smartmatic::System::Exception::CryptoException);
                         * @brief	method decrypto data.
                         * @author	Youssef.tourkmani
                         * @date	29/10/2012
                         * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                         * 																														occurs.
                         * @param	dataIn 	data to crypto.
                         * @param	dataOut	data out of crypto.
                         * @param	key		 	key crypto.
                         * @param	iv		 	iv crypto.
                         * @param	pad		 	padding true=padding
                         * 								@throw CryptoException.
                         */
                        void decryptData ( const Smartmatic::System::SafeBuffer & dataIn,
                                           Smartmatic::System::SafeBuffer & dataOut,
                                           const unsigned char * key,
                                           const unsigned char * iv,
                                           bool pad )
                        throw ( Smartmatic::System::Exception::CryptoException );

                    protected:

                        /**
                         * @fn	void CryptoAES::process(const Smartmatic::System::SafeBuffer & dataIn,
                         * 			Smartmatic::System::SafeBuffer & dataOut, const unsigned char * key, const unsigned char * iv,
                         * 			int enc, bool pad) throw (Smartmatic::System::Exception::CryptoException);
                         * @brief	method encrypt or decrypt data.
                         * @author	Yadickson Soto
                         * @date	29/10/2012
                         * @exception	Smartmatic::System::Exception::CryptoException	Thrown when a crypto error condition
                         * 																														occurs.
                         * @param	dataIn 	data to process.
                         * @param	dataOut	data out (encrypt or decrypt).
                         * @param	key		 	key crypto.
                         * @param	iv		 	iv crypto.
                         * @param	enc		 	0=decrypt, 1=encrypt.
                         * @param	pad		 	padding true=padding
                         * @throw CryptoException.
                         */
                        void process ( const Smartmatic::System::SafeBuffer & dataIn,
                                       Smartmatic::System::SafeBuffer & dataOut,
                                       const unsigned char * key,
                                       const unsigned char * iv,
                                       int enc, bool pad )
                        throw ( Smartmatic::System::Exception::CryptoException );

                    private:

                        const void * cipherMode; ///<cipher mode
                        static Smartmatic::Log::ISMTTLog * logger; ///<logger
                };
            }
        }
    }
}

#endif /*__CRYPTOAES__H__*/
