/**
 * @file   HmacSha256.hxx
 * @brief  Declaration of HmacSha256 class.
 * @author manuel.arguelles
 * @date   sep 20, 2018
 */

#ifndef HMACSHA256_HXX
#define HMACSHA256_HXX

#include <System/SafeBuffer.h>
#include <string>

namespace Smartmatic
{
    namespace System
    {
        namespace Security
        {
            namespace Cryptography
            {

                /**
                 * @class  HmacSha256
                 * @brief  Handles HMAC operation with SHA256.
                 * @author manuel.arguelles
                 * @date   sep 20, 2018
                 */
                class HmacSha256
                {
                    public:

                        /**
                         * @brief  Creates an instance of the class.
                         * @author manuel.arguelles
                         * @date   sep 20, 2018
                         * @param  key The HMAC key.
                         */
                        explicit HmacSha256(const SafeBuffer& key);

                        /**
                         * @brief  Destroys an instance of the class.
                         * @author manuel.arguelles
                         * @date   sep 20, 2018
                         */
                        ~HmacSha256();

                    public:

                        /**
                         * @brief  Adds data for HMAC.
                         * @author manuel.arguelles
                         * @date   sep 20, 2018
                         * @param  data The data that will be HMAC.
                         */
                        void update(const SafeBuffer& data);

                        /**
                         * @brief  Gets the HMAC of the data.
                         * @author manuel.arguelles
                         * @date   sep 20, 2018
                         * @throw  CryptoException on error.
                         * @return The HMAC of the data.
                         */
                        SafeBuffer getDigest() const;

                    private:

                        SafeBuffer m_key;
                        SafeBuffer m_data;
                };

                /**
                 * @brief  Validates the HMAC code against the HMAC object.
                 * @author manuel.arguelles
                 * @date   sep 20, 2018
                 * @throw  CryptoException on error.
                 * @param  hmac The HmacSha256 object with the data to validate.
                 * @param  code The HMAC-256 code to validate.
                 * @return True if the verification success, false if not.
                 */
                bool HmacSha256Validate(const HmacSha256& hmac, const SafeBuffer& code);

            } /* namespace Cryptography */
        } /* namespace Security */
    } /* namespace System */
} /* namespace Smartmatic */

#endif /* HMACSHA256_HXX */
