/**
 * @file MsgDigestUtil.h
 * @brief Header message digest utility class
 * @date 01/04/2012
 * @author Marcel Cordovi
 */

#ifndef MSGDIGESTUTIL_H_
#define MSGDIGESTUTIL_H_

#include <openssl/evp.h>
#include <string>
#include "System/Exception/MessageDigestException.h"
#include "System/SafeBuffer.h"

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace System
    {
        namespace Security
        {
            namespace Cryptography
            {
                /**
                 * @class	MsgDigestUtil
                 * @brief	Message digest utility class.
                 * @author	Yadickson Soto
                 * @date	29/04/2011
                 */
                class MsgDigestUtil
                {
                    public:

                        /**
                         * @enum	DigestAlgType
                         * @brief	Values that represent DigestAlgType.
                         */
                        enum DigestAlgType
                        {
                            DGSTUTIL_SHA224,
                            DGSTUTIL_SHA256,
                            DGSTUTIL_SHA384,
                            DGSTUTIL_SHA512
                        };

                        /**
                         * @brief  Returns the default algorithm for hashing files.
                         * @author manuel.arguelles
                         * @date   Jul 18, 2017
                         * @return The digest algorithm to use.
                         */
                        static DigestAlgType getDefaultFileDigest();

                        /**
                         * @brief  Returns the default algorithm for hashing file names.
                         * @author manuel.arguelles
                         * @date   Jul 18, 2017
                         * @return The digest algorithm to use.
                         */
                        static DigestAlgType getDefaultFileNameDigest();

                        /**
                         * @brief  Returns the default hashing algorithm for certificates.
                         * @author manuel.arguelles
                         * @date   Jul 18, 2017
                         * @return The digest algorithm to use.
                         */
                        static DigestAlgType getDefaultCertificateDigest();

                        /**
                         * @brief  Return the default hashing algorithm for opening code computation.
                         * @author manuel.arguelles
                         * @date   Jul 18, 2017
                         * @return The digest algorithm to use.
                         */
                        static DigestAlgType getDefaultOpeningCodeDigest();

                        /**
                         * @brief  Return the default hashing algorithm for secret key signing.
                         * @author manuel.arguelles
                         * @date   Jul 18, 2017
                         * @return The digest algorithm to use.
                         */
                        static DigestAlgType getDefaultSecretKeyDigest();

                        /**
                         * @brief  Return the default hashing algorithm for compose key computation.
                         * @author manuel.arguelles
                         * @date   Jul 18, 2017
                         * @return The digest algorithm to use.
                         */
                        static DigestAlgType getDefaultComposeKeyDigest();

                        /**
                         * @brief  Return the default hashing algorithm used for (password based) key derivations.
                         * @author manuel.arguelles
                         * @date   Jul 19, 2017
                         * @return The digest algorithm to use.
                         */
                        static DigestAlgType getDefaultGenerateKeyDigest();

                        /**
                         * @brief  Return the default hashing algorithm to use for vote checksum.
                         * @author manuel.arguelles
                         * @date   Jul 20, 2017
                         * @return The digest algorithm to use.
                         */
                        static DigestAlgType getDefaultVoteDigest();

                        /**
                         * @fn  MsgDigestUtil();
                         * @brief   Default constructor.
                         * @author  eduardo.fernandez
                         * @date    17/01/2019
                         * @param   alg algorithm type.
                         */
                        explicit MsgDigestUtil(DigestAlgType alg);

                        /**
                         * @fn	MsgDigestUtil( const Smartmatic::System::SafeBuffer & buffer );
                         * @brief	Default constructor.
                         * @author	Yadickson Soto
                         * @date	29/10/2012
                         * @param buffer data buffer
                         */
                        explicit MsgDigestUtil ( const Smartmatic::System::SafeBuffer & buffer );

                        /**
                         * @fn	virtual MsgDigestUtil::~MsgDigestUtil();
                         * @brief	Destructor.
                         * @author	Youssef.tourkmani
                         * @date	29/10/2012
                         */
                        virtual ~MsgDigestUtil();

                        /**
                         * @fn	void getDigest ( Smartmatic::System::SafeBuffer & result,
                         *   DigestAlgType alg = DGSTUTIL_SHA256)
                         * throw ( Smartmatic::System::Exception::MessageDigestException );
                         * @brief	Calculates hash value of a given binary message with the specified algorithm.
                         * @author	Yadickson Soto
                         * @date	05/10/2012
                         * @param	result	 	The message to digest result.
                         * @param	alg		 	algorithm type.
                         */
                        void getDigest ( Smartmatic::System::SafeBuffer & result, DigestAlgType alg )
                        throw ( Smartmatic::System::Exception::MessageDigestException );

                        /**
                         * @fn  void getDigest ( Smartmatic::System::SafeBuffer & result,
                         *   DigestAlgType alg = DGSTUTIL_SHA256)
                         * throw ( Smartmatic::System::Exception::MessageDigestException );
                         * @brief   Calculates hash value of a given chunk.
                         * @author  eduardo.fernandez
                         * @date    17/01/2019
                         * @param   buffer      The buffer to digest.
                         */
                        void updateDigest ( const SafeBuffer &buffer )
                        throw ( Smartmatic::System::Exception::MessageDigestException );

                        /**
                         * @fn void MsgDigestUtil::obtainResult(const unsigned int md_len, unsigned char *md_value, System::SafeBuffer &result);
                         * @brief Obtains the result of the hashing process.
                         * @author eduardo.fernandez
                         * @date   16/01/2019
                         */
                        void obtainResult(SafeBuffer &result);

                    private:

                        /**
                         * @fn void MsgDigestUtil::initializeHashing(EVP_MD_CTX *mdctx, const EVP_MD * md, const char* algName);
                         * @brief Initializes the hashing process.
                         * @author eduardo.fernandez
                         * @date   16/01/2019
                         */
                        void initializeHashing(EVP_MD_CTX *mdctx, const EVP_MD * md, const char* algName);

                        /**
                         * @fn void MsgDigestUtil::performHashing(EVP_MD_CTX *mdctx, const unsigned char* buffer, const size_t bufferSize);
                         * @brief Performs the hashing process.
                         * @author eduardo.fernandez
                         * @date   16/01/2019
                         */
                        void performHashing(EVP_MD_CTX *mdctx, const unsigned char* buffer, const size_t bufferSize);


                        /**
                         * @fn void MsgDigestUtil::cleanupHashing(EVP_MD_CTX *mdctx, unsigned char *md_value, unsigned int *md_len);
                         * @brief Cleans up the hashing process.
                         * @author eduardo.fernandez
                         * @date   16/01/2019
                         */
                        void cleanupHashing(EVP_MD_CTX *mdctx, unsigned char *md_value, unsigned int *md_len);

                    protected:

                        /**
                         * @fn	virtual const char* MsgDigestUtil::mMapAlgName (DigestAlgType alg);
                         * @brief	Map algorithm name.
                         * @author	marcel
                         * @date	28/04/2011
                         * @param	alg	The algorithm.
                         * @return	null if it fails, else.
                         */

                        virtual const char * mMapAlgName ( DigestAlgType alg );

                    private:

                        Smartmatic::System::SafeBuffer buffer; ///< buffer
                        static Smartmatic::Log::ISMTTLog * logger; ///< The logger

                        EVP_MD_CTX mdctx;
                };
            }
        }
    }
}

#endif /* MSGDIGESTUTIL_H_ */
