/**
 * @file RsaUtil.cpp
 * @brief Body class rsa util
 * @date 24/05/2011
 * @author Marcel Cordovi
 */

#include "RsaUtil.h"
#include <openssl/ossl_typ.h>
#include <openssl/rsa.h>
#include <openssl/rand.h>
#include <openssl/evp.h>
#include <openssl/x509.h>
#include <openssl/safestack.h>
#include <Log/SMTTLogManager.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

#define BUFINV_ERR_MSG N_("Smartmatic.System.Security.Cryptography.RsaUtil.InvalidBuffer") ///< text for log
#define CIPHER_ERR_MSG N_("Smartmatic.System.Security.Cryptography.RsaUtil.ErrorDuringCpherOperation") ///< text for log
#define PADING_ERR_MSG N_("Smartmatic.System.Security.Cryptography.RsaUtil.InvalidPadding") ///< text for log
#define RSAKEY_ERR_MSG N_("Smartmatic.System.Security.Cryptography.RsaUtil.LoadedKeyIsInvalid") ///< text for log

using namespace Smartmatic::System::Exception;
using namespace Smartmatic::System::Security::Cryptography;
using namespace Smartmatic::System;
using namespace Smartmatic::Log;

ISMTTLog * RsaUtil::logger = SMTTLogManager::GetLogger ( "Smartmatic.System.Security.RSAUtil", GETTEXT_PACKAGE );

//============================================================================//
//============================= Life Cycle ===================================//
//============================================================================//

RsaUtil::RsaUtil()
{
    m_Padding = RSAPAD_PKCS1;
}

RsaUtil::~RsaUtil()
{
}

//============================================================================//
//=============================== Access =====================================//
//============================================================================//

RsaUtil::RsaPaddingEnumType RsaUtil::GetPaddingType() const
{
    return this->m_Padding;
}

void RsaUtil::SetPaddingType ( RsaPaddingEnumType pad )
{
    this->m_Padding = pad;
}

//============================================================================//
//============================== Interface ===================================//
//============================================================================//

bool RsaUtil::IsValidPublicKey () const
{
    bool is_public = false;
    // TODO if the class allow m_Rsa to have NULL value on a given time the semantic of this method is broken
    // This should never happen since all errors on loading m_Rsa are handled but m_Rsa can still be NULL right after default constructor instantiation
    EVP_PKEY * pkey = GetPublicKey();

    if ( pkey != NULL )
    {
        RSA * rsakey = EVP_PKEY_get1_RSA ( pkey );

        if ( rsakey != NULL )
        {
            is_public =
                rsakey->d == NULL
                && rsakey->p == NULL
                && rsakey->q == NULL
                && rsakey->dmp1 == NULL
                && rsakey->dmq1 == NULL
                && rsakey->iqmp == NULL;
            RSA_free ( rsakey );
        }
    }

    return is_public;
}

bool RsaUtil::IsValidPrivateKey () const
{
    EVP_PKEY * pkey = GetPrivateKey();

    if ( pkey != NULL )
    {
        RSA * rsakey = EVP_PKEY_get1_RSA ( pkey );

        if ( rsakey != NULL )
        {
            bool result = RSA_check_key ( rsakey ) >= 0;
            RSA_free ( rsakey );
            return result;
        }
    }

    return false;
}

SafeBuffer RsaUtil::PublicEncrypt ( const SafeBuffer & buffer ) throw ( Smartmatic::System::Exception::RSAException )
{
    const unsigned char * plaintext = buffer.GetData();
    int plaintTextLen = buffer.GetSize ();

    unsigned char * cphtext_buff;
    int cphtext_len;

    if ( ! IsValidPrivateKey () && ! IsValidPublicKey () )
    {
        throw RSAException ( _ ( RSAKEY_ERR_MSG ), N_ ( RSAKEY_ERR_MSG ), logger );
    }

    if ( plaintext == NULL )
    {
        throw RSAException ( _ ( BUFINV_ERR_MSG ), N_ ( BUFINV_ERR_MSG ), logger );
    }

    RSA * rsakey = EVP_PKEY_get1_RSA ( GetPublicKey() );

    // Check whether buffer is out of bounds for the current padding
    if ( rsakey == NULL || ! mCheckPadding ( plaintTextLen, rsakey ) )
    {
        throw RSAException ( _ ( PADING_ERR_MSG ), N_ ( PADING_ERR_MSG ), logger );
    }

    /*
     * Pseudo random number generator must be seeded before encryption. If
     * system supports /dev/urandom this is done transparently. Here we are
     * checking if PRNG is seeded with enough entropy in order to proceed
     * properly. If it is not, the PRNG must be explicitly seeded.
     */
    RAND_status ();

    cphtext_buff = ( unsigned char * ) OPENSSL_malloc ( RSA_size ( rsakey ) );

    cphtext_len = RSA_public_encrypt (
                      plaintTextLen
                      , plaintext
                      , cphtext_buff
                      , rsakey
                      , mGetNativePadding () );

    RSA_free ( rsakey );

    SafeBuffer cipher_text ( cphtext_buff, cphtext_len );
    OPENSSL_free ( cphtext_buff );

    return cipher_text;
}

SafeBuffer RsaUtil::PrivateDecrypt ( const SafeBuffer & buffer ) throw ( Smartmatic::System::Exception::RSAException )
{
    const unsigned char * ciphertext = buffer.GetData();
    int cipherTextLen = buffer.GetSize ();

    unsigned char * plaintext_buff;
    int plaintext_len;

    if ( ! IsValidPrivateKey () )
    {
        throw RSAException ( _ ( RSAKEY_ERR_MSG ), N_ ( RSAKEY_ERR_MSG ), logger );
    }

    if ( ciphertext == NULL )
    {
        throw RSAException ( _ ( BUFINV_ERR_MSG ), N_ ( BUFINV_ERR_MSG ), logger );
    }

    RSA * rsakey = EVP_PKEY_get1_RSA ( GetPrivateKey() );
    plaintext_buff = ( unsigned char * ) OPENSSL_malloc ( RSA_size ( rsakey ) );

    plaintext_len = RSA_private_decrypt (
                        cipherTextLen
                        , ciphertext
                        , plaintext_buff
                        , rsakey
                        , mGetNativePadding () );

    RSA_free ( rsakey );

    if ( plaintext_len < 0 )
    {
        OPENSSL_free ( plaintext_buff );
        throw RSAException ( _ ( CIPHER_ERR_MSG ), N_ ( CIPHER_ERR_MSG ), logger );
    }

    SafeBuffer plain_text ( plaintext_buff, plaintext_len );
    OPENSSL_free ( plaintext_buff );

    return plain_text;
}

SafeBuffer RsaUtil::PrivateEncrypt ( const SafeBuffer & buffer ) throw ( Smartmatic::System::Exception::RSAException )
{
    const unsigned char * plaintext = buffer.GetData();
    int plaintTextLen = buffer.GetSize ();

    unsigned char * cphtext_buff;
    int cphtext_len, pad;

    if ( ! IsValidPrivateKey () )
    {
        throw RSAException ( _ ( RSAKEY_ERR_MSG ), N_ ( RSAKEY_ERR_MSG ), logger );
    }

    if ( plaintext == NULL )
    {
        throw RSAException ( _ ( BUFINV_ERR_MSG ), N_ ( BUFINV_ERR_MSG ), logger );
    }

    pad = mGetNativePadding ();

    if ( pad != RSA_PKCS1_PADDING && pad != RSA_NO_PADDING ) // openssl doesn't allow private encryption with other padding than these two
    {
        throw RSAException ( _ ( PADING_ERR_MSG ), N_ ( PADING_ERR_MSG ), logger );
    }

    RSA * rsakey = EVP_PKEY_get1_RSA ( GetPrivateKey() );
    cphtext_buff = ( unsigned char * ) OPENSSL_malloc ( RSA_size ( rsakey ) );

    cphtext_len = RSA_private_encrypt (
                      plaintTextLen
                      , plaintext
                      , cphtext_buff
                      , rsakey
                      , mGetNativePadding () );

    RSA_free ( rsakey );

    SafeBuffer cipher_text ( cphtext_buff, cphtext_len );
    OPENSSL_free ( cphtext_buff );
    return cipher_text;
}

SafeBuffer RsaUtil::PublicDecrypt ( const SafeBuffer & buffer ) throw ( Smartmatic::System::Exception::RSAException )
{
    const unsigned char * ciphertext = buffer.GetData();
    int cipherTextLen = buffer.GetSize ();

    unsigned char * plaintext_buff;
    int plaintext_len;

    if ( ! IsValidPrivateKey () && ! IsValidPublicKey () )
    {
        throw RSAException ( _ ( RSAKEY_ERR_MSG ), N_ ( RSAKEY_ERR_MSG ), logger );
    }

    if ( ciphertext == NULL )
    {
        throw RSAException ( _ ( BUFINV_ERR_MSG ), N_ ( BUFINV_ERR_MSG ), logger );
    }

    RSA * rsakey = EVP_PKEY_get1_RSA ( GetPublicKey() );
    plaintext_buff = ( unsigned char * ) OPENSSL_malloc ( RSA_size ( rsakey ) - 11 );

    plaintext_len = RSA_public_decrypt (
                        cipherTextLen
                        , ciphertext
                        , plaintext_buff
                        , rsakey
                        , mGetNativePadding () );

    RSA_free ( rsakey );

    if ( plaintext_len < 0 )
    {
        OPENSSL_free ( plaintext_buff );
        throw RSAException ( _ ( CIPHER_ERR_MSG ), N_ ( CIPHER_ERR_MSG ), logger );
    }

    SafeBuffer plain_text ( plaintext_buff, plaintext_len );
    OPENSSL_free ( plaintext_buff );
    return plain_text;
}

void RsaUtil::SignData ( const Smartmatic::System::SafeBuffer & inSafeBuffer, Smartmatic::System::SafeBuffer & outSafeBuffer, MsgDigestUtil::DigestAlgType hashKind ) throw ( Smartmatic::System::Exception::SignedException )
{
    outSafeBuffer.Clear();

    int pad = mGetNativePadding ();

    if ( pad != RSA_PKCS1_PADDING && pad != RSA_NO_PADDING ) // openssl doesn't allow private encryption with other padding than these two
    {
        throw SignedException ( _ ( PADING_ERR_MSG ), N_ ( PADING_ERR_MSG ), logger );
    }

    //compute hash
    SafeBuffer hash;
    MsgDigestUtil digest ( inSafeBuffer );
    digest.getDigest ( hash, hashKind );

    try
    {
        //encrypt hash
        Smartmatic::System::SafeBuffer res = PrivateEncrypt ( hash );
        outSafeBuffer.AddData ( res );
    }
    catch ( ... )
    {
        throw SignedException ( _ ( RSAKEY_ERR_MSG ), N_ ( RSAKEY_ERR_MSG ), logger );
    }
}

bool RsaUtil::VerifySignature ( const Smartmatic::System::SafeBuffer & inSafeBuffer, const Smartmatic::System::SafeBuffer & signature, MsgDigestUtil::DigestAlgType hashKind ) throw ( Smartmatic::System::Exception::SignedException )
{
    bool retVal = false;
    int pad = mGetNativePadding ();

    //check conditions
    if ( ! IsValidPublicKey() )
    {
        throw SignedException ( _ ( RSAKEY_ERR_MSG ), N_ ( RSAKEY_ERR_MSG ), logger );
    }

    if ( pad != RSA_PKCS1_PADDING && pad != RSA_NO_PADDING ) // openssl doesn't allow private encryption with other padding than these two
    {
        throw SignedException ( _ ( PADING_ERR_MSG ), N_ ( PADING_ERR_MSG ), logger );
    }

    //compute hash
    SafeBuffer hash;
    MsgDigestUtil digest ( inSafeBuffer );
    digest.getDigest ( hash, hashKind );

    try
    {
        //decrypt signature
        Smartmatic::System::SafeBuffer res = PublicDecrypt ( signature );

        //compare hash
        retVal = hash == res;
    }
    catch ( ... )
    {

    }

    return retVal;
}

//============================================================================//
//============================== Auxiliars ===================================//
//============================================================================//

bool RsaUtil::mCheckPadding ( int buffLen,  void * rsakey )
{
    bool retVal = false;

    switch ( m_Padding )
    {
        case RSAPAD_PKCS1:
            retVal = buffLen < ( RSA_size ( ( RSA * ) rsakey ) - 11 );
            break;

        case RSAPAD_OAEP:
            retVal = buffLen < ( RSA_size ( ( RSA * ) rsakey ) - 41 );
            break;

        case RSAPAD_SSL:
            retVal = buffLen < ( RSA_size ( ( RSA * ) rsakey ) - 11 );
            break;

        case RSAPAD_RAW:
            retVal = buffLen == RSA_size ( ( RSA * ) rsakey );
            break;

        default:
            retVal = false;
            break;
    }

    return retVal;
}

int RsaUtil::mGetNativePadding ()
{
    switch ( m_Padding )
    {
        case RSAPAD_PKCS1:
            return RSA_PKCS1_PADDING;

        case RSAPAD_OAEP:
            return RSA_PKCS1_OAEP_PADDING;

        case RSAPAD_SSL:
            return RSA_SSLV23_PADDING;

        case RSAPAD_RAW:
        default:
            return RSA_NO_PADDING;
    }
}
