/**
 * @file SignUtils.cxx
 * @brief Body class utility certificate signed manager
 * @date 21/01/2014
 * @author Yadickson Soto
 */

#include "SignUtils.hxx"
#include "SignatureAlgorithm.h"
#include "RsaUtil.h"
#include "ECDSASigner.hxx"
#include <openssl/evp.h>

using namespace Smartmatic::System::Security::Cryptography;
using namespace Smartmatic::System;

bool SignUtils::signBuffer ( CertificateExtensionBase * certificate, const SafeBuffer & inputBuffer, SafeBuffer & outputSignature, MsgDigestUtil::DigestAlgType hash )
{
    bool validate = false;
    outputSignature.Clear();

    if ( certificate == NULL )
    {
        return validate;
    }

    EVP_PKEY * privateKey = certificate->LoadCertifiedPrivateKey();
    SignatureAlgorithm * sign = createSignatureAlgorithmByKeyType ( privateKey );

    if ( sign != NULL )
    {
        sign->SetCertificate ( certificate );

        try
        {
            sign->SignData ( inputBuffer, outputSignature, hash );
            validate = true;
        }
        catch ( ... )
        {
        }

        delete sign;
        sign = NULL;
    }

    return validate;
}

bool SignUtils::verifyBuffer ( CertificateExtensionBase * certificate, const SafeBuffer & inputBuffer, const SafeBuffer & signature, MsgDigestUtil::DigestAlgType hash )
{
    bool validate = false;

    if ( certificate == NULL )
    {
        return validate;
    }

    EVP_PKEY * publicKey = certificate->LoadCertifiedPublicKey();
    SignatureAlgorithm * verify = createSignatureAlgorithmByKeyType ( publicKey );

    if ( verify != NULL )
    {
        try
        {
            verify->SetCertificate ( certificate );
            validate = verify->VerifySignature ( inputBuffer, signature, hash );
        }
        catch ( ... )
        {
        }

        delete verify;
        verify = NULL;
    }

    return validate;
}

SignatureAlgorithm * SignUtils::createSignatureAlgorithmByKeyType ( EVP_PKEY * pkey )
{
    SignatureAlgorithm * retVal = NULL;

    if ( pkey == NULL )
    {
        return retVal;
    }

    switch ( pkey->type )
    {
        case EVP_PKEY_RSA:
        case EVP_PKEY_RSA2:
            retVal =  new RsaUtil();
            break;

        case EVP_PKEY_EC:
            retVal =  new ECDSASigner();
            break;

        case EVP_PKEY_DSA:
        case EVP_PKEY_DSA1:
        case EVP_PKEY_DSA2:
        case EVP_PKEY_DSA3:
        case EVP_PKEY_DSA4:
        default:
            break;
    }

    return retVal;
}
