/**
 * @file Signed.hxx
 * @brief Header class for Signed file
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */

#ifndef __SIGNED__H__
#define __SIGNED__H__

#include "System/Exception/CryptoException.hxx"
#include "System/Exception/SignedException.hxx"
#include <System/SafeBuffer.h>
#include <System/Security/SecurityBase.hxx>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace System
    {
        namespace Security
        {
            /**
             * @class	Signed
             * @brief	Class for signed manager.
             * @author	Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
             * @date	31-03-2011
             */
            class Signed : public SecurityBase
            {
                public:

                    /**
                     * @enum SignedType
                     * @brief	Enum signed type config.
                     */
                    typedef enum
                    {
                        SIMPLE_SIGNED /**<Use signed simple*/
                        , FULL_SIGNED /**<Use signed full*/
                        , NOT_SIGNED /**<not used signed*/
                    } SignedType;

                    /**
                     * @fn	Signed::Signed();
                     * @brief	Class constructor.
                     * @author	Yadickson Soto
                     * @date	29/10/2012
                     */
                    Signed();
                    /**
                     * @fn	virtual ~Signed();
                     * @brief	Class destructor.
                     * @author	Yadickson Soto
                     * @date	29/10/2012
                     */
                    virtual ~Signed();

                    /**
                     * @fn	void Signed::signBuffer(const Smartmatic::System::SafeBuffer & bufferIn,
                     *		Smartmatic::System::SafeBuffer & bufferOut,
                     * 			SignedType signedType) throw (Smartmatic::System::Exception::SignedException);
                     * @brief	method to Sign a xml buffer with a certificate.
                     * @author	Yadickson Soto
                     * @date	29/10/2012
                     * @exception	Smartmatic::System::Exception::SignedException	Thrown when a signed error condition occurs.
                     * @param	bufferIn data to signed
                     * @param 	bufferOut result
                     * @param	signedType	signature mode
                     * @throw SignedException.
                     */
                    void signBuffer ( const Smartmatic::System::SafeBuffer & bufferIn,
                                      Smartmatic::System::SafeBuffer & bufferOut, SignedType signedType )
                    throw ( Smartmatic::System::Exception::SignedException );

                    /**
                     * @fn	bool Signed::verifyBuffer( const Smartmatic::System::SafeBuffer & buffer)
                     * 			throw (Smartmatic::System::Exception::SignedException);
                     * @brief	method to validate signed xml buffer.
                     * @author	Yadickson Soto
                     * @date	29/10/2012
                     * @exception	Smartmatic::System::Exception::SignedException	Thrown when a signed error condition occurs.
                     * @param	buffer data to validate
                     * @return	true if valid dom
                     * @throw SignedException.
                     */
                    bool verifyBuffer ( const Smartmatic::System::SafeBuffer & buffer )
                    throw ( Smartmatic::System::Exception::SignedException );

                private:

                    static Smartmatic::Log::ISMTTLog * logger; ///<logger
            };
        }
    }
}

#endif /*__SIGNED__H__*/
