/**
 * @file URILocalResolver.cxx
 * @brief class for resolving only local URIs.
 * @author 05/02/2014
 * @author Yadickson Soto
 */

#include "URILocalResolver.hxx"
#include <xercesc/util/BinFileInputStream.hpp>
#include <xercesc/util/XMLUri.hpp>
#include <xercesc/util/Janitor.hpp>
#include <xsec/framework/XSECError.hpp>
#include <xsec/utils/XSECDOMUtils.hpp>
#include <System/IO/File.h>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <stdio.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

using namespace Smartmatic::System::Security;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::Log;

XERCES_CPP_NAMESPACE_USE

ISMTTLog * URILocalResolver::logger = SMTTLogManager::GetLogger ( "Smartmatic.System.Security.URILocalResolver", GETTEXT_PACKAGE );

URILocalResolver::URILocalResolver()
    : mp_baseURI ( NULL ), memory ( NULL )
{

}

URILocalResolver::URILocalResolver ( const std::string & baseURI )
    : mp_baseURI ( NULL ), memory ( NULL )
{
    mp_baseURI = XMLString::transcode ( baseURI.c_str() );
}

URILocalResolver::~URILocalResolver()
{
    clearBaseURI();
    clearMemory();
}

BinInputStream * URILocalResolver::resolveURI ( const XMLCh * uri )
{
    XMLUri * xmluri;

    // Create the appropriate XMLUri objects

    XMLUri * turi;

    XSECnew ( turi, XMLUri ( mp_baseURI ) );
    Janitor<XMLUri> j_turi ( turi );

    XSECnew ( xmluri, XMLUri ( turi, uri ) );

    Janitor<XMLUri> j_xmluri ( xmluri );

    // Clean hex escapes
    XMLCh * realPath = cleanURIEscapes ( xmluri->getPath() );
    char * xmlString = XMLString::transcode ( realPath );
    std::string pathFile = xmlString;
    XMLString::release ( &xmlString );
    XSEC_RELEASE_XMLCH ( realPath );

    SafeBuffer buffer;

    if ( !File::getInstance()->openFileToBuffer ( pathFile, buffer ) )
    {
        logger->Debug ( "Error resolver file " + pathFile );
        return NULL;
    }

    clearMemory();

    unsigned int length = buffer.GetSize() + 1;

    memory = ( char * ) malloc ( length );
    memcpy ( memory, buffer.GetData(), buffer.GetSize() );

    FileHandle filePointer = ( FileHandle ) fmemopen ( ( char * ) memory, length - 1, "rb" );
    BinFileInputStream * retStrm = new BinFileInputStream ( filePointer );

    if ( !retStrm->getIsOpen() )
    {
        logger->Debug ( "Empty file not support for fmemopen POSIX" );
        delete retStrm;
        retStrm = NULL;
    }

    return retStrm;
}

XSECURIResolver * URILocalResolver::clone ( void )
{
    URILocalResolver * ret;

    ret = new URILocalResolver();
    ret->mp_baseURI = XMLString::replicate ( this->mp_baseURI );

    return ret;
}

void URILocalResolver::clearMemory()
{
    if ( memory )
    {
        free ( memory );
        memory = NULL;
    }
}

void URILocalResolver::clearBaseURI()
{
    if ( mp_baseURI != NULL )
    {
        XSEC_RELEASE_XMLCH ( mp_baseURI );
        mp_baseURI = NULL;
    }
}
