/**
 * @file XmlDetachedSignatureAlgorithm.cxx
 * @brief Body class for XmlDetachedSignatureAlgorithm file.
 * @author  Eduardo Fernández
 * @date    09/10/2018
 */

#include "XmlDetachedSignatureAlgorithm.hxx"

#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <System/IO/Path.h>
#include <System/Security/OpenSSLCertificate.hxx>
#include <System/Security/URILocalResolver.hxx>
#include <System/XML/DomXml.hxx>
#include <System/XML/ExtractCertificate.hxx>
#include <xsec/dsig/DSIGKeyInfoX509.hpp>
#include <xsec/enc/OpenSSL/OpenSSLCryptoKeyEC.hpp>
#include <xsec/framework/XSECException.hpp>
#include <xsec/framework/XSECProvider.hpp>

#include <memory>
#include <sstream>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///< macro for gettext
#else
#define _(String) (String) ///< macro without gettext
#endif

#define N_(String) (String) ///< macro not translate

#define DSIGNS_PREFIX "" ///< prefix signed node

using namespace xercesc;
using namespace Smartmatic::Log;
using namespace Smartmatic::System;
using namespace Smartmatic::System::IO;
using namespace Smartmatic::System::Security;
using namespace Smartmatic::System::Security::Cryptography;
using namespace Smartmatic::System::XML;

ISMTTLog * XmlDetachedSignatureAlgorithm::logger = SMTTLogManager::GetLogger ( "Smartmatic.System.Security.XmlDetachedSignatureAlgorithm", GETTEXT_PACKAGE );

XmlDetachedSignatureAlgorithm::XmlDetachedSignatureAlgorithm()
{

}

XmlDetachedSignatureAlgorithm::~XmlDetachedSignatureAlgorithm()
{

}

bool XmlDetachedSignatureAlgorithm::signBuffer(
    const SafeBuffer& bufferToSign,
    SafeBuffer& bufferOut,
    const Signed::SignedType&,
    CertificateExtensionBase * const certificate,
    const hashMethod& hash,
    const std::string& basePath)
{
    std::string baseURI = "file://";
    baseURI += Path::GetDirectory(basePath);

    EVP_PKEY *privateKey = certificate->LoadCertifiedPrivateKey();

    XSECProvider   provider;
    DSIGSignature *signature = provider.newSignature();

    DomXml      domXml;
    DOMDocument *document = domXml.bufferToDom(bufferToSign);
    DOMElement  *root     = signature->createBlankSignature(document, CANON_C14N_NOC, SIGNATURE_ECDSA, hash);

    document->appendChild(root);

    URILocalResolver resolver(baseURI);
    signature->setURIResolver(&resolver);

    OpenSSLCryptoKeyEC cryptoECKey(privateKey);

    signature->setSigningKey(cryptoECKey.clone());

    safeBuffer sb;
    OpenSSLCryptoX509 x509(certificate->LoadCertifiedX509());
    sb = x509.getDEREncodingSB();
    DSIGKeyInfoX509 *keyInfoX509 = signature->appendX509Data();

    char buf[1024];
    X509_NAME_oneline ( X509_get_subject_name ( x509.getOpenSSLX509() ), buf, 1024 );
    std::string str = buf;
    keyInfoX509->setX509SubjectName ( MAKE_UNICODE_STRING ( str.c_str() ) );
    const char *b64CertStr = sb.rawCharBuffer();
    XMLCh *b64Cert = XMLString::transcode ( b64CertStr );
    keyInfoX509->appendX509Certificate ( b64Cert );
    XSEC_RELEASE_XMLCH ( b64Cert );

    XMLCh* xmlBasePath = XMLString::transcode(Path::GetFileName(basePath).c_str());
    signature->createReference(xmlBasePath, hash);
    XSEC_RELEASE_XMLCH ( xmlBasePath );

    signature->sign();

    std::string signedBuffer;
    domXml.domToString(signature->getParentDocument(), signedBuffer);

    bufferOut.AddData(signedBuffer);

    return true;
}

bool XmlDetachedSignatureAlgorithm::verifyBuffer(
    const SafeBuffer& bufferToVerify,
    X509 * const certificate,
    const std::string& basePath)
{
    bool validate = false;

    std::string baseURI = "file://";
    baseURI += basePath;
    baseURI += "/";

    DomXml domXml;
    DOMDocument * dom = domXml.bufferToDom ( bufferToVerify );

    XSECProvider prov;
    DSIGSignature * sig = NULL;

    try
    {
        OpenSSLCryptoX509 *x509 = NULL;
        std::auto_ptr<OpenSSLCryptoX509> openSSLCert(new OpenSSLCryptoX509(certificate));

        sig = prov.newSignatureFromDOM(dom);
        URILocalResolver resolver(baseURI);
        sig->setURIResolver(&resolver);
        sig->load();

        if (certificate != NULL)
        {
            x509 = openSSLCert.get();
            sig->setSigningKey(x509->clonePublicKey());
        }
        else
        {
            SafeBuffer base64Cert;
            ExtractCertificate extractCertificate(bufferToVerify);

            if (extractCertificate.getBase64Certificate(base64Cert))
            {
                OpenSSLCertificate cert(base64Cert);
                x509 = cert.getX509();
                sig->setSigningKey(x509->clonePublicKey());
            }
            else
            {
                logger->Debug("X509 not found");
                return validate;
            }
        }

        validate = sig->verify();

        if (validate == false)
        {
            char * msg = XMLString::transcode(sig->getErrMsgs());
            logger->Debug("ERROR - " + std::string(msg));
            XMLString::release(&msg);
        }
    }
    catch (XSECException& ex)
    {
        char *msg = XMLString::transcode(ex.getMsg());
        std::stringstream ss;
        ss << "FATAL - " << msg;
        logger->Debug(ss.str());
        XMLString::release(&msg);
    }
    catch (...)
    {
        logger->Debug("FATAL - error " + std::string(__func__));
    }

    if (sig)
    {
        prov.releaseSignature(sig);
    }

    return validate;
}
