/**
 * @file XmlDetachedSignatureAlgorithm.hxx
 * @brief Header class for XML detached signatures.
 * @author	Eduardo Fernández
 * @date	09/10/2018
 */

#ifndef XML_DETACHED_SIGNATURE_ALGORITHM_HXX
#define XML_DETACHED_SIGNATURE_ALGORITHM_HXX

#include <System/SafeBuffer.h>
#include <System/Security/Cryptography/CertificateExtensionBase.hxx>
#include <System/Security/Signed.hxx>
#include <System/Security/XmlDigitalSignature/XmlSignatureAlgorithm.hxx>
#include <xsec/enc/OpenSSL/OpenSSLCryptoX509.hpp>
#include <xsec/enc/XSECCryptoKey.hpp>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace System
    {
        namespace Security
        {
            /**
             * @class	XmlDetachedSignatureAlgorithm
             * @brief	Class for XML detached signatures.
             * @author	Eduardo Fernández
             * @date	10/10/2018
             */
            class XmlDetachedSignatureAlgorithm : public XmlSignatureAlgorithm
            {
                public:

                    /**
                     * @fn	XmlDetachedSignatureAlgorithm();
                     * @brief	Class constructor.
                     * @author	Eduardo Fernández
                     * @date	10/10/2018
                     */
                    XmlDetachedSignatureAlgorithm();

                    /**
                     * @fn	virtual ~XmlDetachedSignatureAlgorithm();
                     * @brief	Class destructor.
                     * @author	Eduardo Fernández
                     * @date	10/10/2018
                     */
                    virtual ~XmlDetachedSignatureAlgorithm();

                    /**
                     * @fn virtual bool signBuffer(
                               const Smartmatic::System::SafeBuffer& bufferToSign,
                               Smartmatic::System::SafeBuffer& signature,
                               const Signed::SignedType& signatureType = Signed::FULL_SIGNED,
                               Smartmatic::System::Security::Cryptography::CertificateExtensionBase * const certificate = NULL,
                               const hashMethod& hash = HASH_SHA256,
                               const std::string& basePath = "");
                     * @brief  method to sign a XML buffer with a certificate.
                     * @author Eduardo Fernández
                     * @date   10/10/2018
                     * @param  bufferToSign buffer to sign.
                     * @param  basePath base path of the buffer to sign.
                     * @param  signature signature buffer.
                     * @param  signatureType signature type.
                     * @param  certificate complete certificate information.
                     * @param  hash hash method.
                     * @return true if signing is successful.
                     */
                    virtual bool signBuffer(
                        const Smartmatic::System::SafeBuffer& bufferToSign,
                        Smartmatic::System::SafeBuffer& signature,
                        const Signed::SignedType& signatureType = Signed::FULL_SIGNED,
                        Smartmatic::System::Security::Cryptography::CertificateExtensionBase * const certificate = NULL,
                        const hashMethod& hash = HASH_SHA256,
                        const std::string& basePath = "");

                    /**
                     * @fn virtual bool verifyBuffer(
                               SafeBuffer& bufferToVerify,
                               X509 * const certificate,
                               const std::string& basePath = "");
                     * @brief  method to verify a XML signature with a certificate.
                     * @author Eduardo Fernández
                     * @date   10/10/2018
                     * @param  bufferToVerify Buffer including XML signature.
                     * @param  certificate X509 certificate.
                     * @param  basePath reference information.
                     * @return true if verification is successful.
                     */
                    virtual bool verifyBuffer(
                        const Smartmatic::System::SafeBuffer& bufferToVerify,
                        X509 * const certificate,
                        const std::string& basePath = "");

                private:

                    static Smartmatic::Log::ISMTTLog * logger; ///<logger
            };
        }
    }
}

#endif /* XML_DETACHED_SIGNATURE_ALGORITHM_HXX */
