/**
 * @file XmlEnvelopedSignatureAlgorithm.cxx
 * @brief Body class for XmlEnvelopedSignatureAlgorithm file.
 * @author  Eduardo Fernández
 * @date    09/10/2018
 */

#include "XmlEnvelopedSignatureAlgorithm.hxx"

#include <System/Security/OpenSSLCertificate.hxx>
#include <System/XML/DomXml.hxx>
#include <System/XML/ExtractCertificate.hxx>
#include <xsec/framework/XSECProvider.hpp>
#include <xsec/enc/OpenSSL/OpenSSLCryptoKeyRSA.hpp>
#include <xsec/enc/OpenSSL/OpenSSLCryptoKeyDSA.hpp>
#include <xsec/enc/OpenSSL/OpenSSLCryptoKeyEC.hpp>
#include <xsec/dsig/DSIGReference.hpp>
#include <xsec/dsig/DSIGKeyInfoX509.hpp>
#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <memory>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///< macro for gettext
#else
#define _(String) (String) ///< macro without gettext
#endif

#define N_(String) (String) ///< macro not translate

#define DSIGNS_PREFIX "" ///< prefix signed node

using namespace xercesc;
using namespace Smartmatic::Log;
using namespace Smartmatic::System;
using namespace Smartmatic::System::Security;
using namespace Smartmatic::System::Security::Cryptography;
using namespace Smartmatic::System::XML;

ISMTTLog * XmlEnvelopedSignatureAlgorithm::logger = SMTTLogManager::GetLogger ( "Smartmatic.System.Security.XmlEnvelopedSignatureAlgorithm", GETTEXT_PACKAGE );

XmlEnvelopedSignatureAlgorithm::XmlEnvelopedSignatureAlgorithm()
{

}

XmlEnvelopedSignatureAlgorithm::~XmlEnvelopedSignatureAlgorithm()
{

}

/**
 * @fn static XSECCryptoKey *getKey(EVP_PKEY *const privateKey, const signatureMethod& sigMethod)
 * @brief  method to get the openssl key depending on the private key and the signature method.
 * @author Eduardo Fernández
 * @date   09/10/2018
 * @param  privateKey the private key.
 * @param  signatureMethod the signature method.
 * @param  logger the logger.
 * @return the openssl key depending on the private key and the signature method.
 */
static XSECCryptoKey * getKey(EVP_PKEY * const primaryKey, signatureMethod& sigMethod, ISMTTLog * const logger)
{
    XSECCryptoKey *key = NULL;

    switch ( primaryKey->type )
    {
        case EVP_PKEY_RSA:
        case EVP_PKEY_RSA2:
            logger->Debug ( "Load Key RSA" );
            sigMethod = SIGNATURE_RSA;
            key = new OpenSSLCryptoKeyRSA ( primaryKey );
            break;

        case EVP_PKEY_DSA:
        case EVP_PKEY_DSA1:
        case EVP_PKEY_DSA2:
        case EVP_PKEY_DSA3:
        case EVP_PKEY_DSA4:
            logger->Debug ( "Load Key DSA" );
            sigMethod = SIGNATURE_DSA;
            key = new OpenSSLCryptoKeyDSA ( primaryKey );
            break;

        case EVP_PKEY_EC:
            logger->Debug ( "Load Key ECDSA" );
            sigMethod = SIGNATURE_ECDSA;
            key = new OpenSSLCryptoKeyEC ( primaryKey );
            break;

        default:
            sigMethod = SIGNATURE_NONE;
            break;
    }

    return key;
}

/**
 * @fn static void setKey(void * signature, OpenSSLCryptoX509 * const x509)
 * @brief  method to append the x509 certificate to the DSIGSignature object.
 * @author Eduardo Fernández
 * @date   09/10/2018
 * @param  signature the signature object.
 * @param  x509 X509 certificate.
 */
static void setKey(void * const sig, OpenSSLCryptoX509 * const x509, const hashMethod& hash)
{
    safeBuffer sb;

    DSIGReference *ref = ( ( DSIGSignature * ) sig )->createReference(MAKE_UNICODE_STRING(""), HASH_SHA256);
    ( void ) ref->appendEnvelopedSignatureTransform();

    sb = x509->getDEREncodingSB();
    DSIGKeyInfoX509 *keyInfoX509 = ( ( DSIGSignature * ) sig )->appendX509Data();
    char buf[1024];
    X509_NAME_oneline ( X509_get_subject_name ( x509->getOpenSSLX509() ), buf, 1024 );
    std::string str = buf;

    keyInfoX509->setX509SubjectName ( MAKE_UNICODE_STRING ( str.c_str() ) );
    const char *b64CertStr = sb.rawCharBuffer();
    XMLCh *b64Cert = XMLString::transcode ( b64CertStr );
    keyInfoX509->appendX509Certificate ( b64Cert );
    XSEC_RELEASE_XMLCH ( b64Cert );
}

/**
 * @fn static bool validatePrivateKeyAndSigningBuffer(
           EVP_PKEY * const privateKey,
           const Smartmatic::System::SafeBuffer& bufferToSign)
 * @brief  method to validate the private key and the buffer to sign.
 * @author Eduardo Fernández
 * @date   09/10/2018
 * @param  privateKey the private key.
 * @param  bufferToSign the buffer to sign.
 * @param  logger the logger.
 * @return true if the the private key and the buffer to sign are valid.
 */
static bool validatePrivateKeyAndSigningBuffer(
    EVP_PKEY * const privateKey,
    const SafeBuffer& bufferToSign,
    ISMTTLog * const logger)
{
    bool validate = true;

    if (privateKey == NULL)
    {
        logger->Debug("ERROR - Private key");
        validate = false;
    }

    if ((validate) && (bufferToSign.GetSize() == 0))
    {
        logger->Debug("ERROR - Xml Buffer");
        validate = false;
    }

    return validate;
}

/**
 * @fn static bool validateBufferToVerify(const Smartmatic::System::SafeBuffer& bufferToVerify)
 * @brief  method to validate the buffer to verify.
 * @author Eduardo Fernández
 * @date   09/10/2018
 * @param  bufferToVerify the buffer to verify.
 * @param  logger the logger.
 * @return true if the buffer to verify is valid.
 */
static bool validateBufferToVerify(const SafeBuffer& bufferToVerify, ISMTTLog * const logger)
{
    bool validate = true;

    if (bufferToVerify.GetSize() == 0)
    {
        logger->Debug("ERROR - Xml Buffer");
        validate = false;
    }

    return validate;
}

bool XmlEnvelopedSignatureAlgorithm::signBuffer(
    const SafeBuffer& bufferToSign,
    Smartmatic::System::SafeBuffer& signature,
    const Signed::SignedType& signatureType,
    CertificateExtensionBase * const certificate,
    const hashMethod& hash,
    const std::string&)
{
    signature.Clear();

    EVP_PKEY * const privateKey = certificate->LoadCertifiedPrivateKey();
    X509 * const cert = signatureType == Signed::FULL_SIGNED ? certificate->LoadCertifiedX509() : NULL;
    bool validate = false;

    if (validatePrivateKeyAndSigningBuffer(privateKey, bufferToSign, logger) == false)
        return validate;

    OpenSSLCryptoX509 *openSSLCert = NULL;

    XSECProvider prov;
    DSIGSignature * const sig = prov.newSignature();

    try
    {
        signatureMethod sigMethod;
        XSECCryptoKey * const key = getKey(privateKey, sigMethod, logger);

        if (key == NULL)
        {
            logger->Debug("ERROR - Error key");
            return validate;
        }

        DomXml domXml;
        DOMDocument * const doc = domXml.bufferToDom(bufferToSign);
        DOMElement * const rootElem = doc->getDocumentElement();

        if ( rootElem == NULL )
        {
            logger->Debug("ERROR - Xml format");
            delete key;
            return validate;
        }

        sig->setSigningKey(key);
        sig->setDSIGNSPrefix(MAKE_UNICODE_STRING(DSIGNS_PREFIX));
        sig->setPrettyPrint(true);

        DOMElement * const sigNode = sig->createBlankSignature(doc, CANON_C14N_NOC, sigMethod, hash);
        rootElem->appendChild(sigNode);
        rootElem->appendChild(doc->createTextNode(MAKE_UNICODE_STRING("\n")));

        if (cert)
        {
            openSSLCert = new OpenSSLCryptoX509(cert);
        }

        if (openSSLCert != NULL)
        {
            setKey(sig, openSSLCert, hash);
        }

        sig->sign();

        prov.releaseSignature(sig);

        validate = true;
        domXml.domToBuffer(doc, signature);
    }
    catch (...)
    {
        logger->Debug("ERROR - Signed");
        prov.releaseSignature(sig);
    }

    if (openSSLCert)
    {
        delete openSSLCert;
        openSSLCert = NULL;
    }

    return validate;
}

bool XmlEnvelopedSignatureAlgorithm::verifyBuffer(
    const SafeBuffer& bufferToVerify,
    X509 * const certificate,
    const std::string&)
{
    bool validate = false;

    if (validateBufferToVerify(bufferToVerify, logger) == false)
        return validate;

    SafeBuffer base64Cert;
    ExtractCertificate extractCertificate(bufferToVerify);
    extractCertificate.getBase64Certificate(base64Cert);
    OpenSSLCertificate base64X509(base64Cert);

    XSECProvider prov;
    DSIGSignature *sig = NULL;

    try
    {
        OpenSSLCryptoX509 *x509 = NULL;
        std::auto_ptr<OpenSSLCryptoX509> openSSLCert(new OpenSSLCryptoX509(certificate));

        if (base64X509.getX509() != NULL)
        {
            x509 = base64X509.getX509();
        }
        else if (certificate != NULL)
        {
            x509 = openSSLCert.get();
        }
        else
        {
            logger->Debug("ERROR - Certificate");
            return validate;
        }

        DomXml domXml;
        DOMDocument * const doc = domXml.bufferToDom(bufferToVerify);
        sig = prov.newSignatureFromDOM(doc);

        sig->setSigningKey(x509->clonePublicKey());
        sig->load();

        validate = sig->verify();
    }
    catch (...)
    {
        logger->Debug("ERROR - Validate");
    }

    if (sig)
    {
        prov.releaseSignature(sig);
    }

    return validate;
}
