/**
 * @file DomXml.cxx
 * @brief Body class DomXml
 */

#include "DomXml.hxx"
#include <sstream>
#include <Log/SMTTLogManager.h>
#include <xsec/framework/XSECProvider.hpp>
#include <xercesc/framework/MemBufInputSource.hpp>
#include <xercesc/framework/MemBufFormatTarget.hpp>
#include <xercesc/dom/DOMImplementation.hpp>
#include <xercesc/dom/DOMImplementationLS.hpp>
#include <xercesc/dom/DOMImplementationRegistry.hpp>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

#define _ERROR_PARSER_XML_TO_STRING_ N_("Smartmatic.System.XML.DomXml.ErrorDuringParserXmlToString") ///< text for log

using namespace Smartmatic::System::XML;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::Log;

ISMTTLog * DomXml::logger = SMTTLogManager::GetLogger ( "Smartmatic.System.XML.DomXml", GETTEXT_PACKAGE );

DomXml::DomXml()
{
    this->fileParser = new xercesc::XercesDOMParser;
}

DomXml::~DomXml()
{
    if ( fileParser )
    {
        fileParser->resetDocumentPool();
        delete fileParser;
        fileParser = NULL;
    }
}

void DomXml::domToString ( xercesc::DOMDocument * doc, std::string & str ) throw ( XmlException )
{
    if ( doc == NULL )
    {
        throw XmlException ( _ ( _ERROR_PARSER_XML_TO_STRING_ ),
                             N_ ( _ERROR_PARSER_XML_TO_STRING_ ), logger );
    }

    XMLCh tempStr[100];
    XMLString::transcode ( "LS", tempStr, 99 );

    xercesc::DOMImplementation *impl = xercesc::DOMImplementationRegistry::getDOMImplementation ( tempStr );
    xercesc::DOMLSSerializer * theSerializer = ( ( xercesc::DOMImplementationLS * ) impl )->createLSSerializer();
    xercesc::DOMLSOutput * theOutputDesc = ( ( xercesc::DOMImplementationLS * ) impl )->createLSOutput();

    XMLCh encodeStr[100];
    XMLString::transcode ( "UTF-8", encodeStr, 99 );
    theOutputDesc->setEncoding ( encodeStr );

    xercesc::MemBufFormatTarget memTarget;
    theOutputDesc->setByteStream ( &memTarget );

    theSerializer->write ( doc, theOutputDesc );

    unsigned int bytesLen = memTarget.getLen();
    const char * rawBytes = ( const char * ) memTarget.getRawBuffer();

    str.append ( rawBytes, bytesLen );

    memTarget.flush();
    theOutputDesc->release();
    theSerializer->release();
}

xercesc::DOMDocument * DomXml::stringToDom ( const std::string & str )
{
    SafeBuffer buffer ( str );
    return bufferToDom ( buffer );
}

void DomXml::domToBuffer ( xercesc::DOMDocument * doc, Smartmatic::System::SafeBuffer & buffer ) throw ( XmlException )
{
    std::string str;
    domToString ( doc, str );
    buffer.Clear();
    buffer.AddData ( ( const unsigned char * ) str.c_str(), str.length() );
}

xercesc::DOMDocument * DomXml::bufferToDom ( const SafeBuffer & buffer )
{
    fileParser->reset();
    fileParser->setValidationScheme ( xercesc::XercesDOMParser::Val_Never );
    fileParser->setDoNamespaces ( true );
    fileParser->setDoSchema ( true );
    fileParser->setLoadExternalDTD ( false );

    xercesc::MemBufInputSource inputSource (
        ( const XMLByte * ) buffer.GetData(),
        ( int ) buffer.GetSize(),
        "system" // fake "system ID"
    );

    fileParser->parse ( inputSource );
    xercesc::DOMDocument *doc = fileParser->getDocument();

    return doc;
}

xercesc::DOMNode * DomXml::getNode ( xercesc::DOMDocument * doc, const std::string & name )
{
    xercesc::DOMNode * node = NULL;

    if ( doc != NULL )
    {
        node = doc->getDocumentElement();
    }

    return getNode ( node, name );
}

xercesc::DOMNode * DomXml::getNode ( xercesc::DOMNode * node, const std::string & name )
{
    xercesc::DOMNode * temp = node;

    if ( temp != NULL && temp->getNodeType() == xercesc::DOMNode::ELEMENT_NODE &&
            strEquals ( temp->getNodeName(), name.c_str() ) )
    {
        return temp;
    }

    if ( temp != NULL )
    {
        temp = temp->getFirstChild();
    }

    while ( temp != NULL &&
            ! ( temp->getNodeType() == xercesc::DOMNode::ELEMENT_NODE &&
                strEquals ( temp->getNodeName(), name.c_str() ) ) )
    {
        temp = temp->getNextSibling();
    }

    return temp;
}
