/**
 * @file DomXml.hxx
 * @brief Header class DomXml
 */

#ifndef __DOM__XML__H__
#define __DOM__XML__H__

#include <string>
#include <xercesc/dom/DOMDocument.hpp>
#include <xercesc/parsers/XercesDOMParser.hpp>
#include <System/Exception/XmlException.hxx>
#include <System/SafeBuffer.h>

namespace Smartmatic
{
    namespace Log
    {
        class ISMTTLog;
    }

    namespace System
    {
        namespace XML
        {
            /**
             * @class	DomXml
             * @brief	class for parser dom to xml file.
             * @author	Yadickson Soto &lt;yadickson.soto@smartmatic.com&gt;
             * @date	03-04-2011
             */
            class DomXml
            {
                public:
                    /**
                     * @fn	DomXml::DomXml()
                     * @brief	class constructor
                     * @author	Yadickson Soto
                     * @date	30/10/2012
                     */
                    DomXml();
                    /**
                     * @fn	virtual DomXml::~DomXml();
                     * @brief	class destructor.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     */
                    virtual ~DomXml();
                    /**
                     * @fn	void DomXml::domToString(xercesc::DOMDocument * doc,
                     * 			std::string & str) throw (Smartmatic::System::Exception::XmlException);
                     * @brief	class for parser dom to string.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @exception	Smartmatic::System::Exception::XmlException	Thrown when an XML error condition occurs.
                     * @param	doc	dom document.
                     * @param	str	string
                     * @throw XmlException.
                     */
                    void domToString ( xercesc::DOMDocument * doc, std::string & str )
                    throw ( Smartmatic::System::Exception::XmlException );
                    /**
                     * @fn	xercesc::DOMDocument * DomXml::stringToDom(const std::string & str);
                     * @brief	class for parser string to dom.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param	str	string.
                     * @return	dom document
                     */
                    xercesc::DOMDocument * stringToDom ( const std::string & str );
                    /**
                     * @fn	void DomXml::domToBuffer(xercesc::DOMDocument * doc,
                     * 			Smartmatic::System::SafeBuffer & buffer) throw (Smartmatic::System::Exception::XmlException);
                     * @brief	class for parser dom to SafeBuffer.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @exception	Smartmatic::System::Exception::XmlException	Thrown when an XML error condition occurs.
                     * @param	doc			dom document.
                     * @param	buffer	buffer
                     * @throw XmlException.
                     */
                    void domToBuffer ( xercesc::DOMDocument * doc, Smartmatic::System::SafeBuffer & buffer )
                    throw ( Smartmatic::System::Exception::XmlException );
                    /**
                     * @fn	xercesc::DOMDocument * DomXml::bufferToDom(const Smartmatic::System::SafeBuffer & buffer);
                     * @brief	class for parser buffer to dom.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param	buffer	buffer.
                     * @return	dom document
                     */
                    xercesc::DOMDocument * bufferToDom ( const Smartmatic::System::SafeBuffer & buffer );
                    /**
                     * @fn	xercesc::DOMNode * DomXml::getNode(xercesc::DOMDocument * doc, const std::string & name );
                     * @brief	getter node from dom document by name.
                     * @author	Yadickson Soto
                     * @date	30/10/2012
                     * @param	doc	document node root.
                     * @param	name	node name.
                     * @return	dom node.
                     */
                    xercesc::DOMNode * getNode ( xercesc::DOMDocument * doc, const std::string & name );
                    /**
                     * @fn	xercesc::DOMNode * DomXml::getNode(xercesc::DOMNode * node, const std::string & name );
                     * @brief	method for get node.
                     * @author	Youssef.tourkmani
                     * @date	30/10/2012
                     * @param	node	dom node root.
                     * @param	name	node name.
                     * @return	dom node.
                     */
                    xercesc::DOMNode * getNode ( xercesc::DOMNode * node, const std::string & name );

                private:

                    /**
                     * @brief  Disable copy constructor.
                     * @author manuel.arguelles
                     * @date   Sep 12, 2017
                     */
                    DomXml(const DomXml&);

                    /**
                     * @brief  Disable copy.
                     * @author manuel.arguelles
                     * @date   Sep 12, 2017
                     */
                    DomXml& operator=(const DomXml&);

                private:

                    xercesc::XercesDOMParser * fileParser; ///<file parser
                    static Smartmatic::Log::ISMTTLog * logger; ///<logger
            };
        }
    }
}

#endif /*__DOM__XML__H__*/
