#ifndef TEST_RESOURCE_PATH_PROVIDER_TEST
#define TEST_RESOURCE_PATH_PROVIDER_TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <tests/MockLanguageController.hxx>
#include <tests/MockResourcePathProvider.hxx>
#include <tests/MockFile.hxx>
#include <tests/MockDirectory.hxx>

class ResourcePathProviderTest : public CPPUNIT_NS::TestFixture
{
    private:

        typedef Smartmatic::Environment::LanguageController LanguageController;
        typedef Smartmatic::Environment::ResourcePathProvider ResourcePathProvider;
        typedef Smartmatic::System::IO::File File;
        typedef Smartmatic::System::IO::Directory Directory;

        ResourcePathProvider * provider;

    public:

        ResourcePathProviderTest ()
        {
            provider = NULL;
        }

        void setUp()
        {
            provider = ResourcePathProvider::getInstance();
        }

        void tearDown()
        {
            delete provider;
            provider = NULL;
        }

        void testGetResourceByLanguageIfFullFileNameIsFound()
        {
            std::string lang = "fr";
            std::string country = "FR";
            std::string expected = "pathfr_FR.ext";
            std::string result;

            MockFile file;
            File::getInstance ( &file );

            MOCKPP_CHAINER_FOR_EXT ( MockFile, canOpenFile, ext ) canOpenFile_mock ( &file );

            canOpenFile_mock
            .expects ( MOCKPP_NS::once () )
            .will ( MOCKPP_NS::returnValue<bool> ( true ) );

            result = provider->getResourcePathByLanguage ( "path.ext", lang, country );

            CPPUNIT_ASSERT ( expected == result );

            file.verify();
        }

        void testGetResourceByLanguageIfSingleFileNameIsFound()
        {
            std::string lang = "fr";
            std::string country = "FR";
            std::string expected = "pathfr.ext";
            std::string result;

            MockFile file;
            File::getInstance ( &file );

            MOCKPP_CHAINER_FOR_EXT ( MockFile, canOpenFile, ext ) canOpenFile_mock ( &file );

            canOpenFile_mock
            .expects ( MOCKPP_NS::exactly ( 2 ) )
            .will ( MOCKPP_NS::onConsecutiveCalls (
                        MOCKPP_NS::returnValue<bool> ( false ),
                        MOCKPP_NS::returnValue<bool> ( true ) ) );

            result = provider->getResourcePathByLanguage ( "path.ext", lang, country );

            CPPUNIT_ASSERT ( expected == result );

            file.verify();
        }

        void testGetResourceByLanguage()
        {
            std::string lang = "fr";
            std::string country = "FR";
            std::string expected = "path.ext";
            std::string result;

            MockFile file;
            File::getInstance ( &file );

            MOCKPP_CHAINER_FOR_EXT ( MockFile, canOpenFile, ext ) canOpenFile_mock ( &file );

            canOpenFile_mock
            .expects ( MOCKPP_NS::exactly ( 2 ) )
            .will ( MOCKPP_NS::onConsecutiveCalls (
                        MOCKPP_NS::returnValue<bool> ( false ),
                        MOCKPP_NS::returnValue<bool> ( false ) ) );

            result = provider->getResourcePathByLanguage ( "path.ext", lang, country );

            CPPUNIT_ASSERT ( expected == result );

            file.verify();
        }

        void testGetResourceBySetLanguage()
        {
            std::string lang = "fr";
            std::string country = "FR";
            std::string expected = "pathfr_FR.ext";
            std::string result;

            MockLanguageController controller;
            LanguageController::getInstance ( &controller );

            MockFile file;
            File::getInstance ( &file );

            MOCKPP_CHAINER_FOR_EXT ( MockFile, canOpenFile, ext ) canOpenFile_mock ( &file );

            canOpenFile_mock
            .expects ( MOCKPP_NS::once () )
            .will ( MOCKPP_NS::returnValue<bool> ( true ) );

            LanguageController::getInstance()->setLanguage ( lang, country );
            result = provider->getResourcePathByLanguage ( "path.ext", lang, country );

            CPPUNIT_ASSERT ( expected == result );

            file.verify();
        }

        void testGetResourcePathToCurrentDirectory()
        {
            std::string lang = "fr";
            std::string country = "FR";
            std::string expected = "current/library/pathfr_FR.ext";
            std::string result;

            MockLanguageController controller;
            LanguageController::getInstance ( &controller );

            MockDirectory directory;
            Directory::getInstance ( &directory );

            MockFile file;
            File::getInstance ( &file );

            MOCKPP_CHAINER_FOR_EXT ( MockFile, canOpenFile, ext ) canOpenFile_mock ( &file );
            MOCKPP_CHAINER_FOR ( MockDirectory, getCurrentDirectory ) getCurrentDirectory_mock ( &directory );
            MOCKPP_CHAINER_FOR_EXT ( MockDirectory, DirectoryExists, ext ) DirectoryExists_mock ( &directory );

            canOpenFile_mock
            .expects ( MOCKPP_NS::exactly ( 2 ) )
            .will ( MOCKPP_NS::onConsecutiveCalls (
                        MOCKPP_NS::returnValue<bool> ( true ),
                        MOCKPP_NS::returnValue<bool> ( true ) ) );

            getCurrentDirectory_mock
            .expects ( MOCKPP_NS::once () )
            .will ( new MOCKPP_NS::ReturnStub<std::string> ( "current" ) );

            DirectoryExists_mock
            .expects ( MOCKPP_NS::once () )
            .will ( MOCKPP_NS::returnValue<bool> ( true ) );

            LanguageController::getInstance()->setLanguage ( lang, country );
            result = provider->GetResourcePath ( "library", "path.ext" );

            CPPUNIT_ASSERT ( expected == result );

            file.verify();
            directory.verify();
        }

        void testGetResourcePathNotCurrentDirectory()
        {
            std::string lang = "fr";
            std::string country = "FR";
            std::string expected = DATADIR"/library/pathfr_FR.ext";
            std::string result;

            MockLanguageController controller;
            LanguageController::getInstance ( &controller );

            MockDirectory directory;
            Directory::getInstance ( &directory );

            MockFile file;
            File::getInstance ( &file );

            MOCKPP_CHAINER_FOR_EXT ( MockFile, canOpenFile, ext ) canOpenFile_mock ( &file );
            MOCKPP_CHAINER_FOR ( MockDirectory, getCurrentDirectory ) getCurrentDirectory_mock ( &directory );
            MOCKPP_CHAINER_FOR_EXT ( MockDirectory, DirectoryExists, ext ) DirectoryExists_mock ( &directory );

            canOpenFile_mock
            .expects ( MOCKPP_NS::exactly ( 2 ) )
            .will ( MOCKPP_NS::onConsecutiveCalls (
                        MOCKPP_NS::returnValue<bool> ( false ),
                        MOCKPP_NS::returnValue<bool> ( true ) ) );

            getCurrentDirectory_mock
            .expects ( MOCKPP_NS::once () )
            .will ( new MOCKPP_NS::ReturnStub<std::string> ( "current" ) );

            DirectoryExists_mock
            .expects ( MOCKPP_NS::once () )
            .will ( MOCKPP_NS::returnValue<bool> ( false ) );

            LanguageController::getInstance()->setLanguage ( lang, country );
            result = provider->GetResourcePath ( "library", "path.ext" );

            CPPUNIT_ASSERT ( expected == result );

            file.verify();
            directory.verify();
        }

        CPPUNIT_TEST_SUITE ( ResourcePathProviderTest );

        CPPUNIT_TEST ( testGetResourceByLanguageIfFullFileNameIsFound );
        CPPUNIT_TEST ( testGetResourceByLanguageIfSingleFileNameIsFound );
        CPPUNIT_TEST ( testGetResourceByLanguage );
        CPPUNIT_TEST ( testGetResourceBySetLanguage );
        CPPUNIT_TEST ( testGetResourcePathToCurrentDirectory );
        CPPUNIT_TEST ( testGetResourcePathNotCurrentDirectory );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_RESOURCE_PATH_PROVIDER_TEST */
