#ifndef TEST_COMPRESSIONTEST
#define TEST_COMPRESSIONTEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <System/Compression/compression-class.h>
#include <System/Compression/ZipCompression.hxx>

class CompressionTest : public CPPUNIT_NS::TestFixture
{
    private:

        Smartmatic::System::Compression::CompressionClass * compressionClass;

    public:

        CompressionTest()
        {
            compressionClass = NULL;
        }

        void setUp()
        {
            compressionClass = Smartmatic::System::Compression::CompressionClass::getInstance ();
        }

        void tearDown()
        {
            delete compressionClass;
        }

        void testInflateNull()
        {
            Smartmatic::System::SafeBuffer dataIn;
            Smartmatic::System::SafeBuffer dataOut;

            CPPUNIT_ASSERT ( false == compressionClass->Inflate ( dataIn, dataOut ) );
        }

        void testInflateNoZip()
        {
            Smartmatic::System::SafeBuffer dataIn ( "abcdf" );
            Smartmatic::System::SafeBuffer dataOut;

            CPPUNIT_ASSERT ( false == compressionClass->Inflate ( dataIn, dataOut ) );
        }

        void testInflate()
        {
            unsigned char data[] = { 0x78, 0xda, 0x73, 0x74, 0x72, 0x76, 0x71, 0x75, 0x03, 0x00, 0x05, 0x7e, 0x01, 0x96 };
            unsigned int size = sizeof ( data ) / sizeof ( unsigned char );

            Smartmatic::System::SafeBuffer dataIn ( data, size );
            Smartmatic::System::SafeBuffer dataOut;

            CPPUNIT_ASSERT ( true == compressionClass->Inflate ( dataIn, dataOut ) );
        }

        void testDeflateNull()
        {
            Smartmatic::System::SafeBuffer dataIn;
            Smartmatic::System::SafeBuffer dataOut;

            CPPUNIT_ASSERT ( false == compressionClass->Deflate ( dataIn, dataOut ) );
        }

        void testDeflateString()
        {
            Smartmatic::System::SafeBuffer dataIn ( "String" );
            Smartmatic::System::SafeBuffer dataOut;

            CPPUNIT_ASSERT ( true == compressionClass->Deflate ( dataIn, dataOut ) );
        }

        void testDeflate()
        {
            Smartmatic::System::SafeBuffer dataIn ( "ABCDEF" );
            Smartmatic::System::SafeBuffer dataOut;

            CPPUNIT_ASSERT ( true == compressionClass->Deflate ( dataIn, dataOut ) );
            CPPUNIT_ASSERT ( "eNpzdHJ2cXUDAAV+AZY=" == dataOut.GetBase64Data() );
        }

        void testRegretion()
        {
            std::string input = "ABCDEF";
            std::string output;

            Smartmatic::System::SafeBuffer dataIn ( input );
            Smartmatic::System::SafeBuffer dataZip;
            Smartmatic::System::SafeBuffer dataOut;

            CPPUNIT_ASSERT ( true == compressionClass->Deflate ( dataIn, dataZip ) );
            CPPUNIT_ASSERT ( true == compressionClass->Inflate ( dataZip, dataOut ) );

            output.assign ( ( const char * ) dataOut.GetData(), dataOut.GetSize() );

            CPPUNIT_ASSERT ( input == output );
        }

        void testInflateNullSplit()
        {
            Smartmatic::System::SafeBuffer dataIn;
            Smartmatic::System::SafeBuffer dataZip;

            std::list < Smartmatic::System::SafeBuffer * > dataOut;
            std::list < Smartmatic::System::SafeBuffer * >::iterator it;

            compressionClass->Inflate ( dataIn, dataOut );

            for ( it = dataOut.begin();
                    it != dataOut.end();
                    ++it )
            {
                Smartmatic::System::SafeBuffer * pointer = ( *it );
                CPPUNIT_ASSERT ( NULL == pointer );
            }

            CPPUNIT_ASSERT ( 1 == dataOut.size() );
        }

        void testInflateNoZipSplit()
        {
            Smartmatic::System::SafeBuffer dataIn ( "XYZ" );
            Smartmatic::System::SafeBuffer dataZip;

            std::list < Smartmatic::System::SafeBuffer * > dataOut;
            std::list < Smartmatic::System::SafeBuffer * >::iterator it;

            compressionClass->Inflate ( dataIn, dataOut );

            for ( it = dataOut.begin();
                    it != dataOut.end();
                    ++it )
            {
                Smartmatic::System::SafeBuffer * pointer = ( *it );
                CPPUNIT_ASSERT ( NULL == pointer );
            }

            CPPUNIT_ASSERT ( 1 == dataOut.size() );
        }

        void testInflateSplit()
        {
            std::string input = "ABCDEF";

            Smartmatic::System::SafeBuffer dataIn ( input );
            Smartmatic::System::SafeBuffer dataZip;

            std::list < Smartmatic::System::SafeBuffer * > dataOut;
            std::list < Smartmatic::System::SafeBuffer * >::iterator it;

            CPPUNIT_ASSERT ( true == compressionClass->Deflate ( dataIn, dataZip ) );

            compressionClass->Inflate ( dataZip, dataOut );

            for ( it = dataOut.begin();
                    it != dataOut.end();
                    ++it )
            {
                Smartmatic::System::SafeBuffer * pointer = ( *it );

                CPPUNIT_ASSERT ( NULL != pointer );

                std::string output;
                output.assign ( ( const char * ) pointer->GetData(), pointer->GetSize() );
                CPPUNIT_ASSERT ( input == output );
            }

            CPPUNIT_ASSERT ( 1 == dataOut.size() );
        }

        void testInflateSplit4()
        {
            std::string input = "ABCDEF";
            unsigned short copy = 4;

            Smartmatic::System::SafeBuffer dataIn ( input );
            Smartmatic::System::SafeBuffer dataZip;
            Smartmatic::System::SafeBuffer dataCopy;

            std::list < Smartmatic::System::SafeBuffer * > dataOut;
            std::list < Smartmatic::System::SafeBuffer * >::iterator it;

            CPPUNIT_ASSERT ( true == compressionClass->Deflate ( dataIn, dataZip ) );

            for ( unsigned short i = 0; i < copy ; i++ )
            {
                dataCopy.AddData ( dataZip );
            }

            compressionClass->Inflate ( dataCopy, dataOut );

            for ( it = dataOut.begin();
                    it != dataOut.end();
                    ++it )
            {
                Smartmatic::System::SafeBuffer * pointer = ( *it );

                CPPUNIT_ASSERT ( NULL != pointer );

                std::string output;
                output.assign ( ( const char * ) pointer->GetData(), pointer->GetSize() );
                CPPUNIT_ASSERT ( input == output );
            }

            CPPUNIT_ASSERT ( copy == ( unsigned short ) dataOut.size() );
        }

        void testInflateSplitCommandAndData()
        {
            std::string input = "ABCDEF";

            Smartmatic::System::SafeBuffer dataIn ( input );
            Smartmatic::System::SafeBuffer dataZip;
            Smartmatic::System::SafeBuffer dataCopy;

            std::list < Smartmatic::System::SafeBuffer * > dataOut;
            std::list < Smartmatic::System::SafeBuffer * >::iterator it;

            CPPUNIT_ASSERT ( true == compressionClass->Deflate ( dataIn, dataZip ) );

            dataCopy.push_back ( 0x06 );
            dataCopy.AddData ( dataZip );

            compressionClass->Inflate ( dataCopy, dataOut );

            CPPUNIT_ASSERT ( 1 == dataOut.size() );

            it = dataOut.begin();
            Smartmatic::System::SafeBuffer * pointer = ( *it );

            CPPUNIT_ASSERT ( NULL == pointer );
        }

        void testInflateSplitDataAndCommand()
        {
            std::string input = "ABCDEF";

            Smartmatic::System::SafeBuffer dataIn ( input );
            Smartmatic::System::SafeBuffer dataZip;
            Smartmatic::System::SafeBuffer dataCopy;

            std::list < Smartmatic::System::SafeBuffer * > dataOut;
            std::list < Smartmatic::System::SafeBuffer * >::iterator it;

            CPPUNIT_ASSERT ( true == compressionClass->Deflate ( dataIn, dataZip ) );

            dataCopy.AddData ( dataZip );
            dataCopy.push_back ( 0x06 );

            compressionClass->Inflate ( dataCopy, dataOut );

            CPPUNIT_ASSERT ( 2 == dataOut.size() );

            it = dataOut.begin();
            Smartmatic::System::SafeBuffer * pointer = ( *it );

            CPPUNIT_ASSERT ( NULL != pointer );

            std::string output;
            output.assign ( ( const char * ) pointer->GetData(), pointer->GetSize() );
            CPPUNIT_ASSERT ( input == output );

            ++it;
            pointer = ( *it );
            CPPUNIT_ASSERT ( NULL == pointer );
        }

        CPPUNIT_TEST_SUITE ( CompressionTest );
        CPPUNIT_TEST ( testInflateNull );
        CPPUNIT_TEST ( testInflateNoZip );
        CPPUNIT_TEST ( testInflate );
        CPPUNIT_TEST ( testDeflateNull );
        CPPUNIT_TEST ( testDeflateString );
        CPPUNIT_TEST ( testDeflate );
        CPPUNIT_TEST ( testRegretion );
        CPPUNIT_TEST ( testInflateNullSplit );
        CPPUNIT_TEST ( testInflateSplit );
        CPPUNIT_TEST ( testInflateSplit4 );
        CPPUNIT_TEST ( testInflateSplitCommandAndData );
        CPPUNIT_TEST ( testInflateSplitDataAndCommand );
        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_COMPRESSIONTEST */
