#ifndef FAT_FILE_MANAGER_TEST
#define FAT_FILE_MANAGER_TEST

#ifdef ECUT_MAIN

#include "FatFileManagerStubs.hxx"
#include "MockBasicFile.hxx"
#include <System/IO/Directory.h>
#include <System/IO/FatFileManager.hxx>
#include <System/IO/File.h>
#include <System/Security/OpenSSLSecurity.hxx>
#include <cppunit/extensions/HelperMacros.h>
#include <tests/MockStorageDevice.hxx>
#include <tests/StoreMockFile.hxx>

class FatFileManagerTest : public CPPUNIT_NS::TestFixture
{
    private:

        typedef Smartmatic::System::IO::FatFileManager FatFileManager;
        typedef Smartmatic::System::IO::File File;
        typedef Smartmatic::System::IO::Directory Directory;
        typedef Smartmatic::System::IO::IDirectory IDirectory;
        typedef Smartmatic::System::SafeBuffer SafeBuffer;

    public:

        FatFileManagerTest() {}

        void setUp()
        {
            Smartmatic::System::Security::OpenSSLSecurity::init();
        }

        void tearDown()
        {
            Smartmatic::System::Security::OpenSSLSecurity::clear();
        }

        void testCopyInvalidPathWithoutDevices()
        {
            FatFileManager fatFiles(*Directory::getInstance(), *File::getInstance());

            CPPUNIT_ASSERT(fatFiles.copyFiles("/tmp/invalid") == false);
        }

        void testVerifyInvalidPathWithoutDevices()
        {
            FatFileManager fatFiles(*Directory::getInstance(), *File::getInstance());

            CPPUNIT_ASSERT(fatFiles.verifyFiles("/tmp/invalid") == false);
        }

        void testCopyWithoutDevices()
        {
            OneFile oneFileDirectory;
            FatFileManager fatFiles(oneFileDirectory, *File::getInstance());

            CPPUNIT_ASSERT(fatFiles.copyFiles("/tmp/") == false);
        }

        void testVerifyWithoutDevices()
        {
            FatFileManager fatFiles(*Directory::getInstance(), *File::getInstance());

            CPPUNIT_ASSERT(fatFiles.verifyFiles("/tmp/") == false);
        }

        void testCopyWithUnmountedDevice()
        {
            OneFile oneFileDirectory;
            FatFileManager fatFiles(oneFileDirectory, *File::getInstance());
            MockStorageDevice device("/dev/invalid");
            bool testResult;

            MOCKPP_CHAINER_FOR(MockStorageDevice, isMounted) chainerIsMount (&device);
            MOCKPP_CHAINER_FOR(MockStorageDevice, getMountPoint) chainerGetMountPoint (&device);
            MOCKPP_CHAINER_FOR(MockStorageDevice, unmount) chainerUnmount (&device);

            chainerIsMount.setDefaultStub(new MOCKPP_NS::ReturnStub<bool>(true));
            chainerGetMountPoint.setDefaultStub(new MOCKPP_NS::ReturnStub<std::string>(""));
            chainerUnmount.setDefaultStub(new MOCKPP_NS::ReturnStub<bool>(true));

            fatFiles.addStorageDevice(&device);
            testResult = fatFiles.copyFiles("/tmp/");
            fatFiles.removeStorageDevice(&device);

            CPPUNIT_ASSERT(testResult == false);
        }

        void testVerifyDirectoryWithOneFile()
        {
            StoreMockFile file;
            bool testResult;
            OneFile oneFile;
            FatFileManager fatFiles(oneFile, file);
            MockStorageDevice device("/dev/ok");

            file.openFileValues.push_back(StoreMockFile::OpenFileValues(SafeBuffer("ABCDE"), true));
            file.openFileValues.push_back(StoreMockFile::OpenFileValues(SafeBuffer("ABCDE"), true));

            MOCKPP_CHAINER_FOR(MockStorageDevice, isMounted) chainerIsMount (&device);
            MOCKPP_CHAINER_FOR(MockStorageDevice, getMountPoint) chainerGetMountPoint (&device);
            MOCKPP_CHAINER_FOR(MockStorageDevice, unmount) chainerUnmount (&device);

            chainerIsMount.setDefaultStub(new MOCKPP_NS::ReturnStub<bool>(true));
            chainerGetMountPoint.setDefaultStub(new MOCKPP_NS::ReturnStub<std::string>("/tmp"));
            chainerUnmount.setDefaultStub(new MOCKPP_NS::ReturnStub<bool>(true));

            fatFiles.addStorageDevice(&device);
            testResult = fatFiles.verifyFiles("/tmp");
            fatFiles.removeStorageDevice(&device);

            CPPUNIT_ASSERT(testResult == true);
        }

        void testVerifySubDirectoryAndFile()
        {
            StoreMockFile file;
            bool testResult;
            OneFilePerOneDirectory oneFilePerDirectory;
            FatFileManager fatFiles(oneFilePerDirectory, file);
            MockStorageDevice device("/dev/ok");
            SafeBuffer fileInRoot("FileInRoot");
            SafeBuffer fileInDirectory("FileInDirectory");

            file.openFileValues.push_back(StoreMockFile::OpenFileValues(fileInRoot, true));
            file.openFileValues.push_back(StoreMockFile::OpenFileValues(fileInRoot, true));
            file.openFileValues.push_back(StoreMockFile::OpenFileValues(fileInDirectory, true));
            file.openFileValues.push_back(StoreMockFile::OpenFileValues(fileInDirectory, true));

            MOCKPP_CHAINER_FOR(MockStorageDevice, isMounted) chainerIsMount (&device);
            MOCKPP_CHAINER_FOR(MockStorageDevice, getMountPoint) chainerGetMountPoint (&device);
            MOCKPP_CHAINER_FOR(MockStorageDevice, unmount) chainerUnmount (&device);

            chainerIsMount.setDefaultStub(new MOCKPP_NS::ReturnStub<bool>(true));
            chainerGetMountPoint.setDefaultStub(new MOCKPP_NS::ReturnStub<std::string>("/tmp"));
            chainerUnmount.setDefaultStub(new MOCKPP_NS::ReturnStub<bool>(true));

            fatFiles.addStorageDevice(&device);
            testResult = fatFiles.verifyFiles("/tmp");
            fatFiles.removeStorageDevice(&device);

            CPPUNIT_ASSERT(testResult == true);
        }

        void testCopyOneFile()
        {
            StoreMockFile file;
            bool testResult;
            OneFile oneFile;
            FatFileManager fatFiles(oneFile, file);
            MockStorageDevice device("/dev/ok");
            SafeBuffer fileBuffer("FileInRootToCopy");

            file.openFileValues.push_back(StoreMockFile::OpenFileValues(fileBuffer, true));
            file.saveReturnValues.push_back(true);

            MOCKPP_CHAINER_FOR(MockStorageDevice, isMounted) chainerIsMount (&device);
            MOCKPP_CHAINER_FOR(MockStorageDevice, getMountPoint) chainerGetMountPoint (&device);
            MOCKPP_CHAINER_FOR(MockStorageDevice, unmount) chainerUnmount (&device);

            chainerIsMount.setDefaultStub(new MOCKPP_NS::ReturnStub<bool>(true));
            chainerGetMountPoint.setDefaultStub(new MOCKPP_NS::ReturnStub<std::string>("/tmp"));
            chainerUnmount.setDefaultStub(new MOCKPP_NS::ReturnStub<bool>(true));

            fatFiles.addStorageDevice(&device);
            testResult = fatFiles.copyFiles("/tmp");
            fatFiles.removeStorageDevice(&device);

            CPPUNIT_ASSERT(testResult == true);
            CPPUNIT_ASSERT(file.saveBufferValues.size() == 1);

            StoreMockFile::SafeBufferValues& value(*(file.saveBufferValues.begin()));
            SafeBuffer& copiedBuffer(value.buffer);

            CPPUNIT_ASSERT(copiedBuffer.GetSize() != 0);
            CPPUNIT_ASSERT(copiedBuffer == fileBuffer);
        }

        void testCopyOneFileInRootAndDirectory()
        {
            StoreMockFile file;
            bool testResult;
            OneFilePerOneDirectory fileInRootAndDirectory;
            FatFileManager fatFiles(fileInRootAndDirectory, file);
            MockStorageDevice device("/dev/ok");
            SafeBuffer fileRootBuffer("FileInRootToCopy");
            SafeBuffer fileDirectoryBuffer("FileInDirectoryToCopy");

            file.openFileValues.push_back(StoreMockFile::OpenFileValues(fileRootBuffer, true));
            file.openFileValues.push_back(StoreMockFile::OpenFileValues(fileDirectoryBuffer, true));

            file.saveReturnValues.push_back(true);
            file.saveReturnValues.push_back(true);

            MOCKPP_CHAINER_FOR(MockStorageDevice, isMounted) chainerIsMount (&device);
            MOCKPP_CHAINER_FOR(MockStorageDevice, getMountPoint) chainerGetMountPoint (&device);
            MOCKPP_CHAINER_FOR(MockStorageDevice, unmount) chainerUnmount (&device);

            chainerIsMount.setDefaultStub(new MOCKPP_NS::ReturnStub<bool>(true));
            chainerGetMountPoint.setDefaultStub(new MOCKPP_NS::ReturnStub<std::string>("/tmp"));
            chainerUnmount.setDefaultStub(new MOCKPP_NS::ReturnStub<bool>(true));

            fatFiles.addStorageDevice(&device);
            testResult = fatFiles.copyFiles("/tmp");
            fatFiles.removeStorageDevice(&device);

            CPPUNIT_ASSERT(testResult == true);
            CPPUNIT_ASSERT(file.saveBufferValues.size() == 2);

            StoreMockFile::SafeBufferValues& value1(*(file.saveBufferValues.begin()));
            SafeBuffer& fileInRoot(value1.buffer);

            StoreMockFile::SafeBufferValues& value2(*(++file.saveBufferValues.begin()));
            SafeBuffer& fileInDirectory(value2.buffer);

            CPPUNIT_ASSERT(fileInRoot.GetSize() != 0);
            CPPUNIT_ASSERT(fileInRoot == fileRootBuffer);
            CPPUNIT_ASSERT(fileInDirectory.GetSize() != 0);
            CPPUNIT_ASSERT(fileInDirectory == fileDirectoryBuffer);
        }

        void testCopyOneFileToTwoDevices()
        {
            StoreMockFile file;
            bool testResult;
            OneFile oneFile;
            FatFileManager fatFiles(oneFile, file);
            MockStorageDevice deviceOne("/dev/first");
            MockStorageDevice deviceTwo("/dev/second");
            SafeBuffer fileBuffer("FileInRootToCopy");

            file.openFileValues.push_back(StoreMockFile::OpenFileValues(fileBuffer, true));
            file.openFileValues.push_back(StoreMockFile::OpenFileValues(fileBuffer, true));
            file.saveReturnValues.push_back(true);
            file.saveReturnValues.push_back(true);

            MOCKPP_CHAINER_FOR(MockStorageDevice, isMounted) chainerIsMount (&deviceOne);
            MOCKPP_CHAINER_FOR(MockStorageDevice, getMountPoint) chainerGetMountPoint (&deviceOne);
            MOCKPP_CHAINER_FOR(MockStorageDevice, unmount) chainerUnmount (&deviceOne);

            MOCKPP_CHAINER_FOR(MockStorageDevice, isMounted) chainer2IsMount (&deviceTwo);
            MOCKPP_CHAINER_FOR(MockStorageDevice, getMountPoint) chainer2GetMountPoint (&deviceTwo);
            MOCKPP_CHAINER_FOR(MockStorageDevice, unmount) chainer2Unmount (&deviceTwo);

            chainerIsMount.setDefaultStub(new MOCKPP_NS::ReturnStub<bool>(true));
            chainerGetMountPoint.setDefaultStub(new MOCKPP_NS::ReturnStub<std::string>("/tmp/disk1"));
            chainerUnmount.setDefaultStub(new MOCKPP_NS::ReturnStub<bool>(true));

            chainer2IsMount.setDefaultStub(new MOCKPP_NS::ReturnStub<bool>(true));
            chainer2GetMountPoint.setDefaultStub(new MOCKPP_NS::ReturnStub<std::string>("/tmp/disk2"));
            chainer2Unmount.setDefaultStub(new MOCKPP_NS::ReturnStub<bool>(true));

            fatFiles.addStorageDevice(&deviceOne);
            fatFiles.addStorageDevice(&deviceTwo);
            testResult = fatFiles.copyFiles("/tmp");
            fatFiles.removeStorageDevice(&deviceOne);
            fatFiles.removeStorageDevice(&deviceTwo);

            CPPUNIT_ASSERT(testResult == true);
            CPPUNIT_ASSERT(file.saveBufferValues.size() == 2);

            StoreMockFile::SafeBufferValues& valueDeviceOne(*(file.saveBufferValues.begin()));
            SafeBuffer& copiedToDeviceOne(valueDeviceOne.buffer);

            StoreMockFile::SafeBufferValues& valueDeviceTwo(*(++file.saveBufferValues.begin()));
            SafeBuffer& copiedToDeviceTwo(valueDeviceTwo.buffer);

            CPPUNIT_ASSERT(copiedToDeviceOne.GetSize() != 0);
            CPPUNIT_ASSERT(copiedToDeviceOne == fileBuffer);
            CPPUNIT_ASSERT(copiedToDeviceTwo.GetSize() != 0);
            CPPUNIT_ASSERT(copiedToDeviceTwo == fileBuffer);

            CPPUNIT_ASSERT(valueDeviceOne.file == "/tmp/disk1/tmp/File.txt");
            CPPUNIT_ASSERT(valueDeviceTwo.file == "/tmp/disk2/tmp/File.txt");
        }

        void testCopyOneFileToTwoDevicesWithPathStripped()
        {
            StoreMockFile file;
            bool testResult;
            OneFile oneFile;
            FatFileManager fatFiles(oneFile, file);
            MockStorageDevice deviceOne("/dev/first");
            MockStorageDevice deviceTwo("/dev/second");
            SafeBuffer fileBuffer("FileInRootToCopy");

            file.openFileValues.push_back(StoreMockFile::OpenFileValues(fileBuffer, true));
            file.openFileValues.push_back(StoreMockFile::OpenFileValues(fileBuffer, true));
            file.saveReturnValues.push_back(true);
            file.saveReturnValues.push_back(true);

            MOCKPP_CHAINER_FOR(MockStorageDevice, isMounted) chainerIsMount (&deviceOne);
            MOCKPP_CHAINER_FOR(MockStorageDevice, getMountPoint) chainerGetMountPoint (&deviceOne);
            MOCKPP_CHAINER_FOR(MockStorageDevice, unmount) chainerUnmount (&deviceOne);

            MOCKPP_CHAINER_FOR(MockStorageDevice, isMounted) chainer2IsMount (&deviceTwo);
            MOCKPP_CHAINER_FOR(MockStorageDevice, getMountPoint) chainer2GetMountPoint (&deviceTwo);
            MOCKPP_CHAINER_FOR(MockStorageDevice, unmount) chainer2Unmount (&deviceTwo);

            chainerIsMount.setDefaultStub(new MOCKPP_NS::ReturnStub<bool>(true));
            chainerGetMountPoint.setDefaultStub(new MOCKPP_NS::ReturnStub<std::string>("/tmp/disk1"));
            chainerUnmount.setDefaultStub(new MOCKPP_NS::ReturnStub<bool>(true));

            chainer2IsMount.setDefaultStub(new MOCKPP_NS::ReturnStub<bool>(true));
            chainer2GetMountPoint.setDefaultStub(new MOCKPP_NS::ReturnStub<std::string>("/tmp/disk2"));
            chainer2Unmount.setDefaultStub(new MOCKPP_NS::ReturnStub<bool>(true));

            fatFiles.addStorageDevice(&deviceOne);
            fatFiles.addStorageDevice(&deviceTwo);
            testResult = fatFiles.copyFiles("/tmp", "/tmp");
            fatFiles.removeStorageDevice(&deviceOne);
            fatFiles.removeStorageDevice(&deviceTwo);

            CPPUNIT_ASSERT(testResult == true);
            CPPUNIT_ASSERT(file.saveBufferValues.size() == 2);

            StoreMockFile::SafeBufferValues& valueDeviceOne(*(file.saveBufferValues.begin()));
            SafeBuffer& copiedToDeviceOne(valueDeviceOne.buffer);

            StoreMockFile::SafeBufferValues& valueDeviceTwo(*(++file.saveBufferValues.begin()));
            SafeBuffer& copiedToDeviceTwo(valueDeviceTwo.buffer);

            CPPUNIT_ASSERT(copiedToDeviceOne.GetSize() != 0);
            CPPUNIT_ASSERT(copiedToDeviceOne == fileBuffer);
            CPPUNIT_ASSERT(copiedToDeviceTwo.GetSize() != 0);
            CPPUNIT_ASSERT(copiedToDeviceTwo == fileBuffer);

            CPPUNIT_ASSERT(valueDeviceOne.file == "/tmp/disk1/File.txt");
            CPPUNIT_ASSERT(valueDeviceTwo.file == "/tmp/disk2/File.txt");
        }

        void testCopyAndVerifyEmptyDirectory()
        {
            StoreMockFile file;
            bool testCopy;
            bool testVerify;
            std::vector<std::string> structure;
            EmptyDirectory directory;
            FatFileManager fatFiles(directory, file);
            MockStorageDevice device("/dev/first");

            MOCKPP_CHAINER_FOR(MockStorageDevice, isMounted) chainerIsMount (&device);
            MOCKPP_CHAINER_FOR(MockStorageDevice, getMountPoint) chainerGetMountPoint (&device);
            MOCKPP_CHAINER_FOR(MockStorageDevice, unmount) chainerUnmount (&device);

            chainerIsMount.setDefaultStub(new MOCKPP_NS::ReturnStub<bool>(true));
            chainerGetMountPoint.setDefaultStub(new MOCKPP_NS::ReturnStub<std::string>("/tmp/device"));
            chainerUnmount.setDefaultStub(new MOCKPP_NS::ReturnStub<bool>(true));

            fatFiles.addStorageDevice(&device);
            testCopy = fatFiles.copyFiles("/tmp/empty", "/tmp");
            fatFiles.removeStorageDevice(&device);
            CPPUNIT_ASSERT(testCopy == true);

            structure = directory.getStructure();
            CPPUNIT_ASSERT(structure.size() != 0);
            CPPUNIT_ASSERT(structure.size() == 1);
            CPPUNIT_ASSERT(structure[0] == "/tmp/device/empty/");

            fatFiles.addStorageDevice(&device);
            testVerify = fatFiles.verifyFiles("/tmp/empty", "/tmp");
            fatFiles.removeStorageDevice(&device);
            CPPUNIT_ASSERT(testVerify == true);
        }

        CPPUNIT_TEST_SUITE(FatFileManagerTest);

        CPPUNIT_TEST(testCopyInvalidPathWithoutDevices);
        CPPUNIT_TEST(testVerifyInvalidPathWithoutDevices);
        CPPUNIT_TEST(testCopyWithoutDevices);
        CPPUNIT_TEST(testVerifyWithoutDevices);
        CPPUNIT_TEST(testCopyWithUnmountedDevice);
        CPPUNIT_TEST(testVerifyDirectoryWithOneFile);
        CPPUNIT_TEST(testVerifySubDirectoryAndFile);
        CPPUNIT_TEST(testCopyOneFile);
        CPPUNIT_TEST(testCopyOneFileInRootAndDirectory);
        CPPUNIT_TEST(testCopyOneFileToTwoDevices);
        CPPUNIT_TEST(testCopyOneFileToTwoDevicesWithPathStripped);
        CPPUNIT_TEST(testCopyAndVerifyEmptyDirectory);

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* FAT_FILE_MANAGER_TEST */
