#ifndef TEST_HMACSHA256TEST
#define TEST_HMACSHA256TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <System/Security/Cryptography/HmacSha256.hxx>
#include <System/Exception/CryptoException.hxx>
#include <System/SafeBuffer.h>
#include <System/Utils/StringUtils.hxx>

using namespace Smartmatic::System::Security::Cryptography;
using namespace Smartmatic::System::Exception;
using namespace Smartmatic::System::Utils;
using namespace Smartmatic::System;

class HmacSha256Test : public CPPUNIT_NS::TestFixture
{
    public:

        void test_empty_key_empty_data ()
        {
            const std::string expected = "b613679a0814d9ec772f95d778c35fc5ff1697c493715653c6c712144292c5ad";
            SafeBuffer expectedBuffer;

            SafeBuffer key;
            SafeBuffer digest;

            HmacSha256 hmac(key);

            digest = hmac.getDigest();

            StringUtils::stringToBufferHex(expected, expectedBuffer);

            CPPUNIT_ASSERT ( digest.GetSize() == 32);
            CPPUNIT_ASSERT ( digest == expectedBuffer );
        }

        void test_empty_key ()
        {
            const std::string expected = "deb693b334862cd9e157d0df7d471b836edd55972a196147e18dd35ea8b1038f";
            SafeBuffer expectedBuffer;

            SafeBuffer key;
            SafeBuffer digest;

            HmacSha256 hmac(key);

            SafeBuffer message("This is the message to sign");
            hmac.update(message);

            digest = hmac.getDigest();

            StringUtils::stringToBufferHex(expected, expectedBuffer);

            CPPUNIT_ASSERT ( digest.GetSize() == 32 );
            CPPUNIT_ASSERT ( digest == expectedBuffer );
        }

        void test_empty_data ()
        {
            size_t rawKeySize = 32;
            unsigned char rawKey[32] = {
                0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08,
                0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18,
                0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28,
                0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38,
            };

            const std::string expected = "7e22f457a54d245eed0185a0d3dd878c062bd4800263f4d50b9d5ae6a26402bb";
            SafeBuffer expectedBuffer;

            SafeBuffer key(rawKey, rawKeySize);
            SafeBuffer digest;

            HmacSha256 hmac(key);

            digest = hmac.getDigest();

            StringUtils::stringToBufferHex(expected, expectedBuffer);

            CPPUNIT_ASSERT ( digest.GetSize() == 32);
            CPPUNIT_ASSERT ( digest == expectedBuffer );
        }

        void test_get_hmac ()
        {
            size_t rawKeySize = 32;
            unsigned char rawKey[32] = {
                0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08,
                0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18,
                0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28,
                0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38,
            };

            const std::string expected = "8ca1566e872189ce66c54696e1db7220a7171b6f549e6b08ad604f1900ff7a5a";
            SafeBuffer expectedBuffer;

            SafeBuffer key(rawKey, rawKeySize);
            SafeBuffer digest;

            HmacSha256 hmac(key);

            SafeBuffer message("This is the message to sign");
            hmac.update(message);

            digest = hmac.getDigest();


            StringUtils::stringToBufferHex(expected, expectedBuffer);

            CPPUNIT_ASSERT ( digest.GetSize() == 32 );
            CPPUNIT_ASSERT ( digest == expectedBuffer );
        }

        void test_check_valid_hmac ()
        {
            SafeBuffer message("This is the message to validate");
            SafeBuffer key("This IS the KeY");
            SafeBuffer codeBuffer;
            const std::string code = "8bc3fff66c9742424a76b9a25c5469a91d5f144796fab74353baacc1f4fb7815";

            StringUtils::stringToBufferHex(code, codeBuffer);

            HmacSha256 hmac(key);

            hmac.update(message);

            CPPUNIT_ASSERT ( HmacSha256Validate(hmac, codeBuffer) == true);
        }

        void test_check_invalid_hmac ()
        {
            SafeBuffer message("This is the message to validate");
            SafeBuffer key("This IS the KeY");
            SafeBuffer codeBuffer;
            const std::string code = "9bc3fff66c9742424a76b9a25c5469a91d5f144796fab74353baacc1f4fb7815";

            StringUtils::stringToBufferHex(code, codeBuffer);

            HmacSha256 hmac(key);

            hmac.update(message);

            CPPUNIT_ASSERT ( HmacSha256Validate(hmac, codeBuffer) == false);
        }

        CPPUNIT_TEST_SUITE (HmacSha256Test );

        CPPUNIT_TEST ( test_empty_key_empty_data );
        CPPUNIT_TEST ( test_empty_data );
        CPPUNIT_TEST ( test_empty_key );
        CPPUNIT_TEST ( test_get_hmac );
        CPPUNIT_TEST ( test_check_valid_hmac );
        CPPUNIT_TEST ( test_check_invalid_hmac );

        CPPUNIT_TEST_SUITE_END ();
};

#endif /* ECUT_MAIN */

#endif /* TEST_HMACSHA256TEST */
