#ifndef TEST_STRING_UTILS_TEST
#define TEST_STRING_UTILS_TEST

#ifdef ECUT_MAIN

#include <cppunit/extensions/HelperMacros.h>
#include <System/XML/DomXml.hxx>
#include <System/Security/XmlSecurityC.hxx>

class DomXmlTest : public CPPUNIT_NS::TestFixture
{
    public:

        typedef Smartmatic::System::Exception::XmlException XmlException;

        Smartmatic::System::XML::DomXml * domXml;

        DomXmlTest()
        {
            domXml = NULL;
        }

        void setUp()
        {
            Smartmatic::System::Security::XmlSecurityC::init();
            domXml = new Smartmatic::System::XML::DomXml();
        }

        void tearDown()
        {
            delete domXml;
            domXml = NULL;

            Smartmatic::System::Security::XmlSecurityC::clear();
        }

        void testEmptyStringToDom()
        {
            std::string xml;
            std::string result;

            xercesc::DOMDocument * doc = domXml->stringToDom ( xml );

            CPPUNIT_ASSERT ( NULL != doc );

            domXml->domToString ( doc, result );

            CPPUNIT_ASSERT ( 0 == std::string ( "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>" ).compare ( result ) );
        }

        void testNotValidStringToDom()
        {
            std::string xml = "<sdsds";
            std::string result;

            xercesc::DOMDocument * doc = domXml->stringToDom ( xml );

            CPPUNIT_ASSERT ( NULL != doc );

            domXml->domToString ( doc, result );

            CPPUNIT_ASSERT ( 0 == std::string ( "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>" ).compare ( result ) );
        }

        void testStringToDom()
        {
            std::string xml = "<a name=\"one\" />";
            std::string result;

            xercesc::DOMDocument * doc = domXml->stringToDom ( xml );

            CPPUNIT_ASSERT ( NULL != doc );

            domXml->domToString ( doc, result );

            CPPUNIT_ASSERT ( 0 == std::string ( "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?><a name=\"one\"/>" ).compare ( result ) );
        }

        void testEmptyBufferToDom()
        {
            Smartmatic::System::SafeBuffer xml;
            Smartmatic::System::SafeBuffer result;

            xercesc::DOMDocument * doc = domXml->bufferToDom ( xml );

            CPPUNIT_ASSERT ( NULL != doc );

            domXml->domToBuffer ( doc, result );

            CPPUNIT_ASSERT ( 0 == std::string ( "PD94bWwgdmVyc2lvbj0iMS4wIiBlbmNvZGluZz0iVVRGLTgiIHN0YW5kYWxvbmU9Im5vIiA/Pg==" ).compare ( result.GetBase64Data() ) );
        }

        void testNotValidBufferToDom()
        {
            Smartmatic::System::SafeBuffer xml ( "<sdsds" );
            Smartmatic::System::SafeBuffer result;

            xercesc::DOMDocument * doc = domXml->bufferToDom ( xml );

            CPPUNIT_ASSERT ( NULL != doc );

            domXml->domToBuffer ( doc, result );

            CPPUNIT_ASSERT ( 0 == std::string ( "PD94bWwgdmVyc2lvbj0iMS4wIiBlbmNvZGluZz0iVVRGLTgiIHN0YW5kYWxvbmU9Im5vIiA/Pg==" ).compare ( result.GetBase64Data() ) );
        }

        void testBufferToDom()
        {
            Smartmatic::System::SafeBuffer xml ( "<b>Text<b/>" );
            Smartmatic::System::SafeBuffer result;

            xercesc::DOMDocument * doc = domXml->bufferToDom ( xml );

            CPPUNIT_ASSERT ( NULL != doc );

            domXml->domToBuffer ( doc, result );

            CPPUNIT_ASSERT ( 0 == std::string ( "PD94bWwgdmVyc2lvbj0iMS4wIiBlbmNvZGluZz0iVVRGLTgiIHN0YW5kYWxvbmU9Im5vIiA/PjxiPlRleHQ8Yi8+PC9iPg==" ).compare ( result.GetBase64Data() ) );
        }

        void testFindIfDocumentIsNull()
        {
            xercesc::DOMDocument * doc = NULL;
            xercesc::DOMNode * node = domXml->getNode ( doc, "x" );

            CPPUNIT_ASSERT ( NULL == node );
        }

        void testFindNodeNotExist()
        {
            Smartmatic::System::SafeBuffer xml ( "<all><a>Image</a><b>Text</b></all>" );
            xercesc::DOMDocument * doc = domXml->bufferToDom ( xml );

            CPPUNIT_ASSERT ( NULL != doc );

            xercesc::DOMNode * node = domXml->getNode ( doc, "x" );

            CPPUNIT_ASSERT ( NULL == node );
        }

        void testFindNodeExist()
        {
            Smartmatic::System::SafeBuffer xml ( "<all><a>Image</a><b>Text</b></all>" );
            xercesc::DOMDocument * doc = domXml->bufferToDom ( xml );

            CPPUNIT_ASSERT ( NULL != doc );

            xercesc::DOMNode * node = domXml->getNode ( doc, "all" );

            CPPUNIT_ASSERT ( NULL != node );
        }

        void testNullDomToString()
        {
            std::string result;
            xercesc::DOMDocument * doc = NULL;
            CPPUNIT_ASSERT_THROW ( domXml->domToString ( doc, result ), XmlException );
        }

        void testNullDomToBuffer()
        {
            Smartmatic::System::SafeBuffer result;
            xercesc::DOMDocument * doc = NULL;
            CPPUNIT_ASSERT_THROW ( domXml->domToBuffer ( doc, result ), XmlException );
        }

        CPPUNIT_TEST_SUITE ( DomXmlTest );

        CPPUNIT_TEST ( testEmptyStringToDom );
        CPPUNIT_TEST ( testNotValidStringToDom );
        CPPUNIT_TEST ( testStringToDom );
        CPPUNIT_TEST ( testEmptyBufferToDom );
        CPPUNIT_TEST ( testNotValidBufferToDom );
        CPPUNIT_TEST ( testBufferToDom );
        CPPUNIT_TEST ( testFindIfDocumentIsNull );
        CPPUNIT_TEST ( testFindNodeNotExist );
        CPPUNIT_TEST ( testFindNodeExist );
        CPPUNIT_TEST ( testNullDomToString );
        CPPUNIT_TEST ( testNullDomToBuffer );

        CPPUNIT_TEST_SUITE_END();
};

#endif /* ECUT_MAIN */

#endif /* TEST_STRING_UTILS_TEST */
