/**
 * @file	s-m-t-t-g-u-i-conf-manager.cc
 * @brief	Body the smtt gui conf manager class.
 */

#include "s-m-t-t-g-u-i-conf-manager.h"
#include <Configuration/configuration-persistence-manager.h>
#include <System/OptParser.hxx>
#include <System/Runtime/ExitProcess.hxx>
#include <System/ZenityWrapper.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::Configuration;
using namespace Smartmatic::System;
using namespace Smartmatic::System::Runtime;

SMTTGUIConfigurationManager * SMTTGUIConfigurationManager::instance = NULL;

SMTTGUIConfigurationManager::SMTTGUIConfigurationManager()
{
    configuration = NULL;
}

SMTTGUIConfigurationManager::~SMTTGUIConfigurationManager()
{
    if ( configuration != NULL )
    {
        delete ( configuration );
        configuration = NULL;
    }

    instance = NULL;
}

void SMTTGUIConfigurationManager::initialize()
{
    try
    {
        std::string semiFile ( ConfigurationPersistenceManager::getInstance()->GetLibraryConfigFile ( "SMTTGUI" ) );
        std::istringstream ss ( semiFile );
        configuration = new SmartmaticGUIConfiguration ( * parseSmartmaticGUIConfiguration ( ss, ConfigurationTypes::Flags::dont_validate ).get() );
    }
    catch ( const ::xsd::cxx::tree::exception< char > & e )
    {
        std::stringstream ss;
        ss << e;
        ZenityWrapper::showError ( "Config SMTTGUI Error \n\n" + ss.str(), "Error" );
        ExitProcess::getInstance()->exitProcess();
    }
}

SmartmaticGUIConfiguration * SMTTGUIConfigurationManager::getConfiguration()
{
    if ( configuration == NULL )
    {
        initialize();
    }

    return configuration;
}

void SMTTGUIConfigurationManager::loadDefault()
{
    SmartmaticGUIConfiguration configGUI;
    BasicWindowConfigurations configBasicWindow;
    EventSoundsConfiguration eventSounds;
    AZERTYKeyboardWidgetConfiguration azertyConf;

    EntryPassPhraseConfiguration entryConf ( EntryPassPhraseConfiguration::CONFIGURATION_CASESENSITIVE );
    ButtonsQuestionConfiguration buttonsQuestionConfiguration ( ButtonsQuestionConfiguration::CONFIGURATION_ACCEPT_CANCEL );
    ButtonLabelSizeConfiguration buttonConf;

    ScreenCaptureConfiguration screenCaptureConfiguration;
    screenCaptureConfiguration.setDefault ( CaptureType::CONFIGURATION_IMPORT );

    ScreenCaptureType importCapture ( CaptureType::CONFIGURATION_IMPORT, "-silent -window root" );
    ScreenCaptureType scrotCapture ( CaptureType::CONFIGURATION_SCROT, "--silent" );

    ScreenCaptureConfiguration::ScreenCaptureSequence captureSeq;
    captureSeq.push_back ( importCapture );
    captureSeq.push_back ( scrotCapture );

    screenCaptureConfiguration.setScreenCapture ( captureSeq );

    SmartmaticGUIMachineModelConfiguration::MachineModelSequence machineModelSequence;
    Smartmatic::GUI::Configuration::MachineModelType machineModelVIU;
    machineModelVIU.setId( "PM-VIU805" );
    Smartmatic::GUI::Configuration::WindowSizeType windowSizeVIU;
    windowSizeVIU.setHeight( 800 );
    windowSizeVIU.setWidth( 1280 );
    machineModelVIU.setWindowSize( windowSizeVIU );
    machineModelVIU.setTitleHeight( 85 );
    Smartmatic::GUI::Configuration::WindowPaddingType windowPaddingVIU;
    windowPaddingVIU.setHorizontalPadding( 5 );
    windowPaddingVIU.setVerticalPadding( 5 );
    machineModelVIU.setWindowPadding( windowPaddingVIU );
    machineModelSequence.push_back( machineModelVIU );

    Smartmatic::GUI::Configuration::MachineModelType machineModelPMA;
    machineModelPMA.setId( "PM-NOTEBOOK-A" );
    Smartmatic::GUI::Configuration::WindowSizeType windowSizePMA;
    windowSizePMA.setHeight( 600 );
    windowSizePMA.setWidth( 1024 );
    machineModelPMA.setWindowSize( windowSizePMA );
    machineModelPMA.setTitleHeight( 85 );
    Smartmatic::GUI::Configuration::WindowPaddingType windowPaddingPMA;
    windowPaddingPMA.setHorizontalPadding( 5 );
    windowPaddingPMA.setVerticalPadding( 5 );
    machineModelPMA.setWindowPadding( windowPaddingPMA );
    machineModelSequence.push_back( machineModelPMA );

    Smartmatic::GUI::Configuration::MachineModelType machineModelPMB;
    machineModelPMB.setId( "PM-NOTEBOOK-B" );
    Smartmatic::GUI::Configuration::WindowSizeType windowSizePMB;
    windowSizePMB.setHeight( 600 );
    windowSizePMB.setWidth( 1024 );
    machineModelPMB.setWindowSize( windowSizePMB );
    machineModelPMB.setTitleHeight( 85 );
    Smartmatic::GUI::Configuration::WindowPaddingType windowPaddingPMB;
    windowPaddingPMB.setHorizontalPadding( 5 );
    windowPaddingPMB.setVerticalPadding( 5 );
    machineModelPMB.setWindowPadding( windowPaddingPMB );
    machineModelSequence.push_back( machineModelPMB );

    Smartmatic::GUI::Configuration::MachineModelType machineModelVM;
    machineModelVM.setId( "VM-SAES3377" );
    Smartmatic::GUI::Configuration::WindowSizeType windowSizeVM;
    windowSizeVM.setHeight( 1280 );
    windowSizeVM.setWidth( 1024 );
    machineModelVM.setWindowSize( windowSizeVM );
    machineModelVM.setTitleHeight( 85 );
    Smartmatic::GUI::Configuration::WindowPaddingType windowPaddingVM;
    windowPaddingVM.setHorizontalPadding( 5 );
    windowPaddingVM.setVerticalPadding( 5 );
    machineModelVM.setWindowPadding( windowPaddingVM );
    machineModelSequence.push_back( machineModelVM );

    SmartmaticGUIMachineModelConfiguration smartmaticGUIMachineModelConfiguration;
    smartmaticGUIMachineModelConfiguration.setMachineModel( machineModelSequence );

    configBasicWindow.setWindowTitle ( "Application" );
    configBasicWindow.setShowBorder ( false );

    configGUI.setBasicWindowConfigurations ( configBasicWindow );

    azertyConf.setShowChars ( false );

    eventSounds.setSmartmaticButtonSound ( "SmartmaticButtonSound" );
    eventSounds.setNumericKeypadSound ( "KeyPadSound" ); //keypadsound
    eventSounds.setCancelKeypadSound ( "KeyPadSound" ); //keypadsound
    eventSounds.setAcceptKeypadSound ( "KeyPadSound" ); //keypadsound
    eventSounds.setConfirmationDialogSound ( "ConfirmationVoteSound" ); //ConfirmationDialogSound
    eventSounds.setChooseDialogSound ( "QuestionVoteSound" ); //ChooseDialogSound
    eventSounds.setAzertyKeypadSound ( "AzertySound" );
    eventSounds.setMenuSound ( "MenuSound" );

    buttonConf.setAllButtons ( 13 );
    buttonConf.setMenuButtons ( 14 );

    configGUI.setEventSoundsConfiguration ( eventSounds );
    configGUI.setAZERTYKeyboardWidgetConfiguration ( azertyConf );
    configGUI.setEntryPassPhraseConfiguration ( entryConf );
    configGUI.setButtonLabelSizeConfiguration ( buttonConf );
    configGUI.setButtonsQuestionConfiguration ( buttonsQuestionConfiguration );
    configGUI.setScreenCaptureConfiguration ( screenCaptureConfiguration );
    configGUI.setGuiThemeConfiguration ( GuiThemeConfiguration ( "", "", "", "" ) );
    configGUI.setSmartmaticGUIMachineModelConfiguration( smartmaticGUIMachineModelConfiguration );

    Smartmatic::ConfigurationTypes::NamespaceInfomap map;
    map[""].name = NAMESPACE_SMARTMATIC_GUI_CONFIGURATION;
    map[""].schema = "";

    std::stringstream ss;
    serializeSmartmaticGUIConfiguration ( ss, configGUI, map, "UTF-8" );

    ConfigurationPersistenceManager::getInstance()->SetLibraryConfigFile ( "SMTTGUI", ss.str() );
}

SMTTGUIConfigurationManager * SMTTGUIConfigurationManager::getInstance ( SMTTGUIConfigurationManager * externManager )
{
    if ( ! instance )
    {
        instance = ( externManager == NULL ? new SMTTGUIConfigurationManager () : externManager );
    }

    return instance;
}

int SMTTGUIConfigurationManager::getAllButtonsSize()
{
    return getConfiguration()->getButtonLabelSizeConfiguration().getAllButtons();
}

int SMTTGUIConfigurationManager::getMenuButtonsSize()
{
    return getConfiguration()->getButtonLabelSizeConfiguration().getMenuButtons();
}

void SMTTGUIConfigurationManager::getWindowSize ( int & width, int & height )
{
    width = getSmartmaticGUIMachineModelConfiguration().getWindowSize().getWidth();
    height = getSmartmaticGUIMachineModelConfiguration().getWindowSize().getHeight();
}

void SMTTGUIConfigurationManager::getPaddings ( int & xPadding, int & yPadding )
{
    xPadding = getSmartmaticGUIMachineModelConfiguration().getWindowPadding().getHorizontalPadding();
    yPadding = getSmartmaticGUIMachineModelConfiguration().getWindowPadding().getVerticalPadding();
}

void SMTTGUIConfigurationManager::getCenterSize ( int & width, int & height )
{
    int xp = getSmartmaticGUIMachineModelConfiguration().getWindowPadding().getHorizontalPadding();

    getWindowSize ( width, height );

    width -= 2 * xp;
    height -= 2 * xp;

    width = ( width < 0 ? 0 : width );
    height = ( height < 0 ? 0 : height );
}

void SMTTGUIConfigurationManager::getHeaderSize ( int & width, int & height )
{
    int xp = getSmartmaticGUIMachineModelConfiguration().getWindowPadding().getHorizontalPadding();

    width = getSmartmaticGUIMachineModelConfiguration().getWindowSize().getWidth();
    height = getSmartmaticGUIMachineModelConfiguration().getTitleHeight();

    width -= 2 * xp;

    width = ( width < 0 ? 0 : width );
}

void SMTTGUIConfigurationManager::getCenterSizeWithHeader ( int & width, int & height )
{
    int xp, yp;

    getPaddings ( xp, yp );
    getWindowSize ( width, height );

    int xh, yh;

    getHeaderSize ( xh, yh );

    width -= 2 * xp;
    height -= 2 * xp + yh + yp;

    width = ( width < 0 ? 0 : width );
    height = ( height < 0 ? 0 : height );
}

bool SMTTGUIConfigurationManager::getAZERTYShowPass()
{
    return getConfiguration()->getAZERTYKeyboardWidgetConfiguration().getShowChars();
}

std::string SMTTGUIConfigurationManager::getSmartmaticButtonSound()
{
    return getConfiguration()->getEventSoundsConfiguration().getSmartmaticButtonSound();
}

std::string SMTTGUIConfigurationManager::getNumericKeypadSound()
{
    return getConfiguration()->getEventSoundsConfiguration().getNumericKeypadSound();
}

std::string SMTTGUIConfigurationManager::getCancelKeypadSound()
{
    return getConfiguration()->getEventSoundsConfiguration().getCancelKeypadSound();
}

std::string SMTTGUIConfigurationManager::getAcceptKeypadSound()
{
    return getConfiguration()->getEventSoundsConfiguration().getAcceptKeypadSound();
}

std::string SMTTGUIConfigurationManager::getAzertyKeypadSound()
{
    return getConfiguration()->getEventSoundsConfiguration().getAzertyKeypadSound();
}

std::string SMTTGUIConfigurationManager::getConfirmationDialogSound()
{
    return getConfiguration()->getEventSoundsConfiguration().getConfirmationDialogSound();
}

std::string SMTTGUIConfigurationManager::getChooseDialogSound()
{
    return getConfiguration()->getEventSoundsConfiguration().getChooseDialogSound();
}

std::string SMTTGUIConfigurationManager::getMenuSound()
{
    return getConfiguration()->getEventSoundsConfiguration().getMenuSound();
}

EntryPassPhraseConfiguration::Value SMTTGUIConfigurationManager::getEntryConfiguration()
{
    return getConfiguration()->getEntryPassPhraseConfiguration();
}

ButtonsQuestionConfiguration::Value SMTTGUIConfigurationManager::getButtonsQuestionConfiguration()
{
    return getConfiguration()->getButtonsQuestionConfiguration();
}

Smartmatic::GUI::Configuration::ScreenCaptureConfiguration & SMTTGUIConfigurationManager::getScreenCaptureConfiguration()
{
    return getConfiguration()->getScreenCaptureConfiguration();
}

Smartmatic::GUI::Configuration::ScreenCaptureType SMTTGUIConfigurationManager::getScreenCaptureType ( Smartmatic::GUI::Configuration::CaptureType type )
{
    Smartmatic::GUI::Configuration::ScreenCaptureType screenType ( type, "" );
    Smartmatic::GUI::Configuration::ScreenCaptureConfiguration::ScreenCaptureIterator it;
    Smartmatic::GUI::Configuration::ScreenCaptureConfiguration::ScreenCaptureSequence & seq ( getScreenCaptureConfiguration().getScreenCapture() );

    for ( it = seq.begin();
            it != seq.end();
            ++it )
    {
        if ( ( *it ).getType() == type )
        {
            screenType = ( *it );
            break;
        }
    }

    return screenType;
}

Smartmatic::GUI::Configuration::GuiThemeConfiguration SMTTGUIConfigurationManager::getGuiThemeConfiguration ()
{
    GuiThemeConfiguration configuration ( "", "", "", "" );

    if ( getConfiguration()->getGuiThemeConfiguration().present() )
    {
        configuration = getConfiguration()->getGuiThemeConfiguration().get();
    }

    return configuration;
}

SmartmaticGUIMachineModelConfiguration::MachineModelType SMTTGUIConfigurationManager::getSmartmaticGUIMachineModelConfiguration ()
{
    std::string machineModel;
    OptParser::getInstance()->getArg( OptParser::MACHINE_MODEL, machineModel );

    SmartmaticGUIMachineModelConfiguration::MachineModelSequence machineModels = getConfiguration()->getSmartmaticGUIMachineModelConfiguration().getMachineModel();

    Smartmatic::GUI::Configuration::MachineModelType configuration = machineModels[ 0 ];

    if ( machineModel.empty() == false )
    {
        for ( size_t i = 0; i < machineModels.size(); ++i )
        {
            if ( machineModels[ i ].getId() == machineModel )
            {
                configuration = machineModels[ i ];
                i = machineModels.size();
            }
        }
    }

    return configuration;
}
