/**
 * @file SmartmaticTextView.cpp
 * @brief Body class text view
 * @date 23/07/2011
 * @author Juan Delgado
 */

#include "SmartmaticTextView.h"

using namespace Smartmatic::GUI::Widgets;

SmartmaticTextView::SmartmaticTextView()
    : SmartmaticBasicScrollContainer(),
      ISmartmaticText ()
{
    initialize();
}

SmartmaticTextView::SmartmaticTextView ( BaseObjectType * cobject, const Glib::RefPtr<Gtk::Builder> & refBuilder )
    : SmartmaticBasicScrollContainer ( cobject ),
      ISmartmaticText ()
{
    initialize();
}

SmartmaticTextView::~SmartmaticTextView()
{
    remove();

    if ( textView )
    {
        delete textView;
        textView = NULL;
    }
}

void SmartmaticTextView::initialize()
{
    offset = 0;
    tempOffset = 0;

    textBuffer = Gtk::TextBuffer::create();
    textBuffer->signal_insert().connect ( sigc::mem_fun ( *this, &SmartmaticTextView::insertText ) );
    textBuffer->signal_erase().connect ( sigc::mem_fun ( *this, &SmartmaticTextView::deleteText ) );
    textBuffer->signal_changed().connect ( sigc::mem_fun ( *this, &SmartmaticTextView::changeText ) );
    textBuffer->signal_mark_set().connect ( sigc::mem_fun ( *this, &SmartmaticTextView::changeMark ) );
    textBuffer->signal_mark_deleted().connect ( sigc::mem_fun ( *this, &SmartmaticTextView::changeMarkOffset ) );

    textView = new Gtk::TextView();
    textView->set_buffer ( textBuffer );
    textView->set_wrap_mode ( Gtk::WRAP_WORD_CHAR );
    textView->set_can_focus ( true );
    textView->set_can_default ( true );
    textView->set_editable ( true );
    textView->set_cursor_visible ( true );

    textView->signal_button_press_event().connect ( sigc::mem_fun ( this, &SmartmaticTextView::on_button_press_event ) );
    textView->signal_key_press_event().connect ( sigc::mem_fun ( this, &SmartmaticTextView::on_key_press_event ) );

    set_policy ( Gtk::POLICY_NEVER, Gtk::POLICY_AUTOMATIC );
    set_shadow_type ( Gtk::SHADOW_IN );
    remove();
    add ( *textView );
    show_all_children();
}

bool SmartmaticTextView::on_button_press_event ( GdkEventButton * event )
{
    return event->button != 3;
}

bool SmartmaticTextView::on_key_press_event ( GdkEventKey * event )
{
    bool result = false;

#ifdef GDK_KEY_Menu

    if ( event->keyval != GDK_KEY_Menu )
    {
        result = true;
    }

#else

    if ( event->keyval != 0xff67 )
    {
        result = true;
    }

#endif
    return result;
}

void SmartmaticTextView::setText ( std::string text )
{
    textView->set_name ( "" );
    textBuffer->set_text ( text );
}

std::string SmartmaticTextView::getText()
{
    return ( std::string ) textBuffer->get_text();
}

void SmartmaticTextView::setBGError()
{
    //textView->set_name ( "entry_error_frame" );
}

void SmartmaticTextView::setBGNormal()
{
    textView->set_name ( getThemeConfiguration() );
}

void SmartmaticTextView::insertText ( const Gtk::TextBuffer::iterator & a, const Glib::ustring & b, int c )
{
    if ( max != -1 && getText().size() > ( unsigned int ) max )
    {
        setText ( getText().substr ( 0, max ) );
        return;
    }

    setBGNormal();
    validate();
}

void SmartmaticTextView::deleteText ( const Gtk::TextBuffer::iterator & a, const Gtk::TextBuffer::iterator & b )
{
    setBGNormal();
    validate();
}

void SmartmaticTextView::changeMark ( const Gtk::TextBuffer::iterator & a, const Glib::RefPtr<Gtk::TextBuffer::Mark> & mark )
{
    tempOffset = a.get_offset();
}

void SmartmaticTextView::changeMarkOffset ( const Glib::RefPtr<Gtk::TextBuffer::Mark> & mark )
{
    offset = tempOffset;
}

void SmartmaticTextView::changeText()
{
    setBGNormal();
    validate();
}

void SmartmaticTextView::focus()
{
    textView->grab_focus();
}

void SmartmaticTextView::vadjustment()
{
    Gtk::Adjustment * adj = get_vadjustment();
    adj->set_value ( adj->get_upper() );
}

void SmartmaticTextView::insert ( const std::string & text )
{
    std::string info = getText();

    if ( info.size() < ( unsigned int ) max )
    {
        std::string tmp;

        unsigned int startOffset = offset;
        unsigned int endOffset = offset;

        if ( textBuffer->get_has_selection() )
        {
            Gtk::TextIter start;
            Gtk::TextIter end;
            textBuffer->get_selection_bounds ( start, end );

            startOffset = start.get_offset();
            endOffset = end.get_offset();
        }

        tmp.append ( info.substr ( 0, startOffset ) );
        tmp.append ( text );
        tmp.append ( info.substr ( endOffset, info.size() ) );

        setText ( tmp );

        offset = startOffset + text.size();

        focusCursor();
    }
}

void SmartmaticTextView::deleted()
{
    if ( !getText().empty() )
    {
        std::string info = getText();
        std::string tmp;

        unsigned int startOffset = offset;
        unsigned int endOffset = offset;

        if ( textBuffer->get_has_selection() )
        {
            Gtk::TextIter start;
            Gtk::TextIter end;
            textBuffer->get_selection_bounds ( start, end );

            startOffset = start.get_offset();
            endOffset = end.get_offset();
        }
        else if ( offset > 0 )
        {
            startOffset--;
        }

        tmp.append ( info.substr ( 0, startOffset ) );
        tmp.append ( info.substr ( endOffset, info.size() ) );

        setText ( tmp );

        offset = startOffset;
    }

    focusCursor();
}

void SmartmaticTextView::up()
{
    gboolean result;
    g_signal_emit_by_name ( textView->gobj(), "move-cursor",
                            GTK_MOVEMENT_DISPLAY_LINES, -1, &result );

    offset = textBuffer->get_insert()->get_iter().get_offset();
    focusCursor();
}

void SmartmaticTextView::down()
{
    gboolean result;
    g_signal_emit_by_name ( textView->gobj(), "move-cursor",
                            GTK_MOVEMENT_DISPLAY_LINES, 1, &result );
    offset = textBuffer->get_insert()->get_iter().get_offset();
    focusCursor();
}

void SmartmaticTextView::left()
{
    gboolean result;
    g_signal_emit_by_name ( textView->gobj(), "move-cursor",
                            GTK_MOVEMENT_VISUAL_POSITIONS, -1, &result );
    offset = textBuffer->get_insert()->get_iter().get_offset();

    focusCursor();
}

void SmartmaticTextView::right()
{
    gboolean result;
    g_signal_emit_by_name ( textView->gobj(), "move-cursor",
                            GTK_MOVEMENT_VISUAL_POSITIONS, 1, &result );
    offset = textBuffer->get_insert()->get_iter().get_offset();

    focusCursor();
}

void SmartmaticTextView::focusCursor()
{
    textBuffer->get_insert()->get_iter().set_offset ( offset );
    Gtk::TextIter iter = textBuffer->get_iter_at_offset ( offset );

    vadjustment();
    textView->scroll_to_iter ( iter, 0.49 );
}
