/**
 * @file	key-board-widget.cc
 * @brief	key board widget class.
 */

/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * Key-Board
 * Copyright (C)  2010 <>
 *
 * Key-Board is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Key-Board is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "key-board-widget.h"

#include "Configuration/s-m-t-t-g-u-i-conf-manager.h"

#include <Log/ISMTTLog.h>
#include <Log/SMTTLogManager.h>
#include <System/Utils/StringUtils.hxx>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef ENABLE_NLS
#include <libintl.h>
#define _(String) dgettext (GETTEXT_PACKAGE, String) ///<macro for gettext
#else
#define _(String) (String) ///<macro without gettext
#endif

#define N_(String) (String) ///<macro for gettext not translate

using namespace Smartmatic::Environment;
using namespace Smartmatic::GUI::Configuration;
using namespace Smartmatic::GUI::Widgets;
using namespace Smartmatic::Log;
using namespace Smartmatic::System::Utils;

static const std::string SPACE = " ";

static const Glib::ustring KEYS_IDENTIFIERS_ARRAY[] =
{
    "~", "1", "2", "3", "4", "5", "6", "7", "8", "9", "0", "-", "=",
    "a", "z", "e", "r", "t", "y", "u", "i", "o", "p", "[", "]", "\\",
    "q", "s", "d", "f", "g", "h", "j", "k", "l", "m", "'",
    "w", "x", "c", "v", "b", "n", ",", ";", ":",
    "_"
};

static const Glib::ustring SPECIAL_KEYS_ARRAY[] =
{
    "Enter", "Shift", "UpArrow", "Backspace", "Escape", "Space", "LeftArrow", "DownArrow", "RightArrow"
};

ISMTTLog * Keyboard::m_logger = SMTTLogManager::GetLogger("Smartmatic.GUI.Widgets.Keyboard", GETTEXT_PACKAGE);

Keyboard::Keyboard(const std::vector<std::string>& mainKeys, const std::vector<std::string>& secondKeys)
: SmartmaticBasicTable()
, m_isShifted(false)
, m_mainKeys(mainKeys)
, m_secondKeys(secondKeys)
{
    try
    {
        m_builder = Gtk::Builder::create_from_file(ResourcePathProvider::getInstance()->GetResourcePath(PACKAGE_NAME, "ui/key-board.glade"));
    }
    catch ( const Glib::FileError & ex )
    {
        m_logger->Debug(ex.what().c_str());
        throw;
    }

    m_builder->get_widget_derived("table1", m_keyBoardTable);

    const std::vector<std::string> keysIdentifiers(KEYS_IDENTIFIERS_ARRAY, KEYS_IDENTIFIERS_ARRAY + sizeof KEYS_IDENTIFIERS_ARRAY / sizeof KEYS_IDENTIFIERS_ARRAY[0]);
    m_keysIdentifiers = keysIdentifiers;

    setKeysButtons(m_mainKeys);
    setSpecialKeysButtons();
    setSpecialKeysLabels();
    setMainKeysActions();

    add ( *m_keyBoardTable );
    show_all_children();

    setAccessibleName("KeyboardContainer");
    m_keyBoardTable->setAccessibleName("Keyboard");

    set_can_focus(false);
}

Keyboard::~Keyboard()
{
}

void Keyboard::setKeysButtons(const std::vector<std::string>& keys)
{
    for (size_t i = 0; i < keys.size(); i++)
    {
        m_builder->get_widget_derived(StringUtils::toLower(m_keysIdentifiers[i]), m_buttonArray[i]);
        m_buttonArray[i]->setAccessibleName(keys[i]);
        m_buttonArray[i]->SetLabel(keys[i]);
    }
}

void Keyboard::setSpecialKeysButtons()
{
    const size_t specialKeysSize = *(&SPECIAL_KEYS_ARRAY + 1) - SPECIAL_KEYS_ARRAY;

    for(size_t i = 0; i < specialKeysSize; i++)
    {
        m_builder->get_widget_derived(SPECIAL_KEYS_ARRAY[i], m_buttonArray[i + m_keysIdentifiers.size()]);
        m_buttonArray[i + m_keysIdentifiers.size()]->setAccessibleName(SPECIAL_KEYS_ARRAY[i]);
    }
}

void Keyboard::setSpecialKeysLabels()
{
    m_buttonArray[enter]->SetLabel(_("Smartmatic.GUI.Glade.Key-board.Enter"));
    m_buttonArray[shift]->SetLabel(_("Smartmatic.GUI.Glade.Key-board.Shift"));
    m_buttonArray[upArrow]->SetLabel(_("Smartmatic.GUI.Glade.Key-board.UpArrow"));
    m_buttonArray[backspace]->SetLabel(_("Smartmatic.GUI.Glade.Key-board.Backspace"));
    m_buttonArray[escape]->SetLabel(_("Smartmatic.GUI.Glade.Key-board.Escape"));
    m_buttonArray[space]->SetLabel(_("Smartmatic.GUI.Glade.Key-board.Space"));
    m_buttonArray[leftArrow]->SetLabel(_("Smartmatic.GUI.Glade.Key-board.LeftArrow"));
    m_buttonArray[downArrow]->SetLabel(_("Smartmatic.GUI.Glade.Key-board.DownArrow"));
    m_buttonArray[rightArrow]->SetLabel(_("Smartmatic.GUI.Glade.Key-board.RightArrow"));
}

void Keyboard::setMainKeysActions()
{
    const size_t buttonArraySize = *(&m_buttonArray + 1) - m_buttonArray;

    for (size_t i = 0; i < buttonArraySize; ++i)
    {
        m_buttonArray[i]->signal_clicked().connect(sigc::bind(sigc::mem_fun(*this, &Keyboard::onButtonClicked), i));
        m_buttonArray[i]->SetSound(SMTTGUIConfigurationManager::getInstance()->getAzertyKeypadSound());
    }
}

void Keyboard::handleSpecialKeyAction(const size_t key)
{
    switch (key)
    {
        case enter:
            m_signalEnter.emit();
            break;

        case shift:
            shiftKeys();
            break;

        case upArrow:
            m_signalUp.emit();
            break;

        case backspace:
            m_signalDelete.emit();
            break;

        case space:
            m_signalDetect.emit(SPACE);
            break;

        default:
            break;
    }
}

void Keyboard::onButtonClicked(const size_t identifier)
{
    if (identifier < m_keysIdentifiers.size())
        m_signalDetect.emit(m_buttonArray[identifier]->GetLabel());
    else
        handleSpecialKeyAction(identifier);
}

void Keyboard::shiftKeys()
{
    if (m_isShifted)
    {
        setKeysButtons(m_mainKeys);
        m_isShifted = false;
    }
    else
    {
        setKeysButtons(m_secondKeys);
        m_isShifted = true;
    }
}

void Keyboard::setSensitiveButton(const SpecialKeys& key, const bool sensitive)
{
    m_buttonArray[key]->set_sensitive(sensitive);
}

Keyboard::DetectSignal Keyboard::getDetectSignal()
{
    return m_signalDetect;
}

Keyboard::DeleteSignal Keyboard::getDeleteSignal()
{
    return m_signalDelete;
}

Keyboard::EnterSignal Keyboard::getEnterSignal()
{
    return m_signalEnter;
}

Keyboard::UpSignal Keyboard::getUpSignal()
{
    return m_signalUp;
}

Keyboard::DownSignal Keyboard::getDownSignal()
{
    return m_signalDown;
}

Keyboard::LeftSignal Keyboard::getLeftSignal()
{
    return m_signalLeft;
}

Keyboard::LeftSignal Keyboard::getRightSignal()
{
    return m_signalRight;
}
